/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Extensions for Dojo
*
* (C) Copyright IBM Corp. 2011.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
dojo.provide("dojoe.table._TableOptions");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("dijit.form.CheckBox");

// NLS
dojo.requireLocalization("dojoe.table", "Table");
// define the widget class
dojo.declare (
	"dojoe.table._TableOptions",[ dijit._Widget, dijit._Container ],{
    grid: null,
	constructor: function() {
		// summary: A reusable widget for displaying a dynamic set of parameters to be 
		//  filled in at runtime.
	    this.resources_ = dojo.i18n.getLocalization("dojoe.table", "Table", this.lang);
	},
	postCreate: function() { 
		this.inherited ("postCreate", arguments);		
		for(var i=0; i<this.grid.structure.length; i++) {
			this.addColumnsVisible(this.grid.structure[i].rows);
		}
	},
	addColumnsVisible: function (columns) {		
		var widget = null;
		var column = null;
		for (var i in columns) {
			if (i) {
				column = columns[i];
				if (column.field) {
					widget = new dojoe.table.ColumnVisible ({
						id: this.id + "column_" + column.field,
						columnid: this.id + "column_" + column.field,
						checked: column.visible===undefined?!column.hidden:column.visible,  
						label: column.name
					});
					this.addChild(widget);
					//console.log ("added column widget");
				} else {
					//console.log ("this column was created internally by data store.  do not show it.");
				}
			}
		}
	},	
	onSelectAll: function(){		
		for(var j=0; j<this.grid.structure.length; j++) {
			var columns = this.grid.structure[j].rows;	 
			for (var i in columns) {
				if (i) {
					var columnWidget = dijit.byId (this.id + "column_" + columns[i].field);
					columnWidget.setVisible(true);	 
				}
			}		
		}
	},
	onDeselectAll: function(){
		for(var j=0; j<this.grid.structure.length; j++) {
			var columns = this.grid.structure[j].rows;
			for (var i in columns) {
				if (i) {
					var columnWidget = dijit.byId (this.id + "column_" + columns[i].field);
					columnWidget.setVisible(false);	 
				}
			}
		}
	},
	onSave: function(){
	for(var j=0; j<this.grid.structure.length; j++) {
			var columns = this.grid.structure[j].rows;
			var allColumns = [];	
			for (var i=0; i<columns.length; i++) {
					var column = columns[i];
					var columnWidget = dijit.byId (this.id + "column_" + column.field);
					var tblColumn = column;
					column.visible = columnWidget.isVisible();
					columns[i].visible=columnWidget.isVisible();					
					if (tblColumn) {
						// In order to preserve the column width, we need to set the width property
						tblColumn.width = tblColumn.unitWidth;
						// set these values on the originalStruture so they will be in memory
						// in case the dialog is opened again
						column.width = tblColumn.unitWidth;
					} else {
						tblColumn = column;
					}				
					for(var k=0; k<this.grid.layout.cells.length; k++) {
						var tempCol = this.grid.layout.cells[k];
						if(tempCol.field === column.field) {
							this.grid.layout.setColumnVisibility(k, columnWidget.isVisible());							
						}
					}					
					allColumns.push (column);
			}	
			//this.grid.savePreferences();
			this.grid.structure[j].rows = allColumns;
		}
	},
	isColumnVisible: function (columnID) {
		var columnWidget = dijit.byId (this.id + "column_" + columnID);
		return (columnWidget.isVisible());
	}	
});

// define the widget class
dojo.declare (
	"dojoe.table.ColumnVisible", [ dijit._Widget, dijit._Templated ],{
	columnid: null,
	label: null,
	checked: false,
	widgetsInTemplate: true,
	// Path to the template
	templateString : dojo.cache("dojoe.table","templates/ColumnVisible.html"),		
	constructor: function () {		
	},
	postCreate: function () {		
		if (this.checked) {
			this.checkbox.setChecked (this.checked);
		}		
	},
	setVisible: function (checked) {
		this.checkbox.setChecked (checked);
	},
	isVisible: function () {
		return this.checkbox.checked;
	},
	destroy: function () {		
		//memory leak issue: custom destroy method to take care of removing all widgets					
		try{
			delete this.resources_;
			//this is for destroying child templated widgets 
			//this should happen by itself but need to figure out why dijit._Templated is not taking care of it...
			var widgets = dijit.findWidgets(this.domNode);
			dojo.forEach(widgets, function(w) {
				if(w){
					if(w.destroyRecursive){
						w.destroyRecursive();
					} else if(w.destroy){
						w.destroy();
					}
				}
			});
			// this is for widgets added as child to this widget and their children..
			//not present in template
			var widgetArr = this.getChildren();
			dojo.forEach(widgetArr, function (child) {
				if(child){
					if(child.destroyRecursive){
						child.destroyRecursive();
					} else if(child.destroy){
						child.destroy();
					}
				}
			});
			this.inherited(arguments);
		}catch(ex){
			console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
		}
	}	
});