/*
	Copyright (c) 2004-2010, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/

/*
	This is an optimized version of Dojo, built for deployment and not for
	development. To get sources and documentation, please visit:

		http://dojotoolkit.org
*/

if(!dojo._hasResource["dojox.gfx._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx._base"] = true;
dojo.provide("dojox.gfx._base");

(function(){
	var g = dojox.gfx, b = g._base;

	// candidates for dojox.style (work on VML and SVG nodes)
	g._hasClass = function(/*DomNode*/node, /*String*/classStr){
		//	summary:
		//		Returns whether or not the specified classes are a portion of the
		//		class list currently applied to the node.
		// return (new RegExp('(^|\\s+)'+classStr+'(\\s+|$)')).test(node.className)	// Boolean
		var cls = node.getAttribute("className");
		return cls && (" " + cls + " ").indexOf(" " + classStr + " ") >= 0;  // Boolean
	}
	g._addClass = function(/*DomNode*/node, /*String*/classStr){
		//	summary:
		//		Adds the specified classes to the end of the class list on the
		//		passed node.
		var cls = node.getAttribute("className") || "";
		if(!cls || (" " + cls + " ").indexOf(" " + classStr + " ") < 0){
			node.setAttribute("className", cls + (cls ? " " : "") + classStr);
		}
	}
	g._removeClass = function(/*DomNode*/node, /*String*/classStr){
		//	summary: Removes classes from node.
		var cls = node.getAttribute("className");
		if(cls){
			node.setAttribute(
				"className", 
				cls.replace(new RegExp('(^|\\s+)' + classStr + '(\\s+|$)'), "$1$2")
			);
		}
	}

	// candidate for dojox.html.metrics (dynamic font resize handler is not implemented here)

	//	derived from Morris John's emResized measurer
	b._getFontMeasurements = function(){
		//	summary:
		//		Returns an object that has pixel equivilents of standard font
		//		size values.
		var heights = {
			'1em': 0, '1ex': 0, '100%': 0, '12pt': 0, '16px': 0, 'xx-small': 0,
			'x-small': 0, 'small': 0, 'medium': 0, 'large': 0, 'x-large': 0,
			'xx-large': 0
		};

		if(dojo.isIE){
			//	we do a font-size fix if and only if one isn't applied already.
			//	NOTE: If someone set the fontSize on the HTML Element, this will kill it.
			dojo.doc.documentElement.style.fontSize="100%";
		}

		//	set up the measuring node.
		var div = dojo.doc.createElement("div");
		var s = div.style;
		s.position = "absolute";
		s.top = "-100px";
		s.left = "0px";
		s.width = "30px";
		s.height = "1000em";
		s.borderWidth = "0px";
		s.margin = "0px";
		s.padding = "0px";
		s.outline = "none";
		s.lineHeight = "1";
		s.overflow = "hidden";
		dojo.body().appendChild(div);

		//	do the measurements.
		for(var p in heights){
			div.style.fontSize = p;
			heights[p] = Math.round(div.offsetHeight * 12/16) * 16/12 / 1000;
		}

		dojo.body().removeChild(div);
		div = null;
		return heights; 	//	object
	};

	var fontMeasurements = null;

	b._getCachedFontMeasurements = function(recalculate){
		if(recalculate || !fontMeasurements){
			fontMeasurements = b._getFontMeasurements();
		}
		return fontMeasurements;
	};

	// candidate for dojox.html.metrics

	var measuringNode = null, empty = {};
	b._getTextBox = function(	/*String*/ text,
								/*Object*/ style,
								/*String?*/ className){
		var m, s, al = arguments.length;
		if(!measuringNode){
			m = measuringNode = dojo.doc.createElement("div");
			s = m.style;
			s.position = "absolute";
			s.top = "-10000px";
			s.left = "0";
			dojo.body().appendChild(m);
		}else{
			m = measuringNode;
			s = m.style;
		}
		// reset styles
		m.className = "";
		s.borderWidth = "0";
		s.margin = "0";
		s.padding = "0";
		s.outline = "0";
		// set new style
		if(al > 1 && style){
			for(var i in style){
				if(i in empty){ continue; }
				s[i] = style[i];
			}
		}
		// set classes
		if(al > 2 && className){
			m.className = className;
		}
		// take a measure
		m.innerHTML = text;

		if(m["getBoundingClientRect"]){
			var bcr = m.getBoundingClientRect();
			return {l: bcr.left, t: bcr.top, w: bcr.width || (bcr.right - bcr.left), h: bcr.height || (bcr.bottom - bcr.top)};
		}else{
			return dojo.marginBox(m);
		}
	};

	// candidate for dojo.dom

	var uniqueId = 0;
	b._getUniqueId = function(){
		// summary: returns a unique string for use with any DOM element
		var id;
		do{
			id = dojo._scopeName + "Unique" + (++uniqueId);
		}while(dojo.byId(id));
		return id;
	};
})();

dojo.mixin(dojox.gfx, {
	//	summary:
	// 		defines constants, prototypes, and utility functions

	// default shapes, which are used to fill in missing parameters
	defaultPath: {
		type: "path", path: ""
	},
	defaultPolyline: {
		type: "polyline", points: []
	},
	defaultRect: {
		type: "rect", x: 0, y: 0, width: 100, height: 100, r: 0
	},
	defaultEllipse: {
		type: "ellipse", cx: 0, cy: 0, rx: 200, ry: 100
	},
	defaultCircle: {
		type: "circle", cx: 0, cy: 0, r: 100
	},
	defaultLine: {
		type: "line", x1: 0, y1: 0, x2: 100, y2: 100
	},
	defaultImage: {
		type: "image", x: 0, y: 0, width: 0, height: 0, src: ""
	},
	defaultText: {
		type: "text", x: 0, y: 0, text: "", align: "start",
		decoration: "none", rotated: false, kerning: true
	},
	defaultTextPath: {
		type: "textpath", text: "", align: "start",
		decoration: "none", rotated: false, kerning: true
	},

	// default geometric attributes
	defaultStroke: {
		type: "stroke", color: "black", style: "solid", width: 1, 
		cap: "butt", join: 4
	},
	defaultLinearGradient: {
		type: "linear", x1: 0, y1: 0, x2: 100, y2: 100,
		colors: [
			{ offset: 0, color: "black" }, { offset: 1, color: "white" }
		]
	},
	defaultRadialGradient: {
		type: "radial", cx: 0, cy: 0, r: 100,
		colors: [
			{ offset: 0, color: "black" }, { offset: 1, color: "white" }
		]
	},
	defaultPattern: {
		type: "pattern", x: 0, y: 0, width: 0, height: 0, src: ""
	},
	defaultFont: {
		type: "font", style: "normal", variant: "normal", 
		weight: "normal", size: "10pt", family: "serif"
	},

	getDefault: (function(){
		var typeCtorCache = {};
		// a memoized delegate()
		return function(/*String*/ type){
			var t = typeCtorCache[type];
			if(t){
				return new t();
			}
			t = typeCtorCache[type] = new Function;
			t.prototype = dojox.gfx[ "default" + type ];
			return new t();
		}
	})(),

	normalizeColor: function(/*Color*/ color){
		//	summary:
		// 		converts any legal color representation to normalized
		// 		dojo.Color object
		return (color instanceof dojo.Color) ? color : new dojo.Color(color); // dojo.Color
	},
	normalizeParameters: function(existed, update){
		//	summary:
		// 		updates an existing object with properties from an "update"
		// 		object
		//	existed: Object
		//		the "target" object to be updated
		//	update:  Object
		//		the "update" object, whose properties will be used to update
		//		the existed object
		if(update){
			var empty = {};
			for(var x in existed){
				if(x in update && !(x in empty)){
					existed[x] = update[x];
				}
			}
		}
		return existed;	// Object
	},
	makeParameters: function(defaults, update){
		//	summary:
		// 		copies the original object, and all copied properties from the
		// 		"update" object
		//	defaults: Object
		//		the object to be cloned before updating
		//	update:   Object
		//		the object, which properties are to be cloned during updating
		if(!update){
			// return dojo.clone(defaults);
			return dojo.delegate(defaults);
		}
		var result = {};
		for(var i in defaults){
			if(!(i in result)){
				result[i] = dojo.clone((i in update) ? update[i] : defaults[i]);
			}
		}
		return result; // Object
	},
	formatNumber: function(x, addSpace){
		// summary: converts a number to a string using a fixed notation
		// x:			Number:		number to be converted
		// addSpace:	Boolean?:	if it is true, add a space before a positive number
		var val = x.toString();
		if(val.indexOf("e") >= 0){
			val = x.toFixed(4);
		}else{
			var point = val.indexOf(".");
			if(point >= 0 && val.length - point > 5){
				val = x.toFixed(4);
			}
		}
		if(x < 0){
			return val; // String
		}
		return addSpace ? " " + val : val; // String
	},
	// font operations
	makeFontString: function(font){
		// summary: converts a font object to a CSS font string
		// font:	Object:	font object (see dojox.gfx.defaultFont)
		return font.style + " " + font.variant + " " + font.weight + " " + font.size + " " + font.family; // Object
	},
	splitFontString: function(str){
		// summary:
		//		converts a CSS font string to a font object
		// description:
		//		Converts a CSS font string to a gfx font object. The CSS font
		//		string components should follow the W3C specified order
		//		(see http://www.w3.org/TR/CSS2/fonts.html#font-shorthand):
		//		style, variant, weight, size, optional line height (will be
		//		ignored), and family.
		// str: String
		//		a CSS font string
		var font = dojox.gfx.getDefault("Font");
		var t = str.split(/\s+/);
		do{
			if(t.length < 5){ break; }
			font.style   = t[0];
			font.variant = t[1];
			font.weight  = t[2];
			var i = t[3].indexOf("/");
			font.size = i < 0 ? t[3] : t[3].substring(0, i);
			var j = 4;
			if(i < 0){
				if(t[4] == "/"){
					j = 6;
				}else if(t[4].charAt(0) == "/"){
					j = 5;
				}
			}
			if(j < t.length){
				font.family = t.slice(j).join(" ");
			}
		}while(false);
		return font;	// Object
	},
	// length operations
	cm_in_pt: 72 / 2.54,	// Number: points per centimeter
	mm_in_pt: 7.2 / 2.54,	// Number: points per millimeter
	px_in_pt: function(){
		// summary: returns a number of pixels per point
		return dojox.gfx._base._getCachedFontMeasurements()["12pt"] / 12;	// Number
	},
	pt2px: function(len){
		// summary: converts points to pixels
		// len: Number: a value in points
		return len * dojox.gfx.px_in_pt();	// Number
	},
	px2pt: function(len){
		// summary: converts pixels to points
		// len: Number: a value in pixels
		return len / dojox.gfx.px_in_pt();	// Number
	},
	normalizedLength: function(len) {
		// summary: converts any length value to pixels
		// len: String: a length, e.g., "12pc"
		if(len.length == 0) return 0;
		if(len.length > 2){
			var px_in_pt = dojox.gfx.px_in_pt();
			var val = parseFloat(len);
			switch(len.slice(-2)){
				case "px": return val;
				case "pt": return val * px_in_pt;
				case "in": return val * 72 * px_in_pt;
				case "pc": return val * 12 * px_in_pt;
				case "mm": return val * dojox.gfx.mm_in_pt * px_in_pt;
				case "cm": return val * dojox.gfx.cm_in_pt * px_in_pt;
			}
		}
		return parseFloat(len);	// Number
	},

	// a constant used to split a SVG/VML path into primitive components
	pathVmlRegExp: /([A-Za-z]+)|(\d+(\.\d+)?)|(\.\d+)|(-\d+(\.\d+)?)|(-\.\d+)/g,
	pathSvgRegExp: /([A-Za-z])|(\d+(\.\d+)?)|(\.\d+)|(-\d+(\.\d+)?)|(-\.\d+)/g,

	equalSources: function(a, b){
		// summary: compares event sources, returns true if they are equal
		return a && b && a == b;
	}
});

}

if(!dojo._hasResource["dojox.gfx.shape"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.shape"] = true;
dojo.provide("dojox.gfx.shape");



dojo.declare("dojox.gfx.Shape", null, {
	// summary: a Shape object, which knows how to apply
	// graphical attributes and transformations

	constructor: function(){
		// rawNode: Node: underlying node
		this.rawNode = null;

		// shape: Object: an abstract shape object
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)
		this.shape = null;

		// matrix: dojox.gfx.Matrix2D: a transformation matrix
		this.matrix = null;

		// fillStyle: Object: a fill object
		//	(see dojox.gfx.defaultLinearGradient,
		//	dojox.gfx.defaultRadialGradient,
		//	dojox.gfx.defaultPattern,
		//	or dojo.Color)
		this.fillStyle = null;

		// strokeStyle: Object: a stroke object
		//	(see dojox.gfx.defaultStroke)
		this.strokeStyle = null;

		// bbox: dojox.gfx.Rectangle: a bounding box of this shape
		//	(see dojox.gfx.defaultRect)
		this.bbox = null;

		// virtual group structure

		// parent: Object: a parent or null
		//	(see dojox.gfx.Surface,
		//	dojox.gfx.shape.VirtualGroup,
		//	or dojox.gfx.Group)
		this.parent = null;

		// parentMatrix: dojox.gfx.Matrix2D
		//	a transformation matrix inherited from the parent
		this.parentMatrix = null;
	},

	// trivial getters

	getNode: function(){
		// summary: returns the current DOM Node or null
		return this.rawNode; // Node
	},
	getShape: function(){
		// summary: returns the current shape object or null
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)
		return this.shape; // Object
	},
	getTransform: function(){
		// summary: returns the current transformation matrix or null
		return this.matrix;	// dojox.gfx.Matrix2D
	},
	getFill: function(){
		// summary: returns the current fill object or null
		//	(see dojox.gfx.defaultLinearGradient,
		//	dojox.gfx.defaultRadialGradient,
		//	dojox.gfx.defaultPattern,
		//	or dojo.Color)
		return this.fillStyle;	// Object
	},
	getStroke: function(){
		// summary: returns the current stroke object or null
		//	(see dojox.gfx.defaultStroke)
		return this.strokeStyle;	// Object
	},
	getParent: function(){
		// summary: returns the parent or null
		//	(see dojox.gfx.Surface,
		//	dojox.gfx.shape.VirtualGroup,
		//	or dojox.gfx.Group)
		return this.parent;	// Object
	},
	getBoundingBox: function(){
		// summary: returns the bounding box or null
		//	(see dojox.gfx.defaultRect)
		return this.bbox;	// dojox.gfx.Rectangle
	},
	getTransformedBoundingBox: function(){
		// summary: returns an array of four points or null
		//	four points represent four corners of the untransformed bounding box
		var b = this.getBoundingBox();
		if(!b){
			return null;	// null
		}
		var m = this._getRealMatrix();
			gm = dojox.gfx.matrix;
		return [	// Array
				gm.multiplyPoint(m, b.x, b.y),
				gm.multiplyPoint(m, b.x + b.width, b.y),
				gm.multiplyPoint(m, b.x + b.width, b.y + b.height),
				gm.multiplyPoint(m, b.x, b.y + b.height)
			];
	},
	getEventSource: function(){
		// summary: returns a Node, which is used as
		//	a source of events for this shape

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		return this.rawNode;	// Node
	},

	// empty settings

	setShape: function(shape){
		// summary: sets a shape object
		//	(the default implementation simply ignores it)
		// shape: Object: a shape object
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		this.shape = dojox.gfx.makeParameters(this.shape, shape);
		this.bbox = null;
		return this;	// self
	},
	setFill: function(fill){
		// summary: sets a fill object
		//	(the default implementation simply ignores it)
		// fill: Object: a fill object
		//	(see dojox.gfx.defaultLinearGradient,
		//	dojox.gfx.defaultRadialGradient,
		//	dojox.gfx.defaultPattern,
		//	or dojo.Color)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		if(!fill){
			// don't fill
			this.fillStyle = null;
			return this;	// self
		}
		var f = null;
		if(typeof(fill) == "object" && "type" in fill){
			// gradient or pattern
			switch(fill.type){
				case "linear":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultLinearGradient, fill);
					break;
				case "radial":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultRadialGradient, fill);
					break;
				case "pattern":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultPattern, fill);
					break;
			}
		}else{
			// color object
			f = dojox.gfx.normalizeColor(fill);
		}
		this.fillStyle = f;
		return this;	// self
	},
	setStroke: function(stroke){
		// summary: sets a stroke object
		//	(the default implementation simply ignores it)
		// stroke: Object: a stroke object
		//	(see dojox.gfx.defaultStroke)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		if(!stroke){
			// don't stroke
			this.strokeStyle = null;
			return this;	// self
		}
		// normalize the stroke
		if(typeof stroke == "string" || dojo.isArray(stroke) || stroke instanceof dojo.Color){
			stroke = {color: stroke};
		}
		var s = this.strokeStyle = dojox.gfx.makeParameters(dojox.gfx.defaultStroke, stroke);
		s.color = dojox.gfx.normalizeColor(s.color);
		return this;	// self
	},
	setTransform: function(matrix){
		// summary: sets a transformation matrix
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D
		//	constructor for a list of acceptable arguments)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		this.matrix = dojox.gfx.matrix.clone(matrix ? dojox.gfx.matrix.normalize(matrix) : dojox.gfx.matrix.identity);
		return this._applyTransform();	// self
	},

	_applyTransform: function(){
		// summary: physically sets a matrix

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		return this;	// self
	},

	// z-index

	moveToFront: function(){
		// summary: moves a shape to front of its parent's list of shapes
		var p = this.getParent();
		if(p){
			p._moveChildToFront(this);
			this._moveToFront();	// execute renderer-specific action
		}
		return this;	// self
	},
	moveToBack: function(){
		// summary: moves a shape to back of its parent's list of shapes
		var p = this.getParent();
		if(p){
			p._moveChildToBack(this);
			this._moveToBack();	// execute renderer-specific action
		}
		return this;
	},
	_moveToFront: function(){
		// summary: renderer-specific hook, see dojox.gfx.shape.Shape.moveToFront()

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!
	},
	_moveToBack: function(){
		// summary: renderer-specific hook, see dojox.gfx.shape.Shape.moveToFront()

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!
	},

	// apply left & right transformation

	applyRightTransform: function(matrix){
		// summary: multiplies the existing matrix with an argument on right side
		//	(this.matrix * matrix)
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D
		//	constructor for a list of acceptable arguments)
		return matrix ? this.setTransform([this.matrix, matrix]) : this;	// self
	},
	applyLeftTransform: function(matrix){
		// summary: multiplies the existing matrix with an argument on left side
		//	(matrix * this.matrix)
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D
		//	constructor for a list of acceptable arguments)
		return matrix ? this.setTransform([matrix, this.matrix]) : this;	// self
	},
	applyTransform: function(matrix){
		// summary: a shortcut for dojox.gfx.Shape.applyRightTransform
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D
		//	constructor for a list of acceptable arguments)
		return matrix ? this.setTransform([this.matrix, matrix]) : this;	// self
	},

	// virtual group methods

	removeShape: function(silently){
		// summary: removes the shape from its parent's list of shapes
		// silently: Boolean?: if true, do not redraw a picture yet
		if(this.parent){
			this.parent.remove(this, silently);
		}
		return this;	// self
	},
	_setParent: function(parent, matrix){
		// summary: sets a parent
		// parent: Object: a parent or null
		//	(see dojox.gfx.Surface,
		//	dojox.gfx.shape.VirtualGroup,
		//	or dojox.gfx.Group)
		// matrix: dojox.gfx.Matrix2D:
		//	a 2D matrix or a matrix-like object
		this.parent = parent;
		return this._updateParentMatrix(matrix);	// self
	},
	_updateParentMatrix: function(matrix){
		// summary: updates the parent matrix with new matrix
		// matrix: dojox.gfx.Matrix2D:
		//	a 2D matrix or a matrix-like object
		this.parentMatrix = matrix ? dojox.gfx.matrix.clone(matrix) : null;
		return this._applyTransform();	// self
	},
	_getRealMatrix: function(){
		// summary: returns the cumulative ("real") transformation matrix
		//	by combining the shape's matrix with its parent's matrix
		var m = this.matrix;
		var p = this.parent;
		while(p){
			if(p.matrix){
				m = dojox.gfx.matrix.multiply(p.matrix, m);
			}
			p = p.parent;
		}
		return m;	// dojox.gfx.Matrix2D
	}
});

dojox.gfx.shape._eventsProcessing = {
	connect: function(name, object, method){
		// summary: connects a handler to an event on this shape

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		return arguments.length > 2 ?	// Object
			dojo.connect(this.getEventSource(), name, object, method) :
			dojo.connect(this.getEventSource(), name, object);
	},
	disconnect: function(token){
		// summary: connects a handler by token from an event on this shape

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		dojo.disconnect(token);
	}
};

dojo.extend(dojox.gfx.Shape, dojox.gfx.shape._eventsProcessing);

dojox.gfx.shape.Container = {
	// summary: a container of shapes, which can be used
	//	as a foundation for renderer-specific groups, or as a way
	//	to logically group shapes (e.g, to propagate matricies)

	_init: function() {
		// children: Array: a list of children
		this.children = [];
	},

	// group management

	openBatch: function() {
		// summary: starts a new batch, subsequent new child shapes will be held in
		//	the batch instead of appending to the container directly
	},
	closeBatch: function() {
		// summary: submits the current batch, append all pending child shapes to DOM
	},
	add: function(shape){
		// summary: adds a shape to the list
		// shape: dojox.gfx.Shape: a shape
		var oldParent = shape.getParent();
		if(oldParent){
			oldParent.remove(shape, true);
		}
		this.children.push(shape);
		return shape._setParent(this, this._getRealMatrix());	// self
	},
	remove: function(shape, silently){
		// summary: removes a shape from the list
		// silently: Boolean?: if true, do not redraw a picture yet
		for(var i = 0; i < this.children.length; ++i){
			if(this.children[i] == shape){
				if(silently){
					// skip for now
				}else{
					shape.parent = null;
					shape.parentMatrix = null;
				}
				this.children.splice(i, 1);
				break;
			}
		}
		return this;	// self
	},
	clear: function(){
		// summary: removes all shapes from a group/surface
		this.children = [];
		return this;	// self
	},

	// moving child nodes

	_moveChildToFront: function(shape){
		// summary: moves a shape to front of the list of shapes
		for(var i = 0; i < this.children.length; ++i){
			if(this.children[i] == shape){
				this.children.splice(i, 1);
				this.children.push(shape);
				break;
			}
		}
		return this;	// self
	},
	_moveChildToBack: function(shape){
		// summary: moves a shape to back of the list of shapes
		for(var i = 0; i < this.children.length; ++i){
			if(this.children[i] == shape){
				this.children.splice(i, 1);
				this.children.unshift(shape);
				break;
			}
		}
		return this;	// self
	}
};

dojo.declare("dojox.gfx.shape.Surface", null, {
	// summary: a surface object to be used for drawings
	constructor: function(){
		// underlying node
		this.rawNode = null;
		// the parent node
		this._parent = null;
		// the list of DOM nodes to be deleted in the case of destruction
		this._nodes = [];
		// the list of events to be detached in the case of destruction
		this._events = [];
	},
	destroy: function(){
		// summary: destroy all relevant external resources and release all
		//	external references to make this object garbage-collectible
		dojo.forEach(this._nodes, dojo.destroy);
		this._nodes = [];
		dojo.forEach(this._events, dojo.disconnect);
		this._events = [];
		this.rawNode = null;	// recycle it in _nodes, if it needs to be recycled
		if(dojo.isIE){
			while(this._parent.lastChild){
				dojo.destroy(this._parent.lastChild);
			}
		}else{
			this._parent.innerHTML = "";
		}
		this._parent = null;
	},
	getEventSource: function(){
		// summary: returns a node, which can be used to attach event listeners
		return this.rawNode; // Node
	},
	_getRealMatrix: function(){
		// summary: always returns the identity matrix
		return null;	// dojox.gfx.Matrix2D
	},
	isLoaded: true,
	onLoad: function(/*dojox.gfx.Surface*/ surface){
		// summary: local event, fired once when the surface is created
		// asynchronously, used only when isLoaded is false, required
		// only for Silverlight.
	},
	whenLoaded: function(
		/*Object?*/ context,
		/*Function|String*/ method
	){
		var f = dojo.hitch(context, method);
		if(this.isLoaded){
			f(this);
		}else{
			var h = dojo.connect(this, "onLoad", function(surface){
				dojo.disconnect(h);
				f(surface);
			});
		}
	}
});

dojo.extend(dojox.gfx.shape.Surface, dojox.gfx.shape._eventsProcessing);

dojo.declare("dojox.gfx.Point", null, {
	// summary: a hypothetical 2D point to be used for drawings - {x, y}
	// description: This object is defined for documentation purposes.
	//	You should use the naked object instead: {x: 1, y: 2}.
});

dojo.declare("dojox.gfx.Rectangle", null, {
	// summary: a hypothetical rectangle - {x, y, width, height}
	// description: This object is defined for documentation purposes.
	//	You should use the naked object instead: {x: 1, y: 2, width: 100, height: 200}.
});

dojo.declare("dojox.gfx.shape.Rect", dojox.gfx.Shape, {
	// summary: a generic rectangle
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.shape = dojox.gfx.getDefault("Rect");
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box (its shape in this case)
		return this.shape;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Ellipse", dojox.gfx.Shape, {
	// summary: a generic ellipse
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.shape = dojox.gfx.getDefault("Ellipse");
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox){
			var shape = this.shape;
			this.bbox = {x: shape.cx - shape.rx, y: shape.cy - shape.ry,
				width: 2 * shape.rx, height: 2 * shape.ry};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Circle", dojox.gfx.Shape, {
	// summary: a generic circle
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.shape = dojox.gfx.getDefault("Circle");
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox){
			var shape = this.shape;
			this.bbox = {x: shape.cx - shape.r, y: shape.cy - shape.r,
				width: 2 * shape.r, height: 2 * shape.r};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Line", dojox.gfx.Shape, {
	// summary: a generic line
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.shape = dojox.gfx.getDefault("Line");
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox){
			var shape = this.shape;
			this.bbox = {
				x:		Math.min(shape.x1, shape.x2),
				y:		Math.min(shape.y1, shape.y2),
				width:	Math.abs(shape.x2 - shape.x1),
				height:	Math.abs(shape.y2 - shape.y1)
			};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Polyline", dojox.gfx.Shape, {
	// summary: a generic polyline/polygon
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.shape = dojox.gfx.getDefault("Polyline");
		this.rawNode = rawNode;
	},
	setShape: function(points, closed){
		// summary: sets a polyline/polygon shape object
		// points: Object: a polyline/polygon shape object
		// closed: Boolean: close the polyline to make a polygon
		if(points && points instanceof Array){
			// points: Array: an array of points
			dojox.gfx.Shape.prototype.setShape.call(this, {points: points});
			if(closed && this.shape.points.length){
				this.shape.points.push(this.shape.points[0]);
			}
		}else{
			dojox.gfx.Shape.prototype.setShape.call(this, points);
		}
		return this;	// self
	},
	_normalizePoints: function(){
		// summary: normalize points to array of {x:number, y:number}
		var p = this.shape.points, l = p && p.length;
		if(l && typeof p[0] == "number"){
			var points = [];
			for(var i = 0; i < l; i += 2){
				points.push({x: p[i], y: p[i + 1]});
			}
			this.shape.points = points;
		}
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox && this.shape.points.length){
			var p = this.shape.points;
			var l = p.length;
			var t = p[0];
			var bbox = {l: t.x, t: t.y, r: t.x, b: t.y};
			for(var i = 1; i < l; ++i){
				t = p[i];
				if(bbox.l > t.x) bbox.l = t.x;
				if(bbox.r < t.x) bbox.r = t.x;
				if(bbox.t > t.y) bbox.t = t.y;
				if(bbox.b < t.y) bbox.b = t.y;
			}
			this.bbox = {
				x:		bbox.l,
				y:		bbox.t,
				width:	bbox.r - bbox.l,
				height:	bbox.b - bbox.t
			};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Image", dojox.gfx.Shape, {
	// summary: a generic image
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.shape = dojox.gfx.getDefault("Image");
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box (its shape in this case)
		return this.shape;	// dojox.gfx.Rectangle
	},
	setStroke: function(){
		// summary: ignore setting a stroke style
		return this;	// self
	},
	setFill: function(){
		// summary: ignore setting a fill style
		return this;	// self
	}
});

dojo.declare("dojox.gfx.shape.Text", dojox.gfx.Shape, {
	// summary: a generic text
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.fontStyle = null;
		this.shape = dojox.gfx.getDefault("Text");
		this.rawNode = rawNode;
	},
	getFont: function(){
		// summary: returns the current font object or null
		return this.fontStyle;	// Object
	},
	setFont: function(newFont){
		// summary: sets a font for text
		// newFont: Object: a font object (see dojox.gfx.defaultFont) or a font string
		this.fontStyle = typeof newFont == "string" ? dojox.gfx.splitFontString(newFont) :
			dojox.gfx.makeParameters(dojox.gfx.defaultFont, newFont);
		this._setFont();
		return this;	// self
	}
});

dojox.gfx.shape.Creator = {
	// summary: shape creators
	createShape: function(shape){
		// summary: creates a shape object based on its type; it is meant to be used
		//	by group-like objects
		// shape: Object: a shape descriptor object
		var gfx = dojox.gfx;
		switch(shape.type){
			case gfx.defaultPath.type:		return this.createPath(shape);
			case gfx.defaultRect.type:		return this.createRect(shape);
			case gfx.defaultCircle.type:		return this.createCircle(shape);
			case gfx.defaultEllipse.type:		return this.createEllipse(shape);
			case gfx.defaultLine.type:		return this.createLine(shape);
			case gfx.defaultPolyline.type:	return this.createPolyline(shape);
			case gfx.defaultImage.type:		return this.createImage(shape);
			case gfx.defaultText.type:		return this.createText(shape);
			case gfx.defaultTextPath.type:	return this.createTextPath(shape);
		}
		return null;
	},
	createGroup: function(){
		// summary: creates a group shape
		return this.createObject(dojox.gfx.Group);	// dojox.gfx.Group
	},
	createRect: function(rect){
		// summary: creates a rectangle shape
		// rect: Object: a path object (see dojox.gfx.defaultRect)
		return this.createObject(dojox.gfx.Rect, rect);	// dojox.gfx.Rect
	},
	createEllipse: function(ellipse){
		// summary: creates an ellipse shape
		// ellipse: Object: an ellipse object (see dojox.gfx.defaultEllipse)
		return this.createObject(dojox.gfx.Ellipse, ellipse);	// dojox.gfx.Ellipse
	},
	createCircle: function(circle){
		// summary: creates a circle shape
		// circle: Object: a circle object (see dojox.gfx.defaultCircle)
		return this.createObject(dojox.gfx.Circle, circle);	// dojox.gfx.Circle
	},
	createLine: function(line){
		// summary: creates a line shape
		// line: Object: a line object (see dojox.gfx.defaultLine)
		return this.createObject(dojox.gfx.Line, line);	// dojox.gfx.Line
	},
	createPolyline: function(points){
		// summary: creates a polyline/polygon shape
		// points: Object: a points object (see dojox.gfx.defaultPolyline)
		//	or an Array of points
		return this.createObject(dojox.gfx.Polyline, points);	// dojox.gfx.Polyline
	},
	createImage: function(image){
		// summary: creates a image shape
		// image: Object: an image object (see dojox.gfx.defaultImage)
		return this.createObject(dojox.gfx.Image, image);	// dojox.gfx.Image
	},
	createText: function(text){
		// summary: creates a text shape
		// text: Object: a text object (see dojox.gfx.defaultText)
		return this.createObject(dojox.gfx.Text, text);	// dojox.gfx.Text
	},
	createPath: function(path){
		// summary: creates a path shape
		// path: Object: a path object (see dojox.gfx.defaultPath)
		return this.createObject(dojox.gfx.Path, path);	// dojox.gfx.Path
	},
	createTextPath: function(text){
		// summary: creates a text shape
		// text: Object: a textpath object (see dojox.gfx.defaultTextPath)
		return this.createObject(dojox.gfx.TextPath, {}).setText(text);	// dojox.gfx.TextPath
	},
	createObject: function(shapeType, rawShape){
		// summary: creates an instance of the passed shapeType class
		// shapeType: Function: a class constructor to create an instance of
		// rawShape: Object: properties to be passed in to the classes "setShape" method

		// SHOULD BE RE-IMPLEMENTED BY THE RENDERER!

		return null;	// dojox.gfx.Shape
	}
};

}

if(!dojo._hasResource["dojox.gfx.matrix"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.matrix"] = true;
dojo.provide("dojox.gfx.matrix");

(function(){
	var m = dojox.gfx.matrix;

	// candidates for dojox.math:
	var _degToRadCache = {};
	m._degToRad = function(degree){
		return _degToRadCache[degree] || (_degToRadCache[degree] = (Math.PI * degree / 180));
	};
	m._radToDeg = function(radian){ return radian / Math.PI * 180; };

	m.Matrix2D = function(arg){
		// summary: a 2D matrix object
		// description: Normalizes a 2D matrix-like object. If arrays is passed,
		//		all objects of the array are normalized and multiplied sequentially.
		// arg: Object
		//		a 2D matrix-like object, a number, or an array of such objects
		if(arg){
			if(typeof arg == "number"){
				this.xx = this.yy = arg;
			}else if(arg instanceof Array){
				if(arg.length > 0){
					var matrix = m.normalize(arg[0]);
					// combine matrices
					for(var i = 1; i < arg.length; ++i){
						var l = matrix, r = dojox.gfx.matrix.normalize(arg[i]);
						matrix = new m.Matrix2D();
						matrix.xx = l.xx * r.xx + l.xy * r.yx;
						matrix.xy = l.xx * r.xy + l.xy * r.yy;
						matrix.yx = l.yx * r.xx + l.yy * r.yx;
						matrix.yy = l.yx * r.xy + l.yy * r.yy;
						matrix.dx = l.xx * r.dx + l.xy * r.dy + l.dx;
						matrix.dy = l.yx * r.dx + l.yy * r.dy + l.dy;
					}
					dojo.mixin(this, matrix);
				}
			}else{
				dojo.mixin(this, arg);
			}
		}
	};

	// the default (identity) matrix, which is used to fill in missing values
	dojo.extend(m.Matrix2D, {xx: 1, xy: 0, yx: 0, yy: 1, dx: 0, dy: 0});

	dojo.mixin(m, {
		// summary: class constants, and methods of dojox.gfx.matrix

		// matrix constants

		// identity: dojox.gfx.matrix.Matrix2D
		//		an identity matrix constant: identity * (x, y) == (x, y)
		identity: new m.Matrix2D(),

		// flipX: dojox.gfx.matrix.Matrix2D
		//		a matrix, which reflects points at x = 0 line: flipX * (x, y) == (-x, y)
		flipX:    new m.Matrix2D({xx: -1}),

		// flipY: dojox.gfx.matrix.Matrix2D
		//		a matrix, which reflects points at y = 0 line: flipY * (x, y) == (x, -y)
		flipY:    new m.Matrix2D({yy: -1}),

		// flipXY: dojox.gfx.matrix.Matrix2D
		//		a matrix, which reflects points at the origin of coordinates: flipXY * (x, y) == (-x, -y)
		flipXY:   new m.Matrix2D({xx: -1, yy: -1}),

		// matrix creators

		translate: function(a, b){
			// summary: forms a translation matrix
			// description: The resulting matrix is used to translate (move) points by specified offsets.
			// a: Number: an x coordinate value
			// b: Number: a y coordinate value
			if(arguments.length > 1){
				return new m.Matrix2D({dx: a, dy: b}); // dojox.gfx.matrix.Matrix2D
			}
			// branch
			// a: dojox.gfx.Point: a point-like object, which specifies offsets for both dimensions
			// b: null
			return new m.Matrix2D({dx: a.x, dy: a.y}); // dojox.gfx.matrix.Matrix2D
		},
		scale: function(a, b){
			// summary: forms a scaling matrix
			// description: The resulting matrix is used to scale (magnify) points by specified offsets.
			// a: Number: a scaling factor used for the x coordinate
			// b: Number: a scaling factor used for the y coordinate
			if(arguments.length > 1){
				return new m.Matrix2D({xx: a, yy: b}); // dojox.gfx.matrix.Matrix2D
			}
			if(typeof a == "number"){
				// branch
				// a: Number: a uniform scaling factor used for the both coordinates
				// b: null
				return new m.Matrix2D({xx: a, yy: a}); // dojox.gfx.matrix.Matrix2D
			}
			// branch
			// a: dojox.gfx.Point: a point-like object, which specifies scale factors for both dimensions
			// b: null
			return new m.Matrix2D({xx: a.x, yy: a.y}); // dojox.gfx.matrix.Matrix2D
		},
		rotate: function(angle){
			// summary: forms a rotating matrix
			// description: The resulting matrix is used to rotate points
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number: an angle of rotation in radians (>0 for CW)
			var c = Math.cos(angle);
			var s = Math.sin(angle);
			return new m.Matrix2D({xx: c, xy: -s, yx: s, yy: c}); // dojox.gfx.matrix.Matrix2D
		},
		rotateg: function(degree){
			// summary: forms a rotating matrix
			// description: The resulting matrix is used to rotate points
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox.gfx.matrix.rotate() for comparison.
			// degree: Number: an angle of rotation in degrees (>0 for CW)
			return m.rotate(m._degToRad(degree)); // dojox.gfx.matrix.Matrix2D
		},
		skewX: function(angle) {
			// summary: forms an x skewing matrix
			// description: The resulting matrix is used to skew points in the x dimension
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number: an skewing angle in radians
			return new m.Matrix2D({xy: Math.tan(angle)}); // dojox.gfx.matrix.Matrix2D
		},
		skewXg: function(degree){
			// summary: forms an x skewing matrix
			// description: The resulting matrix is used to skew points in the x dimension
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox.gfx.matrix.skewX() for comparison.
			// degree: Number: an skewing angle in degrees
			return m.skewX(m._degToRad(degree)); // dojox.gfx.matrix.Matrix2D
		},
		skewY: function(angle){
			// summary: forms a y skewing matrix
			// description: The resulting matrix is used to skew points in the y dimension
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number: an skewing angle in radians
			return new m.Matrix2D({yx: Math.tan(angle)}); // dojox.gfx.matrix.Matrix2D
		},
		skewYg: function(degree){
			// summary: forms a y skewing matrix
			// description: The resulting matrix is used to skew points in the y dimension
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox.gfx.matrix.skewY() for comparison.
			// degree: Number: an skewing angle in degrees
			return m.skewY(m._degToRad(degree)); // dojox.gfx.matrix.Matrix2D
		},
		reflect: function(a, b){
			// summary: forms a reflection matrix
			// description: The resulting matrix is used to reflect points around a vector,
			//		which goes through the origin.
			// a: dojox.gfx.Point: a point-like object, which specifies a vector of reflection
			// b: null
			if(arguments.length == 1){
				b = a.y;
				a = a.x;
			}
			// branch
			// a: Number: an x coordinate value
			// b: Number: a y coordinate value

			// make a unit vector
			var a2 = a * a, b2 = b * b, n2 = a2 + b2, xy = 2 * a * b / n2;
			return new m.Matrix2D({xx: 2 * a2 / n2 - 1, xy: xy, yx: xy, yy: 2 * b2 / n2 - 1}); // dojox.gfx.matrix.Matrix2D
		},
		project: function(a, b){
			// summary: forms an orthogonal projection matrix
			// description: The resulting matrix is used to project points orthogonally on a vector,
			//		which goes through the origin.
			// a: dojox.gfx.Point: a point-like object, which specifies a vector of projection
			// b: null
			if(arguments.length == 1){
				b = a.y;
				a = a.x;
			}
			// branch
			// a: Number: an x coordinate value
			// b: Number: a y coordinate value

			// make a unit vector
			var a2 = a * a, b2 = b * b, n2 = a2 + b2, xy = a * b / n2;
			return new m.Matrix2D({xx: a2 / n2, xy: xy, yx: xy, yy: b2 / n2}); // dojox.gfx.matrix.Matrix2D
		},

		// ensure matrix 2D conformance
		normalize: function(matrix){
			// summary: converts an object to a matrix, if necessary
			// description: Converts any 2D matrix-like object or an array of
			//		such objects to a valid dojox.gfx.matrix.Matrix2D object.
			// matrix: Object: an object, which is converted to a matrix, if necessary
			return (matrix instanceof m.Matrix2D) ? matrix : new m.Matrix2D(matrix); // dojox.gfx.matrix.Matrix2D
		},

		// common operations

		clone: function(matrix){
			// summary: creates a copy of a 2D matrix
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix-like object to be cloned
			var obj = new m.Matrix2D();
			for(var i in matrix){
				if(typeof(matrix[i]) == "number" && typeof(obj[i]) == "number" && obj[i] != matrix[i]) obj[i] = matrix[i];
			}
			return obj; // dojox.gfx.matrix.Matrix2D
		},
		invert: function(matrix){
			// summary: inverts a 2D matrix
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix-like object to be inverted
			var M = m.normalize(matrix),
				D = M.xx * M.yy - M.xy * M.yx,
				M = new m.Matrix2D({
					xx: M.yy/D, xy: -M.xy/D,
					yx: -M.yx/D, yy: M.xx/D,
					dx: (M.xy * M.dy - M.yy * M.dx) / D,
					dy: (M.yx * M.dx - M.xx * M.dy) / D
				});
			return M; // dojox.gfx.matrix.Matrix2D
		},
		_multiplyPoint: function(matrix, x, y){
			// summary: applies a matrix to a point
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix object to be applied
			// x: Number: an x coordinate of a point
			// y: Number: a y coordinate of a point
			return {x: matrix.xx * x + matrix.xy * y + matrix.dx, y: matrix.yx * x + matrix.yy * y + matrix.dy}; // dojox.gfx.Point
		},
		multiplyPoint: function(matrix, /* Number||Point */ a, /* Number, optional */ b){
			// summary: applies a matrix to a point
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix object to be applied
			// a: Number: an x coordinate of a point
			// b: Number: a y coordinate of a point
			var M = m.normalize(matrix);
			if(typeof a == "number" && typeof b == "number"){
				return m._multiplyPoint(M, a, b); // dojox.gfx.Point
			}
			// branch
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix object to be applied
			// a: dojox.gfx.Point: a point
			// b: null
			return m._multiplyPoint(M, a.x, a.y); // dojox.gfx.Point
		},
		multiply: function(matrix){
			// summary: combines matrices by multiplying them sequentially in the given order
			// matrix: dojox.gfx.matrix.Matrix2D...: a 2D matrix-like object,
			//		all subsequent arguments are matrix-like objects too
			var M = m.normalize(matrix);
			// combine matrices
			for(var i = 1; i < arguments.length; ++i){
				var l = M, r = m.normalize(arguments[i]);
				M = new m.Matrix2D();
				M.xx = l.xx * r.xx + l.xy * r.yx;
				M.xy = l.xx * r.xy + l.xy * r.yy;
				M.yx = l.yx * r.xx + l.yy * r.yx;
				M.yy = l.yx * r.xy + l.yy * r.yy;
				M.dx = l.xx * r.dx + l.xy * r.dy + l.dx;
				M.dy = l.yx * r.dx + l.yy * r.dy + l.dy;
			}
			return M; // dojox.gfx.matrix.Matrix2D
		},

		// high level operations

		_sandwich: function(matrix, x, y){
			// summary: applies a matrix at a centrtal point
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix-like object, which is applied at a central point
			// x: Number: an x component of the central point
			// y: Number: a y component of the central point
			return m.multiply(m.translate(x, y), matrix, m.translate(-x, -y)); // dojox.gfx.matrix.Matrix2D
		},
		scaleAt: function(a, b, c, d){
			// summary: scales a picture using a specified point as a center of scaling
			// description: Compare with dojox.gfx.matrix.scale().
			// a: Number: a scaling factor used for the x coordinate
			// b: Number: a scaling factor used for the y coordinate
			// c: Number: an x component of a central point
			// d: Number: a y component of a central point

			// accepts several signatures:
			//	1) uniform scale factor, Point
			//	2) uniform scale factor, x, y
			//	3) x scale, y scale, Point
			//	4) x scale, y scale, x, y

			switch(arguments.length){
				case 4:
					// a and b are scale factor components, c and d are components of a point
					return m._sandwich(m.scale(a, b), c, d); // dojox.gfx.matrix.Matrix2D
				case 3:
					if(typeof c == "number"){
						// branch
						// a: Number: a uniform scaling factor used for both coordinates
						// b: Number: an x component of a central point
						// c: Number: a y component of a central point
						// d: null
						return m._sandwich(m.scale(a), b, c); // dojox.gfx.matrix.Matrix2D
					}
					// branch
					// a: Number: a scaling factor used for the x coordinate
					// b: Number: a scaling factor used for the y coordinate
					// c: dojox.gfx.Point: a central point
					// d: null
					return m._sandwich(m.scale(a, b), c.x, c.y); // dojox.gfx.matrix.Matrix2D
			}
			// branch
			// a: Number: a uniform scaling factor used for both coordinates
			// b: dojox.gfx.Point: a central point
			// c: null
			// d: null
			return m._sandwich(m.scale(a), b.x, b.y); // dojox.gfx.matrix.Matrix2D
		},
		rotateAt: function(angle, a, b){
			// summary: rotates a picture using a specified point as a center of rotation
			// description: Compare with dojox.gfx.matrix.rotate().
			// angle: Number: an angle of rotation in radians (>0 for CW)
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point

			// accepts several signatures:
			//	1) rotation angle in radians, Point
			//	2) rotation angle in radians, x, y

			if(arguments.length > 2){
				return m._sandwich(m.rotate(angle), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// angle: Number: an angle of rotation in radians (>0 for CCW)
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.rotate(angle), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		rotategAt: function(degree, a, b){
			// summary: rotates a picture using a specified point as a center of rotation
			// description: Compare with dojox.gfx.matrix.rotateg().
			// degree: Number: an angle of rotation in degrees (>0 for CW)
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point

			// accepts several signatures:
			//	1) rotation angle in degrees, Point
			//	2) rotation angle in degrees, x, y

			if(arguments.length > 2){
				return m._sandwich(m.rotateg(degree), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// degree: Number: an angle of rotation in degrees (>0 for CCW)
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.rotateg(degree), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewXAt: function(angle, a, b){
			// summary: skews a picture along the x axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewX().
			// angle: Number: an skewing angle in radians
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point

			// accepts several signatures:
			//	1) skew angle in radians, Point
			//	2) skew angle in radians, x, y

			if(arguments.length > 2){
				return m._sandwich(m.skewX(angle), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// angle: Number: an skewing angle in radians
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewX(angle), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewXgAt: function(degree, a, b){
			// summary: skews a picture along the x axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewXg().
			// degree: Number: an skewing angle in degrees
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point

			// accepts several signatures:
			//	1) skew angle in degrees, Point
			//	2) skew angle in degrees, x, y

			if(arguments.length > 2){
				return m._sandwich(m.skewXg(degree), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// degree: Number: an skewing angle in degrees
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewXg(degree), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewYAt: function(angle, a, b){
			// summary: skews a picture along the y axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewY().
			// angle: Number: an skewing angle in radians
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point

			// accepts several signatures:
			//	1) skew angle in radians, Point
			//	2) skew angle in radians, x, y

			if(arguments.length > 2){
				return m._sandwich(m.skewY(angle), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// angle: Number: an skewing angle in radians
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewY(angle), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewYgAt: function(/* Number */ degree, /* Number||Point */ a, /* Number, optional */ b){
			// summary: skews a picture along the y axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewYg().
			// degree: Number: an skewing angle in degrees
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point

			// accepts several signatures:
			//	1) skew angle in degrees, Point
			//	2) skew angle in degrees, x, y

			if(arguments.length > 2){
				return m._sandwich(m.skewYg(degree), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// degree: Number: an skewing angle in degrees
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewYg(degree), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		}

		//TODO: rect-to-rect mapping, scale-to-fit (isotropic and anisotropic versions)

	});
})();

// propagate Matrix2D up
dojox.gfx.Matrix2D = dojox.gfx.matrix.Matrix2D;

}

if(!dojo._hasResource["dojox.gfx.path"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.path"] = true;
dojo.provide("dojox.gfx.path");




dojo.declare("dojox.gfx.path.Path", dojox.gfx.Shape, {
	// summary: a generalized path shape

	constructor: function(rawNode){
		// summary: a path constructor
		// rawNode: Node: a DOM node to be used by this path object
		this.shape = dojo.clone(dojox.gfx.defaultPath);
		this.segments = [];
		this.tbbox = null;
		this.absolute = true;
		this.last = {};
		this.rawNode = rawNode;
		this.segmented = false;
	},

	// mode manipulations
	setAbsoluteMode: function(mode){
		// summary: sets an absolute or relative mode for path points
		// mode: Boolean: true/false or "absolute"/"relative" to specify the mode
		this._confirmSegmented();
		this.absolute = typeof mode == "string" ? (mode == "absolute") : mode;
		return this; // self
	},
	getAbsoluteMode: function(){
		// summary: returns a current value of the absolute mode
		this._confirmSegmented();
		return this.absolute; // Boolean
	},

	getBoundingBox: function(){
		// summary: returns the bounding box {x, y, width, height} or null
		this._confirmSegmented();
		return (this.bbox && ("l" in this.bbox)) ? {x: this.bbox.l, y: this.bbox.t, width: this.bbox.r - this.bbox.l, height: this.bbox.b - this.bbox.t} : null; // dojox.gfx.Rectangle
	},

	_getRealBBox: function(){
		// summary: returns an array of four points or null
		//	four points represent four corners of the untransformed bounding box
		this._confirmSegmented();
		if(this.tbbox){
			return this.tbbox;	// Array
		}
		var bbox = this.bbox, matrix = this._getRealMatrix();
		this.bbox = null;
		for(var i = 0, len = this.segments.length; i < len; ++i){
			this._updateWithSegment(this.segments[i], matrix);
		}
		var t = this.bbox;
		this.bbox = bbox;
		this.tbbox = t ? [
			{x: t.l, y: t.t},
			{x: t.r, y: t.t},
			{x: t.r, y: t.b},
			{x: t.l, y: t.b}
		] : null;
		return this.tbbox;	// Array
	},

	getLastPosition: function(){
		// summary: returns the last point in the path, or null
		this._confirmSegmented();
		return "x" in this.last ? this.last : null; // Object
	},

	_applyTransform: function(){
		this.tbbox = null;
		return dojox.gfx.Shape.prototype._applyTransform.call(this);
	},

	// segment interpretation
	_updateBBox: function(x, y, matrix){
		// summary: updates the bounding box of path with new point
		// x: Number: an x coordinate
		// y: Number: a y coordinate
		
		if(matrix){
			var t = dojox.gfx.matrix.multiplyPoint(matrix, x, y);
			x = t.x;
			y = t.y;
		}

		// we use {l, b, r, t} representation of a bbox
		if(this.bbox && ("l" in this.bbox)){
			if(this.bbox.l > x) this.bbox.l = x;
			if(this.bbox.r < x) this.bbox.r = x;
			if(this.bbox.t > y) this.bbox.t = y;
			if(this.bbox.b < y) this.bbox.b = y;
		}else{
			this.bbox = {l: x, b: y, r: x, t: y};
		}
	},
	_updateWithSegment: function(segment, matrix){
		// summary: updates the bounding box of path with new segment
		// segment: Object: a segment
		var n = segment.args, l = n.length;
		// update internal variables: bbox, absolute, last
		switch(segment.action){
			case "M":
			case "L":
			case "C":
			case "S":
			case "Q":
			case "T":
				for(var i = 0; i < l; i += 2){
					this._updateBBox(n[i], n[i + 1], matrix);
				}
				this.last.x = n[l - 2];
				this.last.y = n[l - 1];
				this.absolute = true;
				break;
			case "H":
				for(var i = 0; i < l; ++i){
					this._updateBBox(n[i], this.last.y, matrix);
				}
				this.last.x = n[l - 1];
				this.absolute = true;
				break;
			case "V":
				for(var i = 0; i < l; ++i){
					this._updateBBox(this.last.x, n[i], matrix);
				}
				this.last.y = n[l - 1];
				this.absolute = true;
				break;
			case "m":
				var start = 0;
				if(!("x" in this.last)){
					this._updateBBox(this.last.x = n[0], this.last.y = n[1], matrix);
					start = 2;
				}
				for(var i = start; i < l; i += 2){
					this._updateBBox(this.last.x += n[i], this.last.y += n[i + 1], matrix);
				}
				this.absolute = false;
				break;
			case "l":
			case "t":
				for(var i = 0; i < l; i += 2){
					this._updateBBox(this.last.x += n[i], this.last.y += n[i + 1], matrix);
				}
				this.absolute = false;
				break;
			case "h":
				for(var i = 0; i < l; ++i){
					this._updateBBox(this.last.x += n[i], this.last.y, matrix);
				}
				this.absolute = false;
				break;
			case "v":
				for(var i = 0; i < l; ++i){
					this._updateBBox(this.last.x, this.last.y += n[i], matrix);
				}
				this.absolute = false;
				break;
			case "c":
				for(var i = 0; i < l; i += 6){
					this._updateBBox(this.last.x + n[i], this.last.y + n[i + 1], matrix);
					this._updateBBox(this.last.x + n[i + 2], this.last.y + n[i + 3], matrix);
					this._updateBBox(this.last.x += n[i + 4], this.last.y += n[i + 5], matrix);
				}
				this.absolute = false;
				break;
			case "s":
			case "q":
				for(var i = 0; i < l; i += 4){
					this._updateBBox(this.last.x + n[i], this.last.y + n[i + 1], matrix);
					this._updateBBox(this.last.x += n[i + 2], this.last.y += n[i + 3], matrix);
				}
				this.absolute = false;
				break;
			case "A":
				for(var i = 0; i < l; i += 7){
					this._updateBBox(n[i + 5], n[i + 6], matrix);
				}
				this.last.x = n[l - 2];
				this.last.y = n[l - 1];
				this.absolute = true;
				break;
			case "a":
				for(var i = 0; i < l; i += 7){
					this._updateBBox(this.last.x += n[i + 5], this.last.y += n[i + 6], matrix);
				}
				this.absolute = false;
				break;
		}
		// add an SVG path segment
		var path = [segment.action];
		for(var i = 0; i < l; ++i){
			path.push(dojox.gfx.formatNumber(n[i], true));
		}
		if(typeof this.shape.path == "string"){
			this.shape.path += path.join("");
		}else{
			Array.prototype.push.apply(this.shape.path, path);
		}
	},

	// a dictionary, which maps segment type codes to a number of their arguments
	_validSegments: {m: 2, l: 2, h: 1, v: 1, c: 6, s: 4, q: 4, t: 2, a: 7, z: 0},

	_pushSegment: function(action, args){
		// summary: adds a segment
		// action: String: valid SVG code for a segment's type
		// args: Array: a list of parameters for this segment
		this.tbbox = null;
		var group = this._validSegments[action.toLowerCase()];
		if(typeof group == "number"){
			if(group){
				if(args.length >= group){
					var segment = {action: action, args: args.slice(0, args.length - args.length % group)};
					this.segments.push(segment);
					this._updateWithSegment(segment);
				}
			}else{
				var segment = {action: action, args: []};
				this.segments.push(segment);
				this._updateWithSegment(segment);
			}
		}
	},

	_collectArgs: function(array, args){
		// summary: converts an array of arguments to plain numeric values
		// array: Array: an output argument (array of numbers)
		// args: Array: an input argument (can be values of Boolean, Number, dojox.gfx.Point, or an embedded array of them)
		for(var i = 0; i < args.length; ++i){
			var t = args[i];
			if(typeof t == "boolean"){
				array.push(t ? 1 : 0);
			}else if(typeof t == "number"){
				array.push(t);
			}else if(t instanceof Array){
				this._collectArgs(array, t);
			}else if("x" in t && "y" in t){
				array.push(t.x, t.y);
			}
		}
	},

	// segments
	moveTo: function(){
		// summary: formes a move segment
		this._confirmSegmented();
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "M" : "m", args);
		return this; // self
	},
	lineTo: function(){
		// summary: formes a line segment
		this._confirmSegmented();
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "L" : "l", args);
		return this; // self
	},
	hLineTo: function(){
		// summary: formes a horizontal line segment
		this._confirmSegmented();
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "H" : "h", args);
		return this; // self
	},
	vLineTo: function(){
		// summary: formes a vertical line segment
		this._confirmSegmented();
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "V" : "v", args);
		return this; // self
	},
	curveTo: function(){
		// summary: formes a curve segment
		this._confirmSegmented();
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "C" : "c", args);
		return this; // self
	},
	smoothCurveTo: function(){
		// summary: formes a smooth curve segment
		this._confirmSegmented();
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "S" : "s", args);
		return this; // self
	},
	qCurveTo: function(){
		// summary: formes a quadratic curve segment
		this._confirmSegmented();
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "Q" : "q", args);
		return this; // self
	},
	qSmoothCurveTo: function(){
		// summary: formes a quadratic smooth curve segment
		this._confirmSegmented();
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "T" : "t", args);
		return this; // self
	},
	arcTo: function(){
		// summary: formes an elliptic arc segment
		this._confirmSegmented();
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "A" : "a", args);
		return this; // self
	},
	closePath: function(){
		// summary: closes a path
		this._confirmSegmented();
		this._pushSegment("Z", []);
		return this; // self
	},

	_confirmSegmented: function() {
		if (!this.segmented) {
			var path = this.shape.path;
			// switch to non-updating version of path building
			this.shape.path = [];
			this._setPath(path);
			// switch back to the string path
			this.shape.path = this.shape.path.join("");
			// become segmented
			this.segmented = true;
		}
	},

	// setShape
	_setPath: function(path){
		// summary: forms a path using an SVG path string
		// path: String: an SVG path string
		var p = dojo.isArray(path) ? path : path.match(dojox.gfx.pathSvgRegExp);
		this.segments = [];
		this.absolute = true;
		this.bbox = {};
		this.last = {};
		if(!p) return;
		// create segments
		var action = "",	// current action
			args = [],		// current arguments
			l = p.length;
		for(var i = 0; i < l; ++i){
			var t = p[i], x = parseFloat(t);
			if(isNaN(x)){
				if(action){
					this._pushSegment(action, args);
				}
				args = [];
				action = t;
			}else{
				args.push(x);
			}
		}
		this._pushSegment(action, args);
	},
	setShape: function(newShape){
		// summary: forms a path using a shape
		// newShape: Object: an SVG path string or a path object (see dojox.gfx.defaultPath)
		dojox.gfx.Shape.prototype.setShape.call(this, typeof newShape == "string" ? {path: newShape} : newShape);
		
		this.segmented = false;
		this.segments = [];
		if(!dojox.gfx.lazyPathSegmentation){
			this._confirmSegmented();
		}
		return this; // self
	},

	// useful constant for descendants
	_2PI: Math.PI * 2
});

dojo.declare("dojox.gfx.path.TextPath", dojox.gfx.path.Path, {
	// summary: a generalized TextPath shape

	constructor: function(rawNode){
		// summary: a TextPath shape constructor
		// rawNode: Node: a DOM node to be used by this TextPath object
		if(!("text" in this)){
			this.text = dojo.clone(dojox.gfx.defaultTextPath);
		}
		if(!("fontStyle" in this)){
			this.fontStyle = dojo.clone(dojox.gfx.defaultFont);
		}
	},
	getText: function(){
		// summary: returns the current text object or null
		return this.text;	// Object
	},
	setText: function(newText){
		// summary: sets a text to be drawn along the path
		this.text = dojox.gfx.makeParameters(this.text,
			typeof newText == "string" ? {text: newText} : newText);
		this._setText();
		return this;	// self
	},
	getFont: function(){
		// summary: returns the current font object or null
		return this.fontStyle;	// Object
	},
	setFont: function(newFont){
		// summary: sets a font for text
		this.fontStyle = typeof newFont == "string" ?
			dojox.gfx.splitFontString(newFont) :
			dojox.gfx.makeParameters(dojox.gfx.defaultFont, newFont);
		this._setFont();
		return this;	// self
	}
});

}

if(!dojo._hasResource["dojox.gfx.svg"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.svg"] = true;
dojo.provide("dojox.gfx.svg");





(function(){
	var d = dojo, g = dojox.gfx, gs = g.shape, svg = g.svg;
	svg.useSvgWeb = (typeof(window.svgweb)!=='undefined');
	
	var _createElementNS = function(ns, nodeType){
		// summary:
		//		Internal helper to deal with creating elements that
		//		are namespaced.  Mainly to get SVG markup output
		//		working on IE.
		if(dojo.doc.createElementNS){
			return dojo.doc.createElementNS(ns,nodeType);
		}else{
			return dojo.doc.createElement(nodeType);
		}
	}
	
	var _createTextNode = function(text) {
		if (svg.useSvgWeb) {
			return dojo.doc.createTextNode(text, true);
		} else {
			return dojo.doc.createTextNode(text);
		}
	}
	
	var _createFragment = function() {
		if (svg.useSvgWeb) {
			return dojo.doc.createDocumentFragment(true);
		} else {
			return dojo.doc.createDocumentFragment();
		}
	}

	svg.xmlns = {
		xlink: "http://www.w3.org/1999/xlink",
		svg:   "http://www.w3.org/2000/svg"
	};

	svg.getRef = function(name){
		// summary: returns a DOM Node specified by the name argument or null
		// name: String: an SVG external reference
		if(!name || name == "none") return null;
		if(name.match(/^url\(#.+\)$/)){
			return d.byId(name.slice(5, -1));	// Node
		}
		// alternative representation of a reference
		if(name.match(/^#dojoUnique\d+$/)){
			// we assume here that a reference was generated by dojox.gfx
			return d.byId(name.slice(1));	// Node
		}
		return null;	// Node
	};

	svg.dasharray = {
		solid:				"none",
		shortdash:			[4, 1],
		shortdot:			[1, 1],
		shortdashdot:		[4, 1, 1, 1],
		shortdashdotdot:	[4, 1, 1, 1, 1, 1],
		dot:				[1, 3],
		dash:				[4, 3],
		longdash:			[8, 3],
		dashdot:			[4, 3, 1, 3],
		longdashdot:		[8, 3, 1, 3],
		longdashdotdot:		[8, 3, 1, 3, 1, 3]
	};

	d.extend(g.Shape, {
		// summary: SVG-specific implementation of dojox.gfx.Shape methods

		setFill: function(fill){
			// summary: sets a fill object (SVG)
			// fill: Object: a fill object
			//	(see dojox.gfx.defaultLinearGradient,
			//	dojox.gfx.defaultRadialGradient,
			//	dojox.gfx.defaultPattern,
			//	or dojo.Color)

			if(!fill){
				// don't fill
				this.fillStyle = null;
				this.rawNode.setAttribute("fill", "none");
				this.rawNode.setAttribute("fill-opacity", 0);
				return this;
			}
			var f;
			// FIXME: slightly magical. We're using the outer scope's "f", but setting it later
			var setter = function(x){
					// we assume that we're executing in the scope of the node to mutate
					this.setAttribute(x, f[x].toFixed(8));
				};
			if(typeof(fill) == "object" && "type" in fill){
				// gradient
				switch(fill.type){
					case "linear":
						f = g.makeParameters(g.defaultLinearGradient, fill);
						var gradient = this._setFillObject(f, "linearGradient");
						d.forEach(["x1", "y1", "x2", "y2"], setter, gradient);
						break;
					case "radial":
						f = g.makeParameters(g.defaultRadialGradient, fill);
						var gradient = this._setFillObject(f, "radialGradient");
						d.forEach(["cx", "cy", "r"], setter, gradient);
						break;
					case "pattern":
						f = g.makeParameters(g.defaultPattern, fill);
						var pattern = this._setFillObject(f, "pattern");
						d.forEach(["x", "y", "width", "height"], setter, pattern);
						break;
				}
				this.fillStyle = f;
				return this;
			}
			// color object
			var f = g.normalizeColor(fill);
			this.fillStyle = f;
			this.rawNode.setAttribute("fill", f.toCss());
			this.rawNode.setAttribute("fill-opacity", f.a);
			this.rawNode.setAttribute("fill-rule", "evenodd");
			return this;	// self
		},

		setStroke: function(stroke){
			//	summary:
			//		sets a stroke object (SVG)
			//	stroke: Object
			// 		a stroke object (see dojox.gfx.defaultStroke)

			var rn = this.rawNode;
			if(!stroke){
				// don't stroke
				this.strokeStyle = null;
				rn.setAttribute("stroke", "none");
				rn.setAttribute("stroke-opacity", 0);
				return this;
			}
			// normalize the stroke
			if(typeof stroke == "string" || d.isArray(stroke) || stroke instanceof d.Color){
				stroke = { color: stroke };
			}
			var s = this.strokeStyle = g.makeParameters(g.defaultStroke, stroke);
			s.color = g.normalizeColor(s.color);
			// generate attributes
			if(s){
				rn.setAttribute("stroke", s.color.toCss());
				rn.setAttribute("stroke-opacity", s.color.a);
				rn.setAttribute("stroke-width",   s.width);
				rn.setAttribute("stroke-linecap", s.cap);
				if(typeof s.join == "number"){
					rn.setAttribute("stroke-linejoin",   "miter");
					rn.setAttribute("stroke-miterlimit", s.join);
				}else{
					rn.setAttribute("stroke-linejoin",   s.join);
				}
				var da = s.style.toLowerCase();
				if(da in svg.dasharray){
					da = svg.dasharray[da];
				}
				if(da instanceof Array){
					da = d._toArray(da);
					for(var i = 0; i < da.length; ++i){
						da[i] *= s.width;
					}
					if(s.cap != "butt"){
						for(var i = 0; i < da.length; i += 2){
							da[i] -= s.width;
							if(da[i] < 1){ da[i] = 1; }
						}
						for(var i = 1; i < da.length; i += 2){
							da[i] += s.width;
						}
					}
					da = da.join(",");
				}
				rn.setAttribute("stroke-dasharray", da);
				rn.setAttribute("dojoGfxStrokeStyle", s.style);
			}
			return this;	// self
		},

		_getParentSurface: function(){
			var surface = this.parent;
			for(; surface && !(surface instanceof g.Surface); surface = surface.parent);
			return surface;
		},

		_setFillObject: function(f, nodeType){
			var svgns = svg.xmlns.svg;
			this.fillStyle = f;
			var surface = this._getParentSurface(),
				defs = surface.defNode,
				fill = this.rawNode.getAttribute("fill"),
				ref  = svg.getRef(fill);
			if(ref){
				fill = ref;
				if(fill.tagName.toLowerCase() != nodeType.toLowerCase()){
					var id = fill.id;
					fill.parentNode.removeChild(fill);
					fill = _createElementNS(svgns, nodeType);
					fill.setAttribute("id", id);
					defs.appendChild(fill);
				}else{
					while(fill.childNodes.length){
						fill.removeChild(fill.lastChild);
					}
				}
			}else{
				fill = _createElementNS(svgns, nodeType);
				fill.setAttribute("id", g._base._getUniqueId());
				defs.appendChild(fill);
			}
			if(nodeType == "pattern"){
				fill.setAttribute("patternUnits", "userSpaceOnUse");
				var img = _createElementNS(svgns, "image");
				img.setAttribute("x", 0);
				img.setAttribute("y", 0);
				img.setAttribute("width",  f.width .toFixed(8));
				img.setAttribute("height", f.height.toFixed(8));
				img.setAttributeNS(svg.xmlns.xlink, "xlink:href", f.src);
				fill.appendChild(img);
			}else{
				fill.setAttribute("gradientUnits", "userSpaceOnUse");
				for(var i = 0; i < f.colors.length; ++i){
					var c = f.colors[i], t = _createElementNS(svgns, "stop"),
						cc = c.color = g.normalizeColor(c.color);
					t.setAttribute("offset",       c.offset.toFixed(8));
					t.setAttribute("stop-color",   cc.toCss());
					t.setAttribute("stop-opacity", cc.a);
					fill.appendChild(t);
				}
			}
			this.rawNode.setAttribute("fill", "url(#" + fill.getAttribute("id") +")");
			this.rawNode.removeAttribute("fill-opacity");
			this.rawNode.setAttribute("fill-rule", "evenodd");
			return fill;
		},

		_applyTransform: function() {
			var matrix = this.matrix;
			if(matrix){
				var tm = this.matrix;
				this.rawNode.setAttribute("transform", "matrix(" +
					tm.xx.toFixed(8) + "," + tm.yx.toFixed(8) + "," +
					tm.xy.toFixed(8) + "," + tm.yy.toFixed(8) + "," +
					tm.dx.toFixed(8) + "," + tm.dy.toFixed(8) + ")");
			}else{
				this.rawNode.removeAttribute("transform");
			}
			return this;
		},

		setRawNode: function(rawNode){
			// summary:
			//	assigns and clears the underlying node that will represent this
			//	shape. Once set, transforms, gradients, etc, can be applied.
			//	(no fill & stroke by default)
			var r = this.rawNode = rawNode;
			if(this.shape.type!="image"){
				r.setAttribute("fill", "none");
			}
			r.setAttribute("fill-opacity", 0);
			r.setAttribute("stroke", "none");
			r.setAttribute("stroke-opacity", 0);
			r.setAttribute("stroke-width", 1);
			r.setAttribute("stroke-linecap", "butt");
			r.setAttribute("stroke-linejoin", "miter");
			r.setAttribute("stroke-miterlimit", 4);
		},

		setShape: function(newShape){
			// summary: sets a shape object (SVG)
			// newShape: Object: a shape object
			//	(see dojox.gfx.defaultPath,
			//	dojox.gfx.defaultPolyline,
			//	dojox.gfx.defaultRect,
			//	dojox.gfx.defaultEllipse,
			//	dojox.gfx.defaultCircle,
			//	dojox.gfx.defaultLine,
			//	or dojox.gfx.defaultImage)
			this.shape = g.makeParameters(this.shape, newShape);
			for(var i in this.shape){
				if(i != "type"){ this.rawNode.setAttribute(i, this.shape[i]); }
			}
			this.bbox = null;
			return this;	// self
		},

		// move family

		_moveToFront: function(){
			// summary: moves a shape to front of its parent's list of shapes (SVG)
			this.rawNode.parentNode.appendChild(this.rawNode);
			return this;	// self
		},
		_moveToBack: function(){
			// summary: moves a shape to back of its parent's list of shapes (SVG)
			this.rawNode.parentNode.insertBefore(this.rawNode, this.rawNode.parentNode.firstChild);
			return this;	// self
		}
	});

	dojo.declare("dojox.gfx.Group", g.Shape, {
		// summary: a group shape (SVG), which can be used
		//	to logically group shapes (e.g, to propagate matricies)
		constructor: function(){
			svg.Container._init.call(this);
		},
		setRawNode: function(rawNode){
			// summary: sets a raw SVG node to be used by this shape
			// rawNode: Node: an SVG node
			this.rawNode = rawNode;
		}
	});
	g.Group.nodeType = "g";

	dojo.declare("dojox.gfx.Rect", gs.Rect, {
		// summary: a rectangle shape (SVG)
		setShape: function(newShape){
			// summary: sets a rectangle shape object (SVG)
			// newShape: Object: a rectangle shape object
			this.shape = g.makeParameters(this.shape, newShape);
			this.bbox = null;
			for(var i in this.shape){
				if(i != "type" && i != "r"){ this.rawNode.setAttribute(i, this.shape[i]); }
			}
			if(this.shape.r){
				this.rawNode.setAttribute("ry", this.shape.r);
				this.rawNode.setAttribute("rx", this.shape.r);
			}
			return this;	// self
		}
	});
	g.Rect.nodeType = "rect";

	g.Ellipse = gs.Ellipse;
	g.Ellipse.nodeType = "ellipse";

	g.Circle = gs.Circle;
	g.Circle.nodeType = "circle";

	g.Line = gs.Line;
	g.Line.nodeType = "line";

	dojo.declare("dojox.gfx.Polyline", gs.Polyline, {
		// summary: a polyline/polygon shape (SVG)
		setShape: function(points, closed){
			// summary: sets a polyline/polygon shape object (SVG)
			// points: Object: a polyline/polygon shape object
			if(points && points instanceof Array){
				// branch
				// points: Array: an array of points
				this.shape = g.makeParameters(this.shape, { points: points });
				if(closed && this.shape.points.length){
					this.shape.points.push(this.shape.points[0]);
				}
			}else{
				this.shape = g.makeParameters(this.shape, points);
			}
			this.bbox = null;
			this._normalizePoints();
			var attr = [], p = this.shape.points;
			for(var i = 0; i < p.length; ++i){
				attr.push(p[i].x.toFixed(8), p[i].y.toFixed(8));
			}
			this.rawNode.setAttribute("points", attr.join(" "));
			return this;	// self
		}
	});
	g.Polyline.nodeType = "polyline";

	dojo.declare("dojox.gfx.Image", gs.Image, {
		// summary: an image (SVG)
		setShape: function(newShape){
			// summary: sets an image shape object (SVG)
			// newShape: Object: an image shape object
			this.shape = g.makeParameters(this.shape, newShape);
			this.bbox = null;
			var rawNode = this.rawNode;
			for(var i in this.shape){
				if(i != "type" && i != "src"){ rawNode.setAttribute(i, this.shape[i]); }
			}
			rawNode.setAttribute("preserveAspectRatio", "none");
			rawNode.setAttributeNS(svg.xmlns.xlink, "xlink:href", this.shape.src);
			return this;	// self
		}
	});
	g.Image.nodeType = "image";

	dojo.declare("dojox.gfx.Text", gs.Text, {
		// summary: an anchored text (SVG)
		setShape: function(newShape){
			// summary: sets a text shape object (SVG)
			// newShape: Object: a text shape object
			this.shape = g.makeParameters(this.shape, newShape);
			this.bbox = null;
			var r = this.rawNode, s = this.shape;
			r.setAttribute("x", s.x);
			r.setAttribute("y", s.y);
			r.setAttribute("text-anchor", s.align);
			r.setAttribute("text-decoration", s.decoration);
			r.setAttribute("rotate", s.rotated ? 90 : 0);
			r.setAttribute("kerning", s.kerning ? "auto" : 0);
			r.setAttribute("text-rendering", "optimizeLegibility");
			
			// update the text content
			if(r.firstChild){
				r.firstChild.nodeValue = s.text;
			}else{
				r.appendChild(_createTextNode(s.text));
			}
			return this;	// self
		},
		getTextWidth: function(){
			// summary: get the text width in pixels
			var rawNode = this.rawNode,
				oldParent = rawNode.parentNode,
				_measurementNode = rawNode.cloneNode(true);
			_measurementNode.style.visibility = "hidden";

			// solution to the "orphan issue" in FF
			var _width = 0, _text = _measurementNode.firstChild.nodeValue;
			oldParent.appendChild(_measurementNode);

			// solution to the "orphan issue" in Opera
			// (nodeValue == "" hangs firefox)
			if(_text!=""){
				while(!_width){
//Yang: work around svgweb bug 417 -- http://code.google.com/p/svgweb/issues/detail?id=417
if (_measurementNode.getBBox)
					_width = parseInt(_measurementNode.getBBox().width);
else
	_width = 68;
				}
			}
			oldParent.removeChild(_measurementNode);
			return _width;
		}
	});
	g.Text.nodeType = "text";

	dojo.declare("dojox.gfx.Path", g.path.Path, {
		// summary: a path shape (SVG)
		_updateWithSegment: function(segment){
			// summary: updates the bounding box of path with new segment
			// segment: Object: a segment
			g.Path.superclass._updateWithSegment.apply(this, arguments);
			if(typeof(this.shape.path) == "string"){
				this.rawNode.setAttribute("d", this.shape.path);
			}
		},
		setShape: function(newShape){
			// summary: forms a path using a shape (SVG)
			// newShape: Object: an SVG path string or a path object (see dojox.gfx.defaultPath)
			g.Path.superclass.setShape.apply(this, arguments);
			this.rawNode.setAttribute("d", this.shape.path);
			return this;	// self
		}
	});
	g.Path.nodeType = "path";

	dojo.declare("dojox.gfx.TextPath", g.path.TextPath, {
		// summary: a textpath shape (SVG)
		_updateWithSegment: function(segment){
			// summary: updates the bounding box of path with new segment
			// segment: Object: a segment
			g.Path.superclass._updateWithSegment.apply(this, arguments);
			this._setTextPath();
		},
		setShape: function(newShape){
			// summary: forms a path using a shape (SVG)
			// newShape: Object: an SVG path string or a path object (see dojox.gfx.defaultPath)
			g.Path.superclass.setShape.apply(this, arguments);
			this._setTextPath();
			return this;	// self
		},
		_setTextPath: function(){
			if(typeof this.shape.path != "string"){ return; }
			var r = this.rawNode;
			if(!r.firstChild){
				var tp = _createElementNS(svg.xmlns.svg, "textPath"),
					tx = _createTextNode("");
				tp.appendChild(tx);
				r.appendChild(tp);
			}
			var ref  = r.firstChild.getAttributeNS(svg.xmlns.xlink, "href"),
				path = ref && svg.getRef(ref);
			if(!path){
				var surface = this._getParentSurface();
				if(surface){
					var defs = surface.defNode;
					path = _createElementNS(svg.xmlns.svg, "path");
					var id = g._base._getUniqueId();
					path.setAttribute("id", id);
					defs.appendChild(path);
					r.firstChild.setAttributeNS(svg.xmlns.xlink, "xlink:href", "#" + id);
				}
			}
			if(path){
				path.setAttribute("d", this.shape.path);
			}
		},
		_setText: function(){
			var r = this.rawNode;
			if(!r.firstChild){
				var tp = _createElementNS(svg.xmlns.svg, "textPath"),
					tx = _createTextNode("");
				tp.appendChild(tx);
				r.appendChild(tp);
			}
			r = r.firstChild;
			var t = this.text;
			r.setAttribute("alignment-baseline", "middle");
			switch(t.align){
				case "middle":
					r.setAttribute("text-anchor", "middle");
					r.setAttribute("startOffset", "50%");
					break;
				case "end":
					r.setAttribute("text-anchor", "end");
					r.setAttribute("startOffset", "100%");
					break;
				default:
					r.setAttribute("text-anchor", "start");
					r.setAttribute("startOffset", "0%");
					break;
			}
			//r.parentNode.setAttribute("alignment-baseline", "central");
			//r.setAttribute("dominant-baseline", "central");
			r.setAttribute("baseline-shift", "0.5ex");
			r.setAttribute("text-decoration", t.decoration);
			r.setAttribute("rotate", t.rotated ? 90 : 0);
			r.setAttribute("kerning", t.kerning ? "auto" : 0);
			r.firstChild.data = t.text;
		}
	});
	g.TextPath.nodeType = "text";

	dojo.declare("dojox.gfx.Surface", gs.Surface, {
		// summary: a surface object to be used for drawings (SVG)
		constructor: function(){
			svg.Container._init.call(this);
		},
		destroy: function(){
			this.defNode = null;	// release the external reference
			this.inherited(arguments);
		},
		setDimensions: function(width, height){
			// summary: sets the width and height of the rawNode
			// width: String: width of surface, e.g., "100px"
			// height: String: height of surface, e.g., "100px"
			if(!this.rawNode){ return this; }
			this.rawNode.setAttribute("width",  width);
			this.rawNode.setAttribute("height", height);
			return this;	// self
		},
		getDimensions: function(){
			// summary: returns an object with properties "width" and "height"
			var t = this.rawNode ? {
				width:  g.normalizedLength(this.rawNode.getAttribute("width")),
				height: g.normalizedLength(this.rawNode.getAttribute("height"))} : null;
			return t;	// Object
		}
	});

	g.createSurface = function(parentNode, width, height){
		// summary: creates a surface (SVG)
		// parentNode: Node: a parent node
		// width: String: width of surface, e.g., "100px"
		// height: String: height of surface, e.g., "100px"

		var s = new g.Surface();
		s.rawNode = _createElementNS(svg.xmlns.svg, "svg");
		if(width){
			s.rawNode.setAttribute("width",  width);
		}
		if(height){
			s.rawNode.setAttribute("height", height);
		}

		var defNode = _createElementNS(svg.xmlns.svg, "defs");
		s.rawNode.appendChild(defNode);
		s.defNode = defNode;

		s._parent = d.byId(parentNode);
		s._parent.appendChild(s.rawNode);

		return s;	// dojox.gfx.Surface
	};

	// Extenders

	svg.Font = {
		_setFont: function(){
			// summary: sets a font object (SVG)
			var f = this.fontStyle;
			// next line doesn't work in Firefox 2 or Opera 9
			//this.rawNode.setAttribute("font", dojox.gfx.makeFontString(this.fontStyle));
			this.rawNode.setAttribute("font-style", f.style);
			this.rawNode.setAttribute("font-variant", f.variant);
			this.rawNode.setAttribute("font-weight", f.weight);
			this.rawNode.setAttribute("font-size", f.size);
			this.rawNode.setAttribute("font-family", f.family);
		}
	};

	svg.Container = {
		_init: function(){
			gs.Container._init.call(this);
		},
		openBatch: function() {
			// summary: starts a new batch, subsequent new child shapes will be held in
			//	the batch instead of appending to the container directly
			this.fragment = _createFragment();
		},
		closeBatch: function() {
			// summary: submits the current batch, append all pending child shapes to DOM
			if (this.fragment) {
				this.rawNode.appendChild(this.fragment);
				delete this.fragment;
			}
		},
		add: function(shape){
			// summary: adds a shape to a group/surface
			// shape: dojox.gfx.Shape: an VML shape object
			if(this != shape.getParent()){
				if (this.fragment) {
					this.fragment.appendChild(shape.rawNode);
				} else {
					this.rawNode.appendChild(shape.rawNode);
				}
				//dojox.gfx.Group.superclass.add.apply(this, arguments);
				//this.inherited(arguments);
				gs.Container.add.apply(this, arguments);
			}
			return this;	// self
		},
		remove: function(shape, silently){
			// summary: remove a shape from a group/surface
			// shape: dojox.gfx.Shape: an VML shape object
			// silently: Boolean?: if true, regenerate a picture
			if(this == shape.getParent()){
				if(this.rawNode == shape.rawNode.parentNode){
					this.rawNode.removeChild(shape.rawNode);
				}
				if(this.fragment && this.fragment == shape.rawNode.parentNode){
					this.fragment.removeChild(shape.rawNode);
				}
				//dojox.gfx.Group.superclass.remove.apply(this, arguments);
				//this.inherited(arguments);
				gs.Container.remove.apply(this, arguments);
			}
			return this;	// self
		},
		clear: function(){
			// summary: removes all shapes from a group/surface
			var r = this.rawNode;
			while(r.lastChild){
				r.removeChild(r.lastChild);
			}
			var defNode = this.defNode;
			if(defNode){
				while(defNode.lastChild){
					defNode.removeChild(defNode.lastChild);
				}
				r.appendChild(defNode);
			}
			//return this.inherited(arguments);	// self
			return gs.Container.clear.apply(this, arguments);
		},
		_moveChildToFront: gs.Container._moveChildToFront,
		_moveChildToBack:  gs.Container._moveChildToBack
	};

	d.mixin(gs.Creator, {
		// summary: SVG shape creators
		createObject: function(shapeType, rawShape){
			// summary: creates an instance of the passed shapeType class
			// shapeType: Function: a class constructor to create an instance of
			// rawShape: Object: properties to be passed in to the classes "setShape" method
			if(!this.rawNode){ return null; }
			var shape = new shapeType(),
				node = _createElementNS(svg.xmlns.svg, shapeType.nodeType);

			shape.setRawNode(node);
			shape.setShape(rawShape);
			// rawNode.appendChild() will be done inside this.add(shape) below
			this.add(shape);
			return shape;	// dojox.gfx.Shape
		}
	});

	d.extend(g.Text, svg.Font);
	d.extend(g.TextPath, svg.Font);

	d.extend(g.Group, svg.Container);
	d.extend(g.Group, gs.Creator);

	d.extend(g.Surface, svg.Container);
	d.extend(g.Surface, gs.Creator);


	// some specific override for svgweb + flash
	if (svg.useSvgWeb) {
		
		// override createSurface()
		g.createSurface = function(parentNode, width, height) {
			var s = new g.Surface();

			// ensure width / height
			if (!width || !height) {
				var pos = d.position(parentNode);
				width  = width  || pos.w;
				height = height || pos.h;
			}

			// ensure id
			parentNode = d.byId(parentNode);
			var id = parentNode.id ? parentNode.id+'_svgweb' : g._base._getUniqueId();
			
			// create dynamic svg root
			var mockSvg = _createElementNS(svg.xmlns.svg, 'svg');
			mockSvg.id = id;
			mockSvg.setAttribute('width', width);
			mockSvg.setAttribute('height', height);
			svgweb.appendChild(mockSvg, parentNode);

			// notice: any call to the raw node before flash init will fail.
			mockSvg.addEventListener('SVGLoad', function() {
				// become loaded
				s.rawNode = this;
				s.isLoaded = true;
				
				// init defs
				var defNode = _createElementNS(svg.xmlns.svg, "defs");
				s.rawNode.appendChild(defNode);
				s.defNode = defNode;
				
				// notify application
				if (s.onLoad)
					s.onLoad(s);
			}, false);

			// flash not loaded yet
			s.isLoaded = false;
			return s;
		}
		
		// override Surface.destroy()
		dojo.extend(dojox.gfx.shape.Surface, {
			destroy: function() {
				var mockSvg = this.rawNode;
				svgweb.removeChild(mockSvg, mockSvg.parentNode);
			}
		});

		// override connect() & disconnect() for Shape & Surface event processing
		gs._eventsProcessing.connect = function(name, object, method) {
			// connect events using the mock addEventListener() provided by svgweb
			if (name.substring(0, 2)==='on') { name = name.substring(2); }
			if (arguments.length == 2) {
				method = object;
			} else {
				method = d.hitch(object, method);
			}
			this.getEventSource().addEventListener(name, method, false);
			return [this, name, method];
		}
		gs._eventsProcessing.disconnect = function(token) {
			// disconnect events using the mock removeEventListener() provided by svgweb
			this.getEventSource().removeEventListener(token[1], token[2], false);
			delete token[0];
		}
		dojo.extend(dojox.gfx.Shape, dojox.gfx.shape._eventsProcessing);
		dojo.extend(dojox.gfx.shape.Surface, dojox.gfx.shape._eventsProcessing);
	}

})();

}

if(!dojo._hasResource["dojox.gfx.arc"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.arc"] = true;
dojo.provide("dojox.gfx.arc");



(function(){
	var m = dojox.gfx.matrix,
		unitArcAsBezier = function(alpha){
			// summary: return a start point, 1st and 2nd control points, and an end point of
			//		a an arc, which is reflected on the x axis
			// alpha: Number: angle in radians, the arc will be 2 * angle size
			var cosa  = Math.cos(alpha), sina  = Math.sin(alpha),
				p2 = {x: cosa + (4 / 3) * (1 - cosa), y: sina - (4 / 3) * cosa * (1 - cosa) / sina};
			return {	// Object
				s:  {x: cosa, y: -sina},
				c1: {x: p2.x, y: -p2.y},
				c2: p2,
				e:  {x: cosa, y: sina}
			};
		},
		twoPI = 2 * Math.PI, pi4 = Math.PI / 4, pi8 = Math.PI / 8,
		pi48 = pi4 + pi8, curvePI4 = unitArcAsBezier(pi8);

	dojo.mixin(dojox.gfx.arc, {
		unitArcAsBezier: unitArcAsBezier,
		curvePI4: curvePI4,
		arcAsBezier: function(last, rx, ry, xRotg, large, sweep, x, y){
			// summary: calculates an arc as a series of Bezier curves
			//	given the last point and a standard set of SVG arc parameters,
			//	it returns an array of arrays of parameters to form a series of
			//	absolute Bezier curves.
			// last: Object: a point-like object as a start of the arc
			// rx: Number: a horizontal radius for the virtual ellipse
			// ry: Number: a vertical radius for the virtual ellipse
			// xRotg: Number: a rotation of an x axis of the virtual ellipse in degrees
			// large: Boolean: which part of the ellipse will be used (the larger arc if true)
			// sweep: Boolean: direction of the arc (CW if true)
			// x: Number: the x coordinate of the end point of the arc
			// y: Number: the y coordinate of the end point of the arc

			// calculate parameters
			large = Boolean(large);
			sweep = Boolean(sweep);
			var xRot = m._degToRad(xRotg),
				rx2 = rx * rx, ry2 = ry * ry,
				pa = m.multiplyPoint(
					m.rotate(-xRot),
					{x: (last.x - x) / 2, y: (last.y - y) / 2}
				),
				pax2 = pa.x * pa.x, pay2 = pa.y * pa.y,
				c1 = Math.sqrt((rx2 * ry2 - rx2 * pay2 - ry2 * pax2) / (rx2 * pay2 + ry2 * pax2));
			if(isNaN(c1)){ c1 = 0; }
			var	ca = {
					x:  c1 * rx * pa.y / ry,
					y: -c1 * ry * pa.x / rx
				};
			if(large == sweep){
				ca = {x: -ca.x, y: -ca.y};
			}
			// the center
			var c = m.multiplyPoint(
				[
					m.translate(
						(last.x + x) / 2,
						(last.y + y) / 2
					),
					m.rotate(xRot)
				],
				ca
			);
			// calculate the elliptic transformation
			var elliptic_transform = m.normalize([
				m.translate(c.x, c.y),
				m.rotate(xRot),
				m.scale(rx, ry)
			]);
			// start, end, and size of our arc
			var inversed = m.invert(elliptic_transform),
				sp = m.multiplyPoint(inversed, last),
				ep = m.multiplyPoint(inversed, x, y),
				startAngle = Math.atan2(sp.y, sp.x),
				endAngle   = Math.atan2(ep.y, ep.x),
				theta = startAngle - endAngle;	// size of our arc in radians
			if(sweep){ theta = -theta; }
			if(theta < 0){
				theta += twoPI;
			}else if(theta > twoPI){
				theta -= twoPI;
			}

			// draw curve chunks
			var alpha = pi8, curve = curvePI4, step  = sweep ? alpha : -alpha,
				result = [];
			for(var angle = theta; angle > 0; angle -= pi4){
				if(angle < pi48){
					alpha = angle / 2;
					curve = unitArcAsBezier(alpha);
					step  = sweep ? alpha : -alpha;
					angle = 0;	// stop the loop
				}
				var c1, c2, e,
					M = m.normalize([elliptic_transform, m.rotate(startAngle + step)]);
				if(sweep){
					c1 = m.multiplyPoint(M, curve.c1);
					c2 = m.multiplyPoint(M, curve.c2);
					e  = m.multiplyPoint(M, curve.e );
				}else{
					c1 = m.multiplyPoint(M, curve.c2);
					c2 = m.multiplyPoint(M, curve.c1);
					e  = m.multiplyPoint(M, curve.s );
				}
				// draw the curve
				result.push([c1.x, c1.y, c2.x, c2.y, e.x, e.y]);
				startAngle += 2 * step;
			}
			return result;	// Array
		}
	});
})();

}

if(!dojo._hasResource["dojox.gfx.gradient"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.gradient"] = true;
dojo.provide("dojox.gfx.gradient");



// Various utilities to deal with a linear gradient (mostly VML-specific)

(function(){
	var d = dojo, m = dojox.gfx.matrix, C = d.Color;
	
	dojox.gfx.gradient.rescale = function(stops, from, to){
		// summary:
		//		recalculates a gradient from 0-1 window to
		//		"from"-"to" window blending and replicating colors,
		//		if necessary
		// stops: Array:
		//		input gradient as a list of colors with offsets
		//		(see dojox.gfx.defaultLinearGradient and dojox.gfx.defaultRadialGradient)
		// from: Number:
		//		the beginning of the window, should be less than "to"
		// to: Number:
		//		the end of the window, should be more than "from"

		var len = stops.length, reverseFlag = (to < from), newStops;

		// do we need to reverse the color table?
		if(reverseFlag){
			var tmp = from;
			from = to;
			to = tmp;
		}
		
		// various edge cases
		if(!len){
			// no colors
			return [];
		}
		if(to <= stops[0].offset){
			// all colors are before the color table
			newStops = [
				{offset: 0, color: stops[0].color},
				{offset: 1, color: stops[0].color}
			];
		}else if(from >= stops[len - 1].offset){
			// all colors are after the color table
			newStops = [
				{offset: 0, color: stops[len - 1].color},
				{offset: 1, color: stops[len - 1].color}
			];
		}else{
			// main scanning algorithm
			var span = to - from, stop, prev, i;
			newStops = [];
			if(from < 0){
				newStops.push({offset: 0, color: new C(stops[0].color)});
			}
			for(i = 0; i < len; ++i){
				stop = stops[i];
				if(stop.offset >= from){
					break;
				}
				// skip this color
			}
			if(i){
				prev = stops[i - 1];
				newStops.push({
					offset: 0,
					color: d.blendColors(new C(prev.color), new C(stop.color), (from - prev.offset) / (stop.offset - prev.offset))
				});
			}else{
				newStops.push({offset: 0, color: new C(stop.color)});
			}
			for(; i < len; ++i){
				stop = stops[i];
				if(stop.offset >= to){
					break;
				}
				newStops.push({offset: (stop.offset - from) / span, color: new C(stop.color)});
			}
			if(i < len){
				prev = stops[i - 1];
				newStops.push({
					offset: 1,
					color: d.blendColors(new C(prev.color), new C(stop.color), (to - prev.offset) / (stop.offset - prev.offset))
				});
			}else{
				newStops.push({offset: 1, color: new C(stops[len - 1].color)});
			}
		}
		
		// reverse the color table, if needed
		if(reverseFlag){
			newStops.reverse();
			for(i = 0, len = newStops.length; i < len; ++i){
				stop = newStops[i];
				stop.offset = 1 - stop.offset;
			}
		}
		
		return newStops;
	};
	
	function getPoint(x, y, matrix, project, shiftAndRotate, scale){
		var r = m.multiplyPoint(matrix, x, y),
			p = m.multiplyPoint(project, r);
		return {r: r, p: p, o: m.multiplyPoint(shiftAndRotate, p).x / scale};
	}
	
	function sortPoints(a, b){
		return a.o - b.o;
	}
	
	dojox.gfx.gradient.project = function(matrix, grad, tl, rb, ttl, trb){
		// summary:
		//		return a new gradient using the "VML algorithm" and suitable for VML
		// matrix: dojox.gfx.Matrix2D|Null:
		//		matrix to apply to a shape and its gradient
		// grad: Object:
		//		a linear gradient object to be transformed
		// tl: dojox.gfx.Point:
		//		top-left corner of shape's bounding box
		// rb: dojox.gfx.Point:
		//		right-bottom corner of shape's bounding box
		// ttl: dojox.gfx.Point:
		//		top-left corner of shape's transformed bounding box
		// trb: dojox.gfx.Point:
		//		right-bottom corner of shape's transformed bounding box
		
		matrix = matrix || m.identity;

		var f1 = m.multiplyPoint(matrix, grad.x1, grad.y1),
			f2 = m.multiplyPoint(matrix, grad.x2, grad.y2),
			angle = Math.atan2(f2.y - f1.y, f2.x - f1.x),
			project = m.project(f2.x - f1.x, f2.y - f1.y),
			pf1 = m.multiplyPoint(project, f1),
			pf2 = m.multiplyPoint(project, f2),
			shiftAndRotate = new m.Matrix2D([m.rotate(-angle), {dx: -pf1.x, dy: -pf1.y}]),
			scale = m.multiplyPoint(shiftAndRotate, pf2).x,
			//comboMatrix = new m.Matrix2D([shiftAndRotate, project, matrix]),
			// bbox-specific calculations
			points = [
					getPoint(tl.x, tl.y, matrix, project, shiftAndRotate, scale),
					getPoint(rb.x, rb.y, matrix, project, shiftAndRotate, scale),
					getPoint(tl.x, rb.y, matrix, project, shiftAndRotate, scale),
					getPoint(rb.x, tl.y, matrix, project, shiftAndRotate, scale)
				].sort(sortPoints),
			from = points[0].o,
			to   = points[3].o,
			stops = dojox.gfx.gradient.rescale(grad.colors, from, to),
			//angle2 = Math.atan2(Math.abs(points[3].r.y - points[0].r.y) * (f2.y - f1.y), Math.abs(points[3].r.x - points[0].r.x) * (f2.x - f1.x));
			angle2 = Math.atan2(points[3].r.y - points[0].r.y, points[3].r.x - points[0].r.x);

		return {
			type: "linear",
			x1: points[0].p.x, y1: points[0].p.y, x2: points[3].p.x, y2: points[3].p.y,
			colors: stops,
			// additional helpers (for VML)
			angle: angle
		};
	};
})();

}

if(!dojo._hasResource["dojox.gfx.vml"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.vml"] = true;
dojo.provide("dojox.gfx.vml");







(function(){
	var d = dojo, g = dojox.gfx, m = g.matrix, gs = g.shape, vml = g.vml;

	// dojox.gfx.vml.xmlns: String: a VML's namespace
	vml.xmlns = "urn:schemas-microsoft-com:vml";

	// dojox.gfx.vml.text_alignment: Object: mapping from SVG alignment to VML alignment
	vml.text_alignment = {start: "left", middle: "center", end: "right"};

	vml._parseFloat = function(str) {
		// summary: a helper function to parse VML-specific floating-point values
		// str: String: a representation of a floating-point number
		return str.match(/^\d+f$/i) ? parseInt(str) / 65536 : parseFloat(str);	// Number
	};

	vml._bool = {"t": 1, "true": 1};

	d.extend(g.Shape, {
		// summary: VML-specific implementation of dojox.gfx.Shape methods

		setFill: function(fill){
			// summary: sets a fill object (VML)
			// fill: Object: a fill object
			//	(see dojox.gfx.defaultLinearGradient,
			//	dojox.gfx.defaultRadialGradient,
			//	dojox.gfx.defaultPattern,
			//	or dojo.Color)

			if(!fill){
				// don't fill
				this.fillStyle = null;
				this.rawNode.filled = "f";
				return this;
			}
			var i, f, fo, a, s;
			if(typeof fill == "object" && "type" in fill){
				// gradient
				switch(fill.type){
					case "linear":
						var matrix = this._getRealMatrix(), bbox = this.getBoundingBox(),
							tbbox = this._getRealBBox ? this._getRealBBox() : this.getTransformedBoundingBox();
						s = [];
						if(this.fillStyle !== fill){
							this.fillStyle = g.makeParameters(g.defaultLinearGradient, fill);
						}
						f = g.gradient.project(matrix, this.fillStyle,
								{x: bbox.x, y: bbox.y},
								{x: bbox.x + bbox.width, y: bbox.y + bbox.height},
								tbbox[0], tbbox[2]);
						a = f.colors;
						if(a[0].offset.toFixed(5) != "0.00000"){
							s.push("0 " + g.normalizeColor(a[0].color).toHex());
						}
						for(i = 0; i < a.length; ++i){
							s.push(a[i].offset.toFixed(5) + " " + g.normalizeColor(a[i].color).toHex());
						}
						i = a.length - 1;
						if(a[i].offset.toFixed(5) != "1.00000"){
							s.push("1 " + g.normalizeColor(a[i].color).toHex());
						}
						fo = this.rawNode.fill;
						fo.colors.value = s.join(";");
						fo.method = "sigma";
						fo.type = "gradient";
						fo.angle = (270 - m._radToDeg(f.angle)) % 360;
						fo.on = true;
						break;
					case "radial":
						f = g.makeParameters(g.defaultRadialGradient, fill);
						this.fillStyle = f;
						var l = parseFloat(this.rawNode.style.left),
							t = parseFloat(this.rawNode.style.top),
							w = parseFloat(this.rawNode.style.width),
							h = parseFloat(this.rawNode.style.height),
							c = isNaN(w) ? 1 : 2 * f.r / w;
						a = [];
						// add a color at the offset 0 (1 in VML coordinates)
						if(f.colors[0].offset > 0){
							a.push({offset: 1, color: g.normalizeColor(f.colors[0].color)});
						}
						// massage colors
						d.forEach(f.colors, function(v, i){
							a.push({offset: 1 - v.offset * c, color: g.normalizeColor(v.color)});
						});
						i = a.length - 1;
						while(i >= 0 && a[i].offset < 0){ --i; }
						if(i < a.length - 1){
							// correct excessive colors
							var q = a[i], p = a[i + 1];
							p.color = d.blendColors(q.color, p.color, q.offset / (q.offset - p.offset));
							p.offset = 0;
							while(a.length - i > 2) a.pop();
						}
						// set colors
						i = a.length - 1, s = [];
						if(a[i].offset > 0){
							s.push("0 " + a[i].color.toHex());
						}
						for(; i >= 0; --i){
							s.push(a[i].offset.toFixed(5) + " " + a[i].color.toHex());
						}
						fo = this.rawNode.fill;
						fo.colors.value = s.join(";");
						fo.method = "sigma";
						fo.type = "gradientradial";
						if(isNaN(w) || isNaN(h) || isNaN(l) || isNaN(t)){
							fo.focusposition = "0.5 0.5";
						}else{
							fo.focusposition = ((f.cx - l) / w).toFixed(5) + " " + ((f.cy - t) / h).toFixed(5);
						}
						fo.focussize = "0 0";
						fo.on = true;
						break;
					case "pattern":
						f = g.makeParameters(g.defaultPattern, fill);
						this.fillStyle = f;
						fo = this.rawNode.fill;
						fo.type = "tile";
						fo.src = f.src;
						if(f.width && f.height){
							// in points
							fo.size.x = g.px2pt(f.width);
							fo.size.y = g.px2pt(f.height);
						}
						fo.alignShape = "f";
						fo.position.x = 0;
						fo.position.y = 0;
						fo.origin.x = f.width  ? f.x / f.width  : 0;
						fo.origin.y = f.height ? f.y / f.height : 0;
						fo.on = true;
						break;
				}
				this.rawNode.fill.opacity = 1;
				return this;
			}
			// color object
			this.fillStyle = g.normalizeColor(fill);
			fo = this.rawNode.fill;
			if(!fo){
				fo = this.rawNode.ownerDocument.createElement("v:fill");
			}
			fo.method = "any";
			fo.type = "solid";
			fo.opacity = this.fillStyle.a;
			this.rawNode.fillcolor = this.fillStyle.toHex();
			this.rawNode.filled = true;
			return this;	// self
		},

		setStroke: function(stroke){
			// summary: sets a stroke object (VML)
			// stroke: Object: a stroke object
			//	(see dojox.gfx.defaultStroke)

			if(!stroke){
				// don't stroke
				this.strokeStyle = null;
				this.rawNode.stroked = "f";
				return this;
			}
			// normalize the stroke
			if(typeof stroke == "string" || d.isArray(stroke) || stroke instanceof d.Color){
				stroke = {color: stroke};
			}
			var s = this.strokeStyle = g.makeParameters(g.defaultStroke, stroke);
			s.color = g.normalizeColor(s.color);
			// generate attributes
			var rn = this.rawNode;
			rn.stroked = true;
			rn.strokecolor = s.color.toCss();
			rn.strokeweight = s.width + "px";	// TODO: should we assume that the width is always in pixels?
			if(rn.stroke) {
				rn.stroke.opacity = s.color.a;
				rn.stroke.endcap = this._translate(this._capMap, s.cap);
				if(typeof s.join == "number") {
					rn.stroke.joinstyle = "miter";
					rn.stroke.miterlimit = s.join;
				}else{
					rn.stroke.joinstyle = s.join;
					// rn.stroke.miterlimit = s.width;
				}
				rn.stroke.dashstyle = s.style == "none" ? "Solid" : s.style;
			}
			return this;	// self
		},

		_capMap: { butt: 'flat' },
		_capMapReversed: { flat: 'butt' },

		_translate: function(dict, value) {
			return (value in dict) ? dict[value] : value;
		},

		_applyTransform: function() {
			var matrix = this._getRealMatrix();
			if(matrix){
				var skew = this.rawNode.skew;
				if(typeof skew == "undefined"){
					for(var i = 0; i < this.rawNode.childNodes.length; ++i){
						if(this.rawNode.childNodes[i].tagName == "skew"){
							skew = this.rawNode.childNodes[i];
							break;
						}
					}
				}
				if(skew){
					skew.on = "f";
					var mt = matrix.xx.toFixed(8) + " " + matrix.xy.toFixed(8) + " " +
						matrix.yx.toFixed(8) + " " + matrix.yy.toFixed(8) + " 0 0",
						offset = Math.floor(matrix.dx).toFixed() + "px " + Math.floor(matrix.dy).toFixed() + "px",
						s = this.rawNode.style,
						l = parseFloat(s.left),
						t = parseFloat(s.top),
						w = parseFloat(s.width),
						h = parseFloat(s.height);
					if(isNaN(l)) l = 0;
					if(isNaN(t)) t = 0;
					if(isNaN(w) || !w) w = 1;
					if(isNaN(h) || !h) h = 1;
					var origin = (-l / w - 0.5).toFixed(8) + " " + (-t / h - 0.5).toFixed(8);
					skew.matrix =  mt;
					skew.origin = origin;
					skew.offset = offset;
					skew.on = true;
				}
			}
			if(this.fillStyle && this.fillStyle.type == "linear"){
				this.setFill(this.fillStyle);
			}
			return this;
		},

		_setDimensions: function(width, height){
			// summary: sets the width and height of the rawNode,
			//	if the surface sixe has been changed
			// width: String: width in pixels
			// height: String: height in pixels

			// default implementation does nothing
			return this; // self
		},

		setRawNode: function(rawNode){
			// summary:
			//	assigns and clears the underlying node that will represent this
			//	shape. Once set, transforms, gradients, etc, can be applied.
			//	(no fill & stroke by default)
			rawNode.stroked = "f";
			rawNode.filled  = "f";
			this.rawNode = rawNode;
		},

		// move family

		_moveToFront: function(){
			// summary: moves a shape to front of its parent's list of shapes (VML)
			this.rawNode.parentNode.appendChild(this.rawNode);
			return this;
		},
		_moveToBack: function(){
			// summary: moves a shape to back of its parent's list of shapes (VML)
			var r = this.rawNode, p = r.parentNode, n = p.firstChild;
			p.insertBefore(r, n);
			if(n.tagName == "rect"){
				// surface has a background rectangle, which position should be preserved
				n.swapNode(r);
			}
			return this;
		},

		_getRealMatrix: function(){
			// summary: returns the cumulative ("real") transformation matrix
			//	by combining the shape's matrix with its parent's matrix
			return this.parentMatrix ? new g.Matrix2D([this.parentMatrix, this.matrix]) : this.matrix;	// dojox.gfx.Matrix2D
		}
	});

	dojo.declare("dojox.gfx.Group", g.Shape, {
		// summary: a group shape (VML), which can be used
		//	to logically group shapes (e.g, to propagate matricies)
		constructor: function(){
			vml.Container._init.call(this);
		},
		// apply transformation
		_applyTransform: function(){
			// summary: applies a transformation matrix to a group
			var matrix = this._getRealMatrix();
			for(var i = 0; i < this.children.length; ++i){
				this.children[i]._updateParentMatrix(matrix);
			}
			return this;	// self
		},
		_setDimensions: function(width, height){
			// summary: sets the width and height of the rawNode,
			//	if the surface sixe has been changed
			// width: String: width in pixels
			// height: String: height in pixels
			var r = this.rawNode, rs = r.style,
				bs = this.bgNode.style;
			rs.width = width;
			rs.height = height;
			r.coordsize = width + " " + height;
			bs.width = width;
			bs.height = height;
			for(var i = 0; i < this.children.length; ++i){
				this.children[i]._setDimensions(width, height);
			}
			return this; // self
		}
	});
	g.Group.nodeType = "group";

	dojo.declare("dojox.gfx.Rect", gs.Rect, {
		// summary: a rectangle shape (VML)
		setShape: function(newShape){
			// summary: sets a rectangle shape object (VML)
			// newShape: Object: a rectangle shape object
			var shape = this.shape = g.makeParameters(this.shape, newShape);
			this.bbox = null;
			var r = Math.min(1, (shape.r / Math.min(parseFloat(shape.width), parseFloat(shape.height)))).toFixed(8);
			// a workaround for the VML's arcsize bug: cannot read arcsize of an instantiated node
			var parent = this.rawNode.parentNode, before = null;
			if(parent){
				if(parent.lastChild !== this.rawNode){
					for(var i = 0; i < parent.childNodes.length; ++i){
						if(parent.childNodes[i] === this.rawNode){
							before = parent.childNodes[i + 1];
							break;
						}
					}
				}
				parent.removeChild(this.rawNode);
			}
			if(d.isIE > 7){
				var node = this.rawNode.ownerDocument.createElement("v:roundrect");
				node.arcsize = r;
				node.style.display = "inline-block";
				this.rawNode = node;
			}else{
				this.rawNode.arcsize = r;
			}
			if(parent){
				if(before){
					parent.insertBefore(this.rawNode, before);
				}else{
					parent.appendChild(this.rawNode);
				}
			}
			var style = this.rawNode.style;
			style.left   = shape.x.toFixed();
			style.top    = shape.y.toFixed();
			style.width  = (typeof shape.width == "string" && shape.width.indexOf("%") >= 0)  ? shape.width  : shape.width.toFixed();
			style.height = (typeof shape.width == "string" && shape.height.indexOf("%") >= 0) ? shape.height : shape.height.toFixed();
			// set all necessary styles, which are lost by VML (yes, it's a VML's bug)
			return this.setTransform(this.matrix).setFill(this.fillStyle).setStroke(this.strokeStyle);	// self
		}
	});
	g.Rect.nodeType = "roundrect"; // use a roundrect so the stroke join type is respected


	dojo.declare("dojox.gfx.Ellipse", gs.Ellipse, {
		// summary: an ellipse shape (VML)
		setShape: function(newShape){
			// summary: sets an ellipse shape object (VML)
			// newShape: Object: an ellipse shape object
			var shape = this.shape = g.makeParameters(this.shape, newShape);
			this.bbox = null;
			var style = this.rawNode.style;
			style.left   = (shape.cx - shape.rx).toFixed();
			style.top    = (shape.cy - shape.ry).toFixed();
			style.width  = (shape.rx * 2).toFixed();
			style.height = (shape.ry * 2).toFixed();
			return this.setTransform(this.matrix);	// self
		}
	});
	g.Ellipse.nodeType = "oval";

	dojo.declare("dojox.gfx.Circle", gs.Circle, {
		// summary: a circle shape (VML)
		setShape: function(newShape){
			// summary: sets a circle shape object (VML)
			// newShape: Object: a circle shape object
			var shape = this.shape = g.makeParameters(this.shape, newShape);
			this.bbox = null;
			var style = this.rawNode.style;
			style.left   = (shape.cx - shape.r).toFixed();
			style.top    = (shape.cy - shape.r).toFixed();
			style.width  = (shape.r * 2).toFixed();
			style.height = (shape.r * 2).toFixed();
			return this;	// self
		}
	});
	g.Circle.nodeType = "oval";

	dojo.declare("dojox.gfx.Line", gs.Line, {
		// summary: a line shape (VML)
		constructor: function(rawNode){
			if(rawNode) rawNode.setAttribute("dojoGfxType", "line");
		},
		setShape: function(newShape){
			// summary: sets a line shape object (VML)
			// newShape: Object: a line shape object
			var shape = this.shape = g.makeParameters(this.shape, newShape);
			this.bbox = null;
			this.rawNode.path.v = "m" + shape.x1.toFixed() + " " + shape.y1.toFixed() +
				"l" + shape.x2.toFixed() + " " + shape.y2.toFixed() + "e";
			return this.setTransform(this.matrix);	// self
		}
	});
	g.Line.nodeType = "shape";

	dojo.declare("dojox.gfx.Polyline", gs.Polyline, {
		// summary: a polyline/polygon shape (VML)
		constructor: function(rawNode){
			if(rawNode) rawNode.setAttribute("dojoGfxType", "polyline");
		},
		setShape: function(points, closed){
			// summary: sets a polyline/polygon shape object (VML)
			// points: Object: a polyline/polygon shape object
			// closed: Boolean?: if true, close the polyline explicitely
			if(points && points instanceof Array){
				// branch
				// points: Array: an array of points
				this.shape = g.makeParameters(this.shape, { points: points });
				if(closed && this.shape.points.length) this.shape.points.push(this.shape.points[0]);
			}else{
				this.shape = g.makeParameters(this.shape, points);
			}
			this.bbox = null;
			this._normalizePoints();
			var attr = [], p = this.shape.points;
			if(p.length > 0){
				attr.push("m");
				attr.push(p[0].x.toFixed(), p[0].y.toFixed());
				if(p.length > 1){
					attr.push("l");
					for(var i = 1; i < p.length; ++i){
						attr.push(p[i].x.toFixed(), p[i].y.toFixed());
					}
				}
			}
			attr.push("e");
			this.rawNode.path.v = attr.join(" ");
			return this.setTransform(this.matrix);	// self
		}
	});
	g.Polyline.nodeType = "shape";

	dojo.declare("dojox.gfx.Image", gs.Image, {
		// summary: an image (VML)
		setShape: function(newShape){
			// summary: sets an image shape object (VML)
			// newShape: Object: an image shape object
			var shape = this.shape = g.makeParameters(this.shape, newShape);
			this.bbox = null;
			this.rawNode.firstChild.src = shape.src;
			return this.setTransform(this.matrix);	// self
		},
		_applyTransform: function() {
			var matrix = this._getRealMatrix(),
				rawNode = this.rawNode,
				s = rawNode.style,
				shape = this.shape;
			if(matrix){
				matrix = m.multiply(matrix, {dx: shape.x, dy: shape.y});
			}else{
				matrix = m.normalize({dx: shape.x, dy: shape.y});
			}
			if(matrix.xy == 0 && matrix.yx == 0 && matrix.xx > 0 && matrix.yy > 0){
				// special case to avoid filters
				s.filter = "";
				s.width  = Math.floor(matrix.xx * shape.width);
				s.height = Math.floor(matrix.yy * shape.height);
				s.left   = Math.floor(matrix.dx);
				s.top    = Math.floor(matrix.dy);
			}else{
				var ps = rawNode.parentNode.style;
				s.left   = "0px";
				s.top    = "0px";
				s.width  = ps.width;
				s.height = ps.height;
				matrix = m.multiply(matrix,
					{xx: shape.width / parseInt(s.width), yy: shape.height / parseInt(s.height)});
				var f = rawNode.filters["DXImageTransform.Microsoft.Matrix"];
				if(f){
					f.M11 = matrix.xx;
					f.M12 = matrix.xy;
					f.M21 = matrix.yx;
					f.M22 = matrix.yy;
					f.Dx = matrix.dx;
					f.Dy = matrix.dy;
				}else{
					s.filter = "progid:DXImageTransform.Microsoft.Matrix(M11=" + matrix.xx +
						", M12=" + matrix.xy + ", M21=" + matrix.yx + ", M22=" + matrix.yy +
						", Dx=" + matrix.dx + ", Dy=" + matrix.dy + ")";
				}
			}
			return this; // self
		},
		_setDimensions: function(width, height){
			// summary: sets the width and height of the rawNode,
			//	if the surface sixe has been changed
			// width: String: width in pixels
			// height: String: height in pixels

			var r = this.rawNode, f = r.filters["DXImageTransform.Microsoft.Matrix"];
			if(f){
				var s = r.style;
				s.width  = width;
				s.height = height;
				return this._applyTransform(); // self
			}
			return this;	// self
		}
	});
	g.Image.nodeType = "rect";

	dojo.declare("dojox.gfx.Text", gs.Text, {
		// summary: an anchored text (VML)
		constructor: function(rawNode){
			if(rawNode){rawNode.setAttribute("dojoGfxType", "text");}
			this.fontStyle = null;
		},
		_alignment: {start: "left", middle: "center", end: "right"},
		setShape: function(newShape){
			// summary: sets a text shape object (VML)
			// newShape: Object: a text shape object
			this.shape = g.makeParameters(this.shape, newShape);
			this.bbox = null;
			var r = this.rawNode, s = this.shape, x = s.x, y = s.y.toFixed(), path;
			switch(s.align){
				case "middle":
					x -= 5;
					break;
				case "end":
					x -= 10;
					break;
			}
			path = "m" + x.toFixed() + "," + y + "l" + (x + 10).toFixed() + "," + y + "e";
			// find path and text path
			var p = null, t = null, c = r.childNodes;
			for(var i = 0; i < c.length; ++i){
				var tag = c[i].tagName;
				if(tag == "path"){
					p = c[i];
					if(t) break;
				}else if(tag == "textpath"){
					t = c[i];
					if(p) break;
				}
			}
			if(!p){
				p = r.ownerDocument.createElement("v:path");
				r.appendChild(p);
			}
			if(!t){
				t = r.ownerDocument.createElement("v:textpath");
				r.appendChild(t);
			}
			p.v = path;
			p.textPathOk = true;
			t.on = true;
			var a = vml.text_alignment[s.align];
			t.style["v-text-align"] = a ? a : "left";
			t.style["text-decoration"] = s.decoration;
			t.style["v-rotate-letters"] = s.rotated;
			t.style["v-text-kern"] = s.kerning;
			t.string = s.text;
			return this.setTransform(this.matrix);	// self
		},
		_setFont: function(){
			// summary: sets a font object (VML)
			var f = this.fontStyle, c = this.rawNode.childNodes;
			for(var i = 0; i < c.length; ++i){
				if(c[i].tagName == "textpath"){
					c[i].style.font = g.makeFontString(f);
					break;
				}
			}
			this.setTransform(this.matrix);
		},
		_getRealMatrix: function(){
			// summary: returns the cumulative ("real") transformation matrix
			//	by combining the shape's matrix with its parent's matrix;
			//	it makes a correction for a font size
			var matrix = g.Shape.prototype._getRealMatrix.call(this);
			// It appears that text is always aligned vertically at a middle of x-height (???).
			// It is impossible to obtain these metrics from VML => I try to approximate it with
			// more-or-less util value of 0.7 * FontSize, which is typical for European fonts.
			if(matrix){
				matrix = m.multiply(matrix,
					{dy: -g.normalizedLength(this.fontStyle ? this.fontStyle.size : "10pt") * 0.35});
			}
			return matrix;	// dojox.gfx.Matrix2D
		},
		getTextWidth: function(){
			// summary: get the text width, in px
			var rawNode = this.rawNode, _display = rawNode.style.display;
			rawNode.style.display = "inline";
			var _width = g.pt2px(parseFloat(rawNode.currentStyle.width));
			rawNode.style.display = _display;
			return _width;
		}
	});
	g.Text.nodeType = "shape";

	dojo.declare("dojox.gfx.Path", g.path.Path, {
		// summary: a path shape (VML)
		constructor: function(rawNode){
			if(rawNode && !rawNode.getAttribute("dojoGfxType")){
				rawNode.setAttribute("dojoGfxType", "path");
			}
			this.vmlPath = "";
			this.lastControl = {};
		},
		_updateWithSegment: function(segment){
			// summary: updates the bounding box of path with new segment
			// segment: Object: a segment
			var last = d.clone(this.last);
			g.Path.superclass._updateWithSegment.apply(this, arguments);
			if(arguments.length > 1){ return; } // skip transfomed bbox calculations
			// add a VML path segment
			var path = this[this.renderers[segment.action]](segment, last);
			if(typeof this.vmlPath == "string"){
				this.vmlPath += path.join("");
				this.rawNode.path.v = this.vmlPath + " r0,0 e";
			}else{
				Array.prototype.push.apply(this.vmlPath, path);
			}
		},
		setShape: function(newShape){
			// summary: forms a path using a shape (VML)
			// newShape: Object: an VML path string or a path object (see dojox.gfx.defaultPath)
			this.vmlPath = [];
			this.lastControl.type = "";	// no prior control point
			g.Path.superclass.setShape.apply(this, arguments);
			this.vmlPath = this.vmlPath.join("");
			this.rawNode.path.v = this.vmlPath + " r0,0 e";
			return this;
		},
		_pathVmlToSvgMap: {m: "M", l: "L", t: "m", r: "l", c: "C", v: "c", qb: "Q", x: "z", e: ""},
		// VML-specific segment renderers
		renderers: {
			M: "_moveToA", m: "_moveToR",
			L: "_lineToA", l: "_lineToR",
			H: "_hLineToA", h: "_hLineToR",
			V: "_vLineToA", v: "_vLineToR",
			C: "_curveToA", c: "_curveToR",
			S: "_smoothCurveToA", s: "_smoothCurveToR",
			Q: "_qCurveToA", q: "_qCurveToR",
			T: "_qSmoothCurveToA", t: "_qSmoothCurveToR",
			A: "_arcTo", a: "_arcTo",
			Z: "_closePath", z: "_closePath"
		},
		_addArgs: function(path, segment, from, upto){
			var n = segment instanceof Array ? segment : segment.args;
			for(var i = from; i < upto; ++i){
				path.push(" ", n[i].toFixed());
			}
		},
		_adjustRelCrd: function(last, segment, step){
			var n = segment instanceof Array ? segment : segment.args, l = n.length,
				result = new Array(l), i = 0, x = last.x, y = last.y;
			if(typeof x != "number"){
				// there is no last coordinate =>
				// treat the first pair as an absolute coordinate
				result[0] = x = n[0];
				result[1] = y = n[1];
				i = 2;
			}
			if(typeof step == "number" && step != 2){
				var j = step;
				while(j <= l){
					for(; i < j; i += 2){
						result[i] = x + n[i];
						result[i + 1] = y + n[i + 1];
					}
					x = result[j - 2];
					y = result[j - 1];
					j += step;
				}
			}else{
				for(; i < l; i += 2){
					result[i] = (x += n[i]);
					result[i + 1] = (y += n[i + 1]);
				}
			}
			return result;
		},
		_adjustRelPos: function(last, segment){
			var n = segment instanceof Array ? segment : segment.args, l = n.length,
				result = new Array(l);
			for(var i = 0; i < l; ++i){
				result[i] = (last += n[i]);
			}
			return result;
		},
		_moveToA: function(segment){
			var p = [" m"], n = segment instanceof Array ? segment : segment.args, l = n.length;
			this._addArgs(p, n, 0, 2);
			if(l > 2){
				p.push(" l");
				this._addArgs(p, n, 2, l);
			}
			this.lastControl.type = "";	// no control point after this primitive
			return p;
		},
		_moveToR: function(segment, last){
			return this._moveToA(this._adjustRelCrd(last, segment));
		},
		_lineToA: function(segment){
			var p = [" l"], n = segment instanceof Array ? segment : segment.args;
			this._addArgs(p, n, 0, n.length);
			this.lastControl.type = "";	// no control point after this primitive
			return p;
		},
		_lineToR: function(segment, last){
			return this._lineToA(this._adjustRelCrd(last, segment));
		},
		_hLineToA: function(segment, last){
			var p = [" l"], y = " " + last.y.toFixed(),
				n = segment instanceof Array ? segment : segment.args, l = n.length;
			for(var i = 0; i < l; ++i){
				p.push(" ", n[i].toFixed(), y);
			}
			this.lastControl.type = "";	// no control point after this primitive
			return p;
		},
		_hLineToR: function(segment, last){
			return this._hLineToA(this._adjustRelPos(last.x, segment), last);
		},
		_vLineToA: function(segment, last){
			var p = [" l"], x = " " + last.x.toFixed(),
				n = segment instanceof Array ? segment : segment.args, l = n.length;
			for(var i = 0; i < l; ++i){
				p.push(x, " ", n[i].toFixed());
			}
			this.lastControl.type = "";	// no control point after this primitive
			return p;
		},
		_vLineToR: function(segment, last){
			return this._vLineToA(this._adjustRelPos(last.y, segment), last);
		},
		_curveToA: function(segment){
			var p = [], n = segment instanceof Array ? segment : segment.args, l = n.length,
				lc = this.lastControl;
			for(var i = 0; i < l; i += 6){
				p.push(" c");
				this._addArgs(p, n, i, i + 6);
			}
			lc.x = n[l - 4];
			lc.y = n[l - 3];
			lc.type = "C";
			return p;
		},
		_curveToR: function(segment, last){
			return this._curveToA(this._adjustRelCrd(last, segment, 6));
		},
		_smoothCurveToA: function(segment, last){
			var p = [], n = segment instanceof Array ? segment : segment.args, l = n.length,
				lc = this.lastControl, i = 0;
			if(lc.type != "C"){
				p.push(" c");
				this._addArgs(p, [last.x, last.y], 0, 2);
				this._addArgs(p, n, 0, 4);
				lc.x = n[0];
				lc.y = n[1];
				lc.type = "C";
				i = 4;
			}
			for(; i < l; i += 4){
				p.push(" c");
				this._addArgs(p, [
					2 * last.x - lc.x,
					2 * last.y - lc.y
				], 0, 2);
				this._addArgs(p, n, i, i + 4);
				lc.x = n[i];
				lc.y = n[i + 1];
			}
			return p;
		},
		_smoothCurveToR: function(segment, last){
			return this._smoothCurveToA(this._adjustRelCrd(last, segment, 4), last);
		},
		_qCurveToA: function(segment){
			var p = [], n = segment instanceof Array ? segment : segment.args, l = n.length,
				lc = this.lastControl;
			for(var i = 0; i < l; i += 4){
				p.push(" qb");
				this._addArgs(p, n, i, i + 4);
			}
			lc.x = n[l - 4];
			lc.y = n[l - 3];
			lc.type = "Q";
			return p;
		},
		_qCurveToR: function(segment, last){
			return this._qCurveToA(this._adjustRelCrd(last, segment, 4));
		},
		_qSmoothCurveToA: function(segment, last){
			var p = [], n = segment instanceof Array ? segment : segment.args, l = n.length,
				lc = this.lastControl, i = 0;
			if(lc.type != "Q"){
				p.push(" qb");
				this._addArgs(p, [
					lc.x = last.x,
					lc.y = last.y
				], 0, 2);
				lc.type = "Q";
				this._addArgs(p, n, 0, 2);
				i = 2;
			}
			for(; i < l; i += 2){
				p.push(" qb");
				this._addArgs(p, [
					lc.x = 2 * last.x - lc.x,
					lc.y = 2 * last.y - lc.y
				], 0, 2);
				this._addArgs(p, n, i, i + 2);
			}
			return p;
		},
		_qSmoothCurveToR: function(segment, last){
			return this._qSmoothCurveToA(this._adjustRelCrd(last, segment, 2), last);
		},
		_arcTo: function(segment, last){
			var p = [], n = segment.args, l = n.length, relative = segment.action == "a";
			for(var i = 0; i < l; i += 7){
				var x1 = n[i + 5], y1 = n[i + 6];
				if(relative){
					x1 += last.x;
					y1 += last.y;
				}
				var result = g.arc.arcAsBezier(
					last, n[i], n[i + 1], n[i + 2],
					n[i + 3] ? 1 : 0, n[i + 4] ? 1 : 0,
					x1, y1
				);
				for(var j = 0; j < result.length; ++j){
					p.push(" c");
					var t = result[j];
					this._addArgs(p, t, 0, t.length);
					this._updateBBox(t[0], t[1]);
					this._updateBBox(t[2], t[3]);
					this._updateBBox(t[4], t[5]);
				}
				last.x = x1;
				last.y = y1;
			}
			this.lastControl.type = "";	// no control point after this primitive
			return p;
		},
		_closePath: function(){
			this.lastControl.type = "";	// no control point after this primitive
			return ["x"];
		}
	});
	g.Path.nodeType = "shape";

	dojo.declare("dojox.gfx.TextPath", g.Path, {
		// summary: a textpath shape (VML)
		constructor: function(rawNode){
			if(rawNode){rawNode.setAttribute("dojoGfxType", "textpath");}
			this.fontStyle = null;
			if(!("text" in this)){
				this.text = d.clone(g.defaultTextPath);
			}
			if(!("fontStyle" in this)){
				this.fontStyle = d.clone(g.defaultFont);
			}
		},
		setText: function(newText){
			// summary: sets a text to be drawn along the path
			this.text = g.makeParameters(this.text,
				typeof newText == "string" ? {text: newText} : newText);
			this._setText();
			return this;	// self
		},
		setFont: function(newFont){
			// summary: sets a font for text
			this.fontStyle = typeof newFont == "string" ?
				g.splitFontString(newFont) :
				g.makeParameters(g.defaultFont, newFont);
			this._setFont();
			return this;	// self
		},

		_setText: function(){
			// summary: sets a text shape object (VML)
			this.bbox = null;
			var r = this.rawNode, s = this.text,
				// find path and text path
				p = null, t = null, c = r.childNodes;
			for(var i = 0; i < c.length; ++i){
				var tag = c[i].tagName;
				if(tag == "path"){
					p = c[i];
					if(t) break;
				}else if(tag == "textpath"){
					t = c[i];
					if(p) break;
				}
			}
			if(!p){
				p = this.rawNode.ownerDocument.createElement("v:path");
				r.appendChild(p);
			}
			if(!t){
				t = this.rawNode.ownerDocument.createElement("v:textpath");
				r.appendChild(t);
			}
			p.textPathOk = true;
			t.on = true;
			var a = vml.text_alignment[s.align];
			t.style["v-text-align"] = a ? a : "left";
			t.style["text-decoration"] = s.decoration;
			t.style["v-rotate-letters"] = s.rotated;
			t.style["v-text-kern"] = s.kerning;
			t.string = s.text;
		},
		_setFont: function(){
			// summary: sets a font object (VML)
			var f = this.fontStyle, c = this.rawNode.childNodes;
			for(var i = 0; i < c.length; ++i){
				if(c[i].tagName == "textpath"){
					c[i].style.font = g.makeFontString(f);
					break;
				}
			}
		}
	});
	g.TextPath.nodeType = "shape";

	dojo.declare("dojox.gfx.Surface", gs.Surface, {
		// summary: a surface object to be used for drawings (VML)
		constructor: function(){
			vml.Container._init.call(this);
		},
		setDimensions: function(width, height){
			// summary: sets the width and height of the rawNode
			// width: String: width of surface, e.g., "100px"
			// height: String: height of surface, e.g., "100px"
			this.width  = g.normalizedLength(width);	// in pixels
			this.height = g.normalizedLength(height);	// in pixels
			if(!this.rawNode) return this;
			var cs = this.clipNode.style,
				r = this.rawNode, rs = r.style,
				bs = this.bgNode.style,
				ps = this._parent.style, i;
			ps.width = width;
			ps.height = height;
			cs.width  = width;
			cs.height = height;
			cs.clip = "rect(0px " + width + "px " + height + "px 0px)";
			rs.width = width;
			rs.height = height;
			r.coordsize = width + " " + height;
			bs.width = width;
			bs.height = height;
			for(i = 0; i < this.children.length; ++i){
				this.children[i]._setDimensions(width, height);
			}
			return this;	// self
		},
		getDimensions: function(){
			// summary: returns an object with properties "width" and "height"
			var t = this.rawNode ? {
				width:  g.normalizedLength(this.rawNode.style.width),
				height: g.normalizedLength(this.rawNode.style.height)} : null;
			if(t.width  <= 0){ t.width  = this.width; }
			if(t.height <= 0){ t.height = this.height; }
			return t;	// Object
		}
	});

	g.createSurface = function(parentNode, width, height){
		// summary: creates a surface (VML)
		// parentNode: Node: a parent node
		// width: String: width of surface, e.g., "100px"
		// height: String: height of surface, e.g., "100px"

		if(!width && !height){
			var pos = d.position(parentNode);
			width  = width  || pos.w;
			height = height || pos.h;
		}
		if(typeof width == "number"){
			width = width + "px";
		}
		if(typeof height == "number"){
			height = height + "px";
		}

		var s = new g.Surface(), p = d.byId(parentNode),
			c = s.clipNode = p.ownerDocument.createElement("div"),
			r = s.rawNode = p.ownerDocument.createElement("v:group"),
			cs = c.style, rs = r.style;

		if(d.isIE > 7){
			rs.display = "inline-block";
		}

		s._parent = p;
		s._nodes.push(c);	// other elements will be deleted as parts of "c"

		p.style.width  = width;
		p.style.height = height;

		cs.position = "absolute";
		cs.width  = width;
		cs.height = height;
		cs.clip = "rect(0px " + width + " " + height + " 0px)";
		rs.position = "absolute";
		rs.width  = width;
		rs.height = height;
		r.coordsize = (width === "100%" ? width : parseFloat(width)) + " " +
			(height === "100%" ? height : parseFloat(height));
		r.coordorigin = "0 0";

		// create a background rectangle, which is required to show all other shapes
		var b = s.bgNode = r.ownerDocument.createElement("v:rect"), bs = b.style;
		bs.left = bs.top = 0;
		bs.width  = rs.width;
		bs.height = rs.height;
		b.filled = b.stroked = "f";

		r.appendChild(b);
		c.appendChild(r);
		p.appendChild(c);

		s.width  = g.normalizedLength(width);	// in pixels
		s.height = g.normalizedLength(height);	// in pixels

		return s;	// dojox.gfx.Surface
	};

	// Extenders


	vml.Container = {
		_init: function(){
			gs.Container._init.call(this);
		},
		add: function(shape){
			// summary: adds a shape to a group/surface
			// shape: dojox.gfx.Shape: an VML shape object
			if(this != shape.getParent()){
				this.rawNode.appendChild(shape.rawNode);
				if(!shape.getParent()){
					// reapply visual attributes
					shape.setFill(shape.getFill());
					shape.setStroke(shape.getStroke());
				}
				//dojox.gfx.Group.superclass.add.apply(this, arguments);
				//this.inherited(arguments);
				gs.Container.add.apply(this, arguments);
			}
			return this;	// self
		},
		remove: function(shape, silently){
			// summary: remove a shape from a group/surface
			// shape: dojox.gfx.Shape: an VML shape object
			// silently: Boolean?: if true, regenerate a picture
			if(this == shape.getParent()){
				if(this.rawNode == shape.rawNode.parentNode){
					this.rawNode.removeChild(shape.rawNode);
				}
				//dojox.gfx.Group.superclass.remove.apply(this, arguments);
				//this.inherited(arguments);
				gs.Container.remove.apply(this, arguments);
			}
			return this;	// self
		},
		clear: function(){
			// summary: removes all shapes from a group/surface
			var r = this.rawNode;
			while(r.firstChild != r.lastChild){
				if(r.firstChild != this.bgNode){
					r.removeChild(r.firstChild);
				}
				if(r.lastChild != this.bgNode){
					r.removeChild(r.lastChild);
				}
			}
			//return this.inherited(arguments);	// self
			return gs.Container.clear.apply(this, arguments);
		},
		_moveChildToFront: gs.Container._moveChildToFront,
		_moveChildToBack:  gs.Container._moveChildToBack
	};

	dojo.mixin(gs.Creator, {
		// summary: VML shape creators
		createGroup: function(){
			// summary: creates a VML group shape
			var node = this.createObject(g.Group, null);	// dojox.gfx.Group
			// create a background rectangle, which is required to show all other shapes
			var r = node.rawNode.ownerDocument.createElement("v:rect");
			r.style.left = r.style.top = 0;
			r.style.width  = node.rawNode.style.width;
			r.style.height = node.rawNode.style.height;
			r.filled = r.stroked = "f";
			node.rawNode.appendChild(r);
			node.bgNode = r;
			return node;	// dojox.gfx.Group
		},
		createImage: function(image){
			// summary: creates a VML image shape
			// image: Object: an image object (see dojox.gfx.defaultImage)
			if(!this.rawNode) return null;
			var shape = new g.Image(),
				doc = this.rawNode.ownerDocument,
				node = doc.createElement('v:rect');
			node.stroked = "f";
			node.style.width  = this.rawNode.style.width;
			node.style.height = this.rawNode.style.height;
			var img  = doc.createElement('v:imagedata');
			node.appendChild(img);
			shape.setRawNode(node);
			this.rawNode.appendChild(node);
			shape.setShape(image);
			this.add(shape);
			return shape;	// dojox.gfx.Image
		},
		createRect: function(rect){
			// summary: creates a rectangle shape
			// rect: Object: a path object (see dojox.gfx.defaultRect)
			if(!this.rawNode) return null;
			var shape = new g.Rect,
				node = this.rawNode.ownerDocument.createElement("v:roundrect");
			if(d.isIE > 7){
				node.style.display = "inline-block";
			}
			shape.setRawNode(node);
			this.rawNode.appendChild(node);
			shape.setShape(rect);
			this.add(shape);
			return shape;	// dojox.gfx.Rect
		},
		createObject: function(shapeType, rawShape) {
			// summary: creates an instance of the passed shapeType class
			// shapeType: Function: a class constructor to create an instance of
			// rawShape: Object: properties to be passed in to the classes "setShape" method
			// overrideSize: Boolean: set the size explicitly, if true
			if(!this.rawNode) return null;
			var shape = new shapeType(),
				node = this.rawNode.ownerDocument.createElement('v:' + shapeType.nodeType);
			shape.setRawNode(node);
			this.rawNode.appendChild(node);
			switch(shapeType){
				case g.Group:
				case g.Line:
				case g.Polyline:
				case g.Image:
				case g.Text:
				case g.Path:
				case g.TextPath:
					this._overrideSize(node);
			}
			shape.setShape(rawShape);
			this.add(shape);
			return shape;	// dojox.gfx.Shape
		},
		_overrideSize: function(node){
			var s = this.rawNode.style, w = s.width, h = s.height;
			node.style.width  = w;
			node.style.height = h;
			node.coordsize = parseInt(w) + " " + parseInt(h);
		}
	});

	d.extend(g.Group, vml.Container);
	d.extend(g.Group, gs.Creator);

	d.extend(g.Surface, vml.Container);
	d.extend(g.Surface, gs.Creator);

})();

}

if(!dojo._hasResource["dojox.json.schema"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.json.schema"] = true;
dojo.provide("dojox.json.schema");


dojox.json.schema.validate = function(/*Any*/instance,/*Object*/schema){
	// summary:
	//  	To use the validator call this with an instance object and an optional schema object.
	// 		If a schema is provided, it will be used to validate. If the instance object refers to a schema (self-validating), 
	// 		that schema will be used to validate and the schema parameter is not necessary (if both exist, 
	// 		both validations will occur).
	//	instance:
	//		The instance value/object to validate
	// schema:
	//		The schema to use to validate
	// description: 
	// 		The validate method will return an object with two properties:
	// 			valid: A boolean indicating if the instance is valid by the schema
	// 			errors: An array of validation errors. If there are no errors, then an 
	// 					empty list will be returned. A validation error will have two properties: 
	// 						property: which indicates which property had the error
	// 						message: which indicates what the error was
	//
	return this._validate(instance,schema,false);
};
dojox.json.schema.checkPropertyChange = function(/*Any*/value,/*Object*/schema, /*String*/ property){
	// summary:
	// 		The checkPropertyChange method will check to see if an value can legally be in property with the given schema
	// 		This is slightly different than the validate method in that it will fail if the schema is readonly and it will
	// 		not check for self-validation, it is assumed that the passed in value is already internally valid.  
	// 		The checkPropertyChange method will return the same object type as validate, see JSONSchema.validate for 
	// 		information.
	//	value:
	//		The new instance value/object to check
	// schema:
	//		The schema to use to validate
	// return: 
	// 		see dojox.validate.jsonSchema.validate
	//
	return this._validate(value,schema, property || "property");
};
dojox.json.schema.mustBeValid = function(result){
	//	summary:
	//		This checks to ensure that the result is valid and will throw an appropriate error message if it is not
	// result: the result returned from checkPropertyChange or validate
	if(!result.valid){
		throw new TypeError(dojo.map(result.errors,function(error){return "for property " + error.property + ': ' + error.message;}).join(", "));
	}	
}
dojox.json.schema._validate = function(/*Any*/instance,/*Object*/schema,/*Boolean*/ _changing){
	
	var errors = [];
		// validate a value against a property definition
	function checkProp(value, schema, path,i){
		var l;
		path += path ? typeof i == 'number' ? '[' + i + ']' : typeof i == 'undefined' ? '' : '.' + i : i;
		function addError(message){
			errors.push({property:path,message:message});
		}
		
		if((typeof schema != 'object' || schema instanceof Array) && (path || typeof schema != 'function')){
			if(typeof schema == 'function'){
				if(!(Object(value) instanceof schema)){
					addError("is not an instance of the class/constructor " + schema.name);
				}
			}else if(schema){
				addError("Invalid schema/property definition " + schema);
			}
			return null;
		}			
		if(_changing && schema.readonly){
			addError("is a readonly field, it can not be changed");
		}
		if(schema['extends']){ // if it extends another schema, it must pass that schema as well
			checkProp(value,schema['extends'],path,i);
		}
		// validate a value against a type definition
		function checkType(type,value){
			if(type){
				if(typeof type == 'string' && type != 'any' && 
						(type == 'null' ? value !== null : typeof value != type) && 
						!(value instanceof Array && type == 'array') &&
						!(type == 'integer' && value%1===0)){
					return [{property:path,message:(typeof value) + " value found, but a " + type + " is required"}];
				}
				if(type instanceof Array){
					var unionErrors=[];
					for(var j = 0; j < type.length; j++){ // a union type 
						if(!(unionErrors=checkType(type[j],value)).length){
							break;
						}
					}
					if(unionErrors.length){
						return unionErrors;
					}
				}else if(typeof type == 'object'){
					var priorErrors = errors;
					errors = []; 
					checkProp(value,type,path);
					var theseErrors = errors;
					errors = priorErrors;
					return theseErrors; 
				} 
			}
			return [];
		}
		if(value === undefined){
			if(!schema.optional){  
				addError("is missing and it is not optional");
			}
		}else{
			errors = errors.concat(checkType(schema.type,value));
			if(schema.disallow && !checkType(schema.disallow,value).length){
				addError(" disallowed value was matched");
			}
			if(value !== null){
				if(value instanceof Array){
					if(schema.items){
						if(schema.items instanceof Array){
							for(i=0,l=value.length; i<l; i++){
								errors.concat(checkProp(value[i],schema.items[i],path,i));
							}
						}else{
							for(i=0,l=value.length; i<l; i++){
								errors.concat(checkProp(value[i],schema.items,path,i));
							}
						}							
					}
					if(schema.minItems && value.length < schema.minItems){
						addError("There must be a minimum of " + schema.minItems + " in the array");
					}
					if(schema.maxItems && value.length > schema.maxItems){
						addError("There must be a maximum of " + schema.maxItems + " in the array");
					}
				}else if(schema.properties){
					errors.concat(checkObj(value,schema.properties,path,schema.additionalProperties));
				}
				if(schema.pattern && typeof value == 'string' && !value.match(schema.pattern)){
					addError("does not match the regex pattern " + schema.pattern);
				}
				if(schema.maxLength && typeof value == 'string' && value.length > schema.maxLength){
					addError("may only be " + schema.maxLength + " characters long");
				}
				if(schema.minLength && typeof value == 'string' && value.length < schema.minLength){
					addError("must be at least " + schema.minLength + " characters long");
				}
				if(typeof schema.minimum !== undefined && typeof value == typeof schema.minimum && 
						schema.minimum > value){
					addError("must have a minimum value of " + schema.minimum);
				}
				if(typeof schema.maximum !== undefined && typeof value == typeof schema.maximum && 
						schema.maximum < value){
					addError("must have a maximum value of " + schema.maximum);
				}
				if(schema['enum']){
					var enumer = schema['enum'];
					l = enumer.length;
					var found;
					for(var j = 0; j < l; j++){
						if(enumer[j]===value){
							found=1;
							break;
						}
					}
					if(!found){
						addError("does not have a value in the enumeration " + enumer.join(", "));
					}
				}
				if(typeof schema.maxDecimal == 'number' && 
					(value.toString().match(new RegExp("\\.[0-9]{" + (schema.maxDecimal + 1) + ",}")))){
					addError("may only have " + schema.maxDecimal + " digits of decimal places");
				}
			}
		}
		return null;
	}
	// validate an object against a schema
	function checkObj(instance,objTypeDef,path,additionalProp){
	
		if(typeof objTypeDef =='object'){
			if(typeof instance != 'object' || instance instanceof Array){
				errors.push({property:path,message:"an object is required"});
			}
			
			for(var i in objTypeDef){ 
				if(objTypeDef.hasOwnProperty(i) && !(i.charAt(0) == '_' && i.charAt(1) == '_')){
					var value = instance[i];
					var propDef = objTypeDef[i];
					checkProp(value,propDef,path,i);
				}
			}
		}
		for(i in instance){
			if(instance.hasOwnProperty(i) && !(i.charAt(0) == '_' && i.charAt(1) == '_') && objTypeDef && !objTypeDef[i] && additionalProp===false){
				errors.push({property:path,message:(typeof value) + "The property " + i +
						" is not defined in the schema and the schema does not allow additional properties"});
			}
			var requires = objTypeDef && objTypeDef[i] && objTypeDef[i].requires;
			if(requires && !(requires in instance)){
				errors.push({property:path,message:"the presence of the property " + i + " requires that " + requires + " also be present"});
			}
			value = instance[i];
			if(objTypeDef && typeof objTypeDef == 'object' && !(i in objTypeDef)){
				checkProp(value,additionalProp,path,i); 
			}
			if(!_changing && value && value.$schema){
				errors = errors.concat(checkProp(value,value.$schema,path,i));
			}
		}
		return errors;
	}
	if(schema){
		checkProp(instance,schema,'',_changing || '');
	}
	if(!_changing && instance && instance.$schema){
		checkProp(instance,instance.$schema,'','');
	}
	return {valid:!errors.length,errors:errors};
};


}

if(!dojo._hasResource["dojo.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.window"] = true;
dojo.provide("dojo.window");

dojo.window.getBox = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	var scrollRoot = (dojo.doc.compatMode == 'BackCompat') ? dojo.body() : dojo.doc.documentElement;

	// get scroll position
	var scroll = dojo._docScroll(); // scrollRoot.scrollTop/Left should work
	return { w: scrollRoot.clientWidth, h: scrollRoot.clientHeight, l: scroll.x, t: scroll.y };
};

dojo.window.get = function(doc){
	// summary:
	// 		Get window object associated with document doc

	// In some IE versions (at least 6.0), document.parentWindow does not return a
	// reference to the real window object (maybe a copy), so we must fix it as well
	// We use IE specific execScript to attach the real window reference to
	// document._parentWindow for later use
	if(dojo.isIE && window !== document.parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc.parentWindow || doc.defaultView;	//	Window
};

dojo.window.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view, if it is not already.
	
	// don't rely on node.scrollIntoView working just because the function is there

	try{ // catch unexpected/unrecreatable errors (#7808) since we can recover using a semi-acceptable native method
		node = dojo.byId(node);
		var doc = node.ownerDocument || dojo.doc,
			body = doc.body || dojo.body(),
			html = doc.documentElement || body.parentNode,
			isIE = dojo.isIE, isWK = dojo.isWebKit;
		// if an untested browser, then use the native method
		if((!(dojo.isMoz || isIE || isWK || dojo.isOpera) || node == body || node == html) && (typeof node.scrollIntoView != "undefined")){
			node.scrollIntoView(false); // short-circuit to native if possible
			return;
		}
		var backCompat = doc.compatMode == 'BackCompat',
			clientAreaRoot = backCompat? body : html,
			scrollRoot = isWK ? body : clientAreaRoot,
			rootWidth = clientAreaRoot.clientWidth,
			rootHeight = clientAreaRoot.clientHeight,
			rtl = !dojo._isBodyLtr(),
			nodePos = pos || dojo.position(node),
			el = node.parentNode,
			isFixed = function(el){
				return ((isIE <= 6 || (isIE && backCompat))? false : (dojo.style(el, 'position').toLowerCase() == "fixed"));
			};
		if(isFixed(node)){ return; } // nothing to do

		while(el){
			if(el == body){ el = scrollRoot; }
			var elPos = dojo.position(el),
				fixedPos = isFixed(el);
	
			if(el == scrollRoot){
				elPos.w = rootWidth; elPos.h = rootHeight;
				if(scrollRoot == html && isIE && rtl){ elPos.x += scrollRoot.offsetWidth-elPos.w; } // IE workaround where scrollbar causes negative x
				if(elPos.x < 0 || !isIE){ elPos.x = 0; } // IE can have values > 0
				if(elPos.y < 0 || !isIE){ elPos.y = 0; }
			}else{
				var pb = dojo._getPadBorderExtents(el);
				elPos.w -= pb.w; elPos.h -= pb.h; elPos.x += pb.l; elPos.y += pb.t;
			}
	
			if(el != scrollRoot){ // body, html sizes already have the scrollbar removed
				var clientSize = el.clientWidth,
					scrollBarSize = elPos.w - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					elPos.w = clientSize;
					if(isIE && rtl){ elPos.x += scrollBarSize; }
				}
				clientSize = el.clientHeight;
				scrollBarSize = elPos.h - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					elPos.h = clientSize;
				}
			}
			if(fixedPos){ // bounded by viewport, not parents
				if(elPos.y < 0){
					elPos.h += elPos.y; elPos.y = 0;
				}
				if(elPos.x < 0){
					elPos.w += elPos.x; elPos.x = 0;
				}
				if(elPos.y + elPos.h > rootHeight){
					elPos.h = rootHeight - elPos.y;
				}
				if(elPos.x + elPos.w > rootWidth){
					elPos.w = rootWidth - elPos.x;
				}
			}
			// calculate overflow in all 4 directions
			var l = nodePos.x - elPos.x, // beyond left: < 0
				t = nodePos.y - Math.max(elPos.y, 0), // beyond top: < 0
				r = l + nodePos.w - elPos.w, // beyond right: > 0
				bot = t + nodePos.h - elPos.h; // beyond bottom: > 0
			if(r * l > 0){
				var s = Math[l < 0? "max" : "min"](l, r);
				nodePos.x += el.scrollLeft;
				el.scrollLeft += (isIE >= 8 && !backCompat && rtl)? -s : s;
				nodePos.x -= el.scrollLeft;
			}
			if(bot * t > 0){
				nodePos.y += el.scrollTop;
				el.scrollTop += Math[t < 0? "max" : "min"](t, bot);
				nodePos.y -= el.scrollTop;
			}
			el = (el != scrollRoot) && !fixedPos && el.parentNode;
		}	
	}catch(error){
		console.error('scrollIntoView: ' + error);
		node.scrollIntoView(false);
	}
};

}

if(!dojo._hasResource["dijit._base.manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.manager"] = true;
dojo.provide("dijit._base.manager");

dojo.declare("dijit.WidgetSet", null, {
	// summary:
	//		A set of widgets indexed by id. A default instance of this class is
	//		available as `dijit.registry`
	//
	// example:
	//		Create a small list of widgets:
	//		|	var ws = new dijit.WidgetSet();
	//		|	ws.add(dijit.byId("one"));
	//		| 	ws.add(dijit.byId("two"));
	//		|	// destroy both:
	//		|	ws.forEach(function(w){ w.destroy(); });
	//
	// example:
	//		Using dijit.registry:
	//		|	dijit.registry.forEach(function(w){ /* do something */ });

	constructor: function(){
		this._hash = {};
		this.length = 0;
	},

	add: function(/*dijit._Widget*/ widget){
		// summary:
		//		Add a widget to this list. If a duplicate ID is detected, a error is thrown.
		//
		// widget: dijit._Widget
		//		Any dijit._Widget subclass.
		if(this._hash[widget.id]){
			throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
		}
		this._hash[widget.id] = widget;
		this.length++;
	},

	remove: function(/*String*/ id){
		// summary:
		//		Remove a widget from this WidgetSet. Does not destroy the widget; simply
		//		removes the reference.
		if(this._hash[id]){
			delete this._hash[id];
			this.length--;
		}
	},

	forEach: function(/*Function*/ func, /* Object? */thisObj){
		// summary:
		//		Call specified function for each widget in this set.
		//
		// func:
		//		A callback function to run for each item. Is passed the widget, the index
		//		in the iteration, and the full hash, similar to `dojo.forEach`.
		//
		// thisObj:
		//		An optional scope parameter
		//
		// example:
		//		Using the default `dijit.registry` instance:
		//		|	dijit.registry.forEach(function(widget){
		//		|		console.log(widget.declaredClass);
		//		|	});
		//
		// returns:
		//		Returns self, in order to allow for further chaining.

		thisObj = thisObj || dojo.global;
		var i = 0, id;
		for(id in this._hash){
			func.call(thisObj, this._hash[id], i++, this._hash);
		}
		return this;	// dijit.WidgetSet
	},

	filter: function(/*Function*/ filter, /* Object? */thisObj){
		// summary:
		//		Filter down this WidgetSet to a smaller new WidgetSet
		//		Works the same as `dojo.filter` and `dojo.NodeList.filter`
		//
		// filter:
		//		Callback function to test truthiness. Is passed the widget
		//		reference and the pseudo-index in the object.
		//
		// thisObj: Object?
		//		Option scope to use for the filter function.
		//
		// example:
		//		Arbitrary: select the odd widgets in this list
		//		|	dijit.registry.filter(function(w, i){
		//		|		return i % 2 == 0;
		//		|	}).forEach(function(w){ /* odd ones */ });

		thisObj = thisObj || dojo.global;
		var res = new dijit.WidgetSet(), i = 0, id;
		for(id in this._hash){
			var w = this._hash[id];
			if(filter.call(thisObj, w, i++, this._hash)){
				res.add(w);
			}
		}
		return res; // dijit.WidgetSet
	},

	byId: function(/*String*/ id){
		// summary:
		//		Find a widget in this list by it's id.
		// example:
		//		Test if an id is in a particular WidgetSet
		//		| var ws = new dijit.WidgetSet();
		//		| ws.add(dijit.byId("bar"));
		//		| var t = ws.byId("bar") // returns a widget
		//		| var x = ws.byId("foo"); // returns undefined

		return this._hash[id];	// dijit._Widget
	},

	byClass: function(/*String*/ cls){
		// summary:
		//		Reduce this widgetset to a new WidgetSet of a particular `declaredClass`
		//
		// cls: String
		//		The Class to scan for. Full dot-notated string.
		//
		// example:
		//		Find all `dijit.TitlePane`s in a page:
		//		|	dijit.registry.byClass("dijit.TitlePane").forEach(function(tp){ tp.close(); });

		var res = new dijit.WidgetSet(), id, widget;
		for(id in this._hash){
			widget = this._hash[id];
			if(widget.declaredClass == cls){
				res.add(widget);
			}
		 }
		 return res; // dijit.WidgetSet
},

	toArray: function(){
		// summary:
		//		Convert this WidgetSet into a true Array
		//
		// example:
		//		Work with the widget .domNodes in a real Array
		//		|	dojo.map(dijit.registry.toArray(), function(w){ return w.domNode; });

		var ar = [];
		for(var id in this._hash){
			ar.push(this._hash[id]);
		}
		return ar;	// dijit._Widget[]
},

	map: function(/* Function */func, /* Object? */thisObj){
		// summary:
		//		Create a new Array from this WidgetSet, following the same rules as `dojo.map`
		// example:
		//		|	var nodes = dijit.registry.map(function(w){ return w.domNode; });
		//
		// returns:
		//		A new array of the returned values.
		return dojo.map(this.toArray(), func, thisObj); // Array
	},

	every: function(func, thisObj){
		// summary:
		// 		A synthetic clone of `dojo.every` acting explicitly on this WidgetSet
		//
		// func: Function
		//		A callback function run for every widget in this list. Exits loop
		//		when the first false return is encountered.
		//
		// thisObj: Object?
		//		Optional scope parameter to use for the callback

		thisObj = thisObj || dojo.global;
		var x = 0, i;
		for(i in this._hash){
			if(!func.call(thisObj, this._hash[i], x++, this._hash)){
				return false; // Boolean
			}
		}
		return true; // Boolean
	},

	some: function(func, thisObj){
		// summary:
		// 		A synthetic clone of `dojo.some` acting explictly on this WidgetSet
		//
		// func: Function
		//		A callback function run for every widget in this list. Exits loop
		//		when the first true return is encountered.
		//
		// thisObj: Object?
		//		Optional scope parameter to use for the callback

		thisObj = thisObj || dojo.global;
		var x = 0, i;
		for(i in this._hash){
			if(func.call(thisObj, this._hash[i], x++, this._hash)){
				return true; // Boolean
			}
		}
		return false; // Boolean
	}

});

(function(){

	/*=====
	dijit.registry = {
		// summary:
		//		A list of widgets on a page.
		// description:
		//		Is an instance of `dijit.WidgetSet`
	};
	=====*/
	dijit.registry = new dijit.WidgetSet();

	var hash = dijit.registry._hash,
		attr = dojo.attr,
		hasAttr = dojo.hasAttr,
		style = dojo.style;

	dijit.byId = function(/*String|dijit._Widget*/ id){
		// summary:
		//		Returns a widget by it's id, or if passed a widget, no-op (like dojo.byId())
		return typeof id == "string" ? hash[id] : id; // dijit._Widget
	};

	var _widgetTypeCtr = {};
	dijit.getUniqueId = function(/*String*/widgetType){
		// summary:
		//		Generates a unique id for a given widgetType
	
		var id;
		do{
			id = widgetType + "_" +
				(widgetType in _widgetTypeCtr ?
					++_widgetTypeCtr[widgetType] : _widgetTypeCtr[widgetType] = 0);
		}while(hash[id]);
		return dijit._scopeName == "dijit" ? id : dijit._scopeName + "_" + id; // String
	};
	
	dijit.findWidgets = function(/*DomNode*/ root){
		// summary:
		//		Search subtree under root returning widgets found.
		//		Doesn't search for nested widgets (ie, widgets inside other widgets).
	
		var outAry = [];
	
		function getChildrenHelper(root){
			for(var node = root.firstChild; node; node = node.nextSibling){
				if(node.nodeType == 1){
					var widgetId = node.getAttribute("widgetId");
					if(widgetId){
						outAry.push(hash[widgetId]);
					}else{
						getChildrenHelper(node);
					}
				}
			}
		}
	
		getChildrenHelper(root);
		return outAry;
	};
	
	dijit._destroyAll = function(){
		// summary:
		//		Code to destroy all widgets and do other cleanup on page unload
	
		// Clean up focus manager lingering references to widgets and nodes
		dijit._curFocus = null;
		dijit._prevFocus = null;
		dijit._activeStack = [];
	
		// Destroy all the widgets, top down
		dojo.forEach(dijit.findWidgets(dojo.body()), function(widget){
			// Avoid double destroy of widgets like Menu that are attached to <body>
			// even though they are logically children of other widgets.
			if(!widget._destroyed){
				if(widget.destroyRecursive){
					widget.destroyRecursive();
				}else if(widget.destroy){
					widget.destroy();
				}
			}
		});
	};
	
	if(dojo.isIE){
		// Only run _destroyAll() for IE because we think it's only necessary in that case,
		// and because it causes problems on FF.  See bug #3531 for details.
		dojo.addOnWindowUnload(function(){
			dijit._destroyAll();
		});
	}
	
	dijit.byNode = function(/*DOMNode*/ node){
		// summary:
		//		Returns the widget corresponding to the given DOMNode
		return hash[node.getAttribute("widgetId")]; // dijit._Widget
	};
	
	dijit.getEnclosingWidget = function(/*DOMNode*/ node){
		// summary:
		//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
		//		the node is not contained within the DOM tree of any widget
		while(node){
			var id = node.getAttribute && node.getAttribute("widgetId");
			if(id){
				return hash[id];
			}
			node = node.parentNode;
		}
		return null;
	};

	var shown = (dijit._isElementShown = function(/*Element*/ elem){
		var s = style(elem);
		return (s.visibility != "hidden")
			&& (s.visibility != "collapsed")
			&& (s.display != "none")
			&& (attr(elem, "type") != "hidden");
	});
	
	dijit.hasDefaultTabStop = function(/*Element*/ elem){
		// summary:
		//		Tests if element is tab-navigable even without an explicit tabIndex setting
	
		// No explicit tabIndex setting, need to investigate node type
		switch(elem.nodeName.toLowerCase()){
			case "a":
				// An <a> w/out a tabindex is only navigable if it has an href
				return hasAttr(elem, "href");
			case "area":
			case "button":
			case "input":
			case "object":
			case "select":
			case "textarea":
				// These are navigable by default
				return true;
			case "iframe":
				// If it's an editor <iframe> then it's tab navigable.
				//TODO: feature detect "designMode" in elem.contentDocument?
				if(dojo.isMoz){
					try{
						return elem.contentDocument.designMode == "on";
					}catch(err){
						return false;
					}
				}else if(dojo.isWebKit){
					var doc = elem.contentDocument,
						body = doc && doc.body;
					return body && body.contentEditable == 'true';
				}else{
					// contentWindow.document isn't accessible within IE7/8
					// if the iframe.src points to a foreign url and this
					// page contains an element, that could get focus
					try{
						doc = elem.contentWindow.document;
						body = doc && doc.body;
						return body && body.firstChild && body.firstChild.contentEditable == 'true';
					}catch(e){
						return false;
					}
				}
			default:
				return elem.contentEditable == 'true';
		}
	};
	
	var isTabNavigable = (dijit.isTabNavigable = function(/*Element*/ elem){
		// summary:
		//		Tests if an element is tab-navigable
	
		// TODO: convert (and rename method) to return effective tabIndex; will save time in _getTabNavigable()
		if(attr(elem, "disabled")){
			return false;
		}else if(hasAttr(elem, "tabIndex")){
			// Explicit tab index setting
			return attr(elem, "tabIndex") >= 0; // boolean
		}else{
			// No explicit tabIndex setting, so depends on node type
			return dijit.hasDefaultTabStop(elem);
		}
	});

	dijit._getTabNavigable = function(/*DOMNode*/ root){
		// summary:
		//		Finds descendants of the specified root node.
		//
		// description:
		//		Finds the following descendants of the specified root node:
		//		* the first tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the last tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the first element in document order with the lowest
		//		  positive tabIndex value
		//		* the last element in document order with the highest
		//		  positive tabIndex value
		var first, last, lowest, lowestTabindex, highest, highestTabindex;
		var walkTree = function(/*DOMNode*/parent){
			dojo.query("> *", parent).forEach(function(child){
				// Skip hidden elements, and also non-HTML elements (those in custom namespaces) in IE,
				// since show() invokes getAttribute("type"), which crash on VML nodes in IE.
				if((dojo.isIE && child.scopeName!=="HTML") || !shown(child)){
					return;
				}

				if(isTabNavigable(child)){
					var tabindex = attr(child, "tabIndex");
					if(!hasAttr(child, "tabIndex") || tabindex == 0){
						if(!first){ first = child; }
						last = child;
					}else if(tabindex > 0){
						if(!lowest || tabindex < lowestTabindex){
							lowestTabindex = tabindex;
							lowest = child;
						}
						if(!highest || tabindex >= highestTabindex){
							highestTabindex = tabindex;
							highest = child;
						}
					}
				}
				if(child.nodeName.toUpperCase() != 'SELECT'){
					walkTree(child);
				}
			});
		};
		if(shown(root)){ walkTree(root) }
		return { first: first, last: last, lowest: lowest, highest: highest };
	}
	dijit.getFirstInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is first in the tabbing order
		var elems = dijit._getTabNavigable(dojo.byId(root));
		return elems.lowest ? elems.lowest : elems.first; // DomNode
	};
	
	dijit.getLastInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is last in the tabbing order
		var elems = dijit._getTabNavigable(dojo.byId(root));
		return elems.last ? elems.last : elems.highest; // DomNode
	};
	
	/*=====
	dojo.mixin(dijit, {
		// defaultDuration: Integer
		//		The default animation speed (in ms) to use for all Dijit
		//		transitional animations, unless otherwise specified
		//		on a per-instance basis. Defaults to 200, overrided by
		//		`djConfig.defaultDuration`
		defaultDuration: 200
	});
	=====*/
	
	dijit.defaultDuration = dojo.config["defaultDuration"] || 200;

})();

}

if(!dojo._hasResource["dijit._base.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.focus"] = true;
dojo.provide("dijit._base.focus");


	// for dijit.isTabNavigable()

// summary:
//		These functions are used to query or set the focus and selection.
//
//		Also, they trace when widgets become activated/deactivated,
//		so that the widget can fire _onFocus/_onBlur events.
//		"Active" here means something similar to "focused", but
//		"focus" isn't quite the right word because we keep track of
//		a whole stack of "active" widgets.  Example: ComboButton --> Menu -->
//		MenuItem.  The onBlur event for ComboButton doesn't fire due to focusing
//		on the Menu or a MenuItem, since they are considered part of the
//		ComboButton widget.  It only happens when focus is shifted
//		somewhere completely different.

dojo.mixin(dijit, {
	// _curFocus: DomNode
	//		Currently focused item on screen
	_curFocus: null,

	// _prevFocus: DomNode
	//		Previously focused item on screen
	_prevFocus: null,

	isCollapsed: function(){
		// summary:
		//		Returns true if there is no text selected
		return dijit.getBookmark().isCollapsed;
	},

	getBookmark: function(){
		// summary:
		//		Retrieves a bookmark that can be used with moveToBookmark to return to the same range
		var bm, rg, tg, sel = dojo.doc.selection, cf = dijit._curFocus;

		if(dojo.global.getSelection){
			//W3C Range API for selections.
			sel = dojo.global.getSelection();
			if(sel){
				if(sel.isCollapsed){
					tg = cf? cf.tagName : "";
					if(tg){
						//Create a fake rangelike item to restore selections.
						tg = tg.toLowerCase();
						if(tg == "textarea" ||
								(tg == "input" && (!cf.type || cf.type.toLowerCase() == "text"))){
							sel = {
								start: cf.selectionStart,
								end: cf.selectionEnd,
								node: cf,
								pRange: true
							};
							return {isCollapsed: (sel.end <= sel.start), mark: sel}; //Object.
						}
					}
					bm = {isCollapsed:true};
				}else{
					rg = sel.getRangeAt(0);
					bm = {isCollapsed: false, mark: rg.cloneRange()};
				}
			}
		}else if(sel){
			// If the current focus was a input of some sort and no selection, don't bother saving
			// a native bookmark.  This is because it causes issues with dialog/page selection restore.
			// So, we need to create psuedo bookmarks to work with.
			tg = cf ? cf.tagName : "";
			tg = tg.toLowerCase();
			if(cf && tg && (tg == "button" || tg == "textarea" || tg == "input")){
				if(sel.type && sel.type.toLowerCase() == "none"){
					return {
						isCollapsed: true,
						mark: null
					}
				}else{
					rg = sel.createRange();
					return {
						isCollapsed: rg.text && rg.text.length?false:true,
						mark: {
							range: rg,
							pRange: true
						}
					};
				}
			}
			bm = {};

			//'IE' way for selections.
			try{
				// createRange() throws exception when dojo in iframe
				//and nothing selected, see #9632
				rg = sel.createRange();
				bm.isCollapsed = !(sel.type == 'Text' ? rg.htmlText.length : rg.length);
			}catch(e){
				bm.isCollapsed = true;
				return bm;
			}
			if(sel.type.toUpperCase() == 'CONTROL'){
				if(rg.length){
					bm.mark=[];
					var i=0,len=rg.length;
					while(i<len){
						bm.mark.push(rg.item(i++));
					}
				}else{
					bm.isCollapsed = true;
					bm.mark = null;
				}
			}else{
				bm.mark = rg.getBookmark();
			}
		}else{
			console.warn("No idea how to store the current selection for this browser!");
		}
		return bm; // Object
	},

	moveToBookmark: function(/*Object*/bookmark){
		// summary:
		//		Moves current selection to a bookmark
		// bookmark:
		//		This should be a returned object from dijit.getBookmark()

		var _doc = dojo.doc,
			mark = bookmark.mark;
		if(mark){
			if(dojo.global.getSelection){
				//W3C Rangi API (FF, WebKit, Opera, etc)
				var sel = dojo.global.getSelection();
				if(sel && sel.removeAllRanges){
					if(mark.pRange){
						var r = mark;
						var n = r.node;
						n.selectionStart = r.start;
						n.selectionEnd = r.end;
					}else{
						sel.removeAllRanges();
						sel.addRange(mark);
					}
				}else{
					console.warn("No idea how to restore selection for this browser!");
				}
			}else if(_doc.selection && mark){
				//'IE' way.
				var rg;
				if(mark.pRange){
					rg = mark.range;
				}else if(dojo.isArray(mark)){
					rg = _doc.body.createControlRange();
					//rg.addElement does not have call/apply method, so can not call it directly
					//rg is not available in "range.addElement(item)", so can't use that either
					dojo.forEach(mark, function(n){
						rg.addElement(n);
					});
				}else{
					rg = _doc.body.createTextRange();
					rg.moveToBookmark(mark);
				}
				rg.select();
			}
		}
	},

	getFocus: function(/*Widget?*/ menu, /*Window?*/ openedForWindow){
		// summary:
		//		Called as getFocus(), this returns an Object showing the current focus
		//		and selected text.
		//
		//		Called as getFocus(widget), where widget is a (widget representing) a button
		//		that was just pressed, it returns where focus was before that button
		//		was pressed.   (Pressing the button may have either shifted focus to the button,
		//		or removed focus altogether.)   In this case the selected text is not returned,
		//		since it can't be accurately determined.
		//
		// menu: dijit._Widget or {domNode: DomNode} structure
		//		The button that was just pressed.  If focus has disappeared or moved
		//		to this button, returns the previous focus.  In this case the bookmark
		//		information is already lost, and null is returned.
		//
		// openedForWindow:
		//		iframe in which menu was opened
		//
		// returns:
		//		A handle to restore focus/selection, to be passed to `dijit.focus`
		var node = !dijit._curFocus || (menu && dojo.isDescendant(dijit._curFocus, menu.domNode)) ? dijit._prevFocus : dijit._curFocus;
		return {
			node: node,
			bookmark: (node == dijit._curFocus) && dojo.withGlobal(openedForWindow || dojo.global, dijit.getBookmark),
			openedForWindow: openedForWindow
		}; // Object
	},

	focus: function(/*Object || DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow,
			collapsed = bookmark ? bookmark.isCollapsed : false;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase() == "iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}
			dijit._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item) or if previous selection was collapsed
		// as it may cause focus shift (Esp in IE).
		if(bookmark && dojo.withGlobal(openedForWindow || dojo.global, dijit.isCollapsed) && !collapsed){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				dojo.withGlobal(openedForWindow || dojo.global, dijit.moveToBookmark, null, [bookmark]);
			}catch(e2){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	},

	// _activeStack: dijit._Widget[]
	//		List of currently active widgets (focused widget and it's ancestors)
	_activeStack: [],

	registerIframe: function(/*DomNode*/ iframe){
		// summary:
		//		Registers listeners on the specified iframe so that any click
		//		or focus event on that iframe (or anything in it) is reported
		//		as a focus/click event on the <iframe> itself.
		// description:
		//		Currently only used by editor.
		// returns:
		//		Handle to pass to unregisterIframe()
		return dijit.registerWin(iframe.contentWindow, iframe);
	},

	unregisterIframe: function(/*Object*/ handle){
		// summary:
		//		Unregisters listeners on the specified iframe created by registerIframe.
		//		After calling be sure to delete or null out the handle itself.
		// handle:
		//		Handle returned by registerIframe()

		dijit.unregisterWin(handle);
	},

	registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
		// summary:
		//		Registers listeners on the specified window (either the main
		//		window or an iframe's window) to detect when the user has clicked somewhere
		//		or focused somewhere.
		// description:
		//		Users should call registerIframe() instead of this method.
		// targetWindow:
		//		If specified this is the window associated with the iframe,
		//		i.e. iframe.contentWindow.
		// effectiveNode:
		//		If specified, report any focus events inside targetWindow as
		//		an event on effectiveNode, rather than on evt.target.
		// returns:
		//		Handle to pass to unregisterWin()

		// TODO: make this function private in 2.0; Editor/users should call registerIframe(),

		var mousedownListener = function(evt){
			dijit._justMouseDowned = true;
			setTimeout(function(){ dijit._justMouseDowned = false; }, 0);
			
			// workaround weird IE bug where the click is on an orphaned node
			// (first time clicking a Select/DropDownButton inside a TooltipDialog)
			if(dojo.isIE && evt && evt.srcElement && evt.srcElement.parentNode == null){
				return;
			}

			dijit._onTouchNode(effectiveNode || evt.target || evt.srcElement, "mouse");
		};
		//dojo.connect(targetWindow, "onscroll", ???);

		// Listen for blur and focus events on targetWindow's document.
		// IIRC, I'm using attachEvent() rather than dojo.connect() because focus/blur events don't bubble
		// through dojo.connect(), and also maybe to catch the focus events early, before onfocus handlers
		// fire.
		// Connect to <html> (rather than document) on IE to avoid memory leaks, but document on other browsers because
		// (at least for FF) the focus event doesn't fire on <html> or <body>.
		var doc = dojo.isIE ? targetWindow.document.documentElement : targetWindow.document;
		if(doc){
			if(dojo.isIE){
				doc.attachEvent('onmousedown', mousedownListener);
				var activateListener = function(evt){
					// IE reports that nodes like <body> have gotten focus, even though they have tabIndex=-1,
					// Should consider those more like a mouse-click than a focus....
					if(evt.srcElement.tagName.toLowerCase() != "#document" &&
						dijit.isTabNavigable(evt.srcElement)){
						dijit._onFocusNode(effectiveNode || evt.srcElement);
					}else{
						dijit._onTouchNode(effectiveNode || evt.srcElement);
					}
				};
				doc.attachEvent('onactivate', activateListener);
				var deactivateListener =  function(evt){
					dijit._onBlurNode(effectiveNode || evt.srcElement);
				};
				doc.attachEvent('ondeactivate', deactivateListener);

				return function(){
					doc.detachEvent('onmousedown', mousedownListener);
					doc.detachEvent('onactivate', activateListener);
					doc.detachEvent('ondeactivate', deactivateListener);
					doc = null;	// prevent memory leak (apparent circular reference via closure)
				};
			}else{
				doc.addEventListener('mousedown', mousedownListener, true);
				var focusListener = function(evt){
					dijit._onFocusNode(effectiveNode || evt.target);
				};
				doc.addEventListener('focus', focusListener, true);
				var blurListener = function(evt){
					dijit._onBlurNode(effectiveNode || evt.target);
				};
				doc.addEventListener('blur', blurListener, true);

				return function(){
					doc.removeEventListener('mousedown', mousedownListener, true);
					doc.removeEventListener('focus', focusListener, true);
					doc.removeEventListener('blur', blurListener, true);
					doc = null;	// prevent memory leak (apparent circular reference via closure)
				};
			}
		}
	},

	unregisterWin: function(/*Handle*/ handle){
		// summary:
		//		Unregisters listeners on the specified window (either the main
		//		window or an iframe's window) according to handle returned from registerWin().
		//		After calling be sure to delete or null out the handle itself.

		// Currently our handle is actually a function
		handle && handle();
	},

	_onBlurNode: function(/*DomNode*/ node){
		// summary:
		// 		Called when focus leaves a node.
		//		Usually ignored, _unless_ it *isn't* follwed by touching another node,
		//		which indicates that we tabbed off the last field on the page,
		//		in which case every widget is marked inactive
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = null;

		if(dijit._justMouseDowned){
			// the mouse down caused a new widget to be marked as active; this blur event
			// is coming late, so ignore it.
			return;
		}

		// if the blur event isn't followed by a focus event then mark all widgets as inactive.
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
		}
		dijit._clearActiveWidgetsTimer = setTimeout(function(){
			delete dijit._clearActiveWidgetsTimer;
			dijit._setStack([]);
			dijit._prevFocus = null;
		}, 100);
	},

	_onTouchNode: function(/*DomNode*/ node, /*String*/ by){
		// summary:
		//		Callback when node is focused or mouse-downed
		// node:
		//		The node that was touched.
		// by:
		//		"mouse" if the focus/touch was caused by a mouse down event

		// ignore the recent blurNode event
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
			delete dijit._clearActiveWidgetsTimer;
		}

		// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
		var newStack=[];
		try{
			while(node){
				var popupParent = dojo.attr(node, "dijitPopupParent");
				if(popupParent){
					node=dijit.byId(popupParent).domNode;
				}else if(node.tagName && node.tagName.toLowerCase() == "body"){
					// is this the root of the document or just the root of an iframe?
					if(node === dojo.body()){
						// node is the root of the main document
						break;
					}
					// otherwise, find the iframe this node refers to (can't access it via parentNode,
					// need to do this trick instead). window.frameElement is supported in IE/FF/Webkit
					node=dojo.window.get(node.ownerDocument).frameElement;
				}else{
					// if this node is the root node of a widget, then add widget id to stack,
					// except ignore clicks on disabled widgets (actually focusing a disabled widget still works,
					// to support MenuItem)
					var id = node.getAttribute && node.getAttribute("widgetId"),
						widget = id && dijit.byId(id);
					if(widget && !(by == "mouse" && widget.get("disabled"))){
						newStack.unshift(id);
					}
					node=node.parentNode;
				}
			}
		}catch(e){ /* squelch */ }

		dijit._setStack(newStack, by);
	},

	_onFocusNode: function(/*DomNode*/ node){
		// summary:
		//		Callback when node is focused

		if(!node){
			return;
		}

		if(node.nodeType == 9){
			// Ignore focus events on the document itself.  This is here so that
			// (for example) clicking the up/down arrows of a spinner
			// (which don't get focus) won't cause that widget to blur. (FF issue)
			return;
		}

		dijit._onTouchNode(node);

		if(node == dijit._curFocus){ return; }
		if(dijit._curFocus){
			dijit._prevFocus = dijit._curFocus;
		}
		dijit._curFocus = node;
		dojo.publish("focusNode", [node]);
	},

	_setStack: function(/*String[]*/ newStack, /*String*/ by){
		// summary:
		//		The stack of active widgets has changed.  Send out appropriate events and records new stack.
		// newStack:
		//		array of widget id's, starting from the top (outermost) widget
		// by:
		//		"mouse" if the focus/touch was caused by a mouse down event

		var oldStack = dijit._activeStack;
		dijit._activeStack = newStack;

		// compare old stack to new stack to see how many elements they have in common
		for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
			if(oldStack[nCommon] != newStack[nCommon]){
				break;
			}
		}

		var widget;
		// for all elements that have gone out of focus, send blur event
		for(var i=oldStack.length-1; i>=nCommon; i--){
			widget = dijit.byId(oldStack[i]);
			if(widget){
				widget._focused = false;
				widget._hasBeenBlurred = true;
				if(widget._onBlur){
					widget._onBlur(by);
				}
				dojo.publish("widgetBlur", [widget, by]);
			}
		}

		// for all element that have come into focus, send focus event
		for(i=nCommon; i<newStack.length; i++){
			widget = dijit.byId(newStack[i]);
			if(widget){
				widget._focused = true;
				if(widget._onFocus){
					widget._onFocus(by);
				}
				dojo.publish("widgetFocus", [widget, by]);
			}
		}
	}
});

// register top window and all the iframes it contains
dojo.addOnLoad(function(){
	var handle = dijit.registerWin(window);
	if(dojo.isIE){
		dojo.addOnWindowUnload(function(){
			dijit.unregisterWin(handle);
			handle = null;
		})
	}
});

}

if(!dojo._hasResource["dojo.AdapterRegistry"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.AdapterRegistry"] = true;
dojo.provide("dojo.AdapterRegistry");

dojo.AdapterRegistry = function(/*Boolean?*/ returnWrappers){
	//	summary:
	//		A registry to make contextual calling/searching easier.
	//	description:
	//		Objects of this class keep list of arrays in the form [name, check,
	//		wrap, directReturn] that are used to determine what the contextual
	//		result of a set of checked arguments is. All check/wrap functions
	//		in this registry should be of the same arity.
	//	example:
	//	|	// create a new registry
	//	|	var reg = new dojo.AdapterRegistry();
	//	|	reg.register("handleString",
	//	|		dojo.isString,
	//	|		function(str){
	//	|			// do something with the string here
	//	|		}
	//	|	);
	//	|	reg.register("handleArr",
	//	|		dojo.isArray,
	//	|		function(arr){
	//	|			// do something with the array here
	//	|		}
	//	|	);
	//	|
	//	|	// now we can pass reg.match() *either* an array or a string and
	//	|	// the value we pass will get handled by the right function
	//	|	reg.match("someValue"); // will call the first function
	//	|	reg.match(["someValue"]); // will call the second

	this.pairs = [];
	this.returnWrappers = returnWrappers || false; // Boolean
}

dojo.extend(dojo.AdapterRegistry, {
	register: function(/*String*/ name, /*Function*/ check, /*Function*/ wrap, /*Boolean?*/ directReturn, /*Boolean?*/ override){
		//	summary: 
		//		register a check function to determine if the wrap function or
		//		object gets selected
		//	name:
		//		a way to identify this matcher.
		//	check:
		//		a function that arguments are passed to from the adapter's
		//		match() function.  The check function should return true if the
		//		given arguments are appropriate for the wrap function.
		//	directReturn:
		//		If directReturn is true, the value passed in for wrap will be
		//		returned instead of being called. Alternately, the
		//		AdapterRegistry can be set globally to "return not call" using
		//		the returnWrappers property. Either way, this behavior allows
		//		the registry to act as a "search" function instead of a
		//		function interception library.
		//	override:
		//		If override is given and true, the check function will be given
		//		highest priority. Otherwise, it will be the lowest priority
		//		adapter.
		this.pairs[((override) ? "unshift" : "push")]([name, check, wrap, directReturn]);
	},

	match: function(/* ... */){
		// summary:
		//		Find an adapter for the given arguments. If no suitable adapter
		//		is found, throws an exception. match() accepts any number of
		//		arguments, all of which are passed to all matching functions
		//		from the registered pairs.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[1].apply(this, arguments)){
				if((pair[3])||(this.returnWrappers)){
					return pair[2];
				}else{
					return pair[2].apply(this, arguments);
				}
			}
		}
		throw new Error("No match found");
	},

	unregister: function(name){
		// summary: Remove a named adapter from the registry

		// FIXME: this is kind of a dumb way to handle this. On a large
		// registry this will be slow-ish and we can use the name as a lookup
		// should we choose to trade memory for speed.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[0] == name){
				this.pairs.splice(i, 1);
				return true;
			}
		}
		return false;
	}
});

}

if(!dojo._hasResource["dijit._base.place"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.place"] = true;
dojo.provide("dijit._base.place");





dijit.getViewport = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	return dojo.window.getBox();
};

/*=====
dijit.__Position = function(){
	// x: Integer
	//		horizontal coordinate in pixels, relative to document body
	// y: Integer
	//		vertical coordinate in pixels, relative to document body

	thix.x = x;
	this.y = y;
}
=====*/


dijit.placeOnScreen = function(
	/* DomNode */			node,
	/* dijit.__Position */	pos,
	/* String[] */			corners,
	/* dijit.__Position? */	padding){
	// summary:
	//		Positions one of the node's corners at specified position
	//		such that node is fully visible in viewport.
	// description:
	//		NOTE: node is assumed to be absolutely or relatively positioned.
	//	pos:
	//		Object like {x: 10, y: 20}
	//	corners:
	//		Array of Strings representing order to try corners in, like ["TR", "BL"].
	//		Possible values are:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//	padding:
	//		set padding to put some buffer around the element you want to position.
	// example:
	//		Try to place node's top right corner at (10,20).
	//		If that makes node go (partially) off screen, then try placing
	//		bottom left corner at (10,20).
	//	|	placeOnScreen(node, {x: 10, y: 20}, ["TR", "BL"])

	var choices = dojo.map(corners, function(corner){
		var c = { corner: corner, pos: {x:pos.x,y:pos.y} };
		if(padding){
			c.pos.x += corner.charAt(1) == 'L' ? padding.x : -padding.x;
			c.pos.y += corner.charAt(0) == 'T' ? padding.y : -padding.y;
		}
		return c;
	});

	return dijit._place(node, choices);
}

dijit._place = function(/*DomNode*/ node, /* Array */ choices, /* Function */ layoutNode){
	// summary:
	//		Given a list of spots to put node, put it at the first spot where it fits,
	//		of if it doesn't fit anywhere then the place with the least overflow
	// choices: Array
	//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
	//		Above example says to put the top-left corner of the node at (10,20)
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.

	// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
	// viewport over document
	var view = dojo.window.getBox();

	// This won't work if the node is inside a <div style="position: relative">,
	// so reattach it to dojo.doc.body.   (Otherwise, the positioning will be wrong
	// and also it might get cutoff)
	if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
		dojo.body().appendChild(node);
	}

	var best = null;
	dojo.some(choices, function(choice){
		var corner = choice.corner;
		var pos = choice.pos;

		// configure node to be displayed in given position relative to button
		// (need to do this in order to get an accurate size for the node, because
		// a tooltips size changes based on position, due to triangle)
		if(layoutNode){
			layoutNode(node, choice.aroundCorner, corner);
		}

		// get node's size
		var style = node.style;
		var oldDisplay = style.display;
		var oldVis = style.visibility;
		style.visibility = "hidden";
		style.display = "";
		var mb = dojo.marginBox(node);
		style.display = oldDisplay;
		style.visibility = oldVis;

		// coordinates and size of node with specified corner placed at pos,
		// and clipped by viewport
		var startX = Math.max(view.l, corner.charAt(1) == 'L' ? pos.x : (pos.x - mb.w)),
			startY = Math.max(view.t, corner.charAt(0) == 'T' ? pos.y : (pos.y - mb.h)),
			endX = Math.min(view.l + view.w, corner.charAt(1) == 'L' ? (startX + mb.w) : pos.x),
			endY = Math.min(view.t + view.h, corner.charAt(0) == 'T' ? (startY + mb.h) : pos.y),
			width = endX - startX,
			height = endY - startY,
			overflow = (mb.w - width) + (mb.h - height);

		if(best == null || overflow < best.overflow){
			best = {
				corner: corner,
				aroundCorner: choice.aroundCorner,
				x: startX,
				y: startY,
				w: width,
				h: height,
				overflow: overflow
			};
		}
		return !overflow;
	});

	node.style.left = best.x + "px";
	node.style.top = best.y + "px";
	if(best.overflow && layoutNode){
		layoutNode(node, best.aroundCorner, best.corner);
	}
	return best;
}

dijit.placeOnScreenAroundNode = function(
	/* DomNode */		node,
	/* DomNode */		aroundNode,
	/* Object */		aroundCorners,
	/* Function? */		layoutNode){

	// summary:
	//		Position node adjacent or kitty-corner to aroundNode
	//		such that it's fully visible in viewport.
	//
	// description:
	//		Place node such that corner of node touches a corner of
	//		aroundNode, and that node is fully visible.
	//
	// aroundCorners:
	//		Ordered list of pairs of corners to try matching up.
	//		Each pair of corners is represented as a key/value in the hash,
	//		where the key corresponds to the aroundNode's corner, and
	//		the value corresponds to the node's corner:
	//
	//	|	{ aroundNodeCorner1: nodeCorner1, aroundNodeCorner2: nodeCorner2, ...}
	//
	//		The following strings are used to represent the four corners:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
	//		For things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.
	//
	// example:
	//	|	dijit.placeOnScreenAroundNode(node, aroundNode, {'BL':'TL', 'TR':'BR'});
	//		This will try to position node such that node's top-left corner is at the same position
	//		as the bottom left corner of the aroundNode (ie, put node below
	//		aroundNode, with left edges aligned).  If that fails it will try to put
	// 		the bottom-right corner of node where the top right corner of aroundNode is
	//		(ie, put node above aroundNode, with right edges aligned)
	//

	// get coordinates of aroundNode
	aroundNode = dojo.byId(aroundNode);
	var oldDisplay = aroundNode.style.display;
	aroundNode.style.display="";
	// #3172: use the slightly tighter border box instead of marginBox
	var aroundNodePos = dojo.position(aroundNode, true);
	aroundNode.style.display=oldDisplay;

	// place the node around the calculated rectangle
	return dijit._placeOnScreenAroundRect(node,
		aroundNodePos.x, aroundNodePos.y, aroundNodePos.w, aroundNodePos.h,	// rectangle
		aroundCorners, layoutNode);
};

/*=====
dijit.__Rectangle = function(){
	// x: Integer
	//		horizontal offset in pixels, relative to document body
	// y: Integer
	//		vertical offset in pixels, relative to document body
	// width: Integer
	//		width in pixels
	// height: Integer
	//		height in pixels

	this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;
}
=====*/


dijit.placeOnScreenAroundRectangle = function(
	/* DomNode */			node,
	/* dijit.__Rectangle */	aroundRect,
	/* Object */			aroundCorners,
	/* Function */			layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except that the "around"
	//		parameter is an arbitrary rectangle on the screen (x, y, width, height)
	//		instead of a dom node.

	return dijit._placeOnScreenAroundRect(node,
		aroundRect.x, aroundRect.y, aroundRect.width, aroundRect.height,	// rectangle
		aroundCorners, layoutNode);
};

dijit._placeOnScreenAroundRect = function(
	/* DomNode */		node,
	/* Number */		x,
	/* Number */		y,
	/* Number */		width,
	/* Number */		height,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts coordinates
	//		of a rectangle to place node adjacent to.

	// TODO: combine with placeOnScreenAroundRectangle()

	// Generate list of possible positions for node
	var choices = [];
	for(var nodeCorner in aroundCorners){
		choices.push( {
			aroundCorner: nodeCorner,
			corner: aroundCorners[nodeCorner],
			pos: {
				x: x + (nodeCorner.charAt(1) == 'L' ? 0 : width),
				y: y + (nodeCorner.charAt(0) == 'T' ? 0 : height)
			}
		});
	}

	return dijit._place(node, choices, layoutNode);
};

dijit.placementRegistry= new dojo.AdapterRegistry();
dijit.placementRegistry.register("node",
	function(n, x){
		return typeof x == "object" &&
			typeof x.offsetWidth != "undefined" && typeof x.offsetHeight != "undefined";
	},
	dijit.placeOnScreenAroundNode);
dijit.placementRegistry.register("rect",
	function(n, x){
		return typeof x == "object" &&
			"x" in x && "y" in x && "width" in x && "height" in x;
	},
	dijit.placeOnScreenAroundRectangle);

dijit.placeOnScreenAroundElement = function(
	/* DomNode */		node,
	/* Object */		aroundElement,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object
	//		for the "around" argument and finds a proper processor to place a node.

	return dijit.placementRegistry.match.apply(dijit.placementRegistry, arguments);
};

dijit.getPopupAroundAlignment = function(/*Array*/ position, /*Boolean*/ leftToRight){
	// summary:
	//		Transforms the passed array of preferred positions into a format suitable for passing as the aroundCorners argument to dijit.placeOnScreenAroundElement.
	//
	// position: String[]
	//		This variable controls the position of the drop down.
	//		It's an array of strings with the following values:
	//
	//			* before: places drop down to the left of the target node/widget, or to the right in
	//			  the case of RTL scripts like Hebrew and Arabic
	//			* after: places drop down to the right of the target node/widget, or to the left in
	//			  the case of RTL scripts like Hebrew and Arabic
	//			* above: drop down goes above target node
	//			* below: drop down goes below target node
	//
	//		The list is positions is tried, in order, until a position is found where the drop down fits
	//		within the viewport.
	//
	// leftToRight: Boolean
	//		Whether the popup will be displaying in leftToRight mode.
	//
	var align = {};
	dojo.forEach(position, function(pos){
		switch(pos){
			case "after":
				align[leftToRight ? "BR" : "BL"] = leftToRight ? "BL" : "BR";
				break;
			case "before":
				align[leftToRight ? "BL" : "BR"] = leftToRight ? "BR" : "BL";
				break;
			case "below":
				// first try to align left borders, next try to align right borders (or reverse for RTL mode)
				align[leftToRight ? "BL" : "BR"] = leftToRight ? "TL" : "TR";
				align[leftToRight ? "BR" : "BL"] = leftToRight ? "TR" : "TL";
				break;
			case "above":
			default:
				// first try to align left borders, next try to align right borders (or reverse for RTL mode)
				align[leftToRight ? "TL" : "TR"] = leftToRight ? "BL" : "BR";
				align[leftToRight ? "TR" : "TL"] = leftToRight ? "BR" : "BL";
				break;
		}
	});
	return align;
};

}

if(!dojo._hasResource["dijit._base.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.window"] = true;
dojo.provide("dijit._base.window");



dijit.getDocumentWindow = function(doc){
	return dojo.window.get(doc);
};

}

if(!dojo._hasResource["dijit._base.popup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.popup"] = true;
dojo.provide("dijit._base.popup");





/*=====
dijit.popup.__OpenArgs = function(){
	// popup: Widget
	//		widget to display
	// parent: Widget
	//		the button etc. that is displaying this popup
	// around: DomNode
	//		DOM node (typically a button); place popup relative to this node.  (Specify this *or* "x" and "y" parameters.)
	// x: Integer
	//		Absolute horizontal position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// y: Integer
	//		Absolute vertical position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// orient: Object|String
	//		When the around parameter is specified, orient should be an
	//		ordered list of tuples of the form (around-node-corner, popup-node-corner).
	//		dijit.popup.open() tries to position the popup according to each tuple in the list, in order,
	//		until the popup appears fully within the viewport.
	//
	//		The default value is {BL:'TL', TL:'BL'}, which represents a list of two tuples:
	//			1. (BL, TL)
	//			2. (TL, BL)
	//		where BL means "bottom left" and "TL" means "top left".
	//		So by default, it first tries putting the popup below the around node, left-aligning them,
	//		and then tries to put it above the around node, still left-aligning them.   Note that the
	//		default is horizontally reversed when in RTL mode.
	//
	//		When an (x,y) position is specified rather than an around node, orient is either
	//		"R" or "L".  R (for right) means that it tries to put the popup to the right of the mouse,
	//		specifically positioning the popup's top-right corner at the mouse position, and if that doesn't
	//		fit in the viewport, then it tries, in order, the bottom-right corner, the top left corner,
	//		and the top-right corner.
	// onCancel: Function
	//		callback when user has canceled the popup by
	//			1. hitting ESC or
	//			2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
	//			   i.e. whenever popupWidget.onCancel() is called, args.onCancel is called
	// onClose: Function
	//		callback whenever this popup is closed
	// onExecute: Function
	//		callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
	// padding: dijit.__Position
	//		adding a buffer around the opening position. This is only useful when around is not set.
	this.popup = popup;
	this.parent = parent;
	this.around = around;
	this.x = x;
	this.y = y;
	this.orient = orient;
	this.onCancel = onCancel;
	this.onClose = onClose;
	this.onExecute = onExecute;
	this.padding = padding;
}
=====*/

dijit.popup = {
	// summary:
	//		This singleton is used to show/hide widgets as popups.

	// _stack: dijit._Widget[]
	//		Stack of currently popped up widgets.
	//		(someone opened _stack[0], and then it opened _stack[1], etc.)
	_stack: [],
	
	// _beginZIndex: Number
	//		Z-index of the first popup.   (If first popup opens other
	//		popups they get a higher z-index.)
	_beginZIndex: 1000,

	_idGen: 1,

	// TODO: in 2.0 change signature to always take widget
	moveOffScreen: function(/*Widget || DomNode*/ widget){
		// summary:
		//		Initialization for widgets that will be used as popups
		//
		// description:
		//		Puts widget inside a wrapper DIV (if not already in one), and
		//		positions wrapper DIV off screen, but not display:none, so that
		//		the widget doesn't appear in the page flow and/or cause a blank
		//		area at the bottom of the viewport (making scrollbar longer), but
		//		initialization of contained widgets works correctly

		var wrapper = widget.declaredClass ? widget._popupWrapper : (dojo.hasClass(widget.parentNode, "dijitPopup") && widget.parentNode),
			node = widget.domNode || widget;

		if(wrapper){
			// Widget already is encased in popup wrapper, just move wrapper off screen
			dojo.style(wrapper, {
				visibility: "hidden",
				// prevent transient scrollbar causing misalign (#5776), and initial flash in upper left (#10111)
				top: "-9999px"
			});
		}else{
			// Create an offscreen wrapper <div> for when this widget (in the future) will be used as a popup.
			// This is done early because of IE bugs where creating/moving DOM nodes causes focus
			// to go wonky, see tests/robot/Toolbar.html to reproduce
			wrapper = dojo.create("div",{
				"class":"dijitPopup",
				style:{
					visibility:"hidden",
					// prevent transient scrollbar causing misalign (#5776), and initial flash in upper left (#10111)
					top: "-9999px"
				},
				role: "presentation"
			}, dojo.body());
			wrapper.appendChild(node);

			var s = node.style;
			s.display = "";
			s.visibility = "";
			s.position = "";
			s.top = "0px";

			if(widget.declaredClass){		// TODO: in 2.0 change signature to always take widget, then remove if()
				widget._popupWrapper = wrapper;
				dojo.connect(widget, "destroy", function(){
					dojo.destroy(wrapper);
					delete widget._popupWrapper;
				});
			}
		}
	},

	getTopPopup: function(){
		// summary:
		//		Compute the closest ancestor popup that's *not* a child of another popup.
		//		Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
		var stack = this._stack;
		for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--){
			/* do nothing, just trying to get right value for pi */
		}
		return stack[pi];
	},

	open: function(/*dijit.popup.__OpenArgs*/ args){
		// summary:
		//		Popup the widget at the specified position
		//
		// example:
		//		opening at the mouse position
		//		|		dijit.popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
		//
		// example:
		//		opening the widget as a dropdown
		//		|		dijit.popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
		//
		//		Note that whatever widget called dijit.popup.open() should also listen to its own _onBlur callback
		//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

		var stack = this._stack,
			widget = args.popup,
			orient = args.orient || (
				(args.parent ? args.parent.isLeftToRight() : dojo._isBodyLtr()) ?
				{'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'} :
				{'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'}
			),
			around = args.around,
			id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+this._idGen++);


		// The wrapper may have already been created, but in case it wasn't, create here
		if(!widget._popupWrapper){
			this.moveOffScreen(widget);
		}
		var wrapper = widget._popupWrapper;

		dojo.attr(wrapper, {
			id: id,
			style: {
				zIndex: this._beginZIndex + stack.length
			},
			"class": "dijitPopup " + (widget.baseClass || widget["class"] || "").split(" ")[0] +"Popup",
			dijitPopupParent: args.parent ? args.parent.id : ""
		});

		if(dojo.isIE || dojo.isMoz){
			if(!widget.bgIframe){
				// setting widget.bgIframe triggers cleanup in _Widget.destroy()
				widget.bgIframe = new dijit.BackgroundIframe(wrapper);
			}
		}

		// position the wrapper node and make it visible
		var best = around ?
			dijit.placeOnScreenAroundElement(wrapper, around, orient, widget.orient ? dojo.hitch(widget, "orient") : null) :
			dijit.placeOnScreen(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR'], args.padding);

		wrapper.style.visibility = "visible";
		widget.domNode.style.visibility = "visible";	// counteract effects from _HasDropDown

		var handlers = [];

		// provide default escape and tab key handling
		// (this will work for any widget, not just menu)
		handlers.push(dojo.connect(wrapper, "onkeypress", this, function(evt){
			if(evt.charOrCode == dojo.keys.ESCAPE && args.onCancel){
				dojo.stopEvent(evt);
				args.onCancel();
			}else if(evt.charOrCode === dojo.keys.TAB){
				dojo.stopEvent(evt);
				var topPopup = this.getTopPopup();
				if(topPopup && topPopup.onCancel){
					topPopup.onCancel();
				}
			}
		}));

		// watch for cancel/execute events on the popup and notify the caller
		// (for a menu, "execute" means clicking an item)
		if(widget.onCancel){
			handlers.push(dojo.connect(widget, "onCancel", args.onCancel));
		}

		handlers.push(dojo.connect(widget, widget.onExecute ? "onExecute" : "onChange", this, function(){
			var topPopup = this.getTopPopup();
			if(topPopup && topPopup.onExecute){
				topPopup.onExecute();
			}
		}));

		stack.push({
			widget: widget,
			parent: args.parent,
			onExecute: args.onExecute,
			onCancel: args.onCancel,
 			onClose: args.onClose,
			handlers: handlers
		});

		if(widget.onOpen){
			// TODO: in 2.0 standardize onShow() (used by StackContainer) and onOpen() (used here)
			widget.onOpen(best);
		}

		return best;
	},

	close: function(/*dijit._Widget*/ popup){
		// summary:
		//		Close specified popup and any popups that it parented

		var stack = this._stack;

		// Basically work backwards from the top of the stack closing popups
		// until we hit the specified popup, but IIRC there was some issue where closing
		// a popup would cause others to close too.  Thus if we are trying to close B in [A,B,C]
		// closing C might close B indirectly and then the while() condition will run where stack==[A]...
		// so the while condition is constructed defensively.
		while(dojo.some(stack, function(elem){return elem.widget == popup;})){
			var top = stack.pop(),
				widget = top.widget,
				onClose = top.onClose;

			if(widget.onClose){
				// TODO: in 2.0 standardize onHide() (used by StackContainer) and onClose() (used here)
				widget.onClose();
			}
			dojo.forEach(top.handlers, dojo.disconnect);

			// Move the widget plus it's wrapper off screen, unless it has already been destroyed in above onClose() etc.
			if(widget && widget.domNode){
				this.moveOffScreen(widget);
			}
                        
			if(onClose){
				onClose();
			}
		}
	}
};

// TODO: remove dijit._frames, it isn't being used much, since popups never release their
// iframes (see [22236])
dijit._frames = new function(){
	// summary:
	//		cache of iframes

	var queue = [];

	this.pop = function(){
		var iframe;
		if(queue.length){
			iframe = queue.pop();
			iframe.style.display="";
		}else{
			if(dojo.isIE){
				var burl = dojo.config["dojoBlankHtmlUrl"] || (dojo.moduleUrl("dojo", "resources/blank.html")+"") || "javascript:\"\"";
				var html="<iframe src='" + burl + "'"
					+ " style='position: absolute; left: 0px; top: 0px;"
					+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
				iframe = dojo.doc.createElement(html);
			}else{
			 	iframe = dojo.create("iframe");
				iframe.src = 'javascript:""';
				iframe.className = "dijitBackgroundIframe";
				dojo.style(iframe, "opacity", 0.1);
			}
			iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didn't work.
			dijit.setWaiRole(iframe,"presentation");
		}
		return iframe;
	};

	this.push = function(iframe){
		iframe.style.display="none";
		queue.push(iframe);
	}
}();


dijit.BackgroundIframe = function(/* DomNode */node){
	// summary:
	//		For IE/FF z-index schenanigans. id attribute is required.
	//
	// description:
	//		new dijit.BackgroundIframe(node)
	//			Makes a background iframe as a child of node, that fills
	//			area (and position) of node

	if(!node.id){ throw new Error("no id"); }
	if(dojo.isIE || dojo.isMoz){
		var iframe = dijit._frames.pop();
		node.appendChild(iframe);
		if(dojo.isIE<7){
			this.resize(node);
			this._conn = dojo.connect(node, 'onresize', this, function(){
				this.resize(node);
			});
		}else{
			dojo.style(iframe, {
				width: '100%',
				height: '100%'
			});
		}
		this.iframe = iframe;
	}
};

dojo.extend(dijit.BackgroundIframe, {
	resize: function(node){
		// summary:
		// 		resize the iframe so its the same size as node
		// description:
		//		this function is a no-op in all browsers except
		//		IE6, which does not support 100% width/height 
		//		of absolute positioned iframes
		if(this.iframe && dojo.isIE<7){
			dojo.style(this.iframe, {
				width: node.offsetWidth + 'px',
				height: node.offsetHeight + 'px'
			});
		}
	},
	destroy: function(){
		// summary:
		//		destroy the iframe
		if(this._conn){
			dojo.disconnect(this._conn);
			this._conn = null;
		}
		if(this.iframe){
			dijit._frames.push(this.iframe);
			delete this.iframe;
		}
	}
});

}

if(!dojo._hasResource["dijit._base.scroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.scroll"] = true;
dojo.provide("dijit._base.scroll");



dijit.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view, if it is not already.
	//		Deprecated, use `dojo.window.scrollIntoView` instead.
	
	dojo.window.scrollIntoView(node, pos);
};

}

if(!dojo._hasResource["dojo.uacss"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.uacss"] = true;
dojo.provide("dojo.uacss");

(function(){
	// summary:
	//		Applies pre-set CSS classes to the top-level HTML node, based on:
	// 			- browser (ex: dj_ie)
	//			- browser version (ex: dj_ie6)
	//			- box model (ex: dj_contentBox)
	//			- text direction (ex: dijitRtl)
	//
	//		In addition, browser, browser version, and box model are
	//		combined with an RTL flag when browser text is RTL.  ex: dj_ie-rtl.

	var d = dojo,
		html = d.doc.documentElement,
		ie = d.isIE,
		opera = d.isOpera,
		maj = Math.floor,
		ff = d.isFF,
		boxModel = d.boxModel.replace(/-/,''),

		classes = {
			dj_ie: ie,
			dj_ie6: maj(ie) == 6,
			dj_ie7: maj(ie) == 7,
			dj_ie8: maj(ie) == 8,
			dj_quirks: d.isQuirks,
			dj_iequirks: ie && d.isQuirks,

			// NOTE: Opera not supported by dijit
			dj_opera: opera,

			dj_khtml: d.isKhtml,

			dj_webkit: d.isWebKit,
			dj_safari: d.isSafari,
			dj_chrome: d.isChrome,

			dj_gecko: d.isMozilla,
			dj_ff3: maj(ff) == 3
		}; // no dojo unsupported browsers

	classes["dj_" + boxModel] = true;

	// apply browser, browser version, and box model class names
	var classStr = "";
	for(var clz in classes){
		if(classes[clz]){
			classStr += clz + " ";
		}
	}
	html.className = d.trim(html.className + " " + classStr);

	// If RTL mode, then add dj_rtl flag plus repeat existing classes with -rtl extension.
	// We can't run the code below until the <body> tag has loaded (so we can check for dir=rtl).  
	// Unshift() is to run sniff code before the parser.
	dojo._loaders.unshift(function(){
		if(!dojo._isBodyLtr()){
			var rtlClassStr = "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl ")
			html.className = d.trim(html.className + " " + rtlClassStr);
		}
	});
})();

}

if(!dojo._hasResource["dijit._base.sniff"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.sniff"] = true;
// summary:
//		Applies pre-set CSS classes to the top-level HTML node, see
//		`dojo.uacss` for details.
//
//		Simply doing a require on this module will
//		establish this CSS.  Modified version of Morris' CSS hack.

dojo.provide("dijit._base.sniff");



}

if(!dojo._hasResource["dijit._base.typematic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.typematic"] = true;
dojo.provide("dijit._base.typematic");

dijit.typematic = {
	// summary:
	//		These functions are used to repetitively call a user specified callback
	//		method when a specific key or mouse click over a specific DOM node is
	//		held down for a specific amount of time.
	//		Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);
		
		// Schedule next event, timer is at most minDelay (default 10ms) to avoid
		// browser overload (particularly avoiding starving DOH robot so it never gets to send a mouseup)
		this._currentTimeout = Math.max(
			this._currentTimeout < 0 ? this._initialDelay :
				(this._subsequentDelay > 1 ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay)),
			this._minDelay);
		this._timer = setTimeout(dojo.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /*Object*/ _this, /*DOMNode*/ node, /*Function*/ callback, /*Object*/ obj, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start a timed, repeating callback sequence.
		//		If already started, the function call is ignored.
		//		This method is not normally called by the user but can be
		//		when the normal listener code is insufficient.
		// evt:
		//		key or mouse event object to pass to the user callback
		// _this:
		//		pointer to the user's widget space.
		// node:
		//		the DOM node object to pass the the callback function
		// callback:
		//		function to call until the sequence is stopped called with 3 parameters:
		// count:
		//		integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		// node:
		//		the DOM node object passed in
		// evt:
		//		key or mouse event object
		// obj:
		//		user space object used to uniquely identify each typematic sequence
		// subsequentDelay (optional):
		//		if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		// initialDelay (optional):
		//		the number of milliseconds until the 2nd event occurs, default=500ms
		// minDelay (optional):
		//		the maximum delay in milliseconds for event to fire, default=10ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._minDelay = minDelay || 10;
			this._obj = obj;
			this._evt = evt;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = dojo.hitch(_this, callback);
			this._fireEventAndReload();
			this._evt = dojo.mixin({faux: true}, evt);
		}
	},

	stop: function(){
		// summary:
		//		Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key.
		//		See also the trigger method for other parameters.
		// keyObject:
		//		an object defining the key to listen for:
		// 		charOrCode:
		//			the printable character (string) or keyCode (number) to listen for.
		// 		keyCode:
		//			(deprecated - use charOrCode) the keyCode (number) to listen for (implies charCode = 0).
		// 		charCode:
		//			(deprecated - use charOrCode) the charCode (number) to listen for.
		// 		ctrlKey:
		//			desired ctrl key state to initiate the callback sequence:
		//			- pressed (true)
		//			- released (false)
		//			- either (unspecified)
		// 		altKey:
		//			same as ctrlKey but for the alt key
		// 		shiftKey:
		//			same as ctrlKey but for the shift key
		// returns:
		//		an array of dojo.connect handles
		if(keyObject.keyCode){
			keyObject.charOrCode = keyObject.keyCode;
			dojo.deprecated("keyCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}else if(keyObject.charCode){
			keyObject.charOrCode = String.fromCharCode(keyObject.charCode);
			dojo.deprecated("charCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}
		return [
			dojo.connect(node, "onkeypress", this, function(evt){
				if(evt.charOrCode == keyObject.charOrCode &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.altKey) &&
				(keyObject.metaKey === undefined || keyObject.metaKey == (evt.metaKey || false)) && // IE doesn't even set metaKey
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.shiftKey)){
					dojo.stopEvent(evt);
					dijit.typematic.trigger(evt, _this, node, callback, keyObject, subsequentDelay, initialDelay, minDelay);
				}else if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			}),
			dojo.connect(node, "onkeyup", this, function(evt){
				if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			})
		];
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a typematic mouse click.
		//		See the trigger method for other parameters.
		// returns:
		//		an array of dojo.connect handles
		var dc = dojo.connect;
		return [
			dc(node, "mousedown", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
			}),
			dc(node, "mouseup", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mouseout", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mousemove", this, function(evt){
				evt.preventDefault();
			}),
			dc(node, "dblclick", this, function(evt){
				dojo.stopEvent(evt);
				if(dojo.isIE){
					dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
					setTimeout(dojo.hitch(this, dijit.typematic.stop), 50);
				}
			})
		];
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key and mouseclick.
		//		This is a thin wrapper to addKeyListener and addMouseListener.
		//		See the addMouseListener and addKeyListener methods for other parameters.
		// mouseNode:
		//		the DOM node object to listen on for mouse events.
		// keyNode:
		//		the DOM node object to listen on for key events.
		// returns:
		//		an array of dojo.connect handles
		return this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay, minDelay).concat(
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay, minDelay));
	}
};

}

if(!dojo._hasResource["dijit._base.wai"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.wai"] = true;
dojo.provide("dijit._base.wai");

dijit.wai = {
	onload: function(){
		// summary:
		//		Detects if we are in high-contrast mode or not

		// This must be a named function and not an anonymous
		// function, so that the widget parsing code can make sure it
		// registers its onload function after this function.
		// DO NOT USE "this" within this function.

		// create div for testing if high contrast mode is on or images are turned off
		var div = dojo.create("div",{
			id: "a11yTestNode",
			style:{
				cssText:'border: 1px solid;'
					+ 'border-color:red green;'
					+ 'position: absolute;'
					+ 'height: 5px;'
					+ 'top: -999px;'
					+ 'background-image: url("' + (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")) + '");'
			}
		}, dojo.body());

		// test it
		var cs = dojo.getComputedStyle(div);
		if(cs){
			var bkImg = cs.backgroundImage;
			var needsA11y = (cs.borderTopColor == cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
			dojo[needsA11y ? "addClass" : "removeClass"](dojo.body(), "dijit_a11y");
			if(dojo.isIE){
				div.outerHTML = "";		// prevent mixed-content warning, see http://support.microsoft.com/kb/925014
			}else{
				dojo.body().removeChild(div);
			}
		}
	}
};

// Test if computer is in high contrast mode.
// Make sure the a11y test runs first, before widgets are instantiated.
if(dojo.isIE || dojo.isMoz){	// NOTE: checking in Safari messes things up
	dojo._loaders.unshift(dijit.wai.onload);
}

dojo.mixin(dijit, {
	_XhtmlRoles: /banner|contentinfo|definition|main|navigation|search|note|secondary|seealso/,

	hasWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Determines if an element has a particular non-XHTML role.
		// returns:
		//		True if elem has the specific non-XHTML role attribute and false if not.
		// 		For backwards compatibility if role parameter not provided,
		// 		returns true if has non XHTML role
		var waiRole = this.getWaiRole(elem);
		return role ? (waiRole.indexOf(role) > -1) : (waiRole.length > 0);
	},

	getWaiRole: function(/*Element*/ elem){
		// summary:
		//		Gets the non-XHTML role for an element (which should be a wai role).
		// returns:
		//		The non-XHTML role of elem or an empty string if elem
		//		does not have a role.
		 return dojo.trim((dojo.attr(elem, "role") || "").replace(this._XhtmlRoles,"").replace("wairole:",""));
	},

	setWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Sets the role on an element.
		// description:
		//		Replace existing role attribute with new role.
		//		If elem already has an XHTML role, append this role to XHTML role
		//		and remove other ARIA roles.

		var curRole = dojo.attr(elem, "role") || "";
		if(!this._XhtmlRoles.test(curRole)){
			dojo.attr(elem, "role", role);
		}else{
			if((" "+ curRole +" ").indexOf(" " + role + " ") < 0){
				var clearXhtml = dojo.trim(curRole.replace(this._XhtmlRoles, ""));
				var cleanRole = dojo.trim(curRole.replace(clearXhtml, ""));
				dojo.attr(elem, "role", cleanRole + (cleanRole ? ' ' : '') + role);
			}
		}
	},

	removeWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Removes the specified non-XHTML role from an element.
		// 		Removes role attribute if no specific role provided (for backwards compat.)

		var roleValue = dojo.attr(elem, "role");
		if(!roleValue){ return; }
		if(role){
			var t = dojo.trim((" " + roleValue + " ").replace(" " + role + " ", " "));
			dojo.attr(elem, "role", t);
		}else{
			elem.removeAttribute("role");
		}
	},

	hasWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Determines if an element has a given state.
		// description:
		//		Checks for an attribute called "aria-"+state.
		// returns:
		//		true if elem has a value for the given state and
		//		false if it does not.

		return elem.hasAttribute ? elem.hasAttribute("aria-"+state) : !!elem.getAttribute("aria-"+state);
	},

	getWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Gets the value of a state on an element.
		// description:
		//		Checks for an attribute called "aria-"+state.
		// returns:
		//		The value of the requested state on elem
		//		or an empty string if elem has no value for state.

		return elem.getAttribute("aria-"+state) || "";
	},

	setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
		// summary:
		//		Sets a state on an element.
		// description:
		//		Sets an attribute called "aria-"+state.

		elem.setAttribute("aria-"+state, value);
	},

	removeWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Removes a state from an element.
		// description:
		//		Sets an attribute called "aria-"+state.

		elem.removeAttribute("aria-"+state);
	}
});

}

if(!dojo._hasResource["dijit._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base"] = true;
dojo.provide("dijit._base");











}

if(!dojo._hasResource["dijit._Widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Widget"] = true;
dojo.provide("dijit._Widget");

dojo.require( "dijit._base" );


// This code is to assist deferring dojo.connect() calls in widgets (connecting to events on the widgets'
// DOM nodes) until someone actually needs to monitor that event.
dojo.connect(dojo, "_connect",
	function(/*dijit._Widget*/ widget, /*String*/ event){
		if(widget && dojo.isFunction(widget._onConnect)){
			widget._onConnect(event);
		}
	});

dijit._connectOnUseEventHandler = function(/*Event*/ event){};

// Keep track of where the last keydown event was, to help avoid generating
// spurious ondijitclick events when:
// 1. focus is on a <button> or <a>
// 2. user presses then releases the ENTER key
// 3. onclick handler fires and shifts focus to another node, with an ondijitclick handler
// 4. onkeyup event fires, causing the ondijitclick handler to fire
dijit._lastKeyDownNode = null;
if(dojo.isIE){
	(function(){
		var keydownCallback = function(evt){
			dijit._lastKeyDownNode = evt.srcElement;
		};
		dojo.doc.attachEvent('onkeydown', keydownCallback);
		dojo.addOnWindowUnload(function(){
			dojo.doc.detachEvent('onkeydown', keydownCallback);
		});
	})();
}else{
	dojo.doc.addEventListener('keydown', function(evt){
		dijit._lastKeyDownNode = evt.target;
	}, true);
}

(function(){

var _attrReg = {},	// cached results from getSetterAttributes
	getSetterAttributes = function(widget){
		// summary:
		//		Returns list of attributes with custom setters for specified widget
		var dc = widget.declaredClass;
		if(!_attrReg[dc]){
			var r = [],
				attrs,
				proto = widget.constructor.prototype;
			for(var fxName in proto){
				if(dojo.isFunction(proto[fxName]) && (attrs = fxName.match(/^_set([a-zA-Z]*)Attr$/)) && attrs[1]){
					r.push(attrs[1].charAt(0).toLowerCase() + attrs[1].substr(1));
				}
			}
			_attrReg[dc] = r;
		}
		return _attrReg[dc] || [];	// String[]
	};

dojo.declare("dijit._Widget", null, {
	// summary:
	//		Base class for all Dijit widgets.

	// id: [const] String
	//		A unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",

	// lang: [const] String
	//		Rarely used.  Overrides the default Dojo locale used to render this widget,
	//		as defined by the [HTML LANG](http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang) attribute.
	//		Value must be among the list of locales specified during by the Dojo bootstrap,
	//		formatted according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt) (like en-us).
	lang: "",

	// dir: [const] String
	//		Bi-directional support, as defined by the [HTML DIR](http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir)
	//		attribute. Either left-to-right "ltr" or right-to-left "rtl".  If undefined, widgets renders in page's
	//		default direction.
	dir: "",

	// class: String
	//		HTML class attribute
	"class": "",

	// style: String||Object
	//		HTML style attributes as cssText string or name/value hash
	style: "",

	// title: String
	//		HTML title attribute.
	//
	//		For form widgets this specifies a tooltip to display when hovering over
	//		the widget (just like the native HTML title attribute).
	//
	//		For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,
	//		etc., it's used to specify the tab label, accordion pane title, etc.
	title: "",

	// tooltip: String
	//		When this widget's title attribute is used to for a tab label, accordion pane title, etc.,
	//		this specifies the tooltip to appear when the mouse is hovered over that text.
	tooltip: "",

	// baseClass: [protected] String
	//		Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate
	//		widget state.
	baseClass: "",

	// srcNodeRef: [readonly] DomNode
	//		pointer to original DOM node
	srcNodeRef: null,

	// domNode: [readonly] DomNode
	//		This is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's dojoAttachPoint syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// containerNode: [readonly] DomNode
	//		Designates where children of the source DOM node will be placed.
	//		"Children" in this case refers to both DOM nodes and widgets.
	//		For example, for myWidget:
	//
	//		|	<div dojoType=myWidget>
	//		|		<b> here's a plain DOM node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//		|	</div>
	//
	//		containerNode would point to:
	//
	//		|		<b> here's a plain DOM node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//
	//		In templated widgets, "containerNode" is set via a
	//		dojoAttachPoint assignment.
	//
	//		containerNode must be defined for any widget that accepts innerHTML
	//		(like ContentPane or BorderContainer or even Button), and conversely
	//		is null for widgets that don't, like TextBox.
	containerNode: null,

/*=====
	// _started: Boolean
	//		startup() has completed.
	_started: false,
=====*/

	// attributeMap: [protected] Object
	//		attributeMap sets up a "binding" between attributes (aka properties)
	//		of the widget and the widget's DOM.
	//		Changes to widget attributes listed in attributeMap will be
	//		reflected into the DOM.
	//
	//		For example, calling attr('title', 'hello')
	//		on a TitlePane will automatically cause the TitlePane's DOM to update
	//		with the new title.
	//
	//		attributeMap is a hash where the key is an attribute of the widget,
	//		and the value reflects a binding to a:
	//
	//		- DOM node attribute
	// |		focus: {node: "focusNode", type: "attribute"}
	// 		Maps this.focus to this.focusNode.focus
	//
	//		- DOM node innerHTML
	//	|		title: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node innerText
	//	|		title: { node: "titleNode", type: "innerText" }
	//		Maps this.title to this.titleNode.innerText
	//
	//		- DOM node CSS class
	// |		myClass: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		There are also some shorthands for backwards compatibility:
	//		- string --> { node: string, type: "attribute" }, for example:
	//	|	"focusNode" ---> { node: "focusNode", type: "attribute" }
	//		- "" --> { node: "domNode", type: "attribute" }
	attributeMap: {id:"", dir:"", lang:"", "class":"", style:"", title:""},

	// _deferredConnects: [protected] Object
	//		attributeMap addendum for event handlers that should be connected only on first use
	_deferredConnects: {
		onClick: "",
		onDblClick: "",
		onKeyDown: "",
		onKeyPress: "",
		onKeyUp: "",
		onMouseMove: "",
		onMouseDown: "",
		onMouseOut: "",
		onMouseOver: "",
		onMouseLeave: "",
		onMouseEnter: "",
		onMouseUp: ""
	},

	onClick: dijit._connectOnUseEventHandler,
	/*=====
	onClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onDblClick: dijit._connectOnUseEventHandler,
	/*=====
	onDblClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse double click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onKeyDown: dijit._connectOnUseEventHandler,
	/*=====
	onKeyDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being pressed down.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyPress: dijit._connectOnUseEventHandler,
	/*=====
	onKeyPress: function(event){
		// summary:
		//		Connect to this function to receive notifications of printable keys being typed.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyUp: dijit._connectOnUseEventHandler,
	/*=====
	onKeyUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being released.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onMouseDown: dijit._connectOnUseEventHandler,
	/*=====
	onMouseDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is pressed down.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseMove: dijit._connectOnUseEventHandler,
	/*=====
	onMouseMove: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOut: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOut: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOver: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOver: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseLeave: dijit._connectOnUseEventHandler,
	/*=====
	onMouseLeave: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseEnter: dijit._connectOnUseEventHandler,
	/*=====
	onMouseEnter: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseUp: dijit._connectOnUseEventHandler,
	/*=====
	onMouseUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is released.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/

	// Constants used in templates

	// _blankGif: [protected] String
	//		Path to a blank 1x1 image.
	//		Used by <img> nodes in templates that really get their image via CSS background-image.
	_blankGif: (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")).toString(),

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(/*Object?*/params, /*DomNode|String*/srcNodeRef){
		// summary:
		//		Kicks off widget instantiation.  See create() for details.
		// tags:
		//		private
		this.create(params, srcNodeRef);
	},

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// summary:
		//		Kick off the life-cycle of a widget
		// params:
		//		Hash of initialization parameters for widget, including
		//		scalar values (like title, duration etc.) and functions,
		//		typically callbacks like onClick.
		// srcNodeRef:
		//		If a srcNodeRef (DOM node) is specified:
		//			- use srcNodeRef.innerHTML as my contents
		//			- if this is a behavioral widget then apply behavior
		//			  to that srcNodeRef
		//			- otherwise, replace srcNodeRef with my generated DOM
		//			  tree
		// description:
		//		Create calls a number of widget methods (postMixInProperties, buildRendering, postCreate,
		//		etc.), some of which of you'll want to override. See http://docs.dojocampus.org/dijit/_Widget
		//		for a discussion of the widget creation lifecycle.
		//
		//		Of course, adventurous developers could override create entirely, but this should
		//		only be done as a last resort.
		// tags:
		//		private

		// store pointer to original DOM tree
		this.srcNodeRef = dojo.byId(srcNodeRef);

		// For garbage collection.  An array of handles returned by Widget.connect()
		// Each handle returned from Widget.connect() is an array of handles from dojo.connect()
		this._connects = [];

		// For garbage collection.  An array of handles returned by Widget.subscribe()
		// The handle returned from Widget.subscribe() is the handle returned from dojo.subscribe()
		this._subscribes = [];

		// To avoid double-connects, remove entries from _deferredConnects
		// that have been setup manually by a subclass (ex, by dojoAttachEvent).
		// If a subclass has redefined a callback (ex: onClick) then assume it's being
		// connected to manually.
		this._deferredConnects = dojo.clone(this._deferredConnects);
		for(var attr in this.attributeMap){
			delete this._deferredConnects[attr]; // can't be in both attributeMap and _deferredConnects
		}
		for(attr in this._deferredConnects){
			if(this[attr] !== dijit._connectOnUseEventHandler){
				delete this._deferredConnects[attr];	// redefined, probably dojoAttachEvent exists
			}
		}

		//mixin our passed parameters
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }
		if(params){
			this.params = params;
			dojo.mixin(this,params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.)
		if(!this.id){
			this.id = dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		dijit.registry.add(this);

		this.buildRendering();

		if(this.domNode){
			// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
			this._applyAttributes();

			var source = this.srcNodeRef;
			if(source && source.parentNode){
				source.parentNode.replaceChild(this.domNode, source);
			}

			// If the developer has specified a handler as a widget parameter
			// (ex: new Button({onClick: ...})
			// then naturally need to connect from DOM node to that handler immediately,
			for(attr in this.params){
				this._onConnect(attr);
			}
		}

		if(this.domNode){
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}

		this._created = true;
	},

	_applyAttributes: function(){
		// summary:
		//		Step during widget creation to copy all widget attributes to the
		//		DOM as per attributeMap and _setXXXAttr functions.
		// description:
		//		Skips over blank/false attribute values, unless they were explicitly specified
		//		as parameters to the widget, since those are the default anyway,
		//		and setting tabIndex="" is different than not setting tabIndex at all.
		//
		//		It processes the attributes in the attribute map first, and then
		//		it goes through and processes the attributes for the _setXXXAttr
		//		functions that have been specified
		// tags:
		//		private
		var condAttrApply = function(attr, scope){
			if((scope.params && attr in scope.params) || scope[attr]){
				scope.set(attr, scope[attr]);
			}
		};

		// Do the attributes in attributeMap
		for(var attr in this.attributeMap){
			condAttrApply(attr, this);
		}

		// And also any attributes with custom setters
		dojo.forEach(getSetterAttributes(this), function(a){
			if(!(a in this.attributeMap)){
				condAttrApply(a, this);
			}
		}, this);
	},

	postMixInProperties: function(){
		// summary:
		//		Called after the parameters to the widget have been read-in,
		//		but before the widget template is instantiated. Especially
		//		useful to set properties that are referenced in the widget
		//		template.
		// tags:
		//		protected
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode
		// description:
		//		Most widgets will mixin `dijit._Templated`, which implements this
		//		method.
		// tags:
		//		protected
		this.domNode = this.srcNodeRef || dojo.create('div');
	},

	postCreate: function(){
		// summary:
		//		Processing after the DOM fragment is created
		// description:
		//		Called after the DOM fragment has been created, but not necessarily
		//		added to the document.  Do not include any operations which rely on
		//		node dimensions or placement.
		// tags:
		//		protected

		// baseClass is a single class name or occasionally a space-separated list of names.
		// Add those classes to the DOMNod.  If RTL mode then also add with Rtl suffix.		
		if(this.baseClass){
			var classes = this.baseClass.split(" ");
			if(!this.isLeftToRight()){
				classes = classes.concat( dojo.map(classes, function(name){ return name+"Rtl"; }));
			}
			dojo.addClass(this.domNode, classes);
		}
	},

	startup: function(){
		// summary:
		//		Processing after the DOM fragment is added to the document
		// description:
		//		Called after a widget and its children have been created and added to the page,
		//		and all related widgets have finished their create() cycle, up through postCreate().
		//		This is useful for composite widgets that need to control or layout sub-widgets.
		//		Many layout widgets can use this as a wiring phase.
		this._started = true;
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean?*/ preserveDom){
		// summary:
		// 		Destroy this widget and its descendants
		// description:
		//		This is the generic "destructor" function that all widget users
		// 		should call to cleanly discard with a widget. Once a widget is
		// 		destroyed, it is removed from the manager object.
		// preserveDom:
		//		If true, this method will leave the original DOM structure
		//		alone of descendant Widgets. Note: This will NOT work with
		//		dijit._Templated widgets.

		this._beingDestroyed = true;
		this.destroyDescendants(preserveDom);
		this.destroy(preserveDom);
	},

	destroy: function(/*Boolean*/ preserveDom){
		// summary:
		// 		Destroy this widget, but not its descendants.
		//		This method will, however, destroy internal widgets such as those used within a template.
		// preserveDom: Boolean
		//		If true, this method will leave the original DOM structure alone.
		//		Note: This will not yet work with _Templated widgets

		this._beingDestroyed = true;
		this.uninitialize();
		var d = dojo,
			dfe = d.forEach,
			dun = d.unsubscribe;
		dfe(this._connects, function(array){
			dfe(array, d.disconnect);
		});
		dfe(this._subscribes, function(handle){
			dun(handle);
		});

		// destroy widgets created as part of template, etc.
		dfe(this._supportingWidgets || [], function(w){
			if(w.destroyRecursive){
				w.destroyRecursive();
			}else if(w.destroy){
				w.destroy();
			}
		});

		this.destroyRendering(preserveDom);
		dijit.registry.remove(this.id);
		this._destroyed = true;
	},

	destroyRendering: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// preserveDom:
		//		If true, this method will leave the original DOM structure alone
		//		during tear-down. Note: this will not work with _Templated
		//		widgets yet.
		// tags:
		//		protected

		if(this.bgIframe){
			this.bgIframe.destroy(preserveDom);
			delete this.bgIframe;
		}

		if(this.domNode){
			if(preserveDom){
				dojo.removeAttr(this.domNode, "widgetId");
			}else{
				dojo.destroy(this.domNode);
			}
			delete this.domNode;
		}

		if(this.srcNodeRef){
			if(!preserveDom){
				dojo.destroy(this.srcNodeRef);
			}
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.
		// preserveDom:
		//		If true, the preserveDom attribute is passed to all descendant
		//		widget's .destroy() method. Not for use with _Templated
		//		widgets.

		// get all direct descendants and destroy them recursively
		dojo.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive(preserveDom);
			}
		});
	},


	uninitialize: function(){
		// summary:
		//		Stub function. Override to implement custom widget tear-down
		//		behavior.
		// tags:
		//		protected
		return false;
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	onFocus: function(){
		// summary:
		//		Called when the widget becomes "active" because
		//		it or a widget inside of it either has focus, or has recently
		//		been clicked.
		// tags:
		//		callback
	},

	onBlur: function(){
		// summary:
		//		Called when the widget stops being "active" because
		//		focus moved to something outside of it, or the user
		//		clicked somewhere outside of it, or the widget was
		//		hidden.
		// tags:
		//		callback
	},

	_onFocus: function(e){
		// summary:
		//		This is where widgets do processing for when they are active,
		//		such as changing CSS classes.  See onFocus() for more details.
		// tags:
		//		protected
		this.onFocus();
	},

	_onBlur: function(){
		// summary:
		//		This is where widgets do processing for when they stop being active,
		//		such as changing CSS classes.  See onBlur() for more details.
		// tags:
		//		protected
		this.onBlur();
	},

	_onConnect: function(/*String*/ event){
		// summary:
		//		Called when someone connects to one of my handlers.
		//		"Turn on" that handler if it isn't active yet.
		//
		//		This is also called for every single initialization parameter
		//		so need to do nothing for parameters like "id".
		// tags:
		//		private
		if(event in this._deferredConnects){
			var mapNode = this[this._deferredConnects[event] || 'domNode'];
			this.connect(mapNode, event.toLowerCase(), event);
			delete this._deferredConnects[event];
		}
	},

	_setClassAttr: function(/*String*/ value){
		// summary:
		//		Custom setter for the CSS "class" attribute
		// tags:
		//		protected
		var mapNode = this[this.attributeMap["class"] || 'domNode'];
		dojo.removeClass(mapNode, this["class"])
		this["class"] = value;
		dojo.addClass(mapNode, value);
	},

	_setStyleAttr: function(/*String||Object*/ value){
		// summary:
		//		Sets the style attribut of the widget according to value,
		//		which is either a hash like {height: "5px", width: "3px"}
		//		or a plain string
		// description:
		//		Determines which node to set the style on based on style setting
		//		in attributeMap.
		// tags:
		//		protected

		var mapNode = this[this.attributeMap.style || 'domNode'];

		// Note: technically we should revert any style setting made in a previous call
		// to his method, but that's difficult to keep track of.

		if(dojo.isObject(value)){
			dojo.style(mapNode, value);
		}else{
			if(mapNode.style.cssText){
				mapNode.style.cssText += "; " + value;
			}else{
				mapNode.style.cssText = value;
			}
		}

		this.style = value;
	},

	setAttribute: function(/*String*/ attr, /*anything*/ value){
		// summary:
		//		Deprecated.  Use set() instead.
		// tags:
		//		deprecated
		dojo.deprecated(this.declaredClass+"::setAttribute(attr, value) is deprecated. Use set() instead.", "", "2.0");
		this.set(attr, value);
	},

	_attrToDom: function(/*String*/ attr, /*String*/ value){
		// summary:
		//		Reflect a widget attribute (title, tabIndex, duration etc.) to
		//		the widget DOM, as specified in attributeMap.
		//
		// description:
		//		Also sets this["attr"] to the new value.
		//		Note some attributes like "type"
		//		cannot be processed this way as they are not mutable.
		//
		// tags:
		//		private

		var commands = this.attributeMap[attr];
		dojo.forEach(dojo.isArray(commands) ? commands : [commands], function(command){

			// Get target node and what we are doing to that node
			var mapNode = this[command.node || command || "domNode"];	// DOM node
			var type = command.type || "attribute";	// class, innerHTML, innerText, or attribute

			switch(type){
				case "attribute":
					if(dojo.isFunction(value)){ // functions execute in the context of the widget
						value = dojo.hitch(this, value);
					}

					// Get the name of the DOM node attribute; usually it's the same
					// as the name of the attribute in the widget (attr), but can be overridden.
					// Also maps handler names to lowercase, like onSubmit --> onsubmit
					var attrName = command.attribute ? command.attribute :
						(/^on[A-Z][a-zA-Z]*$/.test(attr) ? attr.toLowerCase() : attr);

					dojo.attr(mapNode, attrName, value);
					break;
				case "innerText":
					mapNode.innerHTML = "";
					mapNode.appendChild(dojo.doc.createTextNode(value));
					break;
				case "innerHTML":
					mapNode.innerHTML = value;
					break;
				case "class":
					dojo.removeClass(mapNode, this[attr]);
					dojo.addClass(mapNode, value);
					break;
			}
		}, this);
		this[attr] = value;
	},

	attr: function(/*String|Object*/name, /*Object?*/value){
		// summary:
		//		Set or get properties on a widget instance.
		//	name:
		//		The property to get or set. If an object is passed here and not
		//		a string, its keys are used as names of attributes to be set
		//		and the value of the object as values to set in the widget.
		//	value:
		//		Optional. If provided, attr() operates as a setter. If omitted,
		//		the current value of the named property is returned.
		// description:
		//		This method is deprecated, use get() or set() directly.

		// Print deprecation warning but only once per calling function
		if(dojo.config.isDebug){
			var alreadyCalledHash = arguments.callee._ach || (arguments.callee._ach = {}),
				caller = (arguments.callee.caller || "unknown caller").toString();
			if(!alreadyCalledHash[caller]){
				dojo.deprecated(this.declaredClass + "::attr() is deprecated. Use get() or set() instead, called from " +
				caller, "", "2.0");
				alreadyCalledHash[caller] = true;
			}
		}

		var args = arguments.length;
		if(args >= 2 || typeof name === "object"){ // setter
			return this.set.apply(this, arguments);
		}else{ // getter
			return this.get(name);
		}
	},
	
	get: function(name){
		// summary:
		//		Get a property from a widget.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property from a widget. The property may
		//		potentially be retrieved via a getter method. If no getter is defined, this
		// 		just retrieves the object's property.  
		// 		For example, if the widget has a properties "foo"
		//		and "bar" and a method named "_getFooAttr", calling:
		//	|	myWidget.get("foo");
		//		would be equivalent to writing:
		//	|	widget._getFooAttr();
		//		and:
		//	|	myWidget.get("bar");
		//		would be equivalent to writing:
		//	|	widget.bar;
		var names = this._getAttrNames(name);
		return this[names.g] ? this[names.g]() : this[name];
	},
	
	set: function(name, value){
		// summary:
		//		Set a property on a widget
		//	name:
		//		The property to set. 
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a widget which may potentially be handled by a 
		// 		setter in the widget.
		// 		For example, if the widget has a properties "foo"
		//		and "bar" and a method named "_setFooAttr", calling:
		//	|	myWidget.set("foo", "Howdy!");
		//		would be equivalent to writing:
		//	|	widget._setFooAttr("Howdy!");
		//		and:
		//	|	myWidget.set("bar", 3);
		//		would be equivalent to writing:
		//	|	widget.bar = 3;
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myWidget.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)

		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]); 
			}
			return this;
		}
		var names = this._getAttrNames(name);
		if(this[names.s]){
			// use the explicit setter
			var result = this[names.s].apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			// if param is specified as DOM node attribute, copy it
			if(name in this.attributeMap){
				this._attrToDom(name, value);
			}
			var oldValue = this[name];
			// FIXME: what about function assignments? Any way to connect() here?
			this[name] = value;
		}
		return result || this;
	},
	
	_attrPairNames: {},		// shared between all widgets
	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		var uc = name.charAt(0).toUpperCase() + name.substr(1);
		return (apn[name] = {
			n: name+"Node",
			s: "_set"+uc+"Attr",
			g: "_get"+uc+"Attr"
		});
	},

	toString: function(){
		// summary:
		//		Returns a string that represents the widget
		// description:
		//		When a widget is cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversible
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getDescendants: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		This method should generally be avoided as it returns widgets declared in templates, which are
		//		supposed to be internal/hidden, but it's left here for back-compat reasons.

		return this.containerNode ? dojo.query('[widgetId]', this.containerNode).map(dijit.byNode) : []; // dijit._Widget[]
	},

	getChildren: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		Does not return nested widgets, nor widgets that are part of this widget's template.
		return this.containerNode ? dijit.findWidgets(this.containerNode) : []; // dijit._Widget[]
	},

	// nodesWithKeyClick: [private] String[]
	//		List of nodes that correctly handle click events via native browser support,
	//		and don't need dijit's help
	nodesWithKeyClick: ["input", "button"],

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		This version of connect also provides a special "ondijitclick"
		//		event which triggers on a click or space or enter keyup
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var d = dojo,
			dc = d._connect,
			handles = [];
		if(event == "ondijitclick"){
			// add key based click activation for unsupported nodes.
			// do all processing onkey up to prevent spurious clicks
			// for details see comments at top of this file where _lastKeyDownNode is defined
			if(dojo.indexOf(this.nodesWithKeyClick, obj.nodeName.toLowerCase()) == -1){ // is NOT input or button
				var m = d.hitch(this, method);
				handles.push(
					dc(obj, "onkeydown", this, function(e){
						//console.log(this.id + ": onkeydown, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if((e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
							// needed on IE for when focus changes between keydown and keyup - otherwise dropdown menus do not work
							dijit._lastKeyDownNode = e.target;
							e.preventDefault();		// stop event to prevent scrolling on space key in IE
						}
			 		}),
					dc(obj, "onkeyup", this, function(e){
						//console.log(this.id + ": onkeyup, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if( (e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							e.target === dijit._lastKeyDownNode &&
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
								//need reset here or have problems in FF when focus returns to trigger element after closing popup/alert
								dijit._lastKeyDownNode = null;
								return m(e);
						}
					})
				);
			}
			event = "onclick";
		}
		handles.push(dc(obj, event, this, method));

		this._connects.push(handles);
		return handles;		// _Widget.Handle
	},

	disconnect: function(/* _Widget.Handle */ handles){
		// summary:
		//		Disconnects handle created by `connect`.
		//		Also removes handle from this widget's list of connects.
		// tags:
		//		protected
		for(var i=0; i<this._connects.length; i++){
			if(this._connects[i] == handles){
				dojo.forEach(handles, dojo.disconnect);
				this._connects.splice(i, 1);
				return;
			}
		}
	},

	subscribe: function(
			/*String*/ topic,
			/*String|Function*/ method){
		// summary:
		//		Subscribes to the specified topic and calls the specified method
		//		of this object and registers for unsubscribe() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.subscribe, except with the
		//		implicit use of this widget as the target object.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when /my/topic is published, this button changes its label to
		//	|   // be the parameter of the topic.
		//	|	btn.subscribe("/my/topic", function(v){
		//	|		this.set("label", v);
		//	|	});
		var d = dojo,
			handle = d.subscribe(topic, this, method);

		// return handles for Any widget that may need them
		this._subscribes.push(handle);
		return handle;
	},

	unsubscribe: function(/*Object*/ handle){
		// summary:
		//		Unsubscribes handle created by this.subscribe.
		//		Also removes handle from this widget's list of subscriptions
		for(var i=0; i<this._subscribes.length; i++){
			if(this._subscribes[i] == handle){
				dojo.unsubscribe(handle);
				this._subscribes.splice(i, 1);
				return;
			}
		}
	},

	isLeftToRight: function(){
		// summary:
		//		Return this widget's explicit or implicit orientation (true for LTR, false for RTL)
		// tags:
		//		protected
		return this.dir ? (this.dir == "ltr") : dojo._isBodyLtr(); //Boolean
	},

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (dojo.style(this.domNode, "display") != "none");
	},

	placeAt: function(/* String|DomNode|_Widget */reference, /* String?|Int? */position){
		// summary:
		//		Place this widget's domNode reference somewhere in the DOM based
		//		on standard dojo.place conventions, or passing a Widget reference that
		//		contains and addChild member.
		//
		// description:
		//		A convenience function provided in all _Widgets, providing a simple
		//		shorthand mechanism to put an existing (or newly created) Widget
		//		somewhere in the dom, and allow chaining.
		//
		// reference:
		//		The String id of a domNode, a domNode reference, or a reference to a Widget posessing
		//		an addChild method.
		//
		// position:
		//		If passed a string or domNode reference, the position argument
		//		accepts a string just as dojo.place does, one of: "first", "last",
		//		"before", or "after".
		//
		//		If passed a _Widget reference, and that widget reference has an ".addChild" method,
		//		it will be called passing this widget instance into that method, supplying the optional
		//		position index passed.
		//
		// returns:
		//		dijit._Widget
		//		Provides a useful return of the newly created dijit._Widget instance so you
		//		can "chain" this function by instantiating, placing, then saving the return value
		//		to a variable.
		//
		// example:
		// | 	// create a Button with no srcNodeRef, and place it in the body:
		// | 	var button = new dijit.form.Button({ label:"click" }).placeAt(dojo.body());
		// | 	// now, 'button' is still the widget reference to the newly created button
		// | 	dojo.connect(button, "onClick", function(e){ console.log('click'); });
		//
		// example:
		// |	// create a button out of a node with id="src" and append it to id="wrapper":
		// | 	var button = new dijit.form.Button({},"src").placeAt("wrapper");
		//
		// example:
		// |	// place a new button as the first element of some div
		// |	var button = new dijit.form.Button({ label:"click" }).placeAt("wrapper","first");
		//
		// example:
		// |	// create a contentpane and add it to a TabContainer
		// |	var tc = dijit.byId("myTabs");
		// |	new dijit.layout.ContentPane({ href:"foo.html", title:"Wow!" }).placeAt(tc)

		if(reference.declaredClass && reference.addChild){
			reference.addChild(this, position);
		}else{
			dojo.place(this.domNode, reference, position);
		}
		return this;
	},

	_onShow: function(){
		// summary:
		//		Internal method called when this widget is made visible.
		//		See `onShow` for details.
		this.onShow();
	},

	onShow: function(){
		// summary:
		//		Called when this widget becomes the selected pane in a
		//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
		//		`dijit.layout.AccordionContainer`, etc.
		//
		//		Also called to indicate display of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onHide: function(){
		// summary:
			//		Called when another widget becomes the selected pane in a
			//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
			//		`dijit.layout.AccordionContainer`, etc.
			//
			//		Also called to indicate hide of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
			// tags:
			//		callback
	},

	onClose: function(){
		// summary:
		//		Called when this widget is being displayed as a popup (ex: a Calendar popped
		//		up from a DateTextBox), and it is hidden.
		//		This is called from the dijit.popup code, and should not be called directly.
		//
		//		Also used as a parameter for children of `dijit.layout.StackContainer` or subclasses.
		//		Callback if a user tries to close the child.   Child will be closed if this function returns true.
		// tags:
		//		extension

		return true;		// Boolean
	}
});

})();

}

if(!dojo._hasResource["dijit._Container"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Container"] = true;
dojo.provide("dijit._Container");

dojo.declare("dijit._Container",
	null,
	{
		// summary:
		//		Mixin for widgets that contain a set of widget children.
		// description:
		//		Use this mixin for widgets that needs to know about and
		//		keep track of their widget children. Suitable for widgets like BorderContainer
		//		and TabContainer which contain (only) a set of child widgets.
		//
		//		It's not suitable for widgets like ContentPane
		//		which contains mixed HTML (plain DOM nodes in addition to widgets),
		//		and where contained widgets are not necessarily directly below
		//		this.containerNode.   In that case calls like addChild(node, position)
		//		wouldn't make sense.

		// isContainer: [protected] Boolean
		//		Indicates that this widget acts as a "parent" to the descendant widgets.
		//		When the parent is started it will call startup() on the child widgets.
		//		See also `isLayoutContainer`.
		isContainer: true,

		buildRendering: function(){
			this.inherited(arguments);
			if(!this.containerNode){
				// all widgets with descendants must set containerNode
	 				this.containerNode = this.domNode;
			}
		},

		addChild: function(/*dijit._Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Makes the given widget a child of this widget.
			// description:
			//		Inserts specified child widget's dom node as a child of this widget's
			//		container node, and possibly does other processing (such as layout).

			var refNode = this.containerNode;
			if(insertIndex && typeof insertIndex == "number"){
				var children = this.getChildren();
				if(children && children.length >= insertIndex){
					refNode = children[insertIndex-1].domNode;
					insertIndex = "after";
				}
			}
			dojo.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget or int*/ widget){
			// summary:
			//		Removes the passed widget instance from this widget but does
			//		not destroy it.  You can also pass in an integer indicating
			//		the index within the container to remove

			if(typeof widget == "number" && widget > 0){
				widget = this.getChildren()[widget];
			}

			if(widget){
				var node = widget.domNode;
				if(node && node.parentNode){
					node.parentNode.removeChild(node); // detach but don't destroy
				}
			}
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has children, i.e. if this.containerNode contains something.
			return this.getChildren().length > 0;	// Boolean
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			// summary:
			//      Destroys all the widgets inside this.containerNode,
			//      but not this widget itself
			dojo.forEach(this.getChildren(), function(child){ child.destroyRecursive(preserveDom); });
		},

		_getSiblingOfChild: function(/*dijit._Widget*/ child, /*int*/ dir){
			// summary:
			//		Get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			// tags:
			//      private
			var node = child.domNode,
				which = (dir>0 ? "nextSibling" : "previousSibling");
			do{
				node = node[which];
			}while(node && (node.nodeType != 1 || !dijit.byNode(node)));
			return node && dijit.byNode(node);	// dijit._Widget
		},

		getIndexOfChild: function(/*dijit._Widget*/ child){
			// summary:
			//		Gets the index of the child in this container or -1 if not found
			return dojo.indexOf(this.getChildren(), child);	// int
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under dojo.doc.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Startup all children of this widget
			dojo.forEach(this.getChildren(), function(child){ child.startup(); });

			this.inherited(arguments);
		}
	}
);

}

if(!dojo._hasResource["dojo.string"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.string"] = true;
dojo.provide("dojo.string");

/*=====
dojo.string = { 
	// summary: String utilities for Dojo
};
=====*/

dojo.string.rep = function(/*String*/str, /*Integer*/num){
	//	summary:
	//		Efficiently replicate a string `n` times.
	//	str:
	//		the string to replicate
	//	num:
	//		number of times to replicate the string
	
	if(num <= 0 || !str){ return ""; }
	
	var buf = [];
	for(;;){
		if(num & 1){
			buf.push(str);
		}
		if(!(num >>= 1)){ break; }
		str += str;
	}
	return buf.join("");	// String
};

dojo.string.pad = function(/*String*/text, /*Integer*/size, /*String?*/ch, /*Boolean?*/end){
	//	summary:
	//		Pad a string to guarantee that it is at least `size` length by
	//		filling with the character `ch` at either the start or end of the
	//		string. Pads at the start, by default.
	//	text:
	//		the string to pad
	//	size:
	//		length to provide padding
	//	ch:
	//		character to pad, defaults to '0'
	//	end:
	//		adds padding at the end if true, otherwise pads at start
	//	example:
	//	|	// Fill the string to length 10 with "+" characters on the right.  Yields "Dojo++++++".
	//	|	dojo.string.pad("Dojo", 10, "+", true);

	if(!ch){
		ch = '0';
	}
	var out = String(text),
		pad = dojo.string.rep(ch, Math.ceil((size - out.length) / ch.length));
	return end ? out + pad : pad + out;	// String
};

dojo.string.substitute = function(	/*String*/		template, 
									/*Object|Array*/map, 
									/*Function?*/	transform, 
									/*Object?*/		thisObject){
	//	summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	//	template: 
	//		a string with expressions in the form `${key}` to be replaced or
	//		`${key:format}` which specifies a format function. keys are case-sensitive. 
	//	map:
	//		hash to search for substitutions
	//	transform: 
	//		a function to process all parameters before substitution takes
	//		place, e.g. mylib.encodeXML
	//	thisObject: 
	//		where to look for optional format function; default to the global
	//		namespace
	//	example:
	//		Substitutes two expressions in a string from an Array or Object
	//	|	// returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// by providing substitution data in an Array
	//	|	dojo.string.substitute(
	//	|		"File '${0}' is not found in directory '${1}'.",
	//	|		["foo.html","/temp"]
	//	|	);
	//	|
	//	|	// also returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// but provides substitution data in an Object structure.  Dotted
	//	|	// notation may be used to traverse the structure.
	//	|	dojo.string.substitute(
	//	|		"File '${name}' is not found in directory '${info.dir}'.",
	//	|		{ name: "foo.html", info: { dir: "/temp" } }
	//	|	);
	//	example:
	//		Use a transform function to modify the values:
	//	|	// returns "file 'foo.html' is not found in directory '/temp'."
	//	|	dojo.string.substitute(
	//	|		"${0} is not found in ${1}.",
	//	|		["foo.html","/temp"],
	//	|		function(str){
	//	|			// try to figure out the type
	//	|			var prefix = (str.charAt(0) == "/") ? "directory": "file";
	//	|			return prefix + " '" + str + "'";
	//	|		}
	//	|	);
	//	example:
	//		Use a formatter
	//	|	// returns "thinger -- howdy"
	//	|	dojo.string.substitute(
	//	|		"${0:postfix}", ["thinger"], null, {
	//	|			postfix: function(value, key){
	//	|				return value + " -- howdy";
	//	|			}
	//	|		}
	//	|	);

	thisObject = thisObject || dojo.global;
	transform = transform ? 
		dojo.hitch(thisObject, transform) : function(v){ return v; };

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g,
		function(match, key, format){
			var value = dojo.getObject(key, false, map);
			if(format){
				value = dojo.getObject(format, false, thisObject).call(thisObject, value, key);
			}
			return transform(value, key).toString();
		}); // String
};

/*=====
dojo.string.trim = function(str){
	//	summary:
	//		Trims whitespace from both sides of the string
	//	str: String
	//		String to be trimmed
	//	returns: String
	//		Returns the trimmed string
	//	description:
	//		This version of trim() was taken from [Steven Levithan's blog](http://blog.stevenlevithan.com/archives/faster-trim-javascript).
	//		The short yet performant version of this function is dojo.trim(),
	//		which is part of Dojo base.  Uses String.prototype.trim instead, if available.
	return "";	// String
}
=====*/

dojo.string.trim = String.prototype.trim ?
	dojo.trim : // aliasing to the native function
	function(str){
		str = str.replace(/^\s+/, '');
		for(var i = str.length - 1; i >= 0; i--){
			if(/\S/.test(str.charAt(i))){
				str = str.substring(0, i + 1);
				break;
			}
		}
		return str;
	};

}

if(!dojo._hasResource["dojo.date.stamp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date.stamp"] = true;
dojo.provide("dojo.date.stamp");

// Methods to convert dates to or from a wire (string) format using well-known conventions

dojo.date.stamp.fromISOString = function(/*String*/formattedString, /*Number?*/defaultTime){
	//	summary:
	//		Returns a Date object given a string formatted according to a subset of the ISO-8601 standard.
	//
	//	description:
	//		Accepts a string formatted according to a profile of ISO8601 as defined by
	//		[RFC3339](http://www.ietf.org/rfc/rfc3339.txt), except that partial input is allowed.
	//		Can also process dates as specified [by the W3C](http://www.w3.org/TR/NOTE-datetime)
	//		The following combinations are valid:
	//
	//			* dates only
	//			|	* yyyy
	//			|	* yyyy-MM
	//			|	* yyyy-MM-dd
	// 			* times only, with an optional time zone appended
	//			|	* THH:mm
	//			|	* THH:mm:ss
	//			|	* THH:mm:ss.SSS
	// 			* and "datetimes" which could be any combination of the above
	//
	//		timezones may be specified as Z (for UTC) or +/- followed by a time expression HH:mm
	//		Assumes the local time zone if not specified.  Does not validate.  Improperly formatted
	//		input may return null.  Arguments which are out of bounds will be handled
	// 		by the Date constructor (e.g. January 32nd typically gets resolved to February 1st)
	//		Only years between 100 and 9999 are supported.
	//
  	//	formattedString:
	//		A string such as 2005-06-30T08:05:00-07:00 or 2005-06-30 or T08:05:00
	//
	//	defaultTime:
	//		Used for defaults for fields omitted in the formattedString.
	//		Uses 1970-01-01T00:00:00.0Z by default.

	if(!dojo.date.stamp._isoRegExp){
		dojo.date.stamp._isoRegExp =
//TODO: could be more restrictive and check for 00-59, etc.
			/^(?:(\d{4})(?:-(\d{2})(?:-(\d{2}))?)?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?)?$/;
	}

	var match = dojo.date.stamp._isoRegExp.exec(formattedString),
		result = null;

	if(match){
		match.shift();
		if(match[1]){match[1]--;} // Javascript Date months are 0-based
		if(match[6]){match[6] *= 1000;} // Javascript Date expects fractional seconds as milliseconds

		if(defaultTime){
			// mix in defaultTime.  Relatively expensive, so use || operators for the fast path of defaultTime === 0
			defaultTime = new Date(defaultTime);
			dojo.forEach(dojo.map(["FullYear", "Month", "Date", "Hours", "Minutes", "Seconds", "Milliseconds"], function(prop){
				return defaultTime["get" + prop]();
			}), function(value, index){
				match[index] = match[index] || value;
			});
		}
		result = new Date(match[0]||1970, match[1]||0, match[2]||1, match[3]||0, match[4]||0, match[5]||0, match[6]||0); //TODO: UTC defaults
		if(match[0] < 100){
			result.setFullYear(match[0] || 1970);
		}

		var offset = 0,
			zoneSign = match[7] && match[7].charAt(0);
		if(zoneSign != 'Z'){
			offset = ((match[8] || 0) * 60) + (Number(match[9]) || 0);
			if(zoneSign != '-'){ offset *= -1; }
		}
		if(zoneSign){
			offset -= result.getTimezoneOffset();
		}
		if(offset){
			result.setTime(result.getTime() + offset * 60000);
		}
	}

	return result; // Date or null
}

/*=====
	dojo.date.stamp.__Options = function(){
		//	selector: String
		//		"date" or "time" for partial formatting of the Date object.
		//		Both date and time will be formatted by default.
		//	zulu: Boolean
		//		if true, UTC/GMT is used for a timezone
		//	milliseconds: Boolean
		//		if true, output milliseconds
		this.selector = selector;
		this.zulu = zulu;
		this.milliseconds = milliseconds;
	}
=====*/

dojo.date.stamp.toISOString = function(/*Date*/dateObject, /*dojo.date.stamp.__Options?*/options){
	//	summary:
	//		Format a Date object as a string according a subset of the ISO-8601 standard
	//
	//	description:
	//		When options.selector is omitted, output follows [RFC3339](http://www.ietf.org/rfc/rfc3339.txt)
	//		The local time zone is included as an offset from GMT, except when selector=='time' (time without a date)
	//		Does not check bounds.  Only years between 100 and 9999 are supported.
	//
	//	dateObject:
	//		A Date object

	var _ = function(n){ return (n < 10) ? "0" + n : n; };
	options = options || {};
	var formattedDate = [],
		getter = options.zulu ? "getUTC" : "get",
		date = "";
	if(options.selector != "time"){
		var year = dateObject[getter+"FullYear"]();
		date = ["0000".substr((year+"").length)+year, _(dateObject[getter+"Month"]()+1), _(dateObject[getter+"Date"]())].join('-');
	}
	formattedDate.push(date);
	if(options.selector != "date"){
		var time = [_(dateObject[getter+"Hours"]()), _(dateObject[getter+"Minutes"]()), _(dateObject[getter+"Seconds"]())].join(':');
		var millis = dateObject[getter+"Milliseconds"]();
		if(options.milliseconds){
			time += "."+ (millis < 100 ? "0" : "") + _(millis);
		}
		if(options.zulu){
			time += "Z";
		}else if(options.selector != "time"){
			var timezoneOffset = dateObject.getTimezoneOffset();
			var absOffset = Math.abs(timezoneOffset);
			time += (timezoneOffset > 0 ? "-" : "+") + 
				_(Math.floor(absOffset/60)) + ":" + _(absOffset%60);
		}
		formattedDate.push(time);
	}
	return formattedDate.join('T'); // String
}

}

if(!dojo._hasResource["dojo.parser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.parser"] = true;
dojo.provide("dojo.parser");


new Date("X"); // workaround for #11279, new Date("") == NaN

dojo.parser = new function(){
	// summary: The Dom/Widget parsing package

	var d = dojo;
	this._attrName = d._scopeName + "Type";
	this._query = "[" + this._attrName + "]";

	function val2type(/*Object*/ value){
		// summary:
		//		Returns name of type of given value.

		if(d.isString(value)){ return "string"; }
		if(typeof value == "number"){ return "number"; }
		if(typeof value == "boolean"){ return "boolean"; }
		if(d.isFunction(value)){ return "function"; }
		if(d.isArray(value)){ return "array"; } // typeof [] == "object"
		if(value instanceof Date) { return "date"; } // assume timestamp
		if(value instanceof d._Url){ return "url"; }
		return "object";
	}

	function str2obj(/*String*/ value, /*String*/ type){
		// summary:
		//		Convert given string value to given type
		switch(type){
			case "string":
				return value;
			case "number":
				return value.length ? Number(value) : NaN;
			case "boolean":
				// for checked/disabled value might be "" or "checked".  interpret as true.
				return typeof value == "boolean" ? value : !(value.toLowerCase()=="false");
			case "function":
				if(d.isFunction(value)){
					// IE gives us a function, even when we say something like onClick="foo"
					// (in which case it gives us an invalid function "function(){ foo }"). 
					//  Therefore, convert to string
					value=value.toString();
					value=d.trim(value.substring(value.indexOf('{')+1, value.length-1));
				}
				try{
					if(value === "" || value.search(/[^\w\.]+/i) != -1){
						// The user has specified some text for a function like "return x+5"
						return new Function(value);
					}else{
						// The user has specified the name of a function like "myOnClick"
						// or a single word function "return"
						return d.getObject(value, false) || new Function(value);
					}
				}catch(e){ return new Function(); }
			case "array":
				return value ? value.split(/\s*,\s*/) : [];
			case "date":
				switch(value){
					case "": return new Date("");	// the NaN of dates
					case "now": return new Date();	// current date
					default: return d.date.stamp.fromISOString(value);
				}
			case "url":
				return d.baseUrl + value;
			default:
				return d.fromJson(value);
		}
	}

	var instanceClasses = {
		// map from fully qualified name (like "dijit.Button") to structure like
		// { cls: dijit.Button, params: {label: "string", disabled: "boolean"} }
	};

	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	dojo.connect(dojo, "extend", function(){
		instanceClasses = {};
	});

	function getClassInfo(/*String*/ className){
		// className:
		//		fully qualified name (like "dijit.form.Button")
		// returns:
		//		structure like
		//			{ 
		//				cls: dijit.Button, 
		//				params: { label: "string", disabled: "boolean"}
		//			}

		if(!instanceClasses[className]){
			// get pointer to widget class
			var cls = d.getObject(className);
			if(!cls){ return null; }		// class not defined [yet]

			var proto = cls.prototype;
	
			// get table of parameter names & types
			var params = {}, dummyClass = {};
			for(var name in proto){
				if(name.charAt(0)=="_"){ continue; } 	// skip internal properties
				if(name in dummyClass){ continue; }		// skip "constructor" and "toString"
				var defVal = proto[name];
				params[name]=val2type(defVal);
			}

			instanceClasses[className] = { cls: cls, params: params };
		}
		return instanceClasses[className];
	}

	this._functionFromScript = function(script){
		var preamble = "";
		var suffix = "";
		var argsStr = script.getAttribute("args");
		if(argsStr){
			d.forEach(argsStr.split(/\s*,\s*/), function(part, idx){
				preamble += "var "+part+" = arguments["+idx+"]; ";
			});
		}
		var withStr = script.getAttribute("with");
		if(withStr && withStr.length){
			d.forEach(withStr.split(/\s*,\s*/), function(part){
				preamble += "with("+part+"){";
				suffix += "}";
			});
		}
		return new Function(preamble+script.innerHTML+suffix);
	}

	this.instantiate = function(/* Array */nodes, /* Object? */mixin, /* Object? */args){
		// summary:
		//		Takes array of nodes, and turns them into class instances and
		//		potentially calls a startup method to allow them to connect with
		//		any children.
		// nodes: Array
		//		Array of nodes or objects like
		//	|		{
		//	|			type: "dijit.form.Button",
		//	|			node: DOMNode,
		//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
		//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
		//	|		}
		// mixin: Object?
		//		An object that will be mixed in with each node in the array.
		//		Values in the mixin will override values in the node, if they
		//		exist.
		// args: Object?
		//		An object used to hold kwArgs for instantiation.
		//		Supports 'noStart' and inherited.
		var thelist = [], dp = dojo.parser;
		mixin = mixin||{};
		args = args||{};
		
		d.forEach(nodes, function(obj){
			if(!obj){ return; }

			// Get pointers to DOMNode, dojoType string, and clsInfo (metadata about the dojoType), etc.s
			var node, type, clsInfo, clazz, scripts;
			if(obj.node){
				// new format of nodes[] array, object w/lots of properties pre-computed for me
				node = obj.node;
				type = obj.type;
				clsInfo = obj.clsInfo || (type && getClassInfo(type));
				clazz = clsInfo && clsInfo.cls;
				scripts = obj.scripts;
			}else{
				// old (backwards compatible) format of nodes[] array, simple array of DOMNodes
				node = obj;
				type = dp._attrName in mixin ? mixin[dp._attrName] : node.getAttribute(dp._attrName);
				clsInfo = type && getClassInfo(type);
				clazz = clsInfo && clsInfo.cls;
				scripts = (clazz && (clazz._noScript || clazz.prototype._noScript) ? [] : 
							d.query("> script[type^='dojo/']", node));
			}
			if(!clsInfo){
				throw new Error("Could not load class '" + type);
			}

			// Setup hash to hold parameter settings for this widget.   Start with the parameter
			// settings inherited from ancestors ("dir" and "lang").
			// Inherited setting may later be overridden by explicit settings on node itself.
			var params = {},
				attributes = node.attributes;
			if(args.defaults){
				// settings for the document itself (or whatever subtree is being parsed)
				dojo.mixin(params, args.defaults);
			}
			if(obj.inherited){
				// settings from dir=rtl or lang=... on a node above this node
				dojo.mixin(params, obj.inherited);
			}

			// read parameters (ie, attributes) specified on DOMNode
			// clsInfo.params lists expected params like {"checked": "boolean", "n": "number"}
			for(var name in clsInfo.params){
				var item = name in mixin?{value:mixin[name],specified:true}:attributes.getNamedItem(name);
				if(!item || (!item.specified && (!dojo.isIE || name.toLowerCase()!="value"))){ continue; }
				var value = item.value;
				// Deal with IE quirks for 'class' and 'style'
				switch(name){
				case "class":
					value = "className" in mixin?mixin.className:node.className;
					break;
				case "style":
					value = "style" in mixin?mixin.style:(node.style && node.style.cssText); // FIXME: Opera?
				}
				var _type = clsInfo.params[name];
				if(typeof value == "string"){
					params[name] = str2obj(value, _type);
				}else{
					params[name] = value;
				}
			}

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" event="foo"> tags are dojo.connected after instantiation
			// note: dojo/* script tags cannot exist in self closing widgets, like <input />
			var connects = [],	// functions to connect after instantiation
				calls = [];		// functions to call after instantiation

			d.forEach(scripts, function(script){
				node.removeChild(script);
				var event = script.getAttribute("event"),
					type = script.getAttribute("type"),
					nf = d.parser._functionFromScript(script);
				if(event){
					if(type == "dojo/connect"){
						connects.push({event: event, func: nf});
					}else{
						params[event] = nf;
					}
				}else{
					calls.push(nf);
				}
			});

			var markupFactory = clazz.markupFactory || clazz.prototype && clazz.prototype.markupFactory;
			// create the instance
			var instance = markupFactory ? markupFactory(params, node, clazz) : new clazz(params, node);
			thelist.push(instance);

			// map it to the JS namespace if that makes sense
			var jsname = node.getAttribute("jsId");
			if(jsname){
				d.setObject(jsname, instance);
			}

			// process connections and startup functions
			d.forEach(connects, function(connect){
				d.connect(instance, connect.event, null, connect.func);
			});
			d.forEach(calls, function(func){
				func.call(instance);
			});
		});

		// Call startup on each top level instance if it makes sense (as for
		// widgets).  Parent widgets will recursively call startup on their
		// (non-top level) children
		if(!mixin._started){
			// TODO: for 2.0, when old instantiate() API is desupported, store parent-child
			// relationships in the nodes[] array so that no getParent() call is needed.
			// Note that will  require a parse() call from ContentPane setting a param that the
			// ContentPane is the parent widget (so that the parse doesn't call startup() on the
			// ContentPane's children)
			d.forEach(thelist, function(instance){
				if(	!args.noStart && instance  && 
					instance.startup &&
					!instance._started && 
					(!instance.getParent || !instance.getParent())
				){
					instance.startup();
				}
			});
		}
		return thelist;
	};

	this.parse = function(/*DomNode?*/ rootNode, /* Object? */ args){
		// summary:
		//		Scan the DOM for class instances, and instantiate them.
		//
		// description:
		//		Search specified node (or root node) recursively for class instances,
		//		and instantiate them Searches for
		//		dojoType="qualified.class.name"
		//
		// rootNode: DomNode?
		//		A default starting root node from which to start the parsing. Can be
		//		omitted, defaulting to the entire document. If omitted, the `args`
		//		object can be passed in this place. If the `args` object has a 
		//		`rootNode` member, that is used.
		//
		// args:
		//		a kwArgs object passed along to instantiate()
		//		
		//			* noStart: Boolean?
		//				when set will prevent the parser from calling .startup()
		//				when locating the nodes. 
		//			* rootNode: DomNode?
		//				identical to the function's `rootNode` argument, though
		//				allowed to be passed in via this `args object. 
		//			* inherited: Object
		//				Hash possibly containing dir and lang settings to be applied to
		//				parsed widgets, unless there's another setting on a sub-node that overrides
		//
		//
		// example:
		//		Parse all widgets on a page:
		//	|		dojo.parser.parse();
		//
		// example:
		//		Parse all classes within the node with id="foo"
		//	|		dojo.parser.parse(dojo.byId(foo));
		//
		// example:
		//		Parse all classes in a page, but do not call .startup() on any 
		//		child
		//	|		dojo.parser.parse({ noStart: true })
		//
		// example:
		//		Parse all classes in a node, but do not call .startup()
		//	|		dojo.parser.parse(someNode, { noStart:true });
		//	|		// or
		// 	|		dojo.parser.parse({ noStart:true, rootNode: someNode });

		// determine the root node based on the passed arguments.
		var root;
		if(!args && rootNode && rootNode.rootNode){
			args = rootNode;
			root = args.rootNode;
		}else{
			root = rootNode;
		}

		var attrName = this._attrName;
		function scan(parent, list){
			// summary:
			//		Parent is an Object representing a DOMNode, with or without a dojoType specified.
			//		Scan parent's children looking for nodes with dojoType specified, storing in list[].
			//		If parent has a dojoType, also collects <script type=dojo/*> children and stores in parent.scripts[].
			// parent: Object
			//		Object representing the parent node, like
			//	|	{
			//	|		node: DomNode, 			// scan children of this node
			//	|		inherited: {dir: "rtl"},	// dir/lang setting inherited from above node
			//	|
			//	|		// attributes only set if node has dojoType specified
			//	|		scripts: [],			// empty array, put <script type=dojo/*> in here
			//	|		clsInfo: { cls: dijit.form.Button, ...}
			//	|	}
			// list: DomNode[]
			//		Output array of objects (same format as parent) representing nodes to be turned into widgets

			// Effective dir and lang settings on parent node, either set directly or inherited from grandparent
			var inherited = dojo.clone(parent.inherited);
			dojo.forEach(["dir", "lang"], function(name){
				var val = parent.node.getAttribute(name);
				if(val){
					inherited[name] = val;
				}
			});

			// if parent is a widget, then search for <script type=dojo/*> tags and put them in scripts[].
			var scripts = parent.clsInfo && !parent.clsInfo.cls.prototype._noScript ? parent.scripts : null;

			// unless parent is a widget with the stopParser flag set, continue search for dojoType, recursively
			var recurse = !parent.clsInfo || !parent.clsInfo.cls.prototype.stopParser;

			// scan parent's children looking for dojoType and <script type=dojo/*>
			for(var child = parent.node.firstChild; child; child = child.nextSibling){
				if(child.nodeType == 1){
					var type = recurse && child.getAttribute(attrName);
					if(type){
						// if dojoType specified, add to output array of nodes to instantiate
						var params = {
							"type": type,
							clsInfo: getClassInfo(type),	// note: won't find classes declared via dojo.Declaration
							node: child,
							scripts: [], // <script> nodes that are parent's children
							inherited: inherited // dir & lang attributes inherited from parent
						};
						list.push(params);

						// Recurse, collecting <script type="dojo/..."> children, and also looking for
						// descendant nodes with dojoType specified (unless the widget has the stopParser flag),
						scan(params, list);
					}else if(scripts && child.nodeName.toLowerCase() == "script"){
						// if <script type="dojo/...">, save in scripts[]
						type = child.getAttribute("type");
						if (type && /^dojo\/\w/i.test(type)) {
							scripts.push(child);
						}
					}else if(recurse){
						// Recurse, looking for grandchild nodes with dojoType specified
						scan({
							node: child,
							inherited: inherited
						}, list);
					}
				}
			}
		}

		// Make list of all nodes on page w/dojoType specified
		var list = [];
		scan({
			node: root ? dojo.byId(root) : dojo.body(),
			inherited: (args && args.inherited) || {
				dir: dojo._isBodyLtr() ? "ltr" : "rtl"
			}
		}, list);

		// go build the object instances
		return this.instantiate(list, null, args); // Array
	};
}();

//Register the parser callback. It should be the first callback
//after the a11y test.

(function(){
	var parseRunner = function(){ 
		if(dojo.config.parseOnLoad){
			dojo.parser.parse(); 
		}
	};

	// FIXME: need to clobber cross-dependency!!
	if(dojo.exists("dijit.wai.onload") && (dijit.wai.onload === dojo._loaders[0])){
		dojo._loaders.splice(1, 0, parseRunner);
	}else{
		dojo._loaders.unshift(parseRunner);
	}
})();

}

if(!dojo._hasResource["dojo.cache"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cache"] = true;
dojo.provide("dojo.cache");

/*=====
dojo.cache = { 
	// summary:
	// 		A way to cache string content that is fetchable via `dojo.moduleUrl`.
};
=====*/

(function(){
	var cache = {};
	dojo.cache = function(/*String||Object*/module, /*String*/url, /*String||Object?*/value){
		// summary:
		// 		A getter and setter for storing the string content associated with the
		// 		module and url arguments.
		// description:
		// 		module and url are used to call `dojo.moduleUrl()` to generate a module URL.
		// 		If value is specified, the cache value for the moduleUrl will be set to
		// 		that value. Otherwise, dojo.cache will fetch the moduleUrl and store it
		// 		in its internal cache and return that cached value for the URL. To clear
		// 		a cache value pass null for value. Since XMLHttpRequest (XHR) is used to fetch the
		// 		the URL contents, only modules on the same domain of the page can use this capability.
		// 		The build system can inline the cache values though, to allow for xdomain hosting.
		// module: String||Object
		// 		If a String, the module name to use for the base part of the URL, similar to module argument
		// 		to `dojo.moduleUrl`. If an Object, something that has a .toString() method that
		// 		generates a valid path for the cache item. For example, a dojo._Url object.
		// url: String
		// 		The rest of the path to append to the path derived from the module argument. If
		// 		module is an object, then this second argument should be the "value" argument instead.
		// value: String||Object?
		// 		If a String, the value to use in the cache for the module/url combination.
		// 		If an Object, it can have two properties: value and sanitize. The value property
		// 		should be the value to use in the cache, and sanitize can be set to true or false,
		// 		to indicate if XML declarations should be removed from the value and if the HTML
		// 		inside a body tag in the value should be extracted as the real value. The value argument
		// 		or the value property on the value argument are usually only used by the build system
		// 		as it inlines cache content.
		//	example:
		//		To ask dojo.cache to fetch content and store it in the cache (the dojo["cache"] style
		// 		of call is used to avoid an issue with the build system erroneously trying to intern
		// 		this example. To get the build system to intern your dojo.cache calls, use the
		// 		"dojo.cache" style of call):
		// 		|	//If template.html contains "<h1>Hello</h1>" that will be
		// 		|	//the value for the text variable.
		//		|	var text = dojo["cache"]("my.module", "template.html");
		//	example:
		//		To ask dojo.cache to fetch content and store it in the cache, and sanitize the input
		// 		 (the dojo["cache"] style of call is used to avoid an issue with the build system 
		// 		erroneously trying to intern this example. To get the build system to intern your
		// 		dojo.cache calls, use the "dojo.cache" style of call):
		// 		|	//If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		// 		|	//text variable will contain just "<h1>Hello</h1>".
		//		|	var text = dojo["cache"]("my.module", "template.html", {sanitize: true});
		//	example:
		//		Same example as previous, but demostrates how an object can be passed in as
		//		the first argument, then the value argument can then be the second argument.
		// 		|	//If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		// 		|	//text variable will contain just "<h1>Hello</h1>".
		//		|	var text = dojo["cache"](new dojo._Url("my/module/template.html"), {sanitize: true});

		//Module could be a string, or an object that has a toString() method
		//that will return a useful path. If it is an object, then the "url" argument
		//will actually be the value argument.
		if(typeof module == "string"){
			var pathObj = dojo.moduleUrl(module, url);
		}else{
			pathObj = module;
			value = url;
		}
		var key = pathObj.toString();

		var val = value;
		if(value != undefined && !dojo.isString(value)){
			val = ("value" in value ? value.value : undefined);
		}

		var sanitize = value && value.sanitize ? true : false;

		if(typeof val == "string"){
			//We have a string, set cache value
			val = cache[key] = sanitize ? dojo.cache._sanitize(val) : val;
		}else if(val === null){
			//Remove cached value
			delete cache[key];
		}else{
			//Allow cache values to be empty strings. If key property does
			//not exist, fetch it.
			if(!(key in cache)){
				val = dojo._getText(key);
				cache[key] = sanitize ? dojo.cache._sanitize(val) : val;
			}
			val = cache[key];
		}
		return val; //String
	};

	dojo.cache._sanitize = function(/*String*/val){
		// summary: 
		//		Strips <?xml ...?> declarations so that external SVG and XML
		// 		documents can be added to a document without worry. Also, if the string
		//		is an HTML document, only the part inside the body tag is returned.
		// description:
		// 		Copied from dijit._Templated._sanitizeTemplateString.
		if(val){
			val = val.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
			var matches = val.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
			if(matches){
				val = matches[1];
			}
		}else{
			val = "";
		}
		return val; //String
	};
})();

}

if(!dojo._hasResource["dijit._Templated"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Templated"] = true;
dojo.provide("dijit._Templated");






dojo.declare("dijit._Templated",
	null,
	{
		// summary:
		//		Mixin for widgets that are instantiated from a template

		// templateString: [protected] String
		//		A string that represents the widget template. Pre-empts the
		//		templatePath. In builds that have their strings "interned", the
		//		templatePath is converted to an inline templateString, thereby
		//		preventing a synchronous network call.
		//
		//		Use in conjunction with dojo.cache() to load from a file.
		templateString: null,

		// templatePath: [protected deprecated] String
		//		Path to template (HTML file) for this widget relative to dojo.baseUrl.
		//		Deprecated: use templateString with dojo.cache() instead.
		templatePath: null,

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  False by default.
		widgetsInTemplate: false,

		// skipNodeCache: [protected] Boolean
		//		If using a cached widget template node poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

		// _attachPoints: [private] String[]
		//		List of widget attribute names associated with dojoAttachPoint=... in the
		//		template, ex: ["containerNode", "labelNode"]
/*=====
 		_attachPoints: [],
 =====*/

		constructor: function(){
			this._attachPoints = [];
		},

		_stringRepl: function(tmpl){
			// summary:
			//		Does substitution of ${foo} type properties in template string
			// tags:
			//		private
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			return dojo.string.substitute(tmpl, this, function(value, key){
				if(key.charAt(0) == '!'){ value = dojo.getObject(key.substr(1), false, _this); }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(value == null){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
		},

		// method over-ride
		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.
			// tags:
			//		protected

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = dijit._Templated.getCachedTemplate(this.templatePath, this.templateString, this._skipNodeCache);

			var node;
			if(dojo.isString(cached)){
				node = dojo._toDom(this._stringRepl(cached));
				if(node.nodeType != 1){
					// Flag common problems such as templates with multiple top level nodes (nodeType == 11)
					throw new Error("Invalid template: " + cached);
				}
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			this.domNode = node;

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(node);

			if(this.widgetsInTemplate){
				// Make sure dojoType is used for parsing widgets in template.
				// The dojo.parser.query could be changed from multiversion support.
				var parser = dojo.parser, qry, attr;
				if(parser._query != "[dojoType]"){
					qry = parser._query;
					attr = parser._attrName;
					parser._query = "[dojoType]";
					parser._attrName = "dojoType";
				}

				// Store widgets that we need to start at a later point in time
				var cw = (this._startupWidgets = dojo.parser.parse(node, {
					noStart: !this._earlyTemplatedStartup,
					inherited: {dir: this.dir, lang: this.lang}
				}));

				// Restore the query.
				if(qry){
					parser._query = qry;
					parser._attrName = attr;
				}

				this._supportingWidgets = dijit.findWidgets(node);

				this._attachTemplateNodes(cw, function(n,p){
					return n[p];
				});
			}

			this._fillContent(this.srcNodeRef);
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		Iterate through the template and attach functions and nodes accordingly.
			// description:
			//		Map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//			* dojoAttachPoint
			//			* dojoAttachEvent
			//			* waiRole
			//			* waiState
			// rootNode: DomNode|Array[Widgets]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: Function?
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget
			// tags:
			//		private

			getAttrFunc = getAttrFunc || function(n,p){ return n.getAttribute(p); };

			var nodes = dojo.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x = dojo.isArray(rootNode) ? 0 : -1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && getAttrFunc(baseNode, "dojoType")){
					continue;
				}
				// Process dojoAttachPoint
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while((point = points.shift())){
						if(dojo.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
						this._attachPoints.push(point);
					}
				}

				// Process dojoAttachEvent
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent");
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = dojo.trim;
					while((event = events.shift())){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							this.connect(baseNode, event, thisFunc);
						}
					}
				}

				// waiRole, waiState
				var role = getAttrFunc(baseNode, "waiRole");
				if(role){
					dijit.setWaiRole(baseNode, role);
				}
				var values = getAttrFunc(baseNode, "waiState");
				if(values){
					dojo.forEach(values.split(/\s*,\s*/), function(stateValue){
						if(stateValue.indexOf('-') != -1){
							var pair = stateValue.split('-');
							dijit.setWaiState(baseNode, pair[0], pair[1]);
						}
					});
				}
			}
		},

		startup: function(){
			dojo.forEach(this._startupWidgets, function(w){
				if(w && !w._started && w.startup){
					w.startup();
				}
			});
			this.inherited(arguments);
		},

		destroyRendering: function(){
			// Delete all attach points to prevent IE6 memory leaks.
			dojo.forEach(this._attachPoints, function(point){
				delete this[point];
			}, this);
			this._attachPoints = [];

			this.inherited(arguments);
		}
	}
);

// key is either templatePath or templateString; object is either string or DOM tree
dijit._Templated._templateCache = {};

dijit._Templated.getCachedTemplate = function(templatePath, templateString, alwaysUseString){
	// summary:
	//		Static method to get a template based on the templatePath or
	//		templateString key
	// templatePath: String||dojo.uri.Uri
	//		The URL to get the template from.
	// templateString: String?
	//		a string to use in lieu of fetching the template from a URL. Takes precedence
	//		over templatePath
	// returns: Mixed
	//		Either string (if there are ${} variables that need to be replaced) or just
	//		a DOM tree (if the node can be cloned directly)

	// is it already cached?
	var tmplts = dijit._Templated._templateCache;
	var key = templateString || templatePath;
	var cached = tmplts[key];
	if(cached){
		try{
			// if the cached value is an innerHTML string (no ownerDocument) or a DOM tree created within the current document, then use the current cached value
			if(!cached.ownerDocument || cached.ownerDocument == dojo.doc){
				// string or node of the same document
				return cached;
			}
		}catch(e){ /* squelch */ } // IE can throw an exception if cached.ownerDocument was reloaded
		dojo.destroy(cached);
	}

	// If necessary, load template string from template path
	if(!templateString){
		templateString = dojo.cache(templatePath, {sanitize: true});
	}
	templateString = dojo.string.trim(templateString);

	if(alwaysUseString || templateString.match(/\$\{([^\}]+)\}/g)){
		// there are variables in the template so all we can do is cache the string
		return (tmplts[key] = templateString); //String
	}else{
		// there are no variables in the template so we can cache the DOM tree
		var node = dojo._toDom(templateString);
		if(node.nodeType != 1){
			throw new Error("Invalid template: " + templateString);
		}
		return (tmplts[key] = node); //Node
	}
};

if(dojo.isIE){
	dojo.addOnWindowUnload(function(){
		var cache = dijit._Templated._templateCache;
		for(var key in cache){
			var value = cache[key];
			if(typeof value == "object"){ // value is either a string or a DOM node template
				dojo.destroy(value);
			}
			delete cache[key];
		}
	});
}

// These arguments can be specified for widgets which are used in templates.
// Since any widget can be specified as sub widgets in template, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget,{
	dojoAttachEvent: "",
	dojoAttachPoint: "",
	waiRole: "",
	waiState:""
});

}

if(!dojo._hasResource["dijit._KeyNavContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._KeyNavContainer"] = true;
dojo.provide("dijit._KeyNavContainer");


dojo.declare("dijit._KeyNavContainer",
	dijit._Container,
	{

		// summary:
		//		A _Container with keyboard navigation of its children.
		// description:
		//		To use this mixin, call connectKeyNavHandlers() in
		//		postCreate() and call startupKeyNavChildren() in startup().
		//		It provides normalized keyboard and focusing code for Container
		//		widgets.
/*=====
		// focusedChild: [protected] Widget
		//		The currently focused child widget, or null if there isn't one
		focusedChild: null,
=====*/

		// tabIndex: Integer
		//		Tab index of the container; same as HTML tabIndex attribute.
		//		Note then when user tabs into the container, focus is immediately
		//		moved to the first item in the container.
		tabIndex: "0",

		_keyNavCodes: {},

		connectKeyNavHandlers: function(/*dojo.keys[]*/ prevKeyCodes, /*dojo.keys[]*/ nextKeyCodes){
			// summary:
			//		Call in postCreate() to attach the keyboard handlers
			//		to the container.
			// preKeyCodes: dojo.keys[]
			//		Key codes for navigating to the previous child.
			// nextKeyCodes: dojo.keys[]
			//		Key codes for navigating to the next child.
			// tags:
			//		protected

			var keyCodes = (this._keyNavCodes = {});
			var prev = dojo.hitch(this, this.focusPrev);
			var next = dojo.hitch(this, this.focusNext);
			dojo.forEach(prevKeyCodes, function(code){ keyCodes[code] = prev; });
			dojo.forEach(nextKeyCodes, function(code){ keyCodes[code] = next; });
			this.connect(this.domNode, "onkeypress", "_onContainerKeypress");
			this.connect(this.domNode, "onfocus", "_onContainerFocus");
		},

		startupKeyNavChildren: function(){
			// summary:
			//		Call in startup() to set child tabindexes to -1
			// tags:
			//		protected
			dojo.forEach(this.getChildren(), dojo.hitch(this, "_startupChild"));
		},

		addChild: function(/*dijit._Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Add a child to our _Container
			dijit._KeyNavContainer.superclass.addChild.apply(this, arguments);
			this._startupChild(widget);
		},

		focus: function(){
			// summary:
			//		Default focus() implementation: focus the first child.
			this.focusFirstChild();
		},

		focusFirstChild: function(){
			// summary:
			//		Focus the first focusable child in the container.
			// tags:
			//		protected
			var child = this._getFirstFocusableChild();
			if(child){ // edge case: Menu could be empty or hidden
				this.focusChild(child);
			}
		},

		focusNext: function(){
			// summary:
			//		Focus the next widget
			// tags:
			//		protected
			var child = this._getNextFocusableChild(this.focusedChild, 1);
			this.focusChild(child);
		},

		focusPrev: function(){
			// summary:
			//		Focus the last focusable node in the previous widget
			//		(ex: go to the ComboButton icon section rather than button section)
			// tags:
			//		protected
			var child = this._getNextFocusableChild(this.focusedChild, -1);
			this.focusChild(child, true);
		},

		focusChild: function(/*dijit._Widget*/ widget, /*Boolean*/ last){
			// summary:
			//		Focus widget.
			// widget:
			//		Reference to container's child widget
			// last:
			//		If true and if widget has multiple focusable nodes, focus the
			//		last one instead of the first one
			// tags:
			//		protected
			
			if(this.focusedChild && widget !== this.focusedChild){
				this._onChildBlur(this.focusedChild);
			}
			widget.focus(last ? "end" : "start");
			this.focusedChild = widget;
		},

		_startupChild: function(/*dijit._Widget*/ widget){
			// summary:
			//		Setup for each child widget
			// description:
			//		Sets tabIndex=-1 on each child, so that the tab key will 
			//		leave the container rather than visiting each child.
			// tags:
			//		private
			
			widget.set("tabIndex", "-1");
			
			this.connect(widget, "_onFocus", function(){
				// Set valid tabIndex so tabbing away from widget goes to right place, see #10272
				widget.set("tabIndex", this.tabIndex);
			});
			this.connect(widget, "_onBlur", function(){
				widget.set("tabIndex", "-1");
			});
		},

		_onContainerFocus: function(evt){
			// summary:
			//		Handler for when the container gets focus
			// description:
			//		Initially the container itself has a tabIndex, but when it gets
			//		focus, switch focus to first child...
			// tags:
			//		private

			// Note that we can't use _onFocus() because switching focus from the
			// _onFocus() handler confuses the focus.js code
			// (because it causes _onFocusNode() to be called recursively)

			// focus bubbles on Firefox,
			// so just make sure that focus has really gone to the container
			if(evt.target !== this.domNode){ return; }

			this.focusFirstChild();

			// and then set the container's tabIndex to -1,
			// (don't remove as that breaks Safari 4)
			// so that tab or shift-tab will go to the fields after/before
			// the container, rather than the container itself
			dojo.attr(this.domNode, "tabIndex", "-1");
		},

		_onBlur: function(evt){
			// When focus is moved away the container, and its descendant (popup) widgets,
			// then restore the container's tabIndex so that user can tab to it again.
			// Note that using _onBlur() so that this doesn't happen when focus is shifted
			// to one of my child widgets (typically a popup)
			if(this.tabIndex){
				dojo.attr(this.domNode, "tabIndex", this.tabIndex);
			}
			this.inherited(arguments);
		},

		_onContainerKeypress: function(evt){
			// summary:
			//		When a key is pressed, if it's an arrow key etc. then
			//		it's handled here.
			// tags:
			//		private
			if(evt.ctrlKey || evt.altKey){ return; }
			var func = this._keyNavCodes[evt.charOrCode];
			if(func){
				func();
				dojo.stopEvent(evt);
			}
		},

		_onChildBlur: function(/*dijit._Widget*/ widget){
			// summary:
			//		Called when focus leaves a child widget to go
			//		to a sibling widget.
			// tags:
			//		protected
		},

		_getFirstFocusableChild: function(){
			// summary:
			//		Returns first child that can be focused
			return this._getNextFocusableChild(null, 1);	// dijit._Widget
		},

		_getNextFocusableChild: function(child, dir){
			// summary:
			//		Returns the next or previous focusable child, compared
			//		to "child"
			// child: Widget
			//		The current widget
			// dir: Integer
			//		* 1 = after
			//		* -1 = before
			if(child){
				child = this._getSiblingOfChild(child, dir);
			}
			var children = this.getChildren();
			for(var i=0; i < children.length; i++){
				if(!child){
					child = children[(dir>0) ? 0 : (children.length-1)];
				}
				if(child.isFocusable()){
					return child;	// dijit._Widget
				}
				child = this._getSiblingOfChild(child, dir);
			}
			// no focusable child found
			return null;	// dijit._Widget
		}
	}
);

}

if(!dojo._hasResource["dijit._CssStateMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._CssStateMixin"] = true;
dojo.provide("dijit._CssStateMixin");


dojo.declare("dijit._CssStateMixin", [], {
	// summary:
	//		Mixin for widgets to set CSS classes on the widget DOM nodes depending on hover/mouse press/focus
	//		state changes, and also higher-level state changes such becoming disabled or selected.
	//
	// description:
	//		By mixing this class into your widget, and setting the this.baseClass attribute, it will automatically
	//		maintain CSS classes on the widget root node (this.domNode) depending on hover,
	//		active, focus, etc. state.   Ex: with a baseClass of dijitButton, it will apply the classes
	//		dijitButtonHovered and dijitButtonActive, as the user moves the mouse over the widget and clicks it.
	//
	//		It also sets CSS like dijitButtonDisabled based on widget semantic state.
	//
	//		By setting the cssStateNodes attribute, a widget can also track events on subnodes (like buttons
	//		within the widget).

	// cssStateNodes: [protected] Object
	//		List of sub-nodes within the widget that need CSS classes applied on mouse hover/press and focus
	//.
	//		Each entry in the hash is a an attachpoint names (like "upArrowButton") mapped to a CSS class names
	//		(like "dijitUpArrowButton"). Example:
	//	|		{
	//	|			"upArrowButton": "dijitUpArrowButton",
	//	|			"downArrowButton": "dijitDownArrowButton"
	//	|		}
	//		The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it
	//		is hovered, etc.
	cssStateNodes: {},

	postCreate: function(){
		this.inherited(arguments);

		// Automatically monitor mouse events (essentially :hover and :active) on this.domNode
		dojo.forEach(["onmouseenter", "onmouseleave", "onmousedown"], function(e){
			this.connect(this.domNode, e, "_cssMouseEvent");
		}, this);
		
		// Monitoring changes to disabled, readonly, etc. state, and update CSS class of root node
		this.connect(this, "set", function(name, value){
			if(arguments.length >= 2 && {disabled: true, readOnly: true, checked:true, selected:true}[name]){
				this._setStateClass();
			}
		});

		// The widget coming in/out of the focus change affects it's state
		dojo.forEach(["_onFocus", "_onBlur"], function(ap){
			this.connect(this, ap, "_setStateClass");
		}, this);

		// Events on sub nodes within the widget
		for(var ap in this.cssStateNodes){
			this._trackMouseState(this[ap], this.cssStateNodes[ap]);
		}
		// Set state initially; there's probably no hover/active/focus state but widget might be
		// disabled/readonly so we want to set CSS classes for those conditions.
		this._setStateClass();
	},

	_cssMouseEvent: function(/*Event*/ event){
		// summary:
		//	Sets _hovering and _active properties depending on mouse state,
		//	then calls _setStateClass() to set appropriate CSS classes for this.domNode.

		if(!this.disabled){
			switch(event.type){
				case "mouseenter":
				case "mouseover":	// generated on non-IE browsers even though we connected to mouseenter
					this._hovering = true;
					this._active = this._mouseDown;
					break;

				case "mouseleave":
				case "mouseout":	// generated on non-IE browsers even though we connected to mouseleave
					this._hovering = false;
					this._active = false;
					break;

				case "mousedown" :
					this._active = true;
					this._mouseDown = true;
					// Set a global event to handle mouseup, so it fires properly
					// even if the cursor leaves this.domNode before the mouse up event.
					// Alternately could set active=false on mouseout.
					var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
						this._active = false;
						this._mouseDown = false;
						this._setStateClass();
						this.disconnect(mouseUpConnector);
					});
					break;
			}
			this._setStateClass();
		}
	},

	_setStateClass: function(){
		// summary:
		//		Update the visual state of the widget by setting the css classes on this.domNode
		//		(or this.stateNode if defined) by combining this.baseClass with
		//		various suffixes that represent the current widget state(s).
		//
		// description:
		//		In the case where a widget has multiple
		//		states, it sets the class based on all possible
		//	 	combinations.  For example, an invalid form widget that is being hovered
		//		will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
		//
		//		The widget may have one or more of the following states, determined
		//		by this.state, this.checked, this.valid, and this.selected:
		//			- Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
		//			- Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
		//			- Selected - ex: currently selected tab will have this.selected==true
		//
		//		In addition, it may have one or more of the following states,
		//		based on this.disabled and flags set in _onMouse (this._active, this._hovering, this._focused):
		//			- Disabled	- if the widget is disabled
		//			- Active		- if the mouse (or space/enter key?) is being pressed down
		//			- Focused		- if the widget has focus
		//			- Hover		- if the mouse is over the widget

		// Compute new set of classes
		var newStateClasses = this.baseClass.split(" ");

		function multiply(modifier){
			newStateClasses = newStateClasses.concat(dojo.map(newStateClasses, function(c){ return c+modifier; }), "dijit"+modifier);
		}

		if(!this.isLeftToRight()){
			// For RTL mode we need to set an addition class like dijitTextBoxRtl.
			multiply("Rtl");
		}

		if(this.checked){
			multiply("Checked");
		}
		if(this.state){
			multiply(this.state);
		}
		if(this.selected){
			multiply("Selected");
		}

		if(this.disabled){
			multiply("Disabled");
		}else if(this.readOnly){
			multiply("ReadOnly");
		}else{
			if(this._active){
				multiply("Active");
			}else if(this._hovering){
				multiply("Hover");
			}
		}

		if(this._focused){
			multiply("Focused");
		}

		// Remove old state classes and add new ones.
		// For performance concerns we only write into domNode.className once.
		var tn = this.stateNode || this.domNode,
			classHash = {};	// set of all classes (state and otherwise) for node

		dojo.forEach(tn.className.split(" "), function(c){ classHash[c] = true; });

		if("_stateClasses" in this){
			dojo.forEach(this._stateClasses, function(c){ delete classHash[c]; });
		}

		dojo.forEach(newStateClasses, function(c){ classHash[c] = true; });

		var newClasses = [];
		for(var c in classHash){
			newClasses.push(c);
		}
		tn.className = newClasses.join(" ");

		this._stateClasses = newStateClasses;
	},

	_trackMouseState: function(/*DomNode*/ node, /*String*/ clazz){
		// summary:
		//		Track mouse/focus events on specified node and set CSS class on that node to indicate
		//		current state.   Usually not called directly, but via cssStateNodes attribute.
		// description:
		//		Given class=foo, will set the following CSS class on the node
		//			- fooActive: if the user is currently pressing down the mouse button while over the node
		//			- fooHover: if the user is hovering the mouse over the node, but not pressing down a button
		//			- fooFocus: if the node is focused
		//
		//		Note that it won't set any classes if the widget is disabled.
		// node: DomNode
		//		Should be a sub-node of the widget, not the top node (this.domNode), since the top node
		//		is handled specially and automatically just by mixing in this class.
		// clazz: String
		//		CSS class name (ex: dijitSliderUpArrow).

		// Current state of node (initially false)
		// NB: setting specifically to false because dojo.toggleClass() needs true boolean as third arg
		var hovering=false, active=false, focused=false;

		var self = this,
			cn = dojo.hitch(this, "connect", node);

		function setClass(){
			var disabled = ("disabled" in self && self.disabled) || ("readonly" in self && self.readonly);
			dojo.toggleClass(node, clazz+"Hover", hovering && !active && !disabled);
			dojo.toggleClass(node, clazz+"Active", active && !disabled);
			dojo.toggleClass(node, clazz+"Focused", focused && !disabled);
		}

		// Mouse
		cn("onmouseenter", function(){
			hovering = true;
			setClass();
		});
		cn("onmouseleave", function(){
			hovering = false;
			active = false;
			setClass();
		});
		cn("onmousedown", function(){
			active = true;
			setClass();
		});
		cn("onmouseup", function(){
			active = false;
			setClass();
		});

		// Focus
		cn("onfocus", function(){
			focused = true;
			setClass();
		});
		cn("onblur", function(){
			focused = false;
			setClass();
		});

		// Just in case widget is enabled/disabled while it has focus/hover/active state.
		// Maybe this is overkill.
		this.connect(this, "set", function(name, value){
			if(name == "disabled" || name == "readOnly"){
				setClass();
			}
		});
	}
});

}

if(!dojo._hasResource["dijit.form._FormWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormWidget"] = true;
dojo.provide("dijit.form._FormWidget");







dojo.declare("dijit.form._FormWidget", [dijit._Widget, dijit._Templated, dijit._CssStateMixin],
	{
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as <checkbox> or <button>,
	//		which can be children of a <form> node or a `dijit.form.Form` widget.
	//
	// description:
	//		Represents a single HTML element.
	//		All these widgets should have these attributes just like native HTML input elements.
	//		You can set them during widget construction or afterwards, via `dijit._Widget.attr`.
	//
	//		They also share some common methods.

	// name: String
	//		Name used when submitting form; same as "name" attribute or plain HTML elements
	name: "",

	// alt: String
	//		Corresponds to the native HTML <input> element's attribute.
	alt: "",

	// value: String
	//		Corresponds to the native HTML <input> element's attribute.
	value: "",

	// type: String
	//		Corresponds to the native HTML <input> element's attribute.
	type: "text",

	// tabIndex: Integer
	//		Order fields are traversed when user hits the tab key
	tabIndex: "0",

	// disabled: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "disabled='disabled'", or just "disabled".
	disabled: false,

	// intermediateChanges: Boolean
	//		Fires onChange for each value change or only on demand
	intermediateChanges: false,

	// scrollOnFocus: Boolean
	//		On focus, should this widget scroll into view?
	scrollOnFocus: true,

	// These mixins assume that the focus node is an INPUT, as many but not all _FormWidgets are.
	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
		value: "focusNode",
		id: "focusNode",
		tabIndex: "focusNode",
		alt: "focusNode",
		title: "focusNode"
	}),

	postMixInProperties: function(){
		// Setup name=foo string to be referenced from the template (but only if a name has been specified)
		// Unfortunately we can't use attributeMap to set the name due to IE limitations, see #8660
		// Regarding escaping, see heading "Attribute values" in
		// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
		this.nameAttrSetting = this.name ? ('name="' + this.name.replace(/'/g, "&quot;") + '"') : '';
		this.inherited(arguments);
	},

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.domNode, "onmousedown", "_onMouseDown");
	},

	_setDisabledAttr: function(/*Boolean*/ value){
		this.disabled = value;
		dojo.attr(this.focusNode, 'disabled', value);
		if(this.valueNode){
			dojo.attr(this.valueNode, 'disabled', value);
		}
		dijit.setWaiState(this.focusNode, "disabled", value);

		if(value){
			// reset these, because after the domNode is disabled, we can no longer receive
			// mouse related events, see #4200
			this._hovering = false;
			this._active = false;

			// clear tab stop(s) on this widget's focusable node(s)  (ComboBox has two focusable nodes)
			var attachPointNames = "tabIndex" in this.attributeMap ? this.attributeMap.tabIndex : "focusNode";
			dojo.forEach(dojo.isArray(attachPointNames) ? attachPointNames : [attachPointNames], function(attachPointName){
				var node = this[attachPointName];
				// complex code because tabIndex=-1 on a <div> doesn't work on FF
				if(dojo.isWebKit || dijit.hasDefaultTabStop(node)){	// see #11064 about webkit bug
					node.setAttribute('tabIndex', "-1");
				}else{
					node.removeAttribute('tabIndex');				
				}
			}, this);
		}else{
			this.focusNode.setAttribute('tabIndex', this.tabIndex);
		}
	},

	setDisabled: function(/*Boolean*/ disabled){
		// summary:
		//		Deprecated.   Use set('disabled', ...) instead.
		dojo.deprecated("setDisabled("+disabled+") is deprecated. Use set('disabled',"+disabled+") instead.", "", "2.0");
		this.set('disabled', disabled);
	},

	_onFocus: function(e){
		if(this.scrollOnFocus){
			dojo.window.scrollIntoView(this.domNode);
		}
		this.inherited(arguments);
	},

	isFocusable: function(){
		// summary:
		//		Tells if this widget is focusable or not.   Used internally by dijit.
		// tags:
		//		protected
		return !this.disabled && this.focusNode && (dojo.style(this.domNode, "display") != "none");
	},

	focus: function(){
		// summary:
		//		Put focus on this widget
		if(!this.disabled){
			dijit.focus(this.focusNode);
		}
	},

	compare: function(/*anything*/val1, /*anything*/val2){
		// summary:
		//		Compare 2 values (as returned by attr('value') for this widget).
		// tags:
		//		protected
		if(typeof val1 == "number" && typeof val2 == "number"){
			return (isNaN(val1) && isNaN(val2)) ? 0 : val1 - val2;
		}else if(val1 > val2){
			return 1;
		}else if(val1 < val2){
			return -1;
		}else{
			return 0;
		}
	},

	onChange: function(newValue){
		// summary:
		//		Callback when this widget's value is changed.
		// tags:
		//		callback
	},

	// _onChangeActive: [private] Boolean
	//		Indicates that changes to the value should call onChange() callback.
	//		This is false during widget initialization, to avoid calling onChange()
	//		when the initial value is set.
	_onChangeActive: false,

	_handleOnChange: function(/*anything*/ newValue, /* Boolean? */ priorityChange){
		// summary:
		//		Called when the value of the widget is set.  Calls onChange() if appropriate
		// newValue:
		//		the new value
		// priorityChange:
		//		For a slider, for example, dragging the slider is priorityChange==false,
		//		but on mouse up, it's priorityChange==true.  If intermediateChanges==true,
		//		onChange is only called form priorityChange=true events.
		// tags:
		//		private
		this._lastValue = newValue;
		if(this._lastValueReported == undefined && (priorityChange === null || !this._onChangeActive)){
			// this block executes not for a change, but during initialization,
			// and is used to store away the original value (or for ToggleButton, the original checked state)
			this._resetValue = this._lastValueReported = newValue;
		}
		if((this.intermediateChanges || priorityChange || priorityChange === undefined) &&
			((typeof newValue != typeof this._lastValueReported) ||
				this.compare(newValue, this._lastValueReported) != 0)){
			this._lastValueReported = newValue;
			if(this._onChangeActive){
				if(this._onChangeHandle){
					clearTimeout(this._onChangeHandle);
				}
				// setTimout allows hidden value processing to run and
				// also the onChange handler can safely adjust focus, etc
				this._onChangeHandle = setTimeout(dojo.hitch(this,
					function(){
						this._onChangeHandle = null;
						this.onChange(newValue);
					}), 0); // try to collapse multiple onChange's fired faster than can be processed
			}
		}
	},

	create: function(){
		// Overrides _Widget.create()
		this.inherited(arguments);
		this._onChangeActive = true;
	},

	destroy: function(){
		if(this._onChangeHandle){ // destroy called before last onChange has fired
			clearTimeout(this._onChangeHandle);
			this.onChange(this._lastValueReported);
		}
		this.inherited(arguments);
	},

	setValue: function(/*String*/ value){
		// summary:
		//		Deprecated.   Use set('value', ...) instead.
		dojo.deprecated("dijit.form._FormWidget:setValue("+value+") is deprecated.  Use set('value',"+value+") instead.", "", "2.0");
		this.set('value', value);
	},

	getValue: function(){
		// summary:
		//		Deprecated.   Use get('value') instead.
		dojo.deprecated(this.declaredClass+"::getValue() is deprecated. Use get('value') instead.", "", "2.0");
		return this.get('value');
	},
	
	_onMouseDown: function(e){
		// If user clicks on the button, even if the mouse is released outside of it,
		// this button should get focus (to mimics native browser buttons).
		// This is also needed on chrome because otherwise buttons won't get focus at all,
		// which leads to bizarre focus restore on Dialog close etc.
		if(!e.ctrlKey && this.isFocusable()){ // !e.ctrlKey to ignore right-click on mac
			// Set a global event to handle mouseup, so it fires properly
			// even if the cursor leaves this.domNode before the mouse up event.
			var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
				if (this.isFocusable()) {
					this.focus();
				}
				this.disconnect(mouseUpConnector);
			});
		}
	}
});

dojo.declare("dijit.form._FormValueWidget", dijit.form._FormWidget,
{
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as <input> or <select> that have user changeable values.
	// description:
	//		Each _FormValueWidget represents a single input value, and has a (possibly hidden) <input> element,
	//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
	//		works as expected.

	// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
	// directly in the template as read by the parser in order to function. IE is known to specifically
	// require the 'name' attribute at element creation time.   See #8484, #8660.
	// TODO: unclear what that {value: ""} is for; FormWidget.attributeMap copies value to focusNode,
	// so maybe {value: ""} is so the value *doesn't* get copied to focusNode?
	// Seems like we really want value removed from attributeMap altogether
	// (although there's no easy way to do that now)

	// readOnly: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "readOnly".
	//		Similar to disabled except readOnly form values are submitted.
	readOnly: false,

	attributeMap: dojo.delegate(dijit.form._FormWidget.prototype.attributeMap, {
		value: "",
		readOnly: "focusNode"
	}),

	_setReadOnlyAttr: function(/*Boolean*/ value){
		this.readOnly = value;
		dojo.attr(this.focusNode, 'readOnly', value);
		dijit.setWaiState(this.focusNode, "readonly", value);
	},

	postCreate: function(){
		this.inherited(arguments);

		if(dojo.isIE){ // IE won't stop the event with keypress
			this.connect(this.focusNode || this.domNode, "onkeydown", this._onKeyDown);
		}
		// Update our reset value if it hasn't yet been set (because this.set()
		// is only called when there *is* a value)
		if(this._resetValue === undefined){
			this._resetValue = this.value;
		}
	},

	_setValueAttr: function(/*anything*/ newValue, /*Boolean, optional*/ priorityChange){
		// summary:
		//		Hook so attr('value', value) works.
		// description:
		//		Sets the value of the widget.
		//		If the value has changed, then fire onChange event, unless priorityChange
		//		is specified as null (or false?)
		this.value = newValue;
		this._handleOnChange(newValue, priorityChange);
	},

	_getValueAttr: function(){
		// summary:
		//		Hook so attr('value') works.
		return this._lastValue;
	},

	undo: function(){
		// summary:
		//		Restore the value to the last value passed to onChange
		this._setValueAttr(this._lastValueReported, false);
	},

	reset: function(){
		// summary:
		//		Reset the widget's value to what it was at initialization time
		this._hasBeenBlurred = false;
		this._setValueAttr(this._resetValue, true);
	},

	_onKeyDown: function(e){
		if(e.keyCode == dojo.keys.ESCAPE && !(e.ctrlKey || e.altKey || e.metaKey)){
			var te;
			if(dojo.isIE){
				e.preventDefault(); // default behavior needs to be stopped here since keypress is too late
				te = document.createEventObject();
				te.keyCode = dojo.keys.ESCAPE;
				te.shiftKey = e.shiftKey;
				e.srcElement.fireEvent('onkeypress', te);
			}
		}
	},

	_layoutHackIE7: function(){
		// summary:
		//		Work around table sizing bugs on IE7 by forcing redraw

		if(dojo.isIE == 7){ // fix IE7 layout bug when the widget is scrolled out of sight
			var domNode = this.domNode;
			var parent = domNode.parentNode;
			var pingNode = domNode.firstChild || domNode; // target node most unlikely to have a custom filter
			var origFilter = pingNode.style.filter; // save custom filter, most likely nothing
			var _this = this;
			while(parent && parent.clientHeight == 0){ // search for parents that haven't rendered yet
				(function ping(){
					var disconnectHandle = _this.connect(parent, "onscroll",
						function(e){
							_this.disconnect(disconnectHandle); // only call once
							pingNode.style.filter = (new Date()).getMilliseconds(); // set to anything that's unique
							setTimeout(function(){ pingNode.style.filter = origFilter }, 0); // restore custom filter, if any
						}
					);
				})();
				parent = parent.parentNode;
			}
		}
	}
});

}

if(!dojo._hasResource["dijit._HasDropDown"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._HasDropDown"] = true;
dojo.provide("dijit._HasDropDown");




dojo.declare("dijit._HasDropDown",
	null,
	{
		// summary:
		//		Mixin for widgets that need drop down ability.

		// _buttonNode: [protected] DomNode
		//		The button/icon/node to click to display the drop down.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then either focusNode or domNode (if focusNode is also missing) will be used.
		_buttonNode: null,

		// _arrowWrapperNode: [protected] DomNode
		//		Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending
		//		on where the drop down is set to be positioned.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then _buttonNode will be used.
		_arrowWrapperNode: null,

		// _popupStateNode: [protected] DomNode
		//		The node to set the popupActive class on.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then focusNode or _buttonNode (if focusNode is missing) will be used.
		_popupStateNode: null,

		// _aroundNode: [protected] DomNode
		//		The node to display the popup around.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then domNode will be used.
		_aroundNode: null,

		// dropDown: [protected] Widget
		//		The widget to display as a popup.  This widget *must* be
		//		defined before the startup function is called.
		dropDown: null,

		// autoWidth: [protected] Boolean
		//		Set to true to make the drop down at least as wide as this
		//		widget.  Set to false if the drop down should just be its
		//		default width
		autoWidth: true,

		// forceWidth: [protected] Boolean
		//		Set to true to make the drop down exactly as wide as this
		//		widget.  Overrides autoWidth.
		forceWidth: false,

		// maxHeight: [protected] Integer
		//		The max height for our dropdown.  Set to 0 for no max height.
		//		any dropdown taller than this will have scrollbars
		maxHeight: 0,

		// dropDownPosition: [const] String[]
		//		This variable controls the position of the drop down.
		//		It's an array of strings with the following values:
		//
		//			* before: places drop down to the left of the target node/widget, or to the right in
		//			  the case of RTL scripts like Hebrew and Arabic
		//			* after: places drop down to the right of the target node/widget, or to the left in
		//			  the case of RTL scripts like Hebrew and Arabic
		//			* above: drop down goes above target node
		//			* below: drop down goes below target node
		//
		//		The list is positions is tried, in order, until a position is found where the drop down fits
		//		within the viewport.
		//
		dropDownPosition: ["below","above"],

		// _stopClickEvents: Boolean
		//		When set to false, the click events will not be stopped, in
		//		case you want to use them in your subwidget
		_stopClickEvents: true,

		_onDropDownMouseDown: function(/*Event*/ e){
			// summary:
			//		Callback when the user mousedown's on the arrow icon

			if(this.disabled || this.readOnly){ return; }

			this._docHandler = this.connect(dojo.doc, "onmouseup", "_onDropDownMouseUp");

			this.toggleDropDown();
		},

		_onDropDownMouseUp: function(/*Event?*/ e){
			// summary:
			//		Callback when the user lifts their mouse after mouse down on the arrow icon.
			//		If the drop is a simple menu and the mouse is over the menu, we execute it, otherwise, we focus our
			//		dropDown node.  If the event is missing, then we are not
			//		a mouseup event.
			//
			//		This is useful for the common mouse movement pattern
			//		with native browser <select> nodes:
			//			1. mouse down on the select node (probably on the arrow)
			//			2. move mouse to a menu item while holding down the mouse button
			//			3. mouse up.  this selects the menu item as though the user had clicked it.
			if(e && this._docHandler){
				this.disconnect(this._docHandler);
			}
			var dropDown = this.dropDown, overMenu = false;

			if(e && this._opened){
				// This code deals with the corner-case when the drop down covers the original widget,
				// because it's so large.  In that case mouse-up shouldn't select a value from the menu.
				// Find out if our target is somewhere in our dropdown widget,
				// but not over our _buttonNode (the clickable node)
				var c = dojo.position(this._buttonNode, true);
				if(!(e.pageX >= c.x && e.pageX <= c.x + c.w) ||
					!(e.pageY >= c.y && e.pageY <= c.y + c.h)){
					var t = e.target;
					while(t && !overMenu){
						if(dojo.hasClass(t, "dijitPopup")){
							overMenu = true;
						}else{
							t = t.parentNode;
						}
					}
					if(overMenu){
						t = e.target;
						if(dropDown.onItemClick){
							var menuItem;
							while(t && !(menuItem = dijit.byNode(t))){
								t = t.parentNode;
							}
							if(menuItem && menuItem.onClick && menuItem.getParent){
								menuItem.getParent().onItemClick(menuItem, e);
							}
						}
						return;
					}
				}
			}
			if(this._opened && dropDown.focus && dropDown.autoFocus !== false){
				// Focus the dropdown widget - do it on a delay so that we
				// don't steal our own focus.
				window.setTimeout(dojo.hitch(dropDown, "focus"), 1);
			}
		},

		_onDropDownClick: function(/*Event*/ e){
			// the drop down was already opened on mousedown/keydown; just need to call stopEvent()
			if(this._stopClickEvents){
				dojo.stopEvent(e);
			}			
		},

		_setupDropdown: function(){
			// summary:
			//		set up nodes and connect our mouse and keypress events
			this._buttonNode = this._buttonNode || this.focusNode || this.domNode;
			this._popupStateNode = this._popupStateNode || this.focusNode || this._buttonNode;
			this._aroundNode = this._aroundNode || this.domNode;
			this.connect(this._buttonNode, "onmousedown", "_onDropDownMouseDown");
			this.connect(this._buttonNode, "onclick", "_onDropDownClick");
			this.connect(this._buttonNode, "onkeydown", "_onDropDownKeydown");
			this.connect(this._buttonNode, "onkeyup", "_onKey");

			// If we have a _setStateClass function (which happens when
			// we are a form widget), then we need to connect our open/close
			// functions to it
			if(this._setStateClass){
				this.connect(this, "openDropDown", "_setStateClass");
				this.connect(this, "closeDropDown", "_setStateClass");
			}

			// Add a class to the "dijitDownArrowButton" type class to _buttonNode so theme can set direction of arrow
			// based on where drop down will normally appear
			var defaultPos = {
					"after" : this.isLeftToRight() ? "Right" : "Left",
					"before" : this.isLeftToRight() ? "Left" : "Right",
					"above" : "Up",
					"below" : "Down",
					"left" : "Left",
					"right" : "Right"
			}[this.dropDownPosition[0]] || this.dropDownPosition[0] || "Down";
			dojo.addClass(this._arrowWrapperNode || this._buttonNode, "dijit" + defaultPos + "ArrowButton");
		},

		postCreate: function(){
			this._setupDropdown();
			this.inherited(arguments);
		},

		destroyDescendants: function(){
			if(this.dropDown){
				// Destroy the drop down, unless it's already been destroyed.  This can happen because
				// the drop down is a direct child of <body> even though it's logically my child.
				if(!this.dropDown._destroyed){
					this.dropDown.destroyRecursive();
				}
				delete this.dropDown;
			}
			this.inherited(arguments);
		},

		_onDropDownKeydown: function(/*Event*/ e){
			if(e.keyCode == dojo.keys.DOWN_ARROW || e.keyCode == dojo.keys.ENTER || e.keyCode == dojo.keys.SPACE){
				e.preventDefault();	// stop IE screen jump
			}
		},

		_onKey: function(/*Event*/ e){
			// summary:
			//		Callback when the user presses a key while focused on the button node

			if(this.disabled || this.readOnly){ return; }
			var d = this.dropDown;
			if(d && this._opened && d.handleKey){
				if(d.handleKey(e) === false){ return; }
			}
			if(d && this._opened && e.keyCode == dojo.keys.ESCAPE){
				this.toggleDropDown();
			}else if(d && !this._opened && 
					(e.keyCode == dojo.keys.DOWN_ARROW || e.keyCode == dojo.keys.ENTER || e.keyCode == dojo.keys.SPACE)){
				this.toggleDropDown();
				if(d.focus){
					setTimeout(dojo.hitch(d, "focus"), 1);
				}
			}
		},

		_onBlur: function(){
			// summary:
			//		Called magically when focus has shifted away from this widget and it's dropdown

			this.closeDropDown();
			// don't focus on button.  the user has explicitly focused on something else.
			this.inherited(arguments);
		},

		isLoaded: function(){
			// summary:
			//		Returns whether or not the dropdown is loaded.  This can
			//		be overridden in order to force a call to loadDropDown().
			// tags:
			//		protected

			return true;
		},

		loadDropDown: function(/* Function */ loadCallback){
			// summary:
			//		Loads the data for the dropdown, and at some point, calls
			//		the given callback
			// tags:
			//		protected

			loadCallback();
		},

		toggleDropDown: function(){
			// summary:
			//		Toggle the drop-down widget; if it is up, close it, if not, open it
			// tags:
			//		protected

			if(this.disabled || this.readOnly){ return; }
			this.focus();
			var dropDown = this.dropDown;
			if(!dropDown){ return; }
			if(!this._opened){
				// If we aren't loaded, load it first so there isn't a flicker
				if(!this.isLoaded()){
					this.loadDropDown(dojo.hitch(this, "openDropDown"));
					return;
				}else{
					this.openDropDown();
				}
			}else{
				this.closeDropDown();
			}
		},

		openDropDown: function(){
			// summary:
			//		Opens the dropdown for this widget - it returns the
			//		return value of dijit.popup.open
			// tags:
			//		protected

			var dropDown = this.dropDown;
			var ddNode = dropDown.domNode;
			var self = this;

			// Prepare our popup's height and honor maxHeight if it exists.

			// TODO: isn't maxHeight dependent on the return value from dijit.popup.open(),
			// ie, dependent on how much space is available (BK)

			if(!this._preparedNode){
				dijit.popup.moveOffScreen(dropDown);
				this._preparedNode = true;			
				// Check if we have explicitly set width and height on the dropdown widget dom node
				if(ddNode.style.width){
					this._explicitDDWidth = true;
				}
				if(ddNode.style.height){
					this._explicitDDHeight = true;
				}
			}

			// Code for resizing dropdown (height limitation, or increasing width to match my width)
			if(this.maxHeight || this.forceWidth || this.autoWidth){
				var myStyle = {
					display: "",
					visibility: "hidden"
				};
				if(!this._explicitDDWidth){
					myStyle.width = "";
				}
				if(!this._explicitDDHeight){
					myStyle.height = "";
				}
				dojo.style(ddNode, myStyle);
				
				// Get size of drop down, and determine if vertical scroll bar needed
				var mb = dojo.marginBox(ddNode);
				var overHeight = (this.maxHeight && mb.h > this.maxHeight);
				dojo.style(ddNode, {
					overflowX: "hidden",
					overflowY: overHeight ? "auto" : "hidden"
				});
				if(overHeight){
					mb.h = this.maxHeight;
					if("w" in mb){
						mb.w += 16;	// room for vertical scrollbar
					}
				}else{
					delete mb.h;
				}
				delete mb.t;
				delete mb.l;

				// Adjust dropdown width to match or be larger than my width
				if(this.forceWidth){
					mb.w = this.domNode.offsetWidth;
				}else if(this.autoWidth){
					mb.w = Math.max(mb.w, this.domNode.offsetWidth);
				}else{
					delete mb.w;
				}
				
				// And finally, resize the dropdown to calculated height and width
				if(dojo.isFunction(dropDown.resize)){
					dropDown.resize(mb);
				}else{
					dojo.marginBox(ddNode, mb);
				}
			}

			var retVal = dijit.popup.open({
				parent: this,
				popup: dropDown,
				around: this._aroundNode,
				orient: dijit.getPopupAroundAlignment((this.dropDownPosition && this.dropDownPosition.length) ? this.dropDownPosition : ["below"],this.isLeftToRight()),
				onExecute: function(){
					self.closeDropDown(true);
				},
				onCancel: function(){
					self.closeDropDown(true);
				},
				onClose: function(){
					dojo.attr(self._popupStateNode, "popupActive", false);
					dojo.removeClass(self._popupStateNode, "dijitHasDropDownOpen");
					self._opened = false;
					self.state = "";
				}
			});
			dojo.attr(this._popupStateNode, "popupActive", "true");
			dojo.addClass(self._popupStateNode, "dijitHasDropDownOpen");
			this._opened=true;
			this.state="Opened";
			// TODO: set this.checked and call setStateClass(), to affect button look while drop down is shown
			return retVal;
		},

		closeDropDown: function(/*Boolean*/ focus){
			// summary:
			//		Closes the drop down on this widget
			// tags:
			//		protected

			if(this._opened){
				if(focus){ this.focus(); }
				dijit.popup.close(this.dropDown);
				this._opened = false;
				this.state = "";
			}
		}

	}
);

}

if(!dojo._hasResource["dijit.form.Button"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Button"] = true;
dojo.provide("dijit.form.Button");





dojo.declare("dijit.form.Button",
	dijit.form._FormWidget,
	{
	// summary:
	//		Basically the same thing as a normal HTML button, but with special styling.
	// description:
	//		Buttons can display a label, an icon, or both.
	//		A label should always be specified (through innerHTML) or the label
	//		attribute.  It can be hidden via showLabel=false.
	// example:
	// |	<button dojoType="dijit.form.Button" onClick="...">Hello world</button>
	//
	// example:
	// |	var button1 = new dijit.form.Button({label: "hello world", onClick: foo});
	// |	dojo.body().appendChild(button1.domNode);

	// label: HTML String
	//		Text to display in button.
	//		If the label is hidden (showLabel=false) then and no title has
	//		been specified, then label is also set as title attribute of icon.
	label: "",

	// showLabel: Boolean
	//		Set this to true to hide the label text and display only the icon.
	//		(If showLabel=false then iconClass must be specified.)
	//		Especially useful for toolbars.
	//		If showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.
	//
	//		The exception case is for computers in high-contrast mode, where the label
	//		will still be displayed, since the icon doesn't appear.
	showLabel: true,

	// iconClass: String
	//		Class to apply to DOMNode in button to make it display an icon
	iconClass: "",

	// type: String
	//		Defines the type of button.  "button", "submit", or "reset".
	type: "button",

	baseClass: "dijitButton",

	templateString: dojo.cache("dijit.form", "templates/Button.html", "<span class=\"dijit dijitReset dijitInline\"\r\n\t><span class=\"dijitReset dijitInline dijitButtonNode\"\r\n\t\tdojoAttachEvent=\"ondijitclick:_onButtonClick\"\r\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\r\n\t\t\tdojoAttachPoint=\"titleNode,focusNode\"\r\n\t\t\twaiRole=\"button\" waiState=\"labelledby-${id}_label\"\r\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\" dojoAttachPoint=\"iconNode\"></span\r\n\t\t\t><span class=\"dijitReset dijitToggleButtonIconChar\">&#x25CF;</span\r\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\r\n\t\t\t\tid=\"${id}_label\"\r\n\t\t\t\tdojoAttachPoint=\"containerNode\"\r\n\t\t\t></span\r\n\t\t></span\r\n\t></span\r\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\"\r\n\t\tdojoAttachPoint=\"valueNode\"\r\n/></span>\r\n"),

	attributeMap: dojo.delegate(dijit.form._FormWidget.prototype.attributeMap, {
		value: "valueNode",
		iconClass: { node: "iconNode", type: "class" }
	}),


	_onClick: function(/*Event*/ e){
		// summary:
		//		Internal function to handle click actions
		if(this.disabled){
			return false;
		}
		this._clicked(); // widget click actions
		return this.onClick(e); // user click actions
	},

	_onButtonClick: function(/*Event*/ e){
		// summary:
		//		Handler when the user activates the button portion.
		if(this._onClick(e) === false){ // returning nothing is same as true
			e.preventDefault(); // needed for checkbox
		}else if(this.type == "submit" && !(this.valueNode||this.focusNode).form){ // see if a nonform widget needs to be signalled
			for(var node=this.domNode; node.parentNode/*#5935*/; node=node.parentNode){
				var widget=dijit.byNode(node);
				if(widget && typeof widget._onSubmit == "function"){
					widget._onSubmit(e);
					break;
				}
			}
		}else if(this.valueNode){
			this.valueNode.click();
			e.preventDefault(); // cancel BUTTON click and continue with hidden INPUT click
		}
	},

	_fillContent: function(/*DomNode*/ source){
		// Overrides _Templated._fillContent().
		// If button label is specified as srcNodeRef.innerHTML rather than
		// this.params.label, handle it here.
		if(source && (!this.params || !("label" in this.params))){
			this.set('label', source.innerHTML);
		}
	},

	postCreate: function(){
		dojo.setSelectable(this.focusNode, false);
		this.inherited(arguments);
	},

	_setShowLabelAttr: function(val){
		if(this.containerNode){
			dojo.toggleClass(this.containerNode, "dijitDisplayNone", !val);
		}
		this.showLabel = val;
	},

	onClick: function(/*Event*/ e){
		// summary:
		//		Callback for when button is clicked.
		//		If type="submit", return true to perform submit, or false to cancel it.
		// type:
		//		callback
		return true;		// Boolean
	},

	_clicked: function(/*Event*/ e){
		// summary:
		//		Internal overridable function for when the button is clicked
	},

	setLabel: function(/*String*/ content){
		// summary:
		//		Deprecated.  Use set('label', ...) instead.
		dojo.deprecated("dijit.form.Button.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
		this.set("label", content);
	},

	_setLabelAttr: function(/*String*/ content){
		// summary:
		//		Hook for attr('label', ...) to work.
		// description:
		//		Set the label (text) of the button; takes an HTML string.
		this.containerNode.innerHTML = this.label = content;
		if(this.showLabel == false && !this.params.title){
			this.titleNode.title = dojo.trim(this.containerNode.innerText || this.containerNode.textContent || '');
		}
	}
});


dojo.declare("dijit.form.DropDownButton", [dijit.form.Button, dijit._Container, dijit._HasDropDown], {
	// summary:
	//		A button with a drop down
	//
	// example:
	// |	<button dojoType="dijit.form.DropDownButton" label="Hello world">
	// |		<div dojotype="dijit.Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new dijit.form.DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
	// |	dojo.body().appendChild(button1);
	//

	baseClass : "dijitDropDownButton",

	templateString: dojo.cache("dijit.form", "templates/DropDownButton.html", "<span class=\"dijit dijitReset dijitInline\"\r\n\t><span class='dijitReset dijitInline dijitButtonNode'\r\n\t\tdojoAttachEvent=\"ondijitclick:_onButtonClick\" dojoAttachPoint=\"_buttonNode\"\r\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\r\n\t\t\tdojoAttachPoint=\"focusNode,titleNode,_arrowWrapperNode\"\r\n\t\t\twaiRole=\"button\" waiState=\"haspopup-true,labelledby-${id}_label\"\r\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\"\r\n\t\t\t\tdojoAttachPoint=\"iconNode\"\r\n\t\t\t></span\r\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\r\n\t\t\t\tdojoAttachPoint=\"containerNode,_popupStateNode\"\r\n\t\t\t\tid=\"${id}_label\"\r\n\t\t\t></span\r\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonInner\"></span\r\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonChar\">&#9660;</span\r\n\t\t></span\r\n\t></span\r\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\"\r\n\t\tdojoAttachPoint=\"valueNode\"\r\n/></span>\r\n"),

	_fillContent: function(){
		// Overrides Button._fillContent().
		//
		// My inner HTML contains both the button contents and a drop down widget, like
		// <DropDownButton>  <span>push me</span>  <Menu> ... </Menu> </DropDownButton>
		// The first node is assumed to be the button content. The widget is the popup.

		if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
			//FIXME: figure out how to filter out the widget and use all remaining nodes as button
			//	content, not just nodes[0]
			var nodes = dojo.query("*", this.srcNodeRef);
			dijit.form.DropDownButton.superclass._fillContent.call(this, nodes[0]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		if(this._started){ return; }

		// the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
		// make it invisible, and store a reference to pass to the popup code.
		if(!this.dropDown){
			var dropDownNode = dojo.query("[widgetId]", this.dropDownContainer)[0];
			this.dropDown = dijit.byNode(dropDownNode);
			delete this.dropDownContainer;
		}
		dijit.popup.moveOffScreen(this.dropDown);

		this.inherited(arguments);
	},

	isLoaded: function(){
		// Returns whether or not we are loaded - if our dropdown has an href,
		// then we want to check that.
		var dropDown = this.dropDown;
		return (!dropDown.href || dropDown.isLoaded);
	},

	loadDropDown: function(){
		// Loads our dropdown
		var dropDown = this.dropDown;
		if(!dropDown){ return; }
		if(!this.isLoaded()){
			var handler = dojo.connect(dropDown, "onLoad", this, function(){
				dojo.disconnect(handler);
				this.openDropDown();
			});
			dropDown.refresh();
		}else{
			this.openDropDown();
		}
	},

	isFocusable: function(){
		// Overridden so that focus is handled by the _HasDropDown mixin, not by
		// the _FormWidget mixin.
		return this.inherited(arguments) && !this._mouseDown;
	}
});

dojo.declare("dijit.form.ComboButton", dijit.form.DropDownButton, {
	// summary:
	//		A combination button and drop-down button.
	//		Users can click one side to "press" the button, or click an arrow
	//		icon to display the drop down.
	//
	// example:
	// |	<button dojoType="dijit.form.ComboButton" onClick="...">
	// |		<span>Hello world</span>
	// |		<div dojoType="dijit.Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new dijit.form.ComboButton({label: "hello world", onClick: foo, dropDown: "myMenu"});
	// |	dojo.body().appendChild(button1.domNode);
	//

	templateString: dojo.cache("dijit.form", "templates/ComboButton.html", "<table class=\"dijit dijitReset dijitInline dijitLeft\"\r\n\tcellspacing='0' cellpadding='0' waiRole=\"presentation\"\r\n\t><tbody waiRole=\"presentation\"><tr waiRole=\"presentation\"\r\n\t\t><td class=\"dijitReset dijitStretch dijitButtonNode\" dojoAttachPoint=\"buttonNode\" dojoAttachEvent=\"ondijitclick:_onButtonClick,onkeypress:_onButtonKeyPress\"\r\n\t\t><div id=\"${id}_button\" class=\"dijitReset dijitButtonContents\"\r\n\t\t\tdojoAttachPoint=\"titleNode\"\r\n\t\t\twaiRole=\"button\" waiState=\"labelledby-${id}_label\"\r\n\t\t\t><div class=\"dijitReset dijitInline dijitIcon\" dojoAttachPoint=\"iconNode\" waiRole=\"presentation\"></div\r\n\t\t\t><div class=\"dijitReset dijitInline dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\" waiRole=\"presentation\"></div\r\n\t\t></div\r\n\t\t></td\r\n\t\t><td id=\"${id}_arrow\" class='dijitReset dijitRight dijitButtonNode dijitArrowButton'\r\n\t\t\tdojoAttachPoint=\"_popupStateNode,focusNode,_buttonNode\"\r\n\t\t\tdojoAttachEvent=\"onkeypress:_onArrowKeyPress\"\r\n\t\t\ttitle=\"${optionsTitle}\"\r\n\t\t\twaiRole=\"button\" waiState=\"haspopup-true\"\r\n\t\t\t><div class=\"dijitReset dijitArrowButtonInner\" waiRole=\"presentation\"></div\r\n\t\t\t><div class=\"dijitReset dijitArrowButtonChar\" waiRole=\"presentation\">&#9660;</div\r\n\t\t></td\r\n\t\t><td style=\"display:none !important;\"\r\n\t\t\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" dojoAttachPoint=\"valueNode\"\r\n\t\t/></td></tr></tbody\r\n></table>\r\n"),

	attributeMap: dojo.mixin(dojo.clone(dijit.form.Button.prototype.attributeMap), {
		id: "",
		tabIndex: ["focusNode", "titleNode"],
		title: "titleNode"
	}),

	// optionsTitle: String
	//		Text that describes the options menu (accessibility)
	optionsTitle: "",

	baseClass: "dijitComboButton",

	// Set classes like dijitButtonContentsHover or dijitArrowButtonActive depending on
	// mouse action over specified node
	cssStateNodes: {
		"buttonNode": "dijitButtonNode",
		"titleNode": "dijitButtonContents",
		"_popupStateNode": "dijitDownArrowButton"
	},

	_focusedNode: null,

	_onButtonKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handler for right arrow key when focus is on left part of button
		if(evt.charOrCode == dojo.keys[this.isLeftToRight() ? "RIGHT_ARROW" : "LEFT_ARROW"]){
			dijit.focus(this._popupStateNode);
			dojo.stopEvent(evt);
		}
	},

	_onArrowKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handler for left arrow key when focus is on right part of button
		if(evt.charOrCode == dojo.keys[this.isLeftToRight() ? "LEFT_ARROW" : "RIGHT_ARROW"]){
			dijit.focus(this.titleNode);
			dojo.stopEvent(evt);
		}
	},
	
	focus: function(/*String*/ position){
		// summary:
		//		Focuses this widget to according to position, if specified,
		//		otherwise on arrow node
		// position:
		//		"start" or "end"
		if(!this.disabled){
			dijit.focus(position == "start" ? this.titleNode : this._popupStateNode);
		}
	}
});

dojo.declare("dijit.form.ToggleButton", dijit.form.Button, {
	// summary:
	//		A button that can be in two states (checked or not).
	//		Can be base class for things like tabs or checkbox or radio buttons

	baseClass: "dijitToggleButton",

	// checked: Boolean
	//		Corresponds to the native HTML <input> element's attribute.
	//		In markup, specified as "checked='checked'" or just "checked".
	//		True if the button is depressed, or the checkbox is checked,
	//		or the radio button is selected, etc.
	checked: false,

	attributeMap: dojo.mixin(dojo.clone(dijit.form.Button.prototype.attributeMap), {
		checked:"focusNode"
	}),

	_clicked: function(/*Event*/ evt){
		this.set('checked', !this.checked);
	},

	_setCheckedAttr: function(/*Boolean*/ value, /* Boolean? */ priorityChange){
		this.checked = value;
		dojo.attr(this.focusNode || this.domNode, "checked", value);
		dijit.setWaiState(this.focusNode || this.domNode, "pressed", value);
		this._handleOnChange(value, priorityChange);
	},

	setChecked: function(/*Boolean*/ checked){
		// summary:
		//		Deprecated.   Use set('checked', true/false) instead.
		dojo.deprecated("setChecked("+checked+") is deprecated. Use set('checked',"+checked+") instead.", "", "2.0");
		this.set('checked', checked);
	},

	reset: function(){
		// summary:
		//		Reset the widget's value to what it was at initialization time

		this._hasBeenBlurred = false;

		// set checked state to original setting
		this.set('checked', this.params.checked || false);
	}
});

}

if(!dojo._hasResource["dijit._Contained"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Contained"] = true;
dojo.provide("dijit._Contained");

dojo.declare("dijit._Contained",
		null,
		{
			// summary:
			//		Mixin for widgets that are children of a container widget
			//
			// example:
			// | 	// make a basic custom widget that knows about it's parents
			// |	dojo.declare("my.customClass",[dijit._Widget,dijit._Contained],{});

			getParent: function(){
				// summary:
				//		Returns the parent widget of this widget, assuming the parent
				//		specifies isContainer
				var parent = dijit.getEnclosingWidget(this.domNode.parentNode);
				return parent && parent.isContainer ? parent : null;
			},

			_getSibling: function(/*String*/ which){
				// summary:
				//      Returns next or previous sibling
				// which:
				//      Either "next" or "previous"
				// tags:
				//      private
				var node = this.domNode;
				do{
					node = node[which+"Sibling"];
				}while(node && node.nodeType != 1);
				return node && dijit.byNode(node);	// dijit._Widget
			},

			getPreviousSibling: function(){
				// summary:
				//		Returns null if this is the first child of the parent,
				//		otherwise returns the next element sibling to the "left".

				return this._getSibling("previous"); // dijit._Widget
			},

			getNextSibling: function(){
				// summary:
				//		Returns null if this is the last child of the parent,
				//		otherwise returns the next element sibling to the "right".

				return this._getSibling("next"); // dijit._Widget
			},

			getIndexInParent: function(){
				// summary:
				//		Returns the index of this widget within its container parent.
				//		It returns -1 if the parent does not exist, or if the parent
				//		is not a dijit._Container

				var p = this.getParent();
				if(!p || !p.getIndexOfChild){
					return -1; // int
				}
				return p.getIndexOfChild(this); // int
			}
		}
	);


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.Breadcrumb"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.Breadcrumb"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dijit.Breadcrumb");





 
dojo.declare(
		"ibm.tivoli.tip.dijit.Breadcrumb",
		[ dijit.form.Button, dijit._Contained ],
	{
		baseClass: "dijitBreadcrumb",
		templateString:"<div class=\"dijit dijitLeft dijitInline \"\n\tdojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\n\t><div class='dijitRight'\n\t\t><button class=\"dijitStretch dijitBreadcrumb dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\n\t\t\t><span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\" \n \t\t\t\t><span class=\"dijitToggleButtonIconChar\">&#10003</span \n\t\t\t></span\n\t\t\t><span class=\"dijitBreadcrumb\" id=\"${id}_label\" dojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\" dojoAttachPoint=\"containerNode\">${label}</span\n\t\t></button\n\t></div\n></div>\n",
		
		isCurrent: function() {
			return (this.getNextSibling()==null);
		},
		
		constructor: function () {
		},
		postCreate: function() {
			this.inherited(arguments);
		},
		destroy: function () {
			this.destroyDescendants();
			this.inherited (arguments);
		},
		
		_setStateClass:function(){
			if(this.isCurrent()){
				this.baseClass="dijitBreadcrumbCurrent";	//setting special class for the current breadcrumb
			} else {
				this.baseClass="dijitBreadcrumb";
			}; 
			this.inherited(arguments);
		},
		//Added fix for IBM Dojo 1.5
		_onMouse: function(){
		}
	});


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.BreadcrumbSeparator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.BreadcrumbSeparator"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.dijit.BreadcrumbSeparator");




dojo.declare(
		"ibm.tivoli.tip.dijit.BreadcrumbSeparator",
		[ dijit._Widget, dijit._Templated ],
	{
		// summary
		//	A line between two menu items
		templateString: '<div class="breadcrumbDivider dijitInline">&gt;</div>',
		postCreate: function(){ dojo.setSelectable(this.domNode, false); },
		isFocusable: function(){ return false; }
	});


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.BreadcrumbBar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.BreadcrumbBar"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dijit.BreadcrumbBar");









dojo.declare(
	"ibm.tivoli.tip.dijit.BreadcrumbBar",
	[dijit._Widget, dijit._Templated, 
	dijit._KeyNavContainer],
{
	templateString:
		'<div class="dijit dijitToolbar tipBreadcrumbBar" waiRole="toolbar" tabIndex="${tabIndex}" dojoAttachPoint="containerNode">' +
		'</div>',

	tabIndex: "0",
	breadcrumbWidget: [],
	breadcrumbsHidden: false,

	postCreate: function(){
		this.connectKeyNavHandlers(
			this.isLeftToRight() ? [dojo.keys.LEFT_ARROW] : [dojo.keys.RIGHT_ARROW],
			this.isLeftToRight() ? [dojo.keys.RIGHT_ARROW] : [dojo.keys.LEFT_ARROW]
		);
	},

	startup: function(){
		this.startupKeyNavChildren();
	},
	
	destroy: function () {
		this.destroyDescendants();
		this.inherited (arguments);
	},
	
	addChild: function(widget, insertIndex){
		var children = this.getChildren();
		var childToUpdate = null;
		if(children.length>0 && children[children.length -1]._setStateClass){
			childToUpdate=children[children.length -1]; //child which style class needs to be updated after addition
		}
		this.inherited (arguments);	//adding child
		if(childToUpdate!=null){
			childToUpdate._setStateClass(); //updating style class
		}
		//Breadcrumbs Tivoli Guidelines implementation
		var size = dojo.contentBox(this.domNode);
		this.breadcrumbWidget.push(widget);
		var i = 1;
		var isRtL = false;
		// for RtL languages, we need to manipulate the space left in breadcrumbbar considering that it is filled from right.
		// breadcrumbWidth  and  breadcrumbTrailWidth are calculated differently for R to L languages.  isRtL holds if the locale is Right to Left or otherwise.
		if(dojo.locale=="iw" || dojo.locale=="he"|| dojo.locale.indexOf("ar")>-1)
			isRtL = true;
		var breadcrumbWidth = this.breadcrumbWidget[0].containerNode.offsetLeft;
		if(isRtL)
			breadcrumbWidth = size.w - breadcrumbWidth;
		while(i < this.breadcrumbWidget.length){
			if(this.breadcrumbWidget[i].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator")
				breadcrumbWidth = breadcrumbWidth + this.breadcrumbWidget[i-1].containerNode.offsetWidth + this.breadcrumbWidget[i].domNode.offsetWidth;
			else 
				breadcrumbWidth = breadcrumbWidth + this.breadcrumbWidget[i-1].domNode.offsetWidth + this.breadcrumbWidget[i].containerNode.offsetWidth;
			if(breadcrumbWidth > size.w){
				this.breadcrumbsHidden = true;
				var breadcrumbTrailWidth = breadcrumbWidth;
				//Vyshantha: Mar31,2011: #907: Long breadcrumb trail puts ellipses in bad location
				//for(j = i - 2; j > 0 && breadcrumbTrailWidth > size.w ; j = j - 1){
				for(j = i - 4; j > 0 && breadcrumbTrailWidth > size.w ; j = j - 1){
					breadcrumbTrailWidth = 0;
					if(this.breadcrumbWidget[j].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator"){
						dojo.byId(this.breadcrumbWidget[j].id).style.display = "none";
						breadcrumbTrailWidth = this.breadcrumbWidget[j-1].containerNode.offsetLeft + this.breadcrumbWidget[j-1].containerNode.offsetWidth;
					} else {
						dojo.byId(this.breadcrumbWidget[j].id).style.display = "none";
						dojo.byId(this.breadcrumbWidget[j-1].id).style.display = "none";
						breadcrumbTrailWidth = this.breadcrumbWidget[j-2].domNode.offsetLeft + this.breadcrumbWidget[j-2].domNode.offsetWidth;
					}
					if(isRtL)
						breadcrumbTrailWidth = size.w - breadcrumbTrailWidth;
				}
				//Adding Ellipsis to make long breadcrumb trails guideline compliant
				//Vyshantha: Mar31,2011: #907: Long breadcrumb trail puts ellipses in bad location
				//if(this.breadcrumbWidget[i - 1].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator"){
				if(this.breadcrumbWidget[i - 3] !== undefined && this.breadcrumbWidget[i - 3].declaredClass == "ibm.tivoli.tip.dijit.BreadcrumbSeparator"){ // added check for undefined as it would throw exception in R to L languages.
					//dojo.byId(this.breadcrumbWidget[i - 1].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
					dojo.byId(this.breadcrumbWidget[i - 3].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
				} else {
					//dojo.byId(this.breadcrumbWidget[i].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
					if(this.breadcrumbWidget[i - 2] !== undefined) {
						dojo.byId(this.breadcrumbWidget[i - 2].id).innerHTML = "&gt;&#160;...&#160;&#160;&gt;";
						}
				}
			} else {
				this.breadcrumbsHidden = false;
			}
			i = i + 1;
		}
	},
	
	areBreadcrumbsHidden: function(){
		return this.breadcrumbsHidden;
	}
}
);


}

if(!dojo._hasResource["dojo.regexp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.regexp"] = true;
dojo.provide("dojo.regexp");

/*=====
dojo.regexp = {
	// summary: Regular expressions and Builder resources
};
=====*/

dojo.regexp.escapeString = function(/*String*/str, /*String?*/except){
	//	summary:
	//		Adds escape sequences for special characters in regular expressions
	// except:
	//		a String with special characters to be left unescaped

	return str.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(ch){
		if(except && except.indexOf(ch) != -1){
			return ch;
		}
		return "\\" + ch;
	}); // String
}

dojo.regexp.buildGroupRE = function(/*Object|Array*/arr, /*Function*/re, /*Boolean?*/nonCapture){
	//	summary:
	//		Builds a regular expression that groups subexpressions
	//	description:
	//		A utility function used by some of the RE generators. The
	//		subexpressions are constructed by the function, re, in the second
	//		parameter.  re builds one subexpression for each elem in the array
	//		a, in the first parameter. Returns a string for a regular
	//		expression that groups all the subexpressions.
	// arr:
	//		A single value or an array of values.
	// re:
	//		A function. Takes one parameter and converts it to a regular
	//		expression. 
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. Defaults to false

	// case 1: a is a single value.
	if(!(arr instanceof Array)){
		return re(arr); // String
	}

	// case 2: a is an array
	var b = [];
	for(var i = 0; i < arr.length; i++){
		// convert each elem to a RE
		b.push(re(arr[i]));
	}

	 // join the REs as alternatives in a RE group.
	return dojo.regexp.group(b.join("|"), nonCapture); // String
}

dojo.regexp.group = function(/*String*/expression, /*Boolean?*/nonCapture){
	// summary:
	//		adds group match to expression
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. 
	return "(" + (nonCapture ? "?:":"") + expression + ")"; // String
}

}

if(!dojo._hasResource["dojo.data.util.sorter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.util.sorter"] = true;
dojo.provide("dojo.data.util.sorter");

dojo.data.util.sorter.basicComparator = function(	/*anything*/ a, 
													/*anything*/ b){
	//	summary:  
	//		Basic comparision function that compares if an item is greater or less than another item
	//	description:  
	//		returns 1 if a > b, -1 if a < b, 0 if equal.
	//		'null' values (null, undefined) are treated as larger values so that they're pushed to the end of the list.
	//		And compared to each other, null is equivalent to undefined.
	
	//null is a problematic compare, so if null, we set to undefined.
	//Makes the check logic simple, compact, and consistent
	//And (null == undefined) === true, so the check later against null
	//works for undefined and is less bytes.
	var r = -1;
	if(a === null){
		a = undefined;
	}
	if(b === null){
		b = undefined;
	}
	if(a == b){
		r = 0; 
	}else if(a > b || a == null){
		r = 1; 
	}
	return r; //int {-1,0,1}
};

dojo.data.util.sorter.createSortFunction = function(	/* attributes array */sortSpec,
														/*dojo.data.core.Read*/ store){
	//	summary:  
	//		Helper function to generate the sorting function based off the list of sort attributes.
	//	description:  
	//		The sort function creation will look for a property on the store called 'comparatorMap'.  If it exists
	//		it will look in the mapping for comparisons function for the attributes.  If one is found, it will
	//		use it instead of the basic comparator, which is typically used for strings, ints, booleans, and dates.
	//		Returns the sorting function for this particular list of attributes and sorting directions.
	//
	//	sortSpec: array
	//		A JS object that array that defines out what attribute names to sort on and whether it should be descenting or asending.
	//		The objects should be formatted as follows:
	//		{
	//			attribute: "attributeName-string" || attribute,
	//			descending: true|false;   // Default is false.
	//		}
	//	store: object
	//		The datastore object to look up item values from.
	//
	var sortFunctions=[];

	function createSortFunction(attr, dir, comp, s){
		//Passing in comp and s (comparator and store), makes this
		//function much faster.
		return function(itemA, itemB){
			var a = s.getValue(itemA, attr);
			var b = s.getValue(itemB, attr);
			return dir * comp(a,b); //int
		};
	}
	var sortAttribute;
	var map = store.comparatorMap;
	var bc = dojo.data.util.sorter.basicComparator;
	for(var i = 0; i < sortSpec.length; i++){
		sortAttribute = sortSpec[i];
		var attr = sortAttribute.attribute;
		if(attr){
			var dir = (sortAttribute.descending) ? -1 : 1;
			var comp = bc;
			if(map){
				if(typeof attr !== "string" && ("toString" in attr)){
					 attr = attr.toString();
				}
				comp = map[attr] || bc;
			}
			sortFunctions.push(createSortFunction(attr, 
				dir, comp, store));
		}
	}
	return function(rowA, rowB){
		var i=0;
		while(i < sortFunctions.length){
			var ret = sortFunctions[i++](rowA, rowB);
			if(ret !== 0){
				return ret;//int
			}
		}
		return 0; //int  
	}; // Function
};

}

if(!dojo._hasResource["dojo.data.util.simpleFetch"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.util.simpleFetch"] = true;
dojo.provide("dojo.data.util.simpleFetch");


dojo.data.util.simpleFetch.fetch = function(/* Object? */ request){
	//	summary:
	//		The simpleFetch mixin is designed to serve as a set of function(s) that can
	//		be mixed into other datastore implementations to accelerate their development.  
	//		The simpleFetch mixin should work well for any datastore that can respond to a _fetchItems() 
	//		call by returning an array of all the found items that matched the query.  The simpleFetch mixin
	//		is not designed to work for datastores that respond to a fetch() call by incrementally
	//		loading items, or sequentially loading partial batches of the result
	//		set.  For datastores that mixin simpleFetch, simpleFetch 
	//		implements a fetch method that automatically handles eight of the fetch()
	//		arguments -- onBegin, onItem, onComplete, onError, start, count, sort and scope
	//		The class mixing in simpleFetch should not implement fetch(),
	//		but should instead implement a _fetchItems() method.  The _fetchItems() 
	//		method takes three arguments, the keywordArgs object that was passed 
	//		to fetch(), a callback function to be called when the result array is
	//		available, and an error callback to be called if something goes wrong.
	//		The _fetchItems() method should ignore any keywordArgs parameters for
	//		start, count, onBegin, onItem, onComplete, onError, sort, and scope.  
	//		The _fetchItems() method needs to correctly handle any other keywordArgs
	//		parameters, including the query parameter and any optional parameters 
	//		(such as includeChildren).  The _fetchItems() method should create an array of 
	//		result items and pass it to the fetchHandler along with the original request object 
	//		-- or, the _fetchItems() method may, if it wants to, create an new request object 
	//		with other specifics about the request that are specific to the datastore and pass 
	//		that as the request object to the handler.
	//
	//		For more information on this specific function, see dojo.data.api.Read.fetch()
	request = request || {};
	if(!request.store){
		request.store = this;
	}
	var self = this;

	var _errorHandler = function(errorData, requestObject){
		if(requestObject.onError){
			var scope = requestObject.scope || dojo.global;
			requestObject.onError.call(scope, errorData, requestObject);
		}
	};

	var _fetchHandler = function(items, requestObject){
		var oldAbortFunction = requestObject.abort || null;
		var aborted = false;

		var startIndex = requestObject.start?requestObject.start:0;
		var endIndex = (requestObject.count && (requestObject.count !== Infinity))?(startIndex + requestObject.count):items.length;

		requestObject.abort = function(){
			aborted = true;
			if(oldAbortFunction){
				oldAbortFunction.call(requestObject);
			}
		};

		var scope = requestObject.scope || dojo.global;
		if(!requestObject.store){
			requestObject.store = self;
		}
		if(requestObject.onBegin){
			requestObject.onBegin.call(scope, items.length, requestObject);
		}
		if(requestObject.sort){
			items.sort(dojo.data.util.sorter.createSortFunction(requestObject.sort, self));
		}
		if(requestObject.onItem){
			for(var i = startIndex; (i < items.length) && (i < endIndex); ++i){
				var item = items[i];
				if(!aborted){
					requestObject.onItem.call(scope, item, requestObject);
				}
			}
		}
		if(requestObject.onComplete && !aborted){
			var subset = null;
			if(!requestObject.onItem){
				subset = items.slice(startIndex, endIndex);
			}
			requestObject.onComplete.call(scope, subset, requestObject);
		}
	};
	this._fetchItems(request, _fetchHandler, _errorHandler);
	return request;	// Object
};

}

if(!dojo._hasResource["dojo.data.util.filter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.util.filter"] = true;
dojo.provide("dojo.data.util.filter");

dojo.data.util.filter.patternToRegExp = function(/*String*/pattern, /*boolean?*/ ignoreCase){
	//	summary:  
	//		Helper function to convert a simple pattern to a regular expression for matching.
	//	description:
	//		Returns a regular expression object that conforms to the defined conversion rules.
	//		For example:  
	//			ca*   -> /^ca.*$/
	//			*ca*  -> /^.*ca.*$/
	//			*c\*a*  -> /^.*c\*a.*$/
	//			*c\*a?*  -> /^.*c\*a..*$/
	//			and so on.
	//
	//	pattern: string
	//		A simple matching pattern to convert that follows basic rules:
	//			* Means match anything, so ca* means match anything starting with ca
	//			? Means match single character.  So, b?b will match to bob and bab, and so on.
	//      	\ is an escape character.  So for example, \* means do not treat * as a match, but literal character *.
	//				To use a \ as a character in the string, it must be escaped.  So in the pattern it should be 
	//				represented by \\ to be treated as an ordinary \ character instead of an escape.
	//
	//	ignoreCase:
	//		An optional flag to indicate if the pattern matching should be treated as case-sensitive or not when comparing
	//		By default, it is assumed case sensitive.

	var rxp = "^";
	var c = null;
	for(var i = 0; i < pattern.length; i++){
		c = pattern.charAt(i);
		switch(c){
			case '\\':
				rxp += c;
				i++;
				rxp += pattern.charAt(i);
				break;
			case '*':
				rxp += ".*"; break;
			case '?':
				rxp += "."; break;
			case '$':
			case '^':
			case '/':
			case '+':
			case '.':
			case '|':
			case '(':
			case ')':
			case '{':
			case '}':
			case '[':
			case ']':
				rxp += "\\"; //fallthrough
			default:
				rxp += c;
		}
	}
	rxp += "$";
	if(ignoreCase){
		return new RegExp(rxp,"mi"); //RegExp
	}else{
		return new RegExp(rxp,"m"); //RegExp
	}
	
};

}

if(!dojo._hasResource["dojo.i18n"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.i18n"] = true;
dojo.provide("dojo.i18n");

/*=====
dojo.i18n = {
	// summary: Utility classes to enable loading of resources for internationalization (i18n)
};
=====*/

dojo.i18n.getLocalization = function(/*String*/packageName, /*String*/bundleName, /*String?*/locale){
	//	summary:
	//		Returns an Object containing the localization for a given resource
	//		bundle in a package, matching the specified locale.
	//	description:
	//		Returns a hash containing name/value pairs in its prototypesuch
	//		that values can be easily overridden.  Throws an exception if the
	//		bundle is not found.  Bundle must have already been loaded by
	//		`dojo.requireLocalization()` or by a build optimization step.  NOTE:
	//		try not to call this method as part of an object property
	//		definition (`var foo = { bar: dojo.i18n.getLocalization() }`).  In
	//		some loading situations, the bundle may not be available in time
	//		for the object definition.  Instead, call this method inside a
	//		function that is run after all modules load or the page loads (like
	//		in `dojo.addOnLoad()`), or in a widget lifecycle method.
	//	packageName:
	//		package which is associated with this resource
	//	bundleName:
	//		the base filename of the resource bundle (without the ".js" suffix)
	//	locale:
	//		the variant to load (optional).  By default, the locale defined by
	//		the host environment: dojo.locale

	locale = dojo.i18n.normalizeLocale(locale);

	// look for nearest locale match
	var elements = locale.split('-');
	var module = [packageName,"nls",bundleName].join('.');
	var bundle = dojo._loadedModules[module];
	if(bundle){
		var localization;
		for(var i = elements.length; i > 0; i--){
			var loc = elements.slice(0, i).join('_');
			if(bundle[loc]){
				localization = bundle[loc];
				break;
			}
		}
		if(!localization){
			localization = bundle.ROOT;
		}

		// make a singleton prototype so that the caller won't accidentally change the values globally
		if(localization){
			var clazz = function(){};
			clazz.prototype = localization;
			return new clazz(); // Object
		}
	}

	throw new Error("Bundle not found: " + bundleName + " in " + packageName+" , locale=" + locale);
};

dojo.i18n.normalizeLocale = function(/*String?*/locale){
	//	summary:
	//		Returns canonical form of locale, as used by Dojo.
	//
	//  description:
	//		All variants are case-insensitive and are separated by '-' as specified in [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt).
	//		If no locale is specified, the dojo.locale is returned.  dojo.locale is defined by
	//		the user agent's locale unless overridden by djConfig.

	var result = locale ? locale.toLowerCase() : dojo.locale;
	if(result == "root"){
		result = "ROOT";
	}
	return result; // String
};

dojo.i18n._requireLocalization = function(/*String*/moduleName, /*String*/bundleName, /*String?*/locale, /*String?*/availableFlatLocales){
	//	summary:
	//		See dojo.requireLocalization()
	//	description:
	// 		Called by the bootstrap, but factored out so that it is only
	// 		included in the build when needed.

	var targetLocale = dojo.i18n.normalizeLocale(locale);
 	var bundlePackage = [moduleName, "nls", bundleName].join(".");
	// NOTE: 
	//		When loading these resources, the packaging does not match what is
	//		on disk.  This is an implementation detail, as this is just a
	//		private data structure to hold the loaded resources.  e.g.
	//		`tests/hello/nls/en-us/salutations.js` is loaded as the object
	//		`tests.hello.nls.salutations.en_us={...}` The structure on disk is
	//		intended to be most convenient for developers and translators, but
	//		in memory it is more logical and efficient to store in a different
	//		order.  Locales cannot use dashes, since the resulting path will
	//		not evaluate as valid JS, so we translate them to underscores.
	
	//Find the best-match locale to load if we have available flat locales.
	var bestLocale = "";
	if(availableFlatLocales){
		var flatLocales = availableFlatLocales.split(",");
		for(var i = 0; i < flatLocales.length; i++){
			//Locale must match from start of string.
			//Using ["indexOf"] so customBase builds do not see
			//this as a dojo._base.array dependency.
			if(targetLocale["indexOf"](flatLocales[i]) == 0){
				if(flatLocales[i].length > bestLocale.length){
					bestLocale = flatLocales[i];
				}
			}
		}
		if(!bestLocale){
			bestLocale = "ROOT";
		}		
	}

	//See if the desired locale is already loaded.
	var tempLocale = availableFlatLocales ? bestLocale : targetLocale;
	var bundle = dojo._loadedModules[bundlePackage];
	var localizedBundle = null;
	if(bundle){
		if(dojo.config.localizationComplete && bundle._built){return;}
		var jsLoc = tempLocale.replace(/-/g, '_');
		var translationPackage = bundlePackage+"."+jsLoc;
		localizedBundle = dojo._loadedModules[translationPackage];
	}

	if(!localizedBundle){
		bundle = dojo["provide"](bundlePackage);
		var syms = dojo._getModuleSymbols(moduleName);
		var modpath = syms.concat("nls").join("/");
		var parent;

		dojo.i18n._searchLocalePath(tempLocale, availableFlatLocales, function(loc){
			var jsLoc = loc.replace(/-/g, '_');
			var translationPackage = bundlePackage + "." + jsLoc;
			var loaded = false;
			if(!dojo._loadedModules[translationPackage]){
				// Mark loaded whether it's found or not, so that further load attempts will not be made
				dojo["provide"](translationPackage);
				var module = [modpath];
				if(loc != "ROOT"){module.push(loc);}
				module.push(bundleName);
				var filespec = module.join("/") + '.js';
				loaded = dojo._loadPath(filespec, null, function(hash){
					// Use singleton with prototype to point to parent bundle, then mix-in result from loadPath
					var clazz = function(){};
					clazz.prototype = parent;
					bundle[jsLoc] = new clazz();
					for(var j in hash){ bundle[jsLoc][j] = hash[j]; }
				});
			}else{
				loaded = true;
			}
			if(loaded && bundle[jsLoc]){
				parent = bundle[jsLoc];
			}else{
				bundle[jsLoc] = parent;
			}
			
			if(availableFlatLocales){
				//Stop the locale path searching if we know the availableFlatLocales, since
				//the first call to this function will load the only bundle that is needed.
				return true;
			}
		});
	}

	//Save the best locale bundle as the target locale bundle when we know the
	//the available bundles.
	if(availableFlatLocales && targetLocale != bestLocale){
		bundle[targetLocale.replace(/-/g, '_')] = bundle[bestLocale.replace(/-/g, '_')];
	}
};

(function(){
	// If other locales are used, dojo.requireLocalization should load them as
	// well, by default. 
	// 
	// Override dojo.requireLocalization to do load the default bundle, then
	// iterate through the extraLocale list and load those translations as
	// well, unless a particular locale was requested.

	var extra = dojo.config.extraLocale;
	if(extra){
		if(!extra instanceof Array){
			extra = [extra];
		}

		var req = dojo.i18n._requireLocalization;
		dojo.i18n._requireLocalization = function(m, b, locale, availableFlatLocales){
			req(m,b,locale, availableFlatLocales);
			if(locale){return;}
			for(var i=0; i<extra.length; i++){
				req(m,b,extra[i], availableFlatLocales);
			}
		};
	}
})();

dojo.i18n._searchLocalePath = function(/*String*/locale, /*Boolean*/down, /*Function*/searchFunc){
	//	summary:
	//		A helper method to assist in searching for locale-based resources.
	//		Will iterate through the variants of a particular locale, either up
	//		or down, executing a callback function.  For example, "en-us" and
	//		true will try "en-us" followed by "en" and finally "ROOT".

	locale = dojo.i18n.normalizeLocale(locale);

	var elements = locale.split('-');
	var searchlist = [];
	for(var i = elements.length; i > 0; i--){
		searchlist.push(elements.slice(0, i).join('-'));
	}
	searchlist.push(false);
	if(down){searchlist.reverse();}

	for(var j = searchlist.length - 1; j >= 0; j--){
		var loc = searchlist[j] || "ROOT";
		var stop = searchFunc(loc);
		if(stop){ break; }
	}
};

dojo.i18n._preloadLocalizations = function(/*String*/bundlePrefix, /*Array*/localesGenerated){
	//	summary:
	//		Load built, flattened resource bundles, if available for all
	//		locales used in the page. Only called by built layer files.

	function preload(locale){
		locale = dojo.i18n.normalizeLocale(locale);
		dojo.i18n._searchLocalePath(locale, true, function(loc){
			for(var i=0; i<localesGenerated.length;i++){
				if(localesGenerated[i] == loc){
					dojo["require"](bundlePrefix+"_"+loc);
					return true; // Boolean
				}
			}
			return false; // Boolean
		});
	}
	preload();
	var extra = dojo.config.extraLocale||[];
	for(var i=0; i<extra.length; i++){
		preload(extra[i]);
	}
};

}

if(!dojo._hasResource["dijit.form.TextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.TextBox"] = true;
dojo.provide("dijit.form.TextBox");



dojo.declare(
	"dijit.form.TextBox",
	dijit.form._FormValueWidget,
	{
		// summary:
		//		A base class for textbox form inputs

		// trim: Boolean
		//		Removes leading and trailing whitespace if true.  Default is false.
		trim: false,

		// uppercase: Boolean
		//		Converts all characters to uppercase if true.  Default is false.
		uppercase: false,

		// lowercase: Boolean
		//		Converts all characters to lowercase if true.  Default is false.
		lowercase: false,

		// propercase: Boolean
		//		Converts the first character of each word to uppercase if true.
		propercase: false,

		//	maxLength: String
		//		HTML INPUT tag maxLength declaration.
		maxLength: "",

		//	selectOnClick: [const] Boolean
		//		If true, all text will be selected when focused with mouse
		selectOnClick: false,

		//	placeHolder: String
		//		Defines a hint to help users fill out the input field (as defined in HTML 5).
		//		This should only contain plain text (no html markup).
		placeHolder: "",
		
		templateString: dojo.cache("dijit.form", "templates/TextBox.html", "<div class=\"dijit dijitReset dijitInline dijitLeft\" id=\"widget_${id}\" waiRole=\"presentation\"\r\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\r\n\t\t><input class=\"dijitReset dijitInputInner\" dojoAttachPoint='textbox,focusNode' autocomplete=\"off\"\r\n\t\t\t${!nameAttrSetting} type='${type}'\r\n\t/></div\r\n></div>\r\n"),
		_singleNodeTemplate: '<input class="dijit dijitReset dijitLeft dijitInputField" dojoAttachPoint="textbox,focusNode" autocomplete="off" type="${type}" ${!nameAttrSetting} />',

		_buttonInputDisabled: dojo.isIE ? "disabled" : "", // allows IE to disallow focus, but Firefox cannot be disabled for mousedown events

		baseClass: "dijitTextBox",

		attributeMap: dojo.delegate(dijit.form._FormValueWidget.prototype.attributeMap, {
			maxLength: "focusNode"
		}),
		
		postMixInProperties: function(){
			var type = this.type.toLowerCase();
			if(this.templateString && this.templateString.toLowerCase() == "input" || ((type == "hidden" || type == "file") && this.templateString == dijit.form.TextBox.prototype.templateString)){
				this.templateString = this._singleNodeTemplate;
			}
			this.inherited(arguments);
		},

		_setPlaceHolderAttr: function(v){
			this.placeHolder = v;
			if(!this._phspan){
				this._attachPoints.push('_phspan');
				/* dijitInputField class gives placeHolder same padding as the input field
				 * parent node already has dijitInputField class but it doesn't affect this <span>
				 * since it's position: absolute.
				 */
				this._phspan = dojo.create('span',{className:'dijitPlaceHolder dijitInputField'},this.textbox,'after');
			}
			this._phspan.innerHTML="";
			this._phspan.appendChild(document.createTextNode(v));
			
			this._updatePlaceHolder();
		},
		
		_updatePlaceHolder: function(){
			if(this._phspan){
				this._phspan.style.display=(this.placeHolder&&!this._focused&&!this.textbox.value)?"":"none";
			}
		},

		_getValueAttr: function(){
			// summary:
			//		Hook so attr('value') works as we like.
			// description:
			//		For `dijit.form.TextBox` this basically returns the value of the <input>.
			//
			//		For `dijit.form.MappedTextBox` subclasses, which have both
			//		a "displayed value" and a separate "submit value",
			//		This treats the "displayed value" as the master value, computing the
			//		submit value from it via this.parse().
			return this.parse(this.get('displayedValue'), this.constraints);
		},

		_setValueAttr: function(value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Hook so attr('value', ...) works.
			//
			// description:
			//		Sets the value of the widget to "value" which can be of
			//		any type as determined by the widget.
			//
			// value:
			//		The visual element value is also set to a corresponding,
			//		but not necessarily the same, value.
			//
			// formattedValue:
			//		If specified, used to set the visual element value,
			//		otherwise a computed visual value is used.
			//
			// priorityChange:
			//		If true, an onChange event is fired immediately instead of
			//		waiting for the next blur event.

			var filteredValue;
			if(value !== undefined){
				// TODO: this is calling filter() on both the display value and the actual value.
				// I added a comment to the filter() definition about this, but it should be changed.
				filteredValue = this.filter(value);
				if(typeof formattedValue != "string"){
					if(filteredValue !== null && ((typeof filteredValue != "number") || !isNaN(filteredValue))){
						formattedValue = this.filter(this.format(filteredValue, this.constraints));
					}else{ formattedValue = ''; }
				}
			}
			if(formattedValue != null && formattedValue != undefined && ((typeof formattedValue) != "number" || !isNaN(formattedValue)) && this.textbox.value != formattedValue){
				this.textbox.value = formattedValue;
			}

			this._updatePlaceHolder();

			this.inherited(arguments, [filteredValue, priorityChange]);
		},

		// displayedValue: String
		//		For subclasses like ComboBox where the displayed value
		//		(ex: Kentucky) and the serialized value (ex: KY) are different,
		//		this represents the displayed value.
		//
		//		Setting 'displayedValue' through attr('displayedValue', ...)
		//		updates 'value', and vice-versa.  Otherwise 'value' is updated
		//		from 'displayedValue' periodically, like onBlur etc.
		//
		//		TODO: move declaration to MappedTextBox?
		//		Problem is that ComboBox references displayedValue,
		//		for benefit of FilteringSelect.
		displayedValue: "",

		getDisplayedValue: function(){
			// summary:
			//		Deprecated.   Use set('displayedValue') instead.
			// tags:
			//		deprecated
			dojo.deprecated(this.declaredClass+"::getDisplayedValue() is deprecated. Use set('displayedValue') instead.", "", "2.0");
			return this.get('displayedValue');
		},

		_getDisplayedValueAttr: function(){
			// summary:
			//		Hook so attr('displayedValue') works.
			// description:
			//		Returns the displayed value (what the user sees on the screen),
			// 		after filtering (ie, trimming spaces etc.).
			//
			//		For some subclasses of TextBox (like ComboBox), the displayed value
			//		is different from the serialized value that's actually
			//		sent to the server (see dijit.form.ValidationTextBox.serialize)

			return this.filter(this.textbox.value);
		},

		setDisplayedValue: function(/*String*/value){
			// summary:
			//		Deprecated.   Use set('displayedValue', ...) instead.
			// tags:
			//		deprecated
			dojo.deprecated(this.declaredClass+"::setDisplayedValue() is deprecated. Use set('displayedValue', ...) instead.", "", "2.0");
			this.set('displayedValue', value);
		},

		_setDisplayedValueAttr: function(/*String*/value){
			// summary:
			//		Hook so attr('displayedValue', ...) works.
			// description:
			//		Sets the value of the visual element to the string "value".
			//		The widget value is also set to a corresponding,
			//		but not necessarily the same, value.

			if(value === null || value === undefined){ value = '' }
			else if(typeof value != "string"){ value = String(value) }
			this.textbox.value = value;
			this._setValueAttr(this.get('value'), undefined, value);
		},

		format: function(/* String */ value, /* Object */ constraints){
			// summary:
			//		Replacable function to convert a value to a properly formatted string.
			// tags:
			//		protected extension
			return ((value == null || value == undefined) ? "" : (value.toString ? value.toString() : value));
		},

		parse: function(/* String */ value, /* Object */ constraints){
			// summary:
			//		Replacable function to convert a formatted string to a value
			// tags:
			//		protected extension

			return value;	// String
		},

		_refreshState: function(){
			// summary:
			//		After the user types some characters, etc., this method is
			//		called to check the field for validity etc.  The base method
			//		in `dijit.form.TextBox` does nothing, but subclasses override.
			// tags:
			//		protected
		},

		_onInput: function(e){
			if(e && e.type && /key/i.test(e.type) && e.keyCode){
				switch(e.keyCode){
					case dojo.keys.SHIFT:
					case dojo.keys.ALT:
					case dojo.keys.CTRL:
					case dojo.keys.TAB:
						return;
				}
			}
			if(this.intermediateChanges){
				var _this = this;
				// the setTimeout allows the key to post to the widget input box
				setTimeout(function(){ _this._handleOnChange(_this.get('value'), false); }, 0);
			}
			this._refreshState();
		},

		postCreate: function(){
			// setting the value here is needed since value="" in the template causes "undefined"
			// and setting in the DOM (instead of the JS object) helps with form reset actions
			if(dojo.isIE){ // IE INPUT tag fontFamily has to be set directly using STYLE
				// the setTimeout gives IE a chance to render the TextBox and to deal with font inheritance
				setTimeout(dojo.hitch(this, function(){
					var s = dojo.getComputedStyle(this.domNode);
					if(s){
						var ff = s.fontFamily;
						if(ff){
							var inputs = this.domNode.getElementsByTagName("INPUT");
							if(inputs){
								for(var i=0; i < inputs.length; i++){
									inputs[i].style.fontFamily = ff;
								}
							}
						}
					}
				}), 0);
			}
			this.textbox.setAttribute("value", this.textbox.value); // DOM and JS values shuld be the same
			this.inherited(arguments);
			if(dojo.isMoz || dojo.isOpera){
				this.connect(this.textbox, "oninput", this._onInput);
			}else{
				this.connect(this.textbox, "onkeydown", this._onInput);
				this.connect(this.textbox, "onkeyup", this._onInput);
				this.connect(this.textbox, "onpaste", this._onInput);
				this.connect(this.textbox, "oncut", this._onInput);
			}
		},

		_blankValue: '', // if the textbox is blank, what value should be reported
		filter: function(val){
			// summary:
			//		Auto-corrections (such as trimming) that are applied to textbox
			//		value on blur or form submit.
			// description:
			//		For MappedTextBox subclasses, this is called twice
			// 			- once with the display value
			//			- once the value as set/returned by attr('value', ...)
			//		and attr('value'), ex: a Number for NumberTextBox.
			//
			//		In the latter case it does corrections like converting null to NaN.  In
			//		the former case the NumberTextBox.filter() method calls this.inherited()
			//		to execute standard trimming code in TextBox.filter().
			//
			//		TODO: break this into two methods in 2.0
			//
			// tags:
			//		protected extension
			if(val === null){ return this._blankValue; }
			if(typeof val != "string"){ return val; }
			if(this.trim){
				val = dojo.trim(val);
			}
			if(this.uppercase){
				val = val.toUpperCase();
			}
			if(this.lowercase){
				val = val.toLowerCase();
			}
			if(this.propercase){
				val = val.replace(/[^\s]+/g, function(word){
					return word.substring(0,1).toUpperCase() + word.substring(1);
				});
			}
			return val;
		},

		_setBlurValue: function(){
			this._setValueAttr(this.get('value'), true);
		},

		_onBlur: function(e){
			if(this.disabled){ return; }
			this._setBlurValue();
			this.inherited(arguments);

			if(this._selectOnClickHandle){
				this.disconnect(this._selectOnClickHandle);
			}
			if(this.selectOnClick && dojo.isMoz){
				this.textbox.selectionStart = this.textbox.selectionEnd = undefined; // clear selection so that the next mouse click doesn't reselect
			}
			
			this._updatePlaceHolder();
		},

		_onFocus: function(/*String*/ by){
			if(this.disabled || this.readOnly){ return; }

			// Select all text on focus via click if nothing already selected.
			// Since mouse-up will clear the selection need to defer selection until after mouse-up.
			// Don't do anything on focus by tabbing into the widget since there's no associated mouse-up event.
			if(this.selectOnClick && by == "mouse"){
				this._selectOnClickHandle = this.connect(this.domNode, "onmouseup", function(){
					// Only select all text on first click; otherwise users would have no way to clear
					// the selection.
					this.disconnect(this._selectOnClickHandle);

					// Check if the user selected some text manually (mouse-down, mouse-move, mouse-up)
					// and if not, then select all the text
					var textIsNotSelected;
					if(dojo.isIE){
						var range = dojo.doc.selection.createRange();
						var parent = range.parentElement();
						textIsNotSelected = parent == this.textbox && range.text.length == 0;
					}else{
						textIsNotSelected = this.textbox.selectionStart == this.textbox.selectionEnd;
					}
					if(textIsNotSelected){
						dijit.selectInputText(this.textbox);
					}
				});
			}

			this._updatePlaceHolder();
			
			this._refreshState();
			this.inherited(arguments);
		},

		reset: function(){
			// Overrides dijit._FormWidget.reset().
			// Additionally resets the displayed textbox value to ''
			this.textbox.value = '';
			this.inherited(arguments);
		}
	}
);

dijit.selectInputText = function(/*DomNode*/element, /*Number?*/ start, /*Number?*/ stop){
	// summary:
	//		Select text in the input element argument, from start (default 0), to stop (default end).

	// TODO: use functions in _editor/selection.js?
	var _window = dojo.global;
	var _document = dojo.doc;
	element = dojo.byId(element);
	if(isNaN(start)){ start = 0; }
	if(isNaN(stop)){ stop = element.value ? element.value.length : 0; }
	dijit.focus(element);
	if(_document["selection"] && dojo.body()["createTextRange"]){ // IE
		if(element.createTextRange){
			var range = element.createTextRange();
			with(range){
				collapse(true);
				moveStart("character", -99999); // move to 0
				moveStart("character", start); // delta from 0 is the correct position
				moveEnd("character", stop-start);
				select();
			}
		}
	}else if(_window["getSelection"]){
		if(element.setSelectionRange){
			element.setSelectionRange(start, stop);
		}
	}
};

}

if(!dojo._hasResource["dijit.Tooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Tooltip"] = true;
dojo.provide("dijit.Tooltip");




dojo.declare(
	"dijit._MasterTooltip",
	[dijit._Widget, dijit._Templated],
	{
		// summary:
		//		Internal widget that holds the actual tooltip markup,
		//		which occurs once per page.
		//		Called by Tooltip widgets which are just containers to hold
		//		the markup
		// tags:
		//		protected

		// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: dijit.defaultDuration,

		templateString: dojo.cache("dijit", "templates/Tooltip.html", "<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\">\r\n\t<div class=\"dijitTooltipContainer dijitTooltipContents\" dojoAttachPoint=\"containerNode\" waiRole='alert'></div>\r\n\t<div class=\"dijitTooltipConnector\"></div>\r\n</div>\r\n"),

		postCreate: function(){
			dojo.body().appendChild(this.domNode);

			this.bgIframe = new dijit.BackgroundIframe(this.domNode);

			// Setup fade-in and fade-out functions.
			this.fadeIn = dojo.fadeIn({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onShow") });
			this.fadeOut = dojo.fadeOut({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onHide") });

		},

		show: function(/*String*/ innerHTML, /*DomNode*/ aroundNode, /*String[]?*/ position, /*Boolean*/ rtl){
			// summary:
			//		Display tooltip w/specified contents to right of specified node
			//		(To left if there's no space on the right, or if rtl == true)

			if(this.aroundNode && this.aroundNode === aroundNode){
				return;
			}

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;

			var pos = dijit.placeOnScreenAroundElement(this.domNode, aroundNode, dijit.getPopupAroundAlignment((position && position.length) ? position : dijit.Tooltip.defaultPosition, !rtl), dojo.hitch(this, "orient"));

			// show it
			dojo.style(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},

		orient: function(/* DomNode */ node, /* String */ aroundCorner, /* String */ tooltipCorner){
			// summary:
			//		Private function to set CSS for tooltip node based on which position it's in.
			//		This is called by the dijit popup code.
			// tags:
			//		protected

			node.className = "dijitTooltip " +
				{
					"BL-TL": "dijitTooltipBelow dijitTooltipABLeft",
					"TL-BL": "dijitTooltipAbove dijitTooltipABLeft",
					"BR-TR": "dijitTooltipBelow dijitTooltipABRight",
					"TR-BR": "dijitTooltipAbove dijitTooltipABRight",
					"BR-BL": "dijitTooltipRight",
					"BL-BR": "dijitTooltipLeft"
				}[aroundCorner + "-" + tooltipCorner];
		},

		_onShow: function(){
			// summary:
			//		Called at end of fade-in operation
			// tags:
			//		protected
			if(dojo.isIE){
				// the arrow won't show up on a node w/an opacity filter
				this.domNode.style.filter="";
			}
		},

		hide: function(aroundNode){
			// summary:
			//		Hide the tooltip
			if(this._onDeck && this._onDeck[1] == aroundNode){
				// this hide request is for a show() that hasn't even started yet;
				// just cancel the pending show()
				this._onDeck=null;
			}else if(this.aroundNode === aroundNode){
				// this hide request is for the currently displayed tooltip
				this.fadeIn.stop();
				this.isShowingNow = false;
				this.aroundNode = null;
				this.fadeOut.play();
			}else{
				// just ignore the call, it's for a tooltip that has already been erased
			}
		},

		_onHide: function(){
			// summary:
			//		Called at end of fade-out operation
			// tags:
			//		protected

			this.domNode.style.cssText="";	// to position offscreen again
			this.containerNode.innerHTML="";
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		}

	}
);

dijit.showTooltip = function(/*String*/ innerHTML, /*DomNode*/ aroundNode, /*String[]?*/ position, /*Boolean*/ rtl){
	// summary:
	//		Display tooltip w/specified contents in specified position.
	//		See description of dijit.Tooltip.defaultPosition for details on position parameter.
	//		If position is not specified then dijit.Tooltip.defaultPosition is used.
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.show(innerHTML, aroundNode, position, rtl);
};

dijit.hideTooltip = function(aroundNode){
	// summary:
	//		Hide the tooltip
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.hide(aroundNode);
};

dojo.declare(
	"dijit.Tooltip",
	dijit._Widget,
	{
		// summary:
		//		Pops up a tooltip (a help message) when you hover over a node.

		// label: String
		//		Text to display in the tooltip.
		//		Specified as innerHTML when creating the widget from markup.
		label: "",

		// showDelay: Integer
		//		Number of milliseconds to wait after hovering over/focusing on the object, before
		//		the tooltip is displayed.
		showDelay: 400,

		// connectId: [const] String[]
		//		Id's of domNodes to attach the tooltip to.
		//		When user hovers over any of the specified dom nodes, the tooltip will appear.
		//
		//		Note: Currently connectId can only be specified on initialization, it cannot
		//		be changed via attr('connectId', ...)
		//
		//		Note: in 2.0 this will be renamed to connectIds for less confusion.
		connectId: [],

		// position: String[]
		//		See description of `dijit.Tooltip.defaultPosition` for details on position parameter.
		position: [],

		constructor: function(){
			// Map id's of nodes I'm connected to to a list of the this.connect() handles
			this._nodeConnectionsById = {};
		},

		_setConnectIdAttr: function(newIds){
			for(var oldId in this._nodeConnectionsById){
				this.removeTarget(oldId);
			}
			dojo.forEach(dojo.isArrayLike(newIds) ? newIds : [newIds], this.addTarget, this);
		},

		_getConnectIdAttr: function(){
			var ary = [];
			for(var id in this._nodeConnectionsById){
				ary.push(id);
			}
			return ary;
		},

		addTarget: function(/*DOMNODE || String*/ id){
			// summary:
			//		Attach tooltip to specified node, if it's not already connected
			var node = dojo.byId(id);
			if(!node){ return; }
			if(node.id in this._nodeConnectionsById){ return; }//Already connected

			this._nodeConnectionsById[node.id] = [
				this.connect(node, "onmouseenter", "_onTargetMouseEnter"),
				this.connect(node, "onmouseleave", "_onTargetMouseLeave"),
				this.connect(node, "onfocus", "_onTargetFocus"),
				this.connect(node, "onblur", "_onTargetBlur")
			];
		},

		removeTarget: function(/*DOMNODE || String*/ node){
			// summary:
			//		Detach tooltip from specified node

			// map from DOMNode back to plain id string
			var id = node.id || node;

			if(id in this._nodeConnectionsById){
				dojo.forEach(this._nodeConnectionsById[id], this.disconnect, this);
				delete this._nodeConnectionsById[id];
			}
		},

		postCreate: function(){
			dojo.addClass(this.domNode,"dijitTooltipData");
		},

		startup: function(){
			this.inherited(arguments);

			// If this tooltip was created in a template, or for some other reason the specified connectId[s]
			// didn't exist during the widget's initialization, then connect now.
			var ids = this.connectId;
			dojo.forEach(dojo.isArrayLike(ids) ? ids : [ids], this.addTarget, this);
		},

		_onTargetMouseEnter: function(/*Event*/ e){
			// summary:
			//		Handler for mouseenter event on the target node
			// tags:
			//		private
			this._onHover(e);
		},

		_onTargetMouseLeave: function(/*Event*/ e){
			// summary:
			//		Handler for mouseleave event on the target node
			// tags:
			//		private
			this._onUnHover(e);
		},

		_onTargetFocus: function(/*Event*/ e){
			// summary:
			//		Handler for focus event on the target node
			// tags:
			//		private

			this._focus = true;
			this._onHover(e);
		},

		_onTargetBlur: function(/*Event*/ e){
			// summary:
			//		Handler for blur event on the target node
			// tags:
			//		private

			this._focus = false;
			this._onUnHover(e);
		},

		_onHover: function(/*Event*/ e){
			// summary:
			//		Despite the name of this method, it actually handles both hover and focus
			//		events on the target node, setting a timer to show the tooltip.
			// tags:
			//		private
			if(!this._showTimer){
				var target = e.target;
				this._showTimer = setTimeout(dojo.hitch(this, function(){this.open(target)}), this.showDelay);
			}
		},

		_onUnHover: function(/*Event*/ e){
			// summary:
			//		Despite the name of this method, it actually handles both mouseleave and blur
			//		events on the target node, hiding the tooltip.
			// tags:
			//		private

			// keep a tooltip open if the associated element still has focus (even though the
			// mouse moved away)
			if(this._focus){ return; }

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			this.close();
		},

		open: function(/*DomNode*/ target){
 			// summary:
			//		Display the tooltip; usually not called directly.
			// tags:
			//		private

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			dijit.showTooltip(this.label || this.domNode.innerHTML, target, this.position, !this.isLeftToRight());

			this._connectNode = target;
			this.onShow(target, this.position);
		},

		close: function(){
			// summary:
			//		Hide the tooltip or cancel timer for show of tooltip
			// tags:
			//		private

			if(this._connectNode){
				// if tooltip is currently shown
				dijit.hideTooltip(this._connectNode);
				delete this._connectNode;
				this.onHide();
			}
			if(this._showTimer){
				// if tooltip is scheduled to be shown (after a brief delay)
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
		},

		onShow: function(target, position){
			// summary:
			//		Called when the tooltip is shown
			// tags:
			//		callback
		},

		onHide: function(){
			// summary:
			//		Called when the tooltip is hidden
			// tags:
			//		callback
		},

		uninitialize: function(){
			this.close();
			this.inherited(arguments);
		}
	}
);

// dijit.Tooltip.defaultPosition: String[]
//		This variable controls the position of tooltips, if the position is not specified to
//		the Tooltip widget or *TextBox widget itself.  It's an array of strings with the following values:
//
//			* before: places tooltip to the left of the target node/widget, or to the right in
//			  the case of RTL scripts like Hebrew and Arabic
//			* after: places tooltip to the right of the target node/widget, or to the left in
//			  the case of RTL scripts like Hebrew and Arabic
//			* above: tooltip goes above target node
//			* below: tooltip goes below target node
//
//		The list is positions is tried, in order, until a position is found where the tooltip fits
//		within the viewport.
//
//		Be careful setting this parameter.  A value of "above" may work fine until the user scrolls
//		the screen so that there's no room above the target node.   Nodes with drop downs, like
//		DropDownButton or FilteringSelect, are especially problematic, in that you need to be sure
//		that the drop down and tooltip don't overlap, even when the viewport is scrolled so that there
//		is only room below (or above) the target node, but not both.
dijit.Tooltip.defaultPosition = ["after", "before"];

}

if(!dojo._hasResource["dijit.form.ValidationTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ValidationTextBox"] = true;
dojo.provide("dijit.form.ValidationTextBox");








/*=====
	dijit.form.ValidationTextBox.__Constraints = function(){
		// locale: String
		//		locale used for validation, picks up value from this widget's lang attribute
		// _flags_: anything
		//		various flags passed to regExpGen function
		this.locale = "";
		this._flags_ = "";
	}
=====*/

dojo.declare(
	"dijit.form.ValidationTextBox",
	dijit.form.TextBox,
	{
		// summary:
		//		Base class for textbox widgets with the ability to validate content of various types and provide user feedback.
		// tags:
		//		protected

		templateString: dojo.cache("dijit.form", "templates/ValidationTextBox.html", "<div class=\"dijit dijitReset dijitInlineTable dijitLeft\"\r\n\tid=\"widget_${id}\" waiRole=\"presentation\"\r\n\t><div class='dijitReset dijitValidationContainer'\r\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&Chi; \" type=\"text\" tabIndex=\"-1\" readOnly waiRole=\"presentation\"\r\n\t/></div\r\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\r\n\t\t><input class=\"dijitReset dijitInputInner\" dojoAttachPoint='textbox,focusNode' autocomplete=\"off\"\r\n\t\t\t${!nameAttrSetting} type='${type}'\r\n\t/></div\r\n></div>\r\n"),
		baseClass: "dijitTextBox dijitValidationTextBox",

		// required: Boolean
		//		User is required to enter data into this field.
		required: false,

		// promptMessage: String
		//		If defined, display this hint string immediately on focus to the textbox, if empty.
		//		Think of this like a tooltip that tells the user what to do, not an error message
		//		that tells the user what they've done wrong.
		//
		//		Message disappears when user starts typing.
		promptMessage: "",

		// invalidMessage: String
		// 		The message to display if value is invalid.
		//		The translated string value is read from the message file by default.
		// 		Set to "" to use the promptMessage instead.
		invalidMessage: "$_unset_$",

		// missingMessage: String
		// 		The message to display if value is empty and the field is required.
		//		The translated string value is read from the message file by default.
		// 		Set to "" to use the invalidMessage instead.
		missingMessage: "$_unset_$",

		// constraints: dijit.form.ValidationTextBox.__Constraints
		//		user-defined object needed to pass parameters to the validator functions
		constraints: {},

		// regExp: [extension protected] String
		//		regular expression string used to validate the input
		//		Do not specify both regExp and regExpGen
		regExp: ".*",

		regExpGen: function(/*dijit.form.ValidationTextBox.__Constraints*/constraints){
			// summary:
			//		Overridable function used to generate regExp when dependent on constraints.
			//		Do not specify both regExp and regExpGen.
			// tags:
			//		extension protected
			return this.regExp; // String
		},

		// state: [readonly] String
		//		Shows current state (ie, validation result) of input (Normal, Warning, or Error)
		state: "",

		// tooltipPosition: String[]
		//		See description of `dijit.Tooltip.defaultPosition` for details on this parameter.
		tooltipPosition: [],

		_setValueAttr: function(){
			// summary:
			//		Hook so attr('value', ...) works.
			this.inherited(arguments);
			this.validate(this._focused);
		},

		validator: function(/*anything*/value, /*dijit.form.ValidationTextBox.__Constraints*/constraints){
			// summary:
			//		Overridable function used to validate the text input against the regular expression.
			// tags:
			//		protected
			return (new RegExp("^(?:" + this.regExpGen(constraints) + ")"+(this.required?"":"?")+"$")).test(value) &&
				(!this.required || !this._isEmpty(value)) &&
				(this._isEmpty(value) || this.parse(value, constraints) !== undefined); // Boolean
		},

		_isValidSubset: function(){
			// summary:
			//		Returns true if the value is either already valid or could be made valid by appending characters.
			//		This is used for validation while the user [may be] still typing.
			return this.textbox.value.search(this._partialre) == 0;
		},

		isValid: function(/*Boolean*/ isFocused){
			// summary:
			//		Tests if value is valid.
			//		Can override with your own routine in a subclass.
			// tags:
			//		protected
			return this.validator(this.textbox.value, this.constraints);
		},

		_isEmpty: function(value){
			// summary:
			//		Checks for whitespace
			return (this.trim ? /^\s*$/ : /^$/).test(value); // Boolean
		},

		getErrorMessage: function(/*Boolean*/ isFocused){
			// summary:
			//		Return an error message to show if appropriate
			// tags:
			//		protected
			return (this.required && this._isEmpty(this.textbox.value)) ? this.missingMessage : this.invalidMessage; // String
		},

		getPromptMessage: function(/*Boolean*/ isFocused){
			// summary:
			//		Return a hint message to show when widget is first focused
			// tags:
			//		protected
			return this.promptMessage; // String
		},

		_maskValidSubsetError: true,
		validate: function(/*Boolean*/ isFocused){
			// summary:
			//		Called by oninit, onblur, and onkeypress.
			// description:
			//		Show missing or invalid messages if appropriate, and highlight textbox field.
			// tags:
			//		protected
			var message = "";
			var isValid = this.disabled || this.isValid(isFocused);
			if(isValid){ this._maskValidSubsetError = true; }
			var isEmpty = this._isEmpty(this.textbox.value);
			var isValidSubset = !isValid && !isEmpty && isFocused && this._isValidSubset();
			this.state = ((isValid || ((!this._hasBeenBlurred || isFocused) && isEmpty) || isValidSubset) && this._maskValidSubsetError) ? "" : "Error";
			if(this.state == "Error"){ this._maskValidSubsetError = isFocused; } // we want the error to show up afer a blur and refocus
			this._setStateClass();
			dijit.setWaiState(this.focusNode, "invalid", isValid ? "false" : "true");
			if(isFocused){
				if(this.state == "Error"){
					message = this.getErrorMessage(true);
				}else{
					message = this.getPromptMessage(true); // show the prompt whever there's no error
				}
				this._maskValidSubsetError = true; // since we're focused, always mask warnings
			}
			this.displayMessage(message);
			return isValid;
		},

		// _message: String
		//		Currently displayed message
		_message: "",

		displayMessage: function(/*String*/ message){
			// summary:
			//		Overridable method to display validation errors/hints.
			//		By default uses a tooltip.
			// tags:
			//		extension
			if(this._message == message){ return; }
			this._message = message;
			dijit.hideTooltip(this.domNode);
			if(message){
				dijit.showTooltip(message, this.domNode, this.tooltipPosition, !this.isLeftToRight());
			}
		},

		_refreshState: function(){
			// Overrides TextBox._refreshState()
			this.validate(this._focused);
			this.inherited(arguments);
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		constructor: function(){
			this.constraints = {};
		},

		_setConstraintsAttr: function(/* Object */ constraints){
			if(!constraints.locale && this.lang){
				constraints.locale = this.lang;
			}
			this.constraints = constraints;
			this._computePartialRE();
		},

		_computePartialRE: function(){
			var p = this.regExpGen(this.constraints);
			this.regExp = p;
			var partialre = "";
			// parse the regexp and produce a new regexp that matches valid subsets
			// if the regexp is .* then there's no use in matching subsets since everything is valid
			if(p != ".*"){ this.regExp.replace(/\\.|\[\]|\[.*?[^\\]{1}\]|\{.*?\}|\(\?[=:!]|./g,
				function (re){
					switch(re.charAt(0)){
						case '{':
						case '+':
						case '?':
						case '*':
						case '^':
						case '$':
						case '|':
						case '(':
							partialre += re;
							break;
						case ")":
							partialre += "|$)";
							break;
						 default:
							partialre += "(?:"+re+"|$)";
							break;
					}
				}
			);}
			try{ // this is needed for now since the above regexp parsing needs more test verification
				"".search(partialre);
			}catch(e){ // should never be here unless the original RE is bad or the parsing is bad
				partialre = this.regExp;
				console.warn('RegExp error in ' + this.declaredClass + ': ' + this.regExp);
			} // should never be here unless the original RE is bad or the parsing is bad
			this._partialre = "^(?:" + partialre + ")$";
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			this.messages = dojo.i18n.getLocalization("dijit.form", "validate", this.lang);
			if(this.invalidMessage == "$_unset_$"){ this.invalidMessage = this.messages.invalidMessage; }
			if(!this.invalidMessage){ this.invalidMessage = this.promptMessage; }
			if(this.missingMessage == "$_unset_$"){ this.missingMessage = this.messages.missingMessage; }
			if(!this.missingMessage){ this.missingMessage = this.invalidMessage; }
			this._setConstraintsAttr(this.constraints); // this needs to happen now (and later) due to codependency on _set*Attr calls attachPoints
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			this.inherited(arguments);	// call FormValueWidget._setDisabledAttr()
			this._refreshState();
		},

		_setRequiredAttr: function(/*Boolean*/ value){
			this.required = value;
			dijit.setWaiState(this.focusNode, "required", value);
			this._refreshState();
		},

		reset:function(){
			// Overrides dijit.form.TextBox.reset() by also
			// hiding errors about partial matches
			this._maskValidSubsetError = true;
			this.inherited(arguments);
		},

		_onBlur: function(){
			this.displayMessage('');
			this.inherited(arguments);
		}
	}
);

dojo.declare(
	"dijit.form.MappedTextBox",
	dijit.form.ValidationTextBox,
	{
		// summary:
		//		A dijit.form.ValidationTextBox subclass which provides a base class for widgets that have
		//		a visible formatted display value, and a serializable
		//		value in a hidden input field which is actually sent to the server.
		// description:
		//		The visible display may
		//		be locale-dependent and interactive.  The value sent to the server is stored in a hidden
		//		input field which uses the `name` attribute declared by the original widget.  That value sent
		//		to the server is defined by the dijit.form.MappedTextBox.serialize method and is typically
		//		locale-neutral.
		// tags:
		//		protected

		postMixInProperties: function(){
			this.inherited(arguments);

			// we want the name attribute to go to the hidden <input>, not the displayed <input>,
			// so override _FormWidget.postMixInProperties() setting of nameAttrSetting
			this.nameAttrSetting = "";
		},

		serialize: function(/*anything*/val, /*Object?*/options){
			// summary:
			//		Overridable function used to convert the attr('value') result to a canonical
			//		(non-localized) string.  For example, will print dates in ISO format, and
			//		numbers the same way as they are represented in javascript.
			// tags:
			//		protected extension
			return val.toString ? val.toString() : ""; // String
		},

		toString: function(){
			// summary:
			//		Returns widget as a printable string using the widget's value
			// tags:
			//		protected
			var val = this.filter(this.get('value')); // call filter in case value is nonstring and filter has been customized
			return val != null ? (typeof val == "string" ? val : this.serialize(val, this.constraints)) : ""; // String
		},

		validate: function(){
			// Overrides `dijit.form.TextBox.validate`
			this.valueNode.value = this.toString();
			return this.inherited(arguments);
		},

		buildRendering: function(){
			// Overrides `dijit._Templated.buildRendering`

			this.inherited(arguments);

			// Create a hidden <input> node with the serialized value used for submit
			// (as opposed to the displayed value).
			// Passing in name as markup rather than calling dojo.create() with an attrs argument
			// to make dojo.query(input[name=...]) work on IE. (see #8660)
			this.valueNode = dojo.place("<input type='hidden'" + (this.name ? " name='" + this.name + "'" : "") + ">", this.textbox, "after");
		},

		reset:function(){
			// Overrides `dijit.form.ValidationTextBox.reset` to
			// reset the hidden textbox value to ''
			this.valueNode.value = '';
			this.inherited(arguments);
		}
	}
);

/*=====
	dijit.form.RangeBoundTextBox.__Constraints = function(){
		// min: Number
		//		Minimum signed value.  Default is -Infinity
		// max: Number
		//		Maximum signed value.  Default is +Infinity
		this.min = min;
		this.max = max;
	}
=====*/

dojo.declare(
	"dijit.form.RangeBoundTextBox",
	dijit.form.MappedTextBox,
	{
		// summary:
		//		Base class for textbox form widgets which defines a range of valid values.

		// rangeMessage: String
		//		The message to display if value is out-of-range
		rangeMessage: "",

		/*=====
		// constraints: dijit.form.RangeBoundTextBox.__Constraints
		constraints: {},
		======*/

		rangeCheck: function(/*Number*/ primitive, /*dijit.form.RangeBoundTextBox.__Constraints*/ constraints){
			// summary:
			//		Overridable function used to validate the range of the numeric input value.
			// tags:
			//		protected
			return	("min" in constraints? (this.compare(primitive,constraints.min) >= 0) : true) &&
				("max" in constraints? (this.compare(primitive,constraints.max) <= 0) : true); // Boolean
		},

		isInRange: function(/*Boolean*/ isFocused){
			// summary:
			//		Tests if the value is in the min/max range specified in constraints
			// tags:
			//		protected
			return this.rangeCheck(this.get('value'), this.constraints);
		},

		_isDefinitelyOutOfRange: function(){
			// summary:
			//		Returns true if the value is out of range and will remain
			//		out of range even if the user types more characters
			var val = this.get('value');
			var isTooLittle = false;
			var isTooMuch = false;
			if("min" in this.constraints){
				var min = this.constraints.min;
				min = this.compare(val, ((typeof min == "number") && min >= 0 && val !=0) ? 0 : min);
				isTooLittle = (typeof min == "number") && min < 0;
			}
			if("max" in this.constraints){
				var max = this.constraints.max;
				max = this.compare(val, ((typeof max != "number") || max > 0) ? max : 0);
				isTooMuch = (typeof max == "number") && max > 0;
			}
			return isTooLittle || isTooMuch;
		},

		_isValidSubset: function(){
			// summary:
			//		Overrides `dijit.form.ValidationTextBox._isValidSubset`.
			//		Returns true if the input is syntactically valid, and either within
			//		range or could be made in range by more typing.
			return this.inherited(arguments) && !this._isDefinitelyOutOfRange();
		},

		isValid: function(/*Boolean*/ isFocused){
			// Overrides dijit.form.ValidationTextBox.isValid to check that the value is also in range.
			return this.inherited(arguments) &&
				((this._isEmpty(this.textbox.value) && !this.required) || this.isInRange(isFocused)); // Boolean
		},

		getErrorMessage: function(/*Boolean*/ isFocused){
			// Overrides dijit.form.ValidationTextBox.getErrorMessage to print "out of range" message if appropriate
			var v = this.get('value');
			if(v !== null && v !== '' && v !== undefined && (typeof v != "number" || !isNaN(v)) && !this.isInRange(isFocused)){ // don't check isInRange w/o a real value
				return this.rangeMessage; // String
			}
			return this.inherited(arguments);
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			if(!this.rangeMessage){
				this.messages = dojo.i18n.getLocalization("dijit.form", "validate", this.lang);
				this.rangeMessage = this.messages.rangeMessage;
			}
		},

		_setConstraintsAttr: function(/* Object */ constraints){
			this.inherited(arguments);
			if(this.focusNode){ // not set when called from postMixInProperties
				if(this.constraints.min !== undefined){
					dijit.setWaiState(this.focusNode, "valuemin", this.constraints.min);
				}else{
					dijit.removeWaiState(this.focusNode, "valuemin");
				}
				if(this.constraints.max !== undefined){
					dijit.setWaiState(this.focusNode, "valuemax", this.constraints.max);
				}else{
					dijit.removeWaiState(this.focusNode, "valuemax");
				}
			}
		},

		_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so attr('value', ...) works.

			dijit.setWaiState(this.focusNode, "valuenow", value);
			this.inherited(arguments);
		}
	}
);

}

if(!dojo._hasResource["dijit.form.ComboBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ComboBox"] = true;
dojo.provide("dijit.form.ComboBox");












dojo.declare(
	"dijit.form.ComboBoxMixin",
	null,
	{
		// summary:
		//		Implements the base functionality for `dijit.form.ComboBox`/`dijit.form.FilteringSelect`
		// description:
		//		All widgets that mix in dijit.form.ComboBoxMixin must extend `dijit.form._FormValueWidget`.
		// tags:
		//		protected

		// item: Object
		//		This is the item returned by the dojo.data.store implementation that
		//		provides the data for this ComboBox, it's the currently selected item.
		item: null,

		// pageSize: Integer
		//		Argument to data provider.
		//		Specifies number of search results per page (before hitting "next" button)
		pageSize: Infinity,

		// store: Object
		//		Reference to data provider object used by this ComboBox
		store: null,

		// fetchProperties: Object
		//		Mixin to the dojo.data store's fetch.
		//		For example, to set the sort order of the ComboBox menu, pass:
		//	|	{ sort: {attribute:"name",descending: true} }
		//		To override the default queryOptions so that deep=false, do:
		//	|	{ queryOptions: {ignoreCase: true, deep: false} }
		fetchProperties:{},

		// query: Object
		//		A query that can be passed to 'store' to initially filter the items,
		//		before doing further filtering based on `searchAttr` and the key.
		//		Any reference to the `searchAttr` is ignored.
		query: {},

		// autoComplete: Boolean
		//		If user types in a partial string, and then tab out of the `<input>` box,
		//		automatically copy the first entry displayed in the drop down list to
		//		the `<input>` field
		autoComplete: true,

		// highlightMatch: String
		// 		One of: "first", "all" or "none".
		//
		//		If the ComboBox/FilteringSelect opens with the search results and the searched
		//		string can be found, it will be highlighted.  If set to "all"
		//		then will probably want to change `queryExpr` parameter to '*${0}*'
		//
		//		Highlighting is only performed when `labelType` is "text", so as to not
		//		interfere with any HTML markup an HTML label might contain.
		highlightMatch: "first",

		// searchDelay: Integer
		//		Delay in milliseconds between when user types something and we start
		//		searching based on that value
		searchDelay: 100,

		// searchAttr: String
		//		Search for items in the data store where this attribute (in the item)
		//		matches what the user typed
		searchAttr: "name",

		// labelAttr: String?
		//		The entries in the drop down list come from this attribute in the
		//		dojo.data items.
		//		If not specified, the searchAttr attribute is used instead.
		labelAttr: "",

		// labelType: String
		//		Specifies how to interpret the labelAttr in the data store items.
		//		Can be "html" or "text".
		labelType: "text",

		// queryExpr: String
		//		This specifies what query ComboBox/FilteringSelect sends to the data store,
		//		based on what the user has typed.  Changing this expression will modify
		//		whether the drop down shows only exact matches, a "starting with" match,
		//		etc.   Use it in conjunction with highlightMatch.
		//		dojo.data query expression pattern.
		//		`${0}` will be substituted for the user text.
		//		`*` is used for wildcards.
		//		`${0}*` means "starts with", `*${0}*` means "contains", `${0}` means "is"
		queryExpr: "${0}*",

		// ignoreCase: Boolean
		//		Set true if the ComboBox/FilteringSelect should ignore case when matching possible items
		ignoreCase: true,

		// hasDownArrow: [const] Boolean
		//		Set this textbox to have a down arrow button, to display the drop down list.
		//		Defaults to true.
		hasDownArrow: true,

		templateString: dojo.cache("dijit.form", "templates/ComboBox.html", "<div class=\"dijit dijitReset dijitInlineTable dijitLeft\"\r\n\tid=\"widget_${id}\"\r\n\tdojoAttachPoint=\"comboNode\" waiRole=\"combobox\"\r\n\t><div class='dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton dijitArrowButtonContainer'\r\n\t\tdojoAttachPoint=\"downArrowNode\" waiRole=\"presentation\"\r\n\t\tdojoAttachEvent=\"onmousedown:_onArrowMouseDown\"\r\n\t\t><input class=\"dijitReset dijitInputField dijitArrowButtonInner\" value=\"&#9660; \" type=\"text\" tabIndex=\"-1\" readOnly waiRole=\"presentation\"\r\n\t\t\t${_buttonInputDisabled}\r\n\t/></div\r\n\t><div class='dijitReset dijitValidationContainer'\r\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&Chi; \" type=\"text\" tabIndex=\"-1\" readOnly waiRole=\"presentation\"\r\n\t/></div\r\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\r\n\t\t><input class='dijitReset dijitInputInner' ${!nameAttrSetting} type=\"text\" autocomplete=\"off\"\r\n\t\t\tdojoAttachEvent=\"onkeypress:_onKeyPress,compositionend\"\r\n\t\t\tdojoAttachPoint=\"textbox,focusNode\" waiRole=\"textbox\" waiState=\"haspopup-true,autocomplete-list\"\r\n\t/></div\r\n></div>\r\n"),

		baseClass: "dijitTextBox dijitComboBox",

		// Set classes like dijitDownArrowButtonHover depending on
		// mouse action over button node
		cssStateNodes: {
			"downArrowNode": "dijitDownArrowButton"
		},

		_getCaretPos: function(/*DomNode*/ element){
			// khtml 3.5.2 has selection* methods as does webkit nightlies from 2005-06-22
			var pos = 0;
			if(typeof(element.selectionStart) == "number"){
				// FIXME: this is totally borked on Moz < 1.3. Any recourse?
				pos = element.selectionStart;
			}else if(dojo.isIE){
				// in the case of a mouse click in a popup being handled,
				// then the dojo.doc.selection is not the textarea, but the popup
				// var r = dojo.doc.selection.createRange();
				// hack to get IE 6 to play nice. What a POS browser.
				var tr = dojo.doc.selection.createRange().duplicate();
				var ntr = element.createTextRange();
				tr.move("character",0);
				ntr.move("character",0);
				try{
					// If control doesnt have focus, you get an exception.
					// Seems to happen on reverse-tab, but can also happen on tab (seems to be a race condition - only happens sometimes).
					// There appears to be no workaround for this - googled for quite a while.
					ntr.setEndPoint("EndToEnd", tr);
					pos = String(ntr.text).replace(/\r/g,"").length;
				}catch(e){
					// If focus has shifted, 0 is fine for caret pos.
				}
			}
			return pos;
		},

		_setCaretPos: function(/*DomNode*/ element, /*Number*/ location){
			location = parseInt(location);
			dijit.selectInputText(element, location, location);
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			// Additional code to set disabled state of ComboBox node.
			// Overrides _FormValueWidget._setDisabledAttr() or ValidationTextBox._setDisabledAttr().
			this.inherited(arguments);
			dijit.setWaiState(this.comboNode, "disabled", value);
		},

		_abortQuery: function(){
			// stop in-progress query
			if(this.searchTimer){
				clearTimeout(this.searchTimer);
				this.searchTimer = null;
			}
			if(this._fetchHandle){
				if(this._fetchHandle.abort){ this._fetchHandle.abort(); }
				this._fetchHandle = null;
			}
		},

		_onInput: function(/*Event*/ evt){
			// summary:
			//		Handles paste events
			if(!this.searchTimer && (evt.type == 'paste'/*IE|WebKit*/ || evt.type == 'input'/*Firefox*/) && this._lastInput != this.textbox.value){
				this.searchTimer = setTimeout(dojo.hitch(this, function(){
					this._onKeyPress({charOrCode: 229}); // fake IME key to cause a search
				}), 100); // long delay that will probably be preempted by keyboard input
			}
			this.inherited(arguments);
		},

		_onKeyPress: function(/*Event*/ evt){
			// summary:
			//		Handles keyboard events
			var key = evt.charOrCode;
			// except for cutting/pasting case - ctrl + x/v
			if(evt.altKey || ((evt.ctrlKey || evt.metaKey) && (key != 'x' && key != 'v')) || key == dojo.keys.SHIFT){
				return; // throw out weird key combinations and spurious events
			}
			var doSearch = false;
			var searchFunction = "_startSearchFromInput";
			var pw = this._popupWidget;
			var dk = dojo.keys;
			var highlighted = null;
			this._prev_key_backspace = false;
			this._abortQuery();
			if(this._isShowingNow){
				pw.handleKey(key);
				highlighted = pw.getHighlightedOption();
			}
			switch(key){
				case dk.PAGE_DOWN:
				case dk.DOWN_ARROW:
				case dk.PAGE_UP:
				case dk.UP_ARROW:
					if(!this._isShowingNow){
						doSearch = true;
						searchFunction = "_startSearchAll";
					}else{
						this._announceOption(highlighted);
					}
					dojo.stopEvent(evt);
					break;

				case dk.ENTER:
					// prevent submitting form if user presses enter. Also
					// prevent accepting the value if either Next or Previous
					// are selected
					if(highlighted){
						// only stop event on prev/next
						if(highlighted == pw.nextButton){
							this._nextSearch(1);
							dojo.stopEvent(evt);
							break;
						}else if(highlighted == pw.previousButton){
							this._nextSearch(-1);
							dojo.stopEvent(evt);
							break;
						}
					}else{
						// Update 'value' (ex: KY) according to currently displayed text
						this._setBlurValue(); // set value if needed
						this._setCaretPos(this.focusNode, this.focusNode.value.length); // move cursor to end and cancel highlighting
					}
					// default case:
					// prevent submit, but allow event to bubble
					evt.preventDefault();
					// fall through

				case dk.TAB:
					var newvalue = this.get('displayedValue');
					//	if the user had More Choices selected fall into the
					//	_onBlur handler
					if(pw && (
						newvalue == pw._messages["previousMessage"] ||
						newvalue == pw._messages["nextMessage"])
					){
						break;
					}
					if(highlighted){
						this._selectOption();
					}
					if(this._isShowingNow){
						this._lastQuery = null; // in case results come back later
						this._hideResultList();
					}
					break;

				case ' ':
					if(highlighted){
						dojo.stopEvent(evt);
						this._selectOption();
						this._hideResultList();
					}else{
						doSearch = true;
					}
					break;

				case dk.ESCAPE:
					if(this._isShowingNow){
						dojo.stopEvent(evt);
						this._hideResultList();
					}
					break;

				case dk.DELETE:
				case dk.BACKSPACE:
					this._prev_key_backspace = true;
					doSearch = true;
					break;

				default:
					// Non char keys (F1-F12 etc..)  shouldn't open list.
					// Ascii characters and IME input (Chinese, Japanese etc.) should.
					// On IE and safari, IME input produces keycode == 229, and we simulate
					// it on firefox by attaching to compositionend event (see compositionend method)
					doSearch = typeof key == 'string' || key == 229;
			}
			if(doSearch){
				// need to wait a tad before start search so that the event
				// bubbles through DOM and we have value visible
				this.item = undefined; // undefined means item needs to be set
				this.searchTimer = setTimeout(dojo.hitch(this, searchFunction),1);
			}
		},

		_autoCompleteText: function(/*String*/ text){
			// summary:
			// 		Fill in the textbox with the first item from the drop down
			// 		list, and highlight the characters that were
			// 		auto-completed. For example, if user typed "CA" and the
			// 		drop down list appeared, the textbox would be changed to
			// 		"California" and "ifornia" would be highlighted.

			var fn = this.focusNode;

			// IE7: clear selection so next highlight works all the time
			dijit.selectInputText(fn, fn.value.length);
			// does text autoComplete the value in the textbox?
			var caseFilter = this.ignoreCase? 'toLowerCase' : 'substr';
			if(text[caseFilter](0).indexOf(this.focusNode.value[caseFilter](0)) == 0){
				var cpos = this._getCaretPos(fn);
				// only try to extend if we added the last character at the end of the input
				if((cpos+1) > fn.value.length){
					// only add to input node as we would overwrite Capitalisation of chars
					// actually, that is ok
					fn.value = text;//.substr(cpos);
					// visually highlight the autocompleted characters
					dijit.selectInputText(fn, cpos);
				}
			}else{
				// text does not autoComplete; replace the whole value and highlight
				fn.value = text;
				dijit.selectInputText(fn);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ dataObject){
			this._fetchHandle = null;
			if(	this.disabled ||
				this.readOnly ||
				(dataObject.query[this.searchAttr] != this._lastQuery)
			){
				return;
			}
			this._popupWidget.clearResultList();
			if(!results.length && !this._maxOptions){ // this condition needs to match !this._isvalid set in FilteringSelect::_openResultList
				this._hideResultList();
				return;
			}


			// Fill in the textbox with the first item from the drop down list,
			// and highlight the characters that were auto-completed. For
			// example, if user typed "CA" and the drop down list appeared, the
			// textbox would be changed to "California" and "ifornia" would be
			// highlighted.

			dataObject._maxOptions = this._maxOptions;
			var nodes = this._popupWidget.createOptions(
				results,
				dataObject,
				dojo.hitch(this, "_getMenuLabelFromItem")
			);

			// show our list (only if we have content, else nothing)
			this._showResultList();

			// #4091:
			//		tell the screen reader that the paging callback finished by
			//		shouting the next choice
			if(dataObject.direction){
				if(1 == dataObject.direction){
					this._popupWidget.highlightFirstOption();
				}else if(-1 == dataObject.direction){
					this._popupWidget.highlightLastOption();
				}
				this._announceOption(this._popupWidget.getHighlightedOption());
			}else if(this.autoComplete && !this._prev_key_backspace /*&& !dataObject.direction*/
				// when the user clicks the arrow button to show the full list,
				// startSearch looks for "*".
				// it does not make sense to autocomplete
				// if they are just previewing the options available.
				&& !/^[*]+$/.test(dataObject.query[this.searchAttr])){
					this._announceOption(nodes[1]); // 1st real item
			}
		},

		_showResultList: function(){
			this._hideResultList();
			// hide the tooltip
			this.displayMessage("");

			// Position the list and if it's too big to fit on the screen then
			// size it to the maximum possible height
			// Our dear friend IE doesnt take max-height so we need to
			// calculate that on our own every time

			// TODO: want to redo this, see
			//		http://trac.dojotoolkit.org/ticket/3272
			//	and
			//		http://trac.dojotoolkit.org/ticket/4108


			// natural size of the list has changed, so erase old
			// width/height settings, which were hardcoded in a previous
			// call to this function (via dojo.marginBox() call)
			dojo.style(this._popupWidget.domNode, {width: "", height: ""});

			var best = this.open();
			// #3212:
			//		only set auto scroll bars if necessary prevents issues with
			//		scroll bars appearing when they shouldn't when node is made
			//		wider (fractional pixels cause this)
			var popupbox = dojo.marginBox(this._popupWidget.domNode);
			this._popupWidget.domNode.style.overflow =
				((best.h == popupbox.h) && (best.w == popupbox.w)) ? "hidden" : "auto";
			// #4134:
			//		borrow TextArea scrollbar test so content isn't covered by
			//		scrollbar and horizontal scrollbar doesn't appear
			var newwidth = best.w;
			if(best.h < this._popupWidget.domNode.scrollHeight){
				newwidth += 16;
			}
			dojo.marginBox(this._popupWidget.domNode, {
				h: best.h,
				w: Math.max(newwidth, this.domNode.offsetWidth)
			});
			
			// If we increased the width of drop down to match the width of ComboBox.domNode,
			// then need to reposition the drop down (wrapper) so (all of) the drop down still
			// appears underneath the ComboBox.domNode
			if(newwidth < this.domNode.offsetWidth){
				this._popupWidget.domNode.parentNode.style.left = dojo.position(this.domNode, true).x + "px";
			}

			dijit.setWaiState(this.comboNode, "expanded", "true");
		},

		_hideResultList: function(){
			this._abortQuery();
			if(this._isShowingNow){
				dijit.popup.close(this._popupWidget);
				this._isShowingNow=false;
				dijit.setWaiState(this.comboNode, "expanded", "false");
				dijit.removeWaiState(this.focusNode,"activedescendant");
			}
		},

		_setBlurValue: function(){
			// if the user clicks away from the textbox OR tabs away, set the
			// value to the textbox value
			// #4617:
			//		if value is now more choices or previous choices, revert
			//		the value
			var newvalue = this.get('displayedValue');
			var pw = this._popupWidget;
			if(pw && (
				newvalue == pw._messages["previousMessage"] ||
				newvalue == pw._messages["nextMessage"]
				)
			){
				this._setValueAttr(this._lastValueReported, true);
			}else if(typeof this.item == "undefined"){
				// Update 'value' (ex: KY) according to currently displayed text
				this.item = null;
				this.set('displayedValue', newvalue);
			}else{
				if(this.value != this._lastValueReported){
					dijit.form._FormValueWidget.prototype._setValueAttr.call(this, this.value, true);
				}
				this._refreshState();
			}
		},

		_onBlur: function(){
			// summary:
			//		Called magically when focus has shifted away from this widget and it's drop down
			this._hideResultList();
			this.inherited(arguments);
		},

		_setItemAttr: function(/*item*/ item, /*Boolean?*/ priorityChange, /*String?*/ displayedValue){
			// summary:
			//              Set the displayed valued in the input box, and the hidden value
			//              that gets submitted, based on a dojo.data store item.
			// description:
			//              Users shouldn't call this function; they should be calling
			//              attr('item', value)
			// tags:
			//              private
			if(!displayedValue){ displayedValue = this.labelFunc(item, this.store); }
			this.value = this._getValueField() != this.searchAttr? this.store.getIdentity(item) : displayedValue;
			this.item = item;
			dijit.form.ComboBox.superclass._setValueAttr.call(this, this.value, priorityChange, displayedValue);
		},

		_announceOption: function(/*Node*/ node){
			// summary:
			//		a11y code that puts the highlighted option in the textbox.
			//		This way screen readers will know what is happening in the
			//		menu.

			if(!node){
				return;
			}
			// pull the text value from the item attached to the DOM node
			var newValue;
			if(node == this._popupWidget.nextButton ||
				node == this._popupWidget.previousButton){
				newValue = node.innerHTML;
				this.item = undefined;
				this.value = '';
			}else{
				newValue = this.labelFunc(node.item, this.store);
				this.set('item', node.item, false, newValue);
			}
			// get the text that the user manually entered (cut off autocompleted text)
			this.focusNode.value = this.focusNode.value.substring(0, this._lastInput.length);
			// set up ARIA activedescendant
			dijit.setWaiState(this.focusNode, "activedescendant", dojo.attr(node, "id"));
			// autocomplete the rest of the option to announce change
			this._autoCompleteText(newValue);
		},

		_selectOption: function(/*Event*/ evt){
			// summary:
			//		Menu callback function, called when an item in the menu is selected.
			if(evt){
				this._announceOption(evt.target);
			}
			this._hideResultList();
			this._setCaretPos(this.focusNode, this.focusNode.value.length);
			dijit.form._FormValueWidget.prototype._setValueAttr.call(this, this.value, true); // set this.value and fire onChange
		},

		_onArrowMouseDown: function(evt){
			// summary:
			//		Callback when arrow is clicked
			if(this.disabled || this.readOnly){
				return;
			}
			dojo.stopEvent(evt);
			this.focus();
			if(this._isShowingNow){
				this._hideResultList();
			}else{
				// forces full population of results, if they click
				// on the arrow it means they want to see more options
				this._startSearchAll();
			}
		},

		_startSearchAll: function(){
			this._startSearch('');
		},

		_startSearchFromInput: function(){
			this._startSearch(this.focusNode.value.replace(/([\\\*\?])/g, "\\$1"));
		},

		_getQueryString: function(/*String*/ text){
			return dojo.string.substitute(this.queryExpr, [text]);
		},

		_startSearch: function(/*String*/ key){
			if(!this._popupWidget){
				var popupId = this.id + "_popup";
				this._popupWidget = new dijit.form._ComboBoxMenu({
					onChange: dojo.hitch(this, this._selectOption),
					id: popupId,
					dir: this.dir
				});
				dijit.removeWaiState(this.focusNode,"activedescendant");
				dijit.setWaiState(this.textbox,"owns",popupId); // associate popup with textbox
			}
			// create a new query to prevent accidentally querying for a hidden
			// value from FilteringSelect's keyField
			var query = dojo.clone(this.query); // #5970
			this._lastInput = key; // Store exactly what was entered by the user.
			this._lastQuery = query[this.searchAttr] = this._getQueryString(key);
			// #5970: set _lastQuery, *then* start the timeout
			// otherwise, if the user types and the last query returns before the timeout,
			// _lastQuery won't be set and their input gets rewritten
			this.searchTimer=setTimeout(dojo.hitch(this, function(query, _this){
				this.searchTimer = null;
				var fetch = {
					queryOptions: {
						ignoreCase: this.ignoreCase,
						deep: true
					},
					query: query,
					onBegin: dojo.hitch(this, "_setMaxOptions"),
					onComplete: dojo.hitch(this, "_openResultList"),
					onError: function(errText){
						_this._fetchHandle = null;
						console.error('dijit.form.ComboBox: ' + errText);
						dojo.hitch(_this, "_hideResultList")();
					},
					start: 0,
					count: this.pageSize
				};
				dojo.mixin(fetch, _this.fetchProperties);
				this._fetchHandle = _this.store.fetch(fetch);

				var nextSearch = function(dataObject, direction){
					dataObject.start += dataObject.count*direction;
					// #4091:
					//		tell callback the direction of the paging so the screen
					//		reader knows which menu option to shout
					dataObject.direction = direction;
					this._fetchHandle = this.store.fetch(dataObject);
				};
				this._nextSearch = this._popupWidget.onPage = dojo.hitch(this, nextSearch, this._fetchHandle);
			}, query, this), this.searchDelay);
		},

		_setMaxOptions: function(size, request){
			 this._maxOptions = size;
		},

		_getValueField: function(){
			// summmary:
			//		Helper for postMixInProperties() to set this.value based on data inlined into the markup.
			//		Returns the attribute name in the item (in dijit.form._ComboBoxDataStore) to use as the value.
			return this.searchAttr;
		},

		/////////////// Event handlers /////////////////////

		// FIXME: For 2.0, rename to "_compositionEnd"
		compositionend: function(/*Event*/ evt){
			// summary:
			//		When inputting characters using an input method, such as
			//		Asian languages, it will generate this event instead of
			//		onKeyDown event.
			//		Note: this event is only triggered in FF (not in IE/safari)
			// tags:
			//		private

			// 229 is the code produced by IE and safari while pressing keys during
			// IME input mode
			this._onKeyPress({charOrCode: 229});
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		constructor: function(){
			this.query={};
			this.fetchProperties={};
		},

		postMixInProperties: function(){
			if(!this.store){
				var srcNodeRef = this.srcNodeRef;

				// if user didn't specify store, then assume there are option tags
				this.store = new dijit.form._ComboBoxDataStore(srcNodeRef);

				// if there is no value set and there is an option list, set
				// the value to the first value to be consistent with native
				// Select

				// Firefox and Safari set value
				// IE6 and Opera set selectedIndex, which is automatically set
				// by the selected attribute of an option tag
				// IE6 does not set value, Opera sets value = selectedIndex
				if(!("value" in this.params)){
					var item = this.store.fetchSelectedItem();
					if(item){
						var valueField = this._getValueField();
						this.value = valueField != this.searchAttr? this.store.getValue(item, valueField) : this.labelFunc(item, this.store);
					}
				}
			}
			this.inherited(arguments);
		},

		postCreate: function(){
			// summary:
			//		Subclasses must call this method from their postCreate() methods
			// tags:
			//		protected

			if(!this.hasDownArrow){
				this.downArrowNode.style.display = "none";
			}

			// find any associated label element and add to ComboBox node.
			var label=dojo.query('label[for="'+this.id+'"]');
			if(label.length){
				label[0].id = (this.id+"_label");
				var cn=this.comboNode;
				dijit.setWaiState(cn, "labelledby", label[0].id);

			}
			this.inherited(arguments);
		},

		uninitialize: function(){
			if(this._popupWidget && !this._popupWidget._destroyed){
				this._hideResultList();
				this._popupWidget.destroy();
			}
			this.inherited(arguments);
		},

		_getMenuLabelFromItem: function(/*Item*/ item){
			var label = this.labelAttr? this.store.getValue(item, this.labelAttr) : this.labelFunc(item, this.store);
			var labelType = this.labelType;
			// If labelType is not "text" we don't want to screw any markup ot whatever.
			if(this.highlightMatch != "none" && this.labelType == "text" && this._lastInput){
				label = this.doHighlight(label, this._escapeHtml(this._lastInput));
				labelType = "html";
			}
			return {html: labelType == "html", label: label};
		},

		doHighlight: function(/*String*/label, /*String*/find){
			// summary:
			//		Highlights the string entered by the user in the menu.  By default this
			//		highlights the first occurence found. Override this method
			//		to implement your custom highlighing.
			// tags:
			//		protected

			// Add greedy when this.highlightMatch == "all"
			var modifiers = "i"+(this.highlightMatch == "all"?"g":"");
			var escapedLabel = this._escapeHtml(label);
			find = dojo.regexp.escapeString(find); // escape regexp special chars
			var ret = escapedLabel.replace(new RegExp("(^|\\s)("+ find +")", modifiers),
					'$1<span class="dijitComboBoxHighlightMatch">$2</span>');
			return ret;// returns String, (almost) valid HTML (entities encoded)
		},

		_escapeHtml: function(/*string*/str){
			// TODO Should become dojo.html.entities(), when exists use instead
			// summary:
			//		Adds escape sequences for special characters in XML: &<>"'
			str = String(str).replace(/&/gm, "&amp;").replace(/</gm, "&lt;")
				.replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
			return str; // string
		},

		open: function(){
			// summary:
			//		Opens the drop down menu.  TODO: rename to _open.
			// tags:
			//		private
			this._isShowingNow=true;
			return dijit.popup.open({
				popup: this._popupWidget,
				around: this.domNode,
				parent: this
			});
		},

		reset: function(){
			// Overrides the _FormWidget.reset().
			// Additionally reset the .item (to clean up).
			this.item = null;
			this.inherited(arguments);
		},

		labelFunc: function(/*item*/ item, /*dojo.data.store*/ store){
			// summary:
			//              Computes the label to display based on the dojo.data store item.
			// returns:
			//              The label that the ComboBox should display
			// tags:
			//              private

			// Use toString() because XMLStore returns an XMLItem whereas this
			// method is expected to return a String (#9354)
			return store.getValue(item, this.searchAttr).toString(); // String
		}
	}
);

dojo.declare(
	"dijit.form._ComboBoxMenu",
	[dijit._Widget, dijit._Templated, dijit._CssStateMixin],
	{
		// summary:
		//		Focus-less menu for internal use in `dijit.form.ComboBox`
		// tags:
		//		private

		templateString: "<ul class='dijitReset dijitMenu' dojoAttachEvent='onmousedown:_onMouseDown,onmouseup:_onMouseUp,onmouseover:_onMouseOver,onmouseout:_onMouseOut' tabIndex='-1' style='overflow: \"auto\"; overflow-x: \"hidden\";'>"
				+"<li class='dijitMenuItem dijitMenuPreviousButton' dojoAttachPoint='previousButton' waiRole='option'></li>"
				+"<li class='dijitMenuItem dijitMenuNextButton' dojoAttachPoint='nextButton' waiRole='option'></li>"
			+"</ul>",

		// _messages: Object
		//		Holds "next" and "previous" text for paging buttons on drop down
		_messages: null,
		
		baseClass: "dijitComboBoxMenu",

		postMixInProperties: function(){
			this._messages = dojo.i18n.getLocalization("dijit.form", "ComboBox", this.lang);
			this.inherited(arguments);
		},

		_setValueAttr: function(/*Object*/ value){
			this.value = value;
			this.onChange(value);
		},

		// stubs
		onChange: function(/*Object*/ value){
			// summary:
			//		Notifies ComboBox/FilteringSelect that user clicked an option in the drop down menu.
			//		Probably should be called onSelect.
			// tags:
			//		callback
		},
		onPage: function(/*Number*/ direction){
			// summary:
			//		Notifies ComboBox/FilteringSelect that user clicked to advance to next/previous page.
			// tags:
			//		callback
		},

		postCreate: function(){
			// fill in template with i18n messages
			this.previousButton.innerHTML = this._messages["previousMessage"];
			this.nextButton.innerHTML = this._messages["nextMessage"];
			this.inherited(arguments);
		},

		onClose: function(){
			// summary:
			//		Callback from dijit.popup code to this widget, notifying it that it closed
			// tags:
			//		private
			this._blurOptionNode();
		},

		_createOption: function(/*Object*/ item, labelFunc){
			// summary:
			//		Creates an option to appear on the popup menu subclassed by
			//		`dijit.form.FilteringSelect`.

			var labelObject = labelFunc(item);
			var menuitem = dojo.doc.createElement("li");
			dijit.setWaiRole(menuitem, "option");
			if(labelObject.html){
				menuitem.innerHTML = labelObject.label;
			}else{
				menuitem.appendChild(
					dojo.doc.createTextNode(labelObject.label)
				);
			}
			// #3250: in blank options, assign a normal height
			if(menuitem.innerHTML == ""){
				menuitem.innerHTML = "&nbsp;";
			}
			menuitem.item=item;
			return menuitem;
		},

		createOptions: function(results, dataObject, labelFunc){
			// summary:
			//		Fills in the items in the drop down list
			// results:
			//		Array of dojo.data items
			// dataObject:
			//		dojo.data store
			// labelFunc:
			//		Function to produce a label in the drop down list from a dojo.data item

			//this._dataObject=dataObject;
			//this._dataObject.onComplete=dojo.hitch(comboBox, comboBox._openResultList);
			// display "Previous . . ." button
			this.previousButton.style.display = (dataObject.start == 0) ? "none" : "";
			dojo.attr(this.previousButton, "id", this.id + "_prev");
			// create options using _createOption function defined by parent
			// ComboBox (or FilteringSelect) class
			// #2309:
			//		iterate over cache nondestructively
			dojo.forEach(results, function(item, i){
				var menuitem = this._createOption(item, labelFunc);
				menuitem.className = "dijitReset dijitMenuItem" +
					(this.isLeftToRight() ? "" : " dijitMenuItemRtl");
				dojo.attr(menuitem, "id", this.id + i);
				this.domNode.insertBefore(menuitem, this.nextButton);
			}, this);
			// display "Next . . ." button
			var displayMore = false;
			//Try to determine if we should show 'more'...
			if(dataObject._maxOptions && dataObject._maxOptions != -1){
				if((dataObject.start + dataObject.count) < dataObject._maxOptions){
					displayMore = true;
				}else if((dataObject.start + dataObject.count) > dataObject._maxOptions && dataObject.count == results.length){
					//Weird return from a datastore, where a start + count > maxOptions
					// implies maxOptions isn't really valid and we have to go into faking it.
					//And more or less assume more if count == results.length
					displayMore = true;
				}
			}else if(dataObject.count == results.length){
				//Don't know the size, so we do the best we can based off count alone.
				//So, if we have an exact match to count, assume more.
				displayMore = true;
			}

			this.nextButton.style.display = displayMore ? "" : "none";
			dojo.attr(this.nextButton,"id", this.id + "_next");
			return this.domNode.childNodes;
		},

		clearResultList: function(){
			// summary:
			//		Clears the entries in the drop down list, but of course keeps the previous and next buttons.
			while(this.domNode.childNodes.length>2){
				this.domNode.removeChild(this.domNode.childNodes[this.domNode.childNodes.length-2]);
			}
		},

		_onMouseDown: function(/*Event*/ evt){
			dojo.stopEvent(evt);
		},

		_onMouseUp: function(/*Event*/ evt){
			if(evt.target === this.domNode || !this._highlighted_option){
				return;
			}else if(evt.target == this.previousButton){
				this.onPage(-1);
			}else if(evt.target == this.nextButton){
				this.onPage(1);
			}else{
				var tgt = evt.target;
				// while the clicked node is inside the div
				while(!tgt.item){
					// recurse to the top
					tgt = tgt.parentNode;
				}
				this._setValueAttr({ target: tgt }, true);
			}
		},

		_onMouseOver: function(/*Event*/ evt){
			if(evt.target === this.domNode){ return; }
			var tgt = evt.target;
			if(!(tgt == this.previousButton || tgt == this.nextButton)){
				// while the clicked node is inside the div
				while(!tgt.item){
					// recurse to the top
					tgt = tgt.parentNode;
				}
			}
			this._focusOptionNode(tgt);
		},

		_onMouseOut: function(/*Event*/ evt){
			if(evt.target === this.domNode){ return; }
			this._blurOptionNode();
		},

		_focusOptionNode: function(/*DomNode*/ node){
			// summary:
			//		Does the actual highlight.
			if(this._highlighted_option != node){
				this._blurOptionNode();
				this._highlighted_option = node;
				dojo.addClass(this._highlighted_option, "dijitMenuItemSelected");
			}
		},

		_blurOptionNode: function(){
			// summary:
			//		Removes highlight on highlighted option.
			if(this._highlighted_option){
				dojo.removeClass(this._highlighted_option, "dijitMenuItemSelected");
				this._highlighted_option = null;
			}
		},

		_highlightNextOption: function(){
			// summary:
			// 		Highlight the item just below the current selection.
			// 		If nothing selected, highlight first option.

			// because each press of a button clears the menu,
			// the highlighted option sometimes becomes detached from the menu!
			// test to see if the option has a parent to see if this is the case.
			if(!this.getHighlightedOption()){
				var fc = this.domNode.firstChild;
				this._focusOptionNode(fc.style.display == "none" ? fc.nextSibling : fc);
			}else{
				var ns = this._highlighted_option.nextSibling;
				if(ns && ns.style.display != "none"){
					this._focusOptionNode(ns);
				}else{
					this.highlightFirstOption();
				}
			}
			// scrollIntoView is called outside of _focusOptionNode because in IE putting it inside causes the menu to scroll up on mouseover
			dojo.window.scrollIntoView(this._highlighted_option);
		},

		highlightFirstOption: function(){
			// summary:
			// 		Highlight the first real item in the list (not Previous Choices).
			var first = this.domNode.firstChild;
			var second = first.nextSibling;
			this._focusOptionNode(second.style.display == "none" ? first : second); // remotely possible that Previous Choices is the only thing in the list
			dojo.window.scrollIntoView(this._highlighted_option);
		},

		highlightLastOption: function(){
			// summary:
			// 		Highlight the last real item in the list (not More Choices).
			this._focusOptionNode(this.domNode.lastChild.previousSibling);
			dojo.window.scrollIntoView(this._highlighted_option);
		},

		_highlightPrevOption: function(){
			// summary:
			// 		Highlight the item just above the current selection.
			// 		If nothing selected, highlight last option (if
			// 		you select Previous and try to keep scrolling up the list).
			if(!this.getHighlightedOption()){
				var lc = this.domNode.lastChild;
				this._focusOptionNode(lc.style.display == "none" ? lc.previousSibling : lc);
			}else{
				var ps = this._highlighted_option.previousSibling;
				if(ps && ps.style.display != "none"){
					this._focusOptionNode(ps);
				}else{
					this.highlightLastOption();
				}
			}
			dojo.window.scrollIntoView(this._highlighted_option);
		},

		_page: function(/*Boolean*/ up){
			// summary:
			//		Handles page-up and page-down keypresses

			var scrollamount = 0;
			var oldscroll = this.domNode.scrollTop;
			var height = dojo.style(this.domNode, "height");
			// if no item is highlighted, highlight the first option
			if(!this.getHighlightedOption()){
				this._highlightNextOption();
			}
			while(scrollamount<height){
				if(up){
					// stop at option 1
					if(!this.getHighlightedOption().previousSibling ||
						this._highlighted_option.previousSibling.style.display == "none"){
						break;
					}
					this._highlightPrevOption();
				}else{
					// stop at last option
					if(!this.getHighlightedOption().nextSibling ||
						this._highlighted_option.nextSibling.style.display == "none"){
						break;
					}
					this._highlightNextOption();
				}
				// going backwards
				var newscroll=this.domNode.scrollTop;
				scrollamount+=(newscroll-oldscroll)*(up ? -1:1);
				oldscroll=newscroll;
			}
		},

		pageUp: function(){
			// summary:
			//		Handles pageup keypress.
			//		TODO: just call _page directly from handleKey().
			// tags:
			//		private
			this._page(true);
		},

		pageDown: function(){
			// summary:
			//		Handles pagedown keypress.
			//		TODO: just call _page directly from handleKey().
			// tags:
			//		private
			this._page(false);
		},

		getHighlightedOption: function(){
			// summary:
			//		Returns the highlighted option.
			var ho = this._highlighted_option;
			return (ho && ho.parentNode) ? ho : null;
		},

		handleKey: function(key){
			switch(key){
				case dojo.keys.DOWN_ARROW:
					this._highlightNextOption();
					break;
				case dojo.keys.PAGE_DOWN:
					this.pageDown();
					break;
				case dojo.keys.UP_ARROW:
					this._highlightPrevOption();
					break;
				case dojo.keys.PAGE_UP:
					this.pageUp();
					break;
			}
		}
	}
);

dojo.declare(
	"dijit.form.ComboBox",
	[dijit.form.ValidationTextBox, dijit.form.ComboBoxMixin],
	{
		// summary:
		//		Auto-completing text box, and base class for dijit.form.FilteringSelect.
		//
		// description:
		//		The drop down box's values are populated from an class called
		//		a data provider, which returns a list of values based on the characters
		//		that the user has typed into the input box.
		//		If OPTION tags are used as the data provider via markup,
		//		then the OPTION tag's child text node is used as the widget value
		//		when selected.  The OPTION tag's value attribute is ignored.
		//		To set the default value when using OPTION tags, specify the selected
		//		attribute on 1 of the child OPTION tags.
		//
		//		Some of the options to the ComboBox are actually arguments to the data
		//		provider.

		_setValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange, /*String?*/ displayedValue){
			// summary:
			//		Hook so attr('value', value) works.
			// description:
			//		Sets the value of the select.
			this.item = null; // value not looked up in store
			if(!value){ value = ''; } // null translates to blank
			dijit.form.ValidationTextBox.prototype._setValueAttr.call(this, value, priorityChange, displayedValue);
		}
	}
);

dojo.declare("dijit.form._ComboBoxDataStore", null, {
	// summary:
	//		Inefficient but small data store specialized for inlined `dijit.form.ComboBox` data
	//
	// description:
	//		Provides a store for inlined data like:
	//
	//	|	<select>
	//	|		<option value="AL">Alabama</option>
	//	|		...
	//
	//		Actually. just implements the subset of dojo.data.Read/Notification
	//		needed for ComboBox and FilteringSelect to work.
	//
	//		Note that an item is just a pointer to the <option> DomNode.

	constructor: function( /*DomNode*/ root){
		this.root = root;
		if(root.tagName != "SELECT" && root.firstChild){
			root = dojo.query("select", root);
			if(root.length > 0){ // SELECT is a child of srcNodeRef
				root = root[0];
			}else{ // no select, so create 1 to parent the option tags to define selectedIndex
				this.root.innerHTML = "<SELECT>"+this.root.innerHTML+"</SELECT>";
				root = this.root.firstChild;
			}
			this.root = root;
		}
		dojo.query("> option", root).forEach(function(node){
			//	TODO: this was added in #3858 but unclear why/if it's needed;  doesn't seem to be.
			//	If it is needed then can we just hide the select itself instead?
			//node.style.display="none";
			node.innerHTML = dojo.trim(node.innerHTML);
		});

	},

	getValue: function(	/* item */ item,
						/* attribute-name-string */ attribute,
						/* value? */ defaultValue){
		return (attribute == "value") ? item.value : (item.innerText || item.textContent || '');
	},

	isItemLoaded: function(/* anything */ something){
		return true;
	},

	getFeatures: function(){
		return {"dojo.data.api.Read": true, "dojo.data.api.Identity": true};
	},

	_fetchItems: function(	/* Object */ args,
							/* Function */ findCallback,
							/* Function */ errorCallback){
		// summary:
		//		See dojo.data.util.simpleFetch.fetch()
		if(!args.query){ args.query = {}; }
		if(!args.query.name){ args.query.name = ""; }
		if(!args.queryOptions){ args.queryOptions = {}; }
		var matcher = dojo.data.util.filter.patternToRegExp(args.query.name, args.queryOptions.ignoreCase),
			items = dojo.query("> option", this.root).filter(function(option){
				return (option.innerText || option.textContent || '').match(matcher);
			} );
		if(args.sort){
			items.sort(dojo.data.util.sorter.createSortFunction(args.sort, this));
		}
		findCallback(items, args);
	},

	close: function(/*dojo.data.api.Request || args || null */ request){
		return;
	},

	getLabel: function(/* item */ item){
		return item.innerHTML;
	},

	getIdentity: function(/* item */ item){
		return dojo.attr(item, "value");
	},

	fetchItemByIdentity: function(/* Object */ args){
		// summary:
		//		Given the identity of an item, this method returns the item that has
		//		that identity through the onItem callback.
		//		Refer to dojo.data.api.Identity.fetchItemByIdentity() for more details.
		//
		// description:
		//		Given arguments like:
		//
		//	|		{identity: "CA", onItem: function(item){...}
		//
		//		Call `onItem()` with the DOM node `<option value="CA">California</option>`
		var item = dojo.query("> option[value='" + args.identity + "']", this.root)[0];
		args.onItem(item);
	},

	fetchSelectedItem: function(){
		// summary:
		//		Get the option marked as selected, like `<option selected>`.
		//		Not part of dojo.data API.
		var root = this.root,
			si = root.selectedIndex;
		return typeof si == "number"
			? dojo.query("> option:nth-child(" + (si != -1 ? si+1 : 1) + ")", root)[0]
			: null;	// dojo.data.Item
	}
});
//Mix in the simple fetch implementation to this class.
dojo.extend(dijit.form._ComboBoxDataStore,dojo.data.util.simpleFetch);

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.Caret"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.Caret"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

 dojo.provide("ibm.tivoli.tip.dijit.Caret"); 



/**
 * Caret. 
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)   
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.Caret",
	[dijit.form.ComboBoxMixin],
	{
	    /** the DOM node to manage */
	    _node: null,
	    
	    /**
	     * Constructor.
	     */
	    constructor: function(node)
	    {
	        this._node = node;
	    },
	    
        /**
         * Returns the caret position.
         */
        getPosition: function()
		{
			return this._getCaretPos(this._node);
		},
		
		/**
         * Sets the caret position.
         */
		setPosition: function(index)
		{
		    this._setCaretPos(this._node, index);
		}
	}
);

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.Tooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.Tooltip"] = true;
dojo.provide("ibm.tivoli.tip.dijit.Tooltip"); 




/**
 * Tooltip. 
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)    
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.Tooltip",
    [dijit._Widget],
    {
        /** the time (in ms) the tooltip is shown after a splash call */
        splashTime: 2000,
        
        /** the node used to attach the tooltip message */
        _tooltipNode: null,
        /** tooltip widget */
        _tooltip: null,
        /** the close timer (set if a close tooltip has been scheduled) */
        _closeTimer: null,
        /** if true the tooltip must be regularly shown */
        _enabled: true,
       
        constructor: function(enabled)
        {
            this._enabled = enabled;
        },
        
        /**
         * Starts-up the widget.
         */
        startup: function()
        {
            this._tooltip = new dijit.Tooltip();
            this._tooltip.startup();
        },
        
        /**
         * Destroys the widget.
         */
        destroy: function()
        {
            // destroy the tooltip
            this.hide();
            this._tooltip.destroy();
            // call the superclass's method
            this.inherited("destroy", arguments);
        },
        
        /**
         * Sets the tooltip node.
         */
        setNode: function(tooltipNode)
        {
            this._tooltipNode = tooltipNode;
        },
        
        /**
         * Shows the message tooltip containing the given message.
         */
        show: function(message)
        {
            // do not show if not enabled
            if ( this._enabled === false ) {
                return;
            }
            // close the previous tooltip (if any)
            if ( this.isShown() ) {
               this.hide();
            }
            if ( message !== "" && this._tooltipNode ) {
                // open the new tooltip
                this._tooltip.label = message;
                this._tooltip.open(this._tooltipNode);
            }
        },
        
        /**
         * Opens the message tooltip and hides it after a while.
         */
        splash: function(message)
        {
            // do not show if not enabled
            if ( this._enabled === false ) {
                return;
            }
            this.show(message);
            this._closeTimer = setTimeout(dojo.hitch(this, this.hide), this.splashTime);
        },
        
        /**
         * Hides the message tooltip.
         */
        hide: function()
        {
            if ( this.isShown() ) {
                if ( this._closeTimer ){
                    clearTimeout(this._closeTimer);
                    delete this._closeTimer;
                }
                this._tooltip.close();
            }
        },
        
        /**
         * Returns true if the message tooltip is currently shown.
         */
        isShown: function()
        {
            return ( !! this._tooltip._connectNode );   
        }
    }
);

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.Images"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.Images"] = true;
dojo.provide("ibm.tivoli.tip.dijit.Images");

// images cache
ibm.tivoli.tip.dijit.Images._images = 
{
    ICON_COLLAPSED:         dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "icon_collapsed.gif"),
    ICON_EXPANDED:          dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "icon_expanded.gif"),
    ICON_TREE_EMPTY:        dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "icon_tree_empty.gif"),
    OK_ICON:                dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "blank.gif"),
    WARNING_ICON:           dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "warning.gif"),
    WARNING_16_ICON:        dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "tree_warning.gif"),
    ERROR_ICON:             dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "error.gif"),
    ERROR_16_ICON:          dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "tree_error.gif"),
    BLANK_ICON:             dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "blank.gif"),
    ASTERISK_ICON:          dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "asterisk.gif"),
    PICKER_ICON:            dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "asterisk.gif"),
    CALENDAR_ICON:          dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "datatype_date.gif"),
    TIME_ICON:              dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "datatype_time.gif"),
    UP_ARROW_ICON:          dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "uparrow.gif"),
    DOWN_ARROW_ICON:        dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "downarrow.gif"),
    TABLE_ADD_ROW_ICON:     dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "add_row.gif"),
    TABLE_REMOVE_ROW_ICON:  dojo.moduleUrl("ibm.tivoli.tip.dijit.templates.images", "remove_row.gif")
};

ibm.tivoli.tip.dijit.Images.get = function() 
{
    return ibm.tivoli.tip.dijit.Images._images; 
};

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.Widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.Widget"] = true;
dojo.provide("ibm.tivoli.tip.dijit.Widget"); 






/**
 * Widget base class. 
 * 
 * Provides the basic functionalities for a widget and defines 
 * some template methods that must be implemented by inheriting widgets.
 * 
 * The basic functionalities provided by such class are:
 *   
 *      - resources: the labels catalog is already loaded and can be accessed
 *        using the (protected) _resources property.
 *      - tooltip message: the widget shows a tooltip message depending 
 *        on the widget status. Inheriting widgets can specify a DOM node 
 *        (_tooltipNode) which will be used to show the tooltip. If not
 *        specified the widget root DOM node is used.
 *      - status: the widget L&F can be set using the setStatus method.
 *      - focus/blur: inheriting widgets can be notified by focus/blur
 *        events by overriding onFocus/onBlur methods.
 *      - focus node: inheriting widgets can specify a DOM node (_focusNode)
 *        which be used to determine if the widget has got or lost the focus.
 *        If not specified the widget root DOM node is used.
 *      - disabling: the widget can be disabled/enabled by calling the 
 *        setDisabled method. Inheriting widgets can override this method
 *        to add their behaviour.
 * 
 * The template methods are:
 * 
 *      - getValue: returns the widget value
 *      - setValue: sets the widget value
 *      - isEmpty: returns true if the widget's value is empty
 *      - setRequired: sets required the widget's value 
 *      - isRequired: returns true if the widget's value is required
 *      - onValueChanged: called when the widget's value is changed
 *      - onStatusChanged: called when the widget's status is changed
 *      - onFocus: called when the widget has got the focus
 *      - onBlur: called when the widget has lost the focus
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)    
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.Widget",
    dijit._Widget,
    {
        /** widget constants */
        constants: {
            // default view state
            DEFAULT:  0,   
            // required view state
            REQUIRED: 1,   
            // error view state
            ERROR:    2
        },

        /** editable attribute */
        editable: "true",
        /** true if the tooltip is shown if needed */
        useTooltip: "true", 
        
        /** true if the widget can be editable */
        _editable: true,
        /** the node which gives the focus to the whole widget */
        _focusNode: null,
        /** the node used to attach the tooltip message */
        _tooltipNode: null,
        /** true if the widget has focus */
        _hasFocus: false,
        /** the widget message (tooltip) */
        _tooltip: null,       
        /** the widget display status */
        _status: null,
        /** true if the widget is disabled */
        _disabled: false,
        /** widget connections */
        _connections: null,
        /** true if the widget value is wrong */
        _isInError: false,
        /** the focus node tab index */
        _tabIndex: 0,
        /** resources */
        _resources: null,
        /** images */
        _images: null,
       
        /**
         * Constructor
         */
        constructor: function()
        {
            // get the labels
			//console.info("constructor tip.widget");
            this._resources = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
            // get the image
            this._images = ibm.tivoli.tip.dijit.Images.get();
        },
        
        /**
         * Starts-up the widget.
         */
        startup: function()
        {
            // call the superclass's method
			//console.info("startup tip.widget");
            this.inherited("startup", arguments);
           
            // create the tooltip message
            this._tooltip = new ibm.tivoli.tip.dijit.Tooltip();
            this._tooltip.startup();
            this._tooltip.setNode(this._tooltipNode || this.domNode);
           
            // set default widget status
            this.setStatus({ state: this.constants.DEFAULT, message: "" }); 

            // set the focus node
            if ( this._focusNode === null ) {
                // no focus node provided, use the root node
                this._focusNode = this.domNode;
            }
            // get the focus node tabindex
            this._tabIndex = this._focusNode.tabIndex;
            
            // connect callbacks
            this._connections = [];
            this._connections[this._connections.length] = dojo.connect(this._focusNode, "onfocus", this, "_onFocusEvent");
            this._connections[this._connections.length] = dojo.connect(this._focusNode, "onblur", this, "_onBlurEvent");
            
            // set the editable state
            if ( this.editable !== "" ) {
                this.setEditable(this.editable === "true" ? true : false);    
            }            
        },
        
        /**
         * Destroys the widget.
         */
        destroy: function()
        {
            // destroy the tooltip message
			//console.info("destroy tip.widget");
            this._tooltip.destroy();
            // disconnect callbacks
            for ( var i = 0; i < this._connections.length; i++ ) {
                dojo.disconnect(this._connections[i]);
            }
            // call the superclass's method
            this.inherited("destroy", arguments);
        },
        
        /**
         * Enables/disables the widget. 
         */
        setDisabled: function(isDisabled)
        {
            if ( isDisabled != this._disabled ) {
                // update the message
                if ( isDisabled === true ) {
                    // the widget is disabled, hide the message
                    this._tooltip.hide();
                }
                else if ( this._status.state !== this.constants.DEFAULT && this._hasFocus ) {
                    // the widget is in error and has focus                    
                    this._tooltip.show(this._status.message);
                }
                // store the enablement status
                this._disabled = isDisabled;
            }
        },
        
        /**
         * Returns true if the object is disabled.
         */
        isDisabled: function()
        {
            return this._disabled;
        },
        
        /**
         * Enables the widget for editing. 
         */
        setEditable: function(isEditable)
        {
            // store the editable status
            this._editable = isEditable;
            // update the focus node tabindex according to the editable state
            this._focusNode.tabIndex = isEditable ? this._tabIndex : -1;
            // manage the new status
            if ( this._hasFocus && isEditable === false ) {
                // remove the focus
                this._focusNode.blur();
            }
            else if ( isEditable ){
                // close the tooltip 
                this._tooltip.hide();
            }
        },
        
        /**
         * Returns true if the object is editable.
         */
        isEditable: function()
        {
            return this._editable;
        },
        
        /**
         * Sets the widget status: the status is an object with the
         * following two properties:
         * 
         *  - state: the state of the widget (e.g. constants.ERROR)
         *  - message: the message related to the current state.
         */
        setStatus: function(status)
        {
            
            // update the message
            if ( status.state === this.constants.DEFAULT ) {
                // no error in place, hide the message
                this._tooltip.hide();
            }
            else if ( this._hasFocus ) {
                // there is an error, show the message if no message
                // is currently shown or the currently shown message
                // is different from the current one 
                if ( ! this._tooltip.isShown() || status.message !== this._status.message ) {
                    this._tooltip.show(status.message);
                }                      
            }
            // cache old status
            var oldStatus = this._status;
            // set the status            
            this._status = status;
            // notify status changed
            if ( oldStatus && oldStatus.state !== status.state ) {
                this.onStatusChanged(status);    
            }
        },
        
        /**
         * Returns the status object.
         */
        getStatus: function()
        {
            return this._status;
        },
        
        /**        
         * Returns an object which represent the widget value.
         */
        getValue: function()
        {
        },
        
        /**        
         * Sets the widget value.
         */
        setValue: function(value)
        {
        },
        
        /**
         * Returns true if the widget's value is not set.
         */
        isEmpty: function()
        {
        },

        /**
         * Returns true if the widget's value is wrong.
         */
        isInError: function()
        {
            return this._isInError && ! this._disabled;  
        },

        /**
         * Sets the widget required constraint.
         * 
         *  @param required:
         *      true if the value is required, false otherwise.
         */
        setRequired: function(required)
        {
        },
        
        /**
         * Returns true if the widget's value is required.
         */
        isRequired: function()
        {
            return false;
        },
        
        /**
         * Asks the widget to revalidate its value and to set its status accordingly.
         */
        validate: function() 
        {
        },
        
        /**
         * Called by the widget if the widget looses the focus and the 
         * value has changed.
         * 
         *  @param value:
         *      the widget's value (same returned by getValue()). 
         */
        onValueChanged: function(value)
        {
        },
        
        /**
         * Called by the widget if the widget's status changes.
         * 
         *  @param status:
         *      the widget's status (same returned by getStatus()). 
         */
        onStatusChanged: function(status)
        {
        },
        
        /**
         * Returns true if the widget has the focus.
         */
        hasFocus: function()
        {
            return this._hasFocus;    
        },
        
        /**
         * Gives the focus to the widget.
         */
        setFocus: function()
        {
            this.domNode.focus();
        },
         
        /**
         * Called by the widget if the widget has got the focus.
         */
        onFocus: function()
        {
        },
        
        /**
         * Called by the widget if the widget has lost the focus.
         */
        onBlur: function()
        {
        },
        
        /**
         * Sets the tooltip node.
         */
        setTooltipNode: function(node)
        {
            this._tooltip.setNode(node || this.domNode);    
        },        
        
        /**
         * Handles the onfocus event.
         */
        _onFocusEvent: function()
        {
            if ( this._editable ) {
                // the widget is editable give the focus
                this._hasFocus = true;
                // show the message
                this._tooltip.show(this._status.message);
                // notify the event to the superclass
                this.onFocus();
            }
            else {
                // the widget is not editable don't give the focus
                this._tooltip.splash(this._resources.NOT_EDITABLE);
                this._focusNode.blur();
            }
        },
        
        /**
         * Handles the onblur event.
         */
        _onBlurEvent: function()
        {
            this._hasFocus = false;
            if ( this._editable ) {
                // hide the message
                this._tooltip.hide();
                // notify the event to the superclass
                this.onBlur();    
            }
        }
    }
);

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.HtmlSelection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.HtmlSelection"] = true;
dojo.provide("ibm.tivoli.tip.dijit.HtmlSelection"); 




/** 
 * Base class for HTML selection widget (check box and radio button).
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)   
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.HtmlSelection",
    [ibm.tivoli.tip.dijit.Widget, dijit._Templated],
    {
        /** the widget template path */
        templateString:"<input \r\n    name=\"${name}\"\r\n    type=\"${_type}\" \r\n    class=\"htmlSelectionDefault\"\r\n    dojoAttachPoint=\"_htmlSelectionNode\"\r\n    dojoAttachEvent=\"onclick:_onClick\"/>\r\n",
        
        /** HTML name attribute */
        name: "",
        /** HTML checked attribute */
        checked: "false",
        
        /** checkbox DOM node */
        _htmlSelectionNode: null,

        /**
         * Sets the initial value.
         * The initial value can be externally provided by setting the 
         * checked property.  
         */
        startup: function() 
        {
            // call the superclass's method
            this.inherited("startup", arguments);

            // check the checked attribute
            var value = false;
            if ( this.checked === "" || this.checked === "checked" || this.checked === "true" ) {
                value = true;    
            }
            else {
                value = false;
            }
            
            // set the value
            this.setValue(value);
        },
        
        /**        
         * Returns an object which represent the widget value.
         */
        getValue: function()
        {
            return this._htmlSelectionNode.checked; 
        },
        
        /**        
         * Sets the widget value.
         */
        setValue: function(checked)
        {
            // update the selection
            this._htmlSelectionNode.checked = checked; 
            // need for IE > 6.x
            this._htmlSelectionNode.defaultChecked = checked;
        },
        
        /**
         * Returns true if the widget's value is not set.
         */
        isEmpty: function()
        {
            return ( this.getValue() === false );
        },

        /**
         * Enables/disables the widget. 
         */
        setDisabled: function(isDisabled)
        {
            if ( isDisabled != this.isDisabled() ) {
                // update the enablement status
                this.inherited("setDisabled", arguments);
                // update the widget
                if ( isDisabled === true ) {
                    // the widget must be disabled
                    this._htmlSelectionNode.disabled = true;
                    this._htmlSelectionNode.className = "htmlSelectionDisabled";
                }
                else {
                    // the widget must be enabled
                    this._htmlSelectionNode.disabled = false;
                    this._htmlSelectionNode.className = "htmlSelectionDefault";
                }
            }
        },
        
        /**
         * Handles the onclick event.
         */
        _onClick: function(e)
        {
            if ( this.isEditable() ) {
                // notify the change
                this.onValueChanged(this.getValue());
            }
            else {
                // the control is not editable, prevent the change
                // cancel and stop event propagation
                e.preventDefault();
                e.stopPropagation();                
            }
        }
    }
);


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.CheckBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.CheckBox"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
 dojo.provide("ibm.tivoli.tip.dijit.CheckBox"); 



/** 
 * A Check Box widget.
 * 
 * Such widget displays a check box. The widget calls the 
 * onValueChanged callback when the checked state changes
 * even if the widget still has the focus. 
 *   
 * The widget accepts the following parameters:
 * 
 *  - value: the initial value for the check box. 
 *      - "true": the check box must be checked
 *      - "false": the check box must not be checked
 *  - checked: same as the HTML input attribute.
 *      - "", "checked", "true": the check box must be checked
 *      - any other value: the check box must not be checked
 * 
 * Example of a declaration:
 * 
 *       <div 
 *          dojoType="ibm.tivoli.tip.dijit.CheckBox" 
 *          checked="checked"/>
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)   
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.CheckBox",
    [ibm.tivoli.tip.dijit.HtmlSelection],
    {
        /** HTML input type */
        _type: "checkbox" 
    }
);


}

if(!dojo._hasResource["dojo.fx.Toggler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx.Toggler"] = true;
dojo.provide("dojo.fx.Toggler");

dojo.declare("dojo.fx.Toggler", null, {
	// summary:
	//		A simple `dojo.Animation` toggler API.
	//
	// description:
	//		class constructor for an animation toggler. It accepts a packed
	//		set of arguments about what type of animation to use in each
	//		direction, duration, etc. All available members are mixed into 
	//		these animations from the constructor (for example, `node`, 
	//		`showDuration`, `hideDuration`). 
	//
	// example:
	//	|	var t = new dojo.fx.Toggler({
	//	|		node: "nodeId",
	//	|		showDuration: 500,
	//	|		// hideDuration will default to "200"
	//	|		showFunc: dojo.fx.wipeIn, 
	//	|		// hideFunc will default to "fadeOut"
	//	|	});
	//	|	t.show(100); // delay showing for 100ms
	//	|	// ...time passes...
	//	|	t.hide();

	// node: DomNode
	//		the node to target for the showing and hiding animations
	node: null,

	// showFunc: Function
	//		The function that returns the `dojo.Animation` to show the node
	showFunc: dojo.fadeIn,

	// hideFunc: Function	
	//		The function that returns the `dojo.Animation` to hide the node
	hideFunc: dojo.fadeOut,

	// showDuration:
	//		Time in milliseconds to run the show Animation
	showDuration: 200,

	// hideDuration:
	//		Time in milliseconds to run the hide Animation
	hideDuration: 200,

	// FIXME: need a policy for where the toggler should "be" the next
	// time show/hide are called if we're stopped somewhere in the
	// middle.
	// FIXME: also would be nice to specify individual showArgs/hideArgs mixed into
	// each animation individually. 
	// FIXME: also would be nice to have events from the animations exposed/bridged

	/*=====
	_showArgs: null,
	_showAnim: null,

	_hideArgs: null,
	_hideAnim: null,

	_isShowing: false,
	_isHiding: false,
	=====*/

	constructor: function(args){
		var _t = this;

		dojo.mixin(_t, args);
		_t.node = args.node;
		_t._showArgs = dojo.mixin({}, args);
		_t._showArgs.node = _t.node;
		_t._showArgs.duration = _t.showDuration;
		_t.showAnim = _t.showFunc(_t._showArgs);

		_t._hideArgs = dojo.mixin({}, args);
		_t._hideArgs.node = _t.node;
		_t._hideArgs.duration = _t.hideDuration;
		_t.hideAnim = _t.hideFunc(_t._hideArgs);

		dojo.connect(_t.showAnim, "beforeBegin", dojo.hitch(_t.hideAnim, "stop", true));
		dojo.connect(_t.hideAnim, "beforeBegin", dojo.hitch(_t.showAnim, "stop", true));
	},

	show: function(delay){
		// summary: Toggle the node to showing
		// delay: Integer?
		//		Ammount of time to stall playing the show animation
		return this.showAnim.play(delay || 0);
	},

	hide: function(delay){
		// summary: Toggle the node to hidden
		// delay: Integer?
		//		Ammount of time to stall playing the hide animation
		return this.hideAnim.play(delay || 0);
	}
});

}

if(!dojo._hasResource["dojo.fx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx"] = true;
dojo.provide("dojo.fx");
 // FIXME: remove this back-compat require in 2.0 
/*=====
dojo.fx = {
	// summary: Effects library on top of Base animations
};
=====*/
(function(){
	
	var d = dojo, 
		_baseObj = {
			_fire: function(evt, args){
				if(this[evt]){
					this[evt].apply(this, args||[]);
				}
				return this;
			}
		};

	var _chain = function(animations){
		this._index = -1;
		this._animations = animations||[];
		this._current = this._onAnimateCtx = this._onEndCtx = null;

		this.duration = 0;
		d.forEach(this._animations, function(a){
			this.duration += a.duration;
			if(a.delay){ this.duration += a.delay; }
		}, this);
	};
	d.extend(_chain, {
		_onAnimate: function(){
			this._fire("onAnimate", arguments);
		},
		_onEnd: function(){
			d.disconnect(this._onAnimateCtx);
			d.disconnect(this._onEndCtx);
			this._onAnimateCtx = this._onEndCtx = null;
			if(this._index + 1 == this._animations.length){
				this._fire("onEnd");
			}else{
				// switch animations
				this._current = this._animations[++this._index];
				this._onAnimateCtx = d.connect(this._current, "onAnimate", this, "_onAnimate");
				this._onEndCtx = d.connect(this._current, "onEnd", this, "_onEnd");
				this._current.play(0, true);
			}
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			if(!this._current){ this._current = this._animations[this._index = 0]; }
			if(!gotoStart && this._current.status() == "playing"){ return this; }
			var beforeBegin = d.connect(this._current, "beforeBegin", this, function(){
					this._fire("beforeBegin");
				}),
				onBegin = d.connect(this._current, "onBegin", this, function(arg){
					this._fire("onBegin", arguments);
				}),
				onPlay = d.connect(this._current, "onPlay", this, function(arg){
					this._fire("onPlay", arguments);
					d.disconnect(beforeBegin);
					d.disconnect(onBegin);
					d.disconnect(onPlay);
				});
			if(this._onAnimateCtx){
				d.disconnect(this._onAnimateCtx);
			}
			this._onAnimateCtx = d.connect(this._current, "onAnimate", this, "_onAnimate");
			if(this._onEndCtx){
				d.disconnect(this._onEndCtx);
			}
			this._onEndCtx = d.connect(this._current, "onEnd", this, "_onEnd");
			this._current.play.apply(this._current, arguments);
			return this;
		},
		pause: function(){
			if(this._current){
				var e = d.connect(this._current, "onPause", this, function(arg){
						this._fire("onPause", arguments);
						d.disconnect(e);
					});
				this._current.pause();
			}
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			this.pause();
			var offset = this.duration * percent;
			this._current = null;
			d.some(this._animations, function(a){
				if(a.duration <= offset){
					this._current = a;
					return true;
				}
				offset -= a.duration;
				return false;
			});
			if(this._current){
				this._current.gotoPercent(offset / this._current.duration, andPlay);
			}
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			if(this._current){
				if(gotoEnd){
					for(; this._index + 1 < this._animations.length; ++this._index){
						this._animations[this._index].stop(true);
					}
					this._current = this._animations[this._index];
				}
				var e = d.connect(this._current, "onStop", this, function(arg){
						this._fire("onStop", arguments);
						d.disconnect(e);
					});
				this._current.stop();
			}
			return this;
		},
		status: function(){
			return this._current ? this._current.status() : "stopped";
		},
		destroy: function(){
			if(this._onAnimateCtx){ d.disconnect(this._onAnimateCtx); }
			if(this._onEndCtx){ d.disconnect(this._onEndCtx); }
		}
	});
	d.extend(_chain, _baseObj);

	dojo.fx.chain = function(/*dojo.Animation[]*/ animations){
		// summary: 
		//		Chain a list of `dojo.Animation`s to run in sequence
		//
		// description:
		//		Return a `dojo.Animation` which will play all passed
		//		`dojo.Animation` instances in sequence, firing its own
		//		synthesized events simulating a single animation. (eg:
		//		onEnd of this animation means the end of the chain, 
		//		not the individual animations within)
		//
		// example:
		//	Once `node` is faded out, fade in `otherNode`
		//	|	dojo.fx.chain([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		return new _chain(animations) // dojo.Animation
	};

	var _combine = function(animations){
		this._animations = animations||[];
		this._connects = [];
		this._finished = 0;

		this.duration = 0;
		d.forEach(animations, function(a){
			var duration = a.duration;
			if(a.delay){ duration += a.delay; }
			if(this.duration < duration){ this.duration = duration; }
			this._connects.push(d.connect(a, "onEnd", this, "_onEnd"));
		}, this);
		
		this._pseudoAnimation = new d.Animation({curve: [0, 1], duration: this.duration});
		var self = this;
		d.forEach(["beforeBegin", "onBegin", "onPlay", "onAnimate", "onPause", "onStop", "onEnd"], 
			function(evt){
				self._connects.push(d.connect(self._pseudoAnimation, evt,
					function(){ self._fire(evt, arguments); }
				));
			}
		);
	};
	d.extend(_combine, {
		_doAction: function(action, args){
			d.forEach(this._animations, function(a){
				a[action].apply(a, args);
			});
			return this;
		},
		_onEnd: function(){
			if(++this._finished > this._animations.length){
				this._fire("onEnd");
			}
		},
		_call: function(action, args){
			var t = this._pseudoAnimation;
			t[action].apply(t, args);
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			this._finished = 0;
			this._doAction("play", arguments);
			this._call("play", arguments);
			return this;
		},
		pause: function(){
			this._doAction("pause", arguments);
			this._call("pause", arguments);
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			var ms = this.duration * percent;
			d.forEach(this._animations, function(a){
				a.gotoPercent(a.duration < ms ? 1 : (ms / a.duration), andPlay);
			});
			this._call("gotoPercent", arguments);
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			this._doAction("stop", arguments);
			this._call("stop", arguments);
			return this;
		},
		status: function(){
			return this._pseudoAnimation.status();
		},
		destroy: function(){
			d.forEach(this._connects, dojo.disconnect);
		}
	});
	d.extend(_combine, _baseObj);

	dojo.fx.combine = function(/*dojo.Animation[]*/ animations){
		// summary: 
		//		Combine a list of `dojo.Animation`s to run in parallel
		//
		// description:
		//		Combine an array of `dojo.Animation`s to run in parallel, 
		//		providing a new `dojo.Animation` instance encompasing each
		//		animation, firing standard animation events.
		//
		// example:
		//	Fade out `node` while fading in `otherNode` simultaneously
		//	|	dojo.fx.combine([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		// example:
		//	When the longest animation ends, execute a function:
		//	|	var anim = dojo.fx.combine([
		//	|		dojo.fadeIn({ node: n, duration:700 }),
		//	|		dojo.fadeOut({ node: otherNode, duration: 300 })
		//	|	]);
		//	|	dojo.connect(anim, "onEnd", function(){
		//	|		// overall animation is done.
		//	|	});
		//	|	anim.play(); // play the animation
		//
		return new _combine(animations); // dojo.Animation
	};

	dojo.fx.wipeIn = function(/*Object*/ args){
		// summary:
		//		Expand a node to it's natural height.
		//
		// description:
		//		Returns an animation that will expand the
		//		node defined in 'args' object from it's current height to
		//		it's natural height (with no scrollbar).
		//		Node must have no margin/border/padding.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	dojo.fx.wipeIn({
		//	|		node:"someId"
		//	|	}).play()
		var node = args.node = d.byId(args.node), s = node.style, o;

		var anim = d.animateProperty(d.mixin({
			properties: {
				height: {
					// wrapped in functions so we wait till the last second to query (in case value has changed)
					start: function(){
						// start at current [computed] height, but use 1px rather than 0
						// because 0 causes IE to display the whole panel
						o = s.overflow;
						s.overflow = "hidden";
						if(s.visibility == "hidden" || s.display == "none"){
							s.height = "1px";
							s.display = "";
							s.visibility = "";
							return 1;
						}else{
							var height = d.style(node, "height");
							return Math.max(height, 1);
						}
					},
					end: function(){
						return node.scrollHeight;
					}
				}
			}
		}, args));

		d.connect(anim, "onEnd", function(){ 
			s.height = "auto";
			s.overflow = o;
		});

		return anim; // dojo.Animation
	}

	dojo.fx.wipeOut = function(/*Object*/ args){
		// summary:
		//		Shrink a node to nothing and hide it. 
		//
		// description:
		//		Returns an animation that will shrink node defined in "args"
		//		from it's current height to 1px, and then hide it.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		// 
		// example:
		//	|	dojo.fx.wipeOut({ node:"someId" }).play()
		
		var node = args.node = d.byId(args.node), s = node.style, o;
		
		var anim = d.animateProperty(d.mixin({
			properties: {
				height: {
					end: 1 // 0 causes IE to display the whole panel
				}
			}
		}, args));

		d.connect(anim, "beforeBegin", function(){
			o = s.overflow;
			s.overflow = "hidden";
			s.display = "";
		});
		d.connect(anim, "onEnd", function(){
			s.overflow = o;
			s.height = "auto";
			s.display = "none";
		});

		return anim; // dojo.Animation
	}

	dojo.fx.slideTo = function(/*Object*/ args){
		// summary:
		//		Slide a node to a new top/left position
		//
		// description:
		//		Returns an animation that will slide "node" 
		//		defined in args Object from its current position to
		//		the position defined by (args.left, args.top).
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on). Special args members
		//		are `top` and `left`, which indicate the new position to slide to.
		//
		// example:
		//	|	dojo.fx.slideTo({ node: node, left:"40", top:"50", units:"px" }).play()

		var node = args.node = d.byId(args.node), 
			top = null, left = null;

		var init = (function(n){
			return function(){
				var cs = d.getComputedStyle(n);
				var pos = cs.position;
				top = (pos == 'absolute' ? n.offsetTop : parseInt(cs.top) || 0);
				left = (pos == 'absolute' ? n.offsetLeft : parseInt(cs.left) || 0);
				if(pos != 'absolute' && pos != 'relative'){
					var ret = d.position(n, true);
					top = ret.y;
					left = ret.x;
					n.style.position="absolute";
					n.style.top=top+"px";
					n.style.left=left+"px";
				}
			};
		})(node);
		init();

		var anim = d.animateProperty(d.mixin({
			properties: {
				top: args.top || 0,
				left: args.left || 0
			}
		}, args));
		d.connect(anim, "beforeBegin", anim, init);

		return anim; // dojo.Animation
	}

})();

}

if(!dojo._hasResource["dijit.layout._LayoutWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._LayoutWidget"] = true;
dojo.provide("dijit.layout._LayoutWidget");





dojo.declare("dijit.layout._LayoutWidget",
	[dijit._Widget, dijit._Container, dijit._Contained],
	{
		// summary:
		//		Base class for a _Container widget which is responsible for laying out its children.
		//		Widgets which mixin this code must define layout() to manage placement and sizing of the children.

		// baseClass: [protected extension] String
		//		This class name is applied to the widget's domNode
		//		and also may be used to generate names for sub nodes,
		//		for example dijitTabContainer-content.
		baseClass: "dijitLayoutContainer",

		// isLayoutContainer: [protected] Boolean
		//		Indicates that this widget is going to call resize() on its
		//		children widgets, setting their size, when they become visible.
		isLayoutContainer: true,

		postCreate: function(){
			dojo.addClass(this.domNode, "dijitContainer");

			this.inherited(arguments);
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under dojo.doc.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Need to call inherited first - so that child widgets get started
			// up correctly
			this.inherited(arguments);

			// If I am a not being controlled by a parent layout widget...
			var parent = this.getParent && this.getParent()
			if(!(parent && parent.isLayoutContainer)){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// Since my parent isn't a layout container, and my style *may be* width=height=100%
				// or something similar (either set directly or via a CSS class),
				// monitor when my size changes so that I can re-layout.
				// For browsers where I can't directly monitor when my size changes,
				// monitor when the viewport changes size, which *may* indicate a size change for me.
				this.connect(dojo.isIE ? this.domNode : dojo.global, 'onresize', function(){
					// Using function(){} closure to ensure no arguments to resize.
					this.resize();
				});
			}
		},

		resize: function(changeSize, resultSize){
			// summary:
			//		Call this to resize a widget, or after its size has changed.
			// description:
			//		Change size mode:
			//			When changeSize is specified, changes the marginBox of this widget
			//			and forces it to relayout its contents accordingly.
			//			changeSize may specify height, width, or both.
			//
			//			If resultSize is specified it indicates the size the widget will
			//			become after changeSize has been applied.
			//
			//		Notification mode:
			//			When changeSize is null, indicates that the caller has already changed
			//			the size of the widget, or perhaps it changed because the browser
			//			window was resized.  Tells widget to relayout its contents accordingly.
			//
			//			If resultSize is also specified it indicates the size the widget has
			//			become.
			//
			//		In either mode, this method also:
			//			1. Sets this._borderBox and this._contentBox to the new size of
			//				the widget.  Queries the current domNode size if necessary.
			//			2. Calls layout() to resize contents (and maybe adjust child widgets).
			//
			// changeSize: Object?
			//		Sets the widget to this margin-box size and position.
			//		May include any/all of the following properties:
			//	|	{w: int, h: int, l: int, t: int}
			//
			// resultSize: Object?
			//		The margin-box size of this widget after applying changeSize (if
			//		changeSize is specified).  If caller knows this size and
			//		passes it in, we don't need to query the browser to get the size.
			//	|	{w: int, h: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(changeSize){
				dojo.marginBox(node, changeSize);

				// set offset of the node
				if(changeSize.t){ node.style.top = changeSize.t + "px"; }
				if(changeSize.l){ node.style.left = changeSize.l + "px"; }
			}

			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = resultSize || {};
			dojo.mixin(mb, changeSize || {});	// changeSize overrides resultSize
			if( !("h" in mb) || !("w" in mb) ){
				mb = dojo.mixin(dojo.marginBox(node), mb);	// just use dojo.marginBox() to fill in missing values
			}

			// Compute and save the size of my border box and content box
			// (w/out calling dojo.contentBox() since that may fail if size was recently set)
			var cs = dojo.getComputedStyle(node);
			var me = dojo._getMarginExtents(node, cs);
			var be = dojo._getBorderExtents(node, cs);
			var bb = (this._borderBox = {
				w: mb.w - (me.w + be.w),
				h: mb.h - (me.h + be.h)
			});
			var pe = dojo._getPadExtents(node, cs);
			this._contentBox = {
				l: dojo._toPixelValue(node, cs.paddingLeft),
				t: dojo._toPixelValue(node, cs.paddingTop),
				w: bb.w - pe.w,
				h: bb.h - pe.h
			};

			// Callback for widget to adjust size of its children
			this.layout();
		},

		layout: function(){
			// summary:
			//		Widgets override this method to size and position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
			// tags:
			//		protected extension
		},

		_setupChild: function(/*dijit._Widget*/child){
			// summary:
			//		Common setup for initial children and children which are added after startup
			// tags:
			//		protected extension

			dojo.addClass(child.domNode, this.baseClass+"-child");
			if(child.baseClass){
				dojo.addClass(child.domNode, this.baseClass+"-"+child.baseClass);
			}
		},

		addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
			// Overrides _Container.addChild() to call _setupChild()
			this.inherited(arguments);
			if(this._started){
				this._setupChild(child);
			}
		},

		removeChild: function(/*dijit._Widget*/ child){
			// Overrides _Container.removeChild() to remove class added by _setupChild()
			dojo.removeClass(child.domNode, this.baseClass+"-child");
			if(child.baseClass){
				dojo.removeClass(child.domNode, this.baseClass+"-"+child.baseClass);
			}
			this.inherited(arguments);
		}
	}
);

dijit.layout.marginBox2contentBox = function(/*DomNode*/ node, /*Object*/ mb){
	// summary:
	//		Given the margin-box size of a node, return its content box size.
	//		Functions like dojo.contentBox() but is more reliable since it doesn't have
	//		to wait for the browser to compute sizes.
	var cs = dojo.getComputedStyle(node);
	var me = dojo._getMarginExtents(node, cs);
	var pb = dojo._getPadBorderExtents(node, cs);
	return {
		l: dojo._toPixelValue(node, cs.paddingLeft),
		t: dojo._toPixelValue(node, cs.paddingTop),
		w: mb.w - (me.w + pb.w),
		h: mb.h - (me.h + pb.h)
	};
};

(function(){
	var capitalize = function(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	};

	var size = function(widget, dim){
		// size the child
		widget.resize ? widget.resize(dim) : dojo.marginBox(widget.domNode, dim);

		// record child's size, but favor our own numbers when we have them.
		// the browser lies sometimes
		dojo.mixin(widget, dojo.marginBox(widget.domNode));
		dojo.mixin(widget, dim);
	};

	dijit.layout.layoutChildren = function(/*DomNode*/ container, /*Object*/ dim, /*Object[]*/ children){
		// summary
		//		Layout a bunch of child dom nodes within a parent dom node
		// container:
		//		parent node
		// dim:
		//		{l, t, w, h} object specifying dimensions of container into which to place children
		// children:
		//		an array like [ {domNode: foo, layoutAlign: "bottom" }, {domNode: bar, layoutAlign: "client"} ]

		// copy dim because we are going to modify it
		dim = dojo.mixin({}, dim);

		dojo.addClass(container, "dijitLayoutContainer");

		// Move "client" elements to the end of the array for layout.  a11y dictates that the author
		// needs to be able to put them in the document in tab-order, but this algorithm requires that
		// client be last.
		children = dojo.filter(children, function(item){ return item.layoutAlign != "client"; })
			.concat(dojo.filter(children, function(item){ return item.layoutAlign == "client"; }));

		// set positions/sizes
		dojo.forEach(children, function(child){
			var elm = child.domNode,
				pos = child.layoutAlign;

			// set elem to upper left corner of unused space; may move it later
			var elmStyle = elm.style;
			elmStyle.left = dim.l+"px";
			elmStyle.top = dim.t+"px";
			elmStyle.bottom = elmStyle.right = "auto";

			dojo.addClass(elm, "dijitAlign" + capitalize(pos));

			// set size && adjust record of remaining space.
			// note that setting the width of a <div> may affect its height.
			if(pos == "top" || pos == "bottom"){
				size(child, { w: dim.w });
				dim.h -= child.h;
				if(pos == "top"){
					dim.t += child.h;
				}else{
					elmStyle.top = dim.t + dim.h + "px";
				}
			}else if(pos == "left" || pos == "right"){
				size(child, { h: dim.h });
				dim.w -= child.w;
				if(pos == "left"){
					dim.l += child.w;
				}else{
					elmStyle.left = dim.l + dim.w + "px";
				}
			}else if(pos == "client"){
				size(child, dim);
			}
		});
	};

})();

}

if(!dojo._hasResource["dojo.html"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.html"] = true;
dojo.provide("dojo.html");

// the parser might be needed..
 

(function(){ // private scope, sort of a namespace

	// idCounter is incremented with each instantiation to allow asignment of a unique id for tracking, logging purposes
	var idCounter = 0, 
		d = dojo;
	
	dojo.html._secureForInnerHtml = function(/*String*/ cont){
		// summary:
		//		removes !DOCTYPE and title elements from the html string.
		// 
		//		khtml is picky about dom faults, you can't attach a style or <title> node as child of body
		//		must go into head, so we need to cut out those tags
		//	cont:
		//		An html string for insertion into the dom
		//	
		return cont.replace(/(?:\s*<!DOCTYPE\s[^>]+>|<title[^>]*>[\s\S]*?<\/title>)/ig, ""); // String
	};

/*====
	dojo.html._emptyNode = function(node){
		// summary:
		//		removes all child nodes from the given node
		//	node: DOMNode
		//		the parent element
	};
=====*/
	dojo.html._emptyNode = dojo.empty;

	dojo.html._setNodeContent = function(/* DomNode */ node, /* String|DomNode|NodeList */ cont){
		// summary:
		//		inserts the given content into the given node
		//	node:
		//		the parent element
		//	content:
		//		the content to be set on the parent element. 
		//		This can be an html string, a node reference or a NodeList, dojo.NodeList, Array or other enumerable list of nodes
		
		// always empty
		d.empty(node);

		if(cont) {
			if(typeof cont == "string") {
				cont = d._toDom(cont, node.ownerDocument);
			}
			if(!cont.nodeType && d.isArrayLike(cont)) {
				// handle as enumerable, but it may shrink as we enumerate it
				for(var startlen=cont.length, i=0; i<cont.length; i=startlen==cont.length ? i+1 : 0) {
					d.place( cont[i], node, "last");
				}
			} else {
				// pass nodes, documentFragments and unknowns through to dojo.place
				d.place(cont, node, "last");
			}
		}

		// return DomNode
		return node;
	};

	// we wrap up the content-setting operation in a object
	dojo.declare("dojo.html._ContentSetter", null, 
		{
			// node: DomNode|String
			//		An node which will be the parent element that we set content into
			node: "",

			// content: String|DomNode|DomNode[]
			//		The content to be placed in the node. Can be an HTML string, a node reference, or a enumerable list of nodes
			content: "",
			
			// id: String?
			//		Usually only used internally, and auto-generated with each instance 
			id: "",

			// cleanContent: Boolean
			//		Should the content be treated as a full html document, 
			//		and the real content stripped of <html>, <body> wrapper before injection
			cleanContent: false,
			
			// extractContent: Boolean
			//		Should the content be treated as a full html document, and the real content stripped of <html>, <body> wrapper before injection
			extractContent: false,

			// parseContent: Boolean
			//		Should the node by passed to the parser after the new content is set
			parseContent: false,
			
			// lifecyle methods
			constructor: function(/* Object */params, /* String|DomNode */node){
				//	summary:
				//		Provides a configurable, extensible object to wrap the setting on content on a node
				//		call the set() method to actually set the content..
 
				// the original params are mixed directly into the instance "this"
				dojo.mixin(this, params || {});

				// give precedence to params.node vs. the node argument
				// and ensure its a node, not an id string
				node = this.node = dojo.byId( this.node || node );
	
				if(!this.id){
					this.id = [
						"Setter",
						(node) ? node.id || node.tagName : "", 
						idCounter++
					].join("_");
				}
			},
			set: function(/* String|DomNode|NodeList? */ cont, /* Object? */ params){
				// summary:
				//		front-end to the set-content sequence 
				//	cont:
				//		An html string, node or enumerable list of nodes for insertion into the dom
				//		If not provided, the object's content property will be used
				if(undefined !== cont){
					this.content = cont;
				}
				// in the re-use scenario, set needs to be able to mixin new configuration
				if(params){
					this._mixin(params);
				}

				this.onBegin();
				this.setContent();
				this.onEnd();

				return this.node;
			},
			setContent: function(){
				// summary:
				//		sets the content on the node 

				var node = this.node; 
				if(!node) {
				    // can't proceed
					throw new Error(this.declaredClass + ": setContent given no node");
				}
				try{
					node = dojo.html._setNodeContent(node, this.content);
				}catch(e){
					// check if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV
	
					// FIXME: need to allow the user to provide a content error message string
					var errMess = this.onContentError(e); 
					try{
						node.innerHTML = errMess;
					}catch(e){
						console.error('Fatal ' + this.declaredClass + '.setContent could not change content due to '+e.message, e);
					}
				}
				// always put back the node for the next method
				this.node = node; // DomNode
			},
			
			empty: function() {
				// summary
				//	cleanly empty out existing content

				// destroy any widgets from a previous run
				// NOTE: if you dont want this you'll need to empty 
				// the parseResults array property yourself to avoid bad things happenning
				if(this.parseResults && this.parseResults.length) {
					dojo.forEach(this.parseResults, function(w) {
						if(w.destroy){
							w.destroy();
						}
					});
					delete this.parseResults;
				}
				// this is fast, but if you know its already empty or safe, you could 
				// override empty to skip this step
				dojo.html._emptyNode(this.node);
			},
	
			onBegin: function(){
				// summary
				//		Called after instantiation, but before set(); 
				//		It allows modification of any of the object properties 
				//		- including the node and content provided - before the set operation actually takes place
				//		This default implementation checks for cleanContent and extractContent flags to 
				//		optionally pre-process html string content
				var cont = this.content;
	
				if(dojo.isString(cont)){
					if(this.cleanContent){
						cont = dojo.html._secureForInnerHtml(cont);
					}
  
					if(this.extractContent){
						var match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
						if(match){ cont = match[1]; }
					}
				}

				// clean out the node and any cruft associated with it - like widgets
				this.empty();
				
				this.content = cont;
				return this.node; /* DomNode */
			},
	
			onEnd: function(){
				// summary
				//		Called after set(), when the new content has been pushed into the node
				//		It provides an opportunity for post-processing before handing back the node to the caller
				//		This default implementation checks a parseContent flag to optionally run the dojo parser over the new content
				if(this.parseContent){
					// populates this.parseResults if you need those..
					this._parse();
				}
				return this.node; /* DomNode */
			},
	
			tearDown: function(){
				// summary
				//		manually reset the Setter instance if its being re-used for example for another set()
				// description
				//		tearDown() is not called automatically. 
				//		In normal use, the Setter instance properties are simply allowed to fall out of scope
				//		but the tearDown method can be called to explicitly reset this instance.
				delete this.parseResults; 
				delete this.node; 
				delete this.content; 
			},
  
			onContentError: function(err){
				return "Error occured setting content: " + err; 
			},
			
			_mixin: function(params){
				// mix properties/methods into the instance
				// TODO: the intention with tearDown is to put the Setter's state 
				// back to that of the original constructor (vs. deleting/resetting everything regardless of ctor params)
				// so we could do something here to move the original properties aside for later restoration
				var empty = {}, key;
				for(key in params){
					if(key in empty){ continue; }
					// TODO: here's our opportunity to mask the properties we dont consider configurable/overridable
					// .. but history shows we'll almost always guess wrong
					this[key] = params[key]; 
				}
			},
			_parse: function(){
				// summary: 
				//		runs the dojo parser over the node contents, storing any results in this.parseResults
				//		Any errors resulting from parsing are passed to _onError for handling

				var rootNode = this.node;
				try{
					// store the results (widgets, whatever) for potential retrieval
					this.parseResults = dojo.parser.parse({
						rootNode: rootNode,
						dir: this.dir,
						lang: this.lang
					});
				}catch(e){
					this._onError('Content', e, "Error parsing in _ContentSetter#"+this.id);
				}
			},
  
			_onError: function(type, err, consoleText){
				// summary:
				//		shows user the string that is returned by on[type]Error
				//		overide/implement on[type]Error and return your own string to customize
				var errText = this['on' + type + 'Error'].call(this, err);
				if(consoleText){
					console.error(consoleText, err);
				}else if(errText){ // a empty string won't change current content
					dojo.html._setNodeContent(this.node, errText, true);
				}
			}
	}); // end dojo.declare()

	dojo.html.set = function(/* DomNode */ node, /* String|DomNode|NodeList */ cont, /* Object? */ params){
			// summary:
			//		inserts (replaces) the given content into the given node. dojo.place(cont, node, "only")
			//		may be a better choice for simple HTML insertion.
			// description:
			//		Unless you need to use the params capabilities of this method, you should use
			//		dojo.place(cont, node, "only"). dojo.place() has more robust support for injecting
			//		an HTML string into the DOM, but it only handles inserting an HTML string as DOM
			//		elements, or inserting a DOM node. dojo.place does not handle NodeList insertions
			//		or the other capabilities as defined by the params object for this method.
			//	node:
			//		the parent element that will receive the content
			//	cont:
			//		the content to be set on the parent element. 
			//		This can be an html string, a node reference or a NodeList, dojo.NodeList, Array or other enumerable list of nodes
			//	params: 
			//		Optional flags/properties to configure the content-setting. See dojo.html._ContentSetter
			//	example:
			//		A safe string/node/nodelist content replacement/injection with hooks for extension
			//		Example Usage: 
			//		dojo.html.set(node, "some string"); 
			//		dojo.html.set(node, contentNode, {options}); 
			//		dojo.html.set(node, myNode.childNodes, {options}); 
		if(undefined == cont){
			console.warn("dojo.html.set: no cont argument provided, using empty string");
			cont = "";
		}	
		if(!params){
			// simple and fast
			return dojo.html._setNodeContent(node, cont, true);
		}else{ 
			// more options but slower
			// note the arguments are reversed in order, to match the convention for instantiation via the parser
			var op = new dojo.html._ContentSetter(dojo.mixin( 
					params, 
					{ content: cont, node: node } 
			));
			return op.set();
		}
	};
})();

}

if(!dojo._hasResource["dijit.layout.ContentPane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.ContentPane"] = true;
dojo.provide("dijit.layout.ContentPane");



	// for dijit.layout.marginBox2contentBox()






dojo.declare(
	"dijit.layout.ContentPane", dijit._Widget,
{
	// summary:
	//		A widget that acts as a container for mixed HTML and widgets, and includes an Ajax interface
	// description:
	//		A widget that can be used as a stand alone widget
	//		or as a base class for other widgets.
	//
	//		Handles replacement of document fragment using either external uri or javascript
	//		generated markup or DOM content, instantiating widgets within that content.
	//		Don't confuse it with an iframe, it only needs/wants document fragments.
	//		It's useful as a child of LayoutContainer, SplitContainer, or TabContainer.
	//		But note that those classes can contain any widget as a child.
	// example:
	//		Some quick samples:
	//		To change the innerHTML use .set('content', '<b>new content</b>')
	//
	//		Or you can send it a NodeList, .set('content', dojo.query('div [class=selected]', userSelection))
	//		please note that the nodes in NodeList will copied, not moved
	//
	//		To do a ajax update use .set('href', url)

	// href: String
	//		The href of the content that displays now.
	//		Set this at construction if you want to load data externally when the
	//		pane is shown.  (Set preload=true to load it immediately.)
	//		Changing href after creation doesn't have any effect; Use set('href', ...);
	href: "",

/*=====
	// content: String || DomNode || NodeList || dijit._Widget
	//		The innerHTML of the ContentPane.
	//		Note that the initialization parameter / argument to attr("content", ...)
	//		can be a String, DomNode, Nodelist, or _Widget.
	content: "",
=====*/

	// extractContent: Boolean
	//		Extract visible content from inside of <body> .... </body>.
	//		I.e., strip <html> and <head> (and it's contents) from the href
	extractContent: false,

	// parseOnLoad: Boolean
	//		Parse content and create the widgets, if any.
	parseOnLoad: true,

	// preventCache: Boolean
	//		Prevent caching of data from href's by appending a timestamp to the href.
	preventCache: false,

	// preload: Boolean
	//		Force load of data on initialization even if pane is hidden.
	preload: false,

	// refreshOnShow: Boolean
	//		Refresh (re-download) content when pane goes from hidden to shown
	refreshOnShow: false,

	// loadingMessage: String
	//		Message that shows while downloading
	loadingMessage: "<span class='dijitContentPaneLoading'>${loadingState}</span>",

	// errorMessage: String
	//		Message that shows if an error occurs
	errorMessage: "<span class='dijitContentPaneError'>${errorState}</span>",

	// isLoaded: [readonly] Boolean
	//		True if the ContentPane has data in it, either specified
	//		during initialization (via href or inline content), or set
	//		via attr('content', ...) / attr('href', ...)
	//
	//		False if it doesn't have any content, or if ContentPane is
	//		still in the process of downloading href.
	isLoaded: false,

	baseClass: "dijitContentPane",

	// doLayout: Boolean
	//		- false - don't adjust size of children
	//		- true - if there is a single visible child widget, set it's size to
	//				however big the ContentPane is
	doLayout: true,

	// ioArgs: Object
	//		Parameters to pass to xhrGet() request, for example:
	// |	<div dojoType="dijit.layout.ContentPane" href="./bar" ioArgs="{timeout: 500}">
	ioArgs: {},

	// isContainer: [protected] Boolean
	//		Indicates that this widget acts as a "parent" to the descendant widgets.
	//		When the parent is started it will call startup() on the child widgets.
	//		See also `isLayoutContainer`.
	isContainer: true,

	// isLayoutContainer: [protected] Boolean
	//		Indicates that this widget will call resize() on it's child widgets
	//		when they become visible.
	isLayoutContainer: true,

	// onLoadDeferred: [readonly] dojo.Deferred
	//		This is the `dojo.Deferred` returned by attr('href', ...) and refresh().
	//		Calling onLoadDeferred.addCallback() or addErrback() registers your
	//		callback to be called only once, when the prior attr('href', ...) call or
	//		the initial href parameter to the constructor finishes loading.
	//
	//		This is different than an onLoad() handler which gets called any time any href is loaded.
	onLoadDeferred: null,

	// Override _Widget's attributeMap because we don't want the title attribute (used to specify
	// tab labels) to be copied to ContentPane.domNode... otherwise a tooltip shows up over the
	// entire pane.
	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
		title: []
	}),

	postMixInProperties: function(){
		this.inherited(arguments);
		var messages = dojo.i18n.getLocalization("dijit", "loading", this.lang);
		this.loadingMessage = dojo.string.substitute(this.loadingMessage, messages);
		this.errorMessage = dojo.string.substitute(this.errorMessage, messages);

		// Detect if we were initialized with data
		if(!this.href && this.srcNodeRef && this.srcNodeRef.innerHTML){
			this.isLoaded = true;
		}
	},

	buildRendering: function(){
		// Overrides Widget.buildRendering().
		// Since we have no template we need to set this.containerNode ourselves.
		// For subclasses of ContentPane do have a template, does nothing.
		this.inherited(arguments);
		if(!this.containerNode){
			// make getDescendants() work
			this.containerNode = this.domNode;
		}
	},

	postCreate: function(){
		// remove the title attribute so it doesn't show up when hovering
		// over a node
		this.domNode.title = "";

		if(!dojo.attr(this.domNode,"role")){
			dijit.setWaiRole(this.domNode, "group");
		}

		dojo.addClass(this.domNode, this.baseClass);
	},

	startup: function(){
		// summary:
		//		See `dijit.layout._LayoutWidget.startup` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.
		if(this._started){ return; }

		var parent = dijit._Contained.prototype.getParent.call(this);
		this._childOfLayoutWidget = parent && parent.isLayoutContainer;

		// I need to call resize() on my child/children (when I become visible), unless
		// I'm the child of a layout widget in which case my parent will call resize() on me and I'll do it then.
		this._needLayout = !this._childOfLayoutWidget;

		if(this.isLoaded){
			dojo.forEach(this.getChildren(), function(child){
				child.startup();
			});
		}

		if(this._isShown() || this.preload){
			this._onShow();
		}

		this.inherited(arguments);
	},

	_checkIfSingleChild: function(){
		// summary:
		//		Test if we have exactly one visible widget as a child,
		//		and if so assume that we are a container for that widget,
		//		and should propogate startup() and resize() calls to it.
		//		Skips over things like data stores since they aren't visible.

		var childNodes = dojo.query("> *", this.containerNode).filter(function(node){
				return node.tagName !== "SCRIPT"; // or a regexp for hidden elements like script|area|map|etc..
			}),
			childWidgetNodes = childNodes.filter(function(node){
				return dojo.hasAttr(node, "dojoType") || dojo.hasAttr(node, "widgetId");
			}),
			candidateWidgets = dojo.filter(childWidgetNodes.map(dijit.byNode), function(widget){
				return widget && widget.domNode && widget.resize;
			});

		if(
			// all child nodes are widgets
			childNodes.length == childWidgetNodes.length &&

			// all but one are invisible (like dojo.data)
			candidateWidgets.length == 1
		){
			this._singleChild = candidateWidgets[0];
		}else{
			delete this._singleChild;
		}

		// So we can set overflow: hidden to avoid a safari bug w/scrollbars showing up (#9449)
		dojo.toggleClass(this.containerNode, this.baseClass + "SingleChild", !!this._singleChild);
	},

	setHref: function(/*String|Uri*/ href){
		// summary:
		//		Deprecated.   Use set('href', ...) instead.
		dojo.deprecated("dijit.layout.ContentPane.setHref() is deprecated. Use set('href', ...) instead.", "", "2.0");
		return this.set("href", href);
	},
	_setHrefAttr: function(/*String|Uri*/ href){
		// summary:
		//		Hook so attr("href", ...) works.
		// description:
		//		Reset the (external defined) content of this pane and replace with new url
		//		Note: It delays the download until widget is shown if preload is false.
		//	href:
		//		url to the page you want to get, must be within the same domain as your mainpage

		// Cancel any in-flight requests (an attr('href') will cancel any in-flight attr('href', ...))
		this.cancel();

		this.onLoadDeferred = new dojo.Deferred(dojo.hitch(this, "cancel"));

		this.href = href;

		// _setHrefAttr() is called during creation and by the user, after creation.
		// only in the second case do we actually load the URL; otherwise it's done in startup()
		if(this._created && (this.preload || this._isShown())){
			this._load();
		}else{
			// Set flag to indicate that href needs to be loaded the next time the
			// ContentPane is made visible
			this._hrefChanged = true;
		}

		return this.onLoadDeferred;		// dojo.Deferred
	},

	setContent: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Deprecated.   Use set('content', ...) instead.
		dojo.deprecated("dijit.layout.ContentPane.setContent() is deprecated.  Use set('content', ...) instead.", "", "2.0");
		this.set("content", data);
	},
	_setContentAttr: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Hook to make attr("content", ...) work.
		//		Replaces old content with data content, include style classes from old content
		//	data:
		//		the new Content may be String, DomNode or NodeList
		//
		//		if data is a NodeList (or an array of nodes) nodes are copied
		//		so you can import nodes from another document implicitly

		// clear href so we can't run refresh and clear content
		// refresh should only work if we downloaded the content
		this.href = "";

		// Cancel any in-flight requests (an attr('content') will cancel any in-flight attr('href', ...))
		this.cancel();

		// Even though user is just setting content directly, still need to define an onLoadDeferred
		// because the _onLoadHandler() handler is still getting called from setContent()
		this.onLoadDeferred = new dojo.Deferred(dojo.hitch(this, "cancel"));

		this._setContent(data || "");

		this._isDownloaded = false; // mark that content is from a attr('content') not an attr('href')

		return this.onLoadDeferred; 	// dojo.Deferred
	},
	_getContentAttr: function(){
		// summary:
		//		Hook to make attr("content") work
		return this.containerNode.innerHTML;
	},

	cancel: function(){
		// summary:
		//		Cancels an in-flight download of content
		if(this._xhrDfd && (this._xhrDfd.fired == -1)){
			this._xhrDfd.cancel();
		}
		delete this._xhrDfd; // garbage collect

		this.onLoadDeferred = null;
	},

	uninitialize: function(){
		if(this._beingDestroyed){
			this.cancel();
		}
		this.inherited(arguments);
	},

	destroyRecursive: function(/*Boolean*/ preserveDom){
		// summary:
		//		Destroy the ContentPane and its contents

		// if we have multiple controllers destroying us, bail after the first
		if(this._beingDestroyed){
			return;
		}
		this.inherited(arguments);
	},

	resize: function(changeSize, resultSize){
		// summary:
		//		See `dijit.layout._LayoutWidget.resize` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.

		// For the TabContainer --> BorderContainer --> ContentPane case, _onShow() is
		// never called, so resize() is our trigger to do the initial href download.
		if(!this._wasShown){
			this._onShow();
		}

		this._resizeCalled = true;

		// Set margin box size, unless it wasn't specified, in which case use current size.
		if(changeSize){
			dojo.marginBox(this.domNode, changeSize);
		}

		// Compute content box size of containerNode in case we [later] need to size our single child.
		var cn = this.containerNode;
		if(cn === this.domNode){
			// If changeSize or resultSize was passed to this method and this.containerNode ==
			// this.domNode then we can compute the content-box size without querying the node,
			// which is more reliable (similar to LayoutWidget.resize) (see for example #9449).
			var mb = resultSize || {};
			dojo.mixin(mb, changeSize || {}); // changeSize overrides resultSize
			if(!("h" in mb) || !("w" in mb)){
				mb = dojo.mixin(dojo.marginBox(cn), mb); // just use dojo.marginBox() to fill in missing values
			}
			this._contentBox = dijit.layout.marginBox2contentBox(cn, mb);
		}else{
			this._contentBox = dojo.contentBox(cn);
		}

		// Make my children layout, or size my single child widget
		this._layoutChildren();
	},

	_isShown: function(){
		// summary:
		//		Returns true if the content is currently shown.
		// description:
		//		If I am a child of a layout widget then it actually returns true if I've ever been visible,
		//		not whether I'm currently visible, since that's much faster than tracing up the DOM/widget
		//		tree every call, and at least solves the performance problem on page load by deferring loading
		//		hidden ContentPanes until they are first shown

		if(this._childOfLayoutWidget){
			// If we are TitlePane, etc - we return that only *IF* we've been resized
			if(this._resizeCalled && "open" in this){
				return this.open;
			}
			return this._resizeCalled;
		}else if("open" in this){
			return this.open;		// for TitlePane, etc.
		}else{
			// TODO: with _childOfLayoutWidget check maybe this branch no longer necessary?
			var node = this.domNode;
			return (node.style.display != 'none') && (node.style.visibility != 'hidden') && !dojo.hasClass(node, "dijitHidden");
		}
	},

	_onShow: function(){
		// summary:
		//		Called when the ContentPane is made visible
		// description:
		//		For a plain ContentPane, this is called on initialization, from startup().
		//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
		//		called whenever the pane is made visible.
		//
		//		Does necessary processing, including href download and layout/resize of
		//		child widget(s)

		if(this.href){
			if(!this._xhrDfd && // if there's an href that isn't already being loaded
				(!this.isLoaded || this._hrefChanged || this.refreshOnShow)
			){
				this.refresh();
			}
		}else{
			// If we are the child of a layout widget then the layout widget will call resize() on
			// us, and then we will size our child/children.   Otherwise, we need to do it now.
			if(!this._childOfLayoutWidget && this._needLayout){
				// If a layout has been scheduled for when we become visible, do it now
				this._layoutChildren();
			}
		}

		this.inherited(arguments);

		// Need to keep track of whether ContentPane has been shown (which is different than
		// whether or not it's currently visible).
		this._wasShown = true;
	},

	refresh: function(){
		// summary:
		//		[Re]download contents of href and display
		// description:
		//		1. cancels any currently in-flight requests
		//		2. posts "loading..." message
		//		3. sends XHR to download new data

		// Cancel possible prior in-flight request
		this.cancel();

		this.onLoadDeferred = new dojo.Deferred(dojo.hitch(this, "cancel"));
		this._load();
		return this.onLoadDeferred;
	},

	_load: function(){
		// summary:
		//		Load/reload the href specified in this.href

		// display loading message
		this._setContent(this.onDownloadStart(), true);

		var self = this;
		var getArgs = {
			preventCache: (this.preventCache || this.refreshOnShow),
			url: this.href,
			handleAs: "text"
		};
		if(dojo.isObject(this.ioArgs)){
			dojo.mixin(getArgs, this.ioArgs);
		}

		var hand = (this._xhrDfd = (this.ioMethod || dojo.xhrGet)(getArgs));

		hand.addCallback(function(html){
			try{
				self._isDownloaded = true;
				self._setContent(html, false);
				self.onDownloadEnd();
			}catch(err){
				self._onError('Content', err); // onContentError
			}
			delete self._xhrDfd;
			return html;
		});

		hand.addErrback(function(err){
			if(!hand.canceled){
				// show error message in the pane
				self._onError('Download', err); // onDownloadError
			}
			delete self._xhrDfd;
			return err;
		});

		// Remove flag saying that a load is needed
		delete this._hrefChanged;
	},

	_onLoadHandler: function(data){
		// summary:
		//		This is called whenever new content is being loaded
		this.isLoaded = true;
		try{
			this.onLoadDeferred.callback(data);
			this.onLoad(data);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onLoad code: ' + e.message);
		}
	},

	_onUnloadHandler: function(){
		// summary:
		//		This is called whenever the content is being unloaded
		this.isLoaded = false;
		try{
			this.onUnload();
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onUnload code: ' + e.message);
		}
	},

	destroyDescendants: function(){
		// summary:
		//		Destroy all the widgets inside the ContentPane and empty containerNode

		// Make sure we call onUnload (but only when the ContentPane has real content)
		if(this.isLoaded){
			this._onUnloadHandler();
		}

		// Even if this.isLoaded == false there might still be a "Loading..." message
		// to erase, so continue...

		// For historical reasons we need to delete all widgets under this.containerNode,
		// even ones that the user has created manually.
		var setter = this._contentSetter;
		dojo.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive();
			}
		});
		if(setter){
			// Most of the widgets in setter.parseResults have already been destroyed, but
			// things like Menu that have been moved to <body> haven't yet
			dojo.forEach(setter.parseResults, function(widget){
				if(widget.destroyRecursive && widget.domNode && widget.domNode.parentNode == dojo.body()){
					widget.destroyRecursive();
				}
			});
			delete setter.parseResults;
		}

		// And then clear away all the DOM nodes
		dojo.html._emptyNode(this.containerNode);

		// Delete any state information we have about current contents
		delete this._singleChild;
	},

	_setContent: function(cont, isFakeContent){
		// summary:
		//		Insert the content into the container node

		// first get rid of child widgets
		this.destroyDescendants();

		// dojo.html.set will take care of the rest of the details
		// we provide an override for the error handling to ensure the widget gets the errors
		// configure the setter instance with only the relevant widget instance properties
		// NOTE: unless we hook into attr, or provide property setters for each property,
		// we need to re-configure the ContentSetter with each use
		var setter = this._contentSetter;
		if(! (setter && setter instanceof dojo.html._ContentSetter)){
			setter = this._contentSetter = new dojo.html._ContentSetter({
				node: this.containerNode,
				_onError: dojo.hitch(this, this._onError),
				onContentError: dojo.hitch(this, function(e){
					// fires if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV
					var errMess = this.onContentError(e);
					try{
						this.containerNode.innerHTML = errMess;
					}catch(e){
						console.error('Fatal '+this.id+' could not change content due to '+e.message, e);
					}
				})/*,
				_onError */
			});
		};

		var setterParams = dojo.mixin({
			cleanContent: this.cleanContent,
			extractContent: this.extractContent,
			parseContent: this.parseOnLoad,
			dir: this.dir,
			lang: this.lang
		}, this._contentSetterParams || {});

		dojo.mixin(setter, setterParams);

		setter.set( (dojo.isObject(cont) && cont.domNode) ? cont.domNode : cont );

		// setter params must be pulled afresh from the ContentPane each time
		delete this._contentSetterParams;

		if(!isFakeContent){
			// Startup each top level child widget (and they will start their children, recursively)
			dojo.forEach(this.getChildren(), function(child){
				// The parser has already called startup on all widgets *without* a getParent() method
				if(!this.parseOnLoad || child.getParent){
					child.startup();
				}
			}, this);

			// Call resize() on each of my child layout widgets,
			// or resize() on my single child layout widget...
			// either now (if I'm currently visible)
			// or when I become visible
			this._scheduleLayout();

			this._onLoadHandler(cont);
		}
	},

	_onError: function(type, err, consoleText){
		this.onLoadDeferred.errback(err);

		// shows user the string that is returned by on[type]Error
		// overide on[type]Error and return your own string to customize
		var errText = this['on' + type + 'Error'].call(this, err);
		if(consoleText){
			console.error(consoleText, err);
		}else if(errText){// a empty string won't change current content
			this._setContent(errText, true);
		}
	},

	_scheduleLayout: function(){
		// summary:
		//		Call resize() on each of my child layout widgets, either now
		//		(if I'm currently visible) or when I become visible
		if(this._isShown()){
			this._layoutChildren();
		}else{
			this._needLayout = true;
		}
	},

	_layoutChildren: function(){
		// summary:
		//		Since I am a Container widget, each of my children expects me to
		//		call resize() or layout() on them.
		// description:
		//		Should be called on initialization and also whenever we get new content
		//		(from an href, or from attr('content', ...))... but deferred until
		//		the ContentPane is visible

		if(this.doLayout){
			this._checkIfSingleChild();
		}

		if(this._singleChild && this._singleChild.resize){
			var cb = this._contentBox || dojo.contentBox(this.containerNode);

			// note: if widget has padding this._contentBox will have l and t set,
			// but don't pass them to resize() or it will doubly-offset the child
			this._singleChild.resize({w: cb.w, h: cb.h});
		}else{
			// All my child widgets are independently sized (rather than matching my size),
			// but I still need to call resize() on each child to make it layout.
			dojo.forEach(this.getChildren(), function(widget){
				if(widget.resize){
					widget.resize();
				}
			});
		}
		delete this._needLayout;
	},

	// EVENT's, should be overide-able
	onLoad: function(data){
		// summary:
		//		Event hook, is called after everything is loaded and widgetified
		// tags:
		//		callback
	},

	onUnload: function(){
		// summary:
		//		Event hook, is called before old content is cleared
		// tags:
		//		callback
	},

	onDownloadStart: function(){
		// summary:
		//		Called before download starts.
		// description:
		//		The string returned by this function will be the html
		//		that tells the user we are loading something.
		//		Override with your own function if you want to change text.
		// tags:
		//		extension
		return this.loadingMessage;
	},

	onContentError: function(/*Error*/ error){
		// summary:
		//		Called on DOM faults, require faults etc. in content.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		By default (if this method is not overriden), it returns
		//		nothing, so the error message is just printed to the console.
		// tags:
		//		extension
	},

	onDownloadError: function(/*Error*/ error){
		// summary:
		//		Called when download error occurs.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		Default behavior (if this method is not overriden) is to display
		//		the error message inside the pane.
		// tags:
		//		extension
		return this.errorMessage;
	},

	onDownloadEnd: function(){
		// summary:
		//		Called when download is finished.
		// tags:
		//		callback
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.CloseabePanel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.CloseabePanel"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/
dojo.require( "dojo.fx" );
dojo.require( "dijit._Templated" );
dojo.require( "dijit.layout.ContentPane" );

dojo.provide( "ibm.tivoli.tip.dijit.CloseabePanel" );
/******************************************************************************
* Colorful description area for Portlet Descriptions per Tivoli Guidlines
*
* Use innerDIV attachpoint to add graphics to the the description area
* background.
*
* Can be opened and closed.
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.CloseabePanel", [dijit.layout.ContentPane, dijit._Templated],
{
  portletTitle: "Portlet Title Here",
  
  description: "Portlet description here...",

  imageSrc: null,
  
  backgroundClass: null,

  templateString: "<div class='tipDescriptionArea open'><div class='tipDescriptionAreaClose' dojoAttachPoint='closeNode' dojoAttachEvent=\"onclick:_toggle\"></div><div dojoAttachPoint='innerDIV'><div dojoAttachPoint='hideNode'><div class='dijitReset' dojoAttachPoint='wipeNode'><div dojoAttachPoint='containerNode'><table border='0' cellpadding='0' cellspacing='0' width='100%'><tr><td rowspan='2' dojoAttachPoint='imageNode'></td><td width='100%' class='tipDescriptionAreaTitle'>${portletTitle}</td></tr><tr><td  width='100%' valign='top' class='tipDescriptionAreaContent'>${description}</td></tr></table></div></div></div></div></div>",
 // templateString:" <div class='tipDescriptionArea open'>\r\n    <div class='tipDescriptionAreaClose' dojoAttachPoint='closeNode' dojoAttachEvent=\\\"onclick:_toggle\\\"></div>\r\n      <div dojoAttachPoint='innerDIV'>\r\n\t     <div dojoAttachPoint='hideNode'>\r\n\t\t        <div class='dijitReset' dojoAttachPoint='wipeNode'>\r\n\t\t\t\t    <div dojoAttachPoint='containerNode'>\r\n\t\t\t\t\t    <table border='0' cellpadding='0' cellspacing='0' width='100%'>\r\n\t\t\t\t\t\t     <tr>\r\n\t\t\t\t\t\t\t   <td rowspan='2' dojoAttachPoint='imageNode'></td>\r\n\t\t\t\t\t\t\t   <td width='100%' class='tipDescriptionAreaTitle'>${portletTitle}</td></tr>\r\n\t\t\t\t\t\t\t   <tr> \r\n\t\t\t\t\t\t\t     <td  width='100%' valign='top' class='tipDescriptionAreaContent'>${description}</td>\r\n\t\t\t\t\t\t\t\t </tr>\r\n\t\t\t\t\t\t </table>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\t\t\t</div>\r\n\t\t</div>\r\n\t\r\n\t</div>\r\n",
  // open: Boolean
  //    Whether description is opened or closed on first render.
  open: true,

  // duration: Integer
  //    Time in milliseconds to wipe in/wipe out
  duration: 500,
  
  /*****************************************************************************
  * Setup initial styles and state of the widget
  * Create and save all animations
  *****************************************************************************/
  postCreate: function()
  {
    
    if( !this.open )
    {
      this.hideNode.style.display = this.wipeNode.style.display = "none";
      this._updateClass();    
    }
    console.log("postCreate closeablePanel");
    this.inherited("postCreate",arguments);
    
    if ( this.imageSrc !== null )
    {
      var image = dojo.doc.createElement( 'img' );
      image.src = this.imageSrc;
      image.border = "0";
      this.imageNode.appendChild( image );
    }

    if ( this.backgroundClass !== null )
    {
      this.innerDIV.className = this.backgroundClass;
    }
  
    // setup open/close animations
    var hideNode = this.hideNode, 
        wipeNode = this.wipeNode;
        
    this._wipeIn = dojo.fx.wipeIn({
      node: this.wipeNode,
      duration: this.duration,
      beforeBegin: function(){
        hideNode.style.display="";
      }
    });
    
    this._wipeOut = dojo.fx.wipeOut({
      node: this.wipeNode,
      duration: this.duration,
      onEnd: function(){
        hideNode.style.display="none";
      }
    });
  },
  
  /*****************************************************************************
  * The toggle open/close button div was clicked -- toggle open/close state
  *****************************************************************************/
  _toggle: function()
  {
    dojo.forEach([this._wipeIn, this._wipeOut], function(animation){
      if(animation.status() == "playing"){
        animation.stop();
      }
    });

    this[this.open ? "_wipeOut" : "_wipeIn"].play();
    this.open =! this.open;    
    this._updateClass();    
  },
  
  /*****************************************************************************
  * based on the open/close state of widget change CSS to update button
  *****************************************************************************/
  _updateClass: function()
  {
    if ( this.open === true )
    {
      dojo.removeClass( this.domNode, "close"  );
         dojo.addClass( this.domNode, "open" );          
    }
    else
    {
      dojo.removeClass( this.domNode, "open" );
         dojo.addClass( this.domNode, "close"  );          
    }  
  }  
});

}

if(!dojo._hasResource["dijit.MenuItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.MenuItem"] = true;
dojo.provide("dijit.MenuItem");






dojo.declare("dijit.MenuItem",
		[dijit._Widget, dijit._Templated, dijit._Contained, dijit._CssStateMixin],
		{
		// summary:
		//		A line item in a Menu Widget

		// Make 3 columns
		// icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
		templateString: dojo.cache("dijit", "templates/MenuItem.html", "<tr class=\"dijitReset dijitMenuItem\" dojoAttachPoint=\"focusNode\" waiRole=\"menuitem\" tabIndex=\"-1\"\r\n\t\tdojoAttachEvent=\"onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick\">\r\n\t<td class=\"dijitReset dijitMenuItemIconCell\" waiRole=\"presentation\">\r\n\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitIcon dijitMenuItemIcon\" dojoAttachPoint=\"iconNode\"/>\r\n\t</td>\r\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" dojoAttachPoint=\"containerNode\"></td>\r\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" dojoAttachPoint=\"accelKeyNode\"></td>\r\n\t<td class=\"dijitReset dijitMenuArrowCell\" waiRole=\"presentation\">\r\n\t\t<div dojoAttachPoint=\"arrowWrapper\" style=\"visibility: hidden\">\r\n\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuExpand\"/>\r\n\t\t\t<span class=\"dijitMenuExpandA11y\">+</span>\r\n\t\t</div>\r\n\t</td>\r\n</tr>\r\n"),

		attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
			label: { node: "containerNode", type: "innerHTML" },
			iconClass: { node: "iconNode", type: "class" }
		}),

		baseClass: "dijitMenuItem",

		// label: String
		//		Menu text
		label: '',

		// iconClass: String
		//		Class to apply to DOMNode to make it display an icon.
		iconClass: "",

		// accelKey: String
		//		Text for the accelerator (shortcut) key combination.
		//		Note that although Menu can display accelerator keys there
		//		is no infrastructure to actually catch and execute these
		//		accelerators.
		accelKey: "",

		// disabled: Boolean
		//		If true, the menu item is disabled.
		//		If false, the menu item is enabled.
		disabled: false,

		_fillContent: function(/*DomNode*/ source){
			// If button label is specified as srcNodeRef.innerHTML rather than
			// this.params.label, handle it here.
			if(source && !("label" in this.params)){
				this.set('label', source.innerHTML);
			}
		},

		postCreate: function(){
			this.inherited(arguments);
			dojo.setSelectable(this.domNode, false);
			var label = this.id+"_text";
			dojo.attr(this.containerNode, "id", label);
			if(this.accelKeyNode){
				dojo.attr(this.accelKeyNode, "id", this.id + "_accel");
				label += " " + this.id + "_accel";
			}
			dijit.setWaiState(this.domNode, "labelledby", label);
		},

		_onHover: function(){
			// summary:
			//		Handler when mouse is moved onto menu item
			// tags:
			//		protected
			this.getParent().onItemHover(this);
		},

		_onUnhover: function(){
			// summary:
			//		Handler when mouse is moved off of menu item,
			//		possibly to a child menu, or maybe to a sibling
			//		menuitem or somewhere else entirely.
			// tags:
			//		protected

			// if we are unhovering the currently selected item
			// then unselect it
			this.getParent().onItemUnhover(this);

			// _onUnhover() is called when the menu is hidden (collapsed), due to clicking
			// a MenuItem and having it execut.  When that happens, FF and IE don't generate
			// an onmouseout event for the MenuItem, so give _CssStateMixin some help
			this._hovering = false;
			this._setStateClass();
		},

		_onClick: function(evt){
			// summary:
			//		Internal handler for click events on MenuItem.
			// tags:
			//		private
			this.getParent().onItemClick(this, evt);
			dojo.stopEvent(evt);
		},

		onClick: function(/*Event*/ evt){
			// summary:
			//		User defined function to handle clicks
			// tags:
			//		callback
		},

		focus: function(){
			// summary:
			//		Focus on this MenuItem
			try{
				if(dojo.isIE == 8){
					// needed for IE8 which won't scroll TR tags into view on focus yet calling scrollIntoView creates flicker (#10275)
					this.containerNode.focus();
				}
				dijit.focus(this.focusNode);
			}catch(e){
				// this throws on IE (at least) in some scenarios
			}
		},

		_onFocus: function(){
			// summary:
			//		This is called by the focus manager when focus
			//		goes to this MenuItem or a child menu.
			// tags:
			//		protected
			this._setSelected(true);
			this.getParent()._onItemFocus(this);

			this.inherited(arguments);
		},

		_setSelected: function(selected){
			// summary:
			//		Indicate that this node is the currently selected one
			// tags:
			//		private

			/***
			 * TODO: remove this method and calls to it, when _onBlur() is working for MenuItem.
			 * Currently _onBlur() gets called when focus is moved from the MenuItem to a child menu.
			 * That's not supposed to happen, but the problem is:
			 * In order to allow dijit.popup's getTopPopup() to work,a sub menu's popupParent
			 * points to the parent Menu, bypassing the parent MenuItem... thus the
			 * MenuItem is not in the chain of active widgets and gets a premature call to
			 * _onBlur()
			 */

			dojo.toggleClass(this.domNode, "dijitMenuItemSelected", selected);
		},

		setLabel: function(/*String*/ content){
			// summary:
			//		Deprecated.   Use set('label', ...) instead.
			// tags:
			//		deprecated
			dojo.deprecated("dijit.MenuItem.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
			this.set("label", content);
		},

		setDisabled: function(/*Boolean*/ disabled){
			// summary:
			//		Deprecated.   Use set('disabled', bool) instead.
			// tags:
			//		deprecated
			dojo.deprecated("dijit.Menu.setDisabled() is deprecated.  Use set('disabled', bool) instead.", "", "2.0");
			this.set('disabled', disabled);
		},
		_setDisabledAttr: function(/*Boolean*/ value){
			// summary:
			//		Hook for attr('disabled', ...) to work.
			//		Enable or disable this menu item.
			this.disabled = value;
			dijit.setWaiState(this.focusNode, 'disabled', value ? 'true' : 'false');
		},
		_setAccelKeyAttr: function(/*String*/ value){
			// summary:
			//		Hook for attr('accelKey', ...) to work.
			//		Set accelKey on this menu item.
			this.accelKey=value;

			this.accelKeyNode.style.display=value?"":"none";
			this.accelKeyNode.innerHTML=value;
			//have to use colSpan to make it work in IE
			dojo.attr(this.containerNode,'colSpan',value?"1":"2");
		}
	});

}

if(!dojo._hasResource["dijit.PopupMenuItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.PopupMenuItem"] = true;
dojo.provide("dijit.PopupMenuItem");



dojo.declare("dijit.PopupMenuItem",
		dijit.MenuItem,
		{
		_fillContent: function(){
			// summary:
			//		When Menu is declared in markup, this code gets the menu label and
			//		the popup widget from the srcNodeRef.
			// description:
			//		srcNodeRefinnerHTML contains both the menu item text and a popup widget
			//		The first part holds the menu item text and the second part is the popup
			// example:
			// |	<div dojoType="dijit.PopupMenuItem">
			// |		<span>pick me</span>
			// |		<popup> ... </popup>
			// |	</div>
			// tags:
			//		protected

			if(this.srcNodeRef){
				var nodes = dojo.query("*", this.srcNodeRef);
				dijit.PopupMenuItem.superclass._fillContent.call(this, nodes[0]);

				// save pointer to srcNode so we can grab the drop down widget after it's instantiated
				this.dropDownContainer = this.srcNodeRef;
			}
		},

		startup: function(){
			if(this._started){ return; }
			this.inherited(arguments);

			// we didn't copy the dropdown widget from the this.srcNodeRef, so it's in no-man's
			// land now.  move it to dojo.doc.body.
			if(!this.popup){
				var node = dojo.query("[widgetId]", this.dropDownContainer)[0];
				this.popup = dijit.byNode(node);
			}
			dojo.body().appendChild(this.popup.domNode);
			this.popup.startup();

			this.popup.domNode.style.display="none";
			if(this.arrowWrapper){
				dojo.style(this.arrowWrapper, "visibility", "");
			}
			dijit.setWaiState(this.focusNode, "haspopup", "true");
		},

		destroyDescendants: function(){
			if(this.popup){
				// Destroy the popup, unless it's already been destroyed.  This can happen because
				// the popup is a direct child of <body> even though it's logically my child.
				if(!this.popup._destroyed){
					this.popup.destroyRecursive();
				}
				delete this.popup;
			}
			this.inherited(arguments);
		}
	});


}

if(!dojo._hasResource["dijit.CheckedMenuItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.CheckedMenuItem"] = true;
dojo.provide("dijit.CheckedMenuItem");



dojo.declare("dijit.CheckedMenuItem",
		dijit.MenuItem,
		{
		// summary:
		//		A checkbox-like menu item for toggling on and off

		templateString: dojo.cache("dijit", "templates/CheckedMenuItem.html", "<tr class=\"dijitReset dijitMenuItem\" dojoAttachPoint=\"focusNode\" waiRole=\"menuitemcheckbox\" tabIndex=\"-1\"\r\n\t\tdojoAttachEvent=\"onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick\">\r\n\t<td class=\"dijitReset dijitMenuItemIconCell\" waiRole=\"presentation\">\r\n\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuItemIcon dijitCheckedMenuItemIcon\" dojoAttachPoint=\"iconNode\"/>\r\n\t\t<span class=\"dijitCheckedMenuItemIconChar\">&#10003;</span>\r\n\t</td>\r\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" dojoAttachPoint=\"containerNode,labelNode\"></td>\r\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" dojoAttachPoint=\"accelKeyNode\"></td>\r\n\t<td class=\"dijitReset dijitMenuArrowCell\" waiRole=\"presentation\">&nbsp;</td>\r\n</tr>\r\n"),

		// checked: Boolean
		//		Our checked state
		checked: false,
		_setCheckedAttr: function(/*Boolean*/ checked){
			// summary:
			//		Hook so attr('checked', bool) works.
			//		Sets the class and state for the check box.
			dojo.toggleClass(this.domNode, "dijitCheckedMenuItemChecked", checked);
			dijit.setWaiState(this.domNode, "checked", checked);
			this.checked = checked;
		},

		onChange: function(/*Boolean*/ checked){
			// summary:
			//		User defined function to handle check/uncheck events
			// tags:
			//		callback
		},

		_onClick: function(/*Event*/ e){
			// summary:
			//		Clicking this item just toggles its state
			// tags:
			//		private
			if(!this.disabled){
				this.set("checked", !this.checked);
				this.onChange(this.checked);
			}
			this.inherited(arguments);
		}
	});

}

if(!dojo._hasResource["dijit.MenuSeparator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.MenuSeparator"] = true;
dojo.provide("dijit.MenuSeparator");





dojo.declare("dijit.MenuSeparator",
		[dijit._Widget, dijit._Templated, dijit._Contained],
		{
		// summary:
		//		A line between two menu items

		templateString: dojo.cache("dijit", "templates/MenuSeparator.html", "<tr class=\"dijitMenuSeparator\">\r\n\t<td class=\"dijitMenuSeparatorIconCell\">\r\n\t\t<div class=\"dijitMenuSeparatorTop\"></div>\r\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\r\n\t</td>\r\n\t<td colspan=\"3\" class=\"dijitMenuSeparatorLabelCell\">\r\n\t\t<div class=\"dijitMenuSeparatorTop dijitMenuSeparatorLabel\"></div>\r\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\r\n\t</td>\r\n</tr>\r\n"),

		postCreate: function(){
			dojo.setSelectable(this.domNode, false);
		},

		isFocusable: function(){
			// summary:
			//		Override to always return false
			// tags:
			//		protected

			return false; // Boolean
		}
	});


}

if(!dojo._hasResource["dijit.Menu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Menu"] = true;
dojo.provide("dijit.Menu");







dojo.declare("dijit._MenuBase",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	// summary:
	//		Base class for Menu and MenuBar

	// parentMenu: [readonly] Widget
	//		pointer to menu that displayed me
	parentMenu: null,

	// popupDelay: Integer
	//		number of milliseconds before hovering (without clicking) causes the popup to automatically open.
	popupDelay: 500,

	startup: function(){
		if(this._started){ return; }

		dojo.forEach(this.getChildren(), function(child){ child.startup(); });
		this.startupKeyNavChildren();

		this.inherited(arguments);
	},

	onExecute: function(){
		// summary:
		//		Attach point for notification about when a menu item has been executed.
		//		This is an internal mechanism used for Menus to signal to their parent to
		//		close them, because they are about to execute the onClick handler.   In
		//		general developers should not attach to or override this method.
		// tags:
		//		protected
	},

	onCancel: function(/*Boolean*/ closeAll){
		// summary:
		//		Attach point for notification about when the user cancels the current menu
		//		This is an internal mechanism used for Menus to signal to their parent to
		//		close them.  In general developers should not attach to or override this method.
		// tags:
		//		protected
	},

	_moveToPopup: function(/*Event*/ evt){
		// summary:
		//		This handles the right arrow key (left arrow key on RTL systems),
		//		which will either open a submenu, or move to the next item in the
		//		ancestor MenuBar
		// tags:
		//		private

		if(this.focusedChild && this.focusedChild.popup && !this.focusedChild.disabled){
			this.focusedChild._onClick(evt);
		}else{
			var topMenu = this._getTopMenu();
			if(topMenu && topMenu._isMenuBar){
				topMenu.focusNext();
			}
		}
	},

	_onPopupHover: function(/*Event*/ evt){
		// summary:
		//		This handler is called when the mouse moves over the popup.
		// tags:
		//		private

		// if the mouse hovers over a menu popup that is in pending-close state,
		// then stop the close operation.
		// This can't be done in onItemHover since some popup targets don't have MenuItems (e.g. ColorPicker)
		if(this.currentPopup && this.currentPopup._pendingClose_timer){
			var parentMenu = this.currentPopup.parentMenu;
			// highlight the parent menu item pointing to this popup
			if(parentMenu.focusedChild){
				parentMenu.focusedChild._setSelected(false);
			}
			parentMenu.focusedChild = this.currentPopup.from_item;
			parentMenu.focusedChild._setSelected(true);
			// cancel the pending close
			this._stopPendingCloseTimer(this.currentPopup);
		}
	},

	onItemHover: function(/*MenuItem*/ item){
		// summary:
		//		Called when cursor is over a MenuItem.
		// tags:
		//		protected

		// Don't do anything unless user has "activated" the menu by:
		//		1) clicking it
		//		2) opening it from a parent menu (which automatically focuses it)
		if(this.isActive){
			this.focusChild(item);
			if(this.focusedChild.popup && !this.focusedChild.disabled && !this.hover_timer){
				this.hover_timer = setTimeout(dojo.hitch(this, "_openPopup"), this.popupDelay);
			}
		}
		// if the user is mixing mouse and keyboard navigation,
		// then the menu may not be active but a menu item has focus,
		// but it's not the item that the mouse just hovered over.
		// To avoid both keyboard and mouse selections, use the latest.
		if(this.focusedChild){
			this.focusChild(item);
		}
		this._hoveredChild = item;
	},

	_onChildBlur: function(item){
		// summary:
		//		Called when a child MenuItem becomes inactive because focus
		//		has been removed from the MenuItem *and* it's descendant menus.
		// tags:
		//		private
		this._stopPopupTimer();
		item._setSelected(false);
		// Close all popups that are open and descendants of this menu
		var itemPopup = item.popup;
		if(itemPopup){
			this._stopPendingCloseTimer(itemPopup);
			itemPopup._pendingClose_timer = setTimeout(function(){
				itemPopup._pendingClose_timer = null;
				if(itemPopup.parentMenu){
					itemPopup.parentMenu.currentPopup = null;
				}
				dijit.popup.close(itemPopup); // this calls onClose
			}, this.popupDelay);
		}
	},

	onItemUnhover: function(/*MenuItem*/ item){
		// summary:
		//		Callback fires when mouse exits a MenuItem
		// tags:
		//		protected

		if(this.isActive){
			this._stopPopupTimer();
		}
		if(this._hoveredChild == item){ this._hoveredChild = null; }
	},

	_stopPopupTimer: function(){
		// summary:
		//		Cancels the popup timer because the user has stop hovering
		//		on the MenuItem, etc.
		// tags:
		//		private
		if(this.hover_timer){
			clearTimeout(this.hover_timer);
			this.hover_timer = null;
		}
	},

	_stopPendingCloseTimer: function(/*dijit._Widget*/ popup){
		// summary:
		//		Cancels the pending-close timer because the close has been preempted
		// tags:
		//		private
		if(popup._pendingClose_timer){
			clearTimeout(popup._pendingClose_timer);
			popup._pendingClose_timer = null;
		}
	},

	_stopFocusTimer: function(){
		// summary:
		//		Cancels the pending-focus timer because the menu was closed before focus occured
		// tags:
		//		private
		if(this._focus_timer){
			clearTimeout(this._focus_timer);
			this._focus_timer = null;
		}
	},

	_getTopMenu: function(){
		// summary:
		//		Returns the top menu in this chain of Menus
		// tags:
		//		private
		for(var top=this; top.parentMenu; top=top.parentMenu);
		return top;
	},

	onItemClick: function(/*dijit._Widget*/ item, /*Event*/ evt){
		// summary:
		//		Handle clicks on an item.
		// tags:
		//		private

		// this can't be done in _onFocus since the _onFocus events occurs asynchronously
		if(typeof this.isShowingNow == 'undefined'){ // non-popup menu
			this._markActive();
		}

		this.focusChild(item);

		if(item.disabled){ return false; }

		if(item.popup){
			this._openPopup();
		}else{
			// before calling user defined handler, close hierarchy of menus
			// and restore focus to place it was when menu was opened
			this.onExecute();

			// user defined handler for click
			item.onClick(evt);
		}
	},

	_openPopup: function(){
		// summary:
		//		Open the popup to the side of/underneath the current menu item
		// tags:
		//		protected

		this._stopPopupTimer();
		var from_item = this.focusedChild;
		if(!from_item){ return; } // the focused child lost focus since the timer was started
		var popup = from_item.popup;
		if(popup.isShowingNow){ return; }
		if(this.currentPopup){
			this._stopPendingCloseTimer(this.currentPopup);
			dijit.popup.close(this.currentPopup);
		}
		popup.parentMenu = this;
		popup.from_item = from_item; // helps finding the parent item that should be focused for this popup
		var self = this;
		dijit.popup.open({
			parent: this,
			popup: popup,
			around: from_item.domNode,
			orient: this._orient || (this.isLeftToRight() ?
									{'TR': 'TL', 'TL': 'TR', 'BR': 'BL', 'BL': 'BR'} :
									{'TL': 'TR', 'TR': 'TL', 'BL': 'BR', 'BR': 'BL'}),
			onCancel: function(){ // called when the child menu is canceled
				// set isActive=false (_closeChild vs _cleanUp) so that subsequent hovering will NOT open child menus
				// which seems aligned with the UX of most applications (e.g. notepad, wordpad, paint shop pro)
				self.focusChild(from_item);	// put focus back on my node
				self._cleanUp();			// close the submenu (be sure this is done _after_ focus is moved)
				from_item._setSelected(true); // oops, _cleanUp() deselected the item
				self.focusedChild = from_item;	// and unset focusedChild
			},
			onExecute: dojo.hitch(this, "_cleanUp")
		});

		this.currentPopup = popup;
		// detect mouseovers to handle lazy mouse movements that temporarily focus other menu items
		popup.connect(popup.domNode, "onmouseenter", dojo.hitch(self, "_onPopupHover")); // cleaned up when the popped-up widget is destroyed on close

		if(popup.focus){
			// If user is opening the popup via keyboard (right arrow, or down arrow for MenuBar),
			// if the cursor happens to collide with the popup, it will generate an onmouseover event
			// even though the mouse wasn't moved.   Use a setTimeout() to call popup.focus so that
			// our focus() call overrides the onmouseover event, rather than vice-versa.  (#8742)
			popup._focus_timer = setTimeout(dojo.hitch(popup, function(){
				this._focus_timer = null;
				this.focus();
			}), 0);
		}
	},

	_markActive: function(){
		// summary:
		//              Mark this menu's state as active.
		//		Called when this Menu gets focus from:
		//			1) clicking it (mouse or via space/arrow key)
		//			2) being opened by a parent menu.
		//		This is not called just from mouse hover.
		//		Focusing a menu via TAB does NOT automatically set isActive
		//		since TAB is a navigation operation and not a selection one.
		//		For Windows apps, pressing the ALT key focuses the menubar
		//		menus (similar to TAB navigation) but the menu is not active
		//		(ie no dropdown) until an item is clicked.
		this.isActive = true;
		dojo.addClass(this.domNode, "dijitMenuActive");
		dojo.removeClass(this.domNode, "dijitMenuPassive");
	},

	onOpen: function(/*Event*/ e){
		// summary:
		//		Callback when this menu is opened.
		//		This is called by the popup manager as notification that the menu
		//		was opened.
		// tags:
		//		private

		this.isShowingNow = true;
		this._markActive();
	},

	_markInactive: function(){
		// summary:
		//		Mark this menu's state as inactive.
		this.isActive = false; // don't do this in _onBlur since the state is pending-close until we get here
		dojo.removeClass(this.domNode, "dijitMenuActive");
		dojo.addClass(this.domNode, "dijitMenuPassive");
	},

	onClose: function(){
		// summary:
		//		Callback when this menu is closed.
		//		This is called by the popup manager as notification that the menu
		//		was closed.
		// tags:
		//		private

		this._stopFocusTimer();
		this._markInactive();
		this.isShowingNow = false;
		this.parentMenu = null;
	},

	_closeChild: function(){
		// summary:
		//		Called when submenu is clicked or focus is lost.  Close hierarchy of menus.
		// tags:
		//		private
		this._stopPopupTimer();
		if(this.focusedChild){ // unhighlight the focused item
			this.focusedChild._setSelected(false);
			this.focusedChild._onUnhover();
			this.focusedChild = null;
		}
		if(this.currentPopup){
			// Close all popups that are open and descendants of this menu
			dijit.popup.close(this.currentPopup);
			this.currentPopup = null;
		}
	},

	_onItemFocus: function(/*MenuItem*/ item){
		// summary:
		//		Called when child of this Menu gets focus from:
		//			1) clicking it
		//			2) tabbing into it
		//			3) being opened by a parent menu.
		//		This is not called just from mouse hover.
		if(this._hoveredChild && this._hoveredChild != item){
			this._hoveredChild._onUnhover(); // any previous mouse movement is trumped by focus selection
		}
	},

	_onBlur: function(){
		// summary:
		//		Called when focus is moved away from this Menu and it's submenus.
		// tags:
		//		protected
		this._cleanUp();
		this.inherited(arguments);
	},

	_cleanUp: function(){
		// summary:
		//		Called when the user is done with this menu.  Closes hierarchy of menus.
		// tags:
		//		private

		this._closeChild(); // don't call this.onClose since that's incorrect for MenuBar's that never close
		if(typeof this.isShowingNow == 'undefined'){ // non-popup menu doesn't call onClose
			this._markInactive();
		}
	}
});

dojo.declare("dijit.Menu",
	dijit._MenuBase,
	{
	// summary
	//		A context menu you can assign to multiple elements

	// TODO: most of the code in here is just for context menu (right-click menu)
	// support.  In retrospect that should have been a separate class (dijit.ContextMenu).
	// Split them for 2.0

	constructor: function(){
		this._bindings = [];
	},

	templateString: dojo.cache("dijit", "templates/Menu.html", "<table class=\"dijit dijitMenu dijitMenuPassive dijitReset dijitMenuTable\" waiRole=\"menu\" tabIndex=\"${tabIndex}\" dojoAttachEvent=\"onkeypress:_onKeyPress\" cellspacing=0>\r\n\t<tbody class=\"dijitReset\" dojoAttachPoint=\"containerNode\"></tbody>\r\n</table>\r\n"),

	baseClass: "dijitMenu",

	// targetNodeIds: [const] String[]
	//		Array of dom node ids of nodes to attach to.
	//		Fill this with nodeIds upon widget creation and it becomes context menu for those nodes.
	targetNodeIds: [],

	// contextMenuForWindow: [const] Boolean
	//		If true, right clicking anywhere on the window will cause this context menu to open.
	//		If false, must specify targetNodeIds.
	contextMenuForWindow: false,

	// leftClickToOpen: [const] Boolean
	//		If true, menu will open on left click instead of right click, similiar to a file menu.
	leftClickToOpen: false,

	// refocus: Boolean
	// 		When this menu closes, re-focus the element which had focus before it was opened.
	refocus: true,

	postCreate: function(){
		if(this.contextMenuForWindow){
			this.bindDomNode(dojo.body());
		}else{
			// TODO: should have _setTargetNodeIds() method to handle initialization and a possible
			// later attr('targetNodeIds', ...) call.   There's also a problem that targetNodeIds[]
			// gets stale after calls to bindDomNode()/unBindDomNode() as it still is just the original list (see #9610)
			dojo.forEach(this.targetNodeIds, this.bindDomNode, this);
		}
		var k = dojo.keys, l = this.isLeftToRight();
		this._openSubMenuKey = l ? k.RIGHT_ARROW : k.LEFT_ARROW;
		this._closeSubMenuKey = l ? k.LEFT_ARROW : k.RIGHT_ARROW;
		this.connectKeyNavHandlers([k.UP_ARROW], [k.DOWN_ARROW]);
	},

	_onKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handle keyboard based menu navigation.
		// tags:
		//		protected

		if(evt.ctrlKey || evt.altKey){ return; }

		switch(evt.charOrCode){
			case this._openSubMenuKey:
				this._moveToPopup(evt);
				dojo.stopEvent(evt);
				break;
			case this._closeSubMenuKey:
				if(this.parentMenu){
					if(this.parentMenu._isMenuBar){
						this.parentMenu.focusPrev();
					}else{
						this.onCancel(false);
					}
				}else{
					dojo.stopEvent(evt);
				}
				break;
		}
	},

	// thanks burstlib!
	_iframeContentWindow: function(/* HTMLIFrameElement */iframe_el){
		// summary:
		//		Returns the window reference of the passed iframe
		// tags:
		//		private
		var win = dojo.window.get(this._iframeContentDocument(iframe_el)) ||
			// Moz. TODO: is this available when defaultView isn't?
			this._iframeContentDocument(iframe_el)['__parent__'] ||
			(iframe_el.name && dojo.doc.frames[iframe_el.name]) || null;
		return win;	//	Window
	},

	_iframeContentDocument: function(/* HTMLIFrameElement */iframe_el){
		// summary:
		//		Returns a reference to the document object inside iframe_el
		// tags:
		//		protected
		var doc = iframe_el.contentDocument // W3
			|| (iframe_el.contentWindow && iframe_el.contentWindow.document) // IE
			|| (iframe_el.name && dojo.doc.frames[iframe_el.name] && dojo.doc.frames[iframe_el.name].document)
			|| null;
		return doc;	//	HTMLDocument
	},

	bindDomNode: function(/*String|DomNode*/ node){
		// summary:
		//		Attach menu to given node
		node = dojo.byId(node);

		var cn;	// Connect node

		// Support context menus on iframes.   Rather than binding to the iframe itself we need
		// to bind to the <body> node inside the iframe.
		if(node.tagName.toLowerCase() == "iframe"){
			var iframe = node,
				win = this._iframeContentWindow(iframe);
			cn = dojo.withGlobal(win, dojo.body);
		}else{
			
			// To capture these events at the top level, attach to <html>, not <body>.
			// Otherwise right-click context menu just doesn't work.
			cn = (node == dojo.body() ? dojo.doc.documentElement : node);
		}


		// "binding" is the object to track our connection to the node (ie, the parameter to bindDomNode())
		var binding = {
			node: node,
			iframe: iframe
		};

		// Save info about binding in _bindings[], and make node itself record index(+1) into
		// _bindings[] array.   Prefix w/_dijitMenu to avoid setting an attribute that may
		// start with a number, which fails on FF/safari.
		dojo.attr(node, "_dijitMenu" + this.id, this._bindings.push(binding));

		// Setup the connections to monitor click etc., unless we are connecting to an iframe which hasn't finished
		// loading yet, in which case we need to wait for the onload event first, and then connect
		// On linux Shift-F10 produces the oncontextmenu event, but on Windows it doesn't, so
		// we need to monitor keyboard events in addition to the oncontextmenu event.
		var doConnects = dojo.hitch(this, function(cn){
			return [
				// TODO: when leftClickToOpen is true then shouldn't space/enter key trigger the menu,
				// rather than shift-F10?
				dojo.connect(cn, this.leftClickToOpen ? "onclick" : "oncontextmenu", this, function(evt){
					// Schedule context menu to be opened unless it's already been scheduled from onkeydown handler
					dojo.stopEvent(evt);
					this._scheduleOpen(evt.target, iframe, {x: evt.pageX, y: evt.pageY});
				}),
				dojo.connect(cn, "onkeydown", this, function(evt){
					if(evt.shiftKey && evt.keyCode == dojo.keys.F10){
						dojo.stopEvent(evt);
						this._scheduleOpen(evt.target, iframe);	// no coords - open near target node
					}
				})
			];	
		});
		binding.connects = cn ? doConnects(cn) : [];

		if(iframe){
			// Setup handler to [re]bind to the iframe when the contents are initially loaded,
			// and every time the contents change.
			// Need to do this b/c we are actually binding to the iframe's <body> node.
			// Note: can't use dojo.connect(), see #9609.

			binding.onloadHandler = dojo.hitch(this, function(){
				// want to remove old connections, but IE throws exceptions when trying to
				// access the <body> node because it's already gone, or at least in a state of limbo

				var win = this._iframeContentWindow(iframe);
					cn = dojo.withGlobal(win, dojo.body);
				binding.connects = doConnects(cn);
			});
			if(iframe.addEventListener){
				iframe.addEventListener("load", binding.onloadHandler, false);
			}else{
				iframe.attachEvent("onload", binding.onloadHandler);
			}
		}
	},

	unBindDomNode: function(/*String|DomNode*/ nodeName){
		// summary:
		//		Detach menu from given node

		var node;
		try{
			node = dojo.byId(nodeName);
		}catch(e){
			// On IE the dojo.byId() call will get an exception if the attach point was
			// the <body> node of an <iframe> that has since been reloaded (and thus the
			// <body> node is in a limbo state of destruction.
			return;
		}

		// node["_dijitMenu" + this.id] contains index(+1) into my _bindings[] array
		var attrName = "_dijitMenu" + this.id;
		if(node && dojo.hasAttr(node, attrName)){
			var bid = dojo.attr(node, attrName)-1, b = this._bindings[bid];
			dojo.forEach(b.connects, dojo.disconnect);

			// Remove listener for iframe onload events
			var iframe = b.iframe;
			if(iframe){
				if(iframe.removeEventListener){
					iframe.removeEventListener("load", b.onloadHandler, false);
				}else{
					iframe.detachEvent("onload", b.onloadHandler);
				}
			}

			dojo.removeAttr(node, attrName);
			delete this._bindings[bid];
		}
	},

	_scheduleOpen: function(/*DomNode?*/ target, /*DomNode?*/ iframe, /*Object?*/ coords){
		// summary:
		//		Set timer to display myself.  Using a timer rather than displaying immediately solves
		//		two problems:
		//
		//		1. IE: without the delay, focus work in "open" causes the system
		//		context menu to appear in spite of stopEvent.
		//
		//		2. Avoid double-shows on linux, where shift-F10 generates an oncontextmenu event
		//		even after a dojo.stopEvent(e).  (Shift-F10 on windows doesn't generate the
		//		oncontextmenu event.)

		if(!this._openTimer){
			this._openTimer = setTimeout(dojo.hitch(this, function(){
				delete this._openTimer;
				this._openMyself({
					target: target,
					iframe: iframe,
					coords: coords
				});
			}), 1);
		}
	},

	_openMyself: function(args){
		// summary:
		//		Internal function for opening myself when the user does a right-click or something similar.
		// args:
		//		This is an Object containing:
		//		* target:
		//			The node that is being clicked
		//		* iframe:
		//			If an <iframe> is being clicked, iframe points to that iframe
		//		* coords:
		//			Put menu at specified x/y position in viewport, or if iframe is
		//			specified, then relative to iframe.
		//
		//		_openMyself() formerly took the event object, and since various code references
		//		evt.target (after connecting to _openMyself()), using an Object for parameters
		//		(so that old code still works).

		var target = args.target,
			iframe = args.iframe,
			coords = args.coords;

		// Get coordinates to open menu, either at specified (mouse) position or (if triggered via keyboard)
		// then near the node the menu is assigned to.
		if(coords){
			if(iframe){
				// Specified coordinates are on <body> node of an <iframe>, convert to match main document
				var od = target.ownerDocument,
					ifc = dojo.position(iframe, true),
					win = this._iframeContentWindow(iframe),
					scroll = dojo.withGlobal(win, "_docScroll", dojo);
	
				var cs = dojo.getComputedStyle(iframe),
					tp = dojo._toPixelValue,
					left = (dojo.isIE && dojo.isQuirks ? 0 : tp(iframe, cs.paddingLeft)) + (dojo.isIE && dojo.isQuirks ? tp(iframe, cs.borderLeftWidth) : 0),
					top = (dojo.isIE && dojo.isQuirks ? 0 : tp(iframe, cs.paddingTop)) + (dojo.isIE && dojo.isQuirks ? tp(iframe, cs.borderTopWidth) : 0);

				coords.x += ifc.x + left - scroll.x;
				coords.y += ifc.y + top - scroll.y;
			}
		}else{
			coords = dojo.position(target, true);
			coords.x += 10;
			coords.y += 10;
		}

		var self=this;
		var savedFocus = dijit.getFocus(this);
		function closeAndRestoreFocus(){
			// user has clicked on a menu or popup
			if(self.refocus){
				dijit.focus(savedFocus);
			}
			dijit.popup.close(self);
		}
		dijit.popup.open({
			popup: this,
			x: coords.x,
			y: coords.y,
			onExecute: closeAndRestoreFocus,
			onCancel: closeAndRestoreFocus,
			orient: this.isLeftToRight() ? 'L' : 'R'
		});
		this.focus();

		this._onBlur = function(){
			this.inherited('_onBlur', arguments);
			// Usually the parent closes the child widget but if this is a context
			// menu then there is no parent
			dijit.popup.close(this);
			// don't try to restore focus; user has clicked another part of the screen
			// and set focus there
		};
	},

	uninitialize: function(){
 		dojo.forEach(this._bindings, function(b){ if(b){ this.unBindDomNode(b.node); } }, this);
 		this.inherited(arguments);
	}
}
);

// Back-compat (TODO: remove in 2.0)






}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.ContextMenuAdapter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.ContextMenuAdapter"] = true;
/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
dojo.require( "dijit.Menu" );

dojo.provide( "ibm.tivoli.tip.dijit.ContextMenuAdapter" );
/*******************************************************************************
* ContextMenuAdapter is responsible for constructing and displaying dynamic
* context menus.  That is it will ask for the items that should be displayed
* each time it is told to present the context menu. 
*
* It can handle calling a JSON Url to retreive the items for you.
*
* It also has a method that can be overloaded to provide context to the code
* building the menu. (For instance -- the row of a table that is selected)
*
* The menu model is as follows:
* 
* [ { 
*     id: "uniqueID", 
*     label: "label", 
*     iconClass: "iconClass", 
*     submenu_items: [ {},{} ] 
*     actionID: "actionToFire" 
    } 
* ... ]
*
* id, iconClass, and submenu_items are all optional
*
* NOTE:
* ! IF "_separator_" is passed as actionID then a menu separator is created !
*******************************************************************************/
dojo.declare( "ibm.tivoli.tip.dijit.ContextMenuAdapter", null,
{
   /* [Optional] The JSON Url to request context menu items from */
   url: null,
   
   /* [Optional] The unique name of the context menu -- for lookup of items */
   name: null,
   
   /****************************************************************************
   * The constructor of the adapter.  Mixes in parameters.
   ****************************************************************************/
   constructor: function( args )
   {
     dojo.mixin( this, args );
     
     this.menuItems = [];
     this.subMenus = [];
   },
      
   /****************************************************************************
   * Ensures that only one context menu is used during the life of this adapter.
   * It destroys any previously created menus and recreates a new menu.
   ****************************************************************************/
   createContextMenu: function()
   {
     this._menu = new dijit.Menu();
     
     this._onCloseListener = dojo.connect( this._menu, "onClose", this, "_destroyMenu" );
     
     //populate the menu
     this.populateMenu( this._menu ); 
     
     return this._menu;         
   },
   
   /****************************************************************************
   * Return the current dijit.Menu context menu popup or null if there is not
   * a current context menu popup.
   ****************************************************************************/   
   getMenu: function() {
    	return ("_menu" in this) ? this._menu : null;
   },  
   
   /****************************************************************************
   * When the menu closes -- cleanly destroy it.  There is a bug in Dojo that
   * requires us to find all orphaned pieces of the menu to cleanly destroy
   ****************************************************************************/   
   _destroyMenu: function()
   {
     dojo.disconnect( this._onCloseListener );
     
     delete this._onCloseListener;
     
     // This line causes a node is null error in dojo 1.3.1, so get rid of it... ?
     // Get dojo itself to destroy first. 
   	 this._menu.destroyRecursive();

	 for ( var i in this.menuItems )
	 {
	   if (i) {
	   try{
	 	this.menuItems[ i ].destroy();
		}catch(e){ console.log("destory menu items"); console.log(e);
		}
	   }
	 }
     
	 for ( i in this.subMenus )
	 {
	   if (i) {
	   try{
	 	this.subMenus[ i ].destroy();
		}catch(e){ console.log("destory sub menu items");console.log(e);
		}
	   }
	 }
	 
	 // Mark A.: Apr 8,2011: for defect 5339 on CSNext RTC
   	 // Commenting out this line as it destroys other popups on the page, e.g., drop down menus on toolbars.
   	 // It appears this was here for earlier versions of Dojo that did not clean up menus correctly.
     // dojo.query( ".dijitPopup" ).forEach( function( domNode ) { try{dojo._destroyElement( domNode );}catch(e){} } );       
     
     delete this._menu;            
     this.menuItems = [];
     this.subMenus = [];
   },
   
   /****************************************************************************
   * This method must be called with a mouse event to show the Context menu.
   * You could pass in an object with x:xPos and y:yPos for event to trick the 
   * context menu to open at any given point on screen.
   *
   * The context object is any object that you want to send back to the server
   * or pass to the getMenuItems method.
   ****************************************************************************/
   contextMenuOpen: function ( event, context , contentIsolation)
   {
		if(contentIsolation === null || contentIsolation === undefined)
			contentIsolation = false;
		this._context = context;
		// If in TIP content pane or in isolation iframe (i.e. with PopupUtil available)...
		if( contentIsolation == true  && typeof( PopupUtil ) != 'undefined' ){
			// Get the namespace identifying the content that launched the context menu
			var namespace="ns__N1468806017_";
			if(this.cmProvider.global != null)
			 namespace = this.cmProvider.global.namespace;
			//Get the location of the event
			var eCoords = {x:0,y:0};
			try {
				if( event.pageX || event.pageY ) 	{
					eCoords.x = event.pageX;
					eCoords.y = event.pageY;
				}
				else if (event.clientX || event.clientY) 	{
					eCoords.x = event.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
					eCoords.y = event.clientY + document.body.scrollTop + document.documentElement.scrollTop;
				}
			} catch( err ) {
				eCoords = dojo.coords( event.target, true );
			}
			dojo.stopEvent(event);  //To ensure the browser right-click event is stopped.
			//Close (just in case) and then open the popup covering the whole page
			var old_popup_iframe = PopupUtil.ClosePopup( namespace+'ibm.tivoli.tip.dijit.ContextMenuAdapter' );
			//Changes for fixing Content Isolation issue with Claro theme
			var theme = dojo.body().className;
			var popup_iframe = PopupUtil.OpenPopup( 
				namespace, 
				namespace+'ibm.tivoli.tip.dijit.ContextMenuAdapter', 
				true, 
				null, 
				null, 
				null, 
				null, 
				dojo.baseUrl+'../html/TIPCMPopup.html?PopupID='+namespace+'ibm.tivoli.tip.dijit.ContextMenuAdapter&eLeft='+eCoords.x+'&eTop='+eCoords.y+"&theme="+theme, 
				null
			);
			//Initialize the contextMenu when the iframe popup is finished loading
			var tThis = this;
			if( popup_iframe.addEventListener ){
				popup_iframe.addEventListener(
					"load", 
					function(){	

					   if(dojo.isIE){
							popup_iframe.contentWindow.setMenuItems(tThis.cmProvider.getMenuItems(context));
							popup_iframe.contentWindow.setMenuProvider(tThis.cmProvider);
							popup_iframe.contentWindow.makeTIPContextMenu( null,context );
						}
						popup_iframe.contentWindow.makeTIPContextMenu( tThis.cmProvider,context );
					}, 
					false
				);
			}
			else if (popup_iframe.attachEvent){
				popup_iframe.attachEvent(
					"onload", 
					function(){ 

						if(dojo.isIE){
							popup_iframe.contentWindow.setMenuItems(tThis.cmProvider.getMenuItems(context));
							popup_iframe.contentWindow.setMenuProvider(tThis.cmProvider);
							popup_iframe.contentWindow.makeTIPContextMenu( null,context );
						}
						popup_iframe.contentWindow.makeTIPContextMenu( tThis.cmProvider,context );
					}
				);
			}
		}
		else{

		try{

			var menu = this.createContextMenu();
			//Added as a fix for IBM Dojo 1.5
			dojo.stopEvent(event);  //To ensure the browser right-click event is stopped.
			//Vyshantha: Feb18,2011: Defect 903 on CSNext RTC or #158382 on the CS Bug Tracker: start of fix
			coords = {x: event.pageX, y: event.pageY};
			event.coords = coords;
			//end of fix.
			menu._contextMenuWithMouse = true;
			//Vyshantha:Mar3,2011: Hack to fix context menu opening issue on IE 7.0 : start
			if(dojo.isIE === 7){
				var self = menu;
				var savedFocus = dijit.getFocus(menu);
				function closeAndRestoreFocus(){
					// user has clicked on a menu or popup
					if(self.refocus){
						dijit.focus(savedFocus);
					}
					dijit.popup.close(self);
				}
				dijit.popup.open({
					popup: menu,
					x: event.coords.x + 10,
					y: event.coords.y + 10,
					onExecute: closeAndRestoreFocus,
					onCancel: closeAndRestoreFocus,
					orient: menu.isLeftToRight() ? 'L' : 'R'
				});
				if(dojo.isIE !== 7)
					menu.focus();

				menu._onBlur = function(){
					menu.inherited('_onBlur', arguments);
					// Usually the parent closes the child widget but if this is a context
					// menu then there is no parent
					dijit.popup.close(menu);
					// don't try to restore focus; user has clicked another part of the screen
					// and set focus there
				};
			} else 
				menu._openMyself( event );
			}catch(e){console.log("openmyself"); console.log(e);}
		}
   },
   
   /****************************************************************************
   * Asks the JSON URL or callback method for the menu items then recursively
   * builds the menu.
   ****************************************************************************/   
   populateMenu: function( menu )
   {
     var model = null;
     
     if ( this.url !== null )
     { 
       model = this._loadMenuItems(); 
     }
     else
     {
       model = this.getMenuItems( this._context );
     }
     
     if ( model !== undefined && model !== null )
     {
       this._addItemsToMenu( menu, model );

	 
       try{

       menu.startup();

	 
	   }catch(e){console.log("startup");console.log(e);
    	  
      }
     }               
   },
   
   /****************************************************************************
   * Recursive function to construct menu from array of items
   ****************************************************************************/
   _addItemsToMenu:function( parent, items )
   {
      var params;
      var subMenuIndex;
      var menuItemIndex;
	  
      try{
      for ( var i in items )
      {
         if ( items[i].submenu_items )
         {
           subMenuIndex = this.subMenus.length;
           this.subMenus[ subMenuIndex ] = new dijit.Menu();
 
           params = 
           {
               popup: this.subMenus[ subMenuIndex ]
           };

           dojo.mixin( params, items[i] );
        
         //  menuItemIndex = this.menuItems.length; 
          // this.menuItems[ menuItemIndex ] = new dijit.PopupMenuItem( params );
           //parent.addChild(  this.menuItems[ menuItemIndex ] );  
			parent.addChild( new dijit.PopupMenuItem( params ) ); 
           
           this._addItemsToMenu( this.subMenus[ subMenuIndex ], items[i].submenu_items );           
         }
         else
         {
           // [2/15/08] The following if clause added for backward compatibility
           // with the previous version of TIPContextMenu
           if (items[i].id && items[i].id.toLowerCase() === "separator")
           {
              // Previous TIPContextMenu users indicated separators via the id
              items[i].actionID = "_separator_";
           } 
           else if (items[i].id && !items[i].actionID) 
           {
              // Previous TIPContextMenu users didn't use actionID
              items[i].actionID = items[i].id;
           }

           if ( items[i].actionID === "_separator_" )
           {
        //     menuItemIndex = this.menuItems.length;
             
          //   this.menuItems[ menuItemIndex ]  = new dijit.MenuSeparator();
             
            // parent.addChild( this.menuItems[ menuItemIndex ] );
			 
			parent.addChild( new dijit.MenuSeparator() ); 
           }
           else
           {
             params = {
               adapter: this,
               onClick: function() { this.adapter.onMenuItemClick( this ); }
             };
             
             dojo.mixin( params, items[i] );

            // menuItemIndex = this.menuItems.length;
             
             //this.menuItems[ menuItemIndex ] = new dijit.MenuItem( params );
             
             //parent.addChild( this.menuItems[ menuItemIndex ] );
			
			parent.addChild( new dijit.MenuItem( params ) );
  
           }
         }         
      }}catch(e){console.log("in add items to menu");}
   },
      
   /****************************************************************************
   * Callback that should be overloaded to be told when a menu item is clicked
   * and an action should occur
   ****************************************************************************/
   onMenuItemClick: function( menuItem ){ /*required overload*/ },
   
   /****************************************************************************
   * Synchronously Call a JSON Url to get the menu items
   *
   * Calls the provided url with the following:
   *    name: this.name (unique name of context menu to retrieve)
   *    content: this._context (the context object passed to contextMenuOpen)
   *
   * Theoretically this method could be overloaded to do something other than
   * dojo.xhrPost()
   ****************************************************************************/
   _loadMenuItems: function()
   {
     var kw = {
            sync:         true,
            url:          this.url,
            name:         this.name,
            content:      this._context,
            handleAs:     "json",
            mimetype:     "application/json",            
            load:         dojo.hitch( this, function( response ) { this._menuitems = response; } )       
          };
     
     //call synchronously                                                                       
     dojo.xhrGet( kw );
     
     return this._menuitems;
   },  

   /****************************************************************************
   * This method will be called if 'url' parameter is not set.  Provides a means
   * for javascript to build the menu items.
   ****************************************************************************/
   getMenuItems: function( context ){ /* must overload to provide menu items if URL is not used for JSON */ }
   
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TextInputBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TextInputBox"] = true;
dojo.provide("ibm.tivoli.tip.dijit.TextInputBox"); 





/** 
 * A text input box widget.
 * 
 * Such widget displays a text input box which implements
 * the user-provided validations and adapts the look&feel reflecting
 * to its current status.
 * The widget accepts the following parameters:
 * 
 *  - value: the initial value the widget must show when created.
 *  - constraints: an object containing the validation constraints.
 *    The same constraints can be provided as single properties.
 *    A constraint provided as a single property overrides the 
 *    corresponding constraint provided in the constraints object.
 *    The possible constraints are:
 *      
 *      - minlen: (Number) the minimum number of required characters.
 *      - maxlen: (Number) the maximum number of required characters.   
 *      - regexp: (RegExp) a regular expression (object) which must
 *        be matched.
 *      - regexpMessage: (String) the message to show if the regular 
 *        expression has not been matched.
 *      - required: (Boolean) if true the value cannot be empty ("").
 *      - validator: (Function) an external validator function to be 
 *        invoked. The validator accepts the value has parameters and
 *        returns a status object which will affect the widget status.
 *        If null is returned, the validation goes ahead.  
 * 
 *  -  minLength: (String) the same of constraints.minlen.
 *  -  maxLength: (String) the same of constraints.maxlen.
 *  -  regExp: (String) the same of constraints.regexp.
 *  -  regExpMessage: (String) the same of constraints.regexpMessage.
 *  -  required: (String) the same of constraints.required.
 * 
 * Example of a declaration:
 * 
 *       <div 
 *          dojoType="ibm.tivoli.tip.dijit.TextInputBox" 
 *          constraints="{required: true, minlen:5,regexp: /^[0-9]+$/g}"
 *          maxLength="10"/>
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)    
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.TextInputBox",
    [ibm.tivoli.tip.dijit.Widget, dijit._Templated],
    {
        /** the initial value */
        value: "",
        /** the constraints */
        constraints: {},
        /** the minlen */
        minLength: "",
        /** the maxlen */
        maxLength: "",
        /** the regular expression */  
        regExp: "",
        /** the regular expression message */
        regExpMessage: "",
        /** required */
        required: "",
        /** the text box type ("text" or "password") */
        type: "text",
        
        /** the widget template path */
        templateString:"<span  class=\"textInputDefault\"  dojoAttachEvent=\"onclick:_onClick\"  >\r\n\t<img  class=\"textInputIcon\"  src=\"${constants.DEFAULT_ICON}\"   dojoAttachPoint=\"_iconNode\"  />\r\n\t<input  class=\"textInputNode\"  type=\"${type}\"    value=\"\"\r\n        dojoAttachPoint=\"_inputNode, _focusNode\"    dojoAttachEvent=\"onkeyup:_onKeyUp, onkeypress:_onKeyPress, onchange:_onChange\"\r\n\t\t\r\n    />\r\n\t<span  class=\"textInputRight\"  >&nbsp;</span\r\n></span>\r\n   \r\n",
        
        /** input box DOM node */
        _inputNode: null,
        /** the icon node */
        _iconNode: null,
        /** the widget value */
        _value: null,
        /** the caret */
        _caret: null,
        /**
         * Widget constructor.
         */
        constructor: function()
        {
            /** the widget's constraints */
			
            this.constraints = {};

            // set the widget constants
            // default icon url
            this.constants.DEFAULT_ICON = dojo.moduleUrl("ibm.tivoli.tip.dijit", "templates/images/transparent.gif");                
            // error icon url
            this.constants.ERROR_ICON = dojo.moduleUrl("ibm.tivoli.tip.dijit", "templates/images/error.gif");
        },
        
        /**
         * Post mixin properties.
         * Validates attributes.
         */
        postMixInProperties: function()
        {
            // call the superclass's method
            this.inherited("postMixInProperties", arguments);
            
            // validate attributes
            if ( this.type !== "text" && this.type !== "password" ) {
                this.type = "text";
            }     
        },
        
        /**
         * Sets the initial value: this triggers the first validation.
         * The initial value can be externally provided by setting the 
         * value property.  
         */
        startup: function() 
        {
            // call the superclass's method
			//console.info("startup tip.TextInputBox");
            this.inherited("startup", arguments);
           
            // get parameters
            // get the required
            if ( this.required !== "" ) {
                this.constraints.required = this.required === "true" ? true : false;    
            }
            // get the minlen
            if ( this.minLength !== "" ) {
                this.constraints.minlen = parseInt(this.minLength, 10);
            }
            // get the maxlen
            if ( this.maxLength !== "" ) {
                this.constraints.maxlen = parseInt(this.maxLength, 10);
            }          
            // get the regular expression  
            if ( this.regExp !== "" ) {
                this.constraints.regexp = new RegExp(this.regExp);
            }
            // get the regular expression message  
            if ( this.regExpMessage !== "" ) {
                this.constraints.regexpMessage = this.regExpMessage;
            }
            // set the default regexp message if no message has been provided
            if ( ! this.constraints.regexpMessage || this.constraints.regexpMessage === "" ) {
                this.constraints.regexpMessage = this._resources.TEXT_INVALID_CHARS;
            }
            
            // create the caret
            this._caret = new ibm.tivoli.tip.dijit.Caret(this._inputNode);
            
            // get the value
            this.setValue( this.value || "" );
        },

        /**        
         * Returns an object which represent the widget value.
         */
        getValue: function()
        {
            // return the value
            return this._value;
        },
        
        /**        
         * Sets the widget value.
         */
        setValue: function(value)
        {
            // store the internal value
            this._value = value;
            // update the input box
            this._inputNode.value = value;
            // validate
            this._validate();
        },
        
        /**
         * Returns true if the widget's value is not set.
         */
        isEmpty: function()
        {
            return ( this._value === "" );
        },

        /**
         * Enables the widget for editing. 
         */
        setDisabled: function(isDisabled)
        {
            if ( isDisabled != this.isDisabled() ) {
                // update the enablement status
                this.inherited("setDisabled", arguments);
                // update the widget
                this._updateWidget();
            }
        },
        
        /**
         * Sets the widget status: the status is an object with the
         * following two properties:
         * 
         *  - state: the state of the widget (e.g. constants.ERROR)
         *  - message: the message related to the current state.
         */
        setStatus: function(status)
        {
            // update the status
            this.inherited("setStatus", arguments);
            // update the widget
            this._updateWidget();
        },
        
        /**
         * Sets the widget required constraint.
         * 
         *  @param required:
         *      true if the value is required, false otherwise.
         */
        setRequired: function(required)
        {
            if ( this.constraints.required != required ) {
                // store the requried constraint
                this.constraints.required = required;
                // trigger the validation
                this._validate();
            }
        },
        
        /**
         * Returns true if the widget's value is required.
         */
        isRequired: function()
        {
            return this.constraints.required;
        },
        
        /**
         * Asks the widget to revalidate its value and to set its status accordingly.
         */
        validate: function() 
        {
        	this._validate();
        },

        /**
         * Gives the focus to the widget.
         */
        setFocus: function()
        {
            this._inputNode.focus();
        },
        
        /**
         * Returns the caret position.
         */
        getCaretPosition: function()
		{
			return this._caret.getPosition();
		},
		
		/**
         * Sets the caret position.
         */
		setCaretPosition: function(index)
		{
		    this._caret.setPosition(index);
		},
		        
        /**
         * Handles the onkeypress event. Prevents container to catch
         * the key pression. 
         */
        _onKeyPress: function(e)
        {
            // let the enter and tab to be propagated
            if ( e.keyCode !== 13 && e.keyCode !== 9 ) {
                // stop event propagation
                e.stopPropagation();
            }
        },
        
        /**
         * Handles the onkeyup event. The internal value is updated and
         * the validation is triggered. 
         */
        _onKeyUp: function(e)
        {
            // get the value from the input text box
            this._value = this._inputNode.value;
            // trigger the validation
            this._validate();
            // stop event propagation
            e.stopPropagation();
        },
        
        /**
         * Handles the onchange event.  
         */
        _onChange: function()
        {
            // notify the change
            this.onValueChanged(this.getValue());
        },
         
        /**
         * Handles the onclick event.  
         */        
        _onClick: function(e)
        {
			if ( e.target.tagName === "SPAN" ) {
				this.setCaretPosition(this._inputNode.value.length); 	
			}
        },
        
        /**
         * Validates the current value applying all the user-provided
         * constraints. The status is then updated.
         */
        _validate: function()
        {
            // check if we've some constraints 
            if ( this.constraints !== {} ) {
                // set the widget in error, eventually reset ahead in case of no errors  
                this._isInError = true;
                
                // check if the field is required
                if( this.constraints.required && this._value === "" ) {
                    this.setStatus({ 
                        state: this.constants.REQUIRED, 
                        message: this._resources.TEXT_REQUIRED
                    });
                    return;
                }
                                
                // check the min length (if any)
                if ( this.constraints.minlen !== undefined ) {
                    if ( this._value.length < this.constraints.minlen ) {
                        this.setStatus({ 
                            state: this.constants.ERROR, 
                            message: dojo.string.substitute(this._resources.TEXT_MIN_LIMIT, {minlen: this.constraints.minlen})
                        });
                        return;
                    }
                }
                
                // check the max length (if any)
                if ( this.constraints.maxlen !== undefined ) {
                    if ( this._value.length > this.constraints.maxlen ) {
                        this.setStatus({ 
                            state: this.constants.ERROR,
                            message: dojo.string.substitute(this._resources.TEXT_MAX_LIMIT, {maxlen: this.constraints.maxlen})
                        });
                        return;
                    }
                }
                
                // match the regex (if any)
                if ( this.constraints.regexp !== undefined ) {
                    if ( ! this._value.match(this.constraints.regexp) ) {   
                        this.setStatus({ 
                            state: this.constants.ERROR,
                            message: this.constraints.regexpMessage  
                        });
                        return;
                    }
                }
                
                // call the external validator (if any)
                if ( this.constraints.validator !== undefined ) {
                    var status = this.constraints.validator(this._value);
                    if ( status ) {
                        this.setStatus(status);
                        return;
                    }
                }  
            }
                        
            // if we've reached this point, no error is present
            this._isInError = false;
            this.setStatus({ 
                state: this.constants.DEFAULT, 
                message: ""  
            });
        },
        
        /**
         * Updates the widget look&feel depending on the widget' status.
         */
        _updateWidget: function()
        {
            this._inputNode.disabled = this.isDisabled();
            
            if ( this.isDisabled() ) { 
                // the widget is disabled
                this._iconNode.src = this.constants.DEFAULT_ICON;
                this.domNode.className = "textInputDisabled";
            }
            else {  
                // the widget is enabled
                var status = this.getStatus();
                
                if( status.state === this.constants.ERROR ) {
                    this.domNode.className = "textInputError";
                    this._iconNode.src = this.constants.ERROR_ICON;
                }
                else if( this.isRequired() ) {
                    this.domNode.className = "textInputRequired";
                    this._iconNode.src = this.constants.DEFAULT_ICON;
                }
                else {
                    this.domNode.className = "textInputDefault";
                    this._iconNode.src = this.constants.DEFAULT_ICON;
                }
            }
        }
    }
);


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.picker.PickerInputBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.picker.PickerInputBox"] = true;
dojo.provide("ibm.tivoli.tip.dijit.picker.PickerInputBox"); 


 


/** 
 * A text input box with a picker on the right.
 * 
 * Such widget is the base for specific picker implementations.
 * An actual picker must inherit from such widget and set
 * the following attributes and attach points:
 * 
 *      - pickerIcon: the picker icon.
 *      - textMinLength: (String) the minimum number of required characters.
 *      - textMaxLength: (String) the maximum number of required characters.
 *      - textRegExp: a regular expression used to validate the text
 *        displayed in the text input box.
 *      - setPicker: must be used to provide the picker object. 
 *      - getPickerStartValue: must be overridden if the inheriting
 *        class want provide a specific start value to show when the 
 *        picker is popped-up.
 *      - getPickerTextValue: must be overridden by the inheriting
 *        class in order to get a textual representation from an actual
 *        value (which could not be a String but an Object).
 *      - onTextValueChanged: must be overridden by the inheriting
 *        class in order to get notified about changes happened in the 
 *        text input box.
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.picker.PickerInputBox",
    [ibm.tivoli.tip.dijit.Widget, dijit._Templated],
    {
        /** the initial value */
        value: "",
        /** required (String) */
        required: "false",
        /** the picker icon */
        pickerIcon: "",
        /** the minlen */
        textMinLength: "",
        /** the maxlen */
        textMaxLength: "",
        /** regular expression for validation on the text value */
        textRegExp: "",
        /** the message to show if the regular expression is not matched */
        textRegExpMessage: "",        
        /** the tab index */
        tabindex: "0",

        /** the widget contains other widget */        
        widgetsInTemplate: true,
        /** the widget template path */
        templateString:"<span\r\n    class=\"picker\"\r\n    ><span \r\n        tabindex=\"${tabindex}\"\r\n        dojoType=\"ibm.tivoli.tip.dijit.TextInputBox\"\r\n        dojoAttachPoint=\"_textNode,_focusNode\"\r\n        required=\"${required}\"\r\n        minLength=\"${textMinLength}\"\r\n        maxLength=\"${textMaxLength}\"\r\n        regExp=\"${textRegExp}\"\r\n        regExpMessage=\"${textRegExpMessage}\"\r\n    ></span\r\n    ><img \r\n        tabindex=\"${tabindex}\"\r\n        src=\"${pickerIcon}\"\r\n        dojoAttachPoint=\"_pickerButtonNode\"\r\n        dojoAttachEvent=\"onclick:_onClick,onkeyup:_onKeyUp\"\r\n        class=\"pickerButton\"\r\n/></span>\r\n   \r\n",

        /** the text node */
        _textNode: null,
        /** the picker button node */
        _pickerButtonNode: null,
        /** the picker widget */
        _picker: null,
        /** true if the picker is opened */
        _pickerOpened: false,
        /** true if the picker button must not open the picker  */
        _pickerIconDisabled: false,
        
        /**
         * Sets the initial value.  
         */
        startup: function() 
        {
            
			//console.info("startup PickerInputBox");
			// call the superclass's method
            this.inherited("startup", arguments);
            // set the value
            this.setValue(this.value);
            // attach the text "on value changed" event
            this._textNode.onValueChanged = dojo.hitch(this, this.onTextValueChanged);
			
			
            // change the tooltip node
            this._textNode.setTooltipNode(this.domNode);
        },
        
        /**
         * Destroys the widget.
         */
        destroy: function()
        {
            // destroy the picker if any
            if ( this._picker ) {
                this._picker.destroy();
            }            
            // call the superclass's method
            this.inherited("destroy", arguments);
        },
                
        /**
         * Sets the picker widget.
         * 
         *      @param picker
         *          The picker widget.
         */
        setPicker: function(picker)
        {
            // destroy the previous picker if any
            if ( this._picker ) {
                this._picker.destroy();
            }
            
            // set-up the picker
            this._picker = picker;
            if ( this._picker ) {
                // hitch the "onValueSelected" callback
                this._picker.onValueSelected = dojo.hitch(this, this._onPickerValueSelected);
                // set the picker value 
				//MODIFIED By PRASHANT SIROHI to Remove  Warning :DEPRECATED: dijit._TimePicker:setValue() is deprecated. Use attr('value') instead. -- will be removed in version: 2.0 		
                this._picker.attr('value',this.getPickerStartValue());
            }            
        },
        
        /**        
         * Disables/enables the picker button.
         * 
         *  @param disable: (Boolean) if true the picker button is disabled.
         */
        disablePickerIcon: function(disabled)
        {
            this._pickerIconDisabled = disabled;
        },
        
        /**        
         * Returns an object which represent the widget value.
         */
        getValue: function()
        {
            // return the value from the text box
            return this._textNode.getValue();
        },
        
        /**        
         * Sets the widget value.
         */
        setValue: function(value)
        {
            // update the text box value
			
            this._textNode.setValue(value);
        },
        
        /**
         * Enables the widget for editing. 
         */
        setDisabled: function(isDisabled)
        {
            if ( isDisabled != this.isDisabled() ) {
                // update the enablement status
                this.inherited("setDisabled", arguments);
                // update the widget
                this._textNode.setDisabled(isDisabled);
            }
        },
                
        /**
         * Returns true if the widget's value is wrong.
         */
        isInError: function()
        {
            return this._textNode.isInError();
        },
        
        /**
         * Sets the widget required constraint.
         * 
         *  @param required:
         *      true if the value is required, false otherwise.
         */        
        setRequired: function(required)
        {
            this._textNode.setRequired(required);
        },

        /**
         * Returns if the the widget required constraint is required, false otherwise.
         */        
        isRequired: function(required)
        {
        	return this._textNode.isRequired();
        },

        /**
         * Returns the picker start value.
         */
        getPickerStartValue: function()
        {
            return this.getValue();  
        },

        /**
         * Returns the picker text value.
         */
        getPickerTextValue: function(value)
        {
            return value.toString();  
        },
        
        /**
         * Gives the focus to the widget.
         */
        setFocus: function()
        {
            this._textNode.setFocus();
        },
        
        /**
         * Handles a change called if the text value has changed.
         * 
         *      @param value (String)
         *          the text value.
         */        
        onTextValueChanged: function(value)
        {
        },    
         
        /**
         * Opens the picker if closed, closes the picker if opened.
         */
        _onKeyUp: function(e)
        {
            if ( e.target === this._pickerButtonNode && e.keyCode === 32 ) {
                // manage like an onclick event
                this._onClick();
                // stop event propagation
                e.stopPropagation();
            }
        },
               
        /**
         * Opens the picker if closed, closes the picker if opened.
         */
        _onClick: function()
        {
		    // if disabled, skip
			
			//console.warn("PickerInputBox _onClick Start ");
		    if ( this._pickerIconDisabled || this.isDisabled() ) {
		        return;
		    }
		    // if the picker ig going to be opened, set its start value
		    if ( ! this._picker.isOpen() ) {
                var value = this.getValue();
                if ( value ) {
				//MODIFIED By PRASHANT SIROHI to Remove  Warning :DEPRECATED: dijit._TimePicker:setValue() is deprecated. Use attr('value') instead. -- will be removed in version: 2.0 
                    this._picker.attr('value',this.getValue());
                }
		    }
		    // toggle the picker
			
            this._picker.toggle();
			//console.warn("PickerInputBox _onClick End ");
        },
        
        /**
         * Handles the "picker value selected" event.
         * 
         *      @param value (Object):
         *          The value provided by the specific picker implementation.
         *          Its text form is asked to the picker implementation.
         */        
        _onPickerValueSelected: function(value)
        {
            // close the picker
            this._picker.close();
            // set the text value
            this._textNode.setValue(this.getPickerTextValue(value));
            // because the text box has not focus, it will not automatically
            // call the onChangeValue callback, do it
            this.onTextValueChanged(this._textNode.getValue());
        },

        /**
         * Handles onblur event.
         */
        _onBlur: function()
        {
            this._picker.close();
        }        
    }        
);


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.picker.Picker"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.picker.Picker"] = true;
dojo.provide("ibm.tivoli.tip.dijit.picker.Picker"); 



/**
 * The base interface for a picker. 
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.picker.Picker",
    [dijit._Widget],
    {
        
        /**
         * Opens the picker.
         */
        open: function()
        {
        },

        /**
         * Closes the picker.
         */
        close: function()
        {
        },
        
        /**
         * Opens the picker if closed, closes the picker if opened.
         */
        toggle: function()
        {
		   //console.log("Picker toggle Start ");
		   
            this.isOpen() ? this.close() : this.open();
			//console.log("Picker toggle End ");
        },
        
        /**
         * Returns true if the picker is opened.
         */
        isOpen: function()
        {
            return false;
        },        

        /**
         * Called back when a value has been selected from the picker.
         * 
         *      @param value: the value selected. 
         *          The value is provided by the picker and its
         *          type dependes on the specific picker implementation.
         */
        onValueSelected: function(value)
        {
        },
            
        /**
         * Sets the start-value to be shown by the picker when opened.
         * 
         *      @param value: the value to be shown by the picker. 
         *          The value's type dependes on the specific picker 
         *          implementation.
         */                    
        setValue: function(value)
        {
        }
    }        
);


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.picker.PopupPicker"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.picker.PopupPicker"] = true;
dojo.provide("ibm.tivoli.tip.dijit.picker.PopupPicker"); 

 

/**
 * A pop-up picker base class. 
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.picker.PopupPicker",
    [ibm.tivoli.tip.dijit.picker.Picker],
    {
        /** true if the picker is opened */
        _pickerOpened: false,
        /** the widget used to place the popup-picker */
        _parent: null,

        /**
         * Constructor.
         * 
         *      @param params: a set of parameters:
         *          - parent: the widget used to place the popup-picker.
         */
        constructor: function(params)
        {
            this._parent = params.parent;
        },
                       
        /**
         * Opens the picker.
         */
        open: function()
        {
		
		   //console.info("PopupPicker open START ");
		   
            // open the picker (if we've a parent and if not opened)
            if ( this._parent && ! this._pickerOpened ) {
                // open a popup containing the picker
				try {

                dijit.popup.open({
                    parent: this.parent,
                    popup: this,
                    around: this._parent.domNode,
                    onCancel: dojo.hitch(this, this.close),
                    onClose: dojo.hitch(this, function(){ this._pickerOpened = false; })
                });
				}
				catch (e) {
				   // need fix this error(p.getDisplayedValue is not a function) but it is coming in dojo code
					//console.warn(e);
				}

				// Following Line of code is Changed by Prashant Sirohi
                //this._pickerOpened = true;
                dojo.marginBox(this.domNode, { w: this.parent.domNode.offsetWidth });
            }
			//console.info("PopupPicker open END ");
        },

        /**
         * Closes the picker.
         */
        close: function()
        {
		    //console.info("close PopupPicker");
			// Following Lines of code is Changed by Prashant Sirohi
			
            /*if ( this._pickerOpened ) {
                dijit.popup.close(this);
                this._pickerOpened = false;
            }  */
			
			
			dijit.popup.close(this);
            			
        },
        
        /**
         * Returns true if the picker is opened.
         */
        isOpen: function()
        {
            return this._pickerOpened;
        }
    }        
);


}

if(!dojo._hasResource["dojo.cldr.supplemental"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cldr.supplemental"] = true;
dojo.provide("dojo.cldr.supplemental");



dojo.cldr.supplemental.getFirstDayOfWeek = function(/*String?*/locale){
// summary: Returns a zero-based index for first day of the week
// description:
//		Returns a zero-based index for first day of the week, as used by the local (Gregorian) calendar.
//		e.g. Sunday (returns 0), or Monday (returns 1)

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/firstDay
	var firstDay = {/*default is 1=Monday*/
		mv:5,
		af:6,bh:6,dj:6,dz:6,eg:6,er:6,et:6,iq:6,ir:6,jo:6,ke:6,kw:6,
		ly:6,ma:6,om:6,qa:6,sa:6,sd:6,so:6,tn:6,ye:6,
		ar:0,as:0,az:0,bw:0,ca:0,cn:0,fo:0,ge:0,gl:0,gu:0,hk:0,ie:0,
		il:0,'in':0,is:0,jm:0,jp:0,kg:0,kr:0,la:0,mh:0,mn:0,mo:0,mp:0,
		mt:0,nz:0,ph:0,pk:0,sg:0,sy:0,th:0,tt:0,tw:0,um:0,us:0,uz:0,
		vi:0,zw:0
// variant. do not use?		gb:0,
	};

	var country = dojo.cldr.supplemental._region(locale);
	var dow = firstDay[country];
	return (dow === undefined) ? 1 : dow; /*Number*/
};

dojo.cldr.supplemental._region = function(/*String?*/locale){
	locale = dojo.i18n.normalizeLocale(locale);
	var tags = locale.split('-');
	var region = tags[1];
	if(!region){
		// IE often gives language only (#2269)
		// Arbitrary mappings of language-only locales to a country:
		region = {de:"de", en:"us", es:"es", fi:"fi", fr:"fr", he:"il", hu:"hu", it:"it",
			ja:"jp", ko:"kr", nl:"nl", pt:"br", sv:"se", zh:"cn"}[tags[0]];
	}else if(region.length == 4){
		// The ISO 3166 country code is usually in the second position, unless a
		// 4-letter script is given. See http://www.ietf.org/rfc/rfc4646.txt
		region = tags[2];
	}
	return region;
}

dojo.cldr.supplemental.getWeekend = function(/*String?*/locale){
// summary: Returns a hash containing the start and end days of the weekend
// description:
//		Returns a hash containing the start and end days of the weekend according to local custom using locale,
//		or by default in the user's locale.
//		e.g. {start:6, end:0}

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/weekend{Start,End}
	var weekendStart = {/*default is 6=Saturday*/
		'in':0,
		af:4,dz:4,ir:4,om:4,sa:4,ye:4,
		ae:5,bh:5,eg:5,il:5,iq:5,jo:5,kw:5,ly:5,ma:5,qa:5,sd:5,sy:5,tn:5
	};

	var weekendEnd = {/*default is 0=Sunday*/
		af:5,dz:5,ir:5,om:5,sa:5,ye:5,
		ae:6,bh:5,eg:6,il:6,iq:6,jo:6,kw:6,ly:6,ma:6,qa:6,sd:6,sy:6,tn:6
	};

	var country = dojo.cldr.supplemental._region(locale);
	var start = weekendStart[country];
	var end = weekendEnd[country];
	if(start === undefined){start=6;}
	if(end === undefined){end=0;}
	return {start:start, end:end}; /*Object {start,end}*/
};

}

if(!dojo._hasResource["dojo.date"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date"] = true;
dojo.provide("dojo.date");

/*=====
dojo.date = {
	// summary: Date manipulation utilities
}
=====*/

dojo.date.getDaysInMonth = function(/*Date*/dateObject){
	//	summary:
	//		Returns the number of days in the month used by dateObject
	var month = dateObject.getMonth();
	var days = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
	if(month == 1 && dojo.date.isLeapYear(dateObject)){ return 29; } // Number
	return days[month]; // Number
}

dojo.date.isLeapYear = function(/*Date*/dateObject){
	//	summary:
	//		Determines if the year of the dateObject is a leap year
	//	description:
	//		Leap years are years with an additional day YYYY-02-29, where the
	//		year number is a multiple of four with the following exception: If
	//		a year is a multiple of 100, then it is only a leap year if it is
	//		also a multiple of 400. For example, 1900 was not a leap year, but
	//		2000 is one.

	var year = dateObject.getFullYear();
	return !(year%400) || (!(year%4) && !!(year%100)); // Boolean
}

// FIXME: This is not localized
dojo.date.getTimezoneName = function(/*Date*/dateObject){
	//	summary:
	//		Get the user's time zone as provided by the browser
	// dateObject:
	//		Needed because the timezone may vary with time (daylight savings)
	//	description:
	//		Try to get time zone info from toString or toLocaleString method of
	//		the Date object -- UTC offset is not a time zone.  See
	//		http://www.twinsun.com/tz/tz-link.htm Note: results may be
	//		inconsistent across browsers.

	var str = dateObject.toString(); // Start looking in toString
	var tz = ''; // The result -- return empty string if nothing found
	var match;

	// First look for something in parentheses -- fast lookup, no regex
	var pos = str.indexOf('(');
	if(pos > -1){
		tz = str.substring(++pos, str.indexOf(')'));
	}else{
		// If at first you don't succeed ...
		// If IE knows about the TZ, it appears before the year
		// Capital letters or slash before a 4-digit year 
		// at the end of string
		var pat = /([A-Z\/]+) \d{4}$/;
		if((match = str.match(pat))){
			tz = match[1];
		}else{
		// Some browsers (e.g. Safari) glue the TZ on the end
		// of toLocaleString instead of putting it in toString
			str = dateObject.toLocaleString();
			// Capital letters or slash -- end of string, 
			// after space
			pat = / ([A-Z\/]+)$/;
			if((match = str.match(pat))){
				tz = match[1];
			}
		}
	}

	// Make sure it doesn't somehow end up return AM or PM
	return (tz == 'AM' || tz == 'PM') ? '' : tz; // String
}

// Utility methods to do arithmetic calculations with Dates

dojo.date.compare = function(/*Date*/date1, /*Date?*/date2, /*String?*/portion){
	//	summary:
	//		Compare two date objects by date, time, or both.
	//	description:
	//  	Returns 0 if equal, positive if a > b, else negative.
	//	date1:
	//		Date object
	//	date2:
	//		Date object.  If not specified, the current Date is used.
	//	portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	// Extra step required in copy for IE - see #3112
	date1 = new Date(+date1);
	date2 = new Date(+(date2 || new Date()));

	if(portion == "date"){
		// Ignore times and compare dates.
		date1.setHours(0, 0, 0, 0);
		date2.setHours(0, 0, 0, 0);
	}else if(portion == "time"){
		// Ignore dates and compare times.
		date1.setFullYear(0, 0, 0);
		date2.setFullYear(0, 0, 0);
	}
	
	if(date1 > date2){ return 1; } // int
	if(date1 < date2){ return -1; } // int
	return 0; // int
};

dojo.date.add = function(/*Date*/date, /*String*/interval, /*int*/amount){
	//	summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	//	date: Date
	//		Date object to start with
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "quarter", "week", "weekday"
	//	amount:
	//		How much to add to the date.

	var sum = new Date(+date); // convert to Number before copying to accomodate IE (#3112)
	var fixOvershoot = false;
	var property = "Date";

	switch(interval){
		case "day":
			break;
		case "weekday":
			//i18n FIXME: assumes Saturday/Sunday weekend, but this is not always true.  see dojo.cldr.supplemental

			// Divide the increment time span into weekspans plus leftover days
			// e.g., 8 days is one 5-day weekspan / and two leftover days
			// Can't have zero leftover days, so numbers divisible by 5 get
			// a days value of 5, and the remaining days make up the number of weeks
			var days, weeks;
			var mod = amount % 5;
			if(!mod){
				days = (amount > 0) ? 5 : -5;
				weeks = (amount > 0) ? ((amount-5)/5) : ((amount+5)/5);
			}else{
				days = mod;
				weeks = parseInt(amount/5);
			}
			// Get weekday value for orig date param
			var strt = date.getDay();
			// Orig date is Sat / positive incrementer
			// Jump over Sun
			var adj = 0;
			if(strt == 6 && amount > 0){
				adj = 1;
			}else if(strt == 0 && amount < 0){
			// Orig date is Sun / negative incrementer
			// Jump back over Sat
				adj = -1;
			}
			// Get weekday val for the new date
			var trgt = strt + days;
			// New date is on Sat or Sun
			if(trgt == 0 || trgt == 6){
				adj = (amount > 0) ? 2 : -2;
			}
			// Increment by number of weeks plus leftover days plus
			// weekend adjustments
			amount = (7 * weeks) + days + adj;
			break;
		case "year":
			property = "FullYear";
			// Keep increment/decrement from 2/29 out of March
			fixOvershoot = true;
			break;
		case "week":
			amount *= 7;
			break;
		case "quarter":
			// Naive quarter is just three months
			amount *= 3;
			// fallthrough...
		case "month":
			// Reset to last day of month if you overshoot
			fixOvershoot = true;
			property = "Month";
			break;
//		case "hour":
//		case "minute":
//		case "second":
//		case "millisecond":
		default:
			property = "UTC"+interval.charAt(0).toUpperCase() + interval.substring(1) + "s";
	}

	if(property){
		sum["set"+property](sum["get"+property]()+amount);
	}

	if(fixOvershoot && (sum.getDate() < date.getDate())){
		sum.setDate(0);
	}

	return sum; // Date
};

dojo.date.difference = function(/*Date*/date1, /*Date?*/date2, /*String?*/interval){
	//	summary:
	//		Get the difference in a specific unit of time (e.g., number of
	//		months, weeks, days, etc.) between two dates, rounded to the
	//		nearest integer.
	//	date1:
	//		Date object
	//	date2:
	//		Date object.  If not specified, the current Date is used.
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "quarter", "week", "weekday"
	//		Defaults to "day".

	date2 = date2 || new Date();
	interval = interval || "day";
	var yearDiff = date2.getFullYear() - date1.getFullYear();
	var delta = 1; // Integer return value

	switch(interval){
		case "quarter":
			var m1 = date1.getMonth();
			var m2 = date2.getMonth();
			// Figure out which quarter the months are in
			var q1 = Math.floor(m1/3) + 1;
			var q2 = Math.floor(m2/3) + 1;
			// Add quarters for any year difference between the dates
			q2 += (yearDiff * 4);
			delta = q2 - q1;
			break;
		case "weekday":
			var days = Math.round(dojo.date.difference(date1, date2, "day"));
			var weeks = parseInt(dojo.date.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date1.getDay();
				var bDay = date2.getDay();

				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new Date(date1);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();

				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = -1;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 0;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -1;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 2;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			delta = (date2.getMonth() - date1.getMonth()) + (yearDiff * 12);
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(dojo.date.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date2.getTime() - date1.getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

}

if(!dojo._hasResource["dojo.date.locale"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date.locale"] = true;
dojo.provide("dojo.date.locale");

// Localization methods for Date.   Honor local customs using locale-dependent dojo.cldr data.







// Load the bundles containing localization information for
// names and formats


//NOTE: Everything in this module assumes Gregorian calendars.
// Other calendars will be implemented in separate modules.

(function(){
	// Format a pattern without literals
	function formatPattern(dateObject, bundle, options, pattern){
		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s, pad,
				c = match.charAt(0),
				l = match.length,
				widthList = ["abbr", "wide", "narrow"];
			switch(c){
				case 'G':
					s = bundle[(l < 4) ? "eraAbbr" : "eraNames"][dateObject.getFullYear() < 0 ? 0 : 1];
					break;
				case 'y':
					s = dateObject.getFullYear();
					switch(l){
						case 1:
							break;
						case 2:
							if(!options.fullYear){
								s = String(s); s = s.substr(s.length - 2);
								break;
							}
							// fallthrough
						default:
							pad = true;
					}
					break;
				case 'Q':
				case 'q':
					s = Math.ceil((dateObject.getMonth()+1)/3);
//					switch(l){
//						case 1: case 2:
							pad = true;
//							break;
//						case 3: case 4: // unimplemented
//					}
					break;
				case 'M':
					var m = dateObject.getMonth();
					if(l<3){
						s = m+1; pad = true;
					}else{
						var propM = ["months", "format", widthList[l-3]].join("-");
						s = bundle[propM][m];
					}
					break;
				case 'w':
					var firstDay = 0;
					s = dojo.date.locale._getWeekOfYear(dateObject, firstDay); pad = true;
					break;
				case 'd':
					s = dateObject.getDate(); pad = true;
					break;
				case 'D':
					s = dojo.date.locale._getDayOfYear(dateObject); pad = true;
					break;
				case 'E':
					var d = dateObject.getDay();
					if(l<3){
						s = d+1; pad = true;
					}else{
						var propD = ["days", "format", widthList[l-3]].join("-");
						s = bundle[propD][d];
					}
					break;
				case 'a':
					var timePeriod = (dateObject.getHours() < 12) ? 'am' : 'pm';
					s = bundle['dayPeriods-format-wide-' + timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					// strange choices in the date format make it impossible to write this succinctly
					switch (c){
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'v': // FIXME: don't know what this is. seems to be same as z?
				case 'z':
					// We only have one timezone to offer; the one from the browser
					s = dojo.date.locale._getZone(dateObject, true, options);
					if(s){break;}
					l=4;
					// fallthrough... use GMT if tz not available
				case 'Z':
					var offset = dojo.date.locale._getZone(dateObject, false, options);
					var tz = [
						(offset<=0 ? "+" : "-"),
						dojo.string.pad(Math.floor(Math.abs(offset)/60), 2),
						dojo.string.pad(Math.abs(offset)% 60, 2)
					];
					if(l==4){
						tz.splice(0, 0, "GMT");
						tz.splice(3, 0, ":");
					}
					s = tz.join("");
					break;
//				case 'Y': case 'u': case 'W': case 'F': case 'g': case 'A': case 'e':
//					console.log(match+" modifier unimplemented");
				default:
					throw new Error("dojo.date.locale.format: invalid pattern char: "+pattern);
			}
			if(pad){ s = dojo.string.pad(s, l); }
			return s;
		});
	}

/*=====
	dojo.date.locale.__FormatOptions = function(){
	//	selector: String
	//		choice of 'time','date' (default: date and time)
	//	formatLength: String
	//		choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	//	datePattern:String
	//		override pattern with this string
	//	timePattern:String
	//		override pattern with this string
	//	am: String
	//		override strings for am in times
	//	pm: String
	//		override strings for pm in times
	//	locale: String
	//		override the locale used to determine formatting rules
	//	fullYear: Boolean
	//		(format only) use 4 digit years whenever 2 digit years are called for
	//	strict: Boolean
	//		(parse only) strict parsing, off by default
		this.selector = selector;
		this.formatLength = formatLength;
		this.datePattern = datePattern;
		this.timePattern = timePattern;
		this.am = am;
		this.pm = pm;
		this.locale = locale;
		this.fullYear = fullYear;
		this.strict = strict;
	}
=====*/

dojo.date.locale._getZone = function(/*Date*/dateObject, /*boolean*/getName, /*dojo.date.locale.__FormatOptions?*/options){
	// summary:
	//		Returns the zone (or offset) for the given date and options.  This
	//		is broken out into a separate function so that it can be overridden
	//		by timezone-aware code.
	//
	// dateObject:
	//		the date and/or time being formatted.
	//
	// getName:
	//		Whether to return the timezone string (if true), or the offset (if false)
	//
	// options:
	//		The options being used for formatting
	if(getName){
		return dojo.date.getTimezoneName(dateObject);
	}else{
		return dateObject.getTimezoneOffset();
	}
};


dojo.date.locale.format = function(/*Date*/dateObject, /*dojo.date.locale.__FormatOptions?*/options){
	// summary:
	//		Format a Date object as a String, using locale-specific settings.
	//
	// description:
	//		Create a string from a Date object using a known localized pattern.
	//		By default, this method formats both date and time from dateObject.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the dojo.date.locale.addCustomFormats method.
	//		Formatting patterns are implemented using [the syntax described at
	//		unicode.org](http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns)
	//
	// dateObject:
	//		the date and/or time to be formatted.  If a time only is formatted,
	//		the values in the year, month, and day fields are irrelevant.  The
	//		opposite is true when formatting only dates.

	options = options || {};

	var locale = dojo.i18n.normalizeLocale(options.locale),
		formatLength = options.formatLength || 'short',
		bundle = dojo.date.locale._getGregorianBundle(locale),
		str = [],
		sauce = dojo.hitch(this, formatPattern, dateObject, bundle, options);
	if(options.selector == "year"){
		return _processPattern(bundle["dateFormatItem-yyyy"] || "yyyy", sauce);
	}
	var pattern;
	if(options.selector != "date"){
		pattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(pattern){str.push(_processPattern(pattern, sauce));}
	}
	if(options.selector != "time"){
		pattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(pattern){str.push(_processPattern(pattern, sauce));}
	}

	return str.length == 1 ? str[0] : bundle["dateTimeFormat-"+formatLength].replace(/\{(\d+)\}/g,
		function(match, key){ return str[key]; }); // String
};

dojo.date.locale.regexp = function(/*dojo.date.locale.__FormatOptions?*/options){
	// summary:
	//		Builds the regular needed to parse a localized date

	return dojo.date.locale._parseInfo(options).regexp; // String
};

dojo.date.locale._parseInfo = function(/*dojo.date.locale.__FormatOptions?*/options){
	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale),
		bundle = dojo.date.locale._getGregorianBundle(locale),
		formatLength = options.formatLength || 'short',
		datePattern = options.datePattern || bundle["dateFormat-" + formatLength],
		timePattern = options.timePattern || bundle["timeFormat-" + formatLength],
		pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = bundle["dateTimeFormat-"+formatLength].replace(/\{(\d+)\}/g,
			function(match, key){ return [timePattern, datePattern][key]; });
	}

	var tokens = [],
		re = _processPattern(pattern, dojo.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};

dojo.date.locale.parse = function(/*String*/value, /*dojo.date.locale.__FormatOptions?*/options){
	// summary:
	//		Convert a properly formatted string to a primitive Date object,
	//		using locale-specific settings.
	//
	// description:
	//		Create a Date object from a string using a known localized pattern.
	//		By default, this method parses looking for both date and time in the string.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the dojo.date.locale.addCustomFormats method.
	//	
	//		Formatting patterns are implemented using [the syntax described at
	//		unicode.org](http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns)
	//		When two digit years are used, a century is chosen according to a sliding 
	//		window of 80 years before and 20 years after present year, for both `yy` and `yyyy` patterns.
	//		year < 100CE requires strict mode.
	//
	// value:
	//		A string representation of a date

	// remove non-printing bidi control chars from input and pattern
	var controlChars = /[\u200E\u200F\u202A\u202E]/g,
		info = dojo.date.locale._parseInfo(options),
		tokens = info.tokens, bundle = info.bundle,
		re = new RegExp("^" + info.regexp.replace(controlChars, "") + "$",
			info.strict ? "" : "i"),
		match = re.exec(value && value.replace(controlChars, ""));

	if(!match){ return null; } // null

	var widthList = ['abbr', 'wide', 'narrow'],
		result = [1970,0,1,0,0,0,0], // will get converted to a Date at the end
		amPm = "",
		valid = dojo.every(match, function(v, i){
		if(!i){return true;}
		var token=tokens[i-1];
		var l=token.length;
		switch(token.charAt(0)){
			case 'y':
				if(l != 2 && options.strict){
					//interpret year literally, so '5' would be 5 A.D.
					result[0] = v;
				}else{
					if(v<100){
						v = Number(v);
						//choose century to apply, according to a sliding window
						//of 80 years before and 20 years after present year
						var year = '' + new Date().getFullYear(),
							century = year.substring(0, 2) * 100,
							cutoff = Math.min(Number(year.substring(2, 4)) + 20, 99),
							num = (v < cutoff) ? century + v : century - 100 + v;
						result[0] = num;
					}else{
						//we expected 2 digits and got more...
						if(options.strict){
							return false;
						}
						//interpret literally, so '150' would be 150 A.D.
						//also tolerate '1950', if 'yyyy' input passed to 'yy' format
						result[0] = v;
					}
				}
				break;
			case 'M':
				if(l>2){
					var months = bundle['months-format-' + widthList[l-3]].concat();
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = dojo.map(months, function(s){ return s.replace(".","").toLowerCase(); } );
					}
					v = dojo.indexOf(months, v);
					if(v == -1){
//						console.log("dojo.date.locale.parse: Could not parse month name: '" + v + "'.");
						return false;
					}
				}else{
					v--;
				}
				result[1] = v;
				break;
			case 'E':
			case 'e':
				var days = bundle['days-format-' + widthList[l-3]].concat();
				if(!options.strict){
					//Case-insensitive comparison
					v = v.toLowerCase();
					days = dojo.map(days, function(d){return d.toLowerCase();});
				}
				v = dojo.indexOf(days, v);
				if(v == -1){
//					console.log("dojo.date.locale.parse: Could not parse weekday name: '" + v + "'.");
					return false;
				}

				//TODO: not sure what to actually do with this input,
				//in terms of setting something on the Date obj...?
				//without more context, can't affect the actual date
				//TODO: just validate?
				break;
			case 'D':
				result[1] = 0;
				// fallthrough...
			case 'd':
				result[2] = v;
				break;
			case 'a': //am/pm
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
//					console.log("dojo.date.locale.parse: Could not parse am/pm part.");
					return false;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v == 24){ v = 0; }
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//TODO: strict bounds checking, padding
				if(v > 23){
//					console.log("dojo.date.locale.parse: Illegal hours value");
					return false;
				}

				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result[3] = v;
				break;
			case 'm': //minutes
				result[4] = v;
				break;
			case 's': //seconds
				result[5] = v;
				break;
			case 'S': //milliseconds
				result[6] = v;
//				break;
//			case 'w':
//TODO				var firstDay = 0;
//			default:
//TODO: throw?
//				console.log("dojo.date.locale.parse: unsupported pattern char=" + token.charAt(0));
		}
		return true;
	});

	var hours = +result[3];
	if(amPm === 'p' && hours < 12){
		result[3] = hours + 12; //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result[3] = 0; //12am -> 0
	}

	//TODO: implement a getWeekday() method in order to test 
	//validity of input strings containing 'EEE' or 'EEEE'...

	var dateObject = new Date(result[0], result[1], result[2], result[3], result[4], result[5], result[6]); // Date
	if(options.strict){
		dateObject.setFullYear(result[0]);
	}

	// Check for overflow.  The Date() constructor normalizes things like April 32nd...
	//TODO: why isn't this done for times as well?
	var allTokens = tokens.join(""),
		dateToken = allTokens.indexOf('d') != -1,
		monthToken = allTokens.indexOf('M') != -1;

	if(!valid ||
		(monthToken && dateObject.getMonth() > result[1]) ||
		(dateToken && dateObject.getDate() > result[2])){
		return null;
	}

	// Check for underflow, due to DST shifts.  See #9366
	// This assumes a 1 hour dst shift correction at midnight
	// We could compare the timezone offset after the shift and add the difference instead.
	if((monthToken && dateObject.getMonth() < result[1]) ||
		(dateToken && dateObject.getDate() < result[2])){
		dateObject = dojo.date.add(dateObject, "hour", 1);
	}

	return dateObject; // Date
};

function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings) 
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g),
		literal = pattern.charAt(0) == "'";

	dojo.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk.replace(/''/g, "'"));
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE(tokens, bundle, options, pattern){
	pattern = dojo.regexp.escapeString(pattern);
	if(!options.strict){ pattern = pattern.replace(" a", " ?a"); } // kludge to tolerate no space before am/pm
	return pattern.replace(/([a-z])\1*/ig, function(match){
		// Build a simple regexp.  Avoid captures, which would ruin the tokens list
		var s,
			c = match.charAt(0),
			l = match.length,
			p2 = '', p3 = '';
		if(options.strict){
			if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
			if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
		}else{
			p2 = '0?'; p3 = '0{0,2}';
		}
		switch(c){
			case 'y':
				s = '\\d{2,4}';
				break;
			case 'M':
				s = (l>2) ? '\\S+?' : p2+'[1-9]|1[0-2]';
				break;
			case 'D':
				s = p2+'[1-9]|'+p3+'[1-9][0-9]|[12][0-9][0-9]|3[0-5][0-9]|36[0-6]';
				break;
			case 'd':
				s = '3[01]|[12]\\d|'+p2+'[1-9]';
				break;
			case 'w':
				s = p2+'[1-9]|[1-4][0-9]|5[0-3]';
				break;
			case 'E':
				s = '\\S+';
				break;
			case 'h': //hour (1-12)
				s = p2+'[1-9]|1[0-2]';
				break;
			case 'k': //hour (0-11)
				s = p2+'\\d|1[01]';
				break;
			case 'H': //hour (0-23)
				s = p2+'\\d|1\\d|2[0-3]';
				break;
			case 'K': //hour (1-24)
				s = p2+'[1-9]|1\\d|2[0-4]';
				break;
			case 'm':
			case 's':
				s = '[0-5]\\d';
				break;
			case 'S':
				s = '\\d{'+l+'}';
				break;
			case 'a':
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
				if(options.strict){
					s = am + '|' + pm;
				}else{
					s = am + '|' + pm;
					if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
					if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
					if(s.indexOf('.') != -1){ s += '|' + s.replace(/\./g, ""); }
				}
				s = s.replace(/\./g, "\\.");
				break;
			default:
			// case 'v':
			// case 'z':
			// case 'Z':
				s = ".*";
//				console.log("parse of date format, pattern=" + pattern);
		}

		if(tokens){ tokens.push(match); }

		return "(" + s + ")"; // add capture
	}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE.
}
})();

(function(){
var _customFormats = [];
dojo.date.locale.addCustomFormats = function(/*String*/packageName, /*String*/bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	//
	// description:
	//		The user may add custom localized formats where the bundle has properties following the
	//		same naming convention used by dojo.cldr: `dateFormat-xxxx` / `timeFormat-xxxx`
	//		The pattern string should match the format used by the CLDR.
	//		See dojo.date.locale.format() for details.
	//		The resources must be loaded by dojo.requireLocalization() prior to use

	_customFormats.push({pkg:packageName,name:bundleName});
};

dojo.date.locale._getGregorianBundle = function(/*String*/locale){
	var gregorian = {};
	dojo.forEach(_customFormats, function(desc){
		var bundle = dojo.i18n.getLocalization(desc.pkg, desc.name, locale);
		gregorian = dojo.mixin(gregorian, bundle);
	}, this);
	return gregorian; /*Object*/
};
})();

dojo.date.locale.addCustomFormats("dojo.cldr","gregorian");

dojo.date.locale.getNames = function(/*String*/item, /*String*/type, /*String?*/context, /*String?*/locale){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	//
	// item:
	//	'months' || 'days'
	// type:
	//	'wide' || 'narrow' || 'abbr' (e.g. "Monday", "Mon", or "M" respectively, in English)
	// context:
	//	'standAlone' || 'format' (default)
	// locale:
	//	override locale used to find the names

	var label,
		lookup = dojo.date.locale._getGregorianBundle(locale),
		props = [item, context, type];
	if(context == 'standAlone'){
		var key = props.join('-');
		label = lookup[key];
		// Fall back to 'format' flavor of name
		if(label[0] == 1){ label = undefined; } // kludge, in the absence of real aliasing support in dojo.cldr
	}
	props[1] = 'format';

	// return by copy so changes won't be made accidentally to the in-memory model
	return (label || lookup[props.join('-')]).concat(); /*Array*/
};

dojo.date.locale.isWeekend = function(/*Date?*/dateObject, /*String?*/locale){
	// summary:
	//	Determines if the date falls on a weekend, according to local custom.

	var weekend = dojo.cldr.supplemental.getWeekend(locale),
		day = (dateObject || new Date()).getDay();
	if(weekend.end < weekend.start){
		weekend.end += 7;
		if(day < weekend.start){ day += 7; }
	}
	return day >= weekend.start && day <= weekend.end; // Boolean
};

// These are used only by format and strftime.  Do they need to be public?  Which module should they go in?

dojo.date.locale._getDayOfYear = function(/*Date*/dateObject){
	// summary: gets the day of the year as represented by dateObject
	return dojo.date.difference(new Date(dateObject.getFullYear(), 0, 1, dateObject.getHours()), dateObject) + 1; // Number
};

dojo.date.locale._getWeekOfYear = function(/*Date*/dateObject, /*Number*/firstDayOfWeek){
	if(arguments.length == 1){ firstDayOfWeek = 0; } // Sunday

	var firstDayOfYear = new Date(dateObject.getFullYear(), 0, 1).getDay(),
		adj = (firstDayOfYear - firstDayOfWeek + 7) % 7,
		week = Math.floor((dojo.date.locale._getDayOfYear(dateObject) + adj - 1) / 7);

	// if year starts on the specified day, start counting weeks at 1
	if(firstDayOfYear == firstDayOfWeek){ week++; }

	return week; // Number
};

}

if(!dojo._hasResource["dijit.Calendar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Calendar"] = true;
dojo.provide("dijit.Calendar");









dojo.declare(
	"dijit.Calendar",
	[dijit._Widget, dijit._Templated, dijit._CssStateMixin],
	{
		// summary:
		//		A simple GUI for choosing a date in the context of a monthly calendar.
		//
		// description:
		//		A simple GUI for choosing a date in the context of a monthly calendar.
		//		This widget can't be used in a form because it doesn't serialize the date to an
		//		`<input>` field.  For a form element, use dijit.form.DateTextBox instead.
		//
		//		Note that the parser takes all dates attributes passed in the
		//		[RFC 3339 format](http://www.faqs.org/rfcs/rfc3339.html), e.g. `2005-06-30T08:05:00-07:00`
		//		so that they are serializable and locale-independent.
		//
		// example:
		//	|	var calendar = new dijit.Calendar({}, dojo.byId("calendarNode"));
		//
		// example:
		//	|	<div dojoType="dijit.Calendar"></div>

		templateString: dojo.cache("dijit", "templates/Calendar.html", "<table cellspacing=\"0\" cellpadding=\"0\" class=\"dijitCalendarContainer\" role=\"grid\" dojoAttachEvent=\"onkeypress: _onKeyPress\">\r\n\t<thead>\r\n\t\t<tr class=\"dijitReset dijitCalendarMonthContainer\" valign=\"top\">\r\n\t\t\t<th class='dijitReset dijitCalendarArrow' dojoAttachPoint=\"decrementMonth\">\r\n\t\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitCalendarIncrementControl dijitCalendarDecrease\" waiRole=\"presentation\"/>\r\n\t\t\t\t<span dojoAttachPoint=\"decreaseArrowNode\" class=\"dijitA11ySideArrow\">-</span>\r\n\t\t\t</th>\r\n\t\t\t<th class='dijitReset' colspan=\"5\">\r\n\t\t\t\t<div class=\"dijitVisible\">\r\n\t\t\t\t\t<div class=\"dijitPopup dijitMenu dijitMenuPassive dijitHidden\" dojoAttachPoint=\"monthDropDown\" dojoAttachEvent=\"onmouseup: _onMonthSelect, onmouseover: _onMenuHover, onmouseout: _onMenuHover\">\r\n\t\t\t\t\t\t<div class=\"dijitCalendarMonthLabelTemplate dijitCalendarMonthLabel\"></div>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\t\t\t\t<div dojoAttachPoint=\"monthLabelSpacer\" class=\"dijitSpacer\"></div>\r\n\t\t\t\t<div dojoAttachPoint=\"monthLabelNode\" class=\"dijitCalendarMonthLabel dijitInline dijitVisible\" dojoAttachEvent=\"onmousedown: _onMonthToggle\"></div>\r\n\t\t\t</th>\r\n\t\t\t<th class='dijitReset dijitCalendarArrow' dojoAttachPoint=\"incrementMonth\">\r\n\t\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitCalendarIncrementControl dijitCalendarIncrease\" waiRole=\"presentation\"/>\r\n\t\t\t\t<span dojoAttachPoint=\"increaseArrowNode\" class=\"dijitA11ySideArrow\">+</span>\r\n\t\t\t</th>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th class=\"dijitReset dijitCalendarDayLabelTemplate\" role=\"columnheader\"><span class=\"dijitCalendarDayLabel\"></span></th>\r\n\t\t</tr>\r\n\t</thead>\r\n\t<tbody dojoAttachEvent=\"onclick: _onDayClick, onmouseover: _onDayMouseOver, onmouseout: _onDayMouseOut, onmousedown: _onDayMouseDown, onmouseup: _onDayMouseUp\" class=\"dijitReset dijitCalendarBodyContainer\">\r\n\t\t<tr class=\"dijitReset dijitCalendarWeekTemplate\" role=\"row\">\r\n\t\t\t<td class=\"dijitReset dijitCalendarDateTemplate\" role=\"gridcell\"><span class=\"dijitCalendarDateLabel\"></span></td>\r\n\t\t</tr>\r\n\t</tbody>\r\n\t<tfoot class=\"dijitReset dijitCalendarYearContainer\">\r\n\t\t<tr>\r\n\t\t\t<td class='dijitReset' valign=\"top\" colspan=\"7\">\r\n\t\t\t\t<h3 class=\"dijitCalendarYearLabel\">\r\n\t\t\t\t\t<span dojoAttachPoint=\"previousYearLabelNode\" class=\"dijitInline dijitCalendarPreviousYear\"></span>\r\n\t\t\t\t\t<span dojoAttachPoint=\"currentYearLabelNode\" class=\"dijitInline dijitCalendarSelectedYear\"></span>\r\n\t\t\t\t\t<span dojoAttachPoint=\"nextYearLabelNode\" class=\"dijitInline dijitCalendarNextYear\"></span>\r\n\t\t\t\t</h3>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t</tfoot>\r\n</table>\r\n"),

		// value: Date
		//		The currently selected Date
		value: new Date(),

		// datePackage: String
		//		JavaScript namespace to find Calendar routines.  Uses Gregorian Calendar routines
		//		at dojo.date by default.
		datePackage: "dojo.date",

		// dayWidth: String
		//		How to represent the days of the week in the calendar header. See dojo.date.locale
		dayWidth: "narrow",

		// tabIndex: Integer
		//		Order fields are traversed when user hits the tab key
		tabIndex: "0",
		
		baseClass:"dijitCalendar",

		// Set node classes for various mouse events, see dijit._CssStateMixin for more details 
		cssStateNodes: {
			"decrementMonth": "dijitCalendarArrow",
			"incrementMonth": "dijitCalendarArrow",
			"previousYearLabelNode": "dijitCalendarPreviousYear",
			"nextYearLabelNode": "dijitCalendarNextYear"			
		},

		attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
			tabIndex: "domNode"
 		}),

		setValue: function(/*Date*/ value){
			// summary:
			//      Deprecated.   Used attr('value', ...) instead.
			// tags:
			//      deprecated
			dojo.deprecated("dijit.Calendar:setValue() is deprecated.  Use set('value', ...) instead.", "", "2.0");
			this.set('value', value);
		},

		_getValueAttr: function(){
			// summary:
			//		Support getter attr('value')
			var value = new this.dateClassObj(this.value);
			value.setHours(0, 0, 0, 0); // return midnight, local time for back-compat

			// If daylight savings pushes midnight to the previous date, fix the Date
			// object to point at 1am so it will represent the correct day. See #9366
			if(value.getDate() < this.value.getDate()){
				value = this.dateFuncObj.add(value, "hour", 1);
			}
			return value;
		},

		_setValueAttr: function(/*Date*/ value){
			// summary:
			//		Support setter attr("value", ...)
			// description:
			// 		Set the current date and update the UI.  If the date is disabled, the value will
			//		not change, but the display will change to the corresponding month.
			// tags:
			//      protected
			if(!this.value || this.dateFuncObj.compare(value, this.value)){
				value = new this.dateClassObj(value);
				value.setHours(1); // to avoid issues when DST shift occurs at midnight, see #8521, #9366
				this.displayMonth = new this.dateClassObj(value);
				if(!this.isDisabledDate(value, this.lang)){
					this.value = value;
					this.onChange(this.get('value'));
				}
				dojo.attr(this.domNode, "aria-label",
					this.dateLocaleModule.format(value,
						{selector:"date", formatLength:"full"}));
				this._populateGrid();
			}
		},

		_setText: function(node, text){
			// summary:
			//		This just sets the content of node to the specified text.
			//		Can't do "node.innerHTML=text" because of an IE bug w/tables, see #3434.
			// tags:
			//      private
			while(node.firstChild){
				node.removeChild(node.firstChild);
			}
			node.appendChild(dojo.doc.createTextNode(text));
		},

		_populateGrid: function(){
			// summary:
			//      Fills in the calendar grid with each day (1-31)
			// tags:
			//      private
			var month = this.displayMonth;
			month.setDate(1);
			var firstDay = month.getDay(),
				daysInMonth = this.dateFuncObj.getDaysInMonth(month),
				daysInPreviousMonth = this.dateFuncObj.getDaysInMonth(this.dateFuncObj.add(month, "month", -1)),
				today = new this.dateClassObj(),
				dayOffset = dojo.cldr.supplemental.getFirstDayOfWeek(this.lang);
			if(dayOffset > firstDay){ dayOffset -= 7; }

			// Iterate through dates in the calendar and fill in date numbers and style info
			dojo.query(".dijitCalendarDateTemplate", this.domNode).forEach(function(template, i){
				i += dayOffset;
				var date = new this.dateClassObj(month),
					number, clazz = "dijitCalendar", adj = 0;

				if(i < firstDay){
					number = daysInPreviousMonth - firstDay + i + 1;
					adj = -1;
					clazz += "Previous";
				}else if(i >= (firstDay + daysInMonth)){
					number = i - firstDay - daysInMonth + 1;
					adj = 1;
					clazz += "Next";
				}else{
					number = i - firstDay + 1;
					clazz += "Current";
				}

				if(adj){
					date = this.dateFuncObj.add(date, "month", adj);
				}
				date.setDate(number);

				if(!this.dateFuncObj.compare(date, today, "date")){
					clazz = "dijitCalendarCurrentDate " + clazz;
				}

				if(this._isSelectedDate(date, this.lang)){
					clazz = "dijitCalendarSelectedDate " + clazz;
				}

				if(this.isDisabledDate(date, this.lang)){
					clazz = "dijitCalendarDisabledDate " + clazz;
				}

				var clazz2 = this.getClassForDate(date, this.lang);
				if(clazz2){
					clazz = clazz2 + " " + clazz;
				}

				template.className = clazz + "Month dijitCalendarDateTemplate";
				template.dijitDateValue = date.valueOf();
				var label = dojo.query(".dijitCalendarDateLabel", template)[0],
					text = date.getDateLocalized ? date.getDateLocalized(this.lang) : date.getDate();
				this._setText(label, text);
			}, this);

			// Fill in localized month name
			var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month);
			this._setText(this.monthLabelNode, monthNames[month.getMonth()]);
			// Repopulate month list based on current year (Hebrew calendar)
			dojo.query(".dijitCalendarMonthLabelTemplate", this.domNode).forEach(function(node, i){
				dojo.toggleClass(node, "dijitHidden", !(i in monthNames)); // hide leap months (Hebrew)
				this._setText(node, monthNames[i]);
			}, this);

			// Fill in localized prev/current/next years
			var y = month.getFullYear() - 1;
			var d = new this.dateClassObj();
			dojo.forEach(["previous", "current", "next"], function(name){
				d.setFullYear(y++);
				this._setText(this[name+"YearLabelNode"],
					this.dateLocaleModule.format(d, {selector:'year', locale:this.lang}));
			}, this);
		},

		goToToday: function(){
			// summary:
			//      Sets calendar's value to today's date
			this.set('value', new this.dateClassObj());
		},

		constructor: function(/*Object*/args){
			var dateClass = (args.datePackage && (args.datePackage != "dojo.date"))? args.datePackage + ".Date" : "Date";
			this.dateClassObj = dojo.getObject(dateClass, false);
			this.datePackage = args.datePackage || this.datePackage;
			this.dateFuncObj = dojo.getObject(this.datePackage, false);
			this.dateLocaleModule = dojo.getObject(this.datePackage + ".locale", false);
		},

		postMixInProperties: function(){
			// parser.instantiate sometimes passes in NaN for IE.  Use default value in prototype instead.
			if(isNaN(this.value)){ delete this.value; }
			this.inherited(arguments);
		},

		postCreate: function(){
			this.inherited(arguments);
			dojo.setSelectable(this.domNode, false);

			var cloneClass = dojo.hitch(this, function(clazz, n){
				var template = dojo.query(clazz, this.domNode)[0];
	 			for(var i=0; i<n; i++){
					template.parentNode.appendChild(template.cloneNode(true));
				}
			});

			// clone the day label and calendar day templates 6 times to make 7 columns
			cloneClass(".dijitCalendarDayLabelTemplate", 6);
			cloneClass(".dijitCalendarDateTemplate", 6);

			// now make 6 week rows
			cloneClass(".dijitCalendarWeekTemplate", 5);

			// insert localized day names in the header
			var dayNames = this.dateLocaleModule.getNames('days', this.dayWidth, 'standAlone', this.lang);
			var dayOffset = dojo.cldr.supplemental.getFirstDayOfWeek(this.lang);
			dojo.query(".dijitCalendarDayLabel", this.domNode).forEach(function(label, i){
				this._setText(label, dayNames[(i + dayOffset) % 7]);
			}, this);

			var dateObj = new this.dateClassObj(this.value);
			// Fill in spacer/month dropdown element with all the month names (invisible) so that the maximum width will affect layout
			var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, dateObj);
			cloneClass(".dijitCalendarMonthLabelTemplate", monthNames.length-1);
			dojo.query(".dijitCalendarMonthLabelTemplate", this.domNode).forEach(function(node, i){
				dojo.attr(node, "month", i);
				if(i in monthNames){ this._setText(node, monthNames[i]); }
				dojo.place(node.cloneNode(true), this.monthLabelSpacer);
			}, this);

			this.value = null;
			this.set('value', dateObj);

			// Set up repeating mouse behavior
			var _this = this;
			var typematic = function(nodeProp, dateProp, adj){
				_this._connects.push(
					dijit.typematic.addMouseListener(_this[nodeProp], _this, function(count){
						if(count >= 0){ _this._adjustDisplay(dateProp, adj); }
					}, 0.8, 500)
				);
			};
			typematic("incrementMonth", "month", 1);
			typematic("decrementMonth", "month", -1);
			typematic("nextYearLabelNode", "year", 1);
			typematic("previousYearLabelNode", "year", -1);
		},

		_onMenuHover: function(e){
			dojo.stopEvent(e);
			dojo.toggleClass(e.target, "dijitMenuItemHover");
		},

		_adjustDisplay: function(/*String*/ part, /*int*/ amount){
			// summary:
			//      Moves calendar forwards or backwards by months or years
			// part:
			//      "month" or "year"
			// amount:
			//      Number of months or years
			// tags:
			//      private
			this.displayMonth = this.dateFuncObj.add(this.displayMonth, part, amount);
			this._populateGrid();
		},

		_onMonthToggle: function(/*Event*/ evt){
			// summary:
			//      Handler for when user triggers or dismisses the month list
			// tags:
			//      protected
			dojo.stopEvent(evt);

			if(evt.type == "mousedown"){
				var coords = dojo.position(this.monthLabelNode);
//				coords.y -= dojo.position(this.domNode, true).y;
				// Size the dropdown's width to match the label in the widget
				// so that they are horizontally aligned
				var dim = {
					width: coords.w + "px",
					top: -this.displayMonth.getMonth() * coords.h + "px"
				};
				if((dojo.isIE && dojo.isQuirks) || dojo.isIE < 7){
					dim.left = -coords.w/2 + "px";
				}
				dojo.style(this.monthDropDown, dim);
				this._popupHandler = this.connect(document, "onmouseup", "_onMonthToggle");
			}else{
				this.disconnect(this._popupHandler);
				delete this._popupHandler;
			}

			dojo.toggleClass(this.monthDropDown, "dijitHidden");
			dojo.toggleClass(this.monthLabelNode, "dijitVisible");
		},

		_onMonthSelect: function(/*Event*/ evt){
			// summary:
			//      Handler for when user selects a month from a list
			// tags:
			//      protected
			this._onMonthToggle(evt);
			this.displayMonth.setMonth(dojo.attr(evt.target, "month"));
			this._populateGrid();
		},

		_onDayClick: function(/*Event*/ evt){
			// summary:
			//      Handler for day clicks, selects the date if appropriate
			// tags:
			//      protected
			dojo.stopEvent(evt);
			for(var node = evt.target; node && !node.dijitDateValue; node = node.parentNode);
			if(node && !dojo.hasClass(node, "dijitCalendarDisabledDate")){
				this.set('value', node.dijitDateValue);
				this.onValueSelected(this.get('value'));
			}
		},

		_onDayMouseOver: function(/*Event*/ evt){
			// summary:
			//      Handler for mouse over events on days, sets hovered style
			// tags:
			//      protected

			// event can occur on <td> or the <span> inside the td,
			// set node to the <td>.
			var node =
				dojo.hasClass(evt.target, "dijitCalendarDateLabel") ?
				evt.target.parentNode :
				evt.target;

			if(node && (node.dijitDateValue || node == this.previousYearLabelNode || node == this.nextYearLabelNode) ){
				dojo.addClass(node, "dijitCalendarHoveredDate");
				this._currentNode = node;
			}
		},

		_onDayMouseOut: function(/*Event*/ evt){
			// summary:
			//      Handler for mouse out events on days, clears hovered style
			// tags:
			//      protected
	
			if(!this._currentNode){ return; }
			
			// if mouse out occurs moving from <td> to <span> inside <td>, ignore it
			if(evt.relatedTarget && evt.relatedTarget.parentNode == this._currentNode){ return; }

			dojo.removeClass(this._currentNode, "dijitCalendarHoveredDate");
			if(dojo.hasClass(this._currentNode, "dijitCalendarActiveDate")) {
				dojo.removeClass(this._currentNode, "dijitCalendarActiveDate");
			}
			this._currentNode = null;
		},
		
		_onDayMouseDown: function(/*Event*/ evt){
			var node = evt.target.parentNode;
			if(node && node.dijitDateValue){
				dojo.addClass(node, "dijitCalendarActiveDate");
				this._currentNode = node;
			}
		},
		
		_onDayMouseUp: function(/*Event*/ evt){
			var node = evt.target.parentNode;
			if(node && node.dijitDateValue){
				dojo.removeClass(node, "dijitCalendarActiveDate");
			}
		},

//TODO: use typematic
//TODO: skip disabled dates without ending up in a loop
//TODO: could optimize by avoiding populate grid when month does not change
		_onKeyPress: function(/*Event*/evt){
			// summary:
			//		Provides keyboard navigation of calendar
			// tags:
			//		protected
			var dk = dojo.keys,
				increment = -1,
				interval,
				newValue = this.value;
			switch(evt.keyCode){
				case dk.RIGHT_ARROW:
					increment = 1;
					//fallthrough...
				case dk.LEFT_ARROW:
					interval = "day";
					if(!this.isLeftToRight()){ increment *= -1; }
					break;
				case dk.DOWN_ARROW:
					increment = 1;
					//fallthrough...
				case dk.UP_ARROW:
					interval = "week";
					break;
				case dk.PAGE_DOWN:
					increment = 1;
					//fallthrough...
				case dk.PAGE_UP:
					interval = evt.ctrlKey ? "year" : "month";
					break;
				case dk.END:
					// go to the next month
					newValue = this.dateFuncObj.add(newValue, "month", 1);
					// subtract a day from the result when we're done
					interval = "day";
					//fallthrough...
				case dk.HOME:
					newValue = new Date(newValue).setDate(1);
					break;
				case dk.ENTER:
					this.onValueSelected(this.get('value'));
					break;
				case dk.ESCAPE:
					//TODO
				default:
					return;
			}
			dojo.stopEvent(evt);

			if(interval){
				newValue = this.dateFuncObj.add(newValue, interval, increment);
			}

			this.set("value", newValue);
		},

		onValueSelected: function(/*Date*/ date){
			// summary:
			//		Notification that a date cell was selected.  It may be the same as the previous value.
			// description:
			//      Used by `dijit.form._DateTimeTextBox` (and thus `dijit.form.DateTextBox`)
			//      to get notification when the user has clicked a date.
			// tags:
			//      protected
		},

		onChange: function(/*Date*/ date){
			// summary:
			//		Called only when the selected date has changed
		},

		_isSelectedDate: function(/*Date*/ dateObject, /*String?*/ locale){
			// summary:
			//		Extension point so developers can subclass Calendar to
			//		support multiple (concurrently) selected dates
			// tags:
			//		protected extension
			return !this.dateFuncObj.compare(dateObject, this.value, "date")
		},

		isDisabledDate: function(/*Date*/ dateObject, /*String?*/ locale){
			// summary:
			//		May be overridden to disable certain dates in the calendar e.g. `isDisabledDate=dojo.date.locale.isWeekend`
			// tags:
			//      extension
/*=====
			return false; // Boolean
=====*/
		},

		getClassForDate: function(/*Date*/ dateObject, /*String?*/ locale){
			// summary:
			//		May be overridden to return CSS classes to associate with the date entry for the given dateObject,
			//		for example to indicate a holiday in specified locale.
			// tags:
			//      extension

/*=====
			return ""; // String
=====*/
		}
	}
);

}

if(!dojo._hasResource["dijit._Calendar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Calendar"] = true;
dojo.provide("dijit._Calendar");


dojo.deprecated("dijit._Calendar is deprecated", "dijit._Calendar moved to dijit.Calendar", 1.5);

// dijit._Calendar had an underscore all this time merely because it did
// not satisfy dijit's a11y policy.
dijit._Calendar = dijit.Calendar;

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.picker.DatePicker"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.picker.DatePicker"] = true;
dojo.provide("ibm.tivoli.tip.dijit.picker.DatePicker"); 

 


/**
 * The date-picker popup.  
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.picker.DatePicker",
    [ibm.tivoli.tip.dijit.picker.PopupPicker, dijit._Calendar],
    {
        /** constraints */
        constraints: {selector: "date"}
    }        
);


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.DateInputBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.DateInputBox"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dijit.DateInputBox"); 





/** 
 * A date input box widget.
 * 
 * Such widget shows a text input box and a calendar picker icon on the right.
 * By clicking on such icon a calendar picker is shown to help the user
 * to select a date.
 * 
 * The widget supports the following attributes:
 * 
 *      - value (Date | String): the initial value.
 *      - required (String): "true" if the date is required.
 *      - tabindex (Number): the widget tabindex.
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)   
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.DateInputBox",
    [ibm.tivoli.tip.dijit.picker.PickerInputBox],
    {
        /** the initial value (Date or String) */
        value: null,

        /** the picker class */
        pickerClass: "ibm.tivoli.tip.dijit.picker.DatePicker",
        /** the picker icon */
        pickerIcon: ibm.tivoli.tip.dijit.Images.get()["CALENDAR_ICON"],
        /** constraints */
        constraints: {selector: "date"},
        
        /** the value (Date) */
        _value: null,

        /**
         * Constructor.
         */
        constructor: function()
        {
            // set the regex for the text
            this.textRegExp = "^(" + dojo.date.locale.regexp(this.constraints) + "){0,1}$";
            // set the regex message for the text
            this.textRegExpMessage = this._resources.DATE_INVALID_VALUE;
        },
        
        /**
         * Creates the picker and sets the initial value.  
         */
        startup: function() 
        {
            // call the superclass's method
			//console.info("Startup Dateinputbox");
			
            this.inherited("startup", arguments);

            // get the picker protoype
            var PickerClass = dojo.getObject(this.pickerClass, false);
            // create the picker
            var picker = new PickerClass({parent: this._textNode});
            // set the picker
			
            this.setPicker(picker);
           
            // set the initial value
            this.setValue(this.value);
        },
		
        /**        
         * Returns a Date object representing the widget value.
         */
        getValue: function()
        {
            // return the value
            return this._value;
        },
        
        /**        
         * Sets the date value.
         * 
         *      @param value
         *          a Date object.
         */
        setValue: function(value)
        {
           //console.info(" DateInputBox setValue");
			if ( value === null ) {
                // nule value means null (empty) data
                this._value = null;
            }
            else {
                // in case of string parse to a date, otherwise a date object is assumed
                this._value = 
                    dojo.isString(value) ? 
                    dojo.date.locale.parse(value, this.constraints) : 
                    value;
            }
            
            // call the superclass's method 
            arguments[0] = 
                this._value === null ? 
                "" : 
                dojo.date.locale.format(value, this.constraints);
            
            this.inherited("setValue", arguments);
        },
        
        /**
         * Returns true if the widget's value is not set.
         */
        isEmpty: function()
        {
            return this._value === null;
        },

        /**        
         * Returns the text representation of the value (a String)
         */
        getTextValue: function()
        {
            // return the text value
            return ibm.tivoli.tip.dijit.DateInputBox.superclass.getValue.call(this);
        },
        
        /**
         * Returns the picker start value.
         */
        getPickerStartValue: function()
        {
            return this._value || new Date();  
        },
        
        /**
         * Returns the picker text value.
         */
        getPickerTextValue: function(value)
        {
            return dojo.date.locale.format(value, this.constraints);  
        },
        
        /**
         * Handles a change called if the text value has changed.
         * 
         *      @param value (String)
         *          the text value.
         */        
        onTextValueChanged: function(value)
        {
            this._value = dojo.date.locale.parse(value, this.constraints);
            this.onValueChanged(value);
        }        
    }        
);


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.DescriptionArea"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.DescriptionArea"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/
dojo.require( "dojo.fx" );
dojo.require( "dijit._Templated" );
dojo.require( "dijit.layout.ContentPane" );

dojo.provide( "ibm.tivoli.tip.dijit.DescriptionArea" );
/******************************************************************************
* Colorful description area for Portlet Descriptions per Tivoli Guidlines
*
* Use innerDIV attachpoint to add graphics to the the description area
* background.
*
* Can be opened and closed.
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.DescriptionArea", [dijit.layout.ContentPane, dijit._Templated],
{
  portletTitle: "Portlet Title Here",
  
  description: "Portlet description here...",

  imageSrc: "",
  
  backgroundClass: null,

  templateString: "<div class='tipDescriptionArea open'><div dojoAttachPoint='innerDIV'><div dojoAttachPoint='hideNode'><div class='dijitReset' dojoAttachPoint='wipeNode'><div dojoAttachPoint='containerNode'><table border='0' cellpadding='0' cellspacing='0' width='100%'><tr><td rowspan='2' dojoAttachPoint='imageNode'></td><td width='100%' class='tipDescriptionAreaTitle'>${portletTitle}</td></tr><tr><td  width='100%' valign='top' class='tipDescriptionAreaContent'>${description}</td></tr></table></div></div></div></div><div class='tipDescriptionAreaClose' dojoAttachPoint='closeNode' dojoAttachEvent=\"onclick:_toggle\"></div></div>",

  // open: Boolean
  //    Whether description is opened or closed on first render.
  open: true,

  // duration: Integer
  //    Time in milliseconds to wipe in/wipe out
  duration: 500,
  
  /*****************************************************************************
  * Setup initial styles and state of the widget
  * Create and save all animations
  *****************************************************************************/
  postCreate: function()
  {
    if( !this.open )
    {
      this.hideNode.style.display = this.wipeNode.style.display = "none";
      this._updateClass();    
    }

    this.inherited("postCreate",arguments);
    
    if ( this.imageSrc !== null )
    {
      var image = dojo.doc.createElement( 'img' );
      image.src = this.imageSrc;
      image.border = "0";
      this.imageNode.appendChild( image );
    }

    if ( this.backgroundClass !== null )
    {
      this.innerDIV.className = this.backgroundClass;
    }
  
    // setup open/close animations
    var hideNode = this.hideNode, 
        wipeNode = this.wipeNode;
        
    this._wipeIn = dojo.fx.wipeIn({
      node: this.wipeNode,
      duration: this.duration,
      beforeBegin: function(){
        hideNode.style.display="";
      }
    });
    
    this._wipeOut = dojo.fx.wipeOut({
      node: this.wipeNode,
      duration: this.duration,
      onEnd: function(){
        hideNode.style.display="none";
      }
    });
  },  

  /*****************************************************************************
  * The toggle open/close button div was clicked -- toggle open/close state
  *****************************************************************************/
  _toggle: function()
  {
    dojo.forEach([this._wipeIn, this._wipeOut], function(animation){
      if(animation.status() == "playing"){
        animation.stop();
      }
    });

    this[this.open ? "_wipeOut" : "_wipeIn"].play();
    this.open =! this.open;    
    this._updateClass();    
  },
  
  /*****************************************************************************
  * based on the open/close state of widget change CSS to update button
  *****************************************************************************/
  _updateClass: function()
  {
    if ( this.open === true )
    {
      dojo.removeClass( this.domNode, "close"  );
         dojo.addClass( this.domNode, "open" );          
    }
    else
    {
      dojo.removeClass( this.domNode, "open" );
         dojo.addClass( this.domNode, "close"  );          
    }  
  }  
});

}

if(!dojo._hasResource["dojo.dnd.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.common"] = true;
dojo.provide("dojo.dnd.common");

dojo.dnd.getCopyKeyState = dojo.isCopyKey;

dojo.dnd._uniqueId = 0;
dojo.dnd.getUniqueId = function(){
	// summary:
	//		returns a unique string for use with any DOM element
	var id;
	do{
		id = dojo._scopeName + "Unique" + (++dojo.dnd._uniqueId);
	}while(dojo.byId(id));
	return id;
};

dojo.dnd._empty = {};

dojo.dnd.isFormElement = function(/*Event*/ e){
	// summary:
	//		returns true if user clicked on a form element
	var t = e.target;
	if(t.nodeType == 3 /*TEXT_NODE*/){
		t = t.parentNode;
	}
	return " button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
};

}

if(!dojo._hasResource["dojo.dnd.Container"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Container"] = true;
dojo.provide("dojo.dnd.Container");




/*
	Container states:
		""		- normal state
		"Over"	- mouse over a container
	Container item states:
		""		- normal state
		"Over"	- mouse over a container item
*/

/*=====
dojo.declare("dojo.dnd.__ContainerArgs", [], {
	creator: function(){
		// summary:
		//		a creator function, which takes a data item, and returns an object like that:
		//		{node: newNode, data: usedData, type: arrayOfStrings}
	},

	// skipForm: Boolean
	//		don't start the drag operation, if clicked on form elements
	skipForm: false,

	// dropParent: Node||String
	//		node or node's id to use as the parent node for dropped items
	//		(must be underneath the 'node' parameter in the DOM)
	dropParent: null,

	// _skipStartup: Boolean
	//		skip startup(), which collects children, for deferred initialization
	//		(this is used in the markup mode)
	_skipStartup: false
});

dojo.dnd.Item = function(){
	// summary:
	//		Represents (one of) the source node(s) being dragged.
	//		Contains (at least) the "type" and "data" attributes.
	// type: String[]
	//		Type(s) of this item, by default this is ["text"]
	// data: Object
	//		Logical representation of the object being dragged.
	//		If the drag object's type is "text" then data is a String,
	//		if it's another type then data could be a different Object,
	//		perhaps a name/value hash.
	
	this.type = type;
	this.data = data;
}
=====*/

dojo.declare("dojo.dnd.Container", null, {
	// summary:
	//		a Container object, which knows when mouse hovers over it, 
	//		and over which element it hovers
	
	// object attributes (for markup)
	skipForm: false,
	
	/*=====
	// current: DomNode
	//		The DOM node the mouse is currently hovered over
	current: null,
	
	// map: Hash<String, dojo.dnd.Item>
	//		Map from an item's id (which is also the DOMNode's id) to
	//		the dojo.dnd.Item itself.
	map: {},
	=====*/
	
	constructor: function(node, params){
		// summary:
		//		a constructor of the Container
		// node: Node
		//		node or node's id to build the container on
		// params: dojo.dnd.__ContainerArgs
		//		a dictionary of parameters
		this.node = dojo.byId(node);
		if(!params){ params = {}; }
		this.creator = params.creator || null;
		this.skipForm = params.skipForm;
		this.parent = params.dropParent && dojo.byId(params.dropParent);
		
		// class-specific variables
		this.map = {};
		this.current = null;

		// states
		this.containerState = "";
		dojo.addClass(this.node, "dojoDndContainer");
		
		// mark up children
		if(!(params && params._skipStartup)){
			this.startup();
		}

		// set up events
		this.events = [
			dojo.connect(this.node, "onmouseover", this, "onMouseOver"),
			dojo.connect(this.node, "onmouseout",  this, "onMouseOut"),
			// cancel text selection and text dragging
			dojo.connect(this.node, "ondragstart",   this, "onSelectStart"),
			dojo.connect(this.node, "onselectstart", this, "onSelectStart")
		];
	},
	
	// object attributes (for markup)
	creator: function(){
		// summary:
		//		creator function, dummy at the moment
	},
	
	// abstract access to the map
	getItem: function(/*String*/ key){
		// summary:
		//		returns a data item by its key (id)
		return this.map[key];	// dojo.dnd.Item
	},
	setItem: function(/*String*/ key, /*dojo.dnd.Item*/ data){
		// summary:
		//		associates a data item with its key (id)
		this.map[key] = data;
	},
	delItem: function(/*String*/ key){
		// summary:
		//		removes a data item from the map by its key (id)
		delete this.map[key];
	},
	forInItems: function(/*Function*/ f, /*Object?*/ o){
		// summary:
		//		iterates over a data map skipping members that 
		//		are present in the empty object (IE and/or 3rd-party libraries).
		o = o || dojo.global;
		var m = this.map, e = dojo.dnd._empty;
		for(var i in m){
			if(i in e){ continue; }
			f.call(o, m[i], i, this);
		}
		return o;	// Object
	},
	clearItems: function(){
		// summary:
		//		removes all data items from the map
		this.map = {};
	},
	
	// methods
	getAllNodes: function(){
		// summary:
		//		returns a list (an array) of all valid child nodes
		return dojo.query("> .dojoDndItem", this.parent);	// NodeList
	},
	sync: function(){
		// summary:
		//		sync up the node list with the data map
		var map = {};
		this.getAllNodes().forEach(function(node){
			if(node.id){
				var item = this.getItem(node.id);
				if(item){
					map[node.id] = item;
					return;
				}
			}else{
				node.id = dojo.dnd.getUniqueId();
			}
			var type = node.getAttribute("dndType"),
				data = node.getAttribute("dndData");
			map[node.id] = {
				data: data || node.innerHTML,
				type: type ? type.split(/\s*,\s*/) : ["text"]
			};
		}, this);
		this.map = map;
		return this;	// self
	},
	insertNodes: function(data, before, anchor){
		// summary:
		//		inserts an array of new nodes before/after an anchor node
		// data: Array
		//		a list of data items, which should be processed by the creator function
		// before: Boolean
		//		insert before the anchor, if true, and after the anchor otherwise
		// anchor: Node
		//		the anchor node to be used as a point of insertion
		if(!this.parent.firstChild){
			anchor = null;
		}else if(before){
			if(!anchor){
				anchor = this.parent.firstChild;
			}
		}else{
			if(anchor){
				anchor = anchor.nextSibling;
			}
		}
		if(anchor){
			for(var i = 0; i < data.length; ++i){
				var t = this._normalizedCreator(data[i]);
				this.setItem(t.node.id, {data: t.data, type: t.type});
				this.parent.insertBefore(t.node, anchor);
			}
		}else{
			for(var i = 0; i < data.length; ++i){
				var t = this._normalizedCreator(data[i]);
				this.setItem(t.node.id, {data: t.data, type: t.type});
				this.parent.appendChild(t.node);
			}
		}
		return this;	// self
	},
	destroy: function(){
		// summary:
		//		prepares this object to be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		this.clearItems();
		this.node = this.parent = this.current = null;
	},

	// markup methods
	markupFactory: function(params, node){
		params._skipStartup = true;
		return new dojo.dnd.Container(node, params);
	},
	startup: function(){
		// summary:
		//		collects valid child items and populate the map
		
		// set up the real parent node
		if(!this.parent){
			// use the standard algorithm, if not assigned
			this.parent = this.node;
			if(this.parent.tagName.toLowerCase() == "table"){
				var c = this.parent.getElementsByTagName("tbody");
				if(c && c.length){ this.parent = c[0]; }
			}
		}
		this.defaultCreator = dojo.dnd._defaultCreator(this.parent);

		// process specially marked children
		this.sync();
	},

	// mouse events
	onMouseOver: function(e){
		// summary:
		//		event processor for onmouseover
		// e: Event
		//		mouse event
		var n = e.relatedTarget;
		while(n){
			if(n == this.node){ break; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(!n){
			this._changeState("Container", "Over");
			this.onOverEvent();
		}
		n = this._getChildByEvent(e);
		if(this.current == n){ return; }
		if(this.current){ this._removeItemClass(this.current, "Over"); }
		if(n){ this._addItemClass(n, "Over"); }
		this.current = n;
	},
	onMouseOut: function(e){
		// summary:
		//		event processor for onmouseout
		// e: Event
		//		mouse event
		for(var n = e.relatedTarget; n;){
			if(n == this.node){ return; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(this.current){
			this._removeItemClass(this.current, "Over");
			this.current = null;
		}
		this._changeState("Container", "");
		this.onOutEvent();
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skipForm || !dojo.dnd.isFormElement(e)){
			dojo.stopEvent(e);
		}
	},
	
	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
	},
	_changeState: function(type, newState){
		// summary:
		//		changes a named state to new state value
		// type: String
		//		a name of the state to change
		// newState: String
		//		new state
		var prefix = "dojoDnd" + type;
		var state  = type.toLowerCase() + "State";
		//dojo.replaceClass(this.node, prefix + newState, prefix + this[state]);
		dojo.removeClass(this.node, prefix + this[state]);
		dojo.addClass(this.node, prefix + newState);
		this[state] = newState;
	},
	_addItemClass: function(node, type){
		// summary:
		//		adds a class with prefix "dojoDndItem"
		// node: Node
		//		a node
		// type: String
		//		a variable suffix for a class name
		dojo.addClass(node, "dojoDndItem" + type);
	},
	_removeItemClass: function(node, type){
		// summary:
		//		removes a class with prefix "dojoDndItem"
		// node: Node
		//		a node
		// type: String
		//		a variable suffix for a class name
		dojo.removeClass(node, "dojoDndItem" + type);
	},
	_getChildByEvent: function(e){
		// summary:
		//		gets a child, which is under the mouse at the moment, or null
		// e: Event
		//		a mouse event
		var node = e.target;
		if(node){
			for(var parent = node.parentNode; parent; node = parent, parent = node.parentNode){
				if(parent == this.parent && dojo.hasClass(node, "dojoDndItem")){ return node; }
			}
		}
		return null;
	},
	_normalizedCreator: function(/*dojo.dnd.Item*/ item, /*String*/ hint){
		// summary:
		//		adds all necessary data to the output of the user-supplied creator function
		var t = (this.creator || this.defaultCreator).call(this, item, hint);
		if(!dojo.isArray(t.type)){ t.type = ["text"]; }
		if(!t.node.id){ t.node.id = dojo.dnd.getUniqueId(); }
		dojo.addClass(t.node, "dojoDndItem");
		return t;
	}
});

dojo.dnd._createNode = function(tag){
	// summary:
	//		returns a function, which creates an element of given tag 
	//		(SPAN by default) and sets its innerHTML to given text
	// tag: String
	//		a tag name or empty for SPAN
	if(!tag){ return dojo.dnd._createSpan; }
	return function(text){	// Function
		return dojo.create(tag, {innerHTML: text});	// Node
	};
};

dojo.dnd._createTrTd = function(text){
	// summary:
	//		creates a TR/TD structure with given text as an innerHTML of TD
	// text: String
	//		a text for TD
	var tr = dojo.create("tr");
	dojo.create("td", {innerHTML: text}, tr);
	return tr;	// Node
};

dojo.dnd._createSpan = function(text){
	// summary:
	//		creates a SPAN element with given text as its innerHTML
	// text: String
	//		a text for SPAN
	return dojo.create("span", {innerHTML: text});	// Node
};

// dojo.dnd._defaultCreatorNodes: Object
//		a dictionary that maps container tag names to child tag names
dojo.dnd._defaultCreatorNodes = {ul: "li", ol: "li", div: "div", p: "div"};

dojo.dnd._defaultCreator = function(node){
	// summary:
	//		takes a parent node, and returns an appropriate creator function
	// node: Node
	//		a container node
	var tag = node.tagName.toLowerCase();
	var c = tag == "tbody" || tag == "thead" ? dojo.dnd._createTrTd :
			dojo.dnd._createNode(dojo.dnd._defaultCreatorNodes[tag]);
	return function(item, hint){	// Function
		var isObj = item && dojo.isObject(item), data, type, n;
		if(isObj && item.tagName && item.nodeType && item.getAttribute){
			// process a DOM node
			data = item.getAttribute("dndData") || item.innerHTML;
			type = item.getAttribute("dndType");
			type = type ? type.split(/\s*,\s*/) : ["text"];
			n = item;	// this node is going to be moved rather than copied
		}else{
			// process a DnD item object or a string
			data = (isObj && item.data) ? item.data : item;
			type = (isObj && item.type) ? item.type : ["text"];
			n = (hint == "avatar" ? dojo.dnd._createSpan : c)(String(data));
		}
		if(!n.id){
			n.id = dojo.dnd.getUniqueId();
		}
		return {node: n, data: data, type: type};
	};
};

}

if(!dojo._hasResource["dojo.dnd.autoscroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.autoscroll"] = true;
dojo.provide("dojo.dnd.autoscroll");

dojo.dnd.getViewport = function(){
	// summary:
	//		Returns a viewport size (visible part of the window)

	// TODO: remove this when getViewport() moved to dojo core, see #7028

	// FIXME: need more docs!!
	var d = dojo.doc, dd = d.documentElement, w = window, b = dojo.body();
	if(dojo.isMozilla){
		return {w: dd.clientWidth, h: w.innerHeight};	// Object
	}else if(!dojo.isOpera && w.innerWidth){
		return {w: w.innerWidth, h: w.innerHeight};		// Object
	}else if (!dojo.isOpera && dd && dd.clientWidth){
		return {w: dd.clientWidth, h: dd.clientHeight};	// Object
	}else if (b.clientWidth){
		return {w: b.clientWidth, h: b.clientHeight};	// Object
	}
	return null;	// Object
};

dojo.dnd.V_TRIGGER_AUTOSCROLL = 32;
dojo.dnd.H_TRIGGER_AUTOSCROLL = 32;

dojo.dnd.V_AUTOSCROLL_VALUE = 16;
dojo.dnd.H_AUTOSCROLL_VALUE = 16;

dojo.dnd.autoScroll = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the window, if
	//		necesary
	// e: Event
	//		onmousemove event

	// FIXME: needs more docs!
	var v = dojo.dnd.getViewport(), dx = 0, dy = 0;
	if(e.clientX < dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = -dojo.dnd.H_AUTOSCROLL_VALUE;
	}else if(e.clientX > v.w - dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = dojo.dnd.H_AUTOSCROLL_VALUE;
	}
	if(e.clientY < dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = -dojo.dnd.V_AUTOSCROLL_VALUE;
	}else if(e.clientY > v.h - dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = dojo.dnd.V_AUTOSCROLL_VALUE;
	}
	window.scrollBy(dx, dy);
};

dojo.dnd._validNodes = {"div": 1, "p": 1, "td": 1};
dojo.dnd._validOverflow = {"auto": 1, "scroll": 1};

dojo.dnd.autoScrollNodes = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the first avaialble
	//		Dom element, it falls back to dojo.dnd.autoScroll()
	// e: Event
	//		onmousemove event

	// FIXME: needs more docs!
	for(var n = e.target; n;){
		if(n.nodeType == 1 && (n.tagName.toLowerCase() in dojo.dnd._validNodes)){
			var s = dojo.getComputedStyle(n);
			if(s.overflow.toLowerCase() in dojo.dnd._validOverflow){
				var b = dojo._getContentBox(n, s), t = dojo.position(n, true);
				//console.log(b.l, b.t, t.x, t.y, n.scrollLeft, n.scrollTop);
				var w = Math.min(dojo.dnd.H_TRIGGER_AUTOSCROLL, b.w / 2), 
					h = Math.min(dojo.dnd.V_TRIGGER_AUTOSCROLL, b.h / 2),
					rx = e.pageX - t.x, ry = e.pageY - t.y, dx = 0, dy = 0;
				if(dojo.isWebKit || dojo.isOpera){
					// FIXME: this code should not be here, it should be taken into account 
					// either by the event fixing code, or the dojo.position()
					// FIXME: this code doesn't work on Opera 9.5 Beta
					rx += dojo.body().scrollLeft, ry += dojo.body().scrollTop;
				}
				if(rx > 0 && rx < b.w){
					if(rx < w){
						dx = -w;
					}else if(rx > b.w - w){
						dx = w;
					}
				}
				//console.log("ry =", ry, "b.h =", b.h, "h =", h);
				if(ry > 0 && ry < b.h){
					if(ry < h){
						dy = -h;
					}else if(ry > b.h - h){
						dy = h;
					}
				}
				var oldLeft = n.scrollLeft, oldTop = n.scrollTop;
				n.scrollLeft = n.scrollLeft + dx;
				n.scrollTop  = n.scrollTop  + dy;
				if(oldLeft != n.scrollLeft || oldTop != n.scrollTop){ return; }
			}
		}
		try{
			n = n.parentNode;
		}catch(x){
			n = null;
		}
	}
	dojo.dnd.autoScroll(e);
};

}

if(!dojo._hasResource["dojo.dnd.Avatar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Avatar"] = true;
dojo.provide("dojo.dnd.Avatar");



dojo.declare("dojo.dnd.Avatar", null, {
	// summary:
	//		Object that represents transferred DnD items visually
	// manager: Object
	//		a DnD manager object

	constructor: function(manager){
		this.manager = manager;
		this.construct();
	},

	// methods
	construct: function(){
		// summary:
		//		constructor function;
		//		it is separate so it can be (dynamically) overwritten in case of need
		this.isA11y = dojo.hasClass(dojo.body(),"dijit_a11y");
		var a = dojo.create("table", {
				"class": "dojoDndAvatar",
				style: {
					position: "absolute",
					zIndex:   "1999",
					margin:   "0px"
				}
			}),
			source = this.manager.source, node,
			b = dojo.create("tbody", null, a),
			tr = dojo.create("tr", null, b),
			td = dojo.create("td", null, tr),
			icon = this.isA11y ? dojo.create("span", {
						id : "a11yIcon",
						innerHTML : this.manager.copy ? '+' : "<"
					}, td) : null,
			span = dojo.create("span", {
				innerHTML: source.generateText ? this._generateText() : ""
			}, td),
			k = Math.min(5, this.manager.nodes.length), i = 0;
		// we have to set the opacity on IE only after the node is live
		dojo.attr(tr, {
			"class": "dojoDndAvatarHeader",
			style: {opacity: 0.9}
		});
		for(; i < k; ++i){
			if(source.creator){
				// create an avatar representation of the node
				node = source._normalizedCreator(source.getItem(this.manager.nodes[i].id).data, "avatar").node;
			}else{
				// or just clone the node and hope it works
				node = this.manager.nodes[i].cloneNode(true);
				if(node.tagName.toLowerCase() == "tr"){
					// insert extra table nodes
					var table = dojo.create("table"),
						tbody = dojo.create("tbody", null, table);
					tbody.appendChild(node);
					node = table;
				}
			}
			node.id = "";
			tr = dojo.create("tr", null, b);
			td = dojo.create("td", null, tr);
			td.appendChild(node);
			dojo.attr(tr, {
				"class": "dojoDndAvatarItem",
				style: {opacity: (9 - i) / 10}
			});
		}
		this.node = a;
	},
	destroy: function(){
		// summary:
		//		destructor for the avatar; called to remove all references so it can be garbage-collected
		dojo.destroy(this.node);
		this.node = false;
	},
	update: function(){
		// summary:
		//		updates the avatar to reflect the current DnD state
		dojo[(this.manager.canDropFlag ? "add" : "remove") + "Class"](this.node, "dojoDndAvatarCanDrop");
		if (this.isA11y){
			var icon = dojo.byId("a11yIcon");
			var text = '+';   // assume canDrop && copy
			if (this.manager.canDropFlag && !this.manager.copy) {
				text = '< '; // canDrop && move 
			}else if (!this.manager.canDropFlag && !this.manager.copy) {
				text = "o"; //!canDrop && move
			}else if(!this.manager.canDropFlag){
				text = 'x';  // !canDrop && copy
			}
			icon.innerHTML=text;
		}
		// replace text
		dojo.query(("tr.dojoDndAvatarHeader td span" +(this.isA11y ? " span" : "")), this.node).forEach(
			function(node){
				node.innerHTML = this._generateText();
			}, this);
	},
	_generateText: function(){
		// summary: generates a proper text to reflect copying or moving of items
		return this.manager.nodes.length.toString();
	}
});

}

if(!dojo._hasResource["dojo.dnd.Manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Manager"] = true;
dojo.provide("dojo.dnd.Manager");





dojo.declare("dojo.dnd.Manager", null, {
	// summary:
	//		the manager of DnD operations (usually a singleton)
	constructor: function(){
		this.avatar  = null;
		this.source = null;
		this.nodes = [];
		this.copy  = true;
		this.target = null;
		this.canDropFlag = false;
		this.events = [];
	},

	// avatar's offset from the mouse
	OFFSET_X: 16,
	OFFSET_Y: 16,
	
	// methods
	overSource: function(source){
		// summary:
		//		called when a source detected a mouse-over condition
		// source: Object
		//		the reporter
		if(this.avatar){
			this.target = (source && source.targetState != "Disabled") ? source : null;
			this.canDropFlag = Boolean(this.target);
			this.avatar.update();
		}
		dojo.publish("/dnd/source/over", [source]);
	},
	outSource: function(source){
		// summary:
		//		called when a source detected a mouse-out condition
		// source: Object
		//		the reporter
		if(this.avatar){
			if(this.target == source){
				this.target = null;
				this.canDropFlag = false;
				this.avatar.update();
				dojo.publish("/dnd/source/over", [null]);
			}
		}else{
			dojo.publish("/dnd/source/over", [null]);
		}
	},
	startDrag: function(source, nodes, copy){
		// summary:
		//		called to initiate the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		this.source = source;
		this.nodes  = nodes;
		this.copy   = Boolean(copy); // normalizing to true boolean
		this.avatar = this.makeAvatar();
		dojo.body().appendChild(this.avatar.node);
		dojo.publish("/dnd/start", [source, nodes, this.copy]);
		this.events = [
			dojo.connect(dojo.doc, "onmousemove", this, "onMouseMove"),
			dojo.connect(dojo.doc, "onmouseup",   this, "onMouseUp"),
			dojo.connect(dojo.doc, "onkeydown",   this, "onKeyDown"),
			dojo.connect(dojo.doc, "onkeyup",     this, "onKeyUp"),
			// cancel text selection and text dragging
			dojo.connect(dojo.doc, "ondragstart",   dojo.stopEvent),
			dojo.connect(dojo.body(), "onselectstart", dojo.stopEvent)
		];
		var c = "dojoDnd" + (copy ? "Copy" : "Move");
		dojo.addClass(dojo.body(), c); 
	},
	canDrop: function(flag){
		// summary:
		//		called to notify if the current target can accept items
		var canDropFlag = Boolean(this.target && flag);
		if(this.canDropFlag != canDropFlag){
			this.canDropFlag = canDropFlag;
			this.avatar.update();
		}
	},
	stopDrag: function(){
		// summary:
		//		stop the DnD in progress
		dojo.removeClass(dojo.body(), "dojoDndCopy");
		dojo.removeClass(dojo.body(), "dojoDndMove");
		dojo.forEach(this.events, dojo.disconnect);
		this.events = [];
		this.avatar.destroy();
		this.avatar = null;
		this.source = this.target = null;
		this.nodes = [];
	},
	makeAvatar: function(){
		// summary:
		//		makes the avatar; it is separate to be overwritten dynamically, if needed
		return new dojo.dnd.Avatar(this);
	},
	updateAvatar: function(){
		// summary:
		//		updates the avatar; it is separate to be overwritten dynamically, if needed
		this.avatar.update();
	},
	
	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		var a = this.avatar;
		if(a){
			dojo.dnd.autoScrollNodes(e);
			//dojo.dnd.autoScroll(e);
			var s = a.node.style;
			s.left = (e.pageX + this.OFFSET_X) + "px";
			s.top  = (e.pageY + this.OFFSET_Y) + "px";
			var copy = Boolean(this.source.copyState(dojo.isCopyKey(e)));
			if(this.copy != copy){ 
				this._setCopyStatus(copy);
			}
		}
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(this.avatar){
			if(this.target && this.canDropFlag){
				var copy = Boolean(this.source.copyState(dojo.isCopyKey(e))),
				params = [this.source, this.nodes, copy, this.target, e];
				dojo.publish("/dnd/drop/before", params);
				dojo.publish("/dnd/drop", params);
			}else{
				dojo.publish("/dnd/cancel");
			}
			this.stopDrag();
		}
	},
	
	// keyboard event processors
	onKeyDown: function(e){
		// summary:
		//		event processor for onkeydown:
		//		watching for CTRL for copy/move status, watching for ESCAPE to cancel the drag
		// e: Event
		//		keyboard event
		if(this.avatar){
			switch(e.keyCode){
				case dojo.keys.CTRL:
					var copy = Boolean(this.source.copyState(true));
					if(this.copy != copy){ 
						this._setCopyStatus(copy);
					}
					break;
				case dojo.keys.ESCAPE:
					dojo.publish("/dnd/cancel");
					this.stopDrag();
					break;
			}
		}
	},
	onKeyUp: function(e){
		// summary:
		//		event processor for onkeyup, watching for CTRL for copy/move status
		// e: Event
		//		keyboard event
		if(this.avatar && e.keyCode == dojo.keys.CTRL){
			var copy = Boolean(this.source.copyState(false));
			if(this.copy != copy){ 
				this._setCopyStatus(copy);
			}
		}
	},
	
	// utilities
	_setCopyStatus: function(copy){
		// summary:
		//		changes the copy status
		// copy: Boolean
		//		the copy status
		this.copy = copy;
		this.source._markDndStatus(this.copy);
		this.updateAvatar();
		dojo.removeClass(dojo.body(), "dojoDnd" + (this.copy ? "Move" : "Copy"));
		dojo.addClass(dojo.body(), "dojoDnd" + (this.copy ? "Copy" : "Move"));
	}
});

// dojo.dnd._manager:
//		The manager singleton variable. Can be overwritten if needed.
dojo.dnd._manager = null;

dojo.dnd.manager = function(){
	// summary:
	//		Returns the current DnD manager.  Creates one if it is not created yet.
	if(!dojo.dnd._manager){
		dojo.dnd._manager = new dojo.dnd.Manager();
	}
	return dojo.dnd._manager;	// Object
};

}

if(!dojo._hasResource["dojo.dnd.Selector"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Selector"] = true;
dojo.provide("dojo.dnd.Selector");




/*
	Container item states:
		""			- an item is not selected
		"Selected"	- an item is selected
		"Anchor"	- an item is selected, and is an anchor for a "shift" selection
*/

/*=====
dojo.declare("dojo.dnd.__SelectorArgs", [dojo.dnd.__ContainerArgs], {
	//	singular: Boolean
	//		allows selection of only one element, if true
	singular: false,

	//	autoSync: Boolean
	//		autosynchronizes the source with its list of DnD nodes,
	autoSync: false
});
=====*/

dojo.declare("dojo.dnd.Selector", dojo.dnd.Container, {
	// summary:
	//		a Selector object, which knows how to select its children
	
	/*=====
	// selection: Set<String>
	//		The set of id's that are currently selected, such that this.selection[id] == 1
	//		if the node w/that id is selected.  Can iterate over selected node's id's like:
	//	|		for(var id in this.selection)
	selection: {},
	=====*/

	constructor: function(node, params){
		// summary:
		//		constructor of the Selector
		// node: Node||String
		//		node or node's id to build the selector on
		// params: dojo.dnd.__SelectorArgs?
		//		a dictionary of parameters
		if(!params){ params = {}; }
		this.singular = params.singular;
		this.autoSync = params.autoSync;
		// class-specific variables
		this.selection = {};
		this.anchor = null;
		this.simpleSelection = false;
		// set up events
		this.events.push(
			dojo.connect(this.node, "onmousedown", this, "onMouseDown"),
			dojo.connect(this.node, "onmouseup",   this, "onMouseUp"));
	},
	
	// object attributes (for markup)
	singular: false,	// is singular property
	
	// methods
	getSelectedNodes: function(){
		// summary:
		//		returns a list (an array) of selected nodes
		var t = new dojo.NodeList();
		var e = dojo.dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			t.push(dojo.byId(i));
		}
		return t;	// NodeList
	},
	selectNone: function(){
		// summary:
		//		unselects all items
		return this._removeSelection()._removeAnchor();	// self
	},
	selectAll: function(){
		// summary:
		//		selects all items
		this.forInItems(function(data, id){
			this._addItemClass(dojo.byId(id), "Selected");
			this.selection[id] = 1;
		}, this);
		return this._removeAnchor();	// self
	},
	deleteSelectedNodes: function(){
		// summary:
		//		deletes all selected items
		var e = dojo.dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			var n = dojo.byId(i);
			this.delItem(i);
			dojo.destroy(n);
		}
		this.anchor = null;
		this.selection = {};
		return this;	// self
	},
	forInSelectedItems: function(/*Function*/ f, /*Object?*/ o){
		// summary:
		//		iterates over selected items;
		//		see `dojo.dnd.Container.forInItems()` for details
		o = o || dojo.global;
		var s = this.selection, e = dojo.dnd._empty;
		for(var i in s){
			if(i in e){ continue; }
			f.call(o, this.getItem(i), i, this);
		}
	},
	sync: function(){
		// summary:
		//		sync up the node list with the data map
		
		dojo.dnd.Selector.superclass.sync.call(this);
		
		// fix the anchor
		if(this.anchor){
			if(!this.getItem(this.anchor.id)){
				this.anchor = null;
			}
		}
		
		// fix the selection
		var t = [], e = dojo.dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			if(!this.getItem(i)){
				t.push(i);
			}
		}
		dojo.forEach(t, function(i){
			delete this.selection[i];
		}, this);
		
		return this;	// self
	},
	insertNodes: function(addSelected, data, before, anchor){
		// summary:
		//		inserts new data items (see `dojo.dnd.Container.insertNodes()` method for details)
		// addSelected: Boolean
		//		all new nodes will be added to selected items, if true, no selection change otherwise
		// data: Array
		//		a list of data items, which should be processed by the creator function
		// before: Boolean
		//		insert before the anchor, if true, and after the anchor otherwise
		// anchor: Node
		//		the anchor node to be used as a point of insertion
		var oldCreator = this._normalizedCreator;
		this._normalizedCreator = function(item, hint){
			var t = oldCreator.call(this, item, hint);
			if(addSelected){
				if(!this.anchor){
					this.anchor = t.node;
					this._removeItemClass(t.node, "Selected");
					this._addItemClass(this.anchor, "Anchor");
				}else if(this.anchor != t.node){
					this._removeItemClass(t.node, "Anchor");
					this._addItemClass(t.node, "Selected");
				}
				this.selection[t.node.id] = 1;
			}else{
				this._removeItemClass(t.node, "Selected");
				this._removeItemClass(t.node, "Anchor");
			}
			return t;
		};
		dojo.dnd.Selector.superclass.insertNodes.call(this, data, before, anchor);
		this._normalizedCreator = oldCreator;
		return this;	// self
	},
	destroy: function(){
		// summary:
		//		prepares the object to be garbage-collected
		dojo.dnd.Selector.superclass.destroy.call(this);
		this.selection = this.anchor = null;
	},

	// markup methods
	markupFactory: function(params, node){
		params._skipStartup = true;
		return new dojo.dnd.Selector(node, params);
	},

	// mouse events
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if(this.autoSync){ this.sync(); }
		if(!this.current){ return; }
		if(!this.singular && !dojo.isCopyKey(e) && !e.shiftKey && (this.current.id in this.selection)){
			this.simpleSelection = true;
			if(e.button === dojo.mouseButtons.LEFT){
				// accept the left button and stop the event
				// for IE we don't stop event when multiple buttons are pressed
				dojo.stopEvent(e);
			}
			return;
		}
		if(!this.singular && e.shiftKey){
			if(!dojo.isCopyKey(e)){
				this._removeSelection();
			}
			var c = this.getAllNodes();
			if(c.length){
				if(!this.anchor){
					this.anchor = c[0];
					this._addItemClass(this.anchor, "Anchor");
				}
				this.selection[this.anchor.id] = 1;
				if(this.anchor != this.current){
					var i = 0;
					for(; i < c.length; ++i){
						var node = c[i];
						if(node == this.anchor || node == this.current){ break; }
					}
					for(++i; i < c.length; ++i){
						var node = c[i];
						if(node == this.anchor || node == this.current){ break; }
						this._addItemClass(node, "Selected");
						this.selection[node.id] = 1;
					}
					this._addItemClass(this.current, "Selected");
					this.selection[this.current.id] = 1;
				}
			}
		}else{
			if(this.singular){
				if(this.anchor == this.current){
					if(dojo.isCopyKey(e)){
						this.selectNone();
					}
				}else{
					this.selectNone();
					this.anchor = this.current;
					this._addItemClass(this.anchor, "Anchor");
					this.selection[this.current.id] = 1;
				}
			}else{
				if(dojo.isCopyKey(e)){
					if(this.anchor == this.current){
						delete this.selection[this.anchor.id];
						this._removeAnchor();
					}else{
						if(this.current.id in this.selection){
							this._removeItemClass(this.current, "Selected");
							delete this.selection[this.current.id];
						}else{
							if(this.anchor){
								this._removeItemClass(this.anchor, "Anchor");
								this._addItemClass(this.anchor, "Selected");
							}
							this.anchor = this.current;
							this._addItemClass(this.current, "Anchor");
							this.selection[this.current.id] = 1;
						}
					}
				}else{
					if(!(this.current.id in this.selection)){
						this.selectNone();
						this.anchor = this.current;
						this._addItemClass(this.current, "Anchor");
						this.selection[this.current.id] = 1;
					}
				}
			}
		}
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(!this.simpleSelection){ return; }
		this.simpleSelection = false;
		this.selectNone();
		if(this.current){
			this.anchor = this.current;
			this._addItemClass(this.anchor, "Anchor");
			this.selection[this.current.id] = 1;
		}
	},
	onMouseMove: function(e){
		// summary
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		this.simpleSelection = false;
	},
	
	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
		this.onmousemoveEvent = dojo.connect(this.node, "onmousemove", this, "onMouseMove");
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
		dojo.disconnect(this.onmousemoveEvent);
		delete this.onmousemoveEvent;
	},
	_removeSelection: function(){
		// summary:
		//		unselects all items
		var e = dojo.dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			var node = dojo.byId(i);
			if(node){ this._removeItemClass(node, "Selected"); }
		}
		this.selection = {};
		return this;	// self
	},
	_removeAnchor: function(){
		if(this.anchor){
			this._removeItemClass(this.anchor, "Anchor");
			this.anchor = null;
		}
		return this;	// self
	}
});

}

if(!dojo._hasResource["dojo.dnd.Source"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Source"] = true;
dojo.provide("dojo.dnd.Source");




/*
	Container property:
		"Horizontal"- if this is the horizontal container
	Source states:
		""			- normal state
		"Moved"		- this source is being moved
		"Copied"	- this source is being copied
	Target states:
		""			- normal state
		"Disabled"	- the target cannot accept an avatar
	Target anchor state:
		""			- item is not selected
		"Before"	- insert point is before the anchor
		"After"		- insert point is after the anchor
*/

/*=====
dojo.dnd.__SourceArgs = function(){
	//	summary:
	//		a dict of parameters for DnD Source configuration. Note that any
	//		property on Source elements may be configured, but this is the
	//		short-list
	//	isSource: Boolean?
	//		can be used as a DnD source. Defaults to true.
	//	accept: Array?
	//		list of accepted types (text strings) for a target; defaults to
	//		["text"]
	//	autoSync: Boolean
	//		if true refreshes the node list on every operation; false by default
	//	copyOnly: Boolean?
	//		copy items, if true, use a state of Ctrl key otherwise,
	//		see selfCopy and selfAccept for more details
	//	delay: Number
	//		the move delay in pixels before detecting a drag; 0 by default
	//	horizontal: Boolean?
	//		a horizontal container, if true, vertical otherwise or when omitted
	//	selfCopy: Boolean?
	//		copy items by default when dropping on itself,
	//		false by default, works only if copyOnly is true
	//	selfAccept: Boolean?
	//		accept its own items when copyOnly is true,
	//		true by default, works only if copyOnly is true
	//	withHandles: Boolean?
	//		allows dragging only by handles, false by default
	//  generateText: Boolean?
	//		generate text node for drag and drop, true by default
	this.isSource = isSource;
	this.accept = accept;
	this.autoSync = autoSync;
	this.copyOnly = copyOnly;
	this.delay = delay;
	this.horizontal = horizontal;
	this.selfCopy = selfCopy;
	this.selfAccept = selfAccept;
	this.withHandles = withHandles;
	this.generateText = true;
}
=====*/

dojo.declare("dojo.dnd.Source", dojo.dnd.Selector, {
	// summary:
	//		a Source object, which can be used as a DnD source, or a DnD target
	
	// object attributes (for markup)
	isSource: true,
	horizontal: false,
	copyOnly: false,
	selfCopy: false,
	selfAccept: true,
	skipForm: false,
	withHandles: false,
	autoSync: false,
	delay: 0, // pixels
	accept: ["text"],
	generateText: true,
	
	constructor: function(/*DOMNode|String*/node, /*dojo.dnd.__SourceArgs?*/params){
		// summary: 
		//		a constructor of the Source
		// node:
		//		node or node's id to build the source on
		// params: 
		//		any property of this class may be configured via the params
		//		object which is mixed-in to the `dojo.dnd.Source` instance
		dojo.mixin(this, dojo.mixin({}, params));
		var type = this.accept;
		if(type.length){
			this.accept = {};
			for(var i = 0; i < type.length; ++i){
				this.accept[type[i]] = 1;
			}
		}
		// class-specific variables
		this.isDragging = false;
		this.mouseDown = false;
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
		this._lastX = 0;
		this._lastY = 0;
		// states
		this.sourceState  = "";
		if(this.isSource){
			dojo.addClass(this.node, "dojoDndSource");
		}
		this.targetState  = "";
		if(this.accept){
			dojo.addClass(this.node, "dojoDndTarget");
		}
		if(this.horizontal){
			dojo.addClass(this.node, "dojoDndHorizontal");
		}
		// set up events
		this.topics = [
			dojo.subscribe("/dnd/source/over", this, "onDndSourceOver"),
			dojo.subscribe("/dnd/start",  this, "onDndStart"),
			dojo.subscribe("/dnd/drop",   this, "onDndDrop"),
			dojo.subscribe("/dnd/cancel", this, "onDndCancel")
		];
	},
	
	// methods
	checkAcceptance: function(source, nodes){
		// summary:
		//		checks if the target can accept nodes from this source
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		if(this == source){
			return !this.copyOnly || this.selfAccept;
		}
		for(var i = 0; i < nodes.length; ++i){
			var type = source.getItem(nodes[i].id).type;
			// type instanceof Array
			var flag = false;
			for(var j = 0; j < type.length; ++j){
				if(type[j] in this.accept){
					flag = true;
					break;
				}
			}
			if(!flag){
				return false;	// Boolean
			}
		}
		return true;	// Boolean
	},
	copyState: function(keyPressed, self){
		// summary:
		//		Returns true if we need to copy items, false to move.
		//		It is separated to be overwritten dynamically, if needed.
		// keyPressed: Boolean
		//		the "copy" key was pressed
		// self: Boolean?
		//		optional flag that means that we are about to drop on itself
		
		if(keyPressed){ return true; }
		if(arguments.length < 2){
			self = this == dojo.dnd.manager().target;
		}
		if(self){
			if(this.copyOnly){
				return this.selfCopy;
			}
		}else{
			return this.copyOnly;
		}
		return false;	// Boolean
	},
	destroy: function(){
		// summary:
		//		prepares the object to be garbage-collected
		dojo.dnd.Source.superclass.destroy.call(this);
		dojo.forEach(this.topics, dojo.unsubscribe);
		this.targetAnchor = null;
	},

	// markup methods
	markupFactory: function(params, node){
		params._skipStartup = true;
		return new dojo.dnd.Source(node, params);
	},

	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		if(this.isDragging && this.targetState == "Disabled"){ return; }
		dojo.dnd.Source.superclass.onMouseMove.call(this, e);
		var m = dojo.dnd.manager();
		if(!this.isDragging){
			if(this.mouseDown && this.isSource &&
					(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay)){
				var nodes = this.getSelectedNodes();
				if(nodes.length){
					m.startDrag(this, nodes, this.copyState(dojo.isCopyKey(e), true));
				}
			}
		}
		if(this.isDragging){
			// calculate before/after
			var before = false;
			if(this.current){
				if(!this.targetBox || this.targetAnchor != this.current){
					this.targetBox = dojo.position(this.current, true);
				}
				if(this.horizontal){
					before = (e.pageX - this.targetBox.x) < (this.targetBox.w / 2);
				}else{
					before = (e.pageY - this.targetBox.y) < (this.targetBox.h / 2);
				}
			}
			if(this.current != this.targetAnchor || before != this.before){
				this._markTargetAnchor(before);
				m.canDrop(!this.current || m.source != this || !(this.current.id in this.selection));
			}
		}
	},
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if(!this.mouseDown && this._legalMouseDown(e) && (!this.skipForm || !dojo.dnd.isFormElement(e))){
			this.mouseDown = true;
			this._lastX = e.pageX;
			this._lastY = e.pageY;
			dojo.dnd.Source.superclass.onMouseDown.call(this, e);
		}
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(this.mouseDown){
			this.mouseDown = false;
			dojo.dnd.Source.superclass.onMouseUp.call(this, e);
		}
	},
	
	// topic event processors
	onDndSourceOver: function(source){
		// summary:
		//		topic event processor for /dnd/source/over, called when detected a current source
		// source: Object
		//		the source which has the mouse over it
		if(this != source){
			this.mouseDown = false;
			if(this.targetAnchor){
				this._unmarkTargetAnchor();
			}
		}else if(this.isDragging){
			var m = dojo.dnd.manager();
			m.canDrop(this.targetState != "Disabled" && (!this.current || m.source != this || !(this.current.id in this.selection)));
		}
	},
	onDndStart: function(source, nodes, copy){
		// summary:
		//		topic event processor for /dnd/start, called to initiate the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		if(this.autoSync){ this.sync(); }
		if(this.isSource){
			this._changeState("Source", this == source ? (copy ? "Copied" : "Moved") : "");
		}
		var accepted = this.accept && this.checkAcceptance(source, nodes);
		this._changeState("Target", accepted ? "" : "Disabled");
		if(this == source){
			dojo.dnd.manager().overSource(this);
		}
		this.isDragging = true;
	},
	onDndDrop: function(source, nodes, copy, target){
		// summary:
		//		topic event processor for /dnd/drop, called to finish the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		// target: Object
		//		the target which accepts items
		if(this == target){
			// this one is for us => move nodes!
			this.onDrop(source, nodes, copy);
		}
		this.onDndCancel();
	},
	onDndCancel: function(){
		// summary:
		//		topic event processor for /dnd/cancel, called to cancel the DnD operation
		if(this.targetAnchor){
			this._unmarkTargetAnchor();
			this.targetAnchor = null;
		}
		this.before = true;
		this.isDragging = false;
		this.mouseDown = false;
		this._changeState("Source", "");
		this._changeState("Target", "");
	},
	
	// local events
	onDrop: function(source, nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		
		if(this != source){
			this.onDropExternal(source, nodes, copy);
		}else{
			this.onDropInternal(nodes, copy);
		}
	},
	onDropExternal: function(source, nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		//		from an external source
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		
		var oldCreator = this._normalizedCreator;
		// transferring nodes from the source to the target
		if(this.creator){
			// use defined creator
			this._normalizedCreator = function(node, hint){
				return oldCreator.call(this, source.getItem(node.id).data, hint);
			};
		}else{
			// we have no creator defined => move/clone nodes
			if(copy){
				// clone nodes
				this._normalizedCreator = function(node, hint){
					var t = source.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dojo.dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}else{
				// move nodes
				this._normalizedCreator = function(node, hint){
					var t = source.getItem(node.id);
					source.delItem(node.id);
					return {node: node, data: t.data, type: t.type};
				};
			}
		}
		this.selectNone();
		if(!copy && !this.creator){
			source.selectNone();
		}
		this.insertNodes(true, nodes, this.before, this.current);
		if(!copy && this.creator){
			source.deleteSelectedNodes();
		}
		this._normalizedCreator = oldCreator;
	},
	onDropInternal: function(nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		//		from the same target/source
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		
		var oldCreator = this._normalizedCreator;
		// transferring nodes within the single source
		if(this.current && this.current.id in this.selection){
			// do nothing
			return;
		}
		if(copy){
			if(this.creator){
				// create new copies of data items
				this._normalizedCreator = function(node, hint){
					return oldCreator.call(this, this.getItem(node.id).data, hint);
				};
			}else{
				// clone nodes
				this._normalizedCreator = function(node, hint){
					var t = this.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dojo.dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}
		}else{
			// move nodes
			if(!this.current){
				// do nothing
				return;
			}
			this._normalizedCreator = function(node, hint){
				var t = this.getItem(node.id);
				return {node: node, data: t.data, type: t.type};
			};
		}
		this._removeSelection();
		this.insertNodes(true, nodes, this.before, this.current);
		this._normalizedCreator = oldCreator;
	},
	onDraggingOver: function(){
		// summary:
		//		called during the active DnD operation, when items
		//		are dragged over this target, and it is not disabled
	},
	onDraggingOut: function(){
		// summary:
		//		called during the active DnD operation, when items
		//		are dragged away from this target, and it is not disabled
	},
	
	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
		dojo.dnd.Source.superclass.onOverEvent.call(this);
		dojo.dnd.manager().overSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOver();
		}
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
		dojo.dnd.Source.superclass.onOutEvent.call(this);
		dojo.dnd.manager().outSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOut();
		}
	},
	_markTargetAnchor: function(before){
		// summary:
		//		assigns a class to the current target anchor based on "before" status
		// before: Boolean
		//		insert before, if true, after otherwise
		if(this.current == this.targetAnchor && this.before == before){ return; }
		if(this.targetAnchor){
			this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
		this.targetAnchor = this.current;
		this.targetBox = null;
		this.before = before;
		if(this.targetAnchor){
			this._addItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
	},
	_unmarkTargetAnchor: function(){
		// summary:
		//		removes a class of the current target anchor based on "before" status
		if(!this.targetAnchor){ return; }
		this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
	},
	_markDndStatus: function(copy){
		// summary:
		//		changes source's state based on "copy" status
		this._changeState("Source", copy ? "Copied" : "Moved");
	},
	_legalMouseDown: function(e){
		// summary:
		//		checks if user clicked on "approved" items
		// e: Event
		//		mouse event
		
		// accept only the left mouse button
		if(!dojo.mouseButtons.isLeft(e)){ return false; }
		
		if(!this.withHandles){ return true; }
		
		// check for handles
		for(var node = e.target; node && node !== this.node; node = node.parentNode){
			if(dojo.hasClass(node, "dojoDndHandle")){ return true; }
			if(dojo.hasClass(node, "dojoDndItem") || dojo.hasClass(node, "dojoDndIgnore")){ break; }
		}
		return false;	// Boolean
	}
});

dojo.declare("dojo.dnd.Target", dojo.dnd.Source, {
	// summary: a Target object, which can be used as a DnD target
	
	constructor: function(node, params){
		// summary:
		//		a constructor of the Target --- see the `dojo.dnd.Source.constructor` for details
		this.isSource = false;
		dojo.removeClass(this.node, "dojoDndSource");
	},

	// markup methods
	markupFactory: function(params, node){
		params._skipStartup = true;
		return new dojo.dnd.Target(node, params);
	}
});

dojo.declare("dojo.dnd.AutoSource", dojo.dnd.Source, {
	// summary:
	//		a source that syncs its DnD nodes by default
	
	constructor: function(node, params){
		// summary:
		//		constructor of the AutoSource --- see the Source constructor for details
		this.autoSync = true;
	},

	// markup methods
	markupFactory: function(params, node){
		params._skipStartup = true;
		return new dojo.dnd.AutoSource(node, params);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.AccentMapper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.AccentMapper"] = true;
dojo.provide("ibm.tivoli.tip.dojo.AccentMapper");

/**
 * AccentMapper is a mixin providing a function for mapping a string with accented characters
 * to their ascii equivalent. This is useful for more effective searching and filtering.
 * 
 *  Usage:
 *  dojo.declare("my.class", [dijit._Widget, ibm.tivoli.tip.dojo.AccentMapper], 
 *  {
 *  	// test if needle is in haystack
 *  	test: function(needle, haystack) 
 *  	{
 *  		var n = this.mapAccents(needle);
 *  		var h = this.mapAccents(haystack);
 *  		var filter_re = new RegExp(n, "i");
 *  		return filter_re.test(h);
 *  	}
 *  });
 *  			
 * For a more complete example, see ibm.tivoli.tip.dijit.DnDListbox
 */
dojo.declare("ibm.tivoli.tip.dojo.AccentMapper", null, 
{
	// maps accented characters to their ascii equivalent
	_accent_map: {
		 'ẚ':'a',
		 'Á':'a',
		 'á':'a',
		 'À':'a',
		 'à':'a',
		 'Ă':'a',
		 'ă':'a',
		 'Ắ':'a',
		 'ắ':'a',
		 'Ằ':'a',
		 'ằ':'a',
		 'Ẵ':'a',
		 'ẵ':'a',
		 'Ẳ':'a',
		 'ẳ':'a',
		 'Â':'a',
		 'â':'a',
		 'Ấ':'a',
		 'ấ':'a',
		 'Ầ':'a',
		 'ầ':'a',
		 'Ẫ':'a',
		 'ẫ':'a',
		 'Ẩ':'a',
		 'ẩ':'a',
		 'Ǎ':'a',
		 'ǎ':'a',
		 'Å':'a',
		 'å':'a',
		 'Ǻ':'a',
		 'ǻ':'a',
		 'Ä':'a',
		 'ä':'a',
		 'Ǟ':'a',
		 'ǟ':'a',
		 'Ã':'a',
		 'ã':'a',
		 'Ȧ':'a',
		 'ȧ':'a',
		 'Ǡ':'a',
		 'ǡ':'a',
		 'Ą':'a',
		 'ą':'a',
		 'Ā':'a',
		 'ā':'a',
		 'Ả':'a',
		 'ả':'a',
		 'Ȁ':'a',
		 'ȁ':'a',
		 'Ȃ':'a',
		 'ȃ':'a',
		 'Ạ':'a',
		 'ạ':'a',
		 'Ặ':'a',
		 'ặ':'a',
		 'Ậ':'a',
		 'ậ':'a',
		 'Ḁ':'a',
		 'ḁ':'a',
		 'Ⱥ':'a',
		 'ⱥ':'a',
		 'Ǽ':'a',
		 'ǽ':'a',
		 'Ǣ':'a',
		 'ǣ':'a',
		 'Ḃ':'b',
		 'ḃ':'b',
		 'Ḅ':'b',
		 'ḅ':'b',
		 'Ḇ':'b',
		 'ḇ':'b',
		 'Ƀ':'b',
		 'ƀ':'b',
		 'ᵬ':'b',
		 'Ɓ':'b',
		 'ɓ':'b',
		 'Ƃ':'b',
		 'ƃ':'b',
		 'Ć':'c',
		 'ć':'c',
		 'Ĉ':'c',
		 'ĉ':'c',
		 'Č':'c',
		 'č':'c',
		 'Ċ':'c',
		 'ċ':'c',
		 'Ç':'c',
		 'ç':'c',
		 'Ḉ':'c',
		 'ḉ':'c',
		 'Ȼ':'c',
		 'ȼ':'c',
		 'Ƈ':'c',
		 'ƈ':'c',
		 'ɕ':'c',
		 'Ď':'d',
		 'ď':'d',
		 'Ḋ':'d',
		 'ḋ':'d',
		 'Ḑ':'d',
		 'ḑ':'d',
		 'Ḍ':'d',
		 'ḍ':'d',
		 'Ḓ':'d',
		 'ḓ':'d',
		 'Ḏ':'d',
		 'ḏ':'d',
		 'Đ':'d',
		 'đ':'d',
		 'ᵭ':'d',
		 'Ɖ':'d',
		 'ɖ':'d',
		 'Ɗ':'d',
		 'ɗ':'d',
		 'Ƌ':'d',
		 'ƌ':'d',
		 'ȡ':'d',
		 'ð':'d',
		 'É':'e',
		 'Ə':'e',
		 'Ǝ':'e',
		 'ǝ':'e',
		 'é':'e',
		 'È':'e',
		 'è':'e',
		 'Ĕ':'e',
		 'ĕ':'e',
		 'Ê':'e',
		 'ê':'e',
		 'Ế':'e',
		 'ế':'e',
		 'Ề':'e',
		 'ề':'e',
		 'Ễ':'e',
		 'ễ':'e',
		 'Ể':'e',
		 'ể':'e',
		 'Ě':'e',
		 'ě':'e',
		 'Ë':'e',
		 'ë':'e',
		 'Ẽ':'e',
		 'ẽ':'e',
		 'Ė':'e',
		 'ė':'e',
		 'Ȩ':'e',
		 'ȩ':'e',
		 'Ḝ':'e',
		 'ḝ':'e',
		 'Ę':'e',
		 'ę':'e',
		 'Ē':'e',
		 'ē':'e',
		 'Ḗ':'e',
		 'ḗ':'e',
		 'Ḕ':'e',
		 'ḕ':'e',
		 'Ẻ':'e',
		 'ẻ':'e',
		 'Ȅ':'e',
		 'ȅ':'e',
		 'Ȇ':'e',
		 'ȇ':'e',
		 'Ẹ':'e',
		 'ẹ':'e',
		 'Ệ':'e',
		 'ệ':'e',
		 'Ḙ':'e',
		 'ḙ':'e',
		 'Ḛ':'e',
		 'ḛ':'e',
		 'Ɇ':'e',
		 'ɇ':'e',
		 'ɚ':'e',
		 'ɝ':'e',
		 'Ḟ':'f',
		 'ḟ':'f',
		 'ᵮ':'f',
		 'Ƒ':'f',
		 'ƒ':'f',
		 'Ǵ':'g',
		 'ǵ':'g',
		 'Ğ':'g',
		 'ğ':'g',
		 'Ĝ':'g',
		 'ĝ':'g',
		 'Ǧ':'g',
		 'ǧ':'g',
		 'Ġ':'g',
		 'ġ':'g',
		 'Ģ':'g',
		 'ģ':'g',
		 'Ḡ':'g',
		 'ḡ':'g',
		 'Ǥ':'g',
		 'ǥ':'g',
		 'Ɠ':'g',
		 'ɠ':'g',
		 'Ĥ':'h',
		 'ĥ':'h',
		 'Ȟ':'h',
		 'ȟ':'h',
		 'Ḧ':'h',
		 'ḧ':'h',
		 'Ḣ':'h',
		 'ḣ':'h',
		 'Ḩ':'h',
		 'ḩ':'h',
		 'Ḥ':'h',
		 'ḥ':'h',
		 'Ḫ':'h',
		 'ḫ':'h',
		 'H':'h',
		 '̱':'h',
		 'ẖ':'h',
		 'Ħ':'h',
		 'ħ':'h',
		 'Ⱨ':'h',
		 'ⱨ':'h',
		 'Í':'i',
		 'í':'i',
		 'Ì':'i',
		 'ì':'i',
		 'Ĭ':'i',
		 'ĭ':'i',
		 'Î':'i',
		 'î':'i',
		 'Ǐ':'i',
		 'ǐ':'i',
		 'Ï':'i',
		 'ï':'i',
		 'Ḯ':'i',
		 'ḯ':'i',
		 'Ĩ':'i',
		 'ĩ':'i',
		 'İ':'i',
		 'i':'i',
		 'Į':'i',
		 'į':'i',
		 'Ī':'i',
		 'ī':'i',
		 'Ỉ':'i',
		 'ỉ':'i',
		 'Ȉ':'i',
		 'ȉ':'i',
		 'Ȋ':'i',
		 'ȋ':'i',
		 'Ị':'i',
		 'ị':'i',
		 'Ḭ':'i',
		 'ḭ':'i',
		 'I':'i',
		 'ı':'i',
		 'Ɨ':'i',
		 'ɨ':'i',
		 'Ĵ':'j',
		 'ĵ':'j',
		 'J':'j',
		 '̌':'j',
		 'ǰ':'j',
		 'ȷ':'j',
		 'Ɉ':'j',
		 'ɉ':'j',
		 'ʝ':'j',
		 'ɟ':'j',
		 'ʄ':'j',
		 'Ḱ':'k',
		 'ḱ':'k',
		 'Ǩ':'k',
		 'ǩ':'k',
		 'Ķ':'k',
		 'ķ':'k',
		 'Ḳ':'k',
		 'ḳ':'k',
		 'Ḵ':'k',
		 'ḵ':'k',
		 'Ƙ':'k',
		 'ƙ':'k',
		 'Ⱪ':'k',
		 'ⱪ':'k',
		 'Ĺ':'a',
		 'ĺ':'l',
		 'Ľ':'l',
		 'ľ':'l',
		 'Ļ':'l',
		 'ļ':'l',
		 'Ḷ':'l',
		 'ḷ':'l',
		 'Ḹ':'l',
		 'ḹ':'l',
		 'Ḽ':'l',
		 'ḽ':'l',
		 'Ḻ':'l',
		 'ḻ':'l',
		 'Ł':'l',
		 'ł':'l',
		 'Ł':'l',
		 '̣':'l',
		 'ł':'l',
		 '̣':'l',
		 'Ŀ':'l',
		 'ŀ':'l',
		 'Ƚ':'l',
		 'ƚ':'l',
		 'Ⱡ':'l',
		 'ⱡ':'l',
		 'Ɫ':'l',
		 'ɫ':'l',
		 'ɬ':'l',
		 'ɭ':'l',
		 'ȴ':'l',
		 'Ḿ':'m',
		 'ḿ':'m',
		 'Ṁ':'m',
		 'ṁ':'m',
		 'Ṃ':'m',
		 'ṃ':'m',
		 'ɱ':'m',
		 'Ń':'n',
		 'ń':'n',
		 'Ǹ':'n',
		 'ǹ':'n',
		 'Ň':'n',
		 'ň':'n',
		 'Ñ':'n',
		 'ñ':'n',
		 'Ṅ':'n',
		 'ṅ':'n',
		 'Ņ':'n',
		 'ņ':'n',
		 'Ṇ':'n',
		 'ṇ':'n',
		 'Ṋ':'n',
		 'ṋ':'n',
		 'Ṉ':'n',
		 'ṉ':'n',
		 'Ɲ':'n',
		 'ɲ':'n',
		 'Ƞ':'n',
		 'ƞ':'n',
		 'ɳ':'n',
		 'ȵ':'n',
		 'N':'n',
		 '̈':'n',
		 'n':'n',
		 '̈':'n',
		 'Ó':'o',
		 'ó':'o',
		 'Ò':'o',
		 'ò':'o',
		 'Ŏ':'o',
		 'ŏ':'o',
		 'Ô':'o',
		 'ô':'o',
		 'Ố':'o',
		 'ố':'o',
		 'Ồ':'o',
		 'ồ':'o',
		 'Ỗ':'o',
		 'ỗ':'o',
		 'Ổ':'o',
		 'ổ':'o',
		 'Ǒ':'o',
		 'ǒ':'o',
		 'Ö':'o',
		 'ö':'o',
		 'Ȫ':'o',
		 'ȫ':'o',
		 'Ő':'o',
		 'ő':'o',
		 'Õ':'o',
		 'õ':'o',
		 'Ṍ':'o',
		 'ṍ':'o',
		 'Ṏ':'o',
		 'ṏ':'o',
		 'Ȭ':'o',
		 'ȭ':'o',
		 'Ȯ':'o',
		 'ȯ':'o',
		 'Ȱ':'o',
		 'ȱ':'o',
		 'Ø':'o',
		 'ø':'o',
		 'Ǿ':'o',
		 'ǿ':'o',
		 'Ǫ':'o',
		 'ǫ':'o',
		 'Ǭ':'o',
		 'ǭ':'o',
		 'Ō':'o',
		 'ō':'o',
		 'Ṓ':'o',
		 'ṓ':'o',
		 'Ṑ':'o',
		 'ṑ':'o',
		 'Ỏ':'o',
		 'ỏ':'o',
		 'Ȍ':'o',
		 'ȍ':'o',
		 'Ȏ':'o',
		 'ȏ':'o',
		 'Ơ':'o',
		 'ơ':'o',
		 'Ớ':'o',
		 'ớ':'o',
		 'Ờ':'o',
		 'ờ':'o',
		 'Ỡ':'o',
		 'ỡ':'o',
		 'Ở':'o',
		 'ở':'o',
		 'Ợ':'o',
		 'ợ':'o',
		 'Ọ':'o',
		 'ọ':'o',
		 'Ộ':'o',
		 'ộ':'o',
		 'Ɵ':'o',
		 'ɵ':'o',
		 'Ṕ':'p',
		 'ṕ':'p',
		 'Ṗ':'p',
		 'ṗ':'p',
		 'Ᵽ':'p',
		 'Ƥ':'p',
		 'ƥ':'p',
		 'P':'p',
		 '̃':'p',
		 'p':'p',
		 '̃':'p',
		 'ʠ':'q',
		 'Ɋ':'q',
		 'ɋ':'q',
		 'Ŕ':'r',
		 'ŕ':'r',
		 'Ř':'r',
		 'ř':'r',
		 'Ṙ':'r',
		 'ṙ':'r',
		 'Ŗ':'r',
		 'ŗ':'r',
		 'Ȑ':'r',
		 'ȑ':'r',
		 'Ȓ':'r',
		 'ȓ':'r',
		 'Ṛ':'r',
		 'ṛ':'r',
		 'Ṝ':'r',
		 'ṝ':'r',
		 'Ṟ':'r',
		 'ṟ':'r',
		 'Ɍ':'r',
		 'ɍ':'r',
		 'ᵲ':'r',
		 'ɼ':'r',
		 'Ɽ':'r',
		 'ɽ':'r',
		 'ɾ':'r',
		 'ᵳ':'r',
		 'ß':'s',
		 'Ś':'s',
		 'ś':'s',
		 'Ṥ':'s',
		 'ṥ':'s',
		 'Ŝ':'s',
		 'ŝ':'s',
		 'Š':'s',
		 'š':'s',
		 'Ṧ':'s',
		 'ṧ':'s',
		 'Ṡ':'s',
		 'ṡ':'s',
		 'ẛ':'s',
		 'Ş':'s',
		 'ş':'s',
		 'Ṣ':'s',
		 'ṣ':'s',
		 'Ṩ':'s',
		 'ṩ':'s',
		 'Ș':'s',
		 'ș':'s',
		 'ʂ':'s',
		 'S':'s',
		 '̩':'s',
		 's':'s',
		 '̩':'s',
		 'Þ':'t',
		 'þ':'t',
		 'Ť':'t',
		 'ť':'t',
		 'T':'t',
		 '̈':'t',
		 'ẗ':'t',
		 'Ṫ':'t',
		 'ṫ':'t',
		 'Ţ':'t',
		 'ţ':'t',
		 'Ṭ':'t',
		 'ṭ':'t',
		 'Ț':'t',
		 'ț':'t',
		 'Ṱ':'t',
		 'ṱ':'t',
		 'Ṯ':'t',
		 'ṯ':'t',
		 'Ŧ':'t',
		 'ŧ':'t',
		 'Ⱦ':'t',
		 'ⱦ':'t',
		 'ᵵ':'t',
		 'ƫ':'t',
		 'Ƭ':'t',
		 'ƭ':'t',
		 'Ʈ':'t',
		 'ʈ':'t',
		 'ȶ':'t',
		 'Ú':'u',
		 'ú':'u',
		 'Ù':'u',
		 'ù':'u',
		 'Ŭ':'u',
		 'ŭ':'u',
		 'Û':'u',
		 'û':'u',
		 'Ǔ':'u',
		 'ǔ':'u',
		 'Ů':'u',
		 'ů':'u',
		 'Ü':'u',
		 'ü':'u',
		 'Ǘ':'u',
		 'ǘ':'u',
		 'Ǜ':'u',
		 'ǜ':'u',
		 'Ǚ':'u',
		 'ǚ':'u',
		 'Ǖ':'u',
		 'ǖ':'u',
		 'Ű':'u',
		 'ű':'u',
		 'Ũ':'u',
		 'ũ':'u',
		 'Ṹ':'u',
		 'ṹ':'u',
		 'Ų':'u',
		 'ų':'u',
		 'Ū':'u',
		 'ū':'u',
		 'Ṻ':'u',
		 'ṻ':'u',
		 'Ủ':'u',
		 'ủ':'u',
		 'Ȕ':'u',
		 'ȕ':'u',
		 'Ȗ':'u',
		 'ȗ':'u',
		 'Ư':'u',
		 'ư':'u',
		 'Ứ':'u',
		 'ứ':'u',
		 'Ừ':'u',
		 'ừ':'u',
		 'Ữ':'u',
		 'ữ':'u',
		 'Ử':'u',
		 'ử':'u',
		 'Ự':'u',
		 'ự':'u',
		 'Ụ':'u',
		 'ụ':'u',
		 'Ṳ':'u',
		 'ṳ':'u',
		 'Ṷ':'u',
		 'ṷ':'u',
		 'Ṵ':'u',
		 'ṵ':'u',
		 'Ʉ':'u',
		 'ʉ':'u',
		 'Ṽ':'v',
		 'ṽ':'v',
		 'Ṿ':'v',
		 'ṿ':'v',
		 'Ʋ':'v',
		 'ʋ':'v',
		 'Ẃ':'w',
		 'ẃ':'w',
		 'Ẁ':'w',
		 'ẁ':'w',
		 'Ŵ':'w',
		 'ŵ':'w',
		 'W':'w',
		 '̊':'w',
		 'ẘ':'w',
		 'Ẅ':'w',
		 'ẅ':'w',
		 'Ẇ':'w',
		 'ẇ':'w',
		 'Ẉ':'w',
		 'ẉ':'w',
		 'Ẍ':'x',
		 'ẍ':'x',
		 'Ẋ':'x',
		 'ẋ':'x',
		 'Ý':'y',
		 'ý':'y',
		 'Ỳ':'y',
		 'ỳ':'y',
		 'Ŷ':'y',
		 'ŷ':'y',
		 'Y':'y',
		 '̊':'y',
		 'ẙ':'y',
		 'Ÿ':'y',
		 'ÿ':'y',
		 'Ỹ':'y',
		 'ỹ':'y',
		 'Ẏ':'y',
		 'ẏ':'y',
		 'Ȳ':'y',
		 'ȳ':'y',
		 'Ỷ':'y',
		 'ỷ':'y',
		 'Ỵ':'y',
		 'ỵ':'y',
		 'ʏ':'y',
		 'Ɏ':'y',
		 'ɏ':'y',
		 'Ƴ':'y',
		 'ƴ':'y',
		 'Ź':'z',
		 'ź':'z',
		 'Ẑ':'z',
		 'ẑ':'z',
		 'Ž':'z',
		 'ž':'z',
		 'Ż':'z',
		 'ż':'z',
		 'Ẓ':'z',
		 'ẓ':'z',
		 'Ẕ':'z',
		 'ẕ':'z',
		 'Ƶ':'z',
		 'ƶ':'z',
		 'Ȥ':'z',
		 'ȥ':'z',
		 'ʐ':'z',
		 'ʑ':'z',
		 'Ⱬ':'z',
		 'ⱬ':'z',
		 'Ǯ':'z',
		 'ǯ':'z',
		 'ƺ':'z',
		  
		 // Roman fullwidth ascii equivalents: 0xff00 to 0xff5e
		 '２':'2',
		 '６':'6',
		 'Ｂ':'B',
		 'Ｆ':'F',
		 'Ｊ':'J',
		 'Ｎ':'N',
		 'Ｒ':'R',
		 'Ｖ':'V',
		 'Ｚ':'Z',
		 'ｂ':'b',
		 'ｆ':'f',
		 'ｊ':'j',
		 'ｎ':'n',
		 'ｒ':'r',
		 'ｖ':'v',
		 'ｚ':'z',
		 '１':'1',
		 '５':'5',
		 '９':'9',
		 'Ａ':'A',
		 'Ｅ':'E',
		 'Ｉ':'I',
		 'Ｍ':'M',
		 'Ｑ':'Q',
		 'Ｕ':'U',
		 'Ｙ':'Y',
		 'ａ':'a',
		 'ｅ':'e',
		 'ｉ':'i',
		 'ｍ':'m',
		 'ｑ':'q',
		 'ｕ':'u',
		 'ｙ':'y',
		 '０':'0',
		 '４':'4',
		 '８':'8',
		 'Ｄ':'D',
		 'Ｈ':'H',
		 'Ｌ':'L',
		 'Ｐ':'P',
		 'Ｔ':'T',
		 'Ｘ':'X',
		 'ｄ':'d',
		 'ｈ':'h',
		 'ｌ':'l',
		 'ｐ':'p',
		 'ｔ':'t',
		 'ｘ':'x',
		 '３':'3',
		 '７':'7',
		 'Ｃ':'C',
		 'Ｇ':'G',
		 'Ｋ':'K',
		 'Ｏ':'O',
		 'Ｓ':'S',
		 'Ｗ':'W',
		 'ｃ':'c',
		 'ｇ':'g',
		 'ｋ':'k',
		 'ｏ':'o',
		 'ｓ':'s',
		 'ｗ':'w'},
		 
	// given the string, s, return a new string with accented characters converted
	// to their ASCII equivalents 
	mapAccents: function(s) {
	 	if (!s) { return ''; }
	     var ret = '';
	     var i;
	     for (i = 0; i < s.length; ++i) {
	 		ret += this._accent_map[s.charAt(i)] || s.charAt(i);
	     }
	     return ret;
	 }
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.DndListbox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.DndListbox"] = true;
/**
 * A list box supporting drag and drop, filtering and sorting
 * This is different that a select list in that the DndListbox's value is its contents, not what is selected
 * 
 * Creating a DndListbox using markup
 * <div id="l1" dojoType="ibm.tivoli.tip.dijit.DndListbox" filtered="true" sorted="true" accept="foo" type="foo"></div>
 * 		filtered: if true, the filter input box is displayed and the user may filter the list's contents
 * 		sorted: if true, the list is kept sorted. Dragging and dropping within the list is prohibited
 * 		accept: a comma separated list of types that may be dropped into this list box
 * 		type: a comma separated list of the types of items in this list
 * 
 * Creating a DndListbox programmatically
 * 	var zappas = new ibm.tivoli.tip.dijit.DndListbox({filtered:false, sorted:true, accept:["zappa"], type:[""celebrity", "zappa"]}, domNode);
 * 	zappas.startup();
 * 	zappas.attr('value', [ 'Frank', 'Dweezil', 'Moon Unit', 'Ahmet', 'Diva Thin Muffin Pigeen', 'Gail']);
 * 
 * Events:
 * 	OnChange: fired after something is dropped in the list
 * 					Is also fired while the list is being filtered, which can happen quite often while the 
 * 					user is typing, so be careful what you do in the event handler
 * 
 * TODO: the template includes some fixed widths that should be removed
 * TODO: support dojo.data.*Store as a data source
 * TODO: TIP compliant styling
 * TODO: support a disabled attribute (prevent user interaction and form submission)
 * TODO: Accessibility support
 * TODO: improve the creator function so it doesn't morph dropped items into this list's type?
 */
dojo.provide("ibm.tivoli.tip.dijit.DndListbox");








dojo.declare("ibm.tivoli.tip.dijit.DndListbox", [dijit._Widget, dijit._Templated, ibm.tivoli.tip.dojo.AccentMapper],
{
	widgetsInTemplate: true,
	templateString:"<div class=\"tip-DndListbox\" >\r\n\t<label for=\"${id}_listbox\">${label}</label>\r\n\t<div class=\"quick-filter\" dojoAttachPoint=\"_filterBox\" >\r\n\t\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" style=\"width:100%\">\r\n\t\t<tr>\r\n\t\t\t<td class=\"qf-lcap\" >&#160;</td>\r\n\t\t\t<td class=\"qf-mid\" valign=\"middle\">\r\n\t\t\t\t<div style=\"position:relative;\">\r\n\t\t\t\t\t<label title=\"${filterTip}\" >\r\n\t\t\t\t\t\t<input dojoAttachPoint=\"_filterInput\" type=\"text\" class=\"qf-textfield\" tabindex=\"0\"\r\n\t\t\t\t\t\t\tdojoAttachEvent=\"onblur:quickFilterDefault, onfocus:quickFilterFocus, onkeyup:onFilterChange\" \r\n\t\t\t\t\t\t/>\r\n\t\t\t\t\t</label>\r\n\t\t\t\t\t<span dojoAttachPoint=\"_filterHint\" class=\"qf-hint\" dojoAttachEvent=\"onclick:quickFilterClickHint\">${filterHint}</span>\r\n\t\t\t\t</div>\r\n\t\t\t</td>\r\n\t\t\t<td class=\"qf-rcap\" dojoAttachEvent=\"onclick:stopFilter,onmouseover:quickFilterClearOver,onmouseout:quickFilterClearOut\"><div class=\"qf-clear\">x</div></td>\r\n\t\t</tr>\r\n\t\t</table>\r\n\t</div>\r\n\t<div id=\"${id}_listbox\" class=\"balContainer\" dojoAttachPoint=\"_scrollBox\" tabindex=\"-1\">\r\n\t\t<ol class=\"dndContainer\" dojoAttachPoint=\"_sourceContainer\"></ol>\r\n\t</div>\r\n</div>\r\n",
	filtered: true,	// show the filter text box?
	sorted: false,		// sort the list contents?
	accept: ["text"],
	type: ["text"],
	name: "",
	label: "",
	_source: null,		// my dojo.dnd.Source
	_filterTimer: -1,	// timer ID
	// nls strings
	filterTip: "",
	filterHint: "",
	constructor: function()
	{
		
		// preload imagery
		if(dojo.hasClass(dojo.body(), "tundra")) {
			var imgpath = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/");
			var m = new Image(); m.src = imgpath + "dndMove.png";
			var n = new Image(); n.src = imgpath + "dndNoMove.png";
			var o = new Image(); o.src = imgpath + "lb_qf_bkg_cap_l_focus.gif";
			var p = new Image(); p.src = imgpath + "lb_qf_bkg_mid_focus.gif";
			var q = new Image(); q.src = imgpath + "lb_qf_bkg_cap_r_focus.gif";
		}
	},
	postMixInProperties: function()
	{
		this.filterTip = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources").FILTERTIP;
		this.filterHint = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources").FILTERHINT;
		this.inherited(arguments);
	},
	startup: function()
	{
		this._source = new dojo.dnd.Source(this._sourceContainer, {creator: dojo.hitch(this, this._creator), accept: this.accept});
		if(this.sorted) {
			// prevent dragging items w/in the list
			this._source.checkAcceptance = dojo.hitch(this._source, function(source, nodes) {
				if(source === this) 
					return false;
				
				// a copy of dojo.dnd.Source.checkAcceptance
				for(var i = 0; i < nodes.length; ++i){
					var type = source.getItem(nodes[i].id).type;
					// type instanceof Array
					var flag = false;
					for(var j = 0; j < type.length; ++j){
						if(type[j] in this.accept){
							flag = true;
							break;
						}
					}
					if(!flag){
						return false;	// Boolean
					}
				}
				return true;
			});
		}
		
		// keep the list updated after dropping
		this.connect(this._source, "onDrop", this._onDrop);
		
		if(!this.filtered) {
			dojo.style(this._filterBox, "display", "none");
		}
		
		// event handlers to manage focus and keyboard input
		this.connect(this._scrollBox, "onkeypress", this._onListKeyHandler);
		this.connect(this._scrollBox, "onclick", this._findFocusedItem);
			
		this.inherited(arguments);
	},
	
	destroy: function()
	{
		if(this._fhandle) 
			dojo.disconnect(this._fhandle);
		this.inherited(arguments);
	},
	/**
	 * addData()
	 * 
	 * Add data to the available list
	 * 
	 * Input:
	 * 	data: the data being added to the available list
	 * 
	 */
	addData: function(data)
	{
		this._source.insertNodes(false, dojo.isArray(data) ? data : [data]);
		this._onListChange();
	},
	deleteSelected: function()
	{
		this._source.deleteSelectedNodes();
		this._onListChange();
	},
	deleteAll: function()
	{
		this._source.selectAll().deleteSelectedNodes();
		this._onListChange();
	},
	// attribute getters/setters
	_getValueAttr: function()
	{
		var itemArray = [];
		dojo.query("input", this._sourceContainer).forEach(function(n) {
			itemArray.push(n.value);
		});
		return itemArray;
	},
	_setValueAttr: function(value)
	{
		// TODO: setting the value should replace the list contents, not append
		this.addData(value);
	},
	getItem: function(key)
	{
		return this._source.getItem(key);
	},
	delItem: function(key)
	{
		this._source.delItem(key);
		var n = dojo.byId(key);
		dojo.destroy(n);
	},
	getAllNodes: function()
	{
		return this._source.getAllNodes();
	},
	getSelectedNodes: function()
	{
		if(this._source == null)
			return [];
		else
			return this._source.getSelectedNodes();
	},
	deleteSelectedNodes: function()
	{
		return this._source.deleteSelectedNodes();
	},
	selectAll: function()
	{
		this._source.selectAll();
	},
	selectNone: function()
	{
		this._source.selectNone();
	},
	insertNodes: function(nodes)
	{
		this._source.insertNodes(false, nodes);
		this._onListChange();
	},
	_creator: function(item, hint)
	{
		// this function isn't quite right. when you drop an object into this list
		// it morphs into this.type, and forgets what it was.
		var n;
		if("avatar" === hint) {
			n = dojo.create("span", {innerHTML: item.toString()});
		}
		else {
			var val = item.toString();
			n = dojo.create("li", {dndData: val, tabindex: -1});
			var ii = dojo.create("input", {type:"text", readonly:"true", value: val, tabindex: -1, title: item.toString()});
			if(this.name) {
				dojo.attr(ii, {name: this.name});
			}
			n.appendChild(ii);
		}
		//console.log("item: ", item, "type: ",this.type);
		return {node: n, data: item, type: this.type};
	},
	//----------------------- manage focus ----------------------------------------
	focusedItem: null,
	/**
	 * called onclick in the _scrollBox
	 * find the clicked-on item, and give it focus
	 */
	_findFocusedItem: function(evt)
	{
		var nl = this.getAllNodes();
		if(nl.length == 0)
			return;
		
		var elem = evt.target;
		while(elem != null && "string" == typeof elem.tagName && elem.tagName.toLowerCase() != "li") {
			elem = elem.parentNode;
		}
		if(null != elem && "string" == typeof elem.tagName && elem.tagName.toLowerCase() == "li") {
			this._moveFocus(elem);
		}
	},
	_onListFocus: function(evt)
	{
		console.log("DndListbox._onListFocus: ", evt.target);
		console.log("   _onListFocus focusedItem before: ", this.focusedItem);
		var new_focused_item = this._findFirstItem();
		this._moveFocus(new_focused_item);
		console.log("   _onListFocus focusedItem after: ", this.focusedItem);
	},
	_onListKeyHandler: function(evt)
	{
		console.log("DndListBox._onListKeyHandler [%s: %s] key='%s'", evt.target.tagName, evt.target.id, evt.charOrCode);
		
		var new_focused_item = null;
		switch(evt.charOrCode) {
		case dojo.keys.TAB:
			// the list looses focus
			console.log("--- TAB");
			this._moveFocus(null);
			// must be done async do focus can move to the next element before
			// the top list item gets tabindex=0, or shift-tab always lands there
			window.setTimeout(dojo.hitch(this, this._swizzleTabindex), 10);
			break;
		case dojo.keys.DOWN_ARROW:
			// move focus down one list item
			console.log("--- DOWN");
			new_focused_item = this._findNextItem();
			this._moveFocus(new_focused_item);
			dojo.stopEvent(evt);
			break;
		case dojo.keys.UP_ARROW:
			// move focus up one list item
			console.log("--- UP");
			new_focused_item = this._findPrevItem();
			this._moveFocus(new_focused_item);
			dojo.stopEvent(evt);
			break;
		case " ":
			// toggle focused item selection
			console.log("--- SPACE");
			if(null !== this.focusedItem) {
				var isSelected = this._isSelected(this.focusedItem);
				if(!evt.ctrlKey && !evt.shiftKey) {
					this._source.selectNone();
				}
				if(isSelected) {
					dojo.removeClass(this.focusedItem, "dojoDndItemSelected");
					dojo.removeClass(this.focusedItem, "dojoDndItemAnchor");
				}
				else {
					if(this._source.getSelectedNodes().length == 0) {
						this._source.anchor = this.focusedItem;
						dojo.addClass(this.focusedItem, "dojoDndItemAnchor");
					}
					else {
						dojo.addClass(this.focusedItem, "dojoDndItemSelected");
					}
				}
				this._source.selection[this.focusedItem.id] = this._isSelected(this.focusedItem) ? 1 : 0;
				this.OnChange();
			}
			dojo.stopEvent(evt);
			break;
		case dojo.keys.ENTER:
			// toggle focused item selection
			console.log("--- SPACE");
			if(null !== this.focusedItem) {
				var isSelected = this._isSelected(this.focusedItem);
				if(!evt.ctrlKey && !evt.shiftKey) {
					this._source.selectNone();
				}
				if(isSelected) {
					dojo.removeClass(this.focusedItem, "dojoDndItemSelected");
					dojo.removeClass(this.focusedItem, "dojoDndItemAnchor");
				}
				else {
					if(this._source.getSelectedNodes().length == 0) {
						this._source.anchor = this.focusedItem;
						dojo.addClass(this.focusedItem, "dojoDndItemAnchor");
					}
					else {
						dojo.addClass(this.focusedItem, "dojoDndItemSelected");
					}
				}
				this._source.selection[this.focusedItem.id] = this._isSelected(this.focusedItem) ? 1 : 0;
				this.OnChange();
			}
			dojo.stopEvent(evt);
			break;
		}
	},
	_isSelected: function(node)
	{
		return dojo.hasClass(this.focusedItem, "dojoDndItemSelected") 
			|| dojo.hasClass(this.focusedItem, "dojoDndItemAnchor");
	},
	_findFirstItem: function()
	{
		var nl = this._source.getAllNodes();
		return nl.length > 0 ? nl[0] : null;
	},
	_findNextItem: function()
	{
		console.log("DndListbox._findNextItem");
		console.log("   focusedItem: ", this.focusedItem);
		
		var nodes = this._source.getAllNodes();
		if(nodes.length === 0)
			return null;
		
		var i = 0;
		if(null !== this.focusedItem) {
			for(i = 0; i < nodes.length; ++i) {
				if(nodes[i] === this.focusedItem) {
					++i;
					break;
				}
			}
		}
		console.log("   next item: ", i);
		return (i < nodes.length) ? nodes[i] : nodes[nodes.length-1];
	},
	_findPrevItem: function()
	{
		console.log("DndListbox._findPrevItem");
		console.log("   focusedItem: ", this.focusedItem);
		var nodes = this._source.getAllNodes();
		if(nodes.length === 0)
			return null;
		
		var i = nodes.length -1;
		for(;i > 0; --i) {
			if(nodes[i] === this.focusedItem) {
				--i;
				break;
			}
		}
		console.log("   prev item: ", i);
		return (i >= 0) ? nodes[i] : nodes[0];
	},
	_moveFocus: function(new_focused_item)
	{
		console.log("_moveFocus from ", this.focusedItem, " to ", new_focused_item);
		if(this.focusedItem !== new_focused_item) {
			if(this.focusedItem) {
				dojo.removeClass(this.focusedItem, "dojoDndItemFocused");
				dojo.attr(this.focusedItem, "tabindex", "-1");
			}
			if(null !== new_focused_item) {
				console.log("   focusing ", new_focused_item.id);
				dojo.addClass(new_focused_item, "dojoDndItemFocused");
				dojo.attr(new_focused_item, "tabindex", "0");
				new_focused_item.focus();
			}
			this.focusedItem = new_focused_item;
		}
		console.log("  _moveFocus  focusedItem after: ", this.focusedItem);
	},
	_onDrop: function(source, nodes, copy)
	{
		console.log("DndListbox._onDrop");
		this._onListChange();
	},
	_fhandle: null,
	_swizzleTabindex: function()
	{
		var i;
		var nl = this._source.getAllNodes();
		if(nl.length > 0) {
			if(this._fhandle) dojo.disconnect(this._fhandle);
			if(this._bhandle) dojo.disconnect(this._bhandle);
			this._fhandle = dojo.connect(nl[0], "onfocus", this, this._onListFocus);
			dojo.attr(nl[0], "tabindex", "0");
		}
		for(i = 1; i < nl.length; ++i) {
			dojo.attr(nl[i], "tabindex", "-1");
		}
	},
	_onListChange: function()
	{
		console.log(this.id + "._onListChange");
		if(this.filtered) {
			// unfilter
			this.stopFilter();
		}

		if(this.sorted) {
			// sort
			var list = this._source;
			var itemArray = [];
			this._source.forInItems(function(obj, id, container) {
				itemArray.push(obj.data);
				list.delItem(id);
				list.node.removeChild(dojo.byId(id));
			}, null);
			list = null;
			itemArray.sort(function(a, b) {return a.toString().localeCompare(b.toString());});
			this._source.insertNodes(false, itemArray);
		}
		this._source.sync();
		this._swizzleTabindex();
		if(this.focusedItem) {
			this._moveFocus(this._findFirstItem());
		}
		this.OnChange();	// fire 
	},
	// -------------------- filtering support -------------------------------
	quickFilterFocus: function(evt) 
	{
		dojo.addClass(this._filterBox, "quick-filter-focus");
		this.quickFilterHint();
	},
	quickFilterDefault: function (evt) 
	{
		dojo.removeClass(this._filterBox, "quick-filter-focus");
		dojo.removeClass(this._filterBox, "quick-filter-over");
		this.quickFilterHint();
	},

	/**
	* Handle the keyUp event on an input element to start a 
	* filter operation on the list
	*/
	onFilterChange: function(evt)
	{
		var inputElement = evt.target;
		this.startFilter();
	},
	/**
	 * Give the filter text box focus when the hint test is clicked
	 */
	quickFilterClickHint: function()
	{
		this._filterInput.focus();
	},
	/**
	 * Manage visibility of the filter hint
	 */
	quickFilterHint: function()
	{
		if(this._filterInput.value.length > 0) {
			dojo.addClass(this._filterHint, "hidden");
		}
		else {
			dojo.removeClass(this._filterHint, "hidden");
		}
	},
	quickFilterClearOver: function(evt)
	{
		dojo.addClass(dojo.query("div.qf-clear", evt.target.parentNode)[0], "hover");
	},
	quickFilterClearOut: function(evt)
	{
		dojo.removeClass(dojo.query("div.qf-clear", evt.target.parentNode)[0], "hover");
	},
	/**
	* Set a timer that will filter the list when it expires.
	*/
	startFilter: function()
	{
		window.clearTimeout( this._filterTimer );
		this._filterTimer = window.setTimeout( dojo.hitch(this, this.filter), 400 );
		return;
	},
	/**
	* Stops the background filter operation by clearing the window
	* timer. 
	*/
	stopFilter: function()
	{
		// stop the timer
		window.clearTimeout( this._filterTimer );
		this._filterTimer = -1;

		// clear the filter if necessary
		if(this._filterInput.value.length > 0) {
			this._filterInput.value = "";
			this.filter();
		}
		return;
	},
	/**
	 * Filter the list box so only those entries matching the what the user has
	 * typed in the filter box are shown
	 */
	filter: function()
	{
		//	Extract the current source from the input element
		var filter_val = this.mapAccents(dojo.trim(this._filterInput.value));
		var filter_re = new RegExp(filter_val, "i");

		var that = this;
		dojo.query("input", this._sourceContainer).forEach(function(n){
			var dndNode = n.parentNode;
			if(filter_re.test(that.mapAccents(n.value))) {
				dojo.removeClass( dndNode, "qfHide");
				dojo.addClass( dndNode, "dojoDndItem" );
			}
			else
			{
				dojo.removeClass( dndNode, "dojoDndItem" );
				dojo.addClass( dndNode, "qfHide" );
			}
		});
		that = null;
		this._source.sync();
		return;
	},
	
	/**
	* This function will disable the drag n drop for the list. It sets the value of accept to null hence will not allow
	* any DnD.  Before setting the value of accept to null, we save a copy of it so that when the list is enabled, we
	* can set the value back to original.
	*
	*/
	setDisabled: function(/*Boolean*/ value){
		if(this._source.accept!==null) {
			if(value){ // disabling it
				this._source.acceptBuffer=this._source.accept;
				this._source.accept=null;
			}		
			else {
				if(this._source.acceptBuffer!==undefined){
					this._source.accept=this._source.acceptBuffer;
			  } 
			}
		}
		else{
			if(!value){
				this._source.accept=this._source.acceptBuffer;
			}
		}
    },
 
	// ------------------------ external events --------------------
	OnChange: function()
	{
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.DropDownList"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.DropDownList"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
 
dojo.provide("ibm.tivoli.tip.dijit.DropDownList"); 




/** 
 * A drop-down list widget.
 * 
 * Such widget displays a drop-down list based on the browser
 * drop-down control. The usage of that control limits a little
 * bit the look&feel (e.g. the drop-down button).
 * The widget accepts the following parameters:
 * 
 *  - options: an object whose properties are the drop-down options.
 *    Each property's name is used as option's value and the corresponding
 *    property'svalue is used as option's displayed text.
 *    For example the object:
 * 
 *          { 
 *              value1: "this is the value #1",  
 *              value2: "this is the value #2" 
 *          }
 *     
 *    maps to the following HTML code
 * 
 *          <select>
 *              <option value="value1">"this is the value #1"</option>
 *              <option value="value2">"this is the value #2"</option>
 *          </select>
 * 
 *    Values (the option's properties) are keys for the options.
 *  - selected: the value of the option to be selected. "" means no 
 *    selection.
 *  - size: the maximum number of options to display in the drop-down
 *    list. If not specified all the options will be shown.
 * 
 * Example of a declaration:
 * 
 *          <div 
 *              dojoType="ibm.tivoli.tip.dijit.DropDownList" 
 *              options="{value1: 'this is the value #1', value2: 'this is the value #2'}"
 *              selected="value2">
 *          </div>
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)     
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.DropDownList",
    [ibm.tivoli.tip.dijit.Widget, dijit._Templated],
    {
        /** the user's provided set of options */
        options: {},
        /** the user's provided selected option (the value) */
        selected: "",
        /** the user's provided number of options to show */
        size: "",
        
        /** the widget template path */
        templateString:"<select \r\n        class=\"dropDownListDefault\"\r\n        dojoAttachPoint=\"_selectNode\"\r\n        dojoAttachEvent=\"onchange:_onChange\"\r\n></select>\r\n",
        
        /** select DOM node */
        _selectNode: null,
        /** the current set of options */
        _options: {},
        /** contains boolean properties which says if an option is hidden or not */
        _hidden: {},
        
        /**
         * Widget constructor.
         */
        constructor: function()
        {
            this.options = {};
            this._options = {};
            this._hidden = {};
        },
        
        /**
         * Sets the initial value.
         */
        startup: function() 
        {
            // call the superclass's method
            this.inherited("startup", arguments);

            // set the user's provided options
            this.setOptions(this.options);
            
            // select the required number of items to show
            if ( this.size !== "" ) {
                this._selectNode.length = parseInt(this.selected, 10);
            }        

            // select the required item
            if ( this.selected !== "" ) {
                this.setValue(this.selected);
            }
        },
        
        /**        
         * Returns the option's value pointed by the current selection.
         * Returns undefined if no selection is in place.
         */
        getValue: function()
        {
            // return the selected value
            if ( this._selectNode.selectedIndex !== -1 ) {
                return this._selectNode.options[this._selectNode.selectedIndex].value;
            } 
        },
        
        /**        
         * Sets the widget value.
         * 
         *  @param value: 
         *      the value of the option to be selected.
         */
        setValue: function(value)
        {
            var index = this._getIndexByValue(value);
            this._selectNode.selectedIndex = index;
        },
        
        /**
         * Returns true if the widget's value is not set.
         */
        isEmpty: function()
        {
            return ( this._selectNode.selectedIndex === -1 );
        },

        /**
         * Enables the widget for editing. 
         */
        setDisabled: function(isDisabled)
        {
            if ( isDisabled != this.isDisabled() ) {
                // update the enablement status
                this.inherited("setDisabled", arguments);
                // update the widget
                this._updateWidget();
            }
        },
        
        /**        
         * Sets the drop-down options.
         * 
         *  @param options: 
         *      an object whose properties are the drop-down options.
         *      Each property's name is used as option's value and 
         *      the corresponding is used as option's displayed text.
         */
        setOptions: function(options)
        {
            // clean-up the previous options
            var o = this._selectNode.options;
            o.length = 0; 
            // populate the options
            for ( var value in options ) {
                if ( ! this._hidden[value] ) {
                    o[o.length] = new Option(options[value], value);
                }
            }
            // store the options
            this._options = options;  
        },

        /**
         * Returns an object whose property's maps to the option's 
         * value and the property's value maps to the option's displayed text. 
         */
        getOptions: function()
        {
            return this._options;
        },
        
        /**
         * Hides the option corresponding to the given value.
         * 
         *  @param value:
         *      the value to be hidden.
         */
        hideOption: function(value)
        {
            this._hideOption(value, true);
        },
        
        /**
         * Shows the option corresponding to the given value.
         * 
         *  @param value:
         *      the value to be shown.
         */
        showOption: function(value)
        {
            this._hideOption(value, false);
        },
        
        /**
         * Disables/enables theoption corresponding to the given value.
         * 
         *  @param value:
         *      the value to be shown.
         */
        disableOption: function(value, disabled)
        {
            var option = this._selectNode.options[this._getIndexByValue(value)];
            option.disabled = disabled;
        },
        
        /**
         * Returns the text of the current selected item.
         */
        getSelectedText: function()
        {
            return this._selectNode.options[this._selectNode.selectedIndex].text;
        },
        
        /**
         * Handles the onchange event.  
         */
        _onChange: function()
        {
            // notify the change
            this.onValueChanged(this.getValue());
        },
        
        /**
         * Returns the index of the option which matches the given value.
         * If the option is not matched return -1.
         */
        _getIndexByValue: function(value)
        {
            var o = this._selectNode.options;
            var ol = o.length;
            for ( var i = 0; i < ol; i++ ) {
                if ( o[i].value === value ) {
                    return i;   
                }
            }
            return -1;
        },
        
        /**
         * Hides/shows the option corresponding to the given value.
         * 
         *  @param value:
         *      the value to be hidden/shown.
         *  @param hide:
         *      true if the option must be hidden, false if must be shown.
         */
        _hideOption: function(value, hide)
        {
            // store the currently selected value
            var v = this.getValue();
            // hide/show the value
            this._hidden[value] = hide;
            // reset the options 
            this.setOptions(this._options);
            // select the previous value
            if ( v ) {
                this.setValue(v);
            }
        },
        
        /**
         * Updates the widget look&feel depending on the widget' status.
         */
        _updateWidget: function()
        {
            // disable the select node
            this._selectNode.disabled = this.isDisabled();             
            // set the style
            this.domNode.className = this.isDisabled() ? "dropDownListDisabled" : "dropDownListDefault";                
        }
    }
);


}

if(!dojo._hasResource["dojo.cookie"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cookie"] = true;
dojo.provide("dojo.cookie");



/*=====
dojo.__cookieProps = function(){
	//	expires: Date|String|Number?
	//		If a number, the number of days from today at which the cookie
	//		will expire. If a date, the date past which the cookie will expire.
	//		If expires is in the past, the cookie will be deleted.
	//		If expires is omitted or is 0, the cookie will expire when the browser closes. << FIXME: 0 seems to disappear right away? FF3.
	//	path: String?
	//		The path to use for the cookie.
	//	domain: String?
	//		The domain to use for the cookie.
	//	secure: Boolean?
	//		Whether to only send the cookie on secure connections
	this.expires = expires;
	this.path = path;
	this.domain = domain;
	this.secure = secure;
}
=====*/


dojo.cookie = function(/*String*/name, /*String?*/value, /*dojo.__cookieProps?*/props){
	//	summary: 
	//		Get or set a cookie.
	//	description:
	// 		If one argument is passed, returns the value of the cookie
	// 		For two or more arguments, acts as a setter.
	//	name:
	//		Name of the cookie
	//	value:
	//		Value for the cookie
	//	props: 
	//		Properties for the cookie
	//	example:
	//		set a cookie with the JSON-serialized contents of an object which
	//		will expire 5 days from now:
	//	|	dojo.cookie("configObj", dojo.toJson(config), { expires: 5 });
	//	
	//	example:
	//		de-serialize a cookie back into a JavaScript object:
	//	|	var config = dojo.fromJson(dojo.cookie("configObj"));
	//	
	//	example:
	//		delete a cookie:
	//	|	dojo.cookie("configObj", null, {expires: -1});
	var c = document.cookie;
	if(arguments.length == 1){
		var matches = c.match(new RegExp("(?:^|; )" + dojo.regexp.escapeString(name) + "=([^;]*)"));
		return matches ? decodeURIComponent(matches[1]) : undefined; // String or undefined
	}else{
		props = props || {};
// FIXME: expires=0 seems to disappear right away, not on close? (FF3)  Change docs?
		var exp = props.expires;
		if(typeof exp == "number"){ 
			var d = new Date();
			d.setTime(d.getTime() + exp*24*60*60*1000);
			exp = props.expires = d;
		}
		if(exp && exp.toUTCString){ props.expires = exp.toUTCString(); }

		value = encodeURIComponent(value);
		var updatedCookie = name + "=" + value, propName;
		for(propName in props){
			updatedCookie += "; " + propName;
			var propValue = props[propName];
			if(propValue !== true){ updatedCookie += "=" + propValue; }
		}
		document.cookie = updatedCookie;
	}
};

dojo.cookie.isSupported = function(){
	//	summary:
	//		Use to determine if the current browser supports cookies or not.
	//		
	//		Returns true if user allows cookies.
	//		Returns false if user doesn't allow cookies.

	if(!("cookieEnabled" in navigator)){
		this("__djCookieTest__", "CookiesAllowed");
		navigator.cookieEnabled = this("__djCookieTest__") == "CookiesAllowed";
		if(navigator.cookieEnabled){
			this("__djCookieTest__", "", {expires: -1});
		}
	}
	return navigator.cookieEnabled;
};

}

if(!dojo._hasResource["dijit.layout.BorderContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.BorderContainer"] = true;
dojo.provide("dijit.layout.BorderContainer");




dojo.declare(
	"dijit.layout.BorderContainer",
	dijit.layout._LayoutWidget,
{
	// summary:
	//		Provides layout in up to 5 regions, a mandatory center with optional borders along its 4 sides.
	//
	// description:
	//		A BorderContainer is a box with a specified size, such as style="width: 500px; height: 500px;",
	//		that contains a child widget marked region="center" and optionally children widgets marked
	//		region equal to "top", "bottom", "leading", "trailing", "left" or "right".
	//		Children along the edges will be laid out according to width or height dimensions and may
	//		include optional splitters (splitter="true") to make them resizable by the user.  The remaining
	//		space is designated for the center region.
	//
	//		NOTE: Splitters must not be more than 50 pixels in width.
	//
	//		The outer size must be specified on the BorderContainer node.  Width must be specified for the sides
	//		and height for the top and bottom, respectively.  No dimensions should be specified on the center;
	//		it will fill the remaining space.  Regions named "leading" and "trailing" may be used just like
	//		"left" and "right" except that they will be reversed in right-to-left environments.
	//
	// example:
	// |	<div dojoType="dijit.layout.BorderContainer" design="sidebar" gutters="false"
	// |            style="width: 400px; height: 300px;">
	// |		<div dojoType="ContentPane" region="top">header text</div>
	// |		<div dojoType="ContentPane" region="right" splitter="true" style="width: 200px;">table of contents</div>
	// |		<div dojoType="ContentPane" region="center">client area</div>
	// |	</div>

	// design: String
	//		Which design is used for the layout:
	//			- "headline" (default) where the top and bottom extend
	//				the full width of the container
	//			- "sidebar" where the left and right sides extend from top to bottom.
	design: "headline",

	// gutters: Boolean
	//		Give each pane a border and margin.
	//		Margin determined by domNode.paddingLeft.
	//		When false, only resizable panes have a gutter (i.e. draggable splitter) for resizing.
	gutters: true,

	// liveSplitters: Boolean
	//		Specifies whether splitters resize as you drag (true) or only upon mouseup (false)
	liveSplitters: true,

	// persist: Boolean
	//		Save splitter positions in a cookie.
	persist: false,

	baseClass: "dijitBorderContainer",

	// _splitterClass: String
	// 		Optional hook to override the default Splitter widget used by BorderContainer
	_splitterClass: "dijit.layout._Splitter",

	postMixInProperties: function(){
		// change class name to indicate that BorderContainer is being used purely for
		// layout (like LayoutContainer) rather than for pretty formatting.
		if(!this.gutters){
			this.baseClass += "NoGutter";
		}
		this.inherited(arguments);
	},

	postCreate: function(){
		this.inherited(arguments);

		this._splitters = {};
		this._splitterThickness = {};
	},

	startup: function(){
		if(this._started){ return; }
		dojo.forEach(this.getChildren(), this._setupChild, this);
		this.inherited(arguments);
	},

	_setupChild: function(/*dijit._Widget*/ child){
		// Override _LayoutWidget._setupChild().

		var region = child.region;
		if(region){
			this.inherited(arguments);

			dojo.addClass(child.domNode, this.baseClass+"Pane");

			var ltr = this.isLeftToRight();
			if(region == "leading"){ region = ltr ? "left" : "right"; }
			if(region == "trailing"){ region = ltr ? "right" : "left"; }

			//FIXME: redundant?
			this["_"+region] = child.domNode;
			this["_"+region+"Widget"] = child;

			// Create draggable splitter for resizing pane,
			// or alternately if splitter=false but BorderContainer.gutters=true then
			// insert dummy div just for spacing
			if((child.splitter || this.gutters) && !this._splitters[region]){
				var _Splitter = dojo.getObject(child.splitter ? this._splitterClass : "dijit.layout._Gutter");
				var splitter = new _Splitter({
					id: child.id + "_splitter",
					container: this,
					child: child,
					region: region,
					live: this.liveSplitters
				});
				splitter.isSplitter = true;
				this._splitters[region] = splitter.domNode;
				dojo.place(this._splitters[region], child.domNode, "after");

				// Splitters arent added as Contained children, so we need to call startup explicitly
				splitter.startup();
			}
			child.region = region;
		}
	},

	_computeSplitterThickness: function(region){
		this._splitterThickness[region] = this._splitterThickness[region] ||
			dojo.marginBox(this._splitters[region])[(/top|bottom/.test(region) ? 'h' : 'w')];
	},

	layout: function(){
		// Implement _LayoutWidget.layout() virtual method.
		for(var region in this._splitters){ this._computeSplitterThickness(region); }
		this._layoutChildren();
	},

	addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
		// Override _LayoutWidget.addChild().
		this.inherited(arguments);
		if(this._started){
			this.layout(); //OPT
		}
	},

	removeChild: function(/*dijit._Widget*/ child){
		// Override _LayoutWidget.removeChild().
		var region = child.region;
		var splitter = this._splitters[region];
		if(splitter){
			dijit.byNode(splitter).destroy();
			delete this._splitters[region];
			delete this._splitterThickness[region];
		}
		this.inherited(arguments);
		delete this["_"+region];
		delete this["_" +region+"Widget"];
		if(this._started){
			this._layoutChildren();
		}
		dojo.removeClass(child.domNode, this.baseClass+"Pane");
	},

	getChildren: function(){
		// Override _LayoutWidget.getChildren() to only return real children, not the splitters.
		return dojo.filter(this.inherited(arguments), function(widget){
			return !widget.isSplitter;
		});
	},

	getSplitter: function(/*String*/region){
		// summary:
		//		Returns the widget responsible for rendering the splitter associated with region
		var splitter = this._splitters[region];
		return splitter ? dijit.byNode(splitter) : null;
	},

	resize: function(newSize, currentSize){
		// Overrides _LayoutWidget.resize().

		// resetting potential padding to 0px to provide support for 100% width/height + padding
		// TODO: this hack doesn't respect the box model and is a temporary fix
		if(!this.cs || !this.pe){
			var node = this.domNode;
			this.cs = dojo.getComputedStyle(node);
			this.pe = dojo._getPadExtents(node, this.cs);
			this.pe.r = dojo._toPixelValue(node, this.cs.paddingRight);
			this.pe.b = dojo._toPixelValue(node, this.cs.paddingBottom);

			dojo.style(node, "padding", "0px");
		}

		this.inherited(arguments);
	},

	_layoutChildren: function(/*String?*/changedRegion, /*Number?*/ changedRegionSize){
		// summary:
		//		This is the main routine for setting size/position of each child.
		// description:
		//		With no arguments, measures the height of top/bottom panes, the width
		//		of left/right panes, and then sizes all panes accordingly.
		//
		//		With changedRegion specified (as "left", "top", "bottom", or "right"),
		//		it changes that region's width/height to changedRegionSize and
		//		then resizes other regions that were affected.
		// changedRegion:
		//		The region should be changed because splitter was dragged.
		//		"left", "right", "top", or "bottom".
		// changedRegionSize:
		//		The new width/height (in pixels) to make changedRegion

		if(!this._borderBox || !this._borderBox.h){
			// We are currently hidden, or we haven't been sized by our parent yet.
			// Abort.   Someone will resize us later.
			return;
		}

		var sidebarLayout = (this.design == "sidebar");
		var topHeight = 0, bottomHeight = 0, leftWidth = 0, rightWidth = 0;
		var topStyle = {}, leftStyle = {}, rightStyle = {}, bottomStyle = {},
			centerStyle = (this._center && this._center.style) || {};

		var changedSide = /left|right/.test(changedRegion);

		var layoutSides = !changedRegion || (!changedSide && !sidebarLayout);
		var layoutTopBottom = !changedRegion || (changedSide && sidebarLayout);

		// Ask browser for width/height of side panes.
		// Would be nice to cache this but height can change according to width
		// (because words wrap around).  I don't think width will ever change though
		// (except when the user drags a splitter).
		if(this._top){
			topStyle = (changedRegion == "top" || layoutTopBottom) && this._top.style;
			topHeight = changedRegion == "top" ? changedRegionSize : dojo.marginBox(this._top).h;
		}
		if(this._left){
			leftStyle = (changedRegion == "left" || layoutSides) && this._left.style;
			leftWidth = changedRegion == "left" ? changedRegionSize : dojo.marginBox(this._left).w;
		}
		if(this._right){
			rightStyle = (changedRegion == "right" || layoutSides) && this._right.style;
			rightWidth = changedRegion == "right" ? changedRegionSize : dojo.marginBox(this._right).w;
		}
		if(this._bottom){
			bottomStyle = (changedRegion == "bottom" || layoutTopBottom) && this._bottom.style;
			bottomHeight = changedRegion == "bottom" ? changedRegionSize : dojo.marginBox(this._bottom).h;
		}

		var splitters = this._splitters;
		var topSplitter = splitters.top, bottomSplitter = splitters.bottom,
			leftSplitter = splitters.left, rightSplitter = splitters.right;
		var splitterThickness = this._splitterThickness;
		var topSplitterThickness = splitterThickness.top || 0,
			leftSplitterThickness = splitterThickness.left || 0,
			rightSplitterThickness = splitterThickness.right || 0,
			bottomSplitterThickness = splitterThickness.bottom || 0;

		// Check for race condition where CSS hasn't finished loading, so
		// the splitter width == the viewport width (#5824)
		if(leftSplitterThickness > 50 || rightSplitterThickness > 50){
			setTimeout(dojo.hitch(this, function(){
				// Results are invalid.  Clear them out.
				this._splitterThickness = {};

				for(var region in this._splitters){
					this._computeSplitterThickness(region);
				}
				this._layoutChildren();
			}), 50);
			return false;
		}

		var pe = this.pe;

		var splitterBounds = {
			left: (sidebarLayout ? leftWidth + leftSplitterThickness: 0) + pe.l + "px",
			right: (sidebarLayout ? rightWidth + rightSplitterThickness: 0) + pe.r + "px"
		};

		if(topSplitter){
			dojo.mixin(topSplitter.style, splitterBounds);
			topSplitter.style.top = topHeight + pe.t + "px";
		}

		if(bottomSplitter){
			dojo.mixin(bottomSplitter.style, splitterBounds);
			bottomSplitter.style.bottom = bottomHeight + pe.b + "px";
		}

		splitterBounds = {
			top: (sidebarLayout ? 0 : topHeight + topSplitterThickness) + pe.t + "px",
			bottom: (sidebarLayout ? 0 : bottomHeight + bottomSplitterThickness) + pe.b + "px"
		};

		if(leftSplitter){
			dojo.mixin(leftSplitter.style, splitterBounds);
			leftSplitter.style.left = leftWidth + pe.l + "px";
		}

		if(rightSplitter){
			dojo.mixin(rightSplitter.style, splitterBounds);
			rightSplitter.style.right = rightWidth + pe.r +	"px";
		}

		dojo.mixin(centerStyle, {
			top: pe.t + topHeight + topSplitterThickness + "px",
			left: pe.l + leftWidth + leftSplitterThickness + "px",
			right: pe.r + rightWidth + rightSplitterThickness + "px",
			bottom: pe.b + bottomHeight + bottomSplitterThickness + "px"
		});

		var bounds = {
			top: sidebarLayout ? pe.t + "px" : centerStyle.top,
			bottom: sidebarLayout ? pe.b + "px" : centerStyle.bottom
		};
		dojo.mixin(leftStyle, bounds);
		dojo.mixin(rightStyle, bounds);
		leftStyle.left = pe.l + "px"; rightStyle.right = pe.r + "px"; topStyle.top = pe.t + "px"; bottomStyle.bottom = pe.b + "px";
		if(sidebarLayout){
			topStyle.left = bottomStyle.left = leftWidth + leftSplitterThickness + pe.l + "px";
			topStyle.right = bottomStyle.right = rightWidth + rightSplitterThickness + pe.r + "px";
		}else{
			topStyle.left = bottomStyle.left = pe.l + "px";
			topStyle.right = bottomStyle.right = pe.r + "px";
		}

		// More calculations about sizes of panes
		var containerHeight = this._borderBox.h - pe.t - pe.b,
			middleHeight = containerHeight - ( topHeight + topSplitterThickness + bottomHeight + bottomSplitterThickness),
			sidebarHeight = sidebarLayout ? containerHeight : middleHeight;

		var containerWidth = this._borderBox.w - pe.l - pe.r,
			middleWidth = containerWidth - (leftWidth + leftSplitterThickness + rightWidth + rightSplitterThickness),
			sidebarWidth = sidebarLayout ? middleWidth : containerWidth;

		// New margin-box size of each pane
		var dim = {
			top:	{ w: sidebarWidth, h: topHeight },
			bottom: { w: sidebarWidth, h: bottomHeight },
			left:	{ w: leftWidth, h: sidebarHeight },
			right:	{ w: rightWidth, h: sidebarHeight },
			center:	{ h: middleHeight, w: middleWidth }
		};

		if(changedRegion){
			// Respond to splitter drag event by changing changedRegion's width or height
			var child = this["_" + changedRegion + "Widget"],
				mb = {};
				mb[ /top|bottom/.test(changedRegion) ? "h" : "w"] = changedRegionSize;
			child.resize ? child.resize(mb, dim[child.region]) : dojo.marginBox(child.domNode, mb);
		}

		// Nodes in IE<8 don't respond to t/l/b/r, and TEXTAREA doesn't respond in any browser
		var janky = dojo.isIE < 8 || (dojo.isIE && dojo.isQuirks) || dojo.some(this.getChildren(), function(child){
			return child.domNode.tagName == "TEXTAREA" || child.domNode.tagName == "INPUT";
		});
		if(janky){
			// Set the size of the children the old fashioned way, by setting
			// CSS width and height

			var resizeWidget = function(widget, changes, result){
				if(widget){
					(widget.resize ? widget.resize(changes, result) : dojo.marginBox(widget.domNode, changes));
				}
			};

			if(leftSplitter){ leftSplitter.style.height = sidebarHeight; }
			if(rightSplitter){ rightSplitter.style.height = sidebarHeight; }
			resizeWidget(this._leftWidget, {h: sidebarHeight}, dim.left);
			resizeWidget(this._rightWidget, {h: sidebarHeight}, dim.right);

			if(topSplitter){ topSplitter.style.width = sidebarWidth; }
			if(bottomSplitter){ bottomSplitter.style.width = sidebarWidth; }
			resizeWidget(this._topWidget, {w: sidebarWidth}, dim.top);
			resizeWidget(this._bottomWidget, {w: sidebarWidth}, dim.bottom);

			resizeWidget(this._centerWidget, dim.center);
		}else{
			// Calculate which panes need a notification that their size has been changed
			// (we've already set style.top/bottom/left/right on those other panes).
			var notifySides = !changedRegion || (/top|bottom/.test(changedRegion) && this.design != "sidebar"),
				notifyTopBottom = !changedRegion || (/left|right/.test(changedRegion) && this.design == "sidebar"),
				notifyList = {
					center: true,
					left: notifySides,
					right: notifySides,
					top: notifyTopBottom,
					bottom: notifyTopBottom
				};
			
			// Send notification to those panes that have changed size
			dojo.forEach(this.getChildren(), function(child){
				if(child.resize && notifyList[child.region]){
					child.resize(null, dim[child.region]);
				}
			}, this);
		}
	},

	destroy: function(){
		for(var region in this._splitters){
			var splitter = this._splitters[region];
			dijit.byNode(splitter).destroy();
			dojo.destroy(splitter);
		}
		delete this._splitters;
		delete this._splitterThickness;
		this.inherited(arguments);
	}
});

// This argument can be specified for the children of a BorderContainer.
// Since any widget can be specified as a LayoutContainer child, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// region: [const] String
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Values: "top", "bottom", "leading", "trailing", "left", "right", "center".
	//		See the `dijit.layout.BorderContainer` description for details.
	region: '',

	// splitter: [const] Boolean
	//		Parameter for child of `dijit.layout.BorderContainer` where region != "center".
	//		If true, enables user to resize the widget by putting a draggable splitter between
	//		this widget and the region=center widget.
	splitter: false,

	// minSize: [const] Number
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Specifies a minimum size (in pixels) for this widget when resized by a splitter.
	minSize: 0,

	// maxSize: [const] Number
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Specifies a maximum size (in pixels) for this widget when resized by a splitter.
	maxSize: Infinity
});



dojo.declare("dijit.layout._Splitter", [ dijit._Widget, dijit._Templated ],
{
	// summary:
	//		A draggable spacer between two items in a `dijit.layout.BorderContainer`.
	// description:
	//		This is instantiated by `dijit.layout.BorderContainer`.  Users should not
	//		create it directly.
	// tags:
	//		private

/*=====
 	// container: [const] dijit.layout.BorderContainer
 	//		Pointer to the parent BorderContainer
	container: null,

	// child: [const] dijit.layout._LayoutWidget
	//		Pointer to the pane associated with this splitter
	child: null,

	// region: String
	//		Region of pane associated with this splitter.
	//		"top", "bottom", "left", "right".
	region: null,
=====*/

	// live: [const] Boolean
	//		If true, the child's size changes and the child widget is redrawn as you drag the splitter;
	//		otherwise, the size doesn't change until you drop the splitter (by mouse-up)
	live: true,

	templateString: '<div class="dijitSplitter" dojoAttachEvent="onkeypress:_onKeyPress,onmousedown:_startDrag,onmouseenter:_onMouse,onmouseleave:_onMouse" tabIndex="0" waiRole="separator"><div class="dijitSplitterThumb"></div></div>',

	postCreate: function(){
		this.inherited(arguments);
		this.horizontal = /top|bottom/.test(this.region);
		dojo.addClass(this.domNode, "dijitSplitter" + (this.horizontal ? "H" : "V"));
//		dojo.addClass(this.child.domNode, "dijitSplitterPane");
//		dojo.setSelectable(this.domNode, false); //TODO is this necessary?

		this._factor = /top|left/.test(this.region) ? 1 : -1;

		this._cookieName = this.container.id + "_" + this.region;
		if(this.container.persist){
			// restore old size
			var persistSize = dojo.cookie(this._cookieName);
			if(persistSize){
				this.child.domNode.style[this.horizontal ? "height" : "width"] = persistSize;
			}
		}
	},

	_computeMaxSize: function(){
		// summary:
		//		Compute the maximum size that my corresponding pane can be set to

		var dim = this.horizontal ? 'h' : 'w',
			thickness = this.container._splitterThickness[this.region];
			
		// Get DOMNode of opposite pane, if an opposite pane exists.
		// Ex: if I am the _Splitter for the left pane, then get the right pane.
		var flip = {left:'right', right:'left', top:'bottom', bottom:'top', leading:'trailing', trailing:'leading'},
			oppNode = this.container["_" + flip[this.region]];
		
		// I can expand up to the edge of the opposite pane, or if there's no opposite pane, then to
		// edge of BorderContainer
		var available = dojo.contentBox(this.container.domNode)[dim] -
				(oppNode ? dojo.marginBox(oppNode)[dim] : 0) -
				20 - thickness * 2;

		return Math.min(this.child.maxSize, available);
	},

	_startDrag: function(e){
		if(!this.cover){
			this.cover = dojo.doc.createElement('div');
			dojo.addClass(this.cover, "dijitSplitterCover");
			dojo.place(this.cover, this.child.domNode, "after");
		}
		dojo.addClass(this.cover, "dijitSplitterCoverActive");

		// Safeguard in case the stop event was missed.  Shouldn't be necessary if we always get the mouse up.
		if(this.fake){ dojo.destroy(this.fake); }
		if(!(this._resize = this.live)){ //TODO: disable live for IE6?
			// create fake splitter to display at old position while we drag
			(this.fake = this.domNode.cloneNode(true)).removeAttribute("id");
			dojo.addClass(this.domNode, "dijitSplitterShadow");
			dojo.place(this.fake, this.domNode, "after");
		}
		dojo.addClass(this.domNode, "dijitSplitterActive");
		dojo.addClass(this.domNode, "dijitSplitter" + (this.horizontal ? "H" : "V") + "Active");
		if(this.fake){
			dojo.removeClass(this.fake, "dijitSplitterHover");
			dojo.removeClass(this.fake, "dijitSplitter" + (this.horizontal ? "H" : "V") + "Hover");
		}

		//Performance: load data info local vars for onmousevent function closure
		var factor = this._factor,
			max = this._computeMaxSize(),
			min = this.child.minSize || 20,
			isHorizontal = this.horizontal,
			axis = isHorizontal ? "pageY" : "pageX",
			pageStart = e[axis],
			splitterStyle = this.domNode.style,
			dim = isHorizontal ? 'h' : 'w',
			childStart = dojo.marginBox(this.child.domNode)[dim],
			region = this.region,
			splitterStart = parseInt(this.domNode.style[region], 10),
			resize = this._resize,
			childNode = this.child.domNode,
			layoutFunc = dojo.hitch(this.container, this.container._layoutChildren),
			de = dojo.doc;

		this._handlers = (this._handlers || []).concat([
			dojo.connect(de, "onmousemove", this._drag = function(e, forceResize){
				var delta = e[axis] - pageStart,
					childSize = factor * delta + childStart,
					boundChildSize = Math.max(Math.min(childSize, max), min);

				if(resize || forceResize){
					layoutFunc(region, boundChildSize);
				}
				splitterStyle[region] = factor * delta + splitterStart + (boundChildSize - childSize) + "px";
			}),
			dojo.connect(de, "ondragstart", dojo.stopEvent),
			dojo.connect(dojo.body(), "onselectstart", dojo.stopEvent),
			dojo.connect(de, "onmouseup", this, "_stopDrag")
		]);
		dojo.stopEvent(e);
	},

	_onMouse: function(e){
		var o = (e.type == "mouseover" || e.type == "mouseenter");
		dojo.toggleClass(this.domNode, "dijitSplitterHover", o);
		dojo.toggleClass(this.domNode, "dijitSplitter" + (this.horizontal ? "H" : "V") + "Hover", o);
	},

	_stopDrag: function(e){
		try{
			if(this.cover){
				dojo.removeClass(this.cover, "dijitSplitterCoverActive");
			}
			if(this.fake){ dojo.destroy(this.fake); }
			dojo.removeClass(this.domNode, "dijitSplitterActive");
			dojo.removeClass(this.domNode, "dijitSplitter" + (this.horizontal ? "H" : "V") + "Active");
			dojo.removeClass(this.domNode, "dijitSplitterShadow");
			this._drag(e); //TODO: redundant with onmousemove?
			this._drag(e, true);
		}finally{
			this._cleanupHandlers();
			delete this._drag;
		}

		if(this.container.persist){
			dojo.cookie(this._cookieName, this.child.domNode.style[this.horizontal ? "height" : "width"], {expires:365});
		}
	},

	_cleanupHandlers: function(){
		dojo.forEach(this._handlers, dojo.disconnect);
		delete this._handlers;
	},

	_onKeyPress: function(/*Event*/ e){
		// should we apply typematic to this?
		this._resize = true;
		var horizontal = this.horizontal;
		var tick = 1;
		var dk = dojo.keys;
		switch(e.charOrCode){
			case horizontal ? dk.UP_ARROW : dk.LEFT_ARROW:
				tick *= -1;
//				break;
			case horizontal ? dk.DOWN_ARROW : dk.RIGHT_ARROW:
				break;
			default:
//				this.inherited(arguments);
				return;
		}
		var childSize = dojo.marginBox(this.child.domNode)[ horizontal ? 'h' : 'w' ] + this._factor * tick;
		this.container._layoutChildren(this.region, Math.max(Math.min(childSize, this._computeMaxSize()), this.child.minSize));
		dojo.stopEvent(e);
	},

	destroy: function(){
		this._cleanupHandlers();
		delete this.child;
		delete this.container;
		delete this.cover;
		delete this.fake;
		this.inherited(arguments);
	}
});

dojo.declare("dijit.layout._Gutter", [dijit._Widget, dijit._Templated ],
{
	// summary:
	// 		Just a spacer div to separate side pane from center pane.
	//		Basically a trick to lookup the gutter/splitter width from the theme.
	// description:
	//		Instantiated by `dijit.layout.BorderContainer`.  Users should not
	//		create directly.
	// tags:
	//		private

	templateString: '<div class="dijitGutter" waiRole="presentation"></div>',

	postCreate: function(){
		this.horizontal = /top|bottom/.test(this.region);
		dojo.addClass(this.domNode, "dijitGutter" + (this.horizontal ? "H" : "V"));
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.ExpandableContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.ExpandableContainer"] = true;

dojo.provide("ibm.tivoli.tip.dijit.ExpandableContainer");





dojo.declare("ibm.tivoli.tip.dijit.ExpandableContainer",[dijit._Widget,dijit._Templated],{
	scope:(dojo._scopeName||"dojo"),
	templateString:"<div dojoAttachPoint=\"rootpane\"  gutters=\"false\" >\r\n \t  <div class=\"\"  region=\"top\" >\r\n\t       <div dojoAttachPoint=\"toggleIcon\"></div>\r\n\t\t \t<h1 class=\"section h1\" dojoAttachPoint=\"titleNode\"></h1>\r\n\t\t\t\r\n\t  </div> \r\n\t \r\n\t  <div dojoAttachPoint=\"contentPane\"  region=\"center\">\r\n\t\t<div dojoAttachPoint=\"containerNode\"></div>\r\n\t\t\r\n\t  </div>\r\n</div>\r\n",
	widgetsInTemplate:true,
	expanded:true,
	title:"",
	postCreate:function(){
		this.inherited(arguments);
		this.titleNode.setAttribute("tabindex","0");  
		this.titleNode.setAttribute("role","region");  
		this.titleNode.setAttribute("aria-label","Expandable/Collapsable section");  
		this.containerNode.setAttribute("tabindex","0"); 
		this.connect(this.toggleIcon,"onclick",this._toggleContent);
		this.connect(this.titleNode,"onclick",this._toggleContent);
		this.connect(this.titleNode,"onkeydown",this._toggleContentKeyboard);
		this.setTitle(this.title);
		if(this.expanded==false){
		 this._collapse();
		 this.titleNode.className="section iconclose";
		 }
		 else {
		   this.titleNode.className="section iconopen";
		 }
		 this.titleNode.setAttribute("aria-expanded",this.expanded); 
		 this.rootpane = new dijit.layout.BorderContainer();
		 this.contentPane = new dijit.layout.BorderContainer();
	 },	 
	 addWidget: function(widget) {
	   this.rootpane.addChild(widget);
	 },
	 setTitle:function(_1){
	 this.titleNode.innerHTML=this.title=_1;
	 },
	 getTitle:function(){
	 return this.title;
	 },
	 expand:function(){
	    console.info("expand");
		 if(this.expanded==false){
		 this._expand();
		}
	 },
	 _expand:function(){
		 this.expanded=true;
		this.titleNode.className="section iconopen";
		 dojo.fx.wipeIn({
		 node:this.containerNode,duration:500}).play();
	 },
	 collapse:function(){
	      console.info("collapse");
		 if(this.expanded==true){
		 this._collpase();
		 }
	 },
	 _collapse:function(){
		 this.expanded=false;
		 this.titleNode.className="section iconclose";
		 dojo.fx.wipeOut({node:this.containerNode,duration:500}).play();
	 },
	 _toggleContent:function(){
		this.expanded?this._collapse():this._expand();
		this.titleNode.setAttribute("aria-expanded",this.expanded);  
	 },
	 _toggleContentKeyboard:function(event){
		if(event.keyCode == 9 || event.keyCode == 16 || event.keyCode == 18){
			return;
		}
		this._toggleContent();
	 }
 });

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.ListBuilder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.ListBuilder"] = true;
/**
 * ibm.tivoli.tip.dijit.ListBuilder is a pair of ibm.tivoli.tip.dijit.DndListboxes, with Add and Remove buttons between them
 * 
 * Creating a ListBuilder in markup:
 * <div dojoType="ibm.tivoli.tip.dijit.ListBuilder" showAllButtons="true" filtered="true" sorted="false" accept="role" 
 * 		type="role,text" name="myroles" leftLabel="Available roles" rightLabel="Selected roles"></div>
 * 
 * Attributes:
 * shoAllButtons: if true, show the "Add All" and "Remove All" buttons
 * sorted: if true, the right-hand list is kept sorted. the left-hand list is always sorted
 * filtered: if true, the lists include the filter textbox
 * accept: array of types that can be dropped on the 2 DndListboxes
 * type: array of types describing the items in the 2 DndListboxes
 * leftLabel: text displayed above the left-hand list
 * rightLabel: text displayed above the right-hand list
 * 
 * Events:
 * 	fires OnChange when the value of the list changes. (TODO: implement priorityChange to only fire when list looses focus)
 * 
 */
dojo.provide("ibm.tivoli.tip.dijit.ListBuilder");






dojo.declare("ibm.tivoli.tip.dijit.ListBuilder", [dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true,
	templateString: dojo.cache("ibm.tivoli.tip.dijit", "templates/ListBuilder.html", "<div class=\"tip-ListBuilder\">\r\n  <div dojoAttachPoint=\"left\" class=\"balLayoutCell\" dojoType=\"ibm.tivoli.tip.dijit.DndListbox\" \r\n  \t\tfiltered=\"${filtered}\" sorted=\"true\" accept=\"${accept}\" type=\"${type}\" label=\"${leftLabel}\"></div>\r\n  <div class=\"balLayoutCell\">\r\n    <div class=\"balBtnContainer\">\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"_balAddBtn\" dojoAttacheEvent=\"onclick: add\" >${addLabel} &gt;</button>\r\n      <br/>\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"_balAddAllBtn\" dojoAttacheEvent=\"onclick: addAll\">${addAllLabel} &gt;</button>\r\n      <br/>\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"_balRemBtn\" dojoAttacheEvent=\"onclick: remove\">&lt; ${removeLabel}</button>\r\n      <br/>\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"_balRemAllBtn\" dojoAttacheEvent=\"onclick: removeAll\" >&lt; ${removeAllLabel}</button>\r\n    </div>\r\n  </div>\r\n  <div dojoAttachPoint=\"right\" class=\"balLayoutCell\" dojoType=\"ibm.tivoli.tip.dijit.DndListbox\"\r\n\t\tfiltered=\"${filtered}\" sorted=\"${sorted}\" accept=\"${accept}\" type=\"${type}\" label=\"${rightLabel}\" name=\"${name}\"></div>\r\n  \r\n  <div class=\"balLayoutCell\" dojoAttachPoint=\"_reorderButtonContainer\">\r\n    <div class=\"balBtnContainer\">\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"moveUpBtn\" dojoAttachEvent=\"onClick: moveUp\" >${_resources.MOVE_UP_LABEL}</button>\r\n      <br/>      \r\n      <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"moveDownBtn\" dojoAttachEvent=\"onClick: moveDown\" >${_resources.MOVE_DOWN_LABEL}</button>\r\n    </div>\r\n  </div>\r\n  \r\n  <div style=\"clear:both;height:0;width:0\"></div>\r\n</div>\r\n"),
	showAllButtons: true,	// do we show the Add All and Remove All buttons
	filtered: true,			// do we show the filter input text boxes
	sorted: false,			// is the right list kept sorted? (the left list is always sorted)
	accept: ["text"],		// array of types that can be dropped here
	type: ["text"],			// array of types objects in this box represent
	name: "",				// give the widget a name if you want its value submitted with a form
	leftLabel: "",			// label for the left list
	rightLabel: "",			// label for the right list
	required: false,		// is a value required for this list?
	// nls strings
	addLabel: "",
	removeLabel: "",
	addAllLabel: "",
	removeAllLabel: "",
	acceptBuffer:null,
	showReOrderButtons: false,	//if sorted, hide the "move up", "move down" buttons rather than just disable
	disabled:false,

	postMixInProperties: function()
	{
		var string_table = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
		this._resources = string_table;
		this.addLabel = string_table.ADD;
		this.removeLabel = string_table.REMOVE;
		this.addAllLabel = string_table.ADDALL;
		this.removeAllLabel = string_table.REMOVEALL;
		this.inherited(arguments);
	},
	/**
	 * startup()
	 *
	 * Basic start-up rountine, connect event handlers and 
	 * add the first items to the list.
	 */
	startup: function() {
		if(!this.showAllButtons) {
			dojo.style(this._balAddAllBtn.domNode, "display", "none");
			dojo.style(this._balRemAllBtn.domNode, "display", "none");
		}

		// Connect click listeners to the buttons
		this.connect(this._balAddBtn.domNode, 'onclick', this.add);
		this.connect(this._balAddAllBtn.domNode, 'onclick', this.addAll);
		this.connect(this._balRemBtn.domNode, 'onclick', this.remove);
		this.connect(this._balRemAllBtn.domNode, 'onclick', this.removeAll);
		//dojo.subscribe("/dnd/drop", dojo.hitch(this, this.updateButtons));
		this.connect(this.domNode, 'onclick', this.updateButtons);
		
		this.connect(this.left, "OnChange", this.updateButtons);
		this.connect(this.right, "OnChange", this._onChange);
		
		this.updateButtons();
		this.inherited(arguments);
	},	
	destroy: function()
	{
		console.log("ListBuilder.destroy");
		this.inherited(arguments);
	},
	_getValueAttr: function()
	{
		var list = this.right.get("value");
		return list;
	},
	_setValueAttr: function(value)
	{
		var item;
		var items = [];
		var varr = dojo.isArray(value) ? value : [value];
		var from = this.left;
		var to = this.right;

		// move non-matching items from the right list back to the left
		to.getAllNodes().forEach(function(n){
			item = to.getItem(n.id);
			if(item && item.data && dojo.indexOf(varr, item.data.toString()) < 0) {
				items.push(item.data);
				to.delItem(n.id);
			}
		});
		from.insertNodes(items);
		
		// move matching items from the left list to the right
		items = [];
		from.getAllNodes().forEach(function(n){
			item = from.getItem(n.id);
			if(item && item.data && dojo.indexOf(varr, item.data.toString()) >= 0) {
				items.push(item.data);
				from.delItem(n.id);
			}
		});
		this.right.insertNodes(items);

		this.updateButtons();
	},
	
	/**
	* This function provides the ability to enable/disable a ListBuilder.  By default the ListBuilder is enabled.
	* Once user call to disable a listBuilder, all the button needs to be disabled, and Drag n Drop also should
	* be disabled.
	* This function will call DnDListBox.setDisabled(value) which in turn will disable drag n drop.
	*/
	_setDisabledAttr: function(/*Boolean*/ value){
        var i;
		console.log("ListBuilder.js : _setDisabledAttr");
		this.right.setDisabled(value); //target needs to be disabled for DnD
		this.left.setDisabled(value); // Source needs to be disabled for DnD
		if(value){
		this._balAddBtn.set('disabled', value);
		this._balRemBtn.set('disabled', value);
		
		}
		this._balRemAllBtn.set('disabled', value);
		this._balAddAllBtn.set('disabled', value);
		this.disabled=value;
	},
	
	/**
	 * addData()
	 * 
	 * Add data to the available list
	 * 
	 * Input:
	 * 	data: the data being added to the available list
	 * 
	 */
	addData: function(data)
	{
		this.left.addData(data);
		this.updateButtons();
	},
	
	addDataToRight: function(data){
		this.right.addData(data);
		this.updateButtons();
	},
	
	moveUp: function(e){
		var nodes = this.right.getSelectedNodes();
		var anchor = nodes[0].previousSibling;
		var items = [];
		for(var i=0;i<nodes.length;i++){
			items.push(this.right.getItem(nodes[i].id).data);
		}		
		this.right.deleteSelectedNodes();
		this.right._source.insertNodes(true, items, true, anchor);	
		this.updateButtons();
	},
	
	moveDown: function(e){
		var nodes = this.right.getSelectedNodes();	
		var anchor = nodes[nodes.length-1].nextSibling;
		var items = [];
		for(var i=0;i<nodes.length;i++){
			items.push(this.right.getItem(nodes[i].id).data);
		}		
		this.right.deleteSelectedNodes();
		this.right._source.insertNodes(true, items, false, anchor);	
		this.updateButtons();
	},
	
	/**
	 * addAll()
	 *
	 * Add All button handler.  Moves all items from the available list to 
	 * the reserve list.
	 * 
	 * Inputs:
	 *   e- DOM event object from Dojo (assumed fixed)
	 * Returns:
	 *   None.
	 */
	addAll: function(e) {
		if(!this.disabled){
			this.left.selectAll();
			this._move(this.left, this.right);
			this.updateButtons();
		}
	},

	/**
	 * add()
	 *
	 * Add button handler.  Moves selected items from the left list to 
	 * the right list.
	 * 
	 * Inputs:
	 *   e- DOM event object from Dojo (assumed fixed)
	 * Returns:
	 *   None.
	 */
	add: function(e) {
	if(!this.disabled){
		this._move(this.left, this.right);
		}
	},

	/**
	 * removeAll()
	 *
	 * Remove All button handler.  Moves all items reserve from the list to 
	 * the available list.
	 * 
	 * Inputs:
	 *   e- DOM event object from Dojo (assumed fixed)
	 * Returns:
	 *   None.
	 */
	removeAll: function(e) {
	if(!this.disabled){
		 this.right.selectAll();
		 this._move(this.right, this.left);
		 this.updateButtons();
		 }
	},
	/**
	 * remove()
	 *
	 * Remove button handler.  Moves selected items from the right list
	 * to the left list.
	 * 
	 * Inputs:
	 *   e- DOM event object from Dojo (assumed fixed)
	 * Returns:
	 *   None.
	 */
	remove: function(e) {
	if(!this.disabled){
		this._move(this.right, this.left);
		}
	},
	_move: function(from, to)
	{
		var nodes = from.getSelectedNodes();
		// for some reason dojo.dnd can do it, but I can't insert nodes, 
		// but only items.  This also means I loose the items' type in 
		// the process
		var items = [];
		var item;
		nodes.forEach(function(n) {
			item = from.getItem(n.id);
			if(item && item.data)
			items.push(item.data);
		});
		to.insertNodes(items);
		from.deleteSelected();
		this.updateButtons();
	},
	/**
	 * updatesButtons()
	 *
	 * Updates all buttons disabled state based on the content
	 * of the list.  It disables button when the list is empty 
	 * and enable button when their is something to move.
	 * 
	 * Inputs:
	 *   None.
	 * Returns:
	 *   None.
	 */
	updateButtons: function() {
		console.debug(this.id + ".updateButtons");

		if(!this.disabled)
		{
		// If any nodes are selected, enable the add button
		var b = this.left.getSelectedNodes().length < 1;
		this._balAddBtn.set('disabled', b);

		// If the available area contains any nodes, enable the add all button
		b = this.left.get("value").length < 1;
		this._balAddAllBtn.set('disabled', b);

		// If the reserve area contains any seelected items, enable the remove button
		b = this.right.getSelectedNodes().length < 1;
		this._balRemBtn.set('disabled', b);

		// If the available area contains any nodes, enable the remove all button
		b = this.right.get("value").length < 1;
		this._balRemAllBtn.set('disabled', b);
		
		var nodeCount = this.right.getSelectedNodes().length;		
		
		this.moveUpBtn.set('disabled',  nodeCount<1 || !this.right.getSelectedNodes()[0].previousSibling);
		this.moveDownBtn.set('disabled', nodeCount<1 || !this.right.getSelectedNodes()[nodeCount-1].nextSibling);
		
		dojo.style(this._reorderButtonContainer, "display", (this.sorted && !this.showReorderButtons) ? "none" : "");
		}
	},
	isValid: function()
	{
		return !this.required || this.get("value").length > 0;
	},
	_onChange: function()
	{
		this.updateButtons();
		this.OnChange();
	},
	OnChange: function()
	{
		// override or connect to me
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.NumberInputBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.NumberInputBox"] = true;
dojo.provide("ibm.tivoli.tip.dijit.NumberInputBox"); 



/** 
 * A number input box widget.
 * 
 * Such widget displays a text input box which accepts only a valid 
 * number.
 * The widget accepts the following parameters:
 * 
 *  - value: the initial value the widget must show when created.
 *    An empty string means NaN.
 *  - constraints: an object containing the validation constraints.
 *    The same constraints can be provided as single properties.
 *    A constraint provided as a single property overrides the 
 *    corresponding constraint provided in the constraints object.
 *    The possible constraints are:
 *      
 *      - min: (Number) the minimum value.
 *      - max: (Number) the maximum number.   
 *      - required: (Boolean) if true the value cannot be empty (NaN).
 * 
 *  -  min: (String) the same of constraints.min.
 *  -  max: (String) the same of constraints.max.
 *  -  required: (String) the same of constraints.required.
 * 
 * Example of a declaration:
 * 
 *       <div 
 *          dojoType="ibm.tivoli.tip.dijit.NumberInputBox" 
 *          constraints="{required: true, min:5}"
 *          max="10"/>
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)    
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.NumberInputBox",
    [ibm.tivoli.tip.dijit.TextInputBox],
    {
        /** the initial value (an empty string means "not a number")*/
        value: "",
        /** the minimum number */
        min: "",
        /** the maximum number */
        max: "",
        /** the constraints */
        constraints: {},
        
        /**
         * Sets the initial value: this triggers the first validation.
         * The initial value can be externally provided by setting the 
         * value property.  
         */
        startup: function() 
        {
            // call the superclass's method
            this.inherited("startup", arguments);

            // set the regular expression (only number accepted)  
            this.constraints.regexp = /^[\-]?[0-9]*$/g;
            // set the regex message for the text
            this.constraints.regexpMessage = this._resources.NUMBER_INVALID_VALUE;
            
            // get parameters
            var decimalNumber = /^[\-]?[0-9]*[,.]?[0-9]+$/g;
            // get the min
            if ( this.min.match(decimalNumber) ) {
                this.constraints.min = parseInt(this.min, 10);
            }
            // get the max
            if ( this.max.match(decimalNumber) ) {
                this.constraints.max = parseInt(this.max, 10);
            }
            // set the validator function
            this.constraints.validator = dojo.hitch(this, this._validator);

            // set the value
            var n;
            if ( this.value.match(decimalNumber) ) {
                // the passed value is a number
                n = parseInt(this.value, 10);
            }
            else {
                n = NaN;
            }
            this.setValue(n);
        },

        /**        
         * Returns the value (a number)
         */
        getValue: function()
        {
            // get the text value
            var value = this.inherited("getValue", arguments);
            // convert to a numeric value
            var n = NaN;
            if ( value.match(this.constraints.regexp) ) {
                // the value is a number
                n = parseInt(value, 10);
            }
            return n;
        },
        
        /**        
         * Sets the value.
         * 
         *  @param value:
         *      The value (a number) to be set.
         */
        setValue: function(value)
        {
            if ( isNaN(value) ) {
                value = "";
            }
            else {
                value = "" + value;
            }
            this.inherited("setValue", arguments);
        },
        
        /**
         * Returns true if the widget's value is not set.
         */
        isEmpty: function()
        {
            return isNaN(this.getValue());
        },
        
        /**        
         * Returns the text representation of the value (a String)
         */
        getTextValue: function()
        {
            // return the text value
            return ibm.tivoli.tip.dijit.NumberInputBox.superclass.getValue.call(this);
        },
        
        /**
         * The range validator.
         */
        _validator: function()
        {
            // get the value
            var n = this.getValue();

            // check the max constraint
            if ( this.constraints.max !== undefined && n > this.constraints.max ) {
                // the value is greater than max 
                return { state: this.constants.ERROR, message: dojo.string.substitute(this._resources.NUMBER_MAX_LIMIT, {max: this.constraints.max}) };    
            }
            // check the min constraint
            if ( this.constraints.min !== undefined && n < this.constraints.min ) {
                // the value is lower than min
                return { state: this.constants.ERROR, message: dojo.string.substitute(this._resources.NUMBER_MIN_LIMIT, {min: this.constraints.min}) };    
            }
            // - and -0 not allowed
            var s = this.getTextValue();
            if ( s && ( s === "-" || s.indexOf("-0") === 0 ) ) {
                return { state: this.constants.ERROR, message: this._resources.NUMBER_INVALID_VALUE };
            }
            
            // return no status, to permit the validation to go ahead
            return null;
        }
    }
);


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.RadioButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.RadioButton"] = true;
dojo.provide("ibm.tivoli.tip.dijit.RadioButton"); 



/** 
 * A Radio Button widget.
 * 
 * Such widget displays a radio button. 
 * Multiple radio buttons are grouped by sharing the same name. 
 * The widget calls the onValueChanged callback when the checked 
 * state changes even if the widget still has the focus. 
 *   
 * The widget accepts the following parameters:
 * 
 *  - value: the initial value for the radio button. 
 *      - "true": the radio button must be checked
 *      - "false": the radio button must not be checked
 *  - checked: same as the HTML input attribute.
 *      - "", "checked", "true": the radio button must be checked
 *      - any other value: the radio button must not be checked
 * 
 * Example of a declaration:
 * 
 *       <div 
 *          dojoType="ibm.tivoli.tip.dijit.RadioButton" 
 *          name="radioGroup"
 *          checked="checked"/>
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)    
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.RadioButton",
    [ibm.tivoli.tip.dijit.HtmlSelection],
    {
        /** HTML input type */
        _type: "radio" 
    }
);


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.SelectionList"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.SelectionList"] = true;
dojo.provide("ibm.tivoli.tip.dijit.SelectionList");




/**
 * The selection list widget.
 * 
 * Such widget implements a selectable list. Caller provides
 * a list of items which could be individually selected
 * and the widget manages the selection functionalities. 
 * 
 * The caller provides the items as a set of object properties.
 * The property's names are the id for the values and the property's
 * values are the list's values. The ids will be used to identify
 * the list's items to work with. Values can be plain text, HTML or
 * widget declarations (in such case hasWidgets must be true).
 * 
 * The widget exposes the following configuration attributes:
 * 
 *      - items: (Object) the items to show in the selection.
 *      - tabindex: (Number) the tabindex to assign to the selection 
 *        boxes.
 *      - multipleSelection: (Boolean) true if a multiple selection 
 *        is allowed, false if a single selection is allowed.
 *      - hasWidgets: (Boolean) true if the list's items contain
 *        Dojo declarations. 
 *      - colorizeSelection: (Boolean) true if items must be colorized
 *        when selected.
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)    
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.SelectionList",
    [ibm.tivoli.tip.dijit.Widget, dijit._Templated],
    {
        /** the user's provided items */
        items: null,
        /** the user's provided tabindex (needed to navigate with keyboard) */
        tabindex: 0,
        /** if true allows to choose multiple elements in the list */
        multipleSelection: true,
        /** true if at least one item contains a Dojo widget */
        hasWidgets: false,
        /** true if selected items must be colorized */
        colorizeSelection: true,
        
        /** the widget template path */
        templateString:"<div class=\"selectionList\" \r\n     tabindex=\"0\"\r\n    ><div class=\"selectionListIcons\"></div\r\n    ><div \r\n        dojoAttachPoint=\"_bodyNode\"\r\n        dojoAttachEvent=\"onclick:_onClick,onkeyup:_onKeyUp\"\r\n    ></div\r\n></div>\r\n",
        
        /** the list node */
        _bodyNode: null,
        /** the table node */
        _tableNode: null,
        /** the items */
        _items: null,
        /** the input node list */
        _inputNodes: null, 
        /** the table cell node list */
        _cellNodes: null, 
        /** the _rowTemplate template */
        _rowTemplate:
            "<tr itemId='${itemId}'>" +
                "<td name='${cellInputName}' width='16px' class='selectionListTableCell'>" +
                    "<input type='${inputType}' name='${inputName}' itemId='${itemId}' tabindex='${tabindex}'/>" +
                "</td>" +                     
                "<td name='${cellName}' itemId='${itemId}' class='selectionListTableCell'>" +
                    "${item}" +
                "</td>" +                
            "</tr>",
        /** the list of key codes to be intercepted */    
        _keys: null,
        
        /**
         * Prepares the selection list. 
         * If an array of items has been specified the list is populated.  
         */
        startup: function() 
        {
            // call the superclass's method
            this.inherited("startup", arguments);
            
            /** prepare the code array of keys to be managed (should be a static property ) */            
            this._keys = [];
            this._keys[dojo.keys.SPACE] = true;
            this._keys[dojo.keys.LEFT_ARROW] = true;
            this._keys[dojo.keys.UP_ARROW] = true;
            this._keys[dojo.keys.RIGHT_ARROW] = true;
            this._keys[dojo.keys.DOWN_ARROW] = true;

            // set-up some useful constants
            this._INPUT_NAME = this.id + "#input";
            this._CELL_INPUT_NAME = this.id + "#cellInput";
            this._CELL_NAME = this.id + "#item";
               
            // valorize the item template with widget constants
            var p = {
                inputName: this._INPUT_NAME,
                cellInputName: this._CELL_INPUT_NAME,
                cellName: this._CELL_NAME,
                tabindex: this.tabindex,
                itemId: "${itemId}",
                item: "${item}",
                inputType: (this.multipleSelection) ? "checkbox" : "radio"
            };
            this._rowTemplate = dojo.string.substitute(this._rowTemplate, p);
            
            // populate the list from the item parameter
            this.setValue(this.items || []);
        },

        /**        
         * Returns an array containing the name of the selected 
         * values.
         */
        getValue: function()
        {
            var result = [];
            dojo.forEach( 
                this._inputNodes,
                function(node){
                    if ( node.checked ) { 
                       result[result.length] = node.getAttribute("itemId");
                    }
                });

            return result;
        },
        
        /**        
         * Sets the given items into the list.
         * 
         *  @param items
         *      A maps whose values are the elements shown in the selection
         *      and the names are used to identify the selected elements.
         */
        setValue: function(items)
        {
            // destroy enclosing widgets
            if ( this.hasWidgets ) {
                dojo.query("[widgetId]", this._bodyNode).forEach(
                    function(d) {
                        dijit.byId(d.getAttribute("widgetId")).destroy();
                    }
                );
            }
                        
            // store the items
            this._items = items;
            
            // populate the list
            var s = [];
            s[s.length] = "<table class='selectionListTable' tabindex='-1'>";
            s[s.length] = "<tbody>";
            for ( var i in items ) {
                // prepare the tamplate parameters
                var p = {
                    itemId: i,
                    item: items[i]
                };
                // valorize the template
                s[s.length] = dojo.string.substitute(this._rowTemplate, p);
            }
            s[s.length] = "</tbody>";
            s[s.length] = "</table>";
            this._bodyNode.innerHTML = s.join("");
            
            // set the input node list
            this._inputNodes = dojo.query("input", this._bodyNode);
            // set the cell node list
            this._cellNodes = dojo.query("[name='" + this._CELL_NAME + "']", this._bodyNode);
            // set the table node
            this._tableNode = dojo.query("tbody", this._bodyNode)[0];
            
            // parse the mark-up if has Dojo widgets 
            if ( this.hasWidgets ) {
                dojo.parser.parse(this._bodyNode);
            }
        },
        
        /**
         * Returns true if no selection has been done.
         */
        isEmpty: function()
        {
            return ( this.getValue().length === 0 );
        },
        
        /**
         * Returns the item corresponding to the given name.
         * 
         *  @param itemId:
         *      The id of the item to retrieve.
         *  @return 
         *      null if the item has not been found, or a 
         *      NodeList object containing all the child nodes.         
         */
        getItem: function(itemId)
        {
            var node = this._getNodeById(this._cellNodes, itemId);
            return ( node ? node.childNodes : null );
        },
        
        /**
         * Sets the item corresponding to the given name. If the name
         * is not found the item is not set.
         * 
         *  @param itemId:
         *      The id of the item to retrieve.
         *  @param item:
         *      The item to be set. It could be also a mark-up or a 
         *      Dojo declaration (hasWidgets must be true in order to get
         *      the declaration parsed).         
         */
        setItem: function(itemId, item)
        {
            var node = this._getNodeById(this._cellNodes, itemId);
            if ( node ) {
                // destroy widgets
                if ( this.hasWidgets ) {
                    dojo.query("[widgetId]", node).forEach(
                        function(d) {
                            dijit.byId(d.getAttribute("widgetId")).destroy();
                        }
                    );
                }
                // set the item   
                node.innerHTML = item;
                // parse the mark-up if has Dojo widgets 
                if ( this.hasWidgets ) {
                    dojo.parser.parse(node);
                }
            }
        },
        
        /**
         * Removes the item corresponding to the given name. If the name
         * is not found does nothing.
         * 
         *  @param itemId:
         *      The id of the item to remove.
         */
        removeItem: function(itemId)
        {
            // lookup the row with the given id
            var nodes = dojo.query("tr[itemId=" + itemId + "]", this._bodyNode);
            if ( nodes && nodes.length > 0 ) {
                var tr = nodes[0];
                // destroy widgets
                if ( this.hasWidgets ) {
                    dojo.query("[widgetId]", tr).forEach(
                        function(d) {
                            dijit.byId(d.getAttribute("widgetId")).destroy();
                        }
                    );
                }
                // delete references
                this._removeNodeById(this._inputNodes, itemId);
                this._removeNodeById(this._cellNodes, itemId);
                // delete the row    
                this._tableNode.deleteRow(tr.rowIndex);
            }
        },
        
        /**
         * Appends the item corresponding to the given name. If the name
         * is found the item is not appended.
         * 
         *  @param itemId:
         *      The id of the item to append.
         *  @param item:
         *      The item to be append. It could be also a mark-up or a 
         *      Dojo declaration (hasWidgets must be true in order to get
         *      the declaration parsed).         
         */
        appendItem: function(itemId, item)
        {
            var node = this._getNodeById(this._cellNodes, itemId);
            if ( ! node ) {
                // populate the list
                var e = document.createElement("div");
                e.innerHTML = [
                    "<table><tbody>",
                    dojo.string.substitute(this._rowTemplate, {itemId: itemId, item: item}),
                    "</tbody></table>"].join("");
                
                // append the row to the table
                var rowNode = dojo.query("tr", e)[0];
                this._tableNode.appendChild(rowNode);

                // parse the mark-up if has Dojo widgets
                if ( this.hasWidgets ) {
                    dojo.parser.parse(rowNode);
                }
                
                // set the input node list
                if ( ! this._inputNodes ) {
                    this._inputNodes = new dojo.NodeList();
                }
                this._appendNode(this._inputNodes, dojo.query("input", rowNode)[0]);
                // set the cell node list
                if ( ! this._cellNodes ) {
                    this._cellNodes = new dojo.NodeList();
                }
                this._appendNode(this._cellNodes, dojo.query("[name='" + this._CELL_NAME + "']", rowNode)[0]);
            }
        },
        
        /**
         * Removes all the item currently displayed in the list.
         */
        clear: function()
        {
            this._inputNodes = null;
            this._cellNodes = null;
            // delete all the row in the table
            var rows = this._tableNode.rows;
            for ( var i = rows.length - 1; i >= 0 ; i-- ) {
                this._tableNode.deleteRow(i);
            }
        },
        
        /**
         * Returns the number of elements in the selection list.
         */
        size: function()
        {
            var obj = { counter: 0 };
            if ( this._inputNodes ) {
                this._inputNodes.forEach(
                    function() {
                        this.counter++;
                    },
                    obj
                );
            } 
            return obj.counter;   
        },
        
        /**
         * Selects the pointed item.
         * 
         *  @param itemId 
         *      the id of the item to be selected.
         */
        select: function(itemId)
        {
            this._select({itemId: itemId, selected: true});
        },
        
        /**
         * Unselects the pointed item.
         * 
         *  @param itemId 
         *      the id of the item to be selected.
         */
        unselect: function(itemId)
        {
            this._select({itemId: itemId, selected: false});
        },
        
        /**
         * Selects all the items.
         */
        selectAll: function()
        {
            this._select({all: true, selected: true});
        },
        
        /**
         * Unselects all the items.
         */
        unselectAll: function()
        {
            this._select({all: true, selected: false});
        },
        
        /**
         * A generic select method.
         * 
         *  @param params
         *      An object containing the selection parameters.
         *      Parameters are:
         *          - all [boolean]: if true the selection must be done
         *            on all the items
         *          - selected [boolean]: if true the item(s) must be selected
         *          - item [string]: the id of the item to be selected,
         *            only considered if the all parameter is false.
         */
        _select: function(params)
        {
		
		  //console.info("select SelectionList");
            // no nodes in place, skip
            if ( ! this._inputNodes ) {
                return;
            }
        
            // select all on single selection unsupported
            if ( params.all && params.selected && ! this.multipleSelection ) {
                return;
            }
            
            if ( params.all ) {
                // the operation must be done on all the items
                
                // select all the input boxes
                this._inputNodes.forEach( 
                    function(node) {
                        node.checked = params.selected;
                    });
                // select all the cells
                if ( this.colorizeSelection ) {
                    this._cellNodes.forEach( 
                        function(node) {
                            dojo.toggleClass(node, "selectionListTableCellSelected", params.selected);
                        });
                }
            }
            else {
                // the operation must be done on a specific item

                // select the input box
                var inputNode = this._getNodeById(this._inputNodes, params.itemId);
                if ( inputNode ) {
                    inputNode.checked = params.selected;
                }
                
                // select the cell
                if ( this.colorizeSelection ) {
                    var nodes = this._cellNodes;
                    // unselect the previous cell in case of single selection
                    if ( ! this.multipleSelection ) {
                        for ( var i = 0; i < nodes.length; i++ ) {
                            dojo.removeClass(nodes[i], "selectionListTableCellSelected");
                        }
                    }
                    // select the current cell
                    var cellNode = this._getNodeById(nodes, params.itemId);
                    if ( cellNode ) {
                        dojo.toggleClass(cellNode, "selectionListTableCellSelected", params.selected);
                    }
                }
            }
        },
        
        /**
         * Returns the node which has the given id.
         * 
         *  @param nodes
         *      the nodes (NodeList) to be searched in.
         *  @param itemId
         *      the id to be searched.   
         */
        _getNodeById: function(nodes, itemId)
        {
            if ( nodes ) {
            itemId = "" + itemId;
                var name;
				var len=nodes.length;
				//Modified By Prashant Sirohi On 4 Sept 2009
                for (var i=0;i<len;i++ ) {
                    var node = nodes[i];
					
                    if ( ! dojo.isFunction(node) && node.getAttribute("itemId") === itemId ) {
                        // the node is not a function and the id has been matched
                        return node;
                    }
                }
            }
            // node not found
            return null;
        },
           
        /**
         * Removes the node which has the given id.
         * 
         *  @param nodes
         *      the nodes (NodeList) to be removed from.
         *  @param itemId
         *      the id to be removed.   
         */
        _removeNodeById: function(nodes, itemId)
        {
            itemId = "" + itemId;
            var name;
            for ( name in nodes ) {
                var node = nodes[name];
                if ( ! dojo.isFunction(node) && node.getAttribute("itemId") === itemId ) {
                    nodes.splice(nodes.indexOf(node), 1);
                    break;
                }
            }
        },
            
        /**
         * Appends the node.
         * 
         *  @param nodes
         *      the nodes (NodeList) to be appended to.
         *  @param node
         *      the id to be appended.   
         */
        _appendNode: function(nodes, node)
        {
            nodes.splice(nodes.length, 0, node);
        },
           
        /**
         * Handles the onkeyup event.
         * If a space has been pressed on a checkbox, the caller 
         * will be notified.
         */
        _onKeyUp: function(event)
        {
            if ( this._keys[event.keyCode] ) {
                this._onClick(event);       
            }
        },
        
        /**
         * Handles the onclick event.
         * If the selection box has been clicked, the node is 
         * selected/unselected accordingly.
         */
        _onClick: function(event)
        {
            var t = event.target;

            // check if the click comes from a checkbox
            var name = t.getAttribute("name");
            if ( name === this._INPUT_NAME || name === this._CELL_INPUT_NAME ) {
                // get the selection control
                var selection;
                if ( name === this._CELL_INPUT_NAME ) {
                    // this a cell 
                    selection = dojo.query("input", t)[0];
                    selection.checked = ! selection.checked; 
                }
                else {
                    // this is already the selection control
                    selection = t;
                }
                // select the item
                this._select({itemId: selection.getAttribute("itemId"), selected: selection.checked});
                // notify the change
                this.onValueChanged(this.getValue());
                // and that's it
                return;
            }
        }
    }
);

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.SpinnerInputBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.SpinnerInputBox"] = true;
dojo.provide("ibm.tivoli.tip.dijit.SpinnerInputBox"); 




/** 
 * A number input box widget with spinner arrows.
 * 
 * Such widget shows a number input box with two spinner buttons
 * on the right. The buttons can be used to increment (up-arrow)
 * or decrement (down-arrow) of a delta the current set value.
 * 
 * The widget accepts the following parameters:
 * 
 *  - value: the initial value the widget must show when created.
 *  - required: (String) if true the value cannot be empty (NaN).
 *  - min: (String) the minimum value.
 *  - max: (String) the maximum value.
 *  - smallDelta: (String) the delta used to increment or decrement
 *    the value when the corresponding arrow is pressed. 
 * 
 * Example of a declaration:
 * 
 *      <div  
 *          dojoType="ibm.tivoli.tip.dijit.SpinnerInputBox"
 *          required="true"
 *          smallDelta="5"
 *          value="10"
 *          min="5"
 *          max="100"/>
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)    
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.SpinnerInputBox",
    [ibm.tivoli.tip.dijit.Widget, dijit._Templated],
    {
        /** the initial value (an empty string means "not a number") */
        value: "",        
        /** required */
        required: "false",
        /** the tab index */
        tabindex: "0",
        /** the minimum number */
        min: "",
        /** the maximum number */
        max: "",        
        /** the message to show if the regular expression is not matched */
        textRegExpMessage: null,        
        /** adjust the value by this much when spinning using the arrow keys/buttons */
        smallDelta: "1",
        
        /** the widget contains other widget */        
        widgetsInTemplate: true,
        /** the widget template path */
         templateString:"<span\r\n    class=\"spinner\"\r\n    >\r\n\t<table>\r\n\t<tr> <td>\r\n\t<span \r\n        tabindex=\"${tabindex}\"\r\n        dojoType=\"ibm.tivoli.tip.dijit.NumberInputBox\"\r\n        dojoAttachPoint=\"_textNode,_focusNode\"\r\n        value=\"${value}\"\r\n        required=\"${required}\"\r\n        min=\"${min}\"\r\n        max=\"${max}\"\r\n        regExpMessage=\"${textRegExpMessage}\"\r\n    ></span>\r\n\t</td>\r\n\t<td valign=\"middle\">\r\n\t<table  >\r\n\t<tr><td >\r\n\t       <img \r\n                    tabindex=\"${tabindex}\"\r\n                    src=\"${_upArrowIcon}\"\r\n                    dojoAttachPoint=\"_upArrowNode\"\r\n                    dojoAttachEvent=\"onmousedown:_onPushUpArrow,onkeypress:_onPushUpArrow,onmouseup:_onPullArrow,onmouseout:_onPullArrow,onkeyup:_onPullArrow\"\r\n                    class=\"upArrowButton\"\r\n            /></td ></tr >\r\n\t\t<tr >\r\n\t\t<td >\r\n\t\t<img  tabindex=\"${tabindex}\"\r\n                    src=\"${_downArrowIcon}\"\r\n                    dojoAttachPoint=\"_downArrowNode\"\r\n                    dojoAttachEvent=\"onmousedown:_onPushDownArrow,onkeypress:_onPushDownArrow,onmouseup:_onPullArrow,onmouseout:_onPullArrow,onkeyup:_onPullArrow\"\r\n                    class=\"downArrowButton\"\r\n            />\r\n\t\t</td  >\r\n\t\t</tr></table>\r\n\t  </td> \r\n\t  </tr></table>\t\r\n\t\t</span>\r\n   \r\n",

        /** the text node */
        _textNode: null,
        /** the up-arrow button node */
        _upArrowNode: null,
        /** the down-arrow button node */
        _downArrowNode: null,
        /** the up-arrow icon */
        _upArrowIcon: ibm.tivoli.tip.dijit.Images.get()["UP_ARROW_ICON"],
        /** the down-arrow icon */
        _downArrowIcon: ibm.tivoli.tip.dijit.Images.get()["DOWN_ARROW_ICON"],
        
        /** the auto-click slowest delay (milliseconds) */
        _autoClickMaxDelay: 300,
        /** the auto-click fastest delay (milliseconds) */
        _autoClickMinDelay:  25,
        /** the auto-click delay adjustment: auto-click becomes faster with time if this value is > 0, use 0 to disable */
        _autoClickDelayAdjustment: 35,
        /** the auto-click current delay, only valid when auto-clicking */
        _autoClickDelay: 0,
        /** the auto-click current delta, or zero if no auto-clicking is in progress */
        _autoClickDelta: 0,
        /** the auto-click callback */
        _autoClickCallback: null,
        /** the auto-click callback handle (obtained via setTimeout) */
        _autoClickHandle: null,
        /** true if an arrow is currently pushed */
        _isArrowPushed: false,
        
        /**
         * Sets the initial value.  
         */
        startup: function() 
        {
            // call the superclass's method
            this.inherited("startup", arguments);
            // get integer form of the small delta
            this.smallDelta = parseInt(this.smallDelta, 10);
            // set the value
            this.setValue(this.value);
            // change the tooltip node
            this._textNode.setTooltipNode(this.domNode);
            // connect the on value changed event
            this._textNode.onValueChanged = dojo.hitch(this, this.onValueChanged);
            // set the auto-click callback
            this._autoClickCallback = dojo.hitch(this, this._autoClick);
            // set the regex message for the text
            this.textRegExpMessage = this._resources.SPINNER_INVALID_VALUE;
        }, 

        /**
         * Destroys the widget.
         */
        destroy: function()
        {
            // call the superclass's method
            this.inherited("destroy", arguments);
        },
                
        /**        
         * Returns an object which represent the widget value.
         */
        getValue: function()
        {
            // return the value from the text box
            return this._textNode.getValue();
        },
        
        /**        
         * Sets the widget value.
         */
        setValue: function(value)
        {
            // update the text box value
            this._textNode.setValue(parseInt(value, 10));
        },

        /**        
         * Returns the text representation of the value (a String)
         */
        getTextValue: function()
        {
            // return the text value
            return this._textNode.getTextValue();
        },
        
        /**
         * Returns true if the widget's value is not set.
         */
        isEmpty: function()
        {
            return this._textNode.isEmpty();
        },
        
        /**
         * Returns true if the widget's value is wrong.
         */
        isInError: function()
        {
            return this._textNode.isInError();
        },
        
        /**
         * Sets the widget required constraint.
         * 
         *  @param required:
         *      true if the value is required, false otherwise.
         */        
        setRequired: function(required)
        {
            this._textNode.setRequired(required);
        },
        
        /**
         * Handles the up-arrow pushed event.
         */
        _onPushUpArrow: function(e)
        {
            // handle the event only if the has been just pushed
            if ( this._isArrowPushed === false ) {
                this._onPushArrow(e, 1);
            }
        },
           
        /**
         * Handles the down-arrow pushed event.
         */
        _onPushDownArrow: function(e)
        {
            // handle the event only if the has been just pushed
            if ( this._isArrowPushed === false ) {
                this._onPushArrow(e, -1);
            }
        },
        
        /**
         * Handles a generic-arrow pushed event.
         */
        _onPushArrow: function(e, increment)
        {
            // skip if the pressed key is not the enter 
            if ( e.type === "keypress" && e.charCode !== 32 ) {
                return;
            }
            // the arrow has been pushed
            this._isArrowPushed = true;
            // start the autoclick
            this._startAutoClick(increment);
        },
        
        /**
         * Handles a generic-arrow pushed event.
         */
        _onPullArrow: function(e)
        {
            // the arrow has been pulled
            this._isArrowPushed = false;
            // stop the autoclick
            this._stopAutoClick();
        },     

        /**
         * Starts the auto-click (if a valid number is in place) 
         */
        _startAutoClick: function(increment)
        {
            // store the delta increment
            this._autoClickDelta = increment * this.smallDelta;
            // store the auto-click delay
            this._autoClickDelay = this._autoClickMaxDelay;
            // prepare values
            var value = this._textNode.getValue();
            var min = this._textNode.constraints.min;
            var max = this._textNode.constraints.max;
            
            // correct the value if wrong
            if ( isNaN(value) ) {
                // the value is not a number
                if ( increment === 1 && min !== undefined ) {
                    // up-arrow pushed set to the minimum
                    this._textNode.setValue(min);
                }
                else if ( increment === -1 && max !== undefined ) {
                    // up-arrow pushed set to the maximum
                    this._textNode.setValue(max);
                } 
                else {
                    // no constraint available to use, set to 0
                    this._textNode.setValue(0);
                }
                // schedule the next auto-click callback
                this._autoClickHandle = setTimeout(this._autoClickCallback, this._autoClickDelay);
            }
            else if ( this._isOutOfRange(value) ) {
                // the value is out of range
                if ( min !== undefined && value < min ) {
                    this._textNode.setValue(min);
                }
                else if ( max !== undefined && value > max ) {
                    this._textNode.setValue(max);
                }  
                // schedule the next auto-click callback
                this._autoClickHandle = setTimeout(this._autoClickCallback, this._autoClickDelay);
            }
            else {
                // start the auto-click
                this._autoClick();
            }
        },
        
        /**
         * Stops the auto-click.
         */
        _stopAutoClick: function() 
        {
            // stop the auto-click 
            clearTimeout(this._autoClickHandle);
            this._autoClickHandle = null;
            // notify about the change
            this.onValueChanged(this.getValue());
        },             
        
        /**
         * Simulates a click and schedules the next one. 
         */
        _autoClick: function() 
        {
            // compute the next value
            var value = this._textNode.getValue() + this._autoClickDelta;
            // check constraints 
            if ( this._isOutOfRange(value) ) {
                // the value is out of range, stop auto-click 
                this._stopAutoClick();
            }
            else {
                // set the value
                this._textNode.setValue(value);
                // compute the next auto-click delay
                this._autoClickDelay = Math.max(this._autoClickDelay - this._autoClickDelayAdjustment, this._autoClickMinDelay);
                // schedule the next auto-click callback
                this._autoClickHandle = setTimeout(this._autoClickCallback, this._autoClickDelay);
            }
        },
        
        /**
         * Returns true if the given value is out of range.
         */
        _isOutOfRange: function(value)
        {
            return ( this._textNode.constraints.max !== undefined && value > this._textNode.constraints.max ) ||
                   ( this._textNode.constraints.min !== undefined && value < this._textNode.constraints.min );
        }     
    }        
);


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TextArea"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TextArea"] = true;
dojo.provide("ibm.tivoli.tip.dijit.TextArea"); 





/** 
 * A Text Area widget.
 * 
 * Such widget displays a text area.
 * A limit on the number of characters to be accepted can be speficied.
 * If such limit is passed a warning message is shown, but the user
 * can still insert them.
 *   
 * The widget accepts the following parameters:
 * 
 *  - value: the initial value the widget must show when created.
 *  - constraints: an object containing the validation constraints.
 *    The same constraints can be provided as single properties.
 *    A constraint provided as a single property overrides the 
 *    corresponding constraint provided in the constraints object.
 *    The possible constraints are:
 *      
 *      - maxlen: (Number) the maximum number of characters.   
 *      - maxLength: (String) the same of constraints.maxlen.
 *      - rows: (String) the number of rows to display.
 *      - cols: (String) the number of coloumns to display.
 * 
 * Example of a declaration:
 * 
 *       <div 
 *          dojoType="ibm.tivoli.tip.dijit.TextArea" 
 *          maxLength="100"
 *          rows="10"
 *          cols="50"/>
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)    
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.TextArea",
    [ibm.tivoli.tip.dijit.Widget, dijit._Templated],
    {
        /** the widget template path */
        templateString:"<div\r\n    ><textarea \r\n       class=\"textAreaDefault\"\r\n       dojoAttachPoint=\"_textAreaNode, _focusNode\" \r\n       dojoAttachEvent=\"onkeyup:_onKeyUp, onkeypress:_onKeyPress, onchange:_onChange\"\r\n       value=\"\"\r\n     ></textarea\r\n><div\r\n    class=\"textAreaMessageDefault\"\r\n    dojoAttachPoint=\"_messageNode, _tooltipNode\"\r\n    ></div\r\n>\r\n\r\n",
        /** attribute map */
        attributeMap: dojo.mixin(
            dojo.clone(dijit._Widget.prototype.attributeMap),
            {cols: "_textAreaNode", rows: "_textAreaNode"}),
        
        /** the initial value */
        value: "",
        /** the constraints */
        constraints: null,
        /** the maxlen */
        maxLength: "",
        /** number of visible rows for the text area */  
        rows: "10",
        /** number of visible columns for the text area */
        cols: "30",
        
        /** text area DOM node */
        _textAreaNode: null,
        /** message DOM node */
        _messageNode: null,

        /**
         * Widget constructor.
         */
        constructor: function()
        {
            /** the widget's constraints */
            this.constraints = {};
        },
        
        /**
         * Sets the initial value.
         * The initial value can be externally provided by setting the 
         * value property.  
         */
        startup: function() 
        {
            // call the superclass's method
            this.inherited("startup", arguments);

            // get the maxlen
            if ( this.maxLength !== "" ) {
                this.constraints.maxlen = parseInt(this.maxLength, 10);
            }          
            // set the value
            this.setValue( this.value || "" );
        },

        /**        
         * Returns an object which represent the widget value.
         */
        getValue: function()
        {
            // return the value
            return this._textAreaNode.value;
        },
        
        /**        
         * Sets the widget value.
         */
        setValue: function(value)
        {
            // update the input box
            this._textAreaNode.value = value;
            // update the counter
            this._updateCounter();
        },
        
        /**
         * Returns true if the widget's value is not set.
         */
        isEmpty: function()
        {
            return ( this._textAreaNode.value === "" );
        },

        /**
         * Enables the widget for editing. 
         */
        setDisabled: function(isDisabled)
        {
            if ( isDisabled != this.isDisabled() ) {
                // update the enablement status
                this.inherited("setDisabled", arguments);
                // update the widget
                if ( isDisabled === true ) {
                    // the widget must be disabled
                    this._textAreaNode.disabled = true;
                    this._textAreaNode.className = "textAreaDisabled";
                    this._messageNode.className = "textAreaMessageDisabled";
                }
                else {
                    // the widget must be enabled
                    this._textAreaNode.disabled = false;
                    this._textAreaNode.className = "textAreaDefault";
                    this._messageNode.className = "textAreaMessageDefault";
                }
            }
        },

        /**
         * Clear the message text value.
         */
        clear: function()
        {
            this.setValue("");
        },
        
        /**
         * Handles the onkeypress event. Prevents container to catch
         * the key pression. 
         */
        _onKeyPress: function(e)
        {
            // let the tab to be propagated
            if ( e.keyCode !== 9 ) {
                // stop event propagation
                e.stopPropagation();
            }
        },
        
        /**
         * Handles the onkeypress event. The internal value is updated and
         * the validation is triggered. 
         */
        _onKeyUp: function(e)
        {
            // update the counter
            this._updateCounter();
            // stop event propagation
            e.stopPropagation();
        },
        
        /**
         * Handles the onchange event.  
         */
        _onChange: function()
        {
            // notify the change
            this.onValueChanged(this._textAreaNode.value);
        },
        
        /**
         * Handles the onfocus event.  
         */
        onFocus: function()
        {
            this._updateCounter();
        },
        
        /**
         * Updates the text area counter and shows the tooltip message 
         * according to the maxlen constraint.
         */
        _updateCounter: function()
        {
            var maxlen = this.constraints.maxlen;
            if ( ! maxlen ) {
                // there is no contraints on the max length
                return;
            }
            
            // update the counter
            var len = this._textAreaNode.value.length;
            var left = maxlen - len;
            left = left > 0 ? left : 0; 
            this._messageNode.innerHTML = dojo.string.substitute(this._resources.TEXTAREA_MESSAGE, {len: len, left: left});

            // show/hide the tooltip          
            if ( left === 0 ) {
                this.setStatus({ 
                    state: this.constants.ERROR,
                    message: dojo.string.substitute(this._resources.TEXTAREA_MAX_LIMIT, {maxlen: maxlen})
                });                
            }
            else if ( left > 0 ){
               this.setStatus({ 
                    state: this.constants.DEFAULT,
                    message: ""
                });                
            }      
        }
    }
);


}

if(!dojo._hasResource["dijit._TimePicker"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._TimePicker"] = true;
dojo.provide("dijit._TimePicker");




/*=====
dojo.declare(
	"dijit._TimePicker.__Constraints",
	dojo.date.locale.__FormatOptions,
	{
		// clickableIncrement: String
		//		See `dijit._TimePicker.clickableIncrement`
		clickableIncrement: "T00:15:00",

		// visibleIncrement: String
		//		See `dijit._TimePicker.visibleIncrement`
		visibleIncrement: "T01:00:00",

		// visibleRange: String
		//		See `dijit._TimePicker.visibleRange`
		visibleRange: "T05:00:00"
	}
);
=====*/

dojo.declare("dijit._TimePicker",
	[dijit._Widget, dijit._Templated],
	{
		// summary:
		//		A graphical time picker.
		//		This widget is used internally by other widgets and is not available
		//		as a standalone widget due to lack of accessibility support.

		templateString: dojo.cache("dijit", "templates/TimePicker.html", "<div id=\"widget_${id}\" class=\"dijitMenu\"\r\n    ><div dojoAttachPoint=\"upArrow\" class=\"dijitButtonNode dijitUpArrowButton\" dojoAttachEvent=\"onmouseenter:_buttonMouse,onmouseleave:_buttonMouse\"\r\n\t\t><div class=\"dijitReset dijitInline dijitArrowButtonInner\" wairole=\"presentation\" role=\"presentation\">&nbsp;</div\r\n\t\t><div class=\"dijitArrowButtonChar\">&#9650;</div></div\r\n    ><div dojoAttachPoint=\"timeMenu,focusNode\" dojoAttachEvent=\"onclick:_onOptionSelected,onmouseover,onmouseout\"></div\r\n    ><div dojoAttachPoint=\"downArrow\" class=\"dijitButtonNode dijitDownArrowButton\" dojoAttachEvent=\"onmouseenter:_buttonMouse,onmouseleave:_buttonMouse\"\r\n\t\t><div class=\"dijitReset dijitInline dijitArrowButtonInner\" wairole=\"presentation\" role=\"presentation\">&nbsp;</div\r\n\t\t><div class=\"dijitArrowButtonChar\">&#9660;</div></div\r\n></div>\r\n"),

		// baseClass: [protected] String
		//		The root className to use for the various states of this widget
		baseClass: "dijitTimePicker",

		// clickableIncrement: String
		//		ISO-8601 string representing the amount by which
		//		every clickable element in the time picker increases.
		//		Set in local time, without a time zone.
		//		Example: `T00:15:00` creates 15 minute increments
		//		Must divide dijit._TimePicker.visibleIncrement evenly
		clickableIncrement: "T00:15:00",

		// visibleIncrement: String
		//		ISO-8601 string representing the amount by which
		//		every element with a visible time in the time picker increases.
		//		Set in local time, without a time zone.
		//		Example: `T01:00:00` creates text in every 1 hour increment
		visibleIncrement: "T01:00:00",

		// visibleRange: String
		//		ISO-8601 string representing the range of this TimePicker.
		//		The TimePicker will only display times in this range.
		//		Example: `T05:00:00` displays 5 hours of options
		visibleRange: "T05:00:00",

		// value: String
		//		Date to display.
		//		Defaults to current time and date.
		//		Can be a Date object or an ISO-8601 string.
		//		If you specify the GMT time zone (`-01:00`),
		//		the time will be converted to the local time in the local time zone.
		//		Otherwise, the time is considered to be in the local time zone.
		//		If you specify the date and isDate is true, the date is used.
		//		Example: if your local time zone is `GMT -05:00`,
		//		`T10:00:00` becomes `T10:00:00-05:00` (considered to be local time),
		//		`T10:00:00-01:00` becomes `T06:00:00-05:00` (4 hour difference),
		//		`T10:00:00Z` becomes `T05:00:00-05:00` (5 hour difference between Zulu and local time)
		//		`yyyy-mm-ddThh:mm:ss` is the format to set the date and time
		//		Example: `2007-06-01T09:00:00`
		value: new Date(),

		_visibleIncrement:2,
		_clickableIncrement:1,
		_totalIncrements:10,

		// constraints: dijit._TimePicker.__Constraints
		//		Specifies valid range of times (start time, end time)
		constraints:{},

/*=====
		serialize: function(val, options){
			// summary:
			//		User overridable function used to convert the attr('value') result to a String
			// val: Date
			//		The current value
			// options: Object?
			// tags:
			//		protected
		},
=====*/
		serialize: dojo.date.stamp.toISOString,

		// _filterString: string
		//		The string to filter by
		_filterString: "",

		setValue: function(/*Date*/ value){
			// summary:
			//		Deprecated.  Used attr('value') instead.
			// tags:
			//		deprecated
			dojo.deprecated("dijit._TimePicker:setValue() is deprecated.  Use set('value', ...) instead.", "", "2.0");
			this.set('value', value);
		},
		_setValueAttr: function(/*Date*/ date){
			// summary:
			//		Hook so attr('value', ...) works.
			// description:
			//		Set the value of the TimePicker.
			//		Redraws the TimePicker around the new date.
			// tags:
			//		protected
			this.value = date;
			this._showText();
		},

		onOpen: function(best){
			// summary:
			//		This is called by the popup manager when a TimeTextBox is displayed on the screen
			// best:
			//		Whether it is being displayed above or below the `dijit.form.TimeTextBox`
			// tags:
			//		protected
			if(this._beenOpened && this.domNode.parentNode){
				// We've been opened before - so set our filter to to the
				// currently-displayed value (or empty string if it's already
				// valid)
				var p = dijit.byId(this.domNode.parentNode.dijitPopupParent);
				if(p){
					var val = p.get('displayedValue');
					if(val && !p.parse(val, p.constraints)){
						this._filterString = val;
					}else{
						this._filterString = "";
					}
					this._showText();
				}
			}
			this._beenOpened = true;
		},

		isDisabledDate: function(/*Date*/ dateObject, /*String?*/ locale){
			// summary:
			//		May be overridden to disable certain dates in the TimePicker e.g. `isDisabledDate=dojo.date.locale.isWeekend`
			// type:
			//		extension
			return false; // Boolean
		},

		_getFilteredNodes: function(/*number*/ start, /*number*/ maxNum, /*Boolean*/ before){
			// summary:
			//		Returns an array of nodes with the filter applied.  At most maxNum nodes
			//		will be returned - but fewer may be returned as well.  If the
			//		before parameter is set to true, then it will return the elements
			//		before the given index
			// tags:
			//		private
			var nodes = [], n, i = start, max = this._maxIncrement + Math.abs(i),
				chk = before?-1:1, dec = before?1:0, inc = before?0:1;
			do{
				i = i - dec;
				n = this._createOption(i);
				if(n){nodes.push(n);}
				i = i + inc;
			}while(nodes.length < maxNum && (i*chk) < max);
			if(before){ nodes.reverse(); }
			return nodes;
		},

		_showText: function(){
			// summary:
			//		Displays the relevant choices in the drop down list
			// tags:
			//		private
			this.timeMenu.innerHTML = "";
			var fromIso = dojo.date.stamp.fromISOString;
			this._clickableIncrementDate=fromIso(this.clickableIncrement);
			this._visibleIncrementDate=fromIso(this.visibleIncrement);
			this._visibleRangeDate=fromIso(this.visibleRange);
			// get the value of the increments and the range in seconds (since 00:00:00) to find out how many divs to create
			var sinceMidnight = function(/*Date*/ date){
				return date.getHours() * 60 * 60 + date.getMinutes() * 60 + date.getSeconds();
			};

			var clickableIncrementSeconds = sinceMidnight(this._clickableIncrementDate);
			var visibleIncrementSeconds = sinceMidnight(this._visibleIncrementDate);
			var visibleRangeSeconds = sinceMidnight(this._visibleRangeDate);

			// round reference date to previous visible increment
			var time = this.value.getTime();
			this._refDate = new Date(time - time % (visibleIncrementSeconds*1000));
			this._refDate.setFullYear(1970,0,1); // match parse defaults

			// assume clickable increment is the smallest unit
			this._clickableIncrement = 1;
			// divide the visible range by the clickable increment to get the number of divs to create
			// example: 10:00:00/00:15:00 -> display 40 divs
			this._totalIncrements = visibleRangeSeconds / clickableIncrementSeconds;
			// divide the visible increments by the clickable increments to get how often to display the time inline
			// example: 01:00:00/00:15:00 -> display the time every 4 divs
			this._visibleIncrement = visibleIncrementSeconds / clickableIncrementSeconds;
			// divide the number of seconds in a day by the clickable increment in seconds to get the
			// absolute max number of increments.
			this._maxIncrement = (60 * 60 * 24) / clickableIncrementSeconds;

			// find the nodes we should display based on our filter
			var before = this._getFilteredNodes(0, this._totalIncrements >> 1, true);
			var after = this._getFilteredNodes(0, this._totalIncrements >> 1, false);
			if(before.length < this._totalIncrements >> 1){
				before = before.slice(before.length / 2);
				after = after.slice(0, after.length / 2);
			}
			dojo.forEach(before.concat(after), function(n){this.timeMenu.appendChild(n);}, this);

			// TODO:
			// I commented this out because it
			// causes problems for a TimeTextBox in a Dialog, or as the editor of an InlineEditBox,
			// because the timeMenu node isn't visible yet. -- Bill (Bug #????)
			// dijit.focus(this.timeMenu);
		},

		constructor: function(){
			this.constraints = {}; // create instance object
		},

		postMixInProperties: function(){
		        this.inherited(arguments);
			this._setConstraintsAttr(this.constraints); // this needs to happen now (and later) due to codependency on _set*Attr calls
		},

		_setConstraintsAttr: function(/* Object */ constraints){
			// brings in visibleRange, increments, etc.
			dojo.mixin(this, constraints);

			// dojo.date.locale needs the lang in the constraints as locale
			if(!constraints.locale){
				constraints.locale = this.lang;
			}
		},

		postCreate: function(){
			// assign typematic mouse listeners to the arrow buttons
			this.connect(this.timeMenu, dojo.isIE ? "onmousewheel" : 'DOMMouseScroll', "_mouseWheeled");
			var _this = this;
			var typematic = function(){
				_this._connects.push(
					dijit.typematic.addMouseListener.apply(null, arguments)
				);
			};
			typematic(this.upArrow,this,this._onArrowUp, 1.0, 50);
			typematic(this.downArrow,this,this._onArrowDown, 1.0, 50);

			// Connect some callback functions to the hover event of the arrows
			var triggerFx = function(cb){
				return function(cnt){
					// don't run on the first firing
					if(cnt > 0){cb.call(this, arguments);}
				};
			};
			var hoverFx = function(node, cb){
				return function(e){
					dojo.stopEvent(e);
					dijit.typematic.trigger(e, this, node, triggerFx(cb), node, 1.0, 50);
				};
			};
			this.connect(this.upArrow, "onmouseover", hoverFx(this.upArrow, this._onArrowUp));
			this.connect(this.downArrow, "onmouseover", hoverFx(this.downArrow, this._onArrowDown));

			this.inherited(arguments);
		},

		_buttonMouse: function(/*Event*/ e){
			// summary:
			//		Handler for hover (and unhover) on up/down arrows
			// tags:
			//		private

			// in non-IE browser the "mouseenter" event will become "mouseover", 
			// but in IE it's still "mouseenter"
			dojo.toggleClass(e.currentTarget, e.currentTarget == this.upArrow ? "dijitUpArrowHover" : "dijitDownArrowHover", 
				e.type == "mouseenter" || e.type == "mouseover");
		},

		_createOption: function(/*Number*/ index){
			// summary:
			//		Creates a clickable time option
			// tags:
			//		private
			var date = new Date(this._refDate);
			var incrementDate = this._clickableIncrementDate;
			date.setHours(date.getHours() + incrementDate.getHours() * index,
				date.getMinutes() + incrementDate.getMinutes() * index,
				date.getSeconds() + incrementDate.getSeconds() * index);
			if(this.constraints.selector == "time"){
				date.setFullYear(1970,0,1); // make sure each time is for the same date
			}
			var dateString = dojo.date.locale.format(date, this.constraints);
			if(this._filterString && dateString.toLowerCase().indexOf(this._filterString) !== 0){
				// Doesn't match the filter - return null
				return null;
			}

			var div = dojo.create("div", {"class": this.baseClass+"Item"});
			div.date = date;
			div.index = index;
			dojo.create('div',{
				"class": this.baseClass + "ItemInner",
				innerHTML: dateString
			}, div);

			if(index%this._visibleIncrement<1 && index%this._visibleIncrement>-1){
				dojo.addClass(div, this.baseClass+"Marker");
			}else if(!(index%this._clickableIncrement)){
				dojo.addClass(div, this.baseClass+"Tick");
			}

			if(this.isDisabledDate(date)){
				// set disabled
				dojo.addClass(div, this.baseClass+"ItemDisabled");
			}
			if(!dojo.date.compare(this.value, date, this.constraints.selector)){
				div.selected = true;
				dojo.addClass(div, this.baseClass+"ItemSelected");
				if(dojo.hasClass(div, this.baseClass+"Marker")){
					dojo.addClass(div, this.baseClass+"MarkerSelected");
				}else{
					dojo.addClass(div, this.baseClass+"TickSelected");
				}
			}
			return div;
		},

		_onOptionSelected: function(/*Object*/ tgt){
			// summary:
			//		Called when user clicks an option in the drop down list
			// tags:
			//		private
			var tdate = tgt.target.date || tgt.target.parentNode.date;
			if(!tdate || this.isDisabledDate(tdate)){ return; }
			this._highlighted_option = null;
			this.set('value', tdate);
			this.onValueSelected(tdate);
		},

		onValueSelected: function(/*Date*/ time){
			// summary:
			//		Notification that a time was selected.  It may be the same as the previous value.
			// description:
			//      Used by `dijit.form._DateTimeTextBox` (and thus `dijit.form.TimeTextBox`)
			//      to get notification when the user has clicked a time.
			// tags:
			//      protected
		},


		_highlightOption: function(/*node*/ node, /*Boolean*/ highlight){
			// summary:
			//		Turns on/off hover effect on a node based on mouse out/over event
			// tags:
			//		private
			if(!node){return;}
			if(highlight){
				if(this._highlighted_option){
					this._highlightOption(this._highlighted_option, false);
				}
				this._highlighted_option = node;
			}else if(this._highlighted_option !== node){
				return;
			}else{
				this._highlighted_option = null;
			}
			dojo.toggleClass(node, this.baseClass+"ItemHover", highlight);
			if(dojo.hasClass(node, this.baseClass+"Marker")){
				dojo.toggleClass(node, this.baseClass+"MarkerHover", highlight);
			}else{
				dojo.toggleClass(node, this.baseClass+"TickHover", highlight);
			}
		},

		onmouseover: function(/*Event*/ e){
			// summary:
			//		Handler for onmouseover event
			// tags:
			//		private
			this._keyboardSelected = null;
			var tgr = (e.target.parentNode === this.timeMenu) ? e.target : e.target.parentNode;
			// if we aren't targeting an item, then we return
			if(!dojo.hasClass(tgr, this.baseClass+"Item")){return;}
			this._highlightOption(tgr, true);
		},

		onmouseout: function(/*Event*/ e){
			// summary:
			//		Handler for onmouseout event
			// tags:
			//		private
			this._keyboardSelected = null;
			var tgr = (e.target.parentNode === this.timeMenu) ? e.target : e.target.parentNode;
			this._highlightOption(tgr, false);
		},

		_mouseWheeled: function(/*Event*/ e){
			// summary:
			//		Handle the mouse wheel events
			// tags:
			//		private
			this._keyboardSelected = null;
			dojo.stopEvent(e);
			// we're not _measuring_ the scroll amount, just direction
			var scrollAmount = (dojo.isIE ? e.wheelDelta : -e.detail);
			this[(scrollAmount>0 ? "_onArrowUp" : "_onArrowDown")](); // yes, we're making a new dom node every time you mousewheel, or click
		},

		_onArrowUp: function(count){
			// summary:
			//		Handler for up arrow key.
			// description:
			//		Removes the bottom time and add one to the top
			// tags:
			//		private
			if(typeof count == "number" && count == -1){ return; } // typematic end
			if(!this.timeMenu.childNodes.length){ return; }
			var index = this.timeMenu.childNodes[0].index;
			var divs = this._getFilteredNodes(index, 1, true);
			if(divs.length){
				this.timeMenu.removeChild(this.timeMenu.childNodes[this.timeMenu.childNodes.length - 1]);
				this.timeMenu.insertBefore(divs[0], this.timeMenu.childNodes[0]);
			}
		},

		_onArrowDown: function(count){
			// summary:
			//		Handler for up arrow key.
			// description:
			//		Remove the top time and add one to the bottom
			// tags:
			//		private
			if(typeof count == "number" && count == -1){ return; } // typematic end
			if(!this.timeMenu.childNodes.length){ return; }
			var index = this.timeMenu.childNodes[this.timeMenu.childNodes.length - 1].index + 1;
			var divs = this._getFilteredNodes(index, 1, false);
			if(divs.length){
				this.timeMenu.removeChild(this.timeMenu.childNodes[0]);
				this.timeMenu.appendChild(divs[0]);
			}
		},

		handleKey: function(/*Event*/ e){
			// summary:
			//		Called from `dijit.form._DateTimeTextBox` to pass a keypress event
			//		from the `dijit.form.TimeTextBox` to be handled in this widget
			// tags:
			//		protected
			var dk = dojo.keys;
			if(e.keyChar || e.charOrCode === dk.BACKSPACE || e.charOrCode == dk.DELETE){
				// Set a timeout to kick off our filter
				setTimeout(dojo.hitch(this, function(){
					this._filterString = e.target.value.toLowerCase();
					this._showText();
				}),1);
			}else if(e.charOrCode == dk.DOWN_ARROW || e.charOrCode == dk.UP_ARROW){
				dojo.stopEvent(e);
				// Figure out which option to highlight now and then highlight it
				if(this._highlighted_option && !this._highlighted_option.parentNode){
					this._highlighted_option = null;
				}
				var timeMenu = this.timeMenu,
					tgt = this._highlighted_option || dojo.query("." + this.baseClass + "ItemSelected", timeMenu)[0];
				if(!tgt){
					tgt = timeMenu.childNodes[0];
				}else if(timeMenu.childNodes.length){
					if(e.charOrCode == dk.DOWN_ARROW && !tgt.nextSibling){
						this._onArrowDown();
					}else if(e.charOrCode == dk.UP_ARROW && !tgt.previousSibling){
						this._onArrowUp();
					}
					if(e.charOrCode == dk.DOWN_ARROW){
						tgt = tgt.nextSibling;
					}else{
						tgt = tgt.previousSibling;
					}
				}
				this._highlightOption(tgt, true);
				this._keyboardSelected = tgt;
			}else if(this._highlighted_option && (e.charOrCode == dk.ENTER || e.charOrCode === dk.TAB)){
				// Accept the currently-highlighted option as the value
				if(!this._keyboardSelected && e.charOrCode === dk.TAB){ return; } // mouse hover followed by TAB is NO selection
				if(e.charOrCode == dk.ENTER){dojo.stopEvent(e);}
				this._onOptionSelected({target: this._highlighted_option});
			}
		}
	}
);

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.picker.TimePicker"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.picker.TimePicker"] = true;
dojo.provide("ibm.tivoli.tip.dijit.picker.TimePicker"); 

 


/**
 * The time-picker popup. 
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.picker.TimePicker",
    [ibm.tivoli.tip.dijit.picker.PopupPicker, dijit._TimePicker],
    {
        /** constraints */
        constraints: {selector: "time", formatLength: "short"}
    }        
);


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TimeInputBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TimeInputBox"] = true;
dojo.provide("ibm.tivoli.tip.dijit.TimeInputBox"); 






/** 
 * A time input box widget.
 * 
 * Such widget shows a text input box and a time picker icon on the right.
 * By clicking on such icon a time picker is shown to help the user
 * to select a time.
 * 
 * The widget supports the following attributes:
 * 
 *      - value (Date | String): the initial value.
 *      - required (String): "true" if the date is required.
 *      - tabindex (Number): the widget tabindex.
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)    
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.TimeInputBox",
    [ibm.tivoli.tip.dijit.DateInputBox],
    {
        /** the picker class */
        pickerClass: "ibm.tivoli.tip.dijit.picker.TimePicker",
        /** the picker icon */
        pickerIcon: ibm.tivoli.tip.dijit.Images.get()["TIME_ICON"],
        /** constraints */
        constraints: {selector: "time", formatLength: "short"},
        
        /**
         * Constructor.
         */
        constructor: function()
        {
            // set the regex for the text
            this.textRegExp = "^(" + dojo.date.locale.regexp(this.constraints) + "){0,1}$";
            // set the regex message for the text
            this.textRegExpMessage = this._resources.TIME_INVALID_VALUE;
        }
    }        
);


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPButton"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.dijit.TIPButton");


dojo.declare("ibm.tivoli.tip.dijit.TIPButton",
              dijit.form.Button,
{
	//Added fix for IBM Dojo 1.5 Migration
	_onMouse: function(){
	}
});

}

if(!dojo._hasResource["dojo.dnd.Mover"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Mover"] = true;
dojo.provide("dojo.dnd.Mover");




dojo.declare("dojo.dnd.Mover", null, {
	constructor: function(node, e, host){
		// summary:
		//		an object, which makes a node follow the mouse. 
		//		Used as a default mover, and as a base class for custom movers.
		// node: Node
		//		a node (or node's id) to be moved
		// e: Event
		//		a mouse event, which started the move;
		//		only pageX and pageY properties are used
		// host: Object?
		//		object which implements the functionality of the move,
		//	 	and defines proper events (onMoveStart and onMoveStop)
		this.node = dojo.byId(node);
		this.marginBox = {l: e.pageX, t: e.pageY};
		this.mouseButton = e.button;
		var h = this.host = host, d = node.ownerDocument, 
			firstEvent = dojo.connect(d, "onmousemove", this, "onFirstMove");
		this.events = [
			dojo.connect(d, "onmousemove", this, "onMouseMove"),
			dojo.connect(d, "onmouseup",   this, "onMouseUp"),
			// cancel text selection and text dragging
			dojo.connect(d, "ondragstart",   dojo.stopEvent),
			dojo.connect(d.body, "onselectstart", dojo.stopEvent),
			firstEvent
		];
		// notify that the move has started
		if(h && h.onMoveStart){
			h.onMoveStart(this);
		}
	},
	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		dojo.dnd.autoScroll(e);
		var m = this.marginBox;
		this.host.onMove(this, {l: m.l + e.pageX, t: m.t + e.pageY}, e);
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		if(dojo.isWebKit && dojo.isMac && this.mouseButton == 2 ? 
				e.button == 0 : this.mouseButton == e.button){
			this.destroy();
		}
		dojo.stopEvent(e);
	},
	// utilities
	onFirstMove: function(e){
		// summary:
		//		makes the node absolute; it is meant to be called only once. 
		// 		relative and absolutely positioned nodes are assumed to use pixel units
		var s = this.node.style, l, t, h = this.host;
		switch(s.position){
			case "relative":
			case "absolute":
				// assume that left and top values are in pixels already
				l = Math.round(parseFloat(s.left)) || 0;
				t = Math.round(parseFloat(s.top)) || 0;
				break;
			default:
				s.position = "absolute";	// enforcing the absolute mode
				var m = dojo.marginBox(this.node);
				// event.pageX/pageY (which we used to generate the initial
				// margin box) includes padding and margin set on the body.
				// However, setting the node's position to absolute and then
				// doing dojo.marginBox on it *doesn't* take that additional
				// space into account - so we need to subtract the combined
				// padding and margin.  We use getComputedStyle and
				// _getMarginBox/_getContentBox to avoid the extra lookup of
				// the computed style. 
				var b = dojo.doc.body;
				var bs = dojo.getComputedStyle(b);
				var bm = dojo._getMarginBox(b, bs);
				var bc = dojo._getContentBox(b, bs);
				l = m.l - (bc.l - bm.l);
				t = m.t - (bc.t - bm.t);
				break;
		}
		this.marginBox.l = l - this.marginBox.l;
		this.marginBox.t = t - this.marginBox.t;
		if(h && h.onFirstMove){
			h.onFirstMove(this, e);
		}
		dojo.disconnect(this.events.pop());
	},
	destroy: function(){
		// summary:
		//		stops the move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		// undo global settings
		var h = this.host;
		if(h && h.onMoveStop){
			h.onMoveStop(this);
		}
		// destroy objects
		this.events = this.node = this.host = null;
	}
});

}

if(!dojo._hasResource["dojo.dnd.Moveable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Moveable"] = true;
dojo.provide("dojo.dnd.Moveable");



/*=====
dojo.declare("dojo.dnd.__MoveableArgs", [], {
	// handle: Node||String
	//		A node (or node's id), which is used as a mouse handle.
	//		If omitted, the node itself is used as a handle.
	handle: null,

	// delay: Number
	//		delay move by this number of pixels
	delay: 0,

	// skip: Boolean
	//		skip move of form elements
	skip: false,

	// mover: Object
	//		a constructor of custom Mover
	mover: dojo.dnd.Mover
});
=====*/

dojo.declare("dojo.dnd.Moveable", null, {
	// object attributes (for markup)
	handle: "",
	delay: 0,
	skip: false,
	
	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.__MoveableArgs?
		//		optional parameters
		this.node = dojo.byId(node);
		if(!params){ params = {}; }
		this.handle = params.handle ? dojo.byId(params.handle) : null;
		if(!this.handle){ this.handle = this.node; }
		this.delay = params.delay > 0 ? params.delay : 0;
		this.skip  = params.skip;
		this.mover = params.mover ? params.mover : dojo.dnd.Mover;
		this.events = [
			dojo.connect(this.handle, "onmousedown", this, "onMouseDown"),
			// cancel text selection and text dragging
			dojo.connect(this.handle, "ondragstart",   this, "onSelectStart"),
			dojo.connect(this.handle, "onselectstart", this, "onSelectStart")
		];
	},

	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.Moveable(node, params);
	},

	// methods
	destroy: function(){
		// summary:
		//		stops watching for possible move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		this.events = this.node = this.handle = null;
	},
	
	// mouse event processors
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown, creates a Mover for the node
		// e: Event
		//		mouse event
		if(this.skip && dojo.dnd.isFormElement(e)){ return; }
		if(this.delay){
			this.events.push(
				dojo.connect(this.handle, "onmousemove", this, "onMouseMove"),
				dojo.connect(this.handle, "onmouseup", this, "onMouseUp")
			);
			this._lastX = e.pageX;
			this._lastY = e.pageY;
		}else{
			this.onDragDetected(e);
		}
		dojo.stopEvent(e);
	},
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove, used only for delayed drags
		// e: Event
		//		mouse event
		if(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay){
			this.onMouseUp(e);
			this.onDragDetected(e);
		}
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup, used only for delayed drags
		// e: Event
		//		mouse event
		for(var i = 0; i < 2; ++i){
			dojo.disconnect(this.events.pop());
		}
		dojo.stopEvent(e);
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skip || !dojo.dnd.isFormElement(e)){
			dojo.stopEvent(e);
		}
	},
	
	// local events
	onDragDetected: function(/* Event */ e){
		// summary:
		//		called when the drag is detected;
		//		responsible for creation of the mover
		new this.mover(this.node, e, this);
	},
	onMoveStart: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called before every move operation
		dojo.publish("/dnd/move/start", [mover]);
		dojo.addClass(dojo.body(), "dojoMove"); 
		dojo.addClass(this.node, "dojoMoveItem"); 
	},
	onMoveStop: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called after every move operation
		dojo.publish("/dnd/move/stop", [mover]);
		dojo.removeClass(dojo.body(), "dojoMove");
		dojo.removeClass(this.node, "dojoMoveItem");
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover, /* Event */ e){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		
		// default implementation does nothing
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop, /* Event */ e){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		this.onMoving(mover, leftTop);
		var s = mover.node.style;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	},
	onMoving: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called before every incremental move; can be overwritten.
		
		// default implementation does nothing
	},
	onMoved: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called after every incremental move; can be overwritten.
		
		// default implementation does nothing
	}
});

}

if(!dojo._hasResource["dojo.dnd.move"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.move"] = true;
dojo.provide("dojo.dnd.move");




/*=====
dojo.declare("dojo.dnd.move.__constrainedMoveableArgs", [dojo.dnd.__MoveableArgs], {
	// constraints: Function
	//		Calculates a constraint box.
	//		It is called in a context of the moveable object.
	constraints: function(){},

	// within: Boolean
	//		restrict move within boundaries.
	within: false
});
=====*/

dojo.declare("dojo.dnd.move.constrainedMoveable", dojo.dnd.Moveable, {
	// object attributes (for markup)
	constraints: function(){},
	within: false,
	
	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.constrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary:
		//		an object that makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__constrainedMoveableArgs?
		//		an optional object with additional parameters;
		//		the rest is passed to the base class
		if(!params){ params = {}; }
		this.constraints = params.constraints;
		this.within = params.within;
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		var c = this.constraintBox = this.constraints.call(this, mover);
		c.r = c.l + c.w;
		c.b = c.t + c.h;
		if(this.within){
			var mb = dojo.marginBox(mover.node);
			c.r -= mb.w;
			c.b -= mb.h;
		}
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		var c = this.constraintBox, s = mover.node.style;
		s.left = (leftTop.l < c.l ? c.l : c.r < leftTop.l ? c.r : leftTop.l) + "px";
		s.top  = (leftTop.t < c.t ? c.t : c.b < leftTop.t ? c.b : leftTop.t) + "px";
	}
});

/*=====
dojo.declare("dojo.dnd.move.__boxConstrainedMoveableArgs", [dojo.dnd.move.__constrainedMoveableArgs], {
	// box: Object
	//		a constraint box
	box: {}
});
=====*/

dojo.declare("dojo.dnd.move.boxConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// box:
	//		object attributes (for markup)
	box: {},
	
	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.boxConstrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__boxConstrainedMoveableArgs?
		//		an optional object with parameters
		var box = params && params.box;
		this.constraints = function(){ return box; };
	}
});

/*=====
dojo.declare("dojo.dnd.move.__parentConstrainedMoveableArgs", [dojo.dnd.move.__constrainedMoveableArgs], {
	// area: String
	//		A parent's area to restrict the move.
	//		Can be "margin", "border", "padding", or "content".
	area: ""
});
=====*/

dojo.declare("dojo.dnd.move.parentConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// area:
	//		object attributes (for markup)
	area: "content",

	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.parentConstrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__parentConstrainedMoveableArgs?
		//		an optional object with parameters
		var area = params && params.area;
		this.constraints = function(){
			var n = this.node.parentNode, 
				s = dojo.getComputedStyle(n), 
				mb = dojo._getMarginBox(n, s);
			if(area == "margin"){
				return mb;	// Object
			}
			var t = dojo._getMarginExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "border"){
				return mb;	// Object
			}
			t = dojo._getBorderExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "padding"){
				return mb;	// Object
			}
			t = dojo._getPadExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			return mb;	// Object
		};
	}
});

// WARNING: below are obsolete objects, instead of custom movers use custom moveables (above)

dojo.dnd.move.constrainedMover = function(fun, within){
	// summary:
	//		returns a constrained version of dojo.dnd.Mover
	// description:
	//		this function produces n object, which will put a constraint on 
	//		the margin box of dragged object in absolute coordinates
	// fun: Function
	//		called on drag, and returns a constraint box
	// within: Boolean
	//		if true, constraints the whole dragged object withtin the rectangle, 
	//		otherwise the constraint is applied to the left-top corner

	dojo.deprecated("dojo.dnd.move.constrainedMover, use dojo.dnd.move.constrainedMoveable instead");
	var mover = function(node, e, notifier){
		dojo.dnd.Mover.call(this, node, e, notifier);
	};
	dojo.extend(mover, dojo.dnd.Mover.prototype);
	dojo.extend(mover, {
		onMouseMove: function(e){
			// summary: event processor for onmousemove
			// e: Event: mouse event
			dojo.dnd.autoScroll(e);
			var m = this.marginBox, c = this.constraintBox,
				l = m.l + e.pageX, t = m.t + e.pageY;
			l = l < c.l ? c.l : c.r < l ? c.r : l;
			t = t < c.t ? c.t : c.b < t ? c.b : t;
			this.host.onMove(this, {l: l, t: t});
		},
		onFirstMove: function(){
			// summary: called once to initialize things; it is meant to be called only once
			dojo.dnd.Mover.prototype.onFirstMove.call(this);
			var c = this.constraintBox = fun.call(this);
			c.r = c.l + c.w;
			c.b = c.t + c.h;
			if(within){
				var mb = dojo.marginBox(this.node);
				c.r -= mb.w;
				c.b -= mb.h;
			}
		}
	});
	return mover;	// Object
};

dojo.dnd.move.boxConstrainedMover = function(box, within){
	// summary:
	//		a specialization of dojo.dnd.constrainedMover, which constrains to the specified box
	// box: Object
	//		a constraint box (l, t, w, h)
	// within: Boolean
	//		if true, constraints the whole dragged object withtin the rectangle, 
	//		otherwise the constraint is applied to the left-top corner

	dojo.deprecated("dojo.dnd.move.boxConstrainedMover, use dojo.dnd.move.boxConstrainedMoveable instead");
	return dojo.dnd.move.constrainedMover(function(){ return box; }, within);	// Object
};

dojo.dnd.move.parentConstrainedMover = function(area, within){
	// summary:
	//		a specialization of dojo.dnd.constrainedMover, which constrains to the parent node
	// area: String
	//		"margin" to constrain within the parent's margin box, "border" for the border box,
	//		"padding" for the padding box, and "content" for the content box; "content" is the default value.
	// within: Boolean
	//		if true, constraints the whole dragged object within the rectangle, 
	//		otherwise the constraint is applied to the left-top corner

	dojo.deprecated("dojo.dnd.move.parentConstrainedMover, use dojo.dnd.move.parentConstrainedMoveable instead");
	var fun = function(){
		var n = this.node.parentNode, 
			s = dojo.getComputedStyle(n), 
			mb = dojo._getMarginBox(n, s);
		if(area == "margin"){
			return mb;	// Object
		}
		var t = dojo._getMarginExtents(n, s);
		mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
		if(area == "border"){
			return mb;	// Object
		}
		t = dojo._getBorderExtents(n, s);
		mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
		if(area == "padding"){
			return mb;	// Object
		}
		t = dojo._getPadExtents(n, s);
		mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
		return mb;	// Object
	};
	return dojo.dnd.move.constrainedMover(fun, within);	// Object
};

// patching functions one level up for compatibility

dojo.dnd.constrainedMover = dojo.dnd.move.constrainedMover;
dojo.dnd.boxConstrainedMover = dojo.dnd.move.boxConstrainedMover;
dojo.dnd.parentConstrainedMover = dojo.dnd.move.parentConstrainedMover;

}

if(!dojo._hasResource["dojo.dnd.TimedMoveable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.TimedMoveable"] = true;
dojo.provide("dojo.dnd.TimedMoveable");



/*=====
dojo.declare("dojo.dnd.__TimedMoveableArgs", [dojo.dnd.__MoveableArgs], {
	// timeout: Number
	//		delay move by this number of ms,
	//		accumulating position changes during the timeout
	timeout: 0
});
=====*/

(function(){
	// precalculate long expressions
	var oldOnMove = dojo.dnd.Moveable.prototype.onMove;
		
	dojo.declare("dojo.dnd.TimedMoveable", dojo.dnd.Moveable, {
		// summary:
		//		A specialized version of Moveable to support an FPS throttling.
		//		This class puts an upper restriction on FPS, which may reduce 
		//		the CPU load. The additional parameter "timeout" regulates
		//		the delay before actually moving the moveable object.
		
		// object attributes (for markup)
		timeout: 40,	// in ms, 40ms corresponds to 25 fps
	
		constructor: function(node, params){
			// summary:
			//		an object that makes a node moveable with a timer
			// node: Node||String
			//		a node (or node's id) to be moved
			// params: dojo.dnd.__TimedMoveableArgs
			//		object with additional parameters.
			
			// sanitize parameters
			if(!params){ params = {}; }
			if(params.timeout && typeof params.timeout == "number" && params.timeout >= 0){
				this.timeout = params.timeout;
			}
		},
	
		// markup methods
		markupFactory: function(params, node){
			return new dojo.dnd.TimedMoveable(node, params);
		},
	
		onMoveStop: function(/* dojo.dnd.Mover */ mover){
			if(mover._timer){
				// stop timer
				clearTimeout(mover._timer)
				// reflect the last received position
				oldOnMove.call(this, mover, mover._leftTop)
			}
			dojo.dnd.Moveable.prototype.onMoveStop.apply(this, arguments);
		},
		onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
			mover._leftTop = leftTop;
			if(!mover._timer){
				var _t = this;	// to avoid using dojo.hitch()
				mover._timer = setTimeout(function(){
					// we don't have any pending requests
					mover._timer = null;
					// reflect the last received position
					oldOnMove.call(_t, mover, mover._leftTop);
				}, this.timeout);
			}
		}
	});
})();

}

if(!dojo._hasResource["dijit.form._FormMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormMixin"] = true;
dojo.provide("dijit.form._FormMixin");



dojo.declare("dijit.form._FormMixin", null,
	{
	// summary:
	//		Mixin for containers of form widgets (i.e. widgets that represent a single value
	//		and can be children of a <form> node or dijit.form.Form widget)
	// description:
	//		Can extract all the form widgets
	//		values and combine them into a single javascript object, or alternately
	//		take such an object and set the values for all the contained
	//		form widgets

/*=====
    // value: Object
	//		Name/value hash for each child widget with a name and value.
	//		Child widgets without names are not part of the hash.
	// 
	//		If there are multiple child widgets w/the same name, value is an array,
	//		unless they are radio buttons in which case value is a scalar (since only
	//		one radio button can be checked at a time).
	//
	//		If a child widget's name is a dot separated list (like a.b.c.d), it's a nested structure.
	//
	//		Example:
	//	|	{ name: "John Smith", interests: ["sports", "movies"] }
=====*/

	//	TODO:
	//	* Repeater
	//	* better handling for arrays.  Often form elements have names with [] like
	//	* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])
	//
	//

		reset: function(){
			dojo.forEach(this.getDescendants(), function(widget){
				if(widget.reset){
					widget.reset();
				}
			});
		},

		validate: function(){
			// summary:
			//		returns if the form is valid - same as isValid - but
			//			provides a few additional (ui-specific) features.
			//			1 - it will highlight any sub-widgets that are not
			//				valid
			//			2 - it will call focus() on the first invalid
			//				sub-widget
			var didFocus = false;
			return dojo.every(dojo.map(this.getDescendants(), function(widget){
				// Need to set this so that "required" widgets get their
				// state set.
				widget._hasBeenBlurred = true;
				var valid = widget.disabled || !widget.validate || widget.validate();
				if(!valid && !didFocus){
					// Set focus of the first non-valid widget
					dojo.window.scrollIntoView(widget.containerNode || widget.domNode);
					widget.focus();
					didFocus = true;
				}
	 			return valid;
	 		}), function(item){ return item; });
		},

		setValues: function(val){
			dojo.deprecated(this.declaredClass+"::setValues() is deprecated. Use set('value', val) instead.", "", "2.0");
			return this.set('value', val);
		},
		_setValueAttr: function(/*object*/obj){
			// summary:
			//		Fill in form values from according to an Object (in the format returned by attr('value'))

			// generate map from name --> [list of widgets with that name]
			var map = { };
			dojo.forEach(this.getDescendants(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			for(var name in map){
				if(!map.hasOwnProperty(name)){
					continue;
				}
				var widgets = map[name],						// array of widgets w/this name
					values = dojo.getObject(name, false, obj);	// list of values for those widgets

				if(values === undefined){
					continue;
				}
				if(!dojo.isArray(values)){
					values = [ values ];
				}
				if(typeof widgets[0].checked == 'boolean'){
					// for checkbox/radio, values is a list of which widgets should be checked
					dojo.forEach(widgets, function(w, i){
						w.set('value', dojo.indexOf(values, w.value) != -1);
					});
				}else if(widgets[0].multiple){
					// it takes an array (e.g. multi-select)
					widgets[0].set('value', values);
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					dojo.forEach(widgets, function(w, i){
						w.set('value', values[i]);
					});
				}
			}

			/***
			 * 	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets)

			dojo.forEach(this.containerNode.elements, function(element){
				if(element.name == ''){return};	// like "continue"
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined"){
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if(typeof(myObj) == "undefined"){
					return;		// like "continue"
				}
				if(typeof(myObj[name]) == "undefined" && this.ignoreNullValues){
					return;		// like "continue"
				}

				// TODO: widget values (just call attr('value', ...) on the widget)

				// TODO: maybe should call dojo.getNodeProp() instead
				switch(element.type){
					case "checkbox":
						element.checked = (name in myObj) &&
							dojo.some(myObj[name], function(val){ return val == element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name] == element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						dojo.forEach(element.options, function(option){
							option.selected = dojo.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						dojo.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
	  		});
	  		*/
		},

		getValues: function(){
			dojo.deprecated(this.declaredClass+"::getValues() is deprecated. Use get('value') instead.", "", "2.0");
			return this.get('value');
		},
		_getValueAttr: function(){
			// summary:
			// 		Returns Object representing form values.
			// description:
			//		Returns name/value hash for each form element.
			//		If there are multiple elements w/the same name, value is an array,
			//		unless they are radio buttons in which case value is a scalar since only
			//		one can be checked at a time.
			//
			//		If the name is a dot separated list (like a.b.c.d), creates a nested structure.
			//		Only works on widget form elements.
			// example:
			//		| { name: "John Smith", interests: ["sports", "movies"] }

			// get widget values
			var obj = { };
			dojo.forEach(this.getDescendants(), function(widget){
				var name = widget.name;
				if(!name || widget.disabled){ return; }

				// Single value widget (checkbox, radio, or plain <input> type widget
				var value = widget.get('value');

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(typeof widget.checked == 'boolean'){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(value !== false){
							dojo.setObject(name, value, obj);
						}else{
							// give radio widgets a default of null
							value = dojo.getObject(name, false, obj);
							if(value === undefined){
								dojo.setObject(name, null, obj);
							}
						}
					}else{
						// checkbox/toggle button
						var ary=dojo.getObject(name, false, obj);
						if(!ary){
							ary=[];
							dojo.setObject(name, ary, obj);
						}
						if(value !== false){
							ary.push(value);
						}
					}
				}else{
					var prev=dojo.getObject(name, false, obj);
					if(typeof prev != "undefined"){
						if(dojo.isArray(prev)){
							prev.push(value);
						}else{
							dojo.setObject(name, [prev, value], obj);
						}
					}else{
						// unique name
						dojo.setObject(name, value, obj);
					}
				}
			});

			/***
			 * code for plain input boxes (see also dojo.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			dojo.forEach(this.containerNode.elements, function(elm){
				if(!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
					} else if(typeof(myObj[nameA[0]]) == "undefined"){
						myObj[nameA[0]] = { }
					} // if

					if(nameA.length == 1){
						myObj=myObj[nameA[0]];
					} else{
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type == "radio" && elm.checked)){
					if(name == name.split("[")[0]){
						myObj[name]=elm.value;
					} else{
						// can not set value when there is no name
					}
				} else if(elm.type == "checkbox" && elm.checked){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				} else if(elm.type == "select-multiple"){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					for(var jdx=0,len3=elm.options.length; jdx<len3; ++jdx){
						if(elm.options[jdx].selected){
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

		// TODO: ComboBox might need time to process a recently input value.  This should be async?
	 	isValid: function(){
	 		// summary:
	 		//		Returns true if all of the widgets are valid

	 		// This also populate this._invalidWidgets[] array with list of invalid widgets...
	 		// TODO: put that into separate function?   It's confusing to have that as a side effect
	 		// of a method named isValid().

			this._invalidWidgets = dojo.filter(this.getDescendants(), function(widget){
				return !widget.disabled && widget.isValid && !widget.isValid();
	 		});
			return !this._invalidWidgets.length;
		},


		onValidStateChange: function(isValid){
			// summary:
			//		Stub function to connect to if you want to do something
			//		(like disable/enable a submit button) when the valid
			//		state changes on the form as a whole.
		},

		_widgetChange: function(widget){
			// summary:
			//		Connected to a widget's onChange function - update our
			//		valid state, if needed.
			var isValid = this._lastValidState;
			if(!widget || this._lastValidState === undefined){
				// We have passed a null widget, or we haven't been validated
				// yet - let's re-check all our children
				// This happens when we connect (or reconnect) our children
				isValid = this.isValid();
				if(this._lastValidState === undefined){
					// Set this so that we don't fire an onValidStateChange
					// the first time
					this._lastValidState = isValid;
				}
			}else if(widget.isValid){
				this._invalidWidgets = dojo.filter(this._invalidWidgets || [], function(w){
					return (w != widget);
				}, this);
				if(!widget.isValid() && !widget.get("disabled")){
					this._invalidWidgets.push(widget);
				}
				isValid = (this._invalidWidgets.length === 0);
			}
			if(isValid !== this._lastValidState){
				this._lastValidState = isValid;
				this.onValidStateChange(isValid);
			}
		},

		connectChildren: function(){
			// summary:
			//		Connects to the onChange function of all children to
			//		track valid state changes.  You can call this function
			//		directly, ex. in the event that you programmatically
			//		add a widget to the form *after* the form has been
			//		initialized.
			dojo.forEach(this._changeConnections, dojo.hitch(this, "disconnect"));
			var _this = this;

			// we connect to validate - so that it better reflects the states
			// of the widgets - also, we only connect if it has a validate
			// function (to avoid too many unneeded connections)
			var conns = (this._changeConnections = []);
			dojo.forEach(dojo.filter(this.getDescendants(),
				function(item){ return item.validate; }
			),
			function(widget){
				// We are interested in whenever the widget is validated - or
				// whenever the disabled attribute on that widget is changed
				conns.push(_this.connect(widget, "validate",
									dojo.hitch(_this, "_widgetChange", widget)));
				conns.push(_this.connect(widget, "_setDisabledAttr",
									dojo.hitch(_this, "_widgetChange", widget)));
			});

			// Call the widget change function to update the valid state, in
			// case something is different now.
			this._widgetChange(null);
		},

		startup: function(){
			this.inherited(arguments);
			// Initialize our valid state tracking.  Needs to be done in startup
			// because it's not guaranteed that our children are initialized
			// yet.
			this._changeConnections = [];
			this.connectChildren();
		}
	});

}

if(!dojo._hasResource["dijit._DialogMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._DialogMixin"] = true;
dojo.provide("dijit._DialogMixin");



dojo.declare("dijit._DialogMixin", null,
	{
		// summary:
		//		This provides functions useful to Dialog and TooltipDialog

		attributeMap: dijit._Widget.prototype.attributeMap,

		execute: function(/*Object*/ formContents){
			// summary:
			//		Callback when the user hits the submit button.
			//		Override this method to handle Dialog execution.
			// description:
			//		After the user has pressed the submit button, the Dialog
			//		first calls onExecute() to notify the container to hide the
			//		dialog and restore focus to wherever it used to be.
			//
			//		*Then* this method is called.
			// type:
			//		callback
		},

		onCancel: function(){
			// summary:
			//	    Called when user has pressed the Dialog's cancel button, to notify container.
			// description:
			//	    Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit.form.DropDownButton`)
			// type:
			//		protected
		},

		onExecute: function(){
			// summary:
			//	    Called when user has pressed the dialog's OK button, to notify container.
			// description:
			//	    Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit.form.DropDownButton`)
			// type:
			//		protected
		},

		_onSubmit: function(){
			// summary:
			//		Callback when user hits submit button
			// type:
			//		protected
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.get('value'));
		},

		_getFocusItems: function(/*Node*/ dialogNode){
			// summary:
			//		Find focusable Items each time a dialog is opened,
			//		setting _firstFocusItem and _lastFocusItem
			// tags:
			//		protected

			var elems = dijit._getTabNavigable(dojo.byId(dialogNode));
			this._firstFocusItem = elems.lowest || elems.first || dialogNode;
			this._lastFocusItem = elems.last || elems.highest || this._firstFocusItem;
			if(dojo.isMoz && this._firstFocusItem.tagName.toLowerCase() == "input" &&
					dojo.getNodeProp(this._firstFocusItem, "type").toLowerCase() == "file"){
				// FF doesn't behave well when first element is input type=file, set first focusable to dialog container
				dojo.attr(dialogNode, "tabIndex", "0");
				this._firstFocusItem = dialogNode;
			}
		}
	}
);

}

if(!dojo._hasResource["dijit.DialogUnderlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.DialogUnderlay"] = true;
dojo.provide("dijit.DialogUnderlay");






dojo.declare(
	"dijit.DialogUnderlay",
	[dijit._Widget, dijit._Templated],
	{
		// summary:
		//		The component that blocks the screen behind a `dijit.Dialog`
		//
		// description:
		// 		A component used to block input behind a `dijit.Dialog`. Only a single
		//		instance of this widget is created by `dijit.Dialog`, and saved as
		//		a reference to be shared between all Dialogs as `dijit._underlay`
		//
		//		The underlay itself can be styled based on and id:
		//	|	#myDialog_underlay { background-color:red; }
		//
		//		In the case of `dijit.Dialog`, this id is based on the id of the Dialog,
		//		suffixed with _underlay.

		// Template has two divs; outer div is used for fade-in/fade-out, and also to hold background iframe.
		// Inner div has opacity specified in CSS file.
		templateString: "<div class='dijitDialogUnderlayWrapper'><div class='dijitDialogUnderlay' dojoAttachPoint='node'></div></div>",

		// Parameters on creation or updatable later

		// dialogId: String
		//		Id of the dialog.... DialogUnderlay's id is based on this id
		dialogId: "",

		// class: String
		//		This class name is used on the DialogUnderlay node, in addition to dijitDialogUnderlay
		"class": "",

		attributeMap: { id: "domNode" },

		_setDialogIdAttr: function(id){
			dojo.attr(this.node, "id", id + "_underlay");
		},

		_setClassAttr: function(clazz){
			this.node.className = "dijitDialogUnderlay " + clazz;
		},

		postCreate: function(){
			// summary:
			//		Append the underlay to the body
			dojo.body().appendChild(this.domNode);
		},

		layout: function(){
			// summary:
			//		Sets the background to the size of the viewport
			//
			// description:
			//		Sets the background to the size of the viewport (rather than the size
			//		of the document) since we need to cover the whole browser window, even
			//		if the document is only a few lines long.
			// tags:
			//		private

			var is = this.node.style,
				os = this.domNode.style;

			// hide the background temporarily, so that the background itself isn't
			// causing scrollbars to appear (might happen when user shrinks browser
			// window and then we are called to resize)
			os.display = "none";

			// then resize and show
			var viewport = dojo.window.getBox();
			os.top = viewport.t + "px";
			os.left = viewport.l + "px";
			is.width = viewport.w + "px";
			is.height = viewport.h + "px";
			os.display = "block";
		},

		show: function(){
			// summary:
			//		Show the dialog underlay
			this.domNode.style.display = "block";
			this.layout();
			this.bgIframe = new dijit.BackgroundIframe(this.domNode);
		},

		hide: function(){
			// summary:
			//		Hides the dialog underlay
			this.bgIframe.destroy();
			delete this.bgIframe;
			this.domNode.style.display = "none";
		}
	}
);

}

if(!dojo._hasResource["dijit.TooltipDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.TooltipDialog"] = true;
dojo.provide("dijit.TooltipDialog");






dojo.declare(
		"dijit.TooltipDialog",
		[dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin, dijit._DialogMixin],
		{
			// summary:
			//		Pops up a dialog that appears like a Tooltip

			// title: String
			// 		Description of tooltip dialog (required for a11y)
			title: "",

			// doLayout: [protected] Boolean
			//		Don't change this parameter from the default value.
			//		This ContentPane parameter doesn't make sense for TooltipDialog, since TooltipDialog
			//		is never a child of a layout container, nor can you specify the size of
			//		TooltipDialog in order to control the size of an inner widget.
			doLayout: false,

			// autofocus: Boolean
			// 		A Toggle to modify the default focus behavior of a Dialog, which
			// 		is to focus on the first dialog element after opening the dialog.
			//		False will disable autofocusing. Default: true
			autofocus: true,

			// baseClass: [protected] String
			//		The root className to use for the various states of this widget
			baseClass: "dijitTooltipDialog",

			// _firstFocusItem: [private] [readonly] DomNode
			//		The pointer to the first focusable node in the dialog.
			//		Set by `dijit._DialogMixin._getFocusItems`.
			_firstFocusItem: null,

			// _lastFocusItem: [private] [readonly] DomNode
			//		The pointer to which node has focus prior to our dialog.
			//		Set by `dijit._DialogMixin._getFocusItems`.
			_lastFocusItem: null,

			templateString: dojo.cache("dijit", "templates/TooltipDialog.html", "<div waiRole=\"presentation\">\r\n\t<div class=\"dijitTooltipContainer\" waiRole=\"presentation\">\r\n\t\t<div class =\"dijitTooltipContents dijitTooltipFocusNode\" dojoAttachPoint=\"containerNode\" tabindex=\"-1\" waiRole=\"dialog\"></div>\r\n\t</div>\r\n\t<div class=\"dijitTooltipConnector\" waiRole=\"presentation\"></div>\r\n</div>\r\n"),

			postCreate: function(){
				this.inherited(arguments);
				this.connect(this.containerNode, "onkeypress", "_onKey");
				this.containerNode.title = this.title;
			},

			orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ corner){
				// summary:
				//		Configure widget to be displayed in given position relative to the button.
				//		This is called from the dijit.popup code, and should not be called
				//		directly.
				// tags:
				//		protected
				var c = this._currentOrientClass;
				if(c){
					dojo.removeClass(this.domNode, c);
				}
				c = "dijitTooltipAB"+(corner.charAt(1) == 'L'?"Left":"Right")+" dijitTooltip"+(corner.charAt(0) == 'T' ? "Below" : "Above");
				dojo.addClass(this.domNode, c);
				this._currentOrientClass = c;
			},

			focus: function(){
				// summary:
				//		Focus on first field
				this._getFocusItems(this.containerNode);
				dijit.focus(this._firstFocusItem);
			},

			onOpen: function(/*Object*/ pos){
				// summary:
				//		Called when dialog is displayed.
				//		This is called from the dijit.popup code, and should not be called directly.
				// tags:
				//		protected

				this.orient(this.domNode,pos.aroundCorner, pos.corner);
				this._onShow(); // lazy load trigger
			},

			onClose: function(){
				// summary:
				//		Called when dialog is hidden.
				//		This is called from the dijit.popup code, and should not be called directly.
				// tags:
				//		protected
				this.onHide();
			},

			_onKey: function(/*Event*/ evt){
				// summary:
				//		Handler for keyboard events
				// description:
				//		Keep keyboard focus in dialog; close dialog on escape key
				// tags:
				//		private

				var node = evt.target;
				var dk = dojo.keys;
				if(evt.charOrCode === dk.TAB){
					this._getFocusItems(this.containerNode);
				}
				var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
				if(evt.charOrCode == dk.ESCAPE){
					// Use setTimeout to avoid crash on IE, see #10396.
					setTimeout(dojo.hitch(this, "onCancel"), 0);
					dojo.stopEvent(evt);
				}else if(node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === dk.TAB){
					if(!singleFocusItem){
						dijit.focus(this._lastFocusItem); // send focus to last item in dialog
					}
					dojo.stopEvent(evt);
				}else if(node == this._lastFocusItem && evt.charOrCode === dk.TAB && !evt.shiftKey){
					if(!singleFocusItem){
						dijit.focus(this._firstFocusItem); // send focus to first item in dialog
					}
					dojo.stopEvent(evt);
				}else if(evt.charOrCode === dk.TAB){
					// we want the browser's default tab handling to move focus
					// but we don't want the tab to propagate upwards
					evt.stopPropagation();
				}
			}
		}
	);

}

if(!dojo._hasResource["dijit.Dialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Dialog"] = true;
dojo.provide("dijit.Dialog");















/*=====
dijit._underlay = function(kwArgs){
	// summary:
	//		A shared instance of a `dijit.DialogUnderlay`
	//
	// description:
	//		A shared instance of a `dijit.DialogUnderlay` created and
	//		used by `dijit.Dialog`, though never created until some Dialog
	//		or subclass thereof is shown.
};
=====*/

dojo.declare(
	"dijit._DialogBase",
	[dijit._Templated, dijit.form._FormMixin, dijit._DialogMixin, dijit._CssStateMixin],
	{
		// summary:
		//		A modal dialog Widget
		//
		// description:
		//		Pops up a modal dialog window, blocking access to the screen
		//		and also graying out the screen Dialog is extended from
		//		ContentPane so it supports all the same parameters (href, etc.)
		//
		// example:
		// |	<div dojoType="dijit.Dialog" href="test.html"></div>
		//
		// example:
		// |	var foo = new dijit.Dialog({ title: "test dialog", content: "test content" };
		// |	dojo.body().appendChild(foo.domNode);
		// |	foo.startup();

		templateString: dojo.cache("dijit", "templates/Dialog.html", "<div class=\"dijitDialog\" tabindex=\"-1\" waiRole=\"dialog\" waiState=\"labelledby-${id}_title\">\r\n\t<div dojoAttachPoint=\"titleBar\" class=\"dijitDialogTitleBar\">\r\n\t<span dojoAttachPoint=\"titleNode\" class=\"dijitDialogTitle\" id=\"${id}_title\"></span>\r\n\t<span dojoAttachPoint=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" dojoAttachEvent=\"onclick: onCancel\" title=\"${buttonCancel}\">\r\n\t\t<span dojoAttachPoint=\"closeText\" class=\"closeText\" title=\"${buttonCancel}\">x</span>\r\n\t</span>\r\n\t</div>\r\n\t\t<div dojoAttachPoint=\"containerNode\" class=\"dijitDialogPaneContent\"></div>\r\n</div>\r\n"),
		
		baseClass: "dijitDialog",
		
		cssStateNodes: {
			closeButtonNode: "dijitDialogCloseIcon"
		},

		attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
			title: [
				{ node: "titleNode", type: "innerHTML" },
				{ node: "titleBar", type: "attribute" }
			],
			"aria-describedby":""
		}),

		// open: Boolean
		//		True if Dialog is currently displayed on screen.
		open: false,

		// duration: Integer
		//		The time in milliseconds it takes the dialog to fade in and out
		duration: dijit.defaultDuration,

		// refocus: Boolean
		// 		A Toggle to modify the default focus behavior of a Dialog, which
		// 		is to re-focus the element which had focus before being opened.
		//		False will disable refocusing. Default: true
		refocus: true,

		// autofocus: Boolean
		// 		A Toggle to modify the default focus behavior of a Dialog, which
		// 		is to focus on the first dialog element after opening the dialog.
		//		False will disable autofocusing. Default: true
		autofocus: true,

		// _firstFocusItem: [private] [readonly] DomNode
		//		The pointer to the first focusable node in the dialog.
		//		Set by `dijit._DialogMixin._getFocusItems`.
		_firstFocusItem: null,

		// _lastFocusItem: [private] [readonly] DomNode
		//		The pointer to which node has focus prior to our dialog.
		//		Set by `dijit._DialogMixin._getFocusItems`.
		_lastFocusItem: null,

		// doLayout: [protected] Boolean
		//		Don't change this parameter from the default value.
		//		This ContentPane parameter doesn't make sense for Dialog, since Dialog
		//		is never a child of a layout container, nor can you specify the size of
		//		Dialog in order to control the size of an inner widget.
		doLayout: false,

		// draggable: Boolean
		//		Toggles the moveable aspect of the Dialog. If true, Dialog
		//		can be dragged by it's title. If false it will remain centered
		//		in the viewport.
		draggable: true,

		//aria-describedby: String
		//		Allows the user to add an aria-describedby attribute onto the dialog.   The value should
		//		be the id of the container element of text that describes the dialog purpose (usually
		//		the first text in the dialog).
		//		<div dojoType="dijit.Dialog" aria-describedby="intro" .....>
		//			<div id="intro">Introductory text</div>
		//			<div>rest of dialog contents</div>
		//		</div>
		"aria-describedby":"",

		postMixInProperties: function(){
			var _nlsResources = dojo.i18n.getLocalization("dijit", "common");
			dojo.mixin(this, _nlsResources);
			this.inherited(arguments);
		},

		postCreate: function(){
			dojo.style(this.domNode, {
				display: "none",
				position:"absolute"
			});
			dojo.body().appendChild(this.domNode);

			this.inherited(arguments);

			this.connect(this, "onExecute", "hide");
			this.connect(this, "onCancel", "hide");
			this._modalconnects = [];
		},

		onLoad: function(){
			// summary:
			//		Called when data has been loaded from an href.
			//		Unlike most other callbacks, this function can be connected to (via `dojo.connect`)
			//		but should *not* be overriden.
			// tags:
			//		callback

			// when href is specified we need to reposition the dialog after the data is loaded
			// and find the focusable elements
			this._position();
			if(this.autofocus){
				this._getFocusItems(this.domNode);
				dijit.focus(this._firstFocusItem);
			}
			this.inherited(arguments);
		},

		_endDrag: function(e){
			// summary:
			//		Called after dragging the Dialog. Saves the position of the dialog in the viewport.
			// tags:
			//		private
			if(e && e.node && e.node === this.domNode){
				this._relativePosition = dojo.position(e.node);
			}
		},

		_setup: function(){
			// summary:
			//		Stuff we need to do before showing the Dialog for the first
			//		time (but we defer it until right beforehand, for
			//		performance reasons).
			// tags:
			//		private

			var node = this.domNode;

			if(this.titleBar && this.draggable){
				this._moveable = (dojo.isIE == 6) ?
					new dojo.dnd.TimedMoveable(node, { handle: this.titleBar }) :	// prevent overload, see #5285
					new dojo.dnd.Moveable(node, { handle: this.titleBar, timeout: 0 });
				this._dndListener = dojo.subscribe("/dnd/move/stop",this,"_endDrag");
			}else{
				dojo.addClass(node,"dijitDialogFixed");
			}

			this.underlayAttrs = {
				dialogId: this.id,
				"class": dojo.map(this["class"].split(/\s/), function(s){ return s+"_underlay"; }).join(" ")
			};

			this._fadeIn = dojo.fadeIn({
				node: node,
				duration: this.duration,
				beforeBegin: dojo.hitch(this, function(){
					var underlay = dijit._underlay;
					if(!underlay){
						underlay = dijit._underlay = new dijit.DialogUnderlay(this.underlayAttrs);
					}else{
						underlay.set(this.underlayAttrs);
					}

					var ds = dijit._dialogStack,
						zIndex = 948 + ds.length*2;
					if(ds.length == 1){	// first dialog
						underlay.show();
					}
					dojo.style(dijit._underlay.domNode, 'zIndex', zIndex);
					dojo.style(this.domNode, 'zIndex', zIndex + 1);
				}),
				onEnd: dojo.hitch(this, function(){
					if(this.autofocus){
						// find focusable Items each time dialog is shown since if dialog contains a widget the
						// first focusable items can change
						this._getFocusItems(this.domNode);
						dijit.focus(this._firstFocusItem);
					}
				})
			 });

			this._fadeOut = dojo.fadeOut({
				node: node,
				duration: this.duration,
				onEnd: dojo.hitch(this, function(){
					node.style.display = "none";

					// Restore the previous dialog in the stack, or if this is the only dialog
					// then restore to original page
					var ds = dijit._dialogStack;
					if(ds.length == 0){
						dijit._underlay.hide();
					}else{
						dojo.style(dijit._underlay.domNode, 'zIndex', 948 + ds.length*2);
						dijit._underlay.set(ds[ds.length-1].underlayAttrs);
					}

					// Restore focus to wherever it was before this dialog was displayed
					if(this.refocus){
						var focus = this._savedFocus;

						// If we are returning control to a previous dialog but for some reason
						// that dialog didn't have a focused field, set focus to first focusable item.
						// This situation could happen if two dialogs appeared at nearly the same time,
						// since a dialog doesn't set it's focus until the fade-in is finished.
						if(ds.length > 0){
							var pd = ds[ds.length-1];
							if(!dojo.isDescendant(focus.node, pd.domNode)){
								pd._getFocusItems(pd.domNode);
								focus = pd._firstFocusItem;
							}
						}

						dijit.focus(focus);
					}
				})
			 });
		},

		uninitialize: function(){
			var wasPlaying = false;
			if(this._fadeIn && this._fadeIn.status() == "playing"){
				wasPlaying = true;
				this._fadeIn.stop();
			}
			if(this._fadeOut && this._fadeOut.status() == "playing"){
				wasPlaying = true;
				this._fadeOut.stop();
			}
			
			// Hide the underlay, unless the underlay widget has already been destroyed
			// because we are being called during page unload (when all widgets are destroyed)
			if((this.open || wasPlaying) && !dijit._underlay._destroyed){
				dijit._underlay.hide();
			}

			if(this._moveable){
				this._moveable.destroy();
			}
			this.inherited(arguments);
		},

		_size: function(){
			// summary:
			// 		If necessary, shrink dialog contents so dialog fits in viewport
			// tags:
			//		private

			this._checkIfSingleChild();

			// If we resized the dialog contents earlier, reset them back to original size, so
			// that if the user later increases the viewport size, the dialog can display w/out a scrollbar.
			// Need to do this before the dojo.marginBox(this.domNode) call below.
			if(this._singleChild){
				if(this._singleChildOriginalStyle){
					this._singleChild.domNode.style.cssText = this._singleChildOriginalStyle;
				}
				delete this._singleChildOriginalStyle;
			}else{
				dojo.style(this.containerNode, {
					width:"auto",
					height:"auto"
				});
			}

			var mb = dojo.marginBox(this.domNode);
			var viewport = dojo.window.getBox();
			if(mb.w >= viewport.w || mb.h >= viewport.h){
				// Reduce size of dialog contents so that dialog fits in viewport

				var w = Math.min(mb.w, Math.floor(viewport.w * 0.75)),
					h = Math.min(mb.h, Math.floor(viewport.h * 0.75));

				if(this._singleChild && this._singleChild.resize){
					this._singleChildOriginalStyle = this._singleChild.domNode.style.cssText;
					this._singleChild.resize({w: w, h: h});
				}else{
					dojo.style(this.containerNode, {
						width: w + "px",
						height: h + "px",
						overflow: "auto",
						position: "relative"	// workaround IE bug moving scrollbar or dragging dialog
					});
				}
			}else{
				if(this._singleChild && this._singleChild.resize){
					this._singleChild.resize();
				}
			}
		},

		_position: function(){
			// summary:
			//		Position modal dialog in the viewport. If no relative offset
			//		in the viewport has been determined (by dragging, for instance),
			//		center the node. Otherwise, use the Dialog's stored relative offset,
			//		and position the node to top: left: values based on the viewport.
			// tags:
			//		private
			if(!dojo.hasClass(dojo.body(),"dojoMove")){
				var node = this.domNode,
					viewport = dojo.window.getBox(),
					p = this._relativePosition,
					bb = p ? null : dojo._getBorderBox(node),
					l = Math.floor(viewport.l + (p ? p.x : (viewport.w - bb.w) / 2)),
					t = Math.floor(viewport.t + (p ? p.y : (viewport.h - bb.h) / 2))
				;
				dojo.style(node,{
					left: l + "px",
					top: t + "px"
				});
			}
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handles the keyboard events for accessibility reasons
			// tags:
			//		private

			var ds = dijit._dialogStack;
			if(ds[ds.length-1] != this){
				// console.debug(this.id + ': skipping because', this, 'is not the active dialog');
				return;
			}

			if(evt.charOrCode){
				var dk = dojo.keys;
				var node = evt.target;
				if(evt.charOrCode === dk.TAB){
					this._getFocusItems(this.domNode);
				}
				var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
				// see if we are shift-tabbing from first focusable item on dialog
				if(node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === dk.TAB){
					if(!singleFocusItem){
						dijit.focus(this._lastFocusItem); // send focus to last item in dialog
					}
					dojo.stopEvent(evt);
				}else if(node == this._lastFocusItem && evt.charOrCode === dk.TAB && !evt.shiftKey){
					if(!singleFocusItem){
						dijit.focus(this._firstFocusItem); // send focus to first item in dialog
					}
					dojo.stopEvent(evt);
				}else{
					// see if the key is for the dialog
					while(node){
						if(node == this.domNode || dojo.hasClass(node, "dijitPopup")){
							if(evt.charOrCode == dk.ESCAPE){
								this.onCancel();
							}else{
								return; // just let it go
							}
						}
						node = node.parentNode;
					}
					// this key is for the disabled document window
					if(evt.charOrCode !== dk.TAB){ // allow tabbing into the dialog for a11y
						dojo.stopEvent(evt);
					// opera won't tab to a div
					}else if(!dojo.isOpera){
						try{
							this._firstFocusItem.focus();
						}catch(e){ /*squelch*/ }
					}
				}
			}
		},

		show: function(){
			// summary:
			//		Display the dialog
			if(this.open){ return; }

			// first time we show the dialog, there's some initialization stuff to do
			if(!this._alreadyInitialized){
				this._setup();
				this._alreadyInitialized=true;
			}

			if(this._fadeOut.status() == "playing"){
				this._fadeOut.stop();
			}

			this._modalconnects.push(dojo.connect(window, "onscroll", this, "layout"));
			this._modalconnects.push(dojo.connect(window, "onresize", this, function(){
				// IE gives spurious resize events and can actually get stuck
				// in an infinite loop if we don't ignore them
				var viewport = dojo.window.getBox();
				if(!this._oldViewport ||
						viewport.h != this._oldViewport.h ||
						viewport.w != this._oldViewport.w){
					this.layout();
					this._oldViewport = viewport;
				}
			}));
			this._modalconnects.push(dojo.connect(dojo.doc.documentElement, "onkeypress", this, "_onKey"));

			dojo.style(this.domNode, {
				opacity:0,
				display:""
			});

			this.open = true;
			this._onShow(); // lazy load trigger

			this._size();
			this._position();
			dijit._dialogStack.push(this);
			this._fadeIn.play();

			this._savedFocus = dijit.getFocus(this);
		},

		hide: function(){
			// summary:
			//		Hide the dialog

			// if we haven't been initialized yet then we aren't showing and we can just return
			// or if we aren't the active dialog, don't allow us to close yet
			var ds = dijit._dialogStack;
			if(!this._alreadyInitialized || this != ds[ds.length-1]){
				return;
			}

			if(this._fadeIn.status() == "playing"){
				this._fadeIn.stop();
			}

			// throw away current active dialog from stack -- making the previous dialog or the node on the original page active
			ds.pop();

			this._fadeOut.play();

			if(this._scrollConnected){
				this._scrollConnected = false;
			}
			dojo.forEach(this._modalconnects, dojo.disconnect);
			this._modalconnects = [];

			if(this._relativePosition){
				delete this._relativePosition;
			}
			this.open = false;

			this.onHide();
		},

		layout: function(){
			// summary:
			//		Position the Dialog and the underlay
			// tags:
			//		private
			if(this.domNode.style.display != "none"){
				if(dijit._underlay){	// avoid race condition during show()
					dijit._underlay.layout();
				}
				this._position();
			}
		},

		destroy: function(){
			dojo.forEach(this._modalconnects, dojo.disconnect);
			if(this.refocus && this.open){
				setTimeout(dojo.hitch(dijit,"focus",this._savedFocus), 25);
			}
			if(this._dndListener){
				dojo.unsubscribe(this._dndListener);
			}
			this.inherited(arguments);
		}
	}
);

dojo.declare(
	"dijit.Dialog",
	[dijit.layout.ContentPane, dijit._DialogBase],
	{}
);

// Stack of currenctly displayed dialogs, layered on top of each other
dijit._dialogStack = [];

// For back-compat.  TODO: remove in 2.0


}

if(!dojo._hasResource["dijit.form.ToggleButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ToggleButton"] = true;
dojo.provide("dijit.form.ToggleButton");


}

if(!dojo._hasResource["dijit.form.CheckBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.CheckBox"] = true;
dojo.provide("dijit.form.CheckBox");



dojo.declare(
	"dijit.form.CheckBox",
	dijit.form.ToggleButton,
	{
		// summary:
		// 		Same as an HTML checkbox, but with fancy styling.
		//
		// description:
		//		User interacts with real html inputs.
		//		On onclick (which occurs by mouse click, space-bar, or
		//		using the arrow keys to switch the selected radio button),
		//		we update the state of the checkbox/radio.
		//
		//		There are two modes:
		//			1. High contrast mode
		//			2. Normal mode
		//
		//		In case 1, the regular html inputs are shown and used by the user.
		//		In case 2, the regular html inputs are invisible but still used by
		//		the user. They are turned quasi-invisible and overlay the background-image.

		templateString: dojo.cache("dijit.form", "templates/CheckBox.html", "<div class=\"dijit dijitReset dijitInline\" waiRole=\"presentation\"\r\n\t><input\r\n\t \t${!nameAttrSetting} type=\"${type}\" ${checkedAttrSetting}\r\n\t\tclass=\"dijitReset dijitCheckBoxInput\"\r\n\t\tdojoAttachPoint=\"focusNode\"\r\n\t \tdojoAttachEvent=\"onclick:_onClick\"\r\n/></div>\r\n"),

		baseClass: "dijitCheckBox",

		// type: [private] String
		//		type attribute on <input> node.
		//		Overrides `dijit.form.Button.type`.   Users should not change this value.
		type: "checkbox",

		// value: String
		//		As an initialization parameter, equivalent to value field on normal checkbox
		//		(if checked, the value is passed as the value when form is submitted).
		//
		//		However, attr('value') will return either the string or false depending on
		//		whether or not the checkbox is checked.
		//
		//		attr('value', string) will check the checkbox and change the value to the
		//		specified string
		//
		//		attr('value', boolean) will change the checked state.
		value: "on",

		// readOnly: Boolean
		//		Should this widget respond to user input?
		//		In markup, this is specified as "readOnly".
		//		Similar to disabled except readOnly form values are submitted.
		readOnly: false,
		
		// the attributeMap should inherit from dijit.form._FormWidget.prototype.attributeMap 
		// instead of ToggleButton as the icon mapping has no meaning for a CheckBox
		attributeMap: dojo.delegate(dijit.form._FormWidget.prototype.attributeMap, {
			readOnly: "focusNode"
		}),

		_setReadOnlyAttr: function(/*Boolean*/ value){
			this.readOnly = value;
			dojo.attr(this.focusNode, 'readOnly', value);
			dijit.setWaiState(this.focusNode, "readonly", value);
		},

		_setValueAttr: function(/*String or Boolean*/ newValue, /*Boolean*/ priorityChange){
			// summary:
			//		Handler for value= attribute to constructor, and also calls to
			//		attr('value', val).
			// description:
			//		During initialization, just saves as attribute to the <input type=checkbox>.
			//
			//		After initialization,
			//		when passed a boolean, controls whether or not the CheckBox is checked.
			//		If passed a string, changes the value attribute of the CheckBox (the one
			//		specified as "value" when the CheckBox was constructed (ex: <input
			//		dojoType="dijit.CheckBox" value="chicken">)
			if(typeof newValue == "string"){
				this.value = newValue;
				dojo.attr(this.focusNode, 'value', newValue);
				newValue = true;
			}
			if(this._created){
				this.set('checked', newValue, priorityChange);
			}
		},
		_getValueAttr: function(){
			// summary:
			//		Hook so attr('value') works.
			// description:
			//		If the CheckBox is checked, returns the value attribute.
			//		Otherwise returns false.
			return (this.checked ? this.value : false);
		},

		// Override dijit.form.Button._setLabelAttr() since we don't even have a containerNode.
		// Normally users won't try to set label, except when CheckBox or RadioButton is the child of a dojox.layout.TabContainer
		_setLabelAttr: undefined,

		postMixInProperties: function(){
			if(this.value == ""){
				this.value = "on";
			}

			// Need to set initial checked state as part of template, so that form submit works.
			// dojo.attr(node, "checked", bool) doesn't work on IEuntil node has been attached
			// to <body>, see #8666
			this.checkedAttrSetting = this.checked ? "checked" : "";

			this.inherited(arguments);
		},

		 _fillContent: function(/*DomNode*/ source){
			// Override Button::_fillContent() since it doesn't make sense for CheckBox,
			// since CheckBox doesn't even have a container
		},

		reset: function(){
			// Override ToggleButton.reset()

			this._hasBeenBlurred = false;

			this.set('checked', this.params.checked || false);

			// Handle unlikely event that the <input type=checkbox> value attribute has changed
			this.value = this.params.value || "on";
			dojo.attr(this.focusNode, 'value', this.value);
		},

		_onFocus: function(){
			if(this.id){
				dojo.query("label[for='"+this.id+"']").addClass("dijitFocusedLabel");
			}
			this.inherited(arguments);
		},

		_onBlur: function(){
			if(this.id){
				dojo.query("label[for='"+this.id+"']").removeClass("dijitFocusedLabel");
			}
			this.inherited(arguments);
		},

		_onClick: function(/*Event*/ e){
			// summary:
			//		Internal function to handle click actions - need to check
			//		readOnly, since button no longer does that check.
			if(this.readOnly){
				dojo.stopEvent(e);
				return false;
			}
			return this.inherited(arguments);
		}
	}
);

dojo.declare(
	"dijit.form.RadioButton",
	dijit.form.CheckBox,
	{
		// summary:
		// 		Same as an HTML radio, but with fancy styling.

		type: "radio",
		baseClass: "dijitRadio",

		_setCheckedAttr: function(/*Boolean*/ value){
			// If I am being checked then have to deselect currently checked radio button
			this.inherited(arguments);
			if(!this._created){ return; }
			if(value){
				var _this = this;
				// search for radio buttons with the same name that need to be unchecked
				dojo.query("INPUT[type=radio]", this.focusNode.form || dojo.doc).forEach( // can't use name= since dojo.query doesn't support [] in the name
					function(inputNode){
						if(inputNode.name == _this.name && inputNode != _this.focusNode && inputNode.form == _this.focusNode.form){
							var widget = dijit.getEnclosingWidget(inputNode);
							if(widget && widget.checked){
								widget.set('checked', false);
							}
						}
					}
				);
			}
		},

		_clicked: function(/*Event*/ e){
			if(!this.checked){
				this.set('checked', true);
			}
		}
	}
);

}

if(!dojo._hasResource["dojox.layout.ResizeHandle"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.layout.ResizeHandle"] = true;
dojo.provide("dojox.layout.ResizeHandle");
dojo.experimental("dojox.layout.ResizeHandle"); 


 
 


dojo.declare("dojox.layout.ResizeHandle",
	[dijit._Widget, dijit._Templated],
	{
	// summary: A dragable handle used to resize an attached node.
	//
	// description:
	//	The handle on the bottom-right corner of FloatingPane or other widgets that allows
	//	the widget to be resized.
	//	Typically not used directly.
	//
	// targetId: String
	//	id of the Widget OR DomNode that I will size
	targetId: "",
	
	// targetContainer: DomNode
	//	over-ride targetId and attch this handle directly to a reference of a DomNode
	targetContainer: null, 
	
	// resizeAxis: String
	//	one of: x|y|xy limit resizing to a single axis, default to xy ... 
	resizeAxis: "xy",
	
	// activeResize: Boolean
	// 	if true, node will size realtime with mouse movement, 
	//	if false, node will create virtual node, and only resize target on mouseUp
	activeResize: false,
	
	// activeResizeClass: String
	//	css class applied to virtual resize node. 
	activeResizeClass: "dojoxResizeHandleClone",
	
	// animateSizing: Boolean
	//	only applicable if activeResize = false. onMouseup, animate the node to the
	//	new size
	animateSizing: true,
	
	// animateMethod: String
	// 	one of "chain" or "combine" ... visual effect only. combine will "scale" 
	// 	node to size, "chain" will alter width, then height
	animateMethod: "chain",

	// animateDuration: Integer
	//	time in MS to run sizing animation. if animateMethod="chain", total animation 
	//	playtime is 2*animateDuration
	animateDuration: 225,

	// minHeight: Integer
	//	smallest height in px resized node can be
	minHeight: 100,

	// minWidth: Integer
	//	smallest width in px resize node can be
	minWidth: 100,

	// constrainMax: Boolean
	//	Toggle if this widget cares about the maxHeight and maxWidth 
	//	parameters. 
	constrainMax: false,

	// maxHeight: Integer
	//	Largest height size in px the resize node can become. 
	maxHeight:0, 
	
	// maxWidth: Integer
	//	Largest width size in px the reize node can become.
	maxWidth:0,

	// fixedAspect: Boolean
	//		Toggle to enable this widget to maintain the aspect 
	//		ratio of the attached node. 
	fixedAspect: false,

	// intermediateChanges: Boolean
	//		Toggle to enable/disable this widget from firing onResize
	//		events at every step of a resize. If `activeResize` is true,
	//		and this is false, onResize only fires _after_ the drop 
	//		operation. Animated resizing is not affected by this setting.
	intermediateChanges: false,

	// startTopic: String
	//		The name of the topic this resizehandle publishes when resize is starting
	startTopic: "/dojo/resize/start",
	
	// endTopic: String
	//		The name of the topic this resizehandle publishes when resize is complete
	endTopic:"/dojo/resize/stop",

	templateString: '<div dojoAttachPoint="resizeHandle" class="dojoxResizeHandle"><div></div></div>',

	postCreate: function(){
		// summary: setup our one major listener upon creation
		this.connect(this.resizeHandle, "onmousedown", "_beginSizing");
		if(!this.activeResize){ 
			// there shall be only a single resize rubberbox that at the top
			// level so that we can overlay it on anything whenever the user
			// resizes something. Since there is only one mouse pointer he
			// can't at once resize multiple things interactively.
			this._resizeHelper = dijit.byId('dojoxGlobalResizeHelper');
			if(!this._resizeHelper){
				this._resizeHelper = new dojox.layout._ResizeHelper({ 
						id: 'dojoxGlobalResizeHelper'
				}).placeAt(dojo.body());
				dojo.addClass(this._resizeHelper.domNode, this.activeResizeClass);
			}
		}else{ this.animateSizing = false; } 	

		if(!this.minSize){ 
			this.minSize = { w: this.minWidth, h: this.minHeight };
		}
		
		if(this.constrainMax){
			this.maxSize = { w: this.maxWidth, h: this.maxHeight }
		}
		
		// should we modify the css for the cursor hover to n-resize nw-resize and w-resize?
		this._resizeX = this._resizeY = false;
		var addClass = dojo.partial(dojo.addClass, this.resizeHandle); 
		switch(this.resizeAxis.toLowerCase()){
			case "xy" : 
				this._resizeX = this._resizeY = true; 
				// FIXME: need logic to determine NW or NE class to see
				// based on which [todo] corner is clicked
				addClass("dojoxResizeNW"); 
				break; 
			case "x" : 
				this._resizeX = true; 
				addClass("dojoxResizeW");
				break;
			case "y" : 
				this._resizeY = true; 
				addClass("dojoxResizeN");
				break;
		}
	},

	_beginSizing: function(/*Event*/ e){
		// summary: setup movement listeners and calculate initial size
		
		if(this._isSizing){ return false; }

		dojo.publish(this.startTopic, [ this ]);
		this.targetWidget = dijit.byId(this.targetId);

		this.targetDomNode = this.targetWidget ? this.targetWidget.domNode : dojo.byId(this.targetId);
		if(this.targetContainer){ this.targetDomNode = this.targetContainer; }
		if(!this.targetDomNode){ return false; }

		if(!this.activeResize){
			var c = dojo.position(this.targetDomNode, true);
			console.log(c);
			console.log(dojo.window.getBox());
			this._resizeHelper.resize({l: c.x, t: c.y, w: c.w, h: c.h});
			this._resizeHelper.show();
		}

		this._isSizing = true;
		this.startPoint  = { x:e.clientX, y:e.clientY};

		// FIXME: this is funky: marginBox adds height, contentBox ignores padding (expected, but foo!)
		var mb = this.targetWidget ? dojo.marginBox(this.targetDomNode) : dojo.contentBox(this.targetDomNode);  
		this.startSize  = { w:mb.w, h:mb.h };
		
		if(this.fixedAspect){
			var max, val;
			if(mb.w > mb.h){
				max = "w";
				val = mb.w / mb.h
			}else{
				max = "h";
				val = mb.h / mb.w
			}
			this._aspect = { prop: max };
			this._aspect[max] = val;
		}

		this._pconnects = []; 
		this._pconnects.push(dojo.connect(dojo.doc,"onmousemove",this,"_updateSizing")); 
		this._pconnects.push(dojo.connect(dojo.doc,"onmouseup", this, "_endSizing"));
		
		dojo.stopEvent(e); 
	},

	_updateSizing: function(/*Event*/ e){
		// summary: called when moving the ResizeHandle ... determines 
		//	new size based on settings/position and sets styles.

		if(this.activeResize){
			this._changeSizing(e);
		}else{
			var tmp = this._getNewCoords(e);
			if(tmp === false){ return; }
			this._resizeHelper.resize(tmp);
		}
		e.preventDefault();
	},

	_getNewCoords: function(/* Event */ e){
		
		// On IE, if you move the mouse above/to the left of the object being resized,
		// sometimes clientX/Y aren't set, apparently.  Just ignore the event.
		try{
			if(!e.clientX  || !e.clientY){ return false; }
		}catch(e){
			// sometimes you get an exception accessing above fields...
			return false;
		}
		this._activeResizeLastEvent = e; 

		var dx = (this.isLeftToRight()? this.startPoint.x - e.clientX: e.clientX - this.startPoint.x),
			dy = this.startPoint.y - e.clientY,
			newW = this.startSize.w - (this._resizeX ? dx : 0),
			newH = this.startSize.h - (this._resizeY ? dy : 0)
		;
			
		return this._checkConstraints(newW, newH); // Object
	},
	
	_checkConstraints: function(newW, newH){
		// summary: filter through the various possible constaint possibilities.
				
		// minimum size check
		if(this.minSize){
			var tm = this.minSize;
			if(newW < tm.w){
				newW = tm.w;
			}
			if(newH < tm.h){
				newH = tm.h;
			}
		}
		
		// maximum size check:
		if(this.constrainMax && this.maxSize){
			var ms = this.maxSize;
			if(newW > ms.w){
				newW = ms.w;
			}
			if(newH > ms.h){
				newH = ms.h;
			}
		}
		
		if(this.fixedAspect){
			var ta = this._aspect[this._aspect.prop];
			if(newW < newH){
				newH = newW * ta;
			}else if(newH < newW){
				newW = newH * ta;
			}
		}
		
		return { w: newW, h: newH }; // Object
	},
		
	_changeSizing: function(/*Event*/ e){
		// summary: apply sizing information based on information in (e) to attached node
		var tmp = this._getNewCoords(e);
		if(tmp === false){ return; }

		if(this.targetWidget && dojo.isFunction(this.targetWidget.resize)){ 
			this.targetWidget.resize(tmp);
		}else{
			if(this.animateSizing){
				var anim = dojo.fx[this.animateMethod]([
					dojo.animateProperty({
						node: this.targetDomNode,
						properties: { 
							width: { start: this.startSize.w, end: tmp.w } 
						},	
						duration: this.animateDuration
					}),
					dojo.animateProperty({
						node: this.targetDomNode,
						properties: { 
							height: { start: this.startSize.h, end: tmp.h }
						},
						duration: this.animateDuration
					})
				]);
				anim.play();
			}else{
				dojo.style(this.targetDomNode,{
					width: tmp.w + "px",
					height: tmp.h + "px"
				});
			}
		}
		if(this.intermediateChanges){
			this.onResize(e);
		}	
	},

	_endSizing: function(/*Event*/ e){
		// summary: disconnect listenrs and cleanup sizing
		dojo.forEach(this._pconnects, dojo.disconnect);
		var pub = dojo.partial(dojo.publish, this.endTopic, [ this ]);
		if(!this.activeResize){
			this._resizeHelper.hide();
			this._changeSizing(e);
			setTimeout(pub, this.animateDuration + 15);
		}else{
			pub();
		}
		this._isSizing = false;
		this.onResize(e);
	},
	
	onResize: function(e){
		// summary: Stub fired when sizing is done. Fired once 
		//	after resize, or often when `intermediateChanges` is 
		//	set to true. 
	}
	
});

dojo.declare("dojox.layout._ResizeHelper",
	dijit._Widget,
	{
	// summary: A global private resize helper shared between any 
	//		`dojox.layout.ResizeHandle` with activeSizing off.
	
	show: function(){
		// summary: show helper to start resizing
		dojo.fadeIn({ 
			node: this.domNode, 
			duration: 120, 
			beforeBegin: function(n){ dojo.style(n, "display", "") }
		}).play();
	},
	
	hide: function(){
		// summary: hide helper after resizing is complete
		dojo.fadeOut({ 
			node: this.domNode, 
			duration: 250,
			onEnd: function(n){ dojo.style(n, "display", "none") }
		}).play();
	},
	
	resize: function(/* Object */dim){
		// summary: size the widget and place accordingly

		// FIXME: this is off when padding present
		dojo.marginBox(this.domNode, dim);
	}
	
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPMessageDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPMessageDialog"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007, 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
console.warn("'ibm.tivoli.tip.dijit.TIPMessageDialog' is deprecated, use 'dojoe.messagedialog.MessageDialog'(TED) instead.");
dojo.provide("ibm.tivoli.tip.dijit.TIPMessageDialog");

 





// NLS


// define the widget class
dojo.declare(
	"ibm.tivoli.tip.dijit.TIPMessageDialog", 
    [ dijit._Widget ],
{
    // resizeAxis: String
    //  x | xy | y to limit pane's sizing direction
    resizeAxis: "xy",

	// type: String: defines the type of message dialog ("Error", "Warning", "Info", "Confirm")
	type: "Error",
	
	// portlet title associated w/ this message
	portletTitle: null,

	// message: String: the displayable translated message
	message: null,

	// messageId: String: the displayable translated message's unique identifier
	messageId: null,
	
	// width: String: gives the pixel width of the dialog (Default: 500px)
	width: "380px",
	
	// height: String: gives the pixel height of the dialog (Default: 200px)
	height: "225px",
	
	// buttons: String Array: a list of buttons to display (Default: OK)
	buttons: [],
	
	// callback: Function: a callback function when the dialog is closed
	callback: null,
	
	// parameters to pass on callback when dialog closes
	callbackParms: null,
	
	// url: String: a URL that links the message to an InfoCenter page
	url: null,
	
	// show "Never show again checkbox"
	showCheckbox: false,
	
	// contentIsolation: boolean parameter to open PopupUtil in content isolation mode in portlets
	contentIsolation: false,
	
	// Destroy the dialog on close
	destroyOnClose: true,
	
	// namespace: user defined namespace for widget
	namespace :"",

	constructor: function() {
		// summary: An extension to the Dojo Dialog widget that supports 
		//      advanced features.
		// resourceMessages_: JSON
		//		Resource bundle.
		// dialog_: dojo.widget.FloatingPane
		//		An instance of the Dialog widget.
		// title_: String
		//		Title bar string.
		// icon_: String
		//		The dialog icon's uri.
		// buttonPressedId_: String
		//		The ID of the button pressed; this value is passed to the callback function.

	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
	    
	    this.dialog_ = null;

	    this.title_ = null;
	    this.icon_ = null;
	    this.buttonPressedId_ = -1;
	},


	postCreate: function() { 
	    if (this.buttons.length === 0) {
	        this.buttons.push(this.resources_.DIALOG_BUTTON_OK);
	    }

		var className = null;
	    if (this.type.toLowerCase() == "error") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_ERROR, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_ERROR;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_critical.png");
	        className = "critical";
	    }
	    else if (this.type.toLowerCase() == "warning") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_WARNING, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_WARNING;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_warning.png");
	        className = "warning";
	    }
	    else if (this.type.toLowerCase() == "confirm") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_CONFIRM, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_CONFIRM;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_confirm.png");
	        className = "confirm";
	    }
	    else if (this.type.toLowerCase() == "validation") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_VALIDATION, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_VALIDATION;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_validation.png");
	        className = "validation";
	    }
	    else { // if (this.type.toLowerCase() == "info") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_INFO, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_INFO;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_info.png");
	        className = "info";
	    }
	    
		var div = document.createElement('div'); 
//		div.style.width = this.width; 
//		div.style.height = this.height;
        // The following div settings are needed to ensure the dialog is seen when the bottom frames
        // are rendered due to refresh or re-submit during error scenarios (see main_ruleset.jsp).
        // Without the settings, the bottom frame was overlaying the error message. 
		//div.style.top = 0; 
		//div.style.left = 0;
		document.body.appendChild(div);

        this.dialog_ = new dijit.Dialog (
			                             { title: this.title_
		                                 },
		                                 div); 

		var dojoAttachpoint = this.dialog_.containerNode;
		this.dialog_.closeButtonNode.setAttribute("title", this.resources_.DIALOG_BUTTON_CLOSE);
		this.dialog_.closeButtonNode.setAttribute("tabIndex", 0);
		dojo.connect(this.dialog_.closeButtonNode,'onkeypress',this,'onKeyPress');
		dojoAttachpoint.innerHTML = this.getContent(className);

        //Build and place resize handle       
        var resizeHandleDOM = dojo.doc.createElement( 'span' );
        resizeHandleDOM.className = "dojoxFloatingResizeHandle";    
        dojo.place( resizeHandleDOM, this.dialog_.domNode, "last" );
    
        this.resizeHandle = new dojox.layout.ResizeHandle(
        { 
              targetId: this.dialog_.id, 
            resizeAxis: this.resizeAxis 
        }, resizeHandleDOM );   
	
        this.dialog_.domNode.style.width = this.width;	
        this.dialog_.domNode.style.height = this.height;	
        //this.dialog_.containerNode.style.overflow="auto";
        this.dialog_.domNode.style.overflow = "hidden";        
    
		for (var i = 0; i < this.buttons.length; i++) {
			var button = dojo.byId(this.id + "mmd_button_" + i);
			dojo.connect(button, 'onclick', this, 'onClick');    
		}
    
        this.onCloseListener  = dojo.connect( this.dialog_, "hide", this, "_onHide" );
	  this.onResizeListener = dojo.connect( this.dialog_, "resize", this, "customLayout" );
	},
	
    /**************************************************************************
    * The dialog has been hidden.  Wait 500 miliseconds to destroy the dialog.
    *
    * HACK -- Can't figure out a better way to destroy the dialog on close.
    **************************************************************************/
    _onHide:function()
    {
      setTimeout( dojo.hitch( this, "destroy" ), 500 );
    },  
	
	onClick: function(/* HTMLEvent */e) {
		var index = this.id.length + "mmd_button_".length;
		this.buttonPressedId_ = parseInt(e.target.id.substring(index), 0);
	    console.log ("TIPMessageDialog.onClick: buttonPressedId_: " + this.buttonPressedId_);
	    var checkbox = dojo.byId (this.id + "neverShowAgain");
	    var checked = checkbox ? checkbox.checked : false;

	    this.dialog_.hide();
	    
	    if (this.callback) {
	    	this.callback(this.buttonPressedId_, this.messageId, checked, this.callbackParms);
	    }
    },

	show: function() {
	    if( this.contentIsolation == true ){
			// Get the namespace identifying the content that launched the context menu
			console.info("Inside Show if PopupUtil Undefined");
			var namespace="";
			if(this.namespace != null)
			 namespace = this.namespace;
			
			//Close (just in case) and then open the popup covering the whole page
			console.info("Opening Popup namespace "+namespace);
			var xcord=100;
			var ycord=100;
			var dialogNamespace=namespace+'ibm.tivoli.tip.dijit.TipMessageDialog';
			var old_popup_iframe = PopupUtil.ClosePopup( namespace+'ibm.tivoli.tip.dijit.TipMessageDialog' );
			//Changes for fixing Content Isolation issue with Claro theme
			var theme = dojo.body().className;
			//Changes to fix CSBUG #158121
			var popupUrl =dojo.baseUrl+'../html/TIPMessageDialogPopUp.html?PopupID='+dialogNamespace+'&eLeft='+xcord+'&eTop='+ycord+'&locale='+djConfig.locale+"&theme="+theme;
			console.info("url "+popupUrl);
			var popup_iframe = PopupUtil.OpenPopup( 
				namespace, 
				namespace+'ibm.tivoli.tip.dijit.TipMessageDialog', 
				true, 
				null, 
				null, 
				null, 
				null, 
				popupUrl, 
				null
			);
			console.info("after popup opening code");
			//Initialize the contextMenu when the iframe popup is finished loading
			var tThis = this;
			if( popup_iframe.addEventListener ){
				popup_iframe.addEventListener(
					"load", 
					function(){
						popup_iframe.contentWindow.makeMessageDialog( tThis );
					}, 
					false
				);
			}
			else if (popup_iframe.attachEvent){
				popup_iframe.attachEvent(
					"onload", 
					function(){ 
						popup_iframe.contentWindow.makeMessageDialog( tThis);
					}
				);
			}
		}
		else{
			this.dialog_.show();
			this.customLayout();
			this.correctPNG();
        }
	},

      customLayout: function() 
      {
		console.debug( dojo.byId( this.id + '_messagediv' ) );
		console.debug( dojo.contentBox( this.dialog_.domNode ) );
		
		var messageDiv = dojo.byId( this.id + '_messagediv' );
		var dialogSize = dojo.contentBox( this.dialog_.domNode );

		var titleBarSize = dojo.contentBox( this.dialog_.titleBar );

		var newSize = {w:0,h:0};

		newSize.w = dialogSize.w - 75;
		if (this.showCheckbox === true) {
			var neverShowCoords = dojo.coords (this.id + "neverShowAgain");
			//Background issue fix for IE
			if(dojo.isIE)
				newSize.h = dialogSize.h - titleBarSize.h - neverShowCoords.h - 63;
			else	
				newSize.h = dialogSize.h - titleBarSize.h - neverShowCoords.h - 90;
		} else {
			//Defect Fix for Locales with Long translations
			//newSize.h = dialogSize.h - titleBarSize.h - 90;
			//Background issue fix for IE
			if(dojo.isIE)
				newSize.h = dialogSize.h - titleBarSize.h - 63;
			else
				newSize.h = dialogSize.h - titleBarSize.h - 100;
		}

	    dojo.contentBox( messageDiv, newSize );
	},
	
	getContent: function(className) {
		var innerHTML = "";

		innerHTML += "<div tabIndex=0>";
		innerHTML += "<table class=\"" + className + "\">";
		innerHTML += "<tr valign=\"top\">";
		innerHTML += "<td>";

		innerHTML += "<table>";
		innerHTML += "<tr>";
		innerHTML += "<td rowspan=\"2\">";
		innerHTML += "&nbsp;";
		innerHTML += "<img align=\"middle\" src=\"" + this.icon_ + "\"/>";
		innerHTML += "&nbsp;&nbsp;";
		innerHTML += "</td>";
		innerHTML += "<td class=\"message-id\" align=\"left\" tabindex=\"0\" title=\"" + this.messageId + "\">";
		
		// add URL to infocenter
		if (this.url) {
			innerHTML += '<a href="javascript:void(0)" onclick="window.open(\'' + this.url + '\')">';
			innerHTML += this.messageId;
			innerHTML += "</a>";
		}
		else {
			innerHTML += this.messageId;
		}
		
		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "<tr>";
		innerHTML += "<td class=\"message-description\" align=\"left\">";
		innerHTML += "<div id='" + this.id + "_messagediv" + "' style='overflow:auto; width:100%;' tabindex=\"0\" title=\"" + this.message + "\">";
		innerHTML += "<p>";
		innerHTML += this.message;
		innerHTML += "</p>";
		innerHTML += "</div>";
		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "</table>";

		innerHTML += "</td>";
		innerHTML += "</tr>";

		if (this.showCheckbox === true) {
			innerHTML += "<tr>";
			innerHTML += "<td class=\"checkbox\" valign=\"bottom\">";
			innerHTML += "&nbsp;&nbsp;&nbsp;<input type=\"checkbox\" dojoType=\"dijit.form.CheckBox\" id=\"" + this.id + "neverShowAgain\"/><label for=\"" + this.id + "neverShowAgain\">" + this.resources_.NEVER_SHOW + "</label>";
			innerHTML += "</td>";
			innerHTML += "</tr>";
		}

		innerHTML += "<tr>";
		innerHTML += "<td valign=\"bottom\">";

		innerHTML += "<div>";
		innerHTML += "<table width='100%'>";
		innerHTML += "<tr>";
		innerHTML += "<td valign=\"bottom\">";
		innerHTML += "<div align=\"right\">";
		for (var i = 0; i < this.buttons.length; i++) {
			innerHTML += "&nbsp;<button dojoType=\"ibm.tivoli.tip.dijit.TIPButton\" id=\"" + this.id + "mmd_button_" + i + "\" value=\"" + this.buttons[i] + "\">";
			innerHTML += this.buttons[i] + "</button>";
		}
		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "</table>";
		innerHTML += "</div>";

		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "</table>";
		innerHTML += "</div>";
		return innerHTML;
	},
	
	// correctly handle PNG transparency in Win IE 5.5 & 6.
	correctPNG: function() {
		var arVersion = navigator.appVersion.split("MSIE");
		var version = parseFloat(arVersion[1]);
		if ((version >= 5.5) && document.body.filters) {
			for (var i = 0; i < document.images.length; i++) {
				var img = document.images[i];
				var imgName = img.src.toUpperCase();
				if (imgName.substring(imgName.length-3, imgName.length) == "PNG") {
					var imgID = img.id ? "id='" + img.id + "' " : "";
					var imgClass = img.className ? "class='" + img.className + "' " : "";
					var imgTitle = img.title ? "title='" + img.title + "' " : "title='" + img.alt + "' ";
					var imgStyle = "display:inline-block;" + img.style.cssText;
					if (img.align == "left") {
						imgStyle = "float:left;" + imgStyle;
					}
					if (img.align == "right") {
						imgStyle = "float:right;" + imgStyle;
					}
					if (img.parentElement.href) {
						imgStyle = "cursor:hand;" + imgStyle;
					}
					var strNewHTML = "<span " + imgID + imgClass + imgTitle +
									 " style=\"" + "width:" + img.width + "px; height:" + img.height + "px;" + imgStyle + ";" +
									 "filter:progid:DXImageTransform.Microsoft.AlphaImageLoader" +
									 "(src=\'" + img.src + "\', sizingMethod='scale');\"></span>";
					img.outerHTML = strNewHTML;
					i = i - 1;
				}
			}
		}    
	},
	
	destroy: function () {
        dojo.disconnect( this.onCloseListener );
        dojo.disconnect( this.onResizeListener );
        
        //TODO:  The fact this can NOT be called means we have not created the widget correctly
        //       This widget should directly extend dialog to be created correctly
        //this.inherited( "destroy", arguments );
        
        if ( this.dialog_ ) { this.dialog_.destroy(); dijit.registry.remove (this.dialog_.id); delete this.dialog_; }
        
		if (dijit.byId (this.id + "neverShowAgain")) {
			dijit.registry.remove (this.id + "neverShowAgain");
		}

		for (var i = 0; i < this.buttons.length; i++) {
			dijit.registry.remove (this.id + "mmd_button_" + i);
		}

		delete this.type;
		delete this.message;
		delete this.messageId;
		delete this.width;
		delete this.height;
		delete this.buttons;
		delete this.callbackParms;
		delete this.url;
		delete this.showCheckbox;
		delete this.destroyOnClose;
		delete this.title_;
		delete this.icon_;
		delete this.resources_;

		dijit.registry.remove (this.id);
	}

});



}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.Global"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.Global"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* xxxx-xxx 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/



dojo.provide("ibm.tivoli.tip.dojo.Global");
dojo.provide("ibm.tivoli.tip.dojo.Global.InvalidContextError");
dojo.provide("ibm.tivoli.tip.dojo._Global");

/****************************************************************************
* Global 
*
* (Q) What is this?
* (A) Since the TIP applications will be running inside a portal environment
*     it is possible that multiple instances of the application could be
*     running on one browser.  Therefore, it is necessary that our global
*     data and variables are referenced by Portlet Namespace to avoid
*     collisions between multipile instances.
*
*     This object is also used to provide common functions that are namespace
*     dependant.
****************************************************************************/

ibm.tivoli.tip.dojo.Global.IFrame        = "IframeTransport";
ibm.tivoli.tip.dojo.Global.instances     = [];
ibm.tivoli.tip.dojo.Global.anyException  = "*"; 

/****************************************************************************
* Global getSingleton function
*
* namespace (required) -- the namespace of the global object space
* context   (optional: used only if global object does NOT exist) an object of 
*			properties that will get mixed into the Global object for storage
* contentIsolation (optional) -- to show MessageDialog in separate iFrame
*
*  When creating the context, you create it like this:
*  var context = {
*                  controllerURL: "/SimpleExample/AjaxController",
*                  namespace: myNamespace,
*                  nlsResources : dojo.i18n.getLocalization( "ibm.tivoli.tbsm", "resources" )
*                  contextPath: "<%=rootContextPath%>"
*                };
*   controllerURL (required) - specifies the URL to use to send requests to.
*                 this is the URL which handles the requests usually using the
*                 TIP CommandHandler.
*   namespace (optional) - Should always be specified.  If not specified, then
*             this routine will set it on the context.
*   nlsResources (optional) - Must be specified if you intend to use the nls 
*                facility provided in this object
*   contextPath (optional) - Should always be specified but not required. 
*               Must be specified if you intend to use the helper
*               function "contextualURL".
*   ===========
*   any other properties you add into the context will be avaible to your app.
****************************************************************************/
ibm.tivoli.tip.dojo.Global.getSingleton = function( namespace, context, contentIsolation )
{
  var singleton = ibm.tivoli.tip.dojo.Global.instances[ namespace ];
  
  //if a singleton was not found then assume we need to create one now
  if ( singleton === undefined )
  {
  	//check to make sure a context object was passed before creating a Global object
  	//if the context object wasn't given then assume that an attempt was made to 
  	//access a Global object that hasn't been created -- throw an error
  	if ( context === undefined )
  	{
  		throw new ibm.tivoli.tip.dojo.Global.InvalidContextError( "An attempt was made to access a " + 
  		"singleton ibm.tivoli.tip.dojo.Global object that has not been created yet.  Or the context " + 
  		"object was not provided on creation." );
  	}
  	
  	//make sure the namespace is saved on the context object
  	if ( context.namespace === undefined )
  	{
  		context.namespace = namespace;	
  	}  	
	//conditionally setting contentIsolation variable
    if(contentIsolation === null || contentIsolation === undefined)
		contentIsolation = false;
  	
    singleton = new ibm.tivoli.tip.dojo._Global( context, contentIsolation );
   
   	//save the new singleton instance on the Global memory space
    ibm.tivoli.tip.dojo.Global.instances[ namespace ] = singleton;
  }  
  
  return( singleton );	
};

/****************************************************************************
* Global setSingleton function
*
* namespace (required) -- the namespace of the global object space
* context   (required) an object of 
   properties that will get mixed into the Global object for storage
* contentIsolation (optional) -- to show MessageDialog in separate iFrame
****************************************************************************/
ibm.tivoli.tip.dojo.Global.setSingleton = function (namespace, context, contentIsolation) 
{
   //check to make sure a context object was passed before creating a Global object
   //if the context object wasn't given then assume that an attempt was made to 
   //access a Global object that hasn't been created -- throw an error
   if ( context === undefined )
   {
    throw new ibm.tivoli.tip.dojo.Global.InvalidContextError( "An attempt was made to access a " + 
    "singleton ibm.tivoli.tip.dojo.Global object that has not been created yet.  Or the context " + 
    "object was not provided on creation." );
   }
   
   //make sure the namespace is saved on the context object
   if ( context.namespace === undefined )
   {
    context.namespace = namespace; 
   }
   //conditionally setting contentIsolation variable
   if(contentIsolation === null || contentIsolation === undefined)
		contentIsolation = false;
   var singleton = new ibm.tivoli.tip.dojo._Global( context, contentIsolation );
   
   //save the new singleton instance on the Global memory space
   ibm.tivoli.tip.dojo.Global.instances[ namespace ] = singleton; 
   
   return (singleton);
};


/****************************************************************************
* _Global -- the actual Global object
*
* Notice that there is a '_' underscore prefixing this object.  This is
* important because removing it will delete the Global module functions
* such as getSingleton.  This is OK because we will never construct one
* of these things without calling getSingleton.
****************************************************************************/
dojo.declare("ibm.tivoli.tip.dojo._Global",null,
{	 
  _actionBroker:  null,
  _portletTitle: null,
  contentIsolation: false,
  namespace: "",
	
	constructor: function( context, contentIsolation )
	{
      //add all functions, fields, objects inside context to this object
      dojo.mixin( this, context );
      
      //create path to JSON controller servlet
      this.controller = context.controllerURL;
         
      // Create resource bundle for TIP based resource strings
      this.nlsTIPResources = dojo.i18n.getLocalization( "ibm.tivoli.tip.dojo", "resources" );
      
	  //Setting Namespace variable
	  this.namespace = context.namespace;
	  
	  //Setting Boolean variable to show MessageDialog in Content Isolation mode
      this.contentIsolation = contentIsolation;
	},

  /************************************************************************
  * Get the action broker.
  ************************************************************************/
  /* ibm.tivoli.tip.dojo.ActionBroker */ 
  getActionBroker : function() 
  {
    return( this._actionBroker );
  },

  /************************************************************************
  * Set the action broker.
  ************************************************************************/
  /* ibm.tivoli.tip.dojo.ActionBroker */ 
  setActionBroker : function( /* ibm.tivoli.tip.dojo.ActionBroker */ broker ) 
  {
    return( this._actionBroker = broker );
  },
  
  getPortletTitle : function () {
  	return this._portletTitle;
  },
  
  setPortletTitle: function (/*String*/ title) {
  	this._portletTitle = title;
  },
	
	/************************************************************************
	* Generate Conextual URL
	************************************************************************/
	contextualURL : function( baseURL ) 
	{
  	return( this.contextPath + baseURL );
	},

	/************************************************************************
	* Generate namespaced ID from a BASE ID.
	************************************************************************/
	namespaceID: function( baseID ) 
	{
  	return( this.namespace + baseID );
	},	

  /************************************************************************
  * show error message dialog
  ************************************************************************/
  showErrorMessage: function( msgMixin )
  {
    var msgKW = {
              type:         "Error",
              showCheckbox: false,
			  namespace: this.namespace,
			  contentIsolation: this.contentIsolation,
              portletTitle: this._portletTitle
            };

    dojo.mixin( msgKW, msgMixin );
    var x = new ibm.tivoli.tip.dijit.TIPMessageDialog( msgKW );
    x.show();
  },

  /************************************************************************
  * Retrieve a specific NLS string from the TIP resource bundle.  Deployers
  * could create a method like this for their own product that calls nls
  * with their resource bundle for convenience.  This is meant to only
  * be called by TIP code.
  ************************************************************************/
  tipnls: function( nlsKey, inserts )
  {
     return this.nls( nlsKey, inserts, this.nlsTIPResources );
  },

  /************************************************************************
  * Retrieve a specific NLS string optionally performing substitution(s).
  * This is a fairly generic routine meant to be used by exploiters of 
  * the Global object.  If an exploiter specifies a resource bundle in 
  * the Global context via the name 'nlsResources' when creating the Global object
  * then a bundle need not be specified on the invocation of this method
  * because it will use nlsResources instead.
  ************************************************************************/
  nls: function( nlsKey, inserts, myBundle )
  {
    // attempt to retrieve the specified string via its key
    var s = null;
    
    // If a bundle has been specified, then use it, else
    // use the bundle specified on nlsResources.
    if (myBundle !== null)
    {
       s = myBundle[ nlsKey ];
    }
    else
    {
       s = this.nlsResources[ nlsKey ];
    }

    // substitute "key" if not able to resolve
    if ( ! s )
    {
      s = nlsKey;
    }

    // string insert(s) specified?
    if ( inserts ) 
    {
      // if not already a KW object -or- array 
      if ( ! ( typeof inserts == 'object' ) )
      {
        // convert single number/string/... to array
        inserts = [ inserts ];
      }

      // perform the substitutions
      s = dojo.string.substitute( s, inserts );
    }

    return( s );
  },
  
  showMessageDialog: function (type, msgID, msg, showCheckbox) {
	if (!showCheckbox) {
		showCheckbox = false;
	}
    var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: type,
	    	                                          message: msg,
	        	                                      messageId: msgID,
													  namespace: this.namespace,
													  contentIsolation: this.contentIsolation,
	        	                                      showCheckbox: showCheckbox
	                    	                        }); 
	modalWindow.show();
  },

  showMessage: function (msgMixin) {
	var messageid = msgMixin.messageId;
	var messagetype = messageid.substring(messageid.length-1, messageid.length);
	var type = "";
	switch(messagetype)
	{
		case "E":
			type = "error";
			break;
		case "W":
			type = "warning";
			break;
		case "C":
			type = "confirm";
			break;
		case "V":
			type = "validation";
			break;
		default:
			type = "";
	}
	
    var msgKW = {
              type: type,
              showCheckbox: false,
			  namespace: this.namespace,
			  contentIsolation: this.contentIsolation,
              portletTitle: this._portletTitle
            };

    dojo.mixin( msgKW, msgMixin );
    var x = new ibm.tivoli.tip.dijit.TIPMessageDialog( msgKW );
	x.show();
  },
  
  handleServerException: function (exception)  {
 	console.error ("Received server exception: " + exception.messageID + ": " + exception.messageString);
	console.error ("with server stack trace: ");
	console.error (exception.stackTrace);
	this.showMessageDialog (exception.type, exception.messageID, exception.messageString, false);
  },
  
	isBidiChar: function (c){
		if (c >= 0x05d0 && c <= 0x065f ||
			c >= 0x066a && c <= 0x06ef ||
			c >= 0x06fa && c <= 0x07ff ||
			c >= 0xfb1d && c <= 0xfdff ||
			c >= 0xfe70 && c <= 0xfefc) {
			console.log ("isBidiChar: true: " + c);
			return true;
		} else {
			console.log ("isBidiChar: false: " + c);
			return false;
		}
	},

	isLatinChar: function(c){
    	if((c > 64 && c < 91)||(c > 96 && c < 123)) {
	        return true;
		} else {            
    		return false;
    	}    
	},

	checkContextual: function (text){
//		var text = obj.value;
		var len = text.length;
		for(var i = 0;i < len;i++)  {
			symbol = text.charCodeAt(i);
			if(this.isBidiChar(symbol)) {
				return "RTL";
		//	} else if(this.isLatinChar(symbol)) {
		//		return "LTR";
			}
		}
		return "LTR";
	},
  
  bidiEnableLabel: function (text) {
		var result = text;
		
		if (this.bidiTxtDir == "RTL") {
			result = "&#8235;" + text + "&#8236;"; 
		} else if (this.bidiTxtDir == "CONTEXTUAL") {
			var contextDir = this.checkContextual (text)
			if (contextDir == "RTL") {
				result = "&#8235;" + text + "&#8236;";
			} else {
				result = text;
			}
		} else {  // ltf
			result = text;
		}

		return (result);
  }
  
    
} );

//Declare a new Error type to throw when a Global context error is detected
dojo.declare("ibm.tivoli.tip.dojo.Global.InvalidContextError", Error, {});

}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.util"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.util"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2007
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/


dojo.provide("ibm.tivoli.tip.dojo.util");

ibm.tivoli.tip.dojo.util._stdDateFormat = { // selector:     'dateOnly', 
                                      formatLength: 'short' 
                                    };

/************************************************************************
* "apply" any GLOBAL-based data of interest.
************************************************************************/	
ibm.tivoli.tip.dojo.util.applyGlobal = function( global ) 
{
  ibm.tivoli.tip.dojo.util._stdDateFormat = global.l10n.date.stdTS;

  console.debug( "** util.applyGlobal **", dojo.toJson(ibm.tivoli.tip.dojo.util._stdDateFormat ) ); 
};

/************************************************************************
* Format a DATE with NLS rules
************************************************************************/	
ibm.tivoli.tip.dojo.util.formatDate = function( date ) 
{
	var fDate = ( date === null ) ? "" : dojo.date.locale.format( date, ibm.tivoli.tip.dojo.util._stdDateFormat );
 
	return( fDate );
};

/************************************************************************
* Standard formatter for a table column containing Date objects.
************************************************************************/	
ibm.tivoli.tip.dojo.util.dateColumnFormater = function( /* Date */ cellValue )
 {
   // console.debug( "cell: ", cellValue );

   return( ibm.tivoli.tip.dojo.util.formatDate( cellValue ) );
 };

/************************************************************************
* Refresh the page
************************************************************************/	
ibm.tivoli.tip.dojo.util.refreshPage = function()
{
	window.location.reload( false );
};

/*****************************************************************************
* Return a string of all the fields contained in an object.
*****************************************************************************/
ibm.tivoli.tip.dojo.util.fieldsOf = function( object )
{ 
  var s = "";

  if ( object && ( object !== null ) ) 
  {
    for ( var field in object )
    {
    	if (field) {
	      s += field + "; ";
	    }
    }
  }

  return( s );
};

/************************************************************************
* Clear a datastore (must implement dojo.data.api.Write)
* 
* Returns dojo.Deferred() -- must add Callback
*
* example usage:
* ibm.tivoli.tip.dojo.util.clearDataStore( store ).addCallback( function(){} );
************************************************************************/ 
ibm.tivoli.tip.dojo.util.clearDataStore = function( store )
{
  var deferred = new dojo.Deferred();
    
  var callback = 
  {  
        deferred: deferred,
           store: store,        
      onComplete: function( items, request ) 
                  {                    
                    for ( var i in items )
                    {
                    	if (i) {
	                      this.store.deleteItem( items[i] );
	                    }
                    }

                    this.deferred.callback();
                  }
  };
  
  store.fetch( { onComplete: dojo.hitch( callback, "onComplete" ) } );
  
  return deferred;
};

/************************************************************************
* Clear a datastore (must implement dojo.data.api.Write)
************************************************************************/ 
ibm.tivoli.tip.dojo.util.addItemsToDataStore = function( store, items )
{
  for ( var i in items )
  {
  	if (i) {
	    store.newItem( items[i] );
	}
  }

  store.save();
};

/************************************************************************
* Find the default name for a specific value using a name/value array.
*
* [
*  { value: "-1", name: this.global.nls( "CBOX_CONFIG_DEFAULT", { config : config.maxSnapshotsDefault } ) }, 
*  { value:  "0", name: bundle.CBOX_DISABLED }, 
*  { value:  "1", name:  "1" }
* ]
************************************************************************/ 
ibm.tivoli.tip.dojo.util.findDefaultName = function( value, nameValueArray )
{
  var name = nameValueArray[ 0 ].name;     // default to first element

  for ( var i in nameValueArray )
  {
  	if (i) {
    	var nv = nameValueArray[ i ];

    	if ( nv.value == value ) 
    	{
      		name = nv.name;
      		break;
    	}
    }
  }

  // console.debug( "value: " + value + ", name: " + name );

  return( name );
};


}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.Server"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.Server"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/


dojo.provide("ibm.tivoli.tip.dojo.Server");
dojo.provide("ibm.tivoli.tip.dojo._Server");

ibm.tivoli.tip.dojo.Server.anyException  = "*"; 

ibm.tivoli.tip.dojo.Server._singleton    = null;

/****************************************************************************
* Get the "singleton" Server instance.
****************************************************************************/
/* "ibm.tivoli.tip.dojo._Server" */
ibm.tivoli.tip.dojo.Server.getSingleton = function( /* required: ibm.tivoli.tip.dojo._Global */ global )
{
  if ( global === undefined ) {
  	throw new Error( "ibm.tivoli.tip.dojo.Server requires a 'global' object reference" );
  }
  		
  //if the singleton doesn't exist yet --> create it now
  //there must be a unique singleton per global instance
  if ( global._serverSingleton === undefined || global._serverSingleton === null )
  {
    global._serverSingleton = new ibm.tivoli.tip.dojo._Server( { global : global } );
  }  
  
  return global._serverSingleton;	
};

/****************************************************************************
* Provides I/O access to the controller servlet.
* Handles TIP protocol of returning server-side exceptions in a valid
* data response besides Dojo's error and timeout conditions.
****************************************************************************/
dojo.declare("ibm.tivoli.tip.dojo._Server", 
             null,
{	 
	_totalRequests:   0,        // total server requests made
  _activeRequests:  0,        // # server request(s) currently active
	
  /************************************************************************
  * Object constructor
  ************************************************************************/
	constructor: function( mixinData )
	{
    dojo.mixin( this, mixinData );
	},
	
	/************************************************************************
	* Perform a HTTP GET request.
	************************************************************************/
	/* void */ get: function( /* Object */ requestKW, checkTimeout )
	{
    this._preRequest( requestKW, checkTimeout );
	
    //console.debug( "Ready to xhrGet", requestKW );
    //console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(requestKW) );
    //console.debug( "Timeout", requestKW.timeout );

	  dojo.xhrGet( requestKW ); 
	},
	
	/************************************************************************
	* Perform a HTTP POST request.  Preferred for data forms that do
  * NOT contain any file-input fields...
	************************************************************************/
	/* void */ post: function( /* Object */ requestKW, checkTimeout )
	{
    this._preRequest( requestKW, checkTimeout );
	
	  dojo.xhrPost( requestKW ); 
	},

  /************************************************************************
  * Common PRE request processing of the request KW object.
  ************************************************************************/
  /* Object */ _preRequest: function( /* Object */ requestKW, checkTimeout )
  {
  	if (checkTimeout === undefined) { checkTimeout = true; }
    this._requestStarting( ( requestKW.showBusyDialog !== undefined ) ? requestKW.showBusyDialog : true );                 

    requestKW._server      = this;                 // save self reference for use in load/error handlers below   
    requestKW._requestID   = this._totalRequests;  // unofficial request ID

    requestKW.url          = requestKW.appendURL ? 
      this.global.controller + requestKW.appendURL :
      this.global.controller;

    requestKW.handleAs	   = "json";  
    requestKW.mimetype     = "application/json";
    
    if (checkTimeout) {
    	requestKW.content.requestType  = "global";     // indicates that the request was sent here
    }

    requestKW.preventCache = true;     // fix IE cache bug

    // provide default timeout if none specified

    if ( ! requestKW.timeout ) {
           requestKW.timeout = 0; 
    }

    // our code handles it all!  NOTE: trying to set callbacks on "deferred" object did NOT work

    requestKW.load  = this._loadHandler;
    requestKW.error = this._errorHandler;

    return( requestKW ); 
  },
	
	/************************************************************************
	* Standard JSON request "load" handler.
  *
  * Use the appropriate "how to handle" directions from the request:
  * - "onExc"      for server-side errors returned in the 'exc' field of response
  * - "onResponse" for response data
  *
  *	NOTE "this" will be the object passed as argument to dojo.xhrXxxx() used (aka requestKW)
	************************************************************************/
  /* response */ _loadHandler: function( response, ioArgs ) 
	{
    //console.debug( "************************ Server._loadHandler ************************" );

//  console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(this) );      proved its requestKW
//  console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(response) );
//  console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(ioArgs) );    ioArgs.args == requestKW

    //console.debug( "  ** response", dojo.toJson(response) );

    var server = this._server;

    server._requestCompleted();

	  // was an exception detected/reported from the server-side?

    var exc = response.exc;

	  if ( exc )
	  { 
      //console.debug( "    !!!!! handling EXC !!!!!" );

      // ISC session timeout?
      if ( exc == "ISCSessionTimeoutException" ) 
      {
        ibm.tivoli.tip.dojo.util.refreshPage();
        return;
      }

      var howTo  = this.onExc;

      if ( ! howTo ) {
        howTo = { showMsg : true };    // use default
      }

      //console.debug( "  ?? howTo",  dojo.toJson(howTo) );

      // Crop the exc message to maximum size
      server.cropExcMsg( response );

      //TODO: handle new NLS enabled exceptions...
      var msgKW = null;
      
      if (response.excID) {
	      msgKW = { 
                   messageId : response.excID,
                   message   : response.excMsg
                  };
      } else {
    	  msgKW = { 
                   messageId : server.global.tipnls( "BIND_ERROR_ID" ),
                   message   : server.global.tipnls( "BIND_ERROR_MSG", response )
                  };
	  }

      //console.debug( "  ?? msgKW",  dojo.toJson(msgKW) );

      if ( howTo.showMsg ) {
       // server.global.showErrorMessage( msgKW );
	   server.global.showMessage( msgKW);
      }

      if ( howTo.callback ) {
        howTo.callback( msgKW, this /* requestKW */ );
      }
	  }
	
	  // No exception --> forward to callback
	  else
	  {
      howTo = this.onResponse;

      if ( howTo && howTo.callback ) {
	      howTo.callback( response, this /* requestKW */ );
      }
	  }

    return( response ); // Dojo best practice 
	},

	/************************************************************************
	* Standard JSON request error handler
  *
  * How to handle directions are contained in the request's "onExc" field.
  *
  *	NOTE "this" will be the object passed as argument to dojo.xhrXxxx() used
  *
  * error.message:    specific error message, "too much recursion"
  * error.fileName:   "https://localhost:16316/ibm/console/secure/isclite/scripts/dojo/dojo.js"
  * error.lineNumber: 20
  * error.stack:      callback stack (can be very deep )
  * error.name:       "InternalError"
  *
  * error.dojoType:   "timeout" or "cancel", but other error types are possible.
  * dojoType=timeout message=timeout exceeded
  *
  * Error: bad http response code:404 message=bad http response code:404
  * 
  * console.error("HTTP status code: ", ioArgs.xhr.status); 
	************************************************************************/
	/* data */ _errorHandler: function( error, ioArgs ) 
	{
	//alert ("in Server._errorHandler with error: " + dojo.toJson(error));
    console.error( "************************ Server._errorHandler ************************" );

    console.error( "  >> this",   ibm.tivoli.tip.dojo.util.fieldsOf(this) );
    console.error( "  >> error",  ibm.tivoli.tip.dojo.util.fieldsOf(error) );
    console.error( "  >> ioArgs", ibm.tivoli.tip.dojo.util.fieldsOf(ioArgs) );
    console.error( "  >> ioArgs.args", ibm.tivoli.tip.dojo.util.fieldsOf(ioArgs.args) );

    console.error( "  -- error",  dojo.toJson(error) );

    var server = this._server;

    server._requestCompleted();

    var howTo = this.onError;
    var msgKW = null;

    if ( ! howTo ) {
      howTo = { showMsg : true };    // use default
    }

    if ( error.dojoType && ( error.dojoType === "timeout" ) ) 
    {
      msgKW = { 
                messageId : server.global.tipnls( "BIND_TIMEOUT_ID" ),
                message   : server.global.tipnls( "BIND_TIMEOUT_MSG", error )
              }; 
    }
    else if ( error.name && ( error.name === "InternalError" ) ) 
    {
      msgKW = { 
                messageId : server.global.tipnls( "JS_ERROR_ID" ),
                message   : server.global.tipnls( "JS_ERROR_MSG", error )
              }; 
    }
    else
    {
      msgKW = { 
                messageId : server.global.tipnls( "XHR_ERROR_ID" ),
                message   : server.global.tipnls( "XHR_ERROR_MSG", error )
              }; 
    }

    if ( howTo.showMsg ) {
	    server.global.showErrorMessage( msgKW );
    }

    if ( howTo.callback ) {
      howTo.callback( msgKW, this /* requestKW */ );
    }

    return( error ); // Dojo best practice 
	},

  /************************************************************************
  * Starting a new server request.
  ************************************************************************/
  _requestStarting: function( showBusyDialog ) 
  {
    // increment total server requests 
    this._totalRequests++;

    // increment outstanding server requests 
    this._activeRequests++;

    // show wait dialog on first active request
    if ( showBusyDialog === true && this._activeRequests == 1 )
    {
      this.wait_timer = setInterval( dojo.hitch( this, "_showBusyPane" ), 100 );
    }
  },
  
  /************************************************************************
  * Show Busy Pane
  *    
  * Constructing a ibm.tivoli.tip.dijit.BusyPane and passing its ID on 
  * busyPaneID when constructing Server object will provide feedback
  * when requests are made
  ************************************************************************/
  _showBusyPane: function()
  {
  	console.log ("enter _showBusyPane");
    clearInterval( this.wait_timer );
    // in case of race condition, make sure that active requests > 0
    if ( this._activeRequests > 0 ) {
	    var busyPane = dijit.byId( this.busyPaneID );
	    if ( busyPane ) {
	    	console.log ("showing busypane");
	      busyPane.show(true); // size to portlet
	    } else {
	    	console.log ("busypane does not exist");
	    }
    } else {
    	console.log ("active requests ! > 0 so not showing busypane");
    }
  },

  /************************************************************************
  * Completing a server request.
  ************************************************************************/
  _requestCompleted: function() 
  {
	  //decrement the # outstanding requests 
	  this._activeRequests--;

	  // if no more active requests --> close wait dialog
	  if ( this._activeRequests === 0 )
	  {
		  clearInterval( this.wait_timer );
      
          var busyPane = dijit.byId( this.busyPaneID );
          if ( busyPane )
          {
            busyPane.hide();
          }
	  }  	
  },

  /***************************************************************************
  * Crop the message in the 'excMsg' field to the maximum allowed length.
  ***************************************************************************/
  cropExcMsg: function( response )
  {
    // provide a default exc message if field isn't already available
    if ( ! response.excMsg ) {
      response.excMsg = "-?-";
    }

    // access externalized MAX length
    var msgMax = this.global.nlsTIPResources.MAX_WIDTH_EXC_MSG;

    // exc message too long?
    if ( response.excMsg.length > msgMax ) 
    {
      response.excMsg = response.excMsg.substring( 0, msgMax ) + this.global.tipnls( "TEXT_CROPPED" );
    }
  }

} );

}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.EventUtil"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.EventUtil"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dojo.EventUtil");

dojo.declare("ibm.tivoli.tip.dojo.EventUtil",
	          null,
{
	// Simple method to return whether or not a value has a value
	_hasValue: function(obj){
		return typeof obj != "undefined" && obj != null;
	},

	_doEval: function(scope,str){
		if (typeof scope == "undefined" || scope == null)
			return null;
	    var obj = null;
	    try {
	    	var subParts = str.split(".");
	    	obj=scope;
	    	for (var i=0; i<subParts.length; i++){
	    		obj = obj[subParts[i]];
	    	}
	    	//obj=eval("scope."+str);
	    }
	    catch (e) {
	    	obj=null
	    }
	    return obj;
	},
	// Indicates if the expression has a member
	// Scope is the object being checked, str indicates the member to find.
	_hasMember: function(scope, str){
	    var obj = this._doEval(scope,str);
		return typeof obj != "undefined" && obj != null;
	},

	// Pass in a string and this will return a value associated with it.
	// If the string has an error, the returned value is null.  Very
	// useful for deeply nested references.
	_getValue: function(scope,str){
	    var obj = this._doEval(scope,str);
	    if (typeof obj == "undefined")
	    	obj = null;
		return obj;
	},

	// Only adds the specified attribute to the specified object if
	// the attribute does not already exist
	_safeAdd: function(obj,attr,value){
		if (!this._hasValue(obj[attr]))
			obj[attr]=value;
	},

	// This function takes an event using the new event context and 
	// modifies the event by adding the old context to the event by
	// interrogating the proper fields of the new event.  Note that
	// multiple resources are not supported in the old event context.
	// srcEvent: is the event base, it will be updated upon return
	// payload: optional parameter to point to the payload of the event
    //          if specified, this will be used instead of srcEvent.payload
	addOldContext: function(srcEvent, /*optional*/ payload){
		if (this._hasMember(srcEvent,"payload") || this._hasValue(payload)){
			var productCount=1;
			var increment=false;
			var evtPayload=(this._hasValue(payload)?payload:this._getValue(srcEvent,"payload"));
			
			// Check for case if emitter created a resources array of 1
			if (this._hasMember(evtPayload,"resources.length") && (evtPayload.resources.length==1))
				evtPayload=evtPayload.resources[0];

			var clazztype=this._getValue(evtPayload,"cdm.attributes.ClassType");
			
			// Enumerate through all of the payload
			for (var attr in evtPayload) {
			   if (evtPayload[attr]!=null){
				   if (attr=="product"){
						for (var name in evtPayload.product) {
							increment=false;
							var product = evtPayload.product[name];
							// Get all product targeted attributes
							for (var prodName in product){
								// If the attribute is the cdm attribute, need to enumerate it.
								if (prodName == "cdm"){
									for (var prodCdm in product.cdm){
										if (prodCdm == "ClassType")
											this._safeAdd(srcEvent,"CDMClass"+productCount,product.cdm[prodCdm]);
										else
											this._safeAdd(srcEvent,prodCdm+productCount,product.cdm[prodCdm]);
										increment=true;
									}
									if (clazztype != null)
										this._safeAdd(srcEvent,"CDMClass"+productCount,clazztype);
								} else {
									this._safeAdd(srcEvent,prodName+productCount,product[prodName]);
									increment=true;
								}
							}
							if (increment)
								productCount++;
						}
				   }
				   else if (attr=="cdm" && this._hasMember(evtPayload,"cdm.attributes")){
						for (var attr in evtPayload.cdm.attributes) {
							// Add all attributes with their given name; including ClassType
							this._safeAdd(srcEvent,attr,evtPayload.cdm.attributes[attr]);
						}			
				   }
				   else if (attr != "framework" && attr != "resources"){
					   this._safeAdd(srcEvent,attr,evtPayload[attr]);
				   }
			   }
			}
		}
		return srcEvent;
	},

	_parseProductKey: function(mssname){
		var key = null;
		//var pattern = /\+ProductName=.+$/g;
		//var key = mssname.match(pattern);
		//console.debug(key);
		var pattern = "+ProductName=";
		var i = mssname.indexOf(pattern);
		if (i>0){
			var start = i + pattern.length;
			var end = mssname.indexOf("+",start);
			if (end>0)
				key = mssname.substr(start,end-start);
			else
				key = mssname.substr(start);
		}
		if(key != null)
			key = key.replace(/ /g,"");
			
		return key;
	},

	_addCdmAttribute: function(evtPayload, attribute, value){
		if (!this._hasValue(evtPayload.cdm))
			evtPayload.cdm = {};
		if (!this._hasValue(evtPayload.cdm.attributes))
			evtPayload.cdm.attributes = {};
		evtPayload.cdm.attributes[attribute] = value;
	},

	_addMssMember: function(evtPayload, attrList, mssNumber, productkey, attribute){
		if (this._hasMember(attrList,attribute+mssNumber)){

			// Some special processing for CDMClass
			if (attribute=="CDMClass"){
				if (!this._hasMember(evtPayload,"cdm.attributes.ClassType")){
					this._addCdmAttribute(evtPayload, "ClassType", attrList[attribute+mssNumber]);
				}
				this._safeAdd(evtPayload.product[productkey].cdm,"ClassType",attrList[attribute+mssNumber]);
			} else {
				this._safeAdd(evtPayload.product[productkey].cdm,attribute,attrList[attribute+mssNumber]);
			}
			
			delete attrList[attribute+mssNumber];
		}
	},

	_getMssMembers: function(evtPayload, attrList, mssNumber){
		var mssname=attrList["MSSName"+mssNumber];
		var productkey = this._parseProductKey(mssname);
		if (productkey != null){
			if (!this._hasValue(evtPayload.product))
				evtPayload.product={};
			if (!this._hasValue(evtPayload.product[productkey]))
				evtPayload.product[productkey]={};
			if (!this._hasValue(evtPayload.product[productkey].cdm))
				evtPayload.product[productkey].cdm={};
			
			// This case if there are no numbers after MSSName in the old context
			if (mssNumber == ""){
				// The old format only ever added numbers to MSSName, SourceToken, and CDMClass
				this._addMssMember(evtPayload, attrList, mssNumber, productkey, "MSSName");
				this._addMssMember(evtPayload, attrList, mssNumber, productkey, "SourceToken");
				this._addMssMember(evtPayload, attrList, mssNumber, productkey, "CDMClass");
			} else {
				// If here, there are numbers after MSSName (like MSSName1), we include all
				// attributes ending with the same number in that providertype.
				for (var attr in attrList){
					if (attr.lastIndexOf(mssNumber)==(attr.length-(""+mssNumber).length)){
						var prop = attr.substr(0,attr.length-(""+mssNumber).length);
						
						if (prop=="CDMClass" || prop=="MSSName" || prop=="SourceToken"){
							this._addMssMember(evtPayload, attrList, mssNumber, productkey, prop);
						} else {
							this._safeAdd(evtPayload.product[productkey],prop,attrList[attr]);
						}

						delete attrList[attr];
					}
				}
			}
			
		}
	},

	// The purpose of this method is to attempt to create a "new" context out of
	// an event constructed with old context.
	// srcEvent - the source event
	// payload - optional parameter.  By default this method will look for srcEvent.payload
    //           to pull the context from, but you can specify and alternate location if needed.
	addNewContext: function(srcEvent, /*optional*/ payload){
		// bypass if event already has a payload; therefore, only run this method
		// if a payload object was specified, or there is not already a payload
		if ((this._hasValue(payload) || !this._hasMember(srcEvent,"payload")) && this._hasValue(srcEvent)){  
		    // Gather all of the attributes into an array so we can track what has been processed.
			var attrList={};
			for (var attr in srcEvent) {
				attrList[attr]=srcEvent[attr];
			}
			var evtPayload={};
			if (this._hasValue(payload)){
				evtPayload=payload;
			} else {
				srcEvent.payload = {};
				evtPayload = srcEvent.payload;
			}
			
			// See if ClassType was coded in the old context.  If so, add it; otherwise, we will
			// try to get it from CDMClass later.
			if (this._hasMember(attrList,"ClassType") && !this._hasMember(evtPayload,"cdm.attributes.ClassType")){
				this._addCdmAttribute(evtPayload, "ClassType", attrList["ClassType"]);
				delete attrList["ClassType"];
			}
			
			if (this._hasMember(attrList,"MSSName")){
				// There is just "MSSName", so assume just one MSSName, SourceToken, CDMClass.
				this._getMssMembers(evtPayload,attrList,"");
			} else {
				var mssNumber = 1;
				while (this._hasMember(attrList,"MSSName"+mssNumber))
				{
					this._getMssMembers(evtPayload,attrList,mssNumber);
				    mssNumber++;	
				}
			}
			
			// Now place all the other properties inline
			for (var mem in attrList){
				// do not add name, type, or source as they are part of the event meta data
				if (mem != "name" && mem != "source" && mem != "type"){
					this._safeAdd(evtPayload,mem,attrList[mem]);
				}
			}
		}
     }   
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPContextMenu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPContextMenu"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/








dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenu");
dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenuProvider");
dojo.provide("ibm.tivoli.tip.dijit.CMSMenuProvider");
dojo.provide("ibm.tivoli.tip.dijit.CMSMenuProviderV2");
dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenuProxy");


/**
 *  Provides dynamic context menu support.
 *  How do you use this?
 *  First, you create an instance of this context menu similar to:
 *     this._contextMenuAdapter = new ibm.tivoli.tip.dijit.TIPContextMenu({cmProvider: this.cmProvider});
 *  Note: you pass in an instance of a context menu provider (see objects lower in this file)
 *
 *  Second, you latch whatever event you want to create the context menu.  For 
 *  Grids, you might do something like this:
 *     this._onRowContextMenuListener = dojo.connect( this.grid, "onRowContextMenu", this, "_onRowContextMenu" );
 *  Don't forget to disconnect this listener when your object is destroyed.
 *
 *  Third, you create the method that gets called by the above connect, and call
 *  the contextMenuOpen on this context menu with whatever context you want 
 *  processed.  Again, in a Grid example:
 *   _onRowContextMenu: function( e )
 *   { 
 *     var row = e.grid.model.getRow( e.rowIndex );
 *     
 *     if ( row )
 *     {
 *       //select the row that the context menu was initiated on
 *       e.grid.selection.unselectAll( e.rowIndex );
 *       e.grid.selection.setSelected( e.rowIndex, true );
 *               
 *       //tell the context menu adapter to show the menu
 *       // no context is provided because we overload the _loadMenuItems method
 *       this._contextMenuAdapter.contextMenuOpen( e, {selectedData: this.getSelectionData()} );
 *     }
 *   },
 *
 *  
 */
dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenu",
              ibm.tivoli.tip.dijit.ContextMenuAdapter,
{
   cmProvider: null,       // TIPContextMenuProvider
   cmClientProvider: null,
   url: "no_url_needed",   // Note we have to pass something to ContextMenuAdapter
                           // for the url so that we can override _loadMenuItems
   
   /****************************************************************************
   * Callback that should be overloaded to be told when a menu item is clicked
   * and an action should occur
   ****************************************************************************/
   onMenuItemClick: function( menuItem )
   { 
      console.debug("-------------- menuItem :",menuItem);
      try{
    	  this.cmClientProvider.getMenuCommand(menuItem.actionID, this._context, menuItem);
      }catch(e){
    	  this.cmProvider.getMenuCommand(menuItem.actionID, this._context, menuItem);
      }
   },

   /****************************************************************************
   * Synchronously Call a JSON Url to get the menu items
   *
   ****************************************************************************/
   _loadMenuItems: function()
   {
	   return this.cmClientProvider ? this.cmClientProvider.getMenuItems( this._context ).concat (this.cmProvider.getMenuItems( this._context )) : this.cmProvider.getMenuItems( this._context );
   }

});

/**
 *  What is this?
 *  This is a helper for the TIPContextMenu.  It allows you pass in DOM nodes 
 *  and it will set up the context menu action for those nodes.  You must call 
 *  destroy on this object when you are done with it.  You provide the following
 *  when you create it:
 *   - targetNodeIds (required): these are string IDs for the DOM nodes you want 
 *                               context menus on
 *   - getContextCallback (optional): this is a function that will return any 
 *                                    context that you want passed along on the 
 *                                    context menu open call.  It will get 
 *                                    passed to your provider.
 */
dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenuProxy",
              ibm.tivoli.tip.dijit.TIPContextMenu,
{

   targetNodeIds: null,

   getContextCallback: null,
   
   _bindings: null,
   
   constructor: function() 
   {
      this._bindings = [];
      dojo.forEach(this.targetNodeIds, dojo.hitch(this, this.bindDomNode));
   },

   // Called when it is time to notify the user that something was clicked.
   _openContextMenu: function(event)
   {
      var result = null;
      if (this.getContextCallback !== null) 
      {
         result = this.getContextCallback(event);
      }
      this.contextMenuOpen( event, result );
   },
   
   // This method mostly copied from dijit.Menu to provide the binding to the 
   // DOM node.  I needed to copy it so that I could properly inject the 
   // contextMenuAdapter.  I'll note changes with <kjp> tags
	bindDomNode: function(/*String|DomNode*/ node){
		// summary: attach menu to given node
		node = dojo.byId(node);

		//TODO: this is to support context popups in Editor.  Maybe this shouldn't be in dijit.Menu
		var win = dijit.getDocumentWindow(node.ownerDocument);
		if(node.tagName.toLowerCase()=="iframe"){
			win = this._iframeContentWindow(node);
			node = dojo.withGlobal(win, dojo.body);
		}

		// to capture these events at the top level,
		// attach to document, not body
		var cn = (node == dojo.body() ? dojo.doc : node);

		node[this.id] = this._bindings.push([
         // <kjp> Changed this next line to point to my _openContextMenu.
         // The line previously pointed to _openMyself
			dojo.connect(cn, "oncontextmenu", this, "_openContextMenu"),
			dojo.connect(cn, "onkeydown", this, "_contextKey")
         // <kjp> I removed the next line (and removed the comma from the previous)
			//dojo.connect(cn, "onmousedown", this, "_contextMouse")
		]);
	},

   // This is a complete copy from dijit.Menu
	unBindDomNode: function(/*String|DomNode*/ nodeName){
		// summary: detach menu from given node
		var node = dojo.byId(nodeName);
		var bid = node[this.id]-1, b = this._bindings[bid];
		dojo.forEach(b, dojo.disconnect);
		delete this._bindings[bid];
	},
   
   // This method mostly copied from dijit.Menu to provide the binding to the 
   // DOM node.  I needed to copy it so that I could properly inject the 
   // contextMenuAdapter.  I'll note changes with <kjp> tags
	_contextKey: function(e){
		this._contextMenuWithMouse = false;
		if (e.keyCode == dojo.keys.F10) {
			dojo.stopEvent(e);
			if (e.shiftKey && e.type=="keydown") {
				// FF: copying the wrong property from e will cause the system
				// context menu to appear in spite of stopEvent. Don't know
				// exactly which properties cause this effect.
				var _e = { target: e.target, pageX: e.pageX, pageY: e.pageY };
				_e.preventDefault = _e.stopPropagation = function(){};
				// IE: without the delay, focus work in "open" causes the system
				// context menu to appear in spite of stopEvent.
            //<kjp> I changed the following line from calling this_openMyself to this._openContextMenu
				window.setTimeout(dojo.hitch(this, function(){ this._openContextMenu(_e); }), 1);
			}
		}
	},

   destroy: function()
   {
   	console.log ("enter TIPContextMenu.destroy");
      dojo.forEach(this.targetNodeIds, dojo.hitch(this, this.unBindDomNode));
      if (this.cmProvider !== null)
      {
         this.cmProvider.destroy();
         this.cmProvider = null;
      }
   }
});


/**
 *  You provide a context menu provider to do things like populate the context menu with
 *  items, and to handle the actions when one of those items are clicked.
 *  You provide:
 *  - getMenuItemsCallback (required)   (optional alternative: override getMenuItems)
 *      Parameters: (context)
 *                 context (that was passed on the contextMenuOpen call (in your code)
 *
 *      Summary: This callback provides the menu items to be inserted into the context menu.          
 *      The following defines the callback that is used to return menu items to appear
 *      in the list.  This method should return JSON data in the form:
 *       [ 
 *       { 
 *            "label": "testAction",
 *            "iconClass": "tipMenuIcon tipPropertiesIcon", 
 *            "actionID": "testActionID" 
 *       },
 *       { 
 *            "actionID": "_separator_" 
 *       },
 *       { 
 *            "label": "submenu", 
 *            "iconClass": "tipMenuIcon", 
 *            "submenu_items": 
 *            [ 
 *             { 
 *                  "label": "Delete", 
 *                 "iconClass": "tipMenuIcon tipDeleteIcon", 
 *                 "actionID": "testDeleteActionID" 
 *             }
 *            ] 
 *       } 
 *       ]
 *
 *  - getMenuCommandCallback (optional)  (optional alternative: override getMenuCommand)
 *      Parameters: (commandName, context, menuItem)
 *                 commandName - the command name of the context menu item.  Provided on the
 *                               'actionID' item above. 
 *                 context - (that was passed on the contextMenuOpen call (in your code)
 *                 menuItem - the actual menu item object.
 *      This callback provides the function needed to process a menu item click.
 *      The response depends largely on the action you wish to take.  A sample response:
 *
 *      {
 *            "launchType":"JavaScript",
 *            "uri":"alert('Hello World');"
 *      }
 *      Launch types include: JavaScript, WEB_URL, CLIENT_EVENT, PORTLET, PORTAL_PAGE, NOOP
 *
 *      The actions listed above may be included in the original information returned by 
 *      the getMenuItemsCallback.  If these actions are included, you need not specify
 *      a getMenuCommandCallback, and the provider will process these actions for you
 *      automatically.  For example:
 *       { 
 *            "label": "testAction",
 *            "iconClass": "tipMenuIcon tipPropertiesIcon",
 *            "actionID": "testActionID", 
 *            "launchType":"JavaScript",
 *            "uri":"alert('Hello World');"
 *       },
 */
dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenuProvider", null,
{
   iscPortletNamespace: null,   // the namespace of this UI's portlet
                                // (must be set to enable launching of portlet or portlet page from a UI in
                                // a non-iWidget portlet)
   
   iscPortletInstanceID: null,  // the instance ID of this UI's portlet (also called "oid" in portletData)
                                // (must be set to enable launching of portlet or portlet page from a UI in
                                // an iWidget)
   
   is_iWidget: false,  // indicates if this menu provider is being used on a UI in an iWidget
   
   eventFormat: "mixed", // Event context format: "new"=new context, "old"=old context, "mixed"=new and old
   
   iWidgetIDPrefix: "_iw",  // prefix needed for iscPortletInstanceID if UI is an iWidget

   getMenuItemsCallback: null,
   
   getMenuCommandCallback: null,
   
   launchNewWindowOnNullId :true,

   menus_: null, // List of menus saved from getMenuItems

   constructor: function( args ) 
   {
      // summary:
      //      The base class for all providers that populate the TIPContextMenu
      dojo.mixin( this, args );

      this.id_ = "1_";
   },

   getMenuItems: function(context)
   {
      this.menus_ = this.getMenuItemsCallback(context);
      return this.menus_;
   },

   getItem: function(/*String*/ id) 
   {
      var submenus = [];
      var currentMenu = null;
      var x = null;
      submenus.push(this.menus_);
      
      currentMenu = submenus.pop();
      while (currentMenu)
      {
         x = currentMenu.pop();
         while (x)
         {
            if (x) 
            {
               if (x.submenu_items)
               {
                  submenus.push(x.submenu_items);
               }
               else
               {
                  if (x.id == id || x.actionID == id)
                  {
                     return x;
                  }
               }
            }
            x = currentMenu.pop();
         }
         currentMenu = submenus.pop();
      }
   },
   
   isArray: function (obj) 
   {
      if (obj.constructor.toString().indexOf("Array") == -1) 
      {
         return false;
      } else {
         return true;      
      }
   },

   getMenuCommand: function (commandName, context, menuItem) 
   {
      var result = null;
      if (this.getMenuCommandCallback !== null) 
      {
         result = this.getMenuCommandCallback(commandName, context, menuItem);
      }
      else
      {
         result = this.getItem(commandName);
      }

      // console.debug("------------ handleMenuAction result",result);
      this.handleMenuAction(result);
   },

   handleMenuAction: function (data, ioArgs) 
   {
      // console.info ("handleMenuAction: ");
      // console.info (data);

      if (data.launchType == "JavaScript") {
         this.handleJavascript (data);

      } else if (data.launchType == "WEB_URL") {
         this.launchURL (data);

      } else if (data.launchType == "CLIENT_EVENT") {
         this.sendEvent (data);

      } else if (data.launchType == "PORTLET") {
         this.launchPortlet (data);

      } else if (data.launchType == "PORTAL_PAGE") {
         this.launchPage(data);

      } else if (data.launchType == "NOOP") {
         // Do nothing in the case of a no operation type

      } else {
         console.error ("unsupported launchtype: " + data.launchType);
      }
   },

   handleJavascript: function (data) 
   {
      // console.log ("enter handleJavascript");
      var fnc = new Function ("eval (\"" + data.uri + "\");");
      fnc();
   },

   launchURL: function(data) 
   {
	   console.log ("enter launchURL id ");
		  console.info("browserwindowid :"+data.browserwindowid + " launchNewWindowOnNullId :"+this.launchNewWindowOnNullId);
		 var width  = 300;
		 var height = 200;
		 var left   = (screen.width  )/2;
		 var top    = (screen.height)/2;
		 var params = 'top='+top+', left='+left;
		
		 if(data.browserwindowid != null) {
		   window.open(data.uri,data.browserwindowid, params);
		 }
		 else {
		   if(this.launchNewWindowOnNullId == true) {
		      window.open(data.uri,"_blank", params);
		   }
		   else {
		      window.open(data.uri,"_self", params);
		   
		   }
		   
		 }
   },

   launchPortlet: function (data) 
   {
	   var portletEvent = {
                  name: "http://ibm.com/isclite#launchPortlet",
                  portletName: data.portletName,
                  portletApplicationID: data.portletAppUID
               };
	      
	   if (data.parameters)
	   {
		   for (var j=0; j<data.parameters.length; j++)
		   {
			   portletEvent[data.parameters[j].name] = data.parameters[j].value.toString();
		   }
	   }
	      
	   try
	   {
		   EventBroker.sendEvent (this.iscPortletNamespace, this.formatOutboundEvent(portletEvent));
	   } catch (e) {
		   console.error ("Caught javascript exception in TIPContextMenuProvider.launchPortlet: " + e);
		   alert (e);
	   }		  
   },
   
   launchPage: function (data) 
   {
	  if (this.is_iWidget)
	  {
		  // Get the TIP iWidget event service
		  var eventService = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.iwidget.services.EventService.SERVICE_NAME);
		  var eventArray = new Array();
		  var eventData = {};
		  eventData.targetEvent = (("event" in data) && (data.event != null)) ? data.event : "http://ibm.com/isclite#launchPage"; 

		  // Set up the launch parameters as the payload
		  eventData.payload = {};
		  eventData.payloadType = "String";
		  if (data.parameters)
		  {
			  for (var k=0; k<data.parameters.length; k++)
			  {
				  eventData.payload[data.parameters[k].name] = data.parameters[k].value.toString();
			  }
		  }
		  eventData.payload = this.formatOutboundEvent(eventData.payload);
		  eventArray.push(eventData);
		  // Make sure the portletInstanceID has the iWidget prefix
		  var portletInstanceID = this.iscPortletInstanceID;
		  if (portletInstanceID && (portletInstanceID.indexOf(this.iWidgetIDPrefix) == 0))
		  {
			  portletInstanceID = this.iWidgetIDPrefix + portletInstanceID;
		  }
		  try 
		  {
			  eventService.broadcastEvents(eventArray, this.iscPortletInstanceID, data.portletPageID);			  
		  } catch (ex) {
			  console.error ("Caught javascript exception in TIPContextMenuProvider.launchPage: " + ex);
			  alert (ex);
		  }		  
	  }
	  else
	  {
		  var pageEvent = {
                         name: "http://ibm.com/isclite#launchPage",
                         NavigationNode: data.portletPageID,
                         pageInstanceRef: null,
                         switchPage: true
          };
		  
		  if (data.parameters)
		  {
			  for (var k=0; k<data.parameters.length; k++)
			  {
				  pageEvent[data.parameters[k].name] = data.parameters[k].value.toString();
			  }
		  }
		  
		  try
		  {
			  EventBroker.sendEvent (this.iscPortletNamespace, this.formatOutboundEvent(pageEvent));
		  } catch (ex) {
			  console.error ("Caught javascript exception in TIPContextMenuProvider.launchPage: " + ex);
			  alert (ex);
		  }
	  }
   },

   /* Formats an iWidget payload or porlet event.  The input parameter will be the   */
   /* entire event for portlets, and it will be the payload for iwidgets.            */
   /* This method will format the context according to the flag set in this provider */
   formatOutboundEvent: function(payload)
   {
      var newPayload=payload;
      if (this.eventFormat != "old"){ // CMS builds old context by default
         var util = new ibm.tivoli.tip.dojo.EventUtil();
         if (this.eventFormat == "mixed"){
            util.addNewContext(newPayload);
         } else if (this.eventFormat == "new"){
            newPayload={};
            if (typeof payload.name != "undefined"){ // Portlets have the event name coded in
               newPayload.name = payload.name;
            }
            if (typeof payload.source != "undefined"){ // Portlets have the event name coded in
               newPayload.source = payload.source;
            }
            newPayload.payload={};
            util.addNewContext(payload,newPayload.payload);
         }
      }
      return newPayload;
   },
   
   sendEvent: function (data) 
   {
   
   	  var standardEvent = this.createCMSEventCallback ? this.createCMSEventCallback(data) : null;   	  
   
      var parms = data.parameters;
	  if (this.is_iWidget)
	  {
		  // Get the TIP iWidget event service.  aasinha : the below service would not be required. This refers to 
		  // another overloaded fireEvent function which needs source and taget widget IDs.
		  // var eventService = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.iwidget.services.EventService.SERVICE_NAME);
		  var iwId=this.iWidgetIDPrefix + this.iscPortletInstanceID;
		  var iWidgetContainer = com.ibm.mashups.iwidget.model.Factory.getGlobalWidgetModel();
		  var widgetObj = iWidgetContainer.getWidgetById(iwId);
		  if(widgetObj == null) {
			widgetObj = iWBridge.findIframedWidget(iwId);
		  }
		  var widget_iContext = widgetObj.iScope.iContext;
		  if(!standardEvent){
		  	var payload = {};
		  	// Set up the event parameters as the payload
		  	if (parms)
		  	{
			  	for (var i=0; i<parms.length; i++)
			  	{
				  	payload[parms[i].name] = parms[i].value.toString();
			  	}
		  	}
			//	eventService.fireEvent(data.event, "String", this.formatOutboundEvent(payload));  aasinha1 : this call goes to another method which needs source and target widget.
			widget_iContext.iEvents.fireEvent(data.event, "String", this.formatOutboundEvent(payload));
		  }else{
		  	//	eventService.fireEvent(standardEvent.name, "String", this.formatOutboundEvent(standardEvent.payload));
			widget_iContext.iEvents.fireEvent(standardEvent.name, "String", this.formatOutboundEvent(standardEvent.payload));
		  }			  
	  }
	  else
	  {
	  	if(!standardEvent){
		  var event = {
				  name: data.event
                  };

		  if (parms)
		  {
			  for (var i=0; i<parms.length; i++) 
			  {
				  event[parms[i].name] = parms[i].value.toString();
			  }
		  }		  
		  // console.log ("TIPContextMenu: sending event: ");
		  // console.log (event);
		  EventBroker.sendEvent (this.iscPortletNamespace, this.formatOutboundEvent(event));
		  }else{
		  	EventBroker.sendEvent (this.iscPortletNamespace, this.formatOutboundEvent(standardEvent));
		  }
	  }
   },

   destroy: function()
   {
   		this.inherited (arguments);
      // Placeholder: for overrides.  Exploiters should call this as best practice. 
   }
      
});

/**
 *  The CMSMenuProvider is a specialized menu provider for use with the 
 *  context menu service.  Certainly you may use this provider if not using CMS,
 *  but it returns extra data that is used for CMS.  The CMS provider will
 *  make requests to the server to obtain both the menu items as well as the
 *  menu commands.
 *
 *  The CMSMenuProvider expects the following to be included in the context 
 *  passed to contextMenuOpen
 *  {
 *     selectedData: <the selection data>
 *  }
 */
dojo.declare("ibm.tivoli.tip.dijit.CMSMenuProvider", 
             ibm.tivoli.tip.dijit.TIPContextMenuProvider, 
{
   global: null,
   rcfItems: null, 
   menuName: null, 
   parentid: null,
   // This menu uses the same getCMSMenus command, but on the server, we use a different
   // data provider for this version of the CMS menu provider.
   getMenusCommandName: "getCMSMenus",
   getMenuActionCommandName: "getMenuCommand",
   
   
   // When the following is true, then this provider will first look 
   // at the menu items for a launchType, and if one exists, it will
   // execute that action here instead of going back to the host.
   hasLocalActions: false, 

	/* Default timeout for requests made to the server */
	requestTimeout: 30000,

   getIdPrefix: function() 
   {
       return this.id_;
   },
   
   setParentId: function (parentid) 
   {
      this.parentid = parentid;
   },
   
   getParentId: function ()
   {
     return this.parentid;
   },
   
   setResourceContextFilters: function (rcfItems) 
   {
      this.rcfItems = rcfItems;
   },
   
   getRCFURL: function (kw) 
   {
      if (this.rcfItems) 
      {
         for (var i in this.rcfItems) 
         {
            if (i) 
            {
               for (var name in this.rcfItems[i]) 
               {
                  if (name) 
                  {
                       kw.content["rcf_" + name]=this.rcfItems[i][name];
                  }
               }
            }
         }
      }   
   },

   getMenuItems: function(context) 
   {
   	console.log ("enter getMenuItems with context: ");
   	console.log (context);
   	
      var kw = null;

      kw = {
              sync:         true,
              noBusyDialog: true,
              content:      {
                               command:              this.getMenusCommandName,
                               arg_idPrefix:         this.getIdPrefix(),
                               arg_portletNamespace: this.global.namespace,
                               arg_parentId:         this.parentid
                            },
              timeout:      this.requestTimeout,
              onResponse:   {
                               callback: dojo.hitch( this, function( response ) { this.menus_ = response; } )
                            }
            };

      this.dataToURL (context.selectedData,kw);
      
      this.getRCFURL(kw);
         
      if (this.menuName) 
      {
         kw.content.arg_menuName = this.menuName;
      }
      
      ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );

      return this.menus_;
   },
   
   getMenuCommand: function (commandName, context, menuItem) 
   {
      var result = null;
      
      if (this.getMenuCommandCallback !== null) 
      {
         result = this.getMenuCommandCallback(commandName, context, menuItem);
         this.handleMenuAction(result);
      }
      else
      {
         this.getMenuCommandNoCallback(commandName, context, menuItem);
      }
   },

   getMenuCommandNoCallback: function (commandName, context, menuItem) 
   {
      var actionHandled = false;
      if (this.hasLocalActions === true)
      {
         // Check if the menu item contains a launchType.  If so,
         // handle it locally.
         var result = this.getItem(commandName);
         if (result && result.launchType)  // FIXME: look for something other than launch type
         {
             actionHandled = true;
             this.handleMenuAction(result);
         }
      }
      
      if (actionHandled === false)
      {
         this.getMenuCommandFromServer(commandName, context, menuItem);
      }
   },

   getMenuCommandFromServer: function (commandName, context, menuItem) 
   {
      var kw = {
                  content:   {
                                command: this.getMenuActionCommandName,
                                arg_menuCommandName: commandName,
                                arg_idPrefix: this.getIdPrefix(),
                                arg_portletNamespace: this.global.namespace,
                                arg_parentId: this.parentid
                             },
                  timeout:   this.requestTimeout,
                  onResponse:{
                                callback: dojo.hitch(this, this.handleMenuAction)
                             }
               };
               
       this.dataToURL (context.selectedData,kw);
       this.getRCFURL(kw);
       
       if (this.menuName) 
       {
          kw.content.arg_menuName = this.menuName;
       }
      
       ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
   },   
   
   dataToURL: function (selectedData, kw) 
   {
   	  console.log ("enter dataToURL: " + selectedData);
      // if you get an exception here, it means you did not provide
      // selection data in your context.  I'll put a catch of the 
      // exception here because I bet this happens a lot.
      var columns = null; 
      try 
      {
         columns = selectedData.columns;
      } catch (ex) {
         console.error ("Caught javascript exception in cmsContextMenuProvider.dataToURL: " + ex);
         console.error ("You must provide 'selectedData' on your context.");
         alert (ex);
      }
      var rows = selectedData.rows;
      var row = null;
      if (this.isArray (rows)) {
         row = rows[0];
      } else {
         row = rows;
      }
      
      for (var i=0; i<columns.length; i++) 
      {
         var rowField = columns[i].field;
         var value = row[columns[i].field];

         //if (!value) //value that of numberic type and takes 0
         if (value==undefined || value==null)
         {
            value = "";
         }

         if (columns[i].isNamingAttribute === "true") 
         {
            kw.content["rn_"+columns[i].field]=value;
         } else {
            kw.content["sv_"+columns[i].field]=value;
         }
      }
         
      if (row.id) 
      {
        kw.content.arg_rowId = row.id;
      }
   }
   
});

/**
 *  The CMSMenuProviderV2 is an advanced version of the CMSMenuProvider.
 *  This provider allows for resource context fitlers for multiple resources.
 *  It is recommended that you use this version of the CMS menu provider unless you
 *  have some very specific needs.
 *
 * {
 *  rcfs: 
 *  [
 *     [ // per resource
 *       { name : "SourceToken", value : "1.1", isNamingAttribute : false},
 *       { name : "Signature", value : "192.168.1.1", isNamingAttribute : true}
 *     ],
 *     [
 *       { name : "SourceToken", value : "1.2"},
 *       { name : "Signature", value : "192.168.1.2", isNamingAttribute : true }
 *     ]
 *  ],
 *  launchTypes: ["WEB_URL", "PORTAL_PAGE"],
 *  excludedAppIDs: [ { name: "app1", indicator: "isLaunchingProductAndExcludeMyMenus" }, { name: "app2"} ],
 *  substitutionVariables: [ {name: "foo", value: "bar"} ],
 *  options: [ {name: "option", value: "optval"} ]
 * } 
 */
dojo.declare("ibm.tivoli.tip.dijit.CMSMenuProviderV2", 
             ibm.tivoli.tip.dijit.CMSMenuProvider, 
{
   // This menu uses the same getCMSMenus command, but on the server, we use a different
   // data provider for this version of the CMS menu provider.
   getMenusCommandName: "getCMSMenus",

   getMenuActionCommandName: "getCMSAction",

   // Note that this method should not be called from an outside program.  Here we override the base
   // implmementation of this just in case to help with debugging.
   getRCFURL: function (kw) 
   {
      console.error("Not valid for CMSMenuProviderV2 since we use different context.");
   },

   getMenuItems: function(context) 
   {
      var kw = null;

      kw = {
              sync:         true,
              noBusyDialog: true,
              content:      {
                               command:              this.getMenusCommandName,
                               arg_idPrefix:         this.getIdPrefix(),
                               arg_portletNamespace: this.global.namespace,
                               arg_parentId:         this.parentid,
                               arg_context: dojo.toJson(context)
                            },
              timeout:      this.requestTimeout,
              onResponse:   {
                               callback: dojo.hitch( this, function( response ) { this.menus_ = response; } )
                            }
            };

      if (this.menuName) 
      {
         kw.content.arg_menuName = this.menuName;
      }
      
      ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).post( kw );

      return this.menus_;
   },
   
   getMenuCommandFromServer: function (commandName, context, menuItem) 
   {
      var kw = {
                  content:   {
                                command: this.getMenuActionCommandName,
                                arg_menuCommandName: commandName,
                                arg_idPrefix: this.getIdPrefix(),
                                arg_portletNamespace: this.global.namespace,
                                arg_parentId: this.parentid,
                                arg_context: dojo.toJson(context)
                             },
                  timeout:   this.requestTimeout,
                  onResponse:{
                                callback: dojo.hitch(this, this.handleMenuAction)
                             }
               };
               
       if (this.menuName) 
       {
          kw.content.arg_menuName = this.menuName;
       }
      
       ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).post( kw );
   }

});

}

if(!dojo._hasResource["dijit.dijit"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.dijit"] = true;
dojo.provide("dijit.dijit");

/*=====
dijit.dijit = {
	// summary:
	//		A roll-up for common dijit methods
	// description:
	//	A rollup file for the build system including the core and common
	//	dijit files.
	//
	// example:
	// | <script type="text/javascript" src="js/dojo/dijit/dijit.js"></script>
	//
};
=====*/

// All the stuff in _base (these are the function that are guaranteed available without an explicit dojo.require)


// And some other stuff that we tend to pull in all the time anyway







}

if(!dojo._hasResource["dojox.html.metrics"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.html.metrics"] = true;
dojo.provide("dojox.html.metrics");

(function(){
	var dhm = dojox.html.metrics;

	//	derived from Morris John's emResized measurer
	dhm.getFontMeasurements = function(){
		//	summary
		//	Returns an object that has pixel equivilents of standard font size values.
		var heights = {
			'1em':0, '1ex':0, '100%':0, '12pt':0, '16px':0, 'xx-small':0, 'x-small':0,
			'small':0, 'medium':0, 'large':0, 'x-large':0, 'xx-large':0
		};
	
		if(dojo.isIE){
			//	we do a font-size fix if and only if one isn't applied already.
			//	NOTE: If someone set the fontSize on the HTML Element, this will kill it.
			dojo.doc.documentElement.style.fontSize="100%";
		}
	
		//	set up the measuring node.
		var div=dojo.doc.createElement("div");
		var ds = div.style;
		ds.position="absolute";
		ds.left="-100px";
		ds.top="0";
		ds.width="30px";
		ds.height="1000em";
		ds.borderWidth="0";
		ds.margin="0";
		ds.padding="0";
		ds.outline="0";
		ds.lineHeight="1";
		ds.overflow="hidden";
		dojo.body().appendChild(div);
	
		//	do the measurements.
		for(var p in heights){
			ds.fontSize = p;
			heights[p] = Math.round(div.offsetHeight * 12/16) * 16/12 / 1000;
		}
		
		dojo.body().removeChild(div);
		div = null;
		return heights; 	//	object
	};

	var fontMeasurements = null;
	
	dhm.getCachedFontMeasurements = function(recalculate){
		if(recalculate || !fontMeasurements){
			fontMeasurements = dhm.getFontMeasurements();
		}
		return fontMeasurements;
	};

	var measuringNode = null, empty = {};
	dhm.getTextBox = function(/* String */ text, /* Object */ style, /* String? */ className){
		var m, s;
		if(!measuringNode){
			m = measuringNode = dojo.doc.createElement("div");
			// Container that we can set contraints on so that it doesn't
			// trigger a scrollbar.
			var c = dojo.doc.createElement("div"); 
			c.appendChild(m);
			s = c.style;
			s.overflow='scroll';
			s.position = "absolute";
			s.left = "0px";
			s.top = "-10000px";
			s.width = "1px";
			s.height = "1px";
			s.visibility = "hidden";
			s.borderWidth = "0";
			s.margin = "0";
			s.padding = "0";
			s.outline = "0";
			dojo.body().appendChild(c);
		}else{
			m = measuringNode;
		}
		// reset styles
		m.className = "";
		s = m.style;
		s.borderWidth = "0";
		s.margin = "0";
		s.padding = "0";
		s.outline = "0";
		// set new style
		if(arguments.length > 1 && style){
			for(var i in style){
				if(i in empty){ continue; }
				s[i] = style[i];
			}
		}
		// set classes
		if(arguments.length > 2 && className){
			m.className = className;
		}
		// take a measure
		m.innerHTML = text;
		var box = dojo.position(m);
		// position doesn't report right (reports 1, since parent is 1)
		// So we have to look at the scrollWidth to get the real width
		// Height is right.
		box.w = m.parentNode.scrollWidth;
		return box;
	};

	//	determine the scrollbar sizes on load.
	var scroll={ w:16, h:16 };
	dhm.getScrollbar=function(){ return { w:scroll.w, h:scroll.h }; };

	dhm._fontResizeNode = null;

	dhm.initOnFontResize = function(interval){
		var f = dhm._fontResizeNode = dojo.doc.createElement("iframe");
		var fs = f.style;
		fs.position = "absolute";
		fs.width = "5em";
		fs.height = "10em";
		fs.top = "-10000px";
		if(dojo.isIE){
			f.onreadystatechange = function(){
				if(f.contentWindow.document.readyState == "complete"){
					f.onresize = f.contentWindow.parent[dojox._scopeName].html.metrics._fontresize;
				}
			};
		}else{
			f.onload = function(){
				f.contentWindow.onresize = f.contentWindow.parent[dojox._scopeName].html.metrics._fontresize;
			};
		}
		//The script tag is to work around a known firebug race condition.  See comments in bug #9046
		f.setAttribute("src", "javascript:'<html><head><script>if(\"loadFirebugConsole\" in window){window.loadFirebugConsole();}</script></head><body></body></html>'");
		dojo.body().appendChild(f);
		dhm.initOnFontResize = function(){};
	};

	dhm.onFontResize = function(){};
	dhm._fontresize = function(){
		dhm.onFontResize();
	}

	dojo.addOnUnload(function(){
		// destroy our font resize iframe if we have one
		var f = dhm._fontResizeNode;
		if(f){
			if(dojo.isIE && f.onresize){
				f.onresize = null;
			}else if(f.contentWindow && f.contentWindow.onresize){
				f.contentWindow.onresize = null;
			}
			dhm._fontResizeNode = null;
		}
	});

	dojo.addOnLoad(function(){
		// getScrollbar metrics node
		try{
			var n=dojo.doc.createElement("div");
			n.style.cssText = "top:0;left:0;width:100px;height:100px;overflow:scroll;position:absolute;visibility:hidden;";
			dojo.body().appendChild(n);
			scroll.w = n.offsetWidth - n.clientWidth;
			scroll.h = n.offsetHeight - n.clientHeight;
			dojo.body().removeChild(n);
			//console.log("Scroll bar dimensions: ", scroll);
			delete n;
		}catch(e){}

		// text size poll setup
		if("fontSizeWatch" in dojo.config && !!dojo.config.fontSizeWatch){
			dhm.initOnFontResize();
		}
	});
})();

}

if(!dojo._hasResource["dojox.grid.util"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid.util"] = true;
dojo.provide("dojox.grid.util");

// summary: grid utility library
(function(){
	var dgu = dojox.grid.util;

	dgu.na = '...';
	dgu.rowIndexTag = "gridRowIndex";
	dgu.gridViewTag = "gridView";


	dgu.fire = function(ob, ev, args){
		var fn = ob && ev && ob[ev];
		return fn && (args ? fn.apply(ob, args) : ob[ev]());
	};
	
	dgu.setStyleHeightPx = function(inElement, inHeight){
		if(inHeight >= 0){
			var s = inElement.style;
			var v = inHeight + 'px';
			if(inElement && s['height'] != v){
				s['height'] = v;
			}
		}
	};
	
	dgu.mouseEvents = [ 'mouseover', 'mouseout', /*'mousemove',*/ 'mousedown', 'mouseup', 'click', 'dblclick', 'contextmenu' ];

	dgu.keyEvents = [ 'keyup', 'keydown', 'keypress' ];

	dgu.funnelEvents = function(inNode, inObject, inMethod, inEvents){
		var evts = (inEvents ? inEvents : dgu.mouseEvents.concat(dgu.keyEvents));
		for (var i=0, l=evts.length; i<l; i++){
			inObject.connect(inNode, 'on' + evts[i], inMethod);
		}
	};

	dgu.removeNode = function(inNode){
		inNode = dojo.byId(inNode);
		inNode && inNode.parentNode && inNode.parentNode.removeChild(inNode);
		return inNode;
	};
	
	dgu.arrayCompare = function(inA, inB){
		for(var i=0,l=inA.length; i<l; i++){
			if(inA[i] != inB[i]){return false;}
		}
		return (inA.length == inB.length);
	};
	
	dgu.arrayInsert = function(inArray, inIndex, inValue){
		if(inArray.length <= inIndex){
			inArray[inIndex] = inValue;
		}else{
			inArray.splice(inIndex, 0, inValue);
		}
	};
	
	dgu.arrayRemove = function(inArray, inIndex){
		inArray.splice(inIndex, 1);
	};
	
	dgu.arraySwap = function(inArray, inI, inJ){
		var cache = inArray[inI];
		inArray[inI] = inArray[inJ];
		inArray[inJ] = cache;
	};
})();

}

if(!dojo._hasResource["dojox.grid._Scroller"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._Scroller"] = true;
dojo.provide("dojox.grid._Scroller");

(function(){
	var indexInParent = function(inNode){
		var i=0, n, p=inNode.parentNode;
		while((n = p.childNodes[i++])){
			if(n == inNode){
				return i - 1;
			}
		}
		return -1;
	};
	
	var cleanNode = function(inNode){
		if(!inNode){
			return;
		}
		var filter = function(inW){
			return inW.domNode && dojo.isDescendant(inW.domNode, inNode, true);
		};
		var ws = dijit.registry.filter(filter);
		for(var i=0, w; (w=ws[i]); i++){
			w.destroy();
		}
		delete ws;
	};

	var getTagName = function(inNodeOrId){
		var node = dojo.byId(inNodeOrId);
		return (node && node.tagName ? node.tagName.toLowerCase() : '');
	};
	
	var nodeKids = function(inNode, inTag){
		var result = [];
		var i=0, n;
		while((n = inNode.childNodes[i])){
			i++;
			if(getTagName(n) == inTag){
				result.push(n);
			}
		}
		return result;
	};
	
	var divkids = function(inNode){
		return nodeKids(inNode, 'div');
	};

	dojo.declare("dojox.grid._Scroller", null, {
		constructor: function(inContentNodes){
			this.setContentNodes(inContentNodes);
			this.pageHeights = [];
			this.pageNodes = [];
			this.stack = [];
		},
		// specified
		rowCount: 0, // total number of rows to manage
		defaultRowHeight: 32, // default height of a row
		keepRows: 100, // maximum number of rows that should exist at one time
		contentNode: null, // node to contain pages
		scrollboxNode: null, // node that controls scrolling
		// calculated
		defaultPageHeight: 0, // default height of a page
		keepPages: 10, // maximum number of pages that should exists at one time
		pageCount: 0,
		windowHeight: 0,
		firstVisibleRow: 0,
		lastVisibleRow: 0,
		averageRowHeight: 0, // the average height of a row
		// private
		page: 0,
		pageTop: 0,
		// init
		init: function(inRowCount, inKeepRows, inRowsPerPage){
			switch(arguments.length){
				case 3: this.rowsPerPage = inRowsPerPage;
				case 2: this.keepRows = inKeepRows;
				case 1: this.rowCount = inRowCount;
				default: break;
			}
			this.defaultPageHeight = this.defaultRowHeight * this.rowsPerPage;
			this.pageCount = this._getPageCount(this.rowCount, this.rowsPerPage);
			this.setKeepInfo(this.keepRows);
			this.invalidate();
			if(this.scrollboxNode){
				this.scrollboxNode.scrollTop = 0;
				this.scroll(0);
				this.scrollboxNode.onscroll = dojo.hitch(this, 'onscroll');
			}
		},
		_getPageCount: function(rowCount, rowsPerPage){
			return rowCount ? (Math.ceil(rowCount / rowsPerPage) || 1) : 0;
		},
		destroy: function(){
			this.invalidateNodes();
			delete this.contentNodes;
			delete this.contentNode;
			delete this.scrollboxNode;
		},
		setKeepInfo: function(inKeepRows){
			this.keepRows = inKeepRows;
			this.keepPages = !this.keepRows ? this.keepPages : Math.max(Math.ceil(this.keepRows / this.rowsPerPage), 2);
		},
		// nodes
		setContentNodes: function(inNodes){
			this.contentNodes = inNodes;
			this.colCount = (this.contentNodes ? this.contentNodes.length : 0);
			this.pageNodes = [];
			for(var i=0; i<this.colCount; i++){
				this.pageNodes[i] = [];
			}
		},
		getDefaultNodes: function(){
			return this.pageNodes[0] || [];
		},
		// updating
		invalidate: function(){
			this._invalidating = true;
			this.invalidateNodes();
			this.pageHeights = [];
			this.height = (this.pageCount ? (this.pageCount - 1)* this.defaultPageHeight + this.calcLastPageHeight() : 0);
			this.resize();
			this._invalidating = false;
		},
		updateRowCount: function(inRowCount){
			this.invalidateNodes();
			this.rowCount = inRowCount;
			// update page count, adjust document height
			var oldPageCount = this.pageCount;
			if(oldPageCount === 0){
				//We want to have at least 1px in height to keep scroller.  Otherwise with an
				//empty grid you can't scroll to see the header.
				this.height = 1;
			}
			this.pageCount = this._getPageCount(this.rowCount, this.rowsPerPage);
			if(this.pageCount < oldPageCount){
				for(var i=oldPageCount-1; i>=this.pageCount; i--){
					this.height -= this.getPageHeight(i);
					delete this.pageHeights[i];
				}
			}else if(this.pageCount > oldPageCount){
				this.height += this.defaultPageHeight * (this.pageCount - oldPageCount - 1) + this.calcLastPageHeight();
			}
			this.resize();
		},
		// implementation for page manager
		pageExists: function(inPageIndex){
			return Boolean(this.getDefaultPageNode(inPageIndex));
		},
		measurePage: function(inPageIndex){
			if(this.grid.rowHeight){
				var height = this.grid.rowHeight + 1;
				return ((inPageIndex + 1) * this.rowsPerPage > this.rowCount ?
					this.rowCount - inPageIndex * this.rowsPerPage :
					this.rowsPerPage) * height;
					 
			}
			var n = this.getDefaultPageNode(inPageIndex);
			return (n && n.innerHTML) ? n.offsetHeight : undefined;
		},
		positionPage: function(inPageIndex, inPos){
			for(var i=0; i<this.colCount; i++){
				this.pageNodes[i][inPageIndex].style.top = inPos + 'px';
			}
		},
		repositionPages: function(inPageIndex){
			var nodes = this.getDefaultNodes();
			var last = 0;

			for(var i=0; i<this.stack.length; i++){
				last = Math.max(this.stack[i], last);
			}
			//
			var n = nodes[inPageIndex];
			var y = (n ? this.getPageNodePosition(n) + this.getPageHeight(inPageIndex) : 0);
			for(var p=inPageIndex+1; p<=last; p++){
				n = nodes[p];
				if(n){
					if(this.getPageNodePosition(n) == y){
						return;
					}
					this.positionPage(p, y);
				}
				y += this.getPageHeight(p);
			}
		},
		installPage: function(inPageIndex){
			for(var i=0; i<this.colCount; i++){
				this.contentNodes[i].appendChild(this.pageNodes[i][inPageIndex]);
			}
		},
		preparePage: function(inPageIndex, inReuseNode){
			var p = (inReuseNode ? this.popPage() : null);
			for(var i=0; i<this.colCount; i++){
				var nodes = this.pageNodes[i];
				var new_p = (p === null ? this.createPageNode() : this.invalidatePageNode(p, nodes));
				new_p.pageIndex = inPageIndex;
				nodes[inPageIndex] = new_p;
			}
		},
		// rendering implementation
		renderPage: function(inPageIndex){
			var nodes = [];
			var i, j;
			for(i=0; i<this.colCount; i++){
				nodes[i] = this.pageNodes[i][inPageIndex];
			}
			for(i=0, j=inPageIndex*this.rowsPerPage; (i<this.rowsPerPage)&&(j<this.rowCount); i++, j++){
				this.renderRow(j, nodes);
			}
		},
		removePage: function(inPageIndex){
			for(var i=0, j=inPageIndex*this.rowsPerPage; i<this.rowsPerPage; i++, j++){
				this.removeRow(j);
			}
		},
		destroyPage: function(inPageIndex){
			for(var i=0; i<this.colCount; i++){
				var n = this.invalidatePageNode(inPageIndex, this.pageNodes[i]);
				if(n){
					dojo.destroy(n);
				}
			}
		},
		pacify: function(inShouldPacify){
		},
		// pacification
		pacifying: false,
		pacifyTicks: 200,
		setPacifying: function(inPacifying){
			if(this.pacifying != inPacifying){
				this.pacifying = inPacifying;
				this.pacify(this.pacifying);
			}
		},
		startPacify: function(){
			this.startPacifyTicks = new Date().getTime();
		},
		doPacify: function(){
			var result = (new Date().getTime() - this.startPacifyTicks) > this.pacifyTicks;
			this.setPacifying(true);
			this.startPacify();
			return result;
		},
		endPacify: function(){
			this.setPacifying(false);
		},
		// default sizing implementation
		resize: function(){
			if(this.scrollboxNode){
				this.windowHeight = this.scrollboxNode.clientHeight;
			}
			for(var i=0; i<this.colCount; i++){
				//We want to have 1px in height min to keep scroller.  Otherwise can't scroll
				//and see header in empty grid.
				dojox.grid.util.setStyleHeightPx(this.contentNodes[i], Math.max(1,this.height));
			}
			
			// Calculate the average row height and update the defaults (row and page).
			var needPage = (!this._invalidating);
			if(!needPage){
				var ah = this.grid.attr("autoHeight");
				if(typeof ah == "number" && ah <= Math.min(this.rowsPerPage, this.rowCount)){
					needPage = true;
				}
			}
			if(needPage){
				this.needPage(this.page, this.pageTop);
			}
			var rowsOnPage = (this.page < this.pageCount - 1) ? this.rowsPerPage : ((this.rowCount % this.rowsPerPage) || this.rowsPerPage);
			var pageHeight = this.getPageHeight(this.page);
			this.averageRowHeight = (pageHeight > 0 && rowsOnPage > 0) ? (pageHeight / rowsOnPage) : 0;
		},
		calcLastPageHeight: function(){
			if(!this.pageCount){
				return 0;
			}
			var lastPage = this.pageCount - 1;
			var lastPageHeight = ((this.rowCount % this.rowsPerPage)||(this.rowsPerPage)) * this.defaultRowHeight;
			this.pageHeights[lastPage] = lastPageHeight;
			return lastPageHeight;
		},
		updateContentHeight: function(inDh){
			this.height += inDh;
			this.resize();
		},
		updatePageHeight: function(inPageIndex, fromBuild, fromAsynRendering){
			if(this.pageExists(inPageIndex)){
				var oh = this.getPageHeight(inPageIndex);
				var h = (this.measurePage(inPageIndex));
				if(h === undefined){
					h = oh;
				}
				this.pageHeights[inPageIndex] = h;
				if(oh != h){
					this.updateContentHeight(h - oh);
					var ah = this.grid.attr("autoHeight");
					if((typeof ah == "number" && ah > this.rowCount)||(ah === true && !fromBuild)){
						if(!fromAsynRendering){
							this.grid.sizeChange();
						}else{//fix #11101 by using fromAsynRendering to avoid deadlock
							var ns = this.grid.viewsNode.style;
							ns.height = parseInt(ns.height) + h - oh + 'px';
							this.repositionPages(inPageIndex);
						}
					}else{
						this.repositionPages(inPageIndex);
					}
				}
				return h;
			}
			return 0;
		},
		rowHeightChanged: function(inRowIndex, fromAsynRendering){
			this.updatePageHeight(Math.floor(inRowIndex / this.rowsPerPage), false, fromAsynRendering);
		},
		// scroller core
		invalidateNodes: function(){
			while(this.stack.length){
				this.destroyPage(this.popPage());
			}
		},
		createPageNode: function(){
			var p = document.createElement('div');
			dojo.attr(p,"role","presentation");
			p.style.position = 'absolute';
			//p.style.width = '100%';
			p.style[dojo._isBodyLtr() ? "left" : "right"] = '0';
			return p;
		},
		getPageHeight: function(inPageIndex){
			var ph = this.pageHeights[inPageIndex];
			return (ph !== undefined ? ph : this.defaultPageHeight);
		},
		// FIXME: this is not a stack, it's a FIFO list
		pushPage: function(inPageIndex){
			return this.stack.push(inPageIndex);
		},
		popPage: function(){
			return this.stack.shift();
		},
		findPage: function(inTop){
			var i = 0, h = 0;
			for(var ph = 0; i<this.pageCount; i++, h += ph){
				ph = this.getPageHeight(i);
				if(h + ph >= inTop){
					break;
				}
			}
			this.page = i;
			this.pageTop = h;
		},
		buildPage: function(inPageIndex, inReuseNode, inPos){
			this.preparePage(inPageIndex, inReuseNode);
			this.positionPage(inPageIndex, inPos);
			// order of operations is key below
			this.installPage(inPageIndex);
			this.renderPage(inPageIndex);
			// order of operations is key above
			this.pushPage(inPageIndex);
		},
		needPage: function(inPageIndex, inPos){
			var h = this.getPageHeight(inPageIndex), oh = h;
			if(!this.pageExists(inPageIndex)){
				this.buildPage(inPageIndex, (!this.grid._autoHeight/*fix #10543*/ && this.keepPages&&(this.stack.length >= this.keepPages)), inPos);
				h = this.updatePageHeight(inPageIndex, true);
			}else{
				this.positionPage(inPageIndex, inPos);
			}
			return h;
		},
		onscroll: function(){
			this.scroll(this.scrollboxNode.scrollTop);
		},
		scroll: function(inTop){
			this.grid.scrollTop = inTop;
			if(this.colCount){
				this.startPacify();
				this.findPage(inTop);
				var h = this.height;
				var b = this.getScrollBottom(inTop);
				for(var p=this.page, y=this.pageTop; (p<this.pageCount)&&((b<0)||(y<b)); p++){
					y += this.needPage(p, y);
				}
				this.firstVisibleRow = this.getFirstVisibleRow(this.page, this.pageTop, inTop);
				this.lastVisibleRow = this.getLastVisibleRow(p - 1, y, b);
				// indicates some page size has been updated
				if(h != this.height){
					this.repositionPages(p-1);
				}
				this.endPacify();
			}
		},
		getScrollBottom: function(inTop){
			return (this.windowHeight >= 0 ? inTop + this.windowHeight : -1);
		},
		// events
		processNodeEvent: function(e, inNode){
			var t = e.target;
			while(t && (t != inNode) && t.parentNode && (t.parentNode.parentNode != inNode)){
				t = t.parentNode;
			}
			if(!t || !t.parentNode || (t.parentNode.parentNode != inNode)){
				return false;
			}
			var page = t.parentNode;
			e.topRowIndex = page.pageIndex * this.rowsPerPage;
			e.rowIndex = e.topRowIndex + indexInParent(t);
			e.rowTarget = t;
			return true;
		},
		processEvent: function(e){
			return this.processNodeEvent(e, this.contentNode);
		},
		// virtual rendering interface
		renderRow: function(inRowIndex, inPageNode){
		},
		removeRow: function(inRowIndex){
		},
		// page node operations
		getDefaultPageNode: function(inPageIndex){
			return this.getDefaultNodes()[inPageIndex];
		},
		positionPageNode: function(inNode, inPos){
		},
		getPageNodePosition: function(inNode){
			return inNode.offsetTop;
		},
		invalidatePageNode: function(inPageIndex, inNodes){
			var p = inNodes[inPageIndex];
			if(p){
				delete inNodes[inPageIndex];
				this.removePage(inPageIndex, p);
				cleanNode(p);
				p.innerHTML = '';
			}
			return p;
		},
		// scroll control
		getPageRow: function(inPage){
			return inPage * this.rowsPerPage;
		},
		getLastPageRow: function(inPage){
			return Math.min(this.rowCount, this.getPageRow(inPage + 1)) - 1;
		},
		getFirstVisibleRow: function(inPage, inPageTop, inScrollTop){
			if(!this.pageExists(inPage)){
				return 0;
			}
			var row = this.getPageRow(inPage);
			var nodes = this.getDefaultNodes();
			var rows = divkids(nodes[inPage]);
			for(var i=0,l=rows.length; i<l && inPageTop<inScrollTop; i++, row++){
				inPageTop += rows[i].offsetHeight;
			}
			return (row ? row - 1 : row);
		},
		getLastVisibleRow: function(inPage, inBottom, inScrollBottom){
			if(!this.pageExists(inPage)){
				return 0;
			}
			var nodes = this.getDefaultNodes();
			var row = this.getLastPageRow(inPage);
			var rows = divkids(nodes[inPage]);
			for(var i=rows.length-1; i>=0 && inBottom>inScrollBottom; i--, row--){
				inBottom -= rows[i].offsetHeight;
			}
			return row + 1;
		},
		findTopRow: function(inScrollTop){
			var nodes = this.getDefaultNodes();
			var rows = divkids(nodes[this.page]);
			for(var i=0,l=rows.length,t=this.pageTop,h; i<l; i++){
				h = rows[i].offsetHeight;
				t += h;
				if(t >= inScrollTop){
					this.offset = h - (t - inScrollTop);
					return i + this.page * this.rowsPerPage;
				}
			}
			return -1;
		},
		findScrollTop: function(inRow){
			var rowPage = Math.floor(inRow / this.rowsPerPage);
			var t = 0;
			var i, l;
			for(i=0; i<rowPage; i++){
				t += this.getPageHeight(i);
			}
			this.pageTop = t;
			this.page = rowPage;//fix #10543
			this.needPage(rowPage, this.pageTop);

			var nodes = this.getDefaultNodes();
			var rows = divkids(nodes[rowPage]);
			var r = inRow - this.rowsPerPage * rowPage;
			for(i=0,l=rows.length; i<l && i<r; i++){
				t += rows[i].offsetHeight;
			}
			return t;
		},
		dummy: 0
	});
})();

}

if(!dojo._hasResource["dojox.grid.cells._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid.cells._base"] = true;
dojo.provide("dojox.grid.cells._base");




dojo.declare("dojox.grid._DeferredTextWidget", dijit._Widget, {
	deferred: null,
	_destroyOnRemove: true,
	postCreate: function(){
		if(this.deferred){
			this.deferred.addBoth(dojo.hitch(this, function(text){
				if(this.domNode){
					this.domNode.innerHTML = text;
				}
			}));
		}
	}
});

(function(){
	var focusSelectNode = function(inNode){
		try{
			dojox.grid.util.fire(inNode, "focus");
			dojox.grid.util.fire(inNode, "select");
		}catch(e){// IE sux bad
		}
	};
	
	var whenIdle = function(/*inContext, inMethod, args ...*/){
		setTimeout(dojo.hitch.apply(dojo, arguments), 0);
	};

	var dgc = dojox.grid.cells;

	dojo.declare("dojox.grid.cells._Base", null, {
		// summary:
		//	Respresents a grid cell and contains information about column options and methods
		//	for retrieving cell related information.
		//	Each column in a grid layout has a cell object and most events and many methods
		//	provide access to these objects.
		styles: '',
		classes: '',
		editable: false,
		alwaysEditing: false,
		formatter: null,
		defaultValue: '...',
		value: null,
		hidden: false,
		noresize: false,
		draggable: true,
		//private
		_valueProp: "value",
		_formatPending: false,

		constructor: function(inProps){
			this._props = inProps || {};
			dojo.mixin(this, inProps);
			if(this.draggable === undefined){
				this.draggable = true;
			}
		},

		_defaultFormat: function(inValue, callArgs){
			var s = this.grid.formatterScope || this;
			var f = this.formatter;
			if(f && s && typeof f == "string"){
				f = this.formatter = s[f];
			}
			var v = (inValue != this.defaultValue && f) ? f.apply(s, callArgs) : inValue;
			if(typeof v == "undefined"){
				return this.defaultValue;
			}
			if(v && v.addBoth){
				// Check if it's a deferred
				v = new dojox.grid._DeferredTextWidget({deferred: v},
									dojo.create("span", {innerHTML: this.defaultValue}));
			}
			if(v && v.declaredClass && v.startup){
				return "<div class='dojoxGridStubNode' linkWidget='" +
						v.id +
						"' cellIdx='" +
						this.index +
						"'>" +
						this.defaultValue +
						"</div>";
			}
			return v;
		},
		
		// data source
		format: function(inRowIndex, inItem){
			// summary:
			//	provides the html for a given grid cell.
			// inRowIndex: int
			// grid row index
			// returns: html for a given grid cell
			var f, i=this.grid.edit.info, d=this.get ? this.get(inRowIndex, inItem) : (this.value || this.defaultValue);
			d = (d && d.replace && this.grid.escapeHTMLInData) ? d.replace(/&/g, '&amp;').replace(/</g, '&lt;') : d;
			if(this.editable && (this.alwaysEditing || (i.rowIndex==inRowIndex && i.cell==this))){
				return this.formatEditing(d, inRowIndex);
			}else{
				return this._defaultFormat(d, [d, inRowIndex, this]);
			}
		},
		formatEditing: function(inDatum, inRowIndex){
			// summary:
			//	formats the cell for editing
			// inDatum: anything
			//	cell data to edit
			// inRowIndex: int
			//	grid row index
			// returns: string of html to place in grid cell
		},
		// utility
		getNode: function(inRowIndex){
			// summary:
			//	gets the dom node for a given grid cell.
			// inRowIndex: int
			// grid row index
			// returns: dom node for a given grid cell
			return this.view.getCellNode(inRowIndex, this.index);
		},
		getHeaderNode: function(){
			return this.view.getHeaderCellNode(this.index);
		},
		getEditNode: function(inRowIndex){
			return (this.getNode(inRowIndex) || 0).firstChild || 0;
		},
		canResize: function(){
			var uw = this.unitWidth;
			return uw && (uw!=='auto');
		},
		isFlex: function(){
			var uw = this.unitWidth;
			return uw && dojo.isString(uw) && (uw=='auto' || uw.slice(-1)=='%');
		},
		// edit support
		applyEdit: function(inValue, inRowIndex){
			this.grid.edit.applyCellEdit(inValue, this, inRowIndex);
		},
		cancelEdit: function(inRowIndex){
			this.grid.doCancelEdit(inRowIndex);
		},
		_onEditBlur: function(inRowIndex){
			if(this.grid.edit.isEditCell(inRowIndex, this.index)){
				//console.log('editor onblur', e);
				this.grid.edit.apply();
			}
		},
		registerOnBlur: function(inNode, inRowIndex){
			if(this.commitOnBlur){
				dojo.connect(inNode, "onblur", function(e){
					// hack: if editor still thinks this editor is current some ms after it blurs, assume we've focused away from grid
					setTimeout(dojo.hitch(this, "_onEditBlur", inRowIndex), 250);
				});
			}
		},
		//protected
		needFormatNode: function(inDatum, inRowIndex){
			this._formatPending = true;
			whenIdle(this, "_formatNode", inDatum, inRowIndex);
		},
		cancelFormatNode: function(){
			this._formatPending = false;
		},
		//private
		_formatNode: function(inDatum, inRowIndex){
			if(this._formatPending){
				this._formatPending = false;
				// make cell selectable
				dojo.setSelectable(this.grid.domNode, true);
				this.formatNode(this.getEditNode(inRowIndex), inDatum, inRowIndex);
			}
		},
		//protected
		formatNode: function(inNode, inDatum, inRowIndex){
			// summary:
			//	format the editing dom node. Use when editor is a widget.
			// inNode: dom node
			// dom node for the editor
			// inDatum: anything
			//	cell data to edit
			// inRowIndex: int
			//	grid row index
			if(dojo.isIE){
				// IE sux bad
				whenIdle(this, "focus", inRowIndex, inNode);
			}else{
				this.focus(inRowIndex, inNode);
			}
		},
		dispatchEvent: function(m, e){
			if(m in this){
				return this[m](e);
			}
		},
		//public
		getValue: function(inRowIndex){
			// summary:
			//	returns value entered into editor
			// inRowIndex: int
			// grid row index
			// returns:
			//	value of editor
			return this.getEditNode(inRowIndex)[this._valueProp];
		},
		setValue: function(inRowIndex, inValue){
			// summary:
			//	set the value of the grid editor
			// inRowIndex: int
			// grid row index
			// inValue: anything
			//	value of editor
			var n = this.getEditNode(inRowIndex);
			if(n){
				n[this._valueProp] = inValue;
			}
		},
		focus: function(inRowIndex, inNode){
			// summary:
			//	focus the grid editor
			// inRowIndex: int
			// grid row index
			// inNode: dom node
			//	editor node
			focusSelectNode(inNode || this.getEditNode(inRowIndex));
		},
		save: function(inRowIndex){
			// summary:
			//	save editor state
			// inRowIndex: int
			// grid row index
			this.value = this.value || this.getValue(inRowIndex);
			//console.log("save", this.value, inCell.index, inRowIndex);
		},
		restore: function(inRowIndex){
			// summary:
			//	restore editor state
			// inRowIndex: int
			// grid row index
			this.setValue(inRowIndex, this.value);
			//console.log("restore", this.value, inCell.index, inRowIndex);
		},
		//protected
		_finish: function(inRowIndex){
			// summary:
			//	called when editing is completed to clean up editor
			// inRowIndex: int
			// grid row index
			dojo.setSelectable(this.grid.domNode, false);
			this.cancelFormatNode();
		},
		//public
		apply: function(inRowIndex){
			// summary:
			//	apply edit from cell editor
			// inRowIndex: int
			// grid row index
			this.applyEdit(this.getValue(inRowIndex), inRowIndex);
			this._finish(inRowIndex);
		},
		cancel: function(inRowIndex){
			// summary:
			//	cancel cell edit
			// inRowIndex: int
			// grid row index
			this.cancelEdit(inRowIndex);
			this._finish(inRowIndex);
		}
	});
	dgc._Base.markupFactory = function(node, cellDef){
		var d = dojo;
		var formatter = d.trim(d.attr(node, "formatter")||"");
		if(formatter){
			cellDef.formatter = dojo.getObject(formatter)||formatter;
		}
		var get = d.trim(d.attr(node, "get")||"");
		if(get){
			cellDef.get = dojo.getObject(get);
		}
		var getBoolAttr = function(attr, cell, cellAttr){
			var value = d.trim(d.attr(node, attr)||"");
			if(value){ cell[cellAttr||attr] = !(value.toLowerCase()=="false"); }
		};
		getBoolAttr("sortDesc", cellDef);
		getBoolAttr("editable", cellDef);
		getBoolAttr("alwaysEditing", cellDef);
		getBoolAttr("noresize", cellDef);
		getBoolAttr("draggable", cellDef);

		var value = d.trim(d.attr(node, "loadingText")||d.attr(node, "defaultValue")||"");
		if(value){
			cellDef.defaultValue = value;
		}

		var getStrAttr = function(attr, cell, cellAttr){
			var value = d.trim(d.attr(node, attr)||"")||undefined;
			if(value){ cell[cellAttr||attr] = value; }
		};
		getStrAttr("styles", cellDef);
		getStrAttr("headerStyles", cellDef);
		getStrAttr("cellStyles", cellDef);
		getStrAttr("classes", cellDef);
		getStrAttr("headerClasses", cellDef);
		getStrAttr("cellClasses", cellDef);
	};

	dojo.declare("dojox.grid.cells.Cell", dgc._Base, {
		// summary
		// grid cell that provides a standard text input box upon editing
		constructor: function(){
			this.keyFilter = this.keyFilter;
		},
		// keyFilter: RegExp
		//		optional regex for disallowing keypresses
		keyFilter: null,
		formatEditing: function(inDatum, inRowIndex){
			this.needFormatNode(inDatum, inRowIndex);
			return '<input class="dojoxGridInput" type="text" value="' + inDatum + '">';
		},
		formatNode: function(inNode, inDatum, inRowIndex){
			this.inherited(arguments);
			// FIXME: feels too specific for this interface
			this.registerOnBlur(inNode, inRowIndex);
		},
		doKey: function(e){
			if(this.keyFilter){
				var key = String.fromCharCode(e.charCode);
				if(key.search(this.keyFilter) == -1){
					dojo.stopEvent(e);
				}
			}
		},
		_finish: function(inRowIndex){
			this.inherited(arguments);
			var n = this.getEditNode(inRowIndex);
			try{
				dojox.grid.util.fire(n, "blur");
			}catch(e){}
		}
	});
	dgc.Cell.markupFactory = function(node, cellDef){
		dgc._Base.markupFactory(node, cellDef);
		var d = dojo;
		var keyFilter = d.trim(d.attr(node, "keyFilter")||"");
		if(keyFilter){
			cellDef.keyFilter = new RegExp(keyFilter);
		}
	};

	dojo.declare("dojox.grid.cells.RowIndex", dgc.Cell, {
		name: 'Row',

		postscript: function(){
			this.editable = false;
		},
		get: function(inRowIndex){
			return inRowIndex + 1;
		}
	});
	dgc.RowIndex.markupFactory = function(node, cellDef){
		dgc.Cell.markupFactory(node, cellDef);
	};

	dojo.declare("dojox.grid.cells.Select", dgc.Cell, {
		// summary:
		// grid cell that provides a standard select for editing

		// options: Array
		// 		text of each item
		options: null,

		// values: Array
		//		value for each item
		values: null,

		// returnIndex: Integer
		// 		editor returns only the index of the selected option and not the value
		returnIndex: -1,

		constructor: function(inCell){
			this.values = this.values || this.options;
		},
		formatEditing: function(inDatum, inRowIndex){
			this.needFormatNode(inDatum, inRowIndex);
			var h = [ '<select class="dojoxGridSelect">' ];
			for (var i=0, o, v; ((o=this.options[i]) !== undefined)&&((v=this.values[i]) !== undefined); i++){
				h.push("<option", (inDatum==v ? ' selected' : ''), ' value="' + v + '"', ">", o, "</option>");
			}
			h.push('</select>');
			return h.join('');
		},
		getValue: function(inRowIndex){
			var n = this.getEditNode(inRowIndex);
			if(n){
				var i = n.selectedIndex, o = n.options[i];
				return this.returnIndex > -1 ? i : o.value || o.innerHTML;
			}
		}
	});
	dgc.Select.markupFactory = function(node, cell){
		dgc.Cell.markupFactory(node, cell);
		var d=dojo;
		var options = d.trim(d.attr(node, "options")||"");
		if(options){
			var o = options.split(',');
			if(o[0] != options){
				cell.options = o;
			}
		}
		var values = d.trim(d.attr(node, "values")||"");
		if(values){
			var v = values.split(',');
			if(v[0] != values){
				cell.values = v;
			}
		}
	};

	dojo.declare("dojox.grid.cells.AlwaysEdit", dgc.Cell, {
		// summary:
		// grid cell that is always in an editable state, regardless of grid editing state
		alwaysEditing: true,
		_formatNode: function(inDatum, inRowIndex){
			this.formatNode(this.getEditNode(inRowIndex), inDatum, inRowIndex);
		},
		applyStaticValue: function(inRowIndex){
			var e = this.grid.edit;
			e.applyCellEdit(this.getValue(inRowIndex), this, inRowIndex);
			e.start(this, inRowIndex, true);
		}
	});
	dgc.AlwaysEdit.markupFactory = function(node, cell){
		dgc.Cell.markupFactory(node, cell);
	};

	dojo.declare("dojox.grid.cells.Bool", dgc.AlwaysEdit, {
		// summary:
		// grid cell that provides a standard checkbox that is always on for editing
		_valueProp: "checked",
		formatEditing: function(inDatum, inRowIndex){
			return '<input class="dojoxGridInput" type="checkbox"' + (inDatum ? ' checked="checked"' : '') + ' style="width: auto" />';
		},
		doclick: function(e){
			if(e.target.tagName == 'INPUT'){
				this.applyStaticValue(e.rowIndex);
			}
		}
	});
	dgc.Bool.markupFactory = function(node, cell){
		dgc.AlwaysEdit.markupFactory(node, cell);
	};
})();

}

if(!dojo._hasResource["dojox.grid.cells"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid.cells"] = true;
dojo.provide("dojox.grid.cells");


}

if(!dojo._hasResource["dojox.grid._Builder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._Builder"] = true;
dojo.provide("dojox.grid._Builder");




(function(){
	var dg = dojox.grid;

	var getTdIndex = function(td){
		return td.cellIndex >=0 ? td.cellIndex : dojo.indexOf(td.parentNode.cells, td);
	};
	
	var getTrIndex = function(tr){
		return tr.rowIndex >=0 ? tr.rowIndex : dojo.indexOf(tr.parentNode.childNodes, tr);
	};
	
	var getTr = function(rowOwner, index){
		return rowOwner && ((rowOwner.rows||0)[index] || rowOwner.childNodes[index]);
	};

	var findTable = function(node){
		for(var n=node; n && n.tagName!='TABLE'; n=n.parentNode){}
		return n;
	};
	
	var ascendDom = function(inNode, inWhile){
		for(var n=inNode; n && inWhile(n); n=n.parentNode){}
		return n;
	};
	
	var makeNotTagName = function(inTagName){
		var name = inTagName.toUpperCase();
		return function(node){ return node.tagName != name; };
	};

	var rowIndexTag = dojox.grid.util.rowIndexTag;
	var gridViewTag = dojox.grid.util.gridViewTag;

	// base class for generating markup for the views
	dg._Builder = dojo.extend(function(view){
		if(view){
			this.view = view;
			this.grid = view.grid;
		}
	},{
		view: null,
		// boilerplate HTML
		_table: '<table class="dojoxGridRowTable" border="0" cellspacing="0" cellpadding="0" role="presentation"',

		// Returns the table variable as an array - and with the view width, if specified
		getTableArray: function(){
			var html = [this._table];
			if(this.view.viewWidth){
				html.push([' style="width:', this.view.viewWidth, ';"'].join(''));
			}
			html.push('>');
			return html;
		},
		
		// generate starting tags for a cell
		generateCellMarkup: function(inCell, inMoreStyles, inMoreClasses, isHeader){
			var result = [], html;
			if(isHeader){
				var sortInfo = inCell.index != inCell.grid.getSortIndex() ? "" : inCell.grid.sortInfo > 0 ? 'aria-sort="ascending"' : 'aria-sort="descending"';
				if (!inCell.id){
					inCell.id = this.grid.id + "Hdr" + inCell.index;
				}
				// column headers are not editable, mark as aria-readonly=true
				html = ['<th tabIndex="-1" aria-readonly="true" role="columnheader"', sortInfo, 'id="', inCell.id, '"'];
			}else{
				// cells inherit grid aria-readonly property; default value for aria-readonly is false(grid is editable)
				// if grid is editable (had any editable cells), mark non editable cells as aria-readonly=true
				// if no editable cells, grid's aria-readonly value will have been set to true and cells will inherit
				var editInfo = this.grid.editable && !inCell.editable ? 'aria-readonly="true"' : "";
				html = ['<td tabIndex="-1" role="gridcell"', editInfo];
			}
			if(inCell.colSpan){
				html.push(' colspan="', inCell.colSpan, '"');
			}
			if(inCell.rowSpan){
				html.push(' rowspan="', inCell.rowSpan, '"');
			}
			html.push(' class="dojoxGridCell ');
			if(inCell.classes){
				html.push(inCell.classes, ' ');
			}
			if(inMoreClasses){
				html.push(inMoreClasses, ' ');
			}
			// result[0] => td opener, style
			result.push(html.join(''));
			// SLOT: result[1] => td classes 
			result.push('');
			html = ['" idx="', inCell.index, '" style="'];
			if(inMoreStyles && inMoreStyles[inMoreStyles.length-1] != ';'){
				inMoreStyles += ';';
			}
			html.push(inCell.styles, inMoreStyles||'', inCell.hidden?'display:none;':'');
			if(inCell.unitWidth){
				html.push('width:', inCell.unitWidth, ';');
			}
			// result[2] => markup
			result.push(html.join(''));
			// SLOT: result[3] => td style 
			result.push('');
			html = [ '"' ];
			if(inCell.attrs){
				html.push(" ", inCell.attrs);
			}
			html.push('>');
			// result[4] => td postfix
			result.push(html.join(''));
			// SLOT: result[5] => content
			result.push('');
			// result[6] => td closes
			result.push(isHeader?'</th>':'</td>');
			return result; // Array
		},

		// cell finding
		isCellNode: function(inNode){
			return Boolean(inNode && inNode!=dojo.doc && dojo.attr(inNode, "idx"));
		},
		
		getCellNodeIndex: function(inCellNode){
			return inCellNode ? Number(dojo.attr(inCellNode, "idx")) : -1;
		},
		
		getCellNode: function(inRowNode, inCellIndex){
			for(var i=0, row; (row=getTr(inRowNode.firstChild, i)); i++){
				for(var j=0, cell; (cell=row.cells[j]); j++){
					if(this.getCellNodeIndex(cell) == inCellIndex){
						return cell;
					}
				}
			}
			return null;
		},
		
		findCellTarget: function(inSourceNode, inTopNode){
			var n = inSourceNode;
			while(n && (!this.isCellNode(n) || (n.offsetParent && gridViewTag in n.offsetParent.parentNode && n.offsetParent.parentNode[gridViewTag] != this.view.id)) && (n!=inTopNode)){
				n = n.parentNode;
			}
			return n!=inTopNode ? n : null;
		},
		
		// event decoration
		baseDecorateEvent: function(e){
			e.dispatch = 'do' + e.type;
			e.grid = this.grid;
			e.sourceView = this.view;
			e.cellNode = this.findCellTarget(e.target, e.rowNode);
			e.cellIndex = this.getCellNodeIndex(e.cellNode);
			e.cell = (e.cellIndex >= 0 ? this.grid.getCell(e.cellIndex) : null);
		},
		
		// event dispatch
		findTarget: function(inSource, inTag){
			var n = inSource;
			while(n && (n!=this.domNode) && (!(inTag in n) || (gridViewTag in n && n[gridViewTag] != this.view.id))){
				n = n.parentNode;
			}
			return (n != this.domNode) ? n : null; 
		},

		findRowTarget: function(inSource){
			return this.findTarget(inSource, rowIndexTag);
		},

		isIntraNodeEvent: function(e){
			try{
				return (e.cellNode && e.relatedTarget && dojo.isDescendant(e.relatedTarget, e.cellNode));
			}catch(x){
				// e.relatedTarget has permission problem in FF if it's an input: https://bugzilla.mozilla.org/show_bug.cgi?id=208427
				return false;
			}
		},

		isIntraRowEvent: function(e){
			try{
				var row = e.relatedTarget && this.findRowTarget(e.relatedTarget);
				return !row && (e.rowIndex==-1) || row && (e.rowIndex==row.gridRowIndex);			
			}catch(x){
				// e.relatedTarget on INPUT has permission problem in FF: https://bugzilla.mozilla.org/show_bug.cgi?id=208427
				return false;
			}
		},

		dispatchEvent: function(e){
			if(e.dispatch in this){
				return this[e.dispatch](e);
			}
			return false;
		},

		// dispatched event handlers
		domouseover: function(e){
			if(e.cellNode && (e.cellNode!=this.lastOverCellNode)){
				this.lastOverCellNode = e.cellNode;
				this.grid.onMouseOver(e);
			}
			this.grid.onMouseOverRow(e);
		},

		domouseout: function(e){
			if(e.cellNode && (e.cellNode==this.lastOverCellNode) && !this.isIntraNodeEvent(e, this.lastOverCellNode)){
				this.lastOverCellNode = null;
				this.grid.onMouseOut(e);
				if(!this.isIntraRowEvent(e)){
					this.grid.onMouseOutRow(e);
				}
			}
		},
		
		domousedown: function(e){
			if (e.cellNode)
				this.grid.onMouseDown(e);
			this.grid.onMouseDownRow(e);
		}
	});

	// Produces html for grid data content. Owned by grid and used internally 
	// for rendering data. Override to implement custom rendering.
	dg._ContentBuilder = dojo.extend(function(view){
		dg._Builder.call(this, view);
	},dg._Builder.prototype,{
		update: function(){
			this.prepareHtml();
		},

		// cache html for rendering data rows
		prepareHtml: function(){
			var defaultGet=this.grid.get, cells=this.view.structure.cells;
			for(var j=0, row; (row=cells[j]); j++){
				for(var i=0, cell; (cell=row[i]); i++){
					cell.get = cell.get || (cell.value == undefined) && defaultGet;
					cell.markup = this.generateCellMarkup(cell, cell.cellStyles, cell.cellClasses, false);
					if (!this.grid.editable && cell.editable){
						this.grid.editable = true;
					}
				}
			}
		},

		// time critical: generate html using cache and data source
		generateHtml: function(inDataIndex, inRowIndex){
			var
				html = this.getTableArray(),
				v = this.view,
				cells = v.structure.cells,
				item = this.grid.getItem(inRowIndex);

			dojox.grid.util.fire(this.view, "onBeforeRow", [inRowIndex, cells]);
			for(var j=0, row; (row=cells[j]); j++){
				if(row.hidden || row.header){
					continue;
				}
				html.push(!row.invisible ? '<tr>' : '<tr class="dojoxGridInvisible">');
				for(var i=0, cell, m, cc, cs; (cell=row[i]); i++){
					m = cell.markup; cc = cell.customClasses = []; cs = cell.customStyles = [];
					// content (format can fill in cc and cs as side-effects)
					m[5] = cell.format(inRowIndex, item);
					// classes
					m[1] = cc.join(' ');
					// styles
					m[3] = cs.join(';');
					// in-place concat
					html.push.apply(html, m);
				}
				html.push('</tr>');
			}
			html.push('</table>');
			return html.join(''); // String
		},

		decorateEvent: function(e){
			e.rowNode = this.findRowTarget(e.target);
			if(!e.rowNode){return false;}
			e.rowIndex = e.rowNode[rowIndexTag];
			this.baseDecorateEvent(e);
			e.cell = this.grid.getCell(e.cellIndex);
			return true; // Boolean
		}
	});

	// Produces html for grid header content. Owned by grid and used internally 
	// for rendering data. Override to implement custom rendering.
	dg._HeaderBuilder = dojo.extend(function(view){
		this.moveable = null;
		dg._Builder.call(this, view);
	},dg._Builder.prototype,{
		_skipBogusClicks: false,
		overResizeWidth: 4,
		minColWidth: 1,
		
		update: function(){
			if(this.tableMap){
				this.tableMap.mapRows(this.view.structure.cells);
			}else{
				this.tableMap = new dg._TableMap(this.view.structure.cells);
			}
		},

		generateHtml: function(inGetValue, inValue){
			var html = this.getTableArray(), cells = this.view.structure.cells;
			
			dojox.grid.util.fire(this.view, "onBeforeRow", [-1, cells]);
			for(var j=0, row; (row=cells[j]); j++){
				if(row.hidden){
					continue;
				}
				html.push(!row.invisible ? '<tr>' : '<tr class="dojoxGridInvisible">');
				for(var i=0, cell, markup; (cell=row[i]); i++){
					cell.customClasses = [];
					cell.customStyles = [];
					if(this.view.simpleStructure){
						if(cell.draggable){
							if(cell.headerClasses){
								if(cell.headerClasses.indexOf('dojoDndItem') == -1){
									cell.headerClasses += ' dojoDndItem';
								}
							}else{
								cell.headerClasses = 'dojoDndItem';
							}
						}
						if(cell.attrs){
							if(cell.attrs.indexOf("dndType='gridColumn_") == -1){
								cell.attrs += " dndType='gridColumn_" + this.grid.id + "'";
							}
						}else{
							cell.attrs = "dndType='gridColumn_" + this.grid.id + "'";
						}
					}
					markup = this.generateCellMarkup(cell, cell.headerStyles, cell.headerClasses, true);
					// content
					markup[5] = (inValue != undefined ? inValue : inGetValue(cell));
					// styles
					markup[3] = cell.customStyles.join(';');
					// classes
					markup[1] = cell.customClasses.join(' '); //(cell.customClasses ? ' ' + cell.customClasses : '');
					html.push(markup.join(''));
				}
				html.push('</tr>');
			}
			html.push('</table>');
			return html.join('');
		},

		// event helpers
		getCellX: function(e){
			var n, x = e.layerX;
			if(dojo.isMoz){
				n = ascendDom(e.target, makeNotTagName("th"));
				x -= (n && n.offsetLeft) || 0;
				var t = e.sourceView.getScrollbarWidth();
				if(!dojo._isBodyLtr()/*&& e.sourceView.headerNode.scrollLeft < t*/){
					//fix #11253
					table = ascendDom(n,makeNotTagName("table"));
					x -= (table && table.offsetLeft) || 0;
				}
				//x -= getProp(ascendDom(e.target, mkNotTagName("td")), "offsetLeft") || 0;
			}
			n = ascendDom(e.target, function(){
				if(!n || n == e.cellNode){
					return false;
				}
				// Mozilla 1.8 (FF 1.5) has a bug that makes offsetLeft = -parent border width
				// when parent has border, overflow: hidden, and is positioned
				// handle this problem here ... not a general solution!
				x += (n.offsetLeft < 0 ? 0 : n.offsetLeft);
				return true;
			});
			return x;
		},

		// event decoration
		decorateEvent: function(e){
			this.baseDecorateEvent(e);
			e.rowIndex = -1;
			e.cellX = this.getCellX(e);
			return true;
		},

		// event handlers
		// resizing
		prepareResize: function(e, mod){
			do{
				var i = getTdIndex(e.cellNode);
				e.cellNode = (i ? e.cellNode.parentNode.cells[i+mod] : null);
				e.cellIndex = (e.cellNode ? this.getCellNodeIndex(e.cellNode) : -1);
			}while(e.cellNode && e.cellNode.style.display == "none");
			return Boolean(e.cellNode);
		},

		canResize: function(e){
			if(!e.cellNode || e.cellNode.colSpan > 1){
				return false;
			}
			var cell = this.grid.getCell(e.cellIndex); 
			return !cell.noresize && cell.canResize();
		},

		overLeftResizeArea: function(e){
			// We are never over a resize area if we are in the process of moving
			if(dojo.hasClass(dojo.body(), "dojoDndMove")){
				return false;
			}
			//Bugfix for crazy IE problem (#8807).  IE returns position information for the icon and text arrow divs
			//as if they were still on the left instead of returning the position they were 'float: right' to.
			//So, the resize check ends up checking the wrong adjacent cell.  This checks to see if the hover was over 
			//the image or text nodes, then just ignored them/treat them not in scale range.
			if(dojo.isIE){
				var tN = e.target;
				if(dojo.hasClass(tN, "dojoxGridArrowButtonNode") || 
					dojo.hasClass(tN, "dojoxGridArrowButtonChar")){
					return false;
				}
			}

			if(dojo._isBodyLtr()){
				return (e.cellIndex>0) && (e.cellX < this.overResizeWidth) && this.prepareResize(e, -1);
			}
			var t = e.cellNode && (e.cellX < this.overResizeWidth);
			return t;
		},

		overRightResizeArea: function(e){
			// We are never over a resize area if we are in the process of moving
			if(dojo.hasClass(dojo.body(), "dojoDndMove")){
				return false;
			}
			//Bugfix for crazy IE problem (#8807).  IE returns position information for the icon and text arrow divs
			//as if they were still on the left instead of returning the position they were 'float: right' to.
			//So, the resize check ends up checking the wrong adjacent cell.  This checks to see if the hover was over 
			//the image or text nodes, then just ignored them/treat them not in scale range.
			if(dojo.isIE){
				var tN = e.target;
				if(dojo.hasClass(tN, "dojoxGridArrowButtonNode") || 
					dojo.hasClass(tN, "dojoxGridArrowButtonChar")){
					return false;
				}
			}

			if(dojo._isBodyLtr()){
				return e.cellNode && (e.cellX >= e.cellNode.offsetWidth - this.overResizeWidth);
			}
			return (e.cellIndex>0) && (e.cellX >= e.cellNode.offsetWidth - this.overResizeWidth) && this.prepareResize(e, -1);
		},

		domousemove: function(e){
			//console.log(e.cellIndex, e.cellX, e.cellNode.offsetWidth);
			if(!this.moveable){
				var c = (this.overRightResizeArea(e) ? 'dojoxGridColResize' : (this.overLeftResizeArea(e) ? 'dojoxGridColResize' : ''));
				if(c && !this.canResize(e)){
					c = 'dojoxGridColNoResize';
				}
				dojo.toggleClass(e.sourceView.headerNode, "dojoxGridColNoResize", (c == "dojoxGridColNoResize"));
				dojo.toggleClass(e.sourceView.headerNode, "dojoxGridColResize", (c == "dojoxGridColResize"));
				if(dojo.isIE){
					var t = e.sourceView.headerNode.scrollLeft;
					e.sourceView.headerNode.scrollLeft = t;
				}
				if(c){
					dojo.stopEvent(e);
				}
			}
		},

		domousedown: function(e){
			if(!this.moveable){
				if((this.overRightResizeArea(e) || this.overLeftResizeArea(e)) && this.canResize(e)){
					this.beginColumnResize(e);
				}else{
					this.grid.onMouseDown(e);
					this.grid.onMouseOverRow(e);
				}
				//else{
				//	this.beginMoveColumn(e);
				//}
			}
		},

		doclick: function(e) {
			if(this._skipBogusClicks){
				dojo.stopEvent(e);
				return true;
			}
			return false;
		},

		// column resizing
		colResizeSetup: function(/*Event Object*/e, /*boolean*/ isMouse ){
			//Set up the drag object for column resizing
			// Called with mouse event in case of drag and drop,
			// Also called from keyboard shift-arrow event when focus is on a header
			var headContentBox = dojo.contentBox(e.sourceView.headerNode);
			
			if(isMouse){  //IE draws line even with no mouse down so separate from keyboard 
				this.lineDiv = document.createElement('div');

				// NOTE: this is for backwards compatibility with Dojo 1.3
				var vw = (dojo.position||dojo._abs)(e.sourceView.headerNode, true);
				var bodyContentBox = dojo.contentBox(e.sourceView.domNode);
				//fix #11340
				var l = e.clientX;
				if(!dojo._isBodyLtr() && dojo.isIE < 8){
					l -= dojox.html.metrics.getScrollbar().w;
				}				
				dojo.style(this.lineDiv, {
					top: vw.y + "px",
					left: l + "px",
					height: (bodyContentBox.h + headContentBox.h) + "px"
				});
				dojo.addClass(this.lineDiv, "dojoxGridResizeColLine");
				this.lineDiv._origLeft = l;
				dojo.body().appendChild(this.lineDiv);
			}
			var spanners = [], nodes = this.tableMap.findOverlappingNodes(e.cellNode);
			for(var i=0, cell; (cell=nodes[i]); i++){
				spanners.push({ node: cell, index: this.getCellNodeIndex(cell), width: cell.offsetWidth });
				//console.log("spanner: " + this.getCellNodeIndex(cell));
			}

			var view = e.sourceView;
			var adj = dojo._isBodyLtr() ? 1 : -1;
			var views = e.grid.views.views;
			var followers = [];
			for(var j=view.idx+adj, cView; (cView=views[j]); j=j+adj){
				followers.push({ node: cView.headerNode, left: window.parseInt(cView.headerNode.style.left) });
			}
			var table = view.headerContentNode.firstChild;
			var drag = {
				scrollLeft: e.sourceView.headerNode.scrollLeft,
				view: view,
				node: e.cellNode,
				index: e.cellIndex,
				w: dojo.contentBox(e.cellNode).w,
				vw: headContentBox.w,
				table: table,
				tw: dojo.contentBox(table).w,
				spanners: spanners,
				followers: followers
			};
			return drag;
		},
		beginColumnResize: function(e){
			this.moverDiv = document.createElement("div");
			dojo.style(this.moverDiv,{position: "absolute", left:0}); // to make DnD work with dir=rtl
			dojo.body().appendChild(this.moverDiv);
			dojo.addClass(this.grid.domNode, "dojoxGridColumnResizing");
			var m = (this.moveable = new dojo.dnd.Moveable(this.moverDiv));

			var drag = this.colResizeSetup(e,true);

			m.onMove = dojo.hitch(this, "doResizeColumn", drag);

			dojo.connect(m, "onMoveStop", dojo.hitch(this, function(){
				this.endResizeColumn(drag);
				if(drag.node.releaseCapture){
					drag.node.releaseCapture();
				}
				this.moveable.destroy();
				delete this.moveable;
				this.moveable = null;
				dojo.removeClass(this.grid.domNode, "dojoxGridColumnResizing");
			}));

			if(e.cellNode.setCapture){
				e.cellNode.setCapture();
			}
			m.onMouseDown(e);
		},

		doResizeColumn: function(inDrag, mover, leftTop){
			var changeX = leftTop.l;
			var data = {
				deltaX: changeX,
				w: inDrag.w + (dojo._isBodyLtr() ? changeX : -changeX),//fix #11341
				vw: inDrag.vw + changeX,
				tw: inDrag.tw + changeX
			};
			
			this.dragRecord = {inDrag: inDrag, mover: mover, leftTop:leftTop};
			
			if(data.w >= this.minColWidth){
				if (!mover) { // we are using keyboard do immediate resize
					this.doResizeNow(inDrag, data);
				}
				else{
					dojo.style(this.lineDiv, "left", (this.lineDiv._origLeft + data.deltaX) + "px");
				}
			}
		},

		endResizeColumn: function(inDrag){
			if(this.dragRecord){
				var leftTop = this.dragRecord.leftTop;
				var changeX = dojo._isBodyLtr() ? leftTop.l : -leftTop.l;
				// Make sure we are not under our minimum
				// http://bugs.dojotoolkit.org/ticket/9390
				changeX += Math.max(inDrag.w + changeX, this.minColWidth) - (inDrag.w + changeX);
				if(dojo.isWebKit && inDrag.spanners.length){
					// Webkit needs the pad border extents back in
					changeX += dojo._getPadBorderExtents(inDrag.spanners[0].node).w;
				}
				var data = {
					deltaX: changeX,
					w: inDrag.w + changeX,
					vw: inDrag.vw + changeX,
					tw: inDrag.tw + changeX
				};
				// Only resize the columns when the drag has finished
				this.doResizeNow(inDrag, data);
			}
			
			dojo.destroy(this.lineDiv);
 			dojo.destroy(this.moverDiv);
			dojo.destroy(this.moverDiv);
			delete this.moverDiv;
			this._skipBogusClicks = true;
			inDrag.view.update();
			this._skipBogusClicks = false;
			this.grid.onResizeColumn(inDrag.index);
		},
		doResizeNow: function(inDrag, data){
			inDrag.view.convertColPctToFixed();
			if(inDrag.view.flexCells && !inDrag.view.testFlexCells()){
				var t = findTable(inDrag.node);
				if(t){
					(t.style.width = '');
				}
			}
			var i, s, sw, f, fl;
			for(i=0; (s=inDrag.spanners[i]); i++){
				sw = s.width + data.deltaX;
				s.node.style.width = sw + 'px';
				inDrag.view.setColWidth(s.index, sw);
			}
			if(dojo._isBodyLtr() || !dojo.isIE){//fix #11339			
				for(i=0; (f=inDrag.followers[i]); i++){
					fl = f.left + data.deltaX;
					f.node.style.left = fl + 'px';
				}
			}
			inDrag.node.style.width = data.w + 'px';
			inDrag.view.setColWidth(inDrag.index, data.w);
			inDrag.view.headerNode.style.width = data.vw + 'px';
			inDrag.view.setColumnsWidth(data.tw);
			if(!dojo._isBodyLtr()){
				inDrag.view.headerNode.scrollLeft = inDrag.scrollLeft + data.deltaX;
			}
		}
	});

	// Maps an html table into a structure parsable for information about cell row and col spanning.
	// Used by HeaderBuilder.
	dg._TableMap = dojo.extend(function(rows){
		this.mapRows(rows);
	},{
		map: null,

		mapRows: function(inRows){
			// summary: Map table topography

			//console.log('mapRows');
			// # of rows
			var rowCount = inRows.length;
			if(!rowCount){
				return;
			}
			// map which columns and rows fill which cells
			this.map = [];
			var row;
			for(var k=0; (row=inRows[k]); k++){
				this.map[k] = [];
			}
			for(var j=0; (row=inRows[j]); j++){
				for(var i=0, x=0, cell, colSpan, rowSpan; (cell=row[i]); i++){
					while(this.map[j][x]){x++;}
					this.map[j][x] = { c: i, r: j };
					rowSpan = cell.rowSpan || 1;
					colSpan = cell.colSpan || 1;
					for(var y=0; y<rowSpan; y++){
						for(var s=0; s<colSpan; s++){
							this.map[j+y][x+s] = this.map[j][x];
						}
					}
					x += colSpan;
				}
			}
			//this.dumMap();
		},

		dumpMap: function(){
			for(var j=0, row, h=''; (row=this.map[j]); j++,h=''){
				for(var i=0, cell; (cell=row[i]); i++){
					h += cell.r + ',' + cell.c + '   ';
				}
			}
		},

		getMapCoords: function(inRow, inCol){
			// summary: Find node's map coords by it's structure coords
			for(var j=0, row; (row=this.map[j]); j++){
				for(var i=0, cell; (cell=row[i]); i++){
					if(cell.c==inCol && cell.r == inRow){
						return { j: j, i: i };
					}
					//else{console.log(inRow, inCol, ' : ', i, j, " : ", cell.r, cell.c); };
				}
			}
			return { j: -1, i: -1 };
		},
		
		getNode: function(inTable, inRow, inCol){
			// summary: Find a node in inNode's table with the given structure coords
			var row = inTable && inTable.rows[inRow];
			return row && row.cells[inCol];
		},
		
		_findOverlappingNodes: function(inTable, inRow, inCol){
			var nodes = [];
			var m = this.getMapCoords(inRow, inCol);
			//console.log("node j: %d, i: %d", m.j, m.i);
			for(var j=0, row; (row=this.map[j]); j++){
				if(j == m.j){ continue; }
				var rw = row[m.i];
				//console.log("overlaps: r: %d, c: %d", rw.r, rw.c);
				var n = (rw?this.getNode(inTable, rw.r, rw.c):null);
				if(n){ nodes.push(n); }
			}
			//console.log(nodes);
			return nodes;
		},
		
		findOverlappingNodes: function(inNode){
			return this._findOverlappingNodes(findTable(inNode), getTrIndex(inNode.parentNode), getTdIndex(inNode));
		}
	});
})();

}

if(!dojo._hasResource["dojox.grid._View"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._View"] = true;
dojo.provide("dojox.grid._View");










(function(){
	// private
	var getStyleText = function(inNode, inStyleText){
		return inNode.style.cssText == undefined ? inNode.getAttribute("style") : inNode.style.cssText;
	};

	// public
	dojo.declare('dojox.grid._View', [dijit._Widget, dijit._Templated], {
		// summary:
		//		A collection of grid columns. A grid is comprised of a set of views that stack horizontally.
		//		Grid creates views automatically based on grid's layout structure.
		//		Users should typically not need to access individual views directly.
		//
		// defaultWidth: String
		//		Default width of the view
		defaultWidth: "18em",

		// viewWidth: String
		// 		Width for the view, in valid css unit
		viewWidth: "",

		templateString:"<div class=\"dojoxGridView\" wairole=\"presentation\">\r\n\t<div class=\"dojoxGridHeader\" dojoAttachPoint=\"headerNode\" wairole=\"presentation\">\r\n\t\t<div dojoAttachPoint=\"headerNodeContainer\" style=\"width:9000em\" wairole=\"presentation\">\r\n\t\t\t<div dojoAttachPoint=\"headerContentNode\" wairole=\"row\"></div>\r\n\t\t</div>\r\n\t</div>\r\n\t<input type=\"checkbox\" class=\"dojoxGridHiddenFocus\" dojoAttachPoint=\"hiddenFocusNode\" wairole=\"presentation\" />\r\n\t<input type=\"checkbox\" class=\"dojoxGridHiddenFocus\" wairole=\"presentation\" />\r\n\t<div class=\"dojoxGridScrollbox\" dojoAttachPoint=\"scrollboxNode\" wairole=\"presentation\">\r\n\t\t<div class=\"dojoxGridContent\" dojoAttachPoint=\"contentNode\" hidefocus=\"hidefocus\" wairole=\"presentation\"></div>\r\n\t</div>\r\n</div>\r\n",
		
		themeable: false,
		classTag: 'dojoxGrid',
		marginBottom: 0,
		rowPad: 2,

		// _togglingColumn: int
		//		Width of the column being toggled (-1 for none)
		_togglingColumn: -1,
		
		// _headerBuilderClass: Object
		//		The class to use for our header builder
		_headerBuilderClass: dojox.grid._HeaderBuilder,
		
		// _contentBuilderClass: Object
		//		The class to use for our content builder
		_contentBuilderClass: dojox.grid._ContentBuilder,
		
		postMixInProperties: function(){
			this.rowNodes = {};
		},

		postCreate: function(){
			this.connect(this.scrollboxNode,"onscroll","doscroll");
			dojox.grid.util.funnelEvents(this.contentNode, this, "doContentEvent", [ 'mouseover', 'mouseout', 'click', 'dblclick', 'contextmenu', 'mousedown' ]);
			dojox.grid.util.funnelEvents(this.headerNode, this, "doHeaderEvent", [ 'dblclick', 'mouseover', 'mouseout', 'mousemove', 'mousedown', 'click', 'contextmenu' ]);
			this.content = new this._contentBuilderClass(this);
			this.header = new this._headerBuilderClass(this);
			//BiDi: in RTL case, style width='9000em' causes scrolling problem in head node
			if(!dojo._isBodyLtr()){
				this.headerNodeContainer.style.width = "";
			}
		},

		destroy: function(){
			dojo.destroy(this.headerNode);
			delete this.headerNode;
			for(var i in this.rowNodes){
				dojo.destroy(this.rowNodes[i]);
			}
			this.rowNodes = {};
			if(this.source){
				this.source.destroy();
			}
			this.inherited(arguments);
		},

		// focus 
		focus: function(){
			if(dojo.isIE || dojo.isWebKit || dojo.isOpera){
				this.hiddenFocusNode.focus();
			}else{
				this.scrollboxNode.focus();
			}
		},

		setStructure: function(inStructure){
			var vs = (this.structure = inStructure);
			// FIXME: similar logic is duplicated in layout
			if(vs.width && !isNaN(vs.width)){
				this.viewWidth = vs.width + 'em';
			}else{
				this.viewWidth = vs.width || (vs.noscroll ? 'auto' : this.viewWidth); //|| this.defaultWidth;
			}
			this._onBeforeRow = vs.onBeforeRow||function(){};
			this._onAfterRow = vs.onAfterRow||function(){};
			this.noscroll = vs.noscroll;
			if(this.noscroll){
				this.scrollboxNode.style.overflow = "hidden";
			}
			this.simpleStructure = Boolean(vs.cells.length == 1);
			// bookkeeping
			this.testFlexCells();
			// accomodate new structure
			this.updateStructure();
		},
		
		_cleanupRowWidgets: function(inRowNode){
			// Summary:
			//		Cleans up the widgets for the given row node so that
			//		we can reattach them if needed
			if(inRowNode){
				dojo.forEach(dojo.query("[widgetId]", inRowNode).map(dijit.byNode), function(w){
					if(w._destroyOnRemove){
						w.destroy();
						delete w;
					}else if(w.domNode && w.domNode.parentNode){
						w.domNode.parentNode.removeChild(w.domNode);
					}
				});
			}
		},
		
		onBeforeRow: function(inRowIndex, cells){
			this._onBeforeRow(inRowIndex, cells);
			if(inRowIndex >= 0){
				this._cleanupRowWidgets(this.getRowNode(inRowIndex));
			}
		},
		
		onAfterRow: function(inRowIndex, cells, inRowNode){
			this._onAfterRow(inRowIndex, cells, inRowNode);
			var g = this.grid;
			dojo.forEach(dojo.query(".dojoxGridStubNode", inRowNode), function(n){
				if(n && n.parentNode){
					var lw = n.getAttribute("linkWidget");
					var cellIdx = window.parseInt(dojo.attr(n, "cellIdx"), 10);
					var cellDef = g.getCell(cellIdx);
					var w = dijit.byId(lw);
					if(w){
						n.parentNode.replaceChild(w.domNode, n);
						if(!w._started){
							w.startup();
						}
					}else{
						n.innerHTML = "";
					}
				}
			}, this);
		},

		testFlexCells: function(){
			// FIXME: cheater, this function does double duty as initializer and tester
			this.flexCells = false;
			for(var j=0, row; (row=this.structure.cells[j]); j++){
				for(var i=0, cell; (cell=row[i]); i++){
					cell.view = this;
					this.flexCells = this.flexCells || cell.isFlex();
				}
			}
			return this.flexCells;
		},

		updateStructure: function(){
			// header builder needs to update table map
			this.header.update();
			// content builder needs to update markup cache
			this.content.update();
		},

		getScrollbarWidth: function(){
			var hasScrollSpace = this.hasVScrollbar();
			var overflow = dojo.style(this.scrollboxNode, "overflow");
			if(this.noscroll || !overflow || overflow == "hidden"){
				hasScrollSpace = false;
			}else if(overflow == "scroll"){
				hasScrollSpace = true;
			}
			return (hasScrollSpace ? dojox.html.metrics.getScrollbar().w : 0); // Integer
		},

		getColumnsWidth: function(){
			var h = this.headerContentNode;
			return h && h.firstChild ? h.firstChild.offsetWidth : 0; // Integer
		},

		setColumnsWidth: function(width){
			this.headerContentNode.firstChild.style.width = width + 'px';
			if(this.viewWidth){
				this.viewWidth = width + 'px';
			}
		},

		getWidth: function(){
			return this.viewWidth || (this.getColumnsWidth()+this.getScrollbarWidth()) +'px'; // String
		},

		getContentWidth: function(){
			return Math.max(0, dojo._getContentBox(this.domNode).w - this.getScrollbarWidth()) + 'px'; // String
		},

		render: function(){
			this.scrollboxNode.style.height = '';
			this.renderHeader();
			if(this._togglingColumn >= 0){
				this.setColumnsWidth(this.getColumnsWidth() - this._togglingColumn);
				this._togglingColumn = -1;
			}
			var cells = this.grid.layout.cells;
			var getSibling = dojo.hitch(this, function(node, before){
				!dojo._isBodyLtr() && (before = !before);
				var inc = before?-1:1;
				var idx = this.header.getCellNodeIndex(node) + inc;
				var cell = cells[idx];
				while(cell && cell.getHeaderNode() && cell.getHeaderNode().style.display == "none"){
					idx += inc;
					cell = cells[idx];
				}
				if(cell){
					return cell.getHeaderNode();
				}
				return null;
			});
			if(this.grid.columnReordering && this.simpleStructure){
				if(this.source){
					this.source.destroy();
				}
				
				// Create the top and bottom markers
				var bottomMarkerId = "dojoxGrid_bottomMarker";
				var topMarkerId = "dojoxGrid_topMarker";
				if(this.bottomMarker){
					dojo.destroy(this.bottomMarker);
				}
				this.bottomMarker = dojo.byId(bottomMarkerId);
				if(this.topMarker){
					dojo.destroy(this.topMarker);
				}
				this.topMarker = dojo.byId(topMarkerId);
				if (!this.bottomMarker) {
					this.bottomMarker = dojo.create("div", {
						"id": bottomMarkerId,
						"class": "dojoxGridColPlaceBottom"
					}, dojo.body());
					this._hide(this.bottomMarker);

					
					this.topMarker = dojo.create("div", {
						"id": topMarkerId,
						"class": "dojoxGridColPlaceTop"
					}, dojo.body());
					this._hide(this.topMarker);
				}
				this.arrowDim = dojo.contentBox(this.bottomMarker);

				var headerHeight = dojo.contentBox(this.headerContentNode.firstChild.rows[0]).h;
				
				this.source = new dojo.dnd.Source(this.headerContentNode.firstChild.rows[0], {
					horizontal: true,
					accept: [ "gridColumn_" + this.grid.id ],
					viewIndex: this.index,
					generateText: false,
					onMouseDown: dojo.hitch(this, function(e){
						this.header.decorateEvent(e);
						if((this.header.overRightResizeArea(e) || this.header.overLeftResizeArea(e)) &&
							this.header.canResize(e) && !this.header.moveable){
							this.header.beginColumnResize(e);
						}else{
							if(this.grid.headerMenu){
								this.grid.headerMenu.onCancel(true);
							}
							// IE reports a left click as 1, where everything else reports 0
							if(e.button === (dojo.isIE ? 1 : 0)){
								dojo.dnd.Source.prototype.onMouseDown.call(this.source, e);
							}
						}
					}),
					onMouseOver: dojo.hitch(this, function(e){
						var src = this.source;
						if(src._getChildByEvent(e)){
							dojo.dnd.Source.prototype.onMouseOver.apply(src, arguments);
						}
					}),
					_markTargetAnchor: dojo.hitch(this, function(before){
						var src = this.source;
						if(src.current == src.targetAnchor && src.before == before){ return; }
						if(src.targetAnchor && getSibling(src.targetAnchor, src.before)){
							src._removeItemClass(getSibling(src.targetAnchor, src.before), src.before ? "After" : "Before");
						}
						dojo.dnd.Source.prototype._markTargetAnchor.call(src, before);
						
						var target = before ? src.targetAnchor : getSibling(src.targetAnchor, src.before);
						var endAdd = 0;

						if (!target) {
							target = src.targetAnchor;
							endAdd = dojo.contentBox(target).w + this.arrowDim.w/2 + 2;
						}

						// NOTE: this is for backwards compatibility with Dojo 1.3
						var pos = (dojo.position||dojo._abs)(target, true);
						var left = Math.floor(pos.x - this.arrowDim.w/2 + endAdd);

						dojo.style(this.bottomMarker, "visibility", "visible");
						dojo.style(this.topMarker, "visibility", "visible");
						dojo.style(this.bottomMarker, {
							"left": left + "px",
							"top" : (headerHeight + pos.y) + "px"
						});

						dojo.style(this.topMarker, {
							"left": left + "px",
							"top" : (pos.y - this.arrowDim.h) + "px"
						});

						if(src.targetAnchor && getSibling(src.targetAnchor, src.before)){
							src._addItemClass(getSibling(src.targetAnchor, src.before), src.before ? "After" : "Before");
						}
					}),
					_unmarkTargetAnchor: dojo.hitch(this, function(){
						var src = this.source;
						if(!src.targetAnchor){ return; }
						if(src.targetAnchor && getSibling(src.targetAnchor, src.before)){
							src._removeItemClass(getSibling(src.targetAnchor, src.before), src.before ? "After" : "Before");
						}
						this._hide(this.bottomMarker);
						this._hide(this.topMarker);
						dojo.dnd.Source.prototype._unmarkTargetAnchor.call(src);
					}),
					destroy: dojo.hitch(this, function(){
						dojo.disconnect(this._source_conn);
						dojo.unsubscribe(this._source_sub);
						dojo.dnd.Source.prototype.destroy.call(this.source);
						if(this.bottomMarker){
							dojo.destroy(this.bottomMarker);
							delete this.bottomMarker;
						}
						if(this.topMarker){
							dojo.destroy(this.topMarker);
							delete this.topMarker;
						}
					}),
					onDndCancel: dojo.hitch(this, function(){
						dojo.dnd.Source.prototype.onDndCancel.call(this.source);
						this._hide(this.bottomMarker);
						this._hide(this.topMarker);
					})
				});

				this._source_conn = dojo.connect(this.source, "onDndDrop", this, "_onDndDrop");
				this._source_sub = dojo.subscribe("/dnd/drop/before", this, "_onDndDropBefore");
				this.source.startup();
			}
		},
		
		_hide: function(node){
			dojo.style(node, {
				left: "-10000px",
				top: "-10000px",
				"visibility": "hidden"
			});
		},

		_onDndDropBefore: function(source, nodes, copy){
			if(dojo.dnd.manager().target !== this.source){
				return;
			}
			this.source._targetNode = this.source.targetAnchor;
			this.source._beforeTarget = this.source.before;
			var views = this.grid.views.views;
			var srcView = views[source.viewIndex];
			var tgtView = views[this.index];
			if(tgtView != srcView){
				srcView.convertColPctToFixed();
				tgtView.convertColPctToFixed();
			}
		},

		_onDndDrop: function(source, nodes, copy){
			if(dojo.dnd.manager().target !== this.source){
				if(dojo.dnd.manager().source === this.source){
					this._removingColumn = true;
				}
				return;
			}
			this._hide(this.bottomMarker);
			this._hide(this.topMarker);

			var getIdx = function(n){
				return n ? dojo.attr(n, "idx") : null;
			};
			var w = dojo.marginBox(nodes[0]).w;
			if(source.viewIndex !== this.index){
				var views = this.grid.views.views;
				var srcView = views[source.viewIndex];
				var tgtView = views[this.index];
				if(srcView.viewWidth && srcView.viewWidth != "auto"){
					srcView.setColumnsWidth(srcView.getColumnsWidth() - w);
				}
				if(tgtView.viewWidth && tgtView.viewWidth != "auto"){
					tgtView.setColumnsWidth(tgtView.getColumnsWidth());
				}
			}
			var stn = this.source._targetNode;
			var stb = this.source._beforeTarget;
			!dojo._isBodyLtr() && (stb = !stb);
			var layout = this.grid.layout;
			var idx = this.index;
			delete this.source._targetNode;
			delete this.source._beforeTarget;
			
			layout.moveColumn(
				source.viewIndex,
				idx,
				getIdx(nodes[0]),
				getIdx(stn),
				stb);
		},

		renderHeader: function(){
			this.headerContentNode.innerHTML = this.header.generateHtml(this._getHeaderContent);
			if(this.flexCells){
				this.contentWidth = this.getContentWidth();
				this.headerContentNode.firstChild.style.width = this.contentWidth;
			}
			dojox.grid.util.fire(this, "onAfterRow", [-1, this.structure.cells, this.headerContentNode]);
		},

		// note: not called in 'view' context
		_getHeaderContent: function(inCell){
			var n = inCell.name || inCell.grid.getCellName(inCell);
			var ret = [ '<div class="dojoxGridSortNode' ];
			
			if(inCell.index != inCell.grid.getSortIndex()){
				ret.push('">');
			}else{
				ret = ret.concat([ ' ',
							inCell.grid.sortInfo > 0 ? 'dojoxGridSortUp' : 'dojoxGridSortDown',
							'"><div class="dojoxGridArrowButtonChar">',
							inCell.grid.sortInfo > 0 ? '&#9650;' : '&#9660;',
							'</div><div class="dojoxGridArrowButtonNode" role="presentation"></div>',
							'<div class="dojoxGridColCaption">']);
			}
			ret = ret.concat([n, '</div></div>']);
			return ret.join('');
		},

		resize: function(){
			this.adaptHeight();
			this.adaptWidth();
		},

		hasHScrollbar: function(reset){
			var hadScroll = this._hasHScroll||false;
			if(this._hasHScroll == undefined || reset){
				if(this.noscroll){
					this._hasHScroll = false;
				}else{
					var style = dojo.style(this.scrollboxNode, "overflow");
					if(style == "hidden"){
						this._hasHScroll = false;
					}else if(style == "scroll"){
						this._hasHScroll = true;
					}else{
						this._hasHScroll = (this.scrollboxNode.offsetWidth - this.getScrollbarWidth() < this.contentNode.offsetWidth );
					}
				}
			}
			if(hadScroll !== this._hasHScroll){
				this.grid.update();
			}
			return this._hasHScroll; // Boolean
		},

		hasVScrollbar: function(reset){
			var hadScroll = this._hasVScroll||false;
			if(this._hasVScroll == undefined || reset){
				if(this.noscroll){
					this._hasVScroll = false;
				}else{
					var style = dojo.style(this.scrollboxNode, "overflow");
					if(style == "hidden"){
						this._hasVScroll = false;
					}else if(style == "scroll"){
						this._hasVScroll = true;
					}else{
						this._hasVScroll = this.scrollboxNode.clientHeight > 0 && (this.scrollboxNode.scrollHeight > this.scrollboxNode.clientHeight);
					}
				}
			}
			if(hadScroll !== this._hasVScroll){
				this.grid.update();
			}
			return this._hasVScroll; // Boolean
		},
		
		convertColPctToFixed: function(){
			// Fix any percentage widths to be pixel values
			var hasPct = false;
			this.grid.initialWidth = "";
			var cellNodes = dojo.query("th", this.headerContentNode);
			var fixedWidths = dojo.map(cellNodes, function(c, vIdx){
				var w = c.style.width;
				dojo.attr(c, "vIdx", vIdx);
				if(w && w.slice(-1) == "%"){
					hasPct = true;
				}else if(w && w.slice(-2) == "px"){
					return window.parseInt(w, 10);
				}
				return dojo.contentBox(c).w;
			});
			if(hasPct){
				dojo.forEach(this.grid.layout.cells, function(cell, idx){
					if(cell.view == this){
						var cellNode = cell.view.getHeaderCellNode(cell.index);
						if(cellNode && dojo.hasAttr(cellNode, "vIdx")){
							var vIdx = window.parseInt(dojo.attr(cellNode, "vIdx"));
							this.setColWidth(idx, fixedWidths[vIdx]);
							dojo.removeAttr(cellNode, "vIdx");
						}
					}
				}, this);
				return true;
			}
			return false;
		},

		adaptHeight: function(minusScroll){
			if(!this.grid._autoHeight){
				var h = (this.domNode.style.height && parseInt(this.domNode.style.height.replace(/px/,''), 10)) || this.domNode.clientHeight;
				var self = this;
				var checkOtherViewScrollers = function(){
					var v;
					for(var i in self.grid.views.views){
						v = self.grid.views.views[i];
						if(v !== self && v.hasHScrollbar()){
							return true;
						}
					}
					return false;
				};
				if(minusScroll || (this.noscroll && checkOtherViewScrollers())){
					h -= dojox.html.metrics.getScrollbar().h;
				}
				dojox.grid.util.setStyleHeightPx(this.scrollboxNode, h);
			}
			this.hasVScrollbar(true);
		},

		adaptWidth: function(){
			if(this.flexCells){
				// the view content width
				this.contentWidth = this.getContentWidth();
				this.headerContentNode.firstChild.style.width = this.contentWidth;
			}
			// FIXME: it should be easier to get w from this.scrollboxNode.clientWidth, 
			// but clientWidth seemingly does not include scrollbar width in some cases
			var w = this.scrollboxNode.offsetWidth - this.getScrollbarWidth();
			if(!this._removingColumn){
				w = Math.max(w, this.getColumnsWidth()) + 'px';
			}else{
				w = Math.min(w, this.getColumnsWidth()) + 'px';
				this._removingColumn = false;
			}
			var cn = this.contentNode;
			cn.style.width = w;
			this.hasHScrollbar(true);
		},

		setSize: function(w, h){
			var ds = this.domNode.style;
			var hs = this.headerNode.style;

			if(w){
				ds.width = w;
				hs.width = w;
			}
			ds.height = (h >= 0 ? h + 'px' : '');
		},

		renderRow: function(inRowIndex){
			var rowNode = this.createRowNode(inRowIndex);
			this.buildRow(inRowIndex, rowNode);
			this.grid.edit.restore(this, inRowIndex);
			return rowNode;
		},

		createRowNode: function(inRowIndex){
			var node = document.createElement("div");
			node.className = this.classTag + 'Row';
			if (this instanceof dojox.grid._RowSelector){
				dojo.attr(node,"role","presentation");
			}else{
				dojo.attr(node,"role","row");
				if (this.grid.selectionMode != "none") {
					dojo.attr(node, "aria-selected", "false"); //rows can be selected so add aria-selected prop
				}
			}
			node[dojox.grid.util.gridViewTag] = this.id;
			node[dojox.grid.util.rowIndexTag] = inRowIndex;
			this.rowNodes[inRowIndex] = node;
			return node;
		},

		buildRow: function(inRowIndex, inRowNode){
			
			this.buildRowContent(inRowIndex, inRowNode);
		  	
			this.styleRow(inRowIndex, inRowNode);
		  
		 
		},

		buildRowContent: function(inRowIndex, inRowNode){
			inRowNode.innerHTML = this.content.generateHtml(inRowIndex, inRowIndex); 
			if(this.flexCells && this.contentWidth){
				// FIXME: accessing firstChild here breaks encapsulation
				inRowNode.firstChild.style.width = this.contentWidth;
			}
			dojox.grid.util.fire(this, "onAfterRow", [inRowIndex, this.structure.cells, inRowNode]);
		},

		rowRemoved:function(inRowIndex){
			if(inRowIndex >= 0){
				this._cleanupRowWidgets(this.getRowNode(inRowIndex));
			}
			this.grid.edit.save(this, inRowIndex);
			delete this.rowNodes[inRowIndex];
		},

		getRowNode: function(inRowIndex){
			return this.rowNodes[inRowIndex];
		},

		getCellNode: function(inRowIndex, inCellIndex){
			var row = this.getRowNode(inRowIndex);
			if(row){
				return this.content.getCellNode(row, inCellIndex);
			}
		},

		getHeaderCellNode: function(inCellIndex){
			if(this.headerContentNode){
				return this.header.getCellNode(this.headerContentNode, inCellIndex);
			}
		},

		// styling
		styleRow: function(inRowIndex, inRowNode){
			inRowNode._style = getStyleText(inRowNode);
			this.styleRowNode(inRowIndex, inRowNode);
		},

		styleRowNode: function(inRowIndex, inRowNode){
			if(inRowNode){
				this.doStyleRowNode(inRowIndex, inRowNode);
			}
		},

		doStyleRowNode: function(inRowIndex, inRowNode){
			this.grid.styleRowNode(inRowIndex, inRowNode);
		},

		// updating
		updateRow: function(inRowIndex){
			var rowNode = this.getRowNode(inRowIndex);
			if(rowNode){
				rowNode.style.height = '';
				this.buildRow(inRowIndex, rowNode);
			}
			return rowNode;
		},

		updateRowStyles: function(inRowIndex){
			this.styleRowNode(inRowIndex, this.getRowNode(inRowIndex));
		},

		// scrolling
		lastTop: 0,
		firstScroll:0,

		doscroll: function(inEvent){
			//var s = dojo.marginBox(this.headerContentNode.firstChild);
			var isLtr = dojo._isBodyLtr();
			if(this.firstScroll < 2){
				if((!isLtr && this.firstScroll == 1) || (isLtr && this.firstScroll === 0)){
					var s = dojo.marginBox(this.headerNodeContainer);
					if(dojo.isIE){
						this.headerNodeContainer.style.width = s.w + this.getScrollbarWidth() + 'px';
					}else if(dojo.isMoz){
						//TODO currently only for FF, not sure for safari and opera
						this.headerNodeContainer.style.width = s.w - this.getScrollbarWidth() + 'px';
						//this.headerNodeContainer.style.width = s.w + 'px';
						//set scroll to right in FF
						this.scrollboxNode.scrollLeft = isLtr ?
							this.scrollboxNode.clientWidth - this.scrollboxNode.scrollWidth :
							this.scrollboxNode.scrollWidth - this.scrollboxNode.clientWidth;
					}
				}
				this.firstScroll++;
			}
			this.headerNode.scrollLeft = this.scrollboxNode.scrollLeft;
			// 'lastTop' is a semaphore to prevent feedback-loop with setScrollTop below
			var top = this.scrollboxNode.scrollTop;
			if(top !== this.lastTop){
				this.grid.scrollTo(top);
			}
		},

		setScrollTop: function(inTop){
			// 'lastTop' is a semaphore to prevent feedback-loop with doScroll above
			this.lastTop = inTop;
			this.scrollboxNode.scrollTop = inTop;
			return this.scrollboxNode.scrollTop;
		},

		// event handlers (direct from DOM)
		doContentEvent: function(e){
			if(this.content.decorateEvent(e)){
				this.grid.onContentEvent(e);
			}
		},

		doHeaderEvent: function(e){
			if(this.header.decorateEvent(e)){
				this.grid.onHeaderEvent(e);
			}
		},

		// event dispatch(from Grid)
		dispatchContentEvent: function(e){
			return this.content.dispatchEvent(e);
		},

		dispatchHeaderEvent: function(e){
			return this.header.dispatchEvent(e);
		},

		// column resizing
		setColWidth: function(inIndex, inWidth){
			this.grid.setCellWidth(inIndex, inWidth + 'px');
		},

		update: function(){
			if(!this.domNode){
				return;
			}
			this.content.update();
			this.grid.update();
			//get scroll after update or scroll left setting goes wrong on IE.
			//See trac: #8040
			var left = this.scrollboxNode.scrollLeft;
			this.scrollboxNode.scrollLeft = left;
			this.headerNode.scrollLeft = left;
		}
	});

	dojo.declare("dojox.grid._GridAvatar", dojo.dnd.Avatar, {
		construct: function(){
			var dd = dojo.doc;

			var a = dd.createElement("table");
			a.cellPadding = a.cellSpacing = "0";
			a.className = "dojoxGridDndAvatar";
			a.style.position = "absolute";
			a.style.zIndex = 1999;
			a.style.margin = "0px"; // to avoid dojo.marginBox() problems with table's margins
			var b = dd.createElement("tbody");
			var tr = dd.createElement("tr");
			var td = dd.createElement("td");
			var img = dd.createElement("td");
			tr.className = "dojoxGridDndAvatarItem";
			img.className = "dojoxGridDndAvatarItemImage";
			img.style.width = "16px";
			var source = this.manager.source, node;
			if(source.creator){
				// create an avatar representation of the node
				node = source._normalizedCreator(source.getItem(this.manager.nodes[0].id).data, "avatar").node;
			}else{
				// or just clone the node and hope it works
				node = this.manager.nodes[0].cloneNode(true);
				var table, tbody;
				if(node.tagName.toLowerCase() == "tr"){
					// insert extra table nodes
					table = dd.createElement("table");
					tbody = dd.createElement("tbody");
					tbody.appendChild(node);
					table.appendChild(tbody);
					node = table;
				}else if(node.tagName.toLowerCase() == "th"){
					// insert extra table nodes
					table = dd.createElement("table");
					tbody = dd.createElement("tbody");
					var r = dd.createElement("tr");
					table.cellPadding = table.cellSpacing = "0";
					r.appendChild(node);
					tbody.appendChild(r);
					table.appendChild(tbody);
					node = table;
				}
			}
			node.id = "";
			td.appendChild(node);
			tr.appendChild(img);
			tr.appendChild(td);
			dojo.style(tr, "opacity", 0.9);
			b.appendChild(tr);

			a.appendChild(b);
			this.node = a;

			var m = dojo.dnd.manager();
			this.oldOffsetY = m.OFFSET_Y;
			m.OFFSET_Y = 1;
		},
		destroy: function(){
			dojo.dnd.manager().OFFSET_Y = this.oldOffsetY;
			this.inherited(arguments);
		}
	});

	var oldMakeAvatar = dojo.dnd.manager().makeAvatar;
	dojo.dnd.manager().makeAvatar = function(){
		var src = this.source;
		if(src.viewIndex !== undefined && !dojo.hasClass(dojo.body(),"dijit_a11y")){
			return new dojox.grid._GridAvatar(this);
		}
		return oldMakeAvatar.call(dojo.dnd.manager());
	};
})();

}

if(!dojo._hasResource["dojox.grid._RowSelector"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._RowSelector"] = true;
dojo.provide("dojox.grid._RowSelector");


dojo.declare('dojox.grid._RowSelector', dojox.grid._View, {
	// summary:
	//	Custom grid view. If used in a grid structure, provides a small selectable region for grid rows.
	defaultWidth: "2em",
	noscroll: true,
	padBorderWidth: 2,
	buildRendering: function(){
		this.inherited('buildRendering', arguments);
		this.scrollboxNode.style.overflow = "hidden";
		this.headerNode.style.visibility = "hidden";
	},	
	getWidth: function(){
		return this.viewWidth || this.defaultWidth;
	},
	buildRowContent: function(inRowIndex, inRowNode){
		var w = this.contentWidth || 0;
		inRowNode.innerHTML = '<table class="dojoxGridRowbarTable" style="width:' + w + 'px;height:1px;" border="0" cellspacing="0" cellpadding="0" role="presentation"><tr><td class="dojoxGridRowbarInner">&nbsp;</td></tr></table>';
	},
	renderHeader: function(){
	},
	updateRow: function(){
	},
	resize: function(){
		this.adaptHeight();
	},
	adaptWidth: function(){
		// Only calculate this here - rather than every call to buildRowContent
		if(!("contentWidth" in this) && this.contentNode){
			this.contentWidth = this.contentNode.offsetWidth - this.padBorderWidth;
		}
	},
	// styling
	doStyleRowNode: function(inRowIndex, inRowNode){
		var n = [ "dojoxGridRowbar dojoxGridNonNormalizedCell" ];
		if(this.grid.rows.isOver(inRowIndex)){
			n.push("dojoxGridRowbarOver");
		}
		if(this.grid.selection.isSelected(inRowIndex)){
			n.push("dojoxGridRowbarSelected");
		}
		inRowNode.className = n.join(" ");
	},
	// event handlers
	domouseover: function(e){
		this.grid.onMouseOverRow(e);
	},
	domouseout: function(e){
		if(!this.isIntraRowEvent(e)){
			this.grid.onMouseOutRow(e);
		}
	}
});

}

if(!dojo._hasResource["dojox.grid._Layout"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._Layout"] = true;
dojo.provide("dojox.grid._Layout");



dojo.declare("dojox.grid._Layout", null, {
	// summary:
	//	Controls grid cell layout. Owned by grid and used internally.
	constructor: function(inGrid){
		this.grid = inGrid;
	},
	// flat array of grid cells
	cells: [],
	// structured array of grid cells
	structure: null,
	// default cell width
	defaultWidth: '6em',

	// methods
	moveColumn: function(sourceViewIndex, destViewIndex, cellIndex, targetIndex, before){
		var source_cells = this.structure[sourceViewIndex].cells[0];
		var dest_cells = this.structure[destViewIndex].cells[0];

		var cell = null;
		var cell_ri = 0;
		var target_ri = 0;

		for(var i=0, c; c=source_cells[i]; i++){
			if(c.index == cellIndex){
				cell_ri = i;
				break;
			}
		}
		cell = source_cells.splice(cell_ri, 1)[0];
		cell.view = this.grid.views.views[destViewIndex];

		for(i=0, c=null; c=dest_cells[i]; i++){
			if(c.index == targetIndex){
				target_ri = i;
				break;
			}
		}
		if(!before){
			target_ri += 1;
		}
		dest_cells.splice(target_ri, 0, cell);

		var sortedCell = this.grid.getCell(this.grid.getSortIndex());
		if(sortedCell){
			sortedCell._currentlySorted = this.grid.getSortAsc();
		}

		this.cells = [];
		cellIndex = 0;
		var v;
		for(i=0; v=this.structure[i]; i++){
			for(var j=0, cs; cs=v.cells[j]; j++){
				for(var k=0; c=cs[k]; k++){
					c.index = cellIndex;
					this.cells.push(c);
					if("_currentlySorted" in c){
						var si = cellIndex + 1;
						si *= c._currentlySorted ? 1 : -1;
						this.grid.sortInfo = si;
						delete c._currentlySorted;
					}
					cellIndex++;
				}
			}
		}
		
		//Fix #9481 - reset idx in cell markup
		dojo.forEach(this.cells, function(c){
			var marks = c.markup[2].split(" ");
			var oldIdx = parseInt(marks[1].substring(5));//get old "idx"
			if(oldIdx != c.index){
				marks[1] = "idx=\"" + c.index + "\"";
				c.markup[2] = marks.join(" ");
			}
		});
		
		this.grid.setupHeaderMenu();
		//this.grid.renderOnIdle();
	},

	setColumnVisibility: function(columnIndex, visible){
		var cell = this.cells[columnIndex];
		if(cell.hidden == visible){
			cell.hidden = !visible;
			var v = cell.view, w = v.viewWidth;
			if(w && w != "auto"){
				v._togglingColumn = dojo.marginBox(cell.getHeaderNode()).w || 0;
			}
			v.update();
			return true;
		}else{
			return false;
		}
	},
	
	addCellDef: function(inRowIndex, inCellIndex, inDef){
		var self = this;
		var getCellWidth = function(inDef){
			var w = 0;
			if(inDef.colSpan > 1){
				w = 0;
			}else{
				w = inDef.width || self._defaultCellProps.width || self.defaultWidth;

				if(!isNaN(w)){
					w = w + "em";
				}
			}
			return w;
		};

		var props = {
			grid: this.grid,
			subrow: inRowIndex,
			layoutIndex: inCellIndex,
			index: this.cells.length
		};

		if(inDef && inDef instanceof dojox.grid.cells._Base){
			var new_cell = dojo.clone(inDef);
			props.unitWidth = getCellWidth(new_cell._props);
			new_cell = dojo.mixin(new_cell, this._defaultCellProps, inDef._props, props);
			return new_cell;
		}

		var cell_type = inDef.type || this._defaultCellProps.type || dojox.grid.cells.Cell;

		props.unitWidth = getCellWidth(inDef);
		return new cell_type(dojo.mixin({}, this._defaultCellProps, inDef, props));	
	},
	
	addRowDef: function(inRowIndex, inDef){
		var result = [];
		var relSum = 0, pctSum = 0, doRel = true;
		for(var i=0, def, cell; (def=inDef[i]); i++){
			cell = this.addCellDef(inRowIndex, i, def);
			result.push(cell);
			this.cells.push(cell);
			// Check and calculate the sum of all relative widths
			if(doRel && cell.relWidth){
				relSum += cell.relWidth;
			}else if(cell.width){
				var w = cell.width;
				if(typeof w == "string" && w.slice(-1) == "%"){
					pctSum += window.parseInt(w, 10);
				}else if(w == "auto"){
					// relative widths doesn't play nice with auto - since we
					// don't have a way of knowing how much space the auto is 
					// supposed to take up.
					doRel = false;
				}
			}
		}
		if(relSum && doRel){
			// We have some kind of relWidths specified - so change them to %
			dojo.forEach(result, function(cell){
				if(cell.relWidth){
					cell.width = cell.unitWidth = ((cell.relWidth / relSum) * (100 - pctSum)) + "%";
				}
			});
		}
		return result;
	
	},

	addRowsDef: function(inDef){
		var result = [];
		if(dojo.isArray(inDef)){
			if(dojo.isArray(inDef[0])){
				for(var i=0, row; inDef && (row=inDef[i]); i++){
					result.push(this.addRowDef(i, row));
				}
			}else{
				result.push(this.addRowDef(0, inDef));
			}
		}
		return result;	
	},
	
	addViewDef: function(inDef){
		this._defaultCellProps = inDef.defaultCell || {};
		if(inDef.width && inDef.width == "auto"){
			delete inDef.width;
		}
		return dojo.mixin({}, inDef, {cells: this.addRowsDef(inDef.rows || inDef.cells)});
	},
	
	setStructure: function(inStructure){
		this.fieldIndex = 0;
		this.cells = [];
		var s = this.structure = [];

		if(this.grid.rowSelector){
			var sel = { type: dojox._scopeName + ".grid._RowSelector" };

			if(dojo.isString(this.grid.rowSelector)){
				var width = this.grid.rowSelector;

				if(width == "false"){
					sel = null;
				}else if(width != "true"){
					sel['width'] = width;
				}
			}else{
				if(!this.grid.rowSelector){
					sel = null;
				}
			}

			if(sel){
				s.push(this.addViewDef(sel));
			}
		}

		var isCell = function(def){
			return ("name" in def || "field" in def || "get" in def);
		};

		var isRowDef = function(def){
			if(dojo.isArray(def)){
				if(dojo.isArray(def[0]) || isCell(def[0])){
					return true;
				}
			}
			return false;
		};

		var isView = function(def){
			return (def !== null && dojo.isObject(def) &&
					("cells" in def || "rows" in def || ("type" in def && !isCell(def))));
		};

		if(dojo.isArray(inStructure)){
			var hasViews = false;
			for(var i=0, st; (st=inStructure[i]); i++){
				if(isView(st)){
					hasViews = true;
					break;
				}
			}
			if(!hasViews){
				s.push(this.addViewDef({ cells: inStructure }));
			}else{
				for(i=0; (st=inStructure[i]); i++){
					if(isRowDef(st)){
						s.push(this.addViewDef({ cells: st }));
					}else if(isView(st)){
						s.push(this.addViewDef(st));
					}
				}
			}
		}else if(isView(inStructure)){
			// it's a view object
			s.push(this.addViewDef(inStructure));
		}

		this.cellCount = this.cells.length;
		this.grid.setupHeaderMenu();
	}
});

}

if(!dojo._hasResource["dojox.grid._ViewManager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._ViewManager"] = true;
dojo.provide("dojox.grid._ViewManager");

dojo.declare('dojox.grid._ViewManager', null, {
	// summary:
	//		A collection of grid views. Owned by grid and used internally for managing grid views.
	// description:
	//		Grid creates views automatically based on grid's layout structure.
	//		Users should typically not need to access individual views or the views collection directly.
	constructor: function(inGrid){
		this.grid = inGrid;
	},

	defaultWidth: 200,

	views: [],

	// operations
	resize: function(){
		this.onEach("resize");
	},

	render: function(){
		this.onEach("render");
	},

	// views
	addView: function(inView){
		inView.idx = this.views.length;
		this.views.push(inView);
	},

	destroyViews: function(){
		for(var i=0, v; v=this.views[i]; i++){
			v.destroy();
		}
		this.views = [];
	},

	getContentNodes: function(){
		var nodes = [];
		for(var i=0, v; v=this.views[i]; i++){
			nodes.push(v.contentNode);
		}
		return nodes;
	},

	forEach: function(inCallback){
		for(var i=0, v; v=this.views[i]; i++){
			inCallback(v, i);
		}
	},

	onEach: function(inMethod, inArgs){
		inArgs = inArgs || [];
		for(var i=0, v; v=this.views[i]; i++){
			if(inMethod in v){
				v[inMethod].apply(v, inArgs);
			}
		}
	},

	// layout
	normalizeHeaderNodeHeight: function(){
		var rowNodes = [];
		for(var i=0, v; (v=this.views[i]); i++){
			if(v.headerContentNode.firstChild){
				rowNodes.push(v.headerContentNode);
			}
		}
		this.normalizeRowNodeHeights(rowNodes);
	},

	normalizeRowNodeHeights: function(inRowNodes){
		var h = 0;
		var currHeights = [];
		if(this.grid.rowHeight){
			h = this.grid.rowHeight;
		}else{
			if(inRowNodes.length <= 1){ 
				// no need to normalize if we are the only one...
				return; 
			}
			for(var i=0, n; (n=inRowNodes[i]); i++){
				// We only care about the height - so don't use marginBox.  This
				// depends on the container not having any margin (which it shouldn't)
				// Also - we only look up the height if the cell doesn't have the
				// dojoxGridNonNormalizedCell class (like for row selectors)
				if(!dojo.hasClass(n, "dojoxGridNonNormalizedCell")){
					currHeights[i] = n.firstChild.offsetHeight;
					h =  Math.max(h, currHeights[i]);
				}
			}
			h = (h >= 0 ? h : 0);
	
			//Work around odd FF3 rendering bug: #8864.
			//A one px increase fixes FireFox 3's rounding bug for fractional font sizes.
			if(dojo.isMoz && h){h++;}
		}
		for(i=0; (n=inRowNodes[i]); i++){
			if(currHeights[i] != h){
				n.firstChild.style.height = h + "px";
			}
		}
	},
	
	resetHeaderNodeHeight: function(){
		for(var i=0, v, n; (v=this.views[i]); i++){
			n = v.headerContentNode.firstChild;
			if(n){
				n.style.height = "";
			}
		}
	},

	renormalizeRow: function(inRowIndex){
		var rowNodes = [];
		for(var i=0, v, n; (v=this.views[i])&&(n=v.getRowNode(inRowIndex)); i++){
			n.firstChild.style.height = '';
			rowNodes.push(n);
		}
		this.normalizeRowNodeHeights(rowNodes);
	},

	getViewWidth: function(inIndex){
		return this.views[inIndex].getWidth() || this.defaultWidth;
	},

	// must be called after view widths are properly set or height can be miscalculated
	// if there are flex columns
	measureHeader: function(){
		// need to reset view header heights so they are properly measured.
		this.resetHeaderNodeHeight();
		this.forEach(function(inView){
			inView.headerContentNode.style.height = '';
		});
		var h = 0;
		// calculate maximum view header height
		this.forEach(function(inView){
			h = Math.max(inView.headerNode.offsetHeight, h);
		});
		return h;
	},

	measureContent: function(){
		var h = 0;
		this.forEach(function(inView){
			h = Math.max(inView.domNode.offsetHeight, h);
		});
		return h;
	},

	findClient: function(inAutoWidth){
		// try to use user defined client
		var c = this.grid.elasticView || -1;
		// attempt to find implicit client
		if(c < 0){
			for(var i=1, v; (v=this.views[i]); i++){
				if(v.viewWidth){
					for(i=1; (v=this.views[i]); i++){
						if(!v.viewWidth){
							c = i;
							break;
						}
					}
					break;
				}
			}
		}
		// client is in the middle by default
		if(c < 0){
			c = Math.floor(this.views.length / 2);
		}
		return c;
	},

	arrange: function(l, w){
		var i, v, vw, len = this.views.length;
		// find the client
		var c = (w <= 0 ? len : this.findClient());
		// layout views
		var setPosition = function(v, l){
			var ds = v.domNode.style;
			var hs = v.headerNode.style;

			if(!dojo._isBodyLtr()){
				ds.right = l + 'px';
				// fixed rtl, the scrollbar is on the right side in FF
				if (dojo.isMoz) {
					hs.right = l + v.getScrollbarWidth() + 'px';
					hs.width = parseInt(hs.width, 10) - v.getScrollbarWidth() + 'px';
				}else{
					hs.right = l + 'px';					
				}
			}else{
				ds.left = l + 'px';
				hs.left = l + 'px';
			}
			ds.top = 0 + 'px';
			hs.top = 0;
		};
		// for views left of the client
		//BiDi TODO: The left and right should not appear in BIDI environment. Should be replaced with 
		//leading and tailing concept.
		for(i=0; (v=this.views[i])&&(i<c); i++){
			// get width
			vw = this.getViewWidth(i);
			// process boxes
			v.setSize(vw, 0);
			setPosition(v, l);
			if(v.headerContentNode && v.headerContentNode.firstChild){
				vw = v.getColumnsWidth()+v.getScrollbarWidth();
			}else{
				vw = v.domNode.offsetWidth;
			}
			// update position
			l += vw;
		}
		// next view (is the client, i++ == c) 
		i++;
		// start from the right edge
		var r = w;
		// for views right of the client (iterated from the right)
		for(var j=len-1; (v=this.views[j])&&(i<=j); j--){
			// get width
			vw = this.getViewWidth(j);
			// set size
			v.setSize(vw, 0);
			// measure in pixels
			vw = v.domNode.offsetWidth;
			// update position
			r -= vw;
			// set position
			setPosition(v, r);
		}
		if(c<len){
			v = this.views[c];
			// position the client box between left and right boxes	
			vw = Math.max(1, r-l);
			// set size
			v.setSize(vw + 'px', 0);
			setPosition(v, l);
		}
		return l;
	},

	// rendering
	renderRow: function(inRowIndex, inNodes, skipRenorm){
		var rowNodes = [];
		for(var i=0, v, n, rowNode; (v=this.views[i])&&(n=inNodes[i]); i++){
			rowNode = v.renderRow(inRowIndex);
			n.appendChild(rowNode);
			rowNodes.push(rowNode);
		}
		if(!skipRenorm){
			this.normalizeRowNodeHeights(rowNodes);
		}
	},
	
	rowRemoved: function(inRowIndex){
		this.onEach("rowRemoved", [ inRowIndex ]);
	},
	
	// updating
	updateRow: function(inRowIndex, skipRenorm){
		for(var i=0, v; v=this.views[i]; i++){
			v.updateRow(inRowIndex);
		}
		if(!skipRenorm){
			this.renormalizeRow(inRowIndex);
		}
	},
	
	updateRowStyles: function(inRowIndex){
		this.onEach("updateRowStyles", [ inRowIndex ]);
	},
	
	// scrolling
	setScrollTop: function(inTop){
		var top = inTop;
		for(var i=0, v; v=this.views[i]; i++){
			top = v.setScrollTop(inTop);
			// Work around IE not firing scroll events that cause header offset
			// issues to occur.
			if(dojo.isIE && v.headerNode && v.scrollboxNode){
				v.headerNode.scrollLeft = v.scrollboxNode.scrollLeft;
			}
		}
		return top;
		//this.onEach("setScrollTop", [ inTop ]);
	},
	
	getFirstScrollingView: function(){
		// summary: Returns the first grid view with a scroll bar 
		for(var i=0, v; (v=this.views[i]); i++){
			if(v.hasHScrollbar() || v.hasVScrollbar()){
				return v;
			}
		}
		return null;
	}
	
});

}

if(!dojo._hasResource["dojox.grid._RowManager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._RowManager"] = true;
dojo.provide("dojox.grid._RowManager");

(function(){
	var setStyleText = function(inNode, inStyleText){
		if(inNode.style.cssText == undefined){
			inNode.setAttribute("style", inStyleText);
		}else{
			inNode.style.cssText = inStyleText;
		}
	};

	dojo.declare("dojox.grid._RowManager", null, {
		//	Stores information about grid rows. Owned by grid and used internally.
		constructor: function(inGrid){
			this.grid = inGrid;
		},
		linesToEms: 2,
		overRow: -2,
		// styles
		prepareStylingRow: function(inRowIndex, inRowNode){
			return {
				index: inRowIndex, 
				node: inRowNode,
				odd: Boolean(inRowIndex&1),
				selected: !!this.grid.selection.isSelected(inRowIndex),
				over: this.isOver(inRowIndex),
				customStyles: "",
				customClasses: "dojoxGridRow"
			};
		},
		styleRowNode: function(inRowIndex, inRowNode){
			var row = this.prepareStylingRow(inRowIndex, inRowNode);
			this.grid.onStyleRow(row);
			this.applyStyles(row);
		},
		applyStyles: function(inRow){
			var i = inRow;

			i.node.className = i.customClasses;
			var h = i.node.style.height;
			setStyleText(i.node, i.customStyles + ';' + (i.node._style||''));
			i.node.style.height = h;
		},
		updateStyles: function(inRowIndex){
			this.grid.updateRowStyles(inRowIndex);
		},
		// states and events
		setOverRow: function(inRowIndex){
			var last = this.overRow;
			this.overRow = inRowIndex;
			if((last!=this.overRow)&&(dojo.isString(last) || last >= 0)){
				this.updateStyles(last);
			}
			this.updateStyles(this.overRow);
		},
		isOver: function(inRowIndex){
			return (this.overRow == inRowIndex && !dojo.hasClass(this.grid.domNode, "dojoxGridColumnResizing"));
		}
	});
})();

}

if(!dojo._hasResource["dojox.grid._FocusManager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._FocusManager"] = true;
dojo.provide("dojox.grid._FocusManager");



// focus management
dojo.declare("dojox.grid._FocusManager", null, {
	// summary:
	//	Controls grid cell focus. Owned by grid and used internally for focusing.
	//	Note: grid cell actually receives keyboard input only when cell is being edited.
	constructor: function(inGrid){
		this.grid = inGrid;
		this.cell = null;
		this.rowIndex = -1;
		this._connects = [];
		this._headerConnects = [];
		this.headerMenu = this.grid.headerMenu;
		this._connects.push(dojo.connect(this.grid.domNode, "onfocus", this, "doFocus"));
		this._connects.push(dojo.connect(this.grid.domNode, "onblur", this, "doBlur"));
		this._connects.push(dojo.connect(this.grid.domNode, "oncontextmenu", this, "doContextMenu"));
		this._connects.push(dojo.connect(this.grid.lastFocusNode, "onfocus", this, "doLastNodeFocus"));
		this._connects.push(dojo.connect(this.grid.lastFocusNode, "onblur", this, "doLastNodeBlur"));
		this._connects.push(dojo.connect(this.grid,"_onFetchComplete", this, "_delayedCellFocus"));
		this._connects.push(dojo.connect(this.grid,"postrender", this, "_delayedHeaderFocus"));
	},
	destroy: function(){
		dojo.forEach(this._connects, dojo.disconnect);
		dojo.forEach(this._headerConnects, dojo.disconnect);
		delete this.grid;
		delete this.cell;
	},
	_colHeadNode: null,
	_colHeadFocusIdx: null,
	_contextMenuBindNode: null,
	tabbingOut: false,
	focusClass: "dojoxGridCellFocus",
	focusView: null,
	initFocusView: function(){
		this.focusView = this.grid.views.getFirstScrollingView() || this.focusView;
		this._initColumnHeaders();
	},
	isFocusCell: function(inCell, inRowIndex){
		// summary:
		//	states if the given cell is focused
		// inCell: object
		//	grid cell object
		// inRowIndex: int
		//	grid row index
		// returns:
		//	true of the given grid cell is focused
		return (this.cell == inCell) && (this.rowIndex == inRowIndex);
	},
	isLastFocusCell: function(){
		if(this.cell){
			return (this.rowIndex == this.grid.rowCount-1) && (this.cell.index == this.grid.layout.cellCount-1);
		}
		return false;
	},
	isFirstFocusCell: function(){
		if(this.cell){
			return (this.rowIndex === 0) && (this.cell.index === 0);
		}
		return false;
	},
	isNoFocusCell: function(){
		return (this.rowIndex < 0) || !this.cell;
	},
	isNavHeader: function(){
		// summary:
		//	states whether currently navigating among column headers.
		// returns:
		//	true if focus is on a column header; false otherwise. 
		return (!!this._colHeadNode);
	},
	getHeaderIndex: function(){
		// summary:
		//	if one of the column headers currently has focus, return its index.
		// returns:
		//	index of the focused column header, or -1 if none have focus.
		if(this._colHeadNode){
			return dojo.indexOf(this._findHeaderCells(), this._colHeadNode);
		}else{
			return -1;
		}
	},
	_focusifyCellNode: function(inBork){
		var n = this.cell && this.cell.getNode(this.rowIndex);
		if(n){
			dojo.toggleClass(n, this.focusClass, inBork);
			if(inBork){
				var sl = this.scrollIntoView();
				try{
					if(!this.grid.edit.isEditing()){
						dojox.grid.util.fire(n, "focus");
						if(sl){ this.cell.view.scrollboxNode.scrollLeft = sl; }
					}
				}catch(e){}
			}
		}
	},
	_delayedCellFocus: function(){
		if(this.isNavHeader()||!this.grid._focused){
				return;
		}
		var n = this.cell && this.cell.getNode(this.rowIndex);
		if(n){ 
			try{
				if(!this.grid.edit.isEditing()){
					dojo.toggleClass(n, this.focusClass, true);
					this.blurHeader();
					dojox.grid.util.fire(n, "focus");
				}
			} 
			catch(e){}
		}
	},
	_delayedHeaderFocus: function(){
		if(this.isNavHeader()){
			this.focusHeader();
			this.grid.domNode.focus();
		}
	},
	_initColumnHeaders: function(){
		dojo.forEach(this._headerConnects, dojo.disconnect);
		this._headerConnects = [];
		var headers = this._findHeaderCells();
		for(var i = 0; i < headers.length; i++){
			this._headerConnects.push(dojo.connect(headers[i], "onfocus", this, "doColHeaderFocus"));
			this._headerConnects.push(dojo.connect(headers[i], "onblur", this, "doColHeaderBlur"));
		}
	},
	_findHeaderCells: function(){
		// This should be a one liner:
		//	dojo.query("th[tabindex=-1]", this.grid.viewsHeaderNode);
		// But there is a bug in dojo.query() for IE -- see trac #7037.
		var allHeads = dojo.query("th", this.grid.viewsHeaderNode);
		var headers = [];
		for (var i = 0; i < allHeads.length; i++){
			var aHead = allHeads[i];
			var hasTabIdx = dojo.hasAttr(aHead, "tabIndex");
			var tabindex = dojo.attr(aHead, "tabIndex");
			if (hasTabIdx && tabindex < 0) {
				headers.push(aHead);
			}
		}
		return headers;
	},
	_setActiveColHeader: function(/*Node*/colHeaderNode, /*Integer*/colFocusIdx, /*Integer*/ prevColFocusIdx){
		//console.log("setActiveColHeader() - colHeaderNode:colFocusIdx:prevColFocusIdx = " + colHeaderNode + ":" + colFocusIdx + ":" + prevColFocusIdx);
		dojo.attr(this.grid.domNode, "aria-activedescendant",colHeaderNode.id);
		if (prevColFocusIdx != null && prevColFocusIdx >= 0 && prevColFocusIdx != colFocusIdx){
			dojo.toggleClass(this._findHeaderCells()[prevColFocusIdx],this.focusClass,false);
		}
		dojo.toggleClass(colHeaderNode,this.focusClass, true);
		this._colHeadNode = colHeaderNode;
		this._colHeadFocusIdx = colFocusIdx;
		this._scrollHeader(this._colHeadFocusIdx);
	},
	scrollIntoView: function(){
		var info = (this.cell ? this._scrollInfo(this.cell) : null);
		if(!info || !info.s){
			return null;
		}
		var rt = this.grid.scroller.findScrollTop(this.rowIndex);
		// place cell within horizontal view
		if(info.n && info.sr){
			if(info.n.offsetLeft + info.n.offsetWidth > info.sr.l + info.sr.w){
				info.s.scrollLeft = info.n.offsetLeft + info.n.offsetWidth - info.sr.w;
			}else if(info.n.offsetLeft < info.sr.l){
				info.s.scrollLeft = info.n.offsetLeft;
			}
		}
		// place cell within vertical view
		if(info.r && info.sr){
			if(rt + info.r.offsetHeight > info.sr.t + info.sr.h){
				this.grid.setScrollTop(rt + info.r.offsetHeight - info.sr.h);
			}else if(rt < info.sr.t){
				this.grid.setScrollTop(rt);
			}
		}

		return info.s.scrollLeft;
	},
	_scrollInfo: function(cell, domNode){
		if(cell){
			var cl = cell,
				sbn = cl.view.scrollboxNode,
				sbnr = {
					w: sbn.clientWidth,
					l: sbn.scrollLeft,
					t: sbn.scrollTop,
					h: sbn.clientHeight
				},
				rn = cl.view.getRowNode(this.rowIndex);
			return {
				c: cl,
				s: sbn,
				sr: sbnr,
				n: (domNode ? domNode : cell.getNode(this.rowIndex)),
				r: rn
			};
		}
		return null;
	},
	_scrollHeader: function(currentIdx){
		var info = null;
		if(this._colHeadNode){
			var cell = this.grid.getCell(currentIdx);
			info = this._scrollInfo(cell, cell.getNode(0));
		}
		if(info && info.s && info.sr && info.n){
			// scroll horizontally as needed.
			var scroll = info.sr.l + info.sr.w;
			if(info.n.offsetLeft + info.n.offsetWidth > scroll){
				info.s.scrollLeft = info.n.offsetLeft + info.n.offsetWidth - info.sr.w;
			}else if(info.n.offsetLeft < info.sr.l){
				info.s.scrollLeft = info.n.offsetLeft;
			}else if(dojo.isIE <= 7 && cell && cell.view.headerNode){
				// Trac 7158: scroll dojoxGridHeader for IE7 and lower
				cell.view.headerNode.scrollLeft = info.s.scrollLeft;
			}
		}
	},
	_isHeaderHidden: function(){
		// summary:
		//		determine if the grid headers are hidden
		//		relies on documented technique of setting .dojoxGridHeader { display:none; } 
		// returns: Boolean
		//		true if headers are hidden
		//		false if headers are not hidden
		
		var curView = this.focusView;
		if (!curView){
			// find one so we can determine if headers are hidden
			// there is no focusView after adding items to empty grid (test_data_grid_empty.html)
			for (var i = 0, cView; (cView = this.grid.views.views[i]); i++) {
				if(cView.headerNode ){
					curView=cView;
					break;		
				}	
			}
		}
		return (curView && dojo.getComputedStyle(curView.headerNode).display == "none");
	},
	colSizeAdjust: function (e, colIdx, delta){ // adjust the column specified by colIdx by the specified delta px
		var headers = this._findHeaderCells();
		var view = this.focusView;
		if (!view) {
			for (var i = 0, cView; (cView = this.grid.views.views[i]); i++) {
				// find first view with a tableMap in order to work with empty grid
				if(cView.header.tableMap.map ){
					view=cView;
					break;		
				}
			}
		}
		var curHeader = headers[colIdx];
		if (!view || (colIdx == headers.length-1 && colIdx === 0)){
			return; // can't adjust single col. grid
		}	
		view.content.baseDecorateEvent(e);
		// need to adjust event with header cell info since focus is no longer on header cell
		e.cellNode = curHeader; //this.findCellTarget(e.target, e.rowNode);
		e.cellIndex = view.content.getCellNodeIndex(e.cellNode);
		e.cell = (e.cellIndex >= 0 ? this.grid.getCell(e.cellIndex) : null);
		if (view.header.canResize(e)){ 
			var deltaObj = {
				l: delta
			};
			var drag = view.header.colResizeSetup(e,false);
			view.header.doResizeColumn(drag, null, deltaObj);
			view.update();
		}
	},
	styleRow: function(inRow){
		return;
	},
	setFocusIndex: function(inRowIndex, inCellIndex){
		// summary:
		//	focuses the given grid cell
		// inRowIndex: int
		//	grid row index
		// inCellIndex: int
		//	grid cell index
		this.setFocusCell(this.grid.getCell(inCellIndex), inRowIndex);
	},
	setFocusCell: function(inCell, inRowIndex){
		// summary:
		//	focuses the given grid cell
		// inCell: object
		//	grid cell object
		// inRowIndex: int
		//	grid row index
		if(inCell && !this.isFocusCell(inCell, inRowIndex)){
			this.tabbingOut = false;
			if (this._colHeadNode){
				this.blurHeader();
			}
			this._colHeadNode = this._colHeadFocusIdx = null;
			this.focusGridView();
			this._focusifyCellNode(false);
			this.cell = inCell;
			this.rowIndex = inRowIndex;
			this._focusifyCellNode(true);
		}
		// even if this cell isFocusCell, the document focus may need to be rejiggered
		// call opera on delay to prevent keypress from altering focus
		if(dojo.isOpera){
			setTimeout(dojo.hitch(this.grid, 'onCellFocus', this.cell, this.rowIndex), 1);
		}else{
			this.grid.onCellFocus(this.cell, this.rowIndex);
		}
	},
	next: function(){
		// summary:
		//	focus next grid cell
		if(this.cell){
			var row=this.rowIndex, col=this.cell.index+1, cc=this.grid.layout.cellCount-1, rc=this.grid.rowCount-1;
			if(col > cc){
				col = 0;
				row++;
			}
			if(row > rc){
				col = cc;
				row = rc;
			}
			if(this.grid.edit.isEditing()){ //when editing, only navigate to editable cells
				var nextCell = this.grid.getCell(col);
				if (!this.isLastFocusCell() && !nextCell.editable){
					this.cell=nextCell;
					this.rowIndex=row;
					this.next();
					return;
				}
			}
			this.setFocusIndex(row, col);
		}
	},
	previous: function(){
		// summary:
		//	focus previous grid cell
		if(this.cell){
			var row=(this.rowIndex || 0), col=(this.cell.index || 0) - 1;
			if(col < 0){
				col = this.grid.layout.cellCount-1;
				row--;
			}
			if(row < 0){
				row = 0;
				col = 0;
			}
			if(this.grid.edit.isEditing()){ //when editing, only navigate to editable cells
				var prevCell = this.grid.getCell(col);
				if (!this.isFirstFocusCell() && !prevCell.editable){
					this.cell=prevCell;
					this.rowIndex=row;
					this.previous();
					return;
				}
			}
			this.setFocusIndex(row, col);
		}
	},
	move: function(inRowDelta, inColDelta) {
		// summary:
		//	focus grid cell or  simulate focus to column header based on position relative to current focus
		// inRowDelta: int
		// vertical distance from current focus
		// inColDelta: int
		// horizontal distance from current focus

		var colDir = inColDelta < 0 ? -1 : 1;
		// Handle column headers.
		if(this.isNavHeader()){
			var headers = this._findHeaderCells();
			var savedIdx = currentIdx = dojo.indexOf(headers, this._colHeadNode);
			currentIdx += inColDelta;
			while(currentIdx >=0 && currentIdx < headers.length && headers[currentIdx].style.display == "none"){
				// skip over hidden column headers
				currentIdx += colDir;
			}
			if((currentIdx >= 0) && (currentIdx < headers.length)){
				this._setActiveColHeader(headers[currentIdx],currentIdx, savedIdx);
			}
		}else{
			if(this.cell){
				// Handle grid proper.
				var sc = this.grid.scroller,
					r = this.rowIndex,
					rc = this.grid.rowCount-1,
					row = Math.min(rc, Math.max(0, r+inRowDelta));
				if(inRowDelta){
					if(inRowDelta>0){
						if(row > sc.getLastPageRow(sc.page)){
							//need to load additional data, let scroller do that
							this.grid.setScrollTop(this.grid.scrollTop+sc.findScrollTop(row)-sc.findScrollTop(r));
						}
					}else if(inRowDelta<0){
						if(row <= sc.getPageRow(sc.page)){
							//need to load additional data, let scroller do that
							this.grid.setScrollTop(this.grid.scrollTop-sc.findScrollTop(r)-sc.findScrollTop(row));
						}
					}
				}
				var cc = this.grid.layout.cellCount-1,
				i = this.cell.index,
				col = Math.min(cc, Math.max(0, i+inColDelta));
				var cell = this.grid.getCell(col);
				while(col>=0 && col < cc && cell && cell.hidden === true){
					// skip hidden cells
					col += colDir;
					cell = this.grid.getCell(col);
				}
				if (!cell || cell.hidden === true){
					// don't change col if would move to hidden
					col = i;
				}
				this.setFocusIndex(row, col);
				if(inRowDelta){
					this.grid.updateRow(r);
				}
			}
		}
	},
	previousKey: function(e){
		if(this.grid.edit.isEditing()){
			dojo.stopEvent(e);
			this.previous();
		}else if(!this.isNavHeader() && !this._isHeaderHidden()) {
			this.grid.domNode.focus(); // will call doFocus and set focus into header.
			dojo.stopEvent(e);
		}else{
			this.tabOut(this.grid.domNode);
			if (this._colHeadFocusIdx != null) { // clear grid header focus
				dojo.toggleClass(this._findHeaderCells()[this._colHeadFocusIdx], this.focusClass, false);
				this._colHeadFocusIdx = null;
			}
			this._focusifyCellNode(false);
		}
	},
	nextKey: function(e) {
		var isEmpty = (this.grid.rowCount === 0);
		if(e.target === this.grid.domNode && this._colHeadFocusIdx == null){
			this.focusHeader();
			dojo.stopEvent(e);
		}else if(this.isNavHeader()){
			// if tabbing from col header, then go to grid proper. 
			this.blurHeader();
			if(!this.findAndFocusGridCell()){
				this.tabOut(this.grid.lastFocusNode);
			}
			this._colHeadNode = this._colHeadFocusIdx= null;
		}else if(this.grid.edit.isEditing()){
			dojo.stopEvent(e);
			this.next();
		}else{
			this.tabOut(this.grid.lastFocusNode);
		}
	},
	tabOut: function(inFocusNode){
		this.tabbingOut = true;
		inFocusNode.focus();
	},
	focusGridView: function(){
		dojox.grid.util.fire(this.focusView, "focus");
	},
	focusGrid: function(inSkipFocusCell){
		this.focusGridView();
		this._focusifyCellNode(true);
	},
	findAndFocusGridCell: function(){
		// summary:
		//		find the first focusable grid cell 
		// returns: Boolean
		//		true if focus was set to a cell
		//		false if no cell found to set focus onto
		
		var didFocus = true;
		var isEmpty = (this.grid.rowCount === 0); // If grid is empty this.grid.rowCount == 0
		if (this.isNoFocusCell() && !isEmpty){
			var cellIdx = 0;
			var cell = this.grid.getCell(cellIdx);
			if (cell.hidden) {
				// if first cell isn't visible, use _colHeadFocusIdx 
				// could also use a while loop to find first visible cell - not sure that is worth it
				cellIdx = this.isNavHeader() ? this._colHeadFocusIdx : 0;
			}
			this.setFocusIndex(0, cellIdx);
		}
		else if (this.cell && !isEmpty){
			if (this.focusView && !this.focusView.rowNodes[this.rowIndex]){
				// if rowNode for current index is undefined (likely as a result of a sort and because of #7304) 
				// scroll to that row
				this.grid.scrollToRow(this.rowIndex);
			}
			this.focusGrid();
		}else {
			didFocus = false;
		}
		this._colHeadNode = this._colHeadFocusIdx= null;
		return didFocus;
	},
	focusHeader: function(){
		var headerNodes = this._findHeaderCells();
		var saveColHeadFocusIdx = this._colHeadFocusIdx;
		if (this._isHeaderHidden()){
			// grid header is hidden, focus a cell
			this.findAndFocusGridCell();
		}
		else if (!this._colHeadFocusIdx) {
			if (this.isNoFocusCell()) {
				this._colHeadFocusIdx = 0;
			}
			else {
				this._colHeadFocusIdx = this.cell.index;
			}
		}
		this._colHeadNode = headerNodes[this._colHeadFocusIdx];
		while(this._colHeadNode && this._colHeadFocusIdx >=0 && this._colHeadFocusIdx < headerNodes.length &&
				this._colHeadNode.style.display == "none"){
			// skip over hidden column headers
			this._colHeadFocusIdx++;
			this._colHeadNode = headerNodes[this._colHeadFocusIdx];
		}
		if(this._colHeadNode && this._colHeadNode.style.display != "none"){
			// Column header cells know longer receive actual focus.  So, for keyboard invocation of
			// contextMenu to work, the contextMenu must be bound to the grid.domNode rather than the viewsHeaderNode.
			// unbind the contextmenu from the viewsHeaderNode and to the grid when header cells are active.  Reset
			// the binding back to the viewsHeaderNode when header cells are no longer acive (in blurHeader) #10483
			if (this.headerMenu && this._contextMenuBindNode != this.grid.domNode){
				this.headerMenu.unBindDomNode(this.grid.viewsHeaderNode);
				this.headerMenu.bindDomNode(this.grid.domNode);
				this._contextMenuBindNode = this.grid.domNode;
			}
			this._setActiveColHeader(this._colHeadNode, this._colHeadFocusIdx, saveColHeadFocusIdx);
			this._scrollHeader(this._colHeadFocusIdx);
			this._focusifyCellNode(false);
		}else {
			// all col head nodes are hidden - focus the grid
			this.findAndFocusGridCell();
		}
	},
	blurHeader: function(){
		dojo.removeClass(this._colHeadNode, this.focusClass);
		dojo.removeAttr(this.grid.domNode,"aria-activedescendant");
		// reset contextMenu onto viewsHeaderNode so right mouse on header will invoke (see focusHeader)
		if (this.headerMenu && this._contextMenuBindNode == this.grid.domNode) {
			var viewsHeader = this.grid.viewsHeaderNode;
			this.headerMenu.unBindDomNode(this.grid.domNode);
			this.headerMenu.bindDomNode(viewsHeader);
			this._contextMenuBindNode = viewsHeader;
		}
	},
	doFocus: function(e){
		// trap focus only for grid dom node
		if(e && e.target != e.currentTarget){
			dojo.stopEvent(e);
			return;
		}
		// do not focus for scrolling if grid is about to blur
		if(!this.tabbingOut){
			this.focusHeader();
		}
		this.tabbingOut = false;
		dojo.stopEvent(e);
	},
	doBlur: function(e){
		dojo.stopEvent(e);	// FF2
	},
	doContextMenu: function(e){
	//stop contextMenu event if no header Menu to prevent default/browser contextMenu
		if (!this.headerMenu){
			dojo.stopEvent(e); 
		}
	},
	doLastNodeFocus: function(e){
		if (this.tabbingOut){
			this._focusifyCellNode(false);
		}else if(this.grid.rowCount >0){
			if (this.isNoFocusCell()){
				this.setFocusIndex(0,0);
			}
			this._focusifyCellNode(true);
		}else {
			this.focusHeader();
		}
		this.tabbingOut = false;
		dojo.stopEvent(e);	 // FF2
	},
	doLastNodeBlur: function(e){
		dojo.stopEvent(e);	 // FF2
	},
	doColHeaderFocus: function(e){
		this._setActiveColHeader(e.target,dojo.attr(e.target, "idx"),this._colHeadFocusIdx);
		this._scrollHeader(this.getHeaderIndex());
		dojo.stopEvent(e);
	},
	doColHeaderBlur: function(e){
		dojo.toggleClass(e.target, this.focusClass, false);
	}		
});

}

if(!dojo._hasResource["dojox.grid._EditManager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._EditManager"] = true;
dojo.provide("dojox.grid._EditManager");



dojo.declare("dojox.grid._EditManager", null, {
	// summary:
	//		Controls grid cell editing process. Owned by grid and used internally for editing.
	constructor: function(inGrid){
		// inGrid: dojox.Grid
		//		The dojox.Grid this editor should be attached to
		this.grid = inGrid;
		this.connections = [];
		if(dojo.isIE){
			this.connections.push(dojo.connect(document.body, "onfocus", dojo.hitch(this, "_boomerangFocus")));
		}
	},
	
	info: {},

	destroy: function(){
		dojo.forEach(this.connections,dojo.disconnect);
	},

	cellFocus: function(inCell, inRowIndex){
		// summary:
		//		Invoke editing when cell is focused
		// inCell: cell object
		//		Grid cell object
		// inRowIndex: Integer
		//		Grid row index
		if(this.grid.singleClickEdit || this.isEditRow(inRowIndex)){
			// if same row or quick editing, edit
			this.setEditCell(inCell, inRowIndex);
		}else{
			// otherwise, apply any pending row edits
			this.apply();
		}
		// if dynamic or static editing...
		if(this.isEditing() || (inCell && inCell.editable && inCell.alwaysEditing)){
			// let the editor focus itself as needed
			this._focusEditor(inCell, inRowIndex);
		}
	},

	rowClick: function(e){
		if(this.isEditing() && !this.isEditRow(e.rowIndex)){
			this.apply();
		}
	},

	styleRow: function(inRow){
		if(inRow.index == this.info.rowIndex){
			inRow.customClasses += ' dojoxGridRowEditing';
		}
	},

	dispatchEvent: function(e){
		var c = e.cell, ed = (c && c["editable"]) ? c : 0;
		return ed && ed.dispatchEvent(e.dispatch, e);
	},

	// Editing
	isEditing: function(){
		// summary:
		//		Indicates editing state of the grid.
		// returns: Boolean
		//	 	True if grid is actively editing
		return this.info.rowIndex !== undefined;
	},

	isEditCell: function(inRowIndex, inCellIndex){
		// summary:
		//		Indicates if the given cell is being edited.
		// inRowIndex: Integer
		//		Grid row index
		// inCellIndex: Integer
		//		Grid cell index
		// returns: Boolean
		//	 	True if given cell is being edited
		return (this.info.rowIndex === inRowIndex) && (this.info.cell.index == inCellIndex);
	},

	isEditRow: function(inRowIndex){
		// summary:
		//		Indicates if the given row is being edited.
		// inRowIndex: Integer
		//		Grid row index
		// returns: Boolean
		//	 	True if given row is being edited
		return this.info.rowIndex === inRowIndex;
	},

	setEditCell: function(inCell, inRowIndex){
		// summary:
		//		Set the given cell to be edited
		// inRowIndex: Integer
		//		Grid row index
		// inCell: Object
		//		Grid cell object
		if(!this.isEditCell(inRowIndex, inCell.index) && this.grid.canEdit && this.grid.canEdit(inCell, inRowIndex)){
			this.start(inCell, inRowIndex, this.isEditRow(inRowIndex) || inCell.editable);
		}
	},

	_focusEditor: function(inCell, inRowIndex){
		dojox.grid.util.fire(inCell, "focus", [inRowIndex]);
	},

	focusEditor: function(){
		if(this.isEditing()){
			this._focusEditor(this.info.cell, this.info.rowIndex);
		}
	},

	// implement fix for focus boomerang effect on IE
	_boomerangWindow: 500,
	_shouldCatchBoomerang: function(){
		return this._catchBoomerang > new Date().getTime();
	},
	_boomerangFocus: function(){
		//console.log("_boomerangFocus");
		if(this._shouldCatchBoomerang()){
			// make sure we don't utterly lose focus
			this.grid.focus.focusGrid();
			// let the editor focus itself as needed
			this.focusEditor();
			// only catch once
			this._catchBoomerang = 0;
		}
	},
	_doCatchBoomerang: function(){
		// give ourselves a few ms to boomerang IE focus effects
		if(dojo.isIE){this._catchBoomerang = new Date().getTime() + this._boomerangWindow;}
	},
	// end boomerang fix API

	start: function(inCell, inRowIndex, inEditing){
		this.grid.beginUpdate();
		this.editorApply();
		if(this.isEditing() && !this.isEditRow(inRowIndex)){
			this.applyRowEdit();
			this.grid.updateRow(inRowIndex);
		}
		if(inEditing){
			this.info = { cell: inCell, rowIndex: inRowIndex };
			this.grid.doStartEdit(inCell, inRowIndex); 
			this.grid.updateRow(inRowIndex);
		}else{
			this.info = {};
		}
		this.grid.endUpdate();
		// make sure we don't utterly lose focus
		this.grid.focus.focusGrid();
		// let the editor focus itself as needed
		this._focusEditor(inCell, inRowIndex);
		// give ourselves a few ms to boomerang IE focus effects
		this._doCatchBoomerang();
	},

	_editorDo: function(inMethod){
		var c = this.info.cell;
		//c && c.editor && c.editor[inMethod](c, this.info.rowIndex);
		if(c && c.editable){
			c[inMethod](this.info.rowIndex);
		}
	},

	editorApply: function(){
		this._editorDo("apply");
	},

	editorCancel: function(){
		this._editorDo("cancel");
	},

	applyCellEdit: function(inValue, inCell, inRowIndex){
		if(this.grid.canEdit(inCell, inRowIndex)){
			this.grid.doApplyCellEdit(inValue, inRowIndex, inCell.field);
		}
	},

	applyRowEdit: function(){
		this.grid.doApplyEdit(this.info.rowIndex, this.info.cell.field);
	},

	apply: function(){
		// summary:
		//		Apply a grid edit
		if(this.isEditing()){
			this.grid.beginUpdate();
			this.editorApply();
			this.applyRowEdit();
			this.info = {};
			this.grid.endUpdate();
			this.grid.focus.focusGrid();
			this._doCatchBoomerang();
		}
	},

	cancel: function(){
		// summary:
		//		Cancel a grid edit
		if(this.isEditing()){
			this.grid.beginUpdate();
			this.editorCancel();
			this.info = {};
			this.grid.endUpdate();
			this.grid.focus.focusGrid();
			this._doCatchBoomerang();
		}
	},

	save: function(inRowIndex, inView){
		// summary:
		//		Save the grid editing state
		// inRowIndex: Integer
		//		Grid row index
		// inView: Object
		//		Grid view
		var c = this.info.cell;
		if(this.isEditRow(inRowIndex) && (!inView || c.view==inView) && c.editable){
			c.save(c, this.info.rowIndex);
		}
	},

	restore: function(inView, inRowIndex){
		// summary:
		//		Restores the grid editing state
		// inRowIndex: Integer
		//		Grid row index
		// inView: Object
		//		Grid view
		var c = this.info.cell;
		if(this.isEditRow(inRowIndex) && c.view == inView && c.editable){
			c.restore(c, this.info.rowIndex);
		}
	}
});

}

if(!dojo._hasResource['dojox.grid.Selection']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['dojox.grid.Selection'] = true;
dojo.provide('dojox.grid.Selection');

dojo.declare("dojox.grid.Selection", null, {
	// summary:
	//		Manages row selection for grid. Owned by grid and used internally
	//		for selection. Override to implement custom selection.

	constructor: function(inGrid){
		this.grid = inGrid;
		this.selected = [];

		this.setMode(inGrid.selectionMode);
	},

	mode: 'extended',

	selected: null,
	updating: 0,
	selectedIndex: -1,

	setMode: function(mode){
		if(this.selected.length){
			this.deselectAll();
		}
		if(mode != 'extended' && mode != 'multiple' && mode != 'single' && mode != 'none'){
			this.mode = 'extended';
		}else{
			this.mode = mode;
		}
	},

	onCanSelect: function(inIndex){
		return this.grid.onCanSelect(inIndex);
	},

	onCanDeselect: function(inIndex){
		return this.grid.onCanDeselect(inIndex);
	},

	onSelected: function(inIndex){
	},

	onDeselected: function(inIndex){
	},

	//onSetSelected: function(inIndex, inSelect) { };
	onChanging: function(){
	},

	onChanged: function(){
	},

	isSelected: function(inIndex){
		if(this.mode == 'none'){
			return false;
		}
		return this.selected[inIndex];
	},

	getFirstSelected: function(){
		if(!this.selected.length||this.mode == 'none'){ return -1; }
		for(var i=0, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				return i;
			}
		}
		return -1;
	},

	getNextSelected: function(inPrev){
		if(this.mode == 'none'){ return -1; }
		for(var i=inPrev+1, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				return i;
			}
		}
		return -1;
	},

	getSelected: function(){
		var result = [];
		for(var i=0, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				result.push(i);
			}
		}
		return result;
	},

	getSelectedCount: function(){
		var c = 0;
		for(var i=0; i<this.selected.length; i++){
			if(this.selected[i]){
				c++;
			}
		}
		return c;
	},

	_beginUpdate: function(){
		if(this.updating === 0){
			this.onChanging();
		}
		this.updating++;
	},

	_endUpdate: function(){
		this.updating--;
		if(this.updating === 0){
			this.onChanged();
		}
	},

	select: function(inIndex){
		if(this.mode == 'none'){ return; }
		if(this.mode != 'multiple'){
			this.deselectAll(inIndex);
			this.addToSelection(inIndex);
		}else{
			this.toggleSelect(inIndex);
		}
	},

	addToSelection: function(inIndex){
		if(this.mode == 'none'){ return; }
		if(dojo.isArray(inIndex)){
			dojo.forEach(inIndex, this.addToSelection, this);
			return;
		}
		inIndex = Number(inIndex);
		if(this.selected[inIndex]){
			this.selectedIndex = inIndex;
		}else{
			if(this.onCanSelect(inIndex) !== false){
				this.selectedIndex = inIndex;
				var rowNode = this.grid.getRowNode(inIndex);
				if(rowNode){
					dojo.attr(rowNode,"aria-selected","true");
				}
				this._beginUpdate();
				this.selected[inIndex] = true;
				//this.grid.onSelected(inIndex);
				this.onSelected(inIndex);
				//this.onSetSelected(inIndex, true);
				this._endUpdate();
			}
		}
	},

	deselect: function(inIndex){
		if(this.mode == 'none'){ return; }
		if(dojo.isArray(inIndex)){
			dojo.forEach(inIndex, this.deselect, this);
			return;
		}
		inIndex = Number(inIndex);
		if(this.selectedIndex == inIndex){
			this.selectedIndex = -1;
		}
		if(this.selected[inIndex]){
			if(this.onCanDeselect(inIndex) === false){
				return;
			}
			var rowNode = this.grid.getRowNode(inIndex);
			if(rowNode){
				dojo.attr(rowNode,"aria-selected","false");
			}
			this._beginUpdate();
			delete this.selected[inIndex];
			//this.grid.onDeselected(inIndex);
			this.onDeselected(inIndex);
			//this.onSetSelected(inIndex, false);
			this._endUpdate();
		}
	},

	setSelected: function(inIndex, inSelect){
		this[(inSelect ? 'addToSelection' : 'deselect')](inIndex);
	},

	toggleSelect: function(inIndex){
		if(dojo.isArray(inIndex)){
			dojo.forEach(inIndex, this.toggleSelect, this);
			return;
		}
		this.setSelected(inIndex, !this.selected[inIndex]);
	},

	_range: function(inFrom, inTo, func){
		var s = (inFrom >= 0 ? inFrom : inTo), e = inTo;
		if(s > e){
			e = s;
			s = inTo;
		}
		for(var i=s; i<=e; i++){
			func(i);
		}
	},

	selectRange: function(inFrom, inTo){
		this._range(inFrom, inTo, dojo.hitch(this, "addToSelection"));
	},

	deselectRange: function(inFrom, inTo){
		this._range(inFrom, inTo, dojo.hitch(this, "deselect"));
	},

	insert: function(inIndex){
		this.selected.splice(inIndex, 0, false);
		if(this.selectedIndex >= inIndex){
			this.selectedIndex++;
		}
	},

	remove: function(inIndex){
		this.selected.splice(inIndex, 1);
		if(this.selectedIndex >= inIndex){
			this.selectedIndex--;
		}
	},

	deselectAll: function(inExcept){
		for(var i in this.selected){
			if((i!=inExcept)&&(this.selected[i]===true)){
				this.deselect(i);
			}
		}
	},

	clickSelect: function(inIndex, inCtrlKey, inShiftKey){
		if(this.mode == 'none'){ return; }
		this._beginUpdate();
		if(this.mode != 'extended'){
			this.select(inIndex);
		}else{
			var lastSelected = this.selectedIndex;
			if(!inCtrlKey){
				this.deselectAll(inIndex);
			}
			if(inShiftKey){
				this.selectRange(lastSelected, inIndex);
			}else if(inCtrlKey){
				this.toggleSelect(inIndex);
			}else{
				this.addToSelection(inIndex);
			}
		}
		this._endUpdate();
	},

	clickSelectEvent: function(e){
		this.clickSelect(e.rowIndex, dojo.isCopyKey(e), e.shiftKey);
	},

	clear: function(){
		this._beginUpdate();
		this.deselectAll();
		this._endUpdate();
	}
});

}

if(!dojo._hasResource["dojox.grid._Events"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._Events"] = true;
dojo.provide("dojox.grid._Events");

dojo.declare("dojox.grid._Events", null, {
	// summary:
	//		_Grid mixin that provides default implementations for grid events.
	// description: 
	//		Default synthetic events dispatched for _Grid. dojo.connect to events to
	//		retain default implementation or override them for custom handling.
	
	// cellOverClass: String
	// 		css class to apply to grid cells over which the cursor is placed.
	cellOverClass: "dojoxGridCellOver",
	
	onKeyEvent: function(e){
		// summary: top level handler for Key Events
		this.dispatchKeyEvent(e);
	},

	onContentEvent: function(e){
		// summary: Top level handler for Content events
		this.dispatchContentEvent(e);
	},

	onHeaderEvent: function(e){
		// summary: Top level handler for header events
		this.dispatchHeaderEvent(e);
	},

	onStyleRow: function(inRow){
		// summary:
		//		Perform row styling on a given row. Called whenever row styling is updated.
		//
		// inRow: Object
		// 		Object containing row state information: selected, true if the row is selcted; over:
		// 		true of the mouse is over the row; odd: true if the row is odd. Use customClasses and
		// 		customStyles to control row css classes and styles; both properties are strings.
		//
		// example: onStyleRow({ selected: true, over:true, odd:false })
		var i = inRow;
		i.customClasses += (i.odd?" dojoxGridRowOdd":"") + (i.selected?" dojoxGridRowSelected":"") + (i.over?" dojoxGridRowOver":"");
		this.focus.styleRow(inRow);
		this.edit.styleRow(inRow);
	},
	
	onKeyDown: function(e){
		// summary:
		// 		Grid key event handler. By default enter begins editing and applies edits, escape cancels an edit,
		// 		tab, shift-tab, and arrow keys move grid cell focus.
		if(e.altKey || e.metaKey){
			return;
		}
		var dk = dojo.keys;
		var colIdx;
		switch(e.keyCode){
			case dk.ESCAPE:
				this.edit.cancel();
				break;
			case dk.ENTER:
				if(!this.edit.isEditing()){
					colIdx = this.focus.getHeaderIndex();
					if(colIdx >= 0) {
						this.setSortIndex(colIdx);
						break;
					}else {
						this.selection.clickSelect(this.focus.rowIndex, dojo.isCopyKey(e), e.shiftKey);
					}
					dojo.stopEvent(e);
				}
				if(!e.shiftKey){
					var isEditing = this.edit.isEditing();
					this.edit.apply();
					if(!isEditing){
						this.edit.setEditCell(this.focus.cell, this.focus.rowIndex);
					}
				}
				if (!this.edit.isEditing()){
					var curView = this.focus.focusView || this.views.views[0];  //if no focusView than only one view
					curView.content.decorateEvent(e);
					this.onRowClick(e);
				}
				break;
			case dk.SPACE:
				if(!this.edit.isEditing()){
					colIdx = this.focus.getHeaderIndex();
					if(colIdx >= 0) {
						this.setSortIndex(colIdx);
						break;
					}else {
						this.selection.clickSelect(this.focus.rowIndex, dojo.isCopyKey(e), e.shiftKey);
					}
					dojo.stopEvent(e);
				}
				break;
			case dk.TAB:
				this.focus[e.shiftKey ? 'previousKey' : 'nextKey'](e);
				break;
			case dk.LEFT_ARROW:
			case dk.RIGHT_ARROW:
				if(!this.edit.isEditing()){
					var keyCode = e.keyCode;  // IE seems to lose after stopEvent when modifier keys
					dojo.stopEvent(e);
					colIdx = this.focus.getHeaderIndex();
					if (colIdx >= 0 && (e.shiftKey && e.ctrlKey)){
						this.focus.colSizeAdjust(e, colIdx, (keyCode == dk.LEFT_ARROW ? -1 : 1)*5);
					}
					else{
						var offset = (keyCode == dk.LEFT_ARROW) ? 1 : -1;
						if(dojo._isBodyLtr()){ offset *= -1; }
						this.focus.move(0, offset);
					}
				}
				break;
			case dk.UP_ARROW:
				if(!this.edit.isEditing() && this.focus.rowIndex !== 0){
					dojo.stopEvent(e);
					this.focus.move(-1, 0);
				}
				break;
			case dk.DOWN_ARROW:
				if(!this.edit.isEditing() && this.focus.rowIndex+1 != this.rowCount){
					dojo.stopEvent(e);
					this.focus.move(1, 0);
				}
				break;
			case dk.PAGE_UP:
				if(!this.edit.isEditing() && this.focus.rowIndex !== 0){
					dojo.stopEvent(e);
					if(this.focus.rowIndex != this.scroller.firstVisibleRow+1){
						this.focus.move(this.scroller.firstVisibleRow-this.focus.rowIndex, 0);
					}else{
						this.setScrollTop(this.scroller.findScrollTop(this.focus.rowIndex-1));
						this.focus.move(this.scroller.firstVisibleRow-this.scroller.lastVisibleRow+1, 0);
					}
				}
				break;
			case dk.PAGE_DOWN:
				if(!this.edit.isEditing() && this.focus.rowIndex+1 != this.rowCount){
					dojo.stopEvent(e);
					if(this.focus.rowIndex != this.scroller.lastVisibleRow-1){
						this.focus.move(this.scroller.lastVisibleRow-this.focus.rowIndex-1, 0);
					}else{
						this.setScrollTop(this.scroller.findScrollTop(this.focus.rowIndex+1));
						this.focus.move(this.scroller.lastVisibleRow-this.scroller.firstVisibleRow-1, 0);
					}
				}
				break;
			default:
				break;
		}
	},
	
	onMouseOver: function(e){
		// summary:
		//		Event fired when mouse is over the grid.
		// e: Event
		//		Decorated event object contains reference to grid, cell, and rowIndex
		e.rowIndex == -1 ? this.onHeaderCellMouseOver(e) : this.onCellMouseOver(e);
	},
	
	onMouseOut: function(e){
		// summary:
		//		Event fired when mouse moves out of the grid.
		// e: Event
		//		Decorated event object that contains reference to grid, cell, and rowIndex
		e.rowIndex == -1 ? this.onHeaderCellMouseOut(e) : this.onCellMouseOut(e);
	},
	
	onMouseDown: function(e){
		// summary:
		//		Event fired when mouse is down inside grid.
		// e: Event
		//		Decorated event object that contains reference to grid, cell, and rowIndex
		e.rowIndex == -1 ? this.onHeaderCellMouseDown(e) : this.onCellMouseDown(e);
	},
	
	onMouseOverRow: function(e){
		// summary:
		//		Event fired when mouse is over any row (data or header).
		// e: Event
		//		Decorated event object contains reference to grid, cell, and rowIndex
		if(!this.rows.isOver(e.rowIndex)){
			this.rows.setOverRow(e.rowIndex);
			e.rowIndex == -1 ? this.onHeaderMouseOver(e) : this.onRowMouseOver(e);
		}
	},
	onMouseOutRow: function(e){
		// summary:
		//		Event fired when mouse moves out of any row (data or header).
		// e: Event
		//		Decorated event object contains reference to grid, cell, and rowIndex
		if(this.rows.isOver(-1)){
			this.onHeaderMouseOut(e);
		}else if(!this.rows.isOver(-2)){
			this.rows.setOverRow(-2);
			this.onRowMouseOut(e);
		}
	},
	
	onMouseDownRow: function(e){
		// summary:
		//		Event fired when mouse is down inside grid row
		// e: Event
		//		Decorated event object that contains reference to grid, cell, and rowIndex
		if(e.rowIndex != -1)
			this.onRowMouseDown(e);
	},

	// cell events
	onCellMouseOver: function(e){
		// summary:
		//		Event fired when mouse is over a cell.
		// e: Event
		//		Decorated event object contains reference to grid, cell, and rowIndex
		if(e.cellNode){
			dojo.addClass(e.cellNode, this.cellOverClass);
		}
	},
	
	onCellMouseOut: function(e){
		// summary:
		//		Event fired when mouse moves out of a cell.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		if(e.cellNode){
			dojo.removeClass(e.cellNode, this.cellOverClass);
		}
	},
	
	onCellMouseDown: function(e){
		// summary:
		//		Event fired when mouse is down in a header cell.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onCellClick: function(e){
		// summary:
		//		Event fired when a cell is clicked.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		this._click[0] = this._click[1];
		this._click[1] = e;
		if(!this.edit.isEditCell(e.rowIndex, e.cellIndex)){
			this.focus.setFocusCell(e.cell, e.rowIndex);
		}
		this.onRowClick(e);
	},

	onCellDblClick: function(e){
		// summary:
		//		Event fired when a cell is double-clicked.
		// e: Event
		//		Decorated event object contains reference to grid, cell, and rowIndex
		if(this._click.length > 1 && dojo.isIE){
			this.edit.setEditCell(this._click[1].cell, this._click[1].rowIndex);
		}else if(this._click.length > 1 && this._click[0].rowIndex != this._click[1].rowIndex){
			this.edit.setEditCell(this._click[0].cell, this._click[0].rowIndex);
		}else{
			this.edit.setEditCell(e.cell, e.rowIndex);
		}
		this.onRowDblClick(e);
	},

	onCellContextMenu: function(e){
		// summary:
		//		Event fired when a cell context menu is accessed via mouse right click.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		this.onRowContextMenu(e);
	},

	onCellFocus: function(inCell, inRowIndex){
		// summary:
		//		Event fired when a cell receives focus.
		// inCell: Object
		//		Cell object containing properties of the grid column.
		// inRowIndex: Integer
		//		Index of the grid row
		this.edit.cellFocus(inCell, inRowIndex);
	},

	// row events
	onRowClick: function(e){
		// summary:
		//		Event fired when a row is clicked.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		this.edit.rowClick(e);
		this.selection.clickSelectEvent(e);
	},

	onRowDblClick: function(e){
		// summary:
		//		Event fired when a row is double clicked.
		// e: Event
		//		decorated event object which contains reference to grid, cell, and rowIndex
	},

	onRowMouseOver: function(e){
		// summary:
		//		Event fired when mouse moves over a data row.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onRowMouseOut: function(e){
		// summary:
		//		Event fired when mouse moves out of a data row.
		// e: Event
		// 		Decorated event object contains reference to grid, cell, and rowIndex
	},
	
	onRowMouseDown: function(e){
		// summary:
		//		Event fired when mouse is down in a row.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onRowContextMenu: function(e){
		// summary:
		//		Event fired when a row context menu is accessed via mouse right click.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
		dojo.stopEvent(e);
	},

	// header events
	onHeaderMouseOver: function(e){
		// summary:
		//		Event fired when mouse moves over the grid header.
		// e: Event
		// 		Decorated event object contains reference to grid, cell, and rowIndex
	},

	onHeaderMouseOut: function(e){
		// summary:
		//		Event fired when mouse moves out of the grid header.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onHeaderCellMouseOver: function(e){
		// summary:
		//		Event fired when mouse moves over a header cell.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
		if(e.cellNode){
			dojo.addClass(e.cellNode, this.cellOverClass);
		}
	},

	onHeaderCellMouseOut: function(e){
		// summary:
		//		Event fired when mouse moves out of a header cell.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
		if(e.cellNode){
			dojo.removeClass(e.cellNode, this.cellOverClass);
		}
	},
	
	onHeaderCellMouseDown: function(e) {
		// summary:
		//		Event fired when mouse is down in a header cell.
		// e: Event
		// 		Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onHeaderClick: function(e){
		// summary:
		//		Event fired when the grid header is clicked.
		// e: Event
		// Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onHeaderCellClick: function(e){
		// summary:
		//		Event fired when a header cell is clicked.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		this.setSortIndex(e.cell.index);
		this.onHeaderClick(e);
	},

	onHeaderDblClick: function(e){
		// summary:
		//		Event fired when the grid header is double clicked.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
	},

	onHeaderCellDblClick: function(e){
		// summary:
		//		Event fired when a header cell is double clicked.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		this.onHeaderDblClick(e);
	},

	onHeaderCellContextMenu: function(e){
		// summary:
		//		Event fired when a header cell context menu is accessed via mouse right click.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		this.onHeaderContextMenu(e);
	},

	onHeaderContextMenu: function(e){
		// summary:
		//		Event fired when the grid header context menu is accessed via mouse right click.
		// e: Event
		//		Decorated event object which contains reference to grid, cell, and rowIndex
		if(!this.headerMenu){
			dojo.stopEvent(e);
		}
	},

	// editing
	onStartEdit: function(inCell, inRowIndex){
		// summary:
		//		Event fired when editing is started for a given grid cell
		// inCell: Object
		//		Cell object containing properties of the grid column.
		// inRowIndex: Integer
		//		Index of the grid row
	},

	onApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
		// summary:
		//		Event fired when editing is applied for a given grid cell
		// inValue: String
		//		Value from cell editor
		// inRowIndex: Integer
		//		Index of the grid row
		// inFieldIndex: Integer
		//		Index in the grid's data store
	},

	onCancelEdit: function(inRowIndex){
		// summary:
		//		Event fired when editing is cancelled for a given grid cell
		// inRowIndex: Integer
		//		Index of the grid row
	},

	onApplyEdit: function(inRowIndex){
		// summary:
		//		Event fired when editing is applied for a given grid row
		// inRowIndex: Integer
		//		Index of the grid row
	},

	onCanSelect: function(inRowIndex){
		// summary:
		//		Event to determine if a grid row may be selected
		// inRowIndex: Integer
		//		Index of the grid row
		// returns: Boolean
		//		true if the row can be selected
		return true;
	},

	onCanDeselect: function(inRowIndex){
		// summary:
		//		Event to determine if a grid row may be deselected
		// inRowIndex: Integer
		//		Index of the grid row
		// returns: Boolean
		//		true if the row can be deselected
		return true;
	},

	onSelected: function(inRowIndex){
		// summary:
		//		Event fired when a grid row is selected
		// inRowIndex: Integer
		//		Index of the grid row
		this.updateRowStyles(inRowIndex);
	},

	onDeselected: function(inRowIndex){
		// summary:
		//		Event fired when a grid row is deselected
		// inRowIndex: Integer
		//		Index of the grid row
		this.updateRowStyles(inRowIndex);
	},

	onSelectionChanged: function(){
	}
});

}

if(!dojo._hasResource["dojox.grid._Grid"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._Grid"] = true;
dojo.provide("dojox.grid._Grid");




















(function(){
	// NOTE: this is for backwards compatibility with Dojo 1.3
	if(!dojo.isCopyKey){
		dojo.isCopyKey = dojo.dnd.getCopyKeyState;
	}
	/*=====
	dojox.grid.__CellDef = function(){
		//	name: String?
		//		The text to use in the header of the grid for this cell.
		//	get: Function?
		//		function(rowIndex){} rowIndex is of type Integer.  This
		//		function will be called when a cell	requests data.  Returns the
		//		unformatted data for the cell.
		//	value: String?
		//		If "get" is not specified, this is used as the data for the cell.
		//	defaultValue: String?
		//		If "get" and "value" aren't specified or if "get" returns an undefined
		//		value, this is used as the data for the cell.  "formatter" is not run
		//		on this if "get" returns an undefined value.
		//	formatter: Function?
		//		function(data, rowIndex){} data is of type anything, rowIndex
		//		is of type Integer.  This function will be called after the cell
		//		has its data but before it passes it back to the grid to render.
		//		Returns the formatted version of the cell's data.
		//	type: dojox.grid.cells._Base|Function?
		//		TODO
		//	editable: Boolean?
		//		Whether this cell should be editable or not.
		//	hidden: Boolean?
		//		If true, the cell will not be displayed.
		//	noresize: Boolean?
		//		If true, the cell will not be able to be resized.
		//	width: Integer|String?
		//		A CSS size.  If it's an Integer, the width will be in em's.
		//	colSpan: Integer?
		//		How many columns to span this cell.  Will not work in the first
		//		sub-row of cells.
		//	rowSpan: Integer?
		//		How many sub-rows to span this cell.
		//	styles: String?
		//		A string of styles to apply to both the header cell and main
		//		grid cells.  Must end in a ';'.
		//	headerStyles: String?
		//		A string of styles to apply to just the header cell.  Must end
		//		in a ';'
		//	cellStyles: String?
		//		A string of styles to apply to just the main grid cells.  Must
		//		end in a ';'
		//	classes: String?
		//		A space separated list of classes to apply to both the header
		//		cell and the main grid cells.
		//	headerClasses: String?
		//		A space separated list of classes to apply to just the header
		//		cell.
		//	cellClasses: String?
		//		A space separated list of classes to apply to just the main
		//		grid cells.
		//	attrs: String?
		//		A space separated string of attribute='value' pairs to add to
		//		the header cell element and main grid cell elements.
		this.name = name;
		this.value = value;
		this.get = get;
		this.formatter = formatter;
		this.type = type;
		this.editable = editable;
		this.hidden = hidden;
		this.width = width;
		this.colSpan = colSpan;
		this.rowSpan = rowSpan;
		this.styles = styles;
		this.headerStyles = headerStyles;
		this.cellStyles = cellStyles;
		this.classes = classes;
		this.headerClasses = headerClasses;
		this.cellClasses = cellClasses;
		this.attrs = attrs;
	}
	=====*/

	/*=====
	dojox.grid.__ViewDef = function(){
		//	noscroll: Boolean?
		//		If true, no scrollbars will be rendered without scrollbars.
		//	width: Integer|String?
		//		A CSS size.  If it's an Integer, the width will be in em's. If
		//		"noscroll" is true, this value is ignored.
		//	cells: dojox.grid.__CellDef[]|Array[dojox.grid.__CellDef[]]?
		//		The structure of the cells within this grid.
		//	type: String?
		//		A string containing the constructor of a subclass of
		//		dojox.grid._View.  If this is not specified, dojox.grid._View
		//		is used.
		//	defaultCell: dojox.grid.__CellDef?
		//		A cell definition with default values for all cells in this view.  If
		//		a property is defined in a cell definition in the "cells" array and
		//		this property, the cell definition's property will override this
		//		property's property.
		//	onBeforeRow: Function?
		//		function(rowIndex, cells){} rowIndex is of type Integer, cells
		//		is of type Array[dojox.grid.__CellDef[]].  This function is called
		//		before each row of data is rendered.  Before the header is
		//		rendered, rowIndex will be -1.  "cells" is a reference to the
		//		internal structure of this view's cells so any changes you make to
		//		it will persist between calls.
		//	onAfterRow: Function?
		//		function(rowIndex, cells, rowNode){} rowIndex is of type Integer, cells
		//		is of type Array[dojox.grid.__CellDef[]], rowNode is of type DOMNode.
		//		This function is called	after each row of data is rendered.  After the
		//		header is rendered, rowIndex will be -1.  "cells" is a reference to the
		//		internal structure of this view's cells so any changes you make to
		//		it will persist between calls.
		this.noscroll = noscroll;
		this.width = width;
		this.cells = cells;
		this.type = type;
		this.defaultCell = defaultCell;
		this.onBeforeRow = onBeforeRow;
		this.onAfterRow = onAfterRow;
	}
	=====*/

	dojo.declare('dojox.grid._Grid',
		[ dijit._Widget, dijit._Templated, dojox.grid._Events ],
		{
		// summary:
		// 		A grid widget with virtual scrolling, cell editing, complex rows,
		// 		sorting, fixed columns, sizeable columns, etc.
		//
		//	description:
		//		_Grid provides the full set of grid features without any
		//		direct connection to a data store.
		//
		//		The grid exposes a get function for the grid, or optionally
		//		individual columns, to populate cell contents.
		//
		//		The grid is rendered based on its structure, an object describing
		//		column and cell layout.
		//
		//	example:
		//		A quick sample:
		//
		//		define a get function
		//	|	function get(inRowIndex){ // called in cell context
		//	|		return [this.index, inRowIndex].join(', ');
		//	|	}
		//
		//		define the grid structure:
		//	|	var structure = [ // array of view objects
		//	|		{ cells: [// array of rows, a row is an array of cells
		//	|			[
		//	|				{ name: "Alpha", width: 6 },
		//	|				{ name: "Beta" },
		//	|				{ name: "Gamma", get: get }]
		//	|		]}
		//	|	];
		//
		//	|	<div id="grid"
		//	|		rowCount="100" get="get"
		//	|		structure="structure"
		//	|		dojoType="dojox.grid._Grid"></div>

		templateString:"<div hidefocus=\"hidefocus\" wairole=\"grid\" dojoAttachEvent=\"onmouseout:_mouseOut\">\r\n\t<div class=\"dojoxGridMasterHeader\" dojoAttachPoint=\"viewsHeaderNode\" wairole=\"presentation\"></div>\r\n\t<div class=\"dojoxGridMasterView\" dojoAttachPoint=\"viewsNode\" wairole=\"presentation\"></div>\r\n\t<div class=\"dojoxGridMasterMessages\" style=\"display: none;\" dojoAttachPoint=\"messagesNode\"></div>\r\n\t<span dojoAttachPoint=\"lastFocusNode\" tabindex=\"0\"></span>\r\n</div>\r\n",

		// classTag: String
		// 		CSS class applied to the grid's domNode
		classTag: 'dojoxGrid',

		// settings
		// rowCount: Integer
		//		Number of rows to display.
		rowCount: 5,

		// keepRows: Integer
		//		Number of rows to keep in the rendering cache.
		keepRows: 75,

		// rowsPerPage: Integer
		//		Number of rows to render at a time.
		rowsPerPage: 25,

		// autoWidth: Boolean
		//		If autoWidth is true, grid width is automatically set to fit the data.
		autoWidth: false,
		
		// initialWidth: String
		//		A css string to use to set our initial width (only used if autoWidth
		//		is true).  The first rendering of the grid will be this width, any
		//		resizing of columns, etc will result in the grid switching to 
		//		autoWidth mode.  Note, this width will override any styling in a
		//		stylesheet or directly on the node.
		initialWidth: "",

		// autoHeight: Boolean|Integer
		//		If autoHeight is true, grid height is automatically set to fit the data.
		//		If it is an integer, the height will be automatically set to fit the data
		//		if there are fewer than that many rows - and the height will be set to show
		//		that many rows if there are more
		autoHeight: '',

		// rowHeight: Integer
		//		If rowHeight is set to a positive number, it will define the height of the rows
		//		in pixels. This can provide a significant performance advantage, since it
		//		eliminates the need to measure row sizes during rendering, which is one
		// 		the primary bottlenecks in the DataGrid's performance. 
		rowHeight: 0,
		
		// autoRender: Boolean
		//		If autoRender is true, grid will render itself after initialization.
		autoRender: true,

		// defaultHeight: String
		//		default height of the grid, measured in any valid css unit.
		defaultHeight: '15em',
		
		// height: String
		//		explicit height of the grid, measured in any valid css unit.  This will be populated (and overridden)
		//		if the height: css attribute exists on the source node.
		height: '',

		// structure: dojox.grid.__ViewDef|dojox.grid.__ViewDef[]|dojox.grid.__CellDef[]|Array[dojox.grid.__CellDef[]]
		//		View layout defintion.
		structure: null,

		// elasticView: Integer
		//	Override defaults and make the indexed grid view elastic, thus filling available horizontal space.
		elasticView: -1,

		// singleClickEdit: boolean
		//		Single-click starts editing. Default is double-click
		singleClickEdit: false,

		// selectionMode: String
		//		Set the selection mode of grid's Selection.  Value must be 'single', 'multiple',
		//		or 'extended'.  Default is 'extended'.
		selectionMode: 'extended',

		// rowSelector: Boolean|String
		// 		If set to true, will add a row selector view to this grid.  If set to a CSS width, will add
		// 		a row selector of that width to this grid.
		rowSelector: '',

		// columnReordering: Boolean
		// 		If set to true, will add drag and drop reordering to views with one row of columns.
		columnReordering: false,

		// headerMenu: dijit.Menu
		// 		If set to a dijit.Menu, will use this as a context menu for the grid headers.
		headerMenu: null,

		// placeholderLabel: String
		// 		Label of placeholders to search for in the header menu to replace with column toggling
		// 		menu items.
		placeholderLabel: "GridColumns",
		
		// selectable: Boolean
		//		Set to true if you want to be able to select the text within the grid.
		selectable: false,
		
		// Used to store the last two clicks, to ensure double-clicking occurs based on the intended row
		_click: null,
		
		// loadingMessage: String
		//  Message that shows while the grid is loading
		loadingMessage: "<span class='dojoxGridLoading'>${loadingState}</span>",

		// errorMessage: String
		//  Message that shows when the grid encounters an error loading
		errorMessage: "<span class='dojoxGridError'>${errorState}</span>",

		// noDataMessage: String
		//  Message that shows if the grid has no data - wrap it in a 
		//  span with class 'dojoxGridNoData' if you want it to be
		//  styled similar to the loading and error messages
		noDataMessage: "",
		
		// escapeHTMLInData: Boolean
		//		This will escape HTML brackets from the data to prevent HTML from 
		// 		user-inputted data being rendered with may contain JavaScript and result in 
		// 		XSS attacks. This is true by default, and it is recommended that it remain 
		// 		true. Setting this to false will allow data to be displayed in the grid without 
		// 		filtering, and should be only used if it is known that the data won't contain 
		// 		malicious scripts. If HTML is needed in grid cells, it is recommended that 
		// 		you use the formatter function to generate the HTML (the output of 
		// 		formatter functions is not filtered, even with escapeHTMLInData set to true).
		escapeHTMLInData: true,	
		
		// formatterScope: Object
		//		An object to execute format functions within.  If not set, the
		//		format functions will execute within the scope of the cell that
		//		has a format function.
		formatterScope: null,
		
		// editable: boolean
		// indicates if the grid contains editable cells, default is false
		// set to true if editable cell encountered during rendering 
		editable: false,
		
		// private
		sortInfo: 0,
		themeable: true,
		_placeholders: null,

		// _layoutClass: Object
		//	The class to use for our layout - can be overridden by grid subclasses
		_layoutClass: dojox.grid._Layout,

		// initialization
		buildRendering: function(){
			this.inherited(arguments);
			if(!this.domNode.getAttribute('tabIndex')){
				this.domNode.tabIndex = "0";
			}
			this.createScroller();
			this.createLayout();
			this.createViews();
			this.createManagers();

			this.createSelection();

			this.connect(this.selection, "onSelected", "onSelected");
			this.connect(this.selection, "onDeselected", "onDeselected");
			this.connect(this.selection, "onChanged", "onSelectionChanged");

			dojox.html.metrics.initOnFontResize();
			this.connect(dojox.html.metrics, "onFontResize", "textSizeChanged");
			dojox.grid.util.funnelEvents(this.domNode, this, 'doKeyEvent', dojox.grid.util.keyEvents);
			if (this.selectionMode != "none") {
				dojo.attr(this.domNode, "aria-multiselectable", this.selectionMode == "single" ? "false" : "true");
			}

			dojo.addClass(this.domNode, this.classTag);
			if(!this.isLeftToRight()){
				dojo.addClass(this.domNode, this.classTag+"Rtl");
			}
		},
		
		postMixInProperties: function(){
			this.inherited(arguments);
			var messages = dojo.i18n.getLocalization("dijit", "loading", this.lang);
			this.loadingMessage = dojo.string.substitute(this.loadingMessage, messages);
			this.errorMessage = dojo.string.substitute(this.errorMessage, messages);
			if(this.srcNodeRef && this.srcNodeRef.style.height){
				this.height = this.srcNodeRef.style.height;
			}
			// Call this to update our autoheight to start out
			this._setAutoHeightAttr(this.autoHeight, true);
			this.lastScrollTop = this.scrollTop = 0;
		},
		
		postCreate: function(){
			this._placeholders = [];
			this._setHeaderMenuAttr(this.headerMenu);
			this._setStructureAttr(this.structure);
			this._click = [];
			this.inherited(arguments);
			if(this.domNode && this.autoWidth && this.initialWidth){
				this.domNode.style.width = this.initialWidth;
			}
			if (this.domNode && !this.editable){
				// default value for aria-readonly is false, set to true if grid is not editable
				dojo.attr(this.domNode,"aria-readonly", "true");
			}
		},

		destroy: function(){
			this.domNode.onReveal = null;
			this.domNode.onSizeChange = null;

			// Fixes IE domNode leak
			delete this._click;

			this.edit.destroy();
			delete this.edit;

			this.views.destroyViews();
			if(this.scroller){
				this.scroller.destroy();
				delete this.scroller;
			}
			if(this.focus){
				this.focus.destroy();
				delete this.focus;
			}
			if(this.headerMenu&&this._placeholders.length){
				dojo.forEach(this._placeholders, function(p){ p.unReplace(true); });
				this.headerMenu.unBindDomNode(this.viewsHeaderNode);
			}
			this.inherited(arguments);
		},

		_setAutoHeightAttr: function(ah, skipRender){
			// Calculate our autoheight - turn it into a boolean or an integer
			if(typeof ah == "string"){
				if(!ah || ah == "false"){
					ah = false;
				}else if (ah == "true"){
					ah = true;
				}else{
					ah = window.parseInt(ah, 10);
				}
			}
			if(typeof ah == "number"){
				if(isNaN(ah)){
					ah = false;
				}
				// Autoheight must be at least 1, if it's a number.  If it's
				// less than 0, we'll take that to mean "all" rows (same as 
				// autoHeight=true - if it is equal to zero, we'll take that
				// to mean autoHeight=false
				if(ah < 0){
					ah = true;
				}else if (ah === 0){
					ah = false;
				}
			}
			this.autoHeight = ah;
			if(typeof ah == "boolean"){
				this._autoHeight = ah;
			}else if(typeof ah == "number"){
				this._autoHeight = (ah >= this.get('rowCount'));
			}else{
				this._autoHeight = false;
			}
			if(this._started && !skipRender){
				this.render();
			}
		},

		_getRowCountAttr: function(){
			return this.updating && this.invalidated && this.invalidated.rowCount != undefined ?
				this.invalidated.rowCount : this.rowCount;
		},
		
		textSizeChanged: function(){
			this.render();
		},

		sizeChange: function(){
			this.update();
		},

		createManagers: function(){
			// summary:
			//		create grid managers for various tasks including rows, focus, selection, editing

			// row manager
			this.rows = new dojox.grid._RowManager(this);
			// focus manager
			this.focus = new dojox.grid._FocusManager(this);
			// edit manager
			this.edit = new dojox.grid._EditManager(this);
		},

		createSelection: function(){
			// summary:	Creates a new Grid selection manager.

			// selection manager
			this.selection = new dojox.grid.Selection(this);
		},

		createScroller: function(){
			// summary: Creates a new virtual scroller
			this.scroller = new dojox.grid._Scroller();
			this.scroller.grid = this;
			this.scroller.renderRow = dojo.hitch(this, "renderRow");
			this.scroller.removeRow = dojo.hitch(this, "rowRemoved");
		},

		createLayout: function(){
			// summary: Creates a new Grid layout
			this.layout = new this._layoutClass(this);
			this.connect(this.layout, "moveColumn", "onMoveColumn");
		},

		onMoveColumn: function(){
			this.render();
		},
		
		onResizeColumn: function(/*int*/ cellIdx){
			// Called when a column is resized.
		},

		// views
		createViews: function(){
			this.views = new dojox.grid._ViewManager(this);
			this.views.createView = dojo.hitch(this, "createView");
		},

		createView: function(inClass, idx){
			var c = dojo.getObject(inClass);
			var view = new c({ grid: this, index: idx });
			this.viewsNode.appendChild(view.domNode);
			this.viewsHeaderNode.appendChild(view.headerNode);
			this.views.addView(view);
			return view;
		},

		buildViews: function(){
			for(var i=0, vs; (vs=this.layout.structure[i]); i++){
				this.createView(vs.type || dojox._scopeName + ".grid._View", i).setStructure(vs);
			}
			this.scroller.setContentNodes(this.views.getContentNodes());
		},

		_setStructureAttr: function(structure){
			var s = structure;
			if(s && dojo.isString(s)){
				dojo.deprecated("dojox.grid._Grid.attr('structure', 'objVar')", "use dojox.grid._Grid.attr('structure', objVar) instead", "2.0");
				s=dojo.getObject(s);
			}
			this.structure = s;
			if(!s){
				if(this.layout.structure){
					s = this.layout.structure;
				}else{
					return;
				}
			}
			this.views.destroyViews();
			if(s !== this.layout.structure){
				this.layout.setStructure(s);
			}
			this._structureChanged();
		},

		setStructure: function(/* dojox.grid.__ViewDef|dojox.grid.__ViewDef[]|dojox.grid.__CellDef[]|Array[dojox.grid.__CellDef[]] */ inStructure){
			// summary:
			//		Install a new structure and rebuild the grid.
			dojo.deprecated("dojox.grid._Grid.setStructure(obj)", "use dojox.grid._Grid.attr('structure', obj) instead.", "2.0");
			this._setStructureAttr(inStructure);
		},
		
		getColumnTogglingItems: function(){
			// Summary: returns an array of dijit.CheckedMenuItem widgets that can be
			//		added to a menu for toggling columns on and off.
			return dojo.map(this.layout.cells, function(cell){
				if(!cell.menuItems){ cell.menuItems = []; }

				var self = this;
				var item = new dijit.CheckedMenuItem({
					label: cell.name,
					checked: !cell.hidden,
					_gridCell: cell,
					onChange: function(checked){
						if(self.layout.setColumnVisibility(this._gridCell.index, checked)){
							var items = this._gridCell.menuItems;
							if(items.length > 1){
								dojo.forEach(items, function(item){
									if(item !== this){
										item.setAttribute("checked", checked);
									}
								}, this);
							}
							checked = dojo.filter(self.layout.cells, function(c){
								if(c.menuItems.length > 1){
									dojo.forEach(c.menuItems, "item.attr('disabled', false);");
								}else{
									c.menuItems[0].attr('disabled', false);
								}
								return !c.hidden;
							});
							if(checked.length == 1){
								dojo.forEach(checked[0].menuItems, "item.attr('disabled', true);");
							}
						}
					},
					destroy: function(){
						var index = dojo.indexOf(this._gridCell.menuItems, this);
						this._gridCell.menuItems.splice(index, 1);
						delete this._gridCell;
						dijit.CheckedMenuItem.prototype.destroy.apply(this, arguments);
					}
				});
				cell.menuItems.push(item);
				return item;
			}, this); // dijit.CheckedMenuItem[]
		},

		_setHeaderMenuAttr: function(menu){
			if(this._placeholders && this._placeholders.length){
				dojo.forEach(this._placeholders, function(p){
					p.unReplace(true);
				});
				this._placeholders = [];
			}
			if(this.headerMenu){
				this.headerMenu.unBindDomNode(this.viewsHeaderNode);
			}
			this.headerMenu = menu;
			if(!menu){ return; }

			this.headerMenu.bindDomNode(this.viewsHeaderNode);
			if(this.headerMenu.getPlaceholders){
				this._placeholders = this.headerMenu.getPlaceholders(this.placeholderLabel);
			}
		},

		setHeaderMenu: function(/* dijit.Menu */ menu){
			dojo.deprecated("dojox.grid._Grid.setHeaderMenu(obj)", "use dojox.grid._Grid.attr('headerMenu', obj) instead.", "2.0");
			this._setHeaderMenuAttr(menu);
		},
		
		setupHeaderMenu: function(){
			if(this._placeholders && this._placeholders.length){
				dojo.forEach(this._placeholders, function(p){
					if(p._replaced){
						p.unReplace(true);
					}
					p.replace(this.getColumnTogglingItems());
				}, this);
			}
		},

		_fetch: function(start){
			this.setScrollTop(0);
		},

		getItem: function(inRowIndex){
			return null;
		},
		
		showMessage: function(message){
			if(message){
				this.messagesNode.innerHTML = message;
				this.messagesNode.style.display = "";
			}else{
				this.messagesNode.innerHTML = "";
				this.messagesNode.style.display = "none";
			}
		},

		_structureChanged: function() {
			this.buildViews();
			if(this.autoRender && this._started){
				this.render();
			}
		},

		hasLayout: function() {
			return this.layout.cells.length;
		},

		// sizing
		resize: function(changeSize, resultSize){
			// summary:
			//		Update the grid's rendering dimensions and resize it
			
			// Calling sizeChange calls update() which calls _resize...so let's
			// save our input values, if any, and use them there when it gets 
			// called.  This saves us an extra call to _resize(), which can
			// get kind of heavy.
			this._pendingChangeSize = changeSize;
			this._pendingResultSize = resultSize;
			this.sizeChange();
		},

		_getPadBorder: function() {
			this._padBorder = this._padBorder || dojo._getPadBorderExtents(this.domNode);
			return this._padBorder;
		},

		_getHeaderHeight: function(){
			var vns = this.viewsHeaderNode.style, t = vns.display == "none" ? 0 : this.views.measureHeader();
			vns.height = t + 'px';
			// header heights are reset during measuring so must be normalized after measuring.
			this.views.normalizeHeaderNodeHeight();
			return t;
		},
		
		_resize: function(changeSize, resultSize){
			// Restore our pending values, if any
			changeSize = changeSize || this._pendingChangeSize;
			resultSize = resultSize || this._pendingResultSize;
			delete this._pendingChangeSize;
			delete this._pendingResultSize;
			// if we have set up everything except the DOM, we cannot resize
			if(!this.domNode){ return; }
			var pn = this.domNode.parentNode;
			if(!pn || pn.nodeType != 1 || !this.hasLayout() || pn.style.visibility == "hidden" || pn.style.display == "none"){
				return;
			}
			// useful measurement
			var padBorder = this._getPadBorder();
			var hh = undefined;
			var h;
			// grid height
			if(this._autoHeight){
				this.domNode.style.height = 'auto';
				this.viewsNode.style.height = '';
			}else if(typeof this.autoHeight == "number"){
				h = hh = this._getHeaderHeight();
				h += (this.scroller.averageRowHeight * this.autoHeight);
				this.domNode.style.height = h + "px";
			}else if(this.domNode.clientHeight <= padBorder.h){
				if(pn == document.body){
					this.domNode.style.height = this.defaultHeight;
				}else if(this.height){
					this.domNode.style.height = this.height;
				}else{
					this.fitTo = "parent";
				}
			}
			// if we are given dimensions, size the grid's domNode to those dimensions
			if(resultSize){
				changeSize = resultSize;
			}
			if(changeSize){
				dojo.marginBox(this.domNode, changeSize);
				this.height = this.domNode.style.height;
				delete this.fitTo;
			}else if(this.fitTo == "parent"){
				h = this._parentContentBoxHeight = this._parentContentBoxHeight || dojo._getContentBox(pn).h;
				this.domNode.style.height = Math.max(0, h) + "px";
			}
			
			var hasFlex = dojo.some(this.views.views, function(v){ return v.flexCells; });

			if(!this._autoHeight && (h || dojo._getContentBox(this.domNode).h) === 0){
				// We need to hide the header, since the Grid is essentially hidden.
				this.viewsHeaderNode.style.display = "none";
			}else{
				// Otherwise, show the header and give it an appropriate height.
				this.viewsHeaderNode.style.display = "block";
				if(!hasFlex && hh === undefined){
					hh = this._getHeaderHeight();
				}
			}
			if(hasFlex){
				hh = undefined;
			}

			// NOTE: it is essential that width be applied before height
			// Header height can only be calculated properly after view widths have been set.
			// This is because flex column width is naturally 0 in Firefox.
			// Therefore prior to width sizing flex columns with spaces are maximally wrapped
			// and calculated to be too tall.
			this.adaptWidth();
			this.adaptHeight(hh);

			this.postresize();
		},

		adaptWidth: function() {
			// private: sets width and position for views and update grid width if necessary
			var doAutoWidth = (!this.initialWidth && this.autoWidth);
			var w = doAutoWidth ? 0 : this.domNode.clientWidth || (this.domNode.offsetWidth - this._getPadBorder().w),
				vw = this.views.arrange(1, w);
			this.views.onEach("adaptWidth");
			if(doAutoWidth){
				this.domNode.style.width = vw + "px";
			}
		},

		adaptHeight: function(inHeaderHeight){
			// private: measures and normalizes header height, then sets view heights, and then updates scroller
			// content extent
			var t = inHeaderHeight === undefined ? this._getHeaderHeight() : inHeaderHeight;
			var h = (this._autoHeight ? -1 : Math.max(this.domNode.clientHeight - t, 0) || 0);
			this.views.onEach('setSize', [0, h]);
			this.views.onEach('adaptHeight');
			if(!this._autoHeight){
				var numScroll = 0, numNoScroll = 0;
				var noScrolls = dojo.filter(this.views.views, function(v){
					var has = v.hasHScrollbar();
					if(has){ numScroll++; }else{ numNoScroll++; }
					return (!has);
				});
				if(numScroll > 0 && numNoScroll > 0){
					dojo.forEach(noScrolls, function(v){
						v.adaptHeight(true);
					});
				}
			}
			if(this.autoHeight === true || h != -1 || (typeof this.autoHeight == "number" && this.autoHeight >= this.get('rowCount'))){
				this.scroller.windowHeight = h;
			}else{
				this.scroller.windowHeight = Math.max(this.domNode.clientHeight - t, 0);
			}
		},

		// startup
		startup: function(){
			if(this._started){return;}
			this.inherited(arguments);
			if(this.autoRender){
				this.render();
			}
		},

		// render
		render: function(){
			// summary:
			//	Render the grid, headers, and views. Edit and scrolling states are reset. To retain edit and
			// scrolling states, see Update.

			if(!this.domNode){return;}
			if(!this._started){return;}

			if(!this.hasLayout()) {
				this.scroller.init(0, this.keepRows, this.rowsPerPage);
				return;
			}
			//
			this.update = this.defaultUpdate;
			this._render();
		},

		_render: function(){
			this.scroller.init(this.get('rowCount'), this.keepRows, this.rowsPerPage);
			this.prerender();
			this.setScrollTop(0);
			this.postrender();
		},

		prerender: function(){
			// if autoHeight, make sure scroller knows not to virtualize; everything must be rendered.
			this.keepRows = this._autoHeight ? 0 : this.keepRows;
			this.scroller.setKeepInfo(this.keepRows);
			this.views.render();
			this._resize();
		},

		postrender: function(){
			this.postresize();
			this.focus.initFocusView();
			// make rows unselectable
			dojo.setSelectable(this.domNode, this.selectable);
		},

		postresize: function(){
			// views are position absolute, so they do not inflate the parent
			if(this._autoHeight){
				var size = Math.max(this.views.measureContent()) + 'px';
				
				this.viewsNode.style.height = size;
			}
		},

		renderRow: function(inRowIndex, inNodes){
			// summary: private, used internally to render rows
			this.views.renderRow(inRowIndex, inNodes, this._skipRowRenormalize);
		},

		rowRemoved: function(inRowIndex){
			// summary: private, used internally to remove rows
			this.views.rowRemoved(inRowIndex);
		},

		invalidated: null,

		updating: false,

		beginUpdate: function(){
			// summary:
			//		Use to make multiple changes to rows while queueing row updating.
			// NOTE: not currently supporting nested begin/endUpdate calls
			this.invalidated = [];
			this.updating = true;
		},

		endUpdate: function(){
			// summary:
			//		Use after calling beginUpdate to render any changes made to rows.
			this.updating = false;
			var i = this.invalidated, r;
			if(i.all){
				this.update();
			}else if(i.rowCount != undefined){
				this.updateRowCount(i.rowCount);
			}else{
				for(r in i){
					this.updateRow(Number(r));
				}
			}
			this.invalidated = [];
		},

		// update
		defaultUpdate: function(){
			// note: initial update calls render and subsequently this function.
			if(!this.domNode){return;}
			if(this.updating){
				this.invalidated.all = true;
				return;
			}
			//this.edit.saveState(inRowIndex);
			this.lastScrollTop = this.scrollTop;
			this.prerender();
			this.scroller.invalidateNodes();
			this.setScrollTop(this.lastScrollTop);
			this.postrender();
			//this.edit.restoreState(inRowIndex);
		},

		update: function(){
			// summary:
			//		Update the grid, retaining edit and scrolling states.
			this.render();
		},

		updateRow: function(inRowIndex){
			// summary:
			//		Render a single row.
			// inRowIndex: Integer
			//		Index of the row to render
			inRowIndex = Number(inRowIndex);
			if(this.updating){
				this.invalidated[inRowIndex]=true;
			}else{
				this.views.updateRow(inRowIndex);
				this.scroller.rowHeightChanged(inRowIndex);
			}
		},

		updateRows: function(startIndex, howMany){
			// summary:
			//		Render consecutive rows at once.
			// startIndex: Integer
			//		Index of the starting row to render
			// howMany: Integer
			//		How many rows to update.
			startIndex = Number(startIndex);
			howMany = Number(howMany);
			var i;
			if(this.updating){
				for(i=0; i<howMany; i++){
					this.invalidated[i+startIndex]=true;
				}
			}else{
				for(i=0; i<howMany; i++){
					this.views.updateRow(i+startIndex, this._skipRowRenormalize);
				}
				this.scroller.rowHeightChanged(startIndex);
			}
		},

		updateRowCount: function(inRowCount){
			//summary:
			//	Change the number of rows.
			// inRowCount: int
			//	Number of rows in the grid.
			if(this.updating){
				this.invalidated.rowCount = inRowCount;
			}else{
				this.rowCount = inRowCount;
				this._setAutoHeightAttr(this.autoHeight, true);
				if(this.layout.cells.length){
					this.scroller.updateRowCount(inRowCount);
				}
				this._resize();				
				if(this.layout.cells.length){
					this.setScrollTop(this.scrollTop);
				}
			}
		},

		updateRowStyles: function(inRowIndex){
			// summary:
			//		Update the styles for a row after it's state has changed.
			this.views.updateRowStyles(inRowIndex);
		},
		getRowNode: function(inRowIndex){
			// summary:
			//		find the rowNode that is not a rowSelector
			if (this.focus.focusView && !(this.focus.focusView instanceof dojox.grid._RowSelector)){
					return this.focus.focusView.rowNodes[inRowIndex];
			}else{ // search through views
				for (var i = 0, cView; (cView = this.views.views[i]); i++) {
					if (!(cView instanceof dojox.grid._RowSelector)) {
						return cView.rowNodes[inRowIndex];
					}
				}
			}
			return null;
		},
		rowHeightChanged: function(inRowIndex){
			// summary:
			//		Update grid when the height of a row has changed. Row height is handled automatically as rows
			//		are rendered. Use this function only to update a row's height outside the normal rendering process.
			// inRowIndex: Integer
			// 		index of the row that has changed height

			this.views.renormalizeRow(inRowIndex);
			this.scroller.rowHeightChanged(inRowIndex);
		},

		// fastScroll: Boolean
		//		flag modifies vertical scrolling behavior. Defaults to true but set to false for slower
		//		scroll performance but more immediate scrolling feedback
		fastScroll: true,

		delayScroll: false,

		// scrollRedrawThreshold: int
		//	pixel distance a user must scroll vertically to trigger grid scrolling.
		scrollRedrawThreshold: (dojo.isIE ? 100 : 50),

		// scroll methods
		scrollTo: function(inTop){
			// summary:
			//		Vertically scroll the grid to a given pixel position
			// inTop: Integer
			//		vertical position of the grid in pixels
			if(!this.fastScroll){
				this.setScrollTop(inTop);
				return;
			}
			var delta = Math.abs(this.lastScrollTop - inTop);
			this.lastScrollTop = inTop;
			if(delta > this.scrollRedrawThreshold || this.delayScroll){
				this.delayScroll = true;
				this.scrollTop = inTop;
				this.views.setScrollTop(inTop);
				if(this._pendingScroll){
					window.clearTimeout(this._pendingScroll);
				}
				var _this = this;
				this._pendingScroll = window.setTimeout(function(){
					delete _this._pendingScroll;
					_this.finishScrollJob();
				}, 200);
			}else{
				this.setScrollTop(inTop);
			}
		},

		finishScrollJob: function(){
			this.delayScroll = false;
			this.setScrollTop(this.scrollTop);
		},

		setScrollTop: function(inTop){
			this.scroller.scroll(this.views.setScrollTop(inTop));
		},

		scrollToRow: function(inRowIndex){
			// summary:
			//		Scroll the grid to a specific row.
			// inRowIndex: Integer
			// 		grid row index
			this.setScrollTop(this.scroller.findScrollTop(inRowIndex) + 1);
		},

		// styling (private, used internally to style individual parts of a row)
		styleRowNode: function(inRowIndex, inRowNode){
			if(inRowNode){
				this.rows.styleRowNode(inRowIndex, inRowNode);
			}
		},
		
		// called when the mouse leaves the grid so we can deselect all hover rows
		_mouseOut: function(e){
			this.rows.setOverRow(-2);
		},
	
		// cells
		getCell: function(inIndex){
			// summary:
			//		Retrieves the cell object for a given grid column.
			// inIndex: Integer
			// 		Grid column index of cell to retrieve
			// returns:
			//		a grid cell
			return this.layout.cells[inIndex];
		},

		setCellWidth: function(inIndex, inUnitWidth){
			this.getCell(inIndex).unitWidth = inUnitWidth;
		},

		getCellName: function(inCell){
			// summary: Returns the cell name of a passed cell
			return "Cell " + inCell.index; // String
		},

		// sorting
		canSort: function(inSortInfo){
			// summary:
			//		Determines if the grid can be sorted
			// inSortInfo: Integer
			//		Sort information, 1-based index of column on which to sort, positive for an ascending sort
			// 		and negative for a descending sort
			// returns: Boolean
			//		True if grid can be sorted on the given column in the given direction
		},

		sort: function(){
		},

		getSortAsc: function(inSortInfo){
			// summary:
			//		Returns true if grid is sorted in an ascending direction.
			inSortInfo = inSortInfo == undefined ? this.sortInfo : inSortInfo;
			return Boolean(inSortInfo > 0); // Boolean
		},

		getSortIndex: function(inSortInfo){
			// summary:
			//		Returns the index of the column on which the grid is sorted
			inSortInfo = inSortInfo == undefined ? this.sortInfo : inSortInfo;
			return Math.abs(inSortInfo) - 1; // Integer
		},

		setSortIndex: function(inIndex, inAsc){
			// summary:
			// 		Sort the grid on a column in a specified direction
			// inIndex: Integer
			// 		Column index on which to sort.
			// inAsc: Boolean
			// 		If true, sort the grid in ascending order, otherwise in descending order
			var si = inIndex +1;
			if(inAsc != undefined){
				si *= (inAsc ? 1 : -1);
			} else if(this.getSortIndex() == inIndex){
				si = -this.sortInfo;
			}
			this.setSortInfo(si);
		},

		setSortInfo: function(inSortInfo){
			if(this.canSort(inSortInfo)){
				this.sortInfo = inSortInfo;
				this.sort();
				this.update();
			}
		},

		// DOM event handler
		doKeyEvent: function(e){
			e.dispatch = 'do' + e.type;
			this.onKeyEvent(e);
		},

		// event dispatch
		//: protected
		_dispatch: function(m, e){
			if(m in this){
				return this[m](e);
			}
			return false;
		},

		dispatchKeyEvent: function(e){
			this._dispatch(e.dispatch, e);
		},

		dispatchContentEvent: function(e){
			this.edit.dispatchEvent(e) || e.sourceView.dispatchContentEvent(e) || this._dispatch(e.dispatch, e);
		},

		dispatchHeaderEvent: function(e){
			e.sourceView.dispatchHeaderEvent(e) || this._dispatch('doheader' + e.type, e);
		},

		dokeydown: function(e){
			this.onKeyDown(e);
		},

		doclick: function(e){
			if(e.cellNode){
				this.onCellClick(e);
			}else{
				this.onRowClick(e);
			}
		},

		dodblclick: function(e){
			if(e.cellNode){
				this.onCellDblClick(e);
			}else{
				this.onRowDblClick(e);
			}
		},

		docontextmenu: function(e){
			if(e.cellNode){
				this.onCellContextMenu(e);
			}else{
				this.onRowContextMenu(e);
			}
		},

		doheaderclick: function(e){
			if(e.cellNode){
				this.onHeaderCellClick(e);
			}else{
				this.onHeaderClick(e);
			}
		},

		doheaderdblclick: function(e){
			if(e.cellNode){
				this.onHeaderCellDblClick(e);
			}else{
				this.onHeaderDblClick(e);
			}
		},

		doheadercontextmenu: function(e){
			if(e.cellNode){
				this.onHeaderCellContextMenu(e);
			}else{
				this.onHeaderContextMenu(e);
			}
		},

		// override to modify editing process
		doStartEdit: function(inCell, inRowIndex){
			this.onStartEdit(inCell, inRowIndex);
		},

		doApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
			this.onApplyCellEdit(inValue, inRowIndex, inFieldIndex);
		},

		doCancelEdit: function(inRowIndex){
			this.onCancelEdit(inRowIndex);
		},

		doApplyEdit: function(inRowIndex){
			this.onApplyEdit(inRowIndex);
		},

		// row editing
		addRow: function(){
			// summary:
			//		Add a row to the grid.
			this.updateRowCount(this.get('rowCount')+1);
		},

		removeSelectedRows: function(){
			// summary:
			//		Remove the selected rows from the grid.
			if(this.allItemsSelected){
				this.updateRowCount(0);
			}else{
				this.updateRowCount(Math.max(0, this.get('rowCount') - this.selection.getSelected().length));
			}
			this.selection.clear();
		}

	});

	dojox.grid._Grid.markupFactory = function(props, node, ctor, cellFunc){
		var d = dojo;
		var widthFromAttr = function(n){
			var w = d.attr(n, "width")||"auto";
			if((w != "auto")&&(w.slice(-2) != "em")&&(w.slice(-1) != "%")){
				w = parseInt(w, 10)+"px";
			}
			return w;
		};
		// if(!props.store){ console.debug("no store!"); }
		// if a structure isn't referenced, do we have enough
		// data to try to build one automatically?
		if(	!props.structure &&
			node.nodeName.toLowerCase() == "table"){

			// try to discover a structure
			props.structure = d.query("> colgroup", node).map(function(cg){
				var sv = d.attr(cg, "span");
				var v = {
					noscroll: (d.attr(cg, "noscroll") == "true") ? true : false,
					__span: (!!sv ? parseInt(sv, 10) : 1),
					cells: []
				};
				if(d.hasAttr(cg, "width")){
					v.width = widthFromAttr(cg);
				}
				return v; // for vendetta
			});
			if(!props.structure.length){
				props.structure.push({
					__span: Infinity,
					cells: [] // catch-all view
				});
			}
			// check to see if we're gonna have more than one view

			// for each tr in our th, create a row of cells
			d.query("thead > tr", node).forEach(function(tr, tr_idx){
				var cellCount = 0;
				var viewIdx = 0;
				var lastViewIdx;
				var cView = null;
				d.query("> th", tr).map(function(th){
					// what view will this cell go into?

					// NOTE:
					//		to prevent extraneous iteration, we start counters over
					//		for each row, incrementing over the surface area of the
					//		structure that colgroup processing generates and
					//		creating cell objects for each <th> to place into those
					//		cell groups.  There's a lot of state-keepking logic
					//		here, but it is what it has to be.
					if(!cView){ // current view book keeping
						lastViewIdx = 0;
						cView = props.structure[0];
					}else if(cellCount >= (lastViewIdx+cView.__span)){
						viewIdx++;
						// move to allocating things into the next view
						lastViewIdx += cView.__span;
						var lastView = cView;
						cView = props.structure[viewIdx];
					}

					// actually define the cell from what markup hands us
					var cell = {
						name: d.trim(d.attr(th, "name")||th.innerHTML),
						colSpan: parseInt(d.attr(th, "colspan")||1, 10),
						type: d.trim(d.attr(th, "cellType")||""),
						id: d.trim(d.attr(th,"id")||"")
					};
					cellCount += cell.colSpan;
					var rowSpan = d.attr(th, "rowspan");
					if(rowSpan){
						cell.rowSpan = rowSpan;
					}
					if(d.hasAttr(th, "width")){
						cell.width = widthFromAttr(th);
					}
					if(d.hasAttr(th, "relWidth")){
						cell.relWidth = window.parseInt(dojo.attr(th, "relWidth"), 10);
					}
					if(d.hasAttr(th, "hidden")){
						cell.hidden = d.attr(th, "hidden") == "true";
					}

					if(cellFunc){
						cellFunc(th, cell);
					}

					cell.type = cell.type ? dojo.getObject(cell.type) : dojox.grid.cells.Cell;

					if(cell.type && cell.type.markupFactory){
						cell.type.markupFactory(th, cell);
					}

					if(!cView.cells[tr_idx]){
						cView.cells[tr_idx] = [];
					}
					cView.cells[tr_idx].push(cell);
				});
			});
		}

		return new ctor(props, node);
	};
})();

}

if(!dojo._hasResource["dojox.grid.DataSelection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid.DataSelection"] = true;
dojo.provide("dojox.grid.DataSelection");


dojo.declare("dojox.grid.DataSelection", dojox.grid.Selection, {
	getFirstSelected: function(){
		var idx = dojox.grid.Selection.prototype.getFirstSelected.call(this);

		if(idx == -1){ return null; }
		return this.grid.getItem(idx);
	},

	getNextSelected: function(inPrev){
		var old_idx = this.grid.getItemIndex(inPrev);
		var idx = dojox.grid.Selection.prototype.getNextSelected.call(this, old_idx);

		if(idx == -1){ return null; }
		return this.grid.getItem(idx);
	},

	getSelected: function(){
		var result = [];
		for(var i=0, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				result.push(this.grid.getItem(i));
			}
		}
		return result;
	},

	addToSelection: function(inItemOrIndex){
		if(this.mode == 'none'){ return; }
		var idx = null;
		if(typeof inItemOrIndex == "number" || typeof inItemOrIndex == "string"){
			idx = inItemOrIndex;
		}else{
			idx = this.grid.getItemIndex(inItemOrIndex);
		}
		dojox.grid.Selection.prototype.addToSelection.call(this, idx);
	},

	deselect: function(inItemOrIndex){
		if(this.mode == 'none'){ return; }
		var idx = null;
		if(typeof inItemOrIndex == "number" || typeof inItemOrIndex == "string"){
			idx = inItemOrIndex;
		}else{
			idx = this.grid.getItemIndex(inItemOrIndex);
		}
		dojox.grid.Selection.prototype.deselect.call(this, idx);
	},

	deselectAll: function(inItemOrIndex){
		var idx = null;
		if(inItemOrIndex || typeof inItemOrIndex == "number"){
			if(typeof inItemOrIndex == "number" || typeof inItemOrIndex == "string"){
				idx = inItemOrIndex;
			}else{
				idx = this.grid.getItemIndex(inItemOrIndex);
			}
			dojox.grid.Selection.prototype.deselectAll.call(this, idx);
		}else{
			this.inherited(arguments);
		}
	}
});

}

if(!dojo._hasResource["dojox.grid.DataGrid"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid.DataGrid"] = true;
dojo.provide("dojox.grid.DataGrid");




/*=====
dojo.declare("dojox.grid.__DataCellDef", dojox.grid.__CellDef, {
	constructor: function(){
		//	field: String?
		//		The attribute to read from the dojo.data item for the row.
		//  fields: String[]?
		//		An array of fields to grab the values of and pass as an array to the grid
		//	get: Function?
		//		function(rowIndex, item?){} rowIndex is of type Integer, item is of type
		//		Object.  This function will be called when a cell requests data.  Returns
		//		the unformatted data for the cell.
	}
});
=====*/

/*=====
dojo.declare("dojox.grid.__DataViewDef", dojox.grid.__ViewDef, {
	constructor: function(){
		//	cells: dojox.grid.__DataCellDef[]|Array[dojox.grid.__DataCellDef[]]?
		//		The structure of the cells within this grid.
		//	defaultCell: dojox.grid.__DataCellDef?
		//		A cell definition with default values for all cells in this view.  If
		//		a property is defined in a cell definition in the "cells" array and
		//		this property, the cell definition's property will override this
		//		property's property.
	}
});
=====*/

dojo.declare("dojox.grid.DataGrid", dojox.grid._Grid, {
	store: null,
	query: null,
	queryOptions: null,
	fetchText: '...',
	sortFields: null,
	
	// updateDelay: int
	//		Time, in milliseconds, to delay updates automatically so that multiple
	//		calls to onSet/onNew/onDelete don't keep rerendering the grid.  Set
	//		to 0 to immediately cause updates.  A higher value will result in
	//		better performance at the expense of responsiveness of the grid.
	updateDelay: 1,

/*=====
	// structure: dojox.grid.__DataViewDef|dojox.grid.__DataViewDef[]|dojox.grid.__DataCellDef[]|Array[dojox.grid.__DataCellDef[]]
	//		View layout defintion.
	structure: '',
=====*/

	// You can specify items instead of a query, if you like.  They do not need
	// to be loaded - but the must be items in the store
	items: null,
	
	_store_connects: null,
	_by_idty: null,
	_by_idx: null,
	_cache: null,
	_pages: null,
	_pending_requests: null,
	_bop: -1,
	_eop: -1,
	_requests: 0,
	rowCount: 0,

	_isLoaded: false,
	_isLoading: false,
	
	postCreate: function(){
		this._pages = [];
		this._store_connects = [];
		this._by_idty = {};
		this._by_idx = [];
		this._cache = [];
		this._pending_requests = {};

		this._setStore(this.store);
		this.inherited(arguments);
	},

	createSelection: function(){
		this.selection = new dojox.grid.DataSelection(this);
	},

	get: function(inRowIndex, inItem){
		// summary: Default data getter.
		// description:
		//		Provides data to display in a grid cell. Called in grid cell context.
		//		So this.cell.index is the column index.
		// inRowIndex: Integer
		//		Row for which to provide data
		// returns:
		//		Data to display for a given grid cell.
		
		if(inItem && this.field == "_item" && !this.fields){
			return inItem;
		}else if(inItem && this.fields){
			var ret = [];
			var s = this.grid.store;
			dojo.forEach(this.fields, function(f){
				ret = ret.concat(s.getValues(inItem, f));
			});
			return ret;
		}else if(!inItem && typeof inRowIndex === "string"){
			return this.inherited(arguments);
		}
		return (!inItem ? this.defaultValue : (!this.field ? this.value : (this.field == "_item" ? inItem : this.grid.store.getValue(inItem, this.field))));
	},

	_checkUpdateStatus: function(){
		if(this.updateDelay > 0){
			var iStarted = false;
			if(this._endUpdateDelay){
				clearTimeout(this._endUpdateDelay);
				delete this._endUpdateDelay;
				iStarted = true;
			}
			if(!this.updating){
				this.beginUpdate();
				iStarted = true;
			}
			if(iStarted){
				var _this = this;
				this._endUpdateDelay = setTimeout(function(){
					delete _this._endUpdateDelay;
					_this.endUpdate();
				}, this.updateDelay);
			}
		}
	},
	
	_onSet: function(item, attribute, oldValue, newValue){
		this._checkUpdateStatus();
		var idx = this.getItemIndex(item);
		if(idx>-1){
			this.updateRow(idx);
		}
	},
	
	_createItem: function(item, index){
		var idty = this._hasIdentity ? this.store.getIdentity(item) : dojo.toJson(this.query) + ":idx:" + index + ":sort:" + dojo.toJson(this.getSortProps());
		var o = this._by_idty[idty] = { idty: idty, item: item };
		return o;
	},

	_addItem: function(item, index, noUpdate){
		this._by_idx[index] = this._createItem(item, index);
		if(!noUpdate){
			this.updateRow(index);
		}
	},

	_onNew: function(item, parentInfo){
		this._checkUpdateStatus();
		var rowCount = this.get('rowCount');
		this._addingItem = true;
		this.updateRowCount(rowCount+1);
		this._addingItem = false;
		this._addItem(item, rowCount);
		this.showMessage();
	},

	_onDelete: function(item){
		this._checkUpdateStatus();
		var idx = this._getItemIndex(item, true);

		if(idx >= 0){
			// When a row is deleted, all rest rows are shifted down,
			// and migrate from page to page. If some page is not 
			// loaded yet empty rows can migrate to initialized pages
			// without refreshing. It causes empty rows in some pages, see:
			// http://bugs.dojotoolkit.org/ticket/6818
			// this code fix this problem by reseting loaded page info
			this._pages = [];
			this._bop = -1;
			this._eop = -1;

			var o = this._by_idx[idx];
			this._by_idx.splice(idx, 1);
			delete this._by_idty[o.idty];
			this.updateRowCount(this.get('rowCount')-1);
			if(this.get('rowCount') === 0){
				this.showMessage(this.noDataMessage);
			}
		}
	},

	_onRevert: function(){
		this._refresh();
	},

	setStore: function(store, query, queryOptions){
		this._setQuery(query, queryOptions);
		this._setStore(store);
		this._refresh(true);
	},
	
	setQuery: function(query, queryOptions){
		this._setQuery(query, queryOptions);
		this._refresh(true);
	},
	
	setItems: function(items){
		this.items = items;
		this._setStore(this.store);
		this._refresh(true);
	},
	
	_setQuery: function(query, queryOptions){
		this.query = query;
		this.queryOptions = queryOptions || this.queryOptions;		
	},

	_setStore: function(store){
		if(this.store && this._store_connects){
			dojo.forEach(this._store_connects, this.disconnect, this);
		}
		this.store = store;

		if(this.store){
			var f = this.store.getFeatures();
			var h = [];

			this._canEdit = !!f["dojo.data.api.Write"] && !!f["dojo.data.api.Identity"];
			this._hasIdentity = !!f["dojo.data.api.Identity"];

			if(!!f["dojo.data.api.Notification"] && !this.items){
				h.push(this.connect(this.store, "onSet", "_onSet"));
				h.push(this.connect(this.store, "onNew", "_onNew"));
				h.push(this.connect(this.store, "onDelete", "_onDelete"));
			}
			if(this._canEdit){
				h.push(this.connect(this.store, "revert", "_onRevert"));
			}

			this._store_connects = h;
		}
	},

	_onFetchBegin: function(size, req){
		if(!this.scroller){ return; }
		if(this.rowCount != size){
			if(req.isRender){
				this.scroller.init(size, this.keepRows, this.rowsPerPage);
				this.rowCount = size;
				this._setAutoHeightAttr(this.autoHeight, true);
				this._skipRowRenormalize = true;
				this.prerender();
				this._skipRowRenormalize = false;
			}else{
				this.updateRowCount(size);
			}
		}
		if(!size){
			this.views.render();
			this._resize();
			this.showMessage(this.noDataMessage);
			this.focus.initFocusView();
		}else{
			this.showMessage();
		}
	},

	_onFetchComplete: function(items, req){
		if(!this.scroller){ return; }
		if(items && items.length > 0){
			//console.log(items);
			dojo.forEach(items, function(item, idx){
				this._addItem(item, req.start+idx, true);
			}, this);
			this.updateRows(req.start, items.length);
			if(req.isRender){
				this.setScrollTop(0);
				this.postrender();
			}else if(this._lastScrollTop){
				this.setScrollTop(this._lastScrollTop);
			}
		}
		delete this._lastScrollTop;
		if(!this._isLoaded){
			this._isLoading = false;
			this._isLoaded = true;
		}
		this._pending_requests[req.start] = false;
	},

	_onFetchError: function(err, req){
		console.log(err);
		delete this._lastScrollTop;
		if(!this._isLoaded){
			this._isLoading = false;
			this._isLoaded = true;
			this.showMessage(this.errorMessage);
		}
		this._pending_requests[req.start] = false;
		this.onFetchError(err, req);
	},

	onFetchError: function(err, req){
	},

	_fetch: function(start, isRender){
		start = start || 0;
		if(this.store && !this._pending_requests[start]){
			if(!this._isLoaded && !this._isLoading){
				this._isLoading = true;
				this.showMessage(this.loadingMessage);
			}
			this._pending_requests[start] = true;
			//console.log("fetch: ", start);
			try{
				if(this.items){
					var items = this.items;
					var store = this.store;
					this.rowsPerPage = items.length;
					var req = {
						start: start,
						count: this.rowsPerPage,
						isRender: isRender
					};
					this._onFetchBegin(items.length, req);
					
					// Load them if we need to
					var waitCount = 0;
					dojo.forEach(items, function(i){
						if(!store.isItemLoaded(i)){ waitCount++; }
					});
					if(waitCount === 0){
						this._onFetchComplete(items, req);
					}else{
						var onItem = function(item){
							waitCount--;
							if(waitCount === 0){
								this._onFetchComplete(items, req);
							}
						};
						dojo.forEach(items, function(i){
							if(!store.isItemLoaded(i)){
								store.loadItem({item: i, onItem: onItem, scope: this});
							}
						}, this);
					}
				}else{
					this.store.fetch({
						start: start,
						count: this.rowsPerPage,
						query: this.query,
						sort: this.getSortProps(),
						queryOptions: this.queryOptions,
						isRender: isRender,
						onBegin: dojo.hitch(this, "_onFetchBegin"),
						onComplete: dojo.hitch(this, "_onFetchComplete"),
						onError: dojo.hitch(this, "_onFetchError")
					});
				}
			}catch(e){
				this._onFetchError(e, {start: start, count: this.rowsPerPage});
			}
		}
	},

	_clearData: function(){
		this.updateRowCount(0);
		this._by_idty = {};
		this._by_idx = [];
		this._pages = [];
		this._bop = this._eop = -1;
		this._isLoaded = false;
		this._isLoading = false;
	},

	getItem: function(idx){
		var data = this._by_idx[idx];
		if(!data||(data&&!data.item)){
			this._preparePage(idx);
			return null;
		}
		return data.item;
	},

	getItemIndex: function(item){
		return this._getItemIndex(item, false);
	},
	
	_getItemIndex: function(item, isDeleted){
		if(!isDeleted && !this.store.isItem(item)){
			return -1;
		}

		var idty = this._hasIdentity ? this.store.getIdentity(item) : null;

		for(var i=0, l=this._by_idx.length; i<l; i++){
			var d = this._by_idx[i];
			if(d && ((idty && d.idty == idty) || (d.item === item))){
				return i;
			}
		}
		return -1;
	},

	filter: function(query, reRender){
		this.query = query;
		if(reRender){
			this._clearData();
		}
		this._fetch();
	},

	_getItemAttr: function(idx, attr){
		var item = this.getItem(idx);
		return (!item ? this.fetchText : this.store.getValue(item, attr));
	},

	// rendering
	_render: function(){
		if(this.domNode.parentNode){
			this.scroller.init(this.get('rowCount'), this.keepRows, this.rowsPerPage);
			this.prerender();
			this._fetch(0, true);
		}
	},

	// paging
	_requestsPending: function(inRowIndex){
		return this._pending_requests[inRowIndex];
	},

	_rowToPage: function(inRowIndex){
		return (this.rowsPerPage ? Math.floor(inRowIndex / this.rowsPerPage) : inRowIndex);
	},

	_pageToRow: function(inPageIndex){
		return (this.rowsPerPage ? this.rowsPerPage * inPageIndex : inPageIndex);
	},

	_preparePage: function(inRowIndex){
		if((inRowIndex < this._bop || inRowIndex >= this._eop) && !this._addingItem){
			var pageIndex = this._rowToPage(inRowIndex);
			this._needPage(pageIndex);
			this._bop = pageIndex * this.rowsPerPage;
			this._eop = this._bop + (this.rowsPerPage || this.get('rowCount'));
		}
	},

	_needPage: function(inPageIndex){
		if(!this._pages[inPageIndex]){
			this._pages[inPageIndex] = true;
			this._requestPage(inPageIndex);
		}
	},

	_requestPage: function(inPageIndex){
		var row = this._pageToRow(inPageIndex);
		var count = Math.min(this.rowsPerPage, this.get('rowCount') - row);
		if(count > 0){
			this._requests++;
			if(!this._requestsPending(row)){
				setTimeout(dojo.hitch(this, "_fetch", row, false), 1);
				//this.requestRows(row, count);
			}
		}
	},

	getCellName: function(inCell){
		return inCell.field;
		//console.log(inCell);
	},

	_refresh: function(isRender){
		this._clearData();
		this._fetch(0, isRender);
	},

	sort: function(){
		this._lastScrollTop = this.scrollTop;
		this._refresh();
	},

	canSort: function(){
		return (!this._isLoading);
	},

	getSortProps: function(){
		var c = this.getCell(this.getSortIndex());
		if(!c){
			if(this.sortFields){
				return this.sortFields;
			}
			return null;
		}else{
			var desc = c["sortDesc"];
			var si = !(this.sortInfo>0);
			if(typeof desc == "undefined"){
				desc = si;
			}else{
				desc = si ? !desc : desc;
			}
			return [{ attribute: c.field, descending: desc }];
		}
	},

	styleRowState: function(inRow){
		// summary: Perform row styling
		if(this.store && this.store.getState){
			var states=this.store.getState(inRow.index), c='';
			for(var i=0, ss=["inflight", "error", "inserting"], s; s=ss[i]; i++){
				if(states[s]){
					c = ' dojoxGridRow-' + s;
					break;
				}
			}
			inRow.customClasses += c;
		}
	},

	onStyleRow: function(inRow){
		this.styleRowState(inRow);
		this.inherited(arguments);
	},

	// editing
	canEdit: function(inCell, inRowIndex){
		return this._canEdit;
	},

	_copyAttr: function(idx, attr){
		var row = {};
		var backstop = {};
		var src = this.getItem(idx);
		return this.store.getValue(src, attr);
	},

	doStartEdit: function(inCell, inRowIndex){
		if(!this._cache[inRowIndex]){
			this._cache[inRowIndex] = this._copyAttr(inRowIndex, inCell.field);
		}
		this.onStartEdit(inCell, inRowIndex);
	},

	doApplyCellEdit: function(inValue, inRowIndex, inAttrName){
		this.store.fetchItemByIdentity({
			identity: this._by_idx[inRowIndex].idty,
			onItem: dojo.hitch(this, function(item){
				var oldValue = this.store.getValue(item, inAttrName);
				if(typeof oldValue == 'number'){
					inValue = isNaN(inValue) ? inValue : parseFloat(inValue);
				}else if(typeof oldValue == 'boolean'){
					inValue = inValue == 'true' ? true : inValue == 'false' ? false : inValue;
				}else if(oldValue instanceof Date){
					var asDate = new Date(inValue);
					inValue = isNaN(asDate.getTime()) ? inValue : asDate;
				}
				this.store.setValue(item, inAttrName, inValue);
				this.onApplyCellEdit(inValue, inRowIndex, inAttrName);
			})
		});
	},

	doCancelEdit: function(inRowIndex){
		var cache = this._cache[inRowIndex];
		if(cache){
			this.updateRow(inRowIndex);
			delete this._cache[inRowIndex];
		}
		this.onCancelEdit.apply(this, arguments);
	},

	doApplyEdit: function(inRowIndex, inDataAttr){
		var cache = this._cache[inRowIndex];
		/*if(cache){
			var data = this.getItem(inRowIndex);
			if(this.store.getValue(data, inDataAttr) != cache){
				this.update(cache, data, inRowIndex);
			}
			delete this._cache[inRowIndex];
		}*/
		this.onApplyEdit(inRowIndex);
	},

	removeSelectedRows: function(){
		// summary:
		//		Remove the selected rows from the grid.
		if(this._canEdit){
			this.edit.apply();
			var fx = dojo.hitch(this, function(items){
				if(items.length){
					dojo.forEach(items, this.store.deleteItem, this.store);
					this.selection.clear();
				}			
			});
			if(this.allItemsSelected){
				this.store.fetch({
							query: this.query, 
							queryOptions: this.queryOptions,
							onComplete: fx});
			}else{
				fx(this.selection.getSelected());
			}
		}
	}
});

dojox.grid.DataGrid.cell_markupFactory = function(cellFunc, node, cellDef){
	var field = dojo.trim(dojo.attr(node, "field")||"");
	if(field){
		cellDef.field = field;
	}
	cellDef.field = cellDef.field||cellDef.name;
	var fields = dojo.trim(dojo.attr(node, "fields")||"");
	if(fields){
		cellDef.fields = fields.split(",");
	}
	if(cellFunc){
		cellFunc(node, cellDef);
	}
};

dojox.grid.DataGrid.markupFactory = function(props, node, ctor, cellFunc){
	return dojox.grid._Grid.markupFactory(props, node, ctor, 
					dojo.partial(dojox.grid.DataGrid.cell_markupFactory, cellFunc));
};

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPDataGrid"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPDataGrid"] = true;
dojo.provide("ibm.tivoli.tip.dijit.TIPDataGrid");




dojo.declare("ibm.tivoli.tip.dijit.TIPDataGrid", 
			  dojox.grid.DataGrid,
{
	loadCompleteCallback: null,
	prevScrollTop: 0,
	prevScrollLeft:0,
	//totalRows: 0,
	
   	setLoadCompleteCallback: function(aFunction){
        this.loadCompleteCallback = aFunction;
    },

	_onFetchComplete: function (items, req) {
		this.inherited (arguments);
		
		console.log ("TIPDataGrid._onFetchComplete.  items: ");
		console.log (items);
		
		console.log ("request: ");
		console.log (req);
		
		//this.totalRows = (items.numRows) ? items.numRows : items.length;
		if(this.prevScrollTop != 0){
		   this.setScrollTop(this.prevScrollTop);
		}
		if(this.prevScrollLeft != 0){		
			for(var i in this.views.views){
						v = this.views.views[i];
						if(v.viewWidth == null || v.viewWidth == ""){			   

							 v.scrollboxNode.scrollLeft = this.prevScrollLeft;
							 v.headerNode.scrollLeft = this.prevScrollLeft;
							
						   break;
						}
					}
					}
		if (this.loadCompleteCallback) {
		    this.loadCompleteCallback();
		}
	},
	
	getTotalRows: function () {
		return (this.rowCount);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPMessageArea"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPMessageArea"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
console.warn("'ibm.tivoli.tip.dijit.TIPMessageArea' is deprecated, use 'dojoe.messagearea.MessageArea'(TED) instead.");



// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPMessageArea");
dojo.provide("ibm.tivoli.tip.dijit.TIPMessageAreaManager");
/******************************************************************************
* Collapsable Message Area (Info, Warning, Error)
*
* The purpose of this widget is to be placed into dialogs and forms to aid in
* providing feedback on form errors.
*
* Author: Joseph Firebaugh
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPMessageArea", [dijit._Widget, dijit._Templated],
{
	//the message box type: critical, info, warning
	type: "critical",
	
	//the message in the are -- can be HTML string
	message: "",
	
	//if you provide a message id then it will use bigger message images
	messageId: null,

	//the duration of time to play the wipe fx on show and close	
	duration: 200,
	
	// url: String: a URL that links the message to an InfoCenter page	
	url: null,
	
	// boolean: create the message area already open?
	isShowing: false,
	
	//the HTML template for the label placement and null value checkbox
	templateString:"<div>\r\n\t<div dojoAttachPoint=\"wipeNode\">\r\n\t<table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\" dojoAttachPoint=\"styleArea\">\r\n\t<tr>\r\n\t\t<td width=\"17\" valign=\"top\">\r\n\t\t\t<img dojoAttachPoint=\"messageIcon\" src=\"${criticalIcon}\" alt=\"${resources.DIALOG_ERROR}\" border=\"0\" style=\"padding-right:7px\" />\r\n\t\t</td>\r\n\t\t<td>\r\n\t\t\t<div dojoAttachPoint=\"messageNode\" class=\"message-description\"></div>\r\n\t\t</td>\r\n\t\t<td width=\"18\">\r\n\t\t\t<img dojoAttachPoint=\"closeBtn\" class=\"messageAreaCloseIcon\" src=\"${closeIcon}\" alt=\"${resources.DIALOG_BUTTON_CLOSE}\" border=\"0\" dojoAttachEvent=\"onclick: _onCloseClick\" />\r\n\t\t</td>\r\n\t</tr>\r\n\t</table>\r\n\t</div>\r\n</div>\r\n",
	   closeIcon: dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/message_close.gif"),
	criticalIcon: dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/i_message_critical_10.png"),	  
	
	constructor: function () 
	{
		this.resources = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");	
	}, 			
	/**************************************************************************
	* Setup the widget
	**************************************************************************/	
	postCreate: function()
	{
		//call super
		this.inherited( "postCreate", arguments );

		//initialize message
		this.setMessageId( this.messageId );
		
		//initialize message
		this.setMessage( this.message );

		//initialize type
		this.setType( this.type );		
		
		//create a WipeIn Effect Object
		this._wipeIn = dojo.fx.wipeIn(
		{
			node: this.wipeNode,
			duration: this.duration
		});
		
		//create a WipeOut Effect Object
		this._wipeOut = dojo.fx.wipeOut(
		{
			node: this.wipeNode,
			duration: this.duration			
		});
		
		if ( !this.isShowing ) 
		{
			this.wipeNode.style.display="none";
		}
	},
	
	/**************************************************************************
	* Set the type of the message box (critical, warning, info)
	**************************************************************************/		
	setType: function( type )
	{
		this.type = type;
		
		this._updateClass();
	},
	
	/**************************************************************************
	* Set the message ( can be HTML string )
	**************************************************************************/		
	setMessage: function( message )
	{
		this.messageNode.innerHTML = message;
	},
	
	/**************************************************************************
	* Set the messageId ( can be HTML string )
	**************************************************************************/		
	setMessageId: function( messageId )
	{
	   this.messageId = messageId;
	   
	   var suffix = "_30.png";
	   
	   if ( messageId === undefined || messageId === null || messageId == "" )
	   {
	     suffix = "_10.png";
	     
	     if( this.messageIdNode )
	     {
	     	this.messageIdNode.parentNode.removeChild( this.messageIdNode );
	     	delete this.messageIdNode;
	     }
	   }
	   else
	   {
	     if( !this.messageIdNode )
	     {
		   	 this.messageIdNode = dojo.doc.createElement( 'DIV' );
		   	 this.messageIdNode.className = "message-id";
		     dojo.place( this.messageIdNode, this.messageNode, "before" );
		 }
		   	 
	   	 var innerHTML = "";
	   	 
	   	 //if infocenter URL set then create hyperlink
	   	 if ( this.url ) 
	   	 {
	   	   innerHTML += '<a href="javascript:void(0)" onclick="window.open(\'' + this.url + '\')">';
		   innerHTML += this.messageId;
		   innerHTML += "</a>";
		 }
		 else 
		 {
		   innerHTML += this.messageId;
	     }
	   	 
	     this.messageIdNode.innerHTML = innerHTML;		     
	   }
	   
	   this.criticalIcon = dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/i_message_critical" + suffix);
           this.infoIcon = dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/i_message_info"  + suffix);
	   this. warningIcon = dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/i_message_warning"  + suffix);	    
	   
	   this._updateClass();	    
	},
	
	/**************************************************************************
	* Show a info message
	* messageID is optional
	**************************************************************************/		
	showInfoMessage: function( message, messageID )
	{
		this.setMessage( message );
		if (messageID)
		{
			this.setMessageId( messageID );
		}
		else
		{
			this.setMessageId( null );
		}
		this.setType( "info" );
		
		this.show();
	},

	/**************************************************************************
	* Show a warning message
	* messageID is optional
	**************************************************************************/		
	showWarningMessage: function( message, messageID )
	{
		this.setMessage( message );
		
		if (messageID)
		{
			this.setMessageId( messageID );
		}
		else
		{
			this.setMessageId( null );
		}
		
		this.setType( "warning" );
		
		this.show();
	},

	/**************************************************************************
	* Show an error message
	* messageID is optional
	**************************************************************************/		
	showErrorMessage: function( message, messageID )
	{
		this.setMessage( message );
		
		if (messageID)
		{
			this.setMessageId( messageID );
		}
		else
		{
			this.setMessageId( null );
		}
		
		this.setType( "critical" );
		
		this.show();
	},
	
	/**************************************************************************
	* Show the message area -- ensures message area not already visible for you
	**************************************************************************/		
	show: function()
	{
		if ( !this.isShowing )
		{
			this.isShowing = true;
			this._wipeIn.play();
		}
	},
	
	/**************************************************************************
	* Hide the message area -- ensures message area not already hidden for you
	**************************************************************************/		
	hide: function()
	{
		if ( this.isShowing )
		{
			this.isShowing = false;
			this._wipeOut.play();
		}
	},
	
	/**************************************************************************
	* updates the CSS styling and icon based on current type setting
	**************************************************************************/		
	_updateClass: function()
	{
		var type = this.type.toLowerCase();
		
		dojo.removeClass( this.styleArea, "critical" );
		dojo.removeClass( this.styleArea, "info" );
		dojo.removeClass( this.styleArea, "warning" );
		
		if( type == "critical" )
		{
			this.messageIcon.src = this.criticalIcon;
			this.messageIcon.alt = this.resources.DIALOG_ERROR;
		}
		else if ( type == "info" )
		{
			this.messageIcon.src = this.infoIcon;
			this.messageIcon.alt = this.resources.DIALOG_INFO;
		}
		else
		{
			this.messageIcon.src = this.warningIcon;
			this.messageIcon.alt = this.resources.DIALOG_WARNING;
		}
		
		dojo.addClass( this.styleArea, type );
		
		//overload the stylesheet (because it was written for a dialog)
		this.styleArea.style.padding="2px";
	},
		
	/**************************************************************************
	* Called when the close button is clicked
	**************************************************************************/				
	_onCloseClick: function()
	{
		this.hide();
	}
});

}

if(!dojo._hasResource["dojox.data.QueryReadStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.data.QueryReadStore"] = true;
dojo.provide("dojox.data.QueryReadStore");




dojo.declare("dojox.data.QueryReadStore",
	null,
	{
		//	summary:
		//		This class provides a store that is mainly intended to be used
		//		for loading data dynamically from the server, used i.e. for
		//		retreiving chunks of data from huge data stores on the server (by server-side filtering!).
		//		Upon calling the fetch() method of this store the data are requested from
		//		the server if they are not yet loaded for paging (or cached).
		//
		//		For example used for a combobox which works on lots of data. It
		//		can be used to retreive the data partially upon entering the
		//		letters "ac" it returns only items like "action", "acting", etc.
		//
		// note:
		//		The field name "id" in a query is reserved for looking up data
		//		by id. This is necessary as before the first fetch, the store
		//		has no way of knowing which field the server will declare as
		//		identifier.
		//
		//	example:
		// |	// The parameter "query" contains the data that are sent to the server.
		// |	var store = new dojox.data.QueryReadStore({url:'/search.php'});
		// |	store.fetch({query:{name:'a'}, queryOptions:{ignoreCase:false}});
		//
		// |	// Since "serverQuery" is given, it overrules and those data are
		// |	// sent to the server.
		// |	var store = new dojox.data.QueryReadStore({url:'/search.php'});
		// |	store.fetch({serverQuery:{name:'a'}, queryOptions:{ignoreCase:false}});
		//
		// |	<div dojoType="dojox.data.QueryReadStore"
		// |		jsId="store2"
		// |		url="../tests/stores/QueryReadStore.php"
		// |		requestMethod="post"></div>
		// |	<div dojoType="dojox.grid.data.DojoData"
		// |		jsId="model2"
		// |		store="store2"
		// |		sortFields="[{attribute: 'name', descending: true}]"
		// |		rowsPerPage="30"></div>
		// |	<div dojoType="dojox.Grid" id="grid2"
		// |		model="model2"
		// |		structure="gridLayout"
		// |		style="height:300px; width:800px;"></div>
	
		//
		//	todo:
		//		- there is a bug in the paging, when i set start:2, count:5 after an initial fetch() and doClientPaging:true
		//		  it returns 6 elemetns, though count=5, try it in QueryReadStore.html
		//		- add optional caching
		//		- when the first query searched for "a" and the next for a subset of
		//		  the first, i.e. "ab" then we actually dont need a server request, if
		//		  we have client paging, we just need to filter the items we already have
		//		  that might also be tooo much logic
		
		url:"",
		requestMethod:"get",
		//useCache:false,
		
		// We use the name in the errors, once the name is fixed hardcode it, may be.
		_className:"dojox.data.QueryReadStore",
		
		// This will contain the items we have loaded from the server.
		// The contents of this array is optimized to satisfy all read-api requirements
		// and for using lesser storage, so the keys and their content need some explaination:
		// 		this._items[0].i - the item itself 
		//		this._items[0].r - a reference to the store, so we can identify the item
		//			securly. We set this reference right after receiving the item from the
		//			server.
		_items:[],
		
		// Store the last query that triggered xhr request to the server.
		// So we can compare if the request changed and if we shall reload 
		// (this also depends on other factors, such as is caching used, etc).
		_lastServerQuery:null,
		
		// Store how many rows we have so that we can pass it to a clientPaging handler
		_numRows:-1,
		
		// Store a hash of the last server request. Actually I introduced this
		// for testing, so I can check if no unnecessary requests were issued for
		// client-side-paging.
		lastRequestHash:null,
		
		// summary:
		//		By default every request for paging is sent to the server.
		doClientPaging:false,
	
		// summary:
		//		By default all the sorting is done serverside before the data is returned
		//		which is the proper place to be doing it for really large datasets.
		doClientSorting:false,
	
		// Items by identify for Identify API
		_itemsByIdentity:null,
		
		// Identifier used
		_identifier:null,
	
		_features: {'dojo.data.api.Read':true, 'dojo.data.api.Identity':true},
	
		_labelAttr: "label",
		
		constructor: function(/* Object */ params){
			dojo.mixin(this,params);
		},
		
		getValue: function(/* item */ item, /* attribute-name-string */ attribute, /* value? */ defaultValue){
			//	According to the Read API comments in getValue() and exception is
			//	thrown when an item is not an item or the attribute not a string!
			this._assertIsItem(item);
			if(!dojo.isString(attribute)){
				throw new Error(this._className+".getValue(): Invalid attribute, string expected!");
			}
			if(!this.hasAttribute(item, attribute)){
				// read api says: return defaultValue "only if *item* does not have a value for *attribute*." 
				// Is this the case here? The attribute doesn't exist, but a defaultValue, sounds reasonable.
				if(defaultValue){
					return defaultValue;
				}
				console.log(this._className+".getValue(): Item does not have the attribute '"+attribute+"'.");
			}
			return item.i[attribute];
		},
		
		getValues: function(/* item */ item, /* attribute-name-string */ attribute){
			this._assertIsItem(item);
			var ret = [];
			if(this.hasAttribute(item, attribute)){
				ret.push(item.i[attribute]);
			}
			return ret;
		},
		
		getAttributes: function(/* item */ item){
			this._assertIsItem(item);
			var ret = [];
			for(var i in item.i){
				ret.push(i);
			}
			return ret;
		},
	
		hasAttribute: function(/* item */ item,	/* attribute-name-string */ attribute){
			//	summary: 
			//		See dojo.data.api.Read.hasAttribute()
			return this.isItem(item) && typeof item.i[attribute]!="undefined";
		},
		
		containsValue: function(/* item */ item, /* attribute-name-string */ attribute, /* anything */ value){
			var values = this.getValues(item, attribute);
			var len = values.length;
			for(var i=0; i<len; i++){
				if(values[i] == value){
					return true;
				}
			}
			return false;
		},
		
		isItem: function(/* anything */ something){
			// Some basic tests, that are quick and easy to do here.
			// >>> var store = new dojox.data.QueryReadStore({});
			// >>> store.isItem("");
			// false
			//
			// >>> var store = new dojox.data.QueryReadStore({});
			// >>> store.isItem({});
			// false
			//
			// >>> var store = new dojox.data.QueryReadStore({});
			// >>> store.isItem(0);
			// false
			//
			// >>> var store = new dojox.data.QueryReadStore({});
			// >>> store.isItem({name:"me", label:"me too"});
			// false
			//
			if(something){
				return typeof something.r != "undefined" && something.r == this;
			}
			return false;
		},
		
		isItemLoaded: function(/* anything */ something){
			// Currently we dont have any state that tells if an item is loaded or not
			// if the item exists its also loaded.
			// This might change when we start working with refs inside items ...
			return this.isItem(something);
		},
	
		loadItem: function(/* object */ args){
			if(this.isItemLoaded(args.item)){
				return;
			}
			// Actually we have nothing to do here, or at least I dont know what to do here ...
		},
	
		fetch:function(/* Object? */ request){
			//	summary:
			//		See dojo.data.util.simpleFetch.fetch() this is just a copy and I adjusted
			//		only the paging, since it happens on the server if doClientPaging is
			//		false, thx to http://trac.dojotoolkit.org/ticket/4761 reporting this.
			//		Would be nice to be able to use simpleFetch() to reduce copied code,
			//		but i dont know how yet. Ideas please!
			request = request || {};
			if(!request.store){
				request.store = this;
			}
			var self = this;
		
			var _errorHandler = function(errorData, requestObject){
				if(requestObject.onError){
					var scope = requestObject.scope || dojo.global;
					requestObject.onError.call(scope, errorData, requestObject);
				}
			};
		
			var _fetchHandler = function(items, requestObject, numRows){
				var oldAbortFunction = requestObject.abort || null;
				var aborted = false;
				
				var startIndex = requestObject.start?requestObject.start:0;
				if(self.doClientPaging == false){
					// For client paging we dont need no slicing of the result.
					startIndex = 0;
				}
				var endIndex = requestObject.count?(startIndex + requestObject.count):items.length;
		
				requestObject.abort = function(){
					aborted = true;
					if(oldAbortFunction){
						oldAbortFunction.call(requestObject);
					}
				};
		
				var scope = requestObject.scope || dojo.global;
				if(!requestObject.store){
					requestObject.store = self;
				}
				if(requestObject.onBegin){
					requestObject.onBegin.call(scope, numRows, requestObject);
				}
				if(requestObject.sort && self.doClientSorting){
					items.sort(dojo.data.util.sorter.createSortFunction(requestObject.sort, self));
				}
				if(requestObject.onItem){
					for(var i = startIndex; (i < items.length) && (i < endIndex); ++i){
						var item = items[i];
						if(!aborted){
							requestObject.onItem.call(scope, item, requestObject);
						}
					}
				}
				if(requestObject.onComplete && !aborted){
					var subset = null;
					if(!requestObject.onItem){
						subset = items.slice(startIndex, endIndex);
					}
					requestObject.onComplete.call(scope, subset, requestObject);
				}
			};
			this._fetchItems(request, _fetchHandler, _errorHandler);
			return request;	// Object
		},
	
		getFeatures: function(){
			return this._features;
		},
	
		close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
			// I have no idea if this is really needed ... 
		},
	
		getLabel: function(/* item */ item){
			//	summary:
			//		See dojo.data.api.Read.getLabel()
			if(this._labelAttr && this.isItem(item)){
				return this.getValue(item, this._labelAttr); //String
			}
			return undefined; //undefined
		},
	
		getLabelAttributes: function(/* item */ item){
			//	summary:
			//		See dojo.data.api.Read.getLabelAttributes()
			if(this._labelAttr){
				return [this._labelAttr]; //array
			}
			return null; //null
		},
		
		_xhrFetchHandler: function(data, request, fetchHandler, errorHandler){
			data = this._filterResponse(data);
			if(data.label){
				this._labelAttr = data.label;
			}
			var numRows = data.numRows || -1;

			this._items = [];
			// Store a ref to "this" in each item, so we can simply check if an item
			// really origins form here (idea is from ItemFileReadStore, I just don't know
			// how efficient the real storage use, garbage collection effort, etc. is).
			dojo.forEach(data.items,function(e){ 
				this._items.push({i:e, r:this}); 
			},this); 
			
			var identifier = data.identifier;
			this._itemsByIdentity = {};
			if(identifier){
				this._identifier = identifier;
				var i;
				for(i = 0; i < this._items.length; ++i){
					var item = this._items[i].i;
					var identity = item[identifier];
					if(!this._itemsByIdentity[identity]){
						this._itemsByIdentity[identity] = item;
					}else{
						throw new Error(this._className+":  The json data as specified by: [" + this.url + "] is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}
				}
			}else{
				this._identifier = Number;
				for(i = 0; i < this._items.length; ++i){
					this._items[i].n = i;
				}
			}
			
			// TODO actually we should do the same as dojo.data.ItemFileReadStore._getItemsFromLoadedData() to sanitize
			// (does it really sanititze them) and store the data optimal. should we? for security reasons???
			numRows = this._numRows = (numRows === -1) ? this._items.length : numRows;
			fetchHandler(this._items, request, numRows);
			this._numRows = numRows;		
		},
		
		_fetchItems: function(request, fetchHandler, errorHandler){
			//	summary:
			// 		The request contains the data as defined in the Read-API.
			// 		Additionally there is following keyword "serverQuery".
			//
			//	The *serverQuery* parameter, optional.
			//		This parameter contains the data that will be sent to the server.
			//		If this parameter is not given the parameter "query"'s
			//		data are sent to the server. This is done for some reasons:
			//		- to specify explicitly which data are sent to the server, they
			//		  might also be a mix of what is contained in "query", "queryOptions"
			//		  and the paging parameters "start" and "count" or may be even
			//		  completely different things.
			//		- don't modify the request.query data, so the interface using this
			//		  store can rely on unmodified data, as the combobox dijit currently
			//		  does it, it compares if the query has changed
			//		- request.query is required by the Read-API
			//
			// 		I.e. the following examples might be sent via GET:
			//		  fetch({query:{name:"abc"}, queryOptions:{ignoreCase:true}})
			//		  the URL will become:   /url.php?name=abc
			//
			//		  fetch({serverQuery:{q:"abc", c:true}, query:{name:"abc"}, queryOptions:{ignoreCase:true}})
			//		  the URL will become:   /url.php?q=abc&c=true
			//		  // The serverQuery-parameter has overruled the query-parameter
			//		  // but the query parameter stays untouched, but is not sent to the server!
			//		  // The serverQuery contains more data than the query, so they might differ!
			//
	
			var serverQuery = request.serverQuery || request.query || {};
			//Need to add start and count
			if(!this.doClientPaging){
				serverQuery.start = request.start || 0;
				// Count might not be sent if not given.
				if(request.count){
					serverQuery.count = request.count;
				}
			}
			if(!this.doClientSorting && request.sort){
				var sortInfo = [];
				dojo.forEach(request.sort, function(sort){
					if(sort && sort.attribute){
						sortInfo.push((sort.descending ? "-" : "") + sort.attribute);
					}
				});
				serverQuery.sort = sortInfo.join(',');
			}
			// Compare the last query and the current query by simply json-encoding them,
			// so we dont have to do any deep object compare ... is there some dojo.areObjectsEqual()???
			if(this.doClientPaging && this._lastServerQuery !== null &&
				dojo.toJson(serverQuery) == dojo.toJson(this._lastServerQuery)
				){
				this._numRows = (this._numRows === -1) ? this._items.length : this._numRows;
				fetchHandler(this._items, request, this._numRows);
			}else{
				var xhrFunc = this.requestMethod.toLowerCase() == "post" ? dojo.xhrPost : dojo.xhrGet;
				var xhrHandler = xhrFunc({url:this.url, handleAs:"json-comment-optional", content:serverQuery});
				xhrHandler.addCallback(dojo.hitch(this, function(data){
					this._xhrFetchHandler(data, request, fetchHandler, errorHandler);
				}));
				xhrHandler.addErrback(function(error){
					errorHandler(error, request);
				});
				// Generate the hash using the time in milliseconds and a randon number.
				// Since Math.randon() returns something like: 0.23453463, we just remove the "0."
				// probably just for esthetic reasons :-).
				this.lastRequestHash = new Date().getTime()+"-"+String(Math.random()).substring(2);
				this._lastServerQuery = dojo.mixin({}, serverQuery);
			}
		},
		
		_filterResponse: function(data){
			//	summary:
			//		If the data from servers needs to be processed before it can be processed by this
			//		store, then this function should be re-implemented in subclass. This default 
			//		implementation just return the data unchanged.
			//	data:
			//		The data received from server
			return data;
		},
	
		_assertIsItem: function(/* item */ item){
			//	summary:
			//		It throws an error if item is not valid, so you can call it in every method that needs to
			//		throw an error when item is invalid.
			//	item: 
			//		The item to test for being contained by the store.
			if(!this.isItem(item)){
				throw new Error(this._className+": Invalid item argument.");
			}
		},
	
		_assertIsAttribute: function(/* attribute-name-string */ attribute){
			//	summary:
			//		This function tests whether the item passed in is indeed a valid 'attribute' like type for the store.
			//	attribute: 
			//		The attribute to test for being contained by the store.
			if(typeof attribute !== "string"){ 
				throw new Error(this._className+": Invalid attribute argument ('"+attribute+"').");
			}
		},
	
		fetchItemByIdentity: function(/* Object */ keywordArgs){
			//	summary: 
			//		See dojo.data.api.Identity.fetchItemByIdentity()
	
			// See if we have already loaded the item with that id
			// In case there hasn't been a fetch yet, _itemsByIdentity is null
			// and thus a fetch will be triggered below.
			if(this._itemsByIdentity){
				var item = this._itemsByIdentity[keywordArgs.identity];
				if(!(item === undefined)){
					if(keywordArgs.onItem){
						var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
						keywordArgs.onItem.call(scope, {i:item, r:this});
					}
					return;
				}
			}
	
			// Otherwise we need to go remote
			// Set up error handler
			var _errorHandler = function(errorData, requestObject){
				var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
				if(keywordArgs.onError){
					keywordArgs.onError.call(scope, errorData);
				}
			};
			
			// Set up fetch handler
			var _fetchHandler = function(items, requestObject){
				var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
				try{
					// There is supposed to be only one result
					var item = null;
					if(items && items.length == 1){
						item = items[0];
					}
					
					// If no item was found, item is still null and we'll
					// fire the onItem event with the null here
					if(keywordArgs.onItem){
						keywordArgs.onItem.call(scope, item);
					}
				}catch(error){
					if(keywordArgs.onError){
						keywordArgs.onError.call(scope, error);
					}
				}
			};
			
			// Construct query
			var request = {serverQuery:{id:keywordArgs.identity}};
			
			// Dispatch query
			this._fetchItems(request, _fetchHandler, _errorHandler);
		},
		
		getIdentity: function(/* item */ item){
			//	summary: 
			//		See dojo.data.api.Identity.getIdentity()
			var identifier = null;
			if(this._identifier === Number){
				identifier = item.n; // Number
			}else{
				identifier = item.i[this._identifier];
			}
			return identifier;
		},
		
		getIdentityAttributes: function(/* item */ item){
			//	summary:
			//		See dojo.data.api.Identity.getIdentityAttributes()
			return [this._identifier];
		}
	}
);

}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.TIPQueryReadStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.TIPQueryReadStore"] = true;
dojo.provide("ibm.tivoli.tip.dojo.TIPQueryReadStore");




dojo.declare("ibm.tivoli.tip.dojo.TIPQueryReadStore",
	null,
	{
		//	summary:
		//		This class provides a store that is mainly intended to be used
		//		for loading data dynamically from the server, used i.e. for
		//		retreiving chunks of data from huge data stores on the server (by server-side filtering!).
		//		Upon calling the fetch() method of this store the data are requested from
		//		the server if they are not yet loaded for paging (or cached).
		//
		//		For example used for a combobox which works on lots of data. It
		//		can be used to retreive the data partially upon entering the
		//		letters "ac" it returns only items like "action", "acting", etc.
		//
		// note:
		//		The field name "id" in a query is reserved for looking up data
		//		by id. This is necessary as before the first fetch, the store
		//		has no way of knowing which field the server will declare as
		//		identifier.
		//
		//	example:
		// |	// The parameter "query" contains the data that are sent to the server.
		// |	var store = new dojox.data.QueryReadStore({url:'/search.php'});
		// |	store.fetch({query:{name:'a'}, queryOptions:{ignoreCase:false}});
		//
		// |	// Since "serverQuery" is given, it overrules and those data are
		// |	// sent to the server.
		// |	var store = new dojox.data.QueryReadStore({url:'/search.php'});
		// |	store.fetch({serverQuery:{name:'a'}, queryOptions:{ignoreCase:false}});
		//
		// |	<div dojoType="dojox.data.QueryReadStore"
		// |		jsId="store2"
		// |		url="../tests/stores/QueryReadStore.php"
		// |		requestMethod="post"></div>
		// |	<div dojoType="dojox.grid.data.DojoData"
		// |		jsId="model2"
		// |		store="store2"
		// |		sortFields="[{attribute: 'name', descending: true}]"
		// |		rowsPerPage="30"></div>
		// |	<div dojoType="dojox.Grid" id="grid2"
		// |		model="model2"
		// |		structure="gridLayout"
		// |		style="height:300px; width:800px;"></div>
	
		//
		//	todo:
		//		- there is a bug in the paging, when i set start:2, count:5 after an initial fetch() and doClientPaging:true
		//		  it returns 6 elemetns, though count=5, try it in QueryReadStore.html
		//		- add optional caching
		//		- when the first query searched for "a" and the next for a subset of
		//		  the first, i.e. "ab" then we actually dont need a server request, if
		//		  we have client paging, we just need to filter the items we already have
		//		  that might also be tooo much logic
		
		url:"",
		requestMethod:"get",
		//useCache:false,
		
		// We use the name in the errors, once the name is fixed hardcode it, may be.
		_className:"dojox.data.QueryReadStore",
		
		// This will contain the items we have loaded from the server.
		// The contents of this array is optimized to satisfy all read-api requirements
		// and for using lesser storage, so the keys and their content need some explaination:
		// 		this._items[0].i - the item itself 
		//		this._items[0].r - a reference to the store, so we can identify the item
		//			securly. We set this reference right after receiving the item from the
		//			server.
		_items:[],
		
		// Store the last query that triggered xhr request to the server.
		// So we can compare if the request changed and if we shall reload 
		// (this also depends on other factors, such as is caching used, etc).
		_lastServerQuery:null,
		
		// Store how many rows we have so that we can pass it to a clientPaging handler
		_numRows:-1,
		
		// Store a hash of the last server request. Actually I introduced this
		// for testing, so I can check if no unnecessary requests were issued for
		// client-side-paging.
		lastRequestHash:null,
		
		// summary:
		//		By default every request for paging is sent to the server.
		doClientPaging:false,
	
		// summary:
		//		By default all the sorting is done serverside before the data is returned
		//		which is the proper place to be doing it for really large datasets.
		doClientSorting:false,
	
		// Items by identify for Identify API
		_itemsByIdentity:null,
		
		// Identifier used
		_identifier:null,
	
		_features: {'dojo.data.api.Read':true, 'dojo.data.api.Identity':true},
	
		_labelAttr: "label",

		// Begin JSA
	    global: null,
	    tableID: null,
    	loadCompleteCallback: null, 

		/* Default timeout for requests made to the server */
		requestTimeout: 30000,
		// End JSA
	
		constructor: function(/* Object */ params){
			dojo.mixin(this,params);
		},
    
		// Begin JSA
    	setLoadCompleteCallback: function(aFunction){
	        this.loadCompleteCallback = aFunction;
	    },
		// End JSA
		
		getValue: function(/* item */ item, /* attribute-name-string */ attribute, /* value? */ defaultValue){
			//	According to the Read API comments in getValue() and exception is
			//	thrown when an item is not an item or the attribute not a string!
			this._assertIsItem(item);
			if (!dojo.isString(attribute)) {
				throw new Error(this._className+".getValue(): Invalid attribute, string expected!");
			}
			if(!this.hasAttribute(item, attribute)){
				// read api says: return defaultValue "only if *item* does not have a value for *attribute*." 
				// Is this the case here? The attribute doesn't exist, but a defaultValue, sounds reasonable.
				if(defaultValue){
					return defaultValue;
				}
				console.log(this._className+".getValue(): Item does not have the attribute '"+attribute+"'.");
			}
			return item.i[attribute];
		},
		
		getValues: function(/* item */ item, /* attribute-name-string */ attribute){
			this._assertIsItem(item);
			var ret = [];
			if(this.hasAttribute(item, attribute)){
				ret.push(item.i[attribute]);
			}
			return ret;
		},
		
		getAttributes: function(/* item */ item){
			this._assertIsItem(item);
			var ret = [];
			for(var i in item.i){
				if (i) {
					ret.push(i);
				}
			}
			return ret;
		},
	
		hasAttribute: function(/* item */ item,	/* attribute-name-string */ attribute) {
			//	summary: 
			//		See dojo.data.api.Read.hasAttribute()
			return this.isItem(item) && typeof item.i[attribute]!="undefined";
		},
		
		containsValue: function(/* item */ item, /* attribute-name-string */ attribute, /* anything */ value){
			var values = this.getValues(item, attribute);
			var len = values.length;
			for(var i=0; i<len; i++){
				if(values[i]==value){
					return true;
				}
			}
			return false;
		},
		
		isItem: function(/* anything */ something){
			// Some basic tests, that are quick and easy to do here.
			// >>> var store = new dojox.data.QueryReadStore({});
			// >>> store.isItem("");
			// false
			//
			// >>> var store = new dojox.data.QueryReadStore({});
			// >>> store.isItem({});
			// false
			//
			// >>> var store = new dojox.data.QueryReadStore({});
			// >>> store.isItem(0);
			// false
			//
			// >>> var store = new dojox.data.QueryReadStore({});
			// >>> store.isItem({name:"me", label:"me too"});
			// false
			//
			if(something){
				return typeof something.r!="undefined" && something.r==this;
			}
			return false;
		},
		
		isItemLoaded: function(/* anything */ something) {
			// Currently we dont have any state that tells if an item is loaded or not
			// if the item exists its also loaded.
			// This might change when we start working with refs inside items ...
			return this.isItem(something);
		},
	
		loadItem: function(/* object */ args){
			if(this.isItemLoaded(args.item)){
				return;
			}
			// Actually we have nothing to do here, or at least I dont know what to do here ...
		},
	
		fetch:function(/* Object? */ request){
			//	summary:
			//		See dojo.data.util.simpleFetch.fetch() this is just a copy and I adjusted
			//		only the paging, since it happens on the server if doClientPaging is
			//		false, thx to http://trac.dojotoolkit.org/ticket/4761 reporting this.
			//		Would be nice to be able to use simpleFetch() to reduce copied code,
			//		but i dont know how yet. Ideas please!
			request = request || {};
			if(!request.store){
				request.store = this;
			}
			var self = this;
		
			var _errorHandler = function(errorData, requestObject){
				if(requestObject.onError){
					var scope = requestObject.scope || dojo.global;
					requestObject.onError.call(scope, errorData, requestObject);
				}
			};
		
			var _fetchHandler = function(items, requestObject, numRows){
				var oldAbortFunction = requestObject.abort || null;
				var aborted = false;
				
				var startIndex = requestObject.start?requestObject.start:0;
				if (self.doClientPaging===false) {
					// For client paging we dont need no slicing of the result.
					startIndex = 0;
				}
				var endIndex   = requestObject.count?(startIndex + requestObject.count):items.length;
		
				requestObject.abort = function(){
					aborted = true;
					if(oldAbortFunction){
						oldAbortFunction.call(requestObject);
					}
				};
		
				var scope = requestObject.scope || dojo.global;
				if(!requestObject.store){
					requestObject.store = self;
				}
				if(requestObject.onBegin){
					requestObject.onBegin.call(scope, numRows, requestObject);
				}
				if(requestObject.sort && self.doClientSorting){
					items.sort(dojo.data.util.sorter.createSortFunction(requestObject.sort, self));
				}
				if(requestObject.onItem){
					for(var i = startIndex; (i < items.length) && (i < endIndex); ++i){
						var item = items[i];
						if(!aborted){
							requestObject.onItem.call(scope, item, requestObject);
						}
					}
				}
				if(requestObject.onComplete && !aborted){
					var subset = null;
					if (!requestObject.onItem) {
						subset = items.slice(startIndex, endIndex);
					}
					requestObject.onComplete.call(scope, subset, requestObject); 
					dojo.publish ("/ibm.tip/TIPQueryReadStore#rowCountUpdated", [this.id, numRows]);  
				}
			};
			this._fetchItems(request, _fetchHandler, _errorHandler);
			return request;	// Object
		},
	
		getFeatures: function(){
			return this._features;
		},
	
		close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
			// I have no idea if this is really needed ... 
		},
	
		getLabel: function(/* item */ item){
			//	summary:
			//		See dojo.data.api.Read.getLabel()
			if(this._labelAttr && this.isItem(item)){
				return this.getValue(item, this._labelAttr); //String
			}
			return undefined; //undefined
		},
	
		getLabelAttributes: function(/* item */ item){
			//	summary:
			//		See dojo.data.api.Read.getLabelAttributes()
			if(this._labelAttr){
				return [this._labelAttr]; //array
			}
			return null; //null
		},
		
		_xhrFetchHandler: function(data, request, fetchHandler, errorHandler){
			data = this._filterResponse(data);
			if (data.label){
				this._labelAttr = data.label;
			}
			var numRows = data.numRows || -1;

			this._items = [];
			// Store a ref to "this" in each item, so we can simply check if an item
			// really origins form here (idea is from ItemFileReadStore, I just don't know
			// how efficient the real storage use, garbage collection effort, etc. is).
			dojo.forEach(data.items,function(e){ 
				this._items.push({i:e, r:this}); 
			},this); 
			
			var identifier = data.identifier;
			this._itemsByIdentity = {};
			if(identifier){
				this._identifier = identifier;
				var i;
				for(i = 0; i < this._items.length; ++i){
					var item = this._items[i].i;
					var identity = item[identifier];
					if(!this._itemsByIdentity[identity]){
						this._itemsByIdentity[identity] = item;
					}else{
						throw new Error(this._className+":  The json data as specified by: [" + this.url + "] is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}
				}
			}else{
				this._identifier = Number;
				for(i = 0; i < this._items.length; ++i){
					this._items[i].n = i;
				}
			}
			
			// TODO actually we should do the same as dojo.data.ItemFileReadStore._getItemsFromLoadedData() to sanitize
			// (does it really sanititze them) and store the data optimal. should we? for security reasons???
			numRows = this._numRows = (numRows === -1) ? this._items.length : numRows;
			fetchHandler(this._items, request, numRows);
			this._numRows = numRows;
			
			// Begin Change - JSA
			// If a callback is registered, then call it now.  Recall this is used to help
			// smoothly replace models on a refresh to avoid flashing.
			if (this.loadCompleteCallback) {
			    this.loadCompleteCallback();
			}
			// End Change - JSA
		},
		
		_fetchItems: function(request, fetchHandler, errorHandler){
			//	summary:
			// 		The request contains the data as defined in the Read-API.
			// 		Additionally there is following keyword "serverQuery".
			//
			//	The *serverQuery* parameter, optional.
			//		This parameter contains the data that will be sent to the server.
			//		If this parameter is not given the parameter "query"'s
			//		data are sent to the server. This is done for some reasons:
			//		- to specify explicitly which data are sent to the server, they
			//		  might also be a mix of what is contained in "query", "queryOptions"
			//		  and the paging parameters "start" and "count" or may be even
			//		  completely different things.
			//		- don't modify the request.query data, so the interface using this
			//		  store can rely on unmodified data, as the combobox dijit currently
			//		  does it, it compares if the query has changed
			//		- request.query is required by the Read-API
			//
			// 		I.e. the following examples might be sent via GET:
			//		  fetch({query:{name:"abc"}, queryOptions:{ignoreCase:true}})
			//		  the URL will become:   /url.php?name=abc
			//
			//		  fetch({serverQuery:{q:"abc", c:true}, query:{name:"abc"}, queryOptions:{ignoreCase:true}})
			//		  the URL will become:   /url.php?q=abc&c=true
			//		  // The serverQuery-parameter has overruled the query-parameter
			//		  // but the query parameter stays untouched, but is not sent to the server!
			//		  // The serverQuery contains more data than the query, so they might differ!
			//
	
			var serverQuery = request.serverQuery || request.query || {};
			//Need to add start and count
			if(!this.doClientPaging){
				serverQuery.start = request.start || 0;
				// Count might not be sent if not given.
				if (request.count) {
					serverQuery.count = request.count;
				}
			}
			if(!this.doClientSorting){
				if(request.sort){
					var sort = request.sort[0];
					if(sort && sort.attribute){
						var sortStr = sort.attribute;
						if(sort.descending){
							sortStr = "-" + sortStr;
						}
						serverQuery.sort = sortStr;
					}
				}
			}
			// Compare the last query and the current query by simply json-encoding them,
			// so we dont have to do any deep object compare ... is there some dojo.areObjectsEqual()???
			if(this.doClientPaging && this._lastServerQuery!==null &&
				dojo.toJson(serverQuery)==dojo.toJson(this._lastServerQuery)
				){
				this._numRows = (this._numRows === -1) ? this._items.length : this._numRows;
				fetchHandler(this._items, request, this._numRows);
			}else{
				// Begin JSA
				//var xhrFunc = this.requestMethod.toLowerCase()=="post" ? dojo.xhrPost : dojo.xhrGet;
				//var xhrHandler = xhrFunc({url:this.url, handleAs:"json-comment-optional", content:serverQuery});
				var kResponseHandler = dojo.hitch(this, function(data){
					this._xhrFetchHandler(data, request, fetchHandler, errorHandler);
				});
				var kErrorHandler = function(error){
					errorHandler(error, request);
				};
				// Generate the hash using the time in milliseconds and a randon number.
				// Since Math.randon() returns something like: 0.23453463, we just remove the "0."
				// probably just for esthetic reasons :-).
				this.lastRequestHash = new Date().getTime()+"-"+String(Math.random()).substring(2);
				this._lastServerQuery = dojo.mixin({}, serverQuery);

            	var kw = {
                    content:   {
                                  command: "getTableRows",
                                  arg_fqTableId: this.tableID,
                                  arg_portletNamespace: this.global.namespace
                               },
                    timeout:   this.requestTimeout,
                    onResponse:{
                                  callback: kResponseHandler
                               },
                    onError:{
                                  callback: kErrorHandler
                               }
            	};
            
            	// Add server query stuff
            	kw.content = dojo.mixin(kw.content,serverQuery);
            
            	if (this.requestMethod.toLowerCase()=="post") {
	                ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).post( kw );
    	        } else {
        	        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
	            }
            	// End JSA
				
			}
		},
		
		_filterResponse: function(data){
			//	summary:
			//		If the data from servers needs to be processed before it can be processed by this
			//		store, then this function should be re-implemented in subclass. This default 
			//		implementation just return the data unchanged.
			//	data:
			//		The data received from server
			return data;
		},
	
		_assertIsItem: function(/* item */ item){
			//	summary:
			//		It throws an error if item is not valid, so you can call it in every method that needs to
			//		throw an error when item is invalid.
			//	item: 
			//		The item to test for being contained by the store.
			if(!this.isItem(item)){
				throw new Error(this._className+": Invalid item argument.");
			}
		},
	
		_assertIsAttribute: function(/* attribute-name-string */ attribute){
			//	summary:
			//		This function tests whether the item passed in is indeed a valid 'attribute' like type for the store.
			//	attribute: 
			//		The attribute to test for being contained by the store.
			if(typeof attribute !== "string"){ 
				throw new Error(this._className+": Invalid attribute argument ('"+attribute+"').");
			}
		},
	
		fetchItemByIdentity: function(/* Object */ keywordArgs){
			//	summary: 
			//		See dojo.data.api.Identity.fetchItemByIdentity()
	
			// See if we have already loaded the item with that id
			// In case there hasn't been a fetch yet, _itemsByIdentity is null
			// and thus a fetch will be triggered below.
			if(this._itemsByIdentity){
				var item = this._itemsByIdentity[keywordArgs.identity];
				if(!(item === undefined)){
					if(keywordArgs.onItem){
						var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
						keywordArgs.onItem.call(scope, {i:item, r:this});
					}
					return;
				}
			}
	
			// Otherwise we need to go remote
			// Set up error handler
			var _errorHandler = function(errorData, requestObject){
				var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
				if(keywordArgs.onError){
					keywordArgs.onError.call(scope, errorData);
				}
			};
			
			// Set up fetch handler
			var _fetchHandler = function(items, requestObject){
				var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
				try{
					// There is supposed to be only one result
					var item = null;
					if(items && items.length == 1){
						item = items[0];
					}
					
					// If no item was found, item is still null and we'll
					// fire the onItem event with the null here
					if(keywordArgs.onItem){
						keywordArgs.onItem.call(scope, item);
					}
				}catch(error){
					if(keywordArgs.onError){
						keywordArgs.onError.call(scope, error);
					}
				}
			};
			
			// Construct query
			var request = {serverQuery:{id:keywordArgs.identity}};
			
			// Dispatch query
			this._fetchItems(request, _fetchHandler, _errorHandler);
		},
		
		getIdentity: function(/* item */ item){
			//	summary: 
			//		See dojo.data.api.Identity.getIdentity()
			var identifier = null;
			if(this._identifier === Number){
				identifier = item.n; // Number
			}else{
				identifier = item.i[this._identifier];
			}
			return identifier;
		},
		
		getIdentityAttributes: function(/* item */ item){
			//	summary:
			//		See dojo.data.api.Identity.getIdentityAttributes()
			return [this._identifier];
		},
		
		getTotalRows: function () {
			return (this._numRows);
		}
	}
);

}

if(!dojo._hasResource["dijit.ToolbarSeparator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.ToolbarSeparator"] = true;
dojo.provide("dijit.ToolbarSeparator");




dojo.declare("dijit.ToolbarSeparator",
		[ dijit._Widget, dijit._Templated ],
		{
		// summary:
		//		A spacer between two `dijit.Toolbar` items
		templateString: '<div class="dijitToolbarSeparator dijitInline" waiRole="presentation"></div>',
		postCreate: function(){ dojo.setSelectable(this.domNode, false); },
		isFocusable: function(){
			// summary:
			//		This widget isn't focusable, so pass along that fact.
			// tags:
			//		protected
			return false;
		}

	});



}

if(!dojo._hasResource["dijit.Toolbar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Toolbar"] = true;
dojo.provide("dijit.Toolbar");





dojo.declare("dijit.Toolbar",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
	{
	// summary:
	//		A Toolbar widget, used to hold things like `dijit.Editor` buttons

	templateString:
		'<div class="dijit" waiRole="toolbar" tabIndex="${tabIndex}" dojoAttachPoint="containerNode">' +
		//	'<table style="table-layout: fixed" class="dijitReset dijitToolbarTable">' + // factor out style
		//		'<tr class="dijitReset" dojoAttachPoint="containerNode"></tr>'+
		//	'</table>' +
		'</div>',

	baseClass: "dijitToolbar",

	postCreate: function(){
		this.connectKeyNavHandlers(
			this.isLeftToRight() ? [dojo.keys.LEFT_ARROW] : [dojo.keys.RIGHT_ARROW],
			this.isLeftToRight() ? [dojo.keys.RIGHT_ARROW] : [dojo.keys.LEFT_ARROW]
		);
		this.inherited(arguments);
	},

	startup: function(){
		if(this._started){ return; }

		this.startupKeyNavChildren();

		this.inherited(arguments);
	}
}
);

// For back-compat, remove for 2.0


}

if(!dojo._hasResource["dijit.form.DropDownButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.DropDownButton"] = true;
dojo.provide("dijit.form.DropDownButton");



}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableDropDownButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableDropDownButton"] = true;
dojo.provide("ibm.tivoli.tip.dijit.TIPTableDropDownButton");




dojo.declare("ibm.tivoli.tip.dijit.TIPTableDropDownButton", [dijit.form.DropDownButton], {
	// summary:
	//		A button with a drop down
	//
	// example:
	// |	<button dojoType="dijit.form.DropDownButton" label="Hello world">
	// |		<div dojotype="dijit.Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new dijit.form.DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
	// |	dojo.body().appendChild(button1);
	// 	
	
	baseClass : "dijitDropDownButton",

	templateString:"<span class=\"dijit dijitReset dijitLeft dijitInline\" dojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse,onkeydown:_onDropDownKeydown,onblur:_onDropDownBlur,onkeypress:_onKey\">\r\n\t<span class=\"dijitReset dijitRight dijitInline\">\r\n\t\t<span class=\"dijitReset dijitInline dijitButtonNode quickFilterDownArrow\">\r\n\t\t\t<button class=\"dijitReset dijitStretch dijitButtonContents\" style=\"opacity:0;\" ${nameAttrSetting} type=\"${type}\" value=\"${value}\" dojoAttachPoint=\"focusNode,titleNode\" waiRole=\"button\" waiState=\"haspopup-true,labelledby-${id}_label\">\r\n\t\t\t\t<span class=\"dijitReset dijitInline\" dojoAttachPoint=\"iconNode\"></span>\r\n\t\t\t\t<span class=\"dijitReset dijitInline dijitButtonText\"  dojoAttachPoint=\"containerNode,popupStateNode\" id=\"${id}_label\"></span>\r\n\t\t\t\t<span class=\"dijitRest dijitInline\">&nbsp;&nbsp;&nbsp;</span>\r\n\t\t\t</button>\r\n\t\t</span>\r\n\t</span>\r\n</span>\r\n",
	
	//Added fix for IBM Dojo 1.5 Migration
	_onMouse: function(){	
	},	
	_onDropDownBlur: function(){	
	},	
	_onKey: function(){	
	},	
	_onDropDownKeydown: function(){	
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableSearchField"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableSearchField"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/









// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTableSearchField");

dojo.declare("ibm.tivoli.tip.dijit.TIPTableSearchField",
            [ dijit._Widget, dijit._Templated ],
{
  widgetsInTemplate: true,
  
  /* The TIPTable that this search bar is attached to */
  table: null,
  
  _ALL_COLS_NAME: "QF_ALL_COLS",
  
  /** An array of column names that should participate in filtering **/
  columnsToFilterOn: { "QF_ALL_COLS": true },
  
  /** Keeps track of all topic subscriptions to cleanly destroy later **/
  topics: [],
  
  _qf_arrow_src: dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/qf_arrow.gif"),
  
  /** Keeps track of all event connections **/
  objectConnections: [],
  
  /** Keeps track of all children widgets in dropdown **/
  childrenWidgets: [],
  
  templateString:"<div class=\"quick-filter\" align=\"right\">\r\n<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\r\n<tbody>\r\n<tr>\r\n<td dojoAttachPoint=\"colFilter\" class=\"qf-lcap\" valign=\"top\" dojoAttachEvent='onmouseover:_onHover,onmouseout:_onUnHover'>\r\n</td>\r\n<td class=\"qf-mid\" valign=\"middle\">\r\n<label title=\"${resources_.SEARCH_TEXT}\">\r\n<input dojoAttachPoint=\"searchField\" class=\"qf-textfield-label\" type=\"text\" valign=\"middle\" dojoAttachEvent='onfocus:_onFocus,onblur:_onBlur,onkeypress:_onKeyPress' />\r\n</label>\r\n</td>\r\n<td dojoAttachPoint=\"colFilterTail\" class=\"qf-rcap\" valign=\"top\"/>\r\n\t<span>&nbsp;&nbsp;&nbsp;&nbsp;</span>\r\n</td>\r\n</tr>\r\n</tbody>\r\n</table>\r\n</div>\r\n",

  constructor: function( parms ) 
  {
  	this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
  	this.lastColumns = null;
  },
  
  postCreate: function()
  {
	if(dojo.getComputedStyle(dojo.body()).direction == "rtl"){
		this.colFilter.className="qf-lcap-bidi";
		this.colFilterTail.className="qf-rcap-bidi";
	}else{
		this.colFilter.className="qf-lcap";
		this.colFilterTail.className="qf-rcap";
	}
	if ( this.table !== null )
    {
      this.setTable( this.table );
    }        
  },  
   
  /********************************************************************
  * This method must be called to set the underlying table that the 
  * search bar will filter on.  As soon as this method is called
  * the table is queried for its "structure" (columns) and the dropdown
  * column filter dialog is populated
  *********************************************************************/
  setTable: function( table , widgetDef, global )
  {
    console.log ("enter setTable");
  	// Save a reference to the table
  	this.table = table;
  	
  	this.clear(); //ensure that we start with a fresh slate
  	  	
  	var div  = document.createElement( 'DIV' );
  	var div2 = document.createElement( 'DIV' );
  	div2.className="qf-col-filter-title";	  
  
  	div2.appendChild( document.createTextNode( this.resources_.FILTER_ON ) );
  	div.appendChild( div2 );

  	div2 = document.createElement( 'DIV' );
	div2.className="qf-col-chkbox-label-row";
  	
  	var checkbox = new dijit.form.CheckBox( { id     : table.id + this._ALL_COLS_NAME,
  											  pId    : table.id,
  	                                          name   : table.id + this._ALL_COLS_NAME,
  	                                          checked: true } );

	this.childrenWidgets[ checkbox.id ] = checkbox;
	
	checkbox.onColsFilter = function()
	{
		this.attr ("checked", false);
	};
	
	checkbox.onCheckedEvent = function ()
	{
		console.log ("enter checkbox.onCheckedEvent");
  		if ( this.checked )
  		{
	  		dojo.publish( this.pId + "_filterOnAll", ["Filter on all rows"] );
  		}
	};  	           
	
  	this.topics[ this._ALL_COLS_NAME ] = dojo.subscribe( table.id + "_filterByCol", checkbox, "onColsFilter" );                                                                         
	this.objectConnections[ this._ALL_COLS_NAME ] = dojo.connect( checkbox, "_setCheckedAttr", checkbox, "onCheckedEvent" );  	                                          

	var label = document.createElement( 'LABEL' );
	label.className = "qf-col-chkbox-label";
	label["for"] = checkbox.id;
	label.title = this.resources_.ALL_COLUMNS;
  	if (global) {
	  	label.title = global.bidiEnableLabel (label.title);
	  	console.log ("added bidi label for: " + label.title); 
	} else {
		console.log ("global is null... no bidi!");
	}
	
	
	// label.appendChild( document.createTextNode( label.title ) );
	label.innerHTML = label.title;
	
	div2.appendChild( checkbox.domNode );  	                                         
	div2.appendChild( label );  	                                         

	div.appendChild( div2 );

	var columns = [];
	var i = 0;
	var j = 0;
	
	table.views.forEach( function( view, k )
	{
		if ( view.structure.cells )
		{
			for ( var i in view.structure.cells )
			{
				if (i) {
					for ( var j in view.structure.cells[i] )
					{
						if (j) {
						 	var v = view.structure.cells[i][j];				
		    	         	if (v.field !== undefined) {
                	         	if(!widgetDef){
                	         		columns.push( v );
                	         	}else {									
									//if(widgetDef.datasets[0].columns[v.field].type == "java.lang.String"){
									if(widgetDef.datasets[0].columns[j].type == "java.lang.String" || widgetDef.datasets[0].columns[j].type == "string") {
                	         			columns.push( v );
                	         		}
                	         }
	                 	}
					}			
				}
			}
		}
		}
	});  	
  	  	
  	for ( i in columns )
  	{
  		if (i) {
		  	if ( columns[i].isSearchable === false ) {
		    	continue;
			}
  			div2 = document.createElement( 'DIV' );
  			div2.className="qf-col-chkbox-label-row";

  		  	if(columns[i].name !== undefined){
  				checkbox = new dijit.form.CheckBox( { id     : table.id + columns[i].name,
  											  pId    : table.id,
  											  columnID: columns[i].field,
  	                                          name   : table.id + columns[i].name,
  	                                          checked: this.columnsToFilterOn[ columns[i].name ] } );
			} else {
  				checkbox = new dijit.form.CheckBox( { id     : table.id + columns[i].field,
  											  pId    : table.id,
  											  columnID: columns[i].field,
  	                                          name   : table.id + columns[i].field,
  	                                          checked: this.columnsToFilterOn[ columns[i].field ] } );
			}

			this.childrenWidgets[ checkbox.id ] = checkbox;
		
			checkbox.onAllColsFilter = function() {
				this.attr ("checked", false);
			};
		
			checkbox.onCheckedEvent = function () {
	  			if ( this.checked )
	  			{
			  		dojo.publish( this.pId + "_filterByCol", ["Filter on individual columns"] );		
	  			}
			};  	           
		
			this.topics[ columns[i].name ] = dojo.subscribe( table.id + "_filterOnAll", checkbox, "onAllColsFilter" );                               
  	    
			this.objectConnections[ columns[i].name ] = dojo.connect( checkbox, "_setCheckedAttr", checkbox, "onCheckedEvent" );  	                                          
  	                                          
			div2.appendChild( checkbox.domNode );  	                            

			label = document.createElement( 'LABEL' );
			label["for"] = checkbox.id;
			if (columns[i].name !== undefined) {
				label.title = columns[i].name;
			} else {
				label.title = columns[i].field;
			}
  			if (global) {
	  			label.title = global.bidiEnableLabel (label.title);
			}
			label.className = "qf-col-chkbox-label";
		
			// label.appendChild( document.createTextNode( label.title ) );
			label.innerHTML = label.title;
			
		
			div2.appendChild( label );  	                                         		             
  		
  			div.appendChild( div2 );  		
  		}
  	}
  	  	
  	var button = new dijit.form.Button( { id: table.id + "close_btn",
                                        type: "submit", 
                                        onClick: dojo.hitch (this, "onFilterOK"),
  								       label: this.resources_.OK } );

	this.childrenWidgets[ button.id ] = button;	  								       
  								     
	div.appendChild( button.domNode );  								     
  	
  	var tooltipDialog = new dijit.TooltipDialog( { id: table.id + "gf_tooltipdialog" } );

	this.childrenWidgets[ tooltipDialog.id ] = tooltipDialog;	  								       
  	  	
  	tooltipDialog.containerNode.appendChild( div );
  	
  	/** OVERLOAD to allow our own styling to be added **/
//  	tooltipDialog.orient = function(/*Object*/ corner)
//  	{
		// summary: configure widget to be displayed in given position relative to the button
//		this.domNode.className="dijitTooltipDialog " +" dijitTooltipAB"+(corner.charAt(1)=='L'?"Left":"Right")+" dijitTooltip"+(corner.charAt(0)=='T' ? "Below" : "Above" ) + " qf-TooltipDialog";
//	};  	
  	
  	tooltipDialog.execute = function( args )
  	{  		
  	    this.columnsToFilterOn = [];
  	    
  		for ( var i in args )
  		{
  			if (i) {
				this.columnsToFilterOn[i] = args[i][0] ? true : false ;
			}
  		}  	
  	};
  	
	var btn = new ibm.tivoli.tip.dijit.TIPTableDropDownButton( { id: table.id + "_coltrigger", 
												dropDown: tooltipDialog,
												label: "",
												baseClass: "qf-col-trigger" } );
												
	this.tooltip_dropdownhelp = new dijit.Tooltip({label:this.resources_.SEARCH_DROP_DOWN_HELP, connectId:[btn.id]}, document.createElement("div"));
												
	
	this.childrenWidgets[ btn.id ] = btn;	  								       

	this.colFilter.appendChild( btn.domNode );
		
	btn.startup();
	
//	var downArrowSpan =  dojo.query("span.dijitArrowButtonChar", btn.domNode );	
	
//	dojo.query("span.dijitButtonText", btn.domNode ).orphan();
	
//	downArrowSpan[0].innerHTML = "";
//	dojo.removeClass( downArrowSpan[0], "dijitArrowButtonChar" );
//	dojo.removeClass( downArrowSpan[0], "dijitArrowButtonInner" );
//	downArrowSpan[0].style.padding="0px";
	
//	var image = dojo.doc.createElement( 'IMG' );	
//	image.src = this._qf_arrow_src;
//	image.border = 0;
//	image.style.padding="0px";
//	image.style.margin="0px";
//	image.alt = this.resources_.SEARCH_DROP_DOWN_HELP;
	
//	downArrowSpan[0].appendChild( image );	
  },
  
  onFilterOK: function () {
  	console.log ("enter onFilterOK");
	var columns = this.getSelectedColumns();
	if (this.columnsEqual (columns, this.lastColumns)) {
		console.log ("columns match.  not calling search");
	} else {
		console.log ("calling search");
  		dojo.publish (this.id + "searchInvoked", [this, null]);
  		this.lastColumns = columns;
	}
  },
  
  columnsEqual: function (columns1, columns2) {
  	if ((columns1 === null) && (columns2 === null)) {
  		return (true);
  	} else if ((columns1 === null) || (columns2 === null)) {
  		return (false);
  	} else if (columns1.length !== columns2.length) {
  		return (false);
  	} else {
		for (var x in columns1) {
			if (columns1[x] !== columns2[x]) {
				return (false);
			}
  		}
  		
  		return (true);
  	}
  },
  
  _onFocus: function()
  {
	this.domNode.className="quick-filter-focus";  
  },
  
  _onBlur: function()
  {
	this.domNode.className="quick-filter";
  },
  
  _onKeyPress: function(event)
  {
  	if (event.keyCode == 13) {
	  	event.cancelBubble = true;
  		event.returnValue = false;
  		dojo.publish ("onEnter", [this, event]);
  		dojo.publish (this.id + "searchInvoked", [this, event]);
	}
  },
  
  _onHover: function()
  {
	  if(dojo.getComputedStyle(dojo.body()).direction == "rtl"){
		  this.colFilter.className="qf-lcap-over-bidi";
	  }else{
		  this.colFilter.className="qf-lcap-over";
	  }
  },
  
  _onUnHover: function()
  {
	  if(dojo.getComputedStyle(dojo.body()).direction == "rtl"){
		this.colFilter.className="qf-lcap-bidi";
	  }else{
		this.colFilter.className="qf-lcap";
	  }
  },
  
  _setFilterCol: function ()
  {  	
  },
  

  getSelectedColumns: function ()
  {
  	var result = [];
  	
  	if (this.childrenWidgets[this.table.id + this._ALL_COLS_NAME].checked) {
	  	for (var x in this.childrenWidgets) {
	  		if (x) {
	  			if (this.childrenWidgets[x].columnID !== undefined ) {
		  			result.push (this.childrenWidgets[x].columnID);
		  		}
  			}
  		}
  	} else {
	  	for (var y in this.childrenWidgets) {
	  		if (y) {
	  			if (this.childrenWidgets[y].checked) {
		  			if (this.childrenWidgets[y].columnID !== undefined ) {
		  				result.push (this.childrenWidgets[y].columnID);
		  			}
	  			}
	  		}
	  	}
  	}
  	
	return (result);
  },
  
  /*************************************************************
  * Clears the contents of the search drop down to be recreated
  *************************************************************/
  clear: function()
  {
  	/** Destroy all subscriptions **/
  	for ( var i in this.topics )
  	{
  		if (i) {
  			dojo.unsubscribe( this.topics[i] );
  		}
  	}
  	
  	delete this.topics;
  	this.topics = [];

  	/** Destroy all event connections **/
  	for ( i in this.objectConnections )
  	{
  		if (i) {
	  		dojo.disconnect( this.objectConnections[i] );
	  	}
  	}  	

  	delete this.objectConnections;
  	this.objectConnections = [];
  	
  	/** Destroy all children widgets **/
  	for ( i in this.childrenWidgets )
  	{
  		if (i) {
  			this.childrenWidgets[i].destroy();
  		}
  	}  
  	
  	if ( this.tooltip_dropdownhelp )
  	{
  		this.tooltip_dropdownhelp.destroy();
  	}

  	delete this.childrenWidgets;
  	this.childrenWidgets = [];
  	
  	delete this.columnsToFilterOn;
  	this.columnsToFilterOn = [];
  },
  
  /*************************************************************
  * Destroy this widget -- remove all subscriptions and event
  * connections
  *************************************************************/
  destroy: function()
  {  
	this.inherited( "destroy", arguments );
  	this.clear();
  	/** call super **/
  	
  }
  
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableToolbar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableToolbar"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/






dojo.provide("ibm.tivoli.tip.dijit.TIPTableToolbar");

/******************************************************************************
* A dijit.Toolbar and a TIPTableSearchField combined into one.
*
* Authors: Joseph Firebaugh
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableToolbar",
            [ dijit._Widget, dijit._Templated ],
{
  widgetsInTemplate: true,
   shownub : true,
   nubhover :true,
   nub :null,
   stateMinimized:false,
  templateString:"<div>\r\n\t<div >\r\n\t\t<table   border=\"0\" cellpadding=\"0\" cellspacing=\"0\" class=\"tableToolbar\"   dojoAttachPoint=\"toolbarBody\">\r\n\t\t<tbody>\r\n\t\t<tr  id=\"${id}toolbarTR\" dojoAttachPoint=\"toolbarDom\" >\r\n\t\t\t<td   class=\"TIPTableToolbarArea\" width=\"100%\" style=\"height:auto\">\r\n\t\t\t<div style=\"position:relative\" dojoAttachPoint=\"nubArea\" >\r\n\t\t\t<div id=\"${id}toolbar\" dojoType=\"dijit.Toolbar\" dojoAttachPoint=\"toolbar\"  class=\"tableToolbarArea\" ></div>\r\n\t\t\t</div>\r\n\t\t\t</td>\r\n\t\t\t<td id =\"${id}searchbar\" class=\"TIPTableSearchFieldArea\" height=\"auto\">\r\n\t\t\t<div dojoType=\"ibm.tivoli.tip.dijit.TIPTableSearchField\" dojoAttachPoint=\"searchField\"></div>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t\r\n\t\t</tbody>\r\n\t\t\t\r\n\t\t</table>\r\n\t\t\r\n\t\t\r\n\t    </div>\r\n\t\t\r\n</div>\r\n",
  
  getToolbar: function()
  {
    return this.toolbar;
  },
   postCreate : function(){
    if(this.shownub == true) {
	console.warn("postCreate nub");
	var div = document.createElement("div");
		
	this.nub=div;
	div.setAttribute("tabIndex","0");
	dojo.addClass(div,"nubOpen");
	this.resetNubPos();
	dojo.style(div,"position","absolute");
	dojo.style(div,"bottom","-1px");
	dojo.style(div,"height","9px");
	dojo.style(div,"align","center");
	// fix for accessibility, setting aria attributes
	div.setAttribute("role","button");
	div.setAttribute("title","toggle toolbar");
	div.setAttribute("wairole","button");
	div.setAttribute("aria-labelledby","toggle toolbar");
	 if(this.nubhover ==true){
		dojo.style(div,"display","none");
		dojo.connect(this.toolbarDom,'onmouseover',this,this._mouseover);
		dojo.connect(this.toolbarDom,'mouseout',this,this._mouseout);
	  }	
	this.nubArea.appendChild(div);
	}
  },
  resetNubPos : function(){
	if (this.shownub == true) {   
		var coords = dojo.coords(this.toolbarBody);
		//var centerx =  coords.x + (coords.w/2);
		var centerx = coords.w/2;
		dojo.style(this.nub,"left",centerx+"px");
	}
  },
  _mouseover:function() {
   
   if(this.stateMinimized==false){
   dojo.style(this.nub,"display","block");
   }
  },
  _mouseout :function(){
  
   if(this.stateMinimized==false){
   dojo.style(this.nub,"display","none");
  }
  },
  destroy: function()
  {
	this.inherited( "destroy", arguments );
	if(this.searchField){
		this.searchField.destroy();
	}
	if(this.toolbar){
		this.toolbar.destroy();
	}  	
  }    
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableFooter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableFooter"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/



// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTableFooter");
/******************************************************************************
* The footer for the TIP Table
*
* Authors: Joseph Firebaugh, Joshua Allen
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableFooter",
            [ dijit._Widget, dijit._Templated ],
{
  widgetsInTemplate: true,

  templateString:"<table class=\"tip-table-footer-border\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\r\n\t<tbody>\r\n\t\t<tr class=\"tip-table-footer\">\r\n\t\t\t<td class=\"tip-table-footer-data\" nowrap=\"nowrap\" valign=\"middle\"> \r\n\t\t\t\t<div dojoAttachPoint=\"tableUpdateArea\" class=\"footerLabel\"></div>\r\n\t\t\t</td>\t\t\r\n\t\t\t<td class=\"tip-table-footer-data\" width=\"100%\" nowrap=\"nowrap\"> \r\n\t\t\t\t<div dojoType=\"dijit.Toolbar\" dojoAttachPoint=\"toolbar\" class=\"tableToolbarArea\" ></div>\r\n\t\t\t</td>\t\t\t\r\n\t\t\t<td class=\"tip-table-footer-data\" nowrap=\"nowrap\" valign=\"middle\"> \r\n\t\t\t\t<div dojoAttachPoint=\"infoArea\" class=\"footerLabel\"></div>\r\n\t\t\t</td>\r\n\t\t</tr>  \r\n\t</tbody>\r\n</table>\r\n",
  
  //the dojox.Grid that this footer belongs to
  table: null,
  
  constructor: function () 
  {
    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");    
  },
  
  postCreate: function()
  {
    if ( this.table !== null )
    {
      this.setTable( this.table );
    }        
  },
  
  setTable: function( table )
  {
    this.table = table;
    
    this._rowClickListener = dojo.connect( this.table, "onRowClick", this, "_onRowClick");            
    this._updateListener   = dojo.connect( this.table, "setModel",   this, "_onRowClick" );            
    
  },
  
  getToolbar: function () 
  {
  	return (this.toolbar);
  },
  
  _onRowClick: function()
  {      
    var numSelected = 0;
    var totalRows   = 0;
    
    if ( this.table.model )
    {
		if (this.table.model.getRealRowCount) {
			totalRows = this.table.model.getRealRowCount();
		} else {
			totalRows = this.table.model.getRowCount();
		}
    }

    numSelected = this.table.selection.getSelectedCount();
    
    this.updateFooter( numSelected, totalRows );
  },
    
  updateFooter: function( numSelected, totalRows )
  {
    this.setInfoAreaHTML( dojo.string.substitute( this.resources_.TABLE_FOOTER_TEXT, { 0: numSelected, 1: totalRows } ) );  
  },
  
  setInfoAreaHTML: function ( html )
  {
	//jfirebau
	//To fix some race conditions with creating and destroying TIPTables I need
	//to check for the existence of the infoArea before setting content
    if( this.infoArea )
    {
    	this.infoArea.innerHTML = html;    	
    }	
  },
  
  setTableUpdateAreaHTML: function ( html )
  {
	this.tableUpdateArea.innerHTML = html;
  },  
  
  destroy: function()
  {
  	//fix defect 248535
	this.inherited( "destroy", arguments );
    if ( this._rowClickListener ) { dojo.disconnect( this._rowClickListener ); }
    if ( this._updateListener )   { dojo.disconnect( this._updateListener ); }
    //Fix for defect 150409 on bug tracker on CS. Toolbar destroy is called twice throws error on IE
    //if(this.toolbar){
    //	this.toolbar.destroy();
    //}
  }  
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableRefreshButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableRefreshButton"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/



// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTableRefreshButton");
/******************************************************************************
* Provides Table Refresh button that allows for "busy" animation. 
*
* Authors: Joseph Firebaugh
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableRefreshButton", dijit.form.Button,
{
    baseClass: "tipTableRefreshButton",

	templateString:"<div class=\"dijit dijitLeft dijitInline ${baseClass}\"\n\tdojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\n\t><div class='dijitRight'\n\t\t><button class=\"dijitStretch dijitButtonNode dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\n\t\t\t><span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\" \n \t\t\t\t><span class=\"dijitToggleButtonIconChar\">&#10003</span \n\t\t\t></span\n\t\t\t><span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span\n\t\t></button\n\t></div\n></div>\n",
    
    postCreate: function()
    {
		var resources = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
		this.titleNode.title = resources.REFRESH;
    },
	//Added fix for IBM Dojo 1.5 Migration
	_onMouse: function(){
	
	}
    
});

}

if(!dojo._hasResource["dijit.form.Form"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Form"] = true;
dojo.provide("dijit.form.Form");





dojo.declare(
	"dijit.form.Form",
	[dijit._Widget, dijit._Templated, dijit.form._FormMixin],
	{
		// summary:
		//		Widget corresponding to HTML form tag, for validation and serialization
		//
		// example:
		//	|	<form dojoType="dijit.form.Form" id="myForm">
		//	|		Name: <input type="text" name="name" />
		//	|	</form>
		//	|	myObj = {name: "John Doe"};
		//	|	dijit.byId('myForm').set('value', myObj);
		//	|
		//	|	myObj=dijit.byId('myForm').get('value');

		// HTML <FORM> attributes

		// name: String?
		//		Name of form for scripting.
		name: "",

		// action: String?
		//		Server-side form handler.
		action: "",

		// method: String?
		//		HTTP method used to submit the form, either "GET" or "POST".
		method: "",

		// encType: String?
		//		Encoding type for the form, ex: application/x-www-form-urlencoded.
		encType: "",

		// accept-charset: String?
		//		List of supported charsets.
		"accept-charset": "",

		// accept: String?
		//		List of MIME types for file upload.
		accept: "",

		// target: String?
		//		Target frame for the document to be opened in.
		target: "",

		templateString: "<form dojoAttachPoint='containerNode' dojoAttachEvent='onreset:_onReset,onsubmit:_onSubmit' ${!nameAttrSetting}></form>",

		attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
			action: "",
			method: "",
			encType: "",
			"accept-charset": "",
			accept: "",
			target: ""
		}),

		postMixInProperties: function(){
			// Setup name=foo string to be referenced from the template (but only if a name has been specified)
			// Unfortunately we can't use attributeMap to set the name due to IE limitations, see #8660
			this.nameAttrSetting = this.name ? ("name='" + this.name + "'") : "";
			this.inherited(arguments);
		},

		execute: function(/*Object*/ formContents){
			// summary:
			//		Deprecated: use submit()
			// tags:
			//		deprecated
		},

		onExecute: function(){
			// summary:
			//		Deprecated: use onSubmit()
			// tags:
			//		deprecated
		},

		_setEncTypeAttr: function(/*String*/ value){
			this.encType = value;
			dojo.attr(this.domNode, "encType", value);
			if(dojo.isIE){ this.domNode.encoding = value; }
		},

		postCreate: function(){
			// IE tries to hide encType
			// TODO: this code should be in parser, not here.
			if(dojo.isIE && this.srcNodeRef && this.srcNodeRef.attributes){
				var item = this.srcNodeRef.attributes.getNamedItem('encType');
				if(item && !item.specified && (typeof item.value == "string")){
					this.set('encType', item.value);
				}
			}
			this.inherited(arguments);
		},

		reset: function(/*Event?*/ e){
			// summary:
			//		restores all widget values back to their init values,
			//		calls onReset() which can cancel the reset by returning false

			// create fake event so we can know if preventDefault() is called
			var faux = {
				returnValue: true, // the IE way
				preventDefault: function(){ // not IE
							this.returnValue = false;
						},
				stopPropagation: function(){}, 
				currentTarget: e ? e.target : this.domNode, 
				target: e ? e.target : this.domNode
			};
			// if return value is not exactly false, and haven't called preventDefault(), then reset
			if(!(this.onReset(faux) === false) && faux.returnValue){
				this.inherited(arguments, []);
			}
		},

		onReset: function(/*Event?*/ e){
			// summary:
			//		Callback when user resets the form. This method is intended
			//		to be over-ridden. When the `reset` method is called
			//		programmatically, the return value from `onReset` is used
			//		to compute whether or not resetting should proceed
			// tags:
			//		callback
			return true; // Boolean
		},

		_onReset: function(e){
			this.reset(e);
			dojo.stopEvent(e);
			return false;
		},

		_onSubmit: function(e){
			var fp = dijit.form.Form.prototype;
			// TODO: remove this if statement beginning with 2.0
			if(this.execute != fp.execute || this.onExecute != fp.onExecute){
				dojo.deprecated("dijit.form.Form:execute()/onExecute() are deprecated. Use onSubmit() instead.", "", "2.0");
				this.onExecute();
				this.execute(this.getValues());
			}
			if(this.onSubmit(e) === false){ // only exactly false stops submit
				dojo.stopEvent(e);
			}
		},

		onSubmit: function(/*Event?*/e){
			// summary:
			//		Callback when user submits the form.
			// description:
			//		This method is intended to be over-ridden, but by default it checks and
			//		returns the validity of form elements. When the `submit`
			//		method is called programmatically, the return value from
			//		`onSubmit` is used to compute whether or not submission
			//		should proceed
			// tags:
			//		extension

			return this.isValid(); // Boolean
		},

		submit: function(){
			// summary:
			//		programmatically submit form if and only if the `onSubmit` returns true
			if(!(this.onSubmit() === false)){
				this.containerNode.submit();
			}
		}
	}
);

}

if(!dojo._hasResource["dijit.layout.StackController"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.StackController"] = true;
dojo.provide("dijit.layout.StackController");







dojo.declare(
		"dijit.layout.StackController",
		[dijit._Widget, dijit._Templated, dijit._Container],
		{
			// summary:
			//		Set of buttons to select a page in a page list.
			// description:
			//		Monitors the specified StackContainer, and whenever a page is
			//		added, deleted, or selected, updates itself accordingly.

			templateString: "<span wairole='tablist' dojoAttachEvent='onkeypress' class='dijitStackController'></span>",

			// containerId: [const] String
			//		The id of the page container that I point to
			containerId: "",

			// buttonWidget: [const] String
			//		The name of the button widget to create to correspond to each page
			buttonWidget: "dijit.layout._StackButton",

			postCreate: function(){
				dijit.setWaiRole(this.domNode, "tablist");

				this.pane2button = {};		// mapping from pane id to buttons
				this.pane2handles = {};		// mapping from pane id to this.connect() handles

				// Listen to notifications from StackContainer
				this.subscribe(this.containerId+"-startup", "onStartup");
				this.subscribe(this.containerId+"-addChild", "onAddChild");
				this.subscribe(this.containerId+"-removeChild", "onRemoveChild");
				this.subscribe(this.containerId+"-selectChild", "onSelectChild");
				this.subscribe(this.containerId+"-containerKeyPress", "onContainerKeyPress");
			},

			onStartup: function(/*Object*/ info){
				// summary:
				//		Called after StackContainer has finished initializing
				// tags:
				//		private
				dojo.forEach(info.children, this.onAddChild, this);
				if(info.selected){
					// Show button corresponding to selected pane (unless selected
					// is null because there are no panes)
					this.onSelectChild(info.selected);
				}
			},

			destroy: function(){
				for(var pane in this.pane2button){
					this.onRemoveChild(dijit.byId(pane));
				}
				this.inherited(arguments);
			},

			onAddChild: function(/*dijit._Widget*/ page, /*Integer?*/ insertIndex){
				// summary:
				//		Called whenever a page is added to the container.
				//		Create button corresponding to the page.
				// tags:
				//		private

				// create an instance of the button widget
				var cls = dojo.getObject(this.buttonWidget);
				var button = new cls({
					id: this.id + "_" + page.id,
					label: page.title,
					dir: page.dir,
					lang: page.lang,
					showLabel: page.showTitle,
					iconClass: page.iconClass,
					closeButton: page.closable,
					title: page.tooltip
				});
				dijit.setWaiState(button.focusNode,"selected", "false");
				this.pane2handles[page.id] = [
					this.connect(page, 'set', function(name, value){
						var buttonAttr = {
							title: 'label',
							showTitle: 'showLabel',
							iconClass: 'iconClass',
							closable: 'closeButton',
							tooltip: 'title'
						}[name];
						if(buttonAttr){
							button.set(buttonAttr, value);
						}
					}),
					this.connect(button, 'onClick', dojo.hitch(this,"onButtonClick", page)),
					this.connect(button, 'onClickCloseButton', dojo.hitch(this,"onCloseButtonClick", page))
				];
				this.addChild(button, insertIndex);
				this.pane2button[page.id] = button;
				page.controlButton = button;	// this value might be overwritten if two tabs point to same container
				if(!this._currentChild){ // put the first child into the tab order
					button.focusNode.setAttribute("tabIndex", "0");
					dijit.setWaiState(button.focusNode, "selected", "true");
					this._currentChild = page;
				}
				// make sure all tabs have the same length
				if(!this.isLeftToRight() && dojo.isIE && this._rectifyRtlTabList){
					this._rectifyRtlTabList();
				}
			},

			onRemoveChild: function(/*dijit._Widget*/ page){
				// summary:
				//		Called whenever a page is removed from the container.
				//		Remove the button corresponding to the page.
				// tags:
				//		private

				if(this._currentChild === page){ this._currentChild = null; }
				dojo.forEach(this.pane2handles[page.id], this.disconnect, this);
				delete this.pane2handles[page.id];
				var button = this.pane2button[page.id];
				if(button){
					this.removeChild(button);
					delete this.pane2button[page.id];
					button.destroy();
				}
				delete page.controlButton;
			},

			onSelectChild: function(/*dijit._Widget*/ page){
				// summary:
				//		Called when a page has been selected in the StackContainer, either by me or by another StackController
				// tags:
				//		private

				if(!page){ return; }

				if(this._currentChild){
					var oldButton=this.pane2button[this._currentChild.id];
					oldButton.set('checked', false);
					dijit.setWaiState(oldButton.focusNode, "selected", "false");
					oldButton.focusNode.setAttribute("tabIndex", "-1");
				}

				var newButton=this.pane2button[page.id];
				newButton.set('checked', true);
				dijit.setWaiState(newButton.focusNode, "selected", "true");
				this._currentChild = page;
				newButton.focusNode.setAttribute("tabIndex", "0");
				var container = dijit.byId(this.containerId);
				dijit.setWaiState(container.containerNode, "labelledby", newButton.id);
			},

			onButtonClick: function(/*dijit._Widget*/ page){
				// summary:
				//		Called whenever one of my child buttons is pressed in an attempt to select a page
				// tags:
				//		private

				var container = dijit.byId(this.containerId);
				container.selectChild(page);
			},

			onCloseButtonClick: function(/*dijit._Widget*/ page){
				// summary:
				//		Called whenever one of my child buttons [X] is pressed in an attempt to close a page
				// tags:
				//		private

				var container = dijit.byId(this.containerId);
				container.closeChild(page);
				if(this._currentChild){
					var b = this.pane2button[this._currentChild.id];
					if(b){
						dijit.focus(b.focusNode || b.domNode);
					}
				}
			},

			// TODO: this is a bit redundant with forward, back api in StackContainer
			adjacent: function(/*Boolean*/ forward){
				// summary:
				//		Helper for onkeypress to find next/previous button
				// tags:
				//		private

				if(!this.isLeftToRight() && (!this.tabPosition || /top|bottom/.test(this.tabPosition))){ forward = !forward; }
				// find currently focused button in children array
				var children = this.getChildren();
				var current = dojo.indexOf(children, this.pane2button[this._currentChild.id]);
				// pick next button to focus on
				var offset = forward ? 1 : children.length - 1;
				return children[ (current + offset) % children.length ]; // dijit._Widget
			},

			onkeypress: function(/*Event*/ e){
				// summary:
				//		Handle keystrokes on the page list, for advancing to next/previous button
				//		and closing the current page if the page is closable.
				// tags:
				//		private

				if(this.disabled || e.altKey ){ return; }
				var forward = null;
				if(e.ctrlKey || !e._djpage){
					var k = dojo.keys;
					switch(e.charOrCode){
						case k.LEFT_ARROW:
						case k.UP_ARROW:
							if(!e._djpage){ forward = false; }
							break;
						case k.PAGE_UP:
							if(e.ctrlKey){ forward = false; }
							break;
						case k.RIGHT_ARROW:
						case k.DOWN_ARROW:
							if(!e._djpage){ forward = true; }
							break;
						case k.PAGE_DOWN:
							if(e.ctrlKey){ forward = true; }
							break;
						case k.DELETE:
							if(this._currentChild.closable){
								this.onCloseButtonClick(this._currentChild);
							}
							dojo.stopEvent(e);
							break;
						default:
							if(e.ctrlKey){
								if(e.charOrCode === k.TAB){
									this.adjacent(!e.shiftKey).onClick();
									dojo.stopEvent(e);
								}else if(e.charOrCode == "w"){
									if(this._currentChild.closable){
										this.onCloseButtonClick(this._currentChild);
									}
									dojo.stopEvent(e); // avoid browser tab closing.
								}
							}
					}
					// handle page navigation
					if(forward !== null){
						this.adjacent(forward).onClick();
						dojo.stopEvent(e);
					}
				}
			},

			onContainerKeyPress: function(/*Object*/ info){
				// summary:
				//		Called when there was a keypress on the container
				// tags:
				//		private
				info.e._djpage = info.page;
				this.onkeypress(info.e);
			}
	});


dojo.declare("dijit.layout._StackButton",
		dijit.form.ToggleButton,
		{
		// summary:
		//		Internal widget used by StackContainer.
		// description:
		//		The button-like or tab-like object you click to select or delete a page
		// tags:
		//		private

		// Override _FormWidget.tabIndex.
		// StackContainer buttons are not in the tab order by default.
		// Probably we should be calling this.startupKeyNavChildren() instead.
		tabIndex: "-1",

		postCreate: function(/*Event*/ evt){
			dijit.setWaiRole((this.focusNode || this.domNode), "tab");
			this.inherited(arguments);
		},

		onClick: function(/*Event*/ evt){
			// summary:
			//		This is for TabContainer where the tabs are <span> rather than button,
			//		so need to set focus explicitly (on some browsers)
			//		Note that you shouldn't override this method, but you can connect to it.
			dijit.focus(this.focusNode);

			// ... now let StackController catch the event and tell me what to do
		},

		onClickCloseButton: function(/*Event*/ evt){
			// summary:
			//		StackContainer connects to this function; if your widget contains a close button
			//		then clicking it should call this function.
			//		Note that you shouldn't override this method, but you can connect to it.
			evt.stopPropagation();
		}
	});


}

if(!dojo._hasResource["dijit.layout.StackContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.StackContainer"] = true;
dojo.provide("dijit.layout.StackContainer");






dojo.declare(
	"dijit.layout.StackContainer",
	dijit.layout._LayoutWidget,
	{
	// summary:
	//		A container that has multiple children, but shows only
	//		one child at a time
	//
	// description:
	//		A container for widgets (ContentPanes, for example) That displays
	//		only one Widget at a time.
	//
	//		Publishes topics [widgetId]-addChild, [widgetId]-removeChild, and [widgetId]-selectChild
	//
	//		Can be base class for container, Wizard, Show, etc.

	// doLayout: Boolean
	//		If true, change the size of my currently displayed child to match my size
	doLayout: true,

	// persist: Boolean
	//		Remembers the selected child across sessions
	persist: false,

	baseClass: "dijitStackContainer",

/*=====
	// selectedChildWidget: [readonly] dijit._Widget
	//		References the currently selected child widget, if any.
	//		Adjust selected child with selectChild() method.
	selectedChildWidget: null,
=====*/

	postCreate: function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "dijitLayoutContainer");
		dijit.setWaiRole(this.containerNode, "tabpanel");
		this.connect(this.domNode, "onkeypress", this._onKeyPress);
	},

	startup: function(){
		if(this._started){ return; }

		var children = this.getChildren();

		// Setup each page panel to be initially hidden
		dojo.forEach(children, this._setupChild, this);

		// Figure out which child to initially display, defaulting to first one
		if(this.persist){
			this.selectedChildWidget = dijit.byId(dojo.cookie(this.id + "_selectedChild"));
		}else{
			dojo.some(children, function(child){
				if(child.selected){
					this.selectedChildWidget = child;
				}
				return child.selected;
			}, this);
		}
		var selected = this.selectedChildWidget;
		if(!selected && children[0]){
			selected = this.selectedChildWidget = children[0];
			selected.selected = true;
		}

		// Publish information about myself so any StackControllers can initialize.
		// This needs to happen before this.inherited(arguments) so that for
		// TabContainer, this._contentBox doesn't include the space for the tab labels.
		dojo.publish(this.id+"-startup", [{children: children, selected: selected}]);

		// Startup each child widget, and do initial layout like setting this._contentBox,
		// then calls this.resize() which does the initial sizing on the selected child.
		this.inherited(arguments);
	},

	resize: function(){
		// Resize is called when we are first made visible (it's called from startup()
		// if we are initially visible).   If this is the first time we've been made
		// visible then show our first child.
		var selected = this.selectedChildWidget;
		if(selected && !this._hasBeenShown){
			this._hasBeenShown = true;
			this._showChild(selected);
		}
		this.inherited(arguments);
	},

	_setupChild: function(/*dijit._Widget*/ child){
		// Overrides _LayoutWidget._setupChild()

		this.inherited(arguments);

		dojo.removeClass(child.domNode, "dijitVisible");
		dojo.addClass(child.domNode, "dijitHidden");

		// remove the title attribute so it doesn't show up when i hover
		// over a node
		child.domNode.title = "";
	},

	addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
		// Overrides _Container.addChild() to do layout and publish events

		this.inherited(arguments);

		if(this._started){
			dojo.publish(this.id+"-addChild", [child, insertIndex]);

			// in case the tab titles have overflowed from one line to two lines
			// (or, if this if first child, from zero lines to one line)
			// TODO: w/ScrollingTabController this is no longer necessary, although
			// ScrollTabController.resize() does need to get called to show/hide
			// the navigation buttons as appropriate, but that's handled in ScrollingTabController.onAddChild()
			this.layout();

			// if this is the first child, then select it
			if(!this.selectedChildWidget){
				this.selectChild(child);
			}
		}
	},

	removeChild: function(/*dijit._Widget*/ page){
		// Overrides _Container.removeChild() to do layout and publish events

		this.inherited(arguments);

		if(this._started){
			// this will notify any tablists to remove a button; do this first because it may affect sizing
			dojo.publish(this.id + "-removeChild", [page]);
		}

		// If we are being destroyed than don't run the code below (to select another page), because we are deleting
		// every page one by one
		if(this._beingDestroyed){ return; }

		// Select new page to display, also updating TabController to show the respective tab.
		// Do this before layout call because it can affect the height of the TabController.
		if(this.selectedChildWidget === page){
			this.selectedChildWidget = undefined;
			if(this._started){
				var children = this.getChildren();
				if(children.length){
					this.selectChild(children[0]);
				}
			}
		}

		if(this._started){
			// In case the tab titles now take up one line instead of two lines
			// (note though that ScrollingTabController never overflows to multiple lines),
			// or the height has changed slightly because of addition/removal of tab which close icon
			this.layout();
		}
	},

	selectChild: function(/*dijit._Widget|String*/ page, /*Boolean*/ animate){
		// summary:
		//		Show the given widget (which must be one of my children)
		// page:
		//		Reference to child widget or id of child widget

		page = dijit.byId(page);

		if(this.selectedChildWidget != page){
			// Deselect old page and select new one
			this._transition(page, this.selectedChildWidget, animate);
			this.selectedChildWidget = page;
			dojo.publish(this.id+"-selectChild", [page]);

			if(this.persist){
				dojo.cookie(this.id + "_selectedChild", this.selectedChildWidget.id);
			}
		}
	},

	_transition: function(/*dijit._Widget*/newWidget, /*dijit._Widget*/oldWidget){
		// summary:
		//		Hide the old widget and display the new widget.
		//		Subclasses should override this.
		// tags:
		//		protected extension
		if(oldWidget){
			this._hideChild(oldWidget);
		}
		this._showChild(newWidget);

		// Size the new widget, in case this is the first time it's being shown,
		// or I have been resized since the last time it was shown.
		// Note that page must be visible for resizing to work.
		if(newWidget.resize){
			if(this.doLayout){
				newWidget.resize(this._containerContentBox || this._contentBox);
			}else{
				// the child should pick it's own size but we still need to call resize()
				// (with no arguments) to let the widget lay itself out
				newWidget.resize();
			}
		}
	},

	_adjacent: function(/*Boolean*/ forward){
		// summary:
		//		Gets the next/previous child widget in this container from the current selection.
		var children = this.getChildren();
		var index = dojo.indexOf(children, this.selectedChildWidget);
		index += forward ? 1 : children.length - 1;
		return children[ index % children.length ]; // dijit._Widget
	},

	forward: function(){
		// summary:
		//		Advance to next page.
		this.selectChild(this._adjacent(true), true);
	},

	back: function(){
		// summary:
		//		Go back to previous page.
		this.selectChild(this._adjacent(false), true);
	},

	_onKeyPress: function(e){
		dojo.publish(this.id+"-containerKeyPress", [{ e: e, page: this}]);
	},

	layout: function(){
		// Implement _LayoutWidget.layout() virtual method.
		if(this.doLayout && this.selectedChildWidget && this.selectedChildWidget.resize){
			this.selectedChildWidget.resize(this._containerContentBox || this._contentBox);
		}
	},

	_showChild: function(/*dijit._Widget*/ page){
		// summary:
		//		Show the specified child by changing it's CSS, and call _onShow()/onShow() so
		//		it can do any updates it needs regarding loading href's etc.
		var children = this.getChildren();
		page.isFirstChild = (page == children[0]);
		page.isLastChild = (page == children[children.length-1]);
		page.selected = true;

		dojo.removeClass(page.domNode, "dijitHidden");
		dojo.addClass(page.domNode, "dijitVisible");

		page._onShow();
	},

	_hideChild: function(/*dijit._Widget*/ page){
		// summary:
		//		Hide the specified child by changing it's CSS, and call _onHide() so
		//		it's notified.
		page.selected=false;
		dojo.removeClass(page.domNode, "dijitVisible");
		dojo.addClass(page.domNode, "dijitHidden");

		page.onHide();
	},

	closeChild: function(/*dijit._Widget*/ page){
		// summary:
		//		Callback when user clicks the [X] to remove a page.
		//		If onClose() returns true then remove and destroy the child.
		// tags:
		//		private
		var remove = page.onClose(this, page);
		if(remove){
			this.removeChild(page);
			// makes sure we can clean up executeScripts in ContentPane onUnLoad
			page.destroyRecursive();
		}
	},

	destroyDescendants: function(/*Boolean*/preserveDom){
		dojo.forEach(this.getChildren(), function(child){
			this.removeChild(child);
			child.destroyRecursive(preserveDom);
		}, this);
	}
});

// For back-compat, remove for 2.0



// These arguments can be specified for the children of a StackContainer.
// Since any widget can be specified as a StackContainer child, mix them
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// selected: Boolean
	//		Parameter for children of `dijit.layout.StackContainer` or subclasses.
	//		Specifies that this widget should be the initially displayed pane.
	//		Note: to change the selected child use `dijit.layout.StackContainer.selectChild`
	selected: false,

	// closable: Boolean
	//		Parameter for children of `dijit.layout.StackContainer` or subclasses.
	//		True if user can close (destroy) this child, such as (for example) clicking the X on the tab.
	closable: false,

	// iconClass: String
	//		Parameter for children of `dijit.layout.StackContainer` or subclasses.
	//		CSS Class specifying icon to use in label associated with this pane.
	iconClass: "",

	// showTitle: Boolean
	//		Parameter for children of `dijit.layout.StackContainer` or subclasses.
	//		When true, display title of this widget as tab label etc., rather than just using
	//		icon specified in iconClass
	showTitle: true
});

}

if(!dojo._hasResource["dijit.layout._TabContainerBase"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._TabContainerBase"] = true;
dojo.provide("dijit.layout._TabContainerBase");




dojo.declare("dijit.layout._TabContainerBase",
	[dijit.layout.StackContainer, dijit._Templated],
	{
	// summary:
	//		Abstract base class for TabContainer.   Must define _makeController() to instantiate
	//		and return the widget that displays the tab labels
	// description:
	//		A TabContainer is a container that has multiple panes, but shows only
	//		one pane at a time.  There are a set of tabs corresponding to each pane,
	//		where each tab has the name (aka title) of the pane, and optionally a close button.

	// tabPosition: String
	//		Defines where tabs go relative to tab content.
	//		"top", "bottom", "left-h", "right-h"
	tabPosition: "top",

	baseClass: "dijitTabContainer",

	// tabStrip: Boolean
	//		Defines whether the tablist gets an extra class for layouting, putting a border/shading
	//		around the set of tabs.
	tabStrip: false,

	// nested: Boolean
	//		If true, use styling for a TabContainer nested inside another TabContainer.
	//		For tundra etc., makes tabs look like links, and hides the outer
	//		border since the outer TabContainer already has a border.
	nested: false,

	templateString: dojo.cache("dijit.layout", "templates/TabContainer.html", "<div class=\"dijitTabContainer\">\r\n\t<div class=\"dijitTabListWrapper\" dojoAttachPoint=\"tablistNode\"></div>\r\n\t<div dojoAttachPoint=\"tablistSpacer\" class=\"dijitTabSpacer ${baseClass}-spacer\"></div>\r\n\t<div class=\"dijitTabPaneWrapper ${baseClass}-container\" dojoAttachPoint=\"containerNode\"></div>\r\n</div>\r\n"),

	postMixInProperties: function(){
		// set class name according to tab position, ex: dijitTabContainerTop
		this.baseClass += this.tabPosition.charAt(0).toUpperCase() + this.tabPosition.substr(1).replace(/-.*/, "");

		this.srcNodeRef && dojo.style(this.srcNodeRef, "visibility", "hidden");

		this.inherited(arguments);
	},

	postCreate: function(){
		this.inherited(arguments);

		// Create the tab list that will have a tab (a.k.a. tab button) for each tab panel
		this.tablist = this._makeController(this.tablistNode);

		if(!this.doLayout){ dojo.addClass(this.domNode, "dijitTabContainerNoLayout"); }

		if(this.nested){
			/* workaround IE's lack of support for "a > b" selectors by
			 * tagging each node in the template.
			 */
			dojo.addClass(this.domNode, "dijitTabContainerNested");
			dojo.addClass(this.tablist.containerNode, "dijitTabContainerTabListNested");
			dojo.addClass(this.tablistSpacer, "dijitTabContainerSpacerNested");
			dojo.addClass(this.containerNode, "dijitTabPaneWrapperNested");
		}else{
			dojo.addClass(this.domNode, "tabStrip-" + (this.tabStrip ? "enabled" : "disabled"));
		}
	},

	_setupChild: function(/*dijit._Widget*/ tab){
		// Overrides StackContainer._setupChild().
		dojo.addClass(tab.domNode, "dijitTabPane");
		this.inherited(arguments);
	},

	startup: function(){
		if(this._started){ return; }

		// wire up the tablist and its tabs
		this.tablist.startup();

		this.inherited(arguments);
	},

	layout: function(){
		// Overrides StackContainer.layout().
		// Configure the content pane to take up all the space except for where the tabs are

		if(!this._contentBox || typeof(this._contentBox.l) == "undefined"){return;}

		var sc = this.selectedChildWidget;

		if(this.doLayout){
			// position and size the titles and the container node
			var titleAlign = this.tabPosition.replace(/-h/, "");
			this.tablist.layoutAlign = titleAlign;
			var children = [this.tablist, {
				domNode: this.tablistSpacer,
				layoutAlign: titleAlign
			}, {
				domNode: this.containerNode,
				layoutAlign: "client"
			}];
			dijit.layout.layoutChildren(this.domNode, this._contentBox, children);

			// Compute size to make each of my children.
			// children[2] is the margin-box size of this.containerNode, set by layoutChildren() call above
			this._containerContentBox = dijit.layout.marginBox2contentBox(this.containerNode, children[2]);

			if(sc && sc.resize){
				sc.resize(this._containerContentBox);
			}
		}else{
			// just layout the tab controller, so it can position left/right buttons etc.
			if(this.tablist.resize){
				this.tablist.resize({w: dojo.contentBox(this.domNode).w});
			}

			// and call resize() on the selected pane just to tell it that it's been made visible
			if(sc && sc.resize){
				sc.resize();
			}
		}
	},

	destroy: function(){
		if(this.tablist){
			this.tablist.destroy();
		}
		this.inherited(arguments);
	}
});


}

if(!dojo._hasResource["dijit.layout.TabController"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.TabController"] = true;
dojo.provide("dijit.layout.TabController");



// Menu is used for an accessible close button, would be nice to have a lighter-weight solution





dojo.declare("dijit.layout.TabController",
	dijit.layout.StackController,
{
	// summary:
	// 		Set of tabs (the things with titles and a close button, that you click to show a tab panel).
	//		Used internally by `dijit.layout.TabContainer`.
	// description:
	//		Lets the user select the currently shown pane in a TabContainer or StackContainer.
	//		TabController also monitors the TabContainer, and whenever a pane is
	//		added or deleted updates itself accordingly.
	// tags:
	//		private

	templateString: "<div wairole='tablist' dojoAttachEvent='onkeypress:onkeypress'></div>",

	// tabPosition: String
	//		Defines where tabs go relative to the content.
	//		"top", "bottom", "left-h", "right-h"
	tabPosition: "top",

	// buttonWidget: String
	//		The name of the tab widget to create to correspond to each page
	buttonWidget: "dijit.layout._TabButton",

	_rectifyRtlTabList: function(){
		// summary:
		//		For left/right TabContainer when page is RTL mode, rectify the width of all tabs to be equal, otherwise the tab widths are different in IE

		if(0 >= this.tabPosition.indexOf('-h')){ return; }
		if(!this.pane2button){ return; }

		var maxWidth = 0;
		for(var pane in this.pane2button){
			var ow = this.pane2button[pane].innerDiv.scrollWidth;
			maxWidth = Math.max(maxWidth, ow);
		}
		//unify the length of all the tabs
		for(pane in this.pane2button){
			this.pane2button[pane].innerDiv.style.width = maxWidth + 'px';
		}
	}
});

dojo.declare("dijit.layout._TabButton",
	dijit.layout._StackButton,
	{
	// summary:
	//		A tab (the thing you click to select a pane).
	// description:
	//		Contains the title of the pane, and optionally a close-button to destroy the pane.
	//		This is an internal widget and should not be instantiated directly.
	// tags:
	//		private

	// baseClass: String
	//		The CSS class applied to the domNode.
	baseClass: "dijitTab",

	// Apply dijitTabCloseButtonHover when close button is hovered
	cssStateNodes: {
		closeNode: "dijitTabCloseButton"
	},

	templateString: dojo.cache("dijit.layout", "templates/_TabButton.html", "<div waiRole=\"presentation\" dojoAttachPoint=\"titleNode\" dojoAttachEvent='onclick:onClick'>\r\n    <div waiRole=\"presentation\" class='dijitTabInnerDiv' dojoAttachPoint='innerDiv'>\r\n        <div waiRole=\"presentation\" class='dijitTabContent' dojoAttachPoint='tabContent'>\r\n        \t<div waiRole=\"presentation\" dojoAttachPoint='focusNode'>\r\n\t\t        <img src=\"${_blankGif}\" alt=\"\" class=\"dijitIcon\" dojoAttachPoint='iconNode' />\r\n\t\t        <span dojoAttachPoint='containerNode' class='tabLabel'></span>\r\n\t\t        <span class=\"dijitInline dijitTabCloseButton dijitTabCloseIcon\" dojoAttachPoint='closeNode'\r\n\t\t        \t\tdojoAttachEvent='onclick: onClickCloseButton' waiRole=\"presentation\">\r\n\t\t            <span dojoAttachPoint='closeText' class='dijitTabCloseText'>x</span\r\n\t\t        ></span>\r\n\t\t\t</div>\r\n        </div>\r\n    </div>\r\n</div>\r\n"),

	// Override _FormWidget.scrollOnFocus.
	// Don't scroll the whole tab container into view when the button is focused.
	scrollOnFocus: false,

	postMixInProperties: function(){
		// Override blank iconClass from Button to do tab height adjustment on IE6,
		// to make sure that tabs with and w/out close icons are same height
		if(!this.iconClass){
			this.iconClass = "dijitTabButtonIcon";
		}
	},

	postCreate: function(){
		this.inherited(arguments);
		dojo.setSelectable(this.containerNode, false);

		// If a custom icon class has not been set for the
		// tab icon, set its width to one pixel. This ensures
		// that the height styling of the tab is maintained,
		// as it is based on the height of the icon.
		// TODO: I still think we can just set dijitTabButtonIcon to 1px in CSS <Bill>
		if(this.iconNode.className == "dijitTabButtonIcon"){
			dojo.style(this.iconNode, "width", "1px");
		}
	},

	startup: function(){
		this.inherited(arguments);
		var n = this.domNode;

		// Required to give IE6 a kick, as it initially hides the
		// tabs until they are focused on.
		setTimeout(function(){
			n.className = n.className;
		}, 1);
	},

	_setCloseButtonAttr: function(disp){
		this.closeButton = disp;
		dojo.toggleClass(this.innerDiv, "dijitClosable", disp);
		this.closeNode.style.display = disp ? "" : "none";
		if(disp){
			var _nlsResources = dojo.i18n.getLocalization("dijit", "common");
			if(this.closeNode){
				dojo.attr(this.closeNode,"title", _nlsResources.itemClose);
			}
			// add context menu onto title button
			var _nlsResources = dojo.i18n.getLocalization("dijit", "common");
			this._closeMenu = new dijit.Menu({
				id: this.id+"_Menu",
				dir: this.dir,
				lang: this.lang,
				targetNodeIds: [this.domNode]
			});

			this._closeMenu.addChild(new dijit.MenuItem({
				label: _nlsResources.itemClose,
				dir: this.dir,
				lang: this.lang,
				onClick: dojo.hitch(this, "onClickCloseButton")
			}));
		}else{
			if(this._closeMenu){
				this._closeMenu.destroyRecursive();
				delete this._closeMenu;
			}
		}
	},
	_setLabelAttr: function(/*String*/ content){
		// summary:
		//		Hook for attr('label', ...) to work.
		// description:
		//		takes an HTML string.
		//		Inherited ToggleButton implementation will Set the label (text) of the button; 
		//		Need to set the alt attribute of icon on tab buttons if no label displayed
			this.inherited(arguments);
			if(this.showLabel == false && !this.params.title){
				this.iconNode.alt = dojo.trim(this.containerNode.innerText || this.containerNode.textContent || '');
			}
		},

	destroy: function(){
		if(this._closeMenu){
			this._closeMenu.destroyRecursive();
			delete this._closeMenu;
		}
		this.inherited(arguments);
	}
});

}

if(!dojo._hasResource["dijit.layout.ScrollingTabController"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.ScrollingTabController"] = true;
dojo.provide("dijit.layout.ScrollingTabController");




dojo.declare("dijit.layout.ScrollingTabController",
	dijit.layout.TabController,
	{
	// summary:
	//		Set of tabs with left/right arrow keys and a menu to switch between tabs not
	//		all fitting on a single row.
	//		Works only for horizontal tabs (either above or below the content, not to the left
	//		or right).
	// tags:
	//		private

	templateString: dojo.cache("dijit.layout", "templates/ScrollingTabController.html", "<div class=\"dijitTabListContainer-${tabPosition}\" style=\"visibility:hidden\">\r\n\t<div dojoType=\"dijit.layout._ScrollingTabControllerButton\"\r\n\t\t\tclass=\"tabStripButton-${tabPosition}\"\r\n\t\t\tid=\"${id}_menuBtn\" iconClass=\"dijitTabStripMenuIcon\"\r\n\t\t\tdojoAttachPoint=\"_menuBtn\" showLabel=false>&#9660;</div>\r\n\t<div dojoType=\"dijit.layout._ScrollingTabControllerButton\"\r\n\t\t\tclass=\"tabStripButton-${tabPosition}\"\r\n\t\t\tid=\"${id}_leftBtn\" iconClass=\"dijitTabStripSlideLeftIcon\"\r\n\t\t\tdojoAttachPoint=\"_leftBtn\" dojoAttachEvent=\"onClick: doSlideLeft\" showLabel=false>&#9664;</div>\r\n\t<div dojoType=\"dijit.layout._ScrollingTabControllerButton\"\r\n\t\t\tclass=\"tabStripButton-${tabPosition}\"\r\n\t\t\tid=\"${id}_rightBtn\" iconClass=\"dijitTabStripSlideRightIcon\"\r\n\t\t\tdojoAttachPoint=\"_rightBtn\" dojoAttachEvent=\"onClick: doSlideRight\" showLabel=false>&#9654;</div>\r\n\t<div class='dijitTabListWrapper' dojoAttachPoint='tablistWrapper'>\r\n\t\t<div wairole='tablist' dojoAttachEvent='onkeypress:onkeypress'\r\n\t\t\t\tdojoAttachPoint='containerNode' class='nowrapTabStrip'></div>\r\n\t</div>\r\n</div>\r\n"),

	// useMenu:[const] Boolean
	//		True if a menu should be used to select tabs when they are too
	//		wide to fit the TabContainer, false otherwise.
	useMenu: true,

	// useSlider: [const] Boolean
	//		True if a slider should be used to select tabs when they are too
	//		wide to fit the TabContainer, false otherwise.
	useSlider: true,

	// tabStripClass: String
	//		The css class to apply to the tab strip, if it is visible.
	tabStripClass: "",

	widgetsInTemplate: true,

	// _minScroll: Number
	//		The distance in pixels from the edge of the tab strip which,
	//		if a scroll animation is less than, forces the scroll to
	//		go all the way to the left/right.
	_minScroll: 5,

	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
		"class": "containerNode"
	}),

	postCreate: function(){
		this.inherited(arguments);
		var n = this.domNode;

		this.scrollNode = this.tablistWrapper;
		this._initButtons();

		if(!this.tabStripClass){
			this.tabStripClass = "dijitTabContainer" +
				this.tabPosition.charAt(0).toUpperCase() +
				this.tabPosition.substr(1).replace(/-.*/, "") +
				"None";
			dojo.addClass(n, "tabStrip-disabled")
		}

		dojo.addClass(this.tablistWrapper, this.tabStripClass);
	},

	onStartup: function(){
		this.inherited(arguments);

		// Do not show the TabController until the related
		// StackController has added it's children.  This gives
		// a less visually jumpy instantiation.
		dojo.style(this.domNode, "visibility", "visible");
		this._postStartup = true;
	},

	onAddChild: function(page, insertIndex){
		this.inherited(arguments);
		var menuItem;
		if(this.useMenu){
			var containerId = this.containerId;
			menuItem = new dijit.MenuItem({
				id: page.id + "_stcMi",
				label: page.title,
				dir: page.dir,
				lang: page.lang,
				onClick: dojo.hitch(this, function(){
					var container = dijit.byId(containerId);
					container.selectChild(page);
				})
			});
			this._menuChildren[page.id] = menuItem;
			this._menu.addChild(menuItem, insertIndex);
		}

		// update the menuItem label when the button label is updated
		this.pane2handles[page.id].push(
			this.connect(this.pane2button[page.id], "set", function(name, value){
				if(this._postStartup){
					if(name == "label"){
						if(menuItem){
							menuItem.set(name, value);
						}
	
						// The changed label will have changed the width of the
						// buttons, so do a resize
						if(this._dim){
							this.resize(this._dim);
						}
					}
				}
			})
		);

		// Increment the width of the wrapper when a tab is added
		// This makes sure that the buttons never wrap.
		// The value 200 is chosen as it should be bigger than most
		// Tab button widths.
		dojo.style(this.containerNode, "width",
			(dojo.style(this.containerNode, "width") + 200) + "px");
	},

	onRemoveChild: function(page, insertIndex){
		// null out _selectedTab because we are about to delete that dom node
		var button = this.pane2button[page.id];
		if(this._selectedTab === button.domNode){
			this._selectedTab = null;
		}

		// delete menu entry corresponding to pane that was removed from TabContainer
		if(this.useMenu && page && page.id && this._menuChildren[page.id]){
			this._menu.removeChild(this._menuChildren[page.id]);
			this._menuChildren[page.id].destroy();
			delete this._menuChildren[page.id];
		}

		this.inherited(arguments);
	},

	_initButtons: function(){
		// summary:
		//		Creates the buttons used to scroll to view tabs that
		//		may not be visible if the TabContainer is too narrow.
		this._menuChildren = {};

		// Make a list of the buttons to display when the tab labels become
		// wider than the TabContainer, and hide the other buttons.
		// Also gets the total width of the displayed buttons.
		this._btnWidth = 0;
		this._buttons = dojo.query("> .tabStripButton", this.domNode).filter(function(btn){
			if((this.useMenu && btn == this._menuBtn.domNode) ||
				(this.useSlider && (btn == this._rightBtn.domNode || btn == this._leftBtn.domNode))){
				this._btnWidth += dojo.marginBox(btn).w;
				return true;
			}else{
				dojo.style(btn, "display", "none");
				return false;
			}
		}, this);

		if(this.useMenu){
			// Create the menu that is used to select tabs.
			this._menu = new dijit.Menu({
				id: this.id + "_menu",
				dir: this.dir,
				lang: this.lang,
				targetNodeIds: [this._menuBtn.domNode],
				leftClickToOpen: true,
				refocus: false	// selecting a menu item sets focus to a TabButton
			});
			this._supportingWidgets.push(this._menu);
		}
	},

	_getTabsWidth: function(){
		var children = this.getChildren();
		if(children.length){
			var leftTab = children[this.isLeftToRight() ? 0 : children.length - 1].domNode,
				rightTab = children[this.isLeftToRight() ? children.length - 1 : 0].domNode;
			return rightTab.offsetLeft + dojo.style(rightTab, "width") - leftTab.offsetLeft;
		}else{
			return 0;
		}
	},

	_enableBtn: function(width){
		// summary:
		//		Determines if the tabs are wider than the width of the TabContainer, and
		//		thus that we need to display left/right/menu navigation buttons.
		var tabsWidth = this._getTabsWidth();
		width = width || dojo.style(this.scrollNode, "width");
		return tabsWidth > 0 && width < tabsWidth;
	},

	resize: function(dim){
		// summary:
		//		Hides or displays the buttons used to scroll the tab list and launch the menu
		//		that selects tabs.

		if(this.domNode.offsetWidth == 0){
			return;
		}

		// Save the dimensions to be used when a child is renamed.
		this._dim = dim;

		// Set my height to be my natural height (tall enough for one row of tab labels),
		// and my content-box width based on margin-box width specified in dim parameter.
		// But first reset scrollNode.height in case it was set by layoutChildren() call
		// in a previous run of this method.
		this.scrollNode.style.height = "auto";
		this._contentBox = dijit.layout.marginBox2contentBox(this.domNode, {h: 0, w: dim.w});
		this._contentBox.h = this.scrollNode.offsetHeight;
		dojo.contentBox(this.domNode, this._contentBox);

		// Show/hide the left/right/menu navigation buttons depending on whether or not they
		// are needed.
		var enable = this._enableBtn(this._contentBox.w);
		this._buttons.style("display", enable ? "" : "none");

		// Position and size the navigation buttons and the tablist
		this._leftBtn.layoutAlign = "left";
		this._rightBtn.layoutAlign = "right";
		this._menuBtn.layoutAlign = this.isLeftToRight() ? "right" : "left";
		dijit.layout.layoutChildren(this.domNode, this._contentBox,
			[this._menuBtn, this._leftBtn, this._rightBtn, {domNode: this.scrollNode, layoutAlign: "client"}]);

		// set proper scroll so that selected tab is visible
		if(this._selectedTab){
			if(this._anim && this._anim.status() == "playing"){
				this._anim.stop();
			}
			var w = this.scrollNode,
				sl = this._convertToScrollLeft(this._getScrollForSelectedTab());
			w.scrollLeft = sl;
		}

		// Enable/disabled left right buttons depending on whether or not user can scroll to left or right
		this._setButtonClass(this._getScroll());
		
		this._postResize = true;
	},

	_getScroll: function(){
		// summary:
		//		Returns the current scroll of the tabs where 0 means
		//		"scrolled all the way to the left" and some positive number, based on #
		//		of pixels of possible scroll (ex: 1000) means "scrolled all the way to the right"
		var sl = (this.isLeftToRight() || dojo.isIE < 8 || (dojo.isIE && dojo.isQuirks) || dojo.isWebKit) ? this.scrollNode.scrollLeft :
				dojo.style(this.containerNode, "width") - dojo.style(this.scrollNode, "width")
					 + (dojo.isIE == 8 ? -1 : 1) * this.scrollNode.scrollLeft;
		return sl;
	},

	_convertToScrollLeft: function(val){
		// summary:
		//		Given a scroll value where 0 means "scrolled all the way to the left"
		//		and some positive number, based on # of pixels of possible scroll (ex: 1000)
		//		means "scrolled all the way to the right", return value to set this.scrollNode.scrollLeft
		//		to achieve that scroll.
		//
		//		This method is to adjust for RTL funniness in various browsers and versions.
		if(this.isLeftToRight() || dojo.isIE < 8 || (dojo.isIE && dojo.isQuirks) || dojo.isWebKit){
			return val;
		}else{
			var maxScroll = dojo.style(this.containerNode, "width") - dojo.style(this.scrollNode, "width");
			return (dojo.isIE == 8 ? -1 : 1) * (val - maxScroll);
		}
	},

	onSelectChild: function(/*dijit._Widget*/ page){
		// summary:
		//		Smoothly scrolls to a tab when it is selected.

		var tab = this.pane2button[page.id];
		if(!tab || !page){return;}

		// Scroll to the selected tab, except on startup, when scrolling is handled in resize()
		var node = tab.domNode;
		if(this._postResize && node != this._selectedTab){
			this._selectedTab = node;

			var sl = this._getScroll();

			if(sl > node.offsetLeft ||
					sl + dojo.style(this.scrollNode, "width") <
					node.offsetLeft + dojo.style(node, "width")){
				this.createSmoothScroll().play();
			}
		}

		this.inherited(arguments);
	},

	_getScrollBounds: function(){
		// summary:
		//		Returns the minimum and maximum scroll setting to show the leftmost and rightmost
		//		tabs (respectively)
		var children = this.getChildren(),
			scrollNodeWidth = dojo.style(this.scrollNode, "width"),		// about 500px
			containerWidth = dojo.style(this.containerNode, "width"),	// 50,000px
			maxPossibleScroll = containerWidth - scrollNodeWidth,	// scrolling until right edge of containerNode visible
			tabsWidth = this._getTabsWidth();

		if(children.length && tabsWidth > scrollNodeWidth){
			// Scrolling should happen
			return {
				min: this.isLeftToRight() ? 0 : children[children.length-1].domNode.offsetLeft,
				max: this.isLeftToRight() ?
					(children[children.length-1].domNode.offsetLeft + dojo.style(children[children.length-1].domNode, "width")) - scrollNodeWidth :
					maxPossibleScroll
			};
		}else{
			// No scrolling needed, all tabs visible, we stay either scrolled to far left or far right (depending on dir)
			var onlyScrollPosition = this.isLeftToRight() ? 0 : maxPossibleScroll;
			return {
				min: onlyScrollPosition,
				max: onlyScrollPosition
			};
		}
	},

	_getScrollForSelectedTab: function(){
		// summary:
		//		Returns the scroll value setting so that the selected tab
		//		will appear in the center
		var w = this.scrollNode,
			n = this._selectedTab,
			scrollNodeWidth = dojo.style(this.scrollNode, "width"),
			scrollBounds = this._getScrollBounds();

		// TODO: scroll minimal amount (to either right or left) so that
		// selected tab is fully visible, and just return if it's already visible?
		var pos = (n.offsetLeft + dojo.style(n, "width")/2) - scrollNodeWidth/2;
		pos = Math.min(Math.max(pos, scrollBounds.min), scrollBounds.max);

		// TODO:
		// If scrolling close to the left side or right side, scroll
		// all the way to the left or right.  See this._minScroll.
		// (But need to make sure that doesn't scroll the tab out of view...)
		return pos;
	},

	createSmoothScroll : function(x){
		// summary:
		//		Creates a dojo._Animation object that smoothly scrolls the tab list
		//		either to a fixed horizontal pixel value, or to the selected tab.
		// description:
		//		If an number argument is passed to the function, that horizontal
		//		pixel position is scrolled to.  Otherwise the currently selected
		//		tab is scrolled to.
		// x: Integer?
		//		An optional pixel value to scroll to, indicating distance from left.

		// Calculate position to scroll to
		if(arguments.length > 0){
			// position specified by caller, just make sure it's within bounds
			var scrollBounds = this._getScrollBounds();
			x = Math.min(Math.max(x, scrollBounds.min), scrollBounds.max);
		}else{
			// scroll to center the current tab
			x = this._getScrollForSelectedTab();
		}

		if(this._anim && this._anim.status() == "playing"){
			this._anim.stop();
		}

		var self = this,
			w = this.scrollNode,
			anim = new dojo._Animation({
				beforeBegin: function(){
					if(this.curve){ delete this.curve; }
					var oldS = w.scrollLeft,
						newS = self._convertToScrollLeft(x);
					anim.curve = new dojo._Line(oldS, newS);
				},
				onAnimate: function(val){
					w.scrollLeft = val;
				}
			});
		this._anim = anim;

		// Disable/enable left/right buttons according to new scroll position
		this._setButtonClass(x);

		return anim; // dojo._Animation
	},

	_getBtnNode: function(e){
		// summary:
		//		Gets a button DOM node from a mouse click event.
		// e:
		//		The mouse click event.
		var n = e.target;
		while(n && !dojo.hasClass(n, "tabStripButton")){
			n = n.parentNode;
		}
		return n;
	},

	doSlideRight: function(e){
		// summary:
		//		Scrolls the menu to the right.
		// e:
		//		The mouse click event.
		this.doSlide(1, this._getBtnNode(e));
	},

	doSlideLeft: function(e){
		// summary:
		//		Scrolls the menu to the left.
		// e:
		//		The mouse click event.
		this.doSlide(-1,this._getBtnNode(e));
	},

	doSlide: function(direction, node){
		// summary:
		//		Scrolls the tab list to the left or right by 75% of the widget width.
		// direction:
		//		If the direction is 1, the widget scrolls to the right, if it is
		//		-1, it scrolls to the left.

		if(node && dojo.hasClass(node, "dijitTabDisabled")){return;}

		var sWidth = dojo.style(this.scrollNode, "width");
		var d = (sWidth * 0.75) * direction;

		var to = this._getScroll() + d;

		this._setButtonClass(to);

		this.createSmoothScroll(to).play();
	},

	_setButtonClass: function(scroll){
		// summary:
		//		Disables the left scroll button if the tabs are scrolled all the way to the left,
		//		or the right scroll button in the opposite case.
		// scroll: Integer
		//		amount of horizontal scroll

		var scrollBounds = this._getScrollBounds();
		this._leftBtn.set("disabled", scroll <= scrollBounds.min);
		this._rightBtn.set("disabled", scroll >= scrollBounds.max);
	}
});

dojo.declare("dijit.layout._ScrollingTabControllerButton",
	dijit.form.Button,
	{
		baseClass: "dijitTab tabStripButton",

		templateString: dojo.cache("dijit.layout", "templates/_ScrollingTabControllerButton.html", "<div dojoAttachEvent=\"onclick:_onButtonClick\">\r\n\t<div waiRole=\"presentation\" class=\"dijitTabInnerDiv\" dojoattachpoint=\"innerDiv,focusNode\">\r\n\t\t<div waiRole=\"presentation\" class=\"dijitTabContent dijitButtonContents\" dojoattachpoint=\"tabContent\">\r\n\t\t\t<img waiRole=\"presentation\" alt=\"\" src=\"${_blankGif}\" class=\"dijitTabStripIcon\" dojoAttachPoint=\"iconNode\"/>\r\n\t\t\t<span dojoAttachPoint=\"containerNode,titleNode\" class=\"dijitButtonText\"></span>\r\n\t\t</div>\r\n\t</div>\r\n</div>\r\n"),

		// Override inherited tabIndex: 0 from dijit.form.Button, because user shouldn't be
		// able to tab to the left/right/menu buttons
		tabIndex: "-1"
	}
);

}

if(!dojo._hasResource["dijit.layout.TabContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.TabContainer"] = true;
dojo.provide("dijit.layout.TabContainer");





dojo.declare("dijit.layout.TabContainer",
	dijit.layout._TabContainerBase,
	{
		// summary:
		//		A Container with tabs to select each child (only one of which is displayed at a time).
		// description:
		//		A TabContainer is a container that has multiple panes, but shows only
		//		one pane at a time.  There are a set of tabs corresponding to each pane,
		//		where each tab has the name (aka title) of the pane, and optionally a close button.

		// useMenu: [const] Boolean
		//		True if a menu should be used to select tabs when they are too
		//		wide to fit the TabContainer, false otherwise.
		useMenu: true,

		// useSlider: [const] Boolean
		//		True if a slider should be used to select tabs when they are too
		//		wide to fit the TabContainer, false otherwise.
		useSlider: true,

		// controllerWidget: String
		//		An optional parameter to override the widget used to display the tab labels
		controllerWidget: "",

		_makeController: function(/*DomNode*/ srcNode){
			// summary:
			//		Instantiate tablist controller widget and return reference to it.
			//		Callback from _TabContainerBase.postCreate().
			// tags:
			//		protected extension

			var cls = this.baseClass + "-tabs" + (this.doLayout ? "" : " dijitTabNoLayout"),
				TabController = dojo.getObject(this.controllerWidget);

			return new TabController({
				id: this.id + "_tablist",
				dir: this.dir,
				lang: this.lang,
				tabPosition: this.tabPosition,
				doLayout: this.doLayout,
				containerId: this.id,
				"class": cls,
				nested: this.nested,
				useMenu: this.useMenu,
				useSlider: this.useSlider,
				tabStripClass: this.tabStrip ? this.baseClass + (this.tabStrip ? "":"No") + "Strip": null
			}, srcNode);
		},

		postMixInProperties: function(){
			this.inherited(arguments);

			// Scrolling controller only works for horizontal non-nested tabs
			if(!this.controllerWidget){
				this.controllerWidget = (this.tabPosition == "top" || this.tabPosition == "bottom") && !this.nested ?
							"dijit.layout.ScrollingTabController" : "dijit.layout.TabController";
			}
		}
});


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableOptions"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableOptions"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
/*******************************************************************************
 * CMVC Location: %W%
 * Version:       %I%
 * Last Modified: %G%
 *******************************************************************************/

dojo.provide("ibm.tivoli.tip.dijit.TIPTableOptions");





// NLS



// define the widget class
dojo.declare (
	"ibm.tivoli.tip.dijit.TIPTableOptions", 
    [ dijit._Widget, dijit._Container ],
{
    table: null,
    global: null,

	constructor: function() {
		// summary: A reusable widget for displaying a dynamic set of parameters to be 
		//  filled in at runtime.
	    console.info ("enter TIPTableOptions.constructor");
	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
	},
	postCreate: function() { 
		this.inherited ("postCreate", arguments);
		console.info ("enter TIPTableOptions:postCreate");
		this.addColumnsVisible (this.table.originalStructure.cells);
	},
	addColumnsVisible: function (columns) {
		console.info ("enter TIPTableOptions.addColumnsVisible");
		var widget = null;
		var column = null;

		for (var i in columns) {
			if (i) {
				column = columns[i];
				console.log ("got column i: ");
				console.log (column);
				if (column.name) {
					if (this.global) {
						column.name = this.global.bidiEnableLabel (column.name);
						console.log ("bidi enabled label: " + column.name);
					} else {
						console.log ("this.global not defined so not bidi enabling label: " + column.name);
					}						
				
					widget = new ibm.tivoli.tip.dijit.ColumnVisible ({
						id: this.id + "column_" + column.field,
						columnid: this.id + "column_" + column.field,
						baseid: column.field,
						checked: column.visible===undefined?!column.hidden:column.visible,  
						label: column.name,
						global: this.global
					});
					this.addChild (widget);
					console.log ("added column widget");
				} else {
					console.log ("this column was created internally by data store.  do not show it.");
				}
			}
		}		

		console.info ("exit TIPTableOptions.addColumnsVisible");
	},
	
	// add on2009/07/07 by chujing for function selectall
	
	onSelectAll: function(){
		
	var columns = this.table.originalStructure.cells;	 
		for (var i in columns) {
			if (i) {
				var columnWidget = dijit.byId (this.id + "column_" + columns[i].field);
				columnWidget.setVisible(true);	 
			}
		}
		
	},
	// add on2009/07/07 by chujing for function deselectall
	onDeselectAll:function (){
    var columns = this.table.originalStructure.cells;
		 
		for (var i in columns) {
			if (i) {
				var columnWidget = dijit.byId (this.id + "column_" + columns[i].field);
				columnWidget.setVisible(false);	 
			}
		}

	},
	onSave: function () {
		var columns = this.table.originalStructure.cells;
		var allColumns = [];
	
		for (var i in columns) {
			if (i) {
				var column = columns[i];
				var columnWidget = dijit.byId (this.id + "column_" + column.field);
				var tblColumn = this.table.getColumnById (column.field);
				column.visible = columnWidget.isVisible();
				columns[i].visible=columnWidget.isVisible();//added at 2009/04/24 for function show/hidden column
				console.log ("setting column: " + column.field + " to visible: " + column.visible);

				if (tblColumn) {
					// In order to preserve the column width, we need to set the width property
					tblColumn.width = tblColumn.unitWidth;
					// set these values on the originalStruture so they will be in memory
					// in case the dialog is opened again
					column.width = tblColumn.unitWidth;
				} else {
					tblColumn = column;
				}
				
				this.table.grid.layout.setColumnVisibility(i, columnWidget.isVisible());

				console.log ("TIPTableOptions.onSave. pushing column: ");
				console.log (column);
				allColumns.push (column);
			}
		}
	
		//this.table.savePreferences();
		this.table.originalStructure.cells = allColumns;
		console.log ("exit TIPTableOptions.onSave with originalStructure: ");
		console.log (this.table.originalStructure);
	},
	isColumnVisible: function (columnID) {
		var columnWidget = dijit.byId (this.id + "column_" + columnID);
		return (columnWidget.isVisible());
	}
	
});


// define the widget class
dojo.declare (
	"ibm.tivoli.tip.dijit.ColumnVisible", 
    [ dijit._Widget, dijit._Templated ],
{
	columnid: null,
	label: null,
	checked: false,
	global: null,
	widgetsInTemplate: true,
	templateString:"<div>\r\n\t<input dojoType=\"dijit.form.CheckBox\" dojoAttachPoint=\"checkbox\" name=\"${columnid}CB\" id=\"${columnid}CB\" value=\"\" />\r\n\t\t&nbsp;\r\n\t<label for=\"${columnid}CB\">${label}</label>\r\n</div>\r\n",
	
	constructor: function () {
		console.log ("enter ColumnsVisible.constructor");
	},
	postCreate: function () {
		console.log ("enter ColumnVisible.postCreate");
		if (this.checked) {
			this.checkbox.setChecked (this.checked);
		}
		console.log ("exit ColumnVisible.postCreate");
	},
	setVisible: function (checked) {
		this.checkbox.setChecked (checked);
	},
	isVisible: function () {
		return this.checkbox.checked;
	},
	destroy: function () {
		console.log ("enter ColumnVisible.destroy for");
		console.log (this);
		this.inherited (arguments);
		console.log ("exit ColumnVisible.destroy");
	}
	
});


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableOptionsDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableOptionsDialog"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
/*******************************************************************************
 * CMVC Location: %W%
 * Version:       %I%
 * Last Modified: %G%
 *******************************************************************************/

dojo.provide("ibm.tivoli.tip.dijit.TIPTableOptionsDialog");












// NLS



// define the widget class
dojo.declare (
	"ibm.tivoli.tip.dijit.TIPTableOptionsDialog", 
    [ dijit._Widget, dijit._Templated ],
{
	templateString:"<div>\r\n  <div dojoType=\"dijit.Dialog\" dojoAttachPoint=\"dialog\"\ttitle=\"${resources_.TABLE_PREFERENCES}\"\tstyle=\"display:none;\" >\r\n\r\n\t\t<div id=\"${id}tabs\" dojoType=\"dijit.layout.TabContainer\" class=\"dialog\" style=\"width: 550px; height: 350px;\" >\r\n\t\t\t<div dojoType=\"dijit.layout.ContentPane\" selected=\"true\" title=\"${resources_.COLUMNS}\" \r\n\t\t\t \tclass=\"dialog\" style=\"padding: 15px\">\r\n\t\t\t\t<div style=\"width: 480px; height: 280px; overflow: auto\" >\r\n\t\t\t\t \t${resources_.COLUMNS_VISIBLE}\r\n\t\t\t\t \t<br/><br/>\r\n                    <button  iconClass=\"selectAllIcon\"  dojoType=\"ibm.tivoli.tip.dijit.TIPButton\"      dojoAttachEvent=\"onClick:onSelectAll\" title=\"${resources_.SELECT_ALL}\" ></button> \r\n                    <button  iconClass=\"deselectAllIcon\"   dojoType=\"ibm.tivoli.tip.dijit.TIPButton\"      dojoAttachEvent=\"onClick:onDeselectAll\" title=\"${resources_.DESELECT_ALL}\" ></button>\r\n\t\t\t\t \t<div id=\"${id}columnsVisibleDivContainer\">\r\n\t\t\t\t \t</div>\r\n\t\t\t\t </div>\r\n\t\t\t</div>\r\n\t\t</div>\t  \r\n\r\n\t\t<div dojoType=\"dijit.Toolbar\" style=\"padding: 0px\">\r\n\t\t\t<button  \r\n\t\t\t       value=\"${resources_.OK}\"\r\n\t\t\t       dojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\t       dojoAttachEvent=\"onClick:onOK\" \r\n\t\t\t>${resources_.OK}</button>\r\n\t\t\t<button  \r\n\t\t\t\t   dojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\t\t   dojoAttachPoint=\"cancelButton\"\r\n\t\t\t\t   dojoAttachEvent=\"onClick:onCancel\"\r\n\t\t\t       value=\"${resources_.CANCEL}\"\r\n\t \t\t>${resources_.CANCEL}</button>\r\n\t \t</div>\r\n\r\n  </div>\r\n</div>\r\n",
	widgetsInTemplate: true,
	table: null,
	optionsWidget_: null,
	global: null,

	constructor: function() {
		// summary: A reusable widget for displaying a dynamic set of parameters to be 
		//  filled in at runtime.
		console.log ("enter TIPTableOptionsDialog.constructor");
	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
	},
	postCreate: function() { 
		this.inherited ("postCreate", arguments);
		console.info ("enter TIPTableOptions:postCreate");

		this.optionsWidget_=new ibm.tivoli.tip.dijit.TIPTableOptions (
			{
				table: this.table,
				global: this.global
			}, 
			dojo.byId(this.id + "columnsVisibleDivContainer")
		);
		this.optionsWidget_.startup();
		
	},
	//add 2009/07/07 by chujing for  function select all
	onSelectAll: function(){
		
	this.optionsWidget_.onSelectAll();
	},
	//add 2009/07/07 by chujing for  function deselect all
	onDeselectAll: function(){
		
	this.optionsWidget_.onDeselectAll();
	},
	
	onOK: function () {
		// TODO: persist changes
		console.log ("enter TIPOptionsDialog.onOK");
		this.optionsWidget_.onSave();
		this.hide();
	},
	onCancel: function () {
		console.log ("enter TIPOptionsDialog.onCancel");
		this.hide();
		// TODO: revert changes
	},
	show: function () {
		this.dialog.show();
		dijit.byId (this.id + "tabs").resize();
	},
	hide: function () {
		this.dialog.hide();
		setTimeout (dojo.hitch (this, this.kill), 100);
	},
	kill: function () {
		var id = this.id;
		this.destroyRecursive();
		dijit.registry.remove (id);
	},
	getTableOptions: function () {
		return (this.optionsWidget_);
	}
});



}

if(!dojo._hasResource["dijit.form.FilteringSelect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.FilteringSelect"] = true;
dojo.provide("dijit.form.FilteringSelect");



dojo.declare(
	"dijit.form.FilteringSelect",
	[dijit.form.MappedTextBox, dijit.form.ComboBoxMixin],
	{
		// summary:
		//		An enhanced version of the HTML SELECT tag, populated dynamically
		//
		// description:
		//		An enhanced version of the HTML SELECT tag, populated dynamically. It works
		//		very nicely with very large data sets because it can load and page data as needed.
		//		It also resembles ComboBox, but does not allow values outside of the provided ones.
		//		If OPTION tags are used as the data provider via markup, then the
		//		OPTION tag's child text node is used as the displayed value when selected
		//		while the OPTION tag's value attribute is used as the widget value on form submit.
		//		To set the default value when using OPTION tags, specify the selected
		//		attribute on 1 of the child OPTION tags.
		//
		//		Similar features:
		//			- There is a drop down list of possible values.
		//			- You can only enter a value from the drop down list.  (You can't
		//				enter an arbitrary value.)
		//			- The value submitted with the form is the hidden value (ex: CA),
		//				not the displayed value a.k.a. label (ex: California)
		//
		//		Enhancements over plain HTML version:
		//			- If you type in some text then it will filter down the list of
		//				possible values in the drop down list.
		//			- List can be specified either as a static list or via a javascript
		//				function (that can get the list from a server)

		_isvalid: true,

		// required: Boolean
		//		True (default) if user is required to enter a value into this field.
		required: true,

		_lastDisplayedValue: "",

		isValid: function(){
			// Overrides ValidationTextBox.isValid()
			return this._isvalid || (!this.required && this.get('displayedValue') == ""); // #5974
		},

		_refreshState: function(){
			if(!this.searchTimer){ // state will be refreshed after results are returned
				this.inherited(arguments);
			}
		},

		_callbackSetLabel: function(	/*Array*/ result,
						/*Object*/ dataObject,
						/*Boolean?*/ priorityChange){
			// summary:
			//		Callback function that dynamically sets the label of the
			//		ComboBox

			// setValue does a synchronous lookup,
			// so it calls _callbackSetLabel directly,
			// and so does not pass dataObject
			// still need to test against _lastQuery in case it came too late
			if((dataObject && dataObject.query[this.searchAttr] != this._lastQuery) || (!dataObject && result.length && this.store.getIdentity(result[0]) != this._lastQuery)){
				return;
			}
			if(!result.length){
				//#3268: do nothing on bad input
				//#3285: change CSS to indicate error
				this.valueNode.value = "";
				dijit.form.TextBox.superclass._setValueAttr.call(this, "", priorityChange || (priorityChange === undefined && !this._focused));
				this._isvalid = false;
				this.validate(this._focused);
				this.item = null;
			}else{
				this.set('item', result[0], priorityChange);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ dataObject){
			// Overrides ComboBox._openResultList()

			// #3285: tap into search callback to see if user's query resembles a match
			if(dataObject.query[this.searchAttr] != this._lastQuery){
				return;
			}
			if(this.item === undefined){ // item == undefined for keyboard search
				this._isvalid = results.length != 0 || this._maxOptions != 0; // result.length==0 && maxOptions != 0 implies the nextChoices item selected but then the datastore returned 0 more entries
				this.validate(true);
			}
			dijit.form.ComboBoxMixin.prototype._openResultList.apply(this, arguments);
		},

		_getValueAttr: function(){
			// summary:
			//		Hook for attr('value') to work.

			// don't get the textbox value but rather the previously set hidden value.
			// Use this.valueNode.value which isn't always set for other MappedTextBox widgets until blur
			return this.valueNode.value;
		},

		_getValueField: function(){
			// Overrides ComboBox._getValueField()
			return "value";
		},

		_setValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so attr('value', value) works.
			// description:
			//		Sets the value of the select.
			//		Also sets the label to the corresponding value by reverse lookup.
			if(!this._onChangeActive){ priorityChange = null; }
			this._lastQuery = value;

			if(value === null || value === ''){
				this._setDisplayedValueAttr('', priorityChange);
				return;
			}

			//#3347: fetchItemByIdentity if no keyAttr specified
			var self = this;
			this.store.fetchItemByIdentity({
				identity: value,
				onItem: function(item){
					self._callbackSetLabel(item? [item] : [], undefined, priorityChange);
				}
			});
		},

		_setItemAttr: function(/*item*/ item, /*Boolean?*/ priorityChange, /*String?*/ displayedValue){
			// summary:
			//		Set the displayed valued in the input box, and the hidden value
			//		that gets submitted, based on a dojo.data store item.
			// description:
			//		Users shouldn't call this function; they should be calling
			//		attr('item', value)
			// tags:
			//		private
			this._isvalid = true;
			this.inherited(arguments);
			this.valueNode.value = this.value;
			this._lastDisplayedValue = this.textbox.value;
		},

		_getDisplayQueryString: function(/*String*/ text){
			return text.replace(/([\\\*\?])/g, "\\$1");
		},

		_setDisplayedValueAttr: function(/*String*/ label, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so attr('displayedValue', label) works.
			// description:
			//		Sets textbox to display label. Also performs reverse lookup
			//		to set the hidden value.

			// When this is called during initialization it'll ping the datastore
			// for reverse lookup, and when that completes (after an XHR request)
			// will call setValueAttr()... but that shouldn't trigger an onChange()
			// event, even when it happens after creation has finished
			if(!this._created){
				priorityChange = false;
			}

			if(this.store){
				this._hideResultList();
				var query = dojo.clone(this.query); // #6196: populate query with user-specifics
				// escape meta characters of dojo.data.util.filter.patternToRegExp().
				this._lastQuery = query[this.searchAttr] = this._getDisplayQueryString(label);
				// if the label is not valid, the callback will never set it,
				// so the last valid value will get the warning textbox set the
				// textbox value now so that the impending warning will make
				// sense to the user
				this.textbox.value = label;
				this._lastDisplayedValue = label;
				var _this = this;
				var fetch = {
					query: query,
					queryOptions: {
						ignoreCase: this.ignoreCase,
						deep: true
					},
					onComplete: function(result, dataObject){
						_this._fetchHandle = null;
						dojo.hitch(_this, "_callbackSetLabel")(result, dataObject, priorityChange);
					},
					onError: function(errText){
						_this._fetchHandle = null;
						console.error('dijit.form.FilteringSelect: ' + errText);
						dojo.hitch(_this, "_callbackSetLabel")([], undefined, false);
					}
				};
				dojo.mixin(fetch, this.fetchProperties);
				this._fetchHandle = this.store.fetch(fetch);
			}
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			this._isvalid = !this.required;
		},

		undo: function(){
			this.set('displayedValue', this._lastDisplayedValue);
		}
	}
);

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dojo.RESTErrorHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dojo.RESTErrorHandler"] = true;
dojo.provide("ibm.tivoli.tip.charts.dojo.RESTErrorHandler");
dojo.declare("ibm.tivoli.tip.charts.dojo.RESTErrorHandler", null, {
        global: null,
        exeRegExp: "[A-Z]{4,5}[0-9]{4}[A-Z]",        
		constructor: function(params){
			this.global = params;
        },
        showError: function(error){        	
        	var busyPane = dijit.byId(this.global.namespace+"busy_pane");
    		if ( busyPane ){
    			busyPane.hide();
    		}
        	try{
        		var msgKW = null;
        		var exeClassName = error.msgId;
        		var exeErrMess = error.msgText;
        		var parRes = new RegExp(this.exeRegExp).exec(exeErrMess);        		
        		if(parRes){
        			var errMess = {
        					geEXE: exeErrMess.substring(parRes.index+parRes[0].length+2,exeErrMess.indexOf(".")+1),
        					exc: error.msgId,
            				excMsg: exeErrMess.slice(exeErrMess.indexOf(".")+1)
            		};
        			msgKW = { 
                			messageId : exeErrMess.substr(parRes.index,parRes[0].length),
                            message   : this.global.tipnls( "SERVER_ERROR_MSG", errMess )
                    };        			
        		}else{
        			var errMess = {
            				exc: error.msgId,
            				excMsg: error.msgText
            		};
            		var msgKW = { 
                			messageId : this.global.tipnls( "BIND_ERROR_ID" ),
                            message   : this.global.tipnls( "BIND_ERROR_MSG", errMess )
                    };
        		}
            	this.global.showErrorMessage( msgKW );
        	}catch(e){
        		//alert(e.message);
        		console.error(e.message);
        	}        	
        } 
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTable"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007, 2010 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/





dojo.require("dijit.layout.ContentPane" );



//








dojo.require ("ibm.tivoli.tip.charts.dojo.RESTErrorHandler");


// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTable");

/******************************************************************************
* This container places the search bar and toolbar around a given 
* dojox.grid.Grid 
*
* Authors: Joseph Firebaugh, Joshua Allen, Ken Parzygnat
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTable",
			 dijit.layout.BorderContainer,
{
	grid: null,
	gridParms: null,
	global: null,
	requestTimeout: 30000,
	defaultRowsPerPage: 25,
	tableID: null,
	autoSavePreferences : false,
    enableToolbar: true, // Refer defect 889 on csNext. This will hide the toolbar if set to false. The footer will still be visible.
	toolbarHidden_ : false,
	/* Do you want the default buttons added to the toolbar? */
	makeDefaultButtons: true,
	
	/* Table Options is the ability to hide/show columns.  
	   Preferences are handled via the infrastructure, however
	   they must persisted by the implementing application. 
	   Hence, the table options are disabled by default.  */
	tableOptionsEnabled: false,	
	multiSelect: true,
	rowDblClick: null,
	
	// open: Boolean
	//    Whether toolbar is opened or closed on first render.
	open: true,
    shownub : true, // for showing toolbar nub
	nubhover :false, // for hovering the toolbar nub
	// duration: Integer
	//    Time in milliseconds to wipe in/wipe out toolbar
	duration: 300,
	
	_contextMenu: null,
   	_contextMenuAdapter : null,
	cmProvider: null,
	clientRfcItems : null,      // Provide if you are providing above view structure.
	view: null,
	viewData: null,
	applicationID: null,
	widgetID: null,
	tableParams: null,
	
	/*contentIsolation: boolean variable to enable/disable Context Menu in Content Isolation mode*/
	contentIsolation: false,
  
	constructor: function () {
		this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");	
		this.lastPreferences = null;
		this.viewLoaded = false;
		this.tooltips = [];
		this.connections = [];
		this.topics = [];
		this.tableOptionsDialog = null;
		this.originalStructure = null;
		this.toolbar = null;
		this.footerToolbar = null;
		this.filterBar = null;
	},
	
	/**************************************************************************
	* Called after template has been processed.
	*
	* Lets check to see if a dojox.grid.Grid was passed in on params.  If so,
	* add it to the DOM tree and setup.
	**************************************************************************/
	postCreate: function ()	{
		//call super
		this.inherited( "postCreate", arguments );
		
		this.domNode.style.padding = "0px";

		//console.log ("global: " + this.global);
		this.gridParms = this.initializeGridParameters();

        if(this.enableToolbar)
        {
    		this.cPane1 = new dijit.layout.ContentPane( { region: "top" } );
          	this.addChild( this.cPane1 );
                  
          	var div = dojo.doc.createElement( 'div' );
          	this.cPane1.attr( "content", div );
        }
		
  
      //-----------------------------
      // Add Toolbar and filter area
      //-----------------------------
      	if(this.enableToolbar && (!this.tableBuilder || !this.tableBuilder.widgetParams || !this.tableBuilder.widgetParams.standalone ||(this.tableBuilder.widgetParams.standalone && !this.tableBuilder.widgetParams.ignoreToolbar))){
      		this.filterBar = new ibm.tivoli.tip.dijit.TIPTableToolbar( {shownub:this.shownub,nubhover:this.nubhover}, div );
            this.toolbar = this.filterBar.toolbar;      
            this.searchField = this.filterBar.searchField;
            if ( this.makeDefaultButtons === true ) { 
       	    	this.initializeToolbar(); 
            }             
            this.topics.push (dojo.subscribe (this.searchField.id + "searchInvoked", this, "onSearch"));
      	}      
		
		//-----------------------------
      	// Create Grid Pane            
      	//-----------------------------    
	   this.gridPane = new dijit.layout.ContentPane( { region: "center" } );
	   this.gridPane.style.margin="0px";
          
     	this.addChild( this.gridPane );
      
      	this.gridDiv = dojo.doc.createElement( 'div' );
      	dojo.addClass (this.gridDiv, "grid");
      	this.gridPane.attr( "content", this.gridDiv );      

//        this.connections.push (dojo.connect( this.gridPane, "resize", this, "resizeGrid"  ));
      
      //-----------------------------
      // Create and add footer            
      //-----------------------------    
      this.cPane2 = new dijit.layout.ContentPane( { region: "bottom" } );
      this.addChild( this.cPane2 );
  
      div   = dojo.doc.createElement( 'div' );
      this.cPane2.attr( "content", div );
  
      this.footerToolbar = new ibm.tivoli.tip.dijit.TIPTableFooter( {}, div );
      //console.log ("made footer: ");
      //console.log (this.footerToolbar);        
      
      
            		
      //if grid passed on params then set it
      if ( this.grid !== null )
      {
      	this.setGrid( this.grid );
      }
       
	  this.topics.push (dojo.subscribe ("/ibm.tip/TIPQueryReadStore#rowCountUpdated", this, "updateFooter"));
	},
	initializeGridParameters: function () {
		if (this.gridParms === null) {
			this.gridParms = {};
		}
		
		if (this.gridParms.store === undefined) {
			this.gridParms.store = new ibm.tivoli.tip.dojo.TIPQueryReadStore (
				{ global: this.global, tableID: this.tableID, jsId: "jsonStore", doClientPaging:false, 
				  url:this.global.controllerURL,
				  requestTimeout: this.requestTimeout 
				});
		} else {
			console.info ("this.gridParms.store is already defined");
			console.log (this.gridParms.store);
		}
			
		if (this.multiSelect) {
		} else {
			this.gridParms.selectionMode = "single";
		}

		if (this.gridParms.rowsPerPage === undefined) {
			this.gridParms.rowsPerPage=this.defaultRowsPerPage;
		}
		if (this.gridParms.rowSelector === undefined) {
			this.gridParms.rowSelector = "20px";
		}
		//change value of this.gridParms.columnReordering to true on 2009/07/09 for column reordering
		if (this.gridParms.columnReordering === undefined) {
			this.gridParms.columnReordering = true;
		}
		
		var cb = dojo.hitch(this,function(){  
			console.debug("----- Inside the load complete callback ----");
			dojo.publish (this.id + "#LoadComplete", [ "LoadComplete" ]);
        });
        this.gridParms.loadCompleteCallback = cb;
        if (this.global.bidiTxtDir) {
        	if ((this.global.bidiTxtDir == "RTL") || (this.global.bidiTxtDir == "CONTEXTUAL")) {
				this.gridParms.dir = "rtl";
			} else if (this.global.bidiTxtDir == "LTR") {
				this.gridParms.dir = "ltr";
			}
		}
			
		if (this.global.bidiOrientation) {	
			if (this.global.bidiOrientation == "RTL") {
				this.gridParms.layoutAlign = "right";
			} else if (this.global.bidiOrientation == "LTR") {
				this.gridParms.layoutAlign = "left";
			}
		}

		return (this.gridParms);
	},
	startup: function() {
		console.info ("enter TIPTable.startup");
		if (!this.grid) {
			this.makeGrid();
			if(this.filterBar && this.shownub){
				this.connect (this.filterBar.nub, "onclick", "toggleToolbar");
				this.connect (this.filterBar.nub, "onkeydown", "toggleToolbarKeydown");
			}
		}

		if (this.autoSavePreferences && (!this.tableBuilder || (!this.tableBuilder.widgetParams || !this.tableBuilder.widgetParams.standalone))) {
			this.connect (this.grid, "_resize", "onResize");
			//console.log ("connected tiptable resize event");
		}
		
		this.connect (this.grid, "onRowClick", "onRowClick");
		this.connect (this.grid, "onMoveColumn", "saveDOJOPreferences");
		/*
		if(this.tableBuilder && (!this.tableBuilder.widgetParams || !this.tableBuilder.widgetParams.standalone)){
			this.connect (this.grid, "prerender", "savePagePref");
		}
		*/
		console.info ("exit TIPTable.startup");
	},
	
	parseDataGrid: function(){		
		dojo.parser.parse(this.grid.domNode);		
    },
	
	makeGrid: function () {
		console.log ("enter makeGrid");
		var grid = new ibm.tivoli.tip.dijit.TIPDataGrid (this.gridParms, this.gridDiv);
		this.setGrid( grid , true );
		this.connect(grid,"_resize","parseDataGrid");
		this.connect(grid,"onFetchError",dojo.hitch (this, function (error) {										
			var errHandler = new ibm.tivoli.tip.charts.dojo.RESTErrorHandler(this.global);		
			errHandler.showError(dojo.fromJson(error.responseText));	
		}));
		grid.startup();
		/*
		if (this.widgetID !== null) {
		// TODO: We still need to set the widgetID a different way
//			this.query = {};
//			this.query.widgetID = this.widgetID;
//			this.query.applicationID = this.applicationID;
//			this.grid.setQuery (this.query);
			this.query = {
	                navModel: {	                        
	            		itemTypes: ['item'],
	            		parameters: {
							widgetID: this.widgetID
						}
					}
			};
			if(this.tableParams != null){
				for (var row in this.tableParams) {
					if (row) {
					 var param = this.tableParams[row];
					 this.query.navModel.parameters[param.id] = param.value;						
					}
				}
			}
			console.log("Params passed to table");
			console.log(this.query);
			this.grid.setQuery (this.query);
		}
		*/
		if ((this.view === undefined) || (this.view === null)) {
			console.log ("in TIPTable.makeGrid.  this.view is null so fetching from server");
			this.getView();
		} else {
			console.log ("in TIPTable.makeGrid.  this.view is already defined so using it:");
			this.originalStructure = this.viewData;
			console.log (this.originalStructure);
			this.setupView(this.view, this.viewData);
		}
		
		console.log ("exit makeGrid");
	},
	setGrid: function( grid ) {
		this.grid = grid;
		if(this.tableBuilder && this.searchBuild){
			console.log ("calling TIPTable.searchField.setTable");			
			this.searchField.setTable( this.grid, this.tableBuilder.widgetDefinition, this.global );
		}else if(this.searchBuild){
			console.log ("calling TIPTable.searchField.setTable w/o widgetDef");
			this.searchField.setTable( grid, undefined, this.global );
		}		
	},
	getView: function () {
        var kw = {
                    content:   {
                                  command: "getTableColumns",
                                  arg_fqTableId: this.tableID,
                                  arg_portletNamespace: this.global.namespace
                               },
                    timeout:   this.requestTimeout,
                    onResponse:{
                                  callback: dojo.hitch(this, this.getViewCallback)
                               }
                 };
	  	
	  	if (this.tableArgs) {
	  		for (var key in this.tableArgs) {
	  			if (key) {
		            kw.content[key] = this.tableArgs[key];
	  			}	
	  		}
	  	}
	  	
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
	},
	getViewCallback: function (data, ioArgs) {
		this.originalStructure = data;
		var cell = null;
		var cells = [];
		
  		var view1 = { cells: [ data.cells ] };
		// a grid layout is an array of views.
		var layout = [ view1 ];
		this.setupView (layout, data);
	},
	setupView: function (structure, data) {
		console.log ("enter setupView");
//		this.grid.attr('structure', structure);
		this.grid.attr( "structure", structure );
		var cell = null;
		for (var i in data.cells) {
			if (i) {
				cell = data.cells[i];
				if (cell.hidden) {
					this.grid.layout.setColumnVisibility(i, false);
				}
			}
		}
		if(this.tableBuilder && this.searchField){
			console.log ("calling TIPTable.searchField.setTable");			
			this.searchField.setTable( this.grid, this.tableBuilder.widgetDefinition, this.global );
		}else if(this.searchField){
			console.log ("calling TIPTable.searchField.setTable w/o widgetDef");
			this.searchField.setTable( this.grid, undefined, this.global );
		}		
 	    this.afterViewLoad(data.rcfItems);
	},
	afterViewLoad: function (rcfItems) {
		console.log ("Enter TIPTable.afterViewLoad");
		if (this.cmProvider) {
			this.configureContextMenu(rcfItems);  			
	  	}
	  		  	
	  	this.viewLoaded = true;
	},
	configureContextMenu: function (rcfItems) {
		if ((this.cmProvider.parentid === null) || (this.cmProvider.parentid === "")) {
			if ((this.tableBuilder !== undefined) && (this.tableBuilder != null)) {
				this.cmProvider.setParentId (this.tableBuilder.id);
			} else {
				this.cmProvider.setParentId (this.id);
			}
		}
		if ((this.cmProvider.rcfItems === null) && (rcfItems)) {
			this.cmProvider.rcfItems = rcfItems;
		}
		if(!this.tableBuilder || (this.tableBuilder && (!this.tableBuilder.widgetParams || !this.tableBuilder.widgetParams.standalone ||(this.tableBuilder.widgetParams.standalone && !this.tableBuilder.widgetParams.ignoreContextMenu)))){
			this.connect( this.grid, "onRowContextMenu", "_onRowContextMenu" );
		}
      	 //FIXES CONTEXT MENU BUG with selected table rows IN DOJO 1.0.2
		 this.grid.focus.styleRow = function(inRow){};
      
	},
    getContextMenuAdapter: function()
    {
      if (this._contextMenuAdapter === null) {
         // Create the contextMenuAdapter.
         this._contextMenuAdapter = new ibm.tivoli.tip.dijit.TIPContextMenu({cmProvider: this.cmProvider, cmClientProvider: this.cmClientProvider});
      }
      return this._contextMenuAdapter;
    },
    /************************************************************************
    * This method is called each time a context menu is requred on a row
    ************************************************************************/
    _onRowContextMenu: function( e )
    { 
//    	console.log ("enter onrowcontextmenu");
      var row = e.grid.getItem( e.rowIndex );
      
      console.log ("row is: " + row);
      console.log ("e.rowIndex: " + e.rowIndex);
//      console.log ("grid: ");
//      console.log (e.grid);
      if ( row )
      {
        //select the row that the context menu was initiated on
        e.grid.selection.deselectAll( e.rowIndex );
        e.grid.selection.setSelected( e.rowIndex, true );
                
        //tell the context menu adapter to show the menu
        // no context is provided because we overload the _loadMenuItems method
//        var selection = this.getSelectionDataV2();
//        console.log ("selection data v2: ");
//        console.log (selection);
        if(this.tableBuilder){
        	this.getContextMenuAdapter().contextMenuOpen( e, this.getDOJOTableSelectedDataV2(), this.contentIsolation);
        }else{
            var selection = this.getSelectionData();
            console.log ("selection data v1: ");
            console.log (selection);
        	this.getContextMenuAdapter().contextMenuOpen( e, selection, this.contentIsolation );
        }        
      }
    },
	getSelectedRows: function () {
		return this.grid.selection.getSelected();
	},
	getAllColumnIndexes: function () {
//		console.log ("enter getAllColumnIndexes");
//		var count = this.grid.attr ('structure')[0].cells.length;
//		var count = this.grid.structure[0].cells.length;
		var count = this.originalStructure.cells.length;
		var columnIDs = [];
		for (var i=0; i<count; i++) {
			columnIDs.push (i);
		}
		
		return (columnIDs);
	},
	getValueAt: function (rowIndex, columnIndex) {
		var row = rowIndex.i;
//		console.log ("row: ");
//		console.log (row);
		if(row === undefined){
		row = rowIndex;
		}
		var value = row[columnIndex];
//		console.log ("value at row: " + rowIndex + ", column: " + columnIndex + " is: " + value);
		return (value); 
	},
	getModelData: function () {
		var selectedRows = this.getSelectedRows ();
		var row = selectedRows[0];
		var columnIDs = this.getAllColumnIndexes();

		for (var column in columnIDs) {
			if (column) {
				var data = this.getValueAt (row, column);
				//console.log ("cell value for row: " + row + ", and column: " + column + " : " + data);
			}
		}
	},
	getIdentifier: function () {
		return (this.originalStructure.cells.length);
	},
	
	getDOJOTableSelectedDataV2: function () {		
		var contextData = {};
		contextData.rcfs = [];
		var colNames = this.tableBuilder.widgetDefinition.datasets[0].columns;
		var datasourceType = this.tableBuilder.widgetDefinition.datasource.type;
		var datasourceId = this.tableBuilder.datasourceID;		
		var datasetId = this.tableBuilder.widgetDefinition.datasets[0].id;
		var rowIndex = null;
		var sourceToken = null;
		var selectedRows = this.getSelectedRows();
		dojo.forEach (selectedRows,	function (row) {				
			var resourceAttrs = [];							
			for(var i=0; i<colNames.length; i++){
				var attr = {};
				attr.name = colNames[i].label.key;								
				attr.value = this.tableBuilder.jsonStore.getValue (row, colNames[i].id);		
				attr.isNamingAttribute = false;
				resourceAttrs.push(attr);
			}			
			rowIndex = this.tableBuilder.jsonStore.getIdentity(row);
			sourceToken = this.tableBuilder.jsonStore.getValue (row, ""+colNames.length);
			var attr = {};
			attr.name = "Class";
			attr.value = "*";
			attr.isNamingAttribute = false;
			resourceAttrs.push(attr);
			contextData.rcfs.push (resourceAttrs);
			
		}, this);		
		
		contextData.launchTypes = ["WEB_URL", "PORTAL_PAGE", "PORTLET", "CLIENT_EVENT"];
		contextData.excludedAppIDs = [];
		contextData.substitutionVariables = [];		
		var attr = {};
		attr.name = "RowIndex";
		attr.value = rowIndex;		
		contextData.substitutionVariables.push(attr);
		
		var attr = {};
		attr.name = "SourceToken";
		attr.value = sourceToken;
		if(attr.value!==undefined && attr.value!==null){
			contextData.substitutionVariables.push(attr);
		}
		
		var attr = {};
		attr.name = "DatasourceType";
		attr.value = datasourceType;	
		contextData.substitutionVariables.push(attr);
		
		var attr = {};
		attr.name = "DatasourceId";
		attr.value = datasourceId;	
		contextData.substitutionVariables.push(attr);	
		
		var attr = {};
		attr.name = "DatasetId";
		attr.value = datasetId;	
		contextData.substitutionVariables.push(attr);
		contextData.options = [];		
		return contextData;		
	},
	
	getSelectionDataV2: function () {						
		var contextData = {};
		contextData.rcfs = [];		
		var identifier = this.getIdentifier();
		var columnIDs = this.getAllColumnIndexes();
		var selectedRows = this.getSelectedRows();
		console.log ("selectedRows: ");
		console.log (selectedRows);
				
		dojo.forEach (selectedRows,
			function (row) {				
				var resourceAttrs = [];					
				for (var column in columnIDs) {
					if (column) {
						var cellValue = this.getValueAt (row, column);
						var attr = {};
						attr.name = column;
						attr.value = cellValue;
						attr.isNamingAttribute = "false";
						resourceAttrs.push(attr);
					}
				}
				contextData.rcfs.push (resourceAttrs);
			}, this);
		contextData.options = [];
		console.log ("contextData: ");
		console.log (contextData);
		//contextData.options.push ({name:"menuName",value:"listConnectionsMenu"});
		return contextData;		
	},
	
	getSelectionData: function () {		
		//this.getModelData();
		var selectedRows = this.getSelectedRows ();
//		console.log ("selectedRows: " + selectedRows);

		var rowData = null;
		var columnIDs = this.getAllColumnIndexes();
		var identifier = this.getIdentifier();
		var rowJSON = {};
		
		dojo.forEach (selectedRows,
			function (row) {
				var firstTime = true;
				rowJSON.id = this.getValueAt (row, identifier);
				rowJSON[identifier] = this.getValueAt (row, identifier);
				for (var column in columnIDs) {
					if (column) {
//						console.log ("getting cell value for row: " + row + " and column: " + column);
						var cellValue = this.getValueAt (row, column);
						rowJSON[column] = cellValue;
					}
				}
			}, this);

		
		var columnsArray = [];
		var columnJSON = {};
		var firstLoop = true;
		for (var c in columnIDs) {
			if (c) {
				columnJSON = {};
				columnJSON.field = c;
				columnJSON.name = this.originalStructure.cells[c].name;
				columnJSON.isNamingAttribute = this.isNamingAttribute(c).toString();
				columnsArray.push (columnJSON);
			}
		}
		columnsArray.push({
			field: identifier,
			isNamingAttribute: false
		});
		
		var result = { selectedData: { columns: columnsArray, rows: rowJSON }}; 
		
//		console.log ("resultStr: " + resultStr);

		
//		var sample = { 
//		columns: [{ field: "srcToken", hidden: true },
//				  { field: "ManagedSystemName", label: "Managed System Name" },
//				  { field: "column_2", label: "baz" }],
//		
//		 rows: { srcToken: "1234", ManagedSystemName: "bar", column_2: "bang" }
//		 		};
		
		return (result);
	},
	isNamingAttribute: function (field) {
//		console.log ("enter isNamingAttribute");
//		var columns = this.grid.attr('structure')[0].cells[0];
//	  	var columns = this.grid.structure[0].cells[0];
		var columns = this.originalStructure.cells;
	  	if (columns[field]) {
		  	return columns[field].isNamingAttribute;
		} else {
			return (false);
		}
	},
	resize: function (size) {
		console.log ("enter TIPTable.resize: ");
		console.log (size);
		  
	  	if ((size) && (this.grid)) {
		  // remove height of footer
		  if (this.footerToolbar) {
		  	var coords = dojo.coords (this.footerToolbar.domNode);
		  	size.h = size.h - coords.h;
		  }
		  // remove height of toolbar
		  if (this.filterBar) {
		  	// the following line causes error in tip charts.  Please fix before uncommenting.
		    // this.filterBar.resetNubPos();
			coords = dojo.coords (this.filterBar.domNode);
			console.log ("removing height of table toolbar: " + coords.h);
			size.h = size.h - coords.h;
		  }
		  //Commenting the Extra Room changes as there is some extra padding in latest driver
		  // remove some extra room to make things fit better
		  //size.h = size.h - 15;
		  //size.w = size.w - 10;
		  
		  this.grid.autoHeight = false;
		  this.grid.autoWidth = false;
		  var	w = Number(size.w);
		  var h = Number(size.h);
		
//		  console.log ("calling ddojo.contentBox w/ w:: " + w + ", h: " + h);
//		  dojo.contentBox(this.grid.domNode, {w: w, h: h});
		  var updatedSize = { w: w, h: h};
		  this.grid.resize(updatedSize);
		  this.grid.update();
		  //add on 2009/07/09 for reorder columns because autosave is  true, don't need to make ajax request to server ,if auto save is false can try the following method
//		  if(this.gridParms.columnReordering == true){			   
//			  if(this.grid.layout!==null&&this.grid.structure!==null)
//			   { 
//			     pref=this.getPreferences();
//			    this.originalStructure=this.grid.layout; 
//				  if(pref!==null&&pref!==this.getPreferences()){
//			      this.savePreferences();
//	 			
//			   }  
//			  }
//			}
		  if(this.gridParms.columnReordering === true){			
			   
			  if(this.grid.layout!==null&&this.grid.structure!==null)
			   { 
			    this.originalStructure=this.grid.layout;  
			    
			  }
			}
	  	}

		if (this.filterBar) {
			try {
				this.filterBar.resetNubPos();
			} catch (e) {
				console.warn ("TIPTable.resize: error while resetting nub position:");
				console.warn (e);
			}
		}
	  	
	  	console.log ("exit TIPTable.resize");
    },
	onResize: function () {
//		console.log ("enter TIPTable.onResize");
		//var preferences = this.getPreferences();
		if (this.autoSavePreferences && this.viewLoaded && (!this.widgetParams || !this.widgetParams.standalone)) {
			this.savePreferences();
		} else {
//			console.log ("not saving preferences.  autoSavePreferences: " + this.autoSavePreferences + ", viewLoaded: " + this.viewLoaded);
		}		
	},
	
	savePagePref: function () {		
		if(this.tableBuilder){
			this.tableBuilder.saveUserPref();
		}
	},
	
	saveDOJOPreferences: function () {	
		if(this.tableBuilder){
			this.builder = this.tableBuilder;		
			var encodedCID = encodeURIComponent(this.widgetID);		
			var encodedProviderID = encodeURIComponent(this.builder.providerID);		
			var encodedDS = encodeURIComponent(this.builder.datasourceID);		
			var encodedPID = encodeURIComponent(this.global.chartProfileID);		
			//var widgetDefVersion = "2.1.0";
			var widgetDefVersion = this.tableBuilder.widgetDefinition.meta.version;
			var widgetSchema = this.tableBuilder.widgetDefinition.meta.type;
			var controller = "/ibm/tivoli/rest";
			var prefs = dojo.toJson(this.getDOJOPreferences());		
	        var kw = {
	        		sync: true,
	        		headers: { 'Content-Type': 'application/json',
	        			  	   'Content-Length': prefs.length
	        		},
	        		url: controller + "/providers/" + encodedProviderID + "/datasources/" + encodedDS + "/widgets/" + widgetSchema + "/definitions/" + encodedCID + "-" + widgetDefVersion + "?type=user&profileID=" + this.global.chartProfileID + "&registryLocation=local",
	                putData: prefs
	         };          
	        dojo.xhrPut (kw);		
    	}		
	},
	
	getDOJOPreferences: function () {
		var prefs = { 
			"datasetsIdentifier":"id",
				"datasets": [{
					"id": this.tableBuilder.dataset.id					
				}],	
			"tablesIdentifier": "id",
			"tables": [ {
				"id": this.widgetID			     
			}]
		};
		
//		prefs.datasets[0].columns = [];
//		prefs.datasets[0].columnIdentifier = "id";
//		if(this.originalStructure){
//			var columns = this.originalStructure.cells;			
//			for (var x in columns) {
//				if (x) {
//					prefs.datasets[0].columns[x] = {};
//					prefs.datasets[0].columns[x].id = ""+x;	
//					prefs.datasets[0].columns[x].label = {"value": columns[x].name };				
//					prefs.datasets[0].columns[x].type = this.builder.widgetDefinition.datasets[0].columns[columns[x].field].type;
//				}
//			}
//		}
		
		prefs.tables[0].view = [];
		// We want to replace the whole view with an array in the correct order
		// So do NOT use viewIdentifier.  That would replace individual elements
		// and keep the order
//		prefs.tables[0].viewIdentifier = "id";
	
		if (this.originalStructure) {
		  	var columns = this.originalStructure.cells;
			for (var x in columns) {
				if (x) {
					prefs.tables[0].view[x] = {};
					prefs.tables[0].view[x].id = columns[x].field;
					prefs.tables[0].view[x].width = columns[x].width;
					prefs.tables[0].view[x].hidden = columns[x].hidden;
					prefs.tables[0].view[x].searchable = columns[x].isSearchable;
				}
			}			
		}
		
		return (prefs);
	},
	
	savePreferences: function () {
//		console.log ("enter TIPTable.savePreferences");
		// make sure that the grid has structure already.
		var preferences = this.getPreferences();
		
		if ((preferences !== null) && (preferences !== this.lastPreferences)) {
		    var kw = {
                      content:   {
                                    command: "saveTablePreferences",
                                    arg_fqTableId: this.tableID,
                                    arg_portletNamespace: this.global.namespace,
                                    arg_preferences: preferences
                                 },
                      timeout:   this.requestTimeout
                   };

   	      ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
          this.lastPreferences = preferences;
       }
//       console.log ("exit TIPTable.savePreferences");
	},
	getPreferences: function () {
//		console.log ("enter TIPTable.getPreferences");
		var preferences = null;
		
		if (this.originalStructure) {
			//var ids = this.getViewColumnsIDs();
		  	var columns = this.originalStructure.cells;
			preferences = '{ "columns" : [ ';
		
			for (var x in columns) {
				if (x) {
					var visible = columns[x].hidden === undefined ? false : !columns[x].hidden;
					if (this.tableOptionsEnabled && this.tableOptionsDialog) {
						visible = this.tableOptionsDialog.getTableOptions().isColumnVisible(columns[x].field);
					}
					
					if (visible) {
						visible = "t";
					} else {
						visible = "f";
					}
					
					var viewIndex = this.getViewColumnIndex (columns[x].field);
					var width = columns[x].width;
					if (viewIndex !== null) {
						if ((this.grid) && (this.grid.getCell(viewIndex))) {
							width = this.grid.getCell(viewIndex).unitWidth;
						}
					} else {
						console.log ("no view index for column.field: " + columns[x].field);
					}
						
					preferences += ' {"id": "' + columns[x].field + '", "width":"' + width + '", "v":"' + visible + '"} ';
					
					if (x < columns.length - 1) {
						preferences += ", ";
					}
				}
			}
			preferences += " ] }";
//			console.log ("cell widths: ");
//			console.log (preferences);
		} else {
			console.log ("grid has no structure yet");
		}
				
		return (preferences);
	},
	getViewColumnIDs: function () {
//		console.log ("enter getViewColumnIDs");
//		console.log (this.originalStructure);
		//var columns = this.grid.attr('structure')[0].cells[0];
//	  	var columns = this.grid.structure[0].cells[0];
		var columns = this.originalStructure.cells;
	  	var columnIDs = [];
	  	for ( var i in columns ) {
	  		if (i) {
				columnIDs.push (columns[i].field);
			}
		}
		
		return (columnIDs);
	},
	getViewColumnIndex: function (id) {
		var viewColumnIDs = this.getViewColumnIDs();
		for (var i in viewColumnIDs) {
			if (i) {
				if (id == viewColumnIDs[i]) {
					return (i);
				}
			}
		}
		
		return (null);
	},
	/**************************************************************************
	* Initialize the toolbar with default buttons (like refresh)
	**************************************************************************/
	initializeToolbar: function() 
	{
		this._allToolbarButtons = [];
		
		var button = new ibm.tivoli.tip.dijit.TIPTableRefreshButton(
			{	id: this.id + "refreshButton",
			 	onClick: dojo.hitch (this, this.onRefresh) 
			});
		this.toolbar.addChild(button);
		this._allToolbarButtons.push( button );
				
		if (this.multiSelect) {
			this.toolbar.addChild (new dijit.ToolbarSeparator());
		
			button = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.resources_.SELECT_ALL, 
					showLabel: false, 
					iconClass: "selectAllIcon",
				 	onClick: dojo.hitch (this, this.selectAll) 
				});
			this.toolbar.addChild(button);
			this._allToolbarButtons.push( button );
	
			button = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.resources_.DESELECT_ALL, 
					showLabel: false, 
					iconClass: "deselectAllIcon",
				 	onClick: dojo.hitch (this, this.deselectAll) 
				});
			this.toolbar.addChild(button);
			this._allToolbarButtons.push( button );
		}
		
		if (this.tableOptionsEnabled) {
			console.log ("enabling table options");
			this.toolbar.addChild (new dijit.ToolbarSeparator());
		
			button = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.resources_.TABLE_PREFERENCES, 
					showLabel: false, 
					iconClass: "tableOptionsIcon",
				 	onClick: dojo.hitch (this, this.showTableOptions) 
				});
			this.toolbar.addChild(button);
			this._allToolbarButtons.push( button );
		} else {
			console.log ("NOT enabling table options");
		}
		
	    if (this.toolbarHidden_) {
			this.toggleToolbarOff();
		} else {
			this.toggleToolbarOn();
		}
	},
	selectAll: function () 
	{
		var count = this.grid.getTotalRows();
		for (var i=0; i<count; i++) {
			this.grid.selection.addToSelection(i);
		}
		
		this.updateFooter (this.gridParms.store.id);
	},
	deselectAll: function () 
	{
		this.grid.selection.deselectAll();
		this.updateFooter (this.gridParms.store.id);
	},
	onRefresh: function() {
        // What is going on here?
        // Previously, the preceding three lines were used in this function to refresh
        // the table.  Create a new model, set the model, and issue the event.  However,
        // the problem is that TIPTable gets its data from the server, so the new model
        // was set prior to it having any data.  This resulted in the table 
        // "flashing".  That is the table would go all blank, then turn around and fill
        // with the data from the server.  This didn't look good, so the below code fixes
        // this.  A callback was added to the store so that when it finishes handling
        // data from the server, it will call the callback.  This enables us to first 
        // create the model, and load its first page, then swap it in to the table.
        
        // Create the callback function that will swap in the new model when it has some data
        var cb = dojo.hitch(this,function(){  console.debug("----- Inside the refresh callback ----");
                                              this.grid.setLoadCompleteCallback(null);
                                              // I found that the following update is needed when only
                                              // one row is in the model.
//                                              this.grid.update();
                                              this.updateFooter(this.gridParms.store.id);
                                              dojo.publish (this.id + "#Refresh", [ "Refresh" ]);
                                            });
                                            
        this.grid.setLoadCompleteCallback(cb);  // set the callback		
		this.grid.prevScrollTop=this.grid.scrollTop;		
		for(var i in this.grid.views.views){
						v = this.grid.views.views[i];
						if(v.viewWidth == null || v.viewWidth == ""){
						   this.grid.prevScrollLeft = v.scrollboxNode.scrollLeft;
						   break;
						}
					}
		
        this.grid._refresh(true);
	},
	refreshWithArgs: function (args) {
		this.tableArgs = args;
		var dataQuery = {query:this.query};

	  	if (this.tableArgs) {
	  		for (var key in this.tableArgs) {
	  			if (key) {
	  				dataQuery[key] = this.tableArgs[key];
	  			}	
	  		}
	  	}
	
	// TODO: We still need to set the widgetID a different way  	
//	  	if (this.widgetID !== null) {
//			dataQuery.widgetID = this.widgetID;
//		}
	  	
	  	this.grid.setQuery(dataQuery);	
	  	this.onRefresh();
	},
	updateFooter: function (storeID) {
//		console.log ("enter TIPTable.updateFooter");
		if (storeID == this.gridParms.store.id) {
			var total = this.grid.getTotalRows();
			var numSelected = this.grid.selection.getSelectedCount();
			var footerText = dojo.string.substitute(this.resources_.TABLE_FOOTER_TEXT, { 0: numSelected, 1: total });
        	this.getFooterToolbar().setInfoAreaHTML( footerText );
        }
	},
	getFooterToolbar: function () {
		return (this.footerToolbar);
	},
	getToolbar: function () {
		return (this.toolbar);
	}, 
	onSearch: function (tipSearchField, event) 
	{
//		console.log ("enter onSearch with search value: " + tipSearchField.searchField.value);
		
		if (this.search) 
		{
			var columns = this.searchField.getSelectedColumns();
			this.search (columns, tipSearchField.searchField.value);
		}
	},
	search: function (columns, searchFor) {		
		if(this.tableBuilder){
			var queryOptions = {ignoreCase: true};
			/*
			if(this.lastSearchColumns){
				for (var x in this.lastSearchColumns) {
					if (x) {
						this.query[this.lastSearchColumns[x]] = {value:"", op:"CONTAINS", sensitive:false};
					}
				}
			}
			*/
			/*
			this.query.navModel.search = {};
			var search = this.query.navModel.search;
			search.term = searchFor;
			search.attributes = [];	
			for (var x in columns) {
				if (x) {
					//this.query[columns[x]] = {value:searchFor, op:"CONTAINS", sensitive:false};
					search.attributes.push(columns[x]);
				}
			}
			*/	
			//this.lastSearchColumns = columns;	
			if(!this.query){
				this.query = {};
			}
			this.query.ids = this.baseQuery.ids ? this.baseQuery.ids : {};		
			this.query.expression = "";
			for(var i=0; i<columns.length; i++){
				this.query.ids[columns[i]] = true;
				this.query.expression += "("+ columns[i] +" contains '"+searchFor+"')";
				if(i!=columns.length-1){
					this.query.expression += " or ";
				}				
			}
			if(this.baseQuery.expression){
				this.query.expression =  "(" + this.query.expression + ")" + " and " + "(" + this.baseQuery.expression + ")"
			}	
			this.grid.setQuery(this.query,queryOptions);
		}else{
			this.query = {};
			for (var x in columns) {
				if (x) {
					this.query[columns[x]] = searchFor;
				}
			}
		// TODO: We still need to set the widgetID a different way	
//			if (this.widgetID !== null) {
//				this.query.widgetID = this.widgetID;
//			}

//			console.log ("query is: ");
//			console.log (this.query);
			this.grid.setQuery(this.query);
		}	
	},
	
	toggleToolbar: function () {
       console.info("toggleToolbar");
       		if (this.toolbarHidden_) {
			this.toggleToolbarOn();
		} else {
			this.toggleToolbarOff();
		}
		this.saveToolbarStatus(this.toolbarHidden_);
	},
	
	// anurag : there was no keyboard event handler for closing/opening the nub.  If the key is shift/tab, don't do anything.
	toggleToolbarKeydown: function (event) {
		if(event.keyCode == 9 || event.keyCode == 16){
			return;
		}	
       this.toggleToolbar();
	},
	
	saveToolbarStatus: function(toolbarHidden_) {
	// 5831 :  TBSM17995: 2.2.0.12 yields FF error each time table toolbar toggled
	// checking if this.tableBuilder is not defined.
	if(this.tableBuilder != undefined)
	{
	  	if(this.tableBuilder.preferencesDialog === null || this.tableBuilder.preferencesDialog == undefined){
			this.tableBuilder.preferencesDialog = new ibm.tivoli.tip.charts.dijit.ChartPreferences ( 
					{ global: this.global,				  
					  tableBuilder: this.tableBuilder,
					  isTableFirst: this.tableBuilder.isTableFirst
					} 
				);
		}
		this.tableBuilder.preferencesDialog.startup();
		this.tableBuilder.preferencesDialog.builder.toolbarHidden_ = toolbarHidden_;
		this.tableBuilder.preferencesDialog.saveUserPreferences(false);
	}
	},
	toggleToolbarOn: function () {	
		console.info("toggleToolbarOn ");
		this.toolbarHidden_ = false;
		var animatedToolbar = dojo.fx.wipeIn ({    
			node: this.filterBar.id+"toolbar",
		        duration: 500,
                onEnd: dojo.hitch(this, function() {
                	
                })
			});		
		animatedToolbar.play();
		var animatedToolbar1 = dojo.fx.wipeIn ({    
			node: this.filterBar.id+"searchbar",
		        duration: 500,
                onEnd: dojo.hitch(this, function() {
                	
                })
			});		
		animatedToolbar1.play();
		var animatedToolbar1 = dojo.fx.wipeIn ({    
			node: this.filterBar.id+"toolbarTR",
		        duration: 500,
                onEnd: dojo.hitch(this, function() {
                	
                })
			});	
		dojo.style(this.filterBar.toolbar,"height","9px");
		dojo.style(this.filterBar.nub,"position","absolute");
		dojo.removeClass(this.filterBar.nub,"nubClosed");
		dojo.addClass(this.filterBar.nub,"nubOpen");
		this.filterBar.stateMinimized=false;
		dojo.style(this.filterBar.nub,"display","block");
	},
	toggleToolbarOff: function(){
		console.info("toggleToolbarOff");
		this.toolbarHidden_ = true;
		
		var animatedtoolbarOff = dojo.fx.wipeOut ({
				node: this.filterBar.id+"toolbar",
		        duration: 500,
                onEnd: dojo.hitch(this, function() {
                
                })
			});
		animatedtoolbarOff.play();
		var aniOff2 = dojo.fx.wipeOut ({
			node: this.filterBar.id+"searchbar",
		    duration: 500,
            onEnd: dojo.hitch(this, function() {
                
            })
		});
		aniOff2.play();
		dojo.style(this.filterBar.nub,"position","relative");
		//dojo.style(this.filterBar.nubArea,"height","10px");
				
		this.filterBar.stateMinimized=true;
		dojo.removeClass(this.filterBar.nub,"nubOpen");
		dojo.addClass(this.filterBar.nub,"nubClosed");
		
	},
	onRowClick: function (evt) {
		console.log ("enter TIPTable.onRowClick");
		this.updateFooter (this.gridParms.store.id);
	},
	showTableOptions: function () 
	{
//		console.log ("enter TIPTable.showTableOptions");
		this.tableOptionsDialog = new ibm.tivoli.tip.dijit.TIPTableOptionsDialog ( 
				{ table: this, global: this.global } 
			);
		
		this.tableOptionsDialog.show();
	},
	getColumnById: function (id) {
		var viewIndex = this.getViewColumnIndex (id);
		var column = this.grid.getCell (viewIndex);
//		console.log ("TIPTable.getColumnById ( " + id + "). returning column: ");
//		console.log (column);
		return (column);
	},
	getStore: function () {
		return (this.gridParms.store);
	},
	destroy: function ()
	{
		//Vyshantha Jan6,2011: #159935 TIPTable Memory Leak
		delete this.tableBuilder;
		delete this.filterBar;
		delete this.searchField;
		delete this.grid;
		delete this.gridParms;
		delete this.cmProvider;
		delete this._contextMenuAdapter;
		delete this.viewData;
		delete this.widgetID;
		delete this.tableParams;
		delete this.tableOptionsDialog;
		delete this.originalStructure;
		for ( var i in this.topics )
		{
			if (i) {
				dojo.unsubscribe( this.topics[i] );
			}
		}
		this.inherited("destroy",arguments );
		if ( this._allToolbarButtons )
		{
			for ( tbb in this._allToolbarButtons )
			{
				if(this._allToolbarButtons[ tbb ]){
					this._allToolbarButtons[ tbb ].destroy();
				}
			}
		}
		if(this.toolbar){
			this.toolbar.destroy();
		}
		if(this.footerToolbar){
			this.footerToolbar.destroy();
		}		
	}
	
});
	

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableTitleBar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableTitleBar"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/


dojo.provide("ibm.tivoli.tip.dijit.TIPTableTitleBar");
/******************************************************************************
* Provides title bar for table that looks like TIP portlet title
*
* Authors: Joseph Firebaugh
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableTitleBar", [dijit._Widget, dijit._Templated],
{
  label: "",
  
  templateString: "<div class='tipTableTitleBar'><div class='tipTableTitleBarLabel' dojoAttachPoint='titleNode'>${label}</div></div>",

  /*****************************************************************************
  * Programmatically allow changing of the table title. String argument can
  * containe any HTML.
  *****************************************************************************/
  setTitle: function( label /*string*/ )
  {
    this.titleNode.innerHTML = label;
  }  

});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TivPage"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TivPage"] = true;
dojo.provide("ibm.tivoli.tip.dijit.TivPage");
dojo.provide("ibm.tivoli.tip.dijit.TivSubstep");











/**
 * 
 * A page that may be used in both the notebook and wizard widgets.
 * 
 * For the wizard, it handles substeps.
 * 
 */
dojo.declare("ibm.tivoli.tip.dijit.TivPage", [dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin], 
{
  widgetsInTemplate: true,
  title: "",
  visited: false,        // has this page been visited?
  validated: false,      // has this page been validated?

  substeps: null,        // collection of substeps for this main step
  substepsList: null,    // ordered list of the substep list items for this main step
  
  listItem: null,        // list entry
  connectionLink: null,  // its "connect" link

  disabled: false, 
  required: false,
  loops: false,          // if true, this page and substeps form a loop
  titleOnly: false,      // if true and this main step has substeps, main page is never shown; only its
                         // title is show in the ToC

  /**
   * Render the page itself
   */
  buildRendering: function()
  {
    console.log("TivPage.buildRendering");
    this.inherited(arguments);
    
    // TivPage subclasses ContentPane, but adds in _Templated, which ContentPane didn't expect
    // we have to fix some stuff.
    dojo.attr(this.domNode, "region", "center");
    this.containerNode = this.domNode;
    dojo.addClass(this.domNode, "tiv-page");
  },

  /**
   * Render this page's entry in the ToC.  The list item is created once. 
   * For wizard, it is used to jump back to the associated page.
   */
  buildTOCRendring: function() 
  {
    console.log("TivPage.buildTOCRendring - " + this.title);

    // if the list entry hasn't been created yet
    if (this.listItem == null) {

      // create it
      if (this.wizard == null) {
        // vertical notebook
        this.listItem = dojo.create("li", {
          innerHTML: "<div class='tiv-page-toc-icon'></div><div>" + this.title + "</div>",
          pageID: this.id,
          className: this.visited ? "tiv-page-visited" : ""
        });        
      }
      else {
        // wizard
        if ('v' === this.wizard.layout) {
          // vertical toc
          this.listItem = dojo.create("li", {
            innerHTML: "<span class='tiv-page-toc-icon'></span><span class='tiv-page-required-flag'></span>" + "<span class='tiv-page-toc-title'>" + this.title + "</span>",
            pageID: this.id,
            className: this.visited ? "tiv-page-visited" : ""
          });        
        }  
        else {
          // horizontal toc
          if (this.isMainStep()) {
            if (!this.wizard.renderNoArrows) {
              // onlick function is needed so link is not followed
              // link wraps entire arrow for tabbing
              this.listItem = dojo.create("li", {
                innerHTML: "<a href='#' onclick='return false;'><i></i><span><em></em>" + this.title + "</span><b></b></a>",
                pageID: this.id,
                className: this.visited ? "tiv-page-visited" : ""
              });
            }
            else {
              // render without arrow graphics like substeps 
              // required flag is applicable
              this.listItem = dojo.create("li", {
                innerHTML: "<a href='#' onclick='return false;'><span class='tiv-page-toc-title'><em></em>" + this.title + "</span></a>",
                pageID: this.id,
                className: this.visited ? "tiv-page-visited" : ""
              });                      
            }              
          }
          else {
            // substep
            this.listItem = dojo.create("li", {
              innerHTML: "<div class='tiv-page-toc-title'>" + this.title + "</div>",
              pageID: this.id,
              className: this.visited ? "tiv-page-visited" : ""
            });        
          }
        }
        // if this is a disabled page, mark it
        if (this.disabled)
          this.setDisabled(true);
        else
          // connect it to this page
          this.connectionLink = dojo.connect(this.listItem, "onclick", this, this.jumpBack);

        // if this is a required page, mark it
        if (this.required)
          this.setRequired(true);        
      }        
    }

    return this.listItem;
  },

  /**
   * This is always true; substeps return false.
   */
  isMainStep: function() 
  {
    return (true);
  },
  
  /**
   * Return true if page has substeps
   */
  hasSubsteps: function()
  {
    if (this.substeps != null && this.substeps.length > 0)
      return (true);
    else
      return (false);
  },
  
  /**
   * Return ordered list of substep item entries
   */
  getSubstepsList: function()
  {
    return this.substepsList;
  },

  /** 
   * Return true if the page is disabled
   */
  isDisabled: function() 
  {
    return (this.disabled);
  },

  /** 
   * Enable or disable a page
   */
  setDisabled: function (aBoolean) 
  {
    console.log("TivPage.setDisabled - " + aBoolean);

    this.disabled = aBoolean;

    if (this.disabled)
      dojo.addClass(this.listItem, "tiv-page-disabled");
    else
      dojo.removeClass(this.listItem, "tiv-page-disabled");   
  },

  /**
   * Turn on or off the "required" flag, which adds or removes a class
   * that controls the display of an icon.
   */
  setRequired: function(aBoolean) 
  {
    console.log("TivPage.setRequired - " + aBoolean);

    this.required = aBoolean;

    if (this.required)
      dojo.addClass(this.listItem, "tiv-page-required");
    else 
      dojo.removeClass(this.listItem, "tiv-page-required");
    
    // need to update UI to allow for asterisk image
    this.wizard.updateUI();
  },
  
  /**
   * Returns true if step is part of a loop.
   */
  isPartOfLoop: function()
  {
    return this.loops;
  },
  
  /**
   * Returns true if this is a main step with substeps and only the
   * title should show in the ToC, i.e. the main page itself is
   * never shown during navigation.
   */
  showTitleOnly: function()
  {
    if (this.titleOnly && this.hasSubsteps())
      return true;
    else
      return false;
  },
  
  /**
   * Indicate that main page should show or not when it contains substeps.
   */
  setTitleOnly: function(titleOnly)
  {
    this.titleOnly = titleOnly;
  },

  /**
   * The page may want to validate the data before allowing the user to
   * move back a page.
   * 
   * Returning false will block the page container from moving back a page.
   * 
   * Putting up a message and moving the focus to the appropriate field
   * is the responsibility of the page.
   */
  allowBack: function() 
  {
    console.log("TivPage.allowBack");
    return (true);
  },

  /**
   * The page may want to validate the data before allowing the user to
   * move to the next page.
   * 
   * Returning false will block the page container from moving to the next page.
   * 
   * Putting up a message and moving the focus to the appropriate field
   * is the responsibility of the page.
   */
  allowNext: function() 
  {
    console.log("TivPage.allowNext");
    return (true);
  },

  /**
   * The page may want to validate the data before allowing the user to
   * use the "Finish" button.
   * 
   * Returning false will block the page container from allowing "Finish"
   * 
   * Putting up a message and moving the focus to the appropriate field
   * is the responsibility of the page.
   */
  allowFinish: function() 
  {
    console.log("TivPage.allowFinish");
    return (true);
  },

  /**
   * When jumping back to a page, the current page is marked as 
   * visited (or complete).  Override this function and return
   * false to keep the current page as unvisited.
   */
  allowMarkAsComplete: function()
  {
    console.log("TivPage.allowMarkAsComplete");
    return (true);    
  },
  
  /**
   * Disable jumping back to this page
   */
  disableJumpBack: function() 
  {
    console.log("TivPage.disableJumpBack");

    // if there is a connection link
    if (this.connectionLink != null) {

      // disconnect the handler
      dojo.disconnect(this.connectionLink);

      // set it to null so it won't be used again
      this.connectionLink = null;
    }

    // add the "disabled" class
    dojo.addClass(this.listItem, "tiv-page-disabled");
  },

  /**
   * Enable jump back for the item
   * 
   * But not if it is disabled
   */
  enableJumpBack: function() 
  {
    console.log("TivPage.enableJumpBack");

    if (this.disabled == null || this.disabled == false) {

      // if there is no handler set up
      if (this.connectionLink == null) {

        // create the handler
        this.connectionLink = dojo.connect(this.listItem, "onclick", this, this.jumpBack);

        // remove the "disabled" class
        dojo.removeClass(this.listItem, "tiv-page-disabled");
      }
    }
  },

  /**
   * The user wants to jump back to a previously visited page.
   * 
   * However, the "onclick" handler is attached when the table of
   * contents list item is created. The cursor will not be the "hand"
   * but the handler will still be called. So a check to see if this
   * page has been visited is made before letting the parent container
   * jump back to this page.
   */
  jumpBack: function (event) 
  {
    console.log("TivPage.jumpBack");

    // if this page has been visited
    if (this.visited) {
      // then jump back to it
      this.wizard.jumpBackToPage(this);
    }
  },
  
  /**
   * Add another substep to the collection
   */
  addSubstep: function (anotherSubstep) 
  {
    console.log("TivPage.addSubstep");

    if (this.substeps == null)
      this.substeps = [];
    this.substeps[this.substeps.length] = anotherSubstep;
  },

  /**
   * Returns the substeps for this page
   */
  getSubsteps: function()
  {
    return this.substeps;
  },

  /**
   * Hide any substeps that this main step might have
   */
  hideSubsteps: function() 
  {
    console.log("TivPage.hideSubsteps");

    // if there are any substeps
    if (this.substeps != null) {
      // if vertical toc
      if ('v' === this.wizard.layout) {
        // do for all substeps
        for (var i = 0; i < this.substeps.length; i++) {
          // get the table of contents entry
          var tocEntry = this.substeps[i].buildTOCRendring();

          // and hide it
          dojo.style(tocEntry, "display", "none");
        }
      }
      else
      {
        // horizontal mode
        if (this.wizard.tocSubstepsArea.childNodes.length > 0) {
          // remove substeps list
          this.wizard.tocSubstepsArea.removeChild(this.substepsList);
        }
      }
    }
  },
  
  /**
   * Show any substeps that this main step might have
   */
  showSubsteps: function()
  {
    console.log("TivPage.showSubsteps");

    // if there are any substeps
    if (this.substeps != null) {

      // if vertical toc
      if (this.wizard == null || 'v' === this.wizard.layout) {
        // do for all substeps
        for (var i = 0; i < this.substeps.length; i++) {
          // get the table of contents entry
          var tocEntry = this.substeps[i].buildTOCRendring();

          // and make it visible
          dojo.style(tocEntry, "display", "block");
        }
      }
      else {
        // horizontal mode - add substeps ordered list to substeps area
        this.wizard.tocSubstepsArea.appendChild(this.substepsList);
      }
    }    
  },

  /**
   * Mark everything up to this page as having been visited
   */
  markPreviousPagesVisited: function() 
  {
    console.log("TivPage.markPreviousPagesVisited");

    this.wizard.markPreviousPagesVisited(this);
  },

  // dijit.form._FormMixin will walk through all the child widgets checking their validity,
  // if you need to do something else, override
  isValid: function()
  {
    this.validated = this.inherited(arguments); 
    console.log(this.title + ".isValid <= " + this.validated);
    return this.validated;
  },
  // same as isValid, but moves focus to the first invalid child widget
  validate: function()
  {
    this.validated = this.inherited(arguments);
    console.log(this.title + ".validate <= " + this.validated);
    return this.validated;
  },

  /**
   * Slightly different because substeps might be involved.
   * 
   * When a main step becomes the selected page all of its substeps
   * should be made visible.
   */
  OnPageSelected: function(wiz) 
  {
    console.log("TivPage.OnPageSelected - " + this.title);
    
    // show substeps if there are any...
    if (this.wizard)
      this.showSubsteps();
  },
  
  /**
   * catch OnChange events from my child widgets,
   * then let listeners know something's changed
   */
  _onPageContentChange: function(evt)
  {
    this.OnPageContentChange(this);
  },
  
  OnPageContentChange: function(page)
  {
    // override or connect to me
  }
});

/**
 * An extension that is a substep
 */
dojo.declare("ibm.tivoli.tip.dijit.TivSubstep", [ibm.tivoli.tip.dijit.TivPage], 
{
  mainPage: null,  // main page for this substep
  
  constructor: function(arguments) {},

  isMainStep: function() 
  {
    return (false);
  },
  
  isPartOfLoop: function()
  {
    if (this.mainPage.isPartOfLoop())
      return (true);
    else
      return (false);
  },
  
  /**
   * Nested substeps are not supported
   */
  hasSubsteps: function()
  {
    return (false);
  },
  
  getMainPage: function()
  {
    return (this.mainPage);
  },
  
  /** 
   * These items are created with some minor differences in how they are
   * displayed
   */
  buildTOCRendring: function() 
  {
    console.log("TivSubstep.buildTOCRendring - " + this.title);
    
    var listItemToAdd = this.listItem;

    // if the list entry hasn't been created yet
    if (this.listItem == null) {
      // create it
      this.inherited(arguments);
      
      // save main page for this substep
      this.mainPage = this.wizard.getPreviousMainPage(this);
      
      // if vertical toc
      if ('v' === this.wizard.layout)
      {
        listItemToAdd = this.listItem;
        
        // make it invisible
        dojo.style(this.listItem, "display", "none");

        // and indented
        dojo.addClass(this.listItem, "tiv-wiz-toc-substep");
      }
      else {
        // horizontal mode - add to substep list of main page
        listItemToAdd = null;
        if (this.mainPage != null)
        {
          if (this.mainPage.substepsList == null)
            this.mainPage.substepsList = dojo.create("ol", {});
          this.mainPage.substepsList.appendChild(this.listItem);
        }        
      }
    }
    return listItemToAdd;
  }
});


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TivPageContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TivPageContainer"] = true;
dojo.provide("ibm.tivoli.tip.dijit.TivPageContainer");









/*
 * Page container use by notebook and wizard widgets.
 *
 * Events:
 *  OnPageSelected: when a page is made active
 *  OnOK: when the Finish/OK button is clicked
 *  OnCancel: when the Cancel button is clicked
 */

dojo.declare("ibm.tivoli.tip.dijit.TivPageContainer", [dijit._Container],
{
  mode: "",          // "n" for notebook or "w" for wizard (set by subclass)
  layout: "v",       // "vertical" or "horizontal" (actually, anything starting with "v" | "h")
  nested: false,     // contained <form> is used for containerNode (default), if true <div> used instead
  formId: "",        // id of form to submit when container is being nested
  action: ".",       // the action attribute on the contained <form>
  method: "POST",    // the method attribute on the contained <form>
  _pages: null,		 // Jackie: April29,2011: Fix TivWizard issue
  _pageChangeConnections: null, 	// Jackie: April29,2011: Fix TivWizard issue
  _containerSubscriptions: null,	// Jackie: April29,2011: Fix TivWizard issue
  _origOK: null,
  
  constructor: function()
  {
    this._origOK = this.OnOK;
	this._pages = [];					// Jackie: April29,2011: Fix TivWizard issue
	this._pageChangeConnections = {};	// Jackie: April29,2011: Fix TivWizard issue
	this._containerSubscriptions = [];	// Jackie: April29,2011: Fix TivWizard issue
	
  },
  postMixInProperties: function() 
  {
    if (this.layout && this.layout.length > 0)
      this.layout = this.layout.charAt(0).toLowerCase();
    console.log("ToC layout is " + this.layout);
    console.log("nested is " + this.nested);
    this.inherited(arguments);
  },
  buildRendering: function()
  {
    console.log("TivPageContainer.buildRendering: " + this.id);

    this.inherited(arguments);
    
    this._pageContainer.containerNode = this.containerNode;
    this._addFooterButtons();
    
    // if ToC layout is horizontal... 
    if ('h' === this.layout) {
      // move toc region and update class
      var child = this._tocArea;
      this._borderArea.removeChild(child);
      child.region = "top";
      child._setClassAttr("tiv-toc-horizontal");
      this._borderArea.addChild(child, 0);                    
    }
    console.log("rendered");
  },
  postCreate: function()
  {
    console.log("TivPageContainer.postCreate: " + this.id);
    this.inherited(arguments);
  },
  startup: function()
  {
    if(this._started){ return; }
    console.log("TivPageContainer.startup: " + this.id);
    this.inherited(arguments);
    
    this._containerSubscriptions.push(dojo.subscribe(this._pageContainer.id + "-selectChild", this, this._onPageSelected));
    this._containerSubscriptions.push(dojo.subscribe(this._pageContainer.id + "-addChild", this, this._onPageAdded));
    this._containerSubscriptions.push(dojo.subscribe(this._pageContainer.id + "-removeChild", this, this._onPageRemoved));

    // get the pages provided in markup
    this._pages = this._pageContainer.getChildren();
    
    // save reference to ourselves as an attribute so each page has access
    // must occur before _buildToC
    dojo.forEach(this._pages, this._setContainerType, this);
    
    // build the initial table of contents from the pages provided in markup
    this._buildToC();

    // This is copied from StackContainer.startup, which gets called too soon

    // Setup each page
    dojo.forEach(this._pages, this._pageContainer._setupChild, this._pageContainer);
    dojo.forEach(this._pages, function(p, i) {
      this._onPageAdded(p, i);
    }, this);
    
    var selectedChild = null;
    dojo.some(this._pages, function(child){
      if(child.selected){
        selectedChild = child;
      }
      return child.selected;
    }, this);
    if(!selectedChild && this._pages[0]){
      selectedChild = this._pages[0];
    }
    if(selectedChild) {
      //The _onPageSelected function wasn't getting called for a selectPage
      //this.selectPage(selectedChild);
      this._onPageSelected(selectedChild);
    }
    
    this._outerContainer.resize();
  },  
  _addFooterButtons: function()
  {
    // subclass should override
  },  
  _setContainerType: function(page)
  {
    // subclass should override
  },  
  _addToCItemConnections: function(toc_item)
  {
    // subclass should override
  },
  /**
   * submit the contained form
   * wire this function to the OnOK event
   */
  submit: function(wiz)
  {
    var _form = (this.nested) ? this.formId : this.id + "_form";
    if ('' === _form) {
      console.warn("TivPageConainer.submit warning: no form to submit");
      return null;
    }
    console.log("TivPageConainer.submit form with id: " + _form);
    
    // our form is the containerNode
    var f;
    switch(this.method.toLowerCase()) {
    case "get":
      f = dojo.xhrGet;
      break;
    case "post":
      f = dojo.xhrPost;
      break;
    case "put":
      f = dojo.xhrPut;
      break;
    default:
      f = dojo.xhrPost;
    }
    var dfd = f({
      url: this.action,
      preventCache: true,
      form: _form, 
      load: function(response) {
        console.log("TivPageContainer.submit success");
      },
      error: function(response) {
        console.error("TivPageContainer.submit failure");
        console.error(response);
      }
    });
    return dfd;
  },
  _buildToC: function()
  {
    console.log("TivPageContainer._buildToC");
    
    if (this._toc) {    
      while(this._toc.firstChild) {
        this._toc.removeChild(this._toc.firstChild);
      }

      var page_count = this._pages.length;
      dojo.forEach(this._pages, function(p, i) {
        var toc_item = p.buildTOCRendring();
        if (toc_item != null) {
          dojo.place(toc_item, this._toc, "last");
          this._addToCItemConnections(toc_item);
        }
      }, this);    
    }
  },
  destroy: function()
  {
    console.log("TivPageContainer.destroy");
    var alerter;
    while(alerter = this._containerSubscriptions.pop()) {
      dojo.unsubscribe(alerter);
    }
    var p;
    for(p in this._pageChangeConnections) {
      dojo.disconnect(this._pageChangeConnections[p]);
    }
    try{
    	this.inherited(arguments);
	}catch(ex){
		console.warn("Widget destroy failed: "+this.id+", ex="+ex.name+":"+ex.message);
	}
  },
  destroyRecursive: function()
  {
    console.log("TivPageContainer.destroyRecursive");
    //this.inherited(arguments);  TBSM10157: destroying of TivNotebook/TivWizard causing "id is null" error in IE8
    var alerter;
		while(alerter = this._containerSubscriptions.pop()) {
		  dojo.unsubscribe(alerter);
		}
		var p;
		for(p in this._pageChangeConnections) {
		  dojo.disconnect(this._pageChangeConnections[p]);
		}
	this.destroy();
  },
  selectPage: function(/*widget*/ p)
  {
    this._pageContainer.selectChild(p);
  },
  addPage: function(/*widget*/ p)
  {
    this._setContainerType(p);
    this._pageContainer.addChild(p);
    this._pages = this._pageContainer.getChildren();
    this._buildToC();
    this.updateUI();
  }, 
  removePage: function(/*widget*/ p)
  {
    // If removing selected page, StackContainer will select next
    // page by default.
    this._pageContainer.removeChild(p);
    this._pages = this._pageContainer.getChildren();
    this._buildToC();
    this.updateUI();
  }, 
  updateUI: function()
  {
    console.log("TivPageContainer.updateUI");
    
    var curr_page = this._pageContainer.selectedChildWidget;
        
    // update the ToC
    if (this._toc) {
      dojo.query("li[pageID]", this._toc).forEach(function(n, i) {
        var pid = dojo.attr(n, "pageID");
        var p = dijit.byId(pid);
        dojo[(p && p.visited) ? "addClass" : "removeClass"](n, "tiv-page-visited");
        dojo[(p && (!p.visited || p.validated)) ? "removeClass" : "addClass"](n, "tiv-page-invalid");
        dojo[(p == curr_page) ? "addClass" : "removeClass"](n, "tiv-page-selected");
      });
    }

    //To make the current page visible
    if(curr_page){
      dojo.removeClass(curr_page.domNode,"dijitHidden");
      dojo.addClass(curr_page.domNode,"dijitVisible");
    }
  },
  _onPageSelected: function(page) 
  {
    console.log("TivPageContainer._onPageSelected: " + page.id);
    
    this.OnPageSelected(this, page);
    if("function" == typeof page.OnPageSelected) {
      page.OnPageSelected(this);
    }
    this.updateUI();
    
    // if the page can handle this function
    if ("function" == typeof page.AfterPageSelected) {
      // let the page get final control
      page.AfterPageSelected(this);
    }
  },
  _onPageContentChange: function(page)
  {
    console.log("TivPageContainer.onPageContentChange: " + page.id);
    this.updateUI();
  },
  _onPageAdded: function(page, insertIndex)
  {
    console.log("TivPageContainer._onPageAdded: " + page.id + " pos: " + insertIndex);
    this._pageChangeConnections[page.id] = dojo.connect(page, "OnPageContentChange", this, this._onPageContentChange);
    this.updateUI();
  },
  _onPageRemoved: function(page)
  {
    console.log("TivPageContainer._onPageRemoved: " + page.id);
    dojo.disconnect(this._pageChangeConnections[page.id]);
    delete this._pageChangeConnections[page.id];
    this.updateUI();
  },
  _onClickFinish: function(evt)
  {
    console.log("TivPageContainer._onClickFinish");
    this.OnOK(this);
  },
  _onClickCancel: function(evt)
  {
    console.log("TivPageContainer._onClickCancel");
    this.OnCancel(this);
  },
  // fired events
  OnPageSelected: function(wiz, page)
  {
    // override or connect to me
  },
  OnOK: function(wiz)
  {
    // override or connect to me
    
    // if this is still the same function as it was originally,
    // then nobody has connected to the event. do the default
    if(this._origOK === this.OnOK) {
      // nothing connected, do the default
      this.submit(this);
    }
  },
  OnCancel: function(wiz)
  {
    // override or connect to me
  },
  
  /**
   * Enable (or disable) the Finish/OK button
   */
  enableFinishButton: function(shouldEnable) 
  {
    this._finishButton.attr("disabled", !shouldEnable);
  },

  /** 
   * Enable (or disable) the Cancel button
   */
  enableCancelButton: function(shouldEnable) 
  {
    this._cancelButton.attr("disabled", !shouldEnable);
  },

  /**
   * Hide (or show) the Finish/OK button
   */
  hideFinishButton: function(shouldHide) 
  {
    if (shouldHide) 
      dojo.style(this._finishButton.domNode, "display", "none");
    else 
      dojo.style(this._finishButton.domNode, "display", "inline-block");
  },

  /**
   * Hide (or show) the Cancel button
   */
  hideCancelButton: function(shouldHide) 
  {
    if (shouldHide) 
      dojo.style(this._cancelButton.domNode, "display", "none");
    else 
      dojo.style(this._cancelButton.domNode, "display", "inline-block");
  },
  
  /**
   * Returns the currently selected page
   */
  getSelectedPage: function()
  {
    return this._pageContainer.selectedChildWidget; 
  },
  
  /**
   * Resizes this container according to the new dimensions.
   * 
   * @param newSize  the new dimensions with properties w for width 
   *                 and h for height
   */
  resize: function(newSize) 
  {
    if (newSize != null) {
      if (newSize.w != null)
        dojo.style(this.domNode, 'width', newSize.w + 'px');
      if (newSize.h != null)
        dojo.style(this.domNode, 'height', newSize.h + 'px');
      this._outerContainer.resize();
    }
  }  
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TivNotebook"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TivNotebook"] = true;
console.warn("'ibm.tivoli.tip.dijit.TivNotebook' is deprecated, use 'dojoe.notebook.Notebook'(TED) instead.");
dojo.provide("ibm.tivoli.tip.dijit.TivNotebook");








/**
 * Inherit from TivPageContainer which contains the common functions of both
 * the notebook and wizard.
 */


/*
 * Creating a TivNotebook in markup
 * <div id="note" dojoType="ibm.tivoli.tip.dijit.TivNotebook">
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.Welcome"></div>
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.BaseWidget"></div>
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.General"></div>
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.Security"></div>
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.Summary" ></div>
 *  </div> 
 * 
 * Events:
 *  OnPageSelected: when a page is made active
 *  AfterPageSelected: after a page has been made active
 *  OnOK: when the OK button is clicked
 *  OnApply: when the Apply button is clicked
 *  OnCancel: when the Cancel button is clicked
 */
dojo.declare("ibm.tivoli.tip.dijit.TivNotebook", 
            [dijit._Widget, dijit._Templated, ibm.tivoli.tip.dijit.TivPageContainer],
{
  widgetsInTemplate: true,
  templatePath: "",

  mode: "n",               // notebook
  applyLabel: "",
  okLabel: "",  
  cancelLabel: "",  
  _origApply: null,
  
  constructor: function()
  {
    this._origApply = this.OnApply;
  },

  postMixInProperties: function()
  {
    console.log("TivNotebook.postMixInProperties");    
    if (this.nested)
      this.templateString="<div class=\"tiv-notebook\">\r\n  <div dojoType=\"dijit.layout.BorderContainer\" design=\"headline\" gutters=\"false\" class=\"tiv-notebook-outer\" dojoAttachPoint=\"_outerContainer\">\r\n    <div dojoType=\"dijit.layout.BorderContainer\" gutters=\"false\" liveSplitters=\"false\" design=\"sidebar\" region=\"center\" style=\"width:100%;height:100%;\" dojoAttachPoint=\"_borderArea\">\r\n      <div dojoType=\"dijit.layout.ContentPane\" region=\"leading\" splitter=\"false\" class=\"tiv-notebook-toc\" dojoAttachPoint=\"_tocArea\">\r\n        <ol dojoAttachPoint=\"_toc\"></ol>\r\n      </div>\r\n      <div dojoType=\"dijit.layout.TabContainer\" region=\"center\" splitter=\"false\" dojoAttachPoint=\"_pageContainer\" nested=\"true\" class=\"tiv-notebook-work-area\">\r\n        <div dojoAttachPoint=\"containerNode\"></div>              \r\n      </div>\r\n    </div>\r\n    <div dojoType=\"dijit.layout.ContentPane\" region=\"bottom\" splitter=\"false\" class=\"tiv-notebook-footer\" dojoAttachPoint=\"_buttonArea\">\r\n      <div class=\"tiv-notebook-buttons\" dojoAttachPoint=\"_buttons\"> \r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickFinish\" dojoAttachPoint=\"_finishButton\">${okLabel}</button>\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickApply\" dojoAttachPoint=\"_applyButton\">${applyLabel}</button>\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickCancel\" dojoAttachPoint=\"_cancelButton\">${cancelLabel}</button>\r\n      </div>        \r\n    </div>\r\n  </div>\r\n</div>\r\n";
    else
      this.templateString="<div class=\"tiv-notebook\">\r\n  <div dojoType=\"dijit.layout.BorderContainer\" design=\"headline\" gutters=\"false\" class=\"tiv-notebook-outer\" dojoAttachPoint=\"_outerContainer\">\r\n    <div dojoType=\"dijit.layout.BorderContainer\" gutters=\"false\" liveSplitters=\"false\" design=\"sidebar\" region=\"center\" style=\"width:100%;height:100%;\" dojoAttachPoint=\"_borderArea\">\r\n      <div dojoType=\"dijit.layout.ContentPane\" region=\"leading\" splitter=\"false\" class=\"tiv-notebook-toc\" dojoAttachPoint=\"_tocArea\">\r\n        <ol dojoAttachPoint=\"_toc\"></ol>\r\n      </div>\r\n      <div dojoType=\"dijit.layout.TabContainer\" region=\"center\" splitter=\"false\" dojoAttachPoint=\"_pageContainer\" class=\"tiv-notebook-work-area\">\r\n        <form id=\"${id}_form\" dojoAttachPoint=\"containerNode\" action=\"${action}\" method=\"${method}\"></form>              \r\n      </div>\r\n    </div>\r\n    <div dojoType=\"dijit.layout.ContentPane\" region=\"bottom\" splitter=\"false\" class=\"tiv-notebook-footer\" dojoAttachPoint=\"_buttonArea\">\r\n      <div class=\"tiv-notebook-buttons\" dojoAttachPoint=\"_buttons\"> \r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickFinish\" dojoAttachPoint=\"_finishButton\">${okLabel}</button>\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickApply\" dojoAttachPoint=\"_applyButton\">${applyLabel}</button>\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickCancel\" dojoAttachPoint=\"_cancelButton\">${cancelLabel}</button>\r\n      </div>        \r\n    </div>\r\n  </div>\r\n</div>\r\n";
    console.log("templatePath is " + this.templatePath);

    var string_table = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
    this.applyLabel = string_table.APPLY;
    this.okLabel = string_table.OK;
    this.cancelLabel = string_table.CANCEL;
    this.inherited(arguments);
  },
  
  buildRendering: function()
  {
    console.log("TivNotebook.buildRendering");        

    this.inherited(arguments);

    // if horizontal layout...
    if ('h' === this.layout) {
      // remove toc area and use tab container's tab strip
      this._borderArea.removeChild(this._tocArea);
      this._toc = null;
    }
    else {
      // hide tab container's tab strip and use toc area
      this._pageContainer._setClassAttr("tiv-notebook-tabstrip-hidden");
    }
  },
  
  _setContainerType: function(page)
  {
    page.attr("notebook", this);
  },
  
  _addToCItemConnections: function(toc_item)
  {
    // only called for vertical layout
    this.connect(toc_item, "onclick", this._onToCClick);
    this.connect(toc_item, "onmouseover", dojo.hitch(this, this._onToCOver));
    this.connect(toc_item, "onmouseout", dojo.hitch(this, this._onToCOut));
  },
  
  updateUI: function()
  {
    console.log("TivNotebook.updateUI");
    
    // disable the Apply button if the selected page does not validate
    var curr_page = this._pageContainer.selectedChildWidget;
    var isCurrentPageValid = curr_page && curr_page.isValid(); 
    if( !isCurrentPageValid )
      this._applyButton.attr("disabled", true);
        
    // disable the OK button until all pages are valid
    var allValid = dojo.every(this._pages, function(p){
      var v = p.isValid();
      return v;
     }, this);
    this._finishButton.attr("disabled", !allValid);
    this._applyButton.attr("disabled", !allValid);

    // if horizontal layout update invalid icon on tabs
    if ('h' === this.layout) {
      dojo.query(".tabLabel", this._pageContainer.tablist.domNode).forEach(function(n, i) {
        p = this._pages[i];
        dojo[(p && (!p.visited || p.validated)) ? "removeClass" : "addClass"](n, "tiv-page-invalid");
      }, this);     
    }
    this.inherited(arguments);
  },
  
  /**
   * If valid removes invalid icon on ToC item/tab.  If invalid adds invalid icon.  
   * Provided as a utility function when nesting this widget.
   */
  updateToCItemIcon: function(page, valid) {
    var toc_items;
    if ('h' === this.layout)
      toc_items = dojo.query(".tabLabel", this._pageContainer.tablist.domNode);
    else
      toc_items = dojo.query("li", this._toc);
    
    dojo.every(toc_items, function(n, i) {
      p = this._pages[i];
      if (page == p) {
        dojo[(valid) ? "removeClass" : "addClass"](n, "tiv-page-invalid");
        return false;
      }
      else 
        return true;
    }, this);      
  },
  
  _onToCOver: function(evt)
  {
    var li = evt.target;
    while(null != li && li.tagName.toLowerCase() != "li") {
      li = li.parentNode;
    }
    if(null == li) return;
    dojo.addClass(li, "over");
  },
  
  _onToCOut: function(evt)
  {
    var li = evt.target;
    while(null != li && li.tagName.toLowerCase() != "li") {
      li = li.parentNode;
    }
    if(null == li) return;
    dojo.removeClass(li, "over");    
  },
  
  _onToCClick: function(evt)
  {
    var li = evt.target;
    while(null != li && li.tagName.toLowerCase() != "li") {
      li = li.parentNode;
    }
    if(null == li) return;
    var pid = dojo.attr(li, "pageID");
    if(null != pid) {
      this.selectPage(pid);
    }
  },
  
  _onPageSelected: function(page) 
  {
    console.log("TivNotebook._onPageSelected: " + page.id);    
    page.visited = true;
    this.inherited(arguments);    
  },
  
  _onClickApply: function(evt)
  {
    console.log("TivNotebook._onClickApply");
    this.OnApply(this);
  },
  
  OnApply: function(wiz)
  {
    // override or connect to me
    
    // if this is still the same function as it was originally,
    // then nobody has connected to the event. do the default
    if(this._origApply === this.OnApply) {
      // nothing connected, do the default
      this.submit(this);
    }
  },
  
  /**
   * Enable (or disable) the OK button
   */
  enableOKButton: function(shouldEnable) 
  {
    this._finishButton.attr("disabled", !shouldEnable);
  },

  /**
   * Enable (or disable) the Apply button
   */
  enableApplyButton: function(shouldEnable) 
  {
    this._applyButton.attr("disabled", !shouldEnable);
  },

  /**
   * Hide (or show) the "OK" button
   */
  hideOKButton: function(shouldHide) 
  {
    this.hideFinishButton(shouldHide);
  },
  
  /**
   * Hide (or show) the "Apply" button
   */
  hideApplyButton: function(shouldHide) 
  {
    if (shouldHide) 
      dojo.style(this._applyButton.domNode, "display", "none");
    else 
      dojo.style(this._applyButton.domNode, "display", "inline-block");
  }
});


}

if(!dojo._hasResource["dijit.layout.LayoutContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.LayoutContainer"] = true;
dojo.provide("dijit.layout.LayoutContainer");



dojo.declare("dijit.layout.LayoutContainer",
	dijit.layout._LayoutWidget,
	{
	// summary:
	//		Deprecated.  Use `dijit.layout.BorderContainer` instead.
	//
	// description:
	//		Provides Delphi-style panel layout semantics.
	//
	//		A LayoutContainer is a box with a specified size (like style="width: 500px; height: 500px;"),
	//		that contains children widgets marked with "layoutAlign" of "left", "right", "bottom", "top", and "client".
	//		It takes it's children marked as left/top/bottom/right, and lays them out along the edges of the box,
	//		and then it takes the child marked "client" and puts it into the remaining space in the middle.
	//
	//		Left/right positioning is similar to CSS's "float: left" and "float: right",
	//		and top/bottom positioning would be similar to "float: top" and "float: bottom", if there were such
	//		CSS.
	//
	//		Note that there can only be one client element, but there can be multiple left, right, top,
	//		or bottom elements.
	//
	// example:
	// |	<style>
	// |		html, body{ height: 100%; width: 100%; }
	// |	</style>
	// |	<div dojoType="dijit.layout.LayoutContainer" style="width: 100%; height: 100%">
	// |		<div dojoType="dijit.layout.ContentPane" layoutAlign="top">header text</div>
	// |		<div dojoType="dijit.layout.ContentPane" layoutAlign="left" style="width: 200px;">table of contents</div>
	// |		<div dojoType="dijit.layout.ContentPane" layoutAlign="client">client area</div>
	// |	</div>
	//
	//		Lays out each child in the natural order the children occur in.
	//		Basically each child is laid out into the "remaining space", where "remaining space" is initially
	//		the content area of this widget, but is reduced to a smaller rectangle each time a child is added.
	// tags:
	//		deprecated

	baseClass: "dijitLayoutContainer",

	constructor: function(){
		dojo.deprecated("dijit.layout.LayoutContainer is deprecated", "use BorderContainer instead", 2.0);
	},

	layout: function(){
		dijit.layout.layoutChildren(this.domNode, this._contentBox, this.getChildren());
	},

	addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
		this.inherited(arguments);
		if(this._started){
			dijit.layout.layoutChildren(this.domNode, this._contentBox, this.getChildren());
		}
	},

	removeChild: function(/*dijit._Widget*/ widget){
		this.inherited(arguments);
		if(this._started){
			dijit.layout.layoutChildren(this.domNode, this._contentBox, this.getChildren());
		}
	}
});

// This argument can be specified for the children of a LayoutContainer.
// Since any widget can be specified as a LayoutContainer child, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// layoutAlign: String
	//		"none", "left", "right", "bottom", "top", and "client".
	//		See the LayoutContainer description for details on this parameter.
	layoutAlign: 'none'
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPViewChanger"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPViewChanger"] = true;
	/******************************************************* {COPYRIGHT-TOP-OCO} ***
	 * Licensed Materials - Property of IBM
	 *
	 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
	 *
	 * US Government Users Restricted Rights - Use, duplication, or
	 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
	 ******************************************************* {COPYRIGHT-END-OCO} ***/
	
	
	
	
	dojo.require("dijit.layout.ContentPane" );
	
	dojo.provide("ibm.tivoli.tip.dijit.TIPViewChanger");
	/****************************************************************************
	* View Changer Widget
	*    This widget is responsible for providing a toolbar with a dropdown 
	*    button that allows for changing views beneath the toolbar.  It's kinda
	*    like a card layout, in which views are context switched based on the
	*    dropdown selection.
	*
	*    In addition, each view can contribute toolbar icons when inacted.
	*
	* Proud author: Joseph Firebaugh
	****************************************************************************/
	dojo.declare("ibm.tivoli.tip.dijit.TIPViewChanger",
	              dijit.layout.LayoutContainer,
	{
	  viewNames:     [ "viewNames not set" ],
	  viewTooltips:  [ "viewTooltips not set" ],
	  viewIconClass: null,
	  
	  defaultView: 0,
	  	
	  /****************************************************************************
	  * PostCreate
	  *    this method is called after dojo builds the DOM node for this widget
	  *    we will build the contents of this widget programatically.
	  ****************************************************************************/
	  postCreate: function() 
	  {
	    this.inherited( arguments );
		
		//****************************************
	    //* create the toolbar area for the 
	    //* drop down button view changer
	    //****************************************	
	    var div = dojo.doc.createElement('div');
	    this.viewChangerToolbar = new dijit.Toolbar( { layoutAlign: 'top' }, div );
	    this.addChild( this.viewChangerToolbar );
	    
	    this.dropDownMenu = new dijit.Menu({});
	    this.dropDownMenu.domNode.style.display="none";  
	    //****************************************
	    //* create the dropdown menu for the 
	    //* change view button from all view names
	    //****************************************
	    this.menuItems = [];
	    var params = null;
	    for ( var i = 0; i < this.viewNames.length; i++ )
	    {
		    params = {
		    	label: this.viewNames[i],
		    	onClick: dojo.hitch( this, "changeView", i )
		    };    
		    
		    if ( this.viewIconClass !== null )
		    {
		    	params.iconClass = this.viewIconClass + i;
		    }	    
		    
		    this.menuItems[i] = new dijit.MenuItem( params );
		    this.dropDownMenu.addChild( this.menuItems[i] );
	    }
	    
	    //****************************************
	    //* Add the drop down button
	    //****************************************
	    div = dojo.doc.createElement('div');
	    params = {
	    	title: this.viewTooltips[ this.defaultView ],
	    	label: this.viewNames[ this.defaultView ],
	    	id: this.id + "viewChangeBtn",
	    	dropDown: this.dropDownMenu    	
	    };
	      
	    this.viewChangeBtn = new dijit.form.DropDownButton( params, div );
	    
	  	this.viewChangeBtn.containerNode.style.fontFamily = "Verdana,Arial,Helvetica,sans-serif";
	  	this.viewChangeBtn.containerNode.style.fontSize   = "1em";
	    
	    this.viewChangerToolbar.addChild( this.viewChangeBtn ); 
	        
	    setTimeout( dojo.hitch( this, "changeView", this.defaultView ), 500 );
	  },
	  
	  /****************************************************************************
	  * changeView
	  *    this method is called when a user choses a view to display from the
	  *    drop down button's context menu.  The index of the view is passed in
	  ****************************************************************************/
	  changeView: function( viewNum )
	  {
	  	//this changes the label on the drop down button so that the current 
	  	//visible view is the button label -- serving as a "title" of the view
	  	//artificually adding a space between label and icon for better appearances
	  	this.viewChangeBtn.setLabel( "&nbsp;" + this.viewNames[ viewNum ] );
		
		//remove the icon off the button so that it can be replaced with the next view icon
		if ( this.lastIconClass )
		{
	  		dojo.removeClass( this.viewChangeBtn.iconNode, this.lastIconClass );
		}
		
	  	var iconClass;
		if ( this.viewIconClass !== null )
		{
		  iconClass = this.viewIconClass + viewNum;
	
	  	  //add the CSS style to the button for the new icon
	  	  dojo.addClass( this.viewChangeBtn.iconNode, iconClass );
	  	  
	  	  //save a reference to the icon to be removed later if view changes
	  	  this.lastIconClass = iconClass;
		}	    
	  	
	  	//if a view is currently visible then destroy it now.
	  	if ( this.currentView )
	  	{
	  		try
	  		{
	  			this.removeChild( this.currentView );
	  			this.viewRemoved( this.currentView );
	  		}
	  		catch( e )
	  		{
	  			console.debug( e );
	  		}
	  	}
	  	  	
	    this.currentView = this.getViewWidget( viewNum );
	    
	    this.currentView.layoutAlign = "client";  	
	    
	    //record the index of the new view for eventing lookup
	    this.currentViewNum = viewNum;
	        
	    //add the new view to the layout container
	   	this.addChild( this.currentView );
	   	   	
	   	//add toolbar contributions from new view
	   	if ( this.currentView.contributeToolbarItems )
	   	{
	   	  this.currentView.contributeToolbarItems( this.viewChangerToolbar );			        	
	   	}	
	  },
	  
	  /****************************************************************************
	  * getViewWidget
	  *    this method should be overloaded by the using application to provide
	  *    each toggled view.  Preferrably this view widget will extend ContentPane
	  ****************************************************************************/
	  getViewWidget: function( viewNum ){ return new dijit.layout.ContentPane(); },
	  
	  /****************************************************************************
	  * viewRemoved
	  *    this method should be overloaded by the using application if you wish 
	  *    to destroy each view as they are removed.  
	  ****************************************************************************/
	  viewRemoved: function( viewWidget ){},
	
	  /****************************************************************************
	  * destroy
	  *    A well designed widget must implement a destroy method to clean up
	  *    anything necessary
	  ****************************************************************************/
	  destroy: function() 
	  {
	     if ( this.currentView )
	     {
	     	this.currentView.destroy();
	     }
	     
	     for ( var i in this.menuItems )
	     {
	     	if (i) {
	     		this.menuItems[ i ].destroy();
	     	}
	     }
	     
	     this.dropDownMenu.destroy();
	     this.viewChangeBtn.destroy();
	     this.viewChangerToolbar.destroy();
	     
	     this.inherited( "destroy", arguments );
	  }
	});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TivWizard"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TivWizard"] = true;
console.warn("'ibm.tivoli.tip.dijit.TivWizard' is deprecated, use 'dojoe.wizard.Wizard'(TED) instead.");
dojo.provide("ibm.tivoli.tip.dijit.TivWizard");








/**
 * Inherit from TivPageContainer which contains the common functions of both
 * the notebook and wizard.
 */


/**
 * 
 * This class is an extension of TivPageContainer that handles all the
 * wizard specific functions as specified in the UI guidelines.  The table
 * of contents layout may be vertical or horizontal.  It also includes the 
 * advanced navigation features such as the ability to have sub-steps, 
 * sub-steps that loop and to jump backwards to a previous page by clicking 
 * on an item in the table of contents.
 * 
 * Extra capabilities that have been added include the ability to skip from 
 * whatever page is displayed to the next main step, enable and disable steps
 * and buttons, hide the button area, indicate a step is required and restart 
 * the wizard. 
 * 
 * Creating a TivWizard in markup
 * <div id="note" dojoType="ibm.tivoli.tip.dijit.TivWizard">
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.Welcome"></div>
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.BaseWidget"></div>
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.General"></div>
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.Security"></div>
 *    <div dojoType="ibm.tivoli.tip.dijit.tests.tivpages.Summary" ></div>
 *  </div> 
 *
 * Events:
 *  OnPageSelected: when a page is made active
 *  OnBack: when the Back button is clicked
 *  OnNext: when the Next button is clicked
 *  OnOK: when the Finish/OK button is clicked
 *  OnCancel: when the Cancel button is clicked
 */
dojo.declare("ibm.tivoli.tip.dijit.TivWizard", 
            [dijit._Widget, dijit._Templated, ibm.tivoli.tip.dijit.TivPageContainer], 
{  
  widgetsInTemplate: true,
  templatePath: "",

  mode: "w",               // wizard
  renderNoArrows: false,   // if true render ToC without arrow graphics, only applicable in horizontal layout
  slider: null,            // for scrolling ToC, only applicable in horizontal layout
  tocStepsArea: null,      // for main steps of ToC, only applicable in horizontal layout
  tocSubstepsArea: null,   // for sub steps of ToC, only applicable in horizontal layout
  lastPageSelected: null,  // keep track of the last page that was selected
  
  backLabel: "",
  nextLabel: "",
  finishLabel: "",
  cancelLabel: "",
  
  postMixInProperties: function() 
  {
    console.log("TivWizard.postMixInProperties");    
    if (this.nested)
      this.templateString="<div class=\"tiv-wiz\">\r\n  <div dojoType=\"dijit.layout.BorderContainer\" design=\"headline\" gutters=\"false\" class=\"tiv-wiz-outer\" dojoAttachPoint=\"_outerContainer\">\r\n    <div dojoType=\"dijit.layout.BorderContainer\" gutters=\"false\" liveSplitters=\"false\" design=\"sidebar\" region=\"center\" style=\"width:100%;height:100%;\" dojoAttachPoint=\"_borderArea\">\r\n      <div dojoType=\"dijit.layout.ContentPane\" region=\"leading\" splitter=\"false\" class=\"tiv-wiz-toc\" dojoAttachPoint=\"_tocArea\">\r\n        <ol dojoAttachPoint=\"_toc\"></ol>\r\n      </div>\r\n      <div dojoType=\"dijit.layout.StackContainer\" region=\"center\" splitter=\"false\" dojoAttachPoint=\"_pageContainer\" class=\"tiv-wiz-work-area\"> \r\n        <div dojoAttachPoint=\"containerNode\"></div>\r\n      </div>\r\n    </div>\r\n    <div dojoType=\"dijit.layout.ContentPane\" region=\"bottom\" splitter=\"false\" class=\"tiv-wiz-footer\" dojoAttachPoint=\"_buttonArea\">\r\n      <div class=\"tiv-wiz-buttons\" dojoAttachPoint=\"_buttons\">\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickBack\" dojoAttachPoint=\"_backButton\">${backLabel}</button>\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickNext\" dojoAttachPoint=\"_nextButton\">${nextLabel}</button>\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickFinish\" dojoAttachPoint=\"_finishButton\">${finishLabel}</button>\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickCancel\" dojoAttachPoint=\"_cancelButton\">${cancelLabel}</button>\r\n      </div>\r\n    </div>\r\n  </div>\r\n</div>\r\n";
    else
      this.templateString="<div class=\"tiv-wiz\">\r\n  <div dojoType=\"dijit.layout.BorderContainer\" design=\"headline\" gutters=\"false\" class=\"tiv-wiz-outer\" dojoAttachPoint=\"_outerContainer\">\r\n    <div dojoType=\"dijit.layout.BorderContainer\" gutters=\"false\" liveSplitters=\"false\" design=\"sidebar\" region=\"center\" style=\"width:100%;height:100%;\" dojoAttachPoint=\"_borderArea\">\r\n      <div dojoType=\"dijit.layout.ContentPane\" region=\"leading\" splitter=\"false\" class=\"tiv-wiz-toc\" dojoAttachPoint=\"_tocArea\">\r\n        <ol dojoAttachPoint=\"_toc\"></ol>\r\n      </div>\r\n      <div dojoType=\"dijit.layout.StackContainer\" region=\"center\" splitter=\"false\" dojoAttachPoint=\"_pageContainer\" class=\"tiv-wiz-work-area\">\r\n        <form id=\"${id}_form\" dojoAttachPoint=\"containerNode\" action=\"${action}\" method=\"${method}\"></form>              \r\n      </div>\r\n    </div>\r\n    <div dojoType=\"dijit.layout.ContentPane\" region=\"bottom\" splitter=\"false\" class=\"tiv-wiz-footer\" dojoAttachPoint=\"_buttonArea\">\r\n      <div class=\"tiv-wiz-buttons\" dojoAttachPoint=\"_buttons\">\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickBack\" dojoAttachPoint=\"_backButton\">${backLabel}</button>\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickNext\" dojoAttachPoint=\"_nextButton\">${nextLabel}</button>\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickFinish\" dojoAttachPoint=\"_finishButton\">${finishLabel}</button>\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onClickCancel\" dojoAttachPoint=\"_cancelButton\">${cancelLabel}</button>\r\n      </div>\r\n    </div>\r\n  </div>\r\n</div>\r\n";
    console.log("templatePath is " + this.templatePath);
    
    var string_table = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
    this.backLabel = string_table.BACK;
    this.nextLabel = string_table.NEXT;
    this.finishLabel = string_table.FINISH;
    this.cancelLabel = string_table.CANCEL;
    this.inherited(arguments);
  },
  
  buildRendering: function()
  {
    console.log("TivWizard.buildRendering");        

    this.inherited(arguments);

    // if ToC layout is horizontal... 
    if ('h' === this.layout) {
      var child = this._tocArea;
      
      // add main step area
      this.tocStepsArea = dojo.create("div", {});          
      dojo.place(this.tocStepsArea, child.domNode, 0);
      
      if (this.renderNoArrows) {
        dojo.addClass(this.tocStepsArea, "steps-area-noarrows");            
        dojo.place(this._toc, this.tocStepsArea, 0);
      }
      else {
        dojo.addClass(this.tocStepsArea, "steps-area");            
        
        // create slider components which are hidden until needed
        this.slider = new ibm.tivoli.tip.dijit.TivWizardToCSlider({
          toc: this._toc, 
          tocSteps: this.tocStepsArea, 
          ltr: this.isLeftToRight()}, null);
      }
      
      // add hidden sub-step area
      this.tocSubstepsArea = dojo.create("div", {className: "substeps-area"});
      dojo.place(this.tocSubstepsArea, child.domNode, 1);          
    }    
  },
  
  _setContainerType: function(page)
  {
    page.attr("wizard", this);
  },
  
  updateUI: function()
  {        
    // disable the Back/Next buttons if we are on the first/last page
    var curr_page = this._pageContainer.selectedChildWidget;
    var page_count = this._pages.length;

    var isFirst = true, isLast = true;
    if(page_count > 0) {
      isFirst = curr_page === this._pages[0];
      isLast =  curr_page === this._pages[this._pages.length-1];
    }
    this._backButton.attr("disabled", isFirst);
    this._nextButton.attr("disabled", isLast);
        
    // disable the Next button if the selected page does not validate
    var isCurrentPageValid = curr_page && curr_page.isValid(); 
    if( !isCurrentPageValid )
      this._nextButton.attr("disabled", true);    

    // disable the Finish/OK button until all pages are valid
    var allValid = dojo.every(this._pages, function(p){
      var v = p.isValid();
      v = v && (p.visited || p === curr_page);
      return v;
     }, this);
    this._finishButton.attr("disabled", !allValid);
    
    this.inherited(arguments);

    if ('h' === this.layout) {     
      // update UI for all substeps
      // substeps may be hidden due to jump back and need to be updated too
      dojo.forEach(this._pages, function(aPage) {
        if (aPage.hasSubsteps()) {
          dojo.query("li[pageID]", aPage.getSubstepsList()).forEach(function(n, i) {
            var pid = dojo.attr(n, "pageID");
            var p = dijit.byId(pid);
            dojo[(p && p.visited) ? "addClass" : "removeClass"](n, "tiv-page-visited");
            dojo[(p && (!p.visited || p.validated)) ? "removeClass" : "addClass"](n, "tiv-page-invalid");
            dojo[(p == curr_page) ? "addClass" : "removeClass"](n, "tiv-page-selected");                    
          }, this);
        }
      }, this);
      
      // if current page is a substep update main step to look selected
      if (!curr_page.isMainStep()) {
        var mainPage = curr_page.getMainPage();
        dojo.query("li[pageID]", this._toc).every(function(n, i) {
          var pid = dojo.attr(n, "pageID");
          if (pid == mainPage.id) {
            dojo.addClass(n, "tiv-page-selected");
            return false;
          }
          else
            return true;
        });              
      }
      
      // show or hide substeps area
      var newHeight;
      if (curr_page.hasSubsteps() || !curr_page.isMainStep())
        newHeight = this.tocStepsArea.offsetHeight + this.tocSubstepsArea.offsetHeight;
      else
        newHeight = this.tocStepsArea.offsetHeight;
      if (dojo.style(this._pageContainer.domNode, "top") != newHeight)
      {
        dojo.style(this._pageContainer.domNode, "top", newHeight + "px");        
        dojo.style(this._tocArea.domNode, "height", newHeight + "px");
      }
      
      if (this.slider != null) {
        // calculate required width for all step arrows in ToC
        this.slider.recalcToCWidth();
      
        // update slider
        this.slider.updateSliderUI(null);      
      }
    }
  },
  
  /**
   * Overrides base function to also resize the slider.  
   * (Note: for updating the slider, cannot rely on connecting a handler
   * to the resize event for the _tocArea div.  Problems occur in Firefox
   * where the event is not fired often enough so sometimes the slider
   * button will disappear.)
   */
  resize: function(newSize) 
  {
    if (newSize != null) {
      this.inherited(arguments);
      if (this.slider && newSize.w != null)
        this.slider.updateSliderUI(newSize.w);    
    }
  },
  
  /** 
   * Restart the wizard
   * 
   * Enable jumping backwards (may have been disabled)
   * 
   * Show the buttons (may have been hidden)
   * 
   * Remove the "visited" flag
   * 
   * Collapse substeps
   * 
   * Give each page a chance to reset to the original values
   * 
   * Reset to the first page
   */
  restartWizard: function() 
  {
    console.log("TivWizard.restartWizard");

    if (this.slider != null)
      this.slider.reset();
    
    // the bottom button area might have been hidden
    this.showAllButtons();

    // individual buttons might have been hidden
    this.hideBackButton(false);
    this.hideNextButton(false);
    this.hideCancelButton(false);
    this.hideFinishButton(false);

    // or maybe disabled
    this.enableNextButton(true);
    this.enableCancelButton(true);
    this.enableFinishButton(false);

    // get all of the children
    var theKids = this._pageContainer.getChildren();

    // do for all children
    for (var i = 0; i < theKids.length; i++) {
      // mark as not visited
      theKids[i].visited = false;

      // hide any kids
      theKids[i].hideSubsteps();

      // if the child supports the "reset" method
      if ("function" == typeof theKids[i].reset) {

        // give the kid a chance to reset
        theKids[i].reset();
      }
    }

    // the TOC might have been disabled
    this.enableJumpBack();

    // and then select the first page
    this.selectPage(theKids[0]);
  },

  /**
   * Reset the label for the "Finish" button
   */
  resetFinishLabel: function(newName) 
  {
    console.log("TivWizard.resetFinishLabel - " + newName);
    dojo.attr(this._finishButton, "label", newName);
  },

  /**
   * Mark pages as having been visited up to the one specified
   */
  markPreviousPagesVisited: function(aPage) 
  {
    console.log("TivWizard.markPreviousPagesVisited");

    // get all of the children
    var theKids = this._pageContainer.getChildren();

    // do for all children
    var foundPage = false;
    for (var i = 0; !foundPage && i < theKids.length; i++) {
      // if this is not the page specified
      if (aPage != theKids[i]) {
        // if the kid is not disabled
        if (theKids[i].isDisabled() != true) {

          // mark as visited
          theKids[i].visited = true;
        }

        // else this is the end of the line
      } 
      else {
        // mark as found
        foundPage = true;
      }
    }
  },

  /**
   * Disable jumping back to every page except for the one specified
   */
  disableJumpBack: function(exceptFor) 
  {
    console.log("TivWizard:disableJumpBack");

    // get all of the children
    var theKids = this._pageContainer.getChildren();

    // do for all children
    for (var i = 0; i < theKids.length; i++) {

      // if this is not the exception)
      if (exceptFor != theKids[i]) {

        // disable it
        theKids[i].disableJumpBack();
      }
    }
  },

  /**
   * Enable jumping back to every page
   */
  enableJumpBack: function() 
  {
    console.log("TivWizard.enableJumpBack");

    // get all of the children
    var theKids = this._pageContainer.getChildren();

    // do for all children
    for (var i = 0; i < theKids.length; i++) {
      // enable it
      theKids[i].enableJumpBack();
    }
  },

  /**
   * When a page is being added it might be a substep. If so it needs to
   * be added to the previous main step.
   */
  _onPageAdded: function(page, insertIndex) 
  {
    console.log("TivWizard._onPageAdded");

    this.inherited(arguments);

    // if this is a substep
    if (page.isMainStep() == false) {
      // get the substep's main page
      var mainPage = page.getMainPage();
      if (mainPage != null) {
        // add as a substep
        mainPage.addSubstep(page);
      } 
      else {
        // message
        console.error("failed to add substep to main page");
      }
    }
  },
  
  /*
   * Gets the previous main page.  If page is a substep, gets the main
   * page for this substep.  Note the page may be disabled.
   */
  getPreviousMainPage: function(page)
  {
    // get the previous page
    var previousPage = this._pageContainer._getSiblingOfChild(page, -1);

    // while the previous page is a substep
    while (previousPage.isMainStep() == false) {
      // get the previous page
      previousPage = this._pageContainer._getSiblingOfChild(previousPage, -1);
    }
    
    // if the previous page is a main page
    if (previousPage.isMainStep() == false) {
      console.log("no previous main page");
      previousPage = null;
    }
    return previousPage;
  },
  
  /**
   * Gets the next main page.  Note the page may be disabled.
   */
  getNextMainPage: function(page)
  {
    // get the next page
    var nextPage = this._pageContainer._getSiblingOfChild(page, 1);

    // while the next page is a substep
    while (nextPage.isMainStep() == false) {
      // get the next page
      nextPage = this._pageContainer._getSiblingOfChild(nextPage, 1);
    }
    
    // if the next page is a main page
    if (nextPage.isMainStep() == false) {
      console.log("no next main page");
      nextPage = null;
    }
    return nextPage;    
  },
  
  /**
   * Jumps to the next enabled step in the loop which may be a substep
   * or the main step.
   */
  jumpToNextLoopStep: function()
  {
    console.log("TivWizard.jumpToNextLoopStep");
    var nextPage = null;
    
    // get the current page
    var tempPage = this.lastPageSelected;
    
    if (tempPage.isMainStep())
    {
      // next step must be a substep so return first enabled substep
      if (tempPage.hasSubsteps())
      {
        dojo.every(tempPage.getSubsteps(), function(substep) {
          if (!substep.isDisabled()) {
            nextPage = substep;
            return false;
          }
          else
            return true;
        }, this);
      }
    }
    else {
      // current is a substep, get the next substep
      var nextPossible = this._pageContainer._getSiblingOfChild(tempPage, 1);

      // while the next page is a substep and disabled
      while (nextPossible.isMainStep() == false && nextPossible.isDisabled()) {
        // get the next page
        nextPossible = this._pageContainer._getSiblingOfChild(nextPossible, 1);
      }
      
      // if next page is a substep save it for selection
      if (nextPossible != null && !nextPossible.isMainStep())
        nextPage = nextPossible;
      else {
        // otherwise next page is a main page, make sure we
        // loop around and load the main page in this loop
        nextPage = tempPage.getMainPage();
      }
    }

    if (nextPage != null) {
      // select the new page
      this.selectPage(nextPage);
    }
  },
  
  /**
   * Jumps to the previous enabled step in the loop which may be a substep
   * or the main step.  Note if the current page is the main step, the jump
   * will not occur.
   */
  jumpToPreviousLoopStep: function()
  {
    console.log("TivWizard.jumpToPreviousLoopStep");
    
    // get the current page
    var tempPage = this.lastPageSelected;
     
    if (!tempPage.isMainStep()) {
      // get the previous page
      var previousPossible = this._pageContainer._getSiblingOfChild(tempPage, -1);
      
      // while the next page is a substep and disabled
      while(previousPossible.isMainStep() == false && previousPossible.isDisabled()) {
        // get the previous page
        previousPossible = this._pageContainer._getSiblingOfChild(previousPossible, -1);
      }
      
      if (previousPossible != null) {
        // select the new page
        this.selectPage(previousPossible);
      }
    }
  },

  /**
   * Using the skip function takes you to the next main page that is enabled. 
   * But you still want to hide any substeps if you happen to be in a main
   * page with substeps, and you want to hide all of the substeps for a main
   * page if you happen to be in a substep.
   * 
   * Also if the next main step contains substeps but the main step should not
   * show its page, the first enabled substep page will be selected.
   */
  skipToNextMainStep: function() 
  {
    console.log("TivWizard:skipToNextMainStep");

    // get the current page
    var tempPage = this.lastPageSelected;

    // assume it is a main step
    var mainPage = tempPage;

    // while still on a substep
    while (mainPage.isMainStep() == false) {
      // back up another page
      mainPage = this._pageContainer._getSiblingOfChild(mainPage, -1);
    }

    // hide any substeps
    mainPage.hideSubsteps();

    // mark it as visited
    tempPage.visited = true;

    // find the next page
    var whoIsNext = this._pageContainer._getSiblingOfChild(tempPage, 1);

    // while that page is a substep
    while (whoIsNext != null && (whoIsNext.isMainStep() == false || whoIsNext.isDisabled())) {
      // find the next page
      whoIsNext = this._pageContainer._getSiblingOfChild(whoIsNext, 1);
    }
    
    // select the new page
    if (whoIsNext != null) {
      // check if we need to show substep directly
      if (whoIsNext.showTitleOnly())
        this.goToSubstep(whoIsNext);
      else 
        this.selectPage(whoIsNext);
    }     
  },

  /**
   * The user has clicked on something in the table of contents, so the
   * container needs to display that page.
   * 
   * Hide any substeps of the jump off page if the page is not a substep
   * of the jump off page.
   * 
   * Let the developer decide when to mark the pages invalid.
   * 
   * This also means that the page where the jump originated needs to be
   * marked as visited in order to return to that page.  This can be
   * overridden if allowMarkAsComplete returns false.
   */
  jumpBackToPage: function(aPage) 
  {
    console.log("TivWizard.jumpBackToPage");

    // the jump off point might have substeps or be a substep
    // hide them as long as page to jump to isn't one of those substeps
    var jumpOffMainPage = (this.lastPageSelected.isMainStep()) ? this.lastPageSelected
                                                               : this.lastPageSelected.getMainPage();
    var jumpToMainPage = (aPage.isMainStep()) ? aPage : aPage.getMainPage();    
    if (jumpOffMainPage != jumpToMainPage)
      jumpOffMainPage.hideSubsteps();       

    // mark where we were as having been visited
    if (this.lastPageSelected.allowMarkAsComplete())
      this.lastPageSelected.visited = true;
    
    // check if we need to show substep directly
    if (aPage.showTitleOnly())
      this.goToSubstep(aPage);
    else {
      // then let the parent class handle selecting the page
      this.selectPage(aPage);
    }
  },
  
  /**
   * Selects the first enabled substep for the given main page.
   */
  goToSubstep: function(mainPage)
  {
    console.log("TivWizard.goToSubstep");
    mainPage.showSubsteps();
    dojo.every(mainPage.getSubsteps(), function(substep) {
      if (!substep.isDisabled()) {
        this.selectPage(substep);
        return false;
      }
      else 
        return true;
    }, this);    
  },

  /**
   * Hide all of the buttons by adding a class
   */
  hideAllButtons: function() 
  {
    console.log("TivWizard.hideAllButtons");

    dojo.addClass(this._buttonArea.domNode, "tiv-wiz-footer-hidden");
  },

  /**
   * Show all of the buttons by removing a class
   */
  showAllButtons: function() 
  {
    console.log("TivWizard.hideAllButtons");

    dojo.removeClass(this._buttonArea.domNode, "tiv-wiz-footer-hidden");
  },

  /** 
   * Hide (or show) the "back" button 
   */
  hideBackButton: function(shouldHide) 
  {
    if (shouldHide) 
      dojo.style(this._backButton.domNode, "display", "none");
    else 
      dojo.style(this._backButton.domNode, "display", "inline-block");
  },

  /**
   * Hide (or show) the "next" button
   */
  hideNextButton: function(shouldHide) 
  {
    if (shouldHide) 
      dojo.style(this._nextButton.domNode, "display", "none");
    else 
      dojo.style(this._nextButton.domNode, "display", "inline-block");
  },

  /**
   * Enable (or disable) the "back" button
   */
  enableBackButton: function(shouldEnable) 
  {
    this._backButton.attr("disabled", !shouldEnable);
  },

  /**
   * Enable (or disable) the "next" button
   */
  enableNextButton: function(shouldEnable) 
  {
    this._nextButton.attr("disabled", !shouldEnable);
  },

  /**
   * If a change is made to a page that was reached by jumping back,
   * that might cause subsequent pages to have information or choices
   * that are no longer valid.
   * 
   * Call this function to invalidate the selected page and all
   * subsequent pages.
   */
  unmarkVisitedPages: function(aPage) 
  {
    while (aPage != null) {
      aPage.visited = false;
      aPage = this._pageContainer._getSiblingOfChild(aPage, 1);
    }
    this.updateUI();
  },

  /**
   * There are some differences when a page is selected.
   * 
   * This is where the last page selected is reset.
   * 
   * The selected page should not be marked as having been visited, a
   * change from the parent class.
   */
  _onPageSelected: function(page) 
  {
    console.log("TivWizard._onPageSelected");

    // keep track of which page was last selected
    this.lastPageSelected = page;

    // let the parent class handle this
    this.OnPageSelected(this, page);

    // if the page can handle this function
    if ("function" == typeof page.OnPageSelected) {
      // let the page know it has been selected
      page.OnPageSelected(this);
    }

    // update the display
    this.updateUI();

    // if the page can handle this function
    if ("function" == typeof page.AfterPageSelected) {
      // let the page get final control
      page.AfterPageSelected(this);
    }
  },

  /**
   * When "Finish" is clicked any substeps that are open should be
   * hidden
   */
  _onClickFinish: function(event) 
  {
    console.log("TivWizard._onClickFinish");

    // get the page current in view
    var tempPage = this.lastPageSelected;

    // see if the page wants to allow the customer to move on
    var shouldAllow = tempPage.allowFinish();

    // if so
    if (shouldAllow) {

      // while this is a substep
      while (tempPage.isMainStep() == false) {
        // get the page preceding this one
        tempPage = this._pageContainer._getSiblingOfChild(tempPage, -1);
      }

      // then hide any substeps
      tempPage.hideSubsteps();

      // and allow any additional processing to occur
      this.OnOK(this);
    }
  },

  /**
   * This has a number of differences from the parent class.
   * 
   * If a page has substeps, then they should be hidden from view at
   * this time.
   * 
   * If the previous page is really a substep then the page that should
   * be shown is the parent of that substep.
   * 
   * When backing up to the parent of a substep the substeps should be
   * marked as not having been visited.
   * 
   * Disabled pages should be avoided.
   * 
   * Changing behavior:
   * - To completely override the behavior of the Back button, have
   * allowBack() of current page return false and connect customized 
   * handler function to _onClickBack event.
   * - To perform additional processing after previous page has been
   * selected, connect customized handler function to OnBack event.
   */
  _onClickBack: function(evt) 
  {
    console.log("TivWizard._onClickBack");
    
    // allow any additional processing to occur
    this.OnBack(this);
    
    // get the current position
    var tempPage = this.lastPageSelected;
    
    // see if the page wants to allow the customer to move back
    var shouldAllow = tempPage.allowBack();
    
    // if so
    if (shouldAllow) {
      
      // get the next page
      var whoIsNext = tempPage;
      whoIsNext = this._pageContainer._getSiblingOfChild(whoIsNext, 1);   
      
      // if the next page is marked as unvisited (no jump back)
      if (whoIsNext != null && !whoIsNext.visited) {
        // mark current page as unvisited
        tempPage.visited = false;      
      }
  
      var startingPage = tempPage;
      
      // if the current page is not a substep
      if (tempPage.isMainStep() == true) {
  
        // hide any substeps for this page
        tempPage.hideSubsteps();         
      }
      else {
        // current page is a substep
        var mainPage = tempPage.getMainPage();
		//Vyshantha: April7,2011: Fix for the back button issue on TivWizard
		if (whoIsNext != null && !whoIsNext.visited) {
          // mark main step as unvisited as long as no jump back
          mainPage.visited = false;
        }
        
        if (mainPage.showTitleOnly()) {
          // get the previous enabled page
          var previous = this._pageContainer._getSiblingOfChild(tempPage, -1);
          while (previous != null && previous.isDisabled())
              previous = this._pageContainer._getSiblingOfChild(previous, -1);      
          
          // if previous page is this substep's main page, then skip it
          if (previous == mainPage) {
            startingPage = previous;
            mainPage.hideSubsteps();
          }
        }
      }
      
      // get the previous page that is not disabled
      var whoIsPrevious = this._pageContainer._getSiblingOfChild(startingPage, -1);
      
      // while that page is marked disabled or if page is a substep and its
      // main page is disabled
      while (whoIsPrevious != null && (whoIsPrevious.isDisabled() || 
       !whoIsPrevious.isMainStep() && whoIsPrevious.getMainPage().isDisabled())) {
          // get the previous page
          whoIsPrevious = this._pageContainer._getSiblingOfChild(whoIsPrevious, -1);      
      }
          
      // if there is still a page to load
      if (whoIsPrevious != null) {      
        
        // if that page is a substep 
        if (whoIsPrevious.isMainStep() == false) {
          
          // keep track of substep
          var subStepToSelect = whoIsPrevious;
          
          // get previous main page
          whoIsPrevious = subStepToSelect.getMainPage();
          
          // if main page is not part of a loop
          // (otherwise select main page)
          if (!whoIsPrevious.isPartOfLoop()) {
            // make all substeps for this page visible then the substep can be selected
            whoIsPrevious.OnPageSelected(this);
            
            // reset page to be selected to our substep
            whoIsPrevious = subStepToSelect;
          }        
        }
        
        // select that page
        this.selectPage(whoIsPrevious);
      }
  
      // mark this page widget as not having been visited if
      // we didn't get here from jumping back
      if (!tempPage.visited)
        this._pageContainer.selectedChildWidget.visited = false;     
      
    }
  },

  /** 
   * Moving to the next page is slightly different if there are substeps
   * involved.
   * 
   * Substeps should only be seen when their parent is the last main
   * step to be visited. They need to disappear when the next main step
   * is entered.
   * 
   * Disabled pages should be avoided.
   * 
   * Changing behavior:
   * - To completely override the behavior of the Next button, have
   * allowNext() of current page return false and connect customized 
   * handler function to _onClickNext event.
   * - To perform additional processing after next page has been
   * selected, connect customized handler function to OnNext event.
   */
  _onClickNext: function (evt) 
  {
    console.log("TivWizard._onClickNext");
    
    // allow any additional processing to occur
    this.OnNext(this);

    // get the page current in view
    var tempPage = this.lastPageSelected;

    // see if the page wants to allow the customer to move on
    var shouldAllow = tempPage.allowNext();
    
    // if so
    if (shouldAllow) {
      
      // if current page in view has no substeps, mark the page as 
      // having been visited
      if (!tempPage.hasSubsteps())
        tempPage.visited = true;

      // get the next page
      var nextPossible;    
      if (tempPage.isPartOfLoop()) {
        // current page is part of a loop so get next main page
        nextPossible = this.getNextMainPage(tempPage);
      }
      else
        nextPossible = this._pageContainer._getSiblingOfChild(tempPage, 1);

      // while that page is marked disabled
      while (nextPossible != null && nextPossible.isDisabled()) {
        // if disabled page has substeps
        if (nextPossible.hasSubsteps()) {
          // get next main page
          nextPossible = this.getNextMainPage(nextPossible);
        }
        else  {
          // get the next page
          nextPossible = this._pageContainer._getSiblingOfChild(nextPossible, 1);
        }
      }

      // if there is still a next page to load
      if (nextPossible != null) {
        
        // if next page is a main page and current page is a substep or main page for loop
        if (nextPossible.isMainStep() && (!tempPage.isMainStep() || tempPage.isPartOfLoop())) {
          
          // get the current main page
          var currentMainPage;
          
          if (tempPage.isPartOfLoop()) {
            if (tempPage.isMainStep())
              currentMainPage = tempPage;
            else 
              currentMainPage = tempPage.getMainPage();
            
            // mark all loop substeps as visited
            dojo.forEach(currentMainPage.getSubsteps(), function(page) {
              page.visited = true;
            }, this);
          }
          else {
            currentMainPage = tempPage.getMainPage();          
          }
          
          // hide substeps for that main step
          currentMainPage.hideSubsteps();
          
          // mark the main step as having been visited
          currentMainPage.visited = true;
        }       
        
        // check if we need to show substep directly
        if (nextPossible.showTitleOnly())
          this.goToSubstep(nextPossible);
        else {
          // select that page
          this.selectPage(nextPossible);
        }
      }      
    }        
  },
  
  OnBack: function(wiz)
  {
    // override or connect to me
  },
  
  OnNext: function(wiz)
  {
    // override or connect to me
  }
  
});

/**
 * Slider support for horizontal navigation
 */
dojo.declare("ibm.tivoli.tip.dijit.TivWizardToCSlider", null, 
{
  toc: null,        // ordered list for ToC
  tocSteps: null,   // div containing toc main steps
  ltr: true, 
  tocStepsMargin: 10,
  tocMarginLeft: 2,
  oldTocStepsWidth: 0,
  
  totalWidthToCItems: 0,
  oneButtonWidth: 29,
  visibleButtonWidth: 0,  // width plus margin of visible buttons 

  sliderOuterDiv: null,
  sliderLeft: null,
  sliderRight: null,
  sliderLeftImg: dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/tivwizard/slider_left.png"),
  sliderRightImg: dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/tivwizard/slider_right.png"),
  
  t: null,
  slideDelay: 2,  // 2 milliseconds
  slidePixel: 2,
  pos: 0,
  slideDirection: "",

  slideRightLabel: "",
  slideLeftLabel: "",
  

  /**
   * Constructor which adds nodes to ToC content pane for the slider, e.g.
   * <div for main steps - passed in>
   *   <img for left slider button />
   *   <div for containing ToC (outer) >
   *     <ol for ToC (inner) - passed in />
   *   </div>
   *   <img for right slider button>
   * </div>
   * 
   * Takes three arguments:
   *   toc       <ol> for ToC
   *   tocSteps  div for main steps
   *   ltr       flag indicating whether container text direction is left-to-right
   *             or not (right-to-left)
   */
  constructor: function(arguments)
  {
    console.log("TivWizardToCSlider.constructor");
    this.toc = arguments.toc;
    this.tocSteps = arguments.tocSteps;
    this.ltr = arguments.ltr;
        
    // strings for slider buttons
    var string_table = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
    this.slideRightLabel = string_table.SLIDE_RIGHT;
    this.slideLeftLabel = string_table.SLIDE_LEFT;

    this.sliderOuterDiv = dojo.create("div", { className: "slider-outer" });          
    dojo.place(this.toc, this.sliderOuterDiv, 0);
    dojo.addClass(this.toc, "slider-inner");        
    
    this.sliderLeft = dojo.create("img", {
      src: this.sliderLeftImg.toString(),
      alt: this.slideLeftLabel,
      className: "slider-left"
    });
    this.sliderRight = dojo.create("img", {
      src: this.sliderRightImg.toString(),
      alt: this.slideRightLabel,
      className: "slider-right"
    });
    dojo.place(this.sliderLeft, this.tocSteps, 0);
    dojo.place(this.sliderOuterDiv, this.tocSteps, 1);
    dojo.place(this.sliderRight, this.tocSteps, 2);
    
    // initially hide buttons
    dojo.style(this.sliderLeft, "display", "none");        
    dojo.style(this.sliderRight, "display", "none");        
    
    // connect button handlers
    dojo.connect(this.sliderLeft, "onclick", this, this.slideLeft);
    dojo.connect(this.sliderRight, "onclick", this, this.slideRight);
  },
  
  /**
   * Re-calculate total width of all ToC items (step arrows).
   */
  recalcToCWidth: function()
  {
    // With IE arrow image parts may wrap as the arrow styles change.  
    // To ensure we have the correct total width, we temporarily increase
    // the width of the ToC list to prevent any truncation of parts and 
    // then add up the widths of the arrow parts, i.e. front, middle and 
    // tail.
    dojo.style(this.toc, "width", (this.totalWidthToCItems * 2) + "px");  // should be sufficient   
    this.totalWidthToCItems = 0;
    dojo.query("i", this.toc).forEach(function(n) {
      this.totalWidthToCItems += n.offsetWidth; 
    }, this);
    dojo.query("span", this.toc).forEach(function(n) {
      this.totalWidthToCItems += n.offsetWidth;      
    }, this);
    dojo.query("b", this.toc).forEach(function(n) {
      this.totalWidthToCItems += n.offsetWidth;      
    }, this);
    dojo.style(this.toc, "width", this.totalWidthToCItems + "px");
  },
    
  /**
   * Right slider button is made visible if not all step arrows can
   * be displayed in the pane.  Once user clicks on button and slides
   * the arrows, then the left slider button is added.  Both buttons
   * remain visible unless the pane is resized so that all steps can
   * be displayed.
   */
  updateSliderUI: function(newWidth)
  {
    // newWidth will be passed in when toc content pane is being resized;
    // use it instead of scrollWidth since that value may not be updated yet
    var slideWidth = (newWidth != null) ? newWidth : dojo.style(this.tocSteps, "width");
    if (this.totalWidthToCItems >= slideWidth - this.tocStepsMargin)
    {
      if (this.ltr) {
        // Show right slider button only if no buttons currently showing
        if (this.sliderRight.style.display == "none") {
          dojo.style(this.sliderRight, "display", "inline");
          this.visibleButtonWidth = this.oneButtonWidth;        
        }
        else 
          this.finishSlide(this.sliderLeft, slideWidth);
      }
      else {
        // Show left slider button only if no buttons currently showing
        if (this.sliderLeft.style.display == "none") {
          dojo.style(this.sliderLeft, "display", "inline");
          this.visibleButtonWidth = this.oneButtonWidth;        
        }        
        else
          this.finishSlide(this.sliderRight, slideWidth);
      }
      dojo.style(this.sliderOuterDiv, "width", (slideWidth - this.visibleButtonWidth) + "px");
      this.oldTocStepsWidth = slideWidth;
    }              
    else if (this.visibleButtonWidth > 0)
    {
      // no need for slider, hide everything
      dojo.style(this.sliderLeft, "display", "none");        
      dojo.style(this.sliderRight, "display", "none");        
      dojo.style(this.sliderOuterDiv, "width", "100%");
      dojo.style(this.toc, "left", "0px");
      this.pos = 0;
      this.slideDirection = "";
      this.visibleButtonWidth = 0;
    }    
  },

  /**
   * Handles case where both slider buttons are visible, toc has been 
   * scrolled so first and last step(s) are no longer visible and window 
   * is being resized.  As the window width is increased, once the last 
   * step is visible the toc needs to be slid so that the first step(s) 
   * become visible too.
   */
  finishSlide: function(button, slideWidth) 
  {
    if (button.style.display != "none") {
      var tocLeft = dojo.style(this.toc, "left");
      if (this.ltr) {
        // left-to-right direction
        if (tocLeft < 0) {
          var tocWidthShowing = dojo.style(this.toc, "width") + tocLeft;
          if (slideWidth > (tocWidthShowing + this.visibleButtonWidth + this.tocMarginLeft))
          {
            this.pos = tocLeft + slideWidth - this.oldTocStepsWidth;
            dojo.style(this.toc, "left", this.pos + "px");
          }                  
        }
      }
      else {
        // right-to=left direction
        if (tocLeft > 0) {
          var tocWidthShowing = 0 - (dojo.style(this.toc, "width") - tocLeft);
          if (slideWidth > (tocWidthShowing + this.visibleButtonWidth + this.tocMarginLeft))
          {
            this.pos = tocLeft - slideWidth + this.oldTocStepsWidth;
            dojo.style(this.toc, "left", this.pos + "px");
          }                            
        }
      }
    }
  },
  
  /**
   * Called to show the second slider button after first slider button
   * has been clicked.
   */  
  showSecondButton: function(button, marginStyle, multiplier)
  {
    this.visibleButtonWidth *= 2;
    this.pos = multiplier * this.oneButtonWidth;
    dojo.style(button, "display", "inline");
    dojo.style(this.toc, "left", this.pos + "px");
    dojo.style(this.sliderOuterDiv, "width", (dojo.style(this.tocSteps, "width") - this.visibleButtonWidth) + "px");
    dojo.style(this.toc, marginStyle, this.tocMarginLeft + "px");    
  },
  
  /**
   * Returns the number of pixels that have been scrolled due to
   * tabbing.
   */
  getScrollAmount: function()
  {
    var scrollAmt = 0;
    if (this.sliderOuterDiv.scrollLeft > 0)  // IE
      scrollAmt = this.sliderOuterDiv.scrollWidth - this.sliderOuterDiv.scrollLeft - this.sliderOuterDiv.offsetWidth;
    else if (this.sliderOuterDiv.scrollLeft < 0)  // FF
      scrollAmt = 0 - this.sliderOuterDiv.scrollLeft;
    return scrollAmt;
  },
  
  /**
   * Invoked when user clicks on right slider button
   */
  slideRight: function()
  {
    if (this.slideDirection == "")
      this.showSecondButton(this.sliderLeft, "marginLeft", -1);
    else 
      this.slideStop();
    this.slideDirection = "R";
    
    // determine number of pixels to slide adjusting for any scrolling
    // from tabbing if necessary
    var slideWidth = dojo.style(this.sliderOuterDiv, "width") - this.tocMarginLeft;    
    var leftToSlide;
    if (this.ltr)
      leftToSlide = this.totalWidthToCItems - slideWidth + this.pos - this.sliderOuterDiv.scrollLeft;
    else
      leftToSlide = this.pos + this.getScrollAmount();

    // don't slide too much
    if (leftToSlide > slideWidth)
      leftToSlide = slideWidth;
    
    // call recursive function to do actual slide
    if (leftToSlide > 0)
      this._slideRight(leftToSlide - this.pos);
  },
  
  _slideRight: function(totalToSlide)
  {    
    if (this.pos == 0) 
      this.pos = -1;
    this.pos -= this.slidePixel;
    if (this.pos < -totalToSlide)
    {
      var diff = -this.pos - totalToSlide;
      this.pos += this.slidePixel;
      if (diff < this.slidePixel) {
        this.pos -= diff;  // any last pixels if not multiple of slidePixel number
        dojo.style(this.toc, "left", this.pos + "px");
      }
      return;
    }
    dojo.style(this.toc, "left", this.pos + "px");
    this.t = setTimeout(dojo.hitch(this, this._slideRight, totalToSlide), this.slideDelay);    
  },
  
  /**
   * Invoked when user clicks on left slider button
   */
  slideLeft: function()
  {    
    if (this.slideDirection == "")
      this.showSecondButton(this.sliderRight, "marginRight", 1);
    else 
      this.slideStop();
    this.slideDirection = "L";
            
    // determine number of pixels to slide adjusting for any scrolling
    // from tabbing
    var slideWidth = dojo.style(this.sliderOuterDiv, "width") - this.tocMarginLeft;    
    var leftToSlide;
    if (this.ltr)
      leftToSlide = -this.pos + this.sliderOuterDiv.scrollLeft;
    else    
      leftToSlide = this.totalWidthToCItems - slideWidth - this.pos - this.getScrollAmount();
    
    // don't slide too much
    if (leftToSlide > slideWidth)
      leftToSlide = slideWidth;
    
    // call recursive function to do actual slide
    if (leftToSlide > 0)
      this._slideLeft(leftToSlide);
  },
  
  _slideLeft: function(totalToSlide)
  {
    if (this.pos == 0) 
      this.pos = 1;
    this.pos += this.slidePixel;
    totalToSlide -= this.slidePixel;
    if (totalToSlide < 0) 
    {
      this.pos -= this.slidePixel;
      if (-totalToSlide < this.slidePixel) {
        this.pos += -totalToSlide;  // any last pixels if not multiple of slidePixel number
        dojo.style(this.toc, "left", this.pos + "px");
      }
      return;

    }
    dojo.style(this.toc, "left", this.pos + "px");
    this.t = setTimeout(dojo.hitch(this, this._slideLeft, totalToSlide), this.slideDelay);    
  },

  /**
   * Cancels the set timer so sliding is stopped
   */
  slideStop: function()
  {
    clearTimeout(this.t);
  },
  
  /**
   * Resets slider so first arrow is visible.
   */
  reset: function()
  {
    this.pos = 0;
    dojo.style(this.toc, "left", "0px");
    if (this.ltr)
      this.sliderOuterDiv.scrollLeft = 0;
  }
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.Tree"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.Tree"] = true;
dojo.provide("ibm.tivoli.tip.dijit.Tree");




/**
 * Tree widget.
 *
 * @author: Marco Lerro (marco.lerro@it.ibm.com)   
 */
dojo.declare(
    "ibm.tivoli.tip.dijit.Tree",
    [ibm.tivoli.tip.dijit.Widget, 
     dijit._Templated],
    {
        /** the widget template path */
        templateString:"<div \r\n    class=\"tree\"\r\n    dojoAttachEvent=\"onclick:_onClick,onkeyup:_onKeyUp\"\r\n></div>\r\n",
        
        /** true if a node must be selected on user click events */
        autoSelect: true,
        /** the tab index */
        tabindex: "0",
        /** true if at least one node contains a Dojo widget */
        hasWidgets: false,
        /** the tree (TreeItem object tree) */
        tree: null,

        /** the tree */
        _tree: null,
        /** current selected node (DOM node) */
        _selected: null,
        /** the node template */
        _nodeTemplate:
            "<div name='${nodeName}' nodeId=${nodeId}>" +
                "<div name='${rowName}'>" +
                    "<table><tbody><tr class='treeRow'>" +
                        "<td class='treeCell'>" +
                            "<img name='${iconName}' expanded='${expanded}' style='margin-left:${marginLeft}px' tabindex='${tabindex}' src='${imgSource}'></img>" +
                        "</td>" +
                        "<td name='${itemName}' class='treeCell'>" +
                            "${item}" +
                        "</td>" +
                    "</tr></tbody></table>" +
                "</div>" +
                "<div class='treeChildren' name='${childrenName}'>" +
                    "${children}" +
                "</div>" +          
            "</div>",
        
        /**
         * Prepares the selection list. 
         * If an array of tree has been specified the list is populated.  
         */
        startup: function() 
        {
            // call the superclass's method
            this.inherited("startup", arguments);
            
            // set-up some useful constants
            this._NODE_NAME = this.id + "#node";
            this._ROW_NAME  = this.id + "#row";
            this._ICON_NAME = this.id + "#icon";
            this._CHILDREN_NAME = this.id + "#children";
            this._ITEM_NAME = this.id + "#item";
               
            // valorize the node template with widget constants
            var p = {
                nodeName: this._NODE_NAME,
                rowName: this._ROW_NAME,
                iconName: this._ICON_NAME,
                childrenName: this._CHILDREN_NAME,
                itemName: this._ITEM_NAME,
                tabindex: this.tabindex,
                nodeId: "${nodeId}",
                marginLeft: "${marginLeft}", 
                imgSource: "${imgSource}",
                nesting: "${nesting}", 
                expanded: "${expanded}",
                item: "${item}",
                children: "${children}"
            };
            this._nodeTemplate = dojo.string.substitute(this._nodeTemplate, p);
            
            // populate the tree from the tree parameter
            this.setValue(this.tree);
        },

        /**        
         * Returns the tree (TreeItem object tree).
         */
        getValue: function()
        {
            return this._tree;
        },
        
        /**        
         * Sets the given tree into the tree.
         * 
         *  @param tree
         *      A tree of objects that have the following attributes:
         *          - id: a unique id which identifies the node in the tree
         *          - item: the content of the node to be displayed. It could be
         *                  a string, a HTML fragment, a HTML Dojo declaration.
         *                  In the last case the hasWidgets property must be set 
         *                  to true.
         *          - children: any array of child nodes.  
         */
        setValue: function(tree)
        {
            // destroy enclosing widgets
            if ( this.hasWidgets ) {
                dojo.query("[widgetId]", this.domNode).forEach(
                    function(d) {
                        dijit.byId(d.getAttribute("widgetId")).destroy();
                    }
                );
            }
            
            // store the tree
            this._tree = tree;
            if ( tree === null ) {
                this.domNode.innerHTML = "";
            }
            else {
                this.domNode.innerHTML = this._createTreeNode(tree, 0);
            }
            
            // parse the mark-up if has Dojo widgets 
            if ( this.hasWidgets ) {
                dojo.parser.parse(this.domNode);
            }
        },
        
        /**
         * Returns true if the tree has no tree.
         */
        isEmpty: function()
        {
            return ( this.getValue() === null );
        },
        
        /**
         * Selects/Deselects a node corresponding to the given id.
         * 
         *  @param id:
         *      The node id (the same passed in setValue tree).
         *  @param selected:
         *      true if the node must be selected.
         */
        selectNode: function(id, selected)
        {
            // get the node corresponding to the given id
            var node = dojo.query("div[nodeid='" + id + "']")[0];
            if ( node ) {
                var row = dojo.query("div[name='" + this._ROW_NAME + "']", node)[0];
                this._selectRow(row, selected);
            }
        },
        
        /**
         * Deselects the current selected node (if any).
         */
        unselect: function()
        {
            if ( this._selected ) {
                this._selectRow(this._selected, false);
            }
        },
        
        /**        
         * Returns true if the node is selected.
         * 
         *  @param id:
         *      The node id (the same passed in setValue tree).
         *  @return:
         *      true if the node is selected.

         */
        isNodeSelected: function(id)
        {
            var selected = false;
            var node = dojo.query("div[nodeid='" + id + "']")[0];
            if ( node ) {
                var row = dojo.query("div[name='" + this._ROW_NAME + "']", node)[0];
                selected = this._isRowSelected(row);
            }
            return selected;
        },

        /**
         * This callback is called when a node has been selected/unselected.
         * 
         *  @param id:
         *      The id of the selected node.
         *  @param selected:
         *      true if the node has been selected, false otherwise.    
         */        
        onNodeSelect: null,
                
        /**
         * Handles the onkeyup event.
         * If a space has been pressed on a +/- icon, the node is 
         * expanded/collapsed accordingly.
         */
        _onKeyUp: function(event)
        {
            if ( event.keyCode === dojo.keys.SPACE ) {
                this._onClick(event);   
            }
        },
        
        /**
         * Handles the onclick event.
         * If the +/- icon has been clicked, the node is 
         * expanded/collapsed accordingly.
         */
        _onClick: function(event)
        {
            var t = event.target;
            var node;
            
            // check if the click comes from a +/- icon
            if ( t.getAttribute("name") === this._ICON_NAME ) {
                // look-up the tree node
                node = this._getNode(t);
                if ( node ) {
                    // node found, collapse/expand it
                    this._toggleNode(node, t);
                }
                // stop the event bubbling 
                dojo.stopEvent(event);
                // and that's it
                return;
            }
            
            // handle selection only if enabled
            if ( this.autoSelect ) {
                // check if the click comes from the node area
                node = this._getNode(t);
                if ( node ){
                    var row = dojo.query("div[name='" + this._ROW_NAME + "']", node)[0];
                    this._selectRow(row, ! this._isRowSelected(row));
                    // stop the event bubbling 
                    dojo.stopEvent(event);
                    // and that's it
                    return;
                }
            }
        },
        
        /**
         * Returns the DOM node of the tree node which includes
         * the specified child node.
         * 
         *  @param childNode:
         *      A DOM node which is a descendant of a tree node.
         *  @return 
         *      A tree (DOM) node if found, otherwise null.  
         */
        _getNode: function(childNode)
        {
            while ( childNode ) {
                if ( childNode.getAttribute && 
                     childNode.getAttribute("name") === this._NODE_NAME ) {
                    break;
                } 
                childNode = childNode.parentNode;                
            }
            return childNode;   
        },
         
        /**
         * Creates a tree node mark-up.
         * 
         *  @param node:
         *      A tree node. Such node has two properties:
         *          - item: is the mark-up to be shows for such node.
         *          - children: an optional array of children nodes.
         *  @param nesting:
         *      The current nesting level.
         *  @return
         *      The mark-up string corresponding to the given node
         *      including all the descendants. 
         */
        _createTreeNode: function(node, nesting)
        {
            var s = [];
            
            if ( node ) {
                // get the children mark-up
                var children = [];
                if ( node.children && node.children.length > 0 ) {
                    for ( var i = 0; i < node.children.length; i++ ) {
                        children[children.length] = this._createTreeNode(node.children[i], nesting + 1);                                        
                    }
                }
                 
                // set the image and the tabindex depending on the nesting state 
                var imgSource;
                var tabindex = this.tabindex;
                if ( ! node.children || node.children.length === 0 ) {
                    imgSource = this._images.ICON_TREE_EMPTY;
                    tabindex = "-1";
                }
                else {
                    imgSource = this._images.ICON_EXPANDED;
                }
                
                // prepare the template parameters
                var p = {
                    nodeId: node.id || "",
                    marginLeft: (4 + 16 * nesting), 
                    tabindex: tabindex, 
                    imgSource: imgSource,
                    nesting: nesting, 
                    expanded: true,
                    item: node.item || "",
                    children: children.join("")
                };
                
                // valorize the template
                s[s.length] = dojo.string.substitute(this._nodeTemplate, p);
            }
            
            // that's it
            return s.join("");
        },

        /**        
         * Toogles a node (expand it if collapsed, collapse it if expanded).
         * 
         *  @param node:
         *      The (DOM)node to be toggled.
         *  @param img:
         *      The DOM node of the +/- icon. This is optional,
         *      if not provided the method looks-up it.
         */
        _toggleNode: function(node, img)
        {
            // get the children node
            var children = dojo.query("div:first-child[name='" + this._CHILDREN_NAME + "']", node)[0];
            if ( children && children.hasChildNodes() ) {
                // get the image node (if not provided)
                if ( ! img ) {
                    img = dojo.query("img:first-child[name='" + this._ICON_NAME + "']", node)[0];
                }
                var isExpanded = img.getAttribute("expanded") === "true";
                // update the image node
                img.setAttribute("expanded", isExpanded ? "false" : "true");
                img.src = isExpanded ? this._images.ICON_COLLAPSED : this._images.ICON_EXPANDED;
                // hide/show the children
                children.style.display = isExpanded ? "none" : "block";
            }
        },
        
        /**        
         * Selects/unselects a node.
         * 
         *  @param row:
         *      The row DOM node belonging to the node to be selected/unselected.
         *  @param selected:
         *      true if the node must be selected.
         */
        _selectRow: function(row, selected)
        {
            if ( this._selected ) {
                // another row is selected
                if ( this._selected !== row ) {
                    // it's a different row
                    if ( selected ) {
                        // switch the selection
                        dojo.removeClass(this._selected, "treeCellSelected");
                        dojo.addClass(row, "treeCellSelected");
                        this._selected = row;
                    }
                }
                else {
                    // it's the same row
                    if ( ! selected ) {
                        dojo.removeClass(row, "treeCellSelected");
                        this._selected = null;
                    }
                }
            }
            else {
                // no row selected
                if ( selected ) {
                    dojo.addClass(row, "treeCellSelected");
                    this._selected = row;
                }
            }
            
            // call the callback
            if ( this.onNodeSelect ) {
                this.onNodeSelect(row.parentNode.getAttribute("nodeId"), selected);
            }
        },
        
        /**        
         * Returns true if the row is selected.
         * 
         *  @param row:
         *      The row DOM node belonging to the node to be selected/unselected.
         *  @return :
         *      true if the node is selected.
         */
        _isRowSelected: function(row)
        {
            return ( this._selected === row );
        }
    }
);


}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.TIPHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.TIPHelper"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dojo.TIPHelper");

dojo.declare("ibm.tivoli.tip.dojo.TIPHelper",
	          null,
{
	constructor: function () {
	},
	// childNode must be a node within your portlet
	// padding is the amount of padding you want around your portlet
	//   e.g., "0px"
	setPortletPadding: function (childNode, padding) {
		var node = this.getAncestorByClass (childNode, "isc-portlet-bkg");
		if (node) {
			dojo.style (node, "padding", padding);
	    var nextnode = dojo.query( 'table[id$="_portlet_content"]', node )[0];
        if (nextnode) {
        	dojo.style( nextnode, "padding", padding );
        }
		}
		
	},
	getAncestorByClass: function (element, className) {
		var node = null;
		for(node = element.parentNode; node!= null; node = node.parentNode) {
			if(dojo.hasClass(node, className)) {
				break;
			}
		}
		return node;
	},	
	getAncestorByName: function (element, name) {
		var result=null;
		try {
			while(result===null && element!==null) {
				element=element.parentNode;
				var elementname = element.getAttribute ("name");
				if (elementname == name) {
					result=element;
				}
			}
		} catch(e) {
			//no node
		}
		//console.log ("Nodename = " + result);
		return(result);
	},
	getAncestorByID: function (element, id) {
		var result=null;
		try {
			while(result===null && element!==null) {
				element=element.parentNode;
				var elementid = element.getAttribute ("id");
				if (elementid == id) {
					result=element;
				}
			}
		} catch(e) {
			//no node
		}
		//console.log ("Nodename = " + result);
		return(result);
		
	},
	getPortletModel: function (element) {
		console.log ("looking for IscPortlet, starting with element: ");
		console.log (element);
		var node = this.getAncestorByName (element, "IscPortlet");
		console.log ("found IscPortlet node: ");
		console.log (node);
		var dataObject = null;
		
		if (node) {
			var data = node.getAttribute ("portletdataobject");
			console.log ("data: ");
			console.log (data);
			dataObject = dojo.fromJson (data);
			console.log ("get portlet data: ");
			console.log (dataObject);
			return (dataObject);
		} else {
			return (undefined);
		}
	},
	getFreeformPageModel: function (element) {
		node = this.getAncestorByClass (element, "tip-freeform-page");
		if (node === null) {
			console.error ("pageModel cannot be found. node is null");
			return (undefined);
		} 

		console.log ("node with tip-freeform-page class: ");
		console.log (node);
			
		var dataObject = node.modelObject.pageData;
		if ((dataObject === null) || (dataObject === undefined)) {
			var data = node.getAttribute ("pagedataobject");
			dataObject = dojo.fromJson (data);
		}
		
		console.log ("get portlet data: ");
		console.log (dataObject);
		return (dataObject);
	},
	getPortletOID: function (element) {
		var dataObject = this.getPortletModel (element);
		if (dataObject) {
			var result = dataObject.oid;
			return (result);
		} else { 
			return (undefined);
		}
	},
	getPortletWindowID: function (element) {
		var dataObject = this.getPortletModel (element);
		if (dataObject) {
			var result = dataObject.portletName;
			return (result);
		} else {
			return (undefined);
		}
	},
	getPortletPageID: function (element) {
		var dataObject = this.getPortletModel (element);		
		if (dataObject) {
			var result = dataObject.uniquePageName;
			return (result);
		} else {
			dataObject = this.getFreeformPageModel (element);
			var result = dataObject.uniquePageName;
			return (result);
		}
	},
	getPortletPageName: function (element) {
		var parent = this.getAncestorByName (element, "IscPage");
		if (parent) {
			var node = dojo.query ("td.isc-layout-titletext", parent);
			if (node === undefined) {
				node = dojo.query ("div.tip-portlet-titletext", parent);
			}
			
	//		var node = dojo.query ("td.isc-tab-sel-l", frames["taskbar"].document);
			if (node) {
				if (node.length == 1) {
//					var result = node[0].innerHTML.alt;
					var result = node[0].childNodes[0].nodeValue;
					if (result) {
						// strip leading white space
						result = result.replace(/^\s+/,"");
						// strip trailing white space
						result = result.replace(/\s+$/,"");
					}
					console.log ("portlet page name: ");
					console.log (result);
					return (result);
				} else if (node.length === 0) {
					console.log ("could not find node for portlet page name. array length is 0.");
				} else {
					console.log ("found multiple nodes for portlet page names");
					console.log (node);
				}
			} else {
				console.log ("could not find node for portlet page name");
			}
		} else {
			console.log ("no parent node to get portlet page name");
		}
	},
	getTopPortletNodeFreeform: function (element) {
		var node = this.getAncestorByClass (element, "tip-portlet-frame");
		return (node);
	}, 
	getTopPortletNode: function (element) {
		var node = this.getAncestorByName (element, "IscPortlet");
		if (node) {
		} else {
			console.log ("cannot find TopPortletNode with name IscPortlet... looking for tip-portlet-frame class instead... must be freeform page");
			node = this.getTopPortletNodeFreeform (element);
		}
		
		return (node);		
	},
	getPortletTitle: function (element) {
		console.log ("enter getPortletTitle");
		var oid = this.getPortletOID (element);
		if (oid) {
			var node = dojo.byId ("titleText." + oid);
			if (node) {
				return node.innerHTML;
			} else {
				console.log ("cannot find node for title text: titleText." + oid);
			}
		} else {
			console.log ("freeform title search");
			var z = this.getAncestorByClass (element, "tip-portlet-frame");
			if (z) {
				// OMG... what a hack!
				/*
				z = z.childNodes[1];  // tip-portlet-inner
				z = z.childNodes[1];  // tip-portlet-titlebar
				z = z.childNodes[1];  // tip-portlet-titletext
				z = z.childNodes[1];  // h1
				*/
				for(var i=0; i<4 && z.childNodes[1]; i++){
					z = z.childNodes[1];
				}
				if (z.childNodes[0].getAttribute) {  // getAttribute function must exist for it to be an element
					z = z.childNodes[0];  // span
				}

				return z.innerHTML;
			}
		}
		
		return (undefined);
	},
	_setPortletTitleFreeform: function (element, title) {
		console.log ("enter _setPortletTitleFreeform");
		var z = this.getAncestorByClass (element, "tip-portlet-frame");
		if (z) {
			// OMG... what a hack!
			z = z.childNodes[1];  // tip-portlet-inner
			z = z.childNodes[1];  // tip-portlet-titlebar
			z = z.childNodes[1];  // tip-portlet-titletext
			z = z.childNodes[1];  // h1
			if (z.childNodes[0].getAttribute) {  // getAttribute function must exist for it to be an element
				z = z.childNodes[0];  // span
			}
			
			console.log ("found element for title: ");
			console.log (z);
			z.innerHTML = title;
		}
	},
	setPortletTitle: function (element, title) {
		console.log ("enter setPortletTitle: " + title);

		try{
		
		var oid = this.getPortletOID (element);

		if (oid) {
			var node = dojo.byId ("titleText." + oid);
			if (node) {
				node.innerHTML = title;
			} else {
				console.log ("cannot find node for title text: titleText." + oid);
				this._setPortletTitleFreeform (element, title);
			}
		} else {
				var y = this.getAncestorByClass(element, "tip-portlet-inner");
				y.setAttribute ("id", title);
				dojo.query('h1', y).forEach(function(node, index, arr){						
					node.innerHTML = title;					
				});
				/*
				var z = y.childNodes[1];
				z = z.childNodes[1];
				z = z.childNodes[1];
				//z = z.childNodes[1];
				z.innerHTML = title;
				*/
		}

		}
		catch(e){
			console.info("failure="+e);
		}
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.BusyPane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.BusyPane"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/






dojo.provide( "ibm.tivoli.tip.dijit.BusyPane" );
/******************************************************************************
* Busy Pane that blocks all user input and shows progress message.
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.BusyPane", [dijit._Widget, dijit._Templated],
{
  processingImageSrc: dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/processing-toolbar.gif"),
    
  //processing label var
  processingLabel : "",  
  
  templateString: "<div class='tipBusyPane'><table width='100%' height='100%' cellspacing='0' cellpadding='0' border='0'><tr><td valign='middle' align='middle' style='vertical-align: middle;'><div dojoAttachPoint='busypane' class='tipBusyPaneProcessing'><table width='100%' cellspacing='0' cellpadding='0' border='0'><tbody><tr><td class='process-anim' width='1%'><img width='16' height='16' src='${processingImageSrc}'/></td><td class='process-message' dojoAttachPoint='messageArea'>${processingLabel}</td></tr></tbody></table></div></td></tr></table></div>",
  
  constructor: function () {
	  this.processingLabel = dojo.i18n.getLocalization( "ibm.tivoli.tip.dijit", "resources" ).PROCESSING;
  },
  
  postCreate: function()
  {
    this.inherited( arguments );
	this.hide();
  },
  
  show: function( sizeToPortlet )
  {
  	var tipHelper = null;
  	var div       = null;
  	var coords    = null;
  		 
	 dojo.style(this.busypane,"position","absolute"); 
	
	 
	if ( sizeToPortlet ) 
	{
		console.warn("sizeToPortlet =true");
		tipHelper = new ibm.tivoli.tip.dojo.TIPHelper();
		
		div = tipHelper.getTopPortletNode (this.domNode);
		
		if ( div ) {
		    coords = dojo.coords( div );
		    console.log ("in BusyPane.show.  got coords: ");
		    console.log (coords);
		    this.domNode.style.display="";
		    dojo.style (this.domNode, "position", "fixed");
		    dojo.style (this.domNode, "top",  ( coords.y < 0 ) ? "0px" : coords.y+"px" );
		    dojo.style (this.domNode, "left", ( coords.x < 0 ) ? "0px" : coords.x+"px" );
		    dojo.style (this.domNode, "width", coords.w+"px");
			dojo.style (this.domNode, "height", coords.h+"px");
			var panelBoxCoords = dojo.coords( this.busypane );
			var panecenter = coords.w/2 - panelBoxCoords.w/2;
			dojo.style(this.busypane,"left",panecenter+"px");
			
		} else {
			console.log ("top portlet node not found for busy cursor.  styling page instead.");
			this.stylePage();
		}
	} else {
	   
		this.stylePage();
	}
    
    this.domNode.style.display="";
  },
  stylePage: function () {
	     //console.log ("in BusyPane.show.  covering whole page/frame.");
				
	    dojo.style (this.domNode, "top", "0");
	    dojo.style (this.domNode, "align", "center");
	    dojo.style (this.domNode, "width", "100%");
		dojo.style (this.domNode, "height", "100%");
		 var page=dijit.getViewport();
	    var busypanewidth= page.w/2;
		var busypaneheight =page.h/2;
	    dojo.style(this.busypane,"left",busypanewidth+"px");
		dojo.style(this.busypane,"top",busypaneheight+"px");
  },
  hide: function() {
    this.domNode.style.display="none";
  }
});

}

if(!dojo._hasResource["dojox.gfx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx"] = true;
dojo.provide("dojox.gfx");




dojo.loadInit(function(){
	//Since loaderInit can be fired before any dojo.provide/require calls,
	//make sure the dojox.gfx object exists and only run this logic if dojox.gfx.renderer
	//has not been defined yet.
	var gfx = dojo.getObject("dojox.gfx", true), sl, flag, match;
	if(!gfx.renderer){
		//Have a way to force a GFX renderer, if so desired.
		//Useful for being able to serialize GFX data in a particular format.
		if(dojo.config.forceGfxRenderer){
			dojox.gfx.renderer = dojo.config.forceGfxRenderer;
			return;
		}
		var renderers = (typeof dojo.config.gfxRenderer == "string" ?
			dojo.config.gfxRenderer : "svg,vml,silverlight,canvas").split(",");

		// mobile platform detection
		// TODO: move to the base?

		var ua = navigator.userAgent, iPhoneOsBuild = 0, androidVersion = 0;
		if(dojo.isSafari >= 3){
			// detect mobile version of WebKit starting with "version 3"

			//	comprehensive iPhone test.  Have to figure out whether it's SVG or Canvas based on the build.
			//	iPhone OS build numbers from en.wikipedia.org.
			if(ua.indexOf("iPhone") >= 0 || ua.indexOf("iPod") >= 0){
				//	grab the build out of this.  Expression is a little nasty because we want
				//		to be sure we have the whole version string.
				match = ua.match(/Version\/(\d(\.\d)?(\.\d)?)\sMobile\/([^\s]*)\s?/);
				if(match){
					//	grab the build out of the match.  Only use the first three because of specific builds.
					iPhoneOsBuild = parseInt(match[4].substr(0,3), 16);
				}
			}
		}
		if(dojo.isWebKit){
			// Android detection
			if(!iPhoneOsBuild){
				match = ua.match(/Android\s+(\d+\.\d+)/);
				if(match){
					androidVersion = parseFloat(match[1]);
					// Android 1.0-1.1 doesn't support SVG but supports Canvas
				}
			}
		}

		for(var i = 0; i < renderers.length; ++i){
			switch(renderers[i]){
				case "svg":
					//	iPhone OS builds greater than 5F1 should have SVG.
					if(!dojo.isIE && (!iPhoneOsBuild || iPhoneOsBuild >= 0x5f1) && !androidVersion && !dojo.isAIR){
						dojox.gfx.renderer = "svg";
					}
					break;
				case "vml":
					if(dojo.isIE){
						dojox.gfx.renderer = "vml";
					}
					break;
				case "silverlight":
					try{
						if(dojo.isIE){
							sl = new ActiveXObject("AgControl.AgControl");
							if(sl && sl.IsVersionSupported("1.0")){
								flag = true;
							}
						}else{
							if(navigator.plugins["Silverlight Plug-In"]){
								flag = true;
							}
						}
					}catch(e){
						flag = false;
					}finally{
						sl = null;
					}
					if(flag){ dojox.gfx.renderer = "silverlight"; }
					break;
				case "canvas":
					//TODO: need more comprehensive test for Canvas
					if(!dojo.isIE){
						dojox.gfx.renderer = "canvas";
					}
					break;
			}
			if(dojox.gfx.renderer){ break; }
		}
		if(dojo.config.isDebug){
			console.log("gfx renderer = " + dojox.gfx.renderer);
		}
	}
});

// include a renderer conditionally


dojo.requireIf(dojox.gfx.renderer == "silverlight", "dojox.gfx.silverlight");
dojo.requireIf(dojox.gfx.renderer == "canvas", "dojox.gfx.canvas");

}

if(!dojo._hasResource["dojox.lang.functional.lambda"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.functional.lambda"] = true;
dojo.provide("dojox.lang.functional.lambda");

// This module adds high-level functions and related constructs:
//	- anonymous functions built from the string

// Acknoledgements:
//	- lambda() is based on work by Oliver Steele 
//		(http://osteele.com/sources/javascript/functional/functional.js)
//		which was published under MIT License

// Notes:
//	- lambda() produces functions, which after the compilation step are 
//		as fast as regular JS functions (at least theoretically).

// Lambda input values:
//	- returns functions unchanged
//	- converts strings to functions
//	- converts arrays to a functional composition

(function(){
	var df = dojox.lang.functional, lcache = {};

	// split() is augmented on IE6 to ensure the uniform behavior
	var split = "ab".split(/a*/).length > 1 ? String.prototype.split :
			function(sep){
				 var r = this.split.call(this, sep),
					 m = sep.exec(this);
				 if(m && m.index == 0){ r.unshift(""); }
				 return r;
			};
			
	var lambda = function(/*String*/ s){
		var args = [], sects = split.call(s, /\s*->\s*/m);
		if(sects.length > 1){
			while(sects.length){
				s = sects.pop();
				args = sects.pop().split(/\s*,\s*|\s+/m);
				if(sects.length){ sects.push("(function(" + args + "){return (" + s + ")})"); }
			}
		}else if(s.match(/\b_\b/)){
			args = ["_"];
		}else{
			var l = s.match(/^\s*(?:[+*\/%&|\^\.=<>]|!=)/m),
				r = s.match(/[+\-*\/%&|\^\.=<>!]\s*$/m);
			if(l || r){
				if(l){
					args.push("$1");
					s = "$1" + s;
				}
				if(r){
					args.push("$2");
					s = s + "$2";
				}
			}else{
				// the point of the long regex below is to exclude all well-known 
				// lower-case words from the list of potential arguments
				var vars = s.
					replace(/(?:\b[A-Z]|\.[a-zA-Z_$])[a-zA-Z_$\d]*|[a-zA-Z_$][a-zA-Z_$\d]*:|this|true|false|null|undefined|typeof|instanceof|in|delete|new|void|arguments|decodeURI|decodeURIComponent|encodeURI|encodeURIComponent|escape|eval|isFinite|isNaN|parseFloat|parseInt|unescape|dojo|dijit|dojox|window|document|'(?:[^'\\]|\\.)*'|"(?:[^"\\]|\\.)*"/g, "").
					match(/([a-z_$][a-z_$\d]*)/gi) || [], t = {};
				dojo.forEach(vars, function(v){
					if(!(v in t)){
						args.push(v);
						t[v] = 1;
					}
				});
			}
		}
		return {args: args, body: s};	// Object
	};

	var compose = function(/*Array*/ a){
		return a.length ? 
					function(){
						var i = a.length - 1, x = df.lambda(a[i]).apply(this, arguments);
						for(--i; i >= 0; --i){ x = df.lambda(a[i]).call(this, x); }
						return x;
					}
				: 
					// identity
					function(x){ return x; };
	};

	dojo.mixin(df, {
		// lambda
		rawLambda: function(/*String*/ s){
			// summary:
			//		builds a function from a snippet, or array (composing),
			//		returns an object describing the function; functions are
			//		passed through unmodified.
			// description:
			//		This method is to normalize a functional representation (a
			//		text snippet) to an object that contains an array of
			//		arguments, and a body , which is used to calculate the
			//		returning value.
			return lambda(s);	// Object
		},
		buildLambda: function(/*String*/ s){
			// summary:
			//		builds a function from a snippet, returns a string, which
			//		represents the function.
			// description:
			//		This method returns a textual representation of a function
			//		built from the snippet. It is meant to be evaled in the
			//		proper context, so local variables can be pulled from the
			//		environment.
			s = lambda(s);
			return "function(" + s.args.join(",") + "){return (" + s.body + ");}";	// String
		},
		lambda: function(/*Function|String|Array*/ s){
			// summary:
			//		builds a function from a snippet, or array (composing),
			//		returns a function object; functions are passed through
			//		unmodified.
			// description:
			//		This method is used to normalize a functional
			//		representation (a text snippet, an array, or a function) to
			//		a function object.
			if(typeof s == "function"){ return s; }
			if(s instanceof Array){ return compose(s); }
			if(s in lcache){ return lcache[s]; }
			s = lambda(s);
			return lcache[s] = new Function(s.args, "return (" + s.body + ");");	// Function
		},
		clearLambdaCache: function(){
			// summary:
			//		clears internal cache of lambdas
			lcache = {};
		}
	});
})();

}

if(!dojo._hasResource["dojox.lang.functional.array"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.functional.array"] = true;
dojo.provide("dojox.lang.functional.array");



// This module adds high-level functions and related constructs:
//	- array-processing functions similar to standard JS functions

// Notes:
//	- this module provides JS standard methods similar to high-level functions in dojo/_base/array.js: 
//		forEach, map, filter, every, some

// Defined methods:
//	- take any valid lambda argument as the functional argument
//	- operate on dense arrays
//	- take a string as the array argument
//	- take an iterator objects as the array argument

(function(){
	var d = dojo, df = dojox.lang.functional, empty = {};

	d.mixin(df, {
		// JS 1.6 standard array functions, which can take a lambda as a parameter.
		// Consider using dojo._base.array functions, if you don't need the lambda support.
		filter: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: creates a new array with all elements that pass the test 
			//	implemented by the provided function.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var t = [], v, i, n;
			if(d.isArray(a)){
				// array
				for(i = 0, n = a.length; i < n; ++i){
					v = a[i];
					if(f.call(o, v, i, a)){ t.push(v); }
				}
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				for(i = 0; a.hasNext();){
					v = a.next();
					if(f.call(o, v, i++, a)){ t.push(v); }
				}
			}else{
				// object/dictionary
				for(i in a){
					if(!(i in empty)){
						v = a[i];
						if(f.call(o, v, i, a)){ t.push(v); }
					}
				}
			}
			return t;	// Array
		},
		forEach: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: executes a provided function once per array element.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var i, n;
			if(d.isArray(a)){
				// array
				for(i = 0, n = a.length; i < n; f.call(o, a[i], i, a), ++i);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				for(i = 0; a.hasNext(); f.call(o, a.next(), i++, a));
			}else{
				// object/dictionary
				for(i in a){
					if(!(i in empty)){
						f.call(o, a[i], i, a);
					}
				}
			}
			return o;	// Object
		},
		map: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: creates a new array with the results of calling 
			//	a provided function on every element in this array.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var t, n, i;
			if(d.isArray(a)){
				// array
				t = new Array(n = a.length);
				for(i = 0; i < n; t[i] = f.call(o, a[i], i, a), ++i);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				t = [];
				for(i = 0; a.hasNext(); t.push(f.call(o, a.next(), i++, a)));
			}else{
				// object/dictionary
				t = [];
				for(i in a){
					if(!(i in empty)){
						t.push(f.call(o, a[i], i, a));
					}
				}
			}
			return t;	// Array
		},
		every: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: tests whether all elements in the array pass the test 
			//	implemented by the provided function.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var i, n;
			if(d.isArray(a)){
				// array
				for(i = 0, n = a.length; i < n; ++i){
					if(!f.call(o, a[i], i, a)){
						return false;	// Boolean
					}
				}
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				for(i = 0; a.hasNext();){
					if(!f.call(o, a.next(), i++, a)){
						return false;	// Boolean
					}
				}
			}else{
				// object/dictionary
				for(i in a){
					if(!(i in empty)){
						if(!f.call(o, a[i], i, a)){
							return false;	// Boolean
						}
					}
				}
			}
			return true;	// Boolean
		},
		some: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: tests whether some element in the array passes the test 
			//	implemented by the provided function.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var i, n;
			if(d.isArray(a)){
				// array
				for(i = 0, n = a.length; i < n; ++i){
					if(f.call(o, a[i], i, a)){
						return true;	// Boolean
					}
				}
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				for(i = 0; a.hasNext();){
					if(f.call(o, a.next(), i++, a)){
						return true;	// Boolean
					}
				}
			}else{
				// object/dictionary
				for(i in a){
					if(!(i in empty)){
						if(f.call(o, a[i], i, a)){
							return true;	// Boolean
						}
					}
				}
			}
			return false;	// Boolean
		}
	});
})();

}

if(!dojo._hasResource["dojox.lang.functional.object"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.functional.object"] = true;
dojo.provide("dojox.lang.functional.object");



// This module adds high-level functions and related constructs:
//	- object/dictionary helpers

// Defined methods:
//	- take any valid lambda argument as the functional argument
//	- skip all attributes that are present in the empty object 
//		(IE and/or 3rd-party libraries).

(function(){
	var d = dojo, df = dojox.lang.functional, empty = {};

	d.mixin(df, {
		// object helpers
		keys: function(/*Object*/ obj){
			// summary: returns an array of all keys in the object
			var t = [];
			for(var i in obj){
				if(!(i in empty)){
					t.push(i);
				}
			}
			return	t; // Array
		},
		values: function(/*Object*/ obj){
			// summary: returns an array of all values in the object
			var t = [];
			for(var i in obj){
				if(!(i in empty)){
					t.push(obj[i]);
				}
			}
			return	t; // Array
		},
		filterIn: function(/*Object*/ obj, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: creates new object with all attributes that pass the test 
			//	implemented by the provided function.
			o = o || d.global; f = df.lambda(f);
			var t = {}, v, i;
			for(i in obj){
				if(!(i in empty)){
					v = obj[i];
					if(f.call(o, v, i, obj)){ t[i] = v; }
				}
			}
			return t;	// Object
		},
		forIn: function(/*Object*/ obj, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: iterates over all object attributes.
			o = o || d.global; f = df.lambda(f);
			for(var i in obj){
				if(!(i in empty)){
					f.call(o, obj[i], i, obj);
				}
			}
			return o;	// Object
		},
		mapIn: function(/*Object*/ obj, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: creates new object with the results of calling 
			//	a provided function on every attribute in this object.
			o = o || d.global; f = df.lambda(f);
			var t = {}, i;
			for(i in obj){
				if(!(i in empty)){
					t[i] = f.call(o, obj[i], i, obj);
				}
			}
			return t;	// Object
		}
	});
})();

}

if(!dojo._hasResource["dojox.lang.functional"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.functional"] = true;
dojo.provide("dojox.lang.functional");





}

if(!dojo._hasResource["dojox.lang.functional.fold"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.functional.fold"] = true;
dojo.provide("dojox.lang.functional.fold");



// This module adds high-level functions and related constructs:
//	- "fold" family of functions

// Notes:
//	- missing high-level functions are provided with the compatible API: 
//		foldl, foldl1, foldr, foldr1
//	- missing JS standard functions are provided with the compatible API: 
//		reduce, reduceRight
//	- the fold's counterpart: unfold

// Defined methods:
//	- take any valid lambda argument as the functional argument
//	- operate on dense arrays
//	- take a string as the array argument
//	- take an iterator objects as the array argument (only foldl, foldl1, and reduce)

(function(){
	var d = dojo, df = dojox.lang.functional, empty = {};

	d.mixin(df, {
		// classic reduce-class functions
		foldl: function(/*Array|String|Object*/ a, /*Function*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from left
			//	to right using a seed value as a starting point; returns the final
			//	value.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var i, n;
			if(d.isArray(a)){
				// array
				for(i = 0, n = a.length; i < n; z = f.call(o, z, a[i], i, a), ++i);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				for(i = 0; a.hasNext(); z = f.call(o, z, a.next(), i++, a));
			}else{
				// object/dictionary
				for(i in a){
					if(!(i in empty)){
						z = f.call(o, z, a[i], i, a);
					}
				}
			}
			return z;	// Object
		},
		foldl1: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from left
			//	to right; returns the final value.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var z, i, n;
			if(d.isArray(a)){
				// array
				z = a[0];
				for(i = 1, n = a.length; i < n; z = f.call(o, z, a[i], i, a), ++i);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				if(a.hasNext()){
					z = a.next();
					for(i = 1; a.hasNext(); z = f.call(o, z, a.next(), i++, a));
				}
			}else{
				// object/dictionary
				var first = true;
				for(i in a){
					if(!(i in empty)){
						if(first){
							z = a[i];
							first = false;
						}else{
							z = f.call(o, z, a[i], i, a);
						}
					}
				}
			}
			return z;	// Object
		},
		foldr: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from right
			//	to left using a seed value as a starting point; returns the final 
			//	value.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			for(var i = a.length; i > 0; --i, z = f.call(o, z, a[i], i, a));
			return z;	// Object
		},
		foldr1: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from right
			//	to left; returns the final value.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var n = a.length, z = a[n - 1], i = n - 1;
			for(; i > 0; --i, z = f.call(o, z, a[i], i, a));
			return z;	// Object
		},
		// JS 1.8 standard array functions, which can take a lambda as a parameter.
		reduce: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ z){
			// summary: apply a function simultaneously against two values of the array
			//	(from left-to-right) as to reduce it to a single value.
			return arguments.length < 3 ? df.foldl1(a, f) : df.foldl(a, f, z);	// Object
		},
		reduceRight: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ z){
			// summary: apply a function simultaneously against two values of the array
			//	(from right-to-left) as to reduce it to a single value.
			return arguments.length < 3 ? df.foldr1(a, f) : df.foldr(a, f, z);	// Object
		},
		// the fold's counterpart: unfold
		unfold: function(/*Function|String|Array*/ pr, /*Function|String|Array*/ f,
						/*Function|String|Array*/ g, /*Object*/ z, /*Object?*/ o){
			// summary: builds an array by unfolding a value
			o = o || d.global; f = df.lambda(f); g = df.lambda(g); pr = df.lambda(pr);
			var t = [];
			for(; !pr.call(o, z); t.push(f.call(o, z)), z = g.call(o, z));
			return t;	// Array
		}
	});
})();

}

if(!dojo._hasResource["dojox.lang.functional.reversed"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.functional.reversed"] = true;
dojo.provide("dojox.lang.functional.reversed");



// This module adds high-level functions and related constructs:
//	- reversed versions of array-processing functions similar to standard JS functions

// Notes:
//	- this module provides reversed versions of standard array-processing functions: 
//		forEachRev, mapRev, filterRev

// Defined methods:
//	- take any valid lambda argument as the functional argument
//	- operate on dense arrays
//	- take a string as the array argument

(function(){
	var d = dojo, df = dojox.lang.functional;

	d.mixin(df, {
		// JS 1.6 standard array functions, which can take a lambda as a parameter.
		// Consider using dojo._base.array functions, if you don't need the lambda support.
		filterRev: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: creates a new array with all elements that pass the test 
			//	implemented by the provided function.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var t = [], v, i = a.length - 1;
			for(; i >= 0; --i){
				v = a[i];
				if(f.call(o, v, i, a)){ t.push(v); }
			}
			return t;	// Array
		},
		forEachRev: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: executes a provided function once per array element.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			for(var i = a.length - 1; i >= 0; f.call(o, a[i], i, a), --i);
		},
		mapRev: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: creates a new array with the results of calling 
			//	a provided function on every element in this array.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var n = a.length, t = new Array(n), i = n - 1, j = 0;
			for(; i >= 0; t[j++] = f.call(o, a[i], i, a), --i);
			return t;	// Array
		},
		everyRev: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: tests whether all elements in the array pass the test 
			//	implemented by the provided function.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			for(var i = a.length - 1; i >= 0; --i){
				if(!f.call(o, a[i], i, a)){
					return false;	// Boolean
				}
			}
			return true;	// Boolean
		},
		someRev: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: tests whether some element in the array passes the test 
			//	implemented by the provided function.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			for(var i = a.length - 1; i >= 0; --i){
				if(f.call(o, a[i], i, a)){
					return true;	// Boolean
				}
			}
			return false;	// Boolean
		}
	});
})();

}

if(!dojo._hasResource["dojo.colors"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.colors"] = true;
dojo.provide("dojo.colors");

//TODO: this module appears to break naming conventions

/*=====
dojo.colors = {
	// summary: Color utilities
}
=====*/

(function(){
	// this is a standard conversion prescribed by the CSS3 Color Module
	var hue2rgb = function(m1, m2, h){
		if(h < 0){ ++h; }
		if(h > 1){ --h; }
		var h6 = 6 * h;
		if(h6 < 1){ return m1 + (m2 - m1) * h6; }
		if(2 * h < 1){ return m2; }
		if(3 * h < 2){ return m1 + (m2 - m1) * (2 / 3 - h) * 6; }
		return m1;
	};
	
	dojo.colorFromRgb = function(/*String*/ color, /*dojo.Color?*/ obj){
		// summary:
		//		get rgb(a) array from css-style color declarations
		// description:
		//		this function can handle all 4 CSS3 Color Module formats: rgb,
		//		rgba, hsl, hsla, including rgb(a) with percentage values.
		var m = color.toLowerCase().match(/^(rgba?|hsla?)\(([\s\.\-,%0-9]+)\)/);
		if(m){
			var c = m[2].split(/\s*,\s*/), l = c.length, t = m[1], a;
			if((t == "rgb" && l == 3) || (t == "rgba" && l == 4)){
				var r = c[0];
				if(r.charAt(r.length - 1) == "%"){
					// 3 rgb percentage values
					a = dojo.map(c, function(x){
						return parseFloat(x) * 2.56;
					});
					if(l == 4){ a[3] = c[3]; }
					return dojo.colorFromArray(a, obj);	// dojo.Color
				}
				return dojo.colorFromArray(c, obj);	// dojo.Color
			}
			if((t == "hsl" && l == 3) || (t == "hsla" && l == 4)){
				// normalize hsl values
				var H = ((parseFloat(c[0]) % 360) + 360) % 360 / 360,
					S = parseFloat(c[1]) / 100,
					L = parseFloat(c[2]) / 100,
					// calculate rgb according to the algorithm 
					// recommended by the CSS3 Color Module 
					m2 = L <= 0.5 ? L * (S + 1) : L + S - L * S, 
					m1 = 2 * L - m2;
				a = [
					hue2rgb(m1, m2, H + 1 / 3) * 256,
					hue2rgb(m1, m2, H) * 256,
					hue2rgb(m1, m2, H - 1 / 3) * 256,
					1
				];
				if(l == 4){ a[3] = c[3]; }
				return dojo.colorFromArray(a, obj);	// dojo.Color
			}
		}
		return null;	// dojo.Color
	};
	
	var confine = function(c, low, high){
		// summary:
		//		sanitize a color component by making sure it is a number,
		//		and clamping it to valid values
		c = Number(c);
		return isNaN(c) ? high : c < low ? low : c > high ? high : c;	// Number
	};
	
	dojo.Color.prototype.sanitize = function(){
		// summary: makes sure that the object has correct attributes
		var t = this;
		t.r = Math.round(confine(t.r, 0, 255));
		t.g = Math.round(confine(t.g, 0, 255));
		t.b = Math.round(confine(t.b, 0, 255));
		t.a = confine(t.a, 0, 1);
		return this;	// dojo.Color
	};
})();


dojo.colors.makeGrey = function(/*Number*/ g, /*Number?*/ a){
	// summary: creates a greyscale color with an optional alpha
	return dojo.colorFromArray([g, g, g, a]);
};

// mixin all CSS3 named colors not already in _base, along with SVG 1.0 variant spellings
dojo.mixin(dojo.Color.named, {
	aliceblue:	[240,248,255],
	antiquewhite:	[250,235,215],
	aquamarine:	[127,255,212],
	azure:	[240,255,255],
	beige:	[245,245,220],
	bisque:	[255,228,196],
	blanchedalmond:	[255,235,205],
	blueviolet:	[138,43,226],
	brown:	[165,42,42],
	burlywood:	[222,184,135],
	cadetblue:	[95,158,160],
	chartreuse:	[127,255,0],
	chocolate:	[210,105,30],
	coral:	[255,127,80],
	cornflowerblue:	[100,149,237],
	cornsilk:	[255,248,220],
	crimson:	[220,20,60],
	cyan:	[0,255,255],
	darkblue:	[0,0,139],
	darkcyan:	[0,139,139],
	darkgoldenrod:	[184,134,11],
	darkgray:	[169,169,169],
	darkgreen:	[0,100,0],
	darkgrey:	[169,169,169],
	darkkhaki:	[189,183,107],
	darkmagenta:	[139,0,139],
	darkolivegreen:	[85,107,47],
	darkorange:	[255,140,0],
	darkorchid:	[153,50,204],
	darkred:	[139,0,0],
	darksalmon:	[233,150,122],
	darkseagreen:	[143,188,143],
	darkslateblue:	[72,61,139],
	darkslategray:	[47,79,79],
	darkslategrey:	[47,79,79],
	darkturquoise:	[0,206,209],
	darkviolet:	[148,0,211],
	deeppink:	[255,20,147],
	deepskyblue:	[0,191,255],
	dimgray:	[105,105,105],
	dimgrey:	[105,105,105],
	dodgerblue:	[30,144,255],
	firebrick:	[178,34,34],
	floralwhite:	[255,250,240],
	forestgreen:	[34,139,34],
	gainsboro:	[220,220,220],
	ghostwhite:	[248,248,255],
	gold:	[255,215,0],
	goldenrod:	[218,165,32],
	greenyellow:	[173,255,47],
	grey:	[128,128,128],
	honeydew:	[240,255,240],
	hotpink:	[255,105,180],
	indianred:	[205,92,92],
	indigo:	[75,0,130],
	ivory:	[255,255,240],
	khaki:	[240,230,140],
	lavender:	[230,230,250],
	lavenderblush:	[255,240,245],
	lawngreen:	[124,252,0],
	lemonchiffon:	[255,250,205],
	lightblue:	[173,216,230],
	lightcoral:	[240,128,128],
	lightcyan:	[224,255,255],
	lightgoldenrodyellow:	[250,250,210],
	lightgray:	[211,211,211],
	lightgreen:	[144,238,144],
	lightgrey:	[211,211,211],
	lightpink:	[255,182,193],
	lightsalmon:	[255,160,122],
	lightseagreen:	[32,178,170],
	lightskyblue:	[135,206,250],
	lightslategray:	[119,136,153],
	lightslategrey:	[119,136,153],
	lightsteelblue:	[176,196,222],
	lightyellow:	[255,255,224],
	limegreen:	[50,205,50],
	linen:	[250,240,230],
	magenta:	[255,0,255],
	mediumaquamarine:	[102,205,170],
	mediumblue:	[0,0,205],
	mediumorchid:	[186,85,211],
	mediumpurple:	[147,112,219],
	mediumseagreen:	[60,179,113],
	mediumslateblue:	[123,104,238],
	mediumspringgreen:	[0,250,154],
	mediumturquoise:	[72,209,204],
	mediumvioletred:	[199,21,133],
	midnightblue:	[25,25,112],
	mintcream:	[245,255,250],
	mistyrose:	[255,228,225],
	moccasin:	[255,228,181],
	navajowhite:	[255,222,173],
	oldlace:	[253,245,230],
	olivedrab:	[107,142,35],
	orange:	[255,165,0],
	orangered:	[255,69,0],
	orchid:	[218,112,214],
	palegoldenrod:	[238,232,170],
	palegreen:	[152,251,152],
	paleturquoise:	[175,238,238],
	palevioletred:	[219,112,147],
	papayawhip:	[255,239,213],
	peachpuff:	[255,218,185],
	peru:	[205,133,63],
	pink:	[255,192,203],
	plum:	[221,160,221],
	powderblue:	[176,224,230],
	rosybrown:	[188,143,143],
	royalblue:	[65,105,225],
	saddlebrown:	[139,69,19],
	salmon:	[250,128,114],
	sandybrown:	[244,164,96],
	seagreen:	[46,139,87],
	seashell:	[255,245,238],
	sienna:	[160,82,45],
	skyblue:	[135,206,235],
	slateblue:	[106,90,205],
	slategray:	[112,128,144],
	slategrey:	[112,128,144],
	snow:	[255,250,250],
	springgreen:	[0,255,127],
	steelblue:	[70,130,180],
	tan:	[210,180,140],
	thistle:	[216,191,216],
	tomato:	[255,99,71],
	transparent: [0, 0, 0, 0],
	turquoise:	[64,224,208],
	violet:	[238,130,238],
	wheat:	[245,222,179],
	whitesmoke:	[245,245,245],
	yellowgreen:	[154,205,50]
});

}

if(!dojo._hasResource["dojox.color._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.color._base"] = true;
dojo.provide("dojox.color._base");


//	alias all the dojo.Color mechanisms
dojox.color.Color=dojo.Color;
dojox.color.blend=dojo.blendColors;
dojox.color.fromRgb=dojo.colorFromRgb;
dojox.color.fromHex=dojo.colorFromHex;
dojox.color.fromArray=dojo.colorFromArray;
dojox.color.fromString=dojo.colorFromString;

//	alias the dojo.colors mechanisms
dojox.color.greyscale=dojo.colors.makeGrey;

//	static methods
dojo.mixin(dojox.color, {
	fromCmy: function(/* Object|Array|int */cyan, /*int*/magenta, /*int*/yellow){
		//	summary
		//	Create a dojox.color.Color from a CMY defined color.
		//	All colors should be expressed as 0-100 (percentage)

		if(dojo.isArray(cyan)){
			magenta=cyan[1], yellow=cyan[2], cyan=cyan[0];
		} else if(dojo.isObject(cyan)){
			magenta=cyan.m, yellow=cyan.y, cyan=cyan.c;
		}
		cyan/=100, magenta/=100, yellow/=100;

		var r=1-cyan, g=1-magenta, b=1-yellow;
		return new dojox.color.Color({ r:Math.round(r*255), g:Math.round(g*255), b:Math.round(b*255) });	//	dojox.color.Color
	},

	fromCmyk: function(/* Object|Array|int */cyan, /*int*/magenta, /*int*/yellow, /*int*/black){
		//	summary
		//	Create a dojox.color.Color from a CMYK defined color.
		//	All colors should be expressed as 0-100 (percentage)

		if(dojo.isArray(cyan)){
			magenta=cyan[1], yellow=cyan[2], black=cyan[3], cyan=cyan[0];
		} else if(dojo.isObject(cyan)){
			magenta=cyan.m, yellow=cyan.y, black=cyan.b, cyan=cyan.c;
		}
		cyan/=100, magenta/=100, yellow/=100, black/=100;
		var r,g,b;
		r = 1-Math.min(1, cyan*(1-black)+black);
		g = 1-Math.min(1, magenta*(1-black)+black);
		b = 1-Math.min(1, yellow*(1-black)+black);
		return new dojox.color.Color({ r:Math.round(r*255), g:Math.round(g*255), b:Math.round(b*255) });	//	dojox.color.Color
	},
	
	fromHsl: function(/* Object|Array|int */hue, /* int */saturation, /* int */luminosity){
		//	summary
		//	Create a dojox.color.Color from an HSL defined color.
		//	hue from 0-359 (degrees), saturation and luminosity 0-100.

		if(dojo.isArray(hue)){
			saturation=hue[1], luminosity=hue[2], hue=hue[0];
		} else if(dojo.isObject(hue)){
			saturation=hue.s, luminosity=hue.l, hue=hue.h;
		}
		saturation/=100;
		luminosity/=100;

		while(hue<0){ hue+=360; }
		while(hue>=360){ hue-=360; }
		
		var r, g, b;
		if(hue<120){
			r=(120-hue)/60, g=hue/60, b=0;
		} else if (hue<240){
			r=0, g=(240-hue)/60, b=(hue-120)/60;
		} else {
			r=(hue-240)/60, g=0, b=(360-hue)/60;
		}
		
		r=2*saturation*Math.min(r, 1)+(1-saturation);
		g=2*saturation*Math.min(g, 1)+(1-saturation);
		b=2*saturation*Math.min(b, 1)+(1-saturation);
		if(luminosity<0.5){
			r*=luminosity, g*=luminosity, b*=luminosity;
		}else{
			r=(1-luminosity)*r+2*luminosity-1;
			g=(1-luminosity)*g+2*luminosity-1;
			b=(1-luminosity)*b+2*luminosity-1;
		}
		return new dojox.color.Color({ r:Math.round(r*255), g:Math.round(g*255), b:Math.round(b*255) });	//	dojox.color.Color
	},
	
	fromHsv: function(/* Object|Array|int */hue, /* int */saturation, /* int */value){
		//	summary
		//	Create a dojox.color.Color from an HSV defined color.
		//	hue from 0-359 (degrees), saturation and value 0-100.

		if(dojo.isArray(hue)){
			saturation=hue[1], value=hue[2], hue=hue[0];
		} else if (dojo.isObject(hue)){
			saturation=hue.s, value=hue.v, hue=hue.h;
		}
		
		if(hue==360){ hue=0; }
		saturation/=100;
		value/=100;
		
		var r, g, b;
		if(saturation==0){
			r=value, b=value, g=value;
		}else{
			var hTemp=hue/60, i=Math.floor(hTemp), f=hTemp-i;
			var p=value*(1-saturation);
			var q=value*(1-(saturation*f));
			var t=value*(1-(saturation*(1-f)));
			switch(i){
				case 0:{ r=value, g=t, b=p; break; }
				case 1:{ r=q, g=value, b=p; break; }
				case 2:{ r=p, g=value, b=t; break; }
				case 3:{ r=p, g=q, b=value; break; }
				case 4:{ r=t, g=p, b=value; break; }
				case 5:{ r=value, g=p, b=q; break; }
			}
		}
		return new dojox.color.Color({ r:Math.round(r*255), g:Math.round(g*255), b:Math.round(b*255) });	//	dojox.color.Color
	}
});

//	Conversions directly on dojox.color.Color
dojo.extend(dojox.color.Color, {
	toCmy: function(){
		//	summary
		//	Convert this Color to a CMY definition.
		var cyan=1-(this.r/255), magenta=1-(this.g/255), yellow=1-(this.b/255);
		return { c:Math.round(cyan*100), m:Math.round(magenta*100), y:Math.round(yellow*100) };		//	Object
	},
	
	toCmyk: function(){
		//	summary
		//	Convert this Color to a CMYK definition.
		var cyan, magenta, yellow, black;
		var r=this.r/255, g=this.g/255, b=this.b/255;
		black = Math.min(1-r, 1-g, 1-b);
		cyan = (1-r-black)/(1-black);
		magenta = (1-g-black)/(1-black);
		yellow = (1-b-black)/(1-black);
		return { c:Math.round(cyan*100), m:Math.round(magenta*100), y:Math.round(yellow*100), b:Math.round(black*100) };	//	Object
	},
	
	toHsl: function(){
		//	summary
		//	Convert this Color to an HSL definition.
		var r=this.r/255, g=this.g/255, b=this.b/255;
		var min = Math.min(r, b, g), max = Math.max(r, g, b);
		var delta = max-min;
		var h=0, s=0, l=(min+max)/2;
		if(l>0 && l<1){
			s = delta/((l<0.5)?(2*l):(2-2*l));
		}
		if(delta>0){
			if(max==r && max!=g){
				h+=(g-b)/delta;
			}
			if(max==g && max!=b){
				h+=(2+(b-r)/delta);
			}
			if(max==b && max!=r){
				h+=(4+(r-g)/delta);
			}
			h*=60;
		}
		return { h:h, s:Math.round(s*100), l:Math.round(l*100) };	//	Object
	},

	toHsv: function(){
		//	summary
		//	Convert this Color to an HSV definition.
		var r=this.r/255, g=this.g/255, b=this.b/255;
		var min = Math.min(r, b, g), max = Math.max(r, g, b);
		var delta = max-min;
		var h = null, s = (max==0)?0:(delta/max);
		if(s==0){
			h = 0;
		}else{
			if(r==max){
				h = 60*(g-b)/delta;
			}else if(g==max){
				h = 120 + 60*(b-r)/delta;
			}else{
				h = 240 + 60*(r-g)/delta;
			}

			if(h<0){ h+=360; }
		}
		return { h:h, s:Math.round(s*100), v:Math.round(max*100) };	//	Object
	}
});

}

if(!dojo._hasResource["dojox.color"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.color"] = true;
dojo.provide("dojox.color");


}

if(!dojo._hasResource["dojox.color.Palette"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.color.Palette"] = true;
dojo.provide("dojox.color.Palette");


(function(){
	var dxc = dojox.color;
	/***************************************************************
	*	dojox.color.Palette
	*
	*	The Palette object is loosely based on the color palettes
	*	at Kuler (http://kuler.adobe.com).  They are 5 color palettes
	*	with the base color considered to be the third color in the
	*	palette (for generation purposes).
	*
	*	Palettes can be generated from well-known algorithms or they
	* 	can be manually created by passing an array to the constructor.
	*
	*	Palettes can be transformed, using a set of specific params
	*	similar to the way shapes can be transformed with dojox.gfx.
	*	However, unlike with transformations in dojox.gfx, transforming
	* 	a palette will return you a new Palette object, in effect
	* 	a clone of the original.
	***************************************************************/

	//	ctor ----------------------------------------------------------------------------
	dxc.Palette = function(/* String|Array|dojox.color.Color|dojox.color.Palette */base){
		//	summary:
		//		An object that represents a palette of colors.
		//	description:
		//		A Palette is a representation of a set of colors.  While the standard
		//		number of colors contained in a palette is 5, it can really handle any
		//		number of colors.
		//
		//		A palette is useful for the ability to transform all the colors in it
		//		using a simple object-based approach.  In addition, you can generate
		//		palettes using dojox.color.Palette.generate; these generated palettes
		//		are based on the palette generators at http://kuler.adobe.com.
		//
		//	colors: dojox.color.Color[]
		//		The actual color references in this palette.
		this.colors = [];
		if(base instanceof dojox.color.Palette){
			this.colors = base.colors.slice(0);
		}
		else if(base instanceof dojox.color.Color){
			this.colors = [ null, null, base, null, null ];
		}
		else if(dojo.isArray(base)){
			this.colors = dojo.map(base.slice(0), function(item){
				if(dojo.isString(item)){ return new dojox.color.Color(item); }
				return item;
			});
		}
		else if (dojo.isString(base)){
			this.colors = [ null, null, new dojox.color.Color(base), null, null ];
		}
	}

	//	private functions ---------------------------------------------------------------

	//	transformations
	function tRGBA(p, param, val){
		var ret = new dojox.color.Palette();
		ret.colors = [];
		dojo.forEach(p.colors, function(item){
			var r=(param=="dr")?item.r+val:item.r,
				g=(param=="dg")?item.g+val:item.g,
				b=(param=="db")?item.b+val:item.b,
				a=(param=="da")?item.a+val:item.a
			ret.colors.push(new dojox.color.Color({
				r: Math.min(255, Math.max(0, r)),
				g: Math.min(255, Math.max(0, g)),
				b: Math.min(255, Math.max(0, b)),
				a: Math.min(1, Math.max(0, a))
			}));
		});
		return ret;
	}

	function tCMY(p, param, val){
		var ret = new dojox.color.Palette();
		ret.colors = [];
		dojo.forEach(p.colors, function(item){
			var o=item.toCmy(), 
				c=(param=="dc")?o.c+val:o.c,
				m=(param=="dm")?o.m+val:o.m,
				y=(param=="dy")?o.y+val:o.y;
			ret.colors.push(dojox.color.fromCmy(
				Math.min(100, Math.max(0, c)),
				Math.min(100, Math.max(0, m)),
				Math.min(100, Math.max(0, y))
			));
		});
		return ret;
	}

	function tCMYK(p, param, val){
		var ret = new dojox.color.Palette();
		ret.colors = [];
		dojo.forEach(p.colors, function(item){
			var o=item.toCmyk(), 
				c=(param=="dc")?o.c+val:o.c,
				m=(param=="dm")?o.m+val:o.m,
				y=(param=="dy")?o.y+val:o.y,
				k=(param=="dk")?o.b+val:o.b;
			ret.colors.push(dojox.color.fromCmyk(
				Math.min(100, Math.max(0, c)),
				Math.min(100, Math.max(0, m)),
				Math.min(100, Math.max(0, y)),
				Math.min(100, Math.max(0, k))
			));
		});
		return ret;
	}

	function tHSL(p, param, val){
		var ret = new dojox.color.Palette();
		ret.colors = [];
		dojo.forEach(p.colors, function(item){
			var o=item.toHsl(), 
				h=(param=="dh")?o.h+val:o.h,
				s=(param=="ds")?o.s+val:o.s,
				l=(param=="dl")?o.l+val:o.l;
			ret.colors.push(dojox.color.fromHsl(h%360, Math.min(100, Math.max(0, s)), Math.min(100, Math.max(0, l))));
		});
		return ret;
	}

	function tHSV(p, param, val){
		var ret = new dojox.color.Palette();
		ret.colors = [];
		dojo.forEach(p.colors, function(item){
			var o=item.toHsv(), 
				h=(param=="dh")?o.h+val:o.h,
				s=(param=="ds")?o.s+val:o.s,
				v=(param=="dv")?o.v+val:o.v;
			ret.colors.push(dojox.color.fromHsv(h%360, Math.min(100, Math.max(0, s)), Math.min(100, Math.max(0, v))));
		});
		return ret;
	}

	//	helper functions
	function rangeDiff(val, low, high){
		//	given the value in a range from 0 to high, find the equiv
		//		using the range low to high.
		return high-((high-val)*((high-low)/high));
	}

	//	object methods ---------------------------------------------------------------
	dojo.extend(dxc.Palette, {
		transform: function(/* dojox.color.Palette.__transformArgs */kwArgs){
			//	summary:
			//		Transform the palette using a specific transformation function
			//		and a set of transformation parameters.
			//	description:
			//		{palette}.transform is a simple way to uniformly transform
			//		all of the colors in a palette using any of 5 formulae:
			//		RGBA, HSL, HSV, CMYK or CMY.
			//
			//		Once the forumula to be used is determined, you can pass any
			//		number of parameters based on the formula "d"[param]; for instance,
			//		{ use: "rgba", dr: 20, dg: -50 } will take all of the colors in
			//		palette, add 20 to the R value and subtract 50 from the G value.
			//
			//		Unlike other types of transformations, transform does *not* alter
			//		the original palette but will instead return a new one.
			var fn=tRGBA;	//	the default transform function.
			if(kwArgs.use){
				//	we are being specific about the algo we want to use.
				var use=kwArgs.use.toLowerCase();
				if(use.indexOf("hs")==0){
					if(use.charAt(2)=="l"){ fn=tHSL; }
					else { fn=tHSV; }
				}
				else if(use.indexOf("cmy")==0){
					if(use.charAt(3)=="k"){ fn=tCMYK; }
					else { fn=tCMY; }
				}
			}
			//	try to guess the best choice.
			else if("dc" in kwArgs || "dm" in kwArgs || "dy" in kwArgs){
				if("dk" in kwArgs){ fn = tCMYK; }
				else { fn = tCMY; }
			}
			else if("dh" in kwArgs || "ds" in kwArgs){
				if("dv" in kwArgs){ fn = tHSV; }
				else { fn = tHSL; }
			}

			var palette = this;
			for(var p in kwArgs){
				//	ignore use
				if(p=="use"){ continue; }
				palette = fn(palette, p, kwArgs[p]);
			}
			return palette;		//	dojox.color.Palette
		},
		clone: function(){
			//	summary:
			//		Clones the current palette.
			return new dxc.Palette(this);	//	dojox.color.Palette
		}
	});

/*=====
dojox.color.Palette.__transformArgs = function(use, dr, dg, db, da, dc, dm, dy, dk, dh, ds, dv, dl){
	//	summary:
	//		The keywords argument to be passed to the dojox.color.Palette.transform function.  Note that
	//		while all arguments are optional, *some* arguments must be passed.  The basic concept is that
	//		you pass a delta value for a specific aspect of a color model (or multiple aspects of the same
	//		color model); for instance, if you wish to transform a palette based on the HSV color model,
	//		you would pass one of "dh", "ds", or "dv" as a value.
	//
	//	use: String?
	//		Specify the color model to use for the transformation.  Can be "rgb", "rgba", "hsv", "hsl", "cmy", "cmyk".
	//	dr: Number?
	//		The delta to be applied to the red aspect of the RGB/RGBA color model.
	//	dg: Number?
	//		The delta to be applied to the green aspect of the RGB/RGBA color model.
	//	db: Number?
	//		The delta to be applied to the blue aspect of the RGB/RGBA color model.
	//	da: Number?
	//		The delta to be applied to the alpha aspect of the RGBA color model.
	//	dc: Number?
	//		The delta to be applied to the cyan aspect of the CMY/CMYK color model.
	//	dm: Number?
	//		The delta to be applied to the magenta aspect of the CMY/CMYK color model.
	//	dy: Number?
	//		The delta to be applied to the yellow aspect of the CMY/CMYK color model.
	//	dk: Number?
	//		The delta to be applied to the black aspect of the CMYK color model.
	//	dh: Number?
	//		The delta to be applied to the hue aspect of the HSL/HSV color model.
	//	ds: Number?
	//		The delta to be applied to the saturation aspect of the HSL/HSV color model.
	//	dl: Number?
	//		The delta to be applied to the luminosity aspect of the HSL color model.
	//	dv: Number?
	//		The delta to be applied to the value aspect of the HSV color model.
	this.use = use;
	this.dr = dr;
	this.dg = dg;
	this.db = db;
	this.da = da;
	this.dc = dc;
	this.dm = dm;
	this.dy = dy;
	this.dk = dk;
	this.dh = dh;
	this.ds = ds;
	this.dl = dl;
	this.dv = dv;
}
dojox.color.Palette.__generatorArgs = function(base){
	//	summary:
	//		The keyword arguments object used to create a palette based on a base color.
	//
	//	base: dojo.Color
	//		The base color to be used to generate the palette.
	this.base = base;
} 
dojox.color.Palette.__analogousArgs = function(base, high, low){
	//	summary:
	//		The keyword arguments object that is used to create a 5 color palette based on the
	//		analogous rules as implemented at http://kuler.adobe.com, using the HSV color model.
	//
	//	base: dojo.Color
	//		The base color to be used to generate the palette.
	//	high: Number?
	//		The difference between the hue of the base color and the highest hue.  In degrees, default is 60.
	//	low: Number?
	//		The difference between the hue of the base color and the lowest hue.  In degrees, default is 18.
	this.base = base;
	this.high = high;
	this.low = low;
} 
dojox.color.Palette.__splitComplementaryArgs = function(base, da){
	//	summary:
	//		The keyword arguments object used to create a palette based on the split complementary rules
	//		as implemented at http://kuler.adobe.com.
	//
	//	base: dojo.Color
	//		The base color to be used to generate the palette.
	//	da: Number?
	//		The delta angle to be used to determine where the split for the complementary rules happen.
	//		In degrees, the default is 30.
	this.base = base;
	this.da = da;
} 
=====*/
	dojo.mixin(dxc.Palette, {
		generators: {
			analogous:function(/* dojox.color.Palette.__analogousArgs */args){
				//	summary:
				//		Create a 5 color palette based on the analogous rules as implemented at
				//		http://kuler.adobe.com.
				var high=args.high||60, 	//	delta between base hue and highest hue (subtracted from base)
					low=args.low||18,		//	delta between base hue and lowest hue (added to base)
					base = dojo.isString(args.base)?new dojox.color.Color(args.base):args.base,
					hsv=base.toHsv();

				//	generate our hue angle differences
				var h=[
					(hsv.h+low+360)%360,
					(hsv.h+Math.round(low/2)+360)%360,
					hsv.h,
					(hsv.h-Math.round(high/2)+360)%360,
					(hsv.h-high+360)%360
				];

				var s1=Math.max(10, (hsv.s<=95)?hsv.s+5:(100-(hsv.s-95))),
					s2=(hsv.s>1)?hsv.s-1:21-hsv.s,
					v1=(hsv.v>=92)?hsv.v-9:Math.max(hsv.v+9, 20),
					v2=(hsv.v<=90)?Math.max(hsv.v+5, 20):(95+Math.ceil((hsv.v-90)/2)),
					s=[ s1, s2, hsv.s, s1, s1 ],
					v=[ v1, v2, hsv.v, v1, v2 ]

				return new dxc.Palette(dojo.map(h, function(hue, i){
					return dojox.color.fromHsv(hue, s[i], v[i]);
				}));		//	dojox.color.Palette
			},

			monochromatic: function(/* dojox.color.Palette.__generatorArgs */args){
				//	summary:
				//		Create a 5 color palette based on the monochromatic rules as implemented at
				//		http://kuler.adobe.com.
				var base = dojo.isString(args.base)?new dojox.color.Color(args.base):args.base,
					hsv = base.toHsv();
				
				//	figure out the saturation and value
				var s1 = (hsv.s-30>9)?hsv.s-30:hsv.s+30,
					s2 = hsv.s,
					v1 = rangeDiff(hsv.v, 20, 100),
					v2 = (hsv.v-20>20)?hsv.v-20:hsv.v+60,
					v3 = (hsv.v-50>20)?hsv.v-50:hsv.v+30;

				return new dxc.Palette([
					dojox.color.fromHsv(hsv.h, s1, v1),
					dojox.color.fromHsv(hsv.h, s2, v3),
					base,
					dojox.color.fromHsv(hsv.h, s1, v3),
					dojox.color.fromHsv(hsv.h, s2, v2)
				]);		//	dojox.color.Palette
			},

			triadic: function(/* dojox.color.Palette.__generatorArgs */args){
				//	summary:
				//		Create a 5 color palette based on the triadic rules as implemented at
				//		http://kuler.adobe.com.
				var base = dojo.isString(args.base)?new dojox.color.Color(args.base):args.base,
					hsv = base.toHsv();

				var h1 = (hsv.h+57+360)%360,
					h2 = (hsv.h-157+360)%360,
					s1 = (hsv.s>20)?hsv.s-10:hsv.s+10,
					s2 = (hsv.s>90)?hsv.s-10:hsv.s+10,
					s3 = (hsv.s>95)?hsv.s-5:hsv.s+5,
					v1 = (hsv.v-20>20)?hsv.v-20:hsv.v+20,
					v2 = (hsv.v-30>20)?hsv.v-30:hsv.v+30,
					v3 = (hsv.v-30>70)?hsv.v-30:hsv.v+30;

				return new dxc.Palette([
					dojox.color.fromHsv(h1, s1, hsv.v),
					dojox.color.fromHsv(hsv.h, s2, v2),
					base,
					dojox.color.fromHsv(h2, s2, v1),
					dojox.color.fromHsv(h2, s3, v3)
				]);		//	dojox.color.Palette
			},

			complementary: function(/* dojox.color.Palette.__generatorArgs */args){
				//	summary:
				//		Create a 5 color palette based on the complementary rules as implemented at
				//		http://kuler.adobe.com.
				var base = dojo.isString(args.base)?new dojox.color.Color(args.base):args.base,
					hsv = base.toHsv();

				var h1 = ((hsv.h*2)+137<360)?(hsv.h*2)+137:Math.floor(hsv.h/2)-137,
					s1 = Math.max(hsv.s-10, 0),
					s2 = rangeDiff(hsv.s, 10, 100),
					s3 = Math.min(100, hsv.s+20),
					v1 = Math.min(100, hsv.v+30),
					v2 = (hsv.v>20)?hsv.v-30:hsv.v+30;

				return new dxc.Palette([
					dojox.color.fromHsv(hsv.h, s1, v1),
					dojox.color.fromHsv(hsv.h, s2, v2),
					base,
					dojox.color.fromHsv(h1, s3, v2),
					dojox.color.fromHsv(h1, hsv.s, hsv.v)
				]);		//	dojox.color.Palette
			},

			splitComplementary: function(/* dojox.color.Palette.__splitComplementaryArgs */args){
				//	summary:
				//		Create a 5 color palette based on the split complementary rules as implemented at
				//		http://kuler.adobe.com.
				var base = dojo.isString(args.base)?new dojox.color.Color(args.base):args.base,
					dangle = args.da || 30,
					hsv = base.toHsv();

				var baseh = ((hsv.h*2)+137<360)?(hsv.h*2)+137:Math.floor(hsv.h/2)-137,
					h1 = (baseh-dangle+360)%360,
					h2 = (baseh+dangle)%360,
					s1 = Math.max(hsv.s-10, 0),
					s2 = rangeDiff(hsv.s, 10, 100),
					s3 = Math.min(100, hsv.s+20),
					v1 = Math.min(100, hsv.v+30),
					v2 = (hsv.v>20)?hsv.v-30:hsv.v+30;

				return new dxc.Palette([
					dojox.color.fromHsv(h1, s1, v1),
					dojox.color.fromHsv(h1, s2, v2),
					base,
					dojox.color.fromHsv(h2, s3, v2),
					dojox.color.fromHsv(h2, hsv.s, hsv.v)
				]);		//	dojox.color.Palette
			},

			compound: function(/* dojox.color.Palette.__generatorArgs */args){
				//	summary:
				//		Create a 5 color palette based on the compound rules as implemented at
				//		http://kuler.adobe.com.
				var base = dojo.isString(args.base)?new dojox.color.Color(args.base):args.base,
					hsv = base.toHsv();

				var h1 = ((hsv.h*2)+18<360)?(hsv.h*2)+18:Math.floor(hsv.h/2)-18,
					h2 = ((hsv.h*2)+120<360)?(hsv.h*2)+120:Math.floor(hsv.h/2)-120,
					h3 = ((hsv.h*2)+99<360)?(hsv.h*2)+99:Math.floor(hsv.h/2)-99,
					s1 = (hsv.s-40>10)?hsv.s-40:hsv.s+40,
					s2 = (hsv.s-10>80)?hsv.s-10:hsv.s+10,
					s3 = (hsv.s-25>10)?hsv.s-25:hsv.s+25,
					v1 = (hsv.v-40>10)?hsv.v-40:hsv.v+40,
					v2 = (hsv.v-20>80)?hsv.v-20:hsv.v+20,
					v3 = Math.max(hsv.v, 20);

				return new dxc.Palette([
					dojox.color.fromHsv(h1, s1, v1),
					dojox.color.fromHsv(h1, s2, v2),
					base,
					dojox.color.fromHsv(h2, s3, v3),
					dojox.color.fromHsv(h3, s2, v2)
				]);		//	dojox.color.Palette
			},

			shades: function(/* dojox.color.Palette.__generatorArgs */args){
				//	summary:
				//		Create a 5 color palette based on the shades rules as implemented at
				//		http://kuler.adobe.com.
				var base = dojo.isString(args.base)?new dojox.color.Color(args.base):args.base,
					hsv = base.toHsv();

				var s  = (hsv.s==100 && hsv.v==0)?0:hsv.s,
					v1 = (hsv.v-50>20)?hsv.v-50:hsv.v+30,
					v2 = (hsv.v-25>=20)?hsv.v-25:hsv.v+55,
					v3 = (hsv.v-75>=20)?hsv.v-75:hsv.v+5,
					v4 = Math.max(hsv.v-10, 20);

				return new dxc.Palette([
					new dojox.color.fromHsv(hsv.h, s, v1),
					new dojox.color.fromHsv(hsv.h, s, v2),
					base,
					new dojox.color.fromHsv(hsv.h, s, v3),
					new dojox.color.fromHsv(hsv.h, s, v4)
				]);		//	dojox.color.Palette
			}
		},
		generate: function(/* String|dojox.color.Color */base, /* Function|String */type){
			//	summary:
			//		Generate a new Palette using any of the named functions in
			//		dojox.color.Palette.generators or an optional function definition.  Current
			//		generators include "analogous", "monochromatic", "triadic", "complementary",
			//		"splitComplementary", and "shades".
			if(dojo.isFunction(type)){
				return type({ base: base });	//	dojox.color.Palette
			}
			else if(dxc.Palette.generators[type]){
				return dxc.Palette.generators[type]({ base: base });	//	dojox.color.Palette
			}
			throw new Error("dojox.color.Palette.generate: the specified generator ('" + type + "') does not exist.");
		}
	});
})();

}

if(!dojo._hasResource["dojox.lang.utils"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.utils"] = true;
dojo.provide("dojox.lang.utils");

(function(){
	var empty = {}, du = dojox.lang.utils, opts = Object.prototype.toString;

	var clone = function(o){
		if(o){
			switch(opts.call(o)){
				case "[object Array]":
					return o.slice(0);
				case "[object Object]":
					return dojo.delegate(o);
			}
		}
		return o;
	}
	
	dojo.mixin(du, {
		coerceType: function(target, source){
			// summary: Coerces one object to the type of another.
			// target: Object: object, which typeof result is used to coerce "source" object.
			// source: Object: object, which will be forced to change type.
			switch(typeof target){
				case "number":	return Number(eval("(" + source + ")"));
				case "string":	return String(source);
				case "boolean":	return Boolean(eval("(" + source + ")"));
			}
			return eval("(" + source + ")");
		},
		
		updateWithObject: function(target, source, conv){
			// summary: Updates an existing object in place with properties from an "source" object.
			// target: Object: the "target" object to be updated
			// source: Object: the "source" object, whose properties will be used to source the existed object.
			// conv: Boolean?: force conversion to the original type
			if(!source){ return target; }
			for(var x in target){
				if(x in source && !(x in empty)){
					var t = target[x];
					if(t && typeof t == "object"){
						du.updateWithObject(t, source[x], conv);
					}else{
						target[x] = conv ? du.coerceType(t, source[x]) : clone(source[x]);
					}
				}
			}
			return target;	// Object
		},
	
		updateWithPattern: function(target, source, pattern, conv){
			// summary: Updates an existing object in place with properties from an "source" object.
			// target: Object: the "target" object to be updated
			// source: Object: the "source" object, whose properties will be used to source the existed object.
			// pattern: Array: an array of properties to be copied
			// conv: Boolean?: force conversion to the original type
			if(!source || !pattern){ return target; }
			for(var x in pattern){
				if(x in source && !(x in empty)){
					target[x] = conv ? du.coerceType(pattern[x], source[x]) : clone(source[x]);
				}
			}
			return target;	// Object
		},
		
		merge: function(object, mixin){
			// summary: Merge two objects structurally, mixin properties will override object's properties.
			// object: Object: original object.
			// mixin: Object: additional object, which properties will override object's properties.
			if(mixin){
				var otype = opts.call(object), mtype = opts.call(mixin), t, i, l, m;
				switch(mtype){
					case "[object Array]":
						if(mtype == otype){
							t = new Array(Math.max(object.length, mixin.length));
							for(i = 0, l = t.length; i < l; ++i){
								t[i] = du.merge(object[i], mixin[i]);
							}
							return t;
						}
						return mixin.slice(0);
					case "[object Object]":
						if(mtype == otype && object){
							t = dojo.delegate(object);
							for(i in mixin){
								if(i in object){
									l = object[i];
									m = mixin[i];
									if(m !== l){
										t[i] = du.merge(l, m);
									}
								}else{
									t[i] = dojo.clone(mixin[i]);
								}
							}
							return t;
						}
						return dojo.clone(mixin);
				}
			}
			return mixin;
		}
	});
})();

}

if(!dojo._hasResource["dojox.gfx.gradutils"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.gradutils"] = true;
dojo.provide("dojox.gfx.gradutils");



// Various generic utilities to deal with a linear gradient

(function(){
	var d = dojo, m = dojox.gfx.matrix, C = d.Color;
	
	function findColor(o, c){
		if(o <= 0){
			return c[0].color;
		}
		var len = c.length;
		if(o >= 1){
			return c[len - 1].color;
		}
		//TODO: use binary search
		for(var i = 0; i < len; ++i){
			var stop = c[i];
			if(stop.offset >= o){
				if(i){
					var prev = c[i - 1];
					return d.blendColors(new C(prev.color), new C(stop.color),
						(o - prev.offset) / (stop.offset - prev.offset));
				}
				return stop.color;
			}
		}
		return c[len - 1].color;
	}

	dojox.gfx.gradutils.getColor = function(fill, pt){
		// summary:
		//		sample a color from a gradient using a point
		// fill: Object:
		//		fill object
		// pt: dojox.gfx.Point:
		//		point where to sample a color
		var o;
		if(fill){
			switch(fill.type){
				case "linear":
					var angle = Math.atan2(fill.y2 - fill.y1, fill.x2 - fill.x1),
						rotation = m.rotate(-angle),
						projection = m.project(fill.x2 - fill.x1, fill.y2 - fill.y1),
						p = m.multiplyPoint(projection, pt),
						pf1 = m.multiplyPoint(projection, fill.x1, fill.y1),
						pf2 = m.multiplyPoint(projection, fill.x2, fill.y2),
						scale = m.multiplyPoint(rotation, pf2.x - pf1.x, pf2.y - pf1.y).x,
						o = m.multiplyPoint(rotation, p.x - pf1.x, p.y - pf1.y).x / scale;
					break;
				case "radial":
					var dx = pt.x - fill.cx, dy = pt.y - fill.cy,
						o = Math.sqrt(dx * dx + dy * dy) / fill.r;
					break;
			}
			return findColor(o, fill.colors);	// dojo.Color
		}
		// simple color
		return new C(fill || [0, 0, 0, 0]);	// dojo.Color
	};

	dojox.gfx.gradutils.reverse = function(fill){
		// summary:
		//		reverses a gradient
		// fill: Object:
		//		fill object
		if(fill){
			switch(fill.type){
				case "linear":
				case "radial":
					fill = dojo.delegate(fill);
					if(fill.colors){
						var c = fill.colors, l = c.length, i = 0, stop,
							n = fill.colors = new Array(c.length);
						for(; i < l; ++i){
							stop = c[i];
							n[i] = {
								offset: 1 - stop.offset,
								color:  stop.color
							};
						}
						n.sort(function(a, b){ return a.offset - b.offset; });
					}
					break;
			}
		}
		return fill;	// Object
	};
})();

}

if(!dojo._hasResource["dojox.charting.Theme"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.Theme"] = true;
dojo.provide("dojox.charting.Theme");






dojo.declare("dojox.charting.Theme", null, {
	//	summary:
	//		A Theme is a pre-defined object, primarily JSON-based, that makes up the definitions to
	//		style a chart.
	//
	//	description:
	//		While you can set up style definitions on a chart directly (usually through the various add methods
	//		on a dojox.charting.Chart2D object), a Theme simplifies this manual setup by allowing you to
	//		pre-define all of the various visual parameters of each element in a chart.
	//
	//		Most of the properties of a Theme are straight-forward; if something is line-based (such as
	//		an axis or the ticks on an axis), they will be defined using basic stroke parameters.  Likewise,
	//		if an element is primarily block-based (such as the background of a chart), it will be primarily
	//		fill-based.
	//
	//		In addition (for convenience), a Theme definition does not have to contain the entire JSON-based
	//		structure.  Each theme is built on top of a default theme (which serves as the basis for the theme
	//		"GreySkies"), and is mixed into the default theme object.  This allows you to create a theme based,
	//		say, solely on colors for data series.
	//
	//		Defining a new theme is relatively easy; see any of the themes in dojox.charting.themes for examples
	//		on how to define your own.
	//
	//		When you set a theme on a chart, the theme itself is deep-cloned.  This means that you cannot alter
	//		the theme itself after setting the theme value on a chart, and expect it to change your chart.  If you
	//		are looking to make alterations to a theme for a chart, the suggestion would be to create your own
	//		theme, based on the one you want to use, that makes those alterations before it is applied to a chart.
	//
	//		Finally, a Theme contains a number of functions to facilitate rendering operations on a chart--the main
	//		helper of which is the ~next~ method, in which a chart asks for the information for the next data series
	//		to be rendered.
	//
	//		A note on colors:
	//		The Theme constructor was on the use of dojox.color.Palette (in general) for creating a visually distinct
	//		set of colors for usage in a chart.  A palette is usually comprised of 5 different color definitions, and
	//		no more.  If you have a need to render a chart with more than 5 data elements, you can simply "push"
	//		new color definitions into the theme's .color array.  Make sure that you do that with the actual
	//		theme object from a Chart, and not in the theme itself (i.e. either do that before using .setTheme
	//		on a chart).
	//
	//		example:
	//			The default theme (and structure) looks like so:
	//	|	// all objects are structs used directly in dojox.gfx
	//	|	chart:{
	//	|		stroke: null,
	//	|		fill: "white",
	//	|		pageStyle: null // suggested page style as an object suitable for dojo.style()
	//	|	},
	//	|	plotarea:{
	//	|		stroke: null,
	//	|		fill: "white"
	//	|	},
	//	|	axis:{
	//	|		stroke:	{ // the axis itself
	//	|			color: "#333",
	//	|			width: 1
	//	|		},
	//	|		tick: {	// used as a foundation for all ticks
	//	|			color:     "#666",
	//	|			position:  "center",
	//	|			font:      "normal normal normal 7pt Tahoma",	// labels on axis
	//	|			fontColor: "#333"								// color of labels
	//	|		},
	//	|		majorTick:	{ // major ticks on axis, and used for major gridlines
	//	|			width:  1,
	//	|			length: 6
	//	|		},
	//	|		minorTick:	{ // minor ticks on axis, and used for minor gridlines
	//	|			width:  0.8,
	//	|			length: 3
	//	|		},
	//	|		microTick:	{ // minor ticks on axis, and used for minor gridlines
	//	|			width:  0.5,
	//	|			length: 1
	//	|		}
	//	|	},
	//	|	series: {
	//	|		stroke:  {width: 1.5, color: "#333"},		// line
	//	|		outline: {width: 0.1, color: "#ccc"},		// outline
	//	|		//shadow:  {dx: 1, dy: 1, width: 2, color: [0, 0, 0, 0.3]},
	//	|		shadow: null,								// no shadow
	//	|		fill:    "#ccc",							// fill, if appropriate
	//	|		font:    "normal normal normal 8pt Tahoma",	// if there's a label
	//	|		fontColor: "#000"							// color of labels
	//	|	},
	//	|	marker: {	// any markers on a series
	//	|		symbol:  "m-3,3 l3,-6 3,6 z",				// symbol
	//	|		stroke:  {width: 1.5, color: "#333"},		// stroke
	//	|		outline: {width: 0.1, color: "#ccc"},		// outline
	//	|		shadow: null,								// no shadow
	//	|		fill:    "#ccc",							// fill if needed
	//	|		font:    "normal normal normal 8pt Tahoma",	// label
	//	|		fontColor: "#000"
	//	|	}
	//
	//	example:
	//		Defining a new theme is pretty simple:
	//	|	dojox.charting.themes.Grasslands = new dojox.charting.Theme({
	//	|		colors: [ "#70803a", "#dde574", "#788062", "#b1cc5d", "#eff2c2" ]
	//	|	});
	//	|
	//	|	myChart.setTheme(dojox.charting.themes.Grasslands);

	shapeSpaces: {shape: 1, shapeX: 1, shapeY: 1},

	constructor: function(kwArgs){
		//	summary:
		//		Initialize a theme using the keyword arguments.  Note that the arguments
		//		look like the example (above), and may include a few more parameters.
		kwArgs = kwArgs || {};

		// populate theme with defaults updating them if needed
		var def = dojox.charting.Theme.defaultTheme;
		dojo.forEach(["chart", "plotarea", "axis", "series", "marker"], function(name){
			this[name] = dojo.delegate(def[name], kwArgs[name]);
		}, this);

		// personalize theme
		if(kwArgs.seriesThemes && kwArgs.seriesThemes.length){
			this.colors  = null;
			this.seriesThemes = kwArgs.seriesThemes.slice(0);
		}else{
			this.seriesThemes = null;
			this.colors = (kwArgs.colors || dojox.charting.Theme.defaultColors).slice(0);
		}
		this.markerThemes = null;
		if(kwArgs.markerThemes && kwArgs.markerThemes.length){
			this.markerThemes = kwArgs.markerThemes.slice(0);
		}
		this.markers = dojo.delegate(dojox.charting.Theme.defaultMarkers, kwArgs.markers);

		// set flags
		this.noGradConv = kwArgs.noGradConv;
		this.noRadialConv = kwArgs.noRadialConv;
		if(kwArgs.reverseFills){
			this.reverseFills();
		}

		//	private housekeeping
		this._current = 0;
		this._buildMarkerArray();
	},

	clone: function(){
		//	summary:
		//		Clone the current theme.
		//	returns: dojox.charting.Theme
		//		The cloned theme; any alterations made will not affect the original.
		var theme = new dojox.charting.Theme({
			// theme components
			chart: this.chart,
			plotarea: this.plotarea,
			axis: this.axis,
			series: this.series,
			marker: this.marker,
			// individual arrays
			colors: this.colors,
			markers: this.markers,
			seriesThemes: this.seriesThemes,
			markerThemes: this.markerThemes,
			// flags
			noGradConv: this.noGradConv,
			noRadialConv: this.noRadialConv
		});
		// copy custom methods
		dojo.forEach(
			["clone", "clear", "next", "skip", "addMixin", "post", "getTick"],
			function(name){
				if(this.hasOwnProperty(name)){
					theme[name] = this[name];
				}
			},
			this
		);
		return theme;	//	dojox.charting.Theme
	},

	clear: function(){
		//	summary:
		//		Clear and reset the internal pointer to start fresh.
		this._current = 0;
	},

	next: function(elementType, mixin, doPost){
		//	summary:
		//		Get the next color or series theme.
		//	elementType: String?
		//		An optional element type (for use with series themes)
		//	mixin: Object?
		//		An optional object to mix into the theme.
		//	doPost: Boolean?
		//		A flag to post-process the results.
		//	returns: Object
		//		An object of the structure { series, marker, symbol }
		var merge = dojox.lang.utils.merge, series, marker;
		if(this.colors){
			series = dojo.delegate(this.series);
			marker = dojo.delegate(this.marker);
			var color = new dojo.Color(this.colors[this._current % this.colors.length]), old;
			// modify the stroke
			if(series.stroke && series.stroke.color){
				series.stroke = dojo.delegate(series.stroke);
				old = new dojo.Color(series.stroke.color);
				series.stroke.color = new dojo.Color(color);
				series.stroke.color.a = old.a;
			}else{
				series.stroke = {color: color};
			}
			if(marker.stroke && marker.stroke.color){
				marker.stroke = dojo.delegate(marker.stroke);
				old = new dojo.Color(marker.stroke.color);
				marker.stroke.color = new dojo.Color(color);
				marker.stroke.color.a = old.a;
			}else{
				marker.stroke = {color: color};
			}
			// modify the fill
			if(!series.fill || series.fill.type){
				series.fill = color;
			}else{
				old = new dojo.Color(series.fill);
				series.fill = new dojo.Color(color);
				series.fill.a = old.a;
			}
			if(!marker.fill || marker.fill.type){
				marker.fill = color;
			}else{
				old = new dojo.Color(marker.fill);
				marker.fill = new dojo.Color(color);
				marker.fill.a = old.a;
			}
		}else{
			series = this.seriesThemes ?
				merge(this.series, this.seriesThemes[this._current % this.seriesThemes.length]) :
				this.series;
			marker = this.markerThemes ?
				merge(this.marker, this.markerThemes[this._current % this.markerThemes.length]) :
				series;
		}

		var symbol = marker && marker.symbol || this._markers[this._current % this._markers.length];

		var theme = {series: series, marker: marker, symbol: symbol};
		
		// advance the counter
		++this._current;

		if(mixin){
			theme = this.addMixin(theme, elementType, mixin);
		}
		if(doPost){
			theme = this.post(theme, elementType);
		}

		return theme;	//	Object
	},

	skip: function(){
		//	summary:
		//		Skip the next internal color.
		++this._current;
	},

	addMixin: function(theme, elementType, mixin, doPost){
		//	summary:
		//		Add a mixin object to the passed theme and process.
		//	theme: dojox.charting.Theme
		//		The theme to mixin to.
		//	elementType: String
		//		The type of element in question. Can be "line", "bar" or "circle"
		//	mixin: Object|Array
		//		The object or objects to mix into the theme.
		//	doPost: Boolean
		//		If true, run the new theme through the post-processor.
		//	returns: dojox.charting.Theme
		//		The new theme.
		if(dojo.isArray(mixin)){
			dojo.forEach(mixin, function(m){
				theme = this.addMixin(theme, elementType, m);
			}, this);
		}else{
			var t = {};
			if("color" in mixin){
				if(elementType == "line" || elementType == "area"){
					dojo.setObject("series.stroke.color", mixin.color, t);
					dojo.setObject("marker.stroke.color", mixin.color, t);
				}else{
					dojo.setObject("series.fill", mixin.color, t);
				}
			}
			dojo.forEach(["stroke", "outline", "shadow", "fill", "font", "fontColor"], function(name){
				var markerName = "marker" + name.charAt(0).toUpperCase() + name.substr(1),
					b = markerName in mixin;
				if(name in mixin){
					dojo.setObject("series." + name, mixin[name], t);
					if(!b){
						dojo.setObject("marker." + name, mixin[name], t);
					}
				}
				if(b){
					dojo.setObject("marker." + name, mixin[markerName], t);
				}
			});
			if("marker" in mixin){
				t.symbol = mixin.marker;
			}
			theme = dojox.lang.utils.merge(theme, t);
		}
		if(doPost){
			theme = this.post(theme, elementType);
		}
		return theme;	//	dojox.charting.Theme
	},

	post: function(theme, elementType){
		//	summary:
		//		Process any post-shape fills.
		//	theme: dojox.charting.Theme
		//		The theme to post process with.
		//	elementType: String
		//		The type of element being filled.  Can be "bar" or "circle".
		//	returns: dojox.charting.Theme
		//		The post-processed theme.
		var fill = theme.series.fill, t;
		if(!this.noGradConv && this.shapeSpaces[fill.space] && fill.type == "linear"){
			if(elementType == "bar"){
				// transpose start and end points
				t = {
					x1: fill.y1,
					y1: fill.x1,
					x2: fill.y2,
					y2: fill.x2
				};
			}else if(!this.noRadialConv && fill.space == "shape" && (elementType == "slice" || elementType == "circle")){
				// switch to radial
				t = {
					type: "radial",
					cx: 0,
					cy: 0,
					r:  100
				};
			}
			if(t){
				return dojox.lang.utils.merge(theme, {series: {fill: t}});
			}
		}
		return theme;	//	dojox.charting.Theme
	},

	getTick: function(name, mixin){
		//	summary:
		//		Calculates and merges tick parameters.
		//	name: String
		//		Tick name, can be "major", "minor", or "micro".
		//	mixin: Object?
		//		Optional object to mix in to the tick.
		var tick = this.axis.tick, tickName = name + "Tick";
			merge = dojox.lang.utils.merge;
		if(tick){
			if(this.axis[tickName]){
				tick = merge(tick, this.axis[tickName]);
			}
		}else{
			tick = this.axis[tickName];
		}
		if(mixin){
			if(tick){
				if(mixin[tickName]){
					tick = merge(tick, mixin[tickName]);
				}
			}else{
				tick = mixin[tickName];
			}
		}
		return tick;	//	Object
	},

	inspectObjects: function(f){
		dojo.forEach(["chart", "plotarea", "axis", "series", "marker"], function(name){
			f(this[name]);
		}, this);
		if(this.seriesThemes){
			dojo.forEach(this.seriesThemes, f);
		}
		if(this.markerThemes){
			dojo.forEach(this.markerThemes, f);
		}
	},

	reverseFills: function(){
		this.inspectObjects(function(o){
			if(o && o.fill){
				o.fill = dojox.gfx.gradutils.reverse(o.fill);
			}
		});
	},

	addMarker:function(/*String*/ name, /*String*/ segment){
		//	summary:
		//		Add a custom marker to this theme.
		//	example:
		//	|	myTheme.addMarker("Ellipse", foo);
		this.markers[name] = segment;
		this._buildMarkerArray();
	},

	setMarkers:function(/*Object*/ obj){
		//	summary:
		//		Set all the markers of this theme at once.  obj should be a
		//		dictionary of keys and path segments.
		//
		//	example:
		//	|	myTheme.setMarkers({ "CIRCLE": foo });
		this.markers = obj;
		this._buildMarkerArray();
	},

	_buildMarkerArray: function(){
		this._markers = [];
		for(var p in this.markers){
			this._markers.push(this.markers[p]);
		}
	}
});

/*=====
dojox.charting.Theme.__DefineColorArgs = function(num, colors, hue, saturation, low, high, base, generator){
	//	summary:
	//		The arguments object that can be passed to define colors for a theme.
	//	num: Number?
	//		The number of colors to generate.  Defaults to 5.
	//	colors: String[]|dojo.Color[]?
	//		A pre-defined set of colors; this is passed through to the Theme directly.
	//	hue: Number?
	//		A hue to base the generated colors from (a number from 0 - 359).
	//	saturation: Number?
	//		If a hue is passed, this is used for the saturation value (0 - 100).
	//	low: Number?
	//		An optional value to determine the lowest value used to generate a color (HSV model)
	//	high: Number?
	//		An optional value to determine the highest value used to generate a color (HSV model)
	//	base: String|dojo.Color?
	//		A base color to use if we are defining colors using dojox.color.Palette
	//	generator: String?
	//		The generator function name from dojox.color.Palette.
	this.num = num;
	this.colors = colors;
	this.hue = hue;
	this.saturation = saturation;
	this.low = low;
	this.high = high;
	this.base = base;
	this.generator = generator;
}
=====*/
dojo.mixin(dojox.charting.Theme, {
	defaultMarkers: {
		CIRCLE:   "m-3,0 c0,-4 6,-4 6,0 m-6,0 c0,4 6,4 6,0",
		SQUARE:   "m-3,-3 l0,6 6,0 0,-6 z",
		DIAMOND:  "m0,-3 l3,3 -3,3 -3,-3 z",
		CROSS:    "m0,-3 l0,6 m-3,-3 l6,0",
		X:        "m-3,-3 l6,6 m0,-6 l-6,6",
		TRIANGLE: "m-3,3 l3,-6 3,6 z",
		TRIANGLE_INVERTED: "m-3,-3 l3,6 3,-6 z"
	},

	defaultColors:[
		// gray skies
		"#54544c", "#858e94", "#6e767a", "#948585", "#474747"
	],

	defaultTheme: {
		// all objects are structs used directly in dojox.gfx
		chart:{
			stroke: null,
			fill: "white",
			pageStyle: null
		},
		plotarea:{
			stroke: null,
			fill: "white"
		},
		// TODO: label rotation on axis
		axis:{
			stroke:	{ // the axis itself
				color: "#333",
				width: 1
			},
			tick: {	// used as a foundation for all ticks
				color:     "#666",
				position:  "center",
				font:      "normal normal normal 7pt Tahoma",	// labels on axis
				fontColor: "#333"								// color of labels
			},
			majorTick:	{ // major ticks on axis, and used for major gridlines
				width:  1,
				length: 6
			},
			minorTick:	{ // minor ticks on axis, and used for minor gridlines
				width:  0.8,
				length: 3
			},
			microTick:	{ // minor ticks on axis, and used for minor gridlines
				width:  0.5,
				length: 1
			}
		},
		series: {
			// used as a "main" theme for series, sThemes augment it
			stroke:  {width: 1.5, color: "#333"},		// line
			outline: {width: 0.1, color: "#ccc"},		// outline
			//shadow:  {dx: 1, dy: 1, width: 2, color: [0, 0, 0, 0.3]},
			shadow: null,								// no shadow
			fill:    "#ccc",							// fill, if appropriate
			font:    "normal normal normal 8pt Tahoma",	// if there's a label
			fontColor: "#000"							// color of labels
		},
		marker: {	// any markers on a series
			stroke:  {width: 1.5, color: "#333"},		// stroke
			outline: {width: 0.1, color: "#ccc"},		// outline
			//shadow:  {dx: 1, dy: 1, width: 2, color: [0, 0, 0, 0.3]},
			shadow: null,								// no shadow
			fill:    "#ccc",							// fill if needed
			font:    "normal normal normal 8pt Tahoma",	// label
			fontColor: "#000"
		}
	},

	defineColors: function(kwArgs){
		//	summary:
		//		Generate a set of colors for the theme based on keyword
		//		arguments.
		//	kwArgs: dojox.charting.Theme.__DefineColorArgs
		//		The arguments object used to define colors.
		//	returns: dojo.Color[]
		//		An array of colors for use in a theme.
		//
		//	example:
		//	|	var colors = dojox.charting.Theme.defineColors({
		//	|		base: "#369",
		//	|		generator: "compound"
		//	|	});
		//
		//	example:
		//	|	var colors = dojox.charting.Theme.defineColors({
		//	|		hue: 60,
		//	|		saturation: 90,
		//	|		low: 30,
		//	|		high: 80
		//	|	});
		kwArgs = kwArgs || {};
		var c = [], n = kwArgs.num || 5;	// the number of colors to generate
		if(kwArgs.colors){
			// we have an array of colors predefined, so fix for the number of series.
			var l = kwArgs.colors.length;
			for(var i = 0; i < n; i++){
				c.push(kwArgs.colors[i % l]);
			}
			return c;	//	dojo.Color[]
		}
		if(kwArgs.hue){
			// single hue, generate a set based on brightness
			var s = kwArgs.saturation || 100;	// saturation
			var st = kwArgs.low || 30;
			var end = kwArgs.high || 90;
			// we'd like it to be a little on the darker side.
			var l = (end + st) / 2;
			// alternately, use "shades"
			return dojox.color.Palette.generate(
				dojox.color.fromHsv(kwArgs.hue, s, l), "monochromatic"
			).colors;
		}
		if(kwArgs.generator){
			//	pass a base color and the name of a generator
			return dojox.color.Palette.generate(kwArgs.base, kwArgs.generator).colors;
		}
		return c;	//	dojo.Color[]
	},
	
	generateGradient: function(fillPattern, colorFrom, colorTo){
		var fill = dojo.delegate(fillPattern);
		fill.colors = [
			{offset: 0, color: colorFrom},
			{offset: 1, color: colorTo}
		];
		return fill;
	},
	
	generateHslColor: function(color, luminance){
		color = new dojox.color.Color(color);
		var hsl    = color.toHsl(),
			result = dojox.color.fromHsl(hsl.h, hsl.s, luminance);
		result.a = color.a;	// add missing opacity
		return result;
	},

	generateHslGradient: function(color, fillPattern, lumFrom, lumTo){
		color = new dojox.color.Color(color);
		var hsl       = color.toHsl(),
			colorFrom = dojox.color.fromHsl(hsl.h, hsl.s, lumFrom),
			colorTo   = dojox.color.fromHsl(hsl.h, hsl.s, lumTo);
		colorFrom.a = colorTo.a = color.a;	// add missing opacity
		return dojox.charting.Theme.generateGradient(fillPattern, colorFrom, colorTo);	// Object
	}
});

}

if(!dojo._hasResource["dojox.charting.Element"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.Element"] = true;
dojo.provide("dojox.charting.Element");



dojo.declare("dojox.charting.Element", null, {
	//	summary:
	//		A base class that is used to build other elements of a chart, such as
	//		a series.
	//	chart: dojox.charting.Chart2D
	//		The parent chart for this element.
	//	group: dojox.gfx.Group
	//		The visual GFX group representing this element.
	//	htmlElement: Array
	//		Any DOMNodes used as a part of this element (such as HTML-based labels).
	//	dirty: Boolean
	//		A flag indicating whether or not this element needs to be rendered.

	chart: null,
	group: null,
	htmlElements: null,
	dirty: true,

	constructor: function(chart){
		//	summary:
		//		Creates a new charting element.
		//	chart: dojox.charting.Chart2D
		//		The chart that this element belongs to.
		this.chart = chart;
		this.group = null;
		this.htmlElements = [];
		this.dirty = true;
	},
	createGroup: function(creator){
		//	summary:
		//		Convenience function to create a new dojox.gfx.Group.
		//	creator: dojox.gfx.Surface?
		//		An optional surface in which to create this group.
		//	returns: dojox.charting.Element
		//		A reference to this object for functional chaining.
		if(!creator){ creator = this.chart.surface; }
		if(!this.group){
			this.group = creator.createGroup();
		}
		return this;	//	dojox.charting.Element
	},
	purgeGroup: function(){
		//	summary:
		//		Clear any elements out of our group, and destroy the group.
		//	returns: dojox.charting.Element
		//		A reference to this object for functional chaining.
		this.destroyHtmlElements();
		if(this.group){
			this.group.clear();
			this.group.removeShape();
			this.group = null;
		}
		this.dirty = true;
		return this;	//	dojox.charting.Element
	},
	cleanGroup: function(creator){
		//	summary:
		//		Clean any elements (HTML or GFX-based) out of our group, and create a new one.
		//	creator: dojox.gfx.Surface?
		//		An optional surface to work with.
		//	returns: dojox.charting.Element
		//		A reference to this object for functional chaining.
		this.destroyHtmlElements();
		if(!creator){ creator = this.chart.surface; }
		if(this.group){
			this.group.clear();
		}else{
			this.group = creator.createGroup();
		}
		this.dirty = true;
		return this;	//	dojox.charting.Element
	},
	destroyHtmlElements: function(){
		//	summary:
		//		Destroy any DOMNodes that may have been created as a part of this element.
		if(this.htmlElements.length){
			dojo.forEach(this.htmlElements, dojo.destroy);
			this.htmlElements = [];
		}
	},
	destroy: function(){
		//	summary:
		//		API addition to conform to the rest of the Dojo Toolkit's standard.
		this.purgeGroup();
	},
	// fill utilities
	_plotFill: function(fill, dim, offsets){
		// process a plot-wide fill
		if(!fill || !fill.type || !fill.space){
			return fill;
		}
		var space = fill.space;
		switch(fill.type){
			case "linear":
				if(space === "plot" || space === "shapeX" || space === "shapeY"){
					// clone a fill so we can modify properly directly
					fill = dojox.gfx.makeParameters(dojox.gfx.defaultLinearGradient, fill);
					fill.space = space;
					// process dimensions
					if(space === "plot" || space === "shapeX"){
						// process Y
						var span = dim.height - offsets.t - offsets.b;
						fill.y1 = offsets.t + span * fill.y1 / 100;
						fill.y2 = offsets.t + span * fill.y2 / 100;
					}
					if(space === "plot" || space === "shapeY"){
						// process X
						var span = dim.width - offsets.l - offsets.r;
						fill.x1 = offsets.l + span * fill.x1 / 100;
						fill.x2 = offsets.l + span * fill.x2 / 100;
					}
				}
				break;
			case "radial":
				if(space === "plot"){
					// this one is used exclusively for scatter charts
					// clone a fill so we can modify properly directly
					fill = dojox.gfx.makeParameters(dojox.gfx.defaultRadialGradient, fill);
					fill.space = space;
					// process both dimensions
					var spanX = dim.width  - offsets.l - offsets.r,
						spanY = dim.height - offsets.t - offsets.b;
					fill.cx = offsets.l + spanX * fill.cx / 100;
					fill.cy = offsets.t + spanY * fill.cy / 100;
					fill.r  = fill.r * Math.sqrt(spanX * spanX + spanY * spanY) / 200;
				}
				break;
			case "pattern":
				if(space === "plot" || space === "shapeX" || space === "shapeY"){
					// clone a fill so we can modify properly directly
					fill = dojox.gfx.makeParameters(dojox.gfx.defaultPattern, fill);
					fill.space = space;
					// process dimensions
					if(space === "plot" || space === "shapeX"){
						// process Y
						var span = dim.height - offsets.t - offsets.b;
						fill.y = offsets.t + span * fill.y / 100;
						fill.height = span * fill.height / 100;
					}
					if(space === "plot" || space === "shapeY"){
						// process X
						var span = dim.width - offsets.l - offsets.r;
						fill.x = offsets.l + span * fill.x / 100;
						fill.width = span * fill.width / 100;
					}
				}
				break;
		}
		return fill;
	},
	_shapeFill: function(fill, bbox){
		// process shape-specific fill
		if(!fill || !fill.space){
			return fill;
		}
		var space = fill.space;
		switch(fill.type){
			case "linear":
				if(space === "shape" || space === "shapeX" || space === "shapeY"){
					// clone a fill so we can modify properly directly
					fill = dojox.gfx.makeParameters(dojox.gfx.defaultLinearGradient, fill);
					fill.space = space;
					// process dimensions
					if(space === "shape" || space === "shapeX"){
						// process X
						var span = bbox.width;
						fill.x1 = bbox.x + span * fill.x1 / 100;
						fill.x2 = bbox.x + span * fill.x2 / 100;
					}
					if(space === "shape" || space === "shapeY"){
						// process Y
						var span = bbox.height;
						fill.y1 = bbox.y + span * fill.y1 / 100;
						fill.y2 = bbox.y + span * fill.y2 / 100;
					}
				}
				break;
			case "radial":
				if(space === "shape"){
					// this one is used exclusively for bubble charts and pie charts
					// clone a fill so we can modify properly directly
					fill = dojox.gfx.makeParameters(dojox.gfx.defaultRadialGradient, fill);
					fill.space = space;
					// process both dimensions
					fill.cx = bbox.x + bbox.width  / 2;
					fill.cy = bbox.y + bbox.height / 2;
					fill.r  = fill.r * bbox.width  / 200;
				}
				break;
			case "pattern":
				if(space === "shape" || space === "shapeX" || space === "shapeY"){
					// clone a fill so we can modify properly directly
					fill = dojox.gfx.makeParameters(dojox.gfx.defaultPattern, fill);
					fill.space = space;
					// process dimensions
					if(space === "shape" || space === "shapeX"){
						// process X
						var span = bbox.width;
						fill.x = bbox.x + span * fill.x / 100;
						fill.width = span * fill.width / 100;
					}
					if(space === "shape" || space === "shapeY"){
						// process Y
						var span = bbox.height;
						fill.y = bbox.y + span * fill.y / 100;
						fill.height = span * fill.height / 100;
					}
				}
				break;
		}
		return fill;
	},
	_pseudoRadialFill: function(fill, center, radius, start, end){
		// process pseudo-radial fills
		if(!fill || fill.type !== "radial" || fill.space !== "shape"){
			return fill;
		}
		// clone and normalize fill
		var space = fill.space;
		fill = dojox.gfx.makeParameters(dojox.gfx.defaultRadialGradient, fill);
		fill.space = space;
		if(arguments.length < 4){
			// process both dimensions
			fill.cx = center.x;
			fill.cy = center.y;
			fill.r  = fill.r * radius / 100;
			return fill;
		}
		// convert to a linear gradient
		var angle = arguments.length < 5 ? start : (end + start) / 2;
		return {
			type: "linear",
			x1: center.x,
			y1: center.y,
			x2: center.x + fill.r * radius * Math.cos(angle) / 100,
			y2: center.y + fill.r * radius * Math.sin(angle) / 100,
			colors: fill.colors
		};
		return fill;
	}
});

}

if(!dojo._hasResource["dojox.charting.Series"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.Series"] = true;
dojo.provide("dojox.charting.Series");


/*=====
dojox.charting.__SeriesCtorArgs = function(plot){
	//	summary:
	//		An optional arguments object that can be used in the Series constructor.
	//	plot: String?
	//		The plot (by name) that this series belongs to.
	this.plot = plot;
} 
=====*/
dojo.declare("dojox.charting.Series", dojox.charting.Element, {
	//	summary:
	//		An object representing a series of data for plotting on a chart.
	constructor: function(chart, data, kwArgs){
		//	summary:
		//		Create a new data series object for use within charting.
		//	chart: dojox.charting.Chart2D
		//		The chart that this series belongs to.
		//	data: Array|Object:
		//		The array of data points (either numbers or objects) that
		//		represents the data to be drawn. Or it can be an object. In
		//		the latter case, it should have a property "data" (an array),
		//		destroy(), and setSeriesObject().
		//	kwArgs: dojox.charting.__SeriesCtorArgs?
		//		An optional keyword arguments object to set details for this series.
		dojo.mixin(this, kwArgs);
		if(typeof this.plot != "string"){ this.plot = "default"; }
		this.update(data);
	},

	clear: function(){
		//	summary:
		//		Clear the calculated additional parameters set on this series.
		this.dyn = {};
	},
	
	update: function(data){
		//	summary:
		//		Set data and make this object dirty, so it can be redrawn.
		//	data: Array|Object:
		//		The array of data points (either numbers or objects) that
		//		represents the data to be drawn. Or it can be an object. In
		//		the latter case, it should have a property "data" (an array),
		//		destroy(), and setSeriesObject().
		if(dojo.isArray(data)){
			this.data = data;
		}else{
			this.source = data;
			this.data = this.source.data;
			if(this.source.setSeriesObject){
				this.source.setSeriesObject(this);
			}
		}
		this.dirty = true;
		this.clear();
	}
});

}

if(!dojo._hasResource["dojox.charting.scaler.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.scaler.common"] = true;
dojo.provide("dojox.charting.scaler.common");

(function(){
	var eq = function(/*Number*/ a, /*Number*/ b){
		// summary: compare two FP numbers for equality
		return Math.abs(a - b) <= 1e-6 * (Math.abs(a) + Math.abs(b));	// Boolean
	};
	
	dojo.mixin(dojox.charting.scaler.common, {
		findString: function(/*String*/ val, /*Array*/ text){
			val = val.toLowerCase();
			for(var i = 0; i < text.length; ++i){
				if(val == text[i]){ return true; }
			}
			return false;
		},
		getNumericLabel: function(/*Number*/ number, /*Number*/ precision, /*Object*/ kwArgs){
			var def = kwArgs.fixed ? 
						number.toFixed(precision < 0 ? -precision : 0) : 
						number.toString();
			if(kwArgs.labelFunc){
				var r = kwArgs.labelFunc(def, number, precision);
				if(r){ return r; }
				// else fall through to the regular labels search
			}
			if(kwArgs.labels){
				// classic binary search
				var l = kwArgs.labels, lo = 0, hi = l.length;
				while(lo < hi){
					var mid = Math.floor((lo + hi) / 2), val = l[mid].value;
					if(val < number){
						lo = mid + 1;
					}else{
						hi = mid;
					}
				}
				// lets take into account FP errors
				if(lo < l.length && eq(l[lo].value, number)){
					return l[lo].text;
				}
				--lo;
				if(lo >= 0 && lo < l.length && eq(l[lo].value, number)){
					return l[lo].text;
				}
				lo += 2;
				if(lo < l.length && eq(l[lo].value, number)){
					return l[lo].text;
				}
				// otherwise we will produce a number
			}
			return def;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.scaler.linear"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.scaler.linear"] = true;
dojo.provide("dojox.charting.scaler.linear");


(function(){
	var deltaLimit = 3,	// pixels
		dc = dojox.charting, dcs = dc.scaler, dcsc = dcs.common,
		findString = dcsc.findString,
		getLabel = dcsc.getNumericLabel;
	
	var calcTicks = function(min, max, kwArgs, majorTick, minorTick, microTick, span){
		kwArgs = dojo.delegate(kwArgs);
		if(!majorTick){
			if(kwArgs.fixUpper == "major"){ kwArgs.fixUpper = "minor"; }
			if(kwArgs.fixLower == "major"){ kwArgs.fixLower = "minor"; }
		}
		if(!minorTick){
			if(kwArgs.fixUpper == "minor"){ kwArgs.fixUpper = "micro"; }
			if(kwArgs.fixLower == "minor"){ kwArgs.fixLower = "micro"; }
		}
		if(!microTick){
			if(kwArgs.fixUpper == "micro"){ kwArgs.fixUpper = "none"; }
			if(kwArgs.fixLower == "micro"){ kwArgs.fixLower = "none"; }
		}
		var lowerBound = findString(kwArgs.fixLower, ["major"]) ?
				Math.floor(kwArgs.min / majorTick) * majorTick :
					findString(kwArgs.fixLower, ["minor"]) ?
						Math.floor(kwArgs.min / minorTick) * minorTick :
							findString(kwArgs.fixLower, ["micro"]) ?
								Math.floor(kwArgs.min / microTick) * microTick : kwArgs.min,
			upperBound = findString(kwArgs.fixUpper, ["major"]) ?
				Math.ceil(kwArgs.max / majorTick) * majorTick :
					findString(kwArgs.fixUpper, ["minor"]) ?
						Math.ceil(kwArgs.max / minorTick) * minorTick :
							findString(kwArgs.fixUpper, ["micro"]) ?
								Math.ceil(kwArgs.max / microTick) * microTick : kwArgs.max;
								
		if(kwArgs.useMin){ min = lowerBound; }
		if(kwArgs.useMax){ max = upperBound; }
		
		var majorStart = (!majorTick || kwArgs.useMin && findString(kwArgs.fixLower, ["major"])) ?
				min : Math.ceil(min / majorTick) * majorTick,
			minorStart = (!minorTick || kwArgs.useMin && findString(kwArgs.fixLower, ["major", "minor"])) ?
				min : Math.ceil(min / minorTick) * minorTick,
			microStart = (! microTick || kwArgs.useMin && findString(kwArgs.fixLower, ["major", "minor", "micro"])) ?
				min : Math.ceil(min / microTick) * microTick,
			majorCount = !majorTick ? 0 : (kwArgs.useMax && findString(kwArgs.fixUpper, ["major"]) ?
				Math.round((max - majorStart) / majorTick) :
				Math.floor((max - majorStart) / majorTick)) + 1,
			minorCount = !minorTick ? 0 : (kwArgs.useMax && findString(kwArgs.fixUpper, ["major", "minor"]) ?
				Math.round((max - minorStart) / minorTick) :
				Math.floor((max - minorStart) / minorTick)) + 1,
			microCount = !microTick ? 0 : (kwArgs.useMax && findString(kwArgs.fixUpper, ["major", "minor", "micro"]) ?
				Math.round((max - microStart) / microTick) :
				Math.floor((max - microStart) / microTick)) + 1,
			minorPerMajor  = minorTick ? Math.round(majorTick / minorTick) : 0,
			microPerMinor  = microTick ? Math.round(minorTick / microTick) : 0,
			majorPrecision = majorTick ? Math.floor(Math.log(majorTick) / Math.LN10) : 0,
			minorPrecision = minorTick ? Math.floor(Math.log(minorTick) / Math.LN10) : 0,
			scale = span / (max - min);	
		if(!isFinite(scale)){ scale = 1; }
		
		return {
			bounds: {
				lower:	lowerBound,
				upper:	upperBound,
				from:	min,
				to:		max,
				scale:	scale,
				span:	span
			},
			major: {
				tick:	majorTick,
				start:	majorStart,
				count:	majorCount,
				prec:	majorPrecision
			},
			minor: {
				tick:	minorTick,
				start:	minorStart,
				count:	minorCount,
				prec:	minorPrecision
			},
			micro: {
				tick:	microTick,
				start:	microStart,
				count:	microCount,
				prec:	0
			},
			minorPerMajor:	minorPerMajor,
			microPerMinor:	microPerMinor,
			scaler:			dcs.linear
		};
	};
	
	dojo.mixin(dojox.charting.scaler.linear, {
		buildScaler: function(/*Number*/ min, /*Number*/ max, /*Number*/ span, /*Object*/ kwArgs){
			var h = {fixUpper: "none", fixLower: "none", natural: false};
			if(kwArgs){
				if("fixUpper" in kwArgs){ h.fixUpper = String(kwArgs.fixUpper); }
				if("fixLower" in kwArgs){ h.fixLower = String(kwArgs.fixLower); }
				if("natural"  in kwArgs){ h.natural  = Boolean(kwArgs.natural); }
			}
			
			// update bounds
			if("min" in kwArgs){ min = kwArgs.min; }
			if("max" in kwArgs){ max = kwArgs.max; }
			if(kwArgs.includeZero){
				if(min > 0){ min = 0; }
				if(max < 0){ max = 0; }
			}
			h.min = min;
			h.useMin = true;
			h.max = max;
			h.useMax = true;
			
			if("from" in kwArgs){
				min = kwArgs.from;
				h.useMin = false;
			}
			if("to" in kwArgs){
				max = kwArgs.to;
				h.useMax = false;
			}
			
			// check for erroneous condition
			if(max <= min){
				return calcTicks(min, max, h, 0, 0, 0, span);	// Object
			}
			
			var mag = Math.floor(Math.log(max - min) / Math.LN10),
				major = kwArgs && ("majorTickStep" in kwArgs) ? kwArgs.majorTickStep : Math.pow(10, mag), 
				minor = 0, micro = 0, ticks;
				
			// calculate minor ticks
			if(kwArgs && ("minorTickStep" in kwArgs)){
				minor = kwArgs.minorTickStep;
			}else{
				do{
					minor = major / 10;
					if(!h.natural || minor > 0.9){
						ticks = calcTicks(min, max, h, major, minor, 0, span);
						if(ticks.bounds.scale * ticks.minor.tick > deltaLimit){ break; }
					}
					minor = major / 5;
					if(!h.natural || minor > 0.9){
						ticks = calcTicks(min, max, h, major, minor, 0, span);
						if(ticks.bounds.scale * ticks.minor.tick > deltaLimit){ break; }
					}
					minor = major / 2;
					if(!h.natural || minor > 0.9){
						ticks = calcTicks(min, max, h, major, minor, 0, span);
						if(ticks.bounds.scale * ticks.minor.tick > deltaLimit){ break; }
					}
					return calcTicks(min, max, h, major, 0, 0, span);	// Object
				}while(false);
			}
	
			// calculate micro ticks
			if(kwArgs && ("microTickStep" in kwArgs)){
				micro = kwArgs.microTickStep;
				ticks = calcTicks(min, max, h, major, minor, micro, span);
			}else{
				do{
					micro = minor / 10;
					if(!h.natural || micro > 0.9){
						ticks = calcTicks(min, max, h, major, minor, micro, span);
						if(ticks.bounds.scale * ticks.micro.tick > deltaLimit){ break; }
					}
					micro = minor / 5;
					if(!h.natural || micro > 0.9){
						ticks = calcTicks(min, max, h, major, minor, micro, span);
						if(ticks.bounds.scale * ticks.micro.tick > deltaLimit){ break; }
					}
					micro = minor / 2;
					if(!h.natural || micro > 0.9){
						ticks = calcTicks(min, max, h, major, minor, micro, span);
						if(ticks.bounds.scale * ticks.micro.tick > deltaLimit){ break; }
					}
					micro = 0;
				}while(false);
			}
	
			return micro ? ticks : calcTicks(min, max, h, major, minor, 0, span);	// Object
		},
		buildTicks: function(/*Object*/ scaler, /*Object*/ kwArgs){
			var step, next, tick,
				nextMajor = scaler.major.start, 
				nextMinor = scaler.minor.start, 
				nextMicro = scaler.micro.start;
			if(kwArgs.microTicks && scaler.micro.tick){
				step = scaler.micro.tick, next = nextMicro;
			}else if(kwArgs.minorTicks && scaler.minor.tick){
				step = scaler.minor.tick, next = nextMinor;
			}else if(scaler.major.tick){
				step = scaler.major.tick, next = nextMajor;
			}else{
				// no ticks
				return null;
			}
			// make sure that we have finite bounds
			var revScale = 1 / scaler.bounds.scale;
			if(scaler.bounds.to <= scaler.bounds.from || isNaN(revScale) || !isFinite(revScale) ||
					step <= 0 || isNaN(step) || !isFinite(step)){
				// no ticks
				return null;
			}
			// loop over all ticks
			var majorTicks = [], minorTicks = [], microTicks = [];
			while(next <= scaler.bounds.to + revScale){
				if(Math.abs(nextMajor - next) < step / 2){
					// major tick
					tick = {value: nextMajor};
					if(kwArgs.majorLabels){
						tick.label = getLabel(nextMajor, scaler.major.prec, kwArgs);
					}
					majorTicks.push(tick);
					nextMajor += scaler.major.tick;
					nextMinor += scaler.minor.tick;
					nextMicro += scaler.micro.tick;
				}else if(Math.abs(nextMinor - next) < step / 2){
					// minor tick
					if(kwArgs.minorTicks){
						tick = {value: nextMinor};
						if(kwArgs.minorLabels && (scaler.minMinorStep <= scaler.minor.tick * scaler.bounds.scale)){
							tick.label = getLabel(nextMinor, scaler.minor.prec, kwArgs);
						}
						minorTicks.push(tick);
					}
					nextMinor += scaler.minor.tick;
					nextMicro += scaler.micro.tick;
				}else{
					// micro tick
					if(kwArgs.microTicks){
						microTicks.push({value: nextMicro});
					}
					nextMicro += scaler.micro.tick;
				}
				next += step;
			}
			return {major: majorTicks, minor: minorTicks, micro: microTicks};	// Object
		},
		getTransformerFromModel: function(/*Object*/ scaler){
			var offset = scaler.bounds.from, scale = scaler.bounds.scale;
			return function(x){ return (x - offset) * scale; };	// Function
		},
		getTransformerFromPlot: function(/*Object*/ scaler){
			var offset = scaler.bounds.from, scale = scaler.bounds.scale;
			return function(x){ return x / scale + offset; };	// Function
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.axis2d.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.axis2d.common"] = true;
dojo.provide("dojox.charting.axis2d.common");



(function(){
	var g = dojox.gfx;

	var clearNode = function(s){
		s.marginLeft   = "0px";
		s.marginTop    = "0px";
		s.marginRight  = "0px";
		s.marginBottom = "0px";
		s.paddingLeft   = "0px";
		s.paddingTop    = "0px";
		s.paddingRight  = "0px";
		s.paddingBottom = "0px";
		s.borderLeftWidth   = "0px";
		s.borderTopWidth    = "0px";
		s.borderRightWidth  = "0px";
		s.borderBottomWidth = "0px";
	};

	var getBoxWidth = function(n){
		// marginBox is incredibly slow, so avoid it if we can
		if(n["getBoundingClientRect"]){
			var bcr = n.getBoundingClientRect();
			return bcr.width || (bcr.right - bcr.left);
		}else{
			return dojo.marginBox(n).w;
		}
	};

	dojo.mixin(dojox.charting.axis2d.common, {
		//	summary:
		//		Common methods to be used by any axis.  This is considered "static".
		createText: {
			gfx: function(chart, creator, x, y, align, text, font, fontColor){
				//	summary:
				//		Use dojox.gfx to create any text.
				//	chart: dojox.charting.Chart2D
				//		The chart to create the text into.
				//	creator: dojox.gfx.Surface
				//		The graphics surface to use for creating the text.
				//	x: Number
				//		Where to create the text along the x axis (CSS left).
				//	y: Number
				//		Where to create the text along the y axis (CSS top).
				//	align: String
				//		How to align the text.  Can be "left", "right", "center".
				//	text: String
				//		The text to render.
				//	font: String
				//		The font definition, a la CSS "font".
				//	fontColor: String|dojo.Color
				//		The color of the resultant text.
				//	returns: dojox.gfx.Text
				//		The resultant GFX object.
				return creator.createText({
					x: x, y: y, text: text, align: align
				}).setFont(font).setFill(fontColor);	//	dojox.gfx.Text
			},
			html: function(chart, creator, x, y, align, text, font, fontColor, labelWidth){
				//	summary:
				//		Use the HTML DOM to create any text.
				//	chart: dojox.charting.Chart2D
				//		The chart to create the text into.
				//	creator: dojox.gfx.Surface
				//		The graphics surface to use for creating the text.
				//	x: Number
				//		Where to create the text along the x axis (CSS left).
				//	y: Number
				//		Where to create the text along the y axis (CSS top).
				//	align: String
				//		How to align the text.  Can be "left", "right", "center".
				//	text: String
				//		The text to render.
				//	font: String
				//		The font definition, a la CSS "font".
				//	fontColor: String|dojo.Color
				//		The color of the resultant text.
				//	labelWidth: Number?
				//		The maximum width of the resultant DOM node.
				//	returns: DOMNode
				//		The resultant DOMNode (a "div" element).

				// setup the text node
				var p = dojo.doc.createElement("div"), s = p.style, boxWidth;
				clearNode(s);
				s.font = font;
				p.innerHTML = String(text).replace(/\s/g, "&nbsp;");
				s.color = fontColor;
				// measure the size
				s.position = "absolute";
				s.left = "-10000px";
				dojo.body().appendChild(p);
				var size = g.normalizedLength(g.splitFontString(font).size);

				// do we need to calculate the label width?
				if(!labelWidth){
					boxWidth = getBoxWidth(p);
				}

				// new settings for the text node
				dojo.body().removeChild(p);

				s.position = "relative";
				if(labelWidth){
					s.width = labelWidth + "px";
					// s.border = "1px dotted grey";
					switch(align){
						case "middle":
							s.textAlign = "center";
							s.left = (x - labelWidth / 2) + "px";
							break;
						case "end":
							s.textAlign = "right";
							s.left = (x - labelWidth) + "px";
							break;
						default:
							s.left = x + "px";
							s.textAlign = "left";
							break;
					}
				}else{
					switch(align){
						case "middle":
							s.left = Math.floor(x - boxWidth / 2) + "px";
							// s.left = Math.floor(x - p.offsetWidth / 2) + "px";
							break;
						case "end":
							s.left = Math.floor(x - boxWidth) + "px";
							// s.left = Math.floor(x - p.offsetWidth) + "px";
							break;
						//case "start":
						default:
							s.left = Math.floor(x) + "px";
							break;
					}
				}
				s.top = Math.floor(y - size) + "px";
				s.whiteSpace = "nowrap";	// hack for WebKit
				// setup the wrapper node
				var wrap = dojo.doc.createElement("div"), w = wrap.style;
				clearNode(w);
				w.width = "0px";
				w.height = "0px";
				// insert nodes
				wrap.appendChild(p)
				chart.node.insertBefore(wrap, chart.node.firstChild);
				return wrap;	//	DOMNode
			}
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.axis2d.Base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.axis2d.Base"] = true;
dojo.provide("dojox.charting.axis2d.Base");



dojo.declare("dojox.charting.axis2d.Base", dojox.charting.Element, {
	//	summary:
	//		The base class for any axis.  This is more of an interface/API
	//		definition than anything else; see dojox.charting.axis2d.Default
	//		for more details.
	constructor: function(chart, kwArgs){
		//	summary:
		//		Return a new base axis.
		//	chart: dojox.charting.Chart2D
		//		The chart this axis belongs to.
		//	kwArgs: dojox.charting.axis2d.__AxisCtorArgs?
		//		An optional arguments object to define the axis parameters.
		this.vertical = kwArgs && kwArgs.vertical;
	},
	clear: function(){
		//	summary:
		//		Stub function for clearing the axis.
		//	returns: dojox.charting.axis2d.Base
		//		A reference to the axis for functional chaining.
		return this;	//	dojox.charting.axis2d.Base
	},
	initialized: function(){
		//	summary:
		//		Return a flag as to whether or not this axis has been initialized.
		//	returns: Boolean
		//		If the axis is initialized or not.
		return false;	//	Boolean
	},
	calculate: function(min, max, span){
		//	summary:
		//		Stub function to run the calcuations needed for drawing this axis.
		//	returns: dojox.charting.axis2d.Base
		//		A reference to the axis for functional chaining.
		return this;	//	dojox.charting.axis2d.Base
	},
	getScaler: function(){
		//	summary:
		//		A stub function to return the scaler object created during calculate.
		//	returns: Object
		//		The scaler object (see dojox.charting.scaler.linear for more information)
		return null;	//	Object
	},
	getTicks: function(){
		//	summary:
		//		A stub function to return the object that helps define how ticks are rendered.
		//	returns: Object
		//		The ticks object.
		return null;	//	Object
	},
	getOffsets: function(){
		//	summary:
		//		A stub function to return any offsets needed for axis and series rendering.
		//	returns: Object
		//		An object of the form { l, r, t, b }.
		return {l: 0, r: 0, t: 0, b: 0};	//	Object
	},
	render: function(dim, offsets){
		//	summary:
		//		Stub function to render this axis.
		//	returns: dojox.charting.axis2d.Base
		//		A reference to the axis for functional chaining.
		this.dirty = false;
		return this;	//	dojox.charting.axis2d.Base
	}
});

}

if(!dojo._hasResource["dojox.charting.axis2d.Invisible"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.axis2d.Invisible"] = true;
dojo.provide("dojox.charting.axis2d.Invisible");










(function(){
	var dc = dojox.charting,
		df = dojox.lang.functional,
		du = dojox.lang.utils,
		g = dojox.gfx,
		lin = dc.scaler.linear,
		merge = du.merge,
		labelGap = 4,			// in pixels
		centerAnchorLimit = 45;	// in degrees

	dojo.declare("dojox.charting.axis2d.Invisible", dojox.charting.axis2d.Base, {
		//	summary:
		//		The default axis object used in dojox.charting.  See dojox.charting.Chart2D.addAxis for details.
		//
		//	defaultParams: Object
		//		The default parameters used to define any axis.
		//	optionalParams: Object
		//		Any optional parameters needed to define an axis.

		/*
		//	TODO: the documentation tools need these to be pre-defined in order to pick them up
		//	correctly, but the code here is partially predicated on whether or not the properties
		//	actually exist.  For now, we will leave these undocumented but in the code for later. -- TRT

		//	opt: Object
		//		The actual options used to define this axis, created at initialization.
		//	scalar: Object
		//		The calculated helper object to tell charts how to draw an axis and any data.
		//	ticks: Object
		//		The calculated tick object that helps a chart draw the scaling on an axis.
		//	dirty: Boolean
		//		The state of the axis (whether it needs to be redrawn or not)
		//	scale: Number
		//		The current scale of the axis.
		//	offset: Number
		//		The current offset of the axis.

		opt: null,
		scalar: null,
		ticks: null,
		dirty: true,
		scale: 1,
		offset: 0,
		*/
		defaultParams: {
			vertical:    false,		// true for vertical axis
			fixUpper:    "none",	// align the upper on ticks: "major", "minor", "micro", "none"
			fixLower:    "none",	// align the lower on ticks: "major", "minor", "micro", "none"
			natural:     false,		// all tick marks should be made on natural numbers
			leftBottom:  true,		// position of the axis, used with "vertical"
			includeZero: false,		// 0 should be included
			fixed:       true,		// all labels are fixed numbers
			majorLabels: true,		// draw major labels
			minorTicks:  true,		// draw minor ticks
			minorLabels: true,		// draw minor labels
			microTicks:  false,		// draw micro ticks
			rotation:    0			// label rotation angle in degrees
		},
		optionalParams: {
			min:			0,	// minimal value on this axis
			max:			1,	// maximal value on this axis
			from:			0,	// visible from this value
			to:				1,	// visible to this value
			majorTickStep:	4,	// major tick step
			minorTickStep:	2,	// minor tick step
			microTickStep:	1,	// micro tick step
			labels:			[],	// array of labels for major ticks
								// with corresponding numeric values
								// ordered by values
			labelFunc:		null, // function to compute label values
			maxLabelSize:	0	// size in px. For use with labelFunc

			// TODO: add support for minRange!
			// minRange:		1,	// smallest distance from min allowed on the axis
		},

		constructor: function(chart, kwArgs){
			//	summary:
			//		The constructor for an axis.
			//	chart: dojox.charting.Chart2D
			//		The chart the axis belongs to.
			//	kwArgs: dojox.charting.axis2d.__AxisCtorArgs?
			//		Any optional keyword arguments to be used to define this axis.
			this.opt = dojo.delegate(this.defaultParams, kwArgs);
			// du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
		},
		dependOnData: function(){
			//	summary:
			//		Find out whether or not the axis options depend on the data in the axis.
			return !("min" in this.opt) || !("max" in this.opt);	//	Boolean
		},
		clear: function(){
			//	summary:
			//		Clear out all calculated properties on this axis;
			//	returns: dojox.charting.axis2d.Default
			//		The reference to the axis for functional chaining.
			delete this.scaler;
			delete this.ticks;
			this.dirty = true;
			return this;	//	dojox.charting.axis2d.Default
		},
		initialized: function(){
			//	summary:
			//		Finds out if this axis has been initialized or not.
			//	returns: Boolean
			//		Whether a scaler has been calculated and if the axis is not dirty.
			return "scaler" in this && !(this.dirty && this.dependOnData());
		},
		setWindow: function(scale, offset){
			//	summary:
			//		Set the drawing "window" for the axis.
			//	scale: Number
			//		The new scale for the axis.
			//	offset: Number
			//		The new offset for the axis.
			//	returns: dojox.charting.axis2d.Default
			//		The reference to the axis for functional chaining.
			this.scale  = scale;
			this.offset = offset;
			return this.clear();	//	dojox.charting.axis2d.Default
		},
		getWindowScale: function(){
			//	summary:
			//		Get the current windowing scale of the axis.
			return "scale" in this ? this.scale : 1;	//	Number
		},
		getWindowOffset: function(){
			//	summary:
			//		Get the current windowing offset for the axis.
			return "offset" in this ? this.offset : 0;	//	Number
		},
		_groupLabelWidth: function(labels, font){
			if(!labels.length){
				return 0;
			}
			if(dojo.isObject(labels[0])){
				labels = df.map(labels, function(label){ return label.text; });
			}
			var s = labels.join("<br>");
			return dojox.gfx._base._getTextBox(s, {font: font}).w || 0;
		},
		calculate: function(min, max, span, labels){
			//	summary:
			//		Perform all calculations needed to render this axis.
			//	min: Number
			//		The smallest value represented on this axis.
			//	max: Number
			//		The largest value represented on this axis.
			//	span: Number
			//		The span in pixels over which axis calculations are made.
			//	labels: String[]
			//		Optional list of labels.
			//	returns: dojox.charting.axis2d.Default
			//		The reference to the axis for functional chaining.
			if(this.initialized()){
				return this;
			}
			var o = this.opt;
			this.labels = "labels" in o  ? o.labels : labels;
			this.scaler = lin.buildScaler(min, max, span, o);
			var tsb = this.scaler.bounds;
			if("scale" in this){
				// calculate new range
				o.from = tsb.lower + this.offset;
				o.to   = (tsb.upper - tsb.lower) / this.scale + o.from;
				// make sure that bounds are correct
				if( !isFinite(o.from) ||
					isNaN(o.from) ||
					!isFinite(o.to) ||
					isNaN(o.to) ||
					o.to - o.from >= tsb.upper - tsb.lower
				){
					// any error --- remove from/to bounds
					delete o.from;
					delete o.to;
					delete this.scale;
					delete this.offset;
				}else{
					// shift the window, if we are out of bounds
					if(o.from < tsb.lower){
						o.to += tsb.lower - o.from;
						o.from = tsb.lower;
					}else if(o.to > tsb.upper){
						o.from += tsb.upper - o.to;
						o.to = tsb.upper;
					}
					// update the offset
					this.offset = o.from - tsb.lower;
				}
				// re-calculate the scaler
				this.scaler = lin.buildScaler(min, max, span, o);
				tsb = this.scaler.bounds;
				// cleanup
				if(this.scale == 1 && this.offset == 0){
					delete this.scale;
					delete this.offset;
				}
			}

			var ta = this.chart.theme.axis, labelWidth = 0, rotation = o.rotation % 360,
				// TODO: we use one font --- of major tick, we need to use major and minor fonts
				taFont = o.font || (ta.majorTick && ta.majorTick.font) || (ta.tick && ta.tick.font),
				size = taFont ? g.normalizedLength(g.splitFontString(taFont).size) : 0,
				cosr = Math.abs(Math.cos(rotation * Math.PI / 180)),
				sinr = Math.abs(Math.sin(rotation * Math.PI / 180));

			if(rotation < 0){
				rotation += 360;
			}

			if(size){
				if(this.vertical ? rotation != 0 && rotation != 180 : rotation != 90 && rotation != 270){
					// we need width of all labels
					if(o.maxLabelSize){
						labelWidth = o.maxLabelSize;
					}else if(this.labels){
						labelWidth = this._groupLabelWidth(this.labels, taFont);
					}else{
						var labelLength = Math.ceil(
								Math.log(
									Math.max(
										Math.abs(tsb.from),
										Math.abs(tsb.to)
									)
								) / Math.LN10
							),
							t = [];
						if(tsb.from < 0 || tsb.to < 0){
							t.push("-");
						}
						t.push(dojo.string.rep("9", labelLength));
						var precision = Math.floor(
							Math.log( tsb.to - tsb.from ) / Math.LN10
						);
						if(precision > 0){
							t.push(".");
							t.push(dojo.string.rep("9", precision));
						}
						labelWidth = dojox.gfx._base._getTextBox(
							t.join(""),
							{ font: taFont }
						).w;
					}
				}else{
					labelWidth = size;
				}
				switch(rotation){
					case 0:
					case 90:
					case 180:
					case 270:
						// trivial cases: use labelWidth
						break;
					default:
						// rotated labels
						var gap1 = Math.sqrt(labelWidth * labelWidth + size * size),									// short labels
							gap2 = this.vertical ? size * cosr + labelWidth * sinr : labelWidth * cosr + size * sinr;	// slanted labels
						labelWidth = Math.min(gap1, gap2);
						break;
				}
			}

			this.scaler.minMinorStep = labelWidth + labelGap;
			this.ticks = lin.buildTicks(this.scaler, o);
			return this;	//	dojox.charting.axis2d.Default
		},
		getScaler: function(){
			//	summary:
			//		Get the pre-calculated scaler object.
			return this.scaler;	//	Object
		},
		getTicks: function(){
			//	summary:
			//		Get the pre-calculated ticks object.
			return this.ticks;	//	Object
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.axis2d.Default"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.axis2d.Default"] = true;
dojo.provide("dojox.charting.axis2d.Default");












/*=====
	dojox.charting.axis2d.__AxisCtorArgs = function(
		vertical, fixUpper, fixLower, natural, leftBottom,
		includeZero, fixed, majorLabels, minorTicks, minorLabels, microTicks, htmlLabels,
		min, max, from, to, majorTickStep, minorTickStep, microTickStep,
		labels, labelFunc, maxLabelSize,
		stroke, majorTick, minorTick, microTick, tick,
		font, fontColor
	){
	//	summary:
	//		Optional arguments used in the definition of an axis.
	//
	//	vertical: Boolean?
	//		A flag that says whether an axis is vertical (i.e. y axis) or horizontal. Default is false (horizontal).
	//	fixUpper: String?
	//		Align the greatest value on the axis with the specified tick level. Options are "major", "minor", "micro", or "none".  Defaults to "none".
	//	fixLower: String?
	//		Align the smallest value on the axis with the specified tick level. Options are "major", "minor", "micro", or "none".  Defaults to "none".
	//	natural: Boolean?
	//		Ensure tick marks are made on "natural" numbers. Defaults to false.
	//	leftBottom: Boolean?
	//		The position of a vertical axis; if true, will be placed against the left-bottom corner of the chart.  Defaults to true.
	//	includeZero: Boolean?
	//		Include 0 on the axis rendering.  Default is false.
	//	fixed: Boolean?
	//		Force all axis labels to be fixed numbers.  Default is true.
	//	majorLabels: Boolean?
	//		Flag to draw all labels at major ticks. Default is true.
	//	minorTicks: Boolean?
	//		Flag to draw minor ticks on an axis.  Default is true.
	//	minorLabels: Boolean?
	//		Flag to draw labels on minor ticks. Default is true.
	//	microTicks: Boolean?
	//		Flag to draw micro ticks on an axis. Default is false.
	//	htmlLabels: Boolean?
	//		Flag to use HTML (as opposed to the native vector graphics engine) to draw labels. Default is true.
	//	min: Number?
	//		The smallest value on an axis. Default is 0.
	//	max: Number?
	//		The largest value on an axis. Default is 1.
	//	from: Number?
	//		Force the chart to render data visible from this value. Default is 0.
	//	to: Number?
	//		Force the chart to render data visible to this value. Default is 1.
	//	majorTickStep: Number?
	//		The amount to skip before a major tick is drawn.  Default is 4.
	//	minorTickStep: Number?
	//		The amount to skip before a minor tick is drawn. Default is 2.
	//	microTickStep: Number?
	//		The amount to skip before a micro tick is drawn. Default is 1.
	//	labels: Object[]?
	//		An array of labels for major ticks, with corresponding numeric values, ordered by value.
	//	labelFunc: Function?
	//		An optional function used to compute label values.
	//	maxLabelSize: Number?
	//		The maximum size, in pixels, for a label.  To be used with the optional label function.
	//	stroke: dojox.gfx.Stroke?
	//		An optional stroke to be used for drawing an axis.
	//	majorTick: Object?
	//		An object containing a dojox.gfx.Stroke, and a length (number) for a major tick.
	//	minorTick: Object?
	//		An object containing a dojox.gfx.Stroke, and a length (number) for a minor tick.
	//	microTick: Object?
	//		An object containing a dojox.gfx.Stroke, and a length (number) for a micro tick.
	//	tick: Object?
	//		An object containing a dojox.gfx.Stroke, and a length (number) for a tick.
	//	font: String?
	//		An optional font definition (as used in the CSS font property) for labels.
	//	fontColor: String|dojo.Color?
	//		An optional color to be used in drawing labels.

	this.vertical = vertical;
	this.fixUpper = fixUpper;
	this.fixLower = fixLower;
	this.natural = natural;
	this.leftBottom = leftBottom;
	this.includeZero = includeZero;
	this.fixed = fixed;
	this.majorLabels = majorLabels;
	this.minorTicks = minorTicks;
	this.minorLabels = minorLabels;
	this.microTicks = microTicks;
	this.htmlLabels = htmlLabels;
	this.min = min;
	this.max = max;
	this.from = from;
	this.to = to;
	this.majorTickStep = majorTickStep;
	this.minorTickStep = minorTickStep;
	this.microTickStep = microTickStep;
	this.labels = labels;
	this.labelFunc = labelFunc;
	this.maxLabelSize = maxLabelSize;
	this.stroke = stroke;
	this.majorTick = majorTick;
	this.minorTick = minorTick;
	this.microTick = microTick;
	this.tick = tick;
	this.font = font;
	this.fontColor = fontColor;
}
=====*/
(function(){
	var dc = dojox.charting,
		du = dojox.lang.utils,
		g = dojox.gfx,
		lin = dc.scaler.linear,
		labelGap = 4,			// in pixels
		centerAnchorLimit = 45;	// in degrees

	dojo.declare("dojox.charting.axis2d.Default", dojox.charting.axis2d.Invisible, {
		//	summary:
		//		The default axis object used in dojox.charting.  See dojox.charting.Chart2D.addAxis for details.
		//
		//	defaultParams: Object
		//		The default parameters used to define any axis.
		//	optionalParams: Object
		//		Any optional parameters needed to define an axis.

		/*
		//	TODO: the documentation tools need these to be pre-defined in order to pick them up
		//	correctly, but the code here is partially predicated on whether or not the properties
		//	actually exist.  For now, we will leave these undocumented but in the code for later. -- TRT

		//	opt: Object
		//		The actual options used to define this axis, created at initialization.
		//	scalar: Object
		//		The calculated helper object to tell charts how to draw an axis and any data.
		//	ticks: Object
		//		The calculated tick object that helps a chart draw the scaling on an axis.
		//	dirty: Boolean
		//		The state of the axis (whether it needs to be redrawn or not)
		//	scale: Number
		//		The current scale of the axis.
		//	offset: Number
		//		The current offset of the axis.

		opt: null,
		scalar: null,
		ticks: null,
		dirty: true,
		scale: 1,
		offset: 0,
		*/
		defaultParams: {
			vertical:    false,		// true for vertical axis
			fixUpper:    "none",	// align the upper on ticks: "major", "minor", "micro", "none"
			fixLower:    "none",	// align the lower on ticks: "major", "minor", "micro", "none"
			natural:     false,		// all tick marks should be made on natural numbers
			leftBottom:  true,		// position of the axis, used with "vertical"
			includeZero: false,		// 0 should be included
			fixed:       true,		// all labels are fixed numbers
			majorLabels: true,		// draw major labels
			minorTicks:  true,		// draw minor ticks
			minorLabels: true,		// draw minor labels
			microTicks:  false,		// draw micro ticks
			rotation:    0,			// label rotation angle in degrees
			htmlLabels:  true		// use HTML to draw labels
		},
		optionalParams: {
			min:			0,	// minimal value on this axis
			max:			1,	// maximal value on this axis
			from:			0,	// visible from this value
			to:				1,	// visible to this value
			majorTickStep:	4,	// major tick step
			minorTickStep:	2,	// minor tick step
			microTickStep:	1,	// micro tick step
			labels:			[],	// array of labels for major ticks
								// with corresponding numeric values
								// ordered by values
			labelFunc:		null, // function to compute label values
			maxLabelSize:	0,	// size in px. For use with labelFunc

			// TODO: add support for minRange!
			// minRange:		1,	// smallest distance from min allowed on the axis

			// theme components
			stroke:			{},	// stroke for an axis
			majorTick:		{},	// stroke + length for a tick
			minorTick:		{},	// stroke + length for a tick
			microTick:		{},	// stroke + length for a tick
			tick:           {},	// stroke + length for a tick
			font:			"",	// font for labels
			fontColor:		""	// color for labels as a string
		},

		constructor: function(chart, kwArgs){
			//	summary:
			//		The constructor for an axis.
			//	chart: dojox.charting.Chart2D
			//		The chart the axis belongs to.
			//	kwArgs: dojox.charting.axis2d.__AxisCtorArgs?
			//		Any optional keyword arguments to be used to define this axis.
			this.opt = dojo.delegate(this.defaultParams, kwArgs);
			// du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
		},
		getOffsets: function(){
			//	summary:
			//		Get the physical offset values for this axis (used in drawing data series).
			//	returns: Object
			//		The calculated offsets in the form of { l, r, t, b } (left, right, top, bottom).
			var s = this.scaler, offsets = { l: 0, r: 0, t: 0, b: 0 };
			if(!s){
				return offsets;
			}
			var o = this.opt, labelWidth = 0, a, b, c, d,
				gl = dc.scaler.common.getNumericLabel,
				offset = 0, ma = s.major, mi = s.minor,
				ta = this.chart.theme.axis,
				// TODO: we use one font --- of major tick, we need to use major and minor fonts
				taFont = o.font || (ta.majorTick && ta.majorTick.font) || (ta.tick && ta.tick.font),
				taMajorTick = this.chart.theme.getTick("major", o),
				taMinorTick = this.chart.theme.getTick("minor", o),
				size = taFont ? g.normalizedLength(g.splitFontString(taFont).size) : 0,
				rotation = o.rotation % 360, leftBottom = o.leftBottom,
				cosr = Math.abs(Math.cos(rotation * Math.PI / 180)),
				sinr = Math.abs(Math.sin(rotation * Math.PI / 180));
			if(rotation < 0){
				rotation += 360;
			}

			if(size){
				// we need width of all labels
				if(o.maxLabelSize){
					labelWidth = o.maxLabelSize;
				}else if(this.labels){
					labelWidth = this._groupLabelWidth(this.labels, taFont);
				}else{
					labelWidth = this._groupLabelWidth([
						gl(ma.start, ma.prec, o),
						gl(ma.start + ma.count * ma.tick, ma.prec, o),
						gl(mi.start, mi.prec, o),
						gl(mi.start + mi.count * mi.tick, mi.prec, o)
					], taFont);
				}
				if(this.vertical){
					var side = leftBottom ? "l" : "r";
					switch(rotation){
						case 0:
						case 180:
							offsets[side] = labelWidth;
							offsets.t = offsets.b = size / 2;
							break;
						case 90:
						case 270:
							offsets[side] = size;
							offsets.t = offsets.b = labelWidth / 2;
							break;
						default:
							if(rotation <= centerAnchorLimit || (180 < rotation && rotation <= (180 + centerAnchorLimit))){
								offsets[side] = size * sinr / 2 + labelWidth * cosr;
								offsets[leftBottom ? "t" : "b"] = size * cosr / 2 + labelWidth * sinr;
								offsets[leftBottom ? "b" : "t"] = size * cosr / 2;
							}else if(rotation > (360 - centerAnchorLimit) || (180 > rotation && rotation > (180 - centerAnchorLimit))){
								offsets[side] = size * sinr / 2 + labelWidth * cosr;
								offsets[leftBottom ? "b" : "t"] = size * cosr / 2 + labelWidth * sinr;
								offsets[leftBottom ? "t" : "b"] = size * cosr / 2;
							}else if(rotation < 90 || (180 < rotation && rotation < 270)){
								offsets[side] = size * sinr + labelWidth * cosr;
								offsets[leftBottom ? "t" : "b"] = size * cosr + labelWidth * sinr;
							}else{
								offsets[side] = size * sinr + labelWidth * cosr;
								offsets[leftBottom ? "b" : "t"] = size * cosr + labelWidth * sinr;
							}
							break;
					}
					offsets[side] += labelGap + Math.max(taMajorTick.length, taMinorTick.length);
				}else{
					var side = leftBottom ? "b" : "t";
					switch(rotation){
						case 0:
						case 180:
							offsets[side] = size;
							offsets.l = offsets.r = labelWidth / 2;
							break;
						case 90:
						case 270:
							offsets[side] = labelWidth;
							offsets.l = offsets.r = size / 2;
							break;
						default:
							if((90 - centerAnchorLimit) <= rotation && rotation <= 90 || (270 - centerAnchorLimit) <= rotation && rotation <= 270){
								offsets[side] = size * sinr / 2 + labelWidth * cosr;
								offsets[leftBottom ? "r" : "l"] = size * cosr / 2 + labelWidth * sinr;
								offsets[leftBottom ? "l" : "r"] = size * cosr / 2;
							}else if(90 <= rotation && rotation <= (90 + centerAnchorLimit) || 270 <= rotation && rotation <= (270 + centerAnchorLimit)){
								offsets[side] = size * sinr / 2 + labelWidth * cosr;
								offsets[leftBottom ? "l" : "r"] = size * cosr / 2 + labelWidth * sinr;
								offsets[leftBottom ? "r" : "l"] = size * cosr / 2;
							}else if(rotation < centerAnchorLimit || (180 < rotation && rotation < (180 - centerAnchorLimit))){
								offsets[side] = size * sinr + labelWidth * cosr;
								offsets[leftBottom ? "r" : "l"] = size * cosr + labelWidth * sinr;
							}else{
								offsets[side] = size * sinr + labelWidth * cosr;
								offsets[leftBottom ? "l" : "r"] = size * cosr + labelWidth * sinr;
							}
							break;
					}
					offsets[side] += labelGap + Math.max(taMajorTick.length, taMinorTick.length);
				}
			}
			if(labelWidth){
				this._cachedLabelWidth = labelWidth;
			}
			return offsets;	//	Object
		},
		render: function(dim, offsets){
			//	summary:
			//		Render/draw the axis.
			//	dim: Object
			//		An object of the form { width, height}.
			//	offsets: Object
			//		An object of the form { l, r, t, b }.
			//	returns: dojox.charting.axis2d.Default
			//		The reference to the axis for functional chaining.
			if(!this.dirty){
				return this;	//	dojox.charting.axis2d.Default
			}
			// prepare variable
			var o = this.opt, ta = this.chart.theme.axis, leftBottom = o.leftBottom, rotation = o.rotation % 360,
				start, stop, axisVector, tickVector, anchorOffset, labelOffset, labelAlign,

				// TODO: we use one font --- of major tick, we need to use major and minor fonts
				taFont = o.font || (ta.majorTick && ta.majorTick.font) || (ta.tick && ta.tick.font),
				// TODO: we use one font color --- we need to use different colors
				taFontColor = o.fontColor || (ta.majorTick && ta.majorTick.fontColor) || (ta.tick && ta.tick.fontColor) || "black",
				taMajorTick = this.chart.theme.getTick("major", o),
				taMinorTick = this.chart.theme.getTick("minor", o),
				taMicroTick = this.chart.theme.getTick("micro", o),

				tickSize = Math.max(taMajorTick.length, taMinorTick.length, taMicroTick.length),
				taStroke = "stroke" in o ? o.stroke : ta.stroke,
				size = taFont ? g.normalizedLength(g.splitFontString(taFont).size) : 0;
			if(rotation < 0){
				rotation += 360;
			}
			if(this.vertical){
				start = {y: dim.height - offsets.b};
				stop  = {y: offsets.t};
				axisVector = {x: 0, y: -1};
				labelOffset = {x: 0, y: 0};
				tickVector = {x: 1, y: 0};
				anchorOffset = {x: labelGap, y: 0};
				switch(rotation){
					case 0:
						labelAlign = "end";
						labelOffset.y = size * 0.4;
						break;
					case 90:
						labelAlign = "middle";
						labelOffset.x = -size;
						break;
					case 180:
						labelAlign = "start";
						labelOffset.y = -size * 0.4;
						break;
					case 270:
						labelAlign = "middle";
						break;
					default:
						if(rotation < centerAnchorLimit){
							labelAlign = "end";
							labelOffset.y = size * 0.4;
						}else if(rotation < 90){
							labelAlign = "end";
							labelOffset.y = size * 0.4;
						}else if(rotation < (180 - centerAnchorLimit)){
							labelAlign = "start";
						}else if(rotation < (180 + centerAnchorLimit)){
							labelAlign = "start";
							labelOffset.y = -size * 0.4;
						}else if(rotation < 270){
							labelAlign = "start";
							labelOffset.x = leftBottom ? 0 : size * 0.4;
						}else if(rotation < (360 - centerAnchorLimit)){
							labelAlign = "end";
							labelOffset.x = leftBottom ? 0 : size * 0.4;
						}else{
							labelAlign = "end";
							labelOffset.y = size * 0.4;
						}
				}
				if(leftBottom){
					start.x = stop.x = offsets.l;
					tickVector.x = -1;
					anchorOffset.x = -anchorOffset.x;
				}else{
					start.x = stop.x = dim.width - offsets.r;
					switch(labelAlign){
						case "start":
							labelAlign = "end";
							break;
						case "end":
							labelAlign = "start";
							break;
						case "middle":
							labelOffset.x += size;
							break;
					}
				}
			}else{
				start = {x: offsets.l};
				stop  = {x: dim.width - offsets.r};
				axisVector = {x: 1, y: 0};
				labelOffset = {x: 0, y: 0};
				tickVector = {x: 0, y: 1};
				anchorOffset = {x: 0, y: labelGap};
				switch(rotation){
					case 0:
						labelAlign = "middle";
						labelOffset.y = size;
						break;
					case 90:
						labelAlign = "start";
						labelOffset.x = -size * 0.4;
						break;
					case 180:
						labelAlign = "middle";
						break;
					case 270:
						labelAlign = "end";
						labelOffset.x = size * 0.4;
						break;
					default:
						if(rotation < (90 - centerAnchorLimit)){
							labelAlign = "start";
							labelOffset.y = leftBottom ? size : 0;
						}else if(rotation < (90 + centerAnchorLimit)){
							labelAlign = "start";
							labelOffset.x = -size * 0.4;
						}else if(rotation < 180){
							labelAlign = "start";
							labelOffset.y = leftBottom ? 0 : -size;
						}else if(rotation < (270 - centerAnchorLimit)){
							labelAlign = "end";
							labelOffset.y = leftBottom ? 0 : -size;
						}else if(rotation < (270 + centerAnchorLimit)){
							labelAlign = "end";
							labelOffset.y = leftBottom ? size * 0.4 : 0;
						}else{
							labelAlign = "end";
							labelOffset.y = leftBottom ? size : 0;
						}
				}
				if(leftBottom){
					start.y = stop.y = dim.height - offsets.b;
				}else{
					start.y = stop.y = offsets.t;
					tickVector.y = -1;
					anchorOffset.y = -anchorOffset.y;
					switch(labelAlign){
						case "start":
							labelAlign = "end";
							break;
						case "end":
							labelAlign = "start";
							break;
						case "middle":
							labelOffset.y -= size;
							break;
					}
				}
			}

			// render shapes

			this.cleanGroup();

			try{
				var s = this.group,
					c = this.scaler,
					t = this.ticks,
					canLabel,
					f = lin.getTransformerFromModel(this.scaler),
					forceHtmlLabels = (dojox.gfx.renderer == "canvas"),
					labelType = forceHtmlLabels || !rotation && this.opt.htmlLabels && !dojo.isIE && !dojo.isOpera ? "html" : "gfx",
					dx = tickVector.x * taMajorTick.length,
					dy = tickVector.y * taMajorTick.length;

				s.createLine({
					x1: start.x,
					y1: start.y,
					x2: stop.x,
					y2: stop.y
				}).setStroke(taStroke);

				dojo.forEach(t.major, function(tick){
					var offset = f(tick.value), elem,
						x = start.x + axisVector.x * offset,
						y = start.y + axisVector.y * offset;
						s.createLine({
							x1: x, y1: y,
							x2: x + dx,
							y2: y + dy
						}).setStroke(taMajorTick);
						if(tick.label){
							elem = dc.axis2d.common.createText[labelType](
								this.chart,
								s,
								x + dx + anchorOffset.x + (rotation ? 0 : labelOffset.x),
								y + dy + anchorOffset.y + (rotation ? 0 : labelOffset.y),
								labelAlign,
								tick.label,
								taFont,
								taFontColor
								//this._cachedLabelWidth
							);
							if(labelType == "html"){
								this.htmlElements.push(elem);
							}else if(rotation){
								elem.setTransform([
									{dx: labelOffset.x, dy: labelOffset.y},
									g.matrix.rotategAt(
										rotation,
										x + dx + anchorOffset.x,
										y + dy + anchorOffset.y
									)
								]);
							}
						}
				}, this);

				dx = tickVector.x * taMinorTick.length;
				dy = tickVector.y * taMinorTick.length;
				canLabel = c.minMinorStep <= c.minor.tick * c.bounds.scale;
				dojo.forEach(t.minor, function(tick){
					var offset = f(tick.value), elem,
						x = start.x + axisVector.x * offset,
						y = start.y + axisVector.y * offset;
						s.createLine({
							x1: x, y1: y,
							x2: x + dx,
							y2: y + dy
						}).setStroke(taMinorTick);
						if(canLabel && tick.label){
							elem = dc.axis2d.common.createText[labelType](
								this.chart,
								s,
								x + dx + anchorOffset.x + (rotation ? 0 : labelOffset.x),
								y + dy + anchorOffset.y + (rotation ? 0 : labelOffset.y),
								labelAlign,
								tick.label,
								taFont,
								taFontColor
								//this._cachedLabelWidth
							);
							if(labelType == "html"){
								this.htmlElements.push(elem);
							}else if(rotation){
								elem.setTransform([
									{dx: labelOffset.x, dy: labelOffset.y},
									g.matrix.rotategAt(
										rotation,
										x + dx + anchorOffset.x,
										y + dy + anchorOffset.y
									)
								]);
							}
						}
				}, this);

				dx = tickVector.x * taMicroTick.length;
				dy = tickVector.y * taMicroTick.length;
				dojo.forEach(t.micro, function(tick){
					var offset = f(tick.value), elem,
						x = start.x + axisVector.x * offset,
						y = start.y + axisVector.y * offset;
						s.createLine({
							x1: x, y1: y,
							x2: x + dx,
							y2: y + dy
						}).setStroke(taMicroTick);
				}, this);
			}catch(e){
				// squelch
			}

			this.dirty = false;
			return this;	//	dojox.charting.axis2d.Default
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.common"] = true;
dojo.provide("dojox.charting.plot2d.common");





(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common;

	dojo.mixin(dojox.charting.plot2d.common, {
		makeStroke: function(stroke){
			if(!stroke){ return stroke; }
			if(typeof stroke == "string" || stroke instanceof dojo.Color){
				stroke = {color: stroke};
			}
			return dojox.gfx.makeParameters(dojox.gfx.defaultStroke, stroke);
		},
		augmentColor: function(target, color){
			var t = new dojo.Color(target),
				c = new dojo.Color(color);
			c.a = t.a;
			return c;
		},
		augmentStroke: function(stroke, color){
			var s = dc.makeStroke(stroke);
			if(s){
				s.color = dc.augmentColor(s.color, color);
			}
			return s;
		},
		augmentFill: function(fill, color){
			var fc, c = new dojo.Color(color);
			if(typeof fill == "string" || fill instanceof dojo.Color){
				return dc.augmentColor(fill, color);
			}
			return fill;
		},

		defaultStats: {
			hmin: Number.POSITIVE_INFINITY, hmax: Number.NEGATIVE_INFINITY,
			vmin: Number.POSITIVE_INFINITY, vmax: Number.NEGATIVE_INFINITY
		},

		collectSimpleStats: function(series){
			var stats = dojo.delegate(dc.defaultStats);
			for(var i = 0; i < series.length; ++i){
				var run = series[i];
				for(var j = 0; j < run.data.length; j++){
					if(run.data[j] !== null){
						if(typeof run.data[j] == "number"){
							// 1D case
							var old_vmin = stats.vmin, old_vmax = stats.vmax;
							if(!("ymin" in run) || !("ymax" in run)){
								dojo.forEach(run.data, function(val, i){
									if(val !== null){
										var x = i + 1, y = val;
										if(isNaN(y)){ y = 0; }
										stats.hmin = Math.min(stats.hmin, x);
										stats.hmax = Math.max(stats.hmax, x);
										stats.vmin = Math.min(stats.vmin, y);
										stats.vmax = Math.max(stats.vmax, y);
									}
								});
							}
							if("ymin" in run){ stats.vmin = Math.min(old_vmin, run.ymin); }
							if("ymax" in run){ stats.vmax = Math.max(old_vmax, run.ymax); }
						}else{
							// 2D case
							var old_hmin = stats.hmin, old_hmax = stats.hmax,
								old_vmin = stats.vmin, old_vmax = stats.vmax;
							if(!("xmin" in run) || !("xmax" in run) || !("ymin" in run) || !("ymax" in run)){
								dojo.forEach(run.data, function(val, i){
									if(val !== null){
										var x = "x" in val ? val.x : i + 1, y = val.y;
										if(isNaN(x)){ x = 0; }
										if(isNaN(y)){ y = 0; }
										stats.hmin = Math.min(stats.hmin, x);
										stats.hmax = Math.max(stats.hmax, x);
										stats.vmin = Math.min(stats.vmin, y);
										stats.vmax = Math.max(stats.vmax, y);
									}
								});
							}
							if("xmin" in run){ stats.hmin = Math.min(old_hmin, run.xmin); }
							if("xmax" in run){ stats.hmax = Math.max(old_hmax, run.xmax); }
							if("ymin" in run){ stats.vmin = Math.min(old_vmin, run.ymin); }
							if("ymax" in run){ stats.vmax = Math.max(old_vmax, run.ymax); }
						}

						break;
					}
				}
			}
			return stats;
		},

		calculateBarSize: function(/* Number */ availableSize, /* Object */ opt, /* Number? */ clusterSize){
			if(!clusterSize){
				clusterSize = 1;
			}
			var gap = opt.gap, size = (availableSize - 2 * gap) / clusterSize;
			if("minBarSize" in opt){
				size = Math.max(size, opt.minBarSize);
			}
			if("maxBarSize" in opt){
				size = Math.min(size, opt.maxBarSize);
			}
			size = Math.max(size, 1);
			gap = (availableSize - size * clusterSize) / 2;
			return {size: size, gap: gap};	// Object
		},

		collectStackedStats: function(series){
			// collect statistics
			var stats = dojo.clone(dc.defaultStats);
			if(series.length){
				// 1st pass: find the maximal length of runs
				stats.hmin = Math.min(stats.hmin, 1);
				stats.hmax = df.foldl(series, "seed, run -> Math.max(seed, run.data.length)", stats.hmax);
				// 2nd pass: stack values
				for(var i = 0; i < stats.hmax; ++i){
					var v = series[0].data[i];
					if(isNaN(v)){ v = 0; }
					stats.vmin = Math.min(stats.vmin, v);
					for(var j = 1; j < series.length; ++j){
						var t = series[j].data[i];
						if(isNaN(t)){ t = 0; }
						v += t;
					}
					stats.vmax = Math.max(stats.vmax, v);
				}
			}
			return stats;
		},

		curve: function(/* Number[] */a, /* Number|String */tension){
			//	FIX for #7235, submitted by Enzo Michelangeli.
			//	Emulates the smoothing algorithms used in a famous, unnamed spreadsheet
			//		program ;)
			var arr = a.slice(0);
			if(tension == "x") {
				arr[arr.length] = arr[0];   // add a last element equal to the first, closing the loop
			}
			var p=dojo.map(arr, function(item, i){
				if(i==0){ return "M" + item.x + "," + item.y; }
				if(!isNaN(tension)) { // use standard Dojo smoothing in tension is numeric
					var dx=item.x-arr[i-1].x, dy=arr[i-1].y;
					return "C"+(item.x-(tension-1)*(dx/tension))+","+dy+" "+(item.x-(dx/tension))+","+item.y+" "+item.x+","+item.y;
				} else if(tension == "X" || tension == "x" || tension == "S") {
					// use Excel "line smoothing" algorithm (http://xlrotor.com/resources/files.shtml)
					var p0, p1 = arr[i-1], p2 = arr[i], p3;
					var bz1x, bz1y, bz2x, bz2y;
					var f = 1/6;
					if(i==1) {
						if(tension == "x") {
							p0 = arr[arr.length-2];
						} else { // "tension == X || tension == "S"
							p0 = p1;
						}
						f = 1/3;
					} else {
						p0 = arr[i-2];
					}
					if(i==(arr.length-1)) {
						if(tension == "x") {
							p3 = arr[1];
						} else { // "tension == X || tension == "S"
							p3 = p2;
						}
						f = 1/3;
					} else {
						p3 = arr[i+1];
					}
					var p1p2 = Math.sqrt((p2.x-p1.x)*(p2.x-p1.x)+(p2.y-p1.y)*(p2.y-p1.y));
					var p0p2 = Math.sqrt((p2.x-p0.x)*(p2.x-p0.x)+(p2.y-p0.y)*(p2.y-p0.y));
					var p1p3 = Math.sqrt((p3.x-p1.x)*(p3.x-p1.x)+(p3.y-p1.y)*(p3.y-p1.y));

					var p0p2f = p0p2 * f;
					var p1p3f = p1p3 * f;

					if(p0p2f > p1p2/2 && p1p3f > p1p2/2) {
						p0p2f = p1p2/2;
						p1p3f = p1p2/2;
					} else if(p0p2f > p1p2/2) {
						p0p2f = p1p2/2;
						p1p3f = p1p2/2 * p1p3/p0p2;
					} else if(p1p3f > p1p2/2) {
						p1p3f = p1p2/2;
						p0p2f = p1p2/2 * p0p2/p1p3;
					}

					if(tension == "S") {
						if(p0 == p1) { p0p2f = 0; }
						if(p2 == p3) { p1p3f = 0; }
					}

					bz1x = p1.x + p0p2f*(p2.x - p0.x)/p0p2;
					bz1y = p1.y + p0p2f*(p2.y - p0.y)/p0p2;
					bz2x = p2.x - p1p3f*(p3.x - p1.x)/p1p3;
					bz2y = p2.y - p1p3f*(p3.y - p1.y)/p1p3;
				}
				return "C"+(bz1x+","+bz1y+" "+bz2x+","+bz2y+" "+p2.x+","+p2.y);
			});
			return p.join(" ");
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.scaler.primitive"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.scaler.primitive"] = true;
dojo.provide("dojox.charting.scaler.primitive");

dojox.charting.scaler.primitive = {
	buildScaler: function(/*Number*/ min, /*Number*/ max, /*Number*/ span, /*Object*/ kwArgs){
		if(min == max){
			// artificially extend bounds
			min -= 0.5;
			max += 0.5;
			// now the line will be centered
		}
		return {
			bounds: {
				lower: min,
				upper: max,
				from:  min,
				to:    max,
				scale: span / (max - min),
				span:  span
			},
			scaler: dojox.charting.scaler.primitive
		};
	},
	buildTicks: function(/*Object*/ scaler, /*Object*/ kwArgs){
		return {major: [], minor: [], micro: []};	// Object
	},
	getTransformerFromModel: function(/*Object*/ scaler){
		var offset = scaler.bounds.from, scale = scaler.bounds.scale;
		return function(x){ return (x - offset) * scale; };	// Function
	},
	getTransformerFromPlot: function(/*Object*/ scaler){
		var offset = scaler.bounds.from, scale = scaler.bounds.scale;
		return function(x){ return x / scale + offset; };	// Function
	}
};

}

if(!dojo._hasResource["dojox.charting.plot2d._PlotEvents"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d._PlotEvents"] = true;
dojo.provide("dojox.charting.plot2d._PlotEvents");

dojo.declare("dojox.charting.plot2d._PlotEvents", null, {
	constructor: function(){
		this._shapeEvents = [];
		this._eventSeries = {};
	},
	destroy: function(){
		//	summary:
		//		Destroy any internal elements and event handlers.
		this.resetEvents();
		this.inherited(arguments);
	},
	plotEvent: function(o){
		//	summary:
		//		Stub function for use by specific plots.
		//	o: Object
		//		An object intended to represent event parameters.
	},
	raiseEvent: function(o){
		//	summary:
		//		Raises events in predefined order
		//	o: Object
		//		An object intended to represent event parameters.
		this.plotEvent(o);
		var t = dojo.delegate(o);
		t.originalEvent = o.type;
		t.originalPlot  = o.plot;
		t.type = "onindirect";
		dojo.forEach(this.chart.stack, function(plot){
			if(plot !== this && plot.plotEvent){
				t.plot = plot;
				plot.plotEvent(t);
			}
		}, this);
	},
	connect: function(object, method){
		//	summary:
		//		Helper function to connect any object's method to our plotEvent.
		//	object: Object
		//		The object to connect to.
		//	method: String|Function
		//		The method to fire when our plotEvent is fired.
		//	returns: Array
		//		The handle as returned from dojo.connect (see dojo.connect).
		this.dirty = true;
		return dojo.connect(this, "plotEvent", object, method);	//	Array
	},
	events: function(){
		//	summary:
		//		Find out if any event handlers have been connected to our plotEvent.
		//	returns: Boolean
		//		A flag indicating that there are handlers attached.
		var ls = this.plotEvent._listeners;
		if(!ls || !ls.length){ return false; }
		for(var i in ls){
			if(!(i in Array.prototype)){
				return true;
			}
		}
		return false;
	},
	resetEvents: function(){
		//	summary:
		//		Reset all events attached to our plotEvent (i.e. disconnect).
		if(this._shapeEvents.length){
			dojo.forEach(this._shapeEvents, function(item){
				item.shape.disconnect(item.handle);
			});
			this._shapeEvents = [];
		}
		this.raiseEvent({type: "onplotreset", plot: this});
	},
	_connectSingleEvent: function(o, eventName){
		this._shapeEvents.push({
			shape:  o.eventMask,
			handle: o.eventMask.connect(eventName, this, function(e){
				o.type  = eventName;
				o.event = e;
				this.raiseEvent(o);
				o.event = null;
			})
		});
	},
	_connectEvents: function(o){
		o.chart = this.chart;
		o.plot  = this;
		o.hAxis = this.hAxis || null;
		o.vAxis = this.vAxis || null;
		o.eventMask = o.eventMask || o.shape;
		this._connectSingleEvent(o, "onmouseover");
		this._connectSingleEvent(o, "onmouseout");
		this._connectSingleEvent(o, "onclick");
	},
	_reconnectEvents: function(seriesName){
		var a = this._eventSeries[seriesName];
		if(a){
			dojo.forEach(a, this._connectEvents, this);
		}
	},
	fireEvent: function(seriesName, eventName, index, eventObject){
		//	summary:
		//		Emulates firing an event for a given data value (specified by
		//		an index) of a given series.
		//	seriesName: String:
		//		Series name.
		//	eventName: String:
		//		Event name to emulate.
		//	index:	Number:
		//		Valid data value index used to raise an event.
		//	eventObject: Object?:
		//		Optional event object. Especially useful for synthetic events.
		//		Default: null.
		var s = this._eventSeries[seriesName];
		if(s && s.length && index < s.length){
			var o = s[index];
			o.type  = eventName;
			o.event = eventObject || null;
			this.raiseEvent(o);
			o.event = null;
		}
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.Base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Base"] = true;
dojo.provide("dojox.charting.plot2d.Base");






/*=====
dojox.charting.plot2d.__PlotCtorArgs = function(){
	//	summary:
	//		The base keyword arguments object for plot constructors.
	//		Note that the parameters for this may change based on the
	//		specific plot type (see the corresponding plot type for
	//		details).
}
=====*/
dojo.declare("dojox.charting.plot2d.Base", [dojox.charting.Element, dojox.charting.plot2d._PlotEvents], {
	constructor: function(chart, kwArgs){
		//	summary:
		//		Create a base plot for charting.
		//	chart: dojox.chart.Chart2D
		//		The chart this plot belongs to.
		//	kwArgs: dojox.charting.plot2d.__PlotCtorArgs?
		//		An optional arguments object to help define the plot.
		this.zoom = null,
		this.zoomQueue = [];	// zooming action task queue
		this.lastWindow = {vscale: 1, hscale: 1, xoffset: 0, yoffset: 0};
	},
	clear: function(){
		//	summary:
		//		Clear out all of the information tied to this plot.
		//	returns: dojox.charting.plot2d.Base
		//		A reference to this plot for functional chaining.
		this.series = [];
		this._hAxis = null;
		this._vAxis = null;
		this.dirty = true;
		return this;	//	dojox.charting.plot2d.Base
	},
	setAxis: function(axis){
		//	summary:
		//		Set an axis for this plot.
		//	axis: dojox.charting.axis2d.Base
		//		The axis to set.
		//	returns: dojox.charting.plot2d.Base
		//		A reference to this plot for functional chaining.
		if(axis){
			this[axis.vertical ? "_vAxis" : "_hAxis"] = axis;
		}
		return this;	//	dojox.charting.plot2d.Base
	},
	addSeries: function(run){
		//	summary:
		//		Add a data series to this plot.
		//	run: dojox.charting.Series
		//		The series to be added.
		//	returns: dojox.charting.plot2d.Base
		//		A reference to this plot for functional chaining.
		this.series.push(run);
		return this;	//	dojox.charting.plot2d.Base
	},
	getSeriesStats: function(){
		//	summary:
		//		Calculate the min/max on all attached series in both directions.
		//	returns: Object
		//		{hmin, hmax, vmin, vmax} min/max in both directions.
		return dojox.charting.plot2d.common.collectSimpleStats(this.series);
	},
	calculateAxes: function(dim){
		//	summary:
		//		Stub function for running the axis calculations (depricated).
		//	dim: Object
		//		An object of the form { width, height }
		//	returns: dojox.charting.plot2d.Base
		//		A reference to this plot for functional chaining.
		this.initializeScalers(dim, this.getSeriesStats());
		return this;	//	dojox.charting.plot2d.Base
	},
	isDirty: function(){
		//	summary:
		//		Returns whether or not this plot needs to be rendered.
		//	returns: Boolean
		//		The state of the plot.
		return this.dirty || this._hAxis && this._hAxis.dirty || this._vAxis && this._vAxis.dirty;	//	Boolean
	},
	isDataDirty: function(){
		//	summary:
		//		Returns whether or not any of this plot's data series need to be rendered.
		//	returns: Boolean
		//		Flag indicating if any of this plot's series are invalid and need rendering.
		return dojo.some(this.series, function(item){ return item.dirty; });	//	Boolean
	},
	performZoom: function(dim, offsets){
		//	summary:
		//		Create/alter any zooming windows on this plot.
		//	dim: Object
		//		An object of the form { width, height }.
		//	offsets: Object
		//		An object of the form { l, r, t, b }.
		//	returns: dojox.charting.plot2d.Base
		//		A reference to this plot for functional chaining.

		// get current zooming various
		var vs = this._vAxis.scale || 1,
			hs = this._hAxis.scale || 1,
			vOffset = dim.height - offsets.b,
			hBounds = this._hScaler.bounds,
			xOffset = (hBounds.from - hBounds.lower) * hBounds.scale,
			vBounds = this._vScaler.bounds,
			yOffset = (vBounds.from - vBounds.lower) * vBounds.scale;
			// get incremental zooming various
			rVScale = vs / this.lastWindow.vscale,
			rHScale = hs / this.lastWindow.hscale,
			rXOffset = (this.lastWindow.xoffset - xOffset)/
				((this.lastWindow.hscale == 1)? hs : this.lastWindow.hscale),
			rYOffset = (yOffset - this.lastWindow.yoffset)/
				((this.lastWindow.vscale == 1)? vs : this.lastWindow.vscale),

			shape = this.group,
			anim = dojox.gfx.fx.animateTransform(dojo.delegate({
				shape: shape,
				duration: 1200,
				transform:[
					{name:"translate", start:[0, 0], end: [offsets.l * (1 - rHScale), vOffset * (1 - rVScale)]},
					{name:"scale", start:[1, 1], end: [rHScale, rVScale]},
					{name:"original"},
					{name:"translate", start: [0, 0], end: [rXOffset, rYOffset]}
				]}, this.zoom));

		dojo.mixin(this.lastWindow, {vscale: vs, hscale: hs, xoffset: xOffset, yoffset: yOffset});
		//add anim to zooming action queue,
		//in order to avoid several zooming action happened at the same time
		this.zoomQueue.push(anim);
		//perform each anim one by one in zoomQueue
		dojo.connect(anim, "onEnd", this, function(){
			this.zoom = null;
			this.zoomQueue.shift();
			if(this.zoomQueue.length > 0){
				this.zoomQueue[0].play();
			}
		});
		if(this.zoomQueue.length == 1){
			this.zoomQueue[0].play();
		}
		return this;	//	dojox.charting.plot2d.Base
	},
	render: function(dim, offsets){
		//	summary:
		//		Render the plot on the chart.
		//	dim: Object
		//		An object of the form { width, height }.
		//	offsets: Object
		//		An object of the form { l, r, t, b }.
		//	returns: dojox.charting.plot2d.Base
		//		A reference to this plot for functional chaining.
		return this;	//	dojox.charting.plot2d.Base
	},
	getRequiredColors: function(){
		//	summary:
		//		Get how many data series we have, so we know how many colors to use.
		//	returns: Number
		//		The number of colors needed.
		return this.series.length;	//	Number
	},
	initializeScalers: function(dim, stats){
		//	summary:
		//		Initializes scalers using attached axes.
		//	dim: Object:
		//		Size of a plot area in pixels as {width, height}.
		//	stats: Object:
		//		Min/max of data in both directions as {hmin, hmax, vmin, vmax}.
		//	returns: dojox.charting.plot2d.Base
		//		A reference to this plot for functional chaining.
		if(this._hAxis){
			if(!this._hAxis.initialized()){
				this._hAxis.calculate(stats.hmin, stats.hmax, dim.width);
			}
			this._hScaler = this._hAxis.getScaler();
		}else{
			this._hScaler = dojox.charting.scaler.primitive.buildScaler(stats.hmin, stats.hmax, dim.width);
		}
		if(this._vAxis){
			if(!this._vAxis.initialized()){
				this._vAxis.calculate(stats.vmin, stats.vmax, dim.height);
			}
			this._vScaler = this._vAxis.getScaler();
		}else{
			this._vScaler = dojox.charting.scaler.primitive.buildScaler(stats.vmin, stats.vmax, dim.height);
		}
		return this;	//	dojox.charting.plot2d.Base
	}
});

}

if(!dojo._hasResource["dojox.gfx.fx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.fx"] = true;
dojo.provide("dojox.gfx.fx");



(function(){
	var d = dojo, g = dojox.gfx, m = g.matrix;

	// Generic interpolators. Should they be moved to dojox.fx?

	var InterpolNumber = function(start, end){
		this.start = start, this.end = end;
	};
	d.extend(InterpolNumber, {
		getValue: function(r){
			return (this.end - this.start) * r + this.start;
		}
	});

	var InterpolUnit = function(start, end, units){
		this.start = start, this.end = end;
		this.units = units;
	};
	d.extend(InterpolUnit, {
		getValue: function(r){
			return (this.end - this.start) * r + this.start + this.units;
		}
	});

	var InterpolColor = function(start, end){
		this.start = start, this.end = end;
		this.temp = new dojo.Color();
	};
	d.extend(InterpolColor, {
		getValue: function(r){
			return d.blendColors(this.start, this.end, r, this.temp);
		}
	});

	var InterpolValues = function(values){
		this.values = values;
		this.length = values.length;
	};
	d.extend(InterpolValues, {
		getValue: function(r){
			return this.values[Math.min(Math.floor(r * this.length), this.length - 1)];
		}
	});

	var InterpolObject = function(values, def){
		this.values = values;
		this.def = def ? def : {};
	};
	d.extend(InterpolObject, {
		getValue: function(r){
			var ret = dojo.clone(this.def);
			for(var i in this.values){
				ret[i] = this.values[i].getValue(r);
			}
			return ret;
		}
	});

	var InterpolTransform = function(stack, original){
		this.stack = stack;
		this.original = original;
	};
	d.extend(InterpolTransform, {
		getValue: function(r){
			var ret = [];
			dojo.forEach(this.stack, function(t){
				if(t instanceof m.Matrix2D){
					ret.push(t);
					return;
				}
				if(t.name == "original" && this.original){
					ret.push(this.original);
					return;
				}
				if(!(t.name in m)){ return; }
				var f = m[t.name];
				if(typeof f != "function"){
					// constant
					ret.push(f);
					return;
				}
				var val = dojo.map(t.start, function(v, i){
								return (t.end[i] - v) * r + v;
							}),
					matrix = f.apply(m, val);
				if(matrix instanceof m.Matrix2D){
					ret.push(matrix);
				}
			}, this);
			return ret;
		}
	});

	var transparent = new d.Color(0, 0, 0, 0);

	var getColorInterpol = function(prop, obj, name, def){
		if(prop.values){
			return new InterpolValues(prop.values);
		}
		var value, start, end;
		if(prop.start){
			start = g.normalizeColor(prop.start);
		}else{
			start = value = obj ? (name ? obj[name] : obj) : def;
		}
		if(prop.end){
			end = g.normalizeColor(prop.end);
		}else{
			if(!value){
				value = obj ? (name ? obj[name] : obj) : def;
			}
			end = value;
		}
		return new InterpolColor(start, end);
	};

	var getNumberInterpol = function(prop, obj, name, def){
		if(prop.values){
			return new InterpolValues(prop.values);
		}
		var value, start, end;
		if(prop.start){
			start = prop.start;
		}else{
			start = value = obj ? obj[name] : def;
		}
		if(prop.end){
			end = prop.end;
		}else{
			if(typeof value != "number"){
				value = obj ? obj[name] : def;
			}
			end = value;
		}
		return new InterpolNumber(start, end);
	};

	g.fx.animateStroke = function(/*Object*/ args){
		// summary:
		//	Returns an animation which will change stroke properties over time
		// example:
		//	|	dojox.gfx.fx.animateStroke{{
		//	|		shape: shape,
		//	|		duration: 500,
		//	|		color: {start: "red", end: "green"},
		//	|		width: {end: 15},
		//	|		join:  {values: ["miter", "bevel", "round"]}
		//	|	}).play();
		if(!args.easing){ args.easing = d._defaultEasing; }
		var anim = new d.Animation(args), shape = args.shape, stroke;
		d.connect(anim, "beforeBegin", anim, function(){
			stroke = shape.getStroke();
			var prop = args.color, values = {}, value, start, end;
			if(prop){
				values.color = getColorInterpol(prop, stroke, "color", transparent);
			}
			prop = args.style;
			if(prop && prop.values){
				values.style = new InterpolValues(prop.values);
			}
			prop = args.width;
			if(prop){
				values.width = getNumberInterpol(prop, stroke, "width", 1);
			}
			prop = args.cap;
			if(prop && prop.values){
				values.cap = new InterpolValues(prop.values);
			}
			prop = args.join;
			if(prop){
				if(prop.values){
					values.join = new InterpolValues(prop.values);
				}else{
					start = prop.start ? prop.start : (stroke && stroke.join || 0);
					end = prop.end ? prop.end : (stroke && stroke.join || 0);
					if(typeof start == "number" && typeof end == "number"){
						values.join = new InterpolNumber(start, end);
					}
				}
			}
			this.curve = new InterpolObject(values, stroke);
		});
		d.connect(anim, "onAnimate", shape, "setStroke");
		return anim; // dojo.Animation
	};

	g.fx.animateFill = function(/*Object*/ args){
		// summary:
		//	Returns an animation which will change fill color over time.
		//	Only solid fill color is supported at the moment
		// example:
		//	|	dojox.gfx.fx.animateFill{{
		//	|		shape: shape,
		//	|		duration: 500,
		//	|		color: {start: "red", end: "green"}
		//	|	}).play();
		if(!args.easing){ args.easing = d._defaultEasing; }
		var anim = new d.Animation(args), shape = args.shape, fill;
		d.connect(anim, "beforeBegin", anim, function(){
			fill = shape.getFill();
			var prop = args.color, values = {};
			if(prop){
				this.curve = getColorInterpol(prop, fill, "", transparent);
			}
		});
		d.connect(anim, "onAnimate", shape, "setFill");
		return anim; // dojo.Animation
	};

	g.fx.animateFont = function(/*Object*/ args){
		// summary:
		//	Returns an animation which will change font properties over time
		// example:
		//	|	dojox.gfx.fx.animateFont{{
		//	|		shape: shape,
		//	|		duration: 500,
		//	|		variant: {values: ["normal", "small-caps"]},
		//	|		size:  {end: 10, units: "pt"}
		//	|	}).play();
		if(!args.easing){ args.easing = d._defaultEasing; }
		var anim = new d.Animation(args), shape = args.shape, font;
		d.connect(anim, "beforeBegin", anim, function(){
			font = shape.getFont();
			var prop = args.style, values = {}, value, start, end;
			if(prop && prop.values){
				values.style = new InterpolValues(prop.values);
			}
			prop = args.variant;
			if(prop && prop.values){
				values.variant = new InterpolValues(prop.values);
			}
			prop = args.weight;
			if(prop && prop.values){
				values.weight = new InterpolValues(prop.values);
			}
			prop = args.family;
			if(prop && prop.values){
				values.family = new InterpolValues(prop.values);
			}
			prop = args.size;
			if(prop && prop.units){
				start = parseFloat(prop.start ? prop.start : (shape.font && shape.font.size || "0"));
				end = parseFloat(prop.end ? prop.end : (shape.font && shape.font.size || "0"));
				values.size = new InterpolUnit(start, end, prop.units);
			}
			this.curve = new InterpolObject(values, font);
		});
		d.connect(anim, "onAnimate", shape, "setFont");
		return anim; // dojo.Animation
	};

	g.fx.animateTransform = function(/*Object*/ args){
		// summary:
		//	Returns an animation which will change transformation over time
		// example:
		//	|	dojox.gfx.fx.animateTransform{{
		//	|		shape: shape,
		//	|		duration: 500,
		//	|		transform: [
		//	|			{name: "translate", start: [0, 0], end: [200, 200]},
		//	|			{name: "original"}
		//	|		]
		//	|	}).play();
		if(!args.easing){ args.easing = d._defaultEasing; }
		var anim = new d.Animation(args), shape = args.shape, original;
		d.connect(anim, "beforeBegin", anim, function(){
			original = shape.getTransform();
			this.curve = new InterpolTransform(args.transform, original);
		});
		d.connect(anim, "onAnimate", shape, "setTransform");
		return anim; // dojo.Animation
	};
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.Default"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Default"] = true;
dojo.provide("dojox.charting.plot2d.Default");









/*=====
dojo.declare("dojox.charting.plot2d.__DefaultCtorArgs", dojox.charting.plot2d.__PlotCtorArgs, {
	//	summary:
	//		The arguments used for any/most plots.

	//	hAxis: String?
	//		The horizontal axis name.
	hAxis: "x",

	//	vAxis: String?
	//		The vertical axis name
	vAxis: "y",

	//	lines: Boolean?
	//		Whether or not to draw lines on this plot.  Defaults to true.
	lines:   true,

	//	areas: Boolean?
	//		Whether or not to draw areas on this plot. Defaults to false.
	areas:   false,

	//	markers: Boolean?
	//		Whether or not to draw markers at data points on this plot. Default is false.
	markers: false,

	//	tension: Number|String?
	//		Whether or not to apply 'tensioning' to the lines on this chart.
	//		Options include a number, "X", "x", or "S"; if a number is used, the
	//		simpler bezier curve calculations are used to draw the lines.  If X, x or S
	//		is used, the more accurate smoothing algorithm is used.
	tension: "",

	//	animate: Boolean?
	//		Whether or not to animate the chart to place.
	animate: false,

	//	stroke: dojox.gfx.Stroke?
	//		An optional stroke to use for any series on the plot.
	stroke:		{},

	//	outline: dojox.gfx.Stroke?
	//		An optional stroke used to outline any series on the plot.
	outline:	{},

	//	shadow: dojox.gfx.Stroke?
	//		An optional stroke to use to draw any shadows for a series on a plot.
	shadow:		{},

	//	fill: dojox.gfx.Fill?
	//		Any fill to be used for elements on the plot (such as areas).
	fill:		{},

	//	font: String?
	//		A font definition to be used for labels and other text-based elements on the plot.
	font:		"",

	//	fontColor: String|dojo.Color?
	//		The color to be used for any text-based elements on the plot.
	fontColor:	"",

	//	markerStroke: dojo.gfx.Stroke?
	//		An optional stroke to use for any markers on the plot.
	markerStroke:		{},

	//	markerOutline: dojo.gfx.Stroke?
	//		An optional outline to use for any markers on the plot.
	markerOutline:		{},

	//	markerShadow: dojo.gfx.Stroke?
	//		An optional shadow to use for any markers on the plot.
	markerShadow:		{},

	//	markerFill: dojo.gfx.Fill?
	//		An optional fill to use for any markers on the plot.
	markerFill:			{},

	//	markerFont: String?
	//		An optional font definition to use for any markers on the plot.
	markerFont:			"",

	//	markerFontColor: String|dojo.Color?
	//		An optional color to use for any marker text on the plot.
	markerFontColor:	""
});
=====*/
(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	var DEFAULT_ANIMATION_LENGTH = 1200;	// in ms

	dojo.declare("dojox.charting.plot2d.Default", dojox.charting.plot2d.Base, {
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			lines:   true,	// draw lines
			areas:   false,	// draw areas
			markers: false,	// draw markers
			tension: "",	// draw curved lines (tension is "X", "x", or "S")
			animate: false	// animate chart to place
		},
		optionalParams: {
			// theme component
			stroke:		{},
			outline:	{},
			shadow:		{},
			fill:		{},
			font:		"",
			fontColor:	"",
			markerStroke:		{},
			markerOutline:		{},
			markerShadow:		{},
			markerFill:			{},
			markerFont:			"",
			markerFontColor:	""
		},

		constructor: function(chart, kwArgs){
			//	summary:
			//		Return a new plot.
			//	chart: dojox.charting.Chart2D
			//		The chart this plot belongs to.
			//	kwArgs: dojox.charting.plot2d.__DefaultCtorArgs?
			//		An optional arguments object to help define this plot.
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;

			// animation properties
			this.animate = this.opt.animate;
		},

		render: function(dim, offsets){
			//	summary:
			//		Render/draw everything on this plot.
			//	dim: Object
			//		An object of the form { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b }
			//	returns: dojox.charting.plot2d.Default
			//		A reference to this plot for functional chaining.

			// make sure all the series is not modified
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}

			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				this.group.setTransform(null);
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, stroke, outline, marker, events = this.events();

			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				if(!run.data.length){
					run.dirty = false;
					t.skip();
					continue;
				}

				var theme = t.next(this.opt.areas ? "area" : "line", [this.opt, run], true),
					s = run.group, rsegments = [], startindexes = [], rseg = null, lpoly,
					ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
					vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
					eventSeries = this._eventSeries[run.name] = new Array(run.data.length);

                // split the run data into dense segments (each containing no nulls)
                for(var j = 0; j < run.data.length; j++){
                    if(run.data[j] != null){
                        if(!rseg){
                            rseg = [];
                            startindexes.push(j);
                            rsegments.push(rseg)
                        }
                        rseg.push(run.data[j]);
                    }else{
                        rseg = null;
                    }
                }

                for(var seg = 0; seg < rsegments.length; seg++){
					if(typeof rsegments[seg][0] == "number"){
						lpoly = dojo.map(rsegments[seg], function(v, i){
							return {
								x: ht(i + startindexes[seg] + 1) + offsets.l,
								y: dim.height - offsets.b - vt(v)
							};
						}, this);
					}else{
						lpoly = dojo.map(rsegments[seg], function(v, i){
							return {
								x: ht(v.x) + offsets.l,
								y: dim.height - offsets.b - vt(v.y)
							};
						}, this);
					}

					var lpath = this.opt.tension ? dc.curve(lpoly, this.opt.tension) : "";

					if(this.opt.areas && lpoly.length > 1){
						var fill = theme.series.fill;
						var apoly = dojo.clone(lpoly);
						if(this.opt.tension){
							var apath = "L" + apoly[apoly.length-1].x + "," + (dim.height - offsets.b) +
								" L" + apoly[0].x + "," + (dim.height - offsets.b) +
								" L" + apoly[0].x + "," + apoly[0].y;
							run.dyn.fill = s.createPath(lpath + " " + apath).setFill(fill).getFill();
						} else {
							apoly.push({x: lpoly[lpoly.length - 1].x, y: dim.height - offsets.b});
							apoly.push({x: lpoly[0].x, y: dim.height - offsets.b});
							apoly.push(lpoly[0]);
							run.dyn.fill = s.createPolyline(apoly).setFill(fill).getFill();
						}
					}
					if(this.opt.lines || this.opt.markers){
						// need a stroke
						stroke = theme.series.stroke;
						if(theme.series.outline){
							outline = run.dyn.outline = dc.makeStroke(theme.series.outline);
							outline.width = 2 * outline.width + stroke.width;
						}
					}
					if(this.opt.markers){
						run.dyn.marker = theme.symbol;
					}
					var frontMarkers = null, outlineMarkers = null, shadowMarkers = null;
					if(stroke && theme.series.shadow && lpoly.length > 1){
						var shadow = theme.series.shadow,
							spoly = dojo.map(lpoly, function(c){
								return {x: c.x + shadow.dx, y: c.y + shadow.dy};
							});
						if(this.opt.lines){
							if(this.opt.tension){
								run.dyn.shadow = s.createPath(dc.curve(spoly, this.opt.tension)).setStroke(shadow).getStroke();
							} else {
								run.dyn.shadow = s.createPolyline(spoly).setStroke(shadow).getStroke();
							}
						}
						if(this.opt.markers && theme.marker.shadow){
							shadow = theme.marker.shadow;
							shadowMarkers = dojo.map(spoly, function(c){
								return s.createPath("M" + c.x + " " + c.y + " " + theme.symbol).
									setStroke(shadow).setFill(shadow.color);
							}, this);
						}
					}
					if(this.opt.lines && lpoly.length > 1){
						if(outline){
							if(this.opt.tension){
								run.dyn.outline = s.createPath(lpath).setStroke(outline).getStroke();
							} else {
								run.dyn.outline = s.createPolyline(lpoly).setStroke(outline).getStroke();
							}
						}
						if(this.opt.tension){
							run.dyn.stroke = s.createPath(lpath).setStroke(stroke).getStroke();
						} else {
							run.dyn.stroke = s.createPolyline(lpoly).setStroke(stroke).getStroke();
						}
					}
					if(this.opt.markers){
						frontMarkers = new Array(lpoly.length);
						outlineMarkers = new Array(lpoly.length);
						outline = null;
						if(theme.marker.outline){
							outline = dc.makeStroke(theme.marker.outline);
							outline.width = 2 * outline.width + (theme.marker.stroke ? theme.marker.stroke.width : 0);
						}
						dojo.forEach(lpoly, function(c, i){
							var path = "M" + c.x + " " + c.y + " " + theme.symbol;
							if(outline){
								outlineMarkers[i] = s.createPath(path).setStroke(outline);
							}
							frontMarkers[i] = s.createPath(path).setStroke(theme.marker.stroke).setFill(theme.marker.fill);
						}, this);
						if(events){
							dojo.forEach(frontMarkers, function(s, i){
								var o = {
									element: "marker",
									index:   i + startindexes[seg],
									run:     run,
									shape:   s,
									outline: outlineMarkers[i] || null,
									shadow:  shadowMarkers && shadowMarkers[i] || null,
									cx:      lpoly[i].x,
									cy:      lpoly[i].y
								};
								if(typeof rsegments[seg][0] == "number"){
									o.x = i + startindexes[seg] + 1;
									o.y = rsegments[seg][i];
								}else{
									o.x = rsegments[seg][i].x;
									o.y = rsegments[seg][i].y;
								}
								this._connectEvents(o);
								eventSeries[i + startindexes[seg]] = o;
							}, this);
						}else{
							delete this._eventSeries[run.name];
						}
					}
                }
				run.dirty = false;
			}
			if(this.animate){
				// grow from the bottom
				var plotGroup = this.group;
				dojox.gfx.fx.animateTransform(dojo.delegate({
					shape: plotGroup,
					duration: DEFAULT_ANIMATION_LENGTH,
					transform:[
						{name:"translate", start: [0, dim.height - offsets.b], end: [0, 0]},
						{name:"scale", start: [1, 0], end:[1, 1]},
						{name:"original"}
					]
				}, this.animate)).play();
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.Default
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.Lines"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Lines"] = true;
dojo.provide("dojox.charting.plot2d.Lines");



dojo.declare("dojox.charting.plot2d.Lines", dojox.charting.plot2d.Default, {
	//	summary:
	//		A convenience constructor to create a typical line chart.
	constructor: function(){
		//	summary:
		//		Preset our default plot to be line-based.
		this.opt.lines = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.Areas"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Areas"] = true;
dojo.provide("dojox.charting.plot2d.Areas");



dojo.declare("dojox.charting.plot2d.Areas", dojox.charting.plot2d.Default, {
	//	summary:
	//		Represents an area chart.  See dojox.charting.plot2d.Default for details.
	constructor: function(){
		this.opt.lines = true;
		this.opt.areas = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.Markers"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Markers"] = true;
dojo.provide("dojox.charting.plot2d.Markers");



dojo.declare("dojox.charting.plot2d.Markers", dojox.charting.plot2d.Default, {
	//	summary:
	//		A convenience plot to draw a line chart with markers.
	constructor: function(){
		//	summary:
		//		Set up the plot for lines and markers.
		this.opt.markers = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.MarkersOnly"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.MarkersOnly"] = true;
dojo.provide("dojox.charting.plot2d.MarkersOnly");



dojo.declare("dojox.charting.plot2d.MarkersOnly", dojox.charting.plot2d.Default, {
	//	summary:
	//		A convenience object to draw only markers (like a scatter but not quite).
	constructor: function(){
		//	summary:
		//		Set up our default plot to only have markers and no lines.
		this.opt.lines   = false;
		this.opt.markers = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.Scatter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Scatter"] = true;
dojo.provide("dojox.charting.plot2d.Scatter");











(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.Scatter", dojox.charting.plot2d.Base, {
		//	summary:
		//		A plot object representing a typical scatter chart.
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			shadows: null,	// draw shadows
			animate: null	// animate chart to place
		},
		optionalParams: {
			// theme component
			markerStroke:		{},
			markerOutline:		{},
			markerShadow:		{},
			markerFill:			{},
			markerFont:			"",
			markerFontColor:	""
		},

		constructor: function(chart, kwArgs){
			//	summary:
			//		Create the scatter plot.
			//	chart: dojox.charting.Chart2D
			//		The chart this plot belongs to.
			//	kwArgs: dojox.charting.plot2d.__DefaultCtorArgs?
			//		An optional keyword arguments object to help define this plot's parameters.
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
			this.animate = this.opt.animate;
		},

		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojox.charting.plot2d.Scatter
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, events = this.events();
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				if(!run.data.length){
					run.dirty = false;
					t.skip();
					continue;
				}

				var theme = t.next("marker", [this.opt, run]), s = run.group, lpoly,
					ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
					vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler);
				if(typeof run.data[0] == "number"){
					lpoly = dojo.map(run.data, function(v, i){
						return {
							x: ht(i + 1) + offsets.l,
							y: dim.height - offsets.b - vt(v)
						};
					}, this);
				}else{
					lpoly = dojo.map(run.data, function(v, i){
						return {
							x: ht(v.x) + offsets.l,
							y: dim.height - offsets.b - vt(v.y)
						};
					}, this);
				}

				var shadowMarkers  = new Array(lpoly.length),
					frontMarkers   = new Array(lpoly.length),
					outlineMarkers = new Array(lpoly.length);

				dojo.forEach(lpoly, function(c, i){
					var finalTheme = typeof run.data[i] == "number" ?
							t.post(theme, "marker") :
							t.addMixin(theme, "marker", run.data[i], true),
						path = "M" + c.x + " " + c.y + " " + finalTheme.symbol;
					if(finalTheme.marker.shadow){
						shadowMarkers[i] = s.createPath("M" + (c.x + finalTheme.marker.shadow.dx) + " " +
							(c.y + finalTheme.marker.shadow.dy) + " " + finalTheme.symbol).
							setStroke(finalTheme.marker.shadow).setFill(finalTheme.marker.shadow.color);
						if(this.animate){
							this._animateScatter(shadowMarkers[i], dim.height - offsets.b);
						}
					}
					if(finalTheme.marker.outline){
						var outline = dc.makeStroke(finalTheme.marker.outline);
						outline.width = 2 * outline.width + finalTheme.marker.stroke.width;
						outlineMarkers[i] = s.createPath(path).setStroke(outline);
						if(this.animate){
							this._animateScatter(outlineMarkers[i], dim.height - offsets.b);
						}
					}
					var stroke = dc.makeStroke(finalTheme.marker.stroke),
						fill = this._plotFill(finalTheme.marker.fill, dim, offsets);
					if(fill && (fill.type === "linear" || fill.type == "radial")){
						var color = dojox.gfx.gradutils.getColor(fill, {x: c.x, y: c.y});
						if(stroke){
							stroke.color = color;
						}
						frontMarkers[i] = s.createPath(path).setStroke(stroke).setFill(color);
					}else{
						frontMarkers[i] = s.createPath(path).setStroke(stroke).setFill(fill);
					}
					if(this.animate){
						this._animateScatter(frontMarkers[i], dim.height - offsets.b);
					}
				}, this);
				if(frontMarkers.length){
					run.dyn.stroke = frontMarkers[frontMarkers.length - 1].getStroke();
					run.dyn.fill   = frontMarkers[frontMarkers.length - 1].getFill();
				}

				if(events){
					var eventSeries = new Array(frontMarkers.length);
					dojo.forEach(frontMarkers, function(s, i){
						var o = {
							element: "marker",
							index:   i,
							run:     run,
							shape:   s,
							outline: outlineMarkers && outlineMarkers[i] || null,
							shadow:  shadowMarkers && shadowMarkers[i] || null,
							cx:      lpoly[i].x,
							cy:      lpoly[i].y
						};
						if(typeof run.data[0] == "number"){
							o.x = i + 1;
							o.y = run.data[i];
						}else{
							o.x = run.data[i].x;
							o.y = run.data[i].y;
						}
						this._connectEvents(o);
						eventSeries[i] = o;
					}, this);
					this._eventSeries[run.name] = eventSeries;
				}else{
					delete this._eventSeries[run.name];
				}
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.Scatter
		},
		_animateScatter: function(shape, offset){
			dojox.gfx.fx.animateTransform(dojo.delegate({
				shape: shape,
				duration: 1200,
				transform: [
					{name: "translate", start: [0, offset], end: [0, 0]},
					{name: "scale", start: [0, 0], end: [1, 1]},
					{name: "original"}
				]
			}, this.animate)).play();
		}
	});
})();

}

if(!dojo._hasResource["dojox.lang.functional.sequence"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.functional.sequence"] = true;
dojo.provide("dojox.lang.functional.sequence");



// This module adds high-level functions and related constructs:
//	- sequence generators

// If you want more general sequence builders check out listcomp.js and
// unfold() (in fold.js).

// Defined methods:
//	- take any valid lambda argument as the functional argument

(function(){
	var d = dojo, df = dojox.lang.functional;

	d.mixin(df, {
		// sequence generators
		repeat: function(/*Number*/ n, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: builds an array by repeatedly applying a unary function N times
			//	with a seed value Z. N should be greater than 0.
			o = o || d.global; f = df.lambda(f);
			var t = new Array(n), i = 1;
			t[0] = z;
			for(; i < n; t[i] = z = f.call(o, z), ++i);
			return t;	// Array
		},
		until: function(/*Function|String|Array*/ pr, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: builds an array by repeatedly applying a unary function with
			//	a seed value Z until the predicate is satisfied.
			o = o || d.global; f = df.lambda(f); pr = df.lambda(pr);
			var t = [];
			for(; !pr.call(o, z); t.push(z), z = f.call(o, z));
			return t;	// Array
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.Stacked"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Stacked"] = true;
dojo.provide("dojox.charting.plot2d.Stacked");








(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.Stacked", dojox.charting.plot2d.Default, {
		//	summary:
		//		Like the default plot, Stacked sets up lines, areas and markers
		//		in a stacked fashion (values on the y axis added to each other)
		//		as opposed to a direct one.
		getSeriesStats: function(){
			//	summary:
			//		Calculate the min/max on all attached series in both directions.
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			var stats = dc.collectStackedStats(this.series);
			this._maxRunLength = stats.hmax;
			return stats;
		},
		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojox.charting.plot2d.Stacked
			//		A reference to this plot for functional chaining.
			if(this._maxRunLength <= 0){
				return this;
			}

			// stack all values
			var acc = df.repeat(this._maxRunLength, "-> 0", 0);
			for(var i = 0; i < this.series.length; ++i){
				var run = this.series[i];
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j];
					if(v !== null){
						if(isNaN(v)){ v = 0; }
						acc[j] += v;
					}
				}
			}
			// draw runs in backwards
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}

			var t = this.chart.theme, events = this.events(),
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler);

			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next(this.opt.areas ? "area" : "line", [this.opt, run], true),
					s = run.group, outline,
					lpoly = dojo.map(acc, function(v, i){
						return {
							x: ht(i + 1) + offsets.l,
							y: dim.height - offsets.b - vt(v)
						};
					}, this);

				var lpath = this.opt.tension ? dc.curve(lpoly, this.opt.tension) : "";

				if(this.opt.areas){
					var apoly = dojo.clone(lpoly);
					if(this.opt.tension){
						var p=dc.curve(apoly, this.opt.tension);
						p += " L" + lpoly[lpoly.length - 1].x + "," + (dim.height - offsets.b) +
							" L" + lpoly[0].x + "," + (dim.height - offsets.b) +
							" L" + lpoly[0].x + "," + lpoly[0].y;
						run.dyn.fill = s.createPath(p).setFill(theme.series.fill).getFill();
					} else {
						apoly.push({x: lpoly[lpoly.length - 1].x, y: dim.height - offsets.b});
						apoly.push({x: lpoly[0].x, y: dim.height - offsets.b});
						apoly.push(lpoly[0]);
						run.dyn.fill = s.createPolyline(apoly).setFill(theme.series.fill).getFill();
					}
				}
				if(this.opt.lines || this.opt.markers){
					if(theme.series.outline){
						outline = dc.makeStroke(theme.series.outline);
						outline.width = 2 * outline.width + theme.series.stroke.width;
					}
				}
				if(this.opt.markers){
					run.dyn.marker = theme.symbol;
				}
				var frontMarkers, outlineMarkers, shadowMarkers;
				if(theme.series.shadow && theme.series.stroke){
					var shadow = theme.series.shadow,
						spoly = dojo.map(lpoly, function(c){
							return {x: c.x + shadow.dx, y: c.y + shadow.dy};
						});
					if(this.opt.lines){
						if(this.opt.tension){
							run.dyn.shadow = s.createPath(dc.curve(spoly, this.opt.tension)).setStroke(shadow).getStroke();
						} else {
							run.dyn.shadow = s.createPolyline(spoly).setStroke(shadow).getStroke();
						}
					}
					if(this.opt.markers){
						shadow = theme.marker.shadow;
						shadowMarkers = dojo.map(spoly, function(c){
							return s.createPath("M" + c.x + " " + c.y + " " + theme.symbol).
								setStroke(shadow).setFill(shadow.color);
						}, this);
					}
				}
				if(this.opt.lines){
					if(outline){
						if(this.opt.tension){
							run.dyn.outline = s.createPath(lpath).setStroke(outline).getStroke();
						} else {
							run.dyn.outline = s.createPolyline(lpoly).setStroke(outline).getStroke();
						}
					}
					if(this.opt.tension){
						run.dyn.stroke = s.createPath(lpath).setStroke(theme.series.stroke).getStroke();
					} else {
						run.dyn.stroke = s.createPolyline(lpoly).setStroke(theme.series.stroke).getStroke();
					}
				}
				if(this.opt.markers){
					frontMarkers = new Array(lpoly.length);
					outlineMarkers = new Array(lpoly.length);
					outline = null;
					if(theme.marker.outline){
						outline = dc.makeStroke(theme.marker.outline);
						outline.width = 2 * outline.width + (theme.marker.stroke ? theme.marker.stroke.width : 0);
					}
					dojo.forEach(lpoly, function(c, i){
						var path = "M" + c.x + " " + c.y + " " + theme.symbol;
						if(outline){
							outlineMarkers[i] = s.createPath(path).setStroke(outline);
						}
						frontMarkers[i] = s.createPath(path).setStroke(theme.marker.stroke).setFill(theme.marker.fill);
					}, this);
					if(events){
						var eventSeries = new Array(frontMarkers.length);
						dojo.forEach(frontMarkers, function(s, i){
							var o = {
								element: "marker",
								index:   i,
								run:     run,
								shape:   s,
								outline: outlineMarkers[i] || null,
								shadow:  shadowMarkers && shadowMarkers[i] || null,
								cx:      lpoly[i].x,
								cy:      lpoly[i].y,
								x:       i + 1,
								y:       run.data[i]
							};
							this._connectEvents(o);
							eventSeries[i] = o;
						}, this);
						this._eventSeries[run.name] = eventSeries;
					}else{
						delete this._eventSeries[run.name];
					}
				}
				run.dirty = false;
				// update the accumulator
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j];
					if(v !== null){
						if(isNaN(v)){ v = 0; }
						acc[j] -= v;
					}
				}
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.Stacked
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.StackedLines"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.StackedLines"] = true;
dojo.provide("dojox.charting.plot2d.StackedLines");



dojo.declare("dojox.charting.plot2d.StackedLines", dojox.charting.plot2d.Stacked, {
	//	summary:
	//		A convenience object to create a stacked line chart.
	constructor: function(){
		//	summary:
		//		Force our Stacked base to be lines only.
		this.opt.lines = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.StackedAreas"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.StackedAreas"] = true;
dojo.provide("dojox.charting.plot2d.StackedAreas");



dojo.declare("dojox.charting.plot2d.StackedAreas", dojox.charting.plot2d.Stacked, {
	//	summary:
	//		A convenience object to set up a stacked area plot.
	constructor: function(){
		//	summary:
		//		Force our Stacked plotter to include both lines and areas.
		this.opt.lines = true;
		this.opt.areas = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.Columns"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Columns"] = true;
dojo.provide("dojox.charting.plot2d.Columns");









(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.Columns", dojox.charting.plot2d.Base, {
		//	summary:
		//		The plot object representing a column chart (vertical bars).
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			gap:	0,		// gap between columns in pixels
			animate: null   // animate bars into place
		},
		optionalParams: {
			minBarSize:	1,	// minimal column width in pixels
			maxBarSize:	1,	// maximal column width in pixels
			// theme component
			stroke:		{},
			outline:	{},
			shadow:		{},
			fill:		{},
			font:		"",
			fontColor:	""
		},

		constructor: function(chart, kwArgs){
			//	summary:
			//		The constructor for a columns chart.
			//	chart: dojox.charting.Chart2D
			//		The chart this plot belongs to.
			//	kwArgs: dojox.charting.plot2d.__BarCtorArgs?
			//		An optional keyword arguments object to help define the plot.
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
			this.animate = this.opt.animate;
		},

		getSeriesStats: function(){
			//	summary:
			//		Calculate the min/max on all attached series in both directions.
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			var stats = dc.collectSimpleStats(this.series);
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			return stats;
		},

		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojox.charting.plot2d.Columns
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, f, gap, width,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				baseline = Math.max(0, this._vScaler.bounds.lower),
				baselineHeight = vt(baseline),
				events = this.events();
			f = dc.calculateBarSize(this._hScaler.bounds.scale, this.opt);
			gap = f.gap;
			width = f.size;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("column", [this.opt, run]), s = run.group,
					eventSeries = new Array(run.data.length);
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y,
							vv = vt(v),
							height = vv - baselineHeight,
							h = Math.abs(height),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "column", value, true) :
								t.post(theme, "column");
						if(width >= 1 && h >= 1){
							var rect = {
								x: offsets.l + ht(j + 0.5) + gap,
								y: dim.height - offsets.b - (v > baseline ? vv : baselineHeight),
								width: width, height: h
							};
							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							specialFill = this._shapeFill(specialFill, rect);
							var shape = s.createRect(rect).setFill(specialFill).setStroke(finalTheme.series.stroke);
							run.dyn.fill   = shape.getFill();
							run.dyn.stroke = shape.getStroke();
							if(events){
								var o = {
									element: "column",
									index:   j,
									run:     run,
									shape:   shape,
									x:       j + 0.5,
									y:       v
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}
							if(this.animate){
								this._animateColumn(shape, dim.height - offsets.b - baselineHeight, h);
							}
						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.Columns
		},
		_animateColumn: function(shape, voffset, vsize){
			dojox.gfx.fx.animateTransform(dojo.delegate({
				shape: shape,
				duration: 1200,
				transform: [
					{name: "translate", start: [0, voffset - (voffset/vsize)], end: [0, 0]},
					{name: "scale", start: [1, 1/vsize], end: [1, 1]},
					{name: "original"}
				]
			}, this.animate)).play();
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.StackedColumns"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.StackedColumns"] = true;
dojo.provide("dojox.charting.plot2d.StackedColumns");







(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.StackedColumns", dojox.charting.plot2d.Columns, {
		//	summary:
		//		The plot object representing a stacked column chart (vertical bars).
		getSeriesStats: function(){
			//	summary:
			//		Calculate the min/max on all attached series in both directions.
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			var stats = dc.collectStackedStats(this.series);
			this._maxRunLength = stats.hmax;
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			return stats;
		},
		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojox.charting.plot2d.StackedColumns
			//		A reference to this plot for functional chaining.
			if(this._maxRunLength <= 0){
				return this;
			}

			// stack all values
			var acc = df.repeat(this._maxRunLength, "-> 0", 0);
			for(var i = 0; i < this.series.length; ++i){
				var run = this.series[i];
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y;
						if(isNaN(v)){ v = 0; }
						acc[j] += v;
					}
				}
			}
			// draw runs in backwards
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, f, gap, width,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				events = this.events();
			f = dc.calculateBarSize(this._hScaler.bounds.scale, this.opt);
			gap = f.gap;
			width = f.size;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("column", [this.opt, run]), s = run.group,
					eventSeries = new Array(acc.length);
				for(var j = 0; j < acc.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = acc[j],
							height = vt(v),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "column", value, true) :
								t.post(theme, "column");
						if(width >= 1 && height >= 1){
							var rect = {
								x: offsets.l + ht(j + 0.5) + gap,
								y: dim.height - offsets.b - vt(v),
								width: width, height: height
							};
							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							specialFill = this._shapeFill(specialFill, rect);
							var shape = s.createRect(rect).setFill(specialFill).setStroke(finalTheme.series.stroke);
							run.dyn.fill   = shape.getFill();
							run.dyn.stroke = shape.getStroke();
							if(events){
								var o = {
									element: "column",
									index:   j,
									run:     run,
									shape:   shape,
									x:       j + 0.5,
									y:       v
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}
							if(this.animate){
								this._animateColumn(shape, dim.height - offsets.b, height);
							}
						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
				// update the accumulator
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y;
						if(isNaN(v)){ v = 0; }
						acc[j] -= v;
					}
				}
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.StackedColumns
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.ClusteredColumns"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.ClusteredColumns"] = true;
dojo.provide("dojox.charting.plot2d.ClusteredColumns");







(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.ClusteredColumns", dojox.charting.plot2d.Columns, {
		//	summary:
		//		A plot representing grouped or clustered columns (vertical bars).
		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojox.charting.plot2d.ClusteredColumns
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, f, gap, width, thickness,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				baseline = Math.max(0, this._vScaler.bounds.lower),
				baselineHeight = vt(baseline),
				events = this.events();
			f = dc.calculateBarSize(this._hScaler.bounds.scale, this.opt, this.series.length);
			gap = f.gap;
			width = thickness = f.size;
			for(var i = 0; i < this.series.length; ++i){
				var run = this.series[i], shift = thickness * i;
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("column", [this.opt, run]), s = run.group,
					eventSeries = new Array(run.data.length);
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y,
							vv = vt(v),
							height = vv - baselineHeight,
							h = Math.abs(height),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "column", value, true) :
								t.post(theme, "column");
						if(width >= 1 && h >= 1){
							var rect = {
								x: offsets.l + ht(j + 0.5) + gap + shift,
								y: dim.height - offsets.b - (v > baseline ? vv : baselineHeight),
								width: width, height: h
							};
							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							specialFill = this._shapeFill(specialFill, rect);
							var shape = s.createRect(rect).setFill(specialFill).setStroke(finalTheme.series.stroke);
							run.dyn.fill   = shape.getFill();
							run.dyn.stroke = shape.getStroke();
							if(events){
								var o = {
									element: "column",
									index:   j,
									run:     run,
									shape:   shape,
									x:       j + 0.5,
									y:       v
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}
							if(this.animate){
								this._animateColumn(shape, dim.height - offsets.b - baselineHeight, h);
							}
						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.ClusteredColumns
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.Bars"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Bars"] = true;
dojo.provide("dojox.charting.plot2d.Bars");








/*=====
dojo.declare("dojox.charting.plot2d.__BarCtorArgs", dojox.charting.plot2d.__DefaultCtorArgs, {
	//	summary:
	//		Additional keyword arguments for bar charts.

	//	minBarSize: Number?
	//		The minimum size for a bar in pixels.  Default is 1.
	minBarSize: 1,

	//	maxBarSize: Number?
	//		The maximum size for a bar in pixels.  Default is 1.
	maxBarSize: 1
});
=====*/
(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.Bars", dojox.charting.plot2d.Base, {
		//	summary:
		//		The plot object representing a bar chart (horizontal bars).
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			gap:	0,		// gap between columns in pixels
			animate: null   // animate bars into place
		},
		optionalParams: {
			minBarSize:	1,	// minimal bar width in pixels
			maxBarSize:	1,	// maximal bar width in pixels
			// theme component
			stroke:		{},
			outline:	{},
			shadow:		{},
			fill:		{},
			font:		"",
			fontColor:	""
		},

		constructor: function(chart, kwArgs){
			//	summary:
			//		The constructor for a bar chart.
			//	chart: dojox.charting.Chart2D
			//		The chart this plot belongs to.
			//	kwArgs: dojox.charting.plot2d.__BarCtorArgs?
			//		An optional keyword arguments object to help define the plot.
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
			this.animate = this.opt.animate;
		},

		getSeriesStats: function(){
			//	summary:
			//		Calculate the min/max on all attached series in both directions.
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			var stats = dc.collectSimpleStats(this.series), t;
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			t = stats.hmin, stats.hmin = stats.vmin, stats.vmin = t;
			t = stats.hmax, stats.hmax = stats.vmax, stats.vmax = t;
			return stats;
		},

		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojox.charting.plot2d.Bars
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.dirty = this.isDirty();
			this.resetEvents();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, f, gap, height,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				baseline = Math.max(0, this._hScaler.bounds.lower),
				baselineWidth = ht(baseline),
				events = this.events();
			f = dc.calculateBarSize(this._vScaler.bounds.scale, this.opt);
			gap = f.gap;
			height = f.size;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("bar", [this.opt, run]), s = run.group,
					eventSeries = new Array(run.data.length);
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y,
							hv = ht(v),
							width = hv - baselineWidth,
							w = Math.abs(width),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "bar", value, true) :
								t.post(theme, "bar");
						if(w >= 1 && height >= 1){
							var rect = {
								x: offsets.l + (v < baseline ? hv : baselineWidth),
								y: dim.height - offsets.b - vt(j + 1.5) + gap,
								width: w, height: height
							};
							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							specialFill = this._shapeFill(specialFill, rect);
							var shape = s.createRect(rect).setFill(specialFill).setStroke(finalTheme.series.stroke);
							run.dyn.fill   = shape.getFill();
							run.dyn.stroke = shape.getStroke();
							if(events){
								var o = {
									element: "bar",
									index:   j,
									run:     run,
									shape:   shape,
									x:       v,
									y:       j + 1.5
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}
							if(this.animate){
								this._animateBar(shape, offsets.l + baselineWidth, -w);
							}
						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.Bars
		},
		_animateBar: function(shape, hoffset, hsize){
			dojox.gfx.fx.animateTransform(dojo.delegate({
				shape: shape,
				duration: 1200,
				transform: [
					{name: "translate", start: [hoffset - (hoffset/hsize), 0], end: [0, 0]},
					{name: "scale", start: [1/hsize, 1], end: [1, 1]},
					{name: "original"}
				]
			}, this.animate)).play();
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.StackedBars"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.StackedBars"] = true;
dojo.provide("dojox.charting.plot2d.StackedBars");







(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.StackedBars", dojox.charting.plot2d.Bars, {
		//	summary:
		//		The plot object representing a stacked bar chart (horizontal bars).
		getSeriesStats: function(){
			//	summary:
			//		Calculate the min/max on all attached series in both directions.
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			var stats = dc.collectStackedStats(this.series), t;
			this._maxRunLength = stats.hmax;
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			t = stats.hmin, stats.hmin = stats.vmin, stats.vmin = t;
			t = stats.hmax, stats.hmax = stats.vmax, stats.vmax = t;
			return stats;
		},
		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojox.charting.plot2d.StackedBars
			//		A reference to this plot for functional chaining.
			if(this._maxRunLength <= 0){
				return this;
			}

			// stack all values
			var acc = df.repeat(this._maxRunLength, "-> 0", 0);
			for(var i = 0; i < this.series.length; ++i){
				var run = this.series[i];
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y;
						if(isNaN(v)){ v = 0; }
						acc[j] += v;
					}
				}
			}
			// draw runs in backwards
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, f, gap, height,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				events = this.events();
			f = dc.calculateBarSize(this._vScaler.bounds.scale, this.opt);
			gap = f.gap;
			height = f.size;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("bar", [this.opt, run]), s = run.group,
					eventSeries = new Array(acc.length);
				for(var j = 0; j < acc.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = acc[j],
							width = ht(v),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "bar", value, true) :
								t.post(theme, "bar");
						if(width >= 1 && height >= 1){
							var rect = {
								x: offsets.l,
								y: dim.height - offsets.b - vt(j + 1.5) + gap,
								width: width, height: height
							};
							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							specialFill = this._shapeFill(specialFill, rect);
							var shape = s.createRect(rect).setFill(specialFill).setStroke(finalTheme.series.stroke);
							run.dyn.fill   = shape.getFill();
							run.dyn.stroke = shape.getStroke();
							if(events){
								var o = {
									element: "bar",
									index:   j,
									run:     run,
									shape:   shape,
									x:       v,
									y:       j + 1.5
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}
							if(this.animate){
								this._animateBar(shape, offsets.l, -width);
							}
						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
				// update the accumulator
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y;
						if(isNaN(v)){ v = 0; }
						acc[j] -= v;
					}
				}
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.StackedBars
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.ClusteredBars"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.ClusteredBars"] = true;
dojo.provide("dojox.charting.plot2d.ClusteredBars");







(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.ClusteredBars", dojox.charting.plot2d.Bars, {
		//	summary:
		//		A plot representing grouped or clustered bars (horizontal bars)
		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojox.charting.plot2d.ClusteredBars
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, f, gap, height, thickness,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				baseline = Math.max(0, this._hScaler.bounds.lower),
				baselineWidth = ht(baseline),
				events = this.events();
			f = dc.calculateBarSize(this._vScaler.bounds.scale, this.opt, this.series.length);
			gap = f.gap;
			height = thickness = f.size;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i], shift = thickness * (this.series.length - i - 1);
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("bar", [this.opt, run]), s = run.group,
					eventSeries = new Array(run.data.length);
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y,
							hv = ht(v),
							width = hv - baselineWidth,
							w = Math.abs(width),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "bar", value, true) :
								t.post(theme, "bar");
						if(w >= 1 && height >= 1){
							var rect = {
								x: offsets.l + (v < baseline ? hv : baselineWidth),
								y: dim.height - offsets.b - vt(j + 1.5) + gap + shift,
								width: w, height: height
							};
							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							specialFill = this._shapeFill(specialFill, rect);
							var shape = s.createRect(rect).setFill(specialFill).setStroke(finalTheme.series.stroke);
							run.dyn.fill   = shape.getFill();
							run.dyn.stroke = shape.getStroke();
							if(events){
								var o = {
									element: "bar",
									index:   j,
									run:     run,
									shape:   shape,
									x:       v,
									y:       j + 1.5
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}
							if(this.animate){
								this._animateBar(shape, offsets.l + baselineWidth, -width);
							}
						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.ClusteredBars
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.Grid"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Grid"] = true;
dojo.provide("dojox.charting.plot2d.Grid");






/*=====
dojo.declare("dojox.charting.plot2d.__GridCtorArgs", dojox.charting.plot2d.__DefaultCtorArgs, {
	//	summary:
	//		A special keyword arguments object that is specific to a grid "plot".

	//	hMajorLines: Boolean?
	//		Whether to show lines at the major ticks along the horizontal axis. Default is true.
	hMajorLines: true,

	//	hMinorLines: Boolean?
	//		Whether to show lines at the minor ticks along the horizontal axis. Default is false.
	hMinorLines: false,

	//	vMajorLines: Boolean?
	//		Whether to show lines at the major ticks along the vertical axis. Default is true.
	vMajorLines: true,

	//	vMinorLines: Boolean?
	//		Whether to show lines at the major ticks along the vertical axis. Default is false.
	vMinorLines: false,

	//	hStripes: String?
	//		Whether or not to show stripes (alternating fills) along the horizontal axis. Default is "none".
	hStripes: "none",

	//	vStripes: String?
	//		Whether or not to show stripes (alternating fills) along the vertical axis. Default is "none".
	vStripes: "none"
});
=====*/
(function(){
	var du = dojox.lang.utils, dc = dojox.charting.plot2d.common;

	dojo.declare("dojox.charting.plot2d.Grid", dojox.charting.Element, {
		//	summary:
		//		A "faux" plot that can be placed behind other plots to represent
		//		a grid against which other plots can be easily measured.
		defaultParams: {
			hAxis: "x",			// use a horizontal axis named "x"
			vAxis: "y",			// use a vertical axis named "y"
			hMajorLines: true,	// draw horizontal major lines
			hMinorLines: false,	// draw horizontal minor lines
			vMajorLines: true,	// draw vertical major lines
			vMinorLines: false,	// draw vertical minor lines
			hStripes: "none",	// TBD
			vStripes: "none",	// TBD
			animate: null   // animate bars into place
		},
		optionalParams: {},	// no optional parameters

		constructor: function(chart, kwArgs){
			//	summary:
			//		Create the faux Grid plot.
			//	chart: dojox.charting.Chart2D
			//		The chart this plot belongs to.
			//	kwArgs: dojox.charting.plot2d.__GridCtorArgs?
			//		An optional keyword arguments object to help define the parameters of the underlying grid.
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
			this.dirty = true;
			this.animate = this.opt.animate;
			this.zoom = null,
			this.zoomQueue = [];	// zooming action task queue
			this.lastWindow = {vscale: 1, hscale: 1, xoffset: 0, yoffset: 0};
		},
		clear: function(){
			//	summary:
			//		Clear out any parameters set on this plot.
			//	returns: dojox.charting.plot2d.Grid
			//		The reference to this plot for functional chaining.
			this._hAxis = null;
			this._vAxis = null;
			this.dirty = true;
			return this;	//	dojox.charting.plot2d.Grid
		},
		setAxis: function(axis){
			//	summary:
			//		Set an axis for this plot.
			//	returns: dojox.charting.plot2d.Grid
			//		The reference to this plot for functional chaining.
			if(axis){
				this[axis.vertical ? "_vAxis" : "_hAxis"] = axis;
			}
			return this;	//	dojox.charting.plot2d.Grid
		},
		addSeries: function(run){
			//	summary:
			//		Ignored but included as a dummy method.
			//	returns: dojox.charting.plot2d.Grid
			//		The reference to this plot for functional chaining.
			return this;	//	dojox.charting.plot2d.Grid
		},
		getSeriesStats: function(){
			//	summary:
			//		Returns default stats (irrelevant for this type of plot).
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			return dojo.delegate(dc.defaultStats);
		},
		initializeScalers: function(){
			//	summary:
			//		Does nothing (irrelevant for this type of plot).
			return this;
		},
		isDirty: function(){
			//	summary:
			//		Return whether or not this plot needs to be redrawn.
			//	returns: Boolean
			//		If this plot needs to be rendered, this will return true.
			return this.dirty || this._hAxis && this._hAxis.dirty || this._vAxis && this._vAxis.dirty;	//	Boolean
		},
		performZoom: function(dim, offsets){
			//	summary:
			//		Create/alter any zooming windows on this plot.
			//	dim: Object
			//		An object of the form { width, height }.
			//	offsets: Object
			//		An object of the form { l, r, t, b }.
			//	returns: dojox.charting.plot2d.Grid
			//		A reference to this plot for functional chaining.

			// get current zooming various
			var vs = this._vAxis.scale || 1,
				hs = this._hAxis.scale || 1,
				vOffset = dim.height - offsets.b,
				hBounds = this._hAxis.getScaler().bounds,
				xOffset = (hBounds.from - hBounds.lower) * hBounds.scale,
				vBounds = this._vAxis.getScaler().bounds,
				yOffset = (vBounds.from - vBounds.lower) * vBounds.scale;
				// get incremental zooming various
				rVScale = vs / this.lastWindow.vscale,
				rHScale = hs / this.lastWindow.hscale,
				rXOffset = (this.lastWindow.xoffset - xOffset)/
					((this.lastWindow.hscale == 1)? hs : this.lastWindow.hscale),
				rYOffset = (yOffset - this.lastWindow.yoffset)/
					((this.lastWindow.vscale == 1)? vs : this.lastWindow.vscale),

				shape = this.group,
				anim = dojox.gfx.fx.animateTransform(dojo.delegate({
					shape: shape,
					duration: 1200,
					transform:[
						{name:"translate", start:[0, 0], end: [offsets.l * (1 - rHScale), vOffset * (1 - rVScale)]},
						{name:"scale", start:[1, 1], end: [rHScale, rVScale]},
						{name:"original"},
						{name:"translate", start: [0, 0], end: [rXOffset, rYOffset]}
					]}, this.zoom));

			dojo.mixin(this.lastWindow, {vscale: vs, hscale: hs, xoffset: xOffset, yoffset: yOffset});
			//add anim to zooming action queue,
			//in order to avoid several zooming action happened at the same time
			this.zoomQueue.push(anim);
			//perform each anim one by one in zoomQueue
			dojo.connect(anim, "onEnd", this, function(){
				this.zoom = null;
				this.zoomQueue.shift();
				if(this.zoomQueue.length > 0){
					this.zoomQueue[0].play();
				}
			});
			if(this.zoomQueue.length == 1){
				this.zoomQueue[0].play();
			}
			return this;	//	dojox.charting.plot2d.Grid
		},
		getRequiredColors: function(){
			//	summary:
			//		Ignored but included as a dummy method.
			//	returns: Number
			//		Returns 0, since there are no series associated with this plot type.
			return 0;	//	Number
		},
		render: function(dim, offsets){
			//	summary:
			//		Render the plot on the chart.
			//	dim: Object
			//		An object of the form { width, height }.
			//	offsets: Object
			//		An object of the form { l, r, t, b }.
			//	returns: dojox.charting.plot2d.Grid
			//		A reference to this plot for functional chaining.
			if(this.zoom){
				return this.performZoom(dim, offsets);
			}
			this.dirty = this.isDirty();
			if(!this.dirty){ return this; }
			this.cleanGroup();
			var s = this.group, ta = this.chart.theme.axis;
			// draw horizontal stripes and lines
			try{
				var vScaler = this._vAxis.getScaler(),
					vt = vScaler.scaler.getTransformerFromModel(vScaler),
					ticks = this._vAxis.getTicks();
				if(this.opt.hMinorLines){
					dojo.forEach(ticks.minor, function(tick){
						var y = dim.height - offsets.b - vt(tick.value);
						var hMinorLine = s.createLine({
							x1: offsets.l,
							y1: y,
							x2: dim.width - offsets.r,
							y2: y
						}).setStroke(ta.minorTick);
						if(this.animate){
							this._animateGrid(hMinorLine, "h", offsets.l, offsets.r + offsets.l - dim.width);
						}
					}, this);
				}
				if(this.opt.hMajorLines){
					dojo.forEach(ticks.major, function(tick){
						var y = dim.height - offsets.b - vt(tick.value);
						var hMajorLine = s.createLine({
							x1: offsets.l,
							y1: y,
							x2: dim.width - offsets.r,
							y2: y
						}).setStroke(ta.majorTick);
						if(this.animate){
							this._animateGrid(hMajorLine, "h", offsets.l, offsets.r + offsets.l - dim.width);
						}
					}, this);
				}
			}catch(e){
				// squelch
			}
			// draw vertical stripes and lines
			try{
				var hScaler = this._hAxis.getScaler(),
					ht = hScaler.scaler.getTransformerFromModel(hScaler),
					ticks = this._hAxis.getTicks();
				if(ticks && this.opt.vMinorLines){
					dojo.forEach(ticks.minor, function(tick){
						var x = offsets.l + ht(tick.value);
						var vMinorLine = s.createLine({
							x1: x,
							y1: offsets.t,
							x2: x,
							y2: dim.height - offsets.b
						}).setStroke(ta.minorTick);
						if(this.animate){
							this._animateGrid(vMinorLine, "v", dim.height - offsets.b, dim.height - offsets.b - offsets.t);
						}
					}, this);
				}
				if(ticks && this.opt.vMajorLines){
					dojo.forEach(ticks.major, function(tick){
						var x = offsets.l + ht(tick.value);
						var vMajorLine = s.createLine({
							x1: x,
							y1: offsets.t,
							x2: x,
							y2: dim.height - offsets.b
						}).setStroke(ta.majorTick);
						if(this.animate){
							this._animateGrid(vMajorLine, "v", dim.height - offsets.b, dim.height - offsets.b - offsets.t);
						}
					}, this);
				}
			}catch(e){
				// squelch
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.Grid
		},
		_animateGrid: function(shape, type, offset, size){
			var transStart = type == "h" ? [offset, 0] : [0, offset];
			var scaleStart = type == "h" ? [1/size, 1] : [1, 1/size];
			dojox.gfx.fx.animateTransform(dojo.delegate({
				shape: shape,
				duration: 1200,
				transform: [
					{name: "translate", start: transStart, end: [0, 0]},
					{name: "scale", start: scaleStart, end: [1, 1]},
					{name: "original"}
				]
			}, this.animate)).play();
		}
	});
})();

}

if(!dojo._hasResource["dojo.number"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.number"] = true;
dojo.provide("dojo.number");







/*=====
dojo.number = {
	// summary: localized formatting and parsing routines for Number
}

dojo.number.__FormatOptions = function(){
	//	pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.  Literal characters in patterns are not supported.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	//	places: Number?
	//		fixed number of decimal places to show.  This overrides any
	//		information in the provided pattern.
	//	round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means do not round.
	//	locale: String?
	//		override the locale used to determine formatting rules
	//	fractional: Boolean?
	//		If false, show no decimal places, overriding places and pattern settings.
	this.pattern = pattern;
	this.type = type;
	this.places = places;
	this.round = round;
	this.locale = locale;
	this.fractional = fractional;
}
=====*/

dojo.number.format = function(/*Number*/value, /*dojo.number.__FormatOptions?*/options){
	// summary:
	//		Format a Number as a String, using locale-specific settings
	// description:
	//		Create a string from a Number using a known localized pattern.
	//		Formatting patterns appropriate to the locale are chosen from the
	//		[Common Locale Data Repository](http://unicode.org/cldr) as well as the appropriate symbols and
	//		delimiters.
	//		If value is Infinity, -Infinity, or is not a valid JavaScript number, return null.
	// value:
	//		the number to be formatted

	options = dojo.mixin({}, options || {});
	var locale = dojo.i18n.normalizeLocale(options.locale),
		bundle = dojo.i18n.getLocalization("dojo.cldr", "number", locale);
	options.customs = bundle;
	var pattern = options.pattern || bundle[(options.type || "decimal") + "Format"];
	if(isNaN(value) || Math.abs(value) == Infinity){ return null; } // null
	return dojo.number._applyPattern(value, pattern, options); // String
};

//dojo.number._numberPatternRE = /(?:[#0]*,?)*[#0](?:\.0*#*)?/; // not precise, but good enough
dojo.number._numberPatternRE = /[#0,]*[#0](?:\.0*#*)?/; // not precise, but good enough

dojo.number._applyPattern = function(/*Number*/value, /*String*/pattern, /*dojo.number.__FormatOptions?*/options){
	// summary:
	//		Apply pattern to format value as a string using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted.
	// pattern:
	//		a pattern string as described by
	//		[unicode.org TR35](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	// options: dojo.number.__FormatOptions?
	//		_applyPattern is usually called via `dojo.number.format()` which
	//		populates an extra property in the options parameter, "customs".
	//		The customs object specifies group and decimal parameters if set.

	//TODO: support escapes
	options = options || {};
	var group = options.customs.group,
		decimal = options.customs.decimal,
		patternList = pattern.split(';'),
		positivePattern = patternList[0];
	pattern = patternList[(value < 0) ? 1 : 0] || ("-" + positivePattern);

	//TODO: only test against unescaped
	if(pattern.indexOf('%') != -1){
		value *= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		value *= 1000; // per mille
	}else if(pattern.indexOf('\u00a4') != -1){
		group = options.customs.currencyGroup || group;//mixins instead?
		decimal = options.customs.currencyDecimal || decimal;// Should these be mixins instead?
		pattern = pattern.replace(/\u00a4{1,3}/, function(match){
			var prop = ["symbol", "currency", "displayName"][match.length-1];
			return options[prop] || options.currency || "";
		});
	}else if(pattern.indexOf('E') != -1){
		throw new Error("exponential notation not supported");
	}
	
	//TODO: support @ sig figs?
	var numberPatternRE = dojo.number._numberPatternRE;
	var numberPattern = positivePattern.match(numberPatternRE);
	if(!numberPattern){
		throw new Error("unable to find a number expression in pattern: "+pattern);
	}
	if(options.fractional === false){ options.places = 0; }
	return pattern.replace(numberPatternRE,
		dojo.number._formatAbsolute(value, numberPattern[0], {decimal: decimal, group: group, places: options.places, round: options.round}));
}

dojo.number.round = function(/*Number*/value, /*Number?*/places, /*Number?*/increment){
	//	summary:
	//		Rounds to the nearest value with the given number of decimal places, away from zero
	//	description:
	//		Rounds to the nearest value with the given number of decimal places, away from zero if equal.
	//		Similar to Number.toFixed(), but compensates for browser quirks. Rounding can be done by
	//		fractional increments also, such as the nearest quarter.
	//		NOTE: Subject to floating point errors.  See dojox.math.round for experimental workaround.
	//	value:
	//		The number to round
	//	places:
	//		The number of decimal places where rounding takes place.  Defaults to 0 for whole rounding.
	//		Must be non-negative.
	//	increment:
	//		Rounds next place to nearest value of increment/10.  10 by default.
	//	example:
	//		>>> dojo.number.round(-0.5)
	//		-1
	//		>>> dojo.number.round(162.295, 2)
	//		162.29  // note floating point error.  Should be 162.3
	//		>>> dojo.number.round(10.71, 0, 2.5)
	//		10.75
	var factor = 10 / (increment || 10);
	return (factor * +value).toFixed(places) / factor; // Number
}

if((0.9).toFixed() == 0){
	// (isIE) toFixed() bug workaround: Rounding fails on IE when most significant digit
	// is just after the rounding place and is >=5
	(function(){
		var round = dojo.number.round;
		dojo.number.round = function(v, p, m){
			var d = Math.pow(10, -p || 0), a = Math.abs(v);
			if(!v || a >= d || a * Math.pow(10, p + 1) < 5){
				d = 0;
			}
			return round(v, p, m) + (v > 0 ? d : -d);
		}
	})();
}

/*=====
dojo.number.__FormatAbsoluteOptions = function(){
	//	decimal: String?
	//		the decimal separator
	//	group: String?
	//		the group separator
	//	places: Number?|String?
	//		number of decimal places.  the range "n,m" will format to m places.
	//	round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means don't round.
	this.decimal = decimal;
	this.group = group;
	this.places = places;
	this.round = round;
}
=====*/

dojo.number._formatAbsolute = function(/*Number*/value, /*String*/pattern, /*dojo.number.__FormatAbsoluteOptions?*/options){
	// summary: 
	//		Apply numeric pattern to absolute value using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted, ignores sign
	// pattern:
	//		the number portion of a pattern (e.g. `#,##0.00`)
	options = options || {};
	if(options.places === true){options.places=0;}
	if(options.places === Infinity){options.places=6;} // avoid a loop; pick a limit

	var patternParts = pattern.split("."),
		comma = typeof options.places == "string" && options.places.indexOf(","),
		maxPlaces = options.places;
	if(comma){
		maxPlaces = options.places.substring(comma + 1);
	}else if(!(maxPlaces >= 0)){
		maxPlaces = (patternParts[1] || []).length;
	}
	if(!(options.round < 0)){
		value = dojo.number.round(value, maxPlaces, options.round);
	}

	var valueParts = String(Math.abs(value)).split("."),
		fractional = valueParts[1] || "";
	if(patternParts[1] || options.places){
		if(comma){
			options.places = options.places.substring(0, comma);
		}
		// Pad fractional with trailing zeros
		var pad = options.places !== undefined ? options.places : (patternParts[1] && patternParts[1].lastIndexOf("0") + 1);
		if(pad > fractional.length){
			valueParts[1] = dojo.string.pad(fractional, pad, '0', true);
		}

		// Truncate fractional
		if(maxPlaces < fractional.length){
			valueParts[1] = fractional.substr(0, maxPlaces);
		}
	}else{
		if(valueParts[1]){ valueParts.pop(); }
	}

	// Pad whole with leading zeros
	var patternDigits = patternParts[0].replace(',', '');
	pad = patternDigits.indexOf("0");
	if(pad != -1){
		pad = patternDigits.length - pad;
		if(pad > valueParts[0].length){
			valueParts[0] = dojo.string.pad(valueParts[0], pad);
		}

		// Truncate whole
		if(patternDigits.indexOf("#") == -1){
			valueParts[0] = valueParts[0].substr(valueParts[0].length - pad);
		}
	}

	// Add group separators
	var index = patternParts[0].lastIndexOf(','),
		groupSize, groupSize2;
	if(index != -1){
		groupSize = patternParts[0].length - index - 1;
		var remainder = patternParts[0].substr(0, index);
		index = remainder.lastIndexOf(',');
		if(index != -1){
			groupSize2 = remainder.length - index - 1;
		}
	}
	var pieces = [];
	for(var whole = valueParts[0]; whole;){
		var off = whole.length - groupSize;
		pieces.push((off > 0) ? whole.substr(off) : whole);
		whole = (off > 0) ? whole.slice(0, off) : "";
		if(groupSize2){
			groupSize = groupSize2;
			delete groupSize2;
		}
	}
	valueParts[0] = pieces.reverse().join(options.group || ",");

	return valueParts.join(options.decimal || ".");
};

/*=====
dojo.number.__RegexpOptions = function(){
	//	pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	//	locale: String?
	//		override the locale used to determine formatting rules
	//	strict: Boolean?
	//		strict parsing, false by default.  Strict parsing requires input as produced by the format() method.
	//		Non-strict is more permissive, e.g. flexible on white space, omitting thousands separators
	//	places: Number|String?
	//		number of decimal places to accept: Infinity, a positive number, or
	//		a range "n,m".  Defined by pattern or Infinity if pattern not provided.
	this.pattern = pattern;
	this.type = type;
	this.locale = locale;
	this.strict = strict;
	this.places = places;
}
=====*/
dojo.number.regexp = function(/*dojo.number.__RegexpOptions?*/options){
	//	summary:
	//		Builds the regular needed to parse a number
	//	description:
	//		Returns regular expression with positive and negative match, group
	//		and decimal separators
	return dojo.number._parseInfo(options).regexp; // String
}

dojo.number._parseInfo = function(/*Object?*/options){
	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale),
		bundle = dojo.i18n.getLocalization("dojo.cldr", "number", locale),
		pattern = options.pattern || bundle[(options.type || "decimal") + "Format"],
//TODO: memoize?
		group = bundle.group,
		decimal = bundle.decimal,
		factor = 1;

	if(pattern.indexOf('%') != -1){
		factor /= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		factor /= 1000; // per mille
	}else{
		var isCurrency = pattern.indexOf('\u00a4') != -1;
		if(isCurrency){
			group = bundle.currencyGroup || group;
			decimal = bundle.currencyDecimal || decimal;
		}
	}

	//TODO: handle quoted escapes
	var patternList = pattern.split(';');
	if(patternList.length == 1){
		patternList.push("-" + patternList[0]);
	}

	var re = dojo.regexp.buildGroupRE(patternList, function(pattern){
		pattern = "(?:"+dojo.regexp.escapeString(pattern, '.')+")";
		return pattern.replace(dojo.number._numberPatternRE, function(format){
			var flags = {
				signed: false,
				separator: options.strict ? group : [group,""],
				fractional: options.fractional,
				decimal: decimal,
				exponent: false
				},

				parts = format.split('.'),
				places = options.places;

			// special condition for percent (factor != 1)
			// allow decimal places even if not specified in pattern
			if(parts.length == 1 && factor != 1){
			    parts[1] = "###";
			}
			if(parts.length == 1 || places === 0){
				flags.fractional = false;
			}else{
				if(places === undefined){ places = options.pattern ? parts[1].lastIndexOf('0') + 1 : Infinity; }
				if(places && options.fractional == undefined){flags.fractional = true;} // required fractional, unless otherwise specified
				if(!options.places && (places < parts[1].length)){ places += "," + parts[1].length; }
				flags.places = places;
			}
			var groups = parts[0].split(',');
			if(groups.length > 1){
				flags.groupSize = groups.pop().length;
				if(groups.length > 1){
					flags.groupSize2 = groups.pop().length;
				}
			}
			return "("+dojo.number._realNumberRegexp(flags)+")";
		});
	}, true);

	if(isCurrency){
		// substitute the currency symbol for the placeholder in the pattern
		re = re.replace(/([\s\xa0]*)(\u00a4{1,3})([\s\xa0]*)/g, function(match, before, target, after){
			var prop = ["symbol", "currency", "displayName"][target.length-1],
				symbol = dojo.regexp.escapeString(options[prop] || options.currency || "");
			before = before ? "[\\s\\xa0]" : "";
			after = after ? "[\\s\\xa0]" : "";
			if(!options.strict){
				if(before){before += "*";}
				if(after){after += "*";}
				return "(?:"+before+symbol+after+")?";
			}
			return before+symbol+after;
		});
	}

//TODO: substitute localized sign/percent/permille/etc.?

	// normalize whitespace and return
	return {regexp: re.replace(/[\xa0 ]/g, "[\\s\\xa0]"), group: group, decimal: decimal, factor: factor}; // Object
}

/*=====
dojo.number.__ParseOptions = function(){
	//	pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.  Literal characters in patterns are not supported.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	//	locale: String?
	//		override the locale used to determine formatting rules
	//	strict: Boolean?
	//		strict parsing, false by default.  Strict parsing requires input as produced by the format() method.
	//		Non-strict is more permissive, e.g. flexible on white space, omitting thousands separators
	//	fractional: Boolean?|Array?
	//		Whether to include the fractional portion, where the number of decimal places are implied by pattern
	//		or explicit 'places' parameter.  The value [true,false] makes the fractional portion optional.
	this.pattern = pattern;
	this.type = type;
	this.locale = locale;
	this.strict = strict;
	this.fractional = fractional;
}
=====*/
dojo.number.parse = function(/*String*/expression, /*dojo.number.__ParseOptions?*/options){
	// summary:
	//		Convert a properly formatted string to a primitive Number, using
	//		locale-specific settings.
	// description:
	//		Create a Number from a string using a known localized pattern.
	//		Formatting patterns are chosen appropriate to the locale
	//		and follow the syntax described by
	//		[unicode.org TR35](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
    	//		Note that literal characters in patterns are not supported.
	// expression:
	//		A string representation of a Number
	var info = dojo.number._parseInfo(options),
		results = (new RegExp("^"+info.regexp+"$")).exec(expression);
	if(!results){
		return NaN; //NaN
	}
	var absoluteMatch = results[1]; // match for the positive expression
	if(!results[1]){
		if(!results[2]){
			return NaN; //NaN
		}
		// matched the negative pattern
		absoluteMatch =results[2];
		info.factor *= -1;
	}

	// Transform it to something Javascript can parse as a number.  Normalize
	// decimal point and strip out group separators or alternate forms of whitespace
	absoluteMatch = absoluteMatch.
		replace(new RegExp("["+info.group + "\\s\\xa0"+"]", "g"), "").
		replace(info.decimal, ".");
	// Adjust for negative sign, percent, etc. as necessary
	return absoluteMatch * info.factor; //Number
};

/*=====
dojo.number.__RealNumberRegexpFlags = function(){
	//	places: Number?
	//		The integer number of decimal places or a range given as "n,m".  If
	//		not given, the decimal part is optional and the number of places is
	//		unlimited.
	//	decimal: String?
	//		A string for the character used as the decimal point.  Default
	//		is ".".
	//	fractional: Boolean?|Array?
	//		Whether decimal places are used.  Can be true, false, or [true,
	//		false].  Default is [true, false] which means optional.
	//	exponent: Boolean?|Array?
	//		Express in exponential notation.  Can be true, false, or [true,
	//		false]. Default is [true, false], (i.e. will match if the
	//		exponential part is present are not).
	//	eSigned: Boolean?|Array?
	//		The leading plus-or-minus sign on the exponent.  Can be true,
	//		false, or [true, false].  Default is [true, false], (i.e. will
	//		match if it is signed or unsigned).  flags in regexp.integer can be
	//		applied.
	this.places = places;
	this.decimal = decimal;
	this.fractional = fractional;
	this.exponent = exponent;
	this.eSigned = eSigned;
}
=====*/

dojo.number._realNumberRegexp = function(/*dojo.number.__RealNumberRegexpFlags?*/flags){
	// summary:
	//		Builds a regular expression to match a real number in exponential
	//		notation

	// assign default values to missing parameters
	flags = flags || {};
	//TODO: use mixin instead?
	if(!("places" in flags)){ flags.places = Infinity; }
	if(typeof flags.decimal != "string"){ flags.decimal = "."; }
	if(!("fractional" in flags) || /^0/.test(flags.places)){ flags.fractional = [true, false]; }
	if(!("exponent" in flags)){ flags.exponent = [true, false]; }
	if(!("eSigned" in flags)){ flags.eSigned = [true, false]; }

	var integerRE = dojo.number._integerRegexp(flags),
		decimalRE = dojo.regexp.buildGroupRE(flags.fractional,
		function(q){
			var re = "";
			if(q && (flags.places!==0)){
				re = "\\" + flags.decimal;
				if(flags.places == Infinity){ 
					re = "(?:" + re + "\\d+)?"; 
				}else{
					re += "\\d{" + flags.places + "}"; 
				}
			}
			return re;
		},
		true
	);

	var exponentRE = dojo.regexp.buildGroupRE(flags.exponent,
		function(q){ 
			if(q){ return "([eE]" + dojo.number._integerRegexp({ signed: flags.eSigned}) + ")"; }
			return ""; 
		}
	);

	var realRE = integerRE + decimalRE;
	// allow for decimals without integers, e.g. .25
	if(decimalRE){realRE = "(?:(?:"+ realRE + ")|(?:" + decimalRE + "))";}
	return realRE + exponentRE; // String
};

/*=====
dojo.number.__IntegerRegexpFlags = function(){
	//	signed: Boolean?
	//		The leading plus-or-minus sign. Can be true, false, or `[true,false]`.
	//		Default is `[true, false]`, (i.e. will match if it is signed
	//		or unsigned).
	//	separator: String?
	//		The character used as the thousands separator. Default is no
	//		separator. For more than one symbol use an array, e.g. `[",", ""]`,
	//		makes ',' optional.
	//	groupSize: Number?
	//		group size between separators
	//	groupSize2: Number?
	//		second grouping, where separators 2..n have a different interval than the first separator (for India)
	this.signed = signed;
	this.separator = separator;
	this.groupSize = groupSize;
	this.groupSize2 = groupSize2;
}
=====*/

dojo.number._integerRegexp = function(/*dojo.number.__IntegerRegexpFlags?*/flags){
	// summary: 
	//		Builds a regular expression that matches an integer

	// assign default values to missing parameters
	flags = flags || {};
	if(!("signed" in flags)){ flags.signed = [true, false]; }
	if(!("separator" in flags)){
		flags.separator = "";
	}else if(!("groupSize" in flags)){
		flags.groupSize = 3;
	}

	var signRE = dojo.regexp.buildGroupRE(flags.signed,
		function(q){ return q ? "[-+]" : ""; },
		true
	);

	var numberRE = dojo.regexp.buildGroupRE(flags.separator,
		function(sep){
			if(!sep){
				return "(?:\\d+)";
			}

			sep = dojo.regexp.escapeString(sep);
			if(sep == " "){ sep = "\\s"; }
			else if(sep == "\xa0"){ sep = "\\s\\xa0"; }

			var grp = flags.groupSize, grp2 = flags.groupSize2;
			//TODO: should we continue to enforce that numbers with separators begin with 1-9?  See #6933
			if(grp2){
				var grp2RE = "(?:0|[1-9]\\d{0," + (grp2-1) + "}(?:[" + sep + "]\\d{" + grp2 + "})*[" + sep + "]\\d{" + grp + "})";
				return ((grp-grp2) > 0) ? "(?:" + grp2RE + "|(?:0|[1-9]\\d{0," + (grp-1) + "}))" : grp2RE;
			}
			return "(?:0|[1-9]\\d{0," + (grp-1) + "}(?:[" + sep + "]\\d{" + grp + "})*)";
		},
		true
	);

	return signRE + numberRE; // String
}

}

if(!dojo._hasResource["dojox.charting.plot2d.Pie"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Pie"] = true;
dojo.provide("dojox.charting.plot2d.Pie");












/*=====
dojo.declare("dojox.charting.plot2d.__PieCtorArgs", dojox.charting.plot2d.__DefaultCtorArgs, {
	//	summary:
	//		Specialized keyword arguments object for use in defining parameters on a Pie chart.

	//	labels: Boolean?
	//		Whether or not to draw labels within each pie slice.  Default is true.
	labels:			true,

	//	ticks: Boolean?
	//		Whether or not to draw ticks to labels within each slice. Default is false.
	ticks:			false,

	//	fixed: Boolean?
	//		TODO
	fixed:			true,

	//	precision: Number?
	//		The precision at which to sum/add data values. Default is 1.
	precision:		1,

	//	labelOffset: Number?
	//		The amount in pixels by which to offset labels.  Default is 20.
	labelOffset:	20,

	//	labelStyle: String?
	//		Options as to where to draw labels.  Values include "default", "rows", and "auto". Default is "default".
	labelStyle:		"default",	// default/rows/auto

	//	htmlLabels: Boolean?
	//		Whether or not to use HTML to render slice labels. Default is true.
	htmlLabels:		true,

	//	radGrad: String?
	//		The type of radial gradient to use in rendering.  Default is "native".
	radGrad:        "native",

	//	fanSize: Number?
	//		The amount for a radial gradient.  Default is 5.
	fanSize:		5,

	//	startAngle: Number?
	//		Where to being rendering gradients in slices, in degrees.  Default is 0.
	startAngle:     0,

	//	radius: Number?
	//		The size of the radial gradient.  Default is 0.
	radius:		0
});
=====*/
(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		da = dojox.charting.axis2d.common,
		g = dojox.gfx, m = g.matrix,
		FUDGE_FACTOR = 0.2; // use to overlap fans

	dojo.declare("dojox.charting.plot2d.Pie", [dojox.charting.Element, dojox.charting.plot2d._PlotEvents], {
		//	summary:
		//		The plot that represents a typical pie chart.
		defaultParams: {
			labels:			true,
			ticks:			false,
			fixed:			true,
			precision:		1,
			labelOffset:	20,
			labelStyle:		"default",	// default/rows/auto
			htmlLabels:		true,		// use HTML to draw labels
			radGrad:        "native",	// or "linear", or "fan"
			fanSize:		5,			// maximum fan size in degrees
			startAngle:     0			// start angle for slices in degrees
		},
		optionalParams: {
			radius:		0,
			// theme components
			stroke:		{},
			outline:	{},
			shadow:		{},
			fill:		{},
			font:		"",
			fontColor:	""
		},

		constructor: function(chart, kwArgs){
			//	summary:
			//		Create a pie plot.
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
			this.run = null;
			this.dyn = [];
		},
		clear: function(){
			//	summary:
			//		Clear out all of the information tied to this plot.
			//	returns: dojox.charting.plot2d.Pie
			//		A reference to this plot for functional chaining.
			this.dirty = true;
			this.dyn = [];
			this.run = null;
			return this;	//	dojox.charting.plot2d.Pie
		},
		setAxis: function(axis){
			//	summary:
			//		Dummy method, since axes are irrelevant with a Pie chart.
			//	returns: dojox.charting.plot2d.Pie
			//		The reference to this plot for functional chaining.
			return this;	//	dojox.charting.plot2d.Pie
		},
		addSeries: function(run){
			//	summary:
			//		Add a series of data to this plot.
			//	returns: dojox.charting.plot2d.Pie
			//		The reference to this plot for functional chaining.
			this.run = run;
			return this;	//	dojox.charting.plot2d.Pie
		},
		getSeriesStats: function(){
			//	summary:
			//		Returns default stats (irrelevant for this type of plot).
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			return dojo.delegate(dc.defaultStats);
		},
		initializeScalers: function(){
			//	summary:
			//		Does nothing (irrelevant for this type of plot).
			return this;
		},
		getRequiredColors: function(){
			//	summary:
			//		Return the number of colors needed to draw this plot.
			return this.run ? this.run.data.length : 0;
		},

		render: function(dim, offsets){
			//	summary:
			//		Render the plot on the chart.
			//	dim: Object
			//		An object of the form { width, height }.
			//	offsets: Object
			//		An object of the form { l, r, t, b }.
			//	returns: dojox.charting.plot2d.Pie
			//		A reference to this plot for functional chaining.
			if(!this.dirty){ return this; }
			this.resetEvents();
			this.dirty = false;
			this._eventSeries = {};
			this.cleanGroup();
			var s = this.group, t = this.chart.theme;

			if(!this.run || !this.run.data.length){
				return this;
			}

			// calculate the geometry
			var rx = (dim.width  - offsets.l - offsets.r) / 2,
				ry = (dim.height - offsets.t - offsets.b) / 2,
				r  = Math.min(rx, ry),
				taFont = "font" in this.opt ? this.opt.font : t.axis.font,
				size = taFont ? g.normalizedLength(g.splitFontString(taFont).size) : 0,
				taFontColor = "fontColor" in this.opt ? this.opt.fontColor : t.axis.fontColor,
				startAngle = m._degToRad(this.opt.startAngle),
				start = startAngle, step, filteredRun, slices, labels, shift, labelR,
				run = this.run.data,
				events = this.events();
			if(typeof run[0] == "number"){
				filteredRun = df.map(run, "x ? Math.max(x, 0) : 0");
				if(df.every(filteredRun, "<= 0")){
					return this;
				}
				slices = df.map(filteredRun, "/this", df.foldl(filteredRun, "+", 0));
				if(this.opt.labels){
					labels = dojo.map(slices, function(x){
						return x > 0 ? this._getLabel(x * 100) + "%" : "";
					}, this);
				}
			}else{
				filteredRun = df.map(run, "x ? Math.max(x.y, 0) : 0");
				if(df.every(filteredRun, "<= 0")){
					return this;
				}
				slices = df.map(filteredRun, "/this", df.foldl(filteredRun, "+", 0));
				if(this.opt.labels){
					labels = dojo.map(slices, function(x, i){
						if(x <= 0){ return ""; }
						var v = run[i];
						return "text" in v ? v.text : this._getLabel(x * 100) + "%";
					}, this);
				}
			}
			var themes = df.map(run, function(v, i){
				if(v === null || typeof v == "number"){
					return t.next("slice", [this.opt, this.run], true);
				}
				return t.next("slice", [this.opt, this.run, v], true);
			}, this);
			if(this.opt.labels){
				shift = df.foldl1(df.map(labels, function(label, i){
					var font = themes[i].series.font;
					return dojox.gfx._base._getTextBox(label, {font: font}).w;
				}, this), "Math.max(a, b)") / 2;
				if(this.opt.labelOffset < 0){
					r = Math.min(rx - 2 * shift, ry - size) + this.opt.labelOffset;
				}
				labelR = r - this.opt.labelOffset;
			}
			if("radius" in this.opt){
				r = this.opt.radius;
				labelR = r - this.opt.labelOffset;
			}
			var	circle = {
					cx: offsets.l + rx,
					cy: offsets.t + ry,
					r:  r
				};

			this.dyn = [];
			// draw slices
			var eventSeries = new Array(slices.length);
			dojo.some(slices, function(slice, i){
				if(slice <= 0){
					// degenerated slice
					return false;	// continue
				}
				var v = run[i], theme = themes[i], specialFill;
				if(slice >= 1){
					// whole pie
					specialFill = this._plotFill(theme.series.fill, dim, offsets);
					specialFill = this._shapeFill(specialFill,
						{
							x: circle.cx - circle.r, y: circle.cy - circle.r,
							width: 2 * circle.r, height: 2 * circle.r
						});
					specialFill = this._pseudoRadialFill(specialFill, {x: circle.cx, y: circle.cy}, circle.r);
					var shape = s.createCircle(circle).setFill(specialFill).setStroke(theme.series.stroke);
					this.dyn.push({fill: specialFill, stroke: theme.series.stroke});

					if(events){
						var o = {
							element: "slice",
							index:   i,
							run:     this.run,
							shape:   shape,
							x:       i,
							y:       typeof v == "number" ? v : v.y,
							cx:      circle.cx,
							cy:      circle.cy,
							cr:      r
						};
						this._connectEvents(o);
						eventSeries[i] = o;
					}

					return true;	// stop iteration
				}
				// calculate the geometry of the slice
				var end = start + slice * 2 * Math.PI;
				if(i + 1 == slices.length){
					end = startAngle + 2 * Math.PI;
				}
				var	step = end - start,
					x1 = circle.cx + r * Math.cos(start),
					y1 = circle.cy + r * Math.sin(start),
					x2 = circle.cx + r * Math.cos(end),
					y2 = circle.cy + r * Math.sin(end);
				// draw the slice
				var fanSize = m._degToRad(this.opt.fanSize);
				if(theme.series.fill && theme.series.fill.type === "radial" && this.opt.radGrad === "fan" && step > fanSize){
					var group = s.createGroup(), nfans = Math.ceil(step / fanSize), delta = step / nfans;
					specialFill = this._shapeFill(theme.series.fill,
						{x: circle.cx - circle.r, y: circle.cy - circle.r, width: 2 * circle.r, height: 2 * circle.r});
					for(var j = 0; j < nfans; ++j){
						var fansx = j == 0 ? x1 : circle.cx + r * Math.cos(start + (j - FUDGE_FACTOR) * delta),
							fansy = j == 0 ? y1 : circle.cy + r * Math.sin(start + (j - FUDGE_FACTOR) * delta),
							fanex = j == nfans - 1 ? x2 : circle.cx + r * Math.cos(start + (j + 1 + FUDGE_FACTOR) * delta),
							faney = j == nfans - 1 ? y2 : circle.cy + r * Math.sin(start + (j + 1 + FUDGE_FACTOR) * delta),
							fan = group.createPath({}).
								moveTo(circle.cx, circle.cy).
								lineTo(fansx, fansy).
								arcTo(r, r, 0, delta > Math.PI, true, fanex, faney).
								lineTo(circle.cx, circle.cy).
								closePath().
								setFill(this._pseudoRadialFill(specialFill, {x: circle.cx, y: circle.cy}, r, start + (j + 0.5) * delta, start + (j + 0.5) * delta));
					}
					group.createPath({}).
						moveTo(circle.cx, circle.cy).
						lineTo(x1, y1).
						arcTo(r, r, 0, step > Math.PI, true, x2, y2).
						lineTo(circle.cx, circle.cy).
						closePath().
						setStroke(theme.series.stroke);
					shape = group;
				}else{
					shape = s.createPath({}).
						moveTo(circle.cx, circle.cy).
						lineTo(x1, y1).
						arcTo(r, r, 0, step > Math.PI, true, x2, y2).
						lineTo(circle.cx, circle.cy).
						closePath().
						setStroke(theme.series.stroke);
					var specialFill = theme.series.fill;
					if(specialFill && specialFill.type === "radial"){
						specialFill = this._shapeFill(specialFill, {x: circle.cx - circle.r, y: circle.cy - circle.r, width: 2 * circle.r, height: 2 * circle.r});
						if(this.opt.radGrad === "linear"){
							specialFill = this._pseudoRadialFill(specialFill, {x: circle.cx, y: circle.cy}, r, start, end);
						}
					}else if(specialFill && specialFill.type === "linear"){
						specialFill = this._plotFill(specialFill, dim, offsets);
						specialFill = this._shapeFill(specialFill, shape.getBoundingBox());
					}
					shape.setFill(specialFill);
				}
				this.dyn.push({fill: specialFill, stroke: theme.series.stroke});

				if(events){
					var o = {
						element: "slice",
						index:   i,
						run:     this.run,
						shape:   shape,
						x:       i,
						y:       typeof v == "number" ? v : v.y,
						cx:      circle.cx,
						cy:      circle.cy,
						cr:      r
					};
					this._connectEvents(o);
					eventSeries[i] = o;
				}

				start = end;

				return false;	// continue
			}, this);
			// draw labels
			if(this.opt.labels){
				start = startAngle;
				dojo.some(slices, function(slice, i){
					if(slice <= 0){
						// degenerated slice
						return false;	// continue
					}
					var theme = themes[i];
					if(slice >= 1){
						// whole pie
						var v = run[i], elem = da.createText[this.opt.htmlLabels && dojox.gfx.renderer != "vml" ? "html" : "gfx"](
								this.chart, s, circle.cx, circle.cy + size / 2, "middle", labels[i],
								theme.series.font, theme.series.fontColor);
						if(this.opt.htmlLabels){
							this.htmlElements.push(elem);
						}
						return true;	// stop iteration
					}
					// calculate the geometry of the slice
					var end = start + slice * 2 * Math.PI, v = run[i];
					if(i + 1 == slices.length){
						end = startAngle + 2 * Math.PI;
					}
					var	labelAngle = (start + end) / 2,
						x = circle.cx + labelR * Math.cos(labelAngle),
						y = circle.cy + labelR * Math.sin(labelAngle) + size / 2;
					// draw the label
					var elem = da.createText[this.opt.htmlLabels && dojox.gfx.renderer != "vml" ? "html" : "gfx"]
							(this.chart, s, x, y, "middle", labels[i], theme.series.font, theme.series.fontColor);
					if(this.opt.htmlLabels){
						this.htmlElements.push(elem);
					}
					start = end;
					return false;	// continue
				}, this);
			}
			// post-process events to restore the original indexing
			var esi = 0;
			this._eventSeries[this.run.name] = df.map(run, function(v){
				return v <= 0 ? null : eventSeries[esi++];
			});
			return this;	//	dojox.charting.plot2d.Pie
		},

		// utilities
		_getLabel: function(number){
			return this.opt.fixed ? dojo.number.format(number, {places: this.opt.precision}) : number.toString();
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.Bubble"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Bubble"] = true;
dojo.provide("dojox.charting.plot2d.Bubble");




(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.Bubble", dojox.charting.plot2d.Base, {
		//	summary:
		//		A plot representing bubbles.  Note that data for Bubbles requires 3 parameters,
		//		in the form of:  { x, y, size }, where size determines the size of the bubble.
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			animate: null   // animate bars into place
		},
		optionalParams: {
			// theme component
			stroke:		{},
			outline:	{},
			shadow:		{},
			fill:		{},
			font:		"",
			fontColor:	""
		},

		constructor: function(chart, kwArgs){
			//	summary:
			//		Create a plot of bubbles.
			//	chart: dojox.charting.Chart2D
			//		The chart this plot belongs to.
			//	kwArgs: dojox.charting.plot2d.__DefaultCtorArgs?
			//		Optional keyword arguments object to help define plot parameters.
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
			this.animate = this.opt.animate;
		},

		//	override the render so that we are plotting only circles.
		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojox.charting.plot2d.Bubble
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}

			var t = this.chart.theme,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				events = this.events();

			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				if(!run.data.length){
					run.dirty = false;
					t.skip();
					continue;
				}

				if(typeof run.data[0] == "number"){
					console.warn("dojox.charting.plot2d.Bubble: the data in the following series cannot be rendered as a bubble chart; ", run);
					continue;
				}

				var theme = t.next("circle", [this.opt, run]), s = run.group,
					points = dojo.map(run.data, function(v, i){
						return v ? {
							x: ht(v.x) + offsets.l,
							y: dim.height - offsets.b - vt(v.y),
							radius: this._vScaler.bounds.scale * (v.size / 2)
						} : null;
					}, this);

				var frontCircles = null, outlineCircles = null, shadowCircles = null;

				// make shadows if needed
				if(theme.series.shadow){
					shadowCircles = dojo.map(points, function(item){
						if(item !== null){
							var finalTheme = t.addMixin(theme, "circle", item, true),
								shadow = finalTheme.series.shadow;
							var shape = s.createCircle({
								cx: item.x + shadow.dx, cy: item.y + shadow.dy, r: item.radius
							}).setStroke(shadow).setFill(shadow.color);
							if(this.animate){
								this._animateBubble(shape, dim.height - offsets.b, item.radius);
							}
							return shape;
						}
						return null;
					}, this);
					if(shadowCircles.length){
						run.dyn.shadow = shadowCircles[shadowCircles.length - 1].getStroke();
					}
				}

				// make outlines if needed
				if(theme.series.outline){
					outlineCircles = dojo.map(points, function(item){
						if(item !== null){
							var finalTheme = t.addMixin(theme, "circle", item, true),
								outline = dc.makeStroke(finalTheme.series.outline);
							outline.width = 2 * outline.width + theme.series.stroke.width;
							var shape = s.createCircle({
								cx: item.x, cy: item.y, r: item.radius
							}).setStroke(outline);
							if(this.animate){
								this._animateBubble(shape, dim.height - offsets.b, item.radius);
							}
							return shape;
						}
						return null;
					}, this);
					if(outlineCircles.length){
						run.dyn.outline = outlineCircles[outlineCircles.length - 1].getStroke();
					}
				}

				//	run through the data and add the circles.
				frontCircles = dojo.map(points, function(item){
					if(item !== null){
						var finalTheme = t.addMixin(theme, "circle", item, true),
							rect = {
								x: item.x - item.radius,
								y: item.y - item.radius,
								width:  2 * item.radius,
								height: 2 * item.radius
							};
						var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
						specialFill = this._shapeFill(specialFill, rect);
						var shape = s.createCircle({
							cx: item.x, cy: item.y, r: item.radius
						}).setFill(specialFill).setStroke(finalTheme.series.stroke);
						if(this.animate){
							this._animateBubble(shape, dim.height - offsets.b, item.radius);
						}
						return shape;
					}
					return null;
				}, this);
				if(frontCircles.length){
					run.dyn.fill   = frontCircles[frontCircles.length - 1].getFill();
					run.dyn.stroke = frontCircles[frontCircles.length - 1].getStroke();
				}

				if(events){
					var eventSeries = new Array(frontCircles.length);
					dojo.forEach(frontCircles, function(s, i){
						if(s !== null){
							var o = {
								element: "circle",
								index:   i,
								run:     run,
								shape:   s,
								outline: outlineCircles && outlineCircles[i] || null,
								shadow:  shadowCircles && shadowCircles[i] || null,
								x:       run.data[i].x,
								y:       run.data[i].y,
								r:       run.data[i].size / 2,
								cx:      points[i].x,
								cy:      points[i].y,
								cr:      points[i].radius
							};
							this._connectEvents(o);
							eventSeries[i] = o;
						}
					}, this);
					this._eventSeries[run.name] = eventSeries;
				}else{
					delete this._eventSeries[run.name];
				}

				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.Bubble
		},
		_animateBubble: function(shape, offset, size){
			dojox.gfx.fx.animateTransform(dojo.delegate({
				shape: shape,
				duration: 1200,
				transform: [
					{name: "translate", start: [0, offset], end: [0, 0]},
					{name: "scale", start: [0, 1/size], end: [1, 1]},
					{name: "original"}
				]
			}, this.animate)).play();
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.Candlesticks"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Candlesticks"] = true;
dojo.provide("dojox.charting.plot2d.Candlesticks");








(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	//	Candlesticks are based on the Bars plot type; we expect the following passed
	//	as values in a series:
	//	{ x?, open, close, high, low, mid? }
	//	if x is not provided, the array index is used.
	//	failing to provide the OHLC values will throw an error.
	dojo.declare("dojox.charting.plot2d.Candlesticks", dojox.charting.plot2d.Base, {
		//	summary:
		//		A plot that represents typical candlesticks (financial reporting, primarily).
		//		Unlike most charts, the Candlestick expects data points to be represented by
		//		an object of the form { x?, open, close, high, low, mid? }, where both
		//		x and mid are optional parameters.  If x is not provided, the index of the
		//		data array is used.
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			gap:	2,		// gap between columns in pixels
			animate: null   // animate bars into place
		},
		optionalParams: {
			minBarSize:	1,	// minimal candle width in pixels
			maxBarSize:	1,	// maximal candle width in pixels
			// theme component
			stroke:		{},
			outline:	{},
			shadow:		{},
			fill:		{},
			font:		"",
			fontColor:	""
		},

		constructor: function(chart, kwArgs){
			//	summary:
			//		The constructor for a candlestick chart.
			//	chart: dojox.charting.Chart2D
			//		The chart this plot belongs to.
			//	kwArgs: dojox.charting.plot2d.__BarCtorArgs?
			//		An optional keyword arguments object to help define the plot.
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
			this.animate = this.opt.animate;
		},

		collectStats: function(series){
			//	summary:
			//		Collect all statistics for drawing this chart.  Since the common
			//		functionality only assumes x and y, Candlesticks must create it's own
			//		stats (since data has no y value, but open/close/high/low instead).
			//	series: dojox.charting.Series[]
			//		The data series array to be drawn on this plot.
			//	returns: Object
			//		Returns an object in the form of { hmin, hmax, vmin, vmax }.

			//	we have to roll our own, since we need to use all four passed
			//	values to figure out our stats, and common only assumes x and y.
			var stats = dojo.delegate(dc.defaultStats);
			for(var i=0; i<series.length; i++){
				var run = series[i];
				if(!run.data.length){ continue; }
				var old_vmin = stats.vmin, old_vmax = stats.vmax;
				if(!("ymin" in run) || !("ymax" in run)){
					dojo.forEach(run.data, function(val, idx){
						if(val !== null){
							var x = val.x || idx + 1;
							stats.hmin = Math.min(stats.hmin, x);
							stats.hmax = Math.max(stats.hmax, x);
							stats.vmin = Math.min(stats.vmin, val.open, val.close, val.high, val.low);
							stats.vmax = Math.max(stats.vmax, val.open, val.close, val.high, val.low);
						}
					});
				}
				if("ymin" in run){ stats.vmin = Math.min(old_vmin, run.ymin); }
				if("ymax" in run){ stats.vmax = Math.max(old_vmax, run.ymax); }
			}
			return stats;	//	Object
		},

		getSeriesStats: function(){
			//	summary:
			//		Calculate the min/max on all attached series in both directions.
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			var stats = this.collectStats(this.series);
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			return stats;
		},

		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojox.charting.plot2d.Candlesticks
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, f, gap, width,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				baseline = Math.max(0, this._vScaler.bounds.lower),
				baselineHeight = vt(baseline),
				events = this.events();
			f = dc.calculateBarSize(this._hScaler.bounds.scale, this.opt);
			gap = f.gap;
			width = f.size;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("candlestick", [this.opt, run]), s = run.group,
					eventSeries = new Array(run.data.length);
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j];
					if(v !== null){
						var finalTheme = t.addMixin(theme, "candlestick", v, true);

						//	calculate the points we need for OHLC
						var x = ht(v.x || (j+0.5)) + offsets.l + gap,
							y = dim.height - offsets.b,
							open = vt(v.open),
							close = vt(v.close),
							high = vt(v.high),
							low = vt(v.low);
						if("mid" in v){
							var mid = vt(v.mid);
						}
						if(low > high){
							var tmp = high;
							high = low;
							low = tmp;
						}

						if(width >= 1){
							//	draw the line and rect, set up as a group and pass that to the events.
							var doFill = open > close;
							var line = { x1: width/2, x2: width/2, y1: y - high, y2: y - low },
								rect = {
									x: 0, y: y-Math.max(open, close),
									width: width, height: Math.max(doFill ? open-close : close-open, 1)
								};
							shape = s.createGroup();
							shape.setTransform({dx: x, dy: 0 });
							var inner = shape.createGroup();
							inner.createLine(line).setStroke(finalTheme.series.stroke);
							inner.createRect(rect).setStroke(finalTheme.series.stroke).
								setFill(doFill ? finalTheme.series.fill : "white");
							if("mid" in v){
								//	add the mid line.
								inner.createLine({
									x1: (finalTheme.series.stroke.width||1), x2: width - (finalTheme.series.stroke.width || 1),
									y1: y - mid, y2: y - mid
								}).setStroke(doFill ? "white" : finalTheme.series.stroke);
							}

							//	TODO: double check this.
							run.dyn.fill   = finalTheme.series.fill;
							run.dyn.stroke = finalTheme.series.stroke;
							if(events){
								var o = {
									element: "candlestick",
									index:   j,
									run:     run,
									shape:   inner,
									x:       x,
									y:       y-Math.max(open, close),
									cx:		 width/2,
									cy:		 (y-Math.max(open, close)) + (Math.max(doFill ? open-close : close-open, 1)/2),
									width:	 width,
									height:  Math.max(doFill ? open-close : close-open, 1),
									data:	 v
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}
						}
						if(this.animate){
							this._animateCandlesticks(shape, y - low, high - low);
						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.Candlesticks
		},
		_animateCandlesticks: function(shape, voffset, vsize){
			dojox.gfx.fx.animateTransform(dojo.delegate({
				shape: shape,
				duration: 1200,
				transform: [
					{name: "translate", start: [0, voffset - (voffset/vsize)], end: [0, 0]},
					{name: "scale", start: [1, 1/vsize], end: [1, 1]},
					{name: "original"}
				]
			}, this.animate)).play();
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.OHLC"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.OHLC"] = true;
dojo.provide("dojox.charting.plot2d.OHLC");








(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	//	Candlesticks are based on the Bars plot type; we expect the following passed
	//	as values in a series:
	//	{ x?, open, close, high, low }
	//	if x is not provided, the array index is used.
	//	failing to provide the OHLC values will throw an error.
	dojo.declare("dojox.charting.plot2d.OHLC", dojox.charting.plot2d.Base, {
		//	summary:
		//		A plot that represents typical open/high/low/close (financial reporting, primarily).
		//		Unlike most charts, the Candlestick expects data points to be represented by
		//		an object of the form { x?, open, close, high, low, mid? }, where both
		//		x and mid are optional parameters.  If x is not provided, the index of the
		//		data array is used.
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			gap:	2,		// gap between columns in pixels
			animate: null	// animate chart to place
		},
		optionalParams: {
			minBarSize: 1,	// minimal bar size in pixels
			maxBarSize: 1,	// maximal bar size in pixels
			// theme component
			stroke:		{},
			outline:	{},
			shadow:		{},
			fill:		{},
			font:		"",
			fontColor:	""
		},

		constructor: function(chart, kwArgs){
			//	summary:
			//		The constructor for a candlestick chart.
			//	chart: dojox.charting.Chart2D
			//		The chart this plot belongs to.
			//	kwArgs: dojox.charting.plot2d.__BarCtorArgs?
			//		An optional keyword arguments object to help define the plot.
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
			this.animate = this.opt.animate;
		},

		collectStats: function(series){
			//	summary:
			//		Collect all statistics for drawing this chart.  Since the common
			//		functionality only assumes x and y, OHLC must create it's own
			//		stats (since data has no y value, but open/close/high/low instead).
			//	series: dojox.charting.Series[]
			//		The data series array to be drawn on this plot.
			//	returns: Object
			//		Returns an object in the form of { hmin, hmax, vmin, vmax }.

			//	we have to roll our own, since we need to use all four passed
			//	values to figure out our stats, and common only assumes x and y.
			var stats = dojo.delegate(dc.defaultStats);
			for(var i=0; i<series.length; i++){
				var run = series[i];
				if(!run.data.length){ continue; }
				var old_vmin = stats.vmin, old_vmax = stats.vmax;
				if(!("ymin" in run) || !("ymax" in run)){
					dojo.forEach(run.data, function(val, idx){
						if(val !== null){
							var x = val.x || idx + 1;
							stats.hmin = Math.min(stats.hmin, x);
							stats.hmax = Math.max(stats.hmax, x);
							stats.vmin = Math.min(stats.vmin, val.open, val.close, val.high, val.low);
							stats.vmax = Math.max(stats.vmax, val.open, val.close, val.high, val.low);
						}
					});
				}
				if("ymin" in run){ stats.vmin = Math.min(old_vmin, run.ymin); }
				if("ymax" in run){ stats.vmax = Math.max(old_vmax, run.ymax); }
			}
			return stats;
		},

		getSeriesStats: function(){
			//	summary:
			//		Calculate the min/max on all attached series in both directions.
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			var stats = this.collectStats(this.series);
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			return stats;
		},

		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojox.charting.plot2d.OHLC
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, f, gap, width,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				baseline = Math.max(0, this._vScaler.bounds.lower),
				baselineHeight = vt(baseline),
				events = this.events();
			f = dc.calculateBarSize(this._hScaler.bounds.scale, this.opt);
			gap = f.gap;
			width = f.size;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("candlestick", [this.opt, run]), s = run.group,
					eventSeries = new Array(run.data.length);
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j];
					if(v !== null){
						var finalTheme = t.addMixin(theme, "candlestick", v, true);

						//	calculate the points we need for OHLC
						var x = ht(v.x || (j+0.5)) + offsets.l + gap,
							y = dim.height - offsets.b,
							open = vt(v.open),
							close = vt(v.close),
							high = vt(v.high),
							low = vt(v.low);
						if(low > high){
							var tmp = high;
							high = low;
							low = tmp;
						}

						if(width >= 1){
							var hl = {x1: width/2, x2: width/2, y1: y - high, y2: y - low},
								op = {x1: 0, x2: ((width/2) + ((finalTheme.series.stroke.width||1)/2)), y1: y-open, y2: y-open},
								cl = {x1: ((width/2) - ((finalTheme.series.stroke.width||1)/2)), x2: width, y1: y-close, y2: y-close};
							shape = s.createGroup();
							shape.setTransform({dx: x, dy: 0});
							var inner = shape.createGroup();
							inner.createLine(hl).setStroke(finalTheme.series.stroke);
							inner.createLine(op).setStroke(finalTheme.series.stroke);
							inner.createLine(cl).setStroke(finalTheme.series.stroke);

							//	TODO: double check this.
							run.dyn.stroke = finalTheme.series.stroke;
							if(events){
								var o = {
									element: "candlestick",
									index:   j,
									run:     run,
									shape:	 inner,
									x:       x,
									y:       y-Math.max(open, close),
									cx:		 width/2,
									cy:		 (y-Math.max(open, close)) + (Math.max(open > close ? open-close : close-open, 1)/2),
									width:	 width,
									height:  Math.max(open > close ? open-close : close-open, 1),
									data:	 v
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}
						}
						if(this.animate){
							this._animateOHLC(shape, y - low, high - low);
						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojox.charting.plot2d.OHLC
		},
		_animateOHLC: function(shape, voffset, vsize){
			dojox.gfx.fx.animateTransform(dojo.delegate({
				shape: shape,
				duration: 1200,
				transform: [
					{name: "translate", start: [0, voffset - (voffset/vsize)], end: [0, 0]},
					{name: "scale", start: [1, 1/vsize], end: [1, 1]},
					{name: "original"}
				]
			}, this.animate)).play();
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.Chart2D"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.Chart2D"] = true;
dojo.provide("dojox.charting.Chart2D");









// require all axes to support references by name



// require all plots to support references by name





















/*=====
dojox.charting.__Chart2DCtorArgs = function(margins, stroke, fill, delayInMs){
	//	summary:
	//		The keyword arguments that can be passed in a Chart2D constructor.
	//
	//	margins: Object?
	//		Optional margins for the chart, in the form of { l, t, r, b}.
	//	stroke: dojox.gfx.Stroke?
	//		An optional outline/stroke for the chart.
	//	fill: dojox.gfx.Fill?
	//		An optional fill for the chart.
	//	delayInMs: Number
	//		Delay in ms for delayedRender(). Default: 200.
	this.margins = margins;
	this.stroke = stroke;
	this.fill = fill;
	this.delayInMs = delayInMs;
}
 =====*/
(function(){
	var df = dojox.lang.functional, dc = dojox.charting,
		clear = df.lambda("item.clear()"),
		purge = df.lambda("item.purgeGroup()"),
		destroy = df.lambda("item.destroy()"),
		makeClean = df.lambda("item.dirty = false"),
		makeDirty = df.lambda("item.dirty = true"),
		getName = df.lambda("item.name");

	dojo.declare("dojox.charting.Chart2D", null, {
		//	summary:
		//		The main chart object in dojox.charting.  This will create a two dimensional
		//		chart based on dojox.gfx.
		//
		//	description:
		//		dojox.charting.Chart2D is the primary object used for any kind of charts.  It
		//		is simple to create--just pass it a node reference, which is used as the
		//		container for the chart--and a set of optional keyword arguments and go.
		//
		//		Note that like most of dojox.gfx, most of dojox.charting.Chart2D's methods are
		//		designed to return a reference to the chart itself, to allow for functional
		//		chaining.  This makes defining everything on a Chart very easy to do.
		//
		//	example:
		//		Create an area chart, with smoothing.
		//	|	new dojox.charting.Chart2D(node))
		//	|		.addPlot("default", { type: "Areas", tension: "X" })
		//	|		.setTheme(dojox.charting.themes.Shrooms)
		//	|		.addSeries("Series A", [1, 2, 0.5, 1.5, 1, 2.8, 0.4])
		//	|		.addSeries("Series B", [2.6, 1.8, 2, 1, 1.4, 0.7, 2])
		//	|		.addSeries("Series C", [6.3, 1.8, 3, 0.5, 4.4, 2.7, 2])
		//	|		.render();
		//
		//	example:
		//		The form of data in a data series can take a number of forms: a simple array,
		//		an array of objects {x,y}, or something custom (as determined by the plot).
		//		Here's an example of a Candlestick chart, which expects an object of
		//		{ open, high, low, close }.
		//	|	new dojox.charting.Chart2D(node))
		//	|		.addPlot("default", {type: "Candlesticks", gap: 1})
		//	|		.addAxis("x", {fixLower: "major", fixUpper: "major", includeZero: true})
		//	|		.addAxis("y", {vertical: true, fixLower: "major", fixUpper: "major", natural: true})
		//	|		.addSeries("Series A", [
		//	|				{ open: 20, close: 16, high: 22, low: 8 },
		//	|				{ open: 16, close: 22, high: 26, low: 6, mid: 18 },
		//	|				{ open: 22, close: 18, high: 22, low: 11, mid: 21 },
		//	|				{ open: 18, close: 29, high: 32, low: 14, mid: 27 },
		//	|				{ open: 29, close: 24, high: 29, low: 13, mid: 27 },
		//	|				{ open: 24, close: 8, high: 24, low: 5 },
		//	|				{ open: 8, close: 16, high: 22, low: 2 },
		//	|				{ open: 16, close: 12, high: 19, low: 7 },
		//	|				{ open: 12, close: 20, high: 22, low: 8 },
		//	|				{ open: 20, close: 16, high: 22, low: 8 },
		//	|				{ open: 16, close: 22, high: 26, low: 6, mid: 18 },
		//	|				{ open: 22, close: 18, high: 22, low: 11, mid: 21 },
		//	|				{ open: 18, close: 29, high: 32, low: 14, mid: 27 },
		//	|				{ open: 29, close: 24, high: 29, low: 13, mid: 27 },
		//	|				{ open: 24, close: 8, high: 24, low: 5 },
		//	|				{ open: 8, close: 16, high: 22, low: 2 },
		//	|				{ open: 16, close: 12, high: 19, low: 7 },
		//	|				{ open: 12, close: 20, high: 22, low: 8 },
		//	|				{ open: 20, close: 16, high: 22, low: 8 },
		//	|				{ open: 16, close: 22, high: 26, low: 6 },
		//	|				{ open: 22, close: 18, high: 22, low: 11 },
		//	|				{ open: 18, close: 29, high: 32, low: 14 },
		//	|				{ open: 29, close: 24, high: 29, low: 13 },
		//	|				{ open: 24, close: 8, high: 24, low: 5 },
		//	|				{ open: 8, close: 16, high: 22, low: 2 },
		//	|				{ open: 16, close: 12, high: 19, low: 7 },
		//	|				{ open: 12, close: 20, high: 22, low: 8 },
		//	|				{ open: 20, close: 16, high: 22, low: 8 }
		//	|			],
		//	|			{ stroke: { color: "green" }, fill: "lightgreen" }
		//	|		)
		//	|		.render();
		//
		//	theme: dojox.charting.Theme?
		//		An optional theme to use for styling the chart.
		//	axes: dojox.charting.Axis{}?
		//		A map of axes for use in plotting a chart.
		//	stack: dojox.charting.plot2d.Base[]
		//		A stack of plotters.
		//	plots: dojox.charting.plot2d.Base{}
		//		A map of plotter indices
		//	series: dojox.charting.Series[]
		//		The stack of data runs used to create plots.
		//	runs: dojox.charting.Series{}
		//		A map of series indices
		//	margins: Object?
		//		The margins around the chart. Default is { l:10, t:10, r:10, b:10 }.
		//	stroke: dojox.gfx.Stroke?
		//		The outline of the chart (stroke in vector graphics terms).
		//	fill: dojox.gfx.Fill?
		//		The color for the chart.
		//	node: DOMNode
		//		The container node passed to the constructor.
		//	surface: dojox.gfx.Surface
		//		The main graphics surface upon which a chart is drawn.
		//	dirty: Boolean
		//		A boolean flag indicating whether or not the chart needs to be updated/re-rendered.
		//	coords: Object
		//		The coordinates on a page of the containing node, as returned from dojo.coords.

		constructor: function(/* DOMNode */node, /* dojox.charting.__Chart2DCtorArgs? */kwArgs){
			//	summary:
			//		The constructor for a new Chart2D.  Initializes all parameters used for a chart.
			//	returns: dojox.charting.Chart2D
			//		The newly created chart.

			// initialize parameters
			if(!kwArgs){ kwArgs = {}; }
			this.margins   = kwArgs.margins ? kwArgs.margins : {l: 10, t: 10, r: 10, b: 10};
			this.stroke    = kwArgs.stroke;
			this.fill      = kwArgs.fill;
			this.delayInMs = kwArgs.delayInMs || 200;

			// default initialization
			this.theme = null;
			this.axes = {};		// map of axes
			this.stack = [];	// stack of plotters
			this.plots = {};	// map of plotter indices
			this.series = [];	// stack of data runs
			this.runs = {};		// map of data run indices
			this.dirty = true;
			this.coords = null;

			// create a surface
			this.node = dojo.byId(node);
			var box = dojo.marginBox(node);
			this.surface = dojox.gfx.createSurface(this.node, box.w || 400, box.h || 300);
		},
		destroy: function(){
			//	summary:
			//		Cleanup when a chart is to be destroyed.
			//	returns: void
			dojo.forEach(this.series, destroy);
			dojo.forEach(this.stack,  destroy);
			df.forIn(this.axes, destroy);
			this.surface.destroy();
		},
		getCoords: function(){
			//	summary:
			//		Get the coordinates and dimensions of the containing DOMNode, as
			//		returned by dojo.coords.
			//	returns: Object
			//		The resulting coordinates of the chart.  See dojo.coords for details.
			if(!this.coords){
				this.coords = dojo.coords(this.node, true);
			}
			return this.coords;	//	Object
		},
		setTheme: function(theme){
			//	summary:
			//		Set a theme of the chart.
			//	theme: dojox.charting.Theme
			//		The theme to be used for visual rendering.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			this.theme = theme.clone();
			this.dirty = true;
			return this;	//	dojox.charting.Chart2D
		},
		addAxis: function(name, kwArgs){
			//	summary:
			//		Add an axis to the chart, for rendering.
			//	name: String
			//		The name of the axis.
			//	kwArgs: dojox.charting.axis2d.__AxisCtorArgs?
			//		An optional keyword arguments object for use in defining details of an axis.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			var axis;
			if(!kwArgs || !("type" in kwArgs)){
				axis = new dc.axis2d.Default(this, kwArgs);
			}else{
				axis = typeof kwArgs.type == "string" ?
					new dc.axis2d[kwArgs.type](this, kwArgs) :
					new kwArgs.type(this, kwArgs);
			}
			axis.name = name;
			axis.dirty = true;
			if(name in this.axes){
				this.axes[name].destroy();
			}
			this.axes[name] = axis;
			this.dirty = true;
			return this;	//	dojox.charting.Chart2D
		},
		getAxis: function(name){
			//	summary:
			//		Get the given axis, by name.
			//	name: String
			//		The name the axis was defined by.
			//	returns: dojox.charting.axis2d.Default
			//		The axis as stored in the chart's axis map.
			return this.axes[name];	//	dojox.charting.axis2d.Default
		},
		removeAxis: function(name){
			//	summary:
			//		Remove the axis that was defined using name.
			//	name: String
			//		The axis name, as defined in addAxis.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			if(name in this.axes){
				// destroy the axis
				this.axes[name].destroy();
				delete this.axes[name];
				// mark the chart as dirty
				this.dirty = true;
			}
			return this;	//	dojox.charting.Chart2D
		},
		addPlot: function(name, kwArgs){
			//	summary:
			//		Add a new plot to the chart, defined by name and using the optional keyword arguments object.
			//		Note that dojox.charting assumes the main plot to be called "default"; if you do not have
			//		a plot called "default" and attempt to add data series to the chart without specifying the
			//		plot to be rendered on, you WILL get errors.
			//	name: String
			//		The name of the plot to be added to the chart.  If you only plan on using one plot, call it "default".
			//	kwArgs: dojox.charting.plot2d.__PlotCtorArgs
			//		An object with optional parameters for the plot in question.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			var plot;
			if(!kwArgs || !("type" in kwArgs)){
				plot = new dc.plot2d.Default(this, kwArgs);
			}else{
				plot = typeof kwArgs.type == "string" ?
					new dc.plot2d[kwArgs.type](this, kwArgs) :
					new kwArgs.type(this, kwArgs);
			}
			plot.name = name;
			plot.dirty = true;
			if(name in this.plots){
				this.stack[this.plots[name]].destroy();
				this.stack[this.plots[name]] = plot;
			}else{
				this.plots[name] = this.stack.length;
				this.stack.push(plot);
			}
			this.dirty = true;
			return this;	//	dojox.charting.Chart2D
		},
		removePlot: function(name){
			//	summary:
			//		Remove the plot defined using name from the chart's plot stack.
			//	name: String
			//		The name of the plot as defined using addPlot.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			if(name in this.plots){
				// get the index and remove the name
				var index = this.plots[name];
				delete this.plots[name];
				// destroy the plot
				this.stack[index].destroy();
				// remove the plot from the stack
				this.stack.splice(index, 1);
				// update indices to reflect the shift
				df.forIn(this.plots, function(idx, name, plots){
					if(idx > index){
						plots[name] = idx - 1;
					}
				});
				// mark the chart as dirty
				this.dirty = true;
			}
			return this;	//	dojox.charting.Chart2D
		},
		getPlotOrder: function(){
			//	summary:
			//		Returns an array of plot names in the current order
			//		(the top-most plot is the first).
			//	returns: Array
			return df.map(this.stack, getName); // Array
		},
		setPlotOrder: function(newOrder){
			//	summary:
			//		Sets new order of plots. newOrder cannot add or remove
			//		plots. Wrong names, or dups are ignored.
			//	newOrder: Array:
			//		Array of plot names compatible with getPlotOrder().
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			var names = {},
				order = df.filter(newOrder, function(name){
					if(!(name in this.plots) || (name in names)){
						return false;
					}
					names[name] = 1;
					return true;
				}, this);
			if(order.length < this.stack.length){
				df.forEach(this.stack, function(plot){
					var name = plot.name;
					if(!(name in names)){
						order.push(name);
					}
				});
			}
			var newStack = df.map(order, function(name){
					return this.stack[this.plots[name]];
				}, this);
			df.forEach(newStack, function(plot, i){
				this.plots[plot.name] = i;
			}, this);
			this.stack = newStack;
			this.dirty = true;
			return this;	//	dojox.charting.Chart2D
		},
		movePlotToFront: function(name){
			//	summary:
			//		Moves a given plot to front.
			//	name: String:
			//		Plot's name to move.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			if(name in this.plots){
				var index = this.plots[name];
				if(index){
					var newOrder = this.getPlotOrder();
					newOrder.splice(index, 1);
					newOrder.unshift(name);
					return this.setPlotOrder(newOrder);	//	dojox.charting.Chart2D
				}
			}
			return this;	//	dojox.charting.Chart2D
		},
		movePlotToBack: function(name){
			//	summary:
			//		Moves a given plot to back.
			//	name: String:
			//		Plot's name to move.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			if(name in this.plots){
				var index = this.plots[name];
				if(index < this.stack.length - 1){
					var newOrder = this.getPlotOrder();
					newOrder.splice(index, 1);
					newOrder.push(name);
					return this.setPlotOrder(newOrder);	//	dojox.charting.Chart2D
				}
			}
			return this;	//	dojox.charting.Chart2D
		},
		addSeries: function(name, data, kwArgs){
			//	summary:
			//		Add a data series to the chart for rendering.
			//	name: String:
			//		The name of the data series to be plotted.
			//	data: Array|Object:
			//		The array of data points (either numbers or objects) that
			//		represents the data to be drawn. Or it can be an object. In
			//		the latter case, it should have a property "data" (an array),
			//		destroy(), and setSeriesObject().
			//	kwArgs: dojox.charting.__SeriesCtorArgs?:
			//		An optional keyword arguments object that will be mixed into
			//		the resultant series object.
			//	returns: dojox.charting.Chart2D:
			//		A reference to the current chart for functional chaining.
			var run = new dc.Series(this, data, kwArgs);
			run.name = name;
			if(name in this.runs){
				this.series[this.runs[name]].destroy();
				this.series[this.runs[name]] = run;
			}else{
				this.runs[name] = this.series.length;
				this.series.push(run);
			}
			this.dirty = true;
			// fix min/max
			if(!("ymin" in run) && "min" in run){ run.ymin = run.min; }
			if(!("ymax" in run) && "max" in run){ run.ymax = run.max; }
			return this;	//	dojox.charting.Chart2D
		},
		removeSeries: function(name){
			//	summary:
			//		Remove the series defined by name from the chart.
			//	name: String
			//		The name of the series as defined by addSeries.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			if(name in this.runs){
				// get the index and remove the name
				var index = this.runs[name],
					plotName = this.series[index].plot;
				delete this.runs[name];
				// destroy the run
				this.series[index].destroy();
				// remove the run from the stack of series
				this.series.splice(index, 1);
				// update indices to reflect the shift
				df.forIn(this.runs, function(idx, name, runs){
					if(idx > index){
						runs[name] = idx - 1;
					}
				});
				this.dirty = true;
			}
			return this;	//	dojox.charting.Chart2D
		},
		updateSeries: function(name, data){
			//	summary:
			//		Update the given series with a new set of data points.
			//	name: String
			//		The name of the series as defined in addSeries.
			//	data: Array|Object:
			//		The array of data points (either numbers or objects) that
			//		represents the data to be drawn. Or it can be an object. In
			//		the latter case, it should have a property "data" (an array),
			//		destroy(), and setSeriesObject().
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			if(name in this.runs){
				var run = this.series[this.runs[name]];
				run.update(data);
				this._invalidateDependentPlots(run.plot, false);
				this._invalidateDependentPlots(run.plot, true);
			}
			return this;	//	dojox.charting.Chart2D
		},
		getSeriesOrder: function(plotName){
			//	summary:
			//		Returns an array of series names in the current order
			//		(the top-most series is the first) within a plot.
			//	plotName: String:
			//		Plot's name.
			//	returns: Array
			return df.map(df.filter(this.series, function(run){
					return run.plot == plotName;
				}), getName);
		},
		setSeriesOrder: function(newOrder){
			//	summary:
			//		Sets new order of series within a plot. newOrder cannot add
			//		or remove series. Wrong names, or dups are ignored.
			//	newOrder: Array:
			//		Array of series names compatible with getPlotOrder(). All
			//		series should belong to the same plot.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			var plotName, names = {},
				order = df.filter(newOrder, function(name){
					if(!(name in this.runs) || (name in names)){
						return false;
					}
					var run = this.series[this.runs[name]];
					if(plotName){
						if(run.plot != plotName){
							return false;
						}
					}else{
						plotName = run.plot;
					}
					names[name] = 1;
					return true;
				}, this);
			df.forEach(this.series, function(run){
				var name = run.name;
				if(!(name in names) && run.plot == plotName){
					order.push(name);
				}
			});
			var newSeries = df.map(order, function(name){
					return this.series[this.runs[name]];
				}, this);
			this.series = newSeries.concat(df.filter(this.series, function(run){
				return run.plot != plotName;
			}));
			df.forEach(this.series, function(run, i){
				this.runs[run.name] = i;
			}, this);
			this.dirty = true;
			return this;	//	dojox.charting.Chart2D
		},
		moveSeriesToFront: function(name){
			//	summary:
			//		Moves a given series to front of a plot.
			//	name: String:
			//		Series' name to move.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			if(name in this.runs){
				var index = this.runs[name],
					newOrder = this.getSeriesOrder(this.series[index].plot);
				if(name != newOrder[0]){
					newOrder.splice(index, 1);
					newOrder.unshift(name);
					return this.setSeriesOrder(newOrder);	//	dojox.charting.Chart2D
				}
			}
			return this;	//	dojox.charting.Chart2D
		},
		moveSeriesToBack: function(name){
			//	summary:
			//		Moves a given series to back of a plot.
			//	name: String:
			//		Series' name to move.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			if(name in this.runs){
				var index = this.runs[name],
					newOrder = this.getSeriesOrder(this.series[index].plot);
				if(name != newOrder[newOrder.length - 1]){
					newOrder.splice(index, 1);
					newOrder.push(name);
					return this.setSeriesOrder(newOrder);	//	dojox.charting.Chart2D
				}
			}
			return this;	//	dojox.charting.Chart2D
		},
		resize: function(width, height){
			//	summary:
			//		Resize the chart to the dimensions of width and height.
			//	width: Number
			//		The new width of the chart.
			//	height: Number
			//		The new height of the chart.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			var box;
			switch(arguments.length){
				case 0:
					box = dojo.marginBox(this.node);
					break;
				case 1:
					box = width;
					break;
				default:
					box = { w: width, h: height };
					break;
			}
			dojo.marginBox(this.node, box);
			this.surface.setDimensions(box.w, box.h);
			this.dirty = true;
			this.coords = null;
			return this.render();	//	dojox.charting.Chart2D
		},
		getGeometry: function(){
			//	summary:
			//		Returns a map of information about all axes in a chart and what they represent
			//		in terms of scaling (see dojox.charting.axis2d.Default.getScaler).
			//	returns: Object
			//		An map of geometry objects, a one-to-one mapping of axes.
			var ret = {};
			df.forIn(this.axes, function(axis){
				if(axis.initialized()){
					ret[axis.name] = {
						name:		axis.name,
						vertical:	axis.vertical,
						scaler:		axis.scaler,
						ticks:		axis.ticks
					};
				}
			});
			return ret;	//	Object
		},
		setAxisWindow: function(name, scale, offset, zoom){
			//	summary:
			//		Zooms an axis and all dependent plots. Can be used to zoom in 1D.
			//	name: String
			//		The name of the axis as defined by addAxis.
			//	scale: Number
			//		The scale on the target axis.
			//	offset: Number
			//		Any offest, as measured by axis tick
			//	zoom: Boolean|Object?
			//		The chart zooming animation trigger.  This is null by default,
			//		e.g. {duration: 1200}, or just set true.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			var axis = this.axes[name];
			if(axis){
				axis.setWindow(scale, offset);
				dojo.forEach(this.stack,function(plot){
					if(plot.hAxis == name || plot.vAxis == name){
						plot.zoom = zoom;
					}
				})
			}
			return this;	//	dojox.charting.Chart2D
		},
		setWindow: function(sx, sy, dx, dy, zoom){
			//	summary:
			//		Zooms in or out any plots in two dimensions.
			//	sx: Number
			//		The scale for the x axis.
			//	sy: Number
			//		The scale for the y axis.
			//	dx: Number
			//		The pixel offset on the x axis.
			//	dy: Number
			//		The pixel offset on the y axis.
			//	zoom: Boolean|Object?
			//		The chart zooming animation trigger.  This is null by default,
			//		e.g. {duration: 1200}, or just set true.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			if(!("plotArea" in this)){
				this.calculateGeometry();
			}
			df.forIn(this.axes, function(axis){
				var scale, offset, bounds = axis.getScaler().bounds,
					s = bounds.span / (bounds.upper - bounds.lower);
				if(axis.vertical){
					scale  = sy;
					offset = dy / s / scale;
				}else{
					scale  = sx;
					offset = dx / s / scale;
				}
				axis.setWindow(scale, offset);
			});
			dojo.forEach(this.stack, function(plot){ plot.zoom = zoom; });
			return this;	//	dojox.charting.Chart2D
		},
		zoomIn:	function(name, range){
			//	summary:
			//		Zoom the chart to a specific range on one axis.  This calls render()
			//		directly as a convenience method.
			//	name: String
			//		The name of the axis as defined by addAxis.
			//	range: Array
			//		The end points of the zoom range, measured in axis ticks.
			var axis = this.axes[name];
			if(axis){
				var scale, offset, bounds = axis.getScaler().bounds;
				var lower = Math.min(range[0],range[1]);
				var upper = Math.max(range[0],range[1]);
				lower = range[0] < bounds.lower ? bounds.lower : lower;
				upper = range[1] > bounds.upper ? bounds.upper : upper;
				scale = (bounds.upper - bounds.lower) / (upper - lower);
				offset = lower - bounds.lower;
				this.setAxisWindow(name, scale, offset);
				this.render();
			}
		},
		calculateGeometry: function(){
			//	summary:
			//		Calculate the geometry of the chart based on the defined axes of
			//		a chart.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			if(this.dirty){
				return this.fullGeometry();
			}

			// calculate geometry
			var dirty = dojo.filter(this.stack, function(plot){
					return plot.dirty ||
						(plot.hAxis && this.axes[plot.hAxis].dirty) ||
						(plot.vAxis && this.axes[plot.vAxis].dirty);
				}, this);
			calculateAxes(dirty, this.plotArea);

			return this;	//	dojox.charting.Chart2D
		},
		fullGeometry: function(){
			//	summary:
			//		Calculate the full geometry of the chart.  This includes passing
			//		over all major elements of a chart (plots, axes, series, container)
			//		in order to ensure proper rendering.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			this._makeDirty();

			// clear old values
			dojo.forEach(this.stack, clear);

			// rebuild new connections, and add defaults

			// set up a theme
			if(!this.theme){
				this.setTheme(new dojox.charting.Theme(dojox.charting._def));
			}

			// assign series
			dojo.forEach(this.series, function(run){
				if(!(run.plot in this.plots)){
					var plot = new dc.plot2d.Default(this, {});
					plot.name = run.plot;
					this.plots[run.plot] = this.stack.length;
					this.stack.push(plot);
				}
				this.stack[this.plots[run.plot]].addSeries(run);
			}, this);
			// assign axes
			dojo.forEach(this.stack, function(plot){
				if(plot.hAxis){
					plot.setAxis(this.axes[plot.hAxis]);
				}
				if(plot.vAxis){
					plot.setAxis(this.axes[plot.vAxis]);
				}
			}, this);

			// calculate geometry

			// 1st pass
			var dim = this.dim = this.surface.getDimensions();
			dim.width  = dojox.gfx.normalizedLength(dim.width);
			dim.height = dojox.gfx.normalizedLength(dim.height);
			df.forIn(this.axes, clear);
			calculateAxes(this.stack, dim);

			// assumption: we don't have stacked axes yet
			var offsets = this.offsets = { l: 0, r: 0, t: 0, b: 0 };
			df.forIn(this.axes, function(axis){
				df.forIn(axis.getOffsets(), function(o, i){ offsets[i] += o; });
			});
			// add margins
			df.forIn(this.margins, function(o, i){ offsets[i] += o; });

			// 2nd pass with realistic dimensions
			this.plotArea = {
				width: dim.width - offsets.l - offsets.r,
				height: dim.height - offsets.t - offsets.b
			};
			df.forIn(this.axes, clear);
			calculateAxes(this.stack, this.plotArea);

			return this;	//	dojox.charting.Chart2D
		},
		render: function(){
			//	summary:
			//		Render the chart according to the current information defined.  This should
			//		be the last call made when defining/creating a chart, or if data within the
			//		chart has been changed.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			if(this.theme){
				this.theme.clear();
			}

			if(this.dirty){
				return this.fullRender();
			}

			this.calculateGeometry();

			// go over the stack backwards
			df.forEachRev(this.stack, function(plot){ plot.render(this.dim, this.offsets); }, this);

			// go over axes
			df.forIn(this.axes, function(axis){ axis.render(this.dim, this.offsets); }, this);

			this._makeClean();

			// BEGIN FOR HTML CANVAS
			if(this.surface.render){ this.surface.render(); };
			// END FOR HTML CANVAS

			return this;	//	dojox.charting.Chart2D
		},
		fullRender: function(){
			//	summary:
			//		Force a full rendering of the chart, including full resets on the chart itself.
			//		You should not call this method directly unless absolutely necessary.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.

			// calculate geometry
			this.fullGeometry();
			var offsets = this.offsets, dim = this.dim;

			// get required colors
			//var requiredColors = df.foldl(this.stack, "z + plot.getRequiredColors()", 0);
			//this.theme.defineColors({num: requiredColors, cache: false});

			// clear old shapes
			dojo.forEach(this.series, purge);
			df.forIn(this.axes, purge);
			dojo.forEach(this.stack,  purge);
			this.surface.clear();

			// generate shapes

			// draw a plot background
			var t = this.theme,
				fill   = t.plotarea && t.plotarea.fill,
				stroke = t.plotarea && t.plotarea.stroke;
			if(fill){
				this.surface.createRect({
					x: offsets.l - 1, y: offsets.t - 1,
					width:  dim.width  - offsets.l - offsets.r + 2,
					height: dim.height - offsets.t - offsets.b + 2
				}).setFill(fill);
			}
			if(stroke){
				this.surface.createRect({
					x: offsets.l, y: offsets.t,
					width:  dim.width  - offsets.l - offsets.r + 1,
					height: dim.height - offsets.t - offsets.b + 1
				}).setStroke(stroke);
			}

			// go over the stack backwards
			df.foldr(this.stack, function(z, plot){ return plot.render(dim, offsets), 0; }, 0);

			// pseudo-clipping: matting
			fill   = this.fill   !== undefined ? this.fill   : (t.chart && t.chart.fill);
			stroke = this.stroke !== undefined ? this.stroke : (t.chart && t.chart.stroke);

			//	TRT: support for "inherit" as a named value in a theme.
			if(fill == "inherit"){
				//	find the background color of the nearest ancestor node, and use that explicitly.
				var node = this.node, fill = new dojo.Color(dojo.style(node, "backgroundColor"));
				while(fill.a==0 && node!=document.documentElement){
					fill = new dojo.Color(dojo.style(node, "backgroundColor"));
					node = node.parentNode;
				}
			}

			if(fill){
				if(offsets.l){	// left
					this.surface.createRect({
						width:  offsets.l,
						height: dim.height + 1
					}).setFill(fill);
				}
				if(offsets.r){	// right
					this.surface.createRect({
						x: dim.width - offsets.r,
						width:  offsets.r + 1,
						height: dim.height + 2
					}).setFill(fill);
				}
				if(offsets.t){	// top
					this.surface.createRect({
						width:  dim.width + 1,
						height: offsets.t
					}).setFill(fill);
				}
				if(offsets.b){	// bottom
					this.surface.createRect({
						y: dim.height - offsets.b,
						width:  dim.width + 1,
						height: offsets.b + 2
					}).setFill(fill);
				}
			}
			if(stroke){
				this.surface.createRect({
					width:  dim.width - 1,
					height: dim.height - 1
				}).setStroke(stroke);
			}

			// go over axes
			df.forIn(this.axes, function(axis){ axis.render(dim, offsets); });

			this._makeClean();

			// BEGIN FOR HTML CANVAS
			if(this.surface.render){ this.surface.render(); };
			// END FOR HTML CANVAS

			return this;	//	dojox.charting.Chart2D
		},
		delayedRender: function(){
			//	summary:
			//		Delayed render, which is used to collect multiple updates
			//		within a delayInMs time window.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.

			if(!this._delayedRenderHandle){
				this._delayedRenderHandle = setTimeout(
					dojo.hitch(this, function(){
						clearTimeout(this._delayedRenderHandle);
						this._delayedRenderHandle = null;
						this.render();
					}),
					this.delayInMs
				);
			}

			return this;	//	dojox.charting.Chart2D
		},
		connectToPlot: function(name, object, method){
			//	summary:
			//		A convenience method to connect a function to a plot.
			//	name: String
			//		The name of the plot as defined by addPlot.
			//	object: Object
			//		The object to be connected.
			//	method: Function
			//		The function to be executed.
			//	returns: Array
			//		A handle to the connection, as defined by dojo.connect (see dojo.connect).
			return name in this.plots ? this.stack[this.plots[name]].connect(object, method) : null;	//	Array
		},
		fireEvent: function(seriesName, eventName, index){
			//	summary:
			//		Fires a synthetic event for a series item.
			//	seriesName: String:
			//		Series name.
			//	eventName: String:
			//		Event name to simulate: onmouseover, onmouseout, onclick.
			//	index: Number:
			//		Valid data value index for the event.
			//	returns: dojox.charting.Chart2D
			//		A reference to the current chart for functional chaining.
			if(seriesName in this.runs){
				var plotName = this.series[this.runs[seriesName]].plot;
				if(plotName in this.plots){
					var plot = this.stack[this.plots[plotName]];
					if(plot){
						plot.fireEvent(seriesName, eventName, index);
					}
				}
			}
			return this;	//	dojox.charting.Chart2D
		},
		_makeClean: function(){
			// reset dirty flags
			dojo.forEach(this.axes,   makeClean);
			dojo.forEach(this.stack,  makeClean);
			dojo.forEach(this.series, makeClean);
			this.dirty = false;
		},
		_makeDirty: function(){
			// reset dirty flags
			dojo.forEach(this.axes,   makeDirty);
			dojo.forEach(this.stack,  makeDirty);
			dojo.forEach(this.series, makeDirty);
			this.dirty = true;
		},
		_invalidateDependentPlots: function(plotName, /* Boolean */ verticalAxis){
			if(plotName in this.plots){
				var plot = this.stack[this.plots[plotName]], axis,
					axisName = verticalAxis ? "vAxis" : "hAxis";
				if(plot[axisName]){
					axis = this.axes[plot[axisName]];
					if(axis && axis.dependOnData()){
						axis.dirty = true;
						// find all plots and mark them dirty
						dojo.forEach(this.stack, function(p){
							if(p[axisName] && p[axisName] == plot[axisName]){
								p.dirty = true;
							}
						});
					}
				}else{
					plot.dirty = true;
				}
			}
		}
	});

	function hSection(stats){
		return {min: stats.hmin, max: stats.hmax};
	}

	function vSection(stats){
		return {min: stats.vmin, max: stats.vmax};
	}

	function hReplace(stats, h){
		stats.hmin = h.min;
		stats.hmax = h.max;
	}

	function vReplace(stats, v){
		stats.vmin = v.min;
		stats.vmax = v.max;
	}

	function combineStats(target, source){
		if(target && source){
			target.min = Math.min(target.min, source.min);
			target.max = Math.max(target.max, source.max);
		}
		return target || source;
	}

	function calculateAxes(stack, plotArea){
		var plots = {}, axes = {};
		dojo.forEach(stack, function(plot){
			var stats = plots[plot.name] = plot.getSeriesStats();
			if(plot.hAxis){
				axes[plot.hAxis] = combineStats(axes[plot.hAxis], hSection(stats));
			}
			if(plot.vAxis){
				axes[plot.vAxis] = combineStats(axes[plot.vAxis], vSection(stats));
			}
		});
		dojo.forEach(stack, function(plot){
			var stats = plots[plot.name];
			if(plot.hAxis){
				hReplace(stats, axes[plot.hAxis]);
			}
			if(plot.vAxis){
				vReplace(stats, axes[plot.vAxis]);
			}
			plot.initializeScalers(plotArea, stats);
		});
	}
})();

}

if(!dojo._hasResource["dojot.visual.ColumnsEmbossed"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojot.visual.ColumnsEmbossed"] = true;
dojo.provide("dojot.visual.ColumnsEmbossed");









(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojot.visual.ColumnsEmbossed", dojox.charting.plot2d.Base, {
		//	summary:
		//		The plot object representing a column chart (vertical bars).
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			gap:	0,		// gap between columns in pixels
			animate: null,   // animate bars into place
			depth: 5        //redpath
		},
		optionalParams: {
			minBarSize:	1,	// minimal column width in pixels
			maxBarSize:	1,	// maximal column width in pixels
			// theme component
			stroke:		{},
			outline:	{},
			shadow:		{},
			fill:		{},
			font:		"",
			fontColor:	""
		},

		constructor: function(chart, kwArgs){
			//	summary:
			//		The constructor for a columns chart.
			//	chart: dojox.charting.Chart2D
			//		The chart this plot belongs to.
			//	kwArgs: dojox.charting.plot2d.__BarCtorArgs?
			//		An optional keyword arguments object to help define the plot.
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
			this.animate = this.opt.animate;
		},

		getSeriesStats: function(){
			//	summary:
			//		Calculate the min/max on all attached series in both directions.
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			var stats = dc.collectSimpleStats(this.series);
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			return stats;
		},

		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojot.visual.Columns
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			this._vScaler.bounds.span-=this.opt.depth;
            var diff=this._vScaler.bounds.to-this._vScaler.bounds.from;
            this._vScaler.bounds.scale= this._vScaler.bounds.span/diff;

			var t = this.chart.theme, f, gap, width,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				baseline = Math.max(0, this._vScaler.bounds.lower),
				baselineHeight = vt(baseline),
				events = this.events();
			f = dc.calculateBarSize(this._hScaler.bounds.scale, this.opt);
			gap = f.gap;
			width = f.size;

			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("column", [this.opt, run]), s = run.group,
					eventSeries = new Array(run.data.length);
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y,
							vv = vt(v),
							height = vv - baselineHeight,
							h = Math.abs(height),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "column", value, true) :
								t.post(theme, "column");
						if(width >= 1 && h >= 0){
							var rect = {
								x: offsets.l + ht(j + 0.5) + gap,
								y: dim.height - offsets.b - (v > baseline ? vv : baselineHeight),
								width: width, height: h
							};
							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							var work= new dojo.Color(specialFill);
                            var darker = new dojo.Color([ work.r*50/100|0, work.g*50/100|0, work.b*50/100|0 ]);
                            var dark = new dojo.Color([ work.r*90/100|0, work.g*90/100|0, work.b*90/100|0 ]);

                            var x= offsets.l + ht(j + 0.5) + gap;
                            var y= dim.height - offsets.b - vt(v);
                            var off= this.opt.depth;
        
                            var apath = "M" + x           +  "," + y   +
                                 " L" + (x+off)       +  "," + (y-off) +
                                 " L" + (x+width+off) +  "," + (y-off) +
                                 " L" + (x+width)   +  "," + (y) +
                                 " L" + x           +  "," + y+ "z";
                            s.createPath(apath).setFill(dark);
        
                            apath = " M" + (x+width+off) +  "," + (y-off) +
                                 " L" + (x+width+off) +  "," + (y+height-off) +
                                 " L" + (x+width) +  "," + (y+height)+
                                 " L" + (x+width) +  "," + y+" z";
                            s.createPath(apath).setFill(darker);
        
                            var shape = s.createRect({
                     			x: offsets.l + ht(j + 0.5) + gap,
                    			y: dim.height - offsets.b - vt(v),
                      			width: width, height: height
                         	}).setFill(specialFill);

							run.dyn.fill   = shape.getFill();
							run.dyn.stroke = shape.getStroke();
							if(events){
								var o = {
									element: "column",
									index:   j,
									run:     run,
									shape:   shape,
									x:       j + 0.5,
									y:       v
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}
						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojot.visual.Columns
		},
		_animateColumn: function(shape, voffset, vsize){
			dojox.gfx.fx.animateTransform(dojo.delegate({
				shape: shape,
				duration: 1200,
				transform: [
					{name: "translate", start: [0, voffset - (voffset/vsize)], end: [0, 0]},
					{name: "scale", start: [1, 1/vsize], end: [1, 1]},
					{name: "original"}
				]
			}, this.animate)).play();
		}
	});
})();

}

if(!dojo._hasResource["dojot.visual.ClusteredColumnsEmbossed"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojot.visual.ClusteredColumnsEmbossed"] = true;
dojo.provide("dojot.visual.ClusteredColumnsEmbossed");








(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojot.visual.ClusteredColumnsEmbossed", dojot.visual.ColumnsEmbossed, {
		//	summary:
		//		A plot representing grouped or clustered columns (vertical bars).
		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojot.visual.ClusteredColumnsEmbossed
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
			this._vScaler.bounds.span-=this.opt.depth;
            var diff=this._vScaler.bounds.to-this._vScaler.bounds.from;
            this._vScaler.bounds.scale= this._vScaler.bounds.span/diff;

			var t = this.chart.theme, f, gap, width, thickness,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				baseline = Math.max(0, this._vScaler.bounds.lower),
				baselineHeight = vt(baseline),
				events = this.events();
			f = dc.calculateBarSize(this._hScaler.bounds.scale, this.opt, this.series.length);
			gap = f.gap;
			width = thickness = f.size;
            width-=this.opt.depth;

			for(var i = 0; i < this.series.length; ++i){
				var run = this.series[i], shift = thickness * i;
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("column", [this.opt, run]), s = run.group,
					eventSeries = new Array(run.data.length);
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y,
							vv = vt(v),
							height = vv - baselineHeight,
							h = Math.abs(height),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "column", value, true) :
								t.post(theme, "column");
						if(width >= 1 && h >= 0){
							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							var work= new dojo.Color(specialFill);
                            var darker = new dojo.Color([ work.r*50/100|0, work.g*50/100|0, work.b*50/100|0 ]);
                            var dark = new dojo.Color([ work.r*90/100|0, work.g*90/100|0, work.b*90/100|0 ]);
//begin redpath
                            var x= offsets.l + ht(j + 0.5) + gap + shift;
                            var y= dim.height - offsets.b - vt(v);
                            var off= this.opt.depth;
    
    
    
                            var apath = "M" + x           +  "," + y   +
                                 " L" + (x+off)       +  "," + (y-off) +
                                 " L" + (x+width+off) +  "," + (y-off) +
                                 " L" + (x+width)   +  "," + (y) +
                                 " L" + x           +  "," + y+ "z";
                            s.createPath(apath).setFill(dark);
    
                            apath = " M" + (x+width+off) +  "," + (y-off) +
                                 " L" + (x+width+off) +  "," + (y+height-off) +
                                 " L" + (x+width) +  "," + (y+height)+
                                 " L" + (x+width) +  "," + y+" z";
                            s.createPath(apath).setFill(darker);
    
                            var shape = s.createRect({
                                x: offsets.l + ht(j + 0.5) + gap + shift,
    							y: dim.height - offsets.b - (v > baseline ? vv : baselineHeight),
    							width: width, height: h
    						}).setFill(specialFill);
  							run.dyn.fill   = shape.getFill();
   							run.dyn.stroke = shape.getStroke();
   							if(events){
   								var o = {
   									element: "column",
   									index:   j,
   									run:     run,
									shape:   shape,
									x:       j + 0.5,
									y:       v
								};
							this._connectEvents(o);
							eventSeries[j] = o;
							}
						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojot.visual.ClusteredColumnsEmbossed
		}
	});
})();

}

if(!dojo._hasResource["dojot.visual.StackedColumnsEmbossed"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojot.visual.StackedColumnsEmbossed"] = true;
dojo.provide("dojot.visual.StackedColumnsEmbossed");








(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojot.visual.StackedColumnsEmbossed", dojot.visual.ColumnsEmbossed, {
		//	summary:
		//		The plot object representing a stacked column chart (vertical bars).
		getSeriesStats: function(){
			//	summary:
			//		Calculate the min/max on all attached series in both directions.
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			var stats = dc.collectStackedStats(this.series);
			this._maxRunLength = stats.hmax;
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			return stats;
		},
		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojot.visual.StackedColumnsEmbossed
			//		A reference to this plot for functional chaining.
			if(this._maxRunLength <= 0){
				return this;
			}

			// stack all values
			var acc = df.repeat(this._maxRunLength, "-> 0", 0);
			for(var i = 0; i < this.series.length; ++i){
				var run = this.series[i];
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y;
						if(isNaN(v)){ v = 0; }
						acc[j] += v;
					}
				}
			}
			// draw runs in backwards
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}

			this._vScaler.bounds.span-=this.opt.depth;
            var diff=this._vScaler.bounds.to-this._vScaler.bounds.from;
            this._vScaler.bounds.scale= this._vScaler.bounds.span/diff;

			var t = this.chart.theme, f, gap, width,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				events = this.events();
			f = dc.calculateBarSize(this._hScaler.bounds.scale, this.opt);
			gap = f.gap;
			width = f.size;

			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("column", [this.opt, run]), s = run.group,
					eventSeries = new Array(acc.length);
				for(var j = 0; j < acc.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = acc[j],
							height = vt(v),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "column", value, true) :
								t.post(theme, "column");
						if(width >= 1 && height >= 0){

							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							var work= new dojo.Color(specialFill);
                            var darker = new dojo.Color([ work.r*50/100|0, work.g*50/100|0, work.b*50/100|0 ]);
                            var dark = new dojo.Color([ work.r*90/100|0, work.g*90/100|0, work.b*90/100|0 ]);

//
//negative direction
//<path d="M 100 100 L 300 100 L 200 300 z"
                    var x= offsets.l + ht(j + 0.5) + gap;
                    var y= dim.height - offsets.b - vt(v);
                    var off= this.opt.depth;

//s.createPath(apath).setFill("#ff0000");
                    var apath = "M" + x           +  "," + y   +
                         " L" + (x+off)       +  "," + (y-off) +
                         " L" + (x+width+off) +  "," + (y-off) +
                         " L" + (x+width)   +  "," + (y) +
                         " L" + x           +  "," + y+ "z";
                    if (i== (this.series.length-1))
                        s.createPath(apath).setFill(dark);

                    apath = " M" + (x+width+off) +  "," + (y-off) +
                         " L" + (x+width+off) +  "," + (y+height-off) +
                         " L" + (x+width) +  "," + (y+height)+
                         " L" + (x+width) +  "," + y+" z";
                    s.createPath(apath).setFill(darker);

                    var shape = s.createRect({
             			x: offsets.l + ht(j + 0.5) + gap,
            			y: dim.height - offsets.b - vt(v),
              			width: width, height: height
                 	}).setFill(specialFill);

					run.dyn.fill   = shape.getFill();
					run.dyn.stroke = shape.getStroke();
					if(events){
						var o = {
							element: "column",
							index:   j,
							run:     run,
							shape:   shape,
							x:       j + 0.5,
							y:       v
						};
						this._connectEvents(o);
						eventSeries[j] = o;
					}

						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
				// update the accumulator
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y;
						if(isNaN(v)){ v = 0; }
						acc[j] -= v;
					}
				}
			}
			this.dirty = false;
			return this;	//	dojot.visual.StackedColumnsEmbossed
		}
	});
})();

}

if(!dojo._hasResource["dojot.visual.BarsEmbossed"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojot.visual.BarsEmbossed"] = true;
dojo.provide("dojot.visual.BarsEmbossed");








/*=====
dojo.declare("dojox.charting.plot2d.__BarCtorArgs", dojox.charting.plot2d.__DefaultCtorArgs, {
	//	summary:
	//		Additional keyword arguments for bar charts.

	//	minBarSize: Number?
	//		The minimum size for a bar in pixels.  Default is 1.
	minBarSize: 1,

	//	maxBarSize: Number?
	//		The maximum size for a bar in pixels.  Default is 1.
	maxBarSize: 1
});
=====*/
(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojot.visual.BarsEmbossed", dojox.charting.plot2d.Base, {
		//	summary:
		//		The plot object representing a bar chart (horizontal bars).
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			gap:	0,		// gap between columns in pixels
			animate: null,   // animate bars into place
			depth: 5         //redpath

		},
		optionalParams: {
			minBarSize:	1,	// minimal bar width in pixels
			maxBarSize:	1,	// maximal bar width in pixels
			// theme component
			stroke:		{},
			outline:	{},
			shadow:		{},
			fill:		{},
			font:		"",
			fontColor:	""
		},

		constructor: function(chart, kwArgs){
			//	summary:
			//		The constructor for a bar chart.
			//	chart: dojox.charting.Chart2D
			//		The chart this plot belongs to.
			//	kwArgs: dojox.charting.plot2d.__BarCtorArgs?
			//		An optional keyword arguments object to help define the plot.
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
			this.animate = this.opt.animate;
		},


		getSeriesStats: function(){
			//	summary:
			//		Calculate the min/max on all attached series in both directions.
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			var stats = dc.collectSimpleStats(this.series), t;
			stats.hmin -= 0.5;
			stats.hmax += 0.5;

			t = stats.hmin, stats.hmin = stats.vmin, stats.vmin = t;
			t = stats.hmax, stats.hmax = stats.vmax, stats.vmax = t;

			return stats;
		},

		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojot.visual.BarsEmbossed
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.dirty = this.isDirty();
			this.resetEvents();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}

			this._hScaler.bounds.span-=this.opt.depth;
            var diff=this._hScaler.bounds.to-this._hScaler.bounds.from;
            this._hScaler.bounds.scale= this._hScaler.bounds.span/diff;

			var t = this.chart.theme, f, gap, height,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				baseline = Math.max(0, this._hScaler.bounds.lower),
				baselineWidth = ht(baseline),
				events = this.events();

			f = dc.calculateBarSize(this._vScaler.bounds.scale, this.opt);
			gap = f.gap;
			height = f.size;
            height-=this.opt.depth; //redpath

			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("bar", [this.opt, run]), s = run.group,
					eventSeries = new Array(run.data.length);

				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y,
							hv = ht(v),

							width = hv - baselineWidth,
							w = Math.abs(width),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "bar", value, true) :
								t.post(theme, "bar");
						if(w >= 0 && height >= 1){
//begin redpath
                            var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
                            var	x= offsets.l + (v < baseline ? hv : baselineWidth);
                            var	y= dim.height- offsets.b - vt(j + 1.5) + gap;
							var off= this.opt.depth;
                            y+=this.opt.depth;	

							var work= new dojo.Color(specialFill);
                            var darker = new dojo.Color([ work.r*50/100|0, work.g*50/100|0, work.b*50/100|0 ]);
                            var dark = new dojo.Color([ work.r*90/100|0, work.g*90/100|0, work.b*90/100|0 ]);
                            var apath = "M" + x           +  "," + y   +
                                        " L" + (x+off)       +  "," + (y-off) +
                                        " L" + (x+width+off) +  "," + (y-off) +
                                        " L" + (x+width)   +  "," + (y) +
                                        " L" + x           +  "," + y+ "z";
                            s.createPath(apath).setFill(dark);

                            apath = " M" + (x+width+off) +  "," + (y-off) +
                                    " L" + (x+width+off) +  "," + (y+height-off) +
                                    " L" + (x+width) +  "," + (y+height)+
                                    " L" + (x+width) +  "," + y+" z";
                            s.createPath(apath).setFill(darker);

		                    var shape = s.createRect({
                              x: offsets.l + (v < baseline ? hv : baselineWidth),
							  y: dim.height - offsets.b - vt(j + 1.5) + gap + this.opt.depth,
							  width: width, height: height
						    }).setFill(specialFill); //.setStroke(stroke);
//end redpath
							run.dyn.fill   = shape.getFill();
							run.dyn.stroke = shape.getStroke();
							if(events){
								var o = {
									element: "bar",
									index:   j,
									run:     run,
									shape:   shape,
									x:       v,
									y:       j + 1.5
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}
						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojot.visual.BarsEmbossed
		},
		_animateBar: function(shape, hoffset, hsize){
			dojox.gfx.fx.animateTransform(dojo.delegate({
				shape: shape,
				duration: 1200,
				transform: [
					{name: "translate", start: [hoffset - (hoffset/hsize), 0], end: [0, 0]},
					{name: "scale", start: [1/hsize, 1], end: [1, 1]},
					{name: "original"}
				]
			}, this.animate)).play();
		}
	});
})();

}

if(!dojo._hasResource["dojot.visual.ClusteredBarsEmbossed"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojot.visual.ClusteredBarsEmbossed"] = true;
dojo.provide("dojot.visual.ClusteredBarsEmbossed");






(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojot.visual.ClusteredBarsEmbossed", dojot.visual.BarsEmbossed, {
		//	summary:
		//		A plot representing grouped or clustered bars (horizontal bars)
		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojot.visual.ClusteredBarsEmbossed
			//		A reference to this plot for functional chaining.
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}

			this._hScaler.bounds.span-=this.opt.depth;
            var diff=this._hScaler.bounds.to-this._hScaler.bounds.from;
            this._hScaler.bounds.scale= this._hScaler.bounds.span/diff;

			var t = this.chart.theme, f, gap, height, thickness,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				baseline = Math.max(0, this._hScaler.bounds.lower),
				baselineWidth = ht(baseline),
				events = this.events();
			f = dc.calculateBarSize(this._vScaler.bounds.scale, this.opt, this.series.length);
			gap = f.gap;
			height = thickness = f.size;



			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i], shift = thickness * (this.series.length - i - 1);
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("bar", [this.opt, run]), s = run.group,
					eventSeries = new Array(run.data.length);
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y,
							hv = ht(v),
							width = hv - baselineWidth,
							w = Math.abs(width),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "bar", value, true) :
								t.post(theme, "bar");
						if(w >= 0 && height >= 1){

							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							var work= new dojo.Color(specialFill);
                            var darker = new dojo.Color([ work.r*50/100|0, work.g*50/100|0, work.b*50/100|0 ]);
                            var dark = new dojo.Color([ work.r*90/100|0, work.g*90/100|0, work.b*90/100|0 ]);

//begin redpath
                        var x= offsets.l + (v < baseline ? hv : baselineWidth);
                        var y= dim.height - offsets.b - vt(j + 1.5) + this.opt.gap + shift+ this.opt.depth;
                        var off= this.opt.depth;

                        var apath = "M" + x          +  "," + y +
                             " L" + (x+off)       +  "," + (y-off) +
                             " L" + (x+width+off) +  "," + (y-off) +
                             " L" + (x+width)   +  "," + (y) +
                             " L" + x           +  "," + y+ "z";
                        s.createPath(apath).setFill(dark);

						var h= height-off;
                        apath = " M" + (x+width+off) +  "," + (y-off) +
                             " L" + (x+width+off) +  "," + (y+h-off) +
                             " L" + (x+width) +  "," + (y+h)+
                             " L" + (x+width) +  "," + y+" z";
                        s.createPath(apath).setFill(darker);


						var shape = s.createRect({
							x: offsets.l + (v < baseline ? hv : baselineWidth),
							y: dim.height - offsets.b - vt(j + 1.5) + this.opt.gap + shift +this.opt.depth,
							width: w, height: height-this.opt.depth
						}).setFill(specialFill);

							run.dyn.fill   = shape.getFill();
							run.dyn.stroke = shape.getStroke();
							if(events){
								var o = {
									element: "bar",
									index:   j,
									run:     run,
									shape:   shape,
									x:       v,
									y:       j + 1.5
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}

						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
			}
			this.dirty = false;
			return this;	//	dojot.visual.ClusteredBarsEmbossed
		}
	});
})();

}

if(!dojo._hasResource["dojot.visual.StackedBarsEmbossed"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojot.visual.StackedBarsEmbossed"] = true;
dojo.provide("dojot.visual.StackedBarsEmbossed");








(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojot.visual.StackedBarsEmbossed", dojot.visual.BarsEmbossed, {
		//	summary:
		//		The plot object representing a stacked bar chart (horizontal bars).
		getSeriesStats: function(){
			//	summary:
			//		Calculate the min/max on all attached series in both directions.
			//	returns: Object
			//		{hmin, hmax, vmin, vmax} min/max in both directions.
			var stats = dc.collectStackedStats(this.series), t;
			this._maxRunLength = stats.hmax;
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			t = stats.hmin, stats.hmin = stats.vmin, stats.vmin = t;
			t = stats.hmax, stats.hmax = stats.vmax, stats.vmax = t;
			return stats;
		},
		render: function(dim, offsets){
			//	summary:
			//		Run the calculations for any axes for this plot.
			//	dim: Object
			//		An object in the form of { width, height }
			//	offsets: Object
			//		An object of the form { l, r, t, b}.
			//	returns: dojot.visual.StackedBarsEmbossed
			//		A reference to this plot for functional chaining.


			if(this._maxRunLength <= 0){
				return this;
			}

			// stack all values
			var acc = df.repeat(this._maxRunLength, "-> 0", 0);
			for(var i = 0; i < this.series.length; ++i){
				var run = this.series[i];
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y;
						if(isNaN(v)){ v = 0; }
						acc[j] += v;
					}
				}
			}
			// draw runs in backwards
			if(this.zoom && !this.isDataDirty()){
				return this.performZoom(dim, offsets);
			}
			this.resetEvents();
			this.dirty = this.isDirty();
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this._eventSeries = {};
				this.cleanGroup();
				var s = this.group;
				df.forEachRev(this.series, function(item){ item.cleanGroup(s); });
			}
            this._hScaler.bounds.span-=this.opt.depth;
            var diff=this._hScaler.bounds.to-this._hScaler.bounds.from;
            this._hScaler.bounds.scale= this._hScaler.bounds.span/diff;

			var t = this.chart.theme, f, gap, height,
				ht = this._hScaler.scaler.getTransformerFromModel(this._hScaler),
				vt = this._vScaler.scaler.getTransformerFromModel(this._vScaler),
				events = this.events();
			f = dc.calculateBarSize(this._vScaler.bounds.scale, this.opt);

			gap = f.gap;
			height = f.size;
            height-=this.opt.depth; //redpath

			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){
					t.skip();
					this._reconnectEvents(run.name);
					continue;
				}
				run.cleanGroup();
				var theme = t.next("bar", [this.opt, run]), s = run.group,
					eventSeries = new Array(acc.length);
				for(var j = 0; j < acc.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = acc[j],
							width = ht(v),
							finalTheme = typeof value != "number" ?
								t.addMixin(theme, "bar", value, true) :
								t.post(theme, "bar");
						if(width >= 0 && height >= 1){
							var rect = {
								x: offsets.l,
								y: dim.height - offsets.b - vt(j + 1.5) + gap,
								width: width, height: height
							};
//begin redpath
//							width-=this.opt.depth;
							var specialFill = this._plotFill(finalTheme.series.fill, dim, offsets);
							specialFill = this._shapeFill(specialFill, rect);
							var work= new dojo.Color(specialFill);
                            var darker = new dojo.Color([ work.r*50/100|0, work.g*50/100|0, work.b*50/100|0 ]);
                            var dark = new dojo.Color([ work.r*90/100|0, work.g*90/100|0, work.b*90/100|0 ]);

						var x= offsets.l;
						var y= dim.height - offsets.b - vt(j + 1.5) + gap;
                        var off= this.opt.depth;


                        var apath = "M" + x           +  "," + y   +
                             " L" + (x+off)       +  "," + (y-off) +
                             " L" + (x+width+off) +  "," + (y-off) +
                             " L" + (x+width)   +  "," + (y) +
                             " L" + x           +  "," + y+ "z";


                        s.createPath(apath).setFill(dark);

                    if (i== (this.series.length-1)){
var h= height+this.opt.depth;
						apath = " M" + (x+width+off) +  "," + (y-off) +
							 " L" + (x+width+off) +  "," + (y+h-off) +
							 " L" + (x+width) +  "," + (y+h)+
							 " L" + (x+width) +  "," + y+" z";

					   s.createPath(apath).setFill(darker);
					}


                        var shape = s.createRect({
							x: offsets.l,
							y: dim.height - offsets.b - vt(j + 1.5) + gap,
							width: width, height: height+this.opt.depth
						}).setFill(specialFill);   //.setStroke(stroke);

							run.dyn.fill   = shape.getFill();
							run.dyn.stroke = shape.getStroke();
							if(events){
								var o = {
									element: "bar",
									index:   j,
									run:     run,
									shape:   shape,
									x:       v,
									y:       j + 1.5
								};
								this._connectEvents(o);
								eventSeries[j] = o;
							}

						}
					}
				}
				this._eventSeries[run.name] = eventSeries;
				run.dirty = false;
				// update the accumulator
				for(var j = 0; j < run.data.length; ++j){
					var value = run.data[j];
					if(value !== null){
						var v = typeof value == "number" ? value : value.y;
						if(isNaN(v)){ v = 0; }
						acc[j] -= v;
					}
				}
			}
			this.dirty = false;
			return this;	//	dojot.visual.StackedBarsEmbossed
		}
	});
})();

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.themes.ChartTheme"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.themes.ChartTheme"] = true;
dojo.provide("ibm.tivoli.tip.dijit.themes.ChartTheme");


(function(){
	var dxc=dojox.charting;
	ibm.tivoli.tip.dijit.themes.ChartTheme=new dxc.Theme({
		chart:{
			stroke:null,
			fill: "white"
		},
		plotarea:{
			stroke:null,
			fill: "#F0F0F0"
		},
		axis:{
			stroke:{ color:"white",width:1 },			
			line:{ color:"#fff",width:1 },
			majorTick:{ color:"#fff", width:2, length:12 },
			minorTick:{ color:"#fff", width:1, length:8 },
			font:"normal normal normal 8pt San Serif",
			fontColor:"black"
		},
		series:{
			outline:{ width:1, color:"#fff" },
			stroke:{ width:2, color:"#666" },
			fill:new dojo.Color([0x66, 0x66, 0x66, 0.8]),
			font:"normal normal normal 11pt San Serif",	//	label
			fontColor:"black"
		},
		marker:{	//	any markers on a series.
			stroke:{ width:2 },
			fill:"#333",
			font:"normal normal normal 11pt San Serif",	//	label
			fontColor:"black"
		},
		colors:[ "#497c91","#59a0bd","#9dc7d90","#c7e0e9","#7b78a4","#8d88c7","#ada9d6","#c9c6e4","#768b4e","#677e13","#a8c179","#c0d0a0","#b7b35c","#e8e667","#eeea99","#f0eebb","#b39c53","#e9c756","#ebcf81","#efdeb0","#956649","#b17044","#c28b69","#cfb09b","#815454","#a05a5a","#c99999","#ddc0c0","#868686","#a5a5a5","#bebebe","#d8d8d8"  
 ]
	});
	//Added fix for IBM Dojo 1.5 Migration
	//ibm.tivoli.tip.dijit.themes.ChartTheme.defineColors(ibm.tivoli.tip.dijit.themes.ChartTheme.colors);
})();

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandard"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandard"] = true;
dojo.provide("ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandard");


(function(){
	var dxc=dojox.charting;
    var dc = dojox.charting, themes = dc.themes, Theme = dc.Theme, g = Theme.generateGradient,
        defaultFill = {type: "linear", space: "shape", x1: 0, y1: 0, x2: 0, y2: 100};
	
	ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandardCustom = { 
		piestroke: {
			color: "black"
		},
		xlabels: {
			color: "black",
			fontFamily: "Sans Serif",
			size: "12pt", 
			weight: "normal"
		},
		ylabels: {
			color: "black",
			fontFamily: "Sans Serif",
			size: "14pt", 
			weight: "normal"
		},
		legend: {
			textClass: "dojoxLegendTextStandard"
		}
	};
	
	
	ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandard=new dxc.Theme({
		chart:{
			stroke:null,
			fill: "transparent"
		},
		plotarea:{
			stroke:{ color:"#EDEDED",width:0 },			
			line:{ color:"#EDEDED",width:0 },
			fill: "transparent"
		},
		axis:{
			stroke:{ color:"black",width:1 },			
			line:{ color:"black",width:1 },
			majorTick:{ color:"#BFBFBF", width:1, length:12 },
			minorTick:{ color:"#BFBFBF", width:1, length:8 },
			font:"normal normal normal 10pt San Serif",
			fontColor:"black"
		},
        series: {
            stroke:  {width: 2.5, color: "#eaf2cb"},
            outline: null,
            font: "normal normal normal 8pt Helvetica, Arial, sans-serif",
            fontColor: "black"
        },
        marker: {
            stroke:  {width: 1.25, color: "#eaf2cb"},
            outline: {width: 1.25, color: "#eaf2cb"},
            font: "normal normal normal 8pt Helvetica, Arial, sans-serif",
            fontColor: "black"
        },
//        seriesThemes: [
//            {fill: g(defaultFill, "#bf9e0a", "#ecc20c")},
//            {fill: g(defaultFill, "#73b086", "#95e5af")},   
//            {fill: g(defaultFill, "#c7212d", "#ed2835")},   
//            {fill: g(defaultFill, "#87ab41", "#b6e557")},   
//            {fill: g(defaultFill, "#b86c25", "#d37d2a")}    
//        ],
        markerThemes: [
            {fill: "#bf9e0a", stroke: {color: "#ecc20c"}},  
            {fill: "#73b086", stroke: {color: "#95e5af"}},
            {fill: "#c7212d", stroke: {color: "#ed2835"}},  
            {fill: "#87ab41", stroke: {color: "#b6e557"}},  
            {fill: "#b86c25", stroke: {color: "#d37d2a"}}   
        ],
		colors:[ "#205870", "#997fb2", "#f8d080", "#ffaa1c",  "#4ccc33", "#00005d", "#ccb2e5", "#d0a048", "#0758c4", "#c3c300", "#66667f", "#270e96", "#bcd2e8", "#986030", "#ffaa1c", "#205870", "#7faddc", "#99991d", "#997fb2", "#f8d080", "#153c05"
			]
//		colors:[ "#7f6699", "#e28d00", "#8bade7", "#4ccc33", "#00005d", "#ccb2e5", "#d0a048", "#0758c4", "#c3c300", "#66667f", "#270e96", "#bcd2e8", "#986030", "#ffaa1c", "#205870", "#7faddc", "#99991d", "#997fb2", "#f8d080", "#153c05"
// 			]
	});
	//Added fix for IBM Dojo 1.5 Migration
	//ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandard.defineColors(ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandard.colors);
	
    ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandardCustom.next = function(elementType, mixin, doPost){
        var isLine = elementType == "line";
        if(isLine || elementType == "area"){
            // custom processing for lines: substitute colors
            var s = this.seriesThemes[this._current % this.seriesThemes.length];
            s.fill.space = "plot";
            if(isLine){
                s.stroke  = { width: 4, color: s.fill.colors[0].color};
            }
            var theme = Theme.prototype.next.apply(this, arguments);
            // cleanup
            delete s.outline;
            delete s.stroke;
            s.fill.space = "shape";
            return theme;
        }
        return Theme.prototype.next.apply(this, arguments);
    };
    
    ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandardCustom.post = function(theme, elementType){
        theme = Theme.prototype.post.apply(this, arguments);
        if((elementType == "slice" || elementType == "circle") && theme.series.fill && theme.series.fill.type == "radial"){
            theme.series.fill = dojox.gfx.gradutils.reverse(theme.series.fill);
        }
        return theme;
    };
	
})();

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.themes.ChartThemeTivoliDark"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.themes.ChartThemeTivoliDark"] = true;
dojo.provide("ibm.tivoli.tip.dijit.themes.ChartThemeTivoliDark");


(function(){
	var dxc=dojox.charting;
	ibm.tivoli.tip.dijit.themes.ChartThemeTivoliDarkCustom = { 
		piestroke: {
			color: "white"
		},
		xlabels: {
			color: "white",
			fontFamily: "Sans Serif",
			size: "12pt", 
			weight: "normal"
		},
		ylabels: {
			color: "white",
			fontFamily: "Sans Serif",
			size: "14pt", 
			weight: "normal"
		},
		legend: {
			textClass: "dojoxLegendTextDark"
		}
	};
	
	ibm.tivoli.tip.dijit.themes.ChartThemeTivoliDark=new dxc.Theme({
		chart:{
			stroke:null,
			fill: "transparent"
		},
		plotarea:{
			stroke:null,
			fill: "transparent"
		},
		axis:{
			stroke:{ color:"white",width:1 },			
			line:{ color:"#fff",width:1 },
			font:"normal normal normal 10pt San Serif",
			majorTick:{ color:"#BFBFBF", width:1, length:12 },
			minorTick:{ color:"#BFBFBF", width:1, length:8 },
			fontColor:"white"
		},
		series:{
			outline:{ width:0, color:"#fff" },
			stroke:{ width:1, color:"#666" },
			//fill:new dojo.Color([0x66, 0x66, 0x66, 0.8]),
			fill:new dojo.Color([0x66, 0x66, 0x66, 1.0]),
			font:"normal normal normal 11pt San Serif",	//	label
			fontColor:"white"
		},
		marker:{	//	any markers on a series.
			stroke:{ width:2 },
			fill:"#3D4247",
			font:"normal normal normal 11pt San Serif",	//	label
			fontColor:"white"
		},
		colors:[ "#5684bc", "#68588c", "#d9913c", "#842e28", "#b39156", "#788030", "#0a2847", "#5d3080", "#855925", "#4d1915", "#a5792e", "#302212",
		         "#5684bc", "#68588c", "#d9913c", "#842e28", "#b39156", "#788030", "#0a2847", "#5d3080", "#855925"
 			]
	});
	//Added fix for IBM Dojo 1.5 Migration
	//ibm.tivoli.tip.dijit.themes.ChartThemeTivoliDark.defineColors(ibm.tivoli.tip.dijit.themes.ChartThemeTivoliDark.colors);
})();

}

if(!dojo._hasResource["dojo.fx.easing"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx.easing"] = true;
dojo.provide("dojo.fx.easing");

dojo.fx.easing = {
	// summary: 
	//		Collection of easing functions to use beyond the default 
	//		`dojo._defaultEasing` function.
	// 
	// description:
	//
	//		Easing functions are used to manipulate the iteration through
	//		an `dojo.Animation`s _Line. _Line being the properties of an Animation,
	//		and the easing function progresses through that Line determing
	//		how quickly (or slowly) it should go. Or more accurately: modify
	//		the value of the _Line based on the percentage of animation completed.
	//	
	//		All functions follow a simple naming convention of "ease type" + "when".
	//		If the name of the function ends in Out, the easing described appears
	//		towards the end of the animation. "In" means during the beginning,
	//		and InOut means both ranges of the Animation will applied, both
	//		beginning and end. 
	//
	//		One does not call the easing function directly, it must be passed to 
	//		the `easing` property of an animation.
	//
	//	example:
	//	|	
	//	|	var anim = dojo.fadeOut({
	//	|		node: 'node',	
	//	|		duration: 2000,
	//	|		//	note there is no () 
	//	|		easing: dojo.fx.easing.quadIn
	//	|	}).play();
	//
	
	linear: function(/* Decimal? */n){
		// summary: A linear easing function
		return n;
	},

	quadIn: function(/* Decimal? */n){
		return Math.pow(n, 2);
	},

	quadOut: function(/* Decimal? */n){
		return n * (n - 2) * -1;
	},

	quadInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 2) / 2; }
		return -1 * ((--n) * (n - 2) - 1) / 2;
	},

	cubicIn: function(/* Decimal? */n){
		return Math.pow(n, 3);
	},

	cubicOut: function(/* Decimal? */n){
		return Math.pow(n - 1, 3) + 1;
	},

	cubicInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 3) / 2; }
		n -= 2;
		return (Math.pow(n, 3) + 2) / 2;
	},

	quartIn: function(/* Decimal? */n){
		return Math.pow(n, 4);
	},

	quartOut: function(/* Decimal? */n){
		return -1 * (Math.pow(n - 1, 4) - 1);
	},

	quartInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 4) / 2; }
		n -= 2;
		return -1 / 2 * (Math.pow(n, 4) - 2);
	},

	quintIn: function(/* Decimal? */n){
		return Math.pow(n, 5);
	},

	quintOut: function(/* Decimal? */n){
		return Math.pow(n - 1, 5) + 1;
	},

	quintInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 5) / 2; };
		n -= 2;
		return (Math.pow(n, 5) + 2) / 2;
	},

	sineIn: function(/* Decimal? */n){
		return -1 * Math.cos(n * (Math.PI / 2)) + 1;
	},

	sineOut: function(/* Decimal? */n){
		return Math.sin(n * (Math.PI / 2));
	},

	sineInOut: function(/* Decimal? */n){
		return -1 * (Math.cos(Math.PI * n) - 1) / 2;
	},

	expoIn: function(/* Decimal? */n){
		return (n == 0) ? 0 : Math.pow(2, 10 * (n - 1));
	},

	expoOut: function(/* Decimal? */n){
		return (n == 1) ? 1 : (-1 * Math.pow(2, -10 * n) + 1);
	},

	expoInOut: function(/* Decimal? */n){
		if(n == 0){ return 0; }
		if(n == 1){ return 1; }
		n = n * 2;
		if(n < 1){ return Math.pow(2, 10 * (n - 1)) / 2; }
		--n;
		return (-1 * Math.pow(2, -10 * n) + 2) / 2;
	},

	circIn: function(/* Decimal? */n){
		return -1 * (Math.sqrt(1 - Math.pow(n, 2)) - 1);
	},

	circOut: function(/* Decimal? */n){
		n = n - 1;
		return Math.sqrt(1 - Math.pow(n, 2));
	},

	circInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return -1 / 2 * (Math.sqrt(1 - Math.pow(n, 2)) - 1); }
		n -= 2;
		return 1 / 2 * (Math.sqrt(1 - Math.pow(n, 2)) + 1);
	},

	backIn: function(/* Decimal? */n){
		// summary: 
		//		An easing function that starts away from the target, 
		//		and quickly accelerates towards the end value.
		// 
		//		Use caution when the easing will cause values to become 
		//		negative as some properties cannot be set to negative values.
		var s = 1.70158;
		return Math.pow(n, 2) * ((s + 1) * n - s);
	},

	backOut: function(/* Decimal? */n){
		// summary: 
		//		An easing function that pops past the range briefly, and slowly comes back. 
		//
		// description:
		//		An easing function that pops past the range briefly, and slowly comes back. 
		//
		//		Use caution when the easing will cause values to become negative as some 
		//		properties cannot be set to negative values.
		
		n = n - 1;
		var s = 1.70158;
		return Math.pow(n, 2) * ((s + 1) * n + s) + 1;
	},

	backInOut: function(/* Decimal? */n){
		// summary: 
		//		An easing function combining the effects of `backIn` and `backOut`
		//
		// description:
		//		An easing function combining the effects of `backIn` and `backOut`.
		//		Use caution when the easing will cause values to become negative 
		//		as some properties cannot be set to negative values.
		var s = 1.70158 * 1.525;
		n = n * 2;
		if(n < 1){ return (Math.pow(n, 2) * ((s + 1) * n - s)) / 2; }
		n-=2;
		return (Math.pow(n, 2) * ((s + 1) * n + s) + 2) / 2;
	},

	elasticIn: function(/* Decimal? */n){
		// summary: 
		//		An easing function the elastically snaps from the start value
		//
		// description:
		//		An easing function the elastically snaps from the start value
		//	
		//		Use caution when the elasticity will cause values to become negative 
		//		as some properties cannot be set to negative values.
		if(n == 0 || n == 1){ return n; }
		var p = .3;
		var s = p / 4;
		n = n - 1;
		return -1 * Math.pow(2, 10 * n) * Math.sin((n - s) * (2 * Math.PI) / p);
	},

	elasticOut: function(/* Decimal? */n){
		// summary: 
		//		An easing function that elasticly snaps around the target value,
		//		near the end of the Animation
		//
		// description:
		//		An easing function that elasticly snaps around the target value,
		//		near the end of the Animation
		//
		//		Use caution when the elasticity will cause values to become 
		//		negative as some properties cannot be set to negative values.
		if(n==0 || n == 1){ return n; }
		var p = .3;
		var s = p / 4;
		return Math.pow(2, -10 * n) * Math.sin((n - s) * (2 * Math.PI) / p) + 1;
	},

	elasticInOut: function(/* Decimal? */n){
		// summary: 
		//		An easing function that elasticly snaps around the value, near
		//		the beginning and end of the Animation.
		//
		// description:
		//		An easing function that elasticly snaps around the value, near
		//		the beginning and end of the Animation.
		//
		//		Use caution when the elasticity will cause values to become 
		//		negative as some properties cannot be set to negative values.
		if(n == 0) return 0;
		n = n * 2;
		if(n == 2) return 1;
		var p = .3 * 1.5;
		var s = p / 4;
		if(n < 1){
			n -= 1;
			return -.5 * (Math.pow(2, 10 * n) * Math.sin((n - s) * (2 * Math.PI) / p));
		}
		n -= 1;
		return .5 * (Math.pow(2, -10 * n) * Math.sin((n - s) * (2 * Math.PI) / p)) + 1;
	},

	bounceIn: function(/* Decimal? */n){
		// summary: 
		//		An easing function that 'bounces' near the beginning of an Animation
		return (1 - dojo.fx.easing.bounceOut(1 - n)); // Decimal
	},

	bounceOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that 'bounces' near the end of an Animation
		var s = 7.5625;
		var p = 2.75;
		var l; 
		if(n < (1 / p)){
			l = s * Math.pow(n, 2);
		}else if(n < (2 / p)){
			n -= (1.5 / p);
			l = s * Math.pow(n, 2) + .75;
		}else if(n < (2.5 / p)){
			n -= (2.25 / p);
			l = s * Math.pow(n, 2) + .9375;
		}else{
			n -= (2.625 / p);
			l = s * Math.pow(n, 2) + .984375;
		}
		return l;
	},

	bounceInOut: function(/* Decimal? */n){
		// summary: 
		//		An easing function that 'bounces' at the beginning and end of the Animation
		if(n < 0.5){ return dojo.fx.easing.bounceIn(n * 2) / 2; }
		return (dojo.fx.easing.bounceOut(n * 2 - 1) / 2) + 0.5; // Decimal
	}
};

}

if(!dojo._hasResource["dojox.charting.action2d.Base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.action2d.Base"] = true;
dojo.provide("dojox.charting.action2d.Base");





/*=====
dojox.charting.action2d.__BaseCtorArgs = function(duration, easing){
 	//	summary:
	//		The base keyword arguments object for creating an action2d.
	//	duration: Number?
	//		The amount of time in milliseconds for an animation to last.  Default is 400.
	//	easing: dojox.fx.easing.*?
	//		An easing object (see dojo.fx.easing) for use in an animation.  The
	//		default is dojox.fx.easing.backOut.
	this.duration = duration;
	this.easing = easing;
}
=====*/
(function(){
	var DEFAULT_DURATION = 400,	// ms
		DEFAULT_EASING   = dojo.fx.easing.backOut,
		df = dojox.lang.functional;

	dojo.declare("dojox.charting.action2d.Base", null, {

		overOutEvents: {onmouseover: 1, onmouseout: 1},

		constructor: function(chart, plot, kwargs){
			//	summary:
			//		Create a new base Action.
			//	chart: dojox.charting.Chart2D
			//		The chart this action applies to.
			//	plot: String?
			//		The name of the plot this action belongs to.  If none is passed "default" is assumed.
			//	kwargs: dojox.charting.action2d.__BaseCtorArgs?
			//		Optional arguments for the action.
			this.chart = chart;
			this.plot = plot || "default";
			this.anim = {};

			// process common optional named parameters
			if(!kwargs){ kwargs = {}; }
			this.duration = kwargs.duration ? kwargs.duration : DEFAULT_DURATION;
			this.easing   = kwargs.easing   ? kwargs.easing   : DEFAULT_EASING;
		},

		connect: function(){
			//	summary:
			//		Connect this action to the given plot.
			this.handle = this.chart.connectToPlot(this.plot, this, "process");
		},

		disconnect: function(){
			//	summary:
			//		Disconnect this action from the given plot, if connected.
			if(this.handle){
				dojo.disconnect(this.handle);
				this.handle = null;
			}
		},

		reset: function(){
			//	summary:
			//		Reset the action.
		},

		destroy: function(){
			//	summary:
			//		Do any cleanup needed when destroying parent elements.
			this.disconnect();
			df.forIn(this.anim, function(o){
				df.forIn(o, function(anim){
					anim.action.stop(true);
				});
			});
			this.anim = {};
		}
	});
})();

}

if(!dojo._hasResource["dojox.lang.functional.scan"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.functional.scan"] = true;
dojo.provide("dojox.lang.functional.scan");



// This module adds high-level functions and related constructs:
//	- "scan" family of functions

// Notes:
//	- missing high-level functions are provided with the compatible API: 
//		scanl, scanl1, scanr, scanr1

// Defined methods:
//	- take any valid lambda argument as the functional argument
//	- operate on dense arrays
//	- take a string as the array argument
//	- take an iterator objects as the array argument (only scanl, and scanl1)

(function(){
	var d = dojo, df = dojox.lang.functional, empty = {};

	d.mixin(df, {
		// classic reduce-class functions
		scanl: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from left 
			//	to right using a seed value as a starting point; returns an array
			//	of values produced by foldl() at that point.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var t, n, i;
			if(d.isArray(a)){
				// array
				t = new Array((n = a.length) + 1);
				t[0] = z;
				for(i = 0; i < n; z = f.call(o, z, a[i], i, a), t[++i] = z);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				t = [z];
				for(i = 0; a.hasNext(); t.push(z = f.call(o, z, a.next(), i++, a)));
			}else{
				// object/dictionary
				t = [z];
				for(i in a){
					if(!(i in empty)){
						t.push(z = f.call(o, z, a[i], i, a));
					}
				}
			}
			return t;	// Array
		},
		scanl1: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from left 
			//	to right; returns an array of values produced by foldl1() at that 
			//	point.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var t, n, z, first = true;
			if(d.isArray(a)){
				// array
				t = new Array(n = a.length);
				t[0] = z = a[0];
				for(var i = 1; i < n; t[i] = z = f.call(o, z, a[i], i, a), ++i);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				if(a.hasNext()){
					t = [z = a.next()];
					for(var i = 1; a.hasNext(); t.push(z = f.call(o, z, a.next(), i++, a)));
				}
			}else{
				// object/dictionary
				for(var i in a){
					if(!(i in empty)){
						if(first){
							t = [z = a[i]];
							first = false;
						}else{
							t.push(z = f.call(o, z, a[i], i, a));
						}
					}
				}
			}
			return t;	// Array
		},
		scanr: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from right
			//	to left using a seed value as a starting point; returns an array
			//	of values produced by foldr() at that point.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var n = a.length, t = new Array(n + 1), i = n;
			t[n] = z;
			for(; i > 0; --i, z = f.call(o, z, a[i], i, a), t[i] = z);
			return t;	// Array
		},
		scanr1: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from right
			//	to left; returns an array of values produced by foldr1() at that 
			//	point.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var n = a.length, t = new Array(n), z = a[n - 1], i = n - 1;
			t[i] = z;
			for(; i > 0; --i, z = f.call(o, z, a[i], i, a), t[i] = z);
			return t;	// Array
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.action2d.Tooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.action2d.Tooltip"] = true;
dojo.provide("dojox.charting.action2d.Tooltip");









/*=====
dojo.declare("dojox.charting.action2d.__TooltipCtorArgs", dojox.charting.action2d.__BaseCtorArgs, {
	//	summary:
	//		Additional arguments for tooltip actions.

	//	text: Function?
	//		The function that produces the text to be shown within a tooltip.  By default this will be 
	//		set by the plot in question, by returning the value of the element.
	text: null
});
=====*/
(function(){
	var DEFAULT_TEXT = function(o){
		var t = o.run && o.run.data && o.run.data[o.index];
		if(t && typeof t != "number" && (t.tooltip || t.text)){
			return t.tooltip || t.text;
		}
		if(o.element == "candlestick"){
			return '<table cellpadding="1" cellspacing="0" border="0" style="font-size:0.9em;">'
				+ '<tr><td>Open:</td><td align="right"><strong>' + o.data.open + '</strong></td></tr>'
				+ '<tr><td>High:</td><td align="right"><strong>' + o.data.high + '</strong></td></tr>'
				+ '<tr><td>Low:</td><td align="right"><strong>' + o.data.low + '</strong></td></tr>'
				+ '<tr><td>Close:</td><td align="right"><strong>' + o.data.close + '</strong></td></tr>'
				+ (o.data.mid !== undefined ? '<tr><td>Mid:</td><td align="right"><strong>' + o.data.mid + '</strong></td></tr>' : '')
				+ '</table>';
		}
		return o.element == "bar" ? o.x : o.y;
	};

	var df = dojox.lang.functional, m = dojox.gfx.matrix, pi4 = Math.PI / 4, pi2 = Math.PI / 2;
	
	dojo.declare("dojox.charting.action2d.Tooltip", dojox.charting.action2d.Base, {
		//	summary:
		//		Create an action on a plot where a tooltip is shown when hovering over an element.

		// the data description block for the widget parser
		defaultParams: {
			text: DEFAULT_TEXT	// the function to produce a tooltip from the object
		},
		optionalParams: {},	// no optional parameters

		constructor: function(chart, plot, kwArgs){
			//	summary:
			//		Create the tooltip action and connect it to the plot.
			//	chart: dojox.charting.Chart2D
			//		The chart this action belongs to.
			//	plot: String?
			//		The plot this action is attached to.  If not passed, "default" is assumed.
			//	kwArgs: dojox.charting.action2d.__TooltipCtorArgs?
			//		Optional keyword arguments object for setting parameters.
			this.text = kwArgs && kwArgs.text ? kwArgs.text : DEFAULT_TEXT;
			
			this.connect();
		},
		
		process: function(o){
			//	summary:
			//		Process the action on the given object.
			//	o: dojox.gfx.Shape
			//		The object on which to process the highlighting action.
			if(o.type === "onplotreset" || o.type === "onmouseout"){
				hideChartingTooltip(this.aroundRect);
				this.aroundRect = null;
				return;
			}
			
			if(!o.shape || o.type !== "onmouseover"){ return; }
			
			// calculate relative coordinates and the position
			var aroundRect = {type: "rect"}, position = ["after", "before"];
			switch(o.element){
				case "marker":
					aroundRect.x = o.cx;
					aroundRect.y = o.cy;
					aroundRect.width = aroundRect.height = 1;
					break;
				case "circle":
					aroundRect.x = o.cx - o.cr;
					aroundRect.y = o.cy - o.cr;
					aroundRect.width = aroundRect.height = 2 * o.cr;
					break;
				case "column":
					position = ["above", "below"];
					// intentional fall down
				case "bar":
					aroundRect = dojo.clone(o.shape.getShape());
					break;
				case "candlestick":
					aroundRect.x = o.x;
					aroundRect.y = o.y;
					aroundRect.width = o.width;
					aroundRect.height = o.height;
					break;
				default:
				//case "slice":
					if(!this.angles){
						// calculate the running total of slice angles
						if(typeof o.run.data[0] == "number"){
							this.angles = df.map(df.scanl(o.run.data, "+", 0),
								"* 2 * Math.PI / this", df.foldl(o.run.data, "+", 0));
						}else{
							this.angles = df.map(df.scanl(o.run.data, "a + b.y", 0),
								"* 2 * Math.PI / this", df.foldl(o.run.data, "a + b.y", 0));
						}
					}
					var startAngle = m._degToRad(o.plot.opt.startAngle),
						angle = (this.angles[o.index] + this.angles[o.index + 1]) / 2 + startAngle;
					aroundRect.x = o.cx + o.cr * Math.cos(angle);
					aroundRect.y = o.cy + o.cr * Math.sin(angle);
					aroundRect.width = aroundRect.height = 1;
					// calculate the position
					if(angle < pi4){
						// do nothing: the position is right
					}else if(angle < pi2 + pi4){
						position = ["below", "above"];
					}else if(angle < Math.PI + pi4){
						position = ["before", "after"];
					}else if(angle < 2 * Math.PI - pi4){
						position = ["above", "below"];
					}
					/*
					else{
						// do nothing: the position is right
					}
					*/
					break;
			}
			
			// adjust relative coordinates to absolute, and remove fractions
			var lt = dojo.coords(this.chart.node, true);
			aroundRect.x += lt.x;
			aroundRect.y += lt.y;
			aroundRect.x = Math.round(aroundRect.x);
			aroundRect.y = Math.round(aroundRect.y);
			aroundRect.width = Math.ceil(aroundRect.width);
			aroundRect.height = Math.ceil(aroundRect.height);
			this.aroundRect = aroundRect;
			
			showChartingTooltip(this.text(o), this.aroundRect, position, "center");
		}
	});
	
	//TODO: make the charting tooltip a generic one so it can be used with
	// dojox.gfx shapes without charting.
	
	var MasterTooltip = dojo.declare(dijit._MasterTooltip, {
		
		show: function(/*String*/ innerHTML, /*DomNode*/ aroundNode, /*String[]?*/ position, /*String*/alignment){
			// summary:
			//		Display tooltip w/specified contents to right of specified node
			//		(To left if there's no space on the right, or if LTR==right)
			// alignment: String
			//		"center":   tooltip center alignment
			//      "default":  tooltip default alignment (left, right, top, bottom)

			if(this.aroundNode && this.aroundNode === aroundNode){
				return;
			}

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;

			// Firefox bug. when innerHTML changes to be shorter than previous
			// one, the node size will not be updated until it moves.
			this.domNode.style.top = (this.domNode.offsetTop + 1) + "px";
			
			if(!this.connectorNode) this.connectorNode = dojo.query(".dijitTooltipConnector", this.domNode)[0];
			var connectorPos = dojo.coords(this.connectorNode);
			this.arrowWidth = connectorPos.w, this.arrowHeight = connectorPos.h;
			this.place = (alignment && alignment == "center") ? this.placeChartingTooltip : dijit.placeOnScreenAroundElement,
			this.place(
				this.domNode,
				aroundNode,
				dijit.getPopupAroundAlignment(
					(position && position.length) ? position : dijit.Tooltip.defaultPosition,
					this.isLeftToRight()
				),
				dojo.hitch(this, "orient")
			);

			// show it
			dojo.style(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},
		
		placeChartingTooltip: function(node, aroundRect, aroundCorners, layoutNode){
			return this._placeOnScreenAroundRect(node, 
				aroundRect.x, aroundRect.y, aroundRect.width, aroundRect.height,	// rectangle
				aroundCorners, layoutNode);
		},
		
		_placeOnScreenAroundRect: function(node, x, y, width, height, aroundCorners, layoutNode){
			var choices = [];
		    for (var nodeCorner in aroundCorners) {
		        choices.push({
		            aroundCorner: nodeCorner,
		            corner: aroundCorners[nodeCorner],
		            pos: {
		                x: x + (nodeCorner.charAt(1) == 'L' ? 0 : width),
		                y: y + (nodeCorner.charAt(0) == 'T' ? 0 : height),
		                w: width,
		                h: height
		            }
		        });
		    }
		    return this._place(node, choices, layoutNode);
		},
		
		_place: function(/* DomNode */node, /* Array */ choices, /* Function */ layoutNode){
		    var view = dijit.getViewport();
		    
		    if (!node.parentNode ||
		    String(node.parentNode.tagName).toLowerCase() != "body") {
		        dojo.body().appendChild(node);
		    }
		    
		    var best = null;
		    
		    var arrowLeft = null, arrowTop = null;
		    dojo.some(choices, function(choice){
		        var corner = choice.corner;
		        var aroundCorner = choice.aroundCorner;
		        var pos = choice.pos;
		        
		        if (layoutNode) {
		            layoutNode(node, choice.aroundCorner, corner);
		        }
		        
		        // get node's size
		        var style = node.style;
		        var oldDisplay = style.display;
		        var oldVis = style.visibility;
		        style.visibility = "hidden";
		        style.display = "";
		        var mb = dojo.marginBox(node);
		        style.display = oldDisplay;
		        style.visibility = oldVis;
		        
		        var startX, startY, endX, endY, width, height, overflow;
		        
		        arrowLeft = null, arrowTop = null;
		        if (aroundCorner.charAt(0) == corner.charAt(0)) { //left, right
		            startX = (corner.charAt(1) == 'L' ? pos.x : Math.max(view.l, pos.x - mb.w)), 
					startY = (corner.charAt(0) == 'T' ? (pos.y + pos.h / 2 - mb.h / 2) : (pos.y - pos.h / 2 - mb.h / 2)), 
					endX = (corner.charAt(1) == 'L' ? Math.min(view.l + view.w, startX + mb.w) : pos.x), 
					endY = startY + mb.h, 
					width = endX - startX, 
					height = endY - startY, 
					overflow = (mb.w - width) + (mb.h - height);
		            arrowTop = (mb.h - this.arrowHeight) / 2;
		        }
		        else { //top, bottom
		            startX = (corner.charAt(1) == 'L' ? (pos.x + pos.w / 2 - mb.w / 2) : (pos.x - pos.w / 2 - mb.w / 2)), 
					startY = (corner.charAt(0) == 'T' ? pos.y : Math.max(view.t, pos.y - mb.h)), 
					endX = startX + mb.w, 
					endY = (corner.charAt(0) == 'T' ? Math.min(view.t + view.h, startY + mb.h) : pos.y), 
					width = endX - startX, 
					height = endY - startY, 
					overflow = (mb.w - width) + (mb.h - height);
		            arrowLeft = (mb.w - this.arrowWidth) / 2;
		        }
		        
		        if (best == null || overflow < best.overflow) {
		            best = {
		                corner: corner,
		                aroundCorner: choice.aroundCorner,
		                x: startX,
		                y: startY,
		                w: width,
		                h: height,
		                overflow: overflow
		            };
		        }
		        return !overflow;
		    }, this);
		    
		    node.style.left = best.x + "px";
		    node.style.top = best.y + "px";
		    
			this.connectorNode.style.top = "";
	        this.connectorNode.style.left = "";
	        if (arrowTop){
				this.connectorNode.style.top = arrowTop + "px";
			}
	        if (arrowLeft){
				this.connectorNode.style.left = arrowLeft + "px";
			}
	        
		    if (best.overflow && layoutNode) {
		        layoutNode(node, best.aroundCorner, best.corner);
		    }
		    return best;
		}
	});
	
	var masterTT = null;
	
	function showChartingTooltip(innerHTML, aroundNode, position, alignment){
		if(!masterTT){ masterTT = new MasterTooltip(); }
		return masterTT.show(innerHTML, aroundNode, position, alignment);
	}
	
	function hideChartingTooltip(aroundNode){
		if(!masterTT){ masterTT = new MasterTooltip(); }
		return masterTT.hide(aroundNode);
	}
})();

}

if(!dojo._hasResource["dojox.charting.widget.Legend"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.widget.Legend"] = true;
dojo.provide("dojox.charting.widget.Legend");







dojo.declare("dojox.charting.widget.Legend", [dijit._Widget, dijit._Templated], {
	// summary: A legend for a chart. A legend contains summary labels for 
	// each series of data contained in the chart.
	//
	// Set the horizontal attribute to boolean false to layout legend labels vertically.
	// Set the horizontal attribute to a number to layout legend labels in horizontal
	// rows each containing that number of labels (except possibly the last row).
	//
	// (Line or Scatter charts (colored lines with shape symbols) )
	// -o- Series1		-X- Series2		-v- Series3
	//
	// (Area/Bar/Pie charts (letters represent colors))
	// [a] Series1		[b] Series2		[c] Series3
	
	chartRef:   "",
	horizontal: true,
	swatchSize: 18,
	
	templateString: "<table dojoAttachPoint='legendNode' class='dojoxLegendNode'><tbody dojoAttachPoint='legendBody'></tbody></table>",
	
	legendNode: null,
	legendBody: null,
	
	postCreate: function(){
		if(!this.chart){
			if(!this.chartRef){ return; }
			this.chart = dijit.byId(this.chartRef);
			if(!this.chart){
				var node = dojo.byId(this.chartRef);
				if(node){
					this.chart = dijit.byNode(node);
				}else{
					console.log("Could not find chart instance with id: " + this.chartRef);
					return;
				}
			}
			this.series = this.chart.chart.series;
		}else{
			this.series = this.chart.series;
		}
		
		this.refresh();
	},
	refresh: function(){
		// summary: regenerates the legend to reflect changes to the chart
		
		var df = dojox.lang.functional;

		// cleanup
		if(this._surfaces){
			dojo.forEach(this._surfaces, function(surface){
				surface.destroy();
			});
		}
		this._surfaces = [];
		while(this.legendBody.lastChild){
			dojo.destroy(this.legendBody.lastChild);
		}

		if(this.horizontal){
			dojo.addClass(this.legendNode, "dojoxLegendHorizontal");
			// make a container <tr>
			this._tr = dojo.doc.createElement("tr");
			this.legendBody.appendChild(this._tr);
			this._inrow = 0;
		}
		
		var s = this.series;
		if(s.length == 0){
			return;
		}
		if(s[0].chart.stack[0].declaredClass == "dojox.charting.plot2d.Pie"){
			var t = s[0].chart.stack[0];
			if(typeof t.run.data[0] == "number"){
				var filteredRun = df.map(t.run.data, "Math.max(x, 0)");
				if(df.every(filteredRun, "<= 0")){
					return;
				}
				var slices = df.map(filteredRun, "/this", df.foldl(filteredRun, "+", 0));
				dojo.forEach(slices, function(x, i){
					this._addLabel(t.dyn[i], t._getLabel(x * 100) + "%");
				}, this);
			}else{
				dojo.forEach(t.run.data, function(x, i){
					this._addLabel(t.dyn[i], x.legend || x.text || x.y);
				}, this);
			}
		}else{
			dojo.forEach(s, function(x){
				this._addLabel(x.dyn, x.legend || x.name);
			}, this);
		}
	},
	_addLabel: function(dyn, label){
		// create necessary elements
		var icon = dojo.doc.createElement("td"),
			text = dojo.doc.createElement("td"),
			div  = dojo.doc.createElement("div");
		dojo.addClass(icon, "dojoxLegendIcon");
		dojo.addClass(text, "dojoxLegendText");
		div.style.width  = this.swatchSize + "px";
		div.style.height = this.swatchSize + "px";
		icon.appendChild(div);
		
		// create a skeleton
		if(this._tr){
			// horizontal
			this._tr.appendChild(icon);
			this._tr.appendChild(text);
			if(++this._inrow === this.horizontal){
				// make a fresh container <tr>
				this._tr = dojo.doc.createElement("tr");
				this.legendBody.appendChild(this._tr);
				this._inrow = 0;
			}
		}else{
			// vertical
			var tr = dojo.doc.createElement("tr");
			this.legendBody.appendChild(tr);
			tr.appendChild(icon);
			tr.appendChild(text);
		}
		
		// populate the skeleton
		this._makeIcon(div, dyn);
		text.innerHTML = String(label);
	},
	_makeIcon: function(div, dyn){
		var mb = { h: this.swatchSize, w: this.swatchSize };
		var surface = dojox.gfx.createSurface(div, mb.w, mb.h);
		this._surfaces.push(surface);
		if(dyn.fill){
			// regions
			surface.createRect({x: 2, y: 2, width: mb.w - 4, height: mb.h - 4}).
				setFill(dyn.fill).setStroke(dyn.stroke);
		}else if(dyn.stroke || dyn.marker){
			// draw line
			var line = {x1: 0, y1: mb.h / 2, x2: mb.w, y2: mb.h / 2};
			if(dyn.stroke){
				surface.createLine(line).setStroke(dyn.stroke);
			}
			if(dyn.marker){
				// draw marker on top
				var c = {x: mb.w / 2, y: mb.h / 2};
				if(dyn.stroke){
					surface.createPath({path: "M" + c.x + " " + c.y + " " + dyn.marker}).
						setFill(dyn.stroke.color).setStroke(dyn.stroke);
				}else{
					surface.createPath({path: "M" + c.x + " " + c.y + " " + dyn.marker}).
						setFill(dyn.color).setStroke(dyn.color);
				}
			}
		}else{
			// nothing
			surface.createRect({x: 2, y: 2, width: mb.w - 4, height: mb.h - 4}).
				setStroke("black");
			surface.createLine({x1: 2, y1: 2, x2: mb.w - 2, y2: mb.h - 2}).setStroke("black");
			surface.createLine({x1: 2, y1: mb.h - 2, x2: mb.w - 2, y2: 2}).setStroke("black");
		}
	}
});

}

if(!dojo._hasResource["dojox.charting.action2d.MoveSlice"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.action2d.MoveSlice"] = true;
dojo.provide("dojox.charting.action2d.MoveSlice");








/*=====
dojo.declare("dojox.charting.action2d.__MoveSliceCtorArgs", dojox.charting.action2d.__BaseCtorArgs, {
	//	summary:
	//		Additional arguments for highlighting actions.

	//	scale: Number?
	//		The amount to scale the pie slice.  Default is 1.05.
	scale: 1.05,

	//	shift: Number?
	//		The amount in pixels to shift the pie slice.  Default is 7.
	shift: 7
});
=====*/
(function(){
	var DEFAULT_SCALE = 1.05,
		DEFAULT_SHIFT = 7,	// px
		m = dojox.gfx.matrix,
		gf = dojox.gfx.fx,
		df = dojox.lang.functional;

	dojo.declare("dojox.charting.action2d.MoveSlice", dojox.charting.action2d.Base, {
		//	summary:
		//		Create an action for a pie chart that moves and scales a pie slice.

		// the data description block for the widget parser
		defaultParams: {
			duration: 400,	// duration of the action in ms
			easing:   dojo.fx.easing.backOut,	// easing for the action
			scale:    DEFAULT_SCALE,	// scale of magnification
			shift:    DEFAULT_SHIFT		// shift of the slice
		},
		optionalParams: {},	// no optional parameters

		constructor: function(chart, plot, kwArgs){
			//	summary:
			//		Create the slice moving action and connect it to the plot.
			//	chart: dojox.charting.Chart2D
			//		The chart this action belongs to.
			//	plot: String?
			//		The plot this action is attached to.  If not passed, "default" is assumed.
			//	kwArgs: dojox.charting.action2d.__MoveSliceCtorArgs?
			//		Optional keyword arguments object for setting parameters.
			if(!kwArgs){ kwArgs = {}; }
			this.scale = typeof kwArgs.scale == "number" ? kwArgs.scale : DEFAULT_SCALE;
			this.shift = typeof kwArgs.shift == "number" ? kwArgs.shift : DEFAULT_SHIFT;

			this.connect();
		},

		process: function(o){
			//	summary:
			//		Process the action on the given object.
			//	o: dojox.gfx.Shape
			//		The object on which to process the slice moving action.
			if(!o.shape || o.element != "slice" || !(o.type in this.overOutEvents)){ return; }

			if(!this.angles){
				// calculate the running total of slice angles
				var startAngle = m._degToRad(o.plot.opt.startAngle);
				if(typeof o.run.data[0] == "number"){
					this.angles = df.map(df.scanl(o.run.data, "+", startAngle),
						"* 2 * Math.PI / this", df.foldl(o.run.data, "+", 0));
				}else{
					this.angles = df.map(df.scanl(o.run.data, "a + b.y", startAngle),
						"* 2 * Math.PI / this", df.foldl(o.run.data, "a + b.y", 0));
				}
			}

			var index = o.index, anim, startScale, endScale, startOffset, endOffset,
				angle = (this.angles[index] + this.angles[index + 1]) / 2,
				rotateTo0  = m.rotateAt(-angle, o.cx, o.cy),
				rotateBack = m.rotateAt( angle, o.cx, o.cy);

			anim = this.anim[index];

			if(anim){
				anim.action.stop(true);
			}else{
				this.anim[index] = anim = {};
			}

			if(o.type == "onmouseover"){
				startOffset = 0;
				endOffset   = this.shift;
				startScale  = 1;
				endScale    = this.scale;
			}else{
				startOffset = this.shift;
				endOffset   = 0;
				startScale  = this.scale;
				endScale    = 1;
			}

			anim.action = dojox.gfx.fx.animateTransform({
				shape:    o.shape,
				duration: this.duration,
				easing:   this.easing,
				transform: [
					rotateBack,
					{name: "translate", start: [startOffset, 0], end: [endOffset, 0]},
					{name: "scaleAt",   start: [startScale, o.cx, o.cy],  end: [endScale, o.cx, o.cy]},
					rotateTo0
				]
			});

			if(o.type == "onmouseout"){
				dojo.connect(anim.action, "onEnd", this, function(){
					delete this.anim[index];
				});
			}
			anim.action.play();
		},

		reset: function(){
			delete this.angles;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.action2d.Magnify"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.action2d.Magnify"] = true;
dojo.provide("dojox.charting.action2d.Magnify");





/*=====
dojo.declare("dojox.charting.action2d.__MagnifyCtorArgs", dojox.charting.action2d.__BaseCtorArgs, {
	//	summary:
	//		Additional arguments for highlighting actions.

	//	scale: Number?
	//		The amount to magnify the given object to.  Default is 2.
	scale: 2
});
=====*/
(function(){
	var DEFAULT_SCALE = 2,
		m = dojox.gfx.matrix,
		gf = dojox.gfx.fx;

	dojo.declare("dojox.charting.action2d.Magnify", dojox.charting.action2d.Base, {
		//	summary:
		//		Create an action that magnifies the object the action is applied to.

		// the data description block for the widget parser
		defaultParams: {
			duration: 400,	// duration of the action in ms
			easing:   dojo.fx.easing.backOut,	// easing for the action
			scale:    DEFAULT_SCALE	// scale of magnification
		},
		optionalParams: {},	// no optional parameters

		constructor: function(chart, plot, kwArgs){
			//	summary:
			//		Create the magnifying action.
			//	chart: dojox.charting.Chart2D
			//		The chart this action belongs to.
			//	plot: String?
			//		The plot to apply the action to. If not passed, "default" is assumed.
			//	kwArgs: dojox.charting.action2d.__MagnifyCtorArgs?
			//		Optional keyword arguments for this action.

			// process optional named parameters
			this.scale = kwArgs && typeof kwArgs.scale == "number" ? kwArgs.scale : DEFAULT_SCALE;

			this.connect();
		},

		process: function(o){
			//	summary:
			//		Process the action on the given object.
			//	o: dojox.gfx.Shape
			//		The object on which to process the magnifying action.
			if(!o.shape || !(o.type in this.overOutEvents) ||
				!("cx" in o) || !("cy" in o)){ return; }

			var runName = o.run.name, index = o.index, vector = [], anim, init, scale;

			if(runName in this.anim){
				anim = this.anim[runName][index];
			}else{
				this.anim[runName] = {};
			}

			if(anim){
				anim.action.stop(true);
			}else{
				this.anim[runName][index] = anim = {};
			}

			if(o.type == "onmouseover"){
				init  = m.identity;
				scale = this.scale;
			}else{
				init  = m.scaleAt(this.scale, o.cx, o.cy);
				scale = 1 / this.scale;
			}

			var kwArgs = {
				shape:    o.shape,
				duration: this.duration,
				easing:   this.easing,
				transform: [
					{name: "scaleAt", start: [1, o.cx, o.cy], end: [scale, o.cx, o.cy]},
					init
				]
			};
			if(o.shape){
				vector.push(gf.animateTransform(kwArgs));
			}
			if(o.oultine){
				kwArgs.shape = o.outline;
				vector.push(gf.animateTransform(kwArgs));
			}
			if(o.shadow){
				kwArgs.shape = o.shadow;
				vector.push(gf.animateTransform(kwArgs));
			}

			if(!vector.length){
				delete this.anim[runName][index];
				return;
			}

			anim.action = dojo.fx.combine(vector);
			if(o.type == "onmouseout"){
				dojo.connect(anim.action, "onEnd", this, function(){
					if(this.anim[runName]){
						delete this.anim[runName][index];
					}
				});
			}
			anim.action.play();
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.action2d.Shake"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.action2d.Shake"] = true;
dojo.provide("dojox.charting.action2d.Shake");





/*=====
dojo.declare("dojox.charting.action2d.__ShakeCtorArgs", dojox.charting.action2d.__BaseCtorArgs, {
	//	summary:
	//		Additional arguments for highlighting actions.

	//	shift: Number?
	//		The amount in pixels to shift the pie slice.  Default is 3.
	shift: 3
});
=====*/
(function(){
	var DEFAULT_SHIFT = 3,
		m = dojox.gfx.matrix,
		gf = dojox.gfx.fx;

	dojo.declare("dojox.charting.action2d.Shake", dojox.charting.action2d.Base, {
		//	summary:
		//		Create a shaking action for use on an element in a chart.

		// the data description block for the widget parser
		defaultParams: {
			duration: 400,	// duration of the action in ms
			easing:   dojo.fx.easing.backOut,	// easing for the action
			shiftX:   DEFAULT_SHIFT,	// shift of the element along the X axis
			shiftY:   DEFAULT_SHIFT		// shift of the element along the Y axis
		},
		optionalParams: {},	// no optional parameters

		constructor: function(chart, plot, kwArgs){
			//	summary:
			//		Create the shaking action and connect it to the plot.
			//	chart: dojox.charting.Chart2D
			//		The chart this action belongs to.
			//	plot: String?
			//		The plot this action is attached to.  If not passed, "default" is assumed.
			//	kwArgs: dojox.charting.action2d.__ShakeCtorArgs?
			//		Optional keyword arguments object for setting parameters.
			if(!kwArgs){ kwArgs = {}; }
			this.shiftX = typeof kwArgs.shiftX == "number" ? kwArgs.shiftX : DEFAULT_SHIFT;
			this.shiftY = typeof kwArgs.shiftY == "number" ? kwArgs.shiftY : DEFAULT_SHIFT;

			this.connect();
		},

		process: function(o){
			//	summary:
			//		Process the action on the given object.
			//	o: dojox.gfx.Shape
			//		The object on which to process the slice moving action.
			if(!o.shape || !(o.type in this.overOutEvents)){ return; }

			var runName = o.run.name, index = o.index, vector = [], anim,
				shiftX = o.type == "onmouseover" ? this.shiftX : -this.shiftX,
				shiftY = o.type == "onmouseover" ? this.shiftY : -this.shiftY;

			if(runName in this.anim){
				anim = this.anim[runName][index];
			}else{
				this.anim[runName] = {};
			}

			if(anim){
				anim.action.stop(true);
			}else{
				this.anim[runName][index] = anim = {};
			}

			var kwArgs = {
				shape:     o.shape,
				duration:  this.duration,
				easing:    this.easing,
				transform: [
					{name: "translate", start: [this.shiftX, this.shiftY], end: [0, 0]},
					m.identity
				]
			};
			if(o.shape){
				vector.push(gf.animateTransform(kwArgs));
			}
			if(o.oultine){
				kwArgs.shape = o.outline;
				vector.push(gf.animateTransform(kwArgs));
			}
			if(o.shadow){
				kwArgs.shape = o.shadow;
				vector.push(gf.animateTransform(kwArgs));
			}

			if(!vector.length){
				delete this.anim[runName][index];
				return;
			}

			anim.action = dojo.fx.combine(vector);
			if(o.type == "onmouseout"){
				dojo.connect(anim.action, "onEnd", this, function(){
					if(this.anim[runName]){
						delete this.anim[runName][index];
					}
				});
			}
			anim.action.play();
		}
	});
})();

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Status"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Status"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Status"); 
 
////////////////////////////////////
// Status
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Status", null, {
	// private members
	_version: null,
	_status: null,

	/*=====
	constructor_kwArgs: function(version, style) {
		// version: Float
		//		version used to communicate with rest service
		// style: Object
		//		json item style object
	},
	=====*/
	
	 constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Status object.
		this._version = kwArgs.version;
		this._status = kwArgs.status;		
		if (!this._status) {
			this._status = {};
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getAttribute: function(key) {
		return this._status[key];
	},
	
	setAttribute: function(key, value) {
		this._status[key] = value;
	},
	
	getId: function() {
		return this._status.id;
	},
	
	setId: function(id) {
		this._status.id = id;		
	},
	
	getValue: function() {
		return this._status.value;
	},
	
	setValue: function(value) {
		this._status.value = value;
	},
	
	getLabel: function() {
		if (this._status.label) {
			// string
			if (typeof this._status.label === "string") {
				return this._status.label;
			}
			// object with key/value info
			else {
				return this._status.label.value;
			}
		}
		
		return null;
	},
	
	setLabel: function(label) {
		this._status.label = label;
	},
	
	getImage: function() {
		return this._status.image;
	},
	
	setImage: function(image) {
		this._status.image = image;
	},
	
	getImageWidth: function() {
		return this._status.imageWidth;
	},
	
	setImageWidth: function(width) {
		this._status.imageWidth = width;
	},
	
	getImageHeight: function() {
		return this._status.imageHeight;
	},
	
	setImageHeight: function(height) {
		this._status.imageHeight = height;
	},
	
	isImageVisible: function() {
		return (this._status.imageVisible !== undefined ? this._status.imageVisible : true);
	},
	
	setImageVisible: function(visible) {
		this._status.imageVisible = visible;
	},
	
	getScalarImage: function() {
		return this._status.scalarImage;
	},
	
	setScalarImage: function(image) {
		this._status.scalarImage = image;
	},
	
	getScalarImageWidth: function() {
		return this._status.scalarImageWidth;
	},
	
	setScalarImageWidth: function(width) {
		this._status.scalarImageWidth = width;
	},
	
	getScalarImageHeight: function() {
		return this._status.scalarImageHeight;
	},
	
	setScalarImageHeight: function(height) {
		this._status.scalarImageHeight = height;
	},
	
	isScalarImageVisible: function() {
		return (this._status.scalarImageVisible !== undefined ? this._status.scalarImageVisible : true);
	},
	
	setScalarImageVisible: function(visible) {
		this._status.scalarImageVisible = visible;
	},
	
	getBackgroundScalarImage: function() {
		return this._status.backgroundScalarImage;
	},
	
	setBackgroundScalarImage: function(image) {
		this._status.backgroundScalarImage = image;
	},
	
	getBackgroundScalarImageWidth: function() {
		return this._status.backgroundScalarImageWidth;
	},
	
	setBackgroundScalarImageWidth: function(width) {
		this._status.backgroundScalarImageWidth = width;
	},
	
	getBackgroundScalarImageHeight: function() {
		return this._status.backgroundScalarImageHeight;
	},
	
	setBackgroundScalarImageHeight: function(height) {
		this._status.backgroundScalarImageHeight = height;
	},
	
	isBackgroundScalarImageVisible: function() {
		return (this._status.backgroundScalarImageVisible !== undefined ? this._status.backgroundScalarImageVisible : true);
	},
	
	setBackgroundScalarImageVisible: function(visible) {
		this._status.backgroundScalarImageVisible = visible;
	},
	
	getTooltip: function() {
		if (this._status.tooltip) {
			// string
			if (typeof this._status.tooltip === "string") {
				return this._status.tooltip;
			}
			// object with key/value info
			else {
				return this._status.tooltip.value;
			}
		}
		
		return null;
	},
	
	setTooltip: function(tooltip) {
		this._status.tooltip = tooltip;
	},
	
	getMessage: function() {
		return this._status.message;
	},
	
	setMessage: function(message) {
		this._status.message = message;
	},
	
	getColor: function() {
		return this._status.color;
	},
	
	setColor: function(color) {
		this._status.color = color;
	},
	
	clone: function(widgetDef, objectType) {
		var args = {
			status: {},
			version: this._version
		};		
		
		if (widgetDef) {
			// start with widgetDef default status
			var widgetDefaultStatus = null;
			switch(objectType) {
				case "dataset":
					widgetDefaultStatus = widgetDef.getDefaultDatasetStatus();
					break;
				case "item":
					widgetDefaultStatus = widgetDef.getDefaultItemStatus();
					break;
				case "link":
					widgetDefaultStatus = widgetDef.getDefaultLinkStatus();
					break;
				case "relationship":
					widgetDefaultStatus = widgetDef.getDefaultRelationshipStatus();					
			}			
			if (widgetDefaultStatus) {
				dojo.mixin(args.status, widgetDefaultStatus);
			}
		
			// mixin our status
			dojo.mixin(args.status, this._status);
			
			// mixin widgetDef status
			var widgetStatus = null;
			switch(objectType) {
				case "dataset":
					widgetStatus = widgetDef.getDatasetStatus(this.getId(), this.getValue());
					break;
				case "item":
					widgetStatus = widgetDef.getItemStatus(this.getId(), this.getValue());
					break;
				case "link":
					widgetStatus = widgetDef.getLinkStatus(this.getId(), this.getValue());
					break;
				case "relationship":
					widgetStatus = widgetDef.getRelationshipStatus(this.getId(), this.getValue());					
			}					
			if (widgetStatus) {
				dojo.mixin(args.status, widgetStatus);
			}
		}
		// just mixin our status
		else {
			dojo.mixin(args.status, this._status);
		}
			
		return new ibm.tivoli.navmodel.Status(args);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Property"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Property"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Property");
 


////////////////////////////////////
// Property
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Property", null, {
	// private members
	_prop: null,
	_dataset: null,
	_objectType: null,
	_value: null,
	_task: null,

	/*=====
	constructor_kwArgs: function(property, dataset) {
		// property: Object
		//		json property object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this item
		// objectType: String
		//		objectType of the owning object "item", "link", or "relationship"
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Property object.
		this._dataset = kwArgs.dataset;
		this._objectType = (kwArgs.objectType ? kwArgs.objectType : "item");
		this._prop = kwArgs.property;
		
		// optimization of json on the server can leave out:
		// "displayValue"
		if (!this._prop.displayValue) {
			this._prop.displayValue = this._prop.value;
		}		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	 
	clone: function() {		
		// summary:
		//		Clones the property object and returns a new one with the same values
		var args = {
			property: dojo.clone(this._prop),
			dataset: this._dataset,
			objectType: this._objectType
		};		
		
		return new ibm.tivoli.navmodel.Property(args);
	},

	mixin: function(property) {		
		// summary:
		//		Mixin contents of property with ours
		if (property !== this && property.getId() === this.getId()) {
			this._prop = property.getAttributes();
			
			// re-cache values on next request
			this._value = null;
			this._task = null;		
		}
	},

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._prop[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._prop[key] = value;
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the property
		return dojo.clone(this._prop);		
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the property
		dojo.mixin(this._prop, attr);
		
		// re-cache values on next request
		this._value = null;
		this._task = null;
	},

	getId: function() {		
		// summary:
		//		Returns the property id
		return this._prop.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the property id
		this._prop.id = id;
	},

	getLabel: function() {		
		// summary:
		//		Returns the property label
		return this._prop.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the property label
		this._prop.label = label;
	},
	
	getTooltip: function() {
		// summary:
		//		Returns the property tooltip
		return this._prop.tooltip;
	},
	
	setTooltip: function(tooltip) {
		// summary:
		//		Set the property tooltip
		this._prop.tooltip = tooltip;
	},

	getDisplayValue: function() {		
		// summary:
		//		Returns the globalized display value for the property
		return this._prop.displayValue;
	},

	setDisplayValue: function(displayValue) {		
		// summary:
		//		Set the globalized display value for the property
		this._prop.displayValue = displayValue;
	},

	getValue: function() {		
		// summary:
		//		Returns the actual value object for the property
		if (!this._value && this._prop.value) {
			var valueObj = this._prop.value;
			
			switch (this.getValueType()) {
				case "date":
				case "datetime":
				case "time": 
					if (typeof valueObj === "number") {
						this._value = new Date(valueObj);
					}
					else if (typeof valueObj === "string") {
						this._value = dojo.date.stamp.fromISOString(valueObj);										
					}
					break;
				
				case "date[]":
				case "datetime[]":
				case "time[]":
					var dates = [];
					var length = valueObj.length;
					for (var index = 0;  index < length;  ++index) {
						if (typeof valueObj[index] === "number") {
							dates[index] = new Date(valueObj[index]);
						}
						else if (typeof valueObj[index] === "string") {
							dates[index] = dojo.date.stamp.fromISOString(valueObj[index]);						
						}										
					}
					this._value = dates;
					break;
				
				case "status":
					// full status (i.e. was not optimize)
					if (valueObj.label && valueObj.image) {
						this._value = new ibm.tivoli.navmodel.Status({status: valueObj, version: this._dataset.getVersion()});
					}
					// optimized status; look up with dataset
					else {
						var statusArgs = {
							type: this._objectType,
							statusId: valueObj.id,
							statusValue: valueObj.value				
						};
						this._value = this._dataset.getStatus(statusArgs);
					}
					break;
					
				default:
					this._value = valueObj;
			}
		}
		
		return this._value;
	},
	
	/*=====
	getStatusValue_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		WidgetDefinition to use for the merge		
	},
	=====*/
	
	getStatusValue: function(/*getStatusValue_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the "status" value object for the property but
		//		also merges in information from widget definition
		var args = {
			widgetDef: null,
			objectType: this._objectType
		};
		dojo.mixin(args, kwArgs);
		
		var status = null;
		if (this.getValueType() === "status") {
			status = this.getValue();
			if (status && args.widgetDef) {
				status = status.clone(args.widgetDef, args.objectType);
			}
		}
		
		return status;
	},

	setValue: function(value) {		
		// summary:
		//		Modifies the value object for the property
		this._value = value;
				
		switch (this.getValueType()) {
			case "date":
			case "datetime":
			case "time": 
				this._prop.value = value.getTime();
				break;
				
			case "date[]":
			case "datetime[]":
			case "time[]":
				this._prop.value = [];
				var length = value.length;
				for (var index = 0;  index < length;  ++index) {
					this._prop.value[index] = value[index].getTime();
				}
				break;
				
			default:
				this._prop.value = value;
		}
	},

	getValueType: function() {		
		// summary:
		//		Returns the type associated with the property value
		//		|	values:	"itemName", "itemType", "linkName", "linkType", "relationshipName", "relationshipType"
		//		|		"string", "string[]",
		//		|		"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|		"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",	
		//		|		"boolean", "boolean[]"	
		//		|		"enum", "map", "json"
		//		|		"status",
		//		|		"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|		"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|		"float", "double", "percentage", "float[]", "double[]"
		return this._prop.valueType;
	},

	setValueType: function(valueType) {		
		// summary:
		//		Set the type associated with the property value
		this._prop.valueType = valueType;
	},

	getValueState: function() {		
		// summary:
		//		Returns the state assoicated with the property value
		//		|	values:	"ok", "none", "pending", "error"
		return this._prop.valueState ? this._prop.valueState : "ok";
	},

	setValueState: function(valueState) {		
		// summary:
		//		Set the state associated with the property value
		this._prop.valueState = valueState;
	},

	hasMetadata: function() {		
		// summary:
		//		Returns true if the metadata is present in this property object
		return (this._prop.sortWeight !== undefined);
	},

	getValueUnits: function() {		
		// summary:
		//		Returns a units qualifier that can be assoicated with the property value
		//		example:  "seconds", "hours", "mph"
		return this._prop.units;
	},

	setValueUnits: function(units) {		
		// summary:
		//		Set units qualifier
		this._prop.units = units;
	},

	getValueMin: function() {		
		// summary:
		//		Returns minimum value;  for string properties this is minimum length
		return this._getMetadataObj(this._prop.valueMin);
	},

	setValueMin: function(valueMin) {		
		// summary:
		//		Set minimum value
		this._prop.valueMin = valueMin;
	},

	getValueMax: function() {		
		// summary:
		//		Returns maximum value;  for string properties this is maximum length
		return this._getMetadataObj(this._prop.valueMin);
	},

	setValueMax: function(valueMax) {		
		// summary:
		//		Set maximum value
		this._prop.valueMax = valueMax;
	},

	getValueDefault: function() {		
		// summary:
		//		Returns default value for property (i.e. if has no value and 
		//		a set defaults operation is chosen
		return this._getMetadataObj(this._prop.valueDefault);
	},

	setValueDefault: function(valueDefault) {		
		// summary:
		//		Set default value
		this._prop.valueDefault = valueDefault;
	},

	getSortWeight: function() {		
		// summary:
		//		Returns the sort weight for the property;  when sorting a diverse list of properties
		//		the sortWeight can be used to determine sort order.  Properties with equal sortWeight
		//		should be sorted alphabetically by the label.
		return this._prop.sortWeight;
	},

	setSortWeight: function(sortWeight) {		
		// summary:
		//		Set the sort weight
		this._prop.sortWeight = sortWeight;
	},

	isHidden: function() {		
		// summary:
		//		Returns true if the property should be hidden from the user
		return (this._prop.hidden !== undefined ? this._prop.hidden : false);
	},
	
	setHidden: function(hidden) {		
		this._prop.hidden = hidden;
	},

	isEditable: function() {		
		// summary:
		//		Returns true if the property is editable by the user
		return (this._prop.editable !== undefined ? this._prop.editable : true);
	},
	
	setEditable: function(editable) {
		this._prop.editable = editable;
	},

	isRequired: function() {		
		// summary:
		//		Returns true if the property is required (i.e. must have a value if edited)
		return (this._prop.required !== undefined ? this._prop.required : false);
	},
	
	setRequired: function(required) {
		this._prop.required = required;
	},

	isSensitive: function() {		
		// summary:
		//		Returns true if the property is sensitive and should not 
		//		be displayed in the clear
		return (this._prop.sensitive !== undefined ? this._prop.sensitive : false);
	},
	
	setSensitivie: function(sensitive) {
		this._prop.sensitive = sensitive;
	},
	
	getTask: function() {
		if (!this._task && this._prop.task) {
			this._task = new ibm.tivoli.navmodel.Task({task: this._prop.task, dataset: this._dataset});
		}
		
		return this._task;
	},
	
	setTask: function(task) {
		this._task = task;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_getMetadataObj: function(obj) {		
		// summary:
		//		Makes necessary conversion on objects for
		//		metadata functions like min,max,default
		if (obj) {
			switch (this.getValueType()) {
				case "date":
				case "datetime":
				case "time":
				case "date[]":
				case "datetime[]":
				case "time[]":
					if (typeof obj === "number") {
						return new Date(obj);
					}
					else if (typeof obj === "string") {
						return dojo.date.stamp.fromISOString(obj);						
					}									
			}		
		}
		
		return obj;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.LinkStyle"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.LinkStyle"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.LinkStyle"); 
 
////////////////////////////////////
// LinkStyle
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.LinkStyle", null, {
	// private members       
	_version: null,
	_style: null,
	
	/*=====
	constructor_kwArgs: function(version, style) {
		// version: Float
		//		version used to communicate with rest service
		// style: Object
		//		json item style object
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary
		//		Constructs LinkStyle object.
		this._version = kwArgs.version;
		this._style = kwArgs.style;		
		if (!this._style) {
			this._style = {};
		}	
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getAttribute: function(key) {
		return this._style[key];
	},
	
	setAttribute: function(key, value) {
		this._style[key] = value;
	},
	
	getId: function() {
		return this._style.id;
	},
	
	setId: function(id) {
		this._style = id;
	},
	
	getType: function() {
		return this._style.type;
	},
	
	setType: function(type) {
		this._style.type = type;
	},
	
	getLabel: function() {
		if (this._style.label) {
			// string
			if (typeof this._style.label === "string") {
				return this._style.label;
			}
			// object with key/value info
			else {
				return this._style.label.value;
			}
		}
		
		return null;
	},
	
	setLabel: function(label) {
		this._style.label = label;		
	},
	
	getLineStyle: function() {
		return this._style.lineStyle;
	},
	
	setLineStyle: function(lineStyle) {
		this._style.lineStyle = lineStyle;
	},
	
	getLineWidth: function() {
		return this._style.lineWidth;
	},
	
	setLineWidth: function(width) {
		this._style.lineWidth = width;
	},
	
	getLineColor: function() {
		return this._style.lineColor;
	},
	
	setLineColor: function(color) {
		this._style.lineColor = color;
	},
	
	getArrowColor: function() {
		return this._style.arrowColor;
	},

	setArrowColor: function(color) {
		this._style.arrowColor = color;
	},
	
	isLabelVisible: function() {
		return (this._style.labelVisible !== undefined ? this._style.labelVisible : false);
	},
	
	setLabelVisible: function(visible) {
		this._style.labelVisible = visible;
	},
	
	isLabelAutoRotate: function() {
		return (this._style.labelAutoRotate !== undefined ? this._style.labelAutoRotate : false);
	},
	
	setLabelAutoRotate: function(autoRotate) {
		this._style.labelAutoRotate = autoRotate;
	},

	getLabelParallelAlignment: function() {
		return (this._style.labelParallelAlignment !== undefined ? this._style.labelParallelAlignment : "middle");
	},
	
	setLabelParallelAlignment: function(parallelAlignment) {
		this._style.labelParallelAlignment = parallelAlignment;
	},
	
	getLabelOrthogonalAlignment: function() {
		return (this._style.labelOrthogonalAlignment !== undefined ? this._style.labelOrthogonalAlignment : "left");
	},
	
	setLabelOrthogonalAlignment: function(orthogonalAlignment) {
		this._style.labelOrthogonalAlignment = orthogonalAlignment;
	},

	getLabelDistance: function() {
		return (this._style.labelDistance !== undefined ? this._style.labelDistance : 5);
	},
	
	setLabelDistance: function(distance) {
		this._style.labelDistance = distance;
	},

	getLabelPositionType: function() {
		return (this._style.labelPositionType !== undefined ? this._style.labelPositionType : "relative");
	},
	
	setLabelPositionType: function(positionType) {
		this._style.labelPositionType = positionType;
	},

	getLabelPosition: function() {
		return (this._style.labelPosition !== undefined ? this._style.labelPosition : 0.5);
	},
	
	setLabelPosition: function(position) {
		this._style.labelPosition = position;
	},
	
	clone: function(widgetDef, objType) {
		var args = {
			style: {},
			version: this._version
		};		
		
		if (widgetDef) {
			// start with widgetDef default style
			var widgetDefaultStyle = null;
			switch(objType) {
				case "link":
					widgetDefaultStyle = widgetDef.getDefaultLinkStyle();
					break;
				case "relationship":
					widgetDefaultStyle = widgetDef.getDefaultRelationshipStyle();
			}
			if (widgetDefaultStyle) {
				dojo.mixin(args.style, widgetDefaultStyle);
			}
		
			// mixin our link style
			dojo.mixin(args.style, this._style);
			
			// mixin widgetDef style
			var widgetStyle = null;
			switch(objType) {
				case "link":
					widgetStyle = widgetDef.getLinkStyle(this.getType());
					break;
				case "relationship":
					widgetStyle = widgetDef.getRelationshipStyle(this.getType());
			}		
			if (widgetStyle) {
				dojo.mixin(args.style, widgetStyle);
			}
		}
		// just mixin our link style
		else {
			dojo.mixin(args.style, this._style);
		}
			
		return new ibm.tivoli.navmodel.LinkStyle(args);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Exception"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Exception"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Exception"); 
 
////////////////////////////////////
// Exception
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Exception", null, {
	// private members
	_version: null,
	_exception: null,
	
	/*=====
	constructor_kwArgs: function(version, exception) {
		// version: Float
		//		version used to communicate with rest service
		// exception: Object
		//		json exception object
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary
		//		Constructs Exception object.
		this._version = kwArgs.version;
		this._exception = kwArgs.exception;		
	},

	getMsgId: function() {
		return this._exception.msgId;
	},
	
	getMsgSeverity: function() {
		return this._exception.msgSeverity;
	},
	
	getMsgText: function() {
		return this._exception.msgText;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Utils"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Utils"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Utils"); 

////////////////////////////////////
// Utils
////////////////////////////////////

ibm.tivoli.navmodel.Utils.isTIP = function() {
	//	summary:
	//		Returns true if environment is running in Tivoli Integrated Portal  
  	var themes = ["TIPLight", "TIPDark", "ISCTheme"];
  	var themeCount = themes.length;
  	var body = dojo.body();
  	for (var index = 0;  index < themeCount;  ++index) {
  		if (dojo.hasClass(body, themes[index])) {
  			return true;
  		}
  	}
  	
  	return false;  
};

ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion = function() {
	return 1.0;
};

ibm.tivoli.navmodel.Utils.getContentType = function(version, format) {
	// summary:
	//		Return the HttpHeader "Content-Type" that REST service is expecting
	//		;format=xxxx   can be appended to indicate the json format object
	var contentType = "application/vnd.ibm.com.tivolidis.json;version=" + version;	
	if (format) {
		contentType += ";format=" + format;
	}
	
	return contentType;
};

ibm.tivoli.navmodel.Utils.getContent = function() {		
	// summary:
	//		Construct a "content" object for sending an xdr request;
	//		IE will sometimes return http requests from cache without unique parms
	var content = {};
		
	// add date and random
	var now = new Date();		
	content.d = now.getTime();
	content.r = Math.random();
		
	return content;
};

ibm.tivoli.navmodel.Utils.mixinParams = function(content, dataset) {		
	// summary:
	//		Mixin the "param_" qualified arguments;  all parameters are 
	//		prefixed with "param_" so the rest service can use them to identify
	//		dataset cache entry
	var parameters = dataset.getArg("parameters");
	for (var parm in parameters) {
		if (parameters.hasOwnProperty(parm)) {
			content["param_" + parm] = parameters[parm];
		}
	}
};

ibm.tivoli.navmodel.Utils.mixinProperties = function(content, key, properties) {		
	// summary:
	//		Mixin the "properties" into the content as a comma delimited list
	if (properties && properties.length > 0) {
		content[key] = properties.join(",");
	}
};

ibm.tivoli.navmodel.Utils.getParamUrl = function(url, dataset) {		
	// summary:
	//		Tack on dataset version and param_* values to the url
	var questionMark = false;
		
	// add "param_" parms
	var parameters = dataset.getArg("parameters");
	for (var parm in parameters) {
		if (parameters.hasOwnProperty(parm)) {
			if (!questionMark) {
				questionMark = true;
				url += "?";
			}
			else {
				url += "&";
			}
			url += ("param_" + parm + "=" + parameters[parm]);				
		}
	}
		
	// IE will sometimes return http requests from cache without unique parms
	var now = new Date();
	if (!questionMark) {
		questionMark = true;
		url += "?d=" + now.getTime() + "&r=" + Math.random();
	}
	else {
		url += "&d=" + now.getTime() + "&r=" + Math.random();
	}
	
	return url;	
};

ibm.tivoli.navmodel.Utils.getException = function(error, ioArgs, funcName, trace, version) {		
	// summary:
	//		Returns Exception object contained in the dojo.xhr ioArgs
	var exception = null;
	var responseText = ioArgs.xhr.responseText;
	if (responseText) {
		try {
			var jsonEx = dojo.fromJson(responseText);
			if (jsonEx.msgId) {
				trace.error(funcName + " generated exception=" + responseText, "getException");				
				exception = new ibm.tivoli.navmodel.Exception({exception: jsonEx, version: version});
			}
			else {
				trace.error(funcName + " generated exception (non rest) = " + error, "getException");				
			}
		}
		catch(e) {
			trace.error(funcName + " generated exception (non rest) = " + error, "getException");
			exception = new ibm.tivoli.navmodel.Exception({exception: {msgText: error}, version: version});
		}
			
		trace.error("exception url=" + ioArgs.url, "getException");							
	}
	
	return exception;
};

ibm.tivoli.navmodel.Utils.NorthernHemisphere = {
	"GMT-10:00": "US/Aleutian",
  	"GMT-9:00":  "US/Alaska",
  	"GMT-8:00":  "US/Pacific",
  	"GMT-7:00":  "US/Mountain",
	"GMT-6:00":  "US/Central",
	"GMT-5:00":  "US/Eastern",
	"GMT-4:00":  "Canada/Atlantic",
	"GMT-3:30":  "Canada/Newfoundland",
	"GMT-3:00":  "America/Godthab",
    "GMT-2:00":  "America/Noronha",
    "GMT-1:00":  "Atlantic/Azores",
    "GMT":       "Europe/London",
    "GMT+1:00":  "Europe/Amsterdam",
    "GMT+2:00":  "Europe/Athens",
    "GMT+3:00":  "Europe/Moscow",
    "GMT+3:30":  "Asia/Tehran",
    "GMT+4:00":  "Asia/Baku",
    "GMT+5:00":  "Asia/Yekaterinburg",
    "GMT+6:00":  "Asia/Novosibirsk",
    "GMT+7:00":  "Asia/Krasnoyarsk",
    "GMT+8:00":  "Asia/Irkutsk",
    "GMT+9:00":  "Asia/Yakutsk",
    "GMT+10:00": "Asia/Vladivostok"
};

ibm.tivoli.navmodel.Utils.SouthernHemisphere = {
	"GMT-4:00":  "America/Asuncion",
	"GMT-3:00":  "America/Montevideo",
	"GMT+2:00":  "Africa/Windhoek",
	"GMT+9:30":  "Australia/Adelaide",
	"GMT+10:00": "Australia/Sydney",
	"GMT+12:00": "Pacific/Fiji"
};

ibm.tivoli.navmodel.Utils.NoDST = {  
	"GMT-11:00": "US/Samoa",
	"GMT-10:00": "US/Hawaii",
	"GMT-7:00":	 "US/Arizona",
	"GMT-6:00":  "Canada/Saskatchewan",
	"GMT-5:00":  "US/East-Indiana",
	"GMT-4:30":  "America/Caracas",
	"GMT-4:00":  "America/Manaus",
	"GMT-3:00":  "America/Buenos_Aires",
	"GMT-2:00":  "Atlantic/South_Georgia",
	"GMT-1:00":  "Atlantic/Cape_Verde",
	"GMT":       "Africa/Monrovia",
	"GMT+1:00":  "Africa/Algiers",
	"GMT+2:00":  "Africa/Harare",
	"GMT+3:00":  "Asia/Baghdad",
	"GMT+4:00":  "Asia/Dubai",
	"GMT+4:30":  "Asia/Kabul",
	"GMT+5:00":  "Asia/Karachi",
	"GMT+5:30":  "Asia/Kolkata",
	"GMT+5:45":  "Asia/Katmandu",
	"GMT+6:00":  "Asia/Dhaka",
	"GMT+6:30":  "Asia/Rangoon",
	"GMT+7:00":  "Asia/Bangkok",
	"GMT+8:00":  "Asia/Hong_Kong",
	"GMT+9:00":  "Asia/Tokyo",
	"GMT+9:30":  "Australia/Darwin",
	"GMT+10:00": "Australia/Brisbane",
	"GMT+11:00": "Asia/Magadan",
	"GMT+12:00": "Pacific/Wake",
	"GMT+13:00": "Pacific/Tongatapu"
};

// OlsonTimeZone name
ibm.tivoli.navmodel.Utils.timeZoneName = null;

ibm.tivoli.navmodel.Utils.setTimeZone = function(timeZoneName) {
	// summary:
	//		Set the timeZoneName that will be returned by getTimeZone()	
	ibm.tivoli.navmodel.Utils.timeZoneName = timeZoneName;
};

ibm.tivoli.navmodel.Utils.getTimeZone = function() {
	// summary:
	//		Calculate the users Olson timezone name
	
	// if one has been set; just return it
	if (ibm.tivoli.navmodel.Utils.timeZoneName) {
		return ibm.tivoli.navmodel.Utils.timeZoneName;
	}
	
	var janUTC = ibm.tivoli.navmodel.Utils.getUTCOffset(1, 1);
	var janGMT = ibm.tivoli.navmodel.Utils.getGMTOffset(1, 1);
	var julyUTC = ibm.tivoli.navmodel.Utils.getUTCOffset(7, 1);
	var julyGMT = ibm.tivoli.navmodel.Utils.getGMTOffset(7, 1);	
	var dst = (janUTC !== julyUTC);	
	var tzName = null;
	
	// DST
	if (dst) {
		if (janUTC < julyUTC) {
			tzName = ibm.tivoli.navmodel.Utils.NorthernHemisphere[janGMT];
		}
		else {
			tzName = ibm.tivoli.navmodel.Utils.SouthernHemisphere[julyGMT];
		}
	}
	// No DST
	else {
		tzName = ibm.tivoli.navmodel.Utils.NoDST[janGMT];
	}
	
	// can't find it in tables; so return GMT offset
	if (!tzName) {
		tzName = ibm.tivoli.navmodel.Utils.getGMTOffset();
	}
	
	//alert('jan=' + janGMT + '  july=' + julyGMT + ' dst=' + dst + ' tz=' + tzName);
		
	return tzName;
};

ibm.tivoli.navmodel.Utils.getUTCOffset = function(month, day) {
	// summary:
	//		Calculate the users current GMT offset
	var date = new Date();
	if (month) {
		date.setMonth(month);
	}
	if (day) {
		date.setDate(day);
	}
	
	return -(date.getTimezoneOffset());	
};

ibm.tivoli.navmodel.Utils.getGMTOffset = function(month, day) {
	// summary:
	//		Calculate the users current GMT offset
	var date = new Date();
	if (month) {
		date.setMonth(month);
	}
	if (day) {
		date.setDate(day);
	}
	
	var timezone = "";
	var minutesFromGMT = date.getTimezoneOffset();
	var extraMinutes = minutesFromGMT%60;
	var gmtHours = null;
	var gmtMinutes = "00";
		
	if (extraMinutes === 0) {
		gmtHours = -(minutesFromGMT/60);
	}
	else {
		gmtHours = -((minutesFromGMT-extraMinutes)/60);
		gmtMinutes = (Math.abs(extraMinutes)).toString();		
	}	
		
	timezone = "GMT";
	if (gmtHours !== 0) {
		if (gmtHours > 0) {
			timezone += "+";				
		}
		timezone += gmtHours;
		timezone += ":";
		timezone += gmtMinutes;
	}
		
	return timezone;
};

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Trace"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Trace"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Trace");
 
dojo.declare("ibm.tivoli.navmodel.Trace", null, {
    // private members     
	traceName: "",
	level: 0,  // error
        
	constructor: function(name) {
		this.traceName = name;
				
		if (dojo.config.trace) {
			// "all" overrides individual class entries
			if (dojo.config.trace.all) {
				this.level = this._translateLevel("all", dojo.config.trace.all);
			}
			else {
				var lastDot = this.traceName.lastIndexOf(".");
				if (lastDot > 0) {
					var className = this.traceName.substring(lastDot+1);
					if (dojo.config.trace[className]) {
						this.level = this._translateLevel(className, dojo.config.trace[className]);
					}
				}
			}
		}					
 	},
        
	/////////////////////////////
	// public methods
	/////////////////////////////
	
	isError: function() {
		return (this.level >= 0);
	},
	
	isWarn: function() {
		return (this.level >= 1);
	},
	
	isInfo: function() {
		return (this.level >= 2);
	},
	
	isFine: function() {
		return (this.level >= 3);
	},
	
	isFiner: function() {
		return (this.level >= 4);
	},
	
	isFinest: function() {
		return (this.level >= 5);
	},
	
	error: function(dbgString, methodName) {
		if (this.isError()) {
			if (dbgString && dbgString instanceof Error) {
                console.error(this._output("EXCEPTION", methodName, dbgString.toString()));
	    	} 
	    	else if (dbgString && dbgString.isInstanceOf && dbgString.isInstanceOf(ibm.tivoli.navmodel.Exception)) {
	    		console.error(this._output("EXCEPTION", methodName, dbgString.getMsgText()));
	    	} else {
	    		console.error(this._output("ERROR", methodName, dbgString));
	    	}
	    }
	},
	
	warn: function(dbgString, methodName) {
    	if (this.isWarn()) {
    		console.warn(this._output("WARN", methodName, dbgString));
    	}
    },
    
	info: function(dbgString, methodName) {
    	if (this.isInfo()) {
    		console.info(this._output("INFO", methodName, dbgString));
    	}
    },
    
    fine: function(dbgString, methodName) {
    	if (this.isFine()) {
    		console.info(this._output("FINE", methodName, dbgString));
    	}
    },
    
    finer: function(dbgString, methodName) {
    	if (this.isFiner()) {
    		console.info(this._output("FINER", methodName, dbgString));
    	}
    },
    
	finest: function(dbgString, methodName) {
    	if (this.isFinest()) {
    		console.info(this._output("FINEST", methodName, dbgString));
    	}
    },
	
	entering: function(dbgString, methodName) {
    	if (this.isFinest()) {
    		console.info(this._output("ENTERING", methodName, dbgString));
    	}
    },
    
    exiting: function(dbgString, methodName) {
    	if (this.isFinest()) {
    		console.info(this._output("EXITING", methodName, dbgString));
    	}
    },
    
	object: function(dbgObject, methodName) {
    	if (this.isFine()) {
    		console.log(this._output("OBJECT", methodName, dojo.toJson(dbgObject)));
    	}
    },
    
	assert: function(predicate, message, methodName) {
		if (this.isFine()) {
			if (predicate === false ) {
				console.log(this._output("ASSERT", methodName, "assertion failed: " + message));
			}
		}		
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_translateLevel: function(className, value) {
		var level = 0;
		
		switch(value) {
			case "off":
				level = -1;
				break;
			case "error":
				level = 0;
				break;
			case "warn":
				level = 1;
				break;
			case "info":
				level = 2;
				break;
			case "fine":
				level = 3;
				break;
			case "finer":
				level = 4;
				break;
			case "finest":
				level = 5;
				break;
			default:
				console.log("className=" + className + " has invalid level=" + value);
		}
		
		return level;
	},
	
	//***** DEBUG::trace=ibm.tivoli.navmodel.Dataset, method=EXITING; event received, type = itemStatusChanged
	_output: function(level, methodName, message) {
		var outputMessage = "***** " + level;
		if (message) {
			outputMessage = outputMessage + ": " + message;
		}
		outputMessage = outputMessage + " ::trace=" + this.traceName;
		if (methodName) {
			outputMessage = outputMessage + ", method=" + methodName;
		}
		return outputMessage;
	}	
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Item"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Item"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Item"); 
 
// includes








ibm.tivoli.navmodel.Item._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Item");

////////////////////////////////////
// Item
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Item", null, {
	// private members       
	_item: null,
	_dataset: null,
	_props: null,
	_propsById: null,
	_nullPropIds: null,
	_allProps: false,
	_allMetadata: false,
	_status: null,
	_style: null,
	_defaultTask: null,
	_expiration: null,
	_trace: ibm.tivoli.navmodel.Item._trace,
	
	/*=====
	constructor_kwArgs: function(item, dataset, propIds, allProperties, allMetadata, expiration) {
		// item: Object
		//		son item object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this item
		// propIds: String[]
		//		optional array indicating the property Id's that were requested with 
		//		the item; so it can be determined if any properties are null
		// allProperties: Boolean
		//		optional: indicates if the item already has "all" properties
		// allMetadata: Boolean
		//		optional: indicates if item properties already have metadata
		// expiration: Long
		//		optional: expiration time;  will use (current time + interval) if not supplied
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Item object.
		this._item = kwArgs.item;
		if (!this._item) {
			this._item = {};
		}
		
		this._dataset = kwArgs.dataset;
		this._props = [];
		this._propsById = {};
		this._nullPropIds = {};			
		this._allProps = kwArgs.allProperties;
		this._allMetadata = kwArgs.allMetadata;
		
		if (this._item.properties) {
			var propsLength = this._item.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = new ibm.tivoli.navmodel.Property({property: this._item.properties[propIndex], dataset: this._dataset, objectType: "item"});
				this._props[propIndex] = prop;
				this._propsById[prop.getId()] = prop;
			}
			
			// see if we should mark any properties as null because they were not returned
			if (kwArgs.propIds) {
				propsLength = kwArgs.propIds.length;
				for (propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var propId = kwArgs.propIds[propIndex];				
					if (!this._propsById[propId]) {
						this._nullPropIds[propId] = true;
					}
				}
			}
		}
		
		// expiration was supplied
		if (kwArgs.expiration) {
			this._expiration = kwArgs.expiration;
		}
		// reset to current time + interval
		else {
			this.resetExpiration();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	mixin: function(item, resetLazyLoad) {
		// summary:
		//		Mixin properties and status from this Item object	
		if (item && item !== this) {
			this.removeAttribute("style");
			this._style = null;
			
			// reset lazy load indicators;  mixin will
			// provide the current values
			if (resetLazyLoad) {
				this.resetLazyLoad();
			}
			
			this.mixinAttributes(item.getAttributes());
			this.mixinProperties(item._props, item._nullPropIds);
			this._status = item.getStatuses({widgetDef: null});
			this._defaultTask = item.getDefaultTask();
			
			if (item._allProps) {
				this._allProps = true;
				
				if (item._allMetadata) {
					this._allMetadata = true;
				}
			}
		}	
	},

	getDataset: function() {
		// summary:
		//		Returns the dataset that owns the item
		return this._dataset;
	},

	getObjectType: function() {
		// summary:
		//		Indicate that this object is an "item"
		return "item";
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._item[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._item[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._item[key];
	},

	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the item
		var item = dojo.clone(this._item);
		
		// make sure "properties" are current as we may have received events
		if (this._props && this._props.length > 0) {
			item.properties = [];
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				item.properties[propIndex] = this._props[propIndex].getAttributes();
			}
		}
		
		return item;		
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the item
		dojo.mixin(this._item, attr);		
	},

	getUri: function() {
		if (!this._item.uri) {
			this._item.uri = this._dataset.getAttribute("uri") + "/items/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._item.uri;
	},
	
	getId: function() {		
		// summary:
		//		Returns item id
		return this._item.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the item id
		this._item.id = id;		
	},

	getType: function() {		
		// summary:
		//		Returns the type of item
		return this._item.type;
	},
	
	setType: function(type) {		
		// summary:
		//		Set the item type
		this._item.type = type;		
	},

	getTypeLabel: function() {		
		// summary:
		//		Returns globalized label for the item type
		var label = this._item.typeLabel;
		if (!label) {
			var style = this.getItemStyle({widgetDef: null});
			if (style) {
				label = style.getLabel();
			}
		}
		
		return label;
	},

	setTypeLabel: function(typeLabel) {		
		// summary:
		//		Set the globalized label for the item type
		this._item.typeLabel = typeLabel;		
	},

	getLabel: function() {		
		// summary:
		//		Returns globalized label for the item
		return this._item.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the globalized label for the item
		this._item.label = label;		
	},

	getDescription: function() {		
		// summary:
		//		Returns globalized description for the item
		return this._item.description;
	},

	setDescription: function(description) {		
		// summary:
		//		Set the globalized description for the item
		this._item.description = description;		
	},

	getTooltip: function() {		
		// summary:
		//		Returns globalized toolip for the item
		return this._item.tooltip;
	},

	setTooltip: function(tooltip) {		
		// summary:
		//		Set the globalized tooltip for the item
		this._item.tooltip = tooltip;
	},
	
	isLazyLoad: function() {		
		// summary:
		//		Returns true if the item is elligible for "lazy load"
		return (this._item.lazyLoad !== undefined ? this._item.lazyLoad : false);
	},

	setLazyLoad: function(lazyLoad) {		
		// summary:
		//		Set the lazy load attribute
		this._item.lazyLoad = lazyLoad;
	},

	getLazyLoadRelationships: function() {		
		// summary:
		//		Returns the relationships that should be explored for related items
		return this._item.relationshipsKey;
	},

	resetLazyLoad: function() {
		// summary:
		//		Resets current attributes determing whether a lazyLoad
		//		should be performed to find related items
		delete this._item.lazyLoad;
		delete this._item.relationshipsKey;				
	},
	
	setLazyLoadRelationships: function(relationshipsKey) {		
		// summary:
		//		Set the relationships that should be explored for related items
		this._item.relationshipsKey = relationshipsKey;
	},

	isSubgraph: function() {
		// summary:
		//		Returns true if the item represents a nested subgraph parent
		return (this._item.subgraph !== undefined ? this._item.subgraph : false);
	},
	
	setSubgraph: function(subgraph) {		
		// summary:
		//		Set the attribute indicating if the item is a nested subgraph parent
		this._item.subgraph = subgraph;
	},
	
	getSubgraphId: function() {
		// summary:
		//		Returns id of nested subgraph parent; no value if item is not in a subgraph
		return this._item.subgraphId;
	},
	
	setSubgraphId: function(subgraphId) {
		// summary:
		//		Set the id of the sub-graph parent
		this._item.subgraphId = subgraphId;
	},
	
	isSubgraphOrigin: function() {
		// summary:
		//		Returns true if the item is an origin within a nested subgraph
		return (this._item.subgraphOrigin !== undefined ? this._item.subgraphOrigin : false);
	},
	
	setSubgraphOrigin: function(subgraphOrigin) {
		// summary:
		//		Set the attribute indicating if the item is an origin within a nested subgraph
		this._item.subgraphOrigin = subgraphOrigin;
	},
	
	/*=====
	getStatuses_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/

	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns ibm.tivoli.navmodel.Status[] objects indicating the item status values
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if expired mixin from dataset cache
		this._validateExpiration();
				
		if (!this._status && this._item.status) {
			this._status = [];
			var statusLength = this._item.status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				if (this._item.status.label && this._item.status.image) {
					this._status.push(new ibm.tivoli.navmodel.Status({status: this._item.status[statusIndex], version: this._version()}));				
				}
				else {
					var statusArgs = {
						type: "item",
						statusId: this._item.status[statusIndex].id,
						statusValue: this._item.status[statusIndex].value				
					};
				
					var statusObj = this._dataset.getStatus(statusArgs);
					if (statusObj) {				
						this._status.push(statusObj);
					}
					else {
						this._trace.error("Could not locate status with id=" + statusArgs.statusId + " value=" + statusArgs.statusValue, "getStatuses");
					}
				}
			}			
		}
		
		// merge widget definition status
		var status = this._status;
		if (status && args.widgetDef) {
			status = this._status.slice(0);
			var statusLength2 = this._status.length;
			for (var statusIndex2 = 0;  statusIndex2 < statusLength2;  ++statusIndex2) {
				status[statusIndex2] = this._status[statusIndex2].clone(args.widgetDef, "item");
			}			
		}
		
		return status;		
	},

	setStatuses: function(statuses) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Status[] objects indicating the item status values
		this._status = statuses;
	},

	supportsProperties: function() {		
		// summary:
		//		Indicates if "Properties" is supported for the item
		return (this._item.supportsProperties !== undefined ? this._item.supportsProperties : true);
	},

	setSupportsProperties: function(supportsProperties) {		
		// summary:
		//		Set whether item  supports "Properties"
		this._item.supportsProperties = supportsProperties;
	},

	hasCachedMetadata: function() {		
		// summary:
		//		Returns true if item has all properties and associated metadata cached
		return this._allMetadata;
	},

	hasAllCachedProperties: function() {		
		// summary:
		//		Returns true if item has all properties cached
		return this._allProps;
	},

	hasCachedProperty: function(/*String*/ propId) {
		// summary:
		//		Returns true if we have the requested property in the cache
		// propId
		//		property id
		return (this._propsById[propId] ? true : false);
	},
	
	hasCachedProperties: function(/*String[]*/ properties) {		
		// summary:
		//		Returns true if we have all the requested properties in the cache
		// properties:
		//		array of property ids
		if (properties) {
			var propsLength = properties.length;
			
			// asking if we have "all" properties
			if (propsLength === 1 && properties[0] === "all") {
				return this._allProps;
			}
			
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (!this._propsById[properties[propIndex]]) {
					return false;
				}
			}
		}
		
		return true;
	},

	getCachedPropertyIds: function(expandAll) {		
		// summary:
		//		Returns array of property Ids currently cached in the item
		var propIds = [];
		
		// have all the properties
		if (!expandAll && this._allProps) {
			propIds.push("all");
		}
		// get individual property ids	
		else if (this._props) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				propIds.push(this._props[propIndex].getId());
			}
		}
		return propIds;
	},
	
	/*=====
	properties_onComplete: function(properties, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous
	},
	=====*/

	getProperties: function(/*getProperties_kwArgs*/ kwArgs) {	
		// summary:
		//		Asynchronously returns properties for the item
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: ["all"],
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		// if expired mixin from dataset cache
		this._validateExpiration(kwArgs);
				
		// fetch all properties
		if (args.properties.length === 1 && args.properties[0] === "all") {
			if (this._allProps && (!args.metadata || this._allMetadata)) {				
				if (args.sync) {
					args.onComplete(this._props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._props, args), 0);
				}		
			}
			else {
				this._fetchProperties(args);
			}
		}		
		// fetch selective properties
		else {
			// look up properties in cache
			var props = [];
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var propId = args.properties[propIndex];
				var prop = this._propsById[propId];				
				if (prop && (!args.metadata || prop.hasMetadata())) {
					props[propIndex] = prop;					
				}
				else {
					break;
				}
			}
			
			// if we found all properties in cache then return them
			if (props.length === propsLength) {
				if (args.sync) {
					args.onComplete(props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, props, args), 0);
				}
			}
			// otherwise we need to fetch
			else {
				this._fetchProperties(args);
			}
		}
	},

	setProperties: function(properties) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Property[] objects for the item
		//		
		//		Note: client side only operation;  no request to rest service
		this._propsById = {};
		this._nullPropIds = {};
		this._props = [];
		
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = properties[propIndex];
				this._propsById[prop.getId()] = prop;
			}
		}		
	},

	mixinProperties: function(properties, nullPropIds) {		
		// summary:
		//		Mixin the ibm.tivoli.navmodel.Property[] objects for the item
		//		
		//		Note: client side only operation;  no request to rest service
		
		// mixin property objects
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				this.mixinProperty(properties[propIndex]);
			}					
		}
		
		// mixin any propIds that were null
		if (nullPropIds) {
			for (var propId in nullPropIds) {
				if (nullPropIds.hasOwnProperty(propId)) {
					this._nullPropIds[propId] = true;
				}
			}
		}
	},

	getProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// if expired mixin from dataset cache
		this._validateExpiration();
						
		// check prop cache
		var prop = this._propsById[propId];
		
		if (!prop) {
			// we have asked for this property before and it wasn't found
			if (this._nullPropIds[propId]) {
				return null;
			}
			
			// check default properties on item	
			if (this._item.properties) {
				var propsLength = this._item.properties.length;
				for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var rawProp = this._item.properties[propIndex];
					if (rawProp.id === propId) {
						prop = new ibm.tivoli.navmodel.Property({property: rawProp, dataset: this._dataset, objectType: "item"});
						
						// cache property
						this._propsById[propId] = prop;
						break;
					}
				}				
			}
		}	
		
		// fetch it
		if (!prop && !this._allProps) {
			prop = this._fetchProperty(propId);			
		}		
		
		return prop;
	},

	setProperty: function(property) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Property for the item
		//		
		//		Note: client side only operation;  no request to rest service
		var found = this._propsById[property.getId()];		
		if (found) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (this._props[propIndex].getId() === property.getId()) {
					this._props[propIndex] = property;
					break;
				}
			}
		}
		else {
			this._props.push(property);
		}		
		
		this._propsById[property.getId()] = property;
	},

	mixinProperty: function(property) {		
		// summary:
		//		Mixin ibm.tivoli.navmodel.Property for the item
		//		
		//		Note: client side only operation;  no request to rest service
		var prop = this._propsById[property.getId()];
		if (prop) {
			prop.mixin(property);
		}
		else {
			this._props.push(property);
			this._propsById[property.getId()] = property;
		}	
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	editProperties_kwArgs: function(onComplete, onError, properties) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		array of property ids to set
	},
	=====*/

	editProperties: function(/*editProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously modify properties for the item
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.properties && args.properties.length > 0) {
			var item = dojo.clone(this._item);
			item.properties = [];
			
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				item.properties[propIndex] = args.properties[propIndex].getAttributes();
			}
			
			var json = dojo.toJson(item, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: false,			
				load: dojo.hitch(this, function(result, ioArgs) {
					args.onComplete(args);						
				}),
				error: dojo.hitch(function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperties", this._trace, this._version());				               
				})
			});	
		}
		// nothing to set
		else {
			setTimeout(dojo.hitch(this, args.onComplete, args), 0);
		}		
	},

	 editProperty: function(/*String*/ property) {		
		// summary:
		//		Synchronously modify property for the item
		// property:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		
		if (property) {
			var item = dojo.clone(this._item);
			item.properties = [property.getAttributes()];
			
			var json = dojo.toJson(item, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: true,			
				load: dojo.hitch(this, function(result, ioArgs) {						
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperty", this._trace, this._version());									              
				})
			});				
		}
		
		if (exception) {
			throw exception;
		}
	},

	hasOwnItemStyle: function() {		
		// summary:
		//		Returns true if the item has an ItemStyle that overrides the default
		//		item style for the item type  (i.e. different than the one retrieved
		//		from the dataset)
		return (this._item.style ? true: false);
	},
	
	/*=====
	getItemStyle_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/

	getItemStyle: function(/*getItemStyle_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.ItemStyle for this item
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (!this._style) {
			if (this._item.style) {
				this._style = new ibm.tivoli.navmodel.ItemStyle({style: this._item.style, version: this._version()});
			}
			else {
				this._style = this._dataset.getItemStyle({itemType: this.getType(), widgetDef: null});
			}
		}
		
		// merge widget definition styles 
		var style = this._style;
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (style) {
				style = style.clone(args.widgetDef);
			}
			// unknown style; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getItemStyle(this.getType());
				if (widgetStyle) {
					style = this._dataset._createItemStyle(args.widgetDef, widgetStyle);
				}
			}			
		}
		
		return style;
	},

	setItemStyle: function(style) {		
		// summary:
		//		Set item style for the item
		this._style = style;
	},

	getDefaultTask: function() {		
		// summary:
		//		Returns optional default task associated with the item
		if (!this._defaultTask && this._item.defaultTask) {
			this._defaultTask = new ibm.tivoli.navmodel.Task({task: this._item.defaultTask, dataset: this._dataset});
		}
		
		return this._defaultTask;
	},

	setDefaultTask: function(defaultTask) {		
		// summary:
		//		Set optional default task associated with the item
		this._defaultTask = defaultTask;
	},

	isOrigin: function() {		
		// summary:
		//		Returns true if the item is an origin of a topology or tree
		return (this._item.origin !== undefined ? this._item.origin : false);
	},

	setOrigin: function(isOrigin) {		
		// summary:
		//		Set the value indicating if the Item is an origin of a topology or tree
		this._item.origin = isOrigin;		
	},
	
	/*=====
	itemsAndLinks_onComplete: function(items, links, datasetItems, kwArgs){},
	=====*/
	
	/*=====
	getRelatedItems_kwArgs: function(onComplete, onError, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationships: String
		//		comma delimited list of relationships to determine related items
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		keywords: child, parent, all (if null; value from item will be used)
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns items that are related to the requested item
		var args = {                
			onComplete: function(items, links, datasetItems, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			itemId: this.getId(),
			relationships: this.getLazyLoadRelationships(),
			start: 0,
			count: null,
			properties: null,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		if (!args.relationships) {
			args.relationships = this.getLazyLoadRelationships();
		}
		
		if (!args.properties) {
			args.properties = this.getCachedPropertyIds();
		}
		
		// call the dataset to get the related items
		this._dataset.getRelatedItems(args);
	},

	isExpired: function() {		
		// summary:
		//		Returns true if the item is currently expired
		return (this._expiration ? false : true);
	},

	checkExpiration: function(nowTime) {		
		// summary:
		//		Check the expiration time on the item; return true if expired
		var expired = false;
		
		// already set to null from a previous check
		if (!this._expiration) {
			expired = true;
		}
		// check agaist the time to see if expired
		else if (nowTime >= this._expiration) {
			this._expiration = null;
			expired = true;
		}
		
		return expired;		
	},

	resetExpiration: function() {		
		// summary:
		//		Rest the expiration time for this item
		if (this._dataset.cacheExpires()) {
			this._expiration = this._dataset.getNewExpirationTime();
		}
		else {
			this._expiration = -1;
		}
	},
			
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},

	_validateExpiration: function(kwArgs) {		
		// summary:
		//		If item is expired; mixin with a current item from the dataset cache
		if (this._dataset.cacheExpires()) {
			// expired
			if (this.isExpired()) {
				var properties = null;
				var metadata = false;
				if (kwArgs) {
					properties = kwArgs.properties;
					metadata = kwArgs.metadata;
				}			
				if (!properties) {
					properties = this.getCachedPropertyIds();
				}
		
				var item = this._dataset.getItem({itemId: this.getId(), properties: properties, metadata: metadata});
				if (item) {
					this.mixin(item);
				}			
			}
			// NOT expired; reset expiration since touched
			else {
				this.resetExpiration();
			}
		}
	},
	
	/*=====
	fetchProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous	
	},
	=====*/
	
	_fetchProperties: function(/*fetchProperties_kwArgs*/ kwArgs) {	
		// summary:
		//		Asynchronously returns properties for the item
		this._props = [];
		this._allProps = false;
		this._allMetadata = false;
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		content.optimize = true;
		content.metadata = kwArgs.metadata;
		
		// are we fetching "all" properties
		var fetchAll = (kwArgs.properties.length === 1 && kwArgs.properties[0] === "all");
		
		return dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var props = [];
				var rawProps = result.properties;
				var rawPropsLength = rawProps.length;
				for (var propIndex = 0;  propIndex < rawPropsLength;  ++propIndex) {
					var prop = new ibm.tivoli.navmodel.Property({property: rawProps[propIndex], dataset: this._dataset, objectType: "item"});
					props[propIndex] = prop;
					
					if (fetchAll) {
						this._props[propIndex] = prop;
					}
					
					this._propsById[prop.getId()] = prop;
				}
				
				if (fetchAll) {
					this._allProps = true;
					this._allMetadata = kwArgs.metadata;
				}
				
				kwArgs.onComplete(props, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperties", this._trace, this._version()), ioArgs);			
			})
		});		
	},

	_fetchProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		var prop = null;
				
		var content = ibm.tivoli.navmodel.Utils.getContent();		
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		content.optimize = true;
		content.properties = propId;
		
		dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				if (result.properties && result.properties.length > 0) {
					prop = new ibm.tivoli.navmodel.Property({property: result.properties[0], dataset: this._dataset, objectType: "item"});
				
					// cache property
					this._propsById[propId] = prop;
				}
				// property was not found; remember so we don't ask again
				else {
					this._nullPropIds[propId] = true;
				}		
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperty", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return prop;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.ItemStyle"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.ItemStyle"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.ItemStyle"); 
 
////////////////////////////////////
// ItemStyle
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.ItemStyle", null, {
	// private members       
	_version: null,
	_style: null,
	
	/*=====
	constructor_kwArgs: function(version, style) {
		// version: Float
		//		version used to communicate with rest service
		// style: Object
		//		json item style object
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary
		//		Constructs ItemStyle object.
		this._version = kwArgs.version;
		this._style = kwArgs.style;		
		if (!this._style) {
			this._style = {};
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getAttribute: function(key) {
		return this._style[key];
	},
	
	setAttribute: function(key, value) {
		this._style[key] = value;
	},
	
	getId: function() {
		return this._style.id;
	},
	
	setId: function(id) {
		this._style = id;
	},
	
	getType: function() {
		return this._style.type;
	},
	
	setType: function(type) {
		this._style.type = type;
	},
	
	getLabel: function() {
		if (this._style.label) {
			// string
			if (typeof this._style.label === "string") {
				return this._style.label;
			}
			// object with key/value info
			else {
				return this._style.label.value;
			}
		}
		
		return null;
	},
	
	setLabel: function(label) {
		this._style.label = label;		
	},
	
	getDescription: function() {
		if (this._style.description) {
			// string
			if (typeof this._style.description === "string") {
				return this._style.description;
			}
			// object with key/value info
			else {
				return this._style.description.value;
			}
		}
		
		return null;		
	},
	
	setDescription: function(description) {
		this._style.description = description;			
	},
	
	getSmallImage: function() {
		return this._style.smallImage;
	},
	
	setSmallImage: function(image) {
		this._style.smallImage = image;
	},
	
	getSmallImageWidth: function() {
		return this._style.smallImageWidth;
	},
	
	setSmallImageWidth: function(width) {
		this._style.smallImageWidth = width;
	},
	
	getSmallImageHeight: function() {
		return this._style.smallImageHeight;
	},
	
	setSmallImageHeight: function(height) {
		this._style.smallImageHeight = height;
	},
	
	getLargeImage: function() {
		return this._style.largeImage;
	},
	
	setLargeImage: function(image) {
		this._style.largeImage = image;
	},
	
	getLargeImageWidth: function() {
		return this._style.largeImageWidth;
	},
	
	setLargeImageWidth: function(width) {
		this._style.largeImageWidth = width;
	},
	
	getLargeImageHeight: function() {
		return this._style.largeImageHeight;
	},
	
	setLargeImageHeight: function(height) {
		this._style.largeImageHeight = height;
	},
	
	getScalarImage: function() {
		return this._style.scalarImage;
	},
	
	setScalarImage: function(image) {
		this._style.scalarImage = image;
	},
	
	getScalarImageWidth: function() {
		return this._style.scalarImageWidth;
	},
	
	setScalarImageWidth: function(width) {
		this._style.scalarImageWidth = width;
	},
	
	getScalarImageHeight: function() {
		return this._style.scalarImageHeight;
	},
	
	setScalarImageHeight: function(height) {
		this._style.scalarImageHeight = height;
	},
	
	isScalarImageVisible: function() {
		return (this._style.scalarImageVisible !== undefined ? this._style.scalarImageVisible : true);
	},
	
	setScalarImageVisible: function(visible) {
		this._style.scalarImageVisible = visible;
	},
	
	clone: function(widgetDef) {
		var args = {
			style: {},
			version: this._version
		};		
		
		if (widgetDef) {
			// start with widgetDef default style
			var widgetDefaultStyle = widgetDef.getDefaultItemStyle();
			if (widgetDefaultStyle) {
				dojo.mixin(args.style, widgetDefaultStyle);
			}
		
			// mixin our item style
			dojo.mixin(args.style, this._style);
			
			// mixin widgetDef style
			var widgetStyle = widgetDef.getItemStyle(this.getType());			
			if (widgetStyle) {
				dojo.mixin(args.style, widgetStyle);
			}
		}
		// just mixin our item style
		else {
			dojo.mixin(args.style, this._style);
		}
			
		return new ibm.tivoli.navmodel.ItemStyle(args);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Relationship"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Relationship"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Relationship"); 

// includes








ibm.tivoli.navmodel.Relationship._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Relationship");

////////////////////////////////////
// Relationship
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Relationship", null, {
	// private members       
	_relationship: null,
	_dataset: null,
	_props: null,
	_propsById: null,
	_nullPropIds: null,
	_allProps: false,
	_allMetadata: false,
	_status: null,
	_style: null,
	_defaultTask: null,
	_expiration: null,
	_trace: ibm.tivoli.navmodel.Relationship._trace,

	/*=====
	constructor_kwArgs: function(link, dataset, propIds, allProperties, allMetadata, expiration) {
		// link: Object
		//		json link object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this link
		// propIds: String[]
		//		optional array indicating the property Id's that were requested with 
		//		the item; so it can be determined if any properties are null
		// allProperties: Boolean
		//		optional: indicates if the link already has "all" properties
		// allMetadata: Boolean
		//		optional: indicates if link properties already have metadata
		// expiration: Long
		//		optional: expiration time;  will use (current time + interval) if not supplied
	},
	=====*/
	
	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Relationship object.
		this._relationship = kwArgs.relationship;
		if (this._relationship) {
			// optimization of json on the server can leave out the "id"
			// but we know the default format
			if (!this._relationship.id) {
				this._relationship.id = this._relationship.sourceItem + "_" + this._relationship.type + "_" + this._relationship.targetItem;
			}	
		}
		else {
			this._relationship = {};
		}
		
		this._dataset = kwArgs.dataset;		
		this._props = [];
		this._propsById = {};
		this._nullPropIds = {};
		this._allProps = kwArgs.allProperties;		
		this._allMetadata = kwArgs.allMetadata;
		
		if (this._relationship.properties) {
			var propsLength = this._relationship.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = new ibm.tivoli.navmodel.Property({property: this._relationship.properties[propIndex], dataset: this._dataset, objectType: "relationship"});
				this._props[propIndex] = prop;
				this._propsById[prop.getId()] = prop;
			}
			
			// see if we should mark any properties as null because they were not returned
			if (kwArgs.propIds) {
				propsLength = kwArgs.propIds.length;
				for (propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var propId = kwArgs.propIds[propIndex];				
					if (!this._propsById[propId]) {
						this._nullPropIds[propId] = true;
					}
				}
			}
		}
		
		// expiration was supplied
		if (kwArgs.expiration) {
			this._expiration = kwArgs.expiration;
		}
		// reset to current time + interval
		else {
			this.resetExpiration();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	mixin: function(relationship) {		
		// summary:
		//		Mixin properties and status from this Relationship object
		if (relationship && relationship !== this) {
			this.removeAttribute("style");
			this._style = null;
			
			this.mixinAttributes(relationship.getAttributes());
			this.mixinProperties(relationship._props, relationship._nullPropIds);
			this._status = relationship.getStatuses({widgetDef: null});
			this._defaultTask = relationship.getDefaultTask();
			
			if (relationship._allProps) {
				this._allProps = true;
				
				if (relationship._allMetadata) {
					this._allMetadata = true;
				}
			}
		}
	},

	getDataset: function() {		
		// summary:
		//		Returns the dataset that owns the relationship
		return this._dataset;
	},	

	getObjectType: function() {		
		// summary:
		//		Indicate that this object is a "relationship"
		return "relationship";
	},

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._relationship[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._relationship[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._relationship[key];
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the relationship
		var relationship = dojo.clone(this._relationship);
		
		// make sure "properties" are current as we may have received events
		if (this._props && this._props.length > 0) {
			relationship.properties = [];
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				relationship.properties[propIndex] = this._props[propIndex].getAttributes();
			}
		}
		
		return relationship;
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the relationship
		dojo.mixin(this._relationship, attr);		
	},	

	getUri: function() {
		if (!this._relationship.uri) {
			this._relationship.uri = this._dataset.getAttribute("uri") + "/relationships/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._relationship.uri;
	},
	
	getId: function() {		
		// summary:
		//		Returns relationship id
		return this._relationship.id;
	},

	 setId: function(id) {		
		// summary:
		//		Set the relationship id
		this._relationship.id = id;
	},

	getType: function() {		
		// summary:
		//		Returns the type of relationship
		return this._relationship.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the relationship type
		this._relationship.type = type;
	},

	 getLabel: function() {		
		// summary:
		//		Returns globalized label for the relationship
		return this._relationship.label;
	},

	 setLabel: function(label) {		
		// summary:
		//		 Set the globalized label for the relationship
		this._relationship.label = label;
	},

	getTooltip: function() {		
		// summary:
		//		Returns globalized tooltip for the relationship
		return this._relationship.tooltip;
	},

	setTooltip: function(tooltip) {		
		// summary:
		//		Set the globalized tooltip for the relationship
		this._relationship.tooltip = tooltip;
	},

	getSourceDataset: function() {
		// summary:
		//		Returns the sourceDataset involved in the relationship
		return this._lookUpDataset(this.getSourceDatasetId());
	},	
	
	getSourceDatasetId: function() {		
		// summary:
		//		Returns source dataset id
		return this._relationship.sourceDataset;
	},

	setSourceDatasetId: function(sourceDataset) {		
		// summary:
		//		Set source dataset id
		this._relationship.sourceDataset = sourceDataset;
	},

	getTargetDataset: function() {
		// summary:
		//		Returns the targetDataset involved in the relationship
		return this._lookUpDataset(this.getTargetDatasetId());
	},	
	
	getTargetDatasetId: function() {		
		// summary:
		//		Returns target dataset id
		return this._relationship.targetDataset;
	},

	setTargetDatasetId: function(targetDataset) {		
		// summary:
		//		Set target dataset id
		this._relationship.targetDataset = targetDataset;
	},

	getSourceItem: function() {
		// summary:
		//		Returns the sourceItem involved in the relationship	
		var item = null;
		if (this.getSourceItemId()) {
			var sourceDataset = this.getSourceDataset();
			if (sourceDataset) {
				item = sourceDataset.getItem({itemId: this.getSourceItemId()});
			}
		}		
		return item;
	},	
	
	getSourceItemId: function() {		
		// summary:
		//		Returns source item id
		return this._relationship.sourceItem;
	},

	setSourceItemId: function(sourceItem) {		
		// summary:
		//		Set source item id
		this._relationship.sourceItem = sourceItem;
	},

	getTargetItem: function() {
		// summary:
		//		Returns the targetItem involved in the relationship	
		var item = null;
		if (this.getTargetItemId()) {
			var targetDataset = this.getTargetDataset();
			if (targetDataset) {
				item = targetDataset.getItem({itemId: this.getTargetItemId()});
			}
		}		
		return item;
	},	
	
	getTargetItemId: function() {		
		// summary:
		//		Returns target item id
		return this._relationship.targetItem;
	},

	setTargetItemId: function(targetItem) {		
		// summary:
		//		Set target item id
		this._relationship.targetItem = targetItem;
	},
	
	getLink: function() {
		return this._dataset.getLink(this.getLinkId());
	},
	
	getLinkId: function() {		
		// summary:
		//		Returns parent link id
		return (this._relationship.linkId !== undefined ? this._relationship.linkId : "");
	},

	setLinkId: function(linkId) {		
		// summary:
		//		Set parent link id
		this._relationship.linkId = linkId;
	},
	
	/*=====
	getStatuses_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/
	
	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns ibm.tivoli.navmodel.Status[] objects indicating the relationship status values
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		if (!this._status && this._relationship.status) {
			this._status = [];
			var statusLength = this._relationship.status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				if (this._relationship.status.label && this._relationship.status.image) {
					this._status.push(new ibm.tivoli.navmodel.Status({status: this._relationship.status[statusIndex], version: this._version()}));				
				}				
				else {
					var statusArgs = {
						type: "relationship",
						statusId: this._relationship.status[statusIndex].id,
						statusValue: this._relationship.status[statusIndex].value				
					};
				
					var statusObj = this._dataset.getStatus(statusArgs);
					if (statusObj) {				
						this._status.push(statusObj);
					}
					else {
						this._trace.error("Could not locate status with id=" + statusArgs.statusId + " value=" + statusArgs.statusValue, "getStatuses");
					}
				}
			}			
		}
		
		// merge widget definition status
		var status = this._status;
		if (status && args.widgetDef) {
			status = this._status.slice(0);
			var statusLength2 = this._status.length;
			for (var statusIndex2 = 0;  statusIndex2 < statusLength2;  ++statusIndex2) {
				status[statusIndex2] = this._status[statusIndex2].clone(args.widgetDef, "relationship");
			}					
		}
		
		return status;		
	},

	setStatuses: function(statuses) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Status[] objects indicating the relationship status values
		this._status = statuses;
	},

	supportsProperties: function() {		
		// summary:
		//		Indicates if "Properties" is supported for the relationship
		return (this._relationship.supportsProperties !== undefined ? this._relationship.supportsProperties : true);
	},

	setSupportsProperties: function(supportsProperties) {		
		// summary:
		//		Set whether relationship  supports "Properties"
		this._relationship.supportsProperties = supportsProperties;
	},

	hasCachedMetadata: function() {		
		// summary:
		//		Returns true if relationship has all properties and associated metadata cached
		return this._allMetadata;
	},

	hasAllCachedProperties: function() {		
		// summary:
		//		Returns true if relationship has all properties cached
		return this._allProps;
	},

	hasCachedProperty: function(/*String*/ propId) {
		// summary:
		//		Returns true if we have the requested property in the cache
		// propId
		//		property id
		return (this._propsById[propId] ? true : false);
	},
	
	hasCachedProperties: function(/*String[]*/ properties) {		
		// summary:
		//		Returns true if we have all the requested properties in the cache
		// properties:
		//		array of property ids
		if (properties) {
			var propsLength = properties.length;
			
			// asking if we have "all" properties
			if (propsLength === 1 && properties[0] === "all") {
				return this._allProps;
			}
			
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (!this._propsById[properties[propIndex]]) {
					return false;
				}
			}
		}
		
		return true;
	},

	getCachedPropertyIds: function(expandAll) {		
		// summary:
		//		Returns array of property Ids currently cached in the item
		var propIds = [];
		
		// have all the properties
		if (!expandAll && this._allProps) {
			propIds.push("all");
		}
		// get individual property ids	
		else if (this._props) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				propIds.push(this._props[propIndex].getId());
			}
		}
		return propIds;
	},
	
	/*=====
	properties_onComplete: function(properties, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous
	},
	=====*/

	getProperties: function(/*getProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the relationship
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: ["all"],
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// fetch all properties
		if (args.properties.length === 1 && args.properties[0] === "all") {
			if (this._allProps && (!args.metadata || this._allMetadata)) {				
				if (args.sync) {
					args.onComplete(this._props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._props, args), 0);
				}		
			}
			else {
				this._fetchProperties(args);
			}
		}		
		// fetch selective properties
		else {
			// look up properties in cache
			var props = [];
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var propId = args.properties[propIndex];
				var prop = this._propsById[propId];				
				if (prop && (!args.metadata || prop.hasMetadata())) {
					props[propIndex] = prop;					
				}
				else {
					break;
				}
			}
			
			// if we found all properties in cache then return them
			if (props.length === propsLength) {
				if (args.sync) {
					args.onComplete(props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, props, args), 0);
				}
			}
			// otherwise we need to fetch
			else {
				this._fetchProperties(args);
			}
		}		
	},

	setProperties: function(properties) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Property[] objects for the relationship
		//		
		//		Note: client side only operation;  no request to rest service
		this._propsById = {};
		this._nullPropIds = {};
		this._props = [];
		
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = properties[propIndex];
				this._propsById[prop.getId()] = prop;
			}
		}		
	},

	mixinProperties: function(properties, nullPropIds) {		
		// summary:
		//		Mixin the ibm.tivoli.navmodel.Property[] objects for the relationship
		//		
		//		Note: client side only operation;  no request to rest service
		
		// mixin property objects
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				this.mixinProperty(properties[propIndex]);
			}
		}
		
		// mixin any propIds that were null
		if (nullPropIds) {
			for (var propId in nullPropIds) {
				if (nullPropIds.hasOwnProperty(propId)) {
					this._nullPropIds[propId] = true;
				}
			}
		}
	},

	 getProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// check prop cache
		var prop = this._propsById[propId];
			
		if (!prop) {
			// we have asked for this property before and it wasn't found
			if (this._nullPropIds[propId]) {
				return null;
			}
			
			// check default properties on relationship
			if (this._relationship.properties) {
				var propsLength = this._relationship.properties.length;
				for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var rawProp = this._relationship.properties[propIndex];
					if (rawProp.id === propId) {
						prop = new ibm.tivoli.navmodel.Property({property: rawProp, dataset: this._dataset, objectType: "relationship"});
						
						// cache property
						this._propsById[propId] = prop;
						break;
					}
				}				
			}
		}	
		
		// fetch it
		if (!prop && !this._allProps) {
			prop = this._fetchProperty(propId);
			
			// cache property
			if (prop) {
				this._propsById[propId] = prop;
			}
		}		
		
		return prop;
	},

	setProperty: function(property) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Property for the relationship
		//		
		//		Note: client side only operation;  no request to rest service
		var found = this._propsById[property.getId()];		
		if (found) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (this._props[propIndex].getId() === property.getId()) {
					this._props[propIndex] = property;
					break;
				}
			}
		}
		else {
			this._props.push(property);
		}		
		
		this._propsById[property.getId()] = property;
	},
	
	mixinProperty: function(property) {		
		// summary:
		//		Mixin ibm.tivoli.navmodel.Property for the relationship
		//		
		//		Note: client side only operation;  no request to rest service
		var prop = this._propsById[property.getId()];
		if (prop) {
			prop.mixin(property);
		}
		else {
			this._props.push(property);
			this._propsById[property.getId()] = property;
		}	
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	editProperties_kwArgs: function(onComplete, onError, properties) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		array of property ids to set
	},
	=====*/
	
	editProperties: function(/*editProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously modify properties for the relationship
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.properties && args.properties.length > 0) {
			var relationship = dojo.clone(this._relationship);
			relationship.properties = [];
			
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				relationship.properties[propIndex] = args.properties[propIndex].getAttributes();
			}
			
			var json = dojo.toJson(relationship, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "relationshipView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: false,			
				load: dojo.hitch(this, function(result, ioArgs) {
					args.onComplete(args);						
				}),
				error: dojo.hitch(function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperties", this._trace, this._version());				               
				})
			});	
		}
		// nothing to set
		else {
			setTimeout(dojo.hitch(this, args.onComplete, args), 0);
		}		
	},
	
	 editProperty: function(/*String*/ property) {		
		// summary:
		//		Synchronously modify property for the relationship
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		
		if (property) {
			var relationship = dojo.clone(this._relationship);
			relationship.properties = [property.getAttributes()];
			
			var json = dojo.toJson(relationship, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "relationshipView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: true,			
				load: dojo.hitch(this, function(result, ioArgs) {						
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperty", this._trace, this._version());									              
				})
			});				
		}
		
		if (exception) {
			throw exception;
		}
	},

	hasOwnRelationshipStyle: function() {		
		// summary:
		//		Returns true if the relationship has a LinkStyle that overrides the default
		//		relationship style for the relationship type  (i.e. different than the one
		//		retrieved from the dataset)
		return (this._relationship.style ? true: false);
	},

	/*=====
	getRelationshipStyle_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/
	
	getRelationshipStyle: function(/*getRelationshipStyle_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.LinkStyle for this relationship
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (!this._style) {
			if (this._relationship.linkStyle) {
				this._style = new ibm.tivoli.navmodel.LinkStyle({style: this._relationship.style, version: this._version()});
			}
			else {
				this._style = this._dataset.getRelationshipStyle({relationshipType: this.getType(), widgetDef: null});
			}
		}
		
		// merge widget definition styles 
		var style = this._style;
		if (style && args.widgetDef) {
			// clone and merge widget definition attributes				
			if (style) {
				style = style.clone(args.widgetDef, "relationship");
			}
			// unknown style; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getRelationshipStyle(this.getType());
				if (widgetStyle) {
					style = this._dataset._createLinkStyle(args.widgetDef, widgetStyle, "relationship");
				}
			}							
		}
		
		return style;
	},

	setRelationshipStyle: function(style) {		
		// summary:
		//		Set link style for the link
		this._style = style;
	},

	getDefaultTask: function() {		
		// summary:
		//		Returns optional default task associated with the relationship
		if (!this._defaultTask && this._relationship.defaultTask) {
			this._defaultTask = new ibm.tivoli.navmodel.Task({task: this._relationship.defaultTask, dataset: this._dataset});
		}
		
		return this._defaultTask;
	},

	setDefaultTask: function(defaultTask) {		
		// summary:
		//		Set optional default task associated with the link
		this._defaultTask = defaultTask;
	},

	isExpired: function() {		
		// summary:
		//		Returns true if the item is currently expired
		return (this._expiration ? false : true);
	},

	checkExpiration: function(nowTime) {		
		// summary:
		//		Check the expiration time on the item; return true if expired
		var expired = false;
		
		// already set to null from a previous check
		if (!this._expiration) {
			expired = true;
		}
		// check agaist the time to see if expired
		else if (nowTime >= this._expiration) {
			this._expiration = null;
			expired = true;
		}
		
		return expired;		
	},

	resetExpiration: function() {		
		// summary:
		//		Rest the expiration time for this item
		if (this._dataset.cacheExpires()) {
			this._expiration = this._dataset.getNewExpirationTime();
		}
		else {
			this._expiration = -1;
		}
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},

	_validateExpiration: function(kwArgs) {		
		// summary:
		//		If item is expired; mixin with a current item from the dataset cache
		if (this._dataset.cacheExpires()) {
			// expired
			if (this.isExpired()) {
				var properties = null;
				var metadata = false;
				if (kwArgs) {
					properties = kwArgs.properties;
					metadata = kwArgs.metadata;
				}			
				if (!properties) {
					properties = this.getCachedPropertyIds();
				}
		
				var relationship = this._dataset.getRelationship({relationshipId: this.getId(), properties: properties, metadata: metadata});
				if (relationship) {
					this.mixin(relationship);
				}			
			}
			// NOT expired; reset expiration since touched
			else {
				this.resetExpiration();
			}
		}
	},

	_lookUpDataset: function(/*String*/ datasetId) {
		// summary:
		//		Look up dataset (i.e. find descendent dataset if not the main dataset)
		var dataset = this._dataset;
		if (datasetId && datasetId !== this._dataset.getId()) {
			dataset = this._dataset.getDescendentDataset(datasetId);
			
			if (!dataset) {
				this._trace.error("childDatasetId=" + datasetId + " is not a descendent of datasetId=" + this._dataset.getId() , "_lookUpDataset");
			}
		}
		
		return dataset;
	},
	
	/*=====
	fetchProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous	
	},
	=====*/
	
	_fetchProperties: function(/*fetchProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the link
		this._props = [];
		this._allProps = false;
		this._allMetadata = false;
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		content.optimize = true;
		content.metadata = kwArgs.metadata;
		
		// are we fetching "all" properties
		var fetchAll = (kwArgs.properties.length === 1 && kwArgs.properties[0] === "all");
		
		return dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "relationshipView")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var props = [];
				var rawProps = result.properties;
				var rawPropsLength = rawProps.length;
				for (var propIndex = 0;  propIndex < rawPropsLength;  ++propIndex) {
					var prop = new ibm.tivoli.navmodel.Property({property: rawProps[propIndex], dataset: this._dataset, objectType: "relationship"});
					props[propIndex] = prop;
					
					if (fetchAll) {
						this._props[propIndex] = prop;
					}
					
					this._propsById[prop.getId()] = prop;
				}
				
				if (fetchAll) {
					this._allProps = true;
					this._allMetadata = kwArgs.metadata;
				}
				
				kwArgs.onComplete(props, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperties", this._trace, this._version()), ioArgs);			
			})
		});		
	},

	_fetchProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		var prop = null;
				
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		content.optimize = true;
		content.properties = propId;
		
		dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "relationshipView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				if (result.properties && result.properties.length > 0) {
					prop = new ibm.tivoli.navmodel.Property({property: result.properties[0], dataset: this._dataset, objectType: "relationship"});
				
					// cache property
					this._propsById[propId] = prop;
				}
				// property was not found; remember so we don't ask again
				else {
					this._nullPropIds[propId] = true;
				}					
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperty", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return prop;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Link"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Link"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Link"); 
 
// includes









ibm.tivoli.navmodel.Link._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Link");

////////////////////////////////////
// Link
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Link", null, {
	// private members       
	_link: null,
	_dataset: null,
	_relationships: null,
	_props: null,
	_propsById: null,
	_nullPropIds: null,
	_allProps: false,
	_allMetadata: false,
	_status: null,
	_style: null,
	_defaultTask: null,
	_expiration: null,
	_trace: ibm.tivoli.navmodel.Link._trace,
	
	/*=====
	constructor_kwArgs: function(link, dataset, propIds, allProperties, allMetadata, expiration) {
		// link: Object
		//		json link object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this link
		// propIds: String[]
		//		optional array indicating the property Id's that were requested with 
		//		the item; so it can be determined if any properties are null
		// allProperties: Boolean
		//		optional: indicates if the link already has "all" properties
		// allMetadata: Boolean
		//		optional: indicates if link properties already have metadata
		// expiration: Long
		//		optional: expiration time;  will use (current time + interval) if not supplied
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Link object.
		this._link = kwArgs.link;
		if (this._link) {
			// optimization of json on the server can leave out the "id"
			// but we know the default format
			if (!this._link.id) {
				if (this._link.item1 < this._link.item2) {
					this._link.id = this._link.item1 + "_" + this._link.item2;
				}
				else {
					this._link.id = this._link.item2 + "_" + this._link.item1;
				}
			}
		}
		else {
			this._link = {};
		}
		
		this._dataset = kwArgs.dataset;
		this._props = [];
		this._propsById = {};
		this._nullPropIds = {};
		this._allProps = kwArgs.allProperties;		
		this._allMetadata = kwArgs.allMetadata;
		
		if (this._link.properties) {
			var propsLength = this._link.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = new ibm.tivoli.navmodel.Property({property: this._link.properties[propIndex], dataset: this._dataset, objectType: "link"});
				this._props[propIndex] = prop;
				this._propsById[prop.getId()] = prop;
			}
			
			// see if we should mark any properties as null because they were not returned
			if (kwArgs.propIds) {
				propsLength = kwArgs.propIds.length;
				for (propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var propId = kwArgs.propIds[propIndex];				
					if (!this._propsById[propId]) {
						this._nullPropIds[propId] = true;
					}
				}
			}
		}
				
		// expiration was supplied
		if (kwArgs.expiration) {
			this._expiration = kwArgs.expiration;
		}
		// reset to current time + interval
		else {
			this.resetExpiration();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	mixin: function(link) {		
		// summary:
		//		Mixin properties and status from this Link object
		if (link && link !== this) {
			this.removeAttribute("style");
			this._style = null;
			
			this.mixinAttributes(link.getAttributes());
			this.mixinProperties(link._props, link._nullPropIds);
			this._status = link.getStatuses({widgetDef: null});
			this._defaultTask = link.getDefaultTask();
			this._relationships = link.getRelationships();
			
			if (link._allProps) {
				this._allProps = true;
				
				if (link._allMetadata) {
					this._allMetadata = true;
				}
			}
		}		
	},
	
	getDataset: function() {		
		// summary:
		//		Returns the dataset that owns the link
		return this._dataset;
	},

	getObjectType: function() {		
		// summary:
		//		Indicate that this object is a "link"
		return "link";
	},

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._link[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._link[key] = value;
	},

	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._link[key];
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the link
		var link = dojo.clone(this._link);
		
		// make sure "properties" are current as we may have received events
		if (this._props && this._props.length > 0) {
			link.properties = [];
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				link.properties[propIndex] = this._props[propIndex].getAttributes();
			}
		}
		
		return link;
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the link
		dojo.mixin(this._link, attr);		
	},

	getUri: function() {
		if (!this._link.uri) {
			this._link.uri = this._dataset.getAttribute("uri") + "/links/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._link.uri;
	},
	
	getId: function() {		
		// summary:
		//		Returns link id
		return this._link.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the link id
		this._link.id = id;		
	},

	getType: function() {		
		// summary:
		//		Returns the type of link
		return this._link.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the link type
		this._link.type = type;		
	},

	getLabel: function() {		
		// summary:
		//		Returns globalized label for the link
		return this._link.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the globalized label for the link
		this._link.label = label;		
	},

	getTooltip: function() {		
		// summary:
		//		Returns globalized tooltip for the link
		return this._link.tooltip;
	},

	setTooltip: function(tooltip) {		
		// summary:
		//		Set the globalized tooltip for the link
		this._link.tooltip = tooltip;
	},
	
	isContainment: function() {		
		// summary:
		//		Returns true if this link represents containment instead of
		//		connection via normal lines
		return (this._link.containment ? this._link.containment : false);
	},
	
	setContainment: function(isContainment) {		
		// summary:
		//		Set the value indicating containment
		this._link.containment = isContainment;
	},

	getDirection: function() {		
		// summary:
		//		Returns the direction associated with the link
		//		
		//		|	values:	"1_2"   	means direction of arrow flows from item1 to item2
		//		|		"2_1"		means direction of arrow flows from item2 to item1
		//		|		"both"	means that arrow flows in both directions
		return (this._link.direction ? this._link.direction : "1_2");
	},

	setDirection: function(direction) {		
		// summary:
		//		Set the direction associated with the link
		//		
		//		|	values:	"1_2"   	means direction of arrow flows from item1 to item2
		//		|		"2_1"		means direction of arrow flows from item2 to item1
		//		|		"both"	means that arrow flows in both directions
		this._link.direction = direction;
	},
	
	/*=====
	getStatuses_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/

	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns ibm.tivoli.navmodel.Status[] objects indicating the link status values
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		if (!this._status && this._link.status) {
			this._status = [];
			var statusLength = this._link.status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				if (this._link.status.label && this._link.status.image) {
					this._status.push(new ibm.tivoli.navmodel.Status({status: this._link.status[statusIndex], version: this._version()}));				
				}				
				else {
					var statusArgs = {
						type: "link",
						statusId: this._link.status[statusIndex].id,
						statusValue: this._link.status[statusIndex].value				
					};
				
					var statusObj = this._dataset.getStatus(statusArgs);
					if (statusObj) {				
						this._status.push(statusObj);
					}
					else {
						this._trace.error("Could not locate status with id=" + statusArgs.statusId + " value=" + statusArgs.statusValue, "getStatuses");
					}
				}
			}			
		}
		
		// merge widget definition status
		var status = this._status;
		if (status && args.widgetDef) {
			status = this._status.slice(0);
			var statusLength2 = this._status.length;
			for (var statusIndex2 = 0;  statusIndex2 < statusLength2;  ++statusIndex2) {
				status[statusIndex2] = this._status[statusIndex2].clone(args.widgetDef, "link");
			}
		}					
		
		return status;		
	},

	setStatuses: function(statuses) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Status[] objects indicating the link status values
		this._status = statuses;
	},

	getDataset1: function() {
		// summary:
		//		Returns the dataset1 involved in the link	
		return this._lookUpDataset(this.getDataset1Id());
	},
	
	getDataset1Id: function() {		
		// summary:
		//		Returns the id of the dataset1 involved in the link;  this value will
		//		be undefined if the link is NOT between two different datasets
		return this._link.dataset1;
	},

	setDataset1Id: function(dataset1) {		
		// summary:
		//		Set the id of the dataset1 involved in the link
		this._link.dataset1 = dataset1;
	},

	getDataset2: function() {
		// summary:
		//		Returns the dataset2 involved in the link	
		return this._lookUpDataset(this.getDataset2Id());
	},
	
	getDataset2Id: function() {		
		// summary:
		//		Returns the id of the dataset2 involved in the link;  this value will
		//		be undefined if the link is NOT between two different datasets
		return this._link.dataset2;
	},

	setDataset2Id: function(dataset2) {		
		// summary:
		//		Set the id of the dataset2 involved in the link
		this._link.dataset2 = dataset2;
	},

	getItem1: function() {
		// summary:
		//		Returns the item1 involved in the link		
		var item = null;
		if (this.getItem1Id()) {
			var dataset1 = this.getDataset1();
			if (dataset1) {
				item = dataset1.getItem({itemId: this.getItem1Id()});
			}
		}		
		return item;
	},
	
	getItem1Id: function() {		
		// summary:
		//		Returns the id of item1 involved in the link;  this value can be undefined
		//		if the link is between two different datasets and the connection point
		//		is to the dataset and not a link within it
		return this._link.item1;
	},

	setItem1Id: function(item1) {		
		// summary:
		//		Set the id of item1 involved in the link
		this._link.item1 = item1;
	},

	getItem2: function() {
		// summary:
		//		Returns the item2 involved in the link		
		var item = null;
		if (this.getItem2Id()) {
			var dataset2 = this.getDataset2();
			if (dataset2) {
				item = dataset2.getItem({itemId: this.getItem2Id()});
			}
		}		
		return item;
	},
		
	getItem2Id: function() {		
		// summary:
		//		Returns the id of item2 involved in the link;  this value can be undefined
		//		if the link is between two different datasets and the connection point
		//		is to the dataset and not a link within it
		return this._link.item2;
	},

	setItem2Id: function(item2) {		
		// summary:
		//		Set the id of item1 involved in the link
		this._link.item2 = item2;
	},

	getRelationshipCount: function() {		
		// summary:
		//		Returns the number of relationships involved in the link
		return this.getRelationships().length;
	},

	getRelationship: function(relationshipId) {		
		// summary:
		//		Returns requested relationship if it exists
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		var relationships = this.getRelationships();
		if (relationships) {
			var relLength = relationships.length;
			for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
				if (relationships[relIndex].getId() === relationshipId) {
					return relationships[relIndex];
				}
			}
		}
		
		return null;
	},

	getRelationships: function() {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.Relationship[] objects that are involved in the link
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		if (!this._relationships) {
			var direction = this.getDirection();
			this._relationships = [];
			var relationshipsLength = this._link.relationships.length;
			for (var relIndex = 0;  relIndex < relationshipsLength;  ++relIndex) {			
				var relationshipObj = this._link.relationships[relIndex];
				
				if (!relationshipObj.linkId) {
					relationshipObj.linkId = this.getId();
				}
				
				if (!relationshipObj.label) {
					relationshipObj.label = this.getLabel();
				}
				
				if (!relationshipObj.type) {
					relationshipObj.type = this.getType();
				}
				
				if (!relationshipObj.sourceDataset) {
					switch(direction) {
						case "1_2":
						case "none":
							if (this.getDataset1Id()) {
								relationshipObj.sourceDataset = this.getDataset1Id();
							}
							break;
						case "2_1":
							if (this.getDataset2Id()) {
								relationshipObj.sourceDataset = this.getDataset2Id();
							}
					}	
				}
				
				if (!relationshipObj.sourceItem) {
					switch(direction) {
						case "1_2":
						case "none":
							if (this.getItem1Id()) {
								relationshipObj.sourceItem = this.getItem1Id();
							}
							break;
						case "2_1":
							if (this.getItem2Id()) {
								relationshipObj.sourceItem = this.getItem2Id();
							}
					}	
				}
				
				if (!relationshipObj.targetDataset) {
					switch(direction) {
						case "1_2":
						case "none":
							if (this.getDataset2Id()) {
								relationshipObj.targetDataset = this.getDataset2Id();
							}
							break;
						case "2_1":
							if (this.getDataset1Id()) {
								relationshipObj.targetDataset = this.getDataset1Id();
							}
					}
				}
				
				if (!relationshipObj.targetItem) {
					switch(direction) {
						case "1_2":
						case "none":
							if (this.getItem2Id()) {
								relationshipObj.targetItem = this.getItem2Id();
							}
							break;
						case "2_1":
							if (this.getItem1Id()) {
								relationshipObj.targetItem = this.getItem1Id();
							}
					}					
				}
				
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: relationshipObj, dataset: this._dataset});
				this._relationships[relIndex] = this._dataset._cacheRelationship(relationship);
			}
		}

		return this._relationships;
	},

	setRelationships: function(relationships) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Relationship[] objects that are involved in the link
		this._relationships = relationships;
	},

	supportsProperties: function() {		
		// summary:
		//		Indicates if "Properties" is supported for the link
		return (this._link.supportsProperties !== undefined ? this._link.supportsProperties : true);
	},

	setSupportsProperties: function(supportsProperties) {		
		// summary:
		//		Set whether link  supports "Properties"
		this._link.supportsProperties = supportsProperties;
	},

	hasCachedMetadata: function() {		
		// summary:
		//		Returns true if link has all properties and associated metadata cached
		return this._allMetadata;
	},

	hasAllCachedProperties: function() {		
		// summary:
		//		Returns true if link has all proprties cached
		return this._allProps;
	},

	hasCachedProperty: function(/*String*/ propId) {
		// summary:
		//		Returns true if we have the requested property in the cache
		// propId
		//		property id
		return (this._propsById[propId] ? true : false);
	},
	
	hasCachedProperties: function(/*String[]*/ properties) {		
		// summary:
		//		Returns true if we have all the requested properties in the cache
		// properties:
		//		array of property ids
		if (properties) {
			var propsLength = properties.length;
			
			// asking if we have "all" properties
			if (propsLength === 1 && properties[0] === "all") {
				return this._allProps;
			}
			
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (!this._propsById[properties[propIndex]]) {
					return false;
				}
			}
		}
		
		return true;
	},

	getCachedPropertyIds: function(expandAll) {		
		// summary:
		//		Returns array of property Ids currently cached in the item
		var propIds = [];
		
		// have all the properties
		if (!expandAll && this._allProps) {
			propIds.push("all");
		}
		// get individual property ids	
		else if (this._props) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				propIds.push(this._props[propIndex].getId());
			}
		}
		return propIds;
	},
	
	/*=====
	properties_onComplete: function(properties, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous
	},
	=====*/

	getProperties: function(/*getProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the link
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: ["all"],
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// fetch all properties
		if (args.properties.length === 1 && args.properties[0] === "all") {
			if (this._allProps && (!args.metadata || this._allMetadata)) {				
				if (args.sync) {
					args.onComplete(this._props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._props, args), 0);
				}		
			}
			else {
				this._fetchProperties(args);
			}
		}		
		// fetch selective properties
		else {
			// look up properties in cache
			var props = [];
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var propId = args.properties[propIndex];
				var prop = this._propsById[propId];				
				if (prop && (!args.metadata || prop.hasMetadata())) {
					props[propIndex] = prop;					
				}
				else {
					break;
				}
			}
			
			// if we found all properties in cache then return them
			if (props.length === propsLength) {
				if (args.sync) {
					args.onComplete(props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, props, args), 0);
				}
			}
			// otherwise we need to fetch
			else {
				this._fetchProperties(args);
			}
		}		
	},
	
	setProperties: function(properties) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Property[] objects for the link
		//		
		//		Note: client side only operation;  no request to rest service
		this._propsById = {};
		this._nullPropIds = {};
		this._props = [];
		
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = properties[propIndex];
				this._propsById[prop.getId()] = prop;
			}
		}		
	},

	mixinProperties: function(properties, nullPropIds) {		
		// summary:
		//		Mixin the ibm.tivoli.navmodel.Property[] objects for the link
		//		
		//		Note: client side only operation;  no request to rest service
		
		// mixin property objects
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				this.mixinProperty(properties[propIndex]);
			}
		}
		
		// mixin any propIds that were null
		if (nullPropIds) {
			for (var propId in nullPropIds) {
				if (nullPropIds.hasOwnProperty(propId)) {
					this._nullPropIds[propId] = true;
				}
			}
		}
	},

	 getProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// check prop cache
		var prop = this._propsById[propId];
			
		if (!prop) {
			// we have asked for this property before and it wasn't found
			if (this._nullPropIds[propId]) {
				return null;
			}
			
			// check default properties on link	
			if (this._link.properties) {
				var propsLength = this._link.properties.length;
				for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var rawProp = this._link.properties[propIndex];
					if (rawProp.id === propId) {
						prop = new ibm.tivoli.navmodel.Property({property: rawProp, dataset: this._dataset, objectType: "link"});
						
						// cache property
						this._propsById[propId] = prop;
						break;
					}
				}				
			}
		}	
		
		// fetch it
		if (!prop && !this._allProps) {
			prop = this._fetchProperty(propId);
			
			// cache property
			if (prop) {
				this._propsById[propId] = prop;
			}
		}		
		
		return prop;
	},

	setProperty: function(property) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Property for the item
		//		
		//		Note: client side only operation;  no request to rest service
		var found = this._propsById[property.getId()];		
		if (found) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (this._props[propIndex].getId() === property.getId()) {
					this._props[propIndex] = property;
					break;
				}
			}
		}
		else {
			this._props.push(property);
		}		
		
		this._propsById[property.getId()] = property;
	},

	mixinProperty: function(property) {		
		// summary:
		//		Mixin ibm.tivoli.navmodel.Property for the link
		//		
		//		Note: client side only operation;  no request to rest service
		var prop = this._propsById[property.getId()];
		if (prop) {
			prop.mixin(property);
		}
		else {
			this._props.push(property);
			this._propsById[property.getId()] = property;
		}	
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	editProperties_kwArgs: function(onComplete, onError, properties) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		array of property ids to set
	},
	=====*/

	editProperties: function(/*editProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously modify properties for the link
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.properties && args.properties.length > 0) {
			var link = dojo.clone(this._link);
			link.properties = [];
			
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				link.properties[propIndex] = args.properties[propIndex].getAttributes();
			}
			
			var json = dojo.toJson(link, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: false,			
				load: dojo.hitch(this, function(result, ioArgs) {
					args.onComplete(args);						
				}),
				error: dojo.hitch(function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperties", this._trace, this._version());				               
				})
			});	
		}
		// nothing to set
		else {
			setTimeout(dojo.hitch(this, args.onComplete, args), 0);
		}		
	},

	 editProperty: function(/*String*/ property) {		
		// summary:
		//		Synchronously modify property for the link
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		
		if (property) {
			var link = dojo.clone(this._link);
			link.properties = [property.getAttributes()];
			
			var json = dojo.toJson(link, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: true,			
				load: dojo.hitch(this, function(result, ioArgs) {						
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperty", this._trace, this._version());									              
				})
			});				
		}
		
		if (exception) {
			throw exception;
		}
	},

	hasOwnLinkStyle: function() {		
		// summary:
		//		Returns true if the link has a LinkStyle that overrides the default
		//		link style for the link type  (i.e. different than the one retrieved
		//		from the dataset)
		return (this._link.style ? true: false);
	},
	
	/*=====
	getLinkStyle_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override
	},
	=====*/
	
	getLinkStyle: function(/*getLinkStyle_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.LinkStyle for this link
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (!this._style) {
			if (this._link.style) {
				this._style = new ibm.tivoli.navmodel.LinkStyle({style: this._link.style, version: this._version()});
			}
			else {
				this._style = this._dataset.getLinkStyle({linkType: this.getType(), widgetDef: null});
			}
		}
		
		// merge widget definition styles 
		var style = this._style;
		if (style && args.widgetDef) {
			// clone and merge widget definition attributes				
			if (style) {
				style = style.clone(args.widgetDef, "link");
			}
			// unknown style; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getLinkStyle(this.getType());
				if (widgetStyle) {
					style = this._dataset._createLinkStyle(args.widgetDef, widgetStyle, "link");
				}
			}							
		}
		
		return style;
	},

	setLinkStyle: function(style) {		
		// summary:
		//		Set link style for the link
		this._style = style;
	},

	getDefaultTask: function() {		
		// summary:
		//		Returns optional default task associated with the link
		if (!this._defaultTask && this._link.defaultTask) {
			this._defaultTask = new ibm.tivoli.navmodel.Task({task: this._link.defaultTask, dataset: this._dataset});
		}
		
		return this._defaultTask;
	},

	setDefaultTask: function(defaultTask) {		
		// summary:
		//		Set optional default task associated with the link
		this._defaultTask = defaultTask;
	},

	isExpired: function() {		
		// summary:
		//		Returns true if the item is currently expired
		return (this._expiration ? false : true);
	},

	checkExpiration: function(nowTime) {		
		// summary:
		//		Check the expiration time on the item; return true if expired
		var expired = false;
		
		// already set to null from a previous check
		if (!this._expiration) {
			expired = true;
		}
		// check agaist the time to see if expired
		else if (nowTime >= this._expiration) {
			this._expiration = null;
			expired = true;
		}
		
		return expired;		
	},

	resetExpiration: function() {		
		// summary:
		//		Rest the expiration time for this item
		if (this._dataset.cacheExpires()) {
			this._expiration = this._dataset.getNewExpirationTime();
		}
		else {
			this._expiration = -1;
		}
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},

	_validateExpiration: function(kwArgs) {		
		// summary:
		//		If item is expired; mixin with a current item from the dataset cache
		if (this._dataset.cacheExpires()) {
			// expired
			if (this.isExpired()) {
				var properties = null;
				var metadata = false;
				if (kwArgs) {
					properties = kwArgs.properties;
					metadata = kwArgs.metadata;
				}			
				if (!properties) {
					properties = this.getCachedPropertyIds();
				}
		
				var link = this._dataset.getLink({linkId: this.getId(), properties: properties, metadata: metadata});
				if (link) {
					this.mixin(link);
				}			
			}
			// NOT expired; reset expiration since touched
			else {
				this.resetExpiration();
			}
		}
	},	
	
	_lookUpDataset: function(/*String*/ datasetId) {
		// summary:
		//		Look up dataset (i.e. find descendent dataset if not the main dataset)
		var dataset = this._dataset;
		if (datasetId && datasetId !== this._dataset.getId()) {
			dataset = this._dataset.getDescendentDataset(datasetId);
			
			if (!dataset) {
				this._trace.error("childDatasetId=" + datasetId + " is not a descendent of datasetId=" + this._dataset.getId() , "_lookUpDataset");
			}
		}
		
		return dataset;
	},
        
	/*=====
	fetchProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous	
	},
	=====*/
	
	_fetchProperties: function(/*fetchProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the link
		this._props = [];
		this._allProps = false;
		this._allMetadata = false;
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		content.optimize = true;
		content.metadata = kwArgs.metadata;
		
		// are we fetching "all" properties
		var fetchAll = (kwArgs.properties.length === 1 && kwArgs.properties[0] === "all");
		
		return dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var props = [];
				var rawProps = result.properties;
				var rawPropsLength = rawProps.length;
				for (var propIndex = 0;  propIndex < rawPropsLength;  ++propIndex) {
					var prop = new ibm.tivoli.navmodel.Property({property: rawProps[propIndex], dataset: this._dataset, objectType: "link"});
					props[propIndex] = prop;
					
					if (fetchAll) {
						this._props[propIndex] = prop;
					}
					
					this._propsById[prop.getId()] = prop;
				}
				
				if (fetchAll) {
					this._allProps = true;
					this._allMetadata = kwArgs.metadata;
				}
				
				kwArgs.onComplete(props, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperties", this._trace, this._version()), ioArgs);			
			})
		});		
	},

	_fetchProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		var prop = null;
			
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		content.optimize = true;
		content.properties = propId;
		
		dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				if (result.properties && result.properties.length > 0) {
					prop = new ibm.tivoli.navmodel.Property({property: result.properties[0], dataset: this._dataset, objectType: "link"});
				
					// cache property
					this._propsById[propId] = prop;
				}
				// property was not found; remember so we don't ask again
				else {
					this._nullPropIds[propId] = true;
				}
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperty", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return prop;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.ChildDatasetItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.ChildDatasetItem"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.ChildDatasetItem"); 
 
////////////////////////////////////
// ChildDatasetItem
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.ChildDatasetItem", null, {
	// private members       
	_childDataset: null,
	_dataset: null,
	
	/*=====
	constructor_kwArgs: function(childDataset, dataset) {
		// childDataset: Object
		//		json child dataset object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this child dataset
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs ChildDatasetItem object.
		this._childDataset = kwArgs.childDataset;
		if (!this._childDataset) {
			this._childDataset = {};
		}
		
		this._dataset = kwArgs.dataset;		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getDataset: function() {
		// summary:
		//		Returns the dataset that owns this ChildDatasetItem
		return this._dataset;
	},
	
	getChildDataset: function() {
		// summary:
		//		Returns the dataset represented by this ChildDatasetItem
		return this._dataset.getChildDataset(this.getId());
		
	},
	
	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._childDataset[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._childDataset[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._childDataset[key];
	},

	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the child dataset
		return dojo.clone(this._childDataset);		
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the child dataset
		dojo.mixin(this._childDataset, attr);		
	},

	getObjectType: function() {
		// summary:
		//		Indicate that this object is an "childDataset"
		return "childDataset";
	},

	getId: function() {
		// summary:
		//		Returns item id
		return this._childDataset.id;
	},
	
	setId: function(id) {		
		// summary:
		//		Set the child dataset id
		this._childDataset.id = id;		
	},	

	getType: function() {
		// summary:
		//		Returns the type of item
		return this._childDataset.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the child dataset type
		this._childDataset.type = type;		
	},
	
	getLabel: function() {
		//summary:
		//		Returns globalized label for the child dataset	
		return this._childDataset.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the globalized label for the child dataset
		this._childDataset.label = label;		
	},
	
	isOrigin: function() {		
		// summary:
		//		Returns true if the childDataset is an origin of a topology or tree
		return (this._childDataset.origin !== undefined ? this._childDataset.origin : false);
	},

	setOrigin: function(isOrigin) {		
		// summary:
		//		Set the value indicating if the childDataset is an origin of a topology or tree
		this._childDataset.origin = isOrigin;		
	},
		
	isLazyLoad: function() {
		// summary:
		//		Returns true if the dataset designates that it should be lazy loaded
		return (this._childDataset.lazyLoad !== undefined ? this._childDataset.lazyLoad : false);
	},
	
	setLazyLoad: function(lazyLoad) {
		// summary:
		//		Set the lazy load attribute
		this._childDataset.lazyLoad = lazyLoad;
	},
	
	getLazyLoadRelationships: function() {		
		// summary:
		//		Returns the relationships that should be explored for related items
		return this._childDataset.relationshipsKey;
	},

	setLazyLoadRelationships: function(relationshipsKey) {		
		// summary:
		//		Set the relationships that should be explored for related items
		this._childDatasets.relationshipsKey = relationshipsKey;
	},
	
	isExpired: function() {
		// summary:
		//		Returns true if the child dataset item is currently expired
		return false;
	},
	
	getCachedPropertyIds: function(expandAll){
		// summary:
		//		Child dataset does not have properties; but need method to be consistent
		//		with item, link, and relationship
		return [];
	},
	
	getProperties: function(kwArgs) {
		// summary:
		//		Child dataset does not have properties; but need method to be consistent
		//		with item, link, and relationship
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: ["all"],
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		if (args.sync) {
			args.onComplete(null, args);			
		}
		else {
			setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
		}
	},
	
	getProperty: function(propId) {
		// summary:
		//		Child dataset does not have properties; but need method to be consistent
		//		with item, link, and relationship
		return null;
	},
	
	hasSiblings: function() {
		// summary:
		//		Returns true if the child dataset is related to sibling
		//		items or sibling datasets	
		return (this._childDataset.relationshipsKey ? true : false);
	},
	
	/*=====
	itemsAndLinks_onComplete: function(items, links, datasetItems, kwArgs){},
	=====*/
	
	/*=====
	getRelatedItems_kwArgs: function(onComplete, onError, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationships: String
		//		comma delimited list of relationships to determine related items
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		keywords: child, parent, all (if null; value from item will be used)
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns items that are related to the requested child dataset item
		var args = {                
			onComplete: function(items, links, datasetItems, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			childDatasetId: this.getId(),
			relationships: null,
			start: 0,
			count: null,
			properties: null,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		if (!args.relationships) {
			args.relationships = this.getLazyLoadRelationships();
		}
		
		// call the dataset to get the related items
		this._dataset.getChildDatasetRelatedItems(args);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Selection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Selection"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Selection"); 
 
////////////////////////////////////
// Selection
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Selection", null, {
	// private members
	_version: null,
	_selection: null,
	
	/*=====
	constructor_kwArgs: function(version) {
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Selection object.
		this._version = kwArgs.version;
		this.clear();
	},
	
	/////////////////////////////
	// public methods
	/////////////////////////////

	getAttributes: function() {		
		// summary:
		//		Returns selection value
		return this._selection;
	},

	clear: function() {		
		// summary:
		//		Clear out selection contents
		this._selection = {selected: false, itemIds: [], linkIds: [], relationshipIds: [], childDatasets: {}};
	},

	isEmpty: function() {		
		// summary:
		//		Returns true if selection is empty (i.e. no itemIds, linkIds, or childDataset values)
		return this._isEmpty(this._selection);
	},

	addItem: function(qualifiedItemId) {		
		// summary:
		//		Add an item id to the selection
		var ids = this._getIds(qualifiedItemId);
		var selection = this._getSelection(ids);		
		if (selection) {
			this._addItem(selection, ids[ids.length-1]);
		}
	},

	addItems: function(qualifiedItemIds) {		
		// summary:
		//		Add multiple item ids to the selection
		var idsLength = qualifiedItemIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var ids = this._getIds(qualifiedItemIds[idIndex]);
			var selection = this._getSelection(ids);		
			if (selection) {
				this._addItem(selection, ids[ids.length-1]);
			}
		}		
	},

	removeItem: function(qualifiedItemId) {		
		// summary:
		//		Remove an item id from the selection
		var ids = this._getIds(qualifiedItemId);
		var selection = this._getSelection(ids);		
		if (selection) {
			this._removeItem(selection, ids[ids.length-1]);						
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeItems: function(qualifiedItemIds) {		
		// summary:
		//		Remove multiple item ids from the selection
		var idsLength = qualifiedItemIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var ids = this._getIds(qualifiedItemIds[idIndex]);
			var selection = this._getSelection(ids);		
			if (selection) {
				this._removeItem(selection, ids[ids.length-1]);						
			}			
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	addLink: function(qualifiedLinkId) {		
		// summary:
		//		Add a link id to the selection
		var ids = this._getIds(qualifiedLinkId);
		var selection = this._getSelection(ids);		
		if (selection) {
			this._addLink(selection, ids[ids.length-1]);
		}
	},

	addLinks: function(qualifiedLinkIds) {		
		// summary:
		//		Add multiple link ids to the selection
		var idsLength = qualifiedLinkIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var ids = this._getIds(qualifiedLinkIds[idIndex]);
			var selection = this._getSelection(ids);		
			if (selection) {
				this._addLink(selection, ids[ids.length-1]);
			}
		}
	},

	removeLink: function(qualifiedLinkId) {		
		// summary:
		//		Remove a link id from the selection
		var ids = this._getIds(qualifiedLinkId);
		var selection = this._getSelection(ids);		
		if (selection) {
			this._removeLink(selection, ids[ids.length-1]);			
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeLinks: function(qualifiedLinkIds) {		
		// summary:
		//		Remove multiple link ids from the selection
		var idsLength = qualifiedLinkIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var ids = this._getIds(qualifiedLinkIds[idIndex]);
			var selection = this._getSelection(ids);		
			if (selection) {
				this._removeLink(selection, ids[ids.length-1]);			
			}
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},
	
	addRelationship: function(qualifiedRelationshipId) {		
		// summary:
		//		Add a relationship id to the selectio
		var ids = this._getIds(qualifiedRelationshipId);
		var selection = this._getSelection(ids);		
		if (selection) {
			this._addRelationship(selection, ids[ids.length-1]);
		}
	},

	addRelationships: function(qualifiedRelationshipIds) {		
		// summary:
		//		Add multiple relationship ids to the selection
		var idsLength = qualifiedRelationshipIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var ids = this._getIds(qualifiedRelationshipIds[idIndex]);
			var selection = this._getSelection(ids);		
			if (selection) {
				this._addRelationship(selection, ids[ids.length-1]);
			}
		}
	},

	removeRelationship: function(qualifiedRelationshipId) {		
		// summary:
		//		Remove a relationship id from the selection
		var ids = this._getIds(qualifiedRelationshipId);
		var selection = this._getSelection(ids);		
		if (selection) {
			this._removeRelationship(selection, ids[ids.length-1]);			
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeRelationships: function(qualifiedRelationshipIds) {		
		// summary:
		//		Remove multiple relationship ids from the selection
		var idsLength = qualifiedRelationshipIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var ids = this._getIds(qualifiedRelationshipIds[idIndex]);
			var selection = this._getSelection(ids);		
			if (selection) {
				this._removeRelatinship(selection, ids[ids.length-1]);			
			}
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	addChildDataset: function(qualifiedDatasetId) {		
		// summary:
		//		Add a child dataset id to the selection
		var ids = this._getIds(qualifiedDatasetId);
		var selection = this._getSelection(ids, true);		
		if (selection) {
			selection.selected = true;
		}
	},

	addChildDatasets: function(qualifiedDatasetIds) {		
		// summary:
		//		Add multiple child dataset ids to the selection
		var idsLength = qualifiedDatasetIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var ids = this._getIds(qualifiedDatasetIds[idIndex]);
			var selection = this._getSelection(ids, true);		
			if (selection) {
				selection.selected = true;
			}
		}
	},

	removeChildDataset: function(qualifiedDatasetId) {		
		// summary:
		//		Remove a child dataset id from the selection
		var ids = this._getIds(qualifiedDatasetId);
		var selection = this._getSelection(ids, true);		
		if (selection) {
			selection.selected = false;	
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeChildDatasets: function(qualifiedRelationshipIds) {		
		// summary:
		//		Remove multiple child datasest ids from the selection
		var idsLength = qualifiedDatasetIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var ids = this._getIds(qualifiedDatasetIds[idIndex]);
			var selection = this._getSelection(ids, true);		
			if (selection) {
				selection.selected = false;			
			}
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_getIds: function(id) {		
		// summary:
		//		parse qualified id into array of ids
		return id.split(":");
	},

	_getSelection: function(ids, isChildDatasetId) {		
		// summary:
		//		Returns the selection object associated with the requested id array; creates
		//		nested hierarchy as necessary
		var selection = this._selection;
		var idLength = ids.length;
		
		// childDataset
		if (isChildDatasetId) {
			for (var childIndex = 0;  childIndex < idLength;  ++childIndex) {
				var childId = ids[childIndex];
				
				// create child selection			
				if (!selection.childDatasets[childId]) {
					selection.childDatasets[childId] = {selected: false, itemIds: [], linkIds: [], relationshipIds: [], childDatasets: {}};
				}
				
				// work our way down to the correct selection
				selection = selection.childDatasets[childId];
			}
		}
		// item, link, or relationsihps;   search descendants
		else if (idLength > 1) {
			for (var idIndex = 0;  idIndex < idLength - 1;  ++idIndex) {
				var id = ids[idIndex];
			
				// create child selection
				if (!selection.childDatasets[id]) {
					selection.childDatasets[id] = {selected: false, itemIds: [], linkIds: [], relationshipIds: [], childDatasets: {}};
				}
				
				// work our way down to the correct selection
				selection = selection.childDatasets[id];
			}			
		}
		
		return selection;	
	},

	_clean: function(selection) {		
		// summary:
		//		Recursively validate selection; removing any empty descendants
		for (var childDatasetId in selection.childDatasets) {
			if (selection.childDatasets.hasOwnProperty(childDatasetId)) {
				var childDataset = selection.childDatasets[childDatasetId];
				var empty = this._clean(childDataset);
				if (empty) {
					delete selection.childDatasets[childDatasetId];
				}
			}
		}
		
		return this._isEmpty(selection);
	},

	_isEmpty: function(selection) {		
		// summary:
		//		Returns true if the selection is empty
		if (selection.selected) {
			return false;
		}
		
		if (selection.itemIds.length > 0) {
			return false;
		}
		
		if (selection.linkIds.length > 0) {
			return false;
		}
		
		if (selection.relationshipIds.length > 0) {
			return false;
		}
		
		for (var childDatasetId in selection.childDatasets) {
			if (selection.childDatasets.hasOwnProperty(childDatasetId)) {
				return false;
			}
		}
		
		return true;
	},

	_addItem: function(selection, itemId) {		
		// summary:
		//		Add item id to the selection object
		selection.itemIds.push(itemId);
	},
	
	_removeItem: function(selection, itemId) {		
		// summary:
		//		Remove item id from the selection object
		var itemIdsLength = selection.itemIds.length;
		for (var itemIdIndex = 0;  itemIdIndex < itemIdsLength;  ++itemIdIndex) {
			if (itemId === selection.itemIds[itemIdIndex]) {
				selection.itemIds.splice(itemIdIndex, 1);
				break;
			}
		}
	},

	_addLink: function(selection, linkId) {		
		// summary:
		//		Add link id to the selection object
		selection.linkIds.push(linkId);
	},

	_removeLink: function(selection, linkId) {		
		// summary:
		//		Remove link id from the selection object
		var linkIdsLength = selection.linkIds.length;
		for (var linkIdIndex = 0;  linkIdIndex < linkIdsLength;  ++linkIdIndex) {
			if (linkId === selection.linkIds[linkIdIndex]) {
				selection.linkIds.splice(linkIdIndex, 1);
				break;
			}
		}
	},

	_addRelationship: function(selection, relationshipId) {		
		// summary:
		//		Add relationship id to the selection object
		selection.relationshipIds.push(relationshipId);
	},

	_removeRelationship: function(selection, relationshipId) {		
		// summary:
		//		Remove relationship id from the selection object
		var relIdsLength = selection.relationshipIds.length;
		for (var relIdIndex = 0;  relIdIndex < relIdsLength;  ++relIdIndex) {
			if (relationshipId === selection.relationshipIds[relIdIndex]) {
				selection.relationshipIds.splice(relIdIndex, 1);
				break;
			}
		}
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Task"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Task"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Task"); 
 
// includes




ibm.tivoli.navmodel.Task._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Task");

////////////////////////////////////
// Task
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Task", null, {
	// private members
	_task: null,
	_dataset: null,
	_childTasks: null,
	_trace: ibm.tivoli.navmodel.Task._trace,
	
	/*=====
	constructor_kwArgs: function(task, dataset) {
		// task: Object
		//		json task object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that created this task
	},
	=====*/
	
	 constructor: function(/*constructor_kwArgs*/ kwArgs) {				
		// summary:
		//		Constructs Task object.
		this._dataset = kwArgs.dataset;
		this._task = kwArgs.task;
		if (!this._task) {
			this._task = {};
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._task[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._task[key] = value;
	},

	getVersion: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},
		
	getUri: function() {
		if (!this._task.uri) {
			this._task.uri = this._dataset.getAttribute("uri") + "/tasks/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._task.uri;
	},
	
	getId: function() {
		return this._task.id;
	},
	
	setId: function(id) {
		this._task.id = id;
	},
	
	getLabel: function() {
		return this._task.label;
	},
	
	setLabel: function(label) {
		this._task.label = label;
	},
	
	getDescription: function() {
		return this._task.description;
	},
	
	setDescription: function(description) {
		this._task.description = description;
	},
	
	getImage: function() {
		return this._task.image;
	},
	
	setImage: function(image) {
		this._task.image = image;
	},

	getCategory: function() {		
		// summary:
		//		Returns the task category (i.e. where the task is visualized)
		//		
		//		values are: menu, menu:navigation, toolbar, toolbar:button, toolbar:menu,
		//					breadcrumb, breadcrumb:navigation, and property
		return this._task.category;
	},
	
	setCategory: function(category) {
		this._task.category = category;
	},

	isServer: function() {		
		// summary:
		//		Indicates that task should be sent back to the server for execution
		return (this._task.server !== undefined ? this._task.server : false);
	},
	
	setServer: function(isServer) {
		this._task.server = isServer;
	},
	 
	getType: function() {		
		// summary:
		//		Returns the task type
		//		
		//		values are: none, url, page, portlet, clientevent, javascript, native,
		//					dialog, noninteractive, cascade, and separator
		return this._task.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the task type
		//		
		//		values are: none, url, page, portlet, clientevent, javascript, native,
		//					dialog, noninteractive, cascade, and separator
		this._task.type = type;
	},
	
	isNormal: function() {
		return (!this.isNavigation() && !this.isCascade() && !this.isSeparator());
	},

	isNavigation: function() {
		return (this._task.type === "navigation");
	},
	
	isPageLaunch: function() {
		return (this._task.type === "page");
	},
	
	isPortletLaunch: function() {
		return (this._task.type === "portlet");
	},
	
	isUrlLaunch: function() {
		return (this._task.type === "url");
	},
	
	isJavascript: function() {
		return (this._task.type === "javascript");
	},
	
	isClientEvent: function() {
		return (this._task.type === "clientevent");
	},
	
	isCascade: function() {
		return (this._task.type === "cascade" && this._task.subTasks);
	},
	
	isSeparator: function() {
		return (this._task.type === "separator");
	},
	
	isResolved: function() {
		return (this._task.resolved !== undefined ? this._task.resolved : true);
	},
	
	setResolved: function(isResolved) {
		this._task.resolved = isResolved;
	},
	
	isDefault: function() {
		return (this._task["default"] !== undefined ? this._task["default"] : false);
	},
	
	setDefault: function(isDefault) {
		this._task["default"] = isDefault;
	},
	
	isEnabled: function() {
		return (this._task.enabled !== undefined ? this._task.enabled : true);
	},
	
	setEnabled: function(isEnabled) {
		this._task.enabled = isEnabled;
	},
	
	isDisabled: function() {
		return (this._task.enabled !== undefined ? !this._task.enabled : false);
	},
	
	setDisabled: function(isDisabled) {
		this._task.enabled = !isDisabled;
	},
	
	/*=====
	resolve_kwArgs: function(selection, itemIds, linkIds, itemIndexes) {
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds: String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
	},
	=====*/

	resolve: function(/*resolve_kwArgs*/ kwArgs) {		
		// summary:
		//		Re-fetch the task to resolve all client side launch parameters
		var args = {
			selection: null,
			itemIds: null,
			linkIds: null,
			itemIndexes: null
		};
		dojo.mixin(args, kwArgs);
		
		// selection was provided
		var json = "";
		if (args.selection) {
			json = dojo.toJson(args.selection.getAttributes(), false);
		}
			
		return dojo.xhrPost ({
			url: this._getParamURL(args),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "task"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "taskContext"),
				'Content-Length': json.length,
				'X-Method-Override': 'GET'
			},
			handleAs: "json",
			postData: json,
			sync: true,			
			load: dojo.hitch(this, function(result, ioArgs) {
				this._task = result;
			}),
			error: dojo.hitch(function(error, ioArgs) {
				ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "resolveLaunchParms", this._trace, this._version());					              
			})
		});
	},
	
	getLaunchUrl: function() {
		return this._task.url;
	},
	
	setLaunchUrl: function(url) {
		this._task.url = url;
	},
	
	getLaunchBrowserWindowId: function() {
		return this._task.browserWindowId;
	},
	
	setLaunchBrowserWindowId: function(windowId) {
		this._task.browserWindowId = windowId;
	},
	
	getLaunchPageId: function() {
		return this._task.pageId;
	},
	
	setLaunchPageId: function(pageId) {
		this._task.pageId = pageId;
	},
	
	getLaunchPortletName: function() {
		return this._task.portletName;
	},
	
	setLaunchPortletName: function(portletName) {
		this._task.portletName = portletName;
	},
	
	getLaunchPortletReuse: function() {
		return this._task.portletReuse;
	},
	
	setLaunchPortletReuse: function(reuse) {
		this._task.portletReuse = reuse;
	},
	
	getLaunchPortletAppUID: function() {
		return this._task.portletAppUID;
	},
	
	setLaunchPortletAppUID: function(appUID) {
		this._task.portletAppUID = appUID;
	},
	
	getLaunchEvent: function() {
		return this._task.event;
	},
	
	setLaunchEvent: function(event) {
		this._task.event = event;
	},
	
	getLaunchJavascript: function() {
		return this._task.javascript;
	},
	
	setLaunchJavascript: function(javascript) {
		this._task.javascript = javascript;
	},
	
	getLaunchApplicationParms: function() {
		return this._task.applicationParms;
	},
	
	setLaunchApplicationParms: function(applicationParms) {
		this._task.applicationParms = applicationParms;
	},
	
	/*
	 * Returns any child tasks if this task is a cascade
	 */
	getChildTasks: function() {
		if (this.isCascade()) {
			if (!this._childTasks) {
				this._childTasks = [];
				var subTaskLength = this._task.subTasks.length;
				for (var index = 0;  index < subTaskLength;  ++index) {
					this._childTasks[index] = new ibm.tivoli.navmodel.Task({task: this._task.subTasks[index], dataset: this._dataset});
				}
			}
			
			return this._childTasks;
		}
		
		return null;
	},
	
	setChildTasks: function(childTasks) {
		if (this.isCascade()) {
			this._childTasks = childTasks;
		}
	},
	
	/*=====
	result_onComplete: function(result, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	execute_kwArgs: function(onComplete, onError, selection, sync) {
		// onComplete: result_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for successful fetch
		// selection: Object:
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// selectionParms: Object:
		//		extra parameters that can be included in the selection when PUT
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	execute: function(/*execute_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously execute remote task
		
		// this is only for executing regular tasks
		if (this.isNavigation()) {
			alert('execute is only for non-navigation tasks');
			return;
		}
		
		var args = {                
			onComplete: function(result, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},         
			selection: null,
			selectionParms: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// get selection and add additional parameters	
		var selection = args.selection.getAttributes();
		if (args.selectionParms) {
			selection.parameters = args.selectionParms;
		}
			
		var json = dojo.toJson(selection, false);
			
		return dojo.xhrPost ({
			url: this._getParamURL({}),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "taskContext"),
				'Content-Length': json.length				
			},
			handleAs: "json",
			postData: json,
			sync: args.sync,		
			load: dojo.hitch(this, function(result, ioArgs) {
				if (this._trace.isFine()) {
					this._trace.fine("Execute for taskId=" + this.getId() + " returned successfully", "execute");
				}
							
				args.onComplete(result, args);				
			}),
			error: dojo.hitch(function(error, ioArgs) {
				args.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "execute", this._trace, this._version()), ioArgs);	               
			})
		});		
	},
	
	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, selection, sync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for successful fetch
		// selection: Object:
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	navigate: function(/*navigate_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously execute navigation task (i.e. returns new Dataset)
		
		// this is only for executing regular tasks
		if (!this.isNavigation()) {
			alert('navigate is only for navigation tasks');
			return;
		}
		
		var args = {                
			onComplete: function(dataset, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			taskId: this.getId(),
			taskCategory: this.getCategory(),                
			selection: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// call the dataset to do the navigation
		this._dataset.navigate(args);
	},	
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_version: function() {
		return this._dataset.getVersion();
	},

	_getParamURL: function(kwArgs) {		
		// summary:
		//		Build url necessary to do reference task and also includes
		//		all of the "param_" and "version" arguments
		var url = this.getUri();
		var questionMark = false;
		
		// add "param_" parms
		var parameters = this._dataset.getArg("parameters");
		for (var parm in parameters) {
			if (parameters.hasOwnProperty(parm)) {
				if (!questionMark) {
					questionMark = true;
					url += "?";
				}
				else {
					url += "&";
				}
				url += ("param_" + parm + "=" + parameters[parm]);				
			}
		}
		
		// IE will sometimes return http requests from cache without unique parms
		var now = new Date();
		if (!questionMark) {
			questionMark = true;
			url += "?d=" + now.getTime() + "&r=" + Math.random();
		}
		else {
			url += "&d=" + now.getTime() + "&r=" + Math.random();
		}
		
		// add "category"
		if(!questionMark) {
			questionMark = true;
			url += "?category=" + this.getCategory();
		}
		else {
			url += "&category=" + this.getCategory();
		}
		
		// indicate selected items
		if (kwArgs.itemIds) {
			if (!questionMark) {
				questionMark = true;
				url += "?itemIds=" + kwArgs.itemIds;
			}
			else {
				url += "&itemIds=" + kwArgs.itemIds;
			}			
		}
		
		// indicate selected links
		if (kwArgs.linkIds) {
			if (!questionMark) {
				questionMark = true;
				url += "?linkIds=" + kwArgs.linkInds;
			}
			else {
				url += "&linkIds=" + kwArgs.linkIds;
			}			
		}
		
		// indicate seleted item ranges
		if (kwArgs.itemIndexes) {
			if (!questionMark) {
				questionMark = true;
				url += "?itemIndexes=" + kwArgs.itemIndexes;
			}
			else {
				url += "&itemIndexes=" + kwArgs.itemIndexes;
			}			
		}
		
		return url;	
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Column"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Column"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Column"); 

// includes


////////////////////////////////////
// Column
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Column", null, {
	// private members
	_col: null,
	_objectType: null,
	_dataset: null,
	_condition: null,
	
	/*=====
	constructor_kwArgs: function(column, objectType, dataset) {
		// column: Object
		//		json column object
		// objectType: String
		//		object type: "item", "link", or "relationship"
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this column
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Column object.		
		this._col = kwArgs.column;
		if (!this._col) {
			this._col = {};
		}
		
		this._objectType = kwArgs.objectType;
		this._dataset = kwArgs.dataset;
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._col[key];
	},

	setAttribute: function(key, value) {
		// summary:
		//		Set general attribute for requested key
		this._col[key] = value;
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the column
		return dojo.clone(this._col);			
	},

	getDataset: function() {
		// summary:
		//		Returns the dataset that owns the column
		return this._dataset;
	},
	
	getId: function() {
		// summary:
		//		Returns column id
		return this._col.id;
	},

	setId: function(id) {
		// summary:
		//		Set column id
		this._col.id = id;
	},

	getLabel: function() {
		// summary:
		//		Returns globalized column header label
		if (this._col.label) {
			// string
			if (typeof this._col.label === "string") {
				return this._col.label;
			}
			// object with key/value info
			else {
				return this._col.label.value;
			}
		}
		
		return null;		
	},

	setLabel: function(label) {
		// summary:
		//		Set globalized column header label
		this._col.label = label;		
	},	
	
	getDescription: function() {
		// summary:
		//		Returns globalized description
		if (this._col.description) {
			// string
			if (typeof this._col.description === "string") {
				return this._col.description;
			}
			// object with key/value info
			else {
				return this._col.description.value;
			}
		}
		
		return null;
	},

	setDescription: function(description) {
		// summary:
		//		Set globalized description
		this._col.description = description;		
	},	
	
	getValueType: function() {
		// summary:
		//		Returns the type associated with the column
		//		
		//		|	values:	"itemName", "itemType", "linkName", "linkType", "relationshipName", "relationshipType",
		//		|			string", "string[]",
		//		|			"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|			"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",
		//		|			"boolean", "boolean[]"
		//		|			"enum",
		//		|			"status"
		//		|			"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|			"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|			"float", "double", "percentage", "float[]", "double[]"
		return this._col.valueType;
	},

	setValueType: function(valueType) {
		// summary:
		//		Set the type associated with the column	
		this._col.valueType = valueType;
	},
	
	getWidth: function() {
		// summary:
		//		Returns column width string as: pixels, percentage, or em value
		return this._col.width;
	},

	setWidth: function(width) {
		// summary:
		//		Set the column width string as: pixels, percentage, or em value
		// width: String
		//		Column width value
		this._col.width = width;
	},
	
	isResizeable: function() {
		// summary:
		//		Returns true if column is resizeable by the user
		return (this._col.resizeable !== undefined ? this._col.resizeable : true);
	},
	
	setResizeable: function(resizeable) {
		// summary:
		//		Set if the column is to be resizeable by the user
		this._col.resizeable = resizeable;
	},
	
	isLineWrap: function() {
		// summary:
		//		Returns true if column allows line wrap to multiple lines
		return (this._col.lineWrap !== undefined ? this._col.lineWrap : false);
	},
	
	setLineWrap: function(lineWrap) {
		// summary:
		//		Set if the column allows line wrap to multiple lines
		this._col.lineWrap = lineWrap;
	},
	
	isLocked: function() {
		// summary:
		//		Returns true if column locked in non-scrollable region
		return (this._col.locked !== undefined ? this._col.locked : false);
	},
	
	setLocked: function(Locked) {
		// summary:
		//		Set whether the column is locked in non-scrollable region
		this._col.locked = locked;
	},
		
	isHidden: function() {
		// summary:
		//		Returns true if the column is to be hidden from the user
		return (this._col.hidden !== undefined ? this._col.hidden : false);
	},
	
	setHidden: function(hidden) {
		// summary:
		//		Set if the column is to be hidden from the user
		this._col.hidden = hidden;
	},
	
	getImage: function() {
		// summary:
		//		Returns optional image for the column
		return this._col.image;
	},
	
	setImage: function(image) {
		// summary:
		//		Set optional image for the column
		this._col.image = image;
	},
	
	getSortOrder: function() {
		// summary:
		//		Returns sort order priority (value is 0 based and should be -1 if not sorted)
		return (this._col.sortOrder !== undefined ? this._col.sortOrder : -1);
	},

	setSortOrder: function(sortOrder) {
		// summary:
		//		Set sort order priority (value is 0 based and should be -1 if not sorted)
		this._col.sortOrder = sortOrder;
	},

	isSortable: function() {
		// summary:
		//		Returns true if the column is sortable
		return (this._col.sortable !== undefined ? this._col.sortable : true);
	},

	setSortable: function(sortable) {
		// summary:
		//		Set if the column is sortable
		this._col.sortable = sortable;
	},

	isSortAscending: function() {
		// summary:
		//		Returns true if the sort is ascending or descending
		return (this._col.sortAscending !== undefined ? this._col.sortAscending : true);
	},

	setSortAscending: function(ascending) {
		// summary:
		//		Set if the sort is ascending or descending
		this._col.sortAscending = ascending;
	},

	isSearchable: function() {
		// summary:
		//		Returns true if the column is searchable
		return (this._col.searchable !== undefined ? this._col.searchable : true);
	},

	setSearchable: function(searchable) {
		// summary:
		//		Set if the column is searchable
		this._col.searchable = searchable;
	},

	isFilterable: function() {
		// summary:
		//		Returns true if the column is filterable
		return (this._col.filterable !== undefined ? this._col.filterable : true);
	},
	
	setFilterable: function(filterable) {
		// summary:
		//		Set if the column is filterable
		this._col.filterable = filterable;
	},
	
	isFiltered: function() {
		// summary:
		//		Returns true if the column is currently filtered
		return (this._col.filtered !== undefined ? this._col.filtered : false);
	},

	setFiltered: function(filtered) {
		// summary:
		//		Set if the column is currently filtered
		this._col.filtered = filtered;
	},
	
	getFilter: function() {
		// summary:
		//		Returns filter object for the column		
		return this._col.filter;
	},
	
	getUnsupportedFilterOperators: function() {
		// summary:
		//		Returns String[] of operators that are not supported for this
		//		column or null if all operators are supported.
		return (this._col.filter ? this._col.filter.unsupportedOperators : null);		
	},
	
	hasPossibleValues: function() {
		// summary:
		//		Returns true if the column supports a list of possible values		
		var filter = this._col.filter;
		return ((filter && (filter.possibleValues || filter.possibleValuesUri)) ? true : false);
	},
	
	hasInlinedPossibleValues: function() {
		// summary:
		//		Returns true if the column supports possible values and
		//		all of those values have been inlined in the json
		var filter = this._col.filter;
		return (filter && filter.possibleValues ? true : false);
	},
	
	hasUrlAddressablePossibleValues: function() {
		// summary:
		//		Returns true if the column supports possible values and
		//		the values are addressable via a url
		var filter = this._col.filter;
		return (filter && filter.possibleValuesUri ? true : false);
	},
	
	getPossibleValues: function(kwArgs) {
		// summary:
		//		Retrieves a list of possible values for columns with a valueType of
		//		enum, map, itemType, linkType, or relationshipType
		var args = {
			onComplete: function(values, totalCount, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
	
		var filter = this._col.filter;
		
		// possible values were inlined; just return them
		if (filter && filter.possibleValues) {
			if (args.start === 0 && args.count === null) {
				args.onComplete(filter.possibleValues, filter.possibleValues.length, args);
			}
			else {
				var values = [];
				var length = filter.possibleValues.length;
				var count = args.count ? args.count : length;
				for (var index = args.start;  index < length && count > 0;  ++index, --count) {
					values.push(filter.possibleValues[index]);
				}
				args.onComplete(values, length, args);
			}
		}
		// possible values are available via uri
		else if (filter && filter.possibleValuesUri) {
			var content = ibm.tivoli.navmodel.Utils.getContent();
			ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
			if (args.start) {
				content.start = args.start;
			}
			if (args.count) {
				content.count = args.count;
			}
			if (this._objectType) {
				content.type = this._objectType;
			}
			
			dojo.xhrGet({
				url: this._dataset.getArg("baseUrl") + filter.possibleValuesUri,
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "columnValueList")
				},
				handleAs: "json",
				sync: args.sync,
				content: content,		
				load: dojo.hitch(this, function(result, ioArgs){
					args.onComplete(result.items, result.totalRows, args);					
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					args.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getPossibleValues", this._trace, this._version()), ioArgs);
				})
			});	
		}
		// doesn't have any
		else {
			args.onComplete(null, 0, args);
		}		
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.EventRouter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.EventRouter"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.EventRouter"); 



ibm.tivoli.navmodel.EventRouter._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.EventRouter");

////////////////////////////////////
// EventRouter
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.EventRouter", null, {
	// private members
	_args: null,
	_initialized: false,
	_globalObj: null,
	_trace: ibm.tivoli.navmodel.EventRouter._trace,
	
	/*=====
	constructor_kwArgs: function(eventDestination, timeout, pollDelay, pollErrorDelay, contextRoot) {
		// eventDestination: String
		//		destination to listen to (i.e. provided by Dataset)
		// timeout: int?
		//		number of seconds to wait for events  (optional)
		// pollDelay: int?
		//		number of ms to wait in between poll (optional)
		// pollErrorDelay: int?
		//		number of ms to wait after an error before next poll  (optional)
		// contextRoot: String?
		//		context root for requests to event servlet  (optional)		
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs EventRouter object.
		this._args = kwArgs;
		
		// determine the document that we want to anchor the javascript under
		try {
			this._globalObj = ISOLATION_INIFRAME ? parent : window;
		}
		catch(e) {
			this._globalObj = parent;
		}		
		
		// load javascript if we haven't already				
		this._loadEventRouterJavascript();	
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	/*=====
	addListener_event: function(event){},
	=====*/

	addListener: function(/*String*/ eventId, /*addListener_event*/ eventCallback) {
		// summary:
		//		Adds a listener for the requested eventId
		// eventId:
		//		id of events (i.e. provided by Dataset)
		// eventCallback:
		//		callback function to invoke when events are received
		if (this._initialized) {
			if (this._trace.isFinest()) {
				this._trace.finest("adding listener for eventId=" + eventId, "addListener");
			}
			
			this._getGlobalEventRouter().addListener(this, eventId, eventCallback);
		}
		else {
			if (this._trace.isFinest()) {
				this._trace.finest("Waiting on event_router.js to finish loading", "addListener");
			}
					
			setTimeout(dojo.hitch(this, function() {this.addListener(eventId, eventCallback);}), 50);
		}
	},

	removeListener: function(/*String*/ eventId) {
		// summary:
		//		Removes a listener for the requested eventId
		// eventId:
		//		id of events (i.e. provided by Dataset)
		if (this._initialized) {
			return this._getGlobalEventRouter().removeListener(this, eventId);
		}
		
		return 0;
	},

	ping: function() {
		// summary:
		//		Check if the event router is still alive
		console.assert(true);
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_init: function() {	
		// summary:
		//		Initializes the event router
		var globalEventRouter = this._getGlobalEventRouter();
		
		if (globalEventRouter) {
			if (!globalEventRouter.isInitialized()) {
				if (this._trace.isFinest()) {
					this._trace.finest("initializing global event router", "_init");
				}
			
				globalEventRouter.init(this._args);
			}
			
			this._initialized = true;
		}
		else {		
			if (this._trace.isFinest()) {
				this._trace.finest("Waiting on event_router.js to finish loading", "_init");
			}
					
			setTimeout(dojo.hitch(this, function() {this._init();}), 50);
		}	
	},

	_loadEventRouterJavascript: function() {
		// summary:
		//		Load event_router.js using script tag
		var id = "ibm_tivoli_navmodel_event_router";
		var script = this._globalObj.document.getElementById(id);
		if (!script && !this._getGlobalEventRouter()) {
			if (this._trace.isFinest()) {
				this._trace.finest("Inserting event_router.js", "_loadEventRouterJavascript");
			}
			
			// so we can use inside onreadystatechange
			var that = this;
			
			// create the script tag
			script = this._globalObj.document.createElement('script');
			script.id = id;
			script.type = "text/javascript";
			script.src = dojo.moduleUrl("ibm.tivoli.navmodel", "event_router.js").uri;
			script.onload = dojo.hitch(this, this._init);
			script.onreadystatechange = function () { 
				if (this.readyState == 'loaded' || this.readyState == 'complete') {
					that._init();				
				} 
			};
			
			// insert the script tag		
			this._globalObj.document.body.appendChild(script);			
		}
		// script has loaded or in the process of loading
		else {	
			if (this._trace.isFinest()) {
				this._trace.finest("event_router.js is already loaded", "_loadEventRouterJavascript");
			}
			this._init();				
		}		
	},

	_getGlobalEventRouter: function() {
		// summary:
		//		Returns instance of global event router; depending on how javascript was loaded
		var eventRouter = this._globalObj.ibm_tivoli_navmodel_EventRouter;		
		if (!eventRouter) {
			try { 
				eventRouter = ibm_tivoli_navmodel_EventRouter;
			}
			catch (e) {
				// event_router.js has not been loaded
			}
		}
		
		return eventRouter;
	}
});


ibm.tivoli.navmodel.EventRouter.getPageId = function() {	
	// determine the document that we want to anchor the javascript under
	var globalObj = null;
	try {
		globalObj = ISOLATION_INIFRAME ? parent : window;		
	}
	catch(e) {
		globalObj = parent;
	}
	
	if (!globalObj.ibm_tivoli_navmodel_EventRouter_pageId) {
		var now = new Date();
		globalObj.ibm_tivoli_navmodel_EventRouter_pageId = now.getTime().toString();
	}
	
	return globalObj.ibm_tivoli_navmodel_EventRouter_pageId;
};

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Event"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Event"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Event"); 

// includes


////////////////////////////////////
// Event
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Event", null, {
	// private members
	_dataset: null,
	_event: null,
	_items: null,
	_links: null,
	_relationships: null,
	_props: null,
	_childDatasetItems: null,
	_eventRouterStatus: null,
	
	/*=====
	constructor_kwArgs: function(dataset, event) {
		// dataset: Object
		//		the dataset that the event is for
		// event: Object
		//		raw event object
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Event object.
		this._dataset = kwArgs.dataset;		
		this._event = kwArgs.event;
		this._items = [];
		this._links = [];
		this._relationships = [];
		this._props = [];
		this._childDatasetItems = [];
		
		// "eventRouterStatus" events originate from the EventRouter;
		// fill in this info to be consistent with other events
		if (this._event.id == "eventRouter") {
			this._event.providerId = this._dataset.getProviderId();
			this._event.datasourceId = this._dataset.getDatasourceId();
			this._event.datasetId = this._dataset.getId();
			this._event.version = this._dataset.getVersion();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getDataset: function() {
		return this._dataset;
	},
	
	getVersion: function() {
		return this._event.version;
	},
	
	getProviderId: function() {
		return this._event.providerId;
	},
	
	getDatasourceId: function() {
		return this._event.datasourceId;
	},
	
	getDatasetId: function() {
		return this._event.datasetId;
	},
	
	getId: function() {
		return this._event.id;
	},
	
	getType: function() {
		return this._event.type;
	},
	
	getItems: function() {
		return this._items;
	},
	
	getLinks: function() {
		return this._links;
	},
	
	getRelationships: function() {
		return this._relationships;
	},
	
	getProperties: function() {
		return this._props;
	},
	
	getLinkChanged: function() {
		return this._links[0];
	},
	
	getLinkChangedReason: function() {
		return this._event.reason;
	},
	
	getLinkChangedRelationships: function() {
		return this._relationships;
	},
	
	getChildDatasetItems: function() {
		return this._childDatasetItems;
	},
	
	getNewLabel: function() {
		return this._event.label;
	},
	
	getException: function() {
		return new ibm.tivoli.navmodel.Exception({exception: this._event.exception, version: this._dataset.getVersion()});
	},
	
	getEventRouterStatus: function() {
		return this._event.eventRouterStatus;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_addItem: function(item, props) {
		this._items.push(item);
		this._props.push(props);
	},
	
	_addLink: function(link, props) {
		this._links.push(link);
		this._props.push(props);
	},
	
	_addRelationship: function(relationship, props) {
		this._relationships.push(relationship);
		this._props.push(props);
	},
	
	_addChildDatasetItem: function(childDatasetItem) {
		this._childDatasetItems.push(childDatasetItem);
	},
	
	_getEvent: function() {
		return this._event;
	},
	
	_getEventItems: function() {
		return this._event.items;
	},
	
	_getEventLinks: function() {
		return this._event.links;
	},
	
	_getEventLink: function() {
		return this._event.link;
	},
	
	_getEventRelationships: function() {
		return this._event.relationships;
	},
	
	_getEventChildDatasets: function() {
		return this._event.childDatasets;
	},
	
	_getEventNewLabel: function() {
		return this._event.label;
	},
	
	_getEventStatuses: function() {
		return this._event.status;
	},
	
	_getEventDatasetItemCount: function() {
		return this._event.datasetItemCount;
	},
	
	_getEventDatasetLinkCount: function() {
		return this._event.datasetLinkCount;
	},
	
	_getEventDatasetRelationshipCount: function() {
		return this._event.datasetRelationshipCount;
	}
});


////////////////////////////////////////////////////////////////////////
// ibm.tivoli.navmodel.ClientEvent
//
// For creating client initiated ibm.tivoli.navmodel.Event objects
////////////////////////////////////////////////////////////////////////

ibm.tivoli.navmodel.ClientEvent = {
	getChangedEvent: function(dataset) {
		// summary:
		//		Creates client-side event for type "changed"
		var eventObj = this._eventObj(dataset, "changed");		
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemsAddedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemsAdded");
		eventObj.datasetItemCount = dataset.getItemCount() + items.length;		
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemsRemovedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemsRemoved");
		eventObj.datasetItemCount = dataset.getItemCount() - items.length;		
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemStatusChangedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemStatusChanged");
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemPropertiesChangedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemPropertiesChanged");
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinksAddedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linksAdded");
		eventObj.datasetLinkCount = dataset.getLinkCount() + links.length;		
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinksRemovedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linksRemoved");
		eventObj.datasetLinkCount = dataset.getLinkCount() - links.length;		
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinkStatusChangedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linkStatusChanged");
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinkPropertiesChangedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linkPropertiesChanged");
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinkChangedEvent: function(dataset, link, relationships, relationshipAdded) {
		var eventObj = this._eventObj(dataset, "linkPropertiesChanged");
		eventObj.reason = (relationshipAdded ? "relationshipAdded" : "relationshipRemoved");
		eventObj.link = link.getAttributes();
		this._mixinRelationships(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLabelChangedEvent: function(dataset, newLabel) {
		var eventObj = this._eventObj(dataset, "labelChanged");
		eventObj.label = label;
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getChildDatasetsAddedEvent: function(dataset, childDatasetItems) {
		var eventObj = this._eventObj(dataset, "childDatasetsAdded");
		this._mixinChildDatasetItems(eventObj, childDatasetItems);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getChildDatasetsRemovedEvent: function(dataset, childDatasetItems) {
		var eventObj = this._eventObj(dataset, "childDatasetsRemoved");
		this._mixinChildDatasetItems(eventObj, childDatasetItems);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	_eventObj: function(dataset, type) {
		return {
			"id": dataset.getEventId(),
			"type": type,
			"providerId": dataset.getProviderId(),
			"datasourceId": dataset.getDatasourceId(),
			"datasetId": dataset.getId(),
			"version": dataset.getVersion()
		};		
	},
	
	_mixinItems: function(eventObj, items) {
		var itemObjs = [];
		var itemLength = items.length;
		for (var itemIndex = 0;  itemIndex < itemLength;  ++itemIndex) {
			itemObjs[itemIndex] = items[itemIndex].getAttributes();
		}
		eventObj.items = itemObjs;
	},
	
	_mixinLinks: function(eventObj, links) {
		var linkObjs = [];
		var linkLength = links.length;
		for (var linkIndex = 0;  linkIndex < linkLength;  ++linkIndex) {
			linkObjs[linkIndex] = links[linkIndex].getAttributes();
		}
		eventObj.links = linkObjs;
	},
	
	_mixinRelationships: function(eventObj, relationships) {
		var relationshipObjs = [];
		var relationshipLength = relationships.length;
		for (var relationshipIndex = 0;  relationshipIndex < relationshipLength;  ++relationshipIndex) {
			relationshipObjs[relationshipIndex] = relationships[relationshipIndex].getAttributes();
		}
		eventObj.relationships = relationshipObjs;
	},
	
	_mixinChildDatasetItems: function(eventObj, childDatasetItems) {
		var childDatasetObjs = [];
		var childLength = childDatasetItems.length;
		for (var childIndex = 0;  childIndex < childLength;  ++childIndex) {
			childDatasetObjs[childIndex] = childDatasetItems[childIndex].getAttributes();
		}
		eventObj.childDatasets = childDatasetObjs;
	}
};

}

if(!dojo._hasResource["ibm.tivoli.navmodel.WidgetDelta"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.WidgetDelta"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.WidgetDelta"); 
 
// includes


ibm.tivoli.navmodel.WidgetDelta._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.WidgetDelta");

////////////////////////////////////
// WidgetDelta
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.WidgetDelta", null, {
	// private members       
	_widgetDef: null,
	_type: null,
	_delta: null,
	_pending: false,	
	_trace: ibm.tivoli.navmodel.WidgetDelta._trace,

	/*=====
	 constructor_kwArgs: function(widgetDef, type) {
		// widgetDef: ibm.tivoli.navmodel.WidgetDef (or descendent)
		//		widget definition object to manipulate delta
		// type: String
		//		the delta type:  "user" or "shared"		
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		var args = {
			widgetDef: null,
			type: null
		};
		dojo.mixin(args, kwArgs);
		
		// summary:
		//		Constructs WidgetDelta object.
		this._widgetDef = args.widgetDef;
		this._type = (args.type ? args.type : this._widgetDef.getArg("type"));
		
		switch(this._type) {
			case "user":
				this._delta = this._widgetDef.getUserDelta();
				break;
			case "shared":
				this._delta = this._widgetDef.getSharedDelta();
				break;
			default:
				this._trace.error("Unknown delta type=" + this._type, "constructor");
		}
		
		if (this._delta) {
			this._delta = dojo.clone(this._delta);
		}
		else {
			this._delta = {};
		}		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
		
	getType: function() {
		// summary:
		//		Return delta type: either "user" or "shared"
		return this._type;
	},
	
	hasPendingChanges: function() {
		// summary:
		//		Returns true if changes are pending
		return this._pending;
	},
	
	save: function() {
		// summary:
		//		Save current delta changes to the widget registry;
		//		delete the delta if it is empty.
		if (this.hasPendingChanges()) {
			switch(this._type) {
				case "user":
					if (this._hasKeys(this._delta)) {
						this._widgetDef.setUserDelta(this._delta);
					}
					else {
						this._widgetDef.deleteUserDelta();
					}
					break;
				case "shared":			
					if (this._hasKeys(this._delta)) {
						this._widgetDef.setSharedDelta(this._delta);
					}
					else {
						this._widgetDef.deleteSharedDeleta();
					}
			}
			this._pending = false;
		}
	},
	
	clear: function() {
		// summary:
		//		Clear the entire delta
		this._delta = {};
		this._pending = true;
	},
	
	/*=====
	 clearSection_kwArgs: function(sectionKey) {
	 	//	sectionKey: String
		//		The key that identifies the section		
	},
	=====*/
	
	clearSection: function(/*clearSection_kwArgs*/ kwArgs) {
		if (this._delta[kwArgs.sectionKey]) {
			delete this._delta[kwArgs.sectionKey];			
		}
		
		this._pending = true;
	},
	
	/*=====
	 sectionAttribute_kwArgs: function(sectionKey, attrKey) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	attrKey: String
		//		The attribute in the section
	},
	=====*/
	
	getSectionAttribute: function(/*sectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Get a key/value in a specified section
		//
		//	Example:
		//	{	...
		//		"topology: {
		//			"backgroundColor": "white",
		//			...
		//		},
		//		...
		//	}
		//	Get "backgroundColor" attribute in "topology" section 
		//	var mode = delta.getSectionAttribute({sectionKey: "topology", attrKey: "backgroundColor"});		
		var value = null;
		var section = this._delta[kwArgs.sectionKey];
		
		// section exists in delta
		if (section) {
			value = section[kwArgs.attrKey];						
		}
		
		return value;	
	},
	
	/*=====
	 setSectionAttribute_kwArgs: function(sectionKey, attrKey, attrValue) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	attrKey: String
		//		The attribute in the section
		//	attrValue: any
		//		The value to be set on the attribute
	},
	=====*/
		
	setSectionAttribute: function(/*setSectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Set a key/value in a specified section
		//
		//	Example:
		//	{	...
		//		"topology: {
		//			"backgroundColor": "white",
		//			...
		//		},
		//		...
		//	}
		//	Set "backgroundColor" attribute in "topology" section 
		//	delta.setSectionAttribute({sectionKey: "topology", attrKey: "backgroundColor", attrValue: "gray"});		
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// section exists in delta
		if (section) {
			section[kwArgs.attrKey] = kwArgs.attrValue;			
			rc = true;			
		}
		// section does not exist in delta
		else {
			section = {};
			section[kwArgs.attrKey] = kwArgs.attrValue;
			this._delta[kwArgs.sectionKey] = section;
			rc = true;
		}
		
		this._pending = true;
		return rc;		
	},
	
	removeSectionAttribute: function(/*sectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		 Remove a key from a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"topology: {
		//			"backgroundColor": "white",
		//			...
		//		},
		//		...
		//	}
		//	Remove "backgroundColor" attribute from "topology" section
		//	delta.removeSectionAttribute({sectionKey: "topology", attrKey: "backgroundColor"});				
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// section exists in delta
		if (section) {
			delete section[kwArgs.attrKey];
			rc = true;
			
			// if section is empty; we can remove from delta
			if (!this._hasKeys(section)) {
				delete this._delta[kwArgs.sectionKey];
			}					
		}
			
		this._pending = true;					
		return rc;		
	},
	
	/*=====
	 arraySectionAttribute_kwArgs: function(sectionKey, elementKey, elementValue, attrKey) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	elementKey: String
		//		The key used to identify the array element
		//	elementValue: String
		//		The value used to identify the array element (i.e. compare to elementKey)
		//	attrKey: String
		//		The attribute in the array element			
	},
	=====*/
		
	getArraySectionAttribute: function(/*arraySectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Get a key/value in a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"tables: [
		//			{	"id": "ItemsTable",
		//				"filter": {
		//					...
		//				},
		//				...
		//			},
		//			...
		//		],
		//		...
		//	}
		//	Get "filter" attribute in "tables" section (i.e. the element where "id" is "ItemsTable")		
		//	var filter = delta.getArraySectionAttribute({sectionKey: "tables", elementKey: "id", elementValue: "ItemsTable", attrKey: "filter"});
		
		var value = null;
		var section = this._delta[kwArgs.sectionKey];
		
		// array section exists in delta
		if (section && dojo.isArray(section)) {
			var count = section.length;
			for (var index = 0;  index < count;  ++index) {
				var element = section[index];
				if (element[kwArgs.elementKey] === kwArgs.elementValue) {
					value = element[kwArgs.attrKey];
					break;
				}
			}				
		}
		
		return value;	
	},
		
	/*=====
	 setArraySectionAttribute_kwArgs: function(sectionKey, elementKey, elementValue, attrKey, attrValue) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	elementKey: String
		//		The key used to identify the array element
		//	elementValue: String
		//		The value used to identify the array element (i.e. compare to elementKey)
		//	attrKey: String
		//		The attribute to be modified in the array element
		//	attrValue: any
		//		The value to be set on the array element attribute
	},
	=====*/
	
	setArraySectionAttribute: function(/*setArraySectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Set a key/value in a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"tables: [
		//			{	"id": "ItemsTable",
		//				"filter": {
		//					...
		//				},
		//				...
		//			},
		//			...
		//		],
		//		...
		//	}
		//	Set "filter" attribute in "tables" section (i.e. the element that has "id" = "ItemsTable")		
		//	delta.setArraySectionAttribute({sectionKey: "tables", elementKey: "id", elementValue: "ItemsTable", attrKey: "filter", attrValue: value});
		
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// array section exists in delta
		if (section && dojo.isArray(section)) {
			var count = section.length;
			for (var index = 0;  index < count;  ++index) {
				var element = section[index];
				if (element[kwArgs.elementKey] === kwArgs.elementValue) {
					element[kwArgs.attrKey] = kwArgs.attrValue;
					rc = true;
					break;
				}
			}				
		}
		// array section does not exist in delta
		else {
			var newElement = {};
			newElement[kwArgs.elementKey] = kwArgs.elementValue;
			newElement[kwArgs.attrKey] = kwArgs.attrValue;
			this._delta[kwArgs.sectionKey + "Identifier"] = kwArgs.elementKey;			
			this._delta[kwArgs.sectionKey] = [newElement];
			rc = true;
		}
		
		this._pending = true;					
		return rc;		
	},
	
	removeArraySectionAttribute: function(/*arraySectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		 Remove a key from a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"tables: [
		//			{	"id": "ItemsTable",
		//				"filter": {
		//					...
		//				},
		//				...
		//			},
		//			...
		//		],
		//		...
		//	}
		//	Remove "filter" attribute from "tables" section (i.e. the element that has "id" = "ItemsTable")		
		//	delta.removeArraySectionAttribute({sectionKey: "tables", elementKey: "id", elementValue: "ItemsTable", attrKey: "filter"});
				
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// array section exists in delta
		if (section && dojo.isArray(section)) {
			var count = section.length;
			for (var index = 0;  index < count;  ++index) {
				var element = section[index];
				if (element[kwArgs.elementKey] === kwArgs.elementValue) {					
					delete element[kwArgs.attrKey];
					rc = true;
					
					// if the only key left in the array element is the
					// elementKey; then we can remove this element
					var keys = this._getKeys(element);
					if (keys.length === 1 && keys[0] === kwArgs.elementKey) {
						section.splice(index,1);
					}
					
					// if array is empty; we can remove from the delta
					if (section.length === 0) {
						delete this._delta[kwArgs.sectionKey];
						delete this._delta[kwArgs.sectionKey + "Identifier"];
					}					
					break;
				}
			}				
		}
				
		this._pending = true;						
		return rc;		
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	_hasKeys: function(obj) {
		for (var key in obj) {
			if (obj.hasOwnProperty(key)) {
				return true;
			}
		}
		
		return false;
	},
	
	_getKeys: function(obj) {
		var keys = [];		
		for (var key in obj) {
			if (obj.hasOwnProperty(key)) {
				keys.push(key);
			}
		}
		return keys;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.AbstractWidgetDef"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.AbstractWidgetDef"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.AbstractWidgetDef"); 
 
// includes





ibm.tivoli.navmodel.AbstractWidgetDef._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.AbstractWidgetDef");

////////////////////////////////////
// AbstractWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.AbstractWidgetDef", null, {
	// private members       
	_args: null,
	_parentWidgetDef: null,	
	_widgetDef: null,
	_userDelta: null,
	_sharedDelta: null,
	_destroyed: false,
	_trace: ibm.tivoli.navmodel.AbstractWidgetDef._trace,

	/*=====
	 constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, profileID, parentWidgetDef, registryLocation, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// profileID: String
		//		optional unique id to associate with reading and writing the
		//		"user" and "shared" deltas to the widget definition.  If not
		//		supplied; a uniqueId is created consisting of all the related
		//		information (i.e. providerType, datasourceType, widgetType, etc)
		// parentWidgetDef: ibm.tivoli.navmodel.AbstractWidgetDef  (or descendent)
		//		optional parent widget definition
		// registryLocation: String
		//		Indicates if "remote" provider definition is in local registry or is
		//		located at the remote rest service.  
		//		values are: "local" or "remote"  (Default for TIP is "local")
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs AbstractWidgetDef object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		this._args = {
			baseUrl: null,
			provider: null,
			datasource: null,
			widget: null,
			definition: null,
			type: "user",
			profileID: null,
			parentWidgetDef: null,
			// TIP policy is to have remote provider widget definitions in local database
			registryLocation: (ibm.tivoli.navmodel.Utils.isTIP() ? "local" : null),
			version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
		};
		dojo.mixin(this._args, kwArgs);
		
		// convert datasource of "" to null
		if (this._args.datasource && this._args.datasource.length === 0) {
			this._args.datasource = null;
		}
		
		this._parentWidgetDef = this._args.parentWidgetDef;
		this._fetchWidgetDef();
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	validateDefaults: function() {
		// summary:
		//		This method is called after the widget def has been read from the server.
		//		Sub-class can override this method and validate/set any defaults that were
		//		not present in the widget def contents
	},
	
	getParentWidgetDef: function() {
		// summary:
		//		Returns parent widget definition 
		return this._parentWidgetDef;
	},
	
	setParentWidgetDef: function(parent) {
		// summary:
		//		Set parent widget definition; will force a
		//		re-fetch of the widget definition from server if
		//		parent has a profileID set and this definition does not.
		this._parentWidgetDef = parent;
		
		// if we do NOT have a profileID; and the new parent does;
		// then we need to re-fetch with the parents profileID
		if (parent && !this._args.profileID && parent.getProfileID()) {
			this._fetchWidgetDef();
		} 
	},
	
	getProfileID: function() {
		// summary:
		//		Returns delta id value if specified;  will inherit value from
		//		parent widget definition chain if not explicitly set
		var profileID = null;
		
		// profileID explicitly declared		
		if (this._args.profileID) {
			profileID = this._args.profileID;
		}
		// inherit profileID from parent
		else if (this._parentWidgetDef) {
			profileID = this._parentWidgetDef.getProfileID();			
		}
		
		return profileID;
	},
	
	setProfileID: function(/*String*/ profileID) {
		// summary:
		//		Set new profileID for the widget definition;
		//		will force a re-fetch of the widget definition from server
		this._args.profileID = profileID;
		this._fetchWidgetDef();
	},	
			
	getArg: function(key) {
		// summary:
		//		Returns a specific initial argument passed to the widget def
		return this._args[key];
	},
	
	destroy: function() {
		// summary:
		//		Do any necessary clean up 
		this._destroyed = true;
	},

	isDestroyed: function() {
		// summary:
		//		Returns true if the widget def has already been destroyed
		return this._destroyed;
	},

	getDef: function() {
		// summary:
		//		Returns the entire widget definition
		if (this._widgetDef) {
			return this._widgetDef;
		}

		return null;
	},

	getSection: function(section) {
		// summary:
		//		Returns the requested section of the widget definition
		if (this._widgetDef) {
			return this._widgetDef[section];
		}

		return null;
	},
	
	/////////////////////////////
	// meta section data
	/////////////////////////////
	
	getMeta: function() {
		return this._widgetDef.meta;		
	},
	
	getWidgetId: function() {
		return this._widgetDef.meta.id;
	},
	
	getWidgetType: function() {
		return this._widgetDef.meta.type;
	},
	
	getWidgetVersion: function() {
		return this._widgetDef.meta.version;
	},
	
	getWidgetLabel: function() {
		return this._widgetDef.meta.label.value;
	},
	
	getWidgetDescription: function() {
		return this._widgetDef.meta.description.value;
	},
	
	setMeta: function(meta) {
		this._widgetDef.meta = meta;		
	},
	
	setWidgetId: function(id) {
		this._widgetDef.meta.id = id;
	},
	
	setWidgetType: function(type) {
		this._widgetDef.meta.type = type;
	},
	
	setWidgetVersion: function(version) {
		this._widgetDef.meta.version = version;
	},
	
	setWidgetLabel: function(value) {
		this._widgetDef.meta.label.value = value;
	},
	
	setWidgetDescription: function(value) {
		this._widgetDef.meta.description.value = value;
	},

	/////////////////////////////
	// provider section data
	/////////////////////////////
	
	getProvider: function() {
		return this._widgetDef.provider;		
	},

	getProviderType: function() {
		return this._widgetDef.provider.type;
	},
	
	getProviderLabel: function() {
		return this._widgetDef.provider.label.value;
	},
	
	getProviderDescription: function() {
		return this._widgetDef.provider.description.value;
	},
	
	setProvider: function(provider) {
		this._widgetDef.provider = provider;		
	},

	setProviderType: function(type) {
		this._widgetDef.provider.type = type;
	},
	
	setProviderLabel: function(value) {
		this._widgetDef.provider.label.value = value;
	},
	
	setProviderDescription: function(value) {
		this._widgetDef.provider.description.value = value;
	},
	
	/////////////////////////////
	// datasource section data
	/////////////////////////////
	
	getDatasource: function() {
		return this._widgetDef.datasource;		
	},
	
	getDatasourceType: function() {
		if (this._widgetDef.datasource) {
			return this._widgetDef.datasource.type;
		}
		
		return null;
	},
	
	getDatasourceLabel: function() {
		if (this._widgetDef.datasource) {
			return this._widgetDef.datasource.label.value;
		}
		
		return null;
	},
	
	getDatasourceDescription: function() {
		if (this._widgetDef.datasource) {
			return this._widgetDef.datasource.description.value;
		}
		
		return null;
	},
	
	setDatasource: function(datasource) {
		this._widgetDef.datasource = datasource;		
	},
	
	setDataSetDatasourceType: function(type) {
		this._widgetDef.datasets[0].datasourceType = type;
	}, 
	
	setDatasourceType: function(type) {
		this._widgetDef.datasource.type = type;
	},
	
	setDatasourceLabel: function(value) {
		this._widgetDef.datasource.label.value = value;
	},
	
	setDatasourceDescription: function(value) {
		this._widgetDef.datasource.description.value = value;
	},	

	/////////////////////////////
	// details section data
	/////////////////////////////
	
	getDetails: function() {
		return this._widgetDef.details;		
	},
	
	getDetailsAttribute: function(attr) {
		return this._widgetDef.details[attr];
	},
	
	setDetails: function(details) {
		this._widgetDef.details = details;
	},
	
	setDetailsAttribute: function(key, value) {
		if (!this._widgetDef.details) {
			this._widgetDef.details = {};
		}
		
		this._widgetDef.details[key] = value;
	},
		
	/////////////////////////////
	// WidgetDelta
	/////////////////////////////
	getWidgetDelta: function(/*String*/ type) {
		// summary:
		//		Get ibm.tivoli.navmodel.WidgetDelta object
		//	type: String
		//		The delta type: "user" or "shared";  defaults to the 
		//		type passed to widget def constructor if not supplied.		
		if (!type) {
			type = this._args.type;			
		}
		
		if (!type || type === "base") {
			type = "user";
		}
		
		var delta = null;
		switch(type) {
			case "user":
				if (!this._userDelta) {
					this._userDelta = new ibm.tivoli.navmodel.WidgetDelta({widgetDef: this, type: type});
				}
				delta = this._userDelta;
				break;
			case "shared":
				if (!this._sharedDelta) {
					this._sharedDelta = new ibm.tivoli.navmodel.WidgetDelta({widgetDef: this, type: type});
				}
				delta = this._sharedDelta;
		}
				
		return delta;
	},
	
	saveWidgetDelta: function() {
		// summary:
		//		Save pending changes on ibm.tivoli.navmodel.WidgetDelta object(s)
		if (this._userDelta && this._userDelta.hasPendingChanges()) {
			this._userDelta.save();
		}
		
		if (this._sharedDelta && this._sharedDelta.hasPendingChanges()) {
			this._sharedDelta.save();
		}
	},
	
	hasPendingChanges: function() {
		// summary:
		//		Returns true if ibm.tivoli.navmodel.WidgetDelta object(s) have pending changes
		return ((this._userDelta && this._userDelta.hasPendingChanges()) || (this._sharedDelta && this._sharedDelta.hasPendingChanges()));
	},
	
	/////////////////////////////
	// raw user delta
	/////////////////////////////
	hasUserDelta: function() {
		// summary:
		//		indicates if raw user delta json object currently exists
		if (this._widgetDef.user) {
			return true;
		}
		
		return false;
	},
	
	getUserDelta: function() {
		// summary:
		//		Get raw user delta json object		
		return this._widgetDef.user;
	},
	
	setUserDelta: function(userDelta) {
		// summary:
		//		PUT raw user delta json object to the server	
		this._putDelta(userDelta, "user");
		this._fetchWidgetDef();
	},
	
	deleteUserDelta: function() {
		// summary:
		//		DELETE raw user delta json object on server
		this._deleteDelta("user");
		this._fetchWidgetDef();
	},
	
	/////////////////////////////
	// raw shared delta
	/////////////////////////////
	hasSharedDelta: function() {
		// summary:
		//		indicates if raw shared delta json object currently exists
		if (this._widgetDef.shared) {
			return true;
		}
		
		return false;
	},
	
	getSharedDelta: function() {
		// summary:
		//		Get raw shared delta json object		
		return this._widgetDef.shared;
	},
	
	setSharedDelta: function(sharedDelta) {
		// summary:
		//		PUT raw shared delta json object to the server	
		this._putDelta(sharedDelta, "shared");
		this._fetchWidgetDef();
	},
	
	deleteSharedDelta: function() {
		// summary:
		//		DELETE raw shared delta json object on server
		this._deleteDelta("shared");
		this._fetchWidgetDef();
	},
	
	/////////////////////////////
	// abstract methods
	/////////////////////////////
	getSelectionMode: function() {
		return null;
	},
	
	getItemStyles: function() {
		return null;		
	},
	
	getItemStyle: function(type) {
		return null;
	},
	
	getDefaultItemStyle: function() {
		return null;
	},
	
	getLinkStyles: function() {
		return null;		
	},
	
	getLinkStyle: function(type) {
		return null;		
	},

	getDefaultLinkStyle: function() {
		return null;
	},
	
	getItemStatuses: function() {
		return null;
	},
	
	getItemStatus: function(id) {
		return null;
	},
	
	getDefaultItemStatus: function() {
		return null;
	},
	
	getLinkStatuses: function() {
		return null;	
	},
	
	getLinkStatus: function(id) {
		return null;
	},
	
	getDefaultLinkStatus: function() {
		return null;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_version: function() {
		return this._args.version;
	},
	
	_fetchWidgetDef: function() {
		var exception = null;
		var profileID = this.getProfileID();
		var content = this._content();
		content.type = this._args.type;
		if (profileID) {
			content.profileID = profileID;
		}
		
		dojo.xhrGet({
			url: this._getWidgetDefinitionURL(),
			headers: {
				'Accept': "application/json"
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs) {
					this._widgetDef = result;
					this._userDelta = null;
					this._sharedDelta = null;
					this.validateDefaults();		
			}),
			error: dojo.hitch (this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchWidgetDef", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	
	_putDelta: function(/*Object*/ delta, /*String*/ deltaType) {
		// summary:
		//		Save delta information for the widget definition
		// delta:
		//		contains the delta to the definition
		// deltaType:
		//		indicates the delta type:  "user" or "shared"
		var exception = null;		
		var json = dojo.toJson(delta, false);
		
		dojo.xhrPut({
			url: this._getPutURL(deltaType),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
				'Content-Type': 'application/json',
				'Content-Length': json.length
			},
			handleAs: "json",
			putData: json,
			sync: true,
			load: dojo.hitch(this, function(result, ioArgs) {									
			}),
			error: dojo.hitch (this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_putDelta", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	
	_deleteDelta: function(/*String*/ deltaType) {
		// summary:
		//		Delete delta information for the widget definition
		// deltaType:
		//		indicates the delta type:  "user" or "shared"
		var exception = null;
		var profileID = this.getProfileID();
		var content = this._content();
		content.type = deltaType;
		if (profileID) {
			content.profileID = profileID;
		}
		
		dojo.xhrDelete({
			url: this._getWidgetDefinitionURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version())
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: function(result, ioArgs) {									
			},
			error: dojo.hitch (this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_deleteDelta", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	
	_content: function() {
		// summary:
		//		Construct a "content" object for sending an xdr request;
		//		IE will sometimes return http requests from cache without unique parms
		var content = {};
		
		// add date and random
		var now = new Date();		
		content.d = now.getTime();
		content.r = Math.random();
		
		// registryLocation if specified
		if (this._args.registryLocation) {
			content.registryLocation = this._args.registryLocation;
		}
		
		return content;
	},

	_getDatasourceURL: function() {
		// summary:
		//		Returns url for fetching a datasource
		var url = null;
		
		if (this._widgetDef) {
			var providerId = this._args.provider;
			if (this.getProviderType() === "any") {
				providerId = "any";
			}
			
			var datasourceId = this._args.datasource;
			if (this.getDatasourceType() === "any") {
				datasourceId = "any";
			}
			
			url = this._args.baseUrl + "/providers/" + encodeURIComponent(providerId);
			url += (datasourceId ? ("/datasources/" + encodeURIComponent(datasourceId)) : "");
		}
		else {
			url = this._args.baseUrl + "/providers/" + encodeURIComponent(this._args.provider);
			url += (this._args.datasource ? ("/datasources/" + encodeURIComponent(this._args.datasource)) : "");
		}
		
		return url;	
	},

	_getWidgetDefinitionURL: function() {
		// summary:
		//		Returns url for fetching a widget definition
		var url = this._getDatasourceURL();
		
		if (this._widgetDef) {
			url+= "/widgets/"+ this.getWidgetType() + "/definitions/" + this.getWidgetId() + "-" + this.getWidgetVersion();
		}
		else {
			url+= "/widgets/"+ this._args.widget + "/definitions/" + this._args.definition;			
		}
		
		return url;
	},
	
	_getPutURL: function(deltaType) {
		// summary:
		//		Build url necessary to do a put or post operation;
		//		have to add on the "type" and "version" ourself
		//		since the dojo.xhr function doesn't include the content
		var url = this._getWidgetDefinitionURL();
		var questionMark = false;
		
		// delta type
		if (!questionMark) {
			questionMark = true;
			url += "?type=" + deltaType;
		}
		else {
			url += "&type=" + deltaType;
		}
		
		// profileID if specified
		var profileID = this.getProfileID();
		if (profileID) {
			if (!questionMark) {
				questionMark = true;
				url += "?profileID=" + profileID;
			}
			else {
				url += "&profileID=" + profileID;
			}
		}
		
		// registryLocation if specified
		if (this._args.registryLocation) {
			if (!questionMark) {
				questionMark = true;
				url += "?registryLocation=" + this._args.registryLocation;
			}
			else {
				url += "&registryLocation=" + this._args.registryLocation;
			}
		}
						
		// IE will sometimes return http requests from cache without unique parms
		var now = new Date();
		if (!questionMark) {
			questionMark = true;
			url += "?d=" + now.getTime() + "&r=" + Math.random();
		}
		else {
			url += "&d=" + now.getTime() + "&r=" + Math.random();
		}
		
		return url;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.WidgetDef"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.WidgetDef"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.WidgetDef"); 
 
// includes



////////////////////////////////////
// WidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.WidgetDef", ibm.tivoli.navmodel.AbstractWidgetDef, {
	// private members
	_itemStyles: null,
	_itemStatuses: null,
	_linkStyles: null,
	_linkStatuses: null,
	_relationshipStyles: null,
	_relationshipStatuses: null,
	_datasetStatuses: null,
	
	// default styles and status
	_defaultItemStyle: null,
	_defaultItemStatus: null,
	_defaultLinkStyle: null,
	_defaultLinkStatus: null,
	_defaultRelationshipStyle: null,
	_defaultRelationshipStatus: null,
	_defaultDatasetStatus: null,
	
	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs WidgetDef object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		this._itemStyles = {};
		this._itemStatuses = {};
		this._linkStyles = {};
		this._linkStatuses = {};
		this._relationshipStyles = {};
		this._relationshipStatuses = {};
		this._datasetStatuses = {};
		
		this._defaultItemStyle = {
			"scalarImage": "[widget]/resources/common_assets/common_resource_icons/re64_unknownserver1.swf",
           	"scalarImageWidth": 61.011,
           	"scalarImageHeight": 53.596,
           	"smallImage": "[widget]/resources/common_assets/common_resource_icons/re64_unknownserver1_8.png",
           	"smallImageHeight": 16,
         	"smallImageWidth": 16,
         	"largeImage": "[widget]/resources/common_assets/common_resource_icons/re64_unknownserver1_8.png",
           	"largeImageHeight": 32,
         	"largeImageWidth": 32          	
		};
			
		this._defaultItemStatus = {
			"backgroundScalarImage": "[widget]\/resources\/common_assets\/status_icons\/BK_SWF\/unknownBk141x117gradient.swf",
            "backgroundScalarImageHeight": 117,
            "backgroundScalarImageWidth": 141,
            "backgroundScalarImageVisible": true,            
            "scalarImageHeight": 25,
            "scalarImageWidth": 25,
            "scalarImageVisible": true,
            "imageHeight": 16,
            "imageWidth": 16,
            "imageVisible": true
		};
		
		this._defaultLinkStyle = {
			"lineStyle": "solid",
			"lineWidth": 5, //For best results with link selection and seeing link flyovers on roll over, this is the minimum recommended width for links
			"lineColor": "#000000",
			"labelVisible": false,
			"labelAutoRotate": false,
			"labelParallelAlignment": "middle",
			"labelOrthogonalAlignment": "left",
			"labelDistance": 5,
			"labelPositionType": "relative",
			"labelPosition": 0.5
		};
		
		this._defaultLinkStatus = {            
            "scalarImageHeight": 25,
            "scalarImageWidth": 25,
            "scalarImageVisible": true,
            "imageHeight": 16,
            "imageWidth": 16,
            "imageVisible": true,
            "color": "#000000"
		};
		
		this._defaultRelationshipStyle = this._defaultLinkStyle;
		
		this._defaultRelationshipStatus = this._defaultLinkStatus;
		
		this._defaultDatasetStatus = {
			"backgroundScalarImage": "[widget]\/resources\/common_assets\/status_icons\/BK_SWF\/unknownBk141x117gradient.swf",
            "backgroundScalarImageHeight": 117,
            "backgroundScalarImageWidth": 141,
            "backgroundScalarImageVisible": true,            
            "scalarImageHeight": 25,
            "scalarImageWidth": 25,
            "scalarImageVisible": true,
            "imageHeight": 16,
            "imageWidth": 16,
            "imageVisible": true
		};
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	destroy: function() {		
		// summary:
		//		Do any necessary clean up
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},
	
	
	//////////////////
	// pluggableview
	//////////////////
	getPluggableView: function() {
		if (!this._widgetDef.pluggableview) {
			this._widgetDef.pluggableview = {};
		}
		
		return this._widgetDef.pluggableview;
	},
	
	//////////
	// views 
	//////////
	getViews: function(){
		var pluggableview = this.getPluggableView();
		return pluggableview.views;
	},
	
	////////////
	// toolbar 
	////////////
	getToolbar: function() {
		var pluggableview = this.getPluggableView();		
		var returnValue = pluggableview.toolbar;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getToolbar) {
				returnValue = parent.getToolbar();
			}
		}		
		return returnValue;	
	},
	
	getToolbarRows: function(){
		var toolbar = this.getToolbar();
		return (toolbar ? toolbar.rows : []);
	},
	
	isToolbarHidden: function() {
		var toolbar = this.getToolbar();
		var returnValue = (toolbar ? toolbar.hidden : undefined);
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}	
	},
	
	//////////////////
	// selectionMode 
	/////////////////
	getSelectionMode: function(){
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.selectionMode;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getSelectionMode){
			returnValue = this._parentWidgetDef.getSelectionMode();
		}		
		return returnValue;			
	},
	
	///////////////
	// labelStyle 
	//////////////
	getLabelStyle: function(){
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.labelStyle;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getLabelStyle){
			returnValue = this._parentWidgetDef.getLabelStyle();
		}		
		return returnValue;
	},
	
	///////////////
	// itemStyles 
	//////////////
	getItemStyles: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.itemStyles;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getItemStyles){
			returnValue = this._parentWidgetDef.getItemStyles();
		}		
		return returnValue;			
	},
	
	getItemStyle: function(type) {
		var style = this._itemStyles[type];
		
		if (!style) {
			var styles = this.getItemStyles();
			if (styles) {
				var styleLength = styles.length;
				for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					if (styles[styleIndex].type === type) {
						style = styles[styleIndex];
						this._itemStyles[type] = style;
						break;
					}
				}
			}
		}
		
		return style;		
	},
	
	////////////////
	// itemStatuses 
	////////////////
	getItemStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.itemStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getItemStatuses){
			returnValue = this._parentWidgetDef.getItemStatuses();
		}		
		return returnValue;	
	},
	
	getItemStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._itemStatuses[key];
		
		if (!status) {
			var statuses = this.getItemStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id &&  statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._itemStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},
	
	///////////////
	// linkStyles 
	///////////////
	getLinkStyles: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.linkStyles;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getLinkStyles){
			returnValue = this._parentWidgetDef.getLinkStyles();
		}		
		return returnValue;			
	},
	
	getLinkStyle: function(type) {
		var style = this._linkStyles[type];
		
		if (!style) {
			var styles = this.getLinkStyles();
			if (styles) {
				var styleLength = styles.length;
				for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					if (styles[styleIndex].type === type) {
						style = styles[styleIndex];
						this._linkStyles[type] = style;
						break;
					}
				}
			}
		}
		
		return style;		
	},
	
	/////////////////
	// linkStatuses 
	////////////////
	getLinkStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.linkStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getLinkStatuses){
			returnValue = this._parentWidgetDef.getLinkStatuses();
		}		
		return returnValue;			
	},
	
	getLinkStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._linkStatuses[key];
		
		if (!status) {
			var statuses = this.getLinkStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id && statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._linkStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},
		
	///////////////////////
	// relationshipStyles 
	///////////////////////
	getRelationshipStyles: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.relationshipStyles;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getRelationshipStyles){
			returnValue = this._parentWidgetDef.getRelationshipStyles();
		}		
		return returnValue;
	},
	
	getRelationshipStyle: function(type) {
		var style = this._relationshipStyles[type];
		
		if (!style) {
			var styles = this.getRelationshipStyles();
			if (styles) {
				var styleLength = styles.length;
				for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					if (styles[styleIndex].type === type) {
						style = styles[styleIndex];
						this._relationshipStyles[type] = style;
						break;
					}
				}
			}
		}
		
		return style;		
	},
	
	/////////////////////////
	// relationshipStatuses 
	/////////////////////////
	getRelationshipStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.relationshipStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getRelationshipStatuses){
			returnValue = this._parentWidgetDef.getRelationshipStatuses();
		}		
		return returnValue;
	},
	
	getRelationshipStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._relationshipStatuses[key];
		
		if (!status) {
			var statuses = this.getRelationshipStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id && statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._relationshipStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},
	
	///////////////////
	// datasetSatuses 
	///////////////////
	getDatasetStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.datasetStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getDatasetStatuses){
			returnValue = this._parentWidgetDef.getDatasetStatuses();
		}		
		return returnValue;	
	},
	
	getDatasetStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._datasetStatuses[key];
		
		if (!status) {
			var statuses = this.getDatasetStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id && statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._datasetStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},

	getDefaultItemStyle: function() {
		return this._defaultItemStyle;
	},
	
	getDefaultLinkStyle: function() {
		return this._defaultLinkStyle;
	},
	
	getDefaultRelationshipStyle: function() {
		return this._defaultRelationshipStyle;
	},
		
	getDefaultItemStatus: function() {
		return this._defaultItemStatus;
	},
	
	getDefaultLinkStatus: function() {
		return this._defaultLinkStatus;
	},
	
	getDefaultRelationshipStatus: function() {
		return this._defaultRelationshipStatus;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_getTypeSection: function() {
		var type = this.getWidgetType();
		if (type.indexOf("-") > 0) {
			var typeInfo = type.split("-");
			type = typeInfo[0];    		        
        }
        
        return this.getSection(type);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.NavWidgetDef"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.NavWidgetDef"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.NavWidgetDef"); 
 
// includes


////////////////////////////////////
// NavWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.NavWidgetDef", ibm.tivoli.navmodel.WidgetDef, {
	
	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	getBreadcrumbs: function(){
		var returnValue = this._widgetDef.navwidget.breadcrumbs;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
		 	if (parent && parent.getBreadcrumbs) {
				returnValue = parent.getBreadcrumbs();
			}
		}		
		return returnValue;			
	},
	
	getNotebook: function(){
		var returnValue = this._widgetDef.navwidget.notebook;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
		 	if (parent && parent.getNotebook) {
				returnValue = parent.getNotebook();
			}
		}		
		return returnValue;			
	},
	
	getViewsLayout: function(){
		var returnValue = this._widgetDef.navwidget.viewsLayout;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
		 	if (parent && parent.getViewsLayout) {
				returnValue = parent.getViewsLayout();
			}
		}		
		return returnValue;					
	},
	

	destroy: function() {		
		// summary:
		//		Do any necessary clean up 
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	}
	
	/////////////////////////////
	// private methods
	/////////////////////////////

});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.TopoWidgetDef"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.TopoWidgetDef"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.TopoWidgetDef"); 
 
// includes


////////////////////////////////////
// TopoWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.TopoWidgetDef", ibm.tivoli.navmodel.WidgetDef, {

	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs TopoWidgetDef object.
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	validateDefaults: function() {
		// summary:
		//		This method is called after the widget def has been read from the server.
		//		Validate defaults that might not be present in the widget def contents.
		
		var layouts = this.getLayouts();
		if (layouts) {
			var length = layouts.length;
			for (var index = 0;  index < length;  ++index) {
				var layout = layouts[index];
				switch (layout.algorithm) {
					case "tree":
						if (!layout.layoutMode) {
							layout.layoutMode = "alternating radial";
						}
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "straight";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						if (!layout.globalAlignment) {
							layout.globalAlignment = "center";
						}
						if (!layout.levelAlignment) {
							layout.levelAlignment = "center";
						}
						if (!layout.aspectRatio) {
							layout.aspectRatio = "1";
						}
						break;
					case "hierarchical":
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.levelJustification) {
							layout.levelJustification = "bottom";
						}
						if (!layout.levelingStrategy) {
							layout.levelingStrategy = "optimal";
						}
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "straight";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						break;
					case "forceDirected":
						if (!layout.layoutMode) {
							layout.layoutMode = "fast multilevel";
						}
						if (!layout.preferredLinksLength) {
							layout.preferredLinksLength = "25";
						}
						if (!layout.linkStyle) {
							layout.linkStyle = "straight";
						}
						if (!layout.maxAllowedMovePerIteration) {
							layout.maxAllowedMovePerIteration = "5";
						}
						if (!layout.allowedNumberOfIterations) {
							layout.allowedNumberOfIterations = "250";
						}
						if (!layout.convergenceThreshold) {
							layout.convergenceThreshold = "0.1";
						}						
						break;
					case "shortLink":
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "direct";
						}
						if (!layout.globalSelfLinkStyle) {
							layout.globalSelfLinkStyle = "two bends orthogonal";
						}
						if (!layout.globalConnectorStyle) {
							layout.globalConnectorStyle = "automatic";
						}
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.levelJustification) {
							layout.levelJustification = "bottom";
						}
						if (!layout.levelingStrategy) {
							layout.levelingStrategy = "optimal";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						if (!layout.linkOffset) {
							layout.linkOffset = "1";
						}
						if (!layout.minFinalSegmentLength) {
							layout.minFinalSegmentLength = "10";
						}
						break;
					case "longLink":
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "direct";
						}
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.levelJustification) {
							layout.levelJustification = "bottom";
						}
						if (!layout.levelingStrategy) {
							layout.levelingStrategy = "optimal";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						if (!layout.verticalMinOffset) {
							layout.verticalMinOffset = "3";
						}
						if (!layout.horizontalMinOffset) {
							layout.horizontalMinOffset = "3";
						}
						break;
					case "grid":
						if (!layout.layoutMode) {
							layout.layoutMode = "tile to rows";
						}
						if (!layout.globalHorizontalAlignment) {
							layout.globalHorizontalAlignment = "center";
						}
						if (!layout.globalVerticalAlignment) {
							layout.globalVerticalAlignment = "center";
						}
						if (!layout.maxNumberOfNodesPerRowOrColumn) {
							layout.maxNumberOfNodesPerRowOrColumn = "3";
						}
				}
			}
		}
	},
	
	destroy: function() {
		// summary:
		//		Do any necessary clean up 
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},

	getDepth: function() {
		var returnValue = this._widgetDef.topology.depth;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getDepth) {
				returnValue = parent.getDepth();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1;
		}
	},
	
	getInitialZoomLevel: function() {
		var returnValue = this._widgetDef.topology.initialZoomLevel;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getInitialZoomLevel) {
				returnValue = parent.getInitialZoomLevel();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1; //Default to -1 which would result in just fitting the contents
		}
	},	

	zoomOnObjectAdded: function() {
		return this._widgetDef.topology.zoomOnObjectAdded;
	},

	zoomLevelOnObjectAdded: function() {
		return this._widgetDef.topology.zoomLevelOnObjectAdded;
	},

	getMaxDepth: function() {
		var returnValue = this._widgetDef.topology.maxDepth;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getMaxDepth) {
				returnValue = parent.getMaxDepth();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1;
		}
	},

	useNodeLevelAndPosition: function() {
		return this._widgetDef.topology.useNodeLevelAndPosition;
	},

	getNodeOrderDirection: function() {
		return this._widgetDef.topology.nodeOrderDirection;
	},

	getNodeOrderProperty: function() {
		return this._widgetDef.topology.nodeOrderProperty;
	},

	getBackgroundColor: function() {
		var returnValue = this._widgetDef.topology.backgroundColor;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getBackgroundColor) {
				returnValue = parent.getBackgroundColor();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1;
		}
	},
	
	/////////////////////////////
	// footer section data
	/////////////////////////////
	
	getIncludeFooter: function() {
		var returnValue = this._widgetDef.topology.includeFooter;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getIncludeFooter) {
				returnValue = parent.getIncludeFooter();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return true;
		}
	},
	
	setIncludeFooter: function(includeFooter) {
		this._widgetDef.topology.includeFooter = includeFooter;
	},
	
	getHideFooterDateTime: function(){
		var returnValue = this._widgetDef.topology.hideFooterDateTime;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getHideFooterDateTime) {
				returnValue = parent.getHideFooterDateTime();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}
	},

	setHideFooterDateTime: function(hideFooterDateTime) {
		this._widgetDef.topology.hideFooterDateTime = hideFooterDateTime;
	},
	
	getFilters: function(){
		var returnValue = this._widgetDef.topology.filters;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getFilters) {
				returnValue = parent.getFilters();
			}
		}		
		return returnValue;			
	},

	/////////////////////////////
	// search section data
	/////////////////////////////

	isAdvSearchEnabled: function() {
		var returnValue = this._widgetDef.topology.isAdvSearchEnabled;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.isAdvSearchEnabled) {
				returnValue = parent.isAdvSearchEnabled();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}
	},
	
	getAdvSearchProperties: function() {
		var returnValue = this._widgetDef.topology.advSearchProperties;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getAdvSearchProperties) {
				returnValue = parent.getAdvSearchProperties();
			}
		}		
		return returnValue;	
	},	

	/////////////////////////////
	// layout section data
	/////////////////////////////	
	getLayouts: function() {
		var returnValue = this._widgetDef.topology.layout;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getLayouts) {
				returnValue = parent.getLayouts();
			}
		}		
		return returnValue;	
	},
	
	updateLayouts: function(layouts) {
		var delta = this.getWidgetDelta("user");
		if (delta) {
			delta.setSectionAttribute({sectionKey: "topology", attrKey: "layout", attrValue: layouts}); 
		} 
	},
	
	clearLayouts: function() {
		var delta = this.getWidgetDelta("user");
		if (delta) {
			delta.removeSectionAttribute({sectionKey: "topology", attrKey: "layout"}); 
		}
	},
	
	getLayout: function(/*String*/ id) {
		var layouts = this.getLayouts();
		for(var i=0;i<layouts.length;i++){
			if(layouts[i].id === id){
				return layouts[i];
			}
		}
	},
	
	getDefaultLayout: function() {
		var layouts = this.getLayouts();
		for(var i=0;i<layouts.length;i++){
			if(layouts[i].isDefault){
				return layouts[i];
			}
		}
	},
	
	getDefaultLayoutId: function() {
		var layouts = this.getLayouts();
		for(var i=0;i<layouts.length;i++){
			if(layouts[i].isDefault){
				return layouts[i].id;
			}
		}
	},
		
	getRenderRelationshipsInsteadOfLinks: function(){
		var returnValue = this._widgetDef.topology.renderRelationshipsInsteadOfLinks;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getRenderRelationshipsInsteadOfLinks) {
				returnValue = parent.getRenderRelationshipsInsteadOfLinks();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}		
	}
	
	/////////////////////////////
	// private methods
	/////////////////////////////	

});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.TableWidgetDef"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.TableWidgetDef"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.TableWidgetDef"); 
 
// includes


////////////////////////////////////
// TableWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.TableWidgetDef", ibm.tivoli.navmodel.WidgetDef, {

	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs TopoWidgetDef object.
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	destroy: function() {
		// summary:
		//		Do any necessary clean up 
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},
		
	/////////////////////////////
	// dataset section data
	/////////////////////////////	
	getDataSet: function() {
		if (!this._widgetDef.datasets) {
			this._widgetDef.datasets = [];
		}
		
		if (this._widgetDef.datasets.length === 0) {
			this._widgetDef.datasets[0] = {};
		}
		
		return this._widgetDef.datasets;		
	},

	getDataSetID: function() {
		var datasets = this.getDataSet();
		return datasets[0].id;
	},

	getDataSetLabelKey: function() {
		var datasets = this.getDataSet();
		return datasets[0].label.key;
	},
	
	getDataSetLabelValue: function() {
		var datasets = this.getDataSet();
		return datasets[0].label.value;
	},
	
	getDataSetQuery: function() {
		var datasets = this.getDataSet();
		return datasets[0].query;
	},
	
	getDataSetColumns: function() {
		var datasets = this.getDataSet();
		return datasets[0].columns;		
	},
	
	getDataSetThresholds: function(){
		var datasets = this.getDataSet();
		return datasets[0].thresholds;
	},

	getDataSetParameters: function() {
		var datasets = this.getDataSet();
		return datasets[0].parameters;
	},
	
	getDataSetResourceBundle: function() {
		var datasets = this.getDataSet();
		return datasets[0].label.resourceBundle;
	},
	
	
	getDataSetDataSource: function() {
		var datasets = this.getDataSet();
		return datasets[0].datasource;
	},
	
	getDataSetDatasourceType: function() {
		var datasets = this.getDataSet();
		datasets[0].datasourceType = type;
	},
	
	setDataSet: function(datasets) {
		this._widgetDef.datasets = datasets;		
	},

	setDataSetID: function(id) {
		var datasets = this.getDataSet();
		datasets[0].id = id;
	},

	setDataSetLabelKey: function(key) {
		var datasets = this.getDataSet();
		datasets[0].label.key = key;
	},
	
	setDataSetLabelValue: function(value) {
		var datasets = this.getDataSet();
		datasets[0].label.value = value;
	},
	
	setDataSetQuery: function(query) {
		var datasets = this.getDataSet();
		datasets[0].query = query;
	},
	
	setDataSetColumns: function(columns) {
		var datasets = this.getDataSet();
		datasets[0].columns = columns;
	},

	setDataSetParameters: function(parameters) {
		var datasets = this.getDataSet();
		datasets[0].parameters = parameters;
	},
	
	setDataSetResourceBundle: function(bundle) {
		var datasets = this.getDataSet();
		datasets[0].label.resourceBundle = bundle;
	},
	
	/////////////////////////////
	// table section data
	/////////////////////////////	
	getTable: function() {
		if (!this._widgetDef.table) {
			this._widgetDef.table = {};
		}
		
		return this._widgetDef.table;		
	},
	
	getTableID: function() {
		var table = this.getTable();
		return table.id;		
	},
	
	getTableView: function() {
		var table = this.getTable();
		if (!table.view) {
			table.view = [];
		}
		
		return table.view;		
	},
	
	getTableViewElement: function(id) {
		var entry = null;
		var view = this.getTableView();
		if (view) {
			var length = view.length;
			for (var index = 0;  index < length;  ++index) {
				if (view[index].id === id) {
					entry = view[index];
					break;
				}
			}
		}
		
		return entry;
	},
	
	getTableCommon: function() {
		var table = this.getTable();
		return table.common;		
	},

	getTableCommonAutoUpdate: function() {
		var table = this.getTable();
		return table.common.autoUpdate;		
	},
	
	getTableCommonShowLastUpdateTime: function() {
		var table = this.getTable();
		return table.common.showLastUpdateTime;		
	},
	
	getTableCommonPaging: function() {
		var table = this.getTable();
		return table.common.paging;		
	},
	
	getTableCommonPagingSize: function() {
		var table = this.getTable();
		return table.common.pagingSize;		
	},
	
	getTableShowAdvancedFilter: function() {
		var table = this.getTable();
		return table.showAdvancedFilter;
	},
		
	getTableFilter: function() {
		var table = this.getTable();
		return table.filter;
	},
	
	getTableLabelKey: function() {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		
		return table.label.key;		
	},
	
	getTableLabelValue: function() {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		
		return table.label.value;		
	},
	
	getTableDescriptionKey: function() {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		
		return table.description.key;		
	},
	
	getTableDescriptionValue: function() {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		
		return table.description.value;		
	},
	
	getTableColumnReordering: function() {
		var table = this.getTable();
		return table.columnReordering;		
	},

	setTable: function(table) {
		this._widgetDef.table = table;		
	},
	
	setTableID: function(id) {
		var table = this.getTable();
		table.id = id;		
	},
	
	setTableView: function(view) {
		var table = this.getTable();
		table.view = view;
	},
	
	setTableCommon: function(common) {
		var table = this.getTable();
		table.common = common;		
	},

	setTableCommonAutoUpdate: function(autoUpdate) {
		var table = this.getTable();
		table.common.autoUpdate = autoUpdate;		
	},
	
	setTableCommonShowLastUpdateTime: function(showLastUpdateTime) {
		var table = this.getTable();
		table.common.showLastUpdateTime = showLastUpdateTime;		
	},
	
	setTableCommonPaging: function(paging) {
		var table = this.getTable();
		table.common.paging = paging;		
	},
	
	setTableCommonPagingSize: function(pagingSize) {
		var table = this.getTable();
		table.common.pagingSize = pagingSize;		
	},
	
	setTableShowAdvancedFilter: function(showAdvancedFilter) {
		var table = this.getTable();
		table.showAdvancedFilter = showAdvancedFilter;
	},
	
	setTableFilter: function(filter) {
		var table = this.getTable();
		table.filter = filter;
	},
	
	setTableLabelKey: function(key) {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		table.label.key = key;		
	},
	
	setTableLabelValue: function(value) {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		table.label.value = value;		
	},
	
	setTableDescriptionKey: function(key) {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		table.description.key = key;		
	},
	
	setTableDescriptionValue: function(value) {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		table.description.value = value;		
	},
	
	setTableColumnReordering: function(columnReordering) {
		var table = this.getTable();
		table.columnReordering = columnReordering;		
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.NotebookWidgetDef"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.NotebookWidgetDef"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.NotebookWidgetDef"); 
 
// includes


////////////////////////////////////
// NavWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.NotebookWidgetDef", ibm.tivoli.navmodel.WidgetDef, {

	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs NotebookWidgetDef object.
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	destroy: function() {		
		// summary:
		//		Do any necessary clean up
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},
	
	getTabOrientation: function() {
		var returnValue = this._widgetDef.notebook.tabOrientation;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
		 	if (parent && parent.getTabOrientation) {
				returnValue = parent.getTabOrientation();
			}
		}		
		return returnValue;			
	}
	
	/////////////////////////////
	// private methods
	/////////////////////////////	

});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.AbstractDataset"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.AbstractDataset"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.AbstractDataset"); 
 
// includes





















ibm.tivoli.navmodel.AbstractDataset._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.AbstractDataset");

////////////////////////////////////
// AbstractDataset
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.AbstractDataset", null, {
	// private members       
	_args: null,
	
	// dataset attributes
	_dataset: null,
	_version: null,
	
	// widget definition supplied by server
	_widgetDefs: null,
	
	// parent, child and descendent datasets
	_parent: null,
	_childDatasets: null,
	_childDatasetsById: null,
	_childDatasetItems: null,
	_childDatasetItemsById: null,
	_descendentsById: null,
	
	// item, link, relationship, caches
	_itemsById: null,
	_linksById: null,
	_relationshipsById: null,
	_cacheIntervalId: null,
	
	// styles and status caches
	_itemStyles: null,
	_itemStylesById: null,
	_itemStatus: null,
	_itemStatusById: null,	
	_linkStyles: null,
	_linkStylesById: null,
	_linkStatus: null,
	_linkStatusById: null,
	_relStyles: null,
	_relStylesById: null,
	_relStatus: null,
	_relStatusById: null,
	_status: null,
		
	// miscellaneous
	_toolbar: null,
	_eventRouter: null,
	_listeners: null,
	_destroyed: false,
	_trace: ibm.tivoli.navmodel.AbstractDataset._trace, 
	
	/*=====
	constructor_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	constructor_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	 constructor_kwArgs: function(onComplete, onError, baseUrl, provider, datasource, dataset, 
	 					parameters, defaultProperties, defaultLinkProperties, defaultRelProperties, 
	 					cacheThreshold, cacheValidationInterval, cacheExpirationTime, version) {
	 	// onComplete: constructor_onComplete
	 	//		callback to be invoked when the Dataset is initialized and ready for methods
	 	//		to be invoked  (i.e. asynchronous initialization must be performed). 
	 	//		Note: A null value will result in synchronous initialization.
	 	// onError: constructor_onError
	 	//		callback to be invoked if there is a problem constructing the Dataset
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// dataset: String
		//		dataset id
		// parameters: Object
		//		optional: extra parameters sent to dataset provider
		// defaultProperties: String[]
		//		optional: item property ids to fetch by default (can specify "all")
		// defaultLinkProperties: String[]
		//		optional: link property ids to fetch by default (can specify "all")
		// defaultRelProperties: String[]
		//		optional: relationship property ids to fetch by default (can specify "all")
		// cacheThreshold: int
		//		optional: cache objects if number of dataset items >= threshold
		// cacheValidationInterval: int
		//		optional: number of minutes in between cache validations
		// cacheExpirationTime: int
		//		optional: number of minutes objects can stay in cache while inactive
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	 },
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs, /*Object?*/ dataset) {
		// summary:
		//		Constructs AbstractDataset object.
		// description:
		//		The following options are normally not included in constructor (internal use only)
		//			createEventRouter: boolean - True indicates that a new event router should be created (vs. shared)
		//			dataset: object - Optional dataset object; will fetch from server if not provided
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		this._args = {
			onComplete: null,
			onError: null,
			baseUrl: "/ibm/tivoli/rest",
			provider: "",
			datasource: "",
			dataset: "",
			parameters: {},
			defaultProperties: [],
			defaultLinkProperties: [],
			defaultRelProperties: [],
			cacheThreshold:	250,
			cacheValidationInterval: 5,
			cacheExpirationTime: 15,
			version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion(),
			
			// The following options are normally not included in constructor
			createEventRouter: true
		};
		dojo.mixin(this._args, kwArgs);
		
		// init empty caches
		this._itemsById = {};
		this._linksById = {};
		this._relationshipsById = {};
	
		// add param_pageId to help build unique event router queue: <user>_<sessionId>_<pageId>
		this._args.parameters.pageId = ibm.tivoli.navmodel.EventRouter.getPageId();
		
		// add param_tz for the user timezone
		if (!this._args.parameters.tz) {
			this._args.parameters.tz = ibm.tivoli.navmodel.Utils.getTimeZone();
		}
		
		// subclass can initialize the dataset
		var initArgs = {
			onComplete: this._args.onComplete,
			onError: this._args.onError,
			optimize: true,
			dataset: dataset,
			createEventRouter: this._args.createEventRouter
		};
		this._initDataset(initArgs);		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	refresh: function() {
		// summary:
		//		This method forces the dataset object to do a full refresh of its 
		//		data and caches
		
		// clear caches, refresh, and notify listeners
		this.clientChangedEvent();
	},

	destroy: function() {
		// summary:
		//		This method should be called to clean up the Dataset object
		
		if (!this._destroyed) {
			// cancel the cache validation timer
			if (this._cacheIntervalId) {
				clearInterval(this._cacheIntervalId);
			}
			
			// do any clean up like unregistering from event router, etc
			this._cleanupDataset();
			
			// destroy widget definition if we have one
			if (this._widgetDefs) {
				var length = this._widgetDefs.length;
				for (var index = 0;  index < length;  ++index) {
					if (this._widgetDefs[index]) {
						this._widgetDefs[index].destroy();
					}
				}								
			}
			
			this._destroyed = true;
		}
	},
		
	getArgs: function() {
		// summary:
		//		Returns the initial arguments passed to the dataset
		return dojo.clone(this._args);
	},

	getArg: function(key) {
		// summary:
		//		Returns a specific initial argument passed to the dataset
		return this._args[key];
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general dataset attribute for requested key
		return this._dataset[key];
	},

	getAttributes: function(keyPrefix) {
		// summary:
		//		Returns attribute map for all keys that begin with keyPrefix;  if no
		//		keyPrefix is supplied; all attributes are returned
		var attr = {};		
		if (keyPrefix) {
			for (var key in this._dataset) {
				if (this._dataset.hasOwnProperty(key)) {
					if (key.indexOf(keyPrefix) === 0) {
						attr[key] = this._dataset[key];
					}			
				}
			}
		}
		else {
			dojo.mixin(attr, this._dataset);
		}
		
		return attr;
	},

	setAttribute: function(key, value) {
		// summary:
		//		Set general dataset attribute for requested key
		this._dataset[key] = value;		
	},

	mixinAttributes: function(attr) {
		// summary:
		//		Mixin general dataset attributes
		dojo.mixin(this._dataset, attr);		
	},
	
	isClearCacheOnRefresh: function() {
		// summary:
		//		Returns true if the dataset is requesting that
		//		?clearCache be placed on url when doing a Refresh;
		//		which will cause CURI cache to purge the dataset and reconstruct		
		return (!this._dataset.eventsEnabled || this._dataset.clearCacheOnRefresh);
	},
	
	isEventsEnabled: function() {
		// summary:
		//		Returns true if the dataset is capable of producing events
		return this._dataset.eventsEnabled;
	},

	getEventListenerCount: function() {
		// summary:
		//		Returns number of registered event listeners
		var listenerCount = 0;
		if (this._listeners) {
			listenerCount = this._listeners.length;
		}
		
		return listenerCount;
	},
	
	/*=====
	_listenerCallback: function(listenerObj, event) {
		// listenerObj
		// event
	},
	=====*/

	addEventListener: function(/*Object*/ listenerObj, /*_listenerCallback*/ listenerCallback) {
		// summary:
		//		Adds an event listener to the Dataset
		// listenerObj:
		//		unique object (i.e. same object must be passed to removeEventListener)
		// listenerCallback
		//		listener callback function for receiving the events
		
		// remove listener if it already exists
		this.removeEventListener(listenerObj);
			
		// add new listener
		if (!this._listeners) {
			this._listeners = [];
		}		
		this._listeners.push({listenerObj: listenerObj, listenerCallback: listenerCallback});
		
		// add listener to child datasets
		if (this._childDatasets) {
			var childLength = this._childDatasets.length;
			for (var childIndex = 0;  childIndex < childLength;  ++childIndex) {
				this._childDatasets[childIndex].addEventListener(listenerObj, listenerCallback);
			}
		}
	},
	
	removeEventListener: function(/*Object*/ listenerObj) {
		// summary:
		//		Removes an event listener from the Dataset
		// listernObj:
		//		previously registered listener object
		if (this._listeners) {
			var listenerLength = this._listeners.length;
			for (var listenerIndex = 0;  listenerIndex < listenerLength;  ++listenerIndex) {
				if (this._listeners[listenerIndex].listenerObj === listenerObj) {
					// remove listener
					this._listeners.splice(listenerIndex, 1);
					
					// last listener so null out array					
					if (this._listeners.length === 0) {
						this._listeners = null;
					}
					
					break;
				}
			}
			
			// remove listener from child datasets
			if (this._childDatasets) {
				var childLength = this._childDatasets.length;
				for (var childIndex = 0;  childIndex < childLength;  ++childIndex) {
					this._childDatasets[childIndex].removeEventListener(listenerObj);
				}
			}		
		}
	},	

	getVersion: function() {
		// summary:
		//		Returns the version used to communicate with rest service
		//		(i.e. json version)
		return this._version;
	},
	
	_setVersion: function(version) {
		// summary:
		//		Sets the version used to communicate with the rest servce
		//		(i.e. json version)
		this._version = version;
	},

	getId: function() {
		// summary:
		//		Returns the dataset id
		return this._dataset.id;
	},

	setId: function(id) {
		// summary:
		//		Set the dataset id
		this._dataset.id = id;
	},

	getProviderId: function() {
		// summary:
		//		Returns the provider id
		return this._dataset.providerId;
	},

	setProviderId: function(providerId) {
		// summary:
		//		Set the provider id
		this._dataset.providerId = providerId;
	},

	getDatasourceId: function() {
		// summary:
		//		Returns the datasource id
		return this._dataset.datasourceId;
	},
	
	setDatasourceId: function(datasourceId) {
		// summary:
		//		Set the datasource id
		this._dataset.datasourceId = datasourceId;
	},

	getEventId: function() {
		// summary:
		//		Returns the event id
		return this._dataset.eventId;
	},

	setEventId: function(eventId) {
		// summary:
		//		Set the event id
		this._dataset.eventId = eventId;
	},

	getType: function() {
		// summary:
		//		Returns the dataset type
		return this._dataset.type;
	},

	setType: function(type) {
		// summary:
		//		Set the dataset type
		this._dataset.type = type;		
	},

	getViewType: function() {
		// summary:
		//		Returns the preferred view type for rendering
		//		
		//		values: "graph", "table", "tree"
		return this._dataset.viewType;
	},

	setViewType: function(viewType) {
		// summary:
		//		Set the preferred view type for rendering
		this._dataset.viewType = viewType;		
	},
	
	isGraph: function() {
		// summary:
		//		Returns true if the dataset represents a graph
		return this.getViewType() === "graph";
	},

	isTable: function() {
		// summary:
		//		Returns true if the dataset represents a table
		return this.getViewType() === "table";
	},

	isTree: function() {
		// summary:
		//		Returns true if the dataset represents a tree
		return this.getViewType() === "tree";
	},

	isLazyLoad: function() {
		// summary:
		//		Returns true if the dataset designates that it should be lazy loaded
		return (this._dataset.lazyLoad !== undefined ? this._dataset.lazyLoad : false);
	},
	
	setLazyLoad: function(lazyLoad) {
		// summary:
		//		Set lazy load attribute for the dataset
		this._dataset.lazyLoad = lazyLoad;
	},
	
	getLazyLoadRelationships: function() {		
		// summary:
		//		Returns the relationships that should be explored for related items
		return this._dataset.relationshipsKey;
	},

	setLazyLoadRelationships: function(relationshipsKey) {		
		// summary:
		//		Set the relationships that should be explored for related items
		this._dataset.relationshipsKey = relationshipsKey;
	},
	
	getLabel: function() {	
		// summary:
		//		Returns globalized label for the dataset
		return this._dataset.label;
	},

	setLabel: function(label) {
		// summary:
		//		Set globalized label for the dataset
		this._dataset.label = label;		
	},

	getLabelQualifier: function() {
		// summary:
		//		Returns globalized label qualifier that can be used in conjunction with the label
		//		i.e.    label (labelQualifier)
		return this._dataset.labelQualifier;
	},

	setLabelQualifier: function(labelQualifier) {
		// summary:
		//		Set globalized label qualifier that can be used in conjunction with the label
		this._dataset.labelQualifier = labelQualifier;		
	},
	
	/*=====
	depth_kwArgs1: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override the depth value
	},
	=====*/

	getDepth: function(/*depth_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the default depth for the dataset if it is a graph
		//		(otherwise -1)
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.widgetDef) {
			return args.widgetDef.getDepth();
		}
				
		return (this._dataset.depth ? this._dataset.depth : -1);		
	},

	setDepth: function(depth) {
		// summary:
		//		Set the depth for the dataset if it is a graph
		// depth:
		//		New topology depth
		
		// clean up current dataset
		this._cleanupDataset();
		
		// update depth parameter
		var found = false;
		for (var parm in this._args.parameters) {
			if (parm === "depth" && this._args.parameters.hasOwnProperty(parm)) {
				this._args.parameters[parm] = depth;
				found = true;
				break;
			}
		}
		if (!found) {
			this._args.parameters.depth = depth;
		}
		
		// clear caches, refresh, and notify listeners
		var event = ibm.tivoli.navmodel.ClientEvent.getChangedEvent(this);
		event.createEventRouter = true;
		this._eventHandler(event);		
	},

	getMaxDepth: function(/*depth_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the maximum depth for the dataset if it is a graph
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.widgetDef) {
			return args.widgetDef.getMaxDepth();
		}
		
		return (this._dataset.maxDepth ? this._dataset.maxDepth : -1);		
	},

	getDefaultProperties: function() {
		// summary:
		//		Returns default properties that are fetched with each item
		return this._args.defaultProperties;
	},
	
	setDefaultProperties: function(itemProperties) {
		// summary:
		//		Sets default properties that are fetched with each item
		// itemProperties:
		//		Array of item property ids
		this._args.defaultProperties = itemProperties;
	},

	getDefaultLinkProperties: function() {
		// summary:
		//		Returns default properties that are fetched with each link
		return this._args.defaultLinkProperties;
	},
	
	setDefaultLinkProperties: function(linkProperties) {
		// summary:
		//		Sets default properties that are fetched with each link
		// linkProperties:
		//		Array of link properties
		this._args.defaultLinkProperties = linkProperties;
	},

	getDefaultRelProperties: function() {
		// summary:
		//		Returns default properties that are fetched with each relationship
		return this._args.defaultRelProperties;
	},
	
	setDefaultRelProperties: function(relProperties) {
		// summary:
		//		Sets default properties that are fetched with each relationship
		// relProperties:
		//		Array of relationship properties
		this._args.defaultRelProperties = relProperties;
	},
	
	cacheExpires: function() {
		// summary:
		//		Returns true if the items, links, and relationships cache expires for this dataset.  If
		//		there are only a small number of objects the dataset will not bother with expiration.
		return (this._cacheIntervalId ? true : false);
	},

	getNewExpirationTime: function() {
		// summary:
		//		Returns an expiration time for a new item, link, or relationship
		var now = new Date();
		return now.getTime() + (this._args.cacheExpirationTime*60000);
	},
	
	getItemCount: function() {
		// summary:
		//		Returns the number of items associated with the dataset
		return this._dataset.itemCount;
	},
	
	/*=====
	itemsAndLinks_onComplete: function(items, links, datasetItems, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getItemsAndLinks_kwArgs: function(onComplete, onError, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItemsAndLinks: function(/*getItemsAndLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items, links, and datasetItems
		//		(ibm.tivoli.navmodel.ChildDatasetItem  i.e. not a full Dataset);
		//		associated with the dataset
		var args = {                
			onComplete: function(items, links, datasetItems, kwArgs) {},
			onError: function(error, exception, ioArgs) {},
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		// fetch items and links from rest service
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, links, datasetItems, toolbar, args) {
			// cache items and links
			items = this._cacheItems(items);
			links = this._cacheLinks(links);
				
			// cache toolbar tasks				
			this._toolbar = toolbar;
				
			// call orignal callback
			onComplete(items, links, datasetItems, args);
		});			
		this._fetchItemsAndLinks(args);		
	},
	
	/*=====
	items_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	getItems_kwArgs: function(onComplete, onError, recursive, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// recursive: boolean
		//		also fetch all childDataset items if true 
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItems: function(/*getItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset
		var args = {                
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			recursive: false,
			start: 0,
			count: null,
			properties: this._args.defaultProperties,
			metadata: false, 
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		///////////////////////////////////////////////////////////
		// make sure we always return asynchronously in case we are 
		// being invoked by a dojo datastore
		///////////////////////////////////////////////////////////
		
		// are NOT any items
		if (this.getItemCount() === 0) {		
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete(null, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
			}			
		}
		// fetch items from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(items, args) {
				// cache items
				items = this._cacheItems(items);
				
				// call orignal callback
				onComplete(items, args);
			});
			this._fetchItems(args);
		}		
	},
	
	getCachedItems: function(/*Boolean*/ recursive) {
		// summary:
		//		returns array with currently cached items	
		var items = [];
		this._traverseCachedItems(items, recursive);
		return items;
	},
	
	_traverseCachedItems: function(/*ibm.tivoli.navmodel.Item[]*/ items, /*Boolean*/ recursive) {
		if (this._itemsById) {
			for (var id in this._itemsById) { 
    			if (this._itemsById.hasOwnProperty(id)) {
    				items.push(this._itemsById[id]);
    			}
    		}
		}
		
		if (recursive && this.hasChildDatasets()) {
			var childDatasets = this.getChildDatasets();
			var length = childDatasets.length;
			for (var index = 0;  index < length;  ++index) {
				childDatasets[index]._traverseCachedItems(items, recursive);
			}
		}
	},
	
	/*=====
	getItem_kwArgs: function(itemId, properties, metadata) {
		// itemId: String
		//		item id to get
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/

	getItem: function(/*getItem_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the requested item
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var args = {
			itemId: null,
			properties: null,
			metadata: false
		};
		dojo.mixin(args, kwArgs);
		
		// check cache
		var item = this._getCachedItem(args.itemId);
		
		// force a fetch if we do not have all the requested properties		
		if (item && ((args.metadata && !item.hasCachedMetadata()) || (args.properties && !item.hasCachedProperties(args.properties)))) {
			item = null;			
		}
		
		// need to fetch individual item
		if (!item) {
			if (!args.properties || args.properties.length === 0) {
				args.properties = this._args.defaultProperties;
			}
			
			item = this._fetchItem(args);
			item = this._cacheItem(item);			
		}	

		return item;
	},
	
	/*=====
	getOriginItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metdata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getOriginItems: function(/*getOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
		var args = {                
			onComplete: function(items, links, datasetItems, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			properties: this._args.defaultProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, args) {
			// cache items
			items = this._cacheItems(items, true);
				
			// call orignal callback
			onComplete(items, args);
		});
		this._fetchOriginItems(args);
	},
	
	/*=====
	getRelatedItems_kwArgs: function(onComplete, onError, itemId, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of the base item
		// relationships: String
		//		comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested item
		var args = {                
			onComplete: function(items, links, datasetItems, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			itemId: null,
			relationships: null,
			start: 0,
			count: null,                
			properties: this._args.defaultProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, links, datasetItems, args) {
			// cache items and links
			items = this._cacheItems(items, true);
			links = this._cacheLinks(links);			
				
			// if no related items were returned; reset lazyLoad indicators
			if (!items || items.length === 0) {
				var item = this._getCachedItem(kwArgs.itemId);
				if (item) {
					item.resetLazyLoad();
				}
			}
			
			// call orignal callback
			onComplete(items, links, datasetItems, args);
		});		
		this._fetchRelatedItems(args);
	},
	
	/*=====
	getChildDatasetRelatedItems_kwArgs: function(onComplete, onError, childDatasetId, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of the base item
		// relationships: String
		//		comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	getChildDatasetRelatedItems: function(/*getChildDatasetRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested child dataset
		var args = {                
			onComplete: function(items, links, datasetItems, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			childDatasetId: null,
			relationships: null,
			start: 0,
			count: null,         
			properties: this._args.defaultProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, links, datasetItems, args) {
			// cache items and links
			items = this._cacheItems(items, true);
			links = this._cacheLinks(links);			
				
			// call orignal callback
			onComplete(items, links, datasetItems, args);
		});		
		this._fetchChildDatasetRelatedItems(args);
	},

	/*=====
	queryItems_kwArgs: function(onComplete, onError, query, start, count, properties, metadata, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// query: Object
		//		custom query object to be sent as request body
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
		var args = {                
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			query: {},                
			start: 0,
			count: null,
			properties: this._args.defaultProperties,
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, args) {
			// cache items
			items = this._cacheItems(items);
						
			// call orignal callback
			onComplete(items, args);
		});		
		this._queryItems(args);	
	},

	getLinkCount: function() {
		// summary:
		//		Returns the number of links associated with the dataset
		return this._dataset.linkCount;
	},

	/*=====
	links_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	getLinks_kwArgs: function(onComplete, onError, recursive, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// recursive: boolean
		//		also fetch all childDataset links if true 
		// start: int
		//		start index to fetch
		// count: int
		//		number of links to fetch; null means all
		// properties: String[]
		//		link property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	getLinks: function(/*getLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns links associated with the dataset
		var args = {                
			onComplete: function(links, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			recursive: false,
			start: 0,
			count: null,
			properties: this._args.defaultLinkProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         

		// are NOT any links
		if (this.getLinkCount() === 0) {
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete(null, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
			}
		}
		// fetch links from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(links, args) {
				// cache links
				links = this._cacheLinks(links);
				
				// call orignal callback
				onComplete(links, args);
			});
			this._fetchLinks(args);
		}
	},
	
	getCachedLinks: function(/*Boolean*/ recursive) {
		// summary:
		//		returns array with currently cached links		
		var links = [];
		this._traverseCachedLinks(links, recursive);		
		return links;
	},
	
	_traverseCachedLinks: function(/*ibm.tivoli.navmodel.Link[]*/ links, /*Boolean*/ recursive) {
		if (this._linksById) {
			for (var id in this._linksById) { 
    			if (this._linksById.hasOwnProperty(id)) {
    				links.push(this._linksById[id]);
    			}
    		}
		}
		
		if (recursive && this.hasChildDatasets()) {
			var childDatasets = this.getChildDatasets();
			var length = childDatasets.length;
			for (var index = 0;  index < length;  ++index) {
				childDatasets[index]._traverseCachedLinks(links, recursive);
			}
		}
	},

	/*=====
	getLink_kwArgs: function(itemId, properties, metadata) {
		// linkId: String
		//		link id to get
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	getLink: function(/*getLink_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the requested link
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var args = {
			linkId: null,
			properties: null,
			metadata: false
		};
		dojo.mixin(args, kwArgs);
		
		// check cache
		var link = this._getCachedLink(args.linkId);
		
		// force a fetch if we do not have all the requested properties		
		if (link && ((args.metadata && !link.hasCachedMetadata()) || (args.properties && !link.hasCachedProperties(args.properties)))) {
			link = null;			
		}
			
		// need to fetch individual link
		if (!link) {
			if (!args.properties || args.properties.length === 0) {
				args.properties = this._args.defaultLinkProperties;
			}
			
			link = this._fetchLink(args);
			link = this._cacheLink(link);			
		}	
		
		return link;
	},

	getRelationshipCount: function() {
		// summary:
		//		Returns the number of items associated with the dataset
		return this._dataset.relationshipCount;
	},
	
	/*=====
	relationships_onComplete: function(relationships, kwArgs){},
	=====*/
	
	/*=====
	getRelationships_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationships to fetch; null means all
		// properties: String[]
		//		relationship property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelationships: function(/*getRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Returns relationships associated with the dataset
		var args = {                
			onComplete: function(relationships, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			properties: this._args.defaultRelProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         

		///////////////////////////////////////////////////////////
		// make sure we always return asynchronously in case we are 
		// being invoked by a dojo datastore
		///////////////////////////////////////////////////////////
		
		// are NOT any links/relationships
		if (this.getLinkCount() === 0) {
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete(null, args);
			}
			else {		
				setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
			}
		}
		// fetch relationships from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(relationships, args) {
				// cache relationships
				relationships = this._cacheRelationships(relationships);
				
				// call orignal callback
				onComplete(relationships, args);
			});
			this._fetchRelationships(args);
		}
	},
	
	getCachedRelationships: function(/*Boolean*/ recursive) {
		// summary:
		//		return array with currently cached relationships	
		var relationships = [];
		this._traverseCachedRelationships(relationships, recursive);
		return relationships;
	},
	
	_traverseCachedRelationships: function(/*ibm.tivoli.navmodel.Relationship[]*/ relationships, /*Boolean*/ recursive) {
		if (this._relationshipsById) {
			for (var id in this._relationshipsById) { 
    			if (this._relationshipsById.hasOwnProperty(id)) {
    				relationships.push(this._relationshipsById[id]);
    			}
    		}
		}
				
		if (recursive && this.hasChildDatasets()) {
			var childDatasets = this.getChildDatasets();
			var length = childDatasets.length;
			for (var index = 0;  index < length;  ++index) {
				childDatasets[index]._traverseCachedRelationships(relationships, recursive);
			}
		}
	},
	
	/*=====
	getRelationship_kwArgs: function(relationshipId, properties, metadata) {
		// relationshipId: String
		//		relationship id to fetch
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	getRelationship: function(/*getRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the requested relationship
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var args = {
			relationshipId: null,
			properties: null,
			metadata: false
		};
		dojo.mixin(args, kwArgs);
		
		// check cache
		var relationship = this._getCachedRelationship(args.relationshipId);

		// force a fetch if we do not have all the requested properties		
		if (relationship && ((args.metadata && !relationship.hasCachedMetadata()) || (args.properties && !relationship.hasCachedProperties(args.properties)))) {
			relationship = null;			
		}
						
		// need to fetch individual relationship
		if (!relationship) {
			if (!args.properties || args.properties.length === 0) {
				args.properties = this._args.defaultRelProperties;
			}
			
			relationship = this._fetchRelationship(args);
			relationship = this._cacheRelationship(relationship);
		}	
		
		return relationship;
	},

	getParentDataset: function() {
		// summary:
		//		Returns the parent Dataset object for this child Dataset;
		//		returns null if no parent defined
		return this._parent;
	},

	hasChildDatasets: function() {
		// summary:
		//		Returns true if child Dataset objects are present
		return (this.getChildDatasetCount() > 0);
	},
	
	getChildDatasetCount: function() {
		// summary:
		//		Returns the number of child Dataset objects
		return (this._dataset.childDatasets ? this._dataset.childDatasets.length : 0);		
	},

	getChildDatasets: function() {
		// summary:
		//		Returns array of child Dataset objects;  null if there are not any.
		if (this.hasChildDatasets() && !this._childDatasets) {
			this._childDatasets = [];
			this._childDatasetsById = {};
			
			var childLength = this.getChildDatasetCount();
			for (var childIndex = 0;  childIndex < childLength;  ++childIndex) {
				var childDatasetId = this._dataset.childDatasets[childIndex].id;
				var childDataset = this._constructChildDataset(childDatasetId);
				this._childDatasets[childIndex] = childDataset;
				this._childDatasetsById[childDatasetId] = childDataset;			
			}
		}

		return this._childDatasets;		
	},

	getChildDataset: function(/*String*/ childDatasetId) {
		// summary:
		//		Returns requested child Dataset
		var childDataset = null;
		
		// have child datasets
		if (this.hasChildDatasets()) {			
			if (!this._childDatasets) {
				this.getChildDatasets();
			}
			
			childDataset = this._childDatasetsById[childDatasetId];			
		}
		
		return childDataset;
	},
	
	getChildDatasetItems: function() {
		// summary:
		//		Returns array of ibm.tivoli.navmodel.ChildDatasetItem placeholder items 
		//		(i.e. not a full Dataset)
		if (this.hasChildDatasets() && !this._childDatasetItems) {
			this._childDatasetItems = [];
			this._childDatasetItemsById = {};
			
			var childLength = this.getChildDatasetCount();
			for (var childIndex = 0;  childIndex < childLength;  ++childIndex) {
				var childDatasetItem = new ibm.tivoli.navmodel.ChildDatasetItem({childDataset: this._dataset.childDatasets[childIndex], dataset: this});
				this._childDatasetItems[childIndex] = childDatasetItem;
				this._childDatasetItemsById[childDatasetItem.getId()] = childDatasetItem;
			}
		}
		
		return this._childDatasetItems;
	},

	getChildDatasetItem: function(/*String*/ childDatasetId) {
		// summary:
		//		Returns requested ibm.tivoli.navmodel.ChildDatasetItem placeholder item (i.e. not a full Dataset)
		var childDatasetItem = null;
		
		// have child datasets
		if (this.hasChildDatasets()) {
			if (!this._childDatasetItems) {
				this.getChildDatasetItems();
			}
			
			childDatasetItem = this._childDatasetItemsById[childDatasetId];			
		}
		
		return childDatasetItem;
	},

	getDescendentDataset: function(/*String*/ datasetId) {
		// summary:
		//		Return requested descendent Dataset (might be multiple generations)
		var descendent = null;
		
		// check cache
		if (this._descendentsById) {
			descendent = this._descendentsById[datasetId];
			if (descendent) {
				return descendent;
			}
		}
				
		if (this.hasChildDatasets()) {
			descendent = this.getChildDataset(datasetId);
			if (!descendent) {
				var childDatasets = this.getChildDatasets();			
				var childLength = childDatasets.length;
				for (var childIndex = 0;  childIndex < childLength;  ++childIndex) {
					descendent = childDatasets[childIndex].getDescendentDataset(datasetId);
					if (descendent) {
						break;
					}
				}
			}
		}
		
		// cache for next time
		if (descendent) {
			if (!this._descendentsById) {
				this._descendentsById = {};
			}
			this._descendentsById[datasetId] = descendent;
		}
		
		return descendent;
	},
	
	/*=====
	tasks_onComplete: function(tasks, kwArgs){},
	=====*/
	
	/*=====
	getTasks_kwArgs: function(onComplete, onError, taskCategory, selection, itemIds, linkIds, itemIndexes, sync) {
		// onComplete: tasks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskCategory: String
		//		comma delimited list indicating type of tasks to fetch
		//		legal values are: menu, toolbar, and breadcrumb
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds, String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getTasks: function(/*getTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
		var args = {
			onComplete: function(tasks, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			taskCategory: "menu",
			selection: null,
			itemIds: null,
			linkIds: null,
			itemIndexes: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// asking for "toolbar" without any selection and we previously
		// fetched the toolbar tasks with the items and links
		if (args.taskCategory === "toolbar" && this._toolbar &&
			(!args.selection || args.selection.isEmpty()) && !args.itemIds && !args.linkIds && !args.itemIndexes) {
			if (args.sync) {
				args.onComplete(this._toolbar, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._toolbar, args), 0);
			}		
		}		
		// fetch tasks from rest service
		else {
			this._fetchTasks(args);
		}
	},

	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, taskId, taskCategory, selection, sync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskId: String
		//		navigation task id
		// taskCategory: String
		//		navigation task category
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
		var args = {                
			onComplete: function(dataset, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},                
			taskId: null,
			taskCategory: "menu:navigation",
			selection: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		this._navigate(args);     
	},

	/*=====
	itemStyles_onComplete: function(itemStyles, kwArgs){},
	=====*/
	
	/*=====
	getItemStyles_kwArgs: function(onComplete, onError, start, count, widgetDef, sync) {
		// onComplete: itemStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of item styles to fetch;  null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItemStyles: function(/*getItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
		var args = {
			onComplete: function(itemStyles, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// already have the item styles
		if (this._itemStyles) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeItemStyles(this._itemStyles, args.widgetDef), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeItemStyles(this._itemStyles, args.widgetDef), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(this._itemStyles, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._itemStyles, args), 0);
			}			
		}
		// need to fetch the item styles
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(itemStyles, args) {
				// cache styles
				if (args.start === 0 && args.count === null) {
					this._cacheItemStyles(itemStyles);					
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeItemStyles(itemStyles, args.widgetDef), args);
				}
				else {
					onComplete(itemStyles, args);
				}				
			});			
			this._fetchItemStyles(args);
		}
	},
	
	/*=====
	getItemStyle_kwArgs: function(itemType, widgetDef) {
		// itemType: String
		//		type of itemStyle
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/

	getItemStyle: function(/*getItemStyle_kwArgs*/ kwArgs) {
		// summary:
		//		Returns item style for requested itemType
		var itemStyle = null;
		var args = {
			itemType: "",
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if we don't have item styles fetch them
		if (!this._itemStyles) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(itemStyles, args){
					this._cacheItemStyles(itemStyles);					
				}),
				onError: function(error, exception, ioArgs){
				},
				start: 0,
				count: null,
				sync: true
			};
			this._fetchItemStyles(fetchArgs);
		}
		
		// locate requested item style in cache
		if (this._itemStylesById) {
			itemStyle = this._itemStylesById[args.itemType];
		}
		
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (itemStyle) {
				itemStyle = itemStyle.clone(args.widgetDef);					
			}
			// unknown style to dataset; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getItemStyle(args.itemType);
				if (widgetStyle) {
					itemStyle = this._createItemStyle(args.widgetDef, widgetStyle);
				}					
			}
		}
		
		return itemStyle;
	},
	
	/*=====
	linkStyles_onComplete: function(linkStyles, kwArgs){},
	=====*/
	
	/*=====
	getLinkStyles_kwArgs: function(onComplete, onError, start, count, widgetDef, sync) {
		// onComplete: linkStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of link styles to fetch; null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getLinkStyles: function(/*getLinkStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current link styles for the Dataset
		var args = {
			onComplete: function(linkStyles, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// already have the link styles
		if (this._linkStyles) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeLinkStyles(this._linkStyles, args.widgetDef, "link"), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeLinkStyles(this._linkStyles, args.widgetDef, "link"), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(this._linkStyles, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._linkStyles, args), 0);
			}
		}
		// need to fetch the link styles
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(linkStyles, args) {
				// cache styles
				if (args.start === 0 && args.count === null) {
					this._cacheLinkStyles(linkStyles);					
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeLinkStyles(linkStyles, args.widgetDef, "link"), args);
				}
				else {
					onComplete(linkStyles, args);
				}				
			});
			this._fetchLinkStyles(args);
		}
	},
	
	/*=====
	getLinkStyle_kwArgs: function(linkType, widgetDef) {
		// linkType: String
		//		type of linkStyle
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/
	
	getLinkStyle: function(/*getLinkStyle_kwArgs*/ kwArgs) {
		// summary:
		//		Returns link style for requested linkType
		var linkStyle = null;
		var args = {
			linkType: "",
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if we don't have link styles fetch them
		if (!this._linkStyles) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(linkStyles, args){
					this._cacheLinkStyles(linkStyles);
				}),
				onError: function(error, exception, ioArgs){
				},
				sync: true
			};
			this._fetchLinkStyles(fetchArgs);
		}
		
		// locate requested link style in cache
		if (this._linkStylesById) {
			linkStyle = this._linkStylesById[args.linkType];
		}
		
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (linkStyle) {
				linkStyle = linkStyle.clone(args.widgetDef, "link");					
			}
			// unknown style to dataset; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getLinkStyle(args.linkType);
				if (widgetStyle) {
					linkStyle = this._createLinkStyle(args.widgetDef, widgetStyle, "link");
				}
			}
		}
				
		return linkStyle;
	},
	
	/*=====
	relStyles_onComplete: function(relStyles, kwArgs){},
	=====*/
	
	/*=====
	getRelationshipStyles_kwArgs: function(onComplete, onError, start, count, widgetDef, sync) {
		// onComplete: relStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationship styles to fetch; null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelationshipStyles: function(/*getRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
		var args = {
			onComplete: function(relStyles, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// already have the relationship styles
		if (this._relStyles) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeLinkStyles(this._relStyles, args.widgetDef, "relationship"), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeLinkStyles(this._relStyles, args.widgetDef, "relationship"), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(this._relStyles, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._relStyles, args), 0);
			}
		}
		// need to fetch the relationship styles
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(relStyles, args) {
				// cache styles
				if (args.start === 0 && args.count === null) {
					this._cacheRelationshipStyles(relStyles);					
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeLinkStyles(relStyles, args.widgetDef, "relationship"), args);
				}
				else {
					onComplete(relStyles, args);
				}				
			});
			this._fetchRelationshipStyles(args);
		}
	},
	
	/*=====
	getRelationshipStyle_kwArgs: function(relationshipType, widgetDef) {
		// relationshipType: String
		//		type of relationshipStyle
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/

	getRelationshipStyle: function(/*getRelationshipStyle_kwArgs*/ kwArgs) {
		// summary:
		//		Returns relationship style for requested relationshipType
		var relStyle = null;
		var args = {
			relationshipType: "",
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if we don't have relationship styles fetch them
		if (!this._relStyles) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(relStyles, args){
					this._cacheRelationshipStyles(relStyles);
				}),
				onError: function(error, exception, ioArgs){
				},
				sync: true
			};
			this._fetchRelationshipStyles(fetchArgs);
		}
		
		// locate requested relationship style in cache
		if (this._relStylesById) {
			relStyle = this._relStylesById[args.relationshipType];
		}
		
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (relStyle) {
				relStyle = relStyle.clone(args.widgetDef, "relationship");					
			}
			// unknown style to dataset; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getRelationshipStyle(args.relationshipType);
				if (widgetStyle) {
					relStyle = this._createLinkStyle(args.widgetDef, widgetStyle, "relationship");
				}
			}
		}	
		
		return relStyle;
	},
		
	/*=====
	statuses_onComplete: function(statuses, kwArgs){},
	=====*/
	
	/*=====
	getStatuses_kwArgs: function(onComplete, onError, type, start, count, widgetDef, sync) {
		// onComplete: statuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible status values for the object type
		var args = {
			onComplete: function(statuses, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			type: "item",
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		var statuses = null;
		var statusesById = null;
		switch(args.type) {
			case "item":
				statuses = this._itemStatus;
				statusesById = this._itemStatusById;
				break;
			case "link":
				statuses = this._linkStatus;
				statusesById = this._linkStatusById;
				break;
			case "relationship":
				statuses = this._relStatus;
				statusesById = this._relStatusById;			
		}
		
		// already have the status values
		if (statuses) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeStatuses(statuses, args.widgetDef, args.type), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeStatuses(statuses, args.widgetDef, args.type), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(statuses, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, statuses, args), 0);
			}			
		}
		// need to fetch the status values
		else {
			var onComplete = args.onComplete;
			args.onComplete = dojo.hitch(this, function(fetchedStatuses, args) {
				// cache statuses
				if (args.start === 0 && args.count === null) {
					switch(args.type) {
						case "item":
							this._cacheItemStatuses(fetchedStatuses);
							break;
					case "link":
							this._cacheLinkStatuses(fetchedStatuses);
							break;
					case "relationship":
							this._cacheRelationshipStatuses(fetchedStatuses);
					}
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeStatuses(fetchedStatuses, args.widgetDef, args.type), args);
				}
				else {
					onComplete(fetchedStatuses, args);
				}
			});
			this._fetchStatuses(args);
		}
	},
	
	/*=====
	getStatus_kwArgs: function(type, statusId, statusValue, widgetDef) {
		// type: String
		//		object type: "item", "link", "relationship" 
		// statusId: String
		//		id of the status
		// statusValue: int
		//		value of the status
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/

	getStatus: function(/*getStatus_kwArgs*/ kwArgs) {
		// summary:
		//		Returns status for requested status id and value
		var status = null;
		var args = {
			type: "item",
			statusId: "",
			statusValue: 0,
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		var statuses = null;
		switch(args.type) {
			case "item":
				statuses = this._itemStatus;
				break;
			case "link":
				statuses = this._linkStatus;
				break;
			case "relationship":
				statuses = this._relStatus;
		}
		
		// if we don't have statuses fetch them
		if (!statuses) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(fetchedStatuses, args){
					switch(args.type) {
						case "item":
							this._cacheItemStatuses(fetchedStatuses);
							break;
						case "link":
							this._cacheLinkStatuses(fetchedStatuses);
							break;
						case "relationship":
							this._cacheRelationshipStatuses(fetchedStatuses);
					}					
				}),
				onError: function(error, exception, ioArgs){
				},
				type: args.type,
				start: 0,
				count: null,
				sync: true
			};
			this._fetchStatuses(fetchArgs);
		}
		
		var statusesById = null;
		switch(args.type) {
			case "item":
				statusesById = this._itemStatusById;
				break;
			case "link":
				statusesById = this._linkStatusById;
				break;
			case "relationship":
				statusesById = this._relStatusById;			
		}
		
		// locate requested item status in cache
		if (statusesById) {
			var id = args.statusId + "_" + args.statusValue;
			status = statusesById[id];
		}
		
		if (status && args.widgetDef) {
			status = status.clone(widgetDef, args.type);			
		}
		
		return status;
	},
	
	/*=====
	getDatasetStatuses_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override status values
	},
	=====*/

	getDatasetStatuses: function(/*getDatasetStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns ibm.tivoli.navmodel.Status[] objects for the dataset
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (!this._status && this._dataset.status) {
			this._status = [];
			var statusLength = this._dataset.status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				this._status[statusIndex] = new ibm.tivoli.navmodel.Status({status: this._dataset.status[statusIndex], version: this.getVersion()});
			}			
		}
		
		// merge widget definition status
		var status = this._status;
		if (status && args.widgetDef) {
			status = this._status.slice(0);
			var statusLength2 = this._status.length;
			for (var statusIndex2 = 0;  statusIndex2 < statusLength2;  ++statusIndex2) {
				status[statusIndex2] = this._status[statusIndex2].clone(args.widgetDef, "dataset");
			}			
		}
		
		return status;		
	},

	setDatasetStatuses: function(statuses) {
		// summary:
		//		Set ibm.tivoli.navmodel.Status[] objects for the dataset
		this._status = statuses;
	},
	
	/*=====
	cols_onComplete: function(cols, kwArgs){},
	=====*/
	
	/*=====
	getColumns_kwArgs: function(onComplete, onError, type, properties, start, count, metadata, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of column ids; or keyword "current", "default", or "all"
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// metadata: Boolean
		//		indicates if filter metadata should be returned with columns
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	getColumns: function(/*getColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current columns for the Dataset
		//		(i.e. the ones that should currently be dispayed)
		var args = {
			onComplete: function(cols, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			source: "dataset",
			type: "item",
			properties: ["current"],
			start: 0,
			count: null,
			metadata: true,
			sync: false
		};
		dojo.mixin(args, kwArgs);
				
		this._fetchColumns(args);	
	},
	
	/*=====
	setColumns_kwArgs: function(onComplete, onError, columns, type, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// columns: ibm.tivoli.navmodel.Column[]
		//		array of columns to be saved
		// type: String
		//		object type: "item", "link", "relationship" 
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Set list of current columns for the Dataset
		//		(with updated attributes for width, sort, etc)
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			columns: null,
			type: "item",
			sync: false
		};
		dojo.mixin(args, kwArgs);
				
		this._setColumns(args);	
	},
	
	hasWidgetDefs: function() {
		// summary:
		//		Returns true if the dataset defines widget definitions	
		return (this._dataset.widgetDefinitions && dojo.isArray(this._dataset.widgetDefinitions) && this._dataset.widgetDefinitions.length > 0);
	},
	
	/*=====
	getWidgetDef_kwArgs: function() {
		// widgetType: String
		//		the type of widget def (i.e. "navwidget", "topology", "table", etc)
	},
	=====*/
	
	getWidgetDef: function(/*getWidgetDef_kwArgs*/ kwArgs) {
		// summary:
		//		Returns widget definition for the requested widget type
		var args = {
			widgetType: null
		};
		dojo.mixin(args, kwArgs);
		
		return this._getWidgetDef(this._getWidgetDefIndex(args.widgetType));		
	},
	
	getWidgetDefs: function() {
		// summary:
		//		Returns all widget definitions defined by the dataset
		var widgetDefs = null;		
		if (this.hasWidgetDefs()) {
			widgetDefs = [];			
			var length = this._dataset.widgetDefinitions.length;
			for (var index = 0;  index < length;  ++index) {
				var widgetDef = this._getWidgetDef(index);
				if (widgetDef) {
					widgetDefs.push(widgetDef);
				}
			}
		}
		
		return widgetDefs;
	},
	
	/*=====
	item_onComplete: function(item, kwArgs){},
	=====*/
	
	/*=====
	createItem_kwArgs: function(onComplete, onError, item) {
		// onComplete: item_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object
		//		ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/

	createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new item in the dataset
		var args = {
			onComplete: function(item, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			item: null
		};
		dojo.mixin(args, kwArgs);
		
		this._createItem(args);
	},
	
	/*=====
	createItems_kwArgs: function(onComplete, onError, item) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object[]
		//		array of ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/

	createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
		var args = {
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			items: []
		};
		dojo.mixin(args, kwArgs);
		
		this._createItems(args);
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	deleteItem_kwArgs: function(onComplete, onError, itemId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of item to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of item
	},
	=====*/

	deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			itemId: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteItem(args);
	},
	
	/*=====
	deleteItems_kwArgs: function(onComplete, onError, itemIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemIds: String[]
		//		array of item ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of items
	},
	=====*/

	deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			itemIds: [],
			condition: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteItems(args);
	},
	
	/*=====
	link_onComplete: function(link, kwArgs){},
	=====*/
	
	/*=====
	createLink_kwArgs: function(onComplete, onError, link) {
		// onComplete: link_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// link: Object
		//		ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/

	createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
		var args = {
			onComplete: function(link, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			link: null
		};
		dojo.mixin(args, kwArgs);
		
		this._createLink(args);	
	},
	
	/*=====
	createLinks_kwArgs: function(onComplete, onError, links) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// links: Object[]
		//		array of ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/

	createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
		var args = {
			onComplete: function(links, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			links: []
		};
		dojo.mixin(args, kwArgs);
		
		this._createLinks(args);	
	},
	
	/*=====
	deleteLink_kwArgs: function(onComplete, onError, linkId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkId: String
		//		id of link to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of link
	},
	=====*/

	deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			linkId: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteLink(args);
	},
	
	/*=====
	deleteLinks_kwArgs: function(onComplete, onError, linkIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkIds: String[]
		//		array of link ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of links
	},
	=====*/

	deleteLinks: function(/*deleteLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			linkIds: [],
			condition: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteLinks(args);
	},
	
	/*=====
	relationship_onComplete: function(relationship, kwArgs){},
	=====*/
	
	/*=====
	createRelationship_kwArgs: function(onComplete, onError, relationship) {
		// onComplete: relationship_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object
		//		ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
		var args = {
			onComplete: function(relationship, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relationship: null
		};
		dojo.mixin(args, kwArgs);
		
		this._createRelationship(args);	
	},
	
	/*=====
	createRelationships_kwArgs: function(onComplete, onError, relationships) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object[]
		//		array of ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	createRelationships: function(/*createRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
		var args = {
			onComplete: function(relationships, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relationships: []
		};
		dojo.mixin(args, kwArgs);
		
		this._createRelationships(args);	
	},
	
	/*=====
	deleteRelationship_kwArgs: function(onComplete, onError, relationshipId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipId: String
		//		id of relationship to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationship
	},
	=====*/

	deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relatonshipId: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteRelationship(args);
	},
	
	/*=====
	deleteRelationships_kwArgs: function(onComplete, onError, relationshipIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipIds: String[]
		//		array of relationship ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationships
	},
	=====*/

	deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relatonshipIds: [],
			condition: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteRelationships(args);
	},

	clientChangedEvent: function() {
		// summary:
		//		Fire client side "changed" event
		var event = ibm.tivoli.navmodel.ClientEvent.getChangedEvent(this);
		this._eventHandler(event);	
	},

	clientItemsAddedEvent: function(items) {
		// summary:
		//		Fire client side "itemsAdded" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemsAddedEvent(this, items);
		this._eventHandler(event);
	},

	clientItemsRemovedEvent: function(items) {
		// summary:
		//		Fire client side "itemsRemoved" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemsRemovedEvent(this, items);
		this._eventHandler(event);
	},

	clientItemStatusChangedEvent: function(items) {
		// summary:
		//		Fire client side "itemStatusChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemStatusChangedEvent(this, items);
		this._eventHandler(event);	
	},

	clientItemPropertiesChangedEvent: function(items) {
		// summary:
		//		Fire client side "itemPropertiesChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemPropertiesChangedEvent(this, items);
		this._eventHandler(event);	
	},

	clientLinksAddedEvent: function(links) {
		// summary:
		//		Fire client side "linksAdded" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinksAddedEvent(this, items);
		this._eventHandler(event);
	},

	clientLinksRemovedEvent: function(links) {
		// summary:
		//		Fire client side "linksRemoved" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinksRemovedEvent(this, links);
		this._eventHandler(event);
	},

	clientLinkStatusChangedEvent: function(links) {
		// summary:
		//		Fire client side "linkStatusChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinkStatusChangedEvent(this, links);
		this._eventHandler(event);
	},

	clientLinkPropertiesChangedEvent: function(links) {
		// summary:
		//		Fire client side "linkPropertiesChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinkPropertiesChangedEvent(this, links);
		this._eventHandler(event);
	},

	clientLabelChangedEvent: function(newLabel) {
		// summary:
		//		Fire client side "labelChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinkPropertiesChangedEvent(this, newLabel);
		this._eventHandler(event);	
	},

	clientChildDatasetsAddedEvent: function(childDatasetItems) {
		// summary:
		//		Fire client side "childDatasetsAdded" event
		var event = ibm.tivoli.navmodel.ClientEvent.getChildDatasetsAddedEvent(this, childDatasetItems);
		this._eventHandler(event);
	},
	
	clientChildDatasetsRemovedEvent: function(childDatasetItems) {
		// summary:
		//		Fire client side "childDatasetsRemoved" event
		var event = ibm.tivoli.navmodel.ClientEvent.getChildDatasetsRemovedEvent(this, childDatasetItems);
		this._eventHandler(event);
	},
		
	/////////////////////////////
	// protected methods
	/////////////////////////////	
	_initEmptyDataset: function() {
		// summary:
		//		Reset internal dataset to an empty state
		this._version = this._args.version;		
		this._dataset = {};
		this._dataset.providerId = this._args.provider;
		this._dataset.datasourceId = this._args.datasource;
		this._dataset.id = this._args.dataset;
		this._dataset.eventId = this._dataset.id;
		this._dataset.eventsEnabled = false;
		this._dataset.viewType = "graph";		
		this._dataset.itemCount = 0;
		this._dataset.linkCount = 0;
		this._dataset.relationshipCount = 0;
		
		// establish our base dataset uri
		this._dataset.uri  = "/providers/" + encodeURIComponent(this._dataset.providerId);
		this._dataset.uri += (this._dataset.datasourceId !== "" ? ("/datasources/" + encodeURIComponent(this._dataset.datasourceId)) : "");
		this._dataset.uri += "/datasets/" + encodeURIComponent(this._dataset.id);		
	},

	_cleanupDataset: function() {
		// summary:
		//		Clean up by removing listener and attempting a deleteDataset if no other listeners
		
		// unregister from event router
		if (this._eventRouter) {
			var remainingListeners = this._eventRouter.removeListener(this.getEventId());
				
			// ok to delete dataset from REST service cache
			if (remainingListeners === 0) {
				this._deleteDataset();
			}
			
			this._eventRouter = null;
		}
		// no event router;  just do the delete
		else {
			this._deleteDataset();
		}
				
		// destroy child datasets
		if (this._childDatasets) {
			var childLength = this._childDatasets.length;
			for (var childIndex = 0;  childIndex < childLength;  ++childIndex) {
				this._childDatasets[childIndex].destroy();				
			}			
			this._childDatasets = null;
			this._childDatasetsById = null;
			this._childDatasetItems = null;
			this._childDatasetItemsById = null;
		}
		
		// clear all item and link caches
		this._clearAllCaches();		
		this._initEmptyDataset();
	},

	_getWidgetDefIndex: function(/*String*/ widgetType) {
		// summary:
		//		Returns the index into widgetDefinitions array for the requested widgetType		
		if (widgetType && this.hasWidgetDefs()) {
			var length = this._dataset.widgetDefinitions.length;
			for (var index = 0;  index < length;  ++index) {
				var entry = this._dataset.widgetDefinitions[index];
				if (entry) {
					// entry should be in the form of  <widget type>:<widget id>:<widget class>
					var parts = entry.split(":");
					if (parts && parts.length == 3 && parts[0] === widgetType) {
						return index;
					}
				}
			}
		}
		
		return -1;
	},
	
	_getWidgetDef: function(/*int*/ index) {
		// summary:
		//		Return WidgetDef for the requested index
		if (!this._widgetDefs && this.hasWidgetDefs()) {
			this._widgetDefs = [];			
			var widgetDefsLength = this._dataset.widgetDefinitions.length;
			for (var widgetDefIndex = 0;  widgetDefIndex < widgetDefsLength;  ++widgetDefIndex) {
				this._widgetDefs[widgetDefIndex] = null;
			}
		}
		
		if (!this._widgetDefs || index < 0 || index >= this._widgetDefs.length) {
			return null;
		}
		
		if (!this._widgetDefs[index]) {
			var parts = this._dataset.widgetDefinitions[index].split(":");
			var widgetType = parts[0];
			var widgetId = parts[1];
			var widgetClass = parts[2];
				
			var objectClass = dojo.getObject(widgetClass);
			if (objectClass) {
				var args = {
					baseUrl: this._args.baseUrl,
					provider: this.getProviderId(),
					datasource: this.getDatasourceId(),
					widget: widgetType,
					definition: widgetId,
					type: "user",
					version: this.getVersion()
				};					
					
				try {
					this._widgetDefs[index] = new objectClass(args);
				}
				catch(ex) {
					this._trace.error("Exception constructing widget definition: ex=" + ex, "_getWidgetDef");					
				}
			}
			else {
				this._trace.error("Cannot instantiate widget def class=" + widgetClass, "_initWidgetDef");				
			}
		}
			
		return this._widgetDefs[index];		
	},

	_setParentDataset: function(parent) {
		// summary:
		//		Set the parent Dataset object for this child Dataset
		this._parent = parent;
	},

	_setEventRouter: function(eventRouter) {
		// summary:
		//		Associate our dataset with an existing EventRouter object
		if (eventRouter && this.isEventsEnabled()) {
			this._eventRouter = eventRouter;
			this._eventRouter.addListener(this.getEventId(), dojo.hitch(this, this._jsonEventHandler));
		}
	},

	_clearEventRouter: function() {
		// summary:
		//		Clear EventRouter for our dataset
		if (this._eventRouter) {
			this._eventRouter.removeListener(this.getEventId());
			this._eventRouter = null;
			
			if (this._childDatasets) {
				var childLength = this._childDatasets.length;
				for (var childIndex = 0;  childIndex < childLength;  ++childIndex) {
					this._childDatasets[childIndex]._clearEventRouter();
				}									
			}
		}
	},

	_getEventRouter: function() {
		// summary:
		//		Returns current event router
		return this._eventRouter;
	},
	
	_setItemCount: function(itemCount) {
		// summary:
		//		Set internal dataset item count
		this._dataset.itemCount = itemCount;
	},

	_setLinkCount: function(linkCount) {
		// summary:
		//		Set internal dataset link count
		this._dataset.linkCount = linkCount;
	},
	
	_setRelationshipCount: function(relationshipCount) {
		// summary:
		//		Set internal dataset relationship count
		this._dataset.relationshipCount = relationshipCount;
	},

	_clearAllCaches: function() {
		// summary:
		//		Clear all item and link related caches
		this._clearItemCache();
		this._clearLinkCache();
		this._clearRelationshipCache();
		this._clearToolbarCache();
		this._clearItemStyleCache();
		this._clearItemStatusCache();
		this._clearLinkStyleCache();
		this._clearLinkStatusCache();
		this._clearRelationshipStyleCache();
		this._clearRelationshipStatusCache();
		
		if (this._childDatasets) {
			var childLength = this._childDatasets.length;
			for (var childIndex = 0;  childIndex < childLength;  ++childIndex) {
				this._childDatasets[childIndex]._clearEventRouter();
			}	
		}
		
		this._childDatasets = null;
		this._childDatasetsById = null;
		this._childDatasetItems = null;
		this._childDatasetItemsById = null;
		this._descendentsById = null;	
	},

	_clearItemCache: function() {
		// summary:
		//		Clear the item cache
		this._itemsById = {};		
	},
	
	_clearLinkCache: function() {
		// summary:
		//		Clear the link cache
		this._linksById = {};
	},

	_clearRelationshipCache: function() {
		// summary:
		//		Clear the relationship cache
		this._relationshipsById = {};
	},

	_clearToolbarCache: function() {
		// summary:
		//		Clear the "toolbar" cache
		this._toolbar = null;
	},

	_clearItemStyleCache: function() {
		// summary:
		//		Clear item style cache
		this._itemStyles = null;
		this._itemStylesById = null;
	},

	_clearItemStatusCache: function() {
		// summary:
		//		Clear item status cache
		this._itemStatus = null;
		this._itemStatusById = null;
	},

	_clearLinkStyleCache: function() {
		// summary:
		//		Clear link style cache
		this._linkStyles = null;
		this._linkStylesById = null;
	},

	_clearLinkStatusCache: function() {
		// summary:
		//		Clear link status cache
		
		//		protected
		this._linkStatus = null;
		this._linkStatusById = null;
	},

	_clearRelationshipStyleCache: function() {
		// summary:
		//		Clear relationship style cache
		this._relStyles = null;
		this._relStylesById = null;
	},

	_clearRelationshipStatusCache: function() {
		// summary:
		//		Clear relationship status cache
		this._relStatus = null;
		this._relStatusById = null;
	},

	_cacheItems: function(items, resetLazyLoad) {
		// summary:
		//		Cache items; merge if already cached
		if (items) {
			var itemsLength = items.length;
			var hasChildDatasets = this.hasChildDatasets();
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				if (hasChildDatasets && items[itemIndex].getDataset() !== this) {
					var dataset = items[itemIndex].getDataset();
					items[itemIndex] = dataset._cacheItem(items[itemIndex], resetLazyLoad);
				}
				else {
					items[itemIndex] = this._cacheItem(items[itemIndex], resetLazyLoad);
				}
			}
		}
		
		return items;
	},

	_cacheItem: function(item, resetLazyLoad) {
		// summary:
		//		Cache item; merge if already cached
		var cachedItem = null;
		
		if (item) {
			// check item cache		
			cachedItem = this._getCachedItem(item.getId());
			
			// item is already in cache; merge property values
			if (cachedItem) {
				cachedItem.mixin(item, resetLazyLoad);
				
				// reset expiration to (now + interval) since item was just referenced
				if (this.cacheExpires()) {
					cachedItem.resetExpiration();
				}		
			}
			// cache new item
			else {
				this._itemsById[item.getId()] = item;
				cachedItem = item;				
			}
		}
		
		return cachedItem;
	},

	_getCachedItem: function(itemId) {
		// summary:
		//		look up item in cache
		return this._itemsById[itemId];
	},

	_cacheLinks: function(links) {
		// summary:
		//		Cache links; merge if already cached
		if (links) {
			var hasChildDatasets = this.hasChildDatasets();
			var linksLength = links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				if (hasChildDatasets && links[linkIndex].getDataset() !== this) {
					var dataset = links[linkIndex].getDataset();
					links[linkIndex] = dataset._cacheLink(links[linkIndex]);
				}
				else {
					links[linkIndex] = this._cacheLink(links[linkIndex]);
				}
			}
		}
		
		return links;
	},

	_cacheLink: function(link) {
		// summary:
		//		Cache link; merge if already cached
		var cachedLink = null;
		
		if (link) {
			// check link cache		
			cachedLink = this._getCachedLink(link.getId());
			
			// link is already in cache; merge property values
			if (cachedLink) {
				cachedLink.mixin(link);
				
				// reset expiration to (now + interval) since link was just referenced
				if (this.cacheExpires()) {
					cachedLink.resetExpiration();
				}		
			}
			// cache new link
			else {
				this._linksById[link.getId()] = link;
				cachedLink = link;				
			}
		}
		
		return cachedLink;
	},

	_getCachedLink: function(linkId) {
		// summary:
		//		look up link in cache
		return this._linksById[linkId];
	},

	_cacheRelationships: function(relationships) {
		// summary:
		//		Cache relationships; merge if already cached
		if (relationships) {
			var hasChildDatasets = this.hasChildDatasets();
			var relsLength = relationships.length;
			for (var relIndex = 0;  relIndex < relsLength;  ++relIndex) {
				if (hasChildDatasets && relationships[relIndex].getDataset() !== this) {
					var dataset = relationships[relIndex].getDataset();
					relationships[relIndex] = dataset._cacheRelationship(relationships[relIndex]);
				}
				else {
					relationships[relIndex] = this._cacheRelationship(relationships[relIndex]);
				}				
			}
		}
		
		return relationships;
	},

	_cacheRelationship: function(relationship) {
		// summary:
		//		Cache relationship; merge if already cached
		var cachedRelationship = null;
		
		if (relationship) {
			// check relationship cache		
			cachedRelationship = this._getCachedRelationship(relationship.getId());
			
			// relationship is already in cache; merge property values
			if (cachedRelationship) {
				cachedRelationship.mixin(relationship);
				
				// reset expiration to (now + interval) since relationship was just referenced
				if (this.cacheExpires()) {
					cachedRelationship.resetExpiration();
				}			
			}
			// cache new relationship
			else {
				this._relationshipsById[relationship.getId()] = relationship;
				cachedRelationship = relationship;
			}
		}
		
		return cachedRelationship;
	},

	_getCachedRelationship: function(relationshipId) {
		// summary:
		//		look up relationship in cache
		return this._relationshipsById[relationshipId];
	},
	
	_cacheChildDataset: function(rawChildDataset) {
		// summary:
		//		Add a new child dataset into this dataset
		
		// this is the first child dataset
		if (!this._dataset.childDatasets) {
			this._dataset.childDatasets = [];
			this._dataset.childDatasets.push(rawChildDataset);
			this.getChildDatasets();
			this.getChildDatasetItems();
		}
		// this is an additonal child dataset
		else {
			this._dataset.childDatasets.push(rawChildDataset);
					
			// create new child dataset 
			if (this._childDatasets) {
				var childDataset = this._constructChildDataset(rawChildDataset.id);
				this._childDatasets.push(childDataset);
				this._childDatasetsById[rawChildDataset.id] = childDataset;
			}
			
			// create new ibm.tivoli.navmodel.ChildDatasetItem
			if (this._childDatasetItems) {
				var childDatasetItem = new ibm.tivoli.navmodel.ChildDatasetItem({childDataset: rawChildDataset, dataset: this});
				this._childDatasetItems.push(childDatasetItem);
				this._childDatasetItemsById[childDatasetItem.getId()] = childDatasetItem;
			}
		}
	},

	_cacheItemStyles: function(styles) {
		// summary:
		//		Set item style cache
		var stylesById = {};
		if (styles) {
			var stylesLength = styles.length;
			for (var styleIndex = 0;  styleIndex < stylesLength;  ++styleIndex) {
				var style = styles[styleIndex];
				stylesById[style.getType()] = style;
			}
		}
		
		this._itemStylesById = stylesById;
		this._itemStyles = styles;
	},

	_cacheItemStatuses: function(statuses) {
		// summary:
		//		Set item status cache
		var statusesById = {};
		if (statuses) {
			var statusLength = statuses.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				var id = statuses[statusIndex].getId() + "_" + statuses[statusIndex].getValue();
				statusesById[id] = statuses[statusIndex];
			}
		}
		
		this._itemStatusById = statusesById;
		this._itemStatus = statuses;	
	},

	_cacheLinkStyles: function(styles) {
		// summary:
		//		Set link style cache
		var stylesById = {};
		if (styles) {
			var stylesLength = styles.length;
			for (var styleIndex = 0;  styleIndex < stylesLength;  ++styleIndex) {
				var style = styles[styleIndex];
				stylesById[style.getType()] = style;
			}
		}
		
		this._linkStylesById = stylesById;
		this._linkStyles = styles;
	},

	_cacheLinkStatuses: function(statuses) {
		// summary:
		//		Set link status cache
		var statusesById = {};
		if (statuses) {
			var statusLength = statuses.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				var id = statuses[statusIndex].getId() + "_" + statuses[statusIndex].getValue();
				statusesById[id] = statuses[statusIndex];
			}
		}
		
		this._linkStatusById = statusesById;
		this._linkStatus = statuses;
	},

	_cacheRelationshipStyles: function(styles) {
		// summary:
		//		Set relationship style cache
		var stylesById = {};
		if (styles) {
			var stylesLength = styles.length;
			for (var styleIndex = 0;  styleIndex < stylesLength;  ++styleIndex) {
				var style = styles[styleIndex];
				stylesById[style.getType()] = style;
			}
		}
		
		this._relStylesById = stylesById;
		this._relStyles = styles;
	},
	
	_cacheRelationshipStatuses: function(statuses) {
		// summary:
		//		Set relationship status cache
		var statusesById = {};
		if (statuses) {
			var statusLength = statuses.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				var id = statuses[statusIndex].getId() + "_" + statuses[statusIndex].getValue();
				statusesById[id] = statuses[statusIndex];
			}
		}
		
		this._relStatusById = statusesById;
		this._relStatus = statuses;
	},

	_mergeItemStyles: function(itemStyles, widgetDef) {
		// summary:
		//		Merge item styles with any pertinent information from widget definition
		var mergedStyles = itemStyles;
				
		// widget definition was provided
		if (widgetDef) {
			mergedStyles = itemStyles.slice(0);
			
			var styleMap = {};
			var styleLength = itemStyles.length;
			for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
				styleMap[itemStyles[styleIndex].getType()] = itemStyles[styleIndex];
				mergedStyles[styleIndex] = itemStyles[styleIndex].clone(widgetDef);					
			}
			
			// see if any styles are in widget definition that are not known to dataset
			var widgetStyles = widgetDef.getItemStyles();
			if (widgetStyles) {
				styleLength = widgetStyles.length;
				for (styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					var widgetStyle = widgetStyles[styleIndex];
					if (!styleMap[widgetStyle.type]) {
						mergedStyles.push(this._createItemStyle(widgetDef, widgetStyle));
					}
				}
			}			
		}
		
		return mergedStyles;
	},

	_createItemStyle: function(widgetDef, widgetStyle) {
		// summary:
		//		Create an ItemStyle from information in the widget definition
		var styleAttributes = {};
		var widgetDefaultStyle = widgetDef.getDefaultItemStyle();
		if (widgetDefaultStyle) {
			dojo.mixin(styleAttributes, widgetDefaultStyle);
		}
							
		dojo.mixin(styleAttributes, widgetStyle);
					
		return new ibm.tivoli.navmodel.ItemStyle({style: styleAttributes, version: this.getVersion()});
	},

	_mergeLinkStyles: function(linkStyles, widgetDef, objType) {
		// summary:
		//		Merge link styles with any pertinent information from widget definition
		var mergedStyles = linkStyles;
		
		// widget definition was provided
		if (widgetDef) {
			mergedStyles = linkStyles.slice(0);
			
			var styleMap = {};
			var styleLength = linkStyles.length;
			for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
				styleMap[linkStyles[styleIndex].getType()] = linkStyles[styleIndex];
				mergedStyles[styleIndex] = linkStyles[styleIndex].clone(widgetDef, objType);				
			}
			
			// see if any styles are in widget definition that are not known to dataset
			var widgetStyles = widgetDef.getLinkStyles();
			if (widgetStyles) {
				styleLength = widgetStyles.length;
				for (styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					var widgetStyle = widgetStyles[styleIndex];
					if (!styleMap[widgetStyle.type]) {
						mergedStyles.push(this._createLinkStyle(widgetDef, widgetStyle, objType));
					}
				}
			}			
		}
		
		return mergedStyles;
	},

	_createLinkStyle: function(widgetDef, widgetStyle, objType) {
		// summary:
		//		Create a LinkStyle from information in the widget definition
		var styleAttributes = {};
		var widgetDefaultStyle = null;
		switch(objType) {
			case "link":
				widgetDefaultStyle = widgetDef.getDefaultLinkStyle();
				break;
			case "relationship":
				widgetDefaultStyle = widgetDef.getDefaultRelationshipStyle();
		}
		
		if (widgetDefaultStyle) {
			dojo.mixin(styleAttributes, widgetDefaultStyle);
		}
							
		dojo.mixin(styleAttributes, widgetStyle);
					
		return new ibm.tivoli.navmodel.LinkStyle({style: styleAttributes, version: this.getVersion()});
	},

	_mergeStatuses: function(status, widgetDef, objType) {
		// summary:
		//		Merge status with any pertinent information from widget definition
		var mergedStatus = status;
		
		// widget definition was provided
		if (widgetDef) {
			mergedStatus = status.slice(0);
			
			var statusMap = {};
			var statusLength = status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				statusMap[status[statusIndex].getId()] = status[statusIndex];
				mergedStatus[statusIndex] = status[statusIndex].clone(widgetDef, objType);
			}
			
			// see if any status values are in widget definition that are not known to dataset
			var widgetStatuses = null;
			switch(objType) {
				case "item":
					widgetStatuses = widgetDef.getItemStatuses();
					break;
				case "link":
					widgetStatuses = widgetDef.getLinkStatuses();
					break;
				case "relationship":
					widgetStatuses = widgetDef.getRelationshipStatuses();
			}
			
			if (widgetStatuses) {
				statusLength = widgetStatuses.length;
				for (statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					var widgetStatus = widgetStatuses[statusIndex];
					if (!statusMap[widgetStatus.id]) {
						mergedStatus.push(this._createStatus(widgetDef, widgetStatus, objType));
					}
				}
			}
		}
		
		return mergedStatus;
	},

	_createStatus: function(widgetDef, widgetStatus, objType) {
		// summary:
		//		Create an Status from information in the widget definition
		var statusAttributes = {};
		var widgetDefaultStatus = null;
		switch(objType) {
			case "item":
				widgetDefaultStatus = widgetDef.getDefaultItemStatus();
				break;
			case "link":
				widgetDefaultStatus = widgetDef.getDefaultLinkStatus();
				break;
			case "relationship":
				widgetDefaultStatus = widgetDef.getDefaultRelationshipStatus();
		}
		
		if (widgetDefaultStatus) {
			dojo.mixin(statusAttributes, widgetDefaultStatus);
		}
							
		dojo.mixin(statusAttributes, widgetStatus);
					
		return new ibm.tivoli.navmodel.Status({status: statusAttributes, version: this.getVersion()});
	},

	_constructChildDataset: function(childDatasetId) {
		// summary:
		//		Construct and initialized a child ibm.tivoli.navmodel.Dataset object
		var childArgs = this.getArgs();
		childArgs.dataset = childDatasetId;
		childArgs.createEventRouter = false; // will share EventRouter
		childArgs.onComplete = null;
		childArgs.onError = null;
		
		// let sub-class construct the child dataset
		var childDataset = this._newChildDataset(childArgs);
				
		// let child dataset know that we are his parent
		childDataset._setParentDataset(this);
						
		// let the child dataset use the same EventRouter
		childDataset._setEventRouter(this._eventRouter);
		
		// propagate event listener
		if (this._listeners) {
			var listenerLength = this._listeners.length;
			for (var listenerIndex = 0;  listenerIndex < listenerLength;  ++listenerIndex) {
				var listener = this._listeners[listenerIndex];
				childDataset.addEventListener(listener.listenerObj, listener.listenerCallback);
			}					
		}		
		
		return childDataset;
	},
		
	_notifyEventListeners: function(event) {
		// summary:
		//		Pass ibm.tivoli.navmodel.Event to all registered listeners
		if (this._listeners) {
			var listenerLength = this._listeners.length;
			for (var listenerIndex = 0;  listenerIndex < listenerLength;  ++listenerIndex) {
				try {
					var listenerObj = this._listeners[listenerIndex].listenerObj;
					this._listeners[listenerIndex].listenerCallback(listenerObj, event);
				}
				catch (ex) {
					this._trace.error("Exception forwarding eventId=" + event.getId() + " type=" + event.getType() + " ex=" + ex, "_sendEvent");
				}
			}				
		}		
	},
	
	_jsonEventHandler: function(jsonEventObj) {
		// summary:
		//		This is the routine that we register with the EventRouter to handle
		//		raw json events that come from the server
		var event = new ibm.tivoli.navmodel.Event({dataset: this, event: jsonEventObj});
		this._eventHandler(event);
	},

	_eventHandler: function(event) {
		// summary:
		//		Handles ibm.tivoli.navmodel.Event objects
		if (event.getId() === this.getEventId()) {
			if (this._trace.isFine()) {
				this._trace.fine("datasetId=" + this.getId() + " event=" + event.getType() + " eventId=" + event.getId(), "_eventHandler");
			}
			
			switch(event.getType()) {
				case "changed":
					this._handleChangedEvent(event);
					break;
				case "labelChanged":
					this._handleLabelChangedEvent(event);
					break;
				case "statusChanged":
					this._handleStatusChangedEvent(event);
					break;					
				case "itemsAdded":
					this._handleItemsAddedEvent(event);
					break;
				case "itemsRemoved":
					this._handleItemsRemovedEvent(event);
					break;
				case "itemStatusChanged":
					this._handleItemStatusChangedEvent(event);
					break;
				case "itemPropertiesChanged":
					this._handleItemPropertiesChangedEvent(event);
					break;
				case "linksAdded":
					this._handleLinksAddedEvent(event);
					break;
				case "linksRemoved":
					this._handleLinksRemovedEvent(event);
					break;
				case "linkStatusChanged":
					this._handleLinkStatusChangedEvent(event);
					break;
				case "linkPropertiesChanged":
					this._handleLinkPropertiesChangedEvent(event);
					break;
				case "linkChanged":
					this._handleLinkChangedEvent(event);
					break;
				case "relationshipStatusChanged":
					this._handleRelationshipStatusChangedEvent(event);
					break;
				case "relationshipPropertiesChanged":
					this._handleRelationshipPropertiesChangedEvent(event);
					break;
				case "childDatasetsAdded":
					this._handleChildDatasetsAddedEvent(event);
					break;
				case "childDatasetsRemoved":
					this._handleChildDatasetsRemovedEvent(event);
					break;
				case "exception":
					if (this._trace.isFine()) {
						this._trace.fine("	exception msgText=" + event.getException().getMsgText(), "_eventHandler");
					}
					break;
				case "eventRouter":
					if (this._trace.isFine()) {
						this._trace.fine("	eventRouterStatus=" + event.getEventRouterStatus(), "_eventHandler");
					}
			}
			
			this._notifyEventListeners(event);
		}		
	},

	_handleChangedEvent: function(event) {
		// summary:
		//		Handle "changed" event type
		this._clearAllCaches();
		
		var initArgs = {
			optimize: true,
			createEventRouter: event.createEventRouter			
		};
		this._initDataset(initArgs);
	},
	
	_handleLabelChangedEvent: function(event) {
		// summary:
		//		Handle "labelChanged" event type
		this._dataset.label = event._getEventNewLabel();
		
		if (this._trace.isFinest()) {
			this._trace.finest("datasetId=" + this.getId() + " label=" + this._dataset.label, "_handleLabelChangedEvent");
		}
	},

	_handleStatusChangedEvent: function(event) {
		// summary:
		//		Handle "statusChanged" event type
		var eventStatus = event._getEventStatuses();
		if (eventStatus) {		
			var status = [];
			var statusLength = eventStatus.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				status[statusIndex] = new ibm.tivoli.navmodel.Status({status: eventStatus[statusIndex], version: this.getVersion()});
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " status=" + status[statusIndex].getLabel(), "_handleStatusChangedEvent");
				}
			}
			this.setDatasetStatuses(status);
		}
	},

	_handleItemsAddedEvent: function(event) {
		// summary:
		//		Handle "itemsAdded" event type
		
		// get new item count for dataset
		var itemCount = event._getEventDatasetItemCount();
		if (itemCount !== undefined) {
			this._setItemCount(itemCount);
		}
		
		// might have new list of styles
		this._clearItemStyleCache();
			
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItems[itemIndex].id, "_handleItemsAddedEvent");
				}
				
				var eventItem = eventItems[itemIndex];
				var newItem = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});
				newItem = this._cacheItem(newItem);													
				event._addItem(newItem);				
			}					
		}
	},

	_handleItemsRemovedEvent: function(event) {
		// summary:
		//		Handle "itemsRemoved" event type
		
		// get new item count for dataset
		var itemCount = event._getEventDatasetItemCount();
		if (itemCount !== undefined) {
			this._setItemCount(itemCount);
	    	if (itemCount === 0) {
	    		this._clearItemCache();
			}
		}
		
		// might have new list of styles
		this._clearItemStyleCache();
			
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = itemsLength - 1;  itemIndex >= 0;  --itemIndex) {
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItems[itemIndex].id, "_handleItemsRemovedEvent");
				}
				
				var eventItem = eventItems[itemIndex];
				var deletedItem = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});		
				var item = this._getCachedItem(eventItem.id);
				if (item) {
					delete this._itemsById[item.getId()];
					event._addItem(item);
				}
				else {	
					event._addItem(deletedItem);
				}			
			}	
		}
	},

	_handleItemStatusChangedEvent: function(event) {
		// summary:
		//		Handle "itemStatusChanged" event type
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				var eventItem = eventItems[itemIndex];	
				var item = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});		
				item = this._cacheItem(item);					
				event._addItem(item);
				
				if (this._trace.isFinest()) {
					var status = item.getStatuses({widgetDef: null});
					if (status) {
						var statusLength = status.length;
						for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
							this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItem.id + " status=" + status[statusIndex].getLabel(), "_handleItemStatusChangedEvent");
						}					
					}
				}			
			}
		}
	},

	_handleItemPropertiesChangedEvent: function(event) {
		// summary:
		//		Handle "itemPropertiesChanged" event type
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				var eventItem = eventItems[itemIndex];							
				var props = [];
				var properties = eventItem.properties;					
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var property = new ibm.tivoli.navmodel.Property({property: properties[propIndex], dataset: this, objectType: "item"});
						props[propIndex] = property;						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItem.id + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleItemPropertiesChangedEvent");
						}																	
					}
				}
								
				var item = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});				
				item = this._cacheItem(item);					
				event._addItem(item, props);				
			}
		}
	},

	_handleLinksAddedEvent: function(event) {
		// summary:
		//		Handle "linksAdded" event type
		
		// get new link and relationship counts for dataset
		var linkCount = event._getEventDatasetLinkCount();
		var relCount = event._getEventDatasetRelationshipCount();
		if (linkCount !== undefined) {
			this._setLinkCount(linkCount);
		}
		if (relCount !== undefined) {
			this._setRelationshipCount(relCount);
		}
		
		// might have new list of styles
		this._clearLinkStyleCache();
		this._clearRelationshipStyleCache(); 
			
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
		    var eventLinksLength = eventLinks.length;
			for (var linkIndex = 0;  linkIndex < eventLinksLength;  ++linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var newLink = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				newLink = this._cacheLink(newLink);				
				event._addLink(newLink);
				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " linkId=" + newLink.getId(), "_handleLinksAddedEvent");
				}				
			}
		}
	},

	_handleLinksRemovedEvent: function(event) {
		// summary:
		//		Handle "linksRemoved" event type
		
		// get new link count for dataset
		var linkCount = event._getEventDatasetLinkCount();
		var relCount = event._getEventDatasetRelationshipCount();
		if (linkCount !== undefined) {
			this._setLinkCount(linkCount);
			if (linkCount === 0) {
	    		this._clearLinkCache();	    		
			}
		}
		if (relCount !== undefined) {
		 	this._setRelationshipCount(relCount);			
		}
		
		// might have new list of styles
		this._clearLinkStyleCache();
		this._clearRelationshipStyleCache();
		    
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
		    var eventLinksLength = eventLinks.length;
			for (var linkIndex = eventLinksLength - 1;  linkIndex >= 0;  --linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var deletedLink = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				var link = this._getCachedLink(eventLink.id);
				if (link) {
					delete this._linksById[link.getId()];
					event._addLink(link);
				}
				else {	
					event._addLink(deletedLink);
				}
				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " linkId=" + deletedLink.getId(), "_handleLinksRemovedEvent");
				}	
			}
		}
	},
	
	_handleLinkStatusChangedEvent: function(event) {
		// summary:
		//		Handle "linkStatusChanged" event type
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
			var eventLinksLength = eventLinks.length;
			for (var linkIndex = 0;  linkIndex < eventLinksLength;  ++linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var link = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				link = this._cacheLink(link);					
				event._addLink(link);
				
				if (this._trace.isFinest()) {
					var status = link.getStatuses({widgetDef: null});
					if (status) {
						var statusLength = status.length;
						for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
							this._trace.finest("datasetId=" + this.getId() + " linkId=" + link.getId() + " status=" + status[statusIndex].getLabel(), "_handleLinkStatusChangedEvent");
						}					
					}
				}		
			}
		}
	},

	_handleLinkPropertiesChangedEvent: function(event) {
		// summary:
		//		Handle "linkPropertiesChanged" event type
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
			var eventLinksLength = eventLinks.length;
			for (var linkIndex = 0;  linkIndex < eventLinksLength;  ++linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var link = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				var props = [];
				var properties = eventLink.properties;					
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var property = new ibm.tivoli.navmodel.Property({property: properties[propIndex], dataset: this, objectType: "link"});
						props[propIndex] = property;						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.getId() + " linkId=" + link.getId() + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleLinkPropertiesChangedEvent");
						}
					}
				}
				
				link = this._cacheLink(link);					
				event._addLink(link, props);
			}
		}		
	},

	_handleLinkChangedEvent: function(event) {
		// summary:
		//		Handle "linkChanged" event type
		
		// might have new list of styles
		this._clearLinkStyleCache();
		this._clearRelationshipStyleCache();
		    
		var eventLink = event._getEventLink();
		var link = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
		link = this._cacheLink(link);	
		event._addLink(link);
				
		if (this._trace.isFinest()) {
			this._trace.finest("datasetId=" + this.getId() + " linkId=" + link.getId() + " reason=" + event.getLinkChangedReason(), "_handleLinkChangedEvent");
		}						
		
		// affected relationships (reason is "relationshipAdded" or "relationshipRemoved")
		var eventRelationships = event._getEventRelationships();
		if (eventRelationships) {
			var eventRelLength = eventRelationships.length;
			for (var eventRelIndex = 0;  eventRelIndex < eventRelLength;  ++eventRelIndex) {
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: eventRelationships[eventRelIndex], dataset: this});
				event._addRelationship(relationship);
			}
		}
	},

	_handleRelationshipStatusChangedEvent: function(event) {
		// summary:
		//		Handle "relationshipStatusChanged" event type
		var eventRels = event._getEventRelationships();
		if (eventRels) {
			var relLength = eventRels.length;
			for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
				var eventRel = eventRels[relIndex];
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: eventRel, dataset: this});
				relationship = this._cacheRelationship(relationship);
				event._addRelationship(relationship);
				
				if (this._trace.isFinest()) {
					var status = relationship.getStatuses({widgetDef: null});
					if (status) {
						var statusLength = status.length;
						for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
							this._trace.finest("datasetId=" + this.getId() + " relationshipId=" + eventRel.id + " status=" + status[statusIndex].getLabel(), "_handleRelationshipStatusChangedEvent");
						}					
					}
				}
			}
		}		
	},

	_handleRelationshipPropertiesChangedEvent: function(event) {
		// summary:
		//		Handle "relationshipPropertiesChanged" event type
		var eventRels = event._getEventRelationships();
		if (eventRels) {
			var relsLength = eventRels.length;
			for (var relIndex = 0;  relIndex < relsLength;  ++relIndex) {
				var eventRel = eventRels[relIndex];
				var props = [];
				var properties = eventRel.properties;					
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var property = new ibm.tivoli.navmodel.Property({property: properties[propIndex], dataset: this, objectType: "relationship"});
						props[propIndex] = property;							
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.getId() + " relationshipId=" + eventRel.id + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleRelationshipPropertiesChangedEvent");
						}
					}					
				}
					
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: eventRel, dataset: this});
				relationship = this._cacheRelationship(relationship);
				event._addRelationship(relationship, props);				
			}
		}
	},
	
	_handleChildDatasetsAddedEvent: function(event) {
		// summary:
		//		Handle "childDatasetsAdded" event type
		var eventDatasets = event._getEventChildDatasets();
		if (eventDatasets) {
			var eventDatasetsLength = eventDatasets.length;
			for (var datasetIndex = 0;  datasetIndex < eventDatasetsLength;  ++datasetIndex) {
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " childDatastId=" + eventDatasets[datasetIndex].id, "_handleChildDatasetsAddedEvent");
				}
						
				var eventDataset = eventDatasets[datasetIndex];
				var childDatasetItem = this.getChildDatasetItem(eventDataset.id);
				if (!childDatasetItem) {
					this._cacheChildDataset(eventDataset);
					event._addChildDatasetItem(this.getChildDatasetItem(eventDataset.id));
				}
			}
		}
	},
	
	_handleChildDatasetsRemovedEvent: function(event) {
		// summary:
		//		Handle "childDatasetsRemoved" event type
		var eventDatasets = event._getEventChildDatasets();
		if (eventDatasets && this._dataset.childDatasets) {
			var eventDatasetsLength = eventDatasets.length;
			for (var datasetIndex = 0;  datasetIndex < eventDatasetsLength;  ++datasetIndex) {
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " childDatastId=" + eventDatasets[datasetIndex].id, "_handleChildDatasetsRemovedEvent");
				}
				
				var eventDataset = eventDatasets[datasetIndex];				
				var childLength = this._dataset.childDatasets.length;
				for (var childIndex = 0;  childIndex < childLength;  ++childIndex) {
					if (this._dataset.childDatasets[childIndex].id === eventDataset.id) {
						// add childDatasetItem to the event
						var childDatasetItem = this.getChildDatasetItem(eventDataset.id);
						event._addChildDatasetItem(childDatasetItem);
							
						if (this._childDatasets) {
							// destroy child dataset
							var childDataset = this._childDatasets[childIndex];
							childDataset.destroy();
							
							// remove child dataset from cache
							this._childDatasets.splice(childIndex, 1);
							delete this._childDatasetsById[eventDataset.id];											
						}
						
						if (this._descendentsById) {
							delete this._descendentsById[eventDataset.id];
						}
						
						if (this._childDatasetItems) {
							// remove ChildDatasetItem from cache
							this._childDatasetItems.splice(childIndex, 1);
							delete this._childDatasetItemsById[eventDataset.id];							
						}
						
						// remove from "childDatasets" attribute of this._dataset
						this._dataset.childDatasets.splice(childIndex, 1);							
										
						break;
					}
				}
			}
		}
	},
		
	_validateCache: function() {
		// summary:
		//		Check cache for items, links, and relationships that haven't been used for awhile;
		//		remove from cache
		this._trace.entering("starting validate cache", "_validateCache");
		
		this._validate(this._itemsById);
		this._validate(this._linksById);
		this._validate(this._relationshipsById);
		
		this._trace.exiting("ending validate cache", "_validateCache");				
	},
	
	_validate: function(cache) {
		// summary:
		//		Check expiration for supplied cache; items, links, or relationships
		var nowDate = new Date();
		var now = nowDate.getTime();
		
		for (var id in cache) { 
    		if (cache.hasOwnProperty(id)) {
    			var obj = cache[id];
    			if (obj && obj.checkExpiration(now)) {
    				delete cache[id];
    			}
    		}        
		} 		
	},
	
	/////////////////////////////
	// abstract methods
	/////////////////////////////
	
	/*=====
	initDataset_kwArgs: function(createEventRouter) {
		// createEventRouter: Boolean
		//		boolean indicating whether new event router should be created
	},
	=====*/
	
	_initDataset: function(/*initDataset_kwArgs*/ kwArgs) {
		// summary:
		//		Initialize internal dataset object
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// create child datasets now so they are available right away;
		// ran into a timing problem with multiple widgets on the page
		this.getChildDatasets();
		
		// determine if we need to start up the cache interval to purge
		// resources that have not been touched in a while
		if (!this._cacheIntervalId && this.getItemCount() >= this._args.cacheThreshold && this._args.cacheValidationInterval) {
			this._cacheIntervalId = setInterval(dojo.hitch(this, this._validateCache), (this._args.cacheValidationInterval*60000));
		}
	},

	_deleteDataset: function() {
		// summary:
		//		Dataset is no longer in use	
	},
	
	/*=====
	fetchItemsAndLinks_onComplete: function(items, links, datasetItems, toolbar, kwArgs){},
	=====*/
	
	/*=====
	fetchItemsAndLinks_kwArgs: function(onComplete, onError, sync) {
		// onComplete: fetchItemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	_fetchItemsAndLinks: function(/*fetchItemsAndLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items, links, and child dataset items
		//		(ibm.tivoli.navmodel.ChildDatasetItem i.e. not a full Dataset);
		//		associated with the dataset
	},
	
	_fetchItems: function(/*getItems_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch items from rest service
	},

	_fetchItem: function(/*getItem_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual item from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return null;
	},
	
	_fetchOriginItems: function(/*getOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
	},
	
	_fetchRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested item
	},
	
	_fetchChildDatasetRelatedItems: function(/*getChildDatasetRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested child dataset
	},

	_queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
	},

	_fetchLinks: function(/*getLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch links from rest service
	},

	_fetchLink: function(/*getLink_kwArgs*/ linkId) {
		// summary:
		//		Synchronously fetch an individual link from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return null;
	},
		
	_fetchRelationships: function(/*getRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch relationships from rest service
	},

	_fetchRelationship: function(/*getRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual relationship from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return null;
	},

	_fetchItemStyles: function(/*getItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
	},
	
	_fetchLinkStyles: function(/*getLinkStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current link styles for the Dataset
	},
	
	_fetchRelationshipStyles: function(/*getRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
	},
	
	/*=====
	fetchStatuses_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: relStatuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type for columns: "item", "link", "relationship"
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchStatuses: function(/*fetchStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible relationship status values for the Dataset
	},
	
	_fetchTasks: function(/*getTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
	},
	
	_navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
	},
	
	/*=====
	fetchColumns_kwArgs: function(onComplete, onError, type, properties, start, count, metadaa, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of ids in which to fetch columns;
		//		if not specified the "current" columns will be fetched
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// metadata: Boolean
		//		indicates if filter metadata should be returned with columns
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	_fetchColumns: function(/*fetchColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch columns from rest service
	},
	
	_setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Save columns to rest service
	},

	_newChildDataset: function(kwArgs) {
		// summary:
		//		Construct new child dataset object
		return null;
	},

	_createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new item in the dataset
	},
	
	_createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
	},
	
	_deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
	},
	
	_deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
	},
	
	_createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
	},
	
	_createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
	},
	
	_deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
	},
	
	_deleteLinks: function(/*deleteLinks_kwArgs*/  kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
	},

	_createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
	},

	_createRelationships: function(/*createRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
	},
	
	_deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
	},
	
	_deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Dataset"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Dataset"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Dataset"); 
 
// includes


ibm.tivoli.navmodel.Dataset._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Dataset");

////////////////////////////////////
// Dataset
//
// NOTE: do not call any abstract methods directly;
// call the public version of the methods defined in
// AbstractDataset instead and it will in turn call
// the abstract overrides.
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Dataset", ibm.tivoli.navmodel.AbstractDataset, {
	// private members
	_optimize: true,
	_trace: ibm.tivoli.navmodel.Dataset._trace, 
	
	/*=====
	constructor_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	constructor_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	 constructor_kwArgs: function(onComplete, onError, baseUrl, provider, datasource, dataset, 
	 					parameters, defaultProperties, defaultLinkProperties, defaultRelProperties, 
	 					cacheThreshold, cacheValidationInterval, cacheExpirationTime, version) {
		// onComplete: constructor_onComplete
	 	//		callback to be invoked when the Dataset is initialized and ready for methods
	 	//		to be invoked  (i.e. asynchronous initialization must be performed). 
	 	//		Note: A null value will result in synchronous initialization.
	 	// onError: constructor_onError
	 	//		callback to be invoked if there is a problem constructing the Dataset
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// dataset: String
		//		dataset id
		// parameters: Object
		//		optional: extra parameters sent to dataset provider
		// defaultProperties: String[]
		//		optional: item property ids to fetch by default (can specify "all")
		// defaultLinkProperties: String[]
		//		optional: link property ids to fetch by default (can specify "all")
		// defaultRelProperties: String[]
		//		optional: relationship property ids to fetch by default (can specify "all")
		// cacheThreshold: int[]
		//		optional: cache objects if number of dataset items >= threshold
		// cacheValidationInterval: int[]
		//		optional: number of minutes in between cache validations
		// cacheExpirationTime: int[]
		//		optional: number of minutes objects can stay in cache while inactive
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	},
	=====*/

	constructor: function(/*constructor_kwArgs_kwArgs*/ kwArgs, dataset) {	
		// summary: 
		//		Constructs Dataset object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception		
	},

	//////////////////////////////////////////////////////
	// abstract method overrides
	//
	// NOTE: do not call any abstract methods directly;
	// call the public version of the methods defined in
	// AbstractDataset instead and it will in turn call
	// the abstract overrides.
	//////////////////////////////////////////////////////

	/*=====
	initDataset_kwArgs: function(createEventRouter, optimize) {
		// createEventRouter: Boolean
		//		boolean indicating whether new event router should be created
		// optimize: Boolean
		//		optimize json that is returned; remove items that can be calculated
		//		or can be retrieved via additional urls
	},
	=====*/
	
	_initDataset: function(/*initDataset_kwArgs*/ kwArgs) {
		// summary:
		//		Initialize internal dataset object
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// start out with empty dataset
		this._initEmptyDataset();		
		this._optimize = kwArgs.optimize;
		
		// make synchronous call if no onComplete callback
		var synchronous = (kwArgs.onComplete ? false : true);
		var args = arguments;
		var exception = null;
		
		// dataset was NOT provided; fetch it
		if (!kwArgs.dataset) {			
			var content = ibm.tivoli.navmodel.Utils.getContent();
			this._mixinStandard(content, kwArgs);			
			ibm.tivoli.navmodel.Utils.mixinParams(content, this);
			
			dojo.xhrGet({
				url: this._getDatasetURL(),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset")
				},
				handleAs: "json",
				sync: synchronous,
				content: content,
				load: dojo.hitch(this, function(result, ioArgs) {
					// pull version from the Content-Type response header
					this._setVersion(this._extractContentTypeVersion(ioArgs));
					
					// mixin all dataset attributes
					this.mixinAttributes(result);
					
					// listen for asynchronous events
					if (kwArgs.createEventRouter && this.isEventsEnabled()) {
						this._createEventRouter();
					}
		
					// call base class
					this.inherited(args);
					
					// call onComplete callback
					if (kwArgs.onComplete) {
						kwArgs.onComplete(this, kwArgs);
					}							
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					exception = this._getException(error, ioArgs, "_initDataset");					
					if (kwArgs.onError) {
						kwArgs.onError(error, exception, ioArgs);
					}						
				})
			});	
			
			if (synchronous && exception) {
				throw exception;
			}						
		}
		
		// dataset was provided
		else {
			this.mixinAttributes(kwArgs.dataset);
			
			// listen for asynchronous events
			if (kwArgs.createEventRouter && this.isEventsEnabled()) {
				this._createEventRouter();
			}
		
			// call base class
			this.inherited(args);			
		}
	},
	
	_createEventRouter: function() {
		// summary:
		//		create event router object and set on the base class
		var baseUrl = this.getArg("baseUrl");
		var contextRoot = baseUrl.substring(0, baseUrl.lastIndexOf("/"));
		var eventRouterArgs = { 
			eventDestination: this.getAttribute("eventDestination"),
			contextRoot: contextRoot
		};		
		var eventRouter = new ibm.tivoli.navmodel.EventRouter(eventRouterArgs);
			
		this._setEventRouter(eventRouter);		
	},

	_deleteDataset: function() {
		// summary:
		//		Dataset is no longer in use
		if (this._trace.isFinest()) {
			this._trace.finest("sending DELETE for dataset=" + this.getId(), "_deleteDataset");
		}
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrDelete({
			url: this._getDatasetURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){			
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				this._getException(error, ioArgs, "_deleteDataset");
			})
		});
	},
	
	

	/*=====
	fetchItemsAndLinks_onComplete: function(items, links, datasetItems, toolbar, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	fetchItemsAndLinks_kwArgs: function(onComplete, onError, sync) {
		// onComplete: fetchItemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchItemsAndLinks: function(/*fetchItemsAndLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items, links, and child dataset items
		//		(ibm.tivoli.navmodel.ChildDatasetItem  i.e. not a full Dataset);
		//		associated with the dataset
		var defaultProperties = this.getArg("defaultProperties");
		var defaultLinkProperties = this.getArg("defaultLinkProperties");
		var defaultRelProperties = this.getArg("defaultRelProperties");
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "itemProperties", defaultProperties);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "linkProperties", defaultLinkProperties);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "relationshipProperties", defaultRelProperties);		
		content.optimize = this._optimize;
		content.items = true;
		content.links = true;
		content.toolbarTasks = true;
		
		return dojo.xhrGet({
			url: this._getDatasetURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];				
				var links = [];
				var toolbar = [];
				var styles = null;
				var status = null;
				var length;
				var index;
				var version = this.getVersion();
				
				this._setItemCount(result.itemCount);
				this._setLinkCount(result.linkCount);
				this._setRelationshipCount(result.relationshipCount);
										
				if (result.itemStyles) {
					styles = [];
					length = result.itemStyles.length;
					for (index = 0;  index < length;  ++index) {
						styles[index] = new ibm.tivoli.navmodel.ItemStyle({style: result.itemStyles[index], version: version});
					}
					this._cacheItemStyles(styles);
				}
				
				if (result.itemStatus) {
					status = [];
					length = result.itemStatus.length;
					for (index = 0;  index < length;  ++index) {
						status[index] = new ibm.tivoli.navmodel.Status({status: result.itemStatus[index], version: version});
					}
					this._cacheItemStatuses(status);
				}
				
				if (result.items) {
					var allProps = this._isAllProps(defaultProperties);					
					var propIds = this._getPropIds(defaultProperties, allProps);
					length = result.items.length;
					for (index = 0;  index < length;  ++index) {
						items[index] = new ibm.tivoli.navmodel.Item({item: result.items[index], dataset: this, propIds: propIds, allProperties: allProps, expiration: expiration});
					}
				}
				
				if (result.linkStyles) {
					styles = [];
					length = result.linkStyles.length;
					for (index = 0;  index < length;  ++index) {
						styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.linkStyles[index], version: version});
					}
					this._cacheLinkStyles(styles);
				}
				
				if (result.relationshipStyles) {
					styles = [];
					length = result.relationshipStyles.length;
					for (index = 0;  index < length;  ++index) {
						styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.relationshipStyles[index], version: version});
					}
					this._cacheRelationshipStyles(styles);
				}
				
				if (result.linkStatus) {
					status = [];
					length = result.linkStatus.length;
					for (index = 0;  index < length;  ++index) {
						status[index] = new ibm.tivoli.navmodel.Status({status: result.linkStatus[index], version: version});
					}
					this._cacheLinkStatuses(status);
				}
				
				if (result.relationshipStatus) {
					status = [];
					length = result.relationshipStatus.length;
					for (index = 0;  index < length;  ++index) {
						status[index] = new ibm.tivoli.navmodel.Status({status: result.relationshipStatus[index], version: version});
					}
					this._cacheRelationshipStatuses(status);
				}
				
				if (result.links) {
					var allLinkProps = this._isAllProps(defaultLinkProperties);
					var linkPropIds = this._getPropIds(defaultLinkProperties, allLinkProps);
					length = result.links.length;
					for (index = 0;  index < length;  ++index) {
						links[index] = new ibm.tivoli.navmodel.Link({link: result.links[index], dataset: this, propIds: linkPropIds, allProperties: allLinkProps, expiration: expiration});
					}
				}
				
				if (result.toolbarTasks) {
					length = result.toolbarTasks.length;
					for (index = 0;  index < length;  ++index) {
						toolbar[index] = new ibm.tivoli.navmodel.Task({task: result.toolbarTasks[index], dataset: this});
					}
				}
				
				kwArgs.onComplete(items, links, this.getChildDatasetItems(), toolbar, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItemsAndLinks"), ioArgs);
			})
		});
	},

	/*=====
	items_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	fetchItems_kwArgs: function(onComplete, onError, recursive, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// recursive: boolean
		//		also fetch all childDataset links if true 
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchItems: function(/*fetchItems_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch items from rest service
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getItemsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var items = [];				
				if (result.items) {
					var expiration = this.getNewExpirationTime();
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						var item = result.items[itemIndex];
						if (item.datasetId && item.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(item.datasetId);
							items[itemIndex] = new ibm.tivoli.navmodel.Item({item: item, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});							
						}
						else {
							items[itemIndex] = new ibm.tivoli.navmodel.Item({item: item, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
					}
				}
				
				// make sure our count is accurate
				if (!this.hasChildDatasets()) {
					this._setItemCount(result.totalRows);
				}
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				kwArgs.onComplete(items, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItems"), ioArgs);
			})
		});
	},
	
	/*=====
	fetchItem_kwArgs: function(itemId, properties, metadata) {
		// itemId: String
		//		item id to get
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchItem: function(/*fetchItem_kwArg*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual item from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var item = null;
		var exception = null;
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		dojo.xhrGet({
			url: this._getItemURL(kwArgs.itemId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var allProps = this._isAllProps(kwArgs.properties);
				var propIds = this._getPropIds(kwArgs.properties, allProps);
				item = new ibm.tivoli.navmodel.Item({item: result, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata});											
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_fetchItem");
			})
		});
		
		if (exception) {
			throw exception;
		}
				
		return item;
	},

	/*=====
	fetchOriginItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metdata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchOriginItems: function(/*fetchOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getOriginItemsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "originItemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();				
				var items = [];
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
					}
				}
				
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				
				kwArgs.onComplete(items, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchOriginItems"), ioArgs);
			})
		});
	},

	/*=====
	itemsAndLinks_onComplete: function(items, links, datasetItems, kwArgs){},
	=====*/
	
	/*=====
	fetchRelatedItems_kwArgs: function(onComplete, onError, itemId, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of the base item
		// relationships: String
		//		comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelatedItems: function(/*fetchRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested item
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		// relationships to explore
		if (kwArgs.relationships) {
			content.relationships = kwArgs.relationships;
		}
		
		return dojo.xhrGet({
			url: this._getRelatedItemsURL(kwArgs.itemId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relatedItemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];
				if (result.items) {
					if (result.datasetItemCount !== undefined) {
						this._setItemCount(result.datasetItemCount);
					}
					
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});							
					}
				}
				
				var links = [];
				if (result.links) {
					if (result.datasetLinkCount !== undefined) {
						this._setLinkCount(result.datasetLinkCount);
					}
					
					if (result.datasetRelationshipCount !== undefined) {
						this._setRelationshipCount(result.datasetRelationshipCount);
					}					
					
					var linksLength = result.links.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						links[linkIndex] = new ibm.tivoli.navmodel.Link({link: result.links[linkIndex], dataset: this, expiration: expiration});
					}
				}
				
				var datasetItems = [];
				if (result.childDatasets) {
					var childDatasetsLength = result.childDatasets.length;
					for (var childIndex = 0;  childIndex < childDatasetsLength;  ++childIndex) {
						var resultDataset = result.childDatasets[childIndex];
						var childDataset  = this.getChildDataset(resultDataset.id);
						
						// child dataset does not exist yet so create it;
						if (!childDataset) {
							this._cacheChildDataset(resultDataset);								
						}
						
						var childDatasetItem = this.getChildDatasetItem(resultDataset.id);
						datasetItems.push(childDatasetItem);					
					}
				}
				
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				
				kwArgs.onComplete(items, links, datasetItems, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelatedItems"), ioArgs);
			})
		});
	},
	
	/*=====
	fetchChildDatasetRelatedItems_kwArgs: function(onComplete, onError, childDatasetId, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// childDatasetId: String
		//		id of the base child dataset
		// relationships: String
		//		comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchChildDatasetRelatedItems: function(/*fetchChildDatasetRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested child dataset
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		// relationships to explore
		if (kwArgs.relationships) {
			content.relationships = kwArgs.relationships;
		}
		
		return dojo.xhrGet({
			url: this._getChildDatasetRelatedItemsURL(kwArgs.childDatasetId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relatedItemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];
				if (result.items) {
					if (result.datasetItemCount !== undefined) {
						this._setItemCount(result.datasetItemCount);
					}
					
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});							
					}
				}
				
				var links = [];
				if (result.links) {
					if (result.datasetLinkCount !== undefined) {
						this._setLinkCount(result.datasetLinkCount);
					}
					
					if (result.datasetRelationshipCount !== undefined) {
						this._setRelationshipCount(result.datasetRelationshipCount);
					}					
					
					var linksLength = result.links.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						links[linkIndex] = new ibm.tivoli.navmodel.Link({link: result.links[linkIndex], dataset: this, expiration: expiration});
					}
				}
				
				var datasetItems = [];
				if (result.childDatasets) {
					var childDatasetsLength = result.childDatasets.length;
					for (var childIndex = 0;  childIndex < childDatasetsLength;  ++childIndex) {
						var resultDataset = result.childDatasets[childIndex];
						var childDataset  = this.getChildDataset(resultDataset.id);
						
						// child dataset does not exist yet; so create it
						if (!childDataset) {
							this._cacheChildDataset(resultDataset);								
						}
						
						var childDatasetItem = this.getChildDatasetItem(resultDataset.id);
						datasetItems.push(childDatasetItem);						
					}
				}
				
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				
				kwArgs.onComplete(items, links, datasetItems, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelatedItems"), ioArgs);
			})
		});
	},
	
	/*=====
	queryItems_kwArgs: function(onComplete, onError, query, start, count, properties, metadata, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// query: Object
		//		custom query object to be sent as request body
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	 _queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
		var json = dojo.toJson(kwArgs.query, false);
				
		return dojo.xhrPost({
			url: this._getItemsQueryURL(kwArgs),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "customQuery"),
				'Content-Length': json.length,
				'X-Method-Override': 'Custom-GET'
			},
			handleAs: "json",
			postData: json,
			sync: kwArgs.sync,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];				
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);				
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
					}
				}
				
				kwArgs.onComplete(items, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_queryItems"), ioArgs);
			})
		});
	},
	
	/*=====
	links_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	fetchLinks_kwArgs: function(onComplete, onError, recursive, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// recursive: boolean
		//		also fetch all childDataset items if true 
		// start: int
		//		start index to fetch
		// count: int
		//		number of links to fetch; null means all
		// properties: String[]
		//		link property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchLinks: function(/*fetchLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch links from rest service
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getLinksURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var links = [];							
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var linksLength = result.items.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						var link = result.items[linkIndex];
						if (link.datasetId && link.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(link.datasetId);
							links[linkIndex] = new ibm.tivoli.navmodel.Link({link: link, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
						else {									
							links[linkIndex] = new ibm.tivoli.navmodel.Link({link: link, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
					}
				}
				
				// make sure our count is accurate
				if (!this.hasChildDatasets()) {
					this._setLinkCount(result.totalRows);
				}
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;				
				kwArgs.onComplete(links, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchLinks"), ioArgs);
			})
		});
	},
	
	/*=====
	fetchLink_kwArgs: function(itemId, properties, metadata) {
		// linkId: String
		//		link id to get
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchLink: function(/*fetchLink_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual link from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var link = null;
		var exception = null;
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		dojo.xhrGet({
			url: this._getLinkURL(kwArgs.linkId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var allProps = this._isAllProps(kwArgs.properties);
				var propIds = this._getPropIds(kwArgs.properties, allProps);
				link = new ibm.tivoli.navmodel.Link({link: result, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata});			
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_fetchLink");
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return link;
	},

	/*=====
	relationships_onComplete: function(relationships, kwArgs){},
	=====*/
	
	/*=====
	fetchRelationships_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationships to fetch; null means all
		// properties: String[]
		//		relationship property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelationships: function(/*fetchRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch relationships from rest service
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getRelationshipsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var relationships = [];				
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var relLength = result.items.length;
					for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
						var relationship = result.items[relIndex];
						if (relationship.datasetId && relationship.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(relationship.datasetId);
							relationships[relIndex] = new ibm.tivoli.navmodel.Relationship({relationship: relationship, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
						else {									
							relationships[relIndex] = new ibm.tivoli.navmodel.Relationship({relationship: relationship, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}						
					}
				}
								
				// make sure our count is accurate
				if (!this.hasChildDatasets()) {
					this._setRelationshipCount(result.totalRows);
				}
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;				
				kwArgs.onComplete(relationships, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelationships"), ioArgs);
			})
		});
	},

	/*=====
	fetchRelationship_kwArgs: function(relationshipId, properties, metadata) {
		// relationshipId: String
		//		relationship id to fetch
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchRelationship: function(/*fetchRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual relationship from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var relationship = null;
		var exception = null;
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		dojo.xhrGet({
			url: this._getRelationshipURL(kwArgs.relationshipId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var allProps = this._isAllProps(kwArgs.properties);
				var propIds = this._getPropIds(kwArgs.properties, allProps);
				relationship = new ibm.tivoli.navmodel.Relationship({relationship: result, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata});			
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_fetchRelationship");
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return relationship;
	},
	
	/*=====
	itemStyles_onComplete: function(itemStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchItemStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: itemStyle_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of item styles to fetch;  null means all
		// sync: Boolean
		//		indicates if fetch is synchronous		
	},
	=====*/
	
	_fetchItemStyles: function(/*fetchItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrGet({
			url: this._getItemStylesURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemStyleList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var styles = [];
				var stylesLength = result.items.length;
				for (var index = 0;  index < stylesLength;  ++index) {
					styles[index] = new ibm.tivoli.navmodel.ItemStyle({style: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(styles, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItemStyles"), ioArgs);
			})
		});
	},

	/*=====
	linkStyles_onComplete: function(linkStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchLinkStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: linkStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of link styles to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchLinkStyles: function(/*fetchLinkStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current link styles for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrGet({
			url: this._getLinkStylesURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkStyleList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var styles = [];
				var stylesLength = result.items.length;
				for (var index = 0;  index < stylesLength;  ++index) {
					styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(styles, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchLinkStyles"), ioArgs);
			})
		});		
	},

	/*=====
	relStyles_onComplete: function(relStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchRelationshipStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: relStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationship styles to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelationshipStyles: function(/*fetchRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrGet({
			url: this._getRelationshipStylesURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkStyleList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var styles = [];
				var stylesLength = result.items.length;
				for (var index = 0;  index < stylesLength;  ++index) {
					styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(styles, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelationshipStyles"), ioArgs);
			})
		});		
	},

	/*=====
	itemStatuses_onComplete: function(itemStatuses, kwArgs){},
	=====*/
	
	/*=====
	fetchStatuses_kwArgs: function(onComplete, onError, type, start, count, sync) {
		// onComplete: itemStatuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type for columns: "item", "link", "relationship"
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchStatuses: function(/*fetchStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible item status values for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);		
		if (kwArgs.type) {
			content.type = kwArgs.type;
		}
		
		return dojo.xhrGet({
			url: this._getStatusURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "statusList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var status = [];
				var statusLength = result.items.length;
				for (var index = 0;  index < statusLength;  ++index) {
					status[index] = new ibm.tivoli.navmodel.Status({status: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(status, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchStatuses"), ioArgs);
			})
		});
	},

	/*=====
	tasks_onComplete: function(tasks, kwArgs){},
	=====*/
	
	/*=====
	fetchTasks_kwArgs: function(onComplete, onError, taskCategory, selection, itemIds, linkIds, itemIndexes, sync) {
		// onComplete: tasks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskCategory: String
		//		comma delimited list indicating type of tasks to fetch
		//		legal values are: menu, toolbar, and breadcrumb
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds, String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchTasks: function(/*fetchTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
		
		// use "selection"
		if (kwArgs.selection) {
			var json = dojo.toJson(kwArgs.selection.getAttributes(), false);
			
			return dojo.xhrPost ({
				url: this._getTasksParamURL(kwArgs),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskList"),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskContext"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: kwArgs.sync,			
				load: dojo.hitch(this, function(result, ioArgs) {
					var tasks = [];
					var rawTasks = result.items;
					var rawTasksLength = rawTasks.length;
					for (var index = 0;  index < rawTasksLength;  ++index) {
						tasks[index] = new ibm.tivoli.navmodel.Task({task: rawTasks[index], dataset: this});
					}
					kwArgs.onComplete(tasks, kwArgs);
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					kwArgs.onError(error, this._getException(error, ioArgs, "getTasks"), ioArgs);	               
				})
			});
		}
		
		// use "itemIds", "linkIds", and "itemIndexes"
		else {
			// need original "param_" values to locate cached dataset
			var content = ibm.tivoli.navmodel.Utils.getContent();
			this._mixinStandard(content, kwArgs);
			ibm.tivoli.navmodel.Utils.mixinParams(content, this);
			
			// comma delimitted categories (i.e. "menu,toolbar,breadcrumb")
			content.category = kwArgs.taskCategory;

			// indicate what is selected
			if (kwArgs.itemIds) {
				content.itemIds = kwArgs.itemIds;
			}
			if (kwArgs.linkIds) {
				content.linkIds = kwArgs.linkIds;
			}
			if (kwArgs.itemIndexes) {
				content.itemIndexes = kwArgs.itemIndexes;
			}

			return dojo.xhrGet({
				url: this._getTasksURL(),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskList")
				},
				handleAs: "json",
				sync: kwArgs.sync,
				content: content,
				load: dojo.hitch(this, function(result, ioArgs){
					var tasks = [];
					var rawTasks = result.items;
					var rawTasksLength = rawTasks.length;
					for (var index = 0;  index < rawTasksLength;  ++index) {
						tasks[index] = new ibm.tivoli.navmodel.Task({task: rawTasks[index], dataset: this});
					}
					kwArgs.onComplete(tasks, kwArgs);
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					kwArgs.onError(error, this._getException(error, ioArgs, "getTasks()"), ioArgs);
				})
			});
		}
	},

	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, taskId, taskCategory, selection, sync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskId: String
		//		navigation task id
		// taskCategory: String
		//		navigation task category
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
				
		// use "selection"
		var json = dojo.toJson(kwArgs.selection.getAttributes(), false);
			
		return dojo.xhrPost ({
			url: this._getNavigateParamURL(kwArgs),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskContext"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: kwArgs.sync,			
			load: dojo.hitch(this, function(result, ioArgs) {
				var navigateArgs = this.getArgs();
				navigateArgs.dataset = result.id;
				navigateArgs.parameters = {};
				navigateArgs.defaultProperties = [];
				navigateArgs.createEventRouter = false; // will share EventRouter
				navigateArgs.onComplete = null;
				navigateArgs.onError = null;
						
		  		// construct navigation dataset (share EventRouter)
				var dataset = new ibm.tivoli.navmodel.Dataset(navigateArgs, result);
				dataset._setEventRouter(this._getEventRouter());
			
				// pass new dataset back to caller
				kwArgs.onComplete(dataset, kwArgs);		
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				kwArgs.onError(error, this._getException(error, ioArgs, "navigate"), ioArgs);	               
			})
		});			      
	},

	/*=====
	cols_onComplete: function(cols, kwArgs){},
	=====*/
	
	/*=====
	fetchColumns_kwArgs: function(onComplete, onError, type, properties, start, count, metadata, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of column ids; or keyword "current", "default", or "all"
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// metadata: Boolean
		//		indicates if filter metadata should be returned with columns		
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchColumns: function(/*fetchColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch columns from rest service
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);		
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.source) {
			content.source = kwArgs.source;
		}
		if (kwArgs.type) {
			content.type = kwArgs.type;
		}
		if (kwArgs.properties) {
			content.properties = kwArgs.properties.join(",");
		}		

		return dojo.xhrGet({
			url: this._getColumnsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "columnList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var cols = [];
				var colsLength = result.items.length;
				for (var index = 0;  index < colsLength;  ++index) {
					cols[index] = new ibm.tivoli.navmodel.Column({column: result.items[index], objectType: kwArgs.type, dataset: this});
				}
				
				kwArgs.onComplete(cols, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchColumns"), ioArgs);
			})
		});
	},

	/*=====
	setColumns_kwArgs: function(onComplete, onError, columns, type, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// columns: ibm.tivoli.navmodel.Column[]
		//		array of columns to be saved
		// type: String
		//		object type: "item", "link", "relationship" 
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Save columns to rest service
		
		// pull out raw javascript column data
		var columns = [];
		if (kwArgs.columns) {
			var count = kwArgs.columns.length;
			for (var index = 0;  index < count;  ++index) {
				var col = kwArgs.columns[index].getAttributes();
				delete col.filter;
				columns.push(col);
			}
		}		
		var json = dojo.toJson({columns: columns}, false);
		
		// need original "param_" values to locate cached dataset
		return dojo.xhrPost({
			url: this._getColumnsParamURL(kwArgs),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "columnList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: kwArgs.sync,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_setColumns"), ioArgs);
			})
		});
	},
	
	_newChildDataset: function(kwArgs) {
		// summary:
		//		Construct new child dataset object
		return new ibm.tivoli.navmodel.Dataset(kwArgs);
	},

	/*=====
	item_onComplete: function(item, kwArgs){},
	=====*/
	
	/*=====
	createItem_kwArgs: function(onComplete, onError, item) {
		// onComplete: item_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object
		//		ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/
	
	_createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//			Asynchronously create a new item in the dataset	
		var item = kwArgs.item;
		var json = dojo.toJson(item.getAttributes(), false);
		
		return dojo.xhrPost({
			url: this._getItemParamURL(item.getId()),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemView"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemView"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newItem = new ibm.tivoli.navmodel.Item({item: result, dataset: this, allProperties: true, allMetadata: false});
				kwArgs.onComplete(newItem, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createItem"), ioArgs);
			})
		});
	},

	/*=====
	createItems_kwArgs: function(onComplete, onError, item) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object[]
		//		array of ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/
	
	_createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
		var itemsObj = {"items": []};
		if (kwArgs.items) {
			var itemsLength = kwArgs.items.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				itemsObj.items[itemIndex] = kwArgs.items[itemIndex].getAttributes();
			}
		}
	
		var json = dojo.toJson(itemsObj, false);
		
		return dojo.xhrPost({
			url: this._getItemsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newItems = [];				
				if (result.items) {
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						newItems[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, allProperties: true, allMetadata: false});
					}
				}
				kwArgs.onComplete(newItems, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createItems"), ioArgs);
			})
		});
	},

	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	deleteItem_kwArgs: function(onComplete, onError, itemId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of item to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of item
	},
	=====*/
	
	_deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.remove) {
			content.remove = kwArgs.remove;
		}
		
		return dojo.xhrDelete({
			url: this._getItemURL(kwArgs.itemId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: false,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteItem"), ioArgs);
			})
		});
	},
	
	/*=====
	deleteItems_kwArgs: function(onComplete, onError, itemIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemIds: String[]
		//		array of item ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of items
	},
	=====*/
	
	_deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
		
		// build up the delete filter
		var deleteFilter = {};
		if (kwArgs.itemIds && kwArgs.itemIds.length > 0) {
			deleteFilter.identifiers = kwArgs.itemIds;
		}
		if (kwArgs.condition) {
			deleteFilter.condition = kwArgs.condition;
		}
		if (kwArgs.remove) {
			deleteFilter.remove = kwArgs.remove;
		}
		
		var json = dojo.toJson(deleteFilter, false);
				
		return dojo.xhrPost({
			url: this._getItemsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "deleteFilter"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-DELETE'
			},
			handleAs: "json",
			postData: json,
			sync: false,			
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteItems"), ioArgs);
			})
		});
	},

	/*=====
	link_onComplete: function(link, kwArgs){},
	=====*/
	
	/*=====
	createLink_kwArgs: function(onComplete, onError, link) {
		// onComplete: link_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// link: Object
		//		ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/
	
	_createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
		var link = kwArgs.link;
		var json = dojo.toJson(link.getAttributes(), false);
		
		return dojo.xhrPost({
			url: this._getLinkParamURL(link.getId()),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkView"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkView"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newLink = new ibm.tivoli.navmodel.Link({link: result, dataset: this, allProperties: true, allMetadata: false});
				kwArgs.onComplete(newLink, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createLink"), ioArgs);
			})
		});
	},
	
	/*=====
	createLinks_kwArgs: function(onComplete, onError, links) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// links: Object[]
		//		array of ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/
	
	_createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
		var linksObj = {"links": []};
		if (kwArgs.links) {
			var linksLength = kwArgs.links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				linksObj.links[linkIndex] = kwArgs.links[linkIndex].getAttributes();
			}
		}
	
		var json = dojo.toJson(linksObj, false);
		
		return dojo.xhrPost({
			url: this._getLinksParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkViewList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newLinks = [];				
				if (result.items) {
					var linksLength = result.items.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						newLinks[linkIndex] = new ibm.tivoli.navmodel.Link({link: result.items[linkIndex], dataset: this, allProperties: true, allMetadata: false});
					}
				}
				kwArgs.onComplete(newLinks, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createLinks"), ioArgs);
			})
		});
	},

	/*=====
	deleteLink_kwArgs: function(onComplete, onError, linkId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkId: String
		//		id of link to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of link
	},
	=====*/
	
	_deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.remove) {
			content.remove = kwArgs.remove;
		}
		
		return dojo.xhrDelete({
			url: this._getLinkURL(kwArgs.linkId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: false,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteLink"), ioArgs);
			})
		});
	},

	/*=====
	deleteLinks_kwArgs: function(onComplete, onError, linkIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkIds: String[]
		//		array of link ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of links
	},
	=====*/
	
	_deleteLinks: function(/*deleteLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
		
		// build up the delete filter
		var deleteFilter = {};
		if (kwArgs.linkIds && kwArgs.linkIds.length > 0) {
			deleteFilter.identifiers = kwArgs.linkIds;
		}
		if (kwArgs.condition) {
			deleteFilter.condition = kwArgs.condition;
		}
		if (kwArgs.remove) {
			deleteFilter.remove = kwArgs.remove;
		}
		
		var json = dojo.toJson(deleteFilter, false);
				
		return dojo.xhrPost({
			url: this._getLinksParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "deleteFilter"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-DELETE'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteLinks"), ioArgs);
			})
		});
	},
	
	/*=====
	relationship_onComplete: function(relationship, kwArgs){},
	=====*/
	
	/*=====
	createRelationship_kwArgs: function(onComplete, onError, relationship) {
		// onComplete: relationship_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object
		//		ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	_createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
		var relationship = kwArgs.relationship;
		var json = dojo.toJson(relationship.getAttributes(), false);
		
		return dojo.xhrPost({
			url: this._getRelationshipParamURL(relationship.getId()),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipView"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipView"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newRelationship = new ibm.tivoli.navmodel.Relationship({relationship: result, dataset: this, allProperties: true, allMetadata: false});				
				kwArgs.onComplete(newRelationship, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createRelationship"), ioArgs);
			})
		});
	},
	
	/*=====
	createRelationships_kwArgs: function(onComplete, onError, relationships) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object[]
		//		array of ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	_createRelationships: function(/*createRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
		var relationshipsObj = {"relationships": []};
		if (kwArgs.relationships) {
			var relLength = kwArgs.relationships.length;
			for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
				relationshipsObj.relationships[relIndex] = kwArgs.relationships[relIndex].getAttributes();
			}
		}
	
		var json = dojo.toJson(relationshipsObj, false);
		
		return dojo.xhrPost({
			url: this._getRelationshipsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipViewList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newRelationships = [];				
				if (result.items) {
					var relLength = result.items.length;
					for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
						newRelationships[relIndex] = new ibm.tivoli.navmodel.Relationship({relationship: result.items[relIndex], dataset: this, allProperties: true, allMetadata: false});
					}
				}
				kwArgs.onComplete(newRelationships, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createRelationships"), ioArgs);
			})
		});
	},

	/*=====
	deleteRelationship_kwArgs: function(onComplete, onError, relationshipId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipId: String
		//		id of relationship to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationship
	},
	=====*/
	
	_deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.remove) {
			content.remove = kwArgs.remove;
		}
				
		return dojo.xhrDelete({
			url: this._getRelationshipURL(kwArgs.relationshipId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: false,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteRelationship"), ioArgs);
			})
		});
	},

	/*=====
	deleteRelationships_kwArgs: function(onComplete, onError, relationshipIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipIds: String[]
		//		array of relationship ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationships
	},
	=====*/
	
	_deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
		
		// build up the delete filter
		var deleteFilter = {};
		if (kwArgs.relationshipIds && kwArgs.relationshipIds.length > 0) {
			deleteFilter.identifiers = kwArgs.relationshipIds;
		}
		if (kwArgs.condition) {
			deleteFilter.condition = kwArgs.condition;
		}
		if (kwArgs.remove) {
			deleteFilter.remove = kwArgs.remove;
		}
		
		var json = dojo.toJson(deleteFilter, false);
			
		return dojo.xhrPost({
			url: this._getRelationshipsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "deleteFilter"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-DELETE'
			},
			handleAs: "json",
			postData: json,
			sync: false,			
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteRelationships"), ioArgs);
			})
		});
	},	
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_extractContentTypeVersion: function(ioArgs) {
		var version = this.getVersion();
		var contentType = ioArgs.xhr.getResponseHeader("Content-Type");
		if (contentType) {
			var parms = contentType.split(";");
			var length = parms.length;
			for (var index = 0;  index < length;  ++index) {
				if (parms[index].indexOf("version=") === 0) {								
					version = parms[index].substring(8)*1;
				}
			}
		}
		
		return version;		
	},
									
	_mixinStandard: function(content, kwArgs) {
		// summary:
		//		Add common arguments if provided in kwArgs
		if (kwArgs.clearCache) {
			content.clearCache = kwArgs.clearCache;
		}
		
		if (this._optimize) {
			content.optimize = this._optimize;
		}
		
		if (kwArgs.start && kwArgs.start > 0) {
			content.start = kwArgs.start;
		}
		
		if (kwArgs.count && kwArgs.count > 0) {
			content.count = kwArgs.count;
		}
		
		if (kwArgs.metadata) {
			content.metadata = true;
		}
		
		if (kwArgs.sort) {
			content.sort = kwArgs.sort; 
		}
		
		if (kwArgs.condition) {
			content.condition = kwArgs.condition;
		}
		
		if (kwArgs.recursive) {
			content.recursive = kwArgs.recursive;
		}
	},
	
	_getDatasetURL: function() {
		// summary:
		//		Returns url for for fetching dataset
		
		return this.getArg("baseUrl") + this.getAttribute("uri");
	},
	
	_getOriginItemsURL: function() {
		// summary:
		//		Returns url for fetching origin items
		return  this._getDatasetURL() + "/originItems";
	},

	_getItemsURL: function() {
		// summary:
		//		Returns url for fetching items for a dataset 
		return this._getDatasetURL() + "/items";		
	},

	_getItemsParamURL: function() {
		// summary:
		//		Returns url for fetching items and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getItemsURL());	
	},

	_getItemsQueryURL: function(kwArgs) {
		// summary:
		//		Returns url for custom query of items
		var url = this._getItemsParamURL();
		var questionMark = (url.indexOf('?') > 0);
		
		if (this._optimize) {
			if (!questionMark) {
				questionMark = true;
				url += "?optimize=" + this._optimize;
			}
			else {
				url += "&optimize=" + this._optimize;
			}			
		}
		
		if (kwArgs.start && kwArgs.start > 0) {
			if (!questionMark) {
				questionMark = true;
				url += "?start=" + kwArgs.start;
			}
			else {
				url += "&start=" + kwArgs.start;
			}
		}
		
		if (kwArgs.count && kwArgs.count > 0) {
			if (!questionMark) {
				questionMark = true;
				url += "?count=" + kwArgs.count;
			}
			else {
				url += "&count=" + kwArgs.count;
			}
		}
		
		if (kwArgs.properties && kwArgs.properties.length > 0) {
			if (!questionMark) {
				questionMark = true;
				url += "?properties=" + kwArgs.properties.join(",");
			}
			else {
				url += "&properties=" + kwArgs.properties.join(",");
			}
			
			if (kwArgs.metadata) {
				url += "&metadata=true";
			}
		}
				
		return url;
	},

	_getItemURL: function(itemId) {
		// summary:
		//		Returns url for fetching a specific item
		return this._getItemsURL() + "/" + encodeURIComponent(itemId);		
	},
	
	_getItemParamURL: function(itemId) {
		// summary:
		//		Returns url for item and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getItemURL(itemId));				
	},

	_getRelatedItemsURL: function(itemId) {
		// summary:
		//		Returns url for fetching related items
		return this._getItemURL(itemId) + "/relatedItems";	
	},
	
	_getChildDatasetRelatedItemsURL: function(childDatasetId) {
		// summary:
		//		Returns url for fetching related items for a child dataset
		return this._getDatasetURL() + "/childDatasets/" + encodeURIComponent(childDatasetId) + "/relatedItems";	
	},

	_getLinksURL: function() {
		// summary:
		//		Returns url for fetching links for a dataset
		return this._getDatasetURL() + "/links";
	},

	_getLinksParamURL: function() {
		// summary:
		//		Returns url for fetching links and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getLinksURL()); 		
	},

	_getLinkURL: function(linkId) {
		// summary:
		//		Returns url for fetching a specific link
		return this._getLinksURL() + "/" + encodeURIComponent(linkId);		
	},

	_getLinkParamURL: function(linkId) {
		// summary:
		//		Returns url for link and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getLinkURL(linkId));				
	},

	_getRelationshipsURL: function() {
		// summary:
		//		Returns url for fetching relationships for a dataset
		return this._getDatasetURL() + "/relationships";		
	},

	_getRelationshipsParamURL: function() {
		// summary:
		//		Returns url for fetching relationships and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getRelationshipsURL()); 		
	},

	_getRelationshipURL: function(relationshipId) {
		// summary:
		//		Returns url for fetching a specific relationship
		return this._getRelationshipsURL() + "/" + encodeURIComponent(relationshipId);		
	},
	
	_getRelationshipParamURL: function(relationshipId) {
		// summary:
		//		Returns url for relationship and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getRelationshipURL(relationshipId));				
	},
	
	_getTasksURL: function() {
		// summary:
		//		Returns url for fetching tasks for a dataset
		return this._getDatasetURL() + "/tasks";		
	},

	_getTasksParamURL: function(kwArgs) {
		// summary:
		//		Returns url for fetching tasks for a dataset and also includes
		//		all of the "param_" and "version" arguments
		var url = this._appendParams(this._getTasksURL());		
		var questionMark = (url.indexOf('?') > 0);
		
		// optimize
		if (this._optimize) {
			if (!questionMark) {
				questionMark = true;
				url += "?optimize=" + this._optimize;
			}
			else {
				url += "&optimize=" + this._optimize;
			}			
		}
		
		// task category
		if (kwArgs.taskCategory) {
			if (!questionMark) {
				questionMark = true;
				url += "?category=" + kwArgs.taskCategory;
			}
			else {
				url += "&category=" + kwArgs.taskCategory;
			}			
		}
		
		return url;		
	},

	_getTaskURL: function(taskId) {
		// summary:
		//		Returns url for fetching a specific task
		return this._getTasksURL() + "/" + encodeURIComponent(taskId);		
	},

	_getNavigateParamURL: function(kwArgs) {
		// summary:
		//		Returns url for navigating a specific task and also includes
		//		all of the "param_" and "version" arguments
		var url = this._appendParams(this._getTaskURL(kwArgs.taskId));
		var questionMark = (url.indexOf('?') > 0);
		
		// optimize
		if (this._optimize) {
			if (!questionMark) {
				questionMark = true;
				url += "?optimize=" + this._optimize;
			}
			else {
				url += "&optimize=" + this._optimize;
			}			
		}
		
		// task category; should indicate navigation
		if (kwArgs.taskCategory) {
			if (!questionMark) {
				questionMark = true;
				url += "?category=" + kwArgs.taskCategory;
			}
			else {
				url += "&category=" + kwArgs.taskCategory;
			}			
		}
			
		return url;
	},
	
	_getItemStylesURL: function() {
		// summary:
		//		Returns url for fetching current item styles for a dataset
		return this._getDatasetURL() + "/itemStyles";	
	},

	_getLinkStylesURL: function() {
		// summary:
		//		Returns url for fetching current link styles for a dataset
		return this._getDatasetURL() + "/linkStyles";
	},

	_getRelationshipStylesURL: function() {
		// summary:
		//		Returns url for fetching current relationship styles for a dataset
		return this._getDatasetURL() + "/relationshipStyles";
	},

	_getStatusURL: function() {
		// summary:
		//		Returns url for fetching possible status values
		return this._getDatasetURL() + "/status";
	},

	_getColumnsURL: function() {
		// summary:
		//		Returns url for fetching current columns for a dataset
		return this._getDatasetURL() + "/columns";
	},
	
	_getColumnsParamURL: function(kwArgs) {
		// summary:
		//		Returns url for saving current tasks and also includes
		//		all of the "param_" and "version" arguments
		var url = this._appendParams(this._getColumnsURL());
		var questionMark = (url.indexOf('?') > 0);
		
		// type
		if (kwArgs.type) {
			if (!questionMark) {
				questionMark = true;
				url += "?type=" + kwArgs.type;
			}
			else {
				url += "&type=" + kwArgs.type;
			}			
		}
			
		return url;
	},

	_appendParams: function(url) {
		// summary:
		//		Append "param_" and "version" request parameters to the url
		//		so that dataset can be obtained from the REST service cache
		var questionMark = false;
		
		// add "param_" parms
		var parameters = this.getArg("parameters");
		for (var parm in parameters) {
			if (parameters.hasOwnProperty(parm)) {
				if (!questionMark) {
					questionMark = true;
					url += "?";
				}
				else {
					url += "&";
				}
				url += ("param_" + parm + "=" + parameters[parm]);				
			}
		}
		
		// IE will sometimes return http requests from cache without unique parms
		var now = new Date();
		if (!questionMark) {
			questionMark = true;
			url += "?d=" + now.getTime() + "&r=" + Math.random();
		}
		else {
			url += "&d=" + now.getTime() + "&r=" + Math.random();
		}	
		
		return url;
	},
	
	_getException: function(error, ioArgs, functionName) {
		return ibm.tivoli.navmodel.Utils.getException(error, ioArgs, functionName, this._trace, this.getVersion());
	},
	
	_isAllProps: function(properties) {
		return (properties && properties.length === 1 && properties[0] === "all");
	},
	
	_getPropIds: function(properties, allProps) {
		if (properties && properties.length > 0 && !allProps) {
			return properties;
		}
		
		return null;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.DatasetStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.DatasetStore"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
 
dojo.provide("ibm.tivoli.navmodel.DatasetStore");

// includes

	
ibm.tivoli.navmodel.DatasetStore._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetStore");
	
dojo.declare("ibm.tivoli.navmodel.DatasetStore", null, {
	// private members 
	_listening: false,
	_refresh: false,
	_sortIds: null,
	_conditionIds: null,	
	_trace: ibm.tivoli.navmodel.DatasetStore._trace,
	
	/*=====
	constructor_kwArgs: function(dataset, defaultProperties, returnItem) {
		// summary:
		//		An anonymous object used to configure the datastore
		// dataset: Object
		//		A ibm.tivoli.navmodel.Dataset instance
		// defaultProperties: String[]
		//		An array of property id values that are pre-fetched with the items
		// returnItem: Boolean
		//		A boolean indicating if the getValue() method should just return the actual
		//		item so that it can be used for custom formatting of the value
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs){
		// summary:
		//		Constructor for abstract base class dojo datastore;
		//		that receives ids data from an ibm.tivoli.navmodel.Dataset
		var args = {
			dataset: null,
			defaultProperties: ["all"],
			returnItem: false
		};
		dojo.mixin(args, kwArgs);
		dojo.mixin(this, args);
		
		// set up Dataset event handler
		this._setupEventHandler();			
    },
    
    CURIRefresh: function() {
		// summary:
		//		invoked by TED to indicate "Refresh" is occurring now
		this._refresh = true;
		
		if (this.dataset.isClearCacheOnRefresh()) {
			this.dataset._clearAllCaches();
		}
	},
	
	isRefresh: function() {
		// summary:
		//		returns true if TED refresh is occurring
		var refresh = this._refresh;
		this._refresh = false;
		return refresh;
	},
	

	//////////////////////////////////////////////////////////////
	// dojo.data.api.Identity
	//////////////////////////////////////////////////////////////
	
	getIdentityAttributes: function(/*Object*/ item){
		// summary:
		//		Returns an array of attribute names that are used to generate the identity.
		//		For most stores, this is a single attribute, but for some complex stores
		//		such as RDB backed stores that use compound (multi-attribute) identifiers
		//		it can be more than one. If the identity is not composed of attributes
		//		on the item, it will return null. This function is intended to identify
		//		the attributes that comprise the identity so that so that during a render
		//		of all attributes, the UI can hide the the identity information if it
		//		chooses.
		// item:
		//		The item from the store from which to obtain the array of public attributes that
		//		compose the identifier, if any.
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return null;
	},
    
    
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Read
	//////////////////////////////////////////////////////////////
	
	getFeatures: function(){    	
		// summary:
		//		The getFeatures() method returns an simple keyword values object
		//		that specifies what interface features the datastore implements.
		return {
			'dojo.data.api.Read': true,
			'dojo.data.api.Identity': true,
			'dojo.data.api.Notification':true
		};
	},
		
	close: function(/*Object*/ request) {
		// summary:
		//		The close() method is intended for instructing the store to 'close' out
		//		any information associated with a particular request.
		// description:
		//		The close() method is intended for instructing the store to 'close' out
		//		any information associated with a particular request.  In general, this API
		//		expects to receive as a parameter a request object returned from a fetch.
		//		It will then close out anything associated with that request, such as
		//		clearing any internal datastore caches and closing any 'open' connections.
		//		For some store implementations, this call may be a no-op.
		// request:
		//		An instance of a request for the store to use to identify what to close out.
		//		If no request is passed, then the store should clear all internal caches (if any)
		//		and close out all 'open' connections.  It does not render the store unusable from
		//		there on, it merely cleans out any current data and resets the store to initial
		//		state.	
		if (!request) {
			this._cleanupEventHandler();
		}		
	},
  
	isItem: function(/*Object*/ something){
		// summary:
		//		Returns true if *something* is an item and came from the store instance.
		//		Returns false if *something* is a literal, an item from another store instance,
		//		or is any object other than an item.
		// something:
		//		Can be anything.
		if (!something || (typeof something !== "object")) {
			return false;
		}
                
		try {
			return this._getObjectTypes()[something.getObjectType()];			
		}
		catch(err){
			return false;    		
		}	
	},
	
	isItemLoaded: function(/*Object*/ item) {
		// summary:
		//		Returns false if isItem(something) is false.  Returns false if
		//		if isItem(something) is true but the the item is not yet loaded
		//		in local memory (for example, if the item has not yet been read
		//		from the server).
		// item:
		//		Item previously handed out by datastore.
		//		ibm.tivoli.navmodel.Item object
		if (!item || !this.isItem(item)){
  			return false;
  		}
  		
  		return !item.isExpired();
	},	
	
	getLabel: function(/*Object*/ item) {
		// summary:
		//		Method to inspect the item and return a user-readable 'label' for the item
		//		that provides a general/adequate description of what the item is.
		// item:
		//		The item to return the label for.
		//		ibm.tivoli.navmodel.Item object
		// returns:
		// 		A user-readable string representing the item or undefined if no user-readable label can
		//		be generated.	
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return item.getLabel();
	},
  
	getLabelAttributes: function(/*Object*/ item) {
		// summary:
		//		Method to inspect the item and return an array of what attributes of the item were used
		//		to generate its label, if any.
		// item:
		//		The item to return the list of label attributes for.
		//		ibm.tivoli.navmodel.Item object
		// returns:
		//		An array of attribute names that were used to generate the label, or null if public attributes
		//		were not used to generate the label.  
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return ["label"];
	},
  
 	getAttributes: function(/*Object*/ item) {
		// summary:
		//		Returns an array with all the attributes that this item has.  This
		//		method will always return an array; if the item has no attributes
		//		at all, getAttributes() will return an empty array: [].
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// exceptions:
		//		Throws an exception if *item* is not an item
  		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
  		return item.getCachedPropertyIds(true);  		
    },
    
	hasAttribute: function(/*Object*/ item, /*String*/ attribute) {
		// summary:
		//		Returns true if the given *item* has a value for the given *attribute*.
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string		
		if (!item) {
			return false;
		}
		
		if (!this.isItem(item)){
			throw new Error("Specified item argument is not a valid item.");
		}
    	
		if (!dojo.isString(attribute)) {
			throw new Error("Specified attribute argument is not a string.");
		}
        
		var attributes = this.getAttributes(item);
		if (attributes) {
			var attrsLength = attributes.length;
			for (var attrIndex = 0;  attrIndex < attrsLength;  ++ attrIndex) {
				if (attribute === attributes[attrIndex]) {
					return true;
				}
			}
		}
    	
		return false;
	},
    
	getValues: function(/*Object*/ item, /*String*/ attribute) {
		// summary:
		//		This getValues() method works just like the getValue() method, but getValues()
		//		always returns an array rather than a single attribute value.  The array
		//		may be empty, may contain a single attribute value, or may contain many
		//		attribute values.
		//		
		//		If the item does not have a value for the given attribute, then getValues()
		//		will return an empty array: [].  (So, if store.hasAttribute(item, attribute)
		//		returns false, then store.getValues(item, attribute) will return [].)
		// item:
		//		The item to access values on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string 
    	return [this.getValue(item, attribute)];
    },
    
	containsValue: function(/*Object*/ item, /*String*/ attribute, /*Object*/ value) {
		// summary:
		//		Returns true if the given *value* is one of the values that getValues()
		//		would return.
		// item:
		//		The item to access values on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// value:
		//		The value to match as a value for the attribute.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
		var values = this.getValues(item, attribute);
		if (values) {
			var valuesLength = values.length;
				for (var valueIndex = 0;  valueIndex < valuesLength;  ++valueIndex) {
				if (value === values[valueIndex]) {
					return true;
				}
			}
		}
    	
		return false;
    },
	
	
    //////////////////////////////////////////////////////////////
	// dojo.data.api.Notification
	//////////////////////////////////////////////////////////////
	
	onSet: function(/*Object*/ item, 
					/*String*/ attribute, 
					/*Object[]*/ oldValue,
					/*Object[]*/ newValue) {
		// summary: 
		//		See dojo.data.api.Notification.onSet()
		
		// No need to do anything. This method is here just so that the 
		// client code can connect observers to it.
	},

    onNew: function(/*Object*/ newItem, /*Object?*/ parentInfo) {
		// summary: 
		//		See dojo.data.api.Notification.onNew()
		
		// No need to do anything. This method is here just so that the 
		// client code can connect observers to it. 
	},

    onDelete: function(/*Object*/ deletedItem) {
		// summary: 
		//		See See dojo.data.api.Notification.onDelete()
		
		// No need to do anything. This method is here just so that the 
		// client code can connect observers to it. 
	},
	
	onRefresh: function() {
		// summary: 
		//		This is NOT part of the dojo.data.api.Notification but can be attached to
		//		to indicate that the datastore items have changed dramatically and should
		//		be re-fetched.
		//
		// 		Refresh is coming as a result of async notification instead
		// 		of the actual Refresh button being clicked on
		this._refresh = true;		
	},
		
		
	//////////////////////////////////////////////////////////////
	// private methods
	///////////////////////////////////////////////////////////////
	_isChildOfChildDataset: function(/*Object*/ item) {
		// summary: 
		//		Returns true if the item is a member of a child dataset (i.e. grandchild)
    	return (item.getDataset() !== this.dataset);    	
    },
   
	_hasSort: function() {
		// summary: 
		//		Returns true if a sort is active
		return (this._sortIds ? true : false);
	},
	
	_affectsSort: function(/*String*/ propId) {
		// summary: 
		//		Returns true if propId is involved in sort
		return (this._sortIds && this._sortIds[propId]);
	},
	
	_hasCondition: function() {
		// summary: 
		//		Returns true if the filter condition(s) are active
		return (this._conditionIds ? true : false);
	},
	
	_affectsCondition: function(/*String*/ propId) {
		// summary: 
		//		Returns true if propId is involved in filter condition(s)
		return (this._conditionIds && this._conditionIds[propId]);
	},
	
	_getSort: function(/*Object[]*/ sort) {
		// summary: 
		//		Returns comma delimited sort string; indicating descending sort with a preceding "-"
 		this._sortIds = null;		
		var sortOrder = null;	 	
		if (sort && sort.length > 0) {			
			this._sortIds = {};						
			sortOrder = "";
			var sortLength = sort.length;
			for (var sortIndex = 0;  sortIndex < sortLength;  ++sortIndex){
				if (sortIndex > 0) {
					sortOrder += ",";
				}
				
				if (sort[sortIndex].descending) {
					sortOrder += "-";
				}
				
				var propId = sort[sortIndex].attribute;
				sortOrder += propId;
				
        		// remember sort ids
        		this._sortIds[propId] = true;				
        	}        	        	
        }        
        return sortOrder;
 	},
 		
 	_getCondition: function(/*Object*/ query) {
		// summary: 
		//		Returns array of condition objects involved in the current filter
		//			ibm.tivoli.navmodel.[String|Number|Date|Boolean|Enum|Map]Condition
		this._conditionIds = null;
		var condition = null;
 		 		
		if (query) {
			// new query format
			if (query.expression && query.ids) {
				this._conditionIds = query.ids;
				condition = query.expression;
			}
			// old "search" format compatible with original SAPM datastore
			else if (query.navModel && query.navModel.search && query.navModel.search.term) {
				this._conditionIds = {};
				condition = "(";
				
				var searchLength = query.navModel.search.attributes.length;
 				for (var searchIndex = 0;  searchIndex < searchLength;  ++searchIndex) {
 					if (searchIndex > 0) {
 						condition += " or ";
 					}
 					
 					var id = query.navModel.search.attributes[searchIndex];
 					condition += (id + " contains '" + query.navModel.search.term + "'");
 					this._conditionIds[id] = true;
 				}
 				
 				condition += ")";
			}
		}
 		
		return condition;
 	},
 	
 	_validateEventHandler: function() {
 		// summary: 
		//		Ensure that we are listening for events if we are supposed to be;
		//		a close() invocation could have caused us to stop listening
		if (!this._listening) {
			this._setupEventHandler();
		}
	},
    
 	_setupEventHandler: function() {
		// summary: 
		//		Add event listener to the dataset
 		if (!this._listening && this.dataset && this.dataset.isEventsEnabled()) {
			this.dataset.addEventListener(this, dojo.hitch(this, this._eventHandler));
			this._listening = true;
		}
 	},
 	
 	_cleanupEventHandler: function() {
		// summary: 
		//		Remove event listener from the dataset
 		if (this._listening && this.dataset && this.dataset.isEventsEnabled()) {
			this.dataset.removeEventListener(this);
			this._listening = false;			
		}
 	},
 	
	_eventHandler: function(/*Object*/ listenerObj, /*Object*/ event) {
		// summary: 
		//		Handle asynchronous events from the dataset
 		if (this._trace.isFine()) {
				this._trace.fine("datasetId=" + event.getDataset().getId() + " event=" + event.getType() +  " eventId=" + event.getId(), "_eventHandler");
		}
			
		switch(event.getType()) {
			case "changed":
				this._handleChangedEvent(event);
				break;
			case "labelChanged":
				this._handleLabelChangedEvent(event);
				break;
			case "statusChanged":
				this._handleStatusChangedEvent(event);
				break;					
			case "itemsAdded":
				this._handleItemsAddedEvent(event);
				break;
			case "itemsRemoved":
				this._handleItemsRemovedEvent(event);
				break;
			case "itemStatusChanged":
				this._handleItemStatusChangedEvent(event);
				break;
			case "itemPropertiesChanged":
				this._handleItemPropertiesChangedEvent(event);
				break;
			case "linksAdded":
				this._handleLinksAddedEvent(event);
				break;
			case "linksRemoved":
				this._handleLinksRemovedEvent(event);
				break;
			case "linkStatusChanged":
				this._handleLinkStatusChangedEvent(event);
				break;
			case "linkPropertiesChanged":
				this._handleLinkPropertiesChangedEvent(event);
				break;
			case "linkChanged":
				this._handleLinkChangedEvent(event);
				break;
			case "relationshipStatusChanged":
				this._handleRelationshipStatusChangedEvent(event);
				break;
			case "relationshipPropertiesChanged":
				this._handleRelationshipPropertiesChangedEvent(event);
				break;
			case "childDatasetsAdded":
				this._handleChildDatasetsAddedEvent(event);
				break;
			case "childDatasetsRemoved":
				this._handleChildDatasetsRemovedEvent(event);
				break;
			case "exception":
				this._handleExceptionEvent(event);
				break;
			case "eventRouter":
				this._handleEventRouterEvent(event);
				break;
			default:
				if (this._trace.isFine()) {
					this._trace.fine("datasetId=" + event.getDataset().getId() + " unrecognized event=" + event.getType(), "_eventHandler");
				}									
		}
	},
	
	_handleChangedEvent: function(/*Object*/ event) {
		// signal that data from store should be re-requested by consumer
		this.onRefresh();	
	},
	
	_handleLabelChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleStatusChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleItemsAddedEvent: function(/*Object*/ event) {		
	},
	
	_handleItemsRemovedEvent: function(/*Object*/ event) {		
	},
	
	_handleItemStatusChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleItemPropertiesChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleLinksAddedEvent: function(/*Object*/ event) {	
	},
	
	_handleLinksRemovedEvent: function(/*Object*/ event) {		
	},
	
	_handleLinkStatusChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleLinkPropertiesChangedEvent: function(/*Object*/ event) {				
	},
	
	_handleLinkChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleRelationshipStatusChangedEvent: function(/*Object*/ event) {			
	},
	
	_handleRelationshipPropertiesChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleChildDatasetsAddedEvent: function(/*Object*/ event) {		
	},
	
	_handleChildDatasetsRemovedEvent: function(/*Object*/ event) {		
	},
	
	_handleExceptionEvent: function(/*Object*/ event) {		
	},
	
	_handleEventRouterEvent: function(/*Object*/ event) {		
	}
}); 

}

if(!dojo._hasResource["ibm.tivoli.navmodel.DatasetItemsStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.DatasetItemsStore"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
 
dojo.provide("ibm.tivoli.navmodel.DatasetItemsStore");

// includes


	
ibm.tivoli.navmodel.DatasetItemsStore._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetItemsStore");
	
dojo.declare("ibm.tivoli.navmodel.DatasetItemsStore", ibm.tivoli.navmodel.DatasetStore, {
	// private members
	_clearCache: false,
	_trace: ibm.tivoli.navmodel.DatasetItemsStore._trace,
	
	/*=====
	constructor_kwArgs: function(dataset, defaultProperties, returnItem) {
		// dataset: Object
		//		A ibm.tivoli.navmodel.Dataset instance
		// defaultProperties: String[]
		//		An array of property id values that are pre-fetched with the items
		// returnItem: Boolean
		//		A boolean indicating if the getValue() method should just return the actual
		//		item so that it can be used for custom formatting of the value
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructor for dojo datastore;
		//		that receives item data from an ibm.tivoli.navmodel.Dataset
		//		Base class will mixin kwArgs		
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Identity
	//////////////////////////////////////////////////////////////
	
	getIdentity: function(/*Object*/ item){
		// summary:
    	//		Returns a unique identifier for an item. The return value will be
    	//		either a string or something that has a toString() method.
    	// item:
    	//		The item from the store from which to obtain its identifier.
		// exceptions:
    	//		Conforming implementations may throw an exception or return null if
    	//		item is not an item.
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return item.getId();		
    },
    
	/*=====
	_onItem: function(item) {},
	=====*/
	
	/*=====
	_onError: function(error) {},
	=====*/
	
	/*=====
	fetchItemByIdentity_kwArgs: function(identity, onItem, onError, scope) {
		// identity: String|Object
		//		The identity parameter is the identity of the item you wish to locate and load
		//		This attribute is required. It should be a string or an object that toString()
		//		can be called on.
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded. It takes only one
		//		parameter, the item located, or null if none found.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error. It takes 
		//		only one parameter, the error object
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem,
		//		onError, etc) will be invoked in the context of the scope object.
	},
	=====*/
	
 	fetchItemByIdentity: function(/*fetchItemByIdentity_kwArgs*/ kwArgs) {
		// summary:
		//		Given the identity of an item, this method returns the item that has
		//		that identity through the onItem callback. Conforming implementations
		//		should return null if there is no item with the given identity.
		//		Implementations of fetchItemByIdentity() may sometimes return an item
		//		from a local cache and may sometimes fetch an item from a remote server
	 	var args = {
 			identity: null,
 			onItem: function(item) {},
 			onError: function(error) {},
 			scope: null
 		};
 		dojo.mixin(args, kwArgs);
 		
 		var item = null;
 		var error = null;
 		if (args.identity) {
 			try { 			
  				item = this.dataset.getItem({itemId: args.identity, properties: this.defaultProperties});
  			}
  			catch(err) {
  				error = err;
  			}  			
  		}  		
  		
 		if (item) {
 			args.onItem.call(args.scope, item);
 		}
 		else {
 			args.onError.call(args.scope, error ? error : "Could not fetch item for identity=" + args.identity);
 		}		
	},
	
	
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Read
	//////////////////////////////////////////////////////////////
	
	/*=====
	fetch_onBegin: function(size, request){},
	=====*/
	
	/*=====
	fetch_onItem: function(item, request){},
	=====*/
	
	/*=====
	fetch_onComplete: function(items, request){},
	=====*/
	
	/*=====
	fetch_onError: function(errorData, request){},
	=====*/
	
	/*=====
	fetch_kwArgs: function(query, queryOptions, onBegin, onItem, onComplete, onError, scope,
							start, count, sort) {
		// summary:
		//		The kwArgs parameter may either be an instance of conforming to 
		//		dojo.data.api.Request or may be a simple anonymous object
		//		containing any of the above parameters.
		// query: String|Object
		//		The query parameter is used to specify search and filter condition information for the fetch.
		//		Only those items that match the search and/or filter will be returned.
		//		|	{	ids: {
		//		|			propId: boolean		// indicates propIds involved in the expression
		//		|		},
		//		|		expression: string,		// expression involiving propId's and values  (and/or together)
		//		|								// example:  ((Name contains 'abc' || Type contains 'abc)) and (Size > 100)
		//		|
		//		|		// the following is for compatability with legacy datastore
		//		| 		navModel: {
		//		|			search: {
		//		|				term: string,		the search text
		//		|				attributes: array	["property id1", "property id2", ...]
		//		|			}
		//		|		}
		//		|	}
		// queryOptions: Object?
		//		The queryOptions parameter is an optional parameter that can be used to modify the query.  The
		//		ignoreCase case can be specified to globally make all of the filter conditions case sensitive/insensitive.
		//		|	{ ignoreCase: boolean
		//		|	}
		// onBegin: fetch_onBegin
		//		If an onBegin callback function is provided, the callback function will be called just once, 
		//		before the first onItem callback is called. The onBegin callback function will be passed two arguments, 
		//		the the total number of items identified and the Request object.  If the total number is
		//		unknown, then size will be -1.  Note that size is not necessarily the size of the collection of items 
		//		returned from the query, as the request may have specified to return only a subset of the total set 
		//		of items through the use of the start and count parameters.
		// onItem: fetch_onItem
		//		If an onItem callback function is provided, the callback function will be called just once, 
		//		after the last onItem callback is called. Note that if the onItem callback is not present, 
		//		then onComplete will be passed an array containing all items which matched the query and the 
		//		request object. If the onItem callback is present, then onComplete is called as: onComplete(null, request).
		// onComplete: fetch_onComplete
		//		If an onComplete callback function is provided, the callback function will be called just once,
		//		after the last onItem callback is called. Note that if the onItem callback is not present,
		//		then onComplete will be passed an array containing all items which matched the query and the 
		//		request object. If the onItem callback is present, then onComplete is called as: onComplete(null, request).
		// onError: fetch_onError
		//		If an onError callback function is provided, the callback function will be called if there is any 
		//		sort of error while attempting to execute the query. The onError callback function will be passed two arguments:
		//		an Error object and the Request object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onComplete, onError, etc) 
		//		will be invoked in the context of the scope object.
		// start: int
		//		If a start parameter is specified, this is a indication to the datastore to
		//		only start returning items once the start number of items have been located and
		//		skipped.  When this parameter is paired with 'count', the store should be able
		//		to page across queries with millions of hits by only returning subsets of the
		//		hits for each query
		// count: int
		//		 If a count parameter is specified, this is a indication to the datastore to
		//		 only return up to that many items.  This allows a fetch call that may have
		//		millions of item matches to be paired down to something reasonable.
		// sort: Object[]
		//		If a sort parameter is specified, this is a indication to the datastore to
		//		sort the items in some manner before returning the items.  The array is an array of
		//		javascript objects that must conform to the following format to be applied to the
		//		fetching of items:
		//		|	{
		//		|		attribute: attribute || attribute-name-string,
		//		|		descending: true|false;   // Optional.  Default is false.
		//		|	}
	},
	=====*/
	
	fetch: function(/*fetch_kwArgs*/ kwArgs) {
		// summary:
		//		Given a query and set of defined options, such as a start and count of items to return,
		//		this method executes the query and makes the results available as data items.
		//		The format and expectations of stores is that they operate in a generally asynchronous
		//		manner, therefore callbacks are always used to return items located by the fetch parameters.
		// description:
		//		A Request object will always be returned and is returned immediately.
		//		The basic request is nothing more than the keyword args passed to fetch and
		// 		an additional function attached, abort().  The returned request object may then be used
		//		to cancel a fetch.  All data items returns are passed through the callbacks defined in the
		//		fetch parameters and are not present on the 'request' object.
		// returns:
		//		The fetch() method will return a javascript object conforming to the API
		//		defined in dojo.data.api.Request.  In general, it will be the keywordArgs
		//		object returned with the required functions in Request.js attached.
		//		Its general purpose is to provide a convenient way for a caller to abort an
		//		ongoing fetch.
		// exceptions:
		//		Throws an exception if the query is not valid, or if the query is required but was not supplied.
		var args = {
			query: {},
			queryOptions: {},
			onBegin: function(size, request) {},
			onItem: function(item, request) {},
			onComplete: function(items, request){},
			onError: function(errorData, request){},
			scope: null,
			start: 0,
			count: null, //default should be to get all
			sort: null,
			abort: function() {}
		};
		dojo.mixin(args, kwArgs);
		
		if (this._trace.isFinest()) {
			this._trace.finest("dataset=" + this.dataset.getId() + " start=" + args.start + " count=" + args.count, "fetch");
		}
		
		// make sure event handling is set up
		this._validateEventHandler();
		
		// TED refresh is occurring
		this._clearCache = false;
		if (this.isRefresh()) {
			this._clearCache = this.dataset.isClearCacheOnRefresh();						
		}
		
		// translate information from the args
		var sort = this._getSort(args.sort);
		var condition = this._getCondition(args.query);	
			
		var itemArgs = {
			onComplete: dojo.hitch(this, function(items, itemArgs) {
				args.onBegin.call(args.scope, itemArgs.filteredRows, args);
				
				if (items) {
					var itemsLength = items.length;
					for(var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {                 	                	
                		args.onItem.call(args.scope, items[itemIndex], args);                	
					}
				}
				
				args.onComplete.call(args.scope, items, args);				
			}),
			
			onError: dojo.hitch(this, function(error, exception, ioArgs) {
				args.onError.call(args.scope, error, args);
			}),
			
			clearCache: this._clearCache,
			recursive: true,
			start: args.start,
			count: args.count,	
			properties: this.defaultProperties,
			sort: sort,
			condition: condition
		}; 		
		this.dataset.getItems(itemArgs);	
		       
        return args;
    },
    
    
    getValue: function(/*Object*/ item, /*String*/ attribute, /*Object?*/ defaultValue) {
		// summary:
    	//		Returns a single attribute value. Returns defaultValue if and only if item does not have a 
    	//		value for attribute. Returns null if and only if null was explicitly set as the attribute value. 
    	//		Returns undefined if and only if the item does not have a value for the given attribute 
    	//		(which is the same as saying the item does not have the attribute).
    	// item:
    	//		The item to access value on.
    	//		ibm.tivoli.navmodel.Item object
    	// attribute:
    	//		The attribute to access represented as a string.
    	// defaultValue:
    	//		Optional. A default value to use for the getValue return if the attribute does not exist or 
    	//		has no value.
    	// exceptions:
    	//		Throws an exception if *item* is not an item, or *attribute* is not a string
        if (arguments.length < 2) {
            throw new Error("Incorrect number of arguments specified: expecting 2 (item, attribute)");
        }
        
        if (!this.isItem(item)) {
            throw new Error("Specified item argument is not a valid item.");
        }
        
        if (!dojo.isString(attribute)) {
            throw new Error("Specified attribute argument is not a string.");
        }  
          	
        if (this.returnItem) {
        	return item;
        }
        
        var value = defaultValue;
        var prop = item.getProperty(attribute);
        if (prop) {
        	value = prop.getDisplayValue();            
        }        	        
                        
        return value;
    },
    
    /*=====
	loadItem_kwArgs: function(item, onItem, onError, scope) {
		// summary:
		//		An anonymous object that defines the item to load and callbacks to invoke when the
		//		load has completed.
		// item: Object
		//		The item parameter is an object that represents the item in question that should be
		//		ontained by the store.  This attribute is required.
		//		ibm.tivoli.navmodel.Item object
		// onItem: _onItem
		//		The onItem parameter is the callback to invoke when the item has been loaded.
		//		It takes only one parameter, the fully loaded item.
		// onError: _onError
		//		The onError parameter is the callback to invoke when the item load encountered an error.
		//		It takes only one parameter, the error object.
		// scope: Object
		//		If a scope object is provided, all of the callback functions (onItem, onError, etc) 
		//		will be invoked in the context of the scope object. 
		//		onItem.call(dojo.global(), item, request)
	},
	=====*/
    
	loadItem: function(/*loadItem_kwArgs*/ kwArgs) {
		// summary:
		//		Given an item, this method loads the item so that a subsequent call to 
		//		store.isItemLoaded(item) will return true.  If a call to isItemLoaded() returns true 
		//		before loadItem() is even called, then loadItem() need not do any work at all and will 
		//		not even invoke the callback handlers.  So, before invoking this method, check that
		//		the item has not already been loaded.
		var args = {
			item : null,
			onItem: function(item) {},
			onError: function(error){},
			scope: null
		};		
		dojo.advancedMixin(args, kwArgs);
	
		if (!this.isItem(args.item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		if (this.isItemLoaded(args.item)){
			return args.item;
		}
		
		try {
			var dataset = args.item.getDataset();
			var item = dataset.getItem({itemId: args.item.getId(), properties: this.defaultProperties});
			args.onItem.call(args.scope, item);
		}
		catch(err) {
			args.onError.call(args.scope, err);
		}
	},
	
	//////////////////////////////////////////////////////////////
	// private methods
	///////////////////////////////////////////////////////////////
	
	_getObjectTypes: function() {
		// summary:
		//		Returns String[] of object types that can in the datastore
    	return {"item" : true};
    },
    
    _handleItemPropertiesChangedEvent: function(/*Object*/ event) {
		var items = event.getItems();    	
		var propsArray = event.getProperties();			
		if (items && propsArray) {
			var refresh = false;
			var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var properties = propsArray[index];				
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var item = items[index];
						var property = properties[propIndex];
						var propId = property.getId();
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId() + " prop=" + propId + " value=" + property.getDisplayValue(), "_handleItemPropertiesChangedEvent");
						}
						
						this.onSet(item, propId, null, property.getDisplayValue());
						
						if (!refresh && (this._affectsSort(propId) || this._affectsCondition(propId))) {
							refresh = true;							
						}
					}										
				}				
			}
			
			if (refresh) {
				this.onRefresh();
			}	
		}	
	},
	
    _handleItemsAddedEvent: function(/*Object*/ event) {
		var items = event.getItems();
		if (items) {
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var item = items[index];				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId(), "_handleItemsAddedEvent");
				}
				this.onNew(item);
			}
			
			if (this._hasSort() || this._hasCondition()) {
				this.onRefresh();
			}
		}		    		
	},
	
	_handleItemsRemovedEvent: function(/*Object*/ event) {
		var items = event.getItems();
		if (items) {
    		var length = items.length;			
			for (var index = 0;  index < length;  ++index) {
				var item = items[index];				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + event.getDataset().getId() + " itemId=" + item.getId(), "_handleItemsRemovedEvent");
				}
				this.onDelete(item);
			}			
		}
	}
});

}

if(!dojo._hasResource["dojoe.Util"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojoe.Util"] = true;
/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Extensions for Dojo
*
* (C) Copyright IBM Corp. 2011.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
dojo.provide("dojoe.Util");

/*Common utility functions for all*/

isNumber = function(value){
	return (typeof value === 'number' && isFinite(value));
}

startsWith = function(all, part){
	return (all.indexOf(part) === 0);
}

endsWith = function(all, part){
	return (all.lastIndexOf(part) === all.length - part.length);
}

contains = function(all, part){
	return (all.lastIndexOf(part) >= 0);
}

trim = function(stringToTrim) {
	return stringToTrim.replace(/^\s+|\s+$/g,"");
}

ltrim = function(stringToTrim) {
	return stringToTrim.replace(/^\s+/,"");
}

rtrim = function(stringToTrim) {
	return stringToTrim.replace(/\s+$/,"");
}

}

if(!dojo._hasResource["dojoe.messagedialog.MessageDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojoe.messagedialog.MessageDialog"] = true;
/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Extensions for Dojo
*
* (C) Copyright IBM Corp. 2011.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
dojo.provide("dojoe.messagedialog.MessageDialog");


 






//Localization of resource bundles



dojo.declare("dojoe.messagedialog.MessageDialog",[ dijit._Widget ],
{
		// summary:  MessageDialog is a widget to show message dialogs of type error,warning,info,confirm. 
		// description: MessageDialog widget uses the dijit.Dialog for its implementation.
	
	//theme: String
	// defined the theme for Message Dialog
	theme:null,	
	
	//resources:object
	//Reference to the resources
	 resources_:null,
	 
    // resizeAxis: String
    //  x | xy | y to limit pane's sizing direction.
    resizeAxis: "xy",

	// type: String
    // defines the type of message dialog ("Error","Critical", "Warning", "Info", "Confirm").
	type: "Error",
	
	// portlet title associated w/ this message
	portletTitle: null,

	// message: String
	//the displayable translated message
	message: null,

	// messageId: String
	//the displayable translated message's unique identifier
	messageId: null,
	
	// width: String
	//gives the pixel width of the dialog (Default: 500px)
	width: "380px",
	
	// height: String
	//gives the pixel height of the dialog (Default: 200px)
	height: "225px",
	
	// buttons: String Array
	//a list of buttons to display (Default: OK)
	buttons: [],
	
	// callback: Function
	//a callback function when the dialog is closed
	callback: null,
	
	// parameters to pass on callback when dialog closes
	callbackParms: null,
	
	// url: String
	//a URL that links the message to an InfoCenter page
	url: null,
	
	// show "Never show again checkbox"
	showCheckbox: false,
	
	// contentIsolation: boolean parameter to open PopupUtil in content isolation mode in portlets
	contentIsolation: false,
	
	// Destroy the dialog on close
	destroyOnClose: true,
	
	// namespace: user defined namespace for widget
	namespace :"",
	
	dir : "",

	constructor: function() {
		// summary: An extension to the Dojo Dialog widget that supports 
		//      advanced features.
		// resourceMessages_: JSON
		//		Resource bundle.
		// dialog_: dojo.widget.FloatingPane
		//		An instance of the Dialog widget.
		// title_: String
		//		Title bar string.
		// icon_: String
		//		The dialog icon's uri.
		// buttonPressedId_: String
		//		The ID of the button pressed; this value is passed to the callback function.

	    this.resources_ = dojo.i18n.getLocalization("dojoe.messagedialog", "MessageDialog",this.lang);
		
		if (this.contentIsolation == true) {
			var statusAr = startsWith(djConfig.locale,"ar");
			var statusHe = startsWith(djConfig.locale,"he");
			if (statusAr == true || statusHe == true) {
				this.dir = "rtl";
			}
		}
	    
	    this.dialog_ = null;

	    this.title_ = null;
	    this.icon_ = null;
	    this.buttonPressedId_ = -1;
	},


	postCreate: function() { 
		// summary:  Creates and initialises the message Dialog as per its type.
		// tags: public
	    if (this.buttons.length === 0) {
	        this.buttons.push(this.resources_.DIALOG_BUTTON_OK);
	    }

		var className = null;
	    if (this.type.toLowerCase() == "error" ||this.type.toLowerCase() == "critical") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_ERROR, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_ERROR;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/i_message_critical.png");
	        className = "critical";
	    }
	    else if (this.type.toLowerCase() == "warning") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_WARNING, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_WARNING;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/i_message_warning.png");
	        className = "warning";
	    }
	    else if (this.type.toLowerCase() == "confirm") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_CONFIRM, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_CONFIRM;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/i_message_confirm.png");
	        className = "confirm";
	    }
	    else if (this.type.toLowerCase() == "validation") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_VALIDATION, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_VALIDATION;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/i_message_validation.png");
	        className = "validation";
	    }
	    else { // if (this.type.toLowerCase() == "info") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_INFO, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_INFO;
		    }
	        this.icon_ = dojo.moduleUrl("dojoe.common", "themes/images/i_message_info.png");
	        className = "info";
	    }
	    
		var div = document.createElement('div'); 
		document.body.appendChild(div);
		var theme=this.theme;
		var dir = this.dir;
        this.dialog_ = new dijit.Dialog (
			                             { title: this.title_,
											dir: dir
		                                 },
		                                 div); 

		var dojoAttachpoint = this.dialog_.containerNode;
		this.dialog_.closeButtonNode.setAttribute("title", this.resources_.DIALOG_BUTTON_CLOSE);
		this.dialog_.closeButtonNode.setAttribute("tabIndex", 0);
		dojo.connect(this.dialog_.closeButtonNode,'onkeypress',this,'onKeyPress');
		dojo.style(div,'class',theme);
		dojoAttachpoint.innerHTML = this.getContent(className,theme);
		// dojo.connect(dijit.Dialog.prototype, "postMixInProperties", function(){ this.buttonCancel="Close" });
        //Build and place resize handle       
        var resizeHandleDOM = dojo.doc.createElement( 'span' );
        resizeHandleDOM.className = "dojoxFloatingResizeHandle";    
        dojo.place( resizeHandleDOM, this.dialog_.domNode, "last" );
    
        this.resizeHandle = new dojox.layout.ResizeHandle(
        { 
              targetId: this.dialog_.id, 
            resizeAxis: this.resizeAxis 
        }, resizeHandleDOM );   
	
        this.dialog_.domNode.style.width = this.width;	
        this.dialog_.domNode.style.height = this.height;	
        //this.dialog_.containerNode.style.overflow="auto";
        this.dialog_.domNode.style.overflow = "hidden";        
    
		for (var i = 0; i < this.buttons.length; i++) {
			var button = dojo.byId(this.id + "mmd_button_" + i);
			dojo.connect(button, 'onclick', this, 'onClick');    
		}
    
        this.onCloseListener  = dojo.connect( this.dialog_, "hide", this, "_onHide" );
	  this.onResizeListener = dojo.connect( this.dialog_, "resize", this, "customLayout" );
	},
	
    /**************************************************************************
    * The dialog has been hidden.  Wait 500 miliseconds to destroy the dialog.
    *
    * HACK -- Can't figure out a better way to destroy the dialog on close.
    **************************************************************************/
    _onHide:function()
    {// summary: Closes the MessageDialog .
		// tags: private
      setTimeout( dojo.hitch( this, "destroy" ), 500 );
    },  
	
	onKeyPress: function (e) {
		if (e.keyCode == 13 || e.keyCode == 32 || e.keyCode == 0) { //  Space keycode is 32 on IE8 and 0 on mozilla.  Enter keycode is 13 for both. 
			this.dialog_.hide();
		}
	},
	
	onClick: function(/* HTMLEvent */e) {
    	// summary:  Calls the callback on click of Ok else destroys.
		// tags: public
		var index = this.id.length + "mmd_button_".length;
		this.buttonPressedId_ = parseInt(e.target.id.substring(index), 0);
	    console.log ("MessageDialog.onClick: buttonPressedId_: " + this.buttonPressedId_);
	    var checkbox = dojo.byId (this.id + "neverShowAgain");
	    var checked = checkbox ? checkbox.checked : false;

	    this.dialog_.hide();
	    
	    if (this.callback) {
	    	this.callback(this.buttonPressedId_, this.messageId, checked, this.callbackParms);
	    }
    },

	show: function() {
    	// summary:  Displays the messageDialog.
		// tags: public
	    if( this.contentIsolation == true ){
			// Get the namespace identifying the content that launched the context menu
			console.info("Inside Show if PopupUtil Undefined");
			var namespace="";
			if(this.namespace != null)
			 namespace = this.namespace;
			
			//Close (just in case) and then open the popup covering the whole page
			console.info("Opening Popup namespace "+namespace);
			var xcord=100;
			var ycord=100;
			var dialogNamespace=namespace+'dojoe.messagedialog.MessageDialog';
			var old_popup_iframe = PopupUtil.ClosePopup( namespace+'dojoe.messagedialog.MessageDialog' );
			//Changes for fixing Content Isolation issue with Claro theme
			//var theme = dojo.body().className;
			var theme=this.theme;
			this.baseUrl=dojo.baseUrl;
			
			var index = this.baseUrl.lastIndexOf("dojo/dojo/");
			
				if (index != -1) {
				
			this.baseUrl=this.baseUrl.substring(0,index);
			
			}
				 
			
			//Changes to fix CSBUG #158121
			//var popupUrl =dojo.baseUrl+'../html/MessageDialogPopUp.html?PopupID='+dialogNamespace+'&eLeft='+xcord+'&eTop='+ycord+'&locale='+djConfig.locale+"&theme="+theme;
			var popupUrl =this.baseUrl+'sample/jsp/html/MessageDialogPopUp.html?PopupID='+dialogNamespace+'&eLeft='+xcord+'&eTop='+ycord+'&locale='+djConfig.locale+"&theme="+theme;
			console.info("url "+popupUrl);
			var popup_iframe = PopupUtil.OpenPopup( 
				namespace, 
				namespace+'dojoe.messagedialog.MessageDialog', 
				true, 
				null, 
				null, 
				null, 
				null, 
				popupUrl, 
				null
			);
			console.info("after popup opening code");
			//Initialize the contextMenu when the iframe popup is finished loading
			var tThis = this;
			if( popup_iframe.addEventListener ){
				popup_iframe.addEventListener(
					"load", 
					function(){
						popup_iframe.contentWindow.makeMessageDialog( tThis );
					}, 
					false
				);
			}
			else if (popup_iframe.attachEvent){
				popup_iframe.attachEvent(
					"onload", 
					function(){ 
						popup_iframe.contentWindow.makeMessageDialog( tThis);
					}
				);
			}
		}
		else{
			this.dialog_.show();
			this.customLayout();
			this.correctPNG();
        }
	},

      customLayout: function() 
      {
		// summary: Resizes the Message Dialog.
		// tags: public
		console.debug( dojo.byId( this.id + '_messagediv' ) );
		console.debug( dojo.contentBox( this.dialog_.domNode ) );
		
		var messageDiv = dojo.byId( this.id + '_messagediv' );
		var dialogSize = dojo.contentBox( this.dialog_.domNode );

		var titleBarSize = dojo.contentBox( this.dialog_.titleBar );

		var newSize = {w:0,h:0};

		newSize.w = dialogSize.w - 75;
		if (this.showCheckbox === true) {
			var neverShowCoords = dojo.coords (this.id + "neverShowAgain");
			//Background issue fix for IE
			if(dojo.isIE)
				newSize.h = dialogSize.h - titleBarSize.h - neverShowCoords.h - 63;
			else	
				newSize.h = dialogSize.h - titleBarSize.h - neverShowCoords.h - 90;
		} else {
			//Defect Fix for Locales with Long translations
			//newSize.h = dialogSize.h - titleBarSize.h - 90;
			//Background issue fix for IE
			if(dojo.isIE)
				newSize.h = dialogSize.h - titleBarSize.h - 63;
			else
				newSize.h = dialogSize.h - titleBarSize.h - 100;
		}

	    dojo.contentBox( messageDiv, newSize );
	},
	
	getContent: function(className,theme) {
		// summary:  gets the content to show inside messageDialog.
		// tags: public
		var innerHTML = "";
		if (theme == null) {
		theme = "claro";
		}
		//innerHTML += "<table style=\"width: 100%;\" class=\"" + className + "\">";
		
		innerHTML += "<div class=\"" + theme + "\" tabIndex=0>";
		innerHTML += "<table class=\"" + className + "\" >";
		innerHTML += "<tr valign=\"top\">";
		innerHTML += "<td>";

		innerHTML += "<table>";
		innerHTML += "<tr >";
		
		/*innerHTML += "<td rowspan=\"2\">";
		innerHTML += "&nbsp;";
		innerHTML += "<img align=\"middle\" src=\"" + this.icon_ + "\"/>";
		innerHTML += "&nbsp;&nbsp;";
		innerHTML += "</td>";*/
		innerHTML += "<td>";
		innerHTML += "&nbsp;";		
		innerHTML += "&nbsp;&nbsp;";
		innerHTML += "</td>";
		innerHTML += "<td class=\"message-id\" style='font-size: 11px;' tabIndex=0 title=\"" + this.messageId + "\">";
		
		// add URL to infocenter
		if (this.url) {
			innerHTML += '<a href="javascript:void(0)" onclick="window.open(\'' + this.url + "\')\">";
			innerHTML += this.messageId;
			innerHTML += "</a>";
		}
		else {
			innerHTML += this.messageId;
		}
		
		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "<tr>";
		innerHTML += "<td valign=\"top\" style=\"padding:10px;\">";
		innerHTML += "&nbsp;";
		innerHTML += "<img  align=\"top\" src=\"" + this.icon_ + "\"/>";
		innerHTML += "&nbsp;&nbsp;";
		innerHTML += "</td>";
		innerHTML += "<td class=\"message-description\" >";
		innerHTML += "<div id='" + this.id + "_messagediv" + "' style='overflow:auto; width:100%;font-size: 11px;' tabIndex=0 title=\"" + this.message + "\">";
		innerHTML += "<p>";
		innerHTML += this.message;
		innerHTML += "</p>";
		innerHTML += "</div>";
		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "</table>";

		innerHTML += "</td>";
		innerHTML += "</tr>";

		if (this.showCheckbox === true) {
			innerHTML += "<tr >";
			innerHTML += "<td class=\"checkbox\" style=\"font-size: 11px\" valign=\"bottom\">";
			innerHTML += "&nbsp;&nbsp;&nbsp;<input type=\"checkbox\" dojoType=\"dijit.form.CheckBox\" id=\"" + this.id + "neverShowAgain\"/><label for=\"" + this.id + "neverShowAgain\">" + this.resources_.NEVER_SHOW + "</label>";
			innerHTML += "</td>";
			innerHTML += "</tr>";
		}

		innerHTML += "<tr>";
		//innerHTML += "<td valign=\"bottom\" style=\"background-color: rgb(247, 247, 247);\">";
		innerHTML += "<td valign=\"bottom\" >";
		innerHTML += "<div>";
		innerHTML += "<table width='100%' class=\"" + this.theme+ "\">";
		innerHTML += "<tr>";
		innerHTML += "<td valign=\"bottom\">";
		innerHTML += "<div class=\"dialogButtons\">";
		for (var i = 0; i < this.buttons.length; i++) {
			innerHTML += "&nbsp;<button  dojoType=\"dijit.form.Button\"  type=\"button\" style=\"font-size: 11px\" id=\"" + this.id + "mmd_button_" + i + "\" value=\"" + this.buttons[i] + "\">";
			innerHTML += this.buttons[i] + "</button>";
		}
		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "</table>";
		innerHTML += "</div>";

		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "</table>";
		innerHTML += "</div>";
		
		return innerHTML;
	},
	
	// correctly handle PNG transparency in Win IE 5.5 & 6.
	correctPNG: function() {
		var arVersion = navigator.appVersion.split("MSIE");
		var version = parseFloat(arVersion[1]);
		if ((version >= 5.5) && document.body.filters) {
			for (var i = 0; i < document.images.length; i++) {
				var img = document.images[i];
				var imgName = img.src.toUpperCase();
				if (imgName.substring(imgName.length-3, imgName.length) == "PNG") {
					var imgID = img.id ? "id='" + img.id + "' " : "";
					var imgClass = img.className ? "class='" + img.className + "' " : "";
					var imgTitle = img.title ? "title='" + img.title + "' " : "title='" + img.alt + "' ";
					var imgStyle = "display:inline-block;" + img.style.cssText;
					if (img.align == "left") {
						imgStyle = "float:left;" + imgStyle;
					}
					if (img.align == "right") {
						imgStyle = "float:right;" + imgStyle;
					}
					if (img.parentElement.href) {
						imgStyle = "cursor:hand;" + imgStyle;
					}
					var strNewHTML = "<span " + imgID + imgClass + imgTitle +
									 " style=\"" + "width:" + img.width + "px; height:" + img.height + "px;" + imgStyle + ";" +
									 "filter:progid:DXImageTransform.Microsoft.AlphaImageLoader" +
									 "(src=\'" + img.src + "\', sizingMethod='scale');\"></span>";
					img.outerHTML = strNewHTML;
					i = i - 1;
				}
			}
		}    
	},
	
	destroy: function () {
		// summary:  destroys the message Dialog.
		// tags: public
        dojo.disconnect( this.onCloseListener );
        dojo.disconnect( this.onResizeListener );
        
        //TODO:  The fact this can NOT be called means we have not created the widget correctly
        //       This widget should directly extend dialog to be created correctly
        //this.inherited( "destroy", arguments );
        
        if ( this.dialog_ ) { this.dialog_.destroy(); dijit.registry.remove (this.dialog_.id); delete this.dialog_; }
        
		if (dijit.byId (this.id + "neverShowAgain")) {
			dijit.registry.remove (this.id + "neverShowAgain");
		}

		for (var i = 0; i < this.buttons.length; i++) {
			dijit.registry.remove (this.id + "mmd_button_" + i);
		}

		delete this.type;
		delete this.message;
		delete this.messageId;
		delete this.width;
		delete this.height;
		delete this.buttons;
		delete this.callbackParms;
		delete this.url;
		delete this.showCheckbox;
		delete this.destroyOnClose;
		delete this.title_;
		delete this.icon_;
		delete this.resources_;

		dijit.registry.remove (this.id);
	}

});



}

if(!dojo._hasResource["ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler");







ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler");

dojo.declare("ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler",
	      null,
{		
	
	// _trace: [private] ibm.tivoli.navmodel.Trace
	//		Utility for trace and logging
	_trace: ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler._trace,
	
	//Base constructor
	constructor: function () {		
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		//NLS
		this._resources = dojo.i18n.getLocalization("ibm.tivoli.tip.navwidget", "resources");
		this._messages = dojo.i18n.getLocalization("ibm.tivoli.tip.navwidget", "messages");
	},
	
	displayException: function (/*Object*/ ex, /*Object*/dontTraceMessage) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon

		var msgId = null;
		var msgSeverity = null;
		var msgText = null;
		if (ex && ex.isInstanceOf && ex.isInstanceOf(ibm.tivoli.navmodel.Exception)) { //Checking for existence of isInstanceOf before calling it because normal JS exception won't have an isInstanceOf
			msgId = ex.getMsgId();
			msgSeverity = ex.getMsgSeverity();
			msgText = ex.getMsgText();
		} else {
			
			//We have one of the following two messages to choose from. I don't think we're going to have enough in the exception to know if one of these is more appropriate than the
			//other. Also, I don't think we want to do the substitution for ${error_txt} since likely not translated.
			//
			//	- "TCTUI1002E":"An error has occurred.  The data provider could not return the data. ${error_txt}", // ${error_txt} will contain the error message returned from the server
			//	- "TCTUI1003E":"An error has occurred communicating with the server. ${error_txt}"  // ${error_txt} will contain the error message returned from the server
			
			msgId = "TCTUI1003E";
			msgSeverity = "Error";

			var messageTemplate = this._messages.TCTUI1003E;
			msgText = dojo.string.substitute(messageTemplate,
                    {error_txt: ""}); //Doing an empty substitution since message in generic exception probably won't be translated
		}

		//Display the message
		this.displayMessage(msgId, msgSeverity, msgText); 
		
		//Let's trace to the console window too
		if (!dontTraceMessage) {
			if (this._trace.isError()) {
				var methodName = "handleException";
				this._trace.error(ex, methodName);
			}
		}
	},
	
	displayMessage: function(/*Object*/ msgId, /*Object*/ msgSeverity, /*Object*/ msgText) {
		// summary:
		//		Coming soon
		// description:
		//		Coming soon
		
		var msgDlg = new dojoe.messagedialog.MessageDialog(
				{message: msgText,
				 messageId: msgId,
				 type: msgSeverity});
		msgDlg.show();
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dojo.ChartFetcher"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dojo.ChartFetcher"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/


dojo.provide("ibm.tivoli.tip.charts.dojo.ChartFetcher");


dojo.require ("ibm.tivoli.tip.charts.dojo.RESTErrorHandler");




dojo.declare("ibm.tivoli.tip.charts.dojo.ChartFetcher",
	          null,
{
	/* Default timeout for requests made to the server */
	requestTimeout: 30000,
	global: null,
	rowsPerPage: null,
	chartPreferences: null,
	chartDef: null,
	widgetDefinition: null,
	datasourceID: null,
	providerID: null,
	chartParams: null,

	constructor: function (mixinData) {
	    dojo.mixin( this, mixinData );
	    
		this.id = dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));	
		this.columns = null;
		this.rows = null;
		this.start = null;
		this.query = null;
		this.sortColumn = null;
		this.totalCount = -1;
		this.tableID = this.global.namespace + "states";
		this.store = null;
	},
	getQuery: function () {
		var query = this.widgetDefinition.datasets[0].query;
		
		if ((this.chartPreferences) && (this.chartPreferences.getParameters() !== null)) {
			
			query = query + this.chartPreferences.getParameters();
		}
		
		query = query + "&widgetID=" + this.chartDef.id + "&applicationID=" + this.widgetDefinition.datasource.id;
		
		return (query);
	},
	fetchData: function (inRowIndex, inCount) {
		console.log ("enter ChartFetcher.fetchData");
		if(inRowIndex===undefined||inRowIndex===null||inCount===undefined||inCount===null){
			inRowIndex = 0;
			inCount = -1;
		}	
		this.inRowIndex = inRowIndex;
		this.inCount = inCount;
		var url = "/ibm/tivoli/rest";
		var dsParams = {
			widgetID: this.chartDef.id
		};
		if(this.chartParams !== null){
			console.log ("adding chartParams");
			console.log (this.chartParams);
			for (var row in this.chartParams) {
				console.log ("processing row in chartParams");
				if (row) {
					console.log ("processing row in chartParams");
				 	var param = this.chartParams[row];
				 	console.log ("param: ");
				 	console.log (param);
					//queryParam.navModel.parameters[param.id] = param.value;
				 	dsParams[param.id] = param.value;
				} else {
					console.log ("row has no value");
				}
			}
		}
		if(this.global.restTimeParam){
			dsParams[this.global.restTimeParam.id] = this.global.restTimeParam.value;
		}
		var dataSet = new ibm.tivoli.navmodel.Dataset({
			baseUrl: url,	
			provider: this.providerID,
			datasource: this.datasourceID,
			dataset: this.widgetDefinition.datasets[0].id,
			createEventRouter: false,
			parameters: dsParams,
			onComplete: dojo.hitch(this, function(dataSet){
				var navStore = new ibm.tivoli.navmodel.DatasetItemsStore({
					dataset: dataSet,						
					returnItem: false		
				});		
				var query = {};		
				var filCond = this.widgetDefinition.datasets[0].conditions;
				var dsColumns = this.widgetDefinition.datasets[0].columns;
				if(filCond && filCond.filterString && filCond.filter.length>0){
					query.expression = filCond.filterString;	
					query.ids = {};			
					for(var i=0;i<filCond.filter.length;i++){
						if(filCond.filter[i].column == "anycolumn"){
							query.ids = {};
							for(var j=0;j<dsColumns.length;j++){
								query.ids[dsColumns[j].id] = true;
							}
							break;
						}else{
							query.ids[filCond.filter[i].column] = true;
						}
					}
				}		
				this.store = navStore;
				this.store.fetch ( {				
					//onBegin: dojo.hitch(this, "beginReturn"),
					query: query,
					onComplete: dojo.hitch (this, "processRows"),
					onBegin: dojo.hitch(this, "reportTotalRows"),
					onError: dojo.hitch (this, function (error) {										
						var errHandler = new ibm.tivoli.tip.charts.dojo.RESTErrorHandler(this.global);		
						errHandler.showError(dojo.fromJson(error.responseText));	
                	}),
					start: this.inRowIndex,
					count: this.inCount
				});				
			}),
			onError: dojo.hitch(this, function(error, exception, ioArgs) {
				var busyPane = dijit.byId(ibm.tivoli.tip.dojo.Server.getSingleton (this.global).busyPaneID);
				if(busyPane){
					busyPane.hide();
				}
				var msgHandler = new ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler({});
				msgHandler.displayException(exception);
			})
		});
		console.log ("exit ChartFetcher.fetchData");
	},
	
	reportTotalRows: function(inCount){
		console.log("enter reportTotalRows");
		if(this.totalCount==-1){
			this.totalCount = inCount;
			dojo.publish ("ChartFetcher.totalRowsFetched" + this.id, [inCount]);
		}		
		console.log("exit reportTotalRows");
	},
	beginReturn: function (inCount) {
		
		this.totalCount = inCount;
	},
	processRows: function (items, query) {
		this.onRowLoad (items);
	},
//	onColumnLoad: function (data, request) {
//		this.columns = data;
//		this.fetchData ();
//	},
	onRowLoad:function (data) {
		this.rows = data;
		var fetchTime = new Date() - this.start;
		
		
		dojo.publish ("ChartFetcher.dataFetched" + this.id, 
					  [this.columns, this.rows, this.store]
					 );
	},
	getChartDefinition: function () {
		return (this.chartDef);
	},
	getColumns: function () {
		return (this.widgetDefinition.datasets[0].columns);
		//return (this.columns);
	},
	getRows: function () {
		return (this.rows);
	},
	getTotalRows: function () {
		return (this.totalCount);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartPager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartPager"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.charts.dijit.ChartPager");





// NLS


dojo.declare("ibm.tivoli.tip.charts.dijit.ChartPager",
              [ dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true,
	templateString:"<table class=\"tip-table-footer-border\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\r\n\t<tbody>\r\n\t\t<tr class=\"tip-table-footer\" dir=\"ltr\">\r\n\t\t\t<td class=\"tip-table-footer-data\" nowrap=\"nowrap\" valign=\"middle\" align=\"right\"> \r\n\t\t\t\t<div dojoAttachPoint=\"pageLabel\" class=\"footerLabel\"></div>\r\n\t\t\t</td>\r\n\t\t\t<td class=\"tip-table-footer-data\" nowrap=\"nowrap\" valign=\"middle\" align=\"left\"> \t\t\t\t\r\n\t\t\t\t<div type=\"text\" dojoType=\"dijit.form.NumberSpinner\" scrollOnFocus=\"true\" selectOnClick=\"true\" intermediateChanges=\"true\" dojoAttachPoint=\"page_number\" dojoAttachEvent=\"onChange:jumpPage\" style=\"width:50px;\" required=\"true\" value=\"1\">\r\n                </div>\r\n\t\t\t</td>\r\n\t\t\t<td class=\"tip-table-footer-data\" nowrap=\"nowrap\" valign=\"middle\" align=\"left\"> \r\n\t\t\t\t<div dojoAttachPoint=\"totalPage\" class=\"footerLabel\"></div>\r\n\t\t\t</td>\r\n\t\t\t<td class=\"tip-table-footer-data\" width=\"100%\" nowrap=\"nowrap\" valign=\"middle\" align=\"right\"> \r\n\t\t\t\t<div dojoAttachPoint=\"rowsShowRate\" class=\"footerLabel\"></div>\r\n\t\t\t</td>\r\n\t\t</tr>  \r\n\t</tbody>\r\n</table>\r\n",
	chartFetcher: null,
	totalRows: null,
	
	constructor: function () {
		this.currentPage = 0;
		this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");    
	},
	postCreate: function () {
		this.totalRows = this.chartFetcher.getTotalRows();
		this.updateFooter();
		this.configureTextBox();
	},
	configureTextBox: function(){
		var consts = {};
		consts.min = 1;
		consts.max = this.getMaxPages();
		this.page_number.attr('constraints', consts);
		//this.page_number.attr('value', 1);
	},
	removeCharts: function() {
		if(this.chartDef.type =="Pie"){
			var div = dojo.byId (this.chartBuilder.id + "chart0");			
			div.removeChild(div.firstChild);			
			this.chartBuilder.chart = null;
			for(var i=0; i< this.chartBuilder.charts.length;i++){
				this.chartBuilder.charts[i].destroy();
			}
		}
	},
	jumpPage: function (event) {
		var pageNumber = this.page_number.attr('value');
		if (!isNaN(pageNumber)){
			if(pageNumber > this.getMaxPages()){
				this.currentPage = this.getMaxPages()-1;
			}else if(pageNumber<=0){
				this.currentPage = 0;
			}else{
				this.removeCharts();
				this.currentPage = pageNumber - 1;
				var start = this.currentPage * this.chartFetcher.rowsPerPage;
				this.chartFetcher.fetchData (start, this.chartFetcher.rowsPerPage);
				this.updateFooter();
			}
		}
	},
	updatePageNumber: function () {
		this.page_number.attr('value', this.currentPage+1);
	},
	getMaxPages: function () {
		
		
		return (Math.ceil (this.totalRows / this.chartFetcher.rowsPerPage));
	},
	getCurrentPage: function () {
		return (this.currentPage);
	},
	updateFooter: function() {
		//this.setInfoAreaHTML( dojo.string.substitute( this.resources_.PAGE_X_OF_Y, { 0: this.currentPage + 1, 1: this.getMaxPages() } ) );
		this.setPageLabelHTML( dojo.string.substitute( this.resources_.PAGE_LABEL ) );		
		this.setTotalPageHTML( dojo.string.substitute( this.resources_.PAGE_OF_Y, { 0: this.getMaxPages() } ) );
		this.setRowsShowRateHTML( dojo.string.substitute( this.resources_.ROW_SHOW_RATE, { 0: this.chartFetcher.rowsPerPage, 1: this.totalRows }) );
	},
	setPageLabelHTML: function (html) {
		this.pageLabel.innerHTML = html;
	},
	setTotalPageHTML: function (html) {
		this.totalPage.innerHTML = html;
	},
	setRowsShowRateHTML: function (html) {
		this.rowsShowRate.innerHTML = html;
	},
	setInfoAreaHTML: function (html) {
		this.infoArea.innerHTML = html;
	},
	onRefresh: function () {
		var start = this.currentPage * this.chartFetcher.rowsPerPage;
		this.chartFetcher.fetchData (start, this.chartFetcher.rowsPerPage);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.Version"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.Version"] = true;

			dojo.provide("ibm.tivoli.tip.Version");
			
			ibm.tivoli.tip.Version.getVersion = function() {
				  return( "2.2.0.15-2011/07/26 21:43" );
			};
		

}

if(!dojo._hasResource["dijit.form._Spinner"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._Spinner"] = true;
dojo.provide("dijit.form._Spinner");



dojo.declare(
	"dijit.form._Spinner",
	dijit.form.RangeBoundTextBox,
	{
		// summary:
		//		Mixin for validation widgets with a spinner.
		// description:
		//		This class basically (conceptually) extends `dijit.form.ValidationTextBox`.
		//		It modifies the template to have up/down arrows, and provides related handling code.

		// defaultTimeout: Number
		//		Number of milliseconds before a held arrow key or up/down button becomes typematic
		defaultTimeout: 500,

		// minimumTimeout: Number
		//       minimum number of milliseconds that typematic event fires when held key or button is held
		minimumTimeout: 10,

		// timeoutChangeRate: Number
		//		Fraction of time used to change the typematic timer between events.
		//		1.0 means that each typematic event fires at defaultTimeout intervals.
		//		< 1.0 means that each typematic event fires at an increasing faster rate.
		timeoutChangeRate: 0.90,

		// smallDelta: Number
		//	  Adjust the value by this much when spinning using the arrow keys/buttons
		smallDelta: 1,

		// largeDelta: Number
		//	  Adjust the value by this much when spinning using the PgUp/Dn keys
		largeDelta: 10,

		templateString: dojo.cache("dijit.form", "templates/Spinner.html", "<div class=\"dijit dijitReset dijitInlineTable dijitLeft\"\r\n\tid=\"widget_${id}\" waiRole=\"presentation\"\r\n\t><div class=\"dijitReset dijitButtonNode dijitSpinnerButtonContainer\"\r\n\t\t><input class=\"dijitReset dijitInputField dijitSpinnerButtonInner\" type=\"text\" tabIndex=\"-1\" readOnly waiRole=\"presentation\"\r\n\t\t/><div class=\"dijitReset dijitLeft dijitButtonNode dijitArrowButton dijitUpArrowButton\"\r\n\t\t\tdojoAttachPoint=\"upArrowNode\"\r\n\t\t\t><div class=\"dijitArrowButtonInner\"\r\n\t\t\t\t><input class=\"dijitReset dijitInputField\" value=\"&#9650;\" type=\"text\" tabIndex=\"-1\" readOnly waiRole=\"presentation\"\r\n\t\t\t\t\t${_buttonInputDisabled}\r\n\t\t\t/></div\r\n\t\t></div\r\n\t\t><div class=\"dijitReset dijitLeft dijitButtonNode dijitArrowButton dijitDownArrowButton\"\r\n\t\t\tdojoAttachPoint=\"downArrowNode\"\r\n\t\t\t><div class=\"dijitArrowButtonInner\"\r\n\t\t\t\t><input class=\"dijitReset dijitInputField\" value=\"&#9660;\" type=\"text\" tabIndex=\"-1\" readOnly waiRole=\"presentation\"\r\n\t\t\t\t\t${_buttonInputDisabled}\r\n\t\t\t/></div\r\n\t\t></div\r\n\t></div\r\n\t><div class='dijitReset dijitValidationContainer'\r\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&Chi; \" type=\"text\" tabIndex=\"-1\" readOnly waiRole=\"presentation\"\r\n\t/></div\r\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\r\n\t\t><input class='dijitReset dijitInputInner' dojoAttachPoint=\"textbox,focusNode\" type=\"${type}\" dojoAttachEvent=\"onkeypress:_onKeyPress\"\r\n\t\t\twaiRole=\"spinbutton\" autocomplete=\"off\" ${!nameAttrSetting}\r\n\t/></div\r\n></div>\r\n"),

		baseClass: "dijitTextBox dijitSpinner",

		// Set classes like dijitUpArrowButtonHover or dijitDownArrowButtonActive depending on
		// mouse action over specified node
		cssStateNodes: {
			"upArrowNode": "dijitUpArrowButton",
			"downArrowNode": "dijitDownArrowButton"
		},

		adjust: function(/* Object */ val, /*Number*/ delta){
			// summary:
			//		Overridable function used to adjust a primitive value(Number/Date/...) by the delta amount specified.
			// 		The val is adjusted in a way that makes sense to the object type.
			// tags:
			//		protected extension
			return val;
		},

		_arrowPressed: function(/*Node*/ nodePressed, /*Number*/ direction, /*Number*/ increment){
			// summary:
			//		Handler for arrow button or arrow key being pressed
			if(this.disabled || this.readOnly){ return; }
			this._setValueAttr(this.adjust(this.get('value'), direction*increment), false);
			dijit.selectInputText(this.textbox, this.textbox.value.length);
		},

		_arrowReleased: function(/*Node*/ node){
			// summary:
			//		Handler for arrow button or arrow key being released
			this._wheelTimer = null;
			if(this.disabled || this.readOnly){ return; }
		},

		_typematicCallback: function(/*Number*/ count, /*DOMNode*/ node, /*Event*/ evt){
			var inc=this.smallDelta;
			if(node == this.textbox){
				var k=dojo.keys;
				var key = evt.charOrCode;
				inc = (key == k.PAGE_UP || key == k.PAGE_DOWN) ? this.largeDelta : this.smallDelta;
				node = (key == k.UP_ARROW || key == k.PAGE_UP) ? this.upArrowNode : this.downArrowNode;
			}
			if(count == -1){ this._arrowReleased(node); }
			else{ this._arrowPressed(node, (node == this.upArrowNode) ? 1 : -1, inc); }
		},

		_wheelTimer: null,
		_mouseWheeled: function(/*Event*/ evt){
			// summary:
			//		Mouse wheel listener where supported

			dojo.stopEvent(evt);
			// FIXME: Safari bubbles

			// be nice to DOH and scroll as much as the event says to
			var scrollAmount = evt.detail ? (evt.detail * -1) : (evt.wheelDelta / 120);
			if(scrollAmount !== 0){
				var node = this[(scrollAmount > 0 ? "upArrowNode" : "downArrowNode" )];

				this._arrowPressed(node, scrollAmount, this.smallDelta);

				if(!this._wheelTimer){
					clearTimeout(this._wheelTimer);
				}
				this._wheelTimer = setTimeout(dojo.hitch(this,"_arrowReleased",node), 50);
			}

		},

		postCreate: function(){
			this.inherited(arguments);

			// extra listeners
			this.connect(this.domNode, !dojo.isMozilla ? "onmousewheel" : 'DOMMouseScroll', "_mouseWheeled");
			this._connects.push(dijit.typematic.addListener(this.upArrowNode, this.textbox, {charOrCode:dojo.keys.UP_ARROW,ctrlKey:false,altKey:false,shiftKey:false,metaKey:false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout, this.minimumTimeout));
			this._connects.push(dijit.typematic.addListener(this.downArrowNode, this.textbox, {charOrCode:dojo.keys.DOWN_ARROW,ctrlKey:false,altKey:false,shiftKey:false,metaKey:false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout, this.minimumTimeout));
			this._connects.push(dijit.typematic.addListener(this.upArrowNode, this.textbox, {charOrCode:dojo.keys.PAGE_UP,ctrlKey:false,altKey:false,shiftKey:false,metaKey:false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout, this.minimumTimeout));
			this._connects.push(dijit.typematic.addListener(this.downArrowNode, this.textbox, {charOrCode:dojo.keys.PAGE_DOWN,ctrlKey:false,altKey:false,shiftKey:false,metaKey:false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout, this.minimumTimeout));
		}
});

}

if(!dojo._hasResource["dijit.form.NumberTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.NumberTextBox"] = true;
dojo.provide("dijit.form.NumberTextBox");




/*=====
dojo.declare(
	"dijit.form.NumberTextBox.__Constraints",
	[dijit.form.RangeBoundTextBox.__Constraints, dojo.number.__FormatOptions, dojo.number.__ParseOptions], {
	// summary:
	//		Specifies both the rules on valid/invalid values (minimum, maximum,
	//		number of required decimal places), and also formatting options for
	//		displaying the value when the field is not focused.
	// example:
	//		Minimum/maximum:
	//		To specify a field between 0 and 120:
	//	|		{min:0,max:120}
	//		To specify a field that must be an integer:
	//	|		{fractional:false}
	//		To specify a field where 0 to 3 decimal places are allowed on input,
	//		but after the field is blurred the value is displayed with 3 decimal places:
	//	|		{places:'0,3'}
});
=====*/

dojo.declare("dijit.form.NumberTextBoxMixin",
	null,
	{
		// summary:
		//		A mixin for all number textboxes
		// tags:
		//		protected

		// Override ValidationTextBox.regExpGen().... we use a reg-ex generating function rather
		// than a straight regexp to deal with locale (plus formatting options too?)
		regExpGen: dojo.number.regexp,

		/*=====
		// constraints: dijit.form.NumberTextBox.__Constraints
		//		Despite the name, this parameter specifies both constraints on the input
		//		(including minimum/maximum allowed values) as well as
		//		formatting options like places (the number of digits to display after
		//		the decimal point).   See `dijit.form.NumberTextBox.__Constraints` for details.
		constraints: {},
		======*/

		// value: Number
		//		The value of this NumberTextBox as a Javascript Number (i.e., not a String).
		//		If the displayed value is blank, the value is NaN, and if the user types in
		//		an gibberish value (like "hello world"), the value is undefined
		//		(i.e. attr('value') returns undefined).
		//
		//		Symmetrically, attr('value', NaN) will clear the displayed value,
		//		whereas attr('value', undefined) will have no effect.
		value: NaN,

		// editOptions: [protected] Object
		//		Properties to mix into constraints when the value is being edited.
		//		This is here because we edit the number in the format "12345", which is
		//		different than the display value (ex: "12,345")
		editOptions: { pattern: '#.######' },

		/*=====
		_formatter: function(value, options){
			// summary:
			//		_formatter() is called by format().   It's the base routine for formatting a number,
			//		as a string, for example converting 12345 into "12,345".
			// value: Number
			//		The number to be converted into a string.
			// options: dojo.number.__FormatOptions?
			//		Formatting options
			// tags:
			//		protected extension

			return "12345";		// String
		},
		 =====*/
		_formatter: dojo.number.format,

		_setConstraintsAttr: function(/* Object */ constraints){
			var places = typeof constraints.places == "number"? constraints.places : 0;
			if(places){ places++; } // decimal rounding errors take away another digit of precision
			if(typeof constraints.max != "number"){
				constraints.max = 9 * Math.pow(10, 15-places);
			}
			if(typeof constraints.min != "number"){
				constraints.min = -9 * Math.pow(10, 15-places);
			}
			this.inherited(arguments, [ constraints ]);
			if(this.focusNode && this.focusNode.value && !isNaN(this.value)){
				this.set('value', this.value);
			}
		},

		_onFocus: function(){
			if(this.disabled){ return; }
			var val = this.get('value');
			if(typeof val == "number" && !isNaN(val)){
				var formattedValue = this.format(val, this.constraints);
				if(formattedValue !== undefined){
					this.textbox.value = formattedValue;
				}
			}
			this.inherited(arguments);
		},

		format: function(/*Number*/ value, /*dojo.number.__FormatOptions*/ constraints){
			// summary:
			//		Formats the value as a Number, according to constraints.
			// tags:
			//		protected

			var formattedValue = String(value);
			if(typeof value != "number"){ return formattedValue; }
			if(isNaN(value)){ return ""; }
			// check for exponential notation that dojo.number.format chokes on
			if(!("rangeCheck" in this && this.rangeCheck(value, constraints)) && constraints.exponent !== false && /\de[-+]?\d/i.test(formattedValue)){
				return formattedValue;
			}
			if(this.editOptions && this._focused){
				constraints = dojo.mixin({}, constraints, this.editOptions);
			}
			return this._formatter(value, constraints);
		},

		/*=====
		_parser: function(value, constraints){
			// summary:
			//		Parses the string value as a Number, according to constraints.
			// value: String
			//		String representing a number
			// constraints: dojo.number.__ParseOptions
			//		Formatting options
			// tags:
			//		protected

			return 123.45;		// Number
		},
		=====*/
		_parser: dojo.number.parse,

		parse: function(/*String*/ value, /*dojo.number.__FormatOptions*/ constraints){
			// summary:
			//		Replacable function to convert a formatted string to a number value
			// tags:
			//		protected extension
			var v = this._parser(value, dojo.mixin({}, constraints, (this.editOptions && this._focused) ? this.editOptions : {}));
			if(this.editOptions && this._focused && isNaN(v)){
				v = this._parser(value, constraints); // parse w/o editOptions: not technically needed but is nice for the user
			}
			return v;
		},

		_getDisplayedValueAttr: function(){
			var v = this.inherited(arguments);
			return isNaN(v) ? this.textbox.value : v;
		},

		filter: function(/*Number*/ value){
			// summary:
			//		This is called with both the display value (string), and the actual value (a number).
			//		When called with the actual value it does corrections so that '' etc. are represented as NaN.
			//		Otherwise it dispatches to the superclass's filter() method.
			//
			//		See `dijit.form.TextBox.filter` for more details.
			return (value === null || value === '' || value === undefined) ? NaN : this.inherited(arguments); // attr('value', null||''||undefined) should fire onChange(NaN)
		},

		serialize: function(/*Number*/ value, /*Object?*/options){
			// summary:
			//		Convert value (a Number) into a canonical string (ie, how the number literal is written in javascript/java/C/etc.)
			// tags:
			//		protected
			return (typeof value != "number" || isNaN(value)) ? '' : this.inherited(arguments);
		},

		_setBlurValue: function(){
			var val = dojo.hitch(dojo.mixin({}, this, { _focused: true }), "get")('value'); // parse with editOptions
			this._setValueAttr(val, true);
		},

		_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange, /*String?*/formattedValue){
			// summary:
			//		Hook so attr('value', ...) works.
			if(value !== undefined && formattedValue === undefined){
				formattedValue = String(value);
				if(typeof value == "number"){
					if(isNaN(value)){ formattedValue = '' }
					// check for exponential notation that dojo.number.format chokes on
					else if(("rangeCheck" in this && this.rangeCheck(value, this.constraints)) || this.constraints.exponent === false || !/\de[-+]?\d/i.test(formattedValue)){
						formattedValue = undefined; // lets format comnpute a real string value
					}
				}else if(!value){ // 0 processed in if branch above, ''|null|undefined flow thru here
					formattedValue = '';
					value = NaN;
				}else{ // non-numeric values
					value = undefined;
				}
			}
			this.inherited(arguments, [value, priorityChange, formattedValue]);
		},

		_getValueAttr: function(){
			// summary:
			//		Hook so attr('value') works.
			//		Returns Number, NaN for '', or undefined for unparsable text
			var v = this.inherited(arguments); // returns Number for all values accepted by parse() or NaN for all other displayed values

			// If the displayed value of the textbox is gibberish (ex: "hello world"), this.inherited() above
			// returns NaN; this if() branch converts the return value to undefined.
			// Returning undefined prevents user text from being overwritten when doing _setValueAttr(_getValueAttr()).
			// A blank displayed value is still returned as NaN.
			if(isNaN(v) && this.textbox.value !== ''){
				if(this.constraints.exponent !== false && /\de[-+]?\d/i.test(this.textbox.value) && (new RegExp("^"+dojo.number._realNumberRegexp(dojo.mixin({}, this.constraints))+"$").test(this.textbox.value))){	// check for exponential notation that parse() rejected (erroneously?)
					var n = Number(this.textbox.value);
					return isNaN(n) ? undefined : n; // return exponential Number or undefined for random text (may not be possible to do with the above RegExp check)
				}else{
					return undefined; // gibberish
				}
			}else{
				return v; // Number or NaN for ''
			}
		},

		isValid: function(/*Boolean*/ isFocused){
			// Overrides dijit.form.RangeBoundTextBox.isValid to check that the editing-mode value is valid since
			// it may not be formatted according to the regExp vaidation rules
			if(!this._focused || this._isEmpty(this.textbox.value)){
				return this.inherited(arguments);
			}else{
				var v = this.get('value');
				if(!isNaN(v) && this.rangeCheck(v, this.constraints)){
					if(this.constraints.exponent !== false && /\de[-+]?\d/i.test(this.textbox.value)){ // exponential, parse doesn't like it
						return true; // valid exponential number in range
					}else{
						return this.inherited(arguments);
					}
				}else{
					return false;
				}
			}
		}
	}
);

dojo.declare("dijit.form.NumberTextBox",
	[dijit.form.RangeBoundTextBox,dijit.form.NumberTextBoxMixin],
	{
		// summary:
		//		A TextBox for entering numbers, with formatting and range checking
		// description:
		//		NumberTextBox is a textbox for entering and displaying numbers, supporting
		//		the following main features:
		//
		//			1. Enforce minimum/maximum allowed values (as well as enforcing that the user types
		//				a number rather than a random string)
		//			2. NLS support (altering roles of comma and dot as "thousands-separator" and "decimal-point"
		//				depending on locale).
		//			3. Separate modes for editing the value and displaying it, specifically that
		//				the thousands separator character (typically comma) disappears when editing
		//				but reappears after the field is blurred.
		//			4. Formatting and constraints regarding the number of places (digits after the decimal point)
		//				allowed on input, and number of places displayed when blurred (see `constraints` parameter).
	}
);

}

if(!dojo._hasResource["dijit.form.NumberSpinner"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.NumberSpinner"] = true;
dojo.provide("dijit.form.NumberSpinner");




dojo.declare("dijit.form.NumberSpinner",
	[dijit.form._Spinner, dijit.form.NumberTextBoxMixin],
	{
	// summary:
	//		Extends NumberTextBox to add up/down arrows and pageup/pagedown for incremental change to the value
	//
	// description:
	//		A `dijit.form.NumberTextBox` extension to provide keyboard accessible value selection
	//		as well as icons for spinning direction. When using the keyboard, the typematic rules
	//		apply, meaning holding the key will gradually increase or decrease the value and
	// 		accelerate.
	//
	// example:
	//	| new dijit.form.NumberSpinner({ constraints:{ max:300, min:100 }}, "someInput");

	adjust: function(/* Object */val, /* Number*/delta){
		// summary:
		//		Change Number val by the given amount
		// tags:
		//		protected

		var tc = this.constraints,
			v = isNaN(val),
			gotMax = !isNaN(tc.max),
			gotMin = !isNaN(tc.min)
		;
		if(v && delta != 0){ // blank or invalid value and they want to spin, so create defaults
			val = (delta > 0) ?
				gotMin ? tc.min : gotMax ? tc.max : 0 :
				gotMax ? this.constraints.max : gotMin ? tc.min : 0
			;
		}
		var newval = val + delta;
		if(v || isNaN(newval)){ return val; }
		if(gotMax && (newval > tc.max)){
			newval = tc.max;
		}
		if(gotMin && (newval < tc.min)){
			newval = tc.min;
		}
		return newval;
	},

	_onKeyPress: function(e){
		if((e.charOrCode == dojo.keys.HOME || e.charOrCode == dojo.keys.END) && !(e.ctrlKey || e.altKey || e.metaKey)
		&& typeof this.get('value') != 'undefined' /* gibberish, so HOME and END are default editing keys*/){
			var value = this.constraints[(e.charOrCode == dojo.keys.HOME ? "min" : "max")];
			if(typeof value == "number"){
				this._setValueAttr(value, false);
			}
			// eat home or end key whether we change the value or not
			dojo.stopEvent(e);
		}
	}
});

}

if(!dojo._hasResource["dijit.form.SimpleTextarea"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.SimpleTextarea"] = true;
dojo.provide("dijit.form.SimpleTextarea");



dojo.declare("dijit.form.SimpleTextarea",
	dijit.form.TextBox,
	{
	// summary:
	//		A simple textarea that degrades, and responds to
	// 		minimal LayoutContainer usage, and works with dijit.form.Form.
	//		Doesn't automatically size according to input, like Textarea.
	//
	// example:
	//	|	<textarea dojoType="dijit.form.SimpleTextarea" name="foo" value="bar" rows=30 cols=40></textarea>
	//
	// example:
	//	|	new dijit.form.SimpleTextarea({ rows:20, cols:30 }, "foo");

	baseClass: "dijitTextBox dijitTextArea",

	attributeMap: dojo.delegate(dijit.form._FormValueWidget.prototype.attributeMap, {
		rows:"textbox", cols: "textbox"
	}),

	// rows: Number
	//		The number of rows of text.
	rows: "3",

	// rows: Number
	//		The number of characters per line.
	cols: "20",

	templateString: "<textarea ${!nameAttrSetting} dojoAttachPoint='focusNode,containerNode,textbox' autocomplete='off'></textarea>",

	postMixInProperties: function(){
		// Copy value from srcNodeRef, unless user specified a value explicitly (or there is no srcNodeRef)
		if(!this.value && this.srcNodeRef){
			this.value = this.srcNodeRef.value;
		}
		this.inherited(arguments);
	},

	filter: function(/*String*/ value){
		// Override TextBox.filter to deal with newlines... specifically (IIRC) this is for IE which writes newlines
		// as \r\n instead of just \n
		if(value){
			value = value.replace(/\r/g,"");
		}
		return this.inherited(arguments);
	},

	postCreate: function(){
		this.inherited(arguments);
		if(dojo.isIE && this.cols){ // attribute selectors is not supported in IE6
			dojo.addClass(this.textbox, "dijitTextAreaCols");
		}
	},

	_previousValue: "",
	_onInput: function(/*Event?*/ e){
		// Override TextBox._onInput() to enforce maxLength restriction
		if(this.maxLength){
			var maxLength = parseInt(this.maxLength);
			var value = this.textbox.value.replace(/\r/g,'');
			var overflow = value.length - maxLength;
			if(overflow > 0){
				if(e){ dojo.stopEvent(e); }
				var textarea = this.textbox;
				if(textarea.selectionStart){
					var pos = textarea.selectionStart;
					var cr = 0;
					if(dojo.isOpera){
						cr = (this.textbox.value.substring(0,pos).match(/\r/g) || []).length;
					}
					this.textbox.value = value.substring(0,pos-overflow-cr)+value.substring(pos-cr);
					textarea.setSelectionRange(pos-overflow, pos-overflow);
				}else if(dojo.doc.selection){ //IE
					textarea.focus();
					var range = dojo.doc.selection.createRange();
					// delete overflow characters
					range.moveStart("character", -overflow);
					range.text = '';
					// show cursor
					range.select();
				}
			}
			this._previousValue = this.textbox.value;
		}
		this.inherited(arguments);
	}
});

}

if(!dojo._hasResource["dijit.form.Textarea"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Textarea"] = true;
dojo.provide("dijit.form.Textarea");



dojo.declare(
	"dijit.form.Textarea",
	dijit.form.SimpleTextarea,
	{
	// summary:
	//		A textarea widget that adjusts it's height according to the amount of data.
	//
	// description:
	//		A textarea that dynamically expands/contracts (changing it's height) as
	//		the user types, to display all the text without requiring a scroll bar.
	//
	//		Takes nearly all the parameters (name, value, etc.) that a vanilla textarea takes.
	//		Rows is not supported since this widget adjusts the height.
	//
	// example:
	// |	<textarea dojoType="dijit.form.TextArea">...</textarea>


	// Override SimpleTextArea.cols to default to width:100%, for backward compatibility
	cols: "",

	_previousNewlines: 0,
	_strictMode: (dojo.doc.compatMode != 'BackCompat'), // not the same as !dojo.isQuirks

	_getHeight: function(textarea){
		var newH = textarea.scrollHeight;
		if(dojo.isIE){
			newH += textarea.offsetHeight - textarea.clientHeight - ((dojo.isIE < 8 && this._strictMode) ? dojo._getPadBorderExtents(textarea).h : 0);
		}else if(dojo.isMoz){
			newH += textarea.offsetHeight - textarea.clientHeight; // creates room for horizontal scrollbar
		}else if(dojo.isWebKit && !(dojo.isSafari < 4)){ // Safari 4.0 && Chrome
			newH += dojo._getBorderExtents(textarea).h;
		}else{ // Safari 3.x and Opera 9.6
			newH += dojo._getPadBorderExtents(textarea).h;
		}
		return newH;
	},

	_estimateHeight: function(textarea){
		// summary:
		// 		Approximate the height when the textarea is invisible with the number of lines in the text.
		// 		Fails when someone calls setValue with a long wrapping line, but the layout fixes itself when the user clicks inside so . . .
		// 		In IE, the resize event is supposed to fire when the textarea becomes visible again and that will correct the size automatically.
		//
		textarea.style.maxHeight = "";
		textarea.style.height = "auto";
		// #rows = #newlines+1
		// Note: on Moz, the following #rows appears to be 1 too many.
		// Actually, Moz is reserving room for the scrollbar.
		// If you increase the font size, this behavior becomes readily apparent as the last line gets cut off without the +1.
		textarea.rows = (textarea.value.match(/\n/g) || []).length + 1;
	},

	_needsHelpShrinking: dojo.isMoz || dojo.isWebKit,

	_onInput: function(){
		// Override SimpleTextArea._onInput() to deal with height adjustment
		this.inherited(arguments);
		if(this._busyResizing){ return; }
		this._busyResizing = true;
		var textarea = this.textbox;
		if(textarea.scrollHeight && textarea.offsetHeight && textarea.clientHeight){
			var newH = this._getHeight(textarea) + "px";
			if(textarea.style.height != newH){
				textarea.style.maxHeight = textarea.style.height = newH;
			}
			if(this._needsHelpShrinking){
				if(this._setTimeoutHandle){
					clearTimeout(this._setTimeoutHandle);
				}
				this._setTimeoutHandle = setTimeout(dojo.hitch(this, "_shrink"), 0); // try to collapse multiple shrinks into 1
			}
		}else{
			// hidden content of unknown size
			this._estimateHeight(textarea);
		}
		this._busyResizing = false;
	},

	_busyResizing: false,
	_shrink: function(){
		// grow paddingBottom to see if scrollHeight shrinks (when it is unneccesarily big)
		this._setTimeoutHandle = null;
		if(this._needsHelpShrinking && !this._busyResizing){
			this._busyResizing = true;
			var textarea = this.textbox;
			var empty = false;
			if(textarea.value == ''){
				textarea.value = ' '; // prevent collapse all the way back to 0
				empty = true;
			}
			var scrollHeight = textarea.scrollHeight;
			if(!scrollHeight){
				this._estimateHeight(textarea);
			}else{
				var oldPadding = textarea.style.paddingBottom;
				var newPadding = dojo._getPadExtents(textarea);
				newPadding = newPadding.h - newPadding.t;
				textarea.style.paddingBottom = newPadding + 1 + "px"; // tweak padding to see if height can be reduced
				var newH = this._getHeight(textarea) - 1 + "px"; // see if the height changed by the 1px added
				if(textarea.style.maxHeight != newH){ // if can be reduced, so now try a big chunk
					textarea.style.paddingBottom = newPadding + scrollHeight + "px";
					textarea.scrollTop = 0;
					textarea.style.maxHeight = this._getHeight(textarea) - scrollHeight + "px"; // scrollHeight is the added padding
				}
				textarea.style.paddingBottom = oldPadding;
			}
			if(empty){
				textarea.value = '';
			}
			this._busyResizing = false;
		}
	},

	resize: function(){
		// summary:
		//		Resizes the textarea vertically (should be called after a style/value change)
		this._onInput();
	},

	_setValueAttr: function(){
		this.inherited(arguments);
		this.resize();
	},

	postCreate: function(){
		this.inherited(arguments);
		// tweak textarea style to reduce browser differences
		dojo.style(this.textbox, { overflowY: 'hidden', overflowX: 'auto', boxSizing: 'border-box', MsBoxSizing: 'border-box', WebkitBoxSizing: 'border-box', MozBoxSizing: 'border-box' });
		this.connect(this.textbox, "onscroll", this._onInput);
		this.connect(this.textbox, "onresize", this._onInput);
		this.connect(this.textbox, "onfocus", this._onInput); // useful when a previous estimate was off a bit
		this._setTimeoutHandle = setTimeout(dojo.hitch(this, "resize"), 0);
	},

	uninitialize: function(){
		if(this._setTimeoutHandle){
			clearTimeout(this._setTimeoutHandle);
		}
		this.inherited(arguments);
	}
});

}

if(!dojo._hasResource["dijit.form._FormSelectWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormSelectWidget"] = true;
dojo.provide("dijit.form._FormSelectWidget");




/*=====
dijit.form.__SelectOption = function(){
	// value: String
	//		The value of the option.  Setting to empty (or missing) will
	//		place a separator at that location
	// label: String
	//		The label for our option.  It can contain html tags.
	//  selected: Boolean
	//		Whether or not we are a selected option
	// disabled: Boolean
	//		Whether or not this specific option is disabled
	this.value = value;
	this.label = label;
	this.selected = selected;
	this.disabled = disabled;
}
=====*/

dojo.declare("dijit.form._FormSelectWidget", dijit.form._FormValueWidget, {
	// summary:
	//		Extends _FormValueWidget in order to provide "select-specific"
	//		values - i.e., those values that are unique to <select> elements.
	//		This also provides the mechanism for reading the elements from
	//		a store, if desired.

	// multiple: Boolean
	//		Whether or not we are multi-valued
	multiple: false,

	// options: dijit.form.__SelectOption[]
	//		The set of options for our select item.  Roughly corresponds to
	//      the html <option> tag.
	options: null,

	// store: dojo.data.api.Identity
	//		A store which, at the very least impelements dojo.data.api.Identity
	//		to use for getting our list of options - rather than reading them
	//		from the <option> html tags.
	store: null,

	// query: object
	//		A query to use when fetching items from our store
	query: null,

	// queryOptions: object
	//		Query options to use when fetching from the store
	queryOptions: null,

	// onFetch: Function
	//		A callback to do with an onFetch - but before any items are actually
	//		iterated over (i.e. to filter even futher what you want to add)
	onFetch: null,

	// sortByLabel: boolean
	//		Flag to sort the options returned from a store by the label of
	//		the store.
	sortByLabel: true,


	// loadChildrenOnOpen: boolean
	//		By default loadChildren is called when the items are fetched from the
	//		store.  This property allows delaying loadChildren (and the creation
	//		of the options/menuitems) until the user opens the click the button.
	//		dropdown
	loadChildrenOnOpen: false,

	getOptions: function(/* anything */ valueOrIdx){
		// summary:
		//		Returns a given option (or options).
		// valueOrIdx:
		//		If passed in as a string, that string is used to look up the option
		//		in the array of options - based on the value property.
		//		(See dijit.form.__SelectOption).
		//
		//		If passed in a number, then the option with the given index (0-based)
		//		within this select will be returned.
		//
		//		If passed in a dijit.form.__SelectOption, the same option will be
		//		returned if and only if it exists within this select.
		//
		//		If passed an array, then an array will be returned with each element
		//		in the array being looked up.
		//
		//		If not passed a value, then all options will be returned
		//
		// returns:
		//		The option corresponding with the given value or index.  null
		//		is returned if any of the following are true:
		//			- A string value is passed in which doesn't exist
		//			- An index is passed in which is outside the bounds of the array of options
		//			- A dijit.form.__SelectOption is passed in which is not a part of the select

		// NOTE: the compare for passing in a dijit.form.__SelectOption checks
		//		if the value property matches - NOT if the exact option exists
		// NOTE: if passing in an array, null elements will be placed in the returned
		//		array when a value is not found.
		var lookupValue = valueOrIdx, opts = this.options || [], l = opts.length;

		if(lookupValue === undefined){
			return opts; // dijit.form.__SelectOption[]
		}
		if(dojo.isArray(lookupValue)){
			return dojo.map(lookupValue, "return this.getOptions(item);", this); // dijit.form.__SelectOption[]
		}
		if(dojo.isObject(valueOrIdx)){
			// We were passed an option - so see if it's in our array (directly),
			// and if it's not, try and find it by value.
			if(!dojo.some(this.options, function(o, idx){
				if(o === lookupValue ||
					(o.value && o.value === lookupValue.value)){
					lookupValue = idx;
					return true;
				}
				return false;
			})){
				lookupValue = -1;
			}
		}
		if(typeof lookupValue == "string"){
			for(var i=0; i<l; i++){
				if(opts[i].value === lookupValue){
					lookupValue = i;
					break;
				}
			}
		}
		if(typeof lookupValue == "number" && lookupValue >= 0 && lookupValue < l){
			return this.options[lookupValue] // dijit.form.__SelectOption
		}
		return null; // null
	},

	addOption: function(/* dijit.form.__SelectOption, dijit.form.__SelectOption[] */ option){
		// summary:
		//		Adds an option or options to the end of the select.  If value
		//		of the option is empty or missing, a separator is created instead.
		//		Passing in an array of options will yield slightly better performance
		//		since the children are only loaded once.
		if(!dojo.isArray(option)){ option = [option]; }
		dojo.forEach(option, function(i){
			if(i && dojo.isObject(i)){
				this.options.push(i);
			}
		}, this);
		this._loadChildren();
	},

	removeOption: function(/* string, dijit.form.__SelectOption, number, or array */ valueOrIdx){
		// summary:
		//		Removes the given option or options.  You can remove by string
		//		(in which case the value is removed), number (in which case the
		//		index in the options array is removed), or select option (in
		//		which case, the select option with a matching value is removed).
		//		You can also pass in an array of those values for a slightly
		//		better performance since the children are only loaded once.
		if(!dojo.isArray(valueOrIdx)){ valueOrIdx = [valueOrIdx]; }
		var oldOpts = this.getOptions(valueOrIdx);
		dojo.forEach(oldOpts, function(i){
			// We can get null back in our array - if our option was not found.  In
			// that case, we don't want to blow up...
			if(i){
				this.options = dojo.filter(this.options, function(node, idx){
					return (node.value !== i.value);
				});
				this._removeOptionItem(i);
			}
		}, this);
		this._loadChildren();
	},

	updateOption: function(/* dijit.form.__SelectOption, dijit.form.__SelectOption[] */ newOption){
		// summary:
		//		Updates the values of the given option.  The option to update
		//		is matched based on the value of the entered option.  Passing
		//		in an array of new options will yeild better performance since
		//		the children will only be loaded once.
		if(!dojo.isArray(newOption)){ newOption = [newOption]; }
		dojo.forEach(newOption, function(i){
			var oldOpt = this.getOptions(i), k;
			if(oldOpt){
				for(k in i){ oldOpt[k] = i[k]; }
			}
		}, this);
		this._loadChildren();
	},

	setStore: function(/* dojo.data.api.Identity */ store,
						/* anything? */ selectedValue,
						/* Object? */ fetchArgs){
		// summary:
		//		Sets the store you would like to use with this select widget.
		//		The selected value is the value of the new store to set.  This
		//		function returns the original store, in case you want to reuse
		//		it or something.
		// store: dojo.data.api.Identity
		//		The store you would like to use - it MUST implement Identity,
		//		and MAY implement Notification.
		// selectedValue: anything?
		//		The value that this widget should set itself to *after* the store
		//		has been loaded
		// fetchArgs: Object?
		//		The arguments that will be passed to the store's fetch() function
		var oStore = this.store;
		fetchArgs = fetchArgs || {};
		if(oStore !== store){
			// Our store has changed, so update our notifications
			dojo.forEach(this._notifyConnections || [], dojo.disconnect);
			delete this._notifyConnections;
			if(store && store.getFeatures()["dojo.data.api.Notification"]){
				this._notifyConnections = [
					dojo.connect(store, "onNew", this, "_onNewItem"),
					dojo.connect(store, "onDelete", this, "_onDeleteItem"),
					dojo.connect(store, "onSet", this, "_onSetItem")
				];
			}
			this.store = store;
		}

		// Turn off change notifications while we make all these changes
		this._onChangeActive = false;

		// Remove existing options (if there are any)
		if(this.options && this.options.length){
			this.removeOption(this.options);
		}

		// Add our new options
		if(store){
			var cb = function(items){
				if(this.sortByLabel && !fetchArgs.sort && items.length){
					items.sort(dojo.data.util.sorter.createSortFunction([{
						attribute: store.getLabelAttributes(items[0])[0]
					}], store));
				}

				if(fetchArgs.onFetch){
					items = fetchArgs.onFetch(items);
				}
				// TODO: Add these guys as a batch, instead of separately
				dojo.forEach(items, function(i){
					this._addOptionForItem(i);
				}, this);

				// Set our value (which might be undefined), and then tweak
				// it to send a change event with the real value
				this._loadingStore = false;
				this.set("value", (("_pendingValue" in this) ? this._pendingValue : selectedValue));
				delete this._pendingValue;

				if(!this.loadChildrenOnOpen){
					this._loadChildren();
				}else{
					this._pseudoLoadChildren(items);
				}
				this._fetchedWith = opts;
				this._lastValueReported = this.multiple ? [] : null;
				this._onChangeActive = true;
				this.onSetStore();
				this._handleOnChange(this.value);
			};
			var opts = dojo.mixin({onComplete:cb, scope: this}, fetchArgs);
			this._loadingStore = true;
			store.fetch(opts);
		}else{
			delete this._fetchedWith;
		}
		return oStore;	// dojo.data.api.Identity
	},

	_setValueAttr: function(/*anything*/ newValue, /*Boolean, optional*/ priorityChange){
		// summary:
		//		set the value of the widget.
		//		If a string is passed, then we set our value from looking it up.
		if(this._loadingStore){
			// Our store is loading - so save our value, and we'll set it when
			// we're done
			this._pendingValue = newValue;
			return;
		}
		var opts = this.getOptions() || [];
		if(!dojo.isArray(newValue)){
			newValue = [newValue];
		}
		dojo.forEach(newValue, function(i, idx){
			if(!dojo.isObject(i)){
				i = i + "";
			}
			if(typeof i === "string"){
				newValue[idx] = dojo.filter(opts, function(node){
					return node.value === i;
				})[0] || {value: "", label: ""};
			}
		}, this);

		// Make sure some sane default is set
		newValue = dojo.filter(newValue, function(i){ return i && i.value; });
		if(!this.multiple && (!newValue[0] || !newValue[0].value) && opts.length){
			newValue[0] = opts[0];
		}
		dojo.forEach(opts, function(i){
			i.selected = dojo.some(newValue, function(v){ return v.value === i.value; });
		});
		var val = dojo.map(newValue, function(i){ return i.value; }),
			disp = dojo.map(newValue, function(i){ return i.label; });

		this.value = this.multiple ? val : val[0];
		this._setDisplay(this.multiple ? disp : disp[0]);
		this._updateSelection();
		this._handleOnChange(this.value, priorityChange);
	},

	_getDisplayedValueAttr: function(){
		// summary:
		//		returns the displayed value of the widget
		var val = this.get("value");
		if(!dojo.isArray(val)){
			val = [val];
		}
		var ret = dojo.map(this.getOptions(val), function(v){
			if(v && "label" in v){
				return v.label;
			}else if(v){
				return v.value;
			}
			return null;
		}, this);
		return this.multiple ? ret : ret[0];
	},

	_getValueDeprecated: false, // remove when _FormWidget:getValue is removed
	getValue: function(){
		// summary:
		//		get the value of the widget.
		return this._lastValue;
	},

	undo: function(){
		// summary:
		//		restore the value to the last value passed to onChange
		this._setValueAttr(this._lastValueReported, false);
	},

	_loadChildren: function(){
		// summary:
		//		Loads the children represented by this widget's options.
		//		reset the menu to make it "populatable on the next click
		if(this._loadingStore){ return; }
		dojo.forEach(this._getChildren(), function(child){
			child.destroyRecursive();
		});
		// Add each menu item
		dojo.forEach(this.options, this._addOptionItem, this);

		// Update states
		this._updateSelection();
	},

	_updateSelection: function(){
		// summary:
		//		Sets the "selected" class on the item for styling purposes
		this.value = this._getValueFromOpts();
		var val = this.value;
		if(!dojo.isArray(val)){
			val = [val];
		}
		if(val && val[0]){
			dojo.forEach(this._getChildren(), function(child){
				var isSelected = dojo.some(val, function(v){
					return child.option && (v === child.option.value);
				});
				dojo.toggleClass(child.domNode, this.baseClass + "SelectedOption", isSelected);
				dijit.setWaiState(child.domNode, "selected", isSelected);
			}, this);
		}
		this._handleOnChange(this.value);
	},

	_getValueFromOpts: function(){
		// summary:
		//		Returns the value of the widget by reading the options for
		//		the selected flag
		var opts = this.getOptions() || [];
		if(!this.multiple && opts.length){
			// Mirror what a select does - choose the first one
			var opt = dojo.filter(opts, function(i){
				return i.selected;
			})[0];
			if(opt && opt.value){
				return opt.value
			}else{
				opts[0].selected = true;
				return opts[0].value;
			}
		}else if(this.multiple){
			// Set value to be the sum of all selected
			return dojo.map(dojo.filter(opts, function(i){
				return i.selected;
			}), function(i){
				return i.value;
			}) || [];
		}
		return "";
	},

	// Internal functions to call when we have store notifications come in
	_onNewItem: function(/* item */ item, /* Object? */ parentInfo){
		if(!parentInfo || !parentInfo.parent){
			// Only add it if we are top-level
			this._addOptionForItem(item);
		}
	},
	_onDeleteItem: function(/* item */ item){
		var store = this.store;
		this.removeOption(store.getIdentity(item));
	},
	_onSetItem: function(/* item */ item){
		this.updateOption(this._getOptionObjForItem(item));
	},

	_getOptionObjForItem: function(item){
		// summary:
		//		Returns an option object based off the given item.  The "value"
		//		of the option item will be the identity of the item, the "label"
		//		of the option will be the label of the item.  If the item contains
		//		children, the children value of the item will be set
		var store = this.store, label = store.getLabel(item),
			value = (label ? store.getIdentity(item) : null);
		return {value: value, label: label, item:item}; // dijit.form.__SelectOption
	},

	_addOptionForItem: function(/* item */ item){
		// summary:
		//		Creates (and adds) the option for the given item
		var store = this.store;
		if(!store.isItemLoaded(item)){
			// We are not loaded - so let's load it and add later
			store.loadItem({item: item, onComplete: function(i){
				this._addOptionForItem(item);
			},
			scope: this});
			return;
		}
		var newOpt = this._getOptionObjForItem(item);
		this.addOption(newOpt);
	},

	constructor: function(/* Object */ keywordArgs){
		// summary:
		//		Saves off our value, if we have an initial one set so we
		//		can use it if we have a store as well (see startup())
		this._oValue = (keywordArgs || {}).value || null;
	},

	_fillContent: function(){
		// summary:
		//		Loads our options and sets up our dropdown correctly.  We
		//		don't want any content, so we don't call any inherit chain
		//		function.
		var opts = this.options;
		if(!opts){
			opts = this.options = this.srcNodeRef ? dojo.query(">",
						this.srcNodeRef).map(function(node){
							if(node.getAttribute("type") === "separator"){
								return { value: "", label: "", selected: false, disabled: false };
							}
							return { value: node.getAttribute("value"),
										label: String(node.innerHTML),
										selected: node.getAttribute("selected") || false,
										disabled: node.getAttribute("disabled") || false };
						}, this) : [];
		}
		if(!this.value){
			this.value = this._getValueFromOpts();
		}else if(this.multiple && typeof this.value == "string"){
			this.value = this.value.split(",");
		}
	},

	postCreate: function(){
		// summary:
		//		sets up our event handling that we need for functioning
		//		as a select
		dojo.setSelectable(this.focusNode, false);
		this.inherited(arguments);

		// Make our event connections for updating state
		this.connect(this, "onChange", "_updateSelection");
		this.connect(this, "startup", "_loadChildren");

		this._setValueAttr(this.value, null);
	},

	startup: function(){
		// summary:
		//		Connects in our store, if we have one defined
		this.inherited(arguments);
		var store = this.store, fetchArgs = {};
		dojo.forEach(["query", "queryOptions", "onFetch"], function(i){
			if(this[i]){
				fetchArgs[i] = this[i];
			}
			delete this[i];
		}, this);
		if(store && store.getFeatures()["dojo.data.api.Identity"]){
			// Temporarily set our store to null so that it will get set
			// and connected appropriately
			this.store = null;
			this.setStore(store, this._oValue, fetchArgs);
		}
	},

	destroy: function(){
		// summary:
		//		Clean up our connections
		dojo.forEach(this._notifyConnections || [], dojo.disconnect);
		this.inherited(arguments);
	},

	_addOptionItem: function(/* dijit.form.__SelectOption */ option){
		// summary:
		//		User-overridable function which, for the given option, adds an
		//		item to the select.  If the option doesn't have a value, then a
		//		separator is added in that place.  Make sure to store the option
		//		in the created option widget.
	},

	_removeOptionItem: function(/* dijit.form.__SelectOption */ option){
		// summary:
		//		User-overridable function which, for the given option, removes
		//		its item from the select.
	},

	_setDisplay: function(/*String or String[]*/ newDisplay){
		// summary:
		//		Overridable function which will set the display for the
		//		widget.  newDisplay is either a string (in the case of
		//		single selects) or array of strings (in the case of multi-selects)
	},

	_getChildren: function(){
		// summary:
		//		Overridable function to return the children that this widget contains.
		return [];
	},

	_getSelectedOptionsAttr: function(){
		// summary:
		//		hooks into this.attr to provide a mechanism for getting the
		//		option items for the current value of the widget.
		return this.getOptions(this.get("value"));
	},

	_pseudoLoadChildren: function(/* item[] */ items){
		// summary:
		//		a function that will "fake" loading children, if needed, and
		//		if we have set to not load children until the widget opens.
		// items:
		//		An array of items that will be loaded, when needed
	},

	onSetStore: function(){
		// summary:
		//		a function that can be connected to in order to receive a
		//		notification that the store has finished loading and all options
		//		from that store are available
	}
});

}

if(!dojo._hasResource["dijit.form.Select"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Select"] = true;
dojo.provide("dijit.form.Select");








dojo.declare("dijit.form._SelectMenu", dijit.Menu, {
	// summary:
	//		An internally-used menu for dropdown that allows us a vertical scrollbar
	buildRendering: function(){
		// summary:
		//		Stub in our own changes, so that our domNode is not a table
		//		otherwise, we won't respond correctly to heights/overflows
		this.inherited(arguments);
		var o = (this.menuTableNode = this.domNode);
		var n = (this.domNode = dojo.create("div", {style: {overflowX: "hidden", overflowY: "scroll"}}));
		if(o.parentNode){
			o.parentNode.replaceChild(n, o);
		}
		dojo.removeClass(o, "dijitMenuTable");
		n.className = o.className + " dijitSelectMenu";
		o.className = "dijitReset dijitMenuTable";
		dijit.setWaiRole(o,"listbox");
		dijit.setWaiRole(n,"presentation");
		n.appendChild(o);
	},
	resize: function(/*Object*/ mb){
		// summary:
		//		Overridden so that we are able to handle resizing our
		//		internal widget.  Note that this is not a "full" resize
		//		implementation - it only works correctly if you pass it a
		//		marginBox.
		//
		// mb: Object
		//		The margin box to set this dropdown to.
		if(mb){
			dojo.marginBox(this.domNode, mb);
			if("w" in mb){
				// We've explicitly set the wrapper <div>'s width, so set <table> width to match.
				// 100% is safer than a pixel value because there may be a scroll bar with
				// browser/OS specific width.
				this.menuTableNode.style.width = "100%";
			}
		}
	}
});

dojo.declare("dijit.form.Select", [dijit.form._FormSelectWidget, dijit._HasDropDown], {
	// summary:
	//		This is a "styleable" select box - it is basically a DropDownButton which
	//		can take a <select> as its input.

	baseClass: "dijitSelect",

	templateString: dojo.cache("dijit.form", "templates/Select.html", "<table class=\"dijit dijitReset dijitInline dijitLeft\"\r\n\tdojoAttachPoint=\"_buttonNode,tableNode,focusNode\" cellspacing='0' cellpadding='0'\r\n\twaiRole=\"combobox\" waiState=\"haspopup-true\"\r\n\t><tbody waiRole=\"presentation\"><tr waiRole=\"presentation\"\r\n\t\t><td class=\"dijitReset dijitStretch dijitButtonContents dijitButtonNode\" waiRole=\"presentation\"\r\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"  dojoAttachPoint=\"containerNode,_popupStateNode\"></span\r\n\t\t\t><input type=\"hidden\" ${!nameAttrSetting} dojoAttachPoint=\"valueNode\" value=\"${value}\" waiState=\"hidden-true\"\r\n\t\t/></td><td class=\"dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton\"\r\n\t\t\t\tdojoAttachPoint=\"titleNode\" waiRole=\"presentation\"\r\n\t\t\t><div class=\"dijitReset dijitArrowButtonInner\" waiRole=\"presentation\"></div\r\n\t\t\t><div class=\"dijitReset dijitArrowButtonChar\" waiRole=\"presentation\">&#9660;</div\r\n\t\t></td\r\n\t></tr></tbody\r\n></table>\r\n"),

	// attributeMap: Object
	//		Add in our style to be applied to the focus node
	attributeMap: dojo.mixin(dojo.clone(dijit.form._FormSelectWidget.prototype.attributeMap),{style:"tableNode"}),

	// required: Boolean
	//		Can be true or false, default is false.
	required: false,

	// state: String
	//		Shows current state (ie, validation result) of input (Normal, Warning, or Error)
	state: "",

	//	tooltipPosition: String[]
	//		See description of dijit.Tooltip.defaultPosition for details on this parameter.
	tooltipPosition: [],

	// emptyLabel: string
	//		What to display in an "empty" dropdown
	emptyLabel: "",

	// _isLoaded: Boolean
	//		Whether or not we have been loaded
	_isLoaded: false,

	// _childrenLoaded: Boolean
	//		Whether or not our children have been loaded
	_childrenLoaded: false,

	_fillContent: function(){
		// summary:
		//		Set the value to be the first, or the selected index
		this.inherited(arguments);
		if(this.options.length && !this.value && this.srcNodeRef){
			var si = this.srcNodeRef.selectedIndex;
			this.value = this.options[si != -1 ? si : 0].value;
		}

		// Create the dropDown widget
		this.dropDown = new dijit.form._SelectMenu({id: this.id + "_menu"});
		dojo.addClass(this.dropDown.domNode, this.baseClass + "Menu");
	},

	_getMenuItemForOption: function(/*dijit.form.__SelectOption*/ option){
		// summary:
		//		For the given option, return the menu item that should be
		//		used to display it.  This can be overridden as needed
		if(!option.value){
			// We are a separator (no label set for it)
			return new dijit.MenuSeparator();
		}else{
			// Just a regular menu option
			var click = dojo.hitch(this, "_setValueAttr", option);
			var item = new dijit.MenuItem({
				option: option,
				label: option.label,
				onClick: click,
				disabled: option.disabled || false
			});
			dijit.setWaiRole(item.focusNode, "listitem");
			return item;
		}
	},

	_addOptionItem: function(/*dijit.form.__SelectOption*/ option){
		// summary:
		//		For the given option, add an option to our dropdown.
		//		If the option doesn't have a value, then a separator is added
		//		in that place.
		if(this.dropDown){
			this.dropDown.addChild(this._getMenuItemForOption(option));
		}
	},

	_getChildren: function(){
		if(!this.dropDown){
			return [];
		}
		return this.dropDown.getChildren();
	},

	_loadChildren: function(/*Boolean*/ loadMenuItems){
		// summary:
		//		Resets the menu and the length attribute of the button - and
		//		ensures that the label is appropriately set.
		//	loadMenuItems: Boolean
		//		actually loads the child menu items - we only do this when we are
		//		populating for showing the dropdown.

		if(loadMenuItems === true){
			// this.inherited destroys this.dropDown's child widgets (MenuItems).
			// Avoid this.dropDown (Menu widget) having a pointer to a destroyed widget (which will cause
			// issues later in _setSelected). (see #10296)
			if(this.dropDown){
				delete this.dropDown.focusedChild;
			}
			if(this.options.length){
				this.inherited(arguments);
			}else{
				// Drop down menu is blank but add one blank entry just so something appears on the screen
				// to let users know that they are no choices (mimicing native select behavior)
				dojo.forEach(this._getChildren(), function(child){ child.destroyRecursive(); });
				var item = new dijit.MenuItem({label: "&nbsp;"});
				this.dropDown.addChild(item);
			}
		}else{
			this._updateSelection();
		}

		var len = this.options.length;
		this._isLoaded = false;
		this._childrenLoaded = true;

		if(!this._loadingStore){
			// Don't call this if we are loading - since we will handle it later
			this._setValueAttr(this.value);
		}
	},

	_setValueAttr: function(value){
		this.inherited(arguments);
		dojo.attr(this.valueNode, "value", this.get("value"));
	},

	_setDisplay: function(/*String*/ newDisplay){
		// summary:
		//		sets the display for the given value (or values)
		this.containerNode.innerHTML = '<span class="dijitReset dijitInline ' + this.baseClass + 'Label">' +
					(newDisplay || this.emptyLabel || "&nbsp;") +
					'</span>';
		dijit.setWaiState(this.focusNode, "valuetext", (newDisplay || this.emptyLabel || "&nbsp;") );
	},

	validate: function(/*Boolean*/ isFocused){
		// summary:
		//		Called by oninit, onblur, and onkeypress.
		// description:
		//		Show missing or invalid messages if appropriate, and highlight textbox field.
		//		Used when a select is initially set to no value and the user is required to
		//		set the value.
		
		var isValid = this.isValid(isFocused);
		this.state = isValid ? "" : "Error";
		this._setStateClass();
		dijit.setWaiState(this.focusNode, "invalid", isValid ? "false" : "true");
		var message = isValid ? "" : this._missingMsg;
		if(this._message !== message){
			this._message = message;
			dijit.hideTooltip(this.domNode);
			if(message){
				dijit.showTooltip(message, this.domNode, this.tooltipPosition, !this.isLeftToRight());
			}
		}
		return isValid;
	},

	isValid: function(/*Boolean*/ isFocused){
		// summary:
		//		Whether or not this is a valid value.   The only way a Select
		//		can be invalid is when it's required but nothing is selected.
		return (!this.required || !(/^\s*$/.test(this.value)));
	},

	reset: function(){
		// summary:
		//		Overridden so that the state will be cleared.
		this.inherited(arguments);
		dijit.hideTooltip(this.domNode);
		this.state = "";
		this._setStateClass();
		delete this._message;
	},

	postMixInProperties: function(){
		// summary:
		//		set the missing message
		this.inherited(arguments);
		this._missingMsg = dojo.i18n.getLocalization("dijit.form", "validate",
									this.lang).missingMessage;
	},

	postCreate: function(){
		this.inherited(arguments);
		if(this.tableNode.style.width){
			dojo.addClass(this.domNode, this.baseClass + "FixedWidth");
		}
	},

	isLoaded: function(){
		return this._isLoaded;
	},

	loadDropDown: function(/*Function*/ loadCallback){
		// summary:
		//		populates the menu
		this._loadChildren(true);
		this._isLoaded = true;
		loadCallback();
	},

	closeDropDown: function(){
		// overriding _HasDropDown.closeDropDown()
		this.inherited(arguments);

		if(this.dropDown && this.dropDown.menuTableNode){
			// Erase possible width: 100% setting from _SelectMenu.resize().
			// Leaving it would interfere with the next openDropDown() call, which
			// queries the natural size of the drop down.
			this.dropDown.menuTableNode.style.width = "";
		}
	},

	uninitialize: function(preserveDom){
		if(this.dropDown && !this.dropDown._destroyed){
			this.dropDown.destroyRecursive(preserveDom);
			delete this.dropDown;
		}
		this.inherited(arguments);
	}
});

}

if(!dojo._hasResource["dojox.form.DropDownSelect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.form.DropDownSelect"] = true;
dojo.deprecated("dojox.form.DropDownSelect", "Use dijit.form.Select instead", "2.0");

dojo.provide("dojox.form.DropDownSelect");


dojo.setObject("dojox.form.DropDownSelect", dijit.form.Select);

}

if(!dojo._hasResource["dijit.form._DateTimeTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._DateTimeTextBox"] = true;
dojo.provide("dijit.form._DateTimeTextBox");






new Date("X"); // workaround for #11279, new Date("") == NaN

/*=====
dojo.declare(
	"dijit.form._DateTimeTextBox.__Constraints",
	[dijit.form.RangeBoundTextBox.__Constraints, dojo.date.locale.__FormatOptions], {
	// summary:
	//		Specifies both the rules on valid/invalid values (first/last date/time allowed),
	//		and also formatting options for how the date/time is displayed.
	// example:
	//		To restrict to dates within 2004, displayed in a long format like "December 25, 2005":
	//	|		{min:'2004-01-01',max:'2004-12-31', formatLength:'long'}
});
=====*/

dojo.declare(
	"dijit.form._DateTimeTextBox",
	dijit.form.RangeBoundTextBox,
	{
		// summary:
		//		Base class for validating, serializable, range-bound date or time text box.

		// constraints: dijit.form._DateTimeTextBox.__Constraints
		//		Despite the name, this parameter specifies both constraints on the input
		//		(including starting/ending dates/times allowed) as well as
		//		formatting options like whether the date is displayed in long (ex: December 25, 2005)
		//		or short (ex: 12/25/2005) format.   See `dijit.form._DateTimeTextBox.__Constraints` for details.
		/*=====
		constraints: {},
		======*/

		// Override ValidationTextBox.regExpGen().... we use a reg-ex generating function rather
		// than a straight regexp to deal with locale  (plus formatting options too?)
		regExpGen: dojo.date.locale.regexp,

		// datePackage: String
		//	JavaScript namespace to find calendar routines.  Uses Gregorian calendar routines
		//	at dojo.date, by default.
		datePackage: "dojo.date",

		// Override _FormWidget.compare() to work for dates/times
		compare: dojo.date.compare,

		format: function(/*Date*/ value, /*dojo.date.locale.__FormatOptions*/ constraints){
			// summary:
			//		Formats the value as a Date, according to specified locale (second argument)
			// tags:
			//		protected
			if(!value){ return ''; }
			return this.dateLocaleModule.format(value, constraints);
		},

		parse: function(/*String*/ value, /*dojo.date.locale.__FormatOptions*/ constraints){
			// summary:
			//		Parses as string as a Date, according to constraints
			// tags:
			//		protected

			return this.dateLocaleModule.parse(value, constraints) || (this._isEmpty(value) ? null : undefined);	 // Date
		},

		// Overrides ValidationTextBox.serialize() to serialize a date in canonical ISO format.
		serialize: function(/*anything*/val, /*Object?*/options){
			if(val.toGregorian){
				val = val.toGregorian();
			}
			return dojo.date.stamp.toISOString(val, options);
		},

		// value: Date
		//		The value of this widget as a JavaScript Date object.  Use get("value") / set("value", val) to manipulate.
		//		When passed to the parser in markup, must be specified according to `dojo.date.stamp.fromISOString`
		value: new Date(""),	// value.toString()="NaN"
		_blankValue: null,	// used by filter() when the textbox is blank

		//	popupClass: [protected extension] String
		//		Name of the popup widget class used to select a date/time.
		//		Subclasses should specify this.
		popupClass: "", // default is no popup = text only


		// _selector: [protected extension] String
		//		Specifies constraints.selector passed to dojo.date functions, should be either
		//		"date" or "time".
		//		Subclass must specify this.
		_selector: "",

		constructor: function(/*Object*/args){
			var dateClass = args.datePackage ? args.datePackage + ".Date" : "Date";
			this.dateClassObj = dojo.getObject(dateClass, false);
			this.value = new this.dateClassObj("");

			this.datePackage = args.datePackage || this.datePackage;
			this.dateLocaleModule = dojo.getObject(this.datePackage + ".locale", false);
			this.regExpGen = this.dateLocaleModule.regexp;
		},

		_setConstraintsAttr: function(/* Object */ constraints){
			constraints.selector = this._selector;
			constraints.fullYear = true; // see #5465 - always format with 4-digit years
			var fromISO = dojo.date.stamp.fromISOString;
			if(typeof constraints.min == "string"){ constraints.min = fromISO(constraints.min); }
 			if(typeof constraints.max == "string"){ constraints.max = fromISO(constraints.max); }
			this.inherited(arguments, [constraints]);
		},

		_onFocus: function(/*Event*/ evt){
			// summary:
			//		open the popup
			this._open();
			this.inherited(arguments);
		},

		_setValueAttr: function(/*Date*/ value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Sets the date on this textbox.  Note that `value` must be like a Javascript Date object.
			if(value !== undefined){
				if(!value || value.toString() == dijit.form._DateTimeTextBox.prototype.value.toString()){
					value = null;
				}
				if(value instanceof Date && !(this.dateClassObj instanceof Date)){
					value = new this.dateClassObj(value);
				}
			}
			this.inherited(arguments, [value, priorityChange, formattedValue]);
			if(this._picker){
				// #3948: fix blank date on popup only
				if(!value){value = new this.dateClassObj();}
				this._picker.set('value', value);
			}
		},

		_open: function(){
			// summary:
			//		opens the TimePicker, and sets the onValueSelected value

			if(this.disabled || this.readOnly || !this.popupClass){return;}

			var textBox = this;

			if(!this._picker){
				var PopupProto = dojo.getObject(this.popupClass, false);
				this._picker = new PopupProto({
					onValueSelected: function(value){
						if(textBox._tabbingAway){
							delete textBox._tabbingAway;
						}else{
							textBox.focus(); // focus the textbox before the popup closes to avoid reopening the popup
						}
						setTimeout(dojo.hitch(textBox, "_close"), 1); // allow focus time to take

						// this will cause InlineEditBox and other handlers to do stuff so make sure it's last
						dijit.form._DateTimeTextBox.superclass._setValueAttr.call(textBox, value, true);
					},
					id: this.id + "_popup",
					dir: textBox.dir,
					lang: textBox.lang,
					value: this.get('value') || new this.dateClassObj(),
					constraints: textBox.constraints,

					datePackage: textBox.datePackage,

					isDisabledDate: function(/*Date*/ date){
						// summary:
						// 	disables dates outside of the min/max of the _DateTimeTextBox
						var compare = dojo.date.compare;
						var constraints = textBox.constraints;
						return constraints && (
							(constraints.min && compare(constraints.min, date, textBox._selector) > 0) ||
							(constraints.max && compare(constraints.max, date, textBox._selector) < 0)
						);
					}
				});
			}
			if(!this._opened){
				// Open drop down.  Align left sides of input box and drop down, even in RTL mode,
				// otherwise positioning thrown off when the drop down width is changed in marginBox call below (#10676)
				dijit.popup.open({
					parent: this,
					popup: this._picker,
					orient: {'BL':'TL', 'TL':'BL'},
					around: this.domNode,
					onCancel: dojo.hitch(this, this._close),
					onClose: function(){ textBox._opened=false; }
				});
				this._opened=true;
			}

			dojo.marginBox(this._picker.domNode,{ w:this.domNode.offsetWidth });
		},

		_close: function(){
			if(this._opened){
				dijit.popup.close(this._picker);
				this._opened=false;
			}
		},

		_onBlur: function(){
			// summary:
			//		Called magically when focus has shifted away from this widget and it's dropdown
			this._close();
			if(this._picker){
				// teardown so that constraints will be rebuilt next time (redundant reference: #6002)
				this._picker.destroy();
				delete this._picker;
			}
			this.inherited(arguments);
			// don't focus on <input>.  the user has explicitly focused on something else.
		},

		_getDisplayedValueAttr: function(){
			return this.textbox.value;
		},

		_setDisplayedValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange){
			this._setValueAttr(this.parse(value, this.constraints), priorityChange, value);
		},

		destroy: function(){
			if(this._picker){
				this._picker.destroy();
				delete this._picker;
			}
			this.inherited(arguments);
		},

		postCreate: function(){
			this.inherited(arguments);
			this.connect(this.focusNode, 'onkeypress', this._onKeyPress);
			this.connect(this.focusNode, 'onclick', this._open);
		},

		_onKeyPress: function(/*Event*/ e){
			// summary:
			//		Handler for keypress events

			var p = this._picker, dk = dojo.keys;
			// Handle the key in the picker, if it has a handler.  If the handler
			// returns false, then don't handle any other keys.
			if(p && this._opened && p.handleKey){
				if(p.handleKey(e) === false){ return; }
			}
			if(this._opened && e.charOrCode == dk.ESCAPE && !(e.shiftKey || e.ctrlKey || e.altKey || e.metaKey)){
				this._close();
				dojo.stopEvent(e);
			}else if(!this._opened && e.charOrCode == dk.DOWN_ARROW){
				this._open();
				dojo.stopEvent(e);
			}else if(e.charOrCode === dk.TAB){
				this._tabbingAway = true;
			}else if(this._opened && (e.keyChar || e.charOrCode === dk.BACKSPACE || e.charOrCode == dk.DELETE)){
				// Replace the element - but do it after a delay to allow for
				// filtering to occur
				setTimeout(dojo.hitch(this, function(){
					if(this._picker && this._opened){
						dijit.placeOnScreenAroundElement(p.domNode.parentNode, this.domNode, {'BL':'TL', 'TL':'BL'}, p.orient ? dojo.hitch(p, "orient") : null);
					}
				}), 1);
			}
		}
	}
);

}

if(!dojo._hasResource["dijit.form.DateTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.DateTextBox"] = true;
dojo.provide("dijit.form.DateTextBox");




dojo.declare(
	"dijit.form.DateTextBox",
	dijit.form._DateTimeTextBox,
	{
		// summary:
		//		A validating, serializable, range-bound date text box with a drop down calendar
		//
		//		Example:
		// |	new dijit.form.DateTextBox({value: new Date(2009, 0, 20)})
		//
		//		Example:
		// |	<input dojotype='dijit.form.DateTextBox' value='2009-01-20'>

		baseClass: "dijitTextBox dijitDateTextBox",
		popupClass: "dijit.Calendar",
		_selector: "date",

		// value: Date
		//		The value of this widget as a JavaScript Date object, with only year/month/day specified.
		//		If specified in markup, use the format specified in `dojo.date.stamp.fromISOString`
		value: new Date("")	// value.toString()="NaN"
	}
);

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.ParametersWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.ParametersWidget"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
/*******************************************************************************
 * CMVC Location: TIPChartPortlet/WebContent/js/ibm/tivoli/tip/charts/dijit/ParametersWidget.js, tipcharts
 * Version:       1.0
 * Last Modified: 11/24/09
 *******************************************************************************/

dojo.provide("ibm.tivoli.tip.charts.dijit.ParametersWidget");







// NLS



// define the widget class
dojo.declare (
	"ibm.tivoli.tip.charts.dijit.ParametersWidget", 
    [ dijit._Widget, dijit._Templated, dijit._Container ],
{
    namespace: "namespace1_",
    widgetsInTemplate: true,
    data: [],
    context: [],
    hasContext_: false,
	templateString:"<div id=\"${id}parametersWidgetDiv\">\r\n</div>\r\n",

	constructor: function() {
		// summary: A reusable widget for displaying a dynamic set of parameters to be 
		//  filled in at runtime.
	    //this.resourceMessages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "resources");
	},

	postCreate: function() { 
		this.inherited ("postCreate", arguments);
		this.addParameters (this.data);
	},
	
	addParameters: function (model) {
		try {		
		var param = null;
		var widget = null;

		for (var row in model) {
			if (row) {
				param = model[row];
				widget = new ibm.tivoli.tip.charts.dijit.Parameter ({
					parmid: this.id + "param_" + param.id + param.type,
					baseid: param.name,
					value: param.value,
					type: param.type,
					required: param.required,
					options : param.options,
					label: param.label.value
				});				
				this.addChild (widget);
			}
		}
		} catch (ex) {
			console.error (ex);
			throw (ex);
		}
	},	
	isContextOn: function () {
		var receiveContextRadio = dijit.byId(this.id + "contextualParametersSelected");		
		return receiveContextRadio.checked;
	},
	
	hasContext: function() {
		return this.hasContext_;
	},
	
	getParameterValue: function (parameterId, parameterType) {
		
		var value = null;
		
		/*if (this.isContextOn() && this.hasContext()) {
			console.log ("context is on and we have context");
			if (this.context[parameterName]) {
				console.log ("from context w/ parameter: " + parameterName + " and value: " + this.context[parameterName]);
				value = this.context[parameterName];
			} else {
				console.log ("context value is null for name: " + parameterName);
				value = null;
			}
		} else {*/
			var textbox = dijit.byId (this.id + "param_" + parameterId + parameterType);

			if(parameterType == "java.util.Boolean") {
				value = textbox.checked;
			}
			else {
				value = textbox.getValue();
			}
	//	}
		
		//value = this.urlizeValue (value);
		
		return (value);
	},
	
	urlizeValue: function (value) {
		if (value !== null) {
			value = value.replace ("&", ":::");
		}
		
		return (value);
	},

	setParameterValue: function (parameterName, parameterValue) {
		var textbox = dijit.byId (this.namespace + "param_" + parameterName);
		textbox.setValue (parameterValue);
	},

	setAllContext: function (ctxt) {
		var params = this.getParameters();
		var param = null;
		var value = null;
		var x = null;
		
		for (x in params) {
			if (x) {
				param = params[x];
				value = eval ("ctxt." + param.name);
				if (value) {
					this.setContext (param.name, value);
				}
			}
		}
		
		this.hasContext_ = true;
	},
	
	setContext: function (parameterName, parameterValue) {
		this.context[parameterName] = parameterValue;
		this.hasContext_ = true;
	},
	
	getParameters: function () {
		return this.data;
	},
		
	removeParameters: function () {
		dojo.forEach (this.getChildren(), function (child) {
			this.removeChild (child);
			child.destroy();
		}, this);
	},
	
	resetParameters: function () {
		var param = null;
		var widget = null;
		for (var row in this.data) {
			if (row) {
				param = this.data[row];
				widget = dijit.byId (this.namespace + "param_" + param.name);
				if (param.value === undefined) {
					param.value = "";
				}
				widget.setValue (param.value);
			}
		}
	},
	
	destroy: function () {
//		this.removeParameters();
		dojo.forEach (this.data, function (datum) {
			var param = dijit.byId (this.id + "param_" + datum.name);
			if (param) {
				param.destroy();
			}
		}, this);
		
		this.data = null;
		this.destroyDescendants();
		this.inherited (arguments);
	}

});
	

// define the widget class
dojo.declare (
	"ibm.tivoli.tip.charts.dijit.Parameter", 
    [ dijit._Widget, dijit._Container ],
{
	parmid: null,
	baseid: null,
	label: null,
	description: null,
	value: null,
	type: null,
	required: null,
	options: null,

	constructor: function () {
		console.log ("enter ParametersWidget.constructor");
		console.log ("exit ParametersWidget.constructor");
	},
	postCreate: function () {
		console.log ("enter ParametersWidget.postCreate");
		 var label = dojo.doc.createElement("label");              
         dojo.attr(label, { "id":this.parmid,"for":"${id}"+this.parmid });        
         label.innerHTML = this.label; 
         this.domNode.appendChild(label); 
		 this.domNode.appendChild(document.createElement("br"));

		 var parWidget = null;
		console.log ("ParametersWidget.postCreate... this.type: " + this.type); 
		 if ((this.type == "java.util.Boolean") || (this.type == "boolean")) {
			var checkBoxChecked = false;
			if(this.value == "true") {
				checkBoxChecked = true;
			}				
			parWidget = new dijit.form.CheckBox({
				id : this.parmid,
				value : this.value,
				name : this.parmid,
				checked : checkBoxChecked
			});
			this.addChild (parWidget);	 
			
		 } else if ((this.type == "java.util.Date") || (this.type == "date")) {
			parWidget = new dijit.form.DateTextBox({
				id : this.parmid,
				value : this.value,
				name : this.parmid,
                constraints:{datePattern:'MM/dd/yyyy'},
                lang:"en",
                required:this.required,
                promptMessage:"mm/dd/yyyy",
                invalidMessage:"Invalid date. Use mm/dd/yy format."
            });            
            this.addChild(parWidget); 
		 } else if ((this.type == "java.lang.Integer") || (this.type == "int")) {
			parWidget = new dijit.form.NumberTextBox({
				id : this.parmid,
				value : this.value,
				name : this.parmid,
				constraints:{min:0,max:1000000,places:0,strict: true,pattern: '#,######0.0#'},
				invalidMessage:"",
                promptMessage:""
			});
			this.addChild(parWidget);
		 } else if (this.type == "java.util.List") {
			var options = dojo.clone(this.options);
			for(var i=0;i<options.length;i++){
				options[i].label = options[i].label.value;
			}
			var selectOptions = new dojo.data.ItemFileWriteStore({data: {identifier: 'label', items:options}});
			parWidget = new dijit.form.ComboBox({
				id: this.parmid,
				name: this.parmid,
				value: this.value,
				store: selectOptions,
				searchAttr: "label"
         	},this.parmid);
		 	this.addChild(parWidget);
		} else { 			  
			parWidget = new dijit.form.ValidationTextBox({
                    type: "text",
                    id: this.parmid,
                    name: this.parmid,
                    size: "20",
                    required: this.required,
                    promptMessage: "Update "+this.label,
                    invalidMessage: "",
                    trim: true,
					value: this.value,
                    style: "width: 14em;"
                },
                document.createElement("input"));
				this.addChild (parWidget);
			if ((this.type == "java.lang.String") || (this.type == "string")) {
				// nothing to do here... 
			} else {
				console.warn ("Unknown type in ParametersWidget.postCreate.  Rendering as text box: " + this.type);
				parWidget.set('disabled', true);
			}
		}
	},
	setValue: function (value) {
		this.value = value;
	},
	getValue: function () {
		return this.value;
	},
	destroy: function () {
//		if (dijit.byId(this.parmid)) {
//			console.log ("about to destroy: ");
//			console.log (dijit.byId (this.parmid));
//			dijit.byId (this.parmid).destroy();
//		} else {
//			console.log ("not found to destroy: " + this.parmid);
//		}

//		this.destroyDescendants();
		this.inherited (arguments);
	}
	
});


}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartPreferences"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartPreferences"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
/*******************************************************************************
 * CMVC Location: %W%
 * Version:       %I%
 * Last Modified: %G%
 *******************************************************************************/

dojo.provide("ibm.tivoli.tip.charts.dijit.ChartPreferences");













  // also contains RadioButton








dojo.require ("ibm.tivoli.tip.charts.dojo.RESTErrorHandler");

// NLS



dojo.declare("ibm.tivoli.tip.charts.dijit.ChartPreferences",
	          [ dijit._Widget, dijit._Templated, dijit._Container ],
{
	widgetsInTemplate: true,
	templateString:"<div>\r\n<div dojoType=\"dijit.Dialog\"\r\n\tdojoAttachPoint=\"dialog\"\r\n\ttitle=\"${resources_.PREFERENCES}\"\r\n\tstyle=\"display:none;\" >\r\n  <form name=\"${id}propertiesForm\" id=\"${id}propertiesForm\" \r\n\t  dojoType=\"dijit.form.Form\" enctype=\"multipart/form-data\" \r\n\t  style=\"padding: 0px; margin:0px;\">\r\n\r\n    <div id=\"${id}tabs\" dojoType=\"dijit.layout.TabContainer\" controllerWidget=\"dijit.layout.StackController\" doLayout=\"false\" class=\"chartsDialog\" style=\"width: 550px; height: 375px;\" >\r\n      <div dojoType=\"dijit.layout.ContentPane\" selected=\"true\" title=\"${resources_.GENERAL}\" \r\n\t\t\t  style=\"padding: 15px; height: 345px;\">\r\n        <table border=\"0\" cellpadding=\"0\" cellspacing=\"5px\" width=\"95%\">\r\n          <tr valign=\"top\">\r\n            <td> ${resources_.CHART_NAME}\r\n              <label class=\"label\" for=\"${id}chartName\" title=\"${resources_.CHART_NAME}\"> </label>\r\n              <input id=\"${id}chartName\" type=\"text\" name=\"${id}chartName\" dojoAttachPoint=\"labelField\" dojoType=\"dijit.form.ValidationTextBox\" \r\n\t\t\t\t\tdojoAttachEvent=\"onBlur:onBlur, onKeyUp:onBlur\" required=\"true\" /></td>\r\n            <td rowspan=\"4\"> ${resources_.DESCRIPTION}\r\n              <label class=\"h2\" for=\"${id}description\" title=\"${resources_.DESCRIPTION}\"></label>\r\n              <textarea id=\"${id}description\" \r\n\t\t\t\t\t\t\tdojoType=\"dijit.form.SimpleTextarea\"\r\n\t\t\t\t\t   \t\tname=\"${id}description\" \r\n\t\t\t\t\t\t\tdojoAttachPoint=\"descField\"\r\n\t\t\t\t\t   \t\tstyle=\"width:200px; height:150px;\"\r\n\t\t\t\t\t   \t\trows=\"4\"\r\n\t\t\t\t\t\t    cols=\"30\" >\r\n\t\t\t\t  </textarea>\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\r\n\t\t\t\t<tr valign=\"top\">\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<div id=\"${id}chartTypeLabelDiv\"> ${resources_.VIEW_AS} </div>\r\n\t\t\t\t</td>\r\n\t\t\t\t</tr>\r\n\r\n\t\t\t\t<tr valign=\"top\">\r\n\t\t\t\t<td>\t\t\t\t\r\n\t\t\t\t\t<div id=\"${id}chartTypeDiv\">\r\n\t\t\t\t\t\t<select id=\"${id}chartType\" name=\"${id}chartType\" dojoAttachPoint=\"selectedChartType\" dojoType=\"dojox.form.DropDownSelect\" style=\"width: auto;\">\t\r\n\t\t\t\t\t\t\t<option value=\"Pie\">${resources_.PIE}</option>\r\n\t\t\t\t\t\t\t<option value=\"Table\">${resources_.TABLE}</option>\r\n\t\t\t\t\t\t\t<option value=\"Lines\">${resources_.LINE}</option>\r\n\t\t\t\t\t\t\t<option value=\"dojot.visual.ClusteredColumnsEmbossed\">${resources_.CLUSTERED_COLUMNS}</option>\r\n\t\t\t\t\t\t\t<option value=\"dojot.visual.ColumnsEmbossed\">${resources_.COLUMNS}</option>\r\n\t\t\t\t\t\t\t<option value=\"dojot.visual.StackedColumnsEmbossed\">${resources_.STACKED_COLUMNS}</option>\r\n\t\t\t\t\t\t\t<option value=\"dojot.visual.BarsEmbossed\">${resources_.BARS}</option>\r\n\t\t\t\t\t\t\t<option value=\"dojot.visual.StackedBarsEmbossed\">${resources_.STACKED_BARS}</option>\r\n\t\t\t\t\t\t\t<option value=\"dojot.visual.ClusteredBarsEmbossed\">${resources_.CLUSTERED_BARS}</option>\r\n\t\t\t\t\t\t</select>\r\n\t\t\t\t<br/>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\r\n\t\t\t<tr valign=\"top\">\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<div id=\"${id}legendDiv\">\r\n\t\t\t\t\t\t<input type=\"checkbox\"\r\n\t\t\t\t\t\tname=\"${id}showLegendCB\"\r\n\t\t\t\t\t\tid=\"${id}showLegendCB\"\r\n\t\t\t\t\t\tdojoType=\"dijit.form.CheckBox\" dojoAttachPoint=\"isLegendEnabled\"/>\r\n                <label class=\"label\" for=\"${id}showLegendCB\">${resources_.SHOW_LEGEND}</label>\r\n                <br/>\r\n                <label class=\"label\" for=\"${id}legendLocation\">${resources_.LEGEND_LOCATION}</label>\r\n                <br/>\r\n                <select id=\"${id}legendLocation\" name=\"${id}legendLocation\" dojoAttachPoint=\"legendLoc\" dojoType=\"dojox.form.DropDownSelect\" style=\"width: 40%;\">\r\n                  <option value=\"bottom\" selected=\"selected\">${resources_.LEGEND_LOCATION_BOTTOM}</option>\r\n                  <option value=\"right\">${resources_.LEGEND_LOCATION_RIGHT}</option>\r\n                </select>\r\n                <br/>\r\n              </div></td>\r\n          </tr>\r\n          <tr valign=\"top\">\r\n            <td> ${resources_.OPTIONS}<br />\r\n              <div><input type=\"checkbox\"\r\n\t\t\t\t\t\t\tname=\"${id}checkForUpdatesCB\"\r\n\t\t\t\t\t\t\tid=\"${id}checkForUpdatesCB\"\r\n\t\t\t\t\t\t\tdojoType=\"dijit.form.CheckBox\" dojoAttachPoint=\"isAutoUpdate\"/>\r\n              <label class=\"label\"\r\n\t\t\t              for=\"${id}checkForUpdatesCB\">${resources_.AUTO_UPDATE}</label>\r\n              </div>\r\n\t\t\t  <br/>\r\n\t\t\t  <div>\r\n              <input type=\"checkbox\"\r\n\t\t\t\t\t\t\tname=\"${id}showLastUpdated\"\r\n\t\t\t\t\t\t\tid=\"${id}showLastUpdated\"\r\n\t\t\t\t\t\t\tdojoType=\"dijit.form.CheckBox\" dojoAttachPoint=\"showLastUpdate\"/>\r\n              <label class=\"label\"\r\n\t\t\t              for=\"${id}showLastUpdated\">${resources_.SHOW_LAST_UPDATED}</label>\r\n              </div><br/>\r\n\t\t\t  </td>\r\n            <td> ${resources_.PAGING}<br />\r\n              <div><input type=\"radio\"\r\n\t\t\t\t\t dojoType=\"dijit.form.RadioButton\"\r\n\t\t\t\t\t name=\"${id}pagingType\"\r\n\t\t\t\t\t value=\"${resources_.RETURN_ALL_ROWS}\"\r\n\t\t\t\t\t id=\"${id}allRows\" dojoAttachEvent=\"onClick:setPagingOption\">\r\n              <label class=\"label\">${resources_.RETURN_ALL_ROWS}</label>\r\n\t\t\t  </div>\r\n              <br>\r\n\t\t\t  <div>\r\n              <input type=\"radio\"\r\n\t\t\t\t\t dojoType=\"dijit.form.RadioButton\"\r\n\t\t\t\t\t name=\"${id}pagingType\"\r\n\t\t\t\t\t value=\"${resources_.PAGE_THE_DATA}\"\r\n\t\t\t\t\t id=\"${id}pageSize\" dojoAttachEvent=\"onClick:setPagingOption\">\r\n              <label class=\"label\">${resources_.PAGE_THE_DATA}</label>\r\n\t\t\t  </div>\r\n              <br>\r\n              &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\r\n\t\t\t  <div>\r\n              <label class=\"label\" for=\"${id}pageSizeValue\">${resources_.PAGING_SIZE}</label>\r\n              &nbsp;&nbsp;&nbsp;\r\n              <input id=\"${id}pageSizeValue\" type=\"text\" name=\"${id}pageSizeValue\" dojoAttachPoint=\"pageSizeValue\" dojoType=\"dijit.form.NumberTextBox\" constraints=\"{min:1,max:500,places:0,strict: true,pattern: '#,######0.0#'}\"\r\n\t\t\t\t\t invalidMessage=\"${messages_.TIPUI1002E}\" promptMessage=\"\" required=\"false\"\r\n\t\t\t\t\t maxlength=\"3\" style=\"width: 5em;\"/>\r\n\t\t\t\t\t </div>\r\n\t\t\t</td>\r\n          </tr>\r\n        </table>\r\n      </div>\r\n      <div dojoType=\"dijit.layout.ContentPane\" title=\"${resources_.PARAMETERS}\"\r\n\t\t\t\t style=\"padding: 15px; height: 345px;\">\r\n        <div style=\"width: 510px; height: 285px; overflow: auto\" >\r\n          <table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\r\n            <tr valign=\"top\">\r\n              <td> ${resources_.OPTIONS} </td>\r\n            </tr>\r\n            <tr valign=\"top\">\r\n              <td><div class=\"label\">\r\n                  <input type=\"radio\"\r\n\t\t\t\t\t  dojoType=\"dijit.form.RadioButton\"\r\n\t\t\t\t\t  name=\"${id}whichParametersGroup\"\r\n\t\t\t\t\t  id=\"${id}contextualParametersSelected\"\r\n\t\t\t\t\t  value=\"true\"\r\n\t\t\t\t\t  checked\r\n\t\t\t\t\t  />\r\n                  <label for=\"${id}contextualParametersSelected\" title=\"${resources_.USE_CONTEXT}\">${resources_.USE_CONTEXT}</label>\r\n                </div></td>\r\n            </tr>\r\n            <tr valign=\"top\">\r\n              <td><div class=\"label\">\r\n                  <input type=\"radio\"\r\n\t\t\t\t\t  dojoType=\"dijit.form.RadioButton\"\r\n\t\t\t\t\t  name=\"${id}whichParametersGroup\"\r\n\t\t\t\t\t  id=\"${id}defaultParametersSelected\"\r\n\t\t\t\t\t  value=\"false\"\r\n\t\t\t\t\t  />\r\n                  <label for=\"${id}defaultParametersSelected\" title=\"${resources_.USE_DEFAULTS}\">${resources_.USE_DEFAULTS}</label>\r\n                </div></td>\r\n            </tr>\r\n          </table>\r\n          <br/>\r\n          <div id=\"${id}parametersDivContainer\" > </div>\r\n        </div>\r\n      </div>\r\n      <div dojoType=\"dijit.layout.ContentPane\" title=\"${resources_.COLUMNS}\"\r\n\t\t\t\t style=\"padding: 15px; height: 345px;\" id=\"${id}columnsDivTab\" >\r\n        <table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\r\n          <tr valign=\"top\">\r\n            <td><div style=\"width: 480px; height: 280px; overflow: auto\" > ${resources_.COLUMNS_VISIBLE} <br/>\r\n                <br/>\r\n                <button  iconClass=\"selectAllIcon\"  dojoType=\"ibm.tivoli.tip.dijit.TIPButton\"      dojoAttachEvent=\"onClick:onSelectAll\" title=\"${resources_.SELECT_ALL}\" ></button>\r\n                <button  iconClass=\"deselectAllIcon\"   dojoType=\"ibm.tivoli.tip.dijit.TIPButton\"      dojoAttachEvent=\"onClick:onDeselectAll\" title=\"${resources_.DESELECT_ALL}\" ></button>\r\n                <div id=\"${id}columnsDivContainer\"> </div>\r\n              </div></td>\r\n          </tr>\r\n        </table>\r\n      </div>\r\n      <div dojoType=\"dijit.layout.ContentPane\" title=\"${resources_.ABOUT}\"\r\n\t\t\t\t  style=\"padding: 15px; height: 345px;\">\r\n        <table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\r\n          <tr>\r\n            <td></td>\r\n          </tr>\r\n          <tr>\r\n            <td> ${tipDojoVersion} </td>\r\n          </tr>\r\n          <tr>\r\n            <td> ${dojoVersion} </td>\r\n          </tr>\r\n        </table>\r\n      </div>\r\n    </div>\r\n\r\n    <div class=\"tip-table-footer\">\r\n      <div dojoType=\"dijit.Toolbar\" style=\"padding: 0px\">\r\n        <button \r\n\t\t\t\t       value=\"${resources_.OK}\"\r\n\t\t\t\t       dojoType=\"dijit.form.Button\"\r\n\t\t\t\t       dojoAttachPoint=\"okButton\"\r\n\t\t\t\t       dojoAttachEvent=\"onClick:onOK\" \r\n\t\t\t>${resources_.OK}</button>\r\n        <button \r\n\t\t\t\t\t   dojoType=\"dijit.form.Button\"\r\n\t\t\t\t\t   dojoAttachPoint=\"cancelButton\"\r\n\t\t\t\t\t   dojoAttachEvent=\"onClick:onCancel\"\r\n\t\t\t\t       value=\"${resources_.CANCEL}\"\r\n\t\t >${resources_.CANCEL}</button>\r\n      </div>\r\n\t</div>\t  \r\n  </form>\r\n</div>\r\n</div>\r\n",
	global: null,
	chartBuilder: null,
	datasourceID: null,
    namespace: null,
	tableBuilder: null,
	builder: null,
	isTableFirst: null,

	constructor: function () {
		console.log ("enter ChartPreferences.ctor");
	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
		this.messages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "messages");
	    this.tipDojoVersion = dojo.string.substitute(this.resources_.TIPDOJO_VERSION, { 0: ibm.tivoli.tip.Version.getVersion() });
	    console.log ("tipDojoVersion: " + this.tipDojoVersion);
	    this.dojoVersion = dojo.string.substitute(this.resources_.DOJO_VERSION, { 0: dojo.version });
	    console.log ("dojoVersion: " + this.dojoVersion);
	    console.log ("exit ChartPreferences.ctor");
	},
	postCreate: function () {
		this.populate();
	},
	populate: function () {
		this.builder = this.tableBuilder;
		
		if(this.tableBuilder === null) {
			this.builder = this.chartBuilder;
			var legendLocation = this.builder.chartDef.legend.location;
			var legendEnabled = this.builder.chartDef.legend.hidden;	
			this.legendLoc.setValue (legendLocation);		
			this.isLegendEnabled.attr('checked', !legendEnabled);

			var columnsDivTab = dijit.byId (this.id + "columnsDivTab");		
			var mainDom = dijit.byId(this.id + "tabs");
			mainDom.removeChild(columnsDivTab);			
		}
		else
		{
			if(this.isTableFirst === true && !this.builder.widgetDefinition.charts) {
				var chartTypeLabelDiv = dojo.byId(this.id + "chartTypeLabelDiv");
				dojo.style(chartTypeLabelDiv, 'display', 'none');
				var chartTypeDiv = dojo.byId(this.id + "chartTypeDiv");
				dojo.style(chartTypeDiv, 'display', 'none');
			}
			var legendDiv = dojo.byId(this.id + "legendDiv");
			dojo.style(legendDiv, 'display', 'none');

			var colDiv = dijit.byId (this.id + "columnsDiv");
			if (colDiv) {
				colDiv.destroy();
			}
			
			var columnsDiv = dojo.doc.createElement( 'div' );
			columnsDiv.id = this.id + "columnsDiv";
			dojo.place(columnsDiv, dojo.byId (this.id + "columnsDivContainer"), "after" );
			this.tableOptions = new ibm.tivoli.tip.dijit.TIPTableOptions ( 
					{ table: this.builder.table,
					  global: this.global
					}, 
				dojo.byId(this.id + "columnsDiv")
			);
			this.tableOptions.startup();	
		}
		var label = this.builder.chartDef.label.value;
		var description = this.builder.chartDef.description.value;			
		var chartType = this.builder.chartDef.type;		
		this.labelField.setValue (label);		
		this.descField.setValue (description);		
		this.selectedChartType.setValue (chartType);
		
		var pagingOption = this.builder.chartDef.common.paging;		
		var pagingSize = this.builder.chartDef.common.pagingSize;		
		
		if(pagingOption === true) {
			dijit.byId(this.id + "pageSize").attr('checked', true);
			this.pageSizeValue.setValue(pagingSize);
		}
		else {			
			dijit.byId(this.id + "allRows").attr('checked', true);			
			//this.pageSizeValue.setValue(0);
		}
		this.setPagingOption();

		var paramDiv = dijit.byId (this.id + "parametersDiv");
		if (paramDiv) {
			paramDiv.destroy();
		}
		
		var div = dojo.doc.createElement( 'div' );
		div.id = this.id + "parametersDiv";
    	        dojo.place(  div, dojo.byId (this.id + "parametersDivContainer"), "after" );
		
		this.paramWidget_=new ibm.tivoli.tip.charts.dijit.ParametersWidget (
			{
				namespace: this.namespace,
				data: this.builder.dataset.parameters
			}, 
			dojo.byId(this.id + "parametersDiv")
		);
		this.paramWidget_.startup();

		var isAutoUpdateEnabled = this.builder.chartDef.common.autoUpdate;
		this.isAutoUpdate.attr('checked', isAutoUpdateEnabled);

		var showLastUpdatedTime = this.builder.chartDef.common.showLastUpdateTime;
		this.showLastUpdate.attr('checked', showLastUpdatedTime);
		
		var contextRadio = dijit.byId (this.id + "contextualParametersSelected");
		var defaultRadio = dijit.byId (this.id + "defaultParametersSelected");
		var supportsWire = this.builder.widgetDefinition.datasets[0].supportsWire;

		if ((supportsWire === undefined) || (supportsWire === true)) {
			contextRadio.attr ('checked', true);
			defaultRadio.attr ('checked', false);
		} else {
			contextRadio.attr ('checked', false);
			defaultRadio.attr ('checked', true);
		}
	},
	getUserPreferences: function () {				
		this.builder = this.tableBuilder;				
		var userPrefs = null;
		var pagingOption = this.setPagingOption();		
		if((this.pageSizeValue.getValue() == "") || (isNaN(this.pageSizeValue.getValue()))|| !this.pageSizeValue.isInRange() || !this.pageSizeValue.isValid())
		{			
			dojo.style(dijit.byId(this.id + "pageSizeValue").textbox,{visibility:'hidden'});			
			this.pageSizeValue.setValue(25);
		}		
		if(this.tableBuilder === null) {
			this.builder = this.chartBuilder;					
			userPrefs = { 		
			"details": {				
				"preferredView": this.selectedChartType.getValue()
			},	
			"datasetsIdentifier":"id",
			"datasets": [{
				"id": this.builder.dataset.id,
				"parametersIdentifier": "id",
				"parameters": []
			}],	
			"chartsIdentifier":"id",
			"charts": [{
                   "common": {
					"autoUpdate": this.isAutoUpdate.checked,
					"showLastUpdateTime": this.showLastUpdate.checked,
					"paging": (pagingOption === this.resources_.PAGE_THE_DATA) ? true : false,
					"pagingSize": this.pageSizeValue.getValue()
				},
				"id": this.builder.chartID,				
				"label": {
					"key": "",
					"value": this.labelField.getValue()
				},
				"legend": {
					"hidden":!this.isLegendEnabled.checked,
					"location":this.legendLoc.getValue()
						},
                "type": this.selectedChartType.getValue(),
				"toolbar": {
						"hidden":this.builder.toolbarSelf.toolbarHidden_							
						},
				"description": {
					"key": "",
					"value": this.descField.getValue()
				}
			}]
			};
		}
		else
		{
			userPrefs = { 		
			"details": {				
				"preferredView": this.selectedChartType.getValue()
			},
			"datasetsIdentifier":"id",
			"datasets": [{
				"id": this.builder.dataset.id,
				"parametersIdentifier": "id",
				"parameters": []
			}],	
			"tablesIdentifier":"id",
			"tables": [{
                   "common": {
					"autoUpdate": this.isAutoUpdate.checked,
					"showLastUpdateTime": this.showLastUpdate.checked,
					"paging": (pagingOption === this.resources_.PAGE_THE_DATA) ? true : false,
					"pagingSize": this.pageSizeValue.getValue()
				},
				"datasource": this.builder.datasourceID,
				"dataset": this.builder.getTableFromDef().dataset,
				"selectionMode": "single",
				"id": this.builder.tableID,
				"label": {
					"key": "",
					"value": this.labelField.getValue()
				},
				"type": this.selectedChartType.getValue(),
				"toolbar": {
					"hidden":this.builder.toolbarHidden_							
				},
				"description": {
					"key": "",
					"value": this.descField.getValue()
				}
			}]
			};
			if(this.selectedChartType.getValue()==="Table" || !this.tableBuilder.widgetDefinition.charts){
			if(this.tableOptions){				
				var columns = this.builder.table.originalStructure.cells;
				if (columns !== undefined && columns.length > 0)
				{
					userPrefs.tables[0].viewIdentifier = "id";
					userPrefs.tables[0].view = [];							
				
					for (var x in columns) {
						if (x) {
							userPrefs.tables[0].view[x] = {};
							userPrefs.tables[0].view[x].id = columns[x].field;	
							userPrefs.tables[0].view[x].searchable = true;				
							userPrefs.tables[0].view[x].hidden = !dijit.byId (this.tableOptions.id + "column_" + columns[x].field).isVisible();															
							userPrefs.tables[0].view[x].width = columns[x].unitWidth;
						}
					}
				}
			}	
			} else{
				userPrefs.charts = this.tableBuilder.widgetDefinition.charts;
				dojo.mixin (userPrefs.charts[0], userPrefs.tables[0]);				
				delete userPrefs.tables;
			}
		}

		 if (this.paramWidget_ !== null) {			
				var model = this.paramWidget_.getParameters();
				var param = null;
				var value = null;
				
				var contextRadio = dijit.byId (this.id + "contextualParametersSelected");
				userPrefs.datasets[0].supportsWire = contextRadio.checked;
			
				for (var row in model) {
					if (row) {
						
						param = model[row];					
						value = this.paramWidget_.getParameterValue(param.id, param.type);		
						console.log ("retuned value from getparameter value: ");
						console.log (value);
						if (value !== null) {
							userPrefs.datasets[0].parameters[row] = {
								"id": param.id,
								"value": value
							};
						}
					}
				}
		}		
		return (userPrefs);		
	},
	setPagingOption: function () {	
		var pagingTypeOption1 = dojo.byId(this.id + "allRows");
		var pagingTypeOption2 = dojo.byId(this.id + "pageSize");	
		var result;
		
		if (pagingTypeOption1.checked)
		  {
			dojo.byId(this.id + "pageSizeValue").style.backgroundColor = "#ffffff";
			dojo.byId(this.id + "pageSizeValue").readOnly = 1;
			dojo.byId(this.id + "pageSizeValue").value = "";
			dojo.byId(this.id + "pageSizeValue").required = false;
			this.pageSizeValue.attr('promptMessage', "");
			result = pagingTypeOption1.value;
		  }
		 if (pagingTypeOption2.checked)
		  {			
			dojo.byId(this.id + "pageSizeValue").readOnly = 0;
			dojo.byId(this.id + "pageSizeValue").required = true;
			this.pageSizeValue.attr('promptMessage', this.resources_.PAGE_SIZE_RANGE_MESSAGE);
			result = pagingTypeOption2.value;
		  } 	
		 return result;
	},
	show: function () {
		this.dialog.show();
//		dijit.byId (this.id + "tabs").resize();
	},
	close: function () {
		this.dialog.hide();
		setTimeout (dojo.hitch (this, this.kill), 100);
	},
	kill: function () {
		var id = this.id;
		this.destroyRecursive();
		dijit.registry.remove (id);
	},
	onOK: function () {
		console.log ("enter chartPreferences.onOK");
		var widgetDef = this.saveUserPreferences();
		this.builder = this.tableBuilder;	
		var encodedCID = null;
		if(this.tableBuilder === null) {
			this.builder = this.chartBuilder;	
			encodedCID = encodeURIComponent(this.builder.chartID);
		}
		else {
			encodedCID = encodeURIComponent(this.builder.tableID);
			this.tableOptions.onSave();
		}
		var encodedProviderID = encodeURIComponent(this.builder.providerID);		
		var encodedDS = encodeURIComponent(this.builder.datasourceID);
		var encodedPID = encodeURIComponent(this.global.chartProfileID);
		var drillUpItems = this.builder.drillUpItems;
		//show busyPane
		var busyPane = dijit.byId(this.global.namespace+"busy_pane");
	    if ( busyPane ) {
	      console.log ("showing busypane");
	      busyPane.show(true); // size to portlet
		}
	    
		//when table is displayed first
		if ((this.isTableFirst !== undefined) && (this.isTableFirst !== null))
		{
			//when chart type changes from chart to table
			if(widgetDef.tables === undefined) {
				widgetDef.tables = widgetDef.user.charts;
			}
			if(this.isTableFirst === true || (this.tableBuilder !== null && widgetDef.details.preferredView == "Table"))
			{				
				widgetDef.details.preferredView = "Table";
				var userPrefs = this.getUserPreferences();
				if(userPrefs.tables === undefined) {
					userPrefs = this.updatePrefs(userPrefs);
				}
				this.builder.update (userPrefs, widgetDef);
				this.close();
				return ;
			}
		}	
		if(!this.global.widgetFetcher){
			this.global.widgetFetcher = new ibm.tivoli.tip.charts.dojo.WidgetFetcher ({
				global: this.global
			});
		}
		var fetcher = this.global.widgetFetcher;
		var domNode = dojo.byId (this.global.namespace+"widgetDiv");		
		this.builder.destroyRecursive();		
		dojo.destroy(domNode);
		domNode = dojo.doc.createElement("div");
		domNode.id = this.global.namespace+"widgetDiv";
		dojo.byId (this.global.namespace + "widgetContainer").appendChild(domNode);
		/*
		var domNode = dojo.byId (this.global.namespace+"widget");
		this.builder.destroy();		
		dojo.empty(domNode);
		dojo.place (domNode, dojo.byId (this.global.namespace + "birtChartWidget"), "after");
		*/	
		if (widgetDef.details.preferredView == "Table") {			
			this.builder = fetcher.buildTable (encodedProviderID, encodedDS, widgetDef, domNode, encodedCID, null, this.isTableFirst, drillUpItems);			
		} else {			
			//widgetDef.charts[0].type = widgetDef.details.preferredView;
			//when chart type changes from table to chart
			if(widgetDef.user.charts === undefined) {
				widgetDef.user.charts = widgetDef.user.tables;
			}
			this.builder = fetcher.buildChart (encodedProviderID, encodedDS, widgetDef, encodedCID, domNode, null, drillUpItems);
		}
		this.global.builder = this.builder;
		this.close();
		console.log ("exit ChartPreferences.onOK");
	},
	updatePrefs: function() {	
		var userPrefs = this.getUserPreferences();
		if(userPrefs.tables === undefined) {
			userPrefs.tables = userPrefs.charts;
		}
		return userPrefs;
	},	
	saveUserPreferences: function(syncflag) {	
		this.builder = this.tableBuilder;		
		var encodedCID = null;
		if(this.tableBuilder === null) {
			this.builder = this.chartBuilder;			
			encodedCID = encodeURIComponent(this.builder.chartID);
		}
		else
		{
			encodedCID = encodeURIComponent(this.builder.tableID);
		}
		var encodedProviderID = encodeURIComponent(this.builder.providerID);
		var result = null;
		var encodedDS = encodeURIComponent(this.builder.datasourceID);		
		var encodedPID = encodeURIComponent(this.global.chartProfileID);
		var widgetDefVersion = null;
		var widgetSchema = null;
		if(this.tableBuilder !== null){
			widgetDefVersion = this.tableBuilder.widgetDefinition.meta.version;
			widgetSchema = this.tableBuilder.widgetDefinition.meta.type;
		}else{
			widgetDefVersion = this.chartBuilder.widgetDefinition.meta.version;	
			widgetSchema = this.chartBuilder.widgetDefinition.meta.type;
		}
		var controller = "/ibm/tivoli/rest";
		var prefs = dojo.toJson(this.getUserPreferences());
		console.log ("preferences to be saved");
		console.log (prefs);
		console.log (controller + "/providers/tip.charts/datasources/" + encodedDS + "/widgets/" + widgetSchema + "/definitions/" + encodedCID + "-" + widgetDefVersion + "?type=user&profileID=" + this.global.chartProfileID + "&registryLocation=local");
		//    /ibm/tivoli/rest/providers/tip.charts/datasources/ITM/widgets/charts-2.1.0/definitions/MemoryAllocation-2.1.0?type=user&profileID=123
		
		if(syncflag === undefined || syncflag === null) {
			syncflag = true;
		}
		var profileIdUriParam = this.global.chartProfileID==null||this.global.chartProfileID==undefined||this.global.chartProfileID=="null"||this.global.chartProfileID=="undefined" ? "" : "&profileID=" + this.global.chartProfileID;
		var kw = {
        			  sync: syncflag,
        			  headers: {'Content-Type': 'application/json',
        			  		   'Content-Length': prefs.length
        			  		  },
        			  url: controller + "/providers/" + encodedProviderID + "/datasources/" + encodedDS + "/widgets/" + widgetSchema + "/definitions/" + encodedCID + "-" + widgetDefVersion + "?type=user" + profileIdUriParam + "&registryLocation=local",
                      putData: prefs,					  
                      load: dojo.hitch(this, function () {
                      		var fetcher = new ibm.tivoli.tip.charts.dojo.WidgetFetcher({global: this.global});
							result = fetcher.fetchWidgetDefinition(encodedProviderID, encodedDS, encodedCID, widgetDefVersion, encodedPID);
                      }),
                      error: dojo.hitch (this, function (error) {											
							var errHandler = new ibm.tivoli.tip.charts.dojo.RESTErrorHandler(this.global);		
							errHandler.showError(dojo.fromJson(error.responseText));	
		 			  })
                   };

          //ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).post( kw );
          dojo.xhrPut (kw);
        
          console.log ("exit ChartPreferences.saveUserPreferences");  
          return (result);
	},
	onSelectAll: function(){		
		this.tableOptions.onSelectAll();
	},	
	onDeselectAll: function(){		
		this.tableOptions.onDeselectAll();
	},
	saveSharedPreferences: function() {
	},
	onBlur: function(){		
		var chartName = this.labelField.attr('value');
		if(!chartName||chartName==""){
			this.okButton.attr('disabled',true);
		}else{
			this.okButton.attr('disabled',false);
		}
	},
	onCancel: function () {
		this.close();
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartToolbar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartToolbar"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.charts.dijit.ChartToolbar");





dojo.require ("ibm.tivoli.tip.charts.dojo.RESTErrorHandler");

// NLS


dojo.declare("ibm.tivoli.tip.charts.dijit.ChartToolbar",
              [ dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true,
	templateString:"<table width=\"100%\"  cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\r\n\t<tbody  dojoAttachPoint=\"toolbarBody\">\r\n\t\t<tr id=\"${id}toolbarTR\" >\r\n\t\t\t<td class=\"TIPChartToolbarArea\" width=\"100%\" nowrap=\"nowrap\" align=\"left\"  style=\"height:auto\"> \r\n\t\t\t\t<div style=\"position:relative; height:100%;\" dojoAttachPoint=\"nubArea\" >\r\n\t\t\t\t<div id=\"${id}toolbar\" dojoAttachPoint=\"toolbar\" dojoType=\"dijit.Toolbar\">\r\n\t\t\t\t\t<div dojoType=\"dijit.form.Button\" iconClass=\"refreshIcon\"   \r\n            \t\t\tshowLabel=\"false\" dojoAttachPoint=\"refresh\" dojoAttachEvent=\"onClick:onRefresh\">${resources_.REFRESH}\r\n        \t\t\t</div>\r\n\t\t   \t\t</div>\r\n\t\t\t\t</div>\r\n\t\t\t</td>\t\t\t\r\n\t\t\t<!--td class=\"tip-table-footer-data\" nowrap=\"nowrap\" valign=\"middle\" align=\"right\"> \r\n\t\t\t\t<div dojoAttachPoint=\"infoArea\" class=\"footerLabel\"></div>\r\n\t\t\t</td-->\r\n\t\t</tr>  \r\n\t\t\r\n\t\t<!--\r\n\t\t<tr>\r\n\t\t<td align=\"center\">\r\n\t\t\t<div id=\"${id}nub\" dojoAttachPoint=\"nub\" class=\"nubOpen\" tabindex=\"0\">\r\n\t\t\t</div>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t<div  id=\"${id}nub\" dojoAttachPoint=\"nub\" tabindex=\"0\" class=\"nubOpen\" style=\"left: 385px; position: absolute; bottom: -1px; height: 9px; display: block;\"></div>\t\t\t\t\t\t\t\t\t\r\n\t\t-->\r\n\t</tbody>\r\n</table>\r\n",
	chartFetcher: null,
	chartBuilder: null,
	global: null,
	toolbarHidden_:null,
	CHART_SOURCE_TYPE_PAGE: "CHART_SOURCE_TYPE_PAGE",
	
	namespace:"namespace_",
	
	
	constructor: function () {
		try {
			console.log ("enter ChartToolbar.ctor");
		
			this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");    
	    	this.preferencesDialog = null;
			this.connections = [];
			this.allowSubmit_ = true;
			this.interval = null;
			this.nub=null;
		
			this.currentPage = this.CHART_SOURCE_TYPE_PAGE;
		} catch (ex) {
			alert("caught error:"+dojo.toJson(ex));
			console.error ("caught error in ChartToolbar.ctor: " + dojo.toJson(ex));
		}
	},
	postCreate: function () {
		if (this.chartBuilder.standalone === true) {
		} else {
			console.log ("instantiating chart messages");
			console.log (this.chartBuilder);
			this.messages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.chartportlet.dijit", "chart_messages");
			dojo.require ("ibm.tivoli.tip.dojo.TIPHelper");
			this.tipHelper_ = new ibm.tivoli.tip.dojo.TIPHelper();
		}
		
		//this.toolbarHidden_ = (this.global.mode == "view") ? true : false;
		this.mode = this.global.mode;

		var div = document.createElement("div");
		this.nub=div;
		div.setAttribute("tabIndex","0");
		dojo.addClass(div,"nubOpen");
		dojo.style(div,"position","absolute");
		dojo.style(div,"bottom","-1px");
		dojo.style(div,"height","9px");
		dojo.style(div,"align","center");
		var coords = dojo.coords(this.toolbarBody);
		//var centerx =  coords.x + (coords.w/2);
		var centerx = coords.w/2;
		dojo.style(div,"left",centerx+"px");
		dojo.style(div,"display","block");

		this.nubArea.appendChild(div);		
		
		// connect toggle event to nub
		this.connections.push (dojo.connect (this.nub, "onclick", this, "toggleToolbar"));
		this.connections.push (dojo.connect (this.nub, "onkeypress", this, "onNubKeyPress"));
		
		this.chartNavigator = dijit.byId (this.global.namespace + "mainChartNavigatorDiv");
		if (this.toolbarHidden_ === true) {
			dojo.style (dojo.byId (this.id + "toolbar"), "display", "none");
			dojo.addClass (this.nub, "nubClosed");
			dojo.removeClass (this.nub, "nubOpen");
			dojo.style(this.nub,"position","relative");
		}
			
		console.log ("exit ChartToolbar.postCreate");
	},
	showChartSelector: function () {
    	if (this.interval !== null) {
      		// if auto update is on, cancel it
			clearTimeout (this.interval);
			this.interval = null;
		}
		var chartNavigator = dijit.byId (this.global.namespace + "mainChartNavigatorDiv");
		chartNavigator.setCurrentPage (chartNavigator.CHART_SOURCE_TYPE_PAGE);
		chartNavigator.showChartNavigator();
		this.restorePortletTitle();
		this.destroyRecreate();
	},
	destroyRecreate: function () {
		this.chartBuilder.destroyRecursive();
		var node = dojo.doc.createElement ("div");
		node.id = this.global.namespace + "widget";
		dojo.place (node, dojo.byId (this.global.namespace + "birtChartWidget"), "after");
	},
	
	restorePortletTitle: function () {
		// restore portlet title to translated Charting
		this.tipHelper_.setPortletTitle (this.chartBuilder.domNode, this.global.getPortletTitle());
	},

	onResetPrimitive: function (action, reloadPrefs) {
		var callback = null;
		
		if (reloadPrefs === false) {
			callback = null;
		} else {
			callback = dojo.hitch(this, this.afterReset);
		}
		
		var kw = {
	             content:   {
                        command: "savePreferences",
                        arg_portletNamespace: this.global.namespace,
                        action: action
                        },
            	 onResponse:{
                         callback: callback
       	                }
    	     };
	
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		
		// do not submit
		return (false);
	},
	onReset: function () {
		//this.onResetPrimitive ("resetDefaults");
		this.chartBuilder.chartUtil.onResetPrimitive ("resetDefaults");
	},
	onResetMyPreferences: function (reloadDefaultPrefs) {
		//this.onResetPrimitive ("resetMine", reloadDefaultPrefs);
		this.chartBuilder.chartUtil.onResetPrimitive ("resetMine", reloadDefaultPrefs);
	},

	afterReset: function () {
		var chartNav = dijit.byId (this.global.namespace + "mainChartNavigatorDiv");
		chartNav.showChartNavigator();
		this.showChartSelector();
		chartNav.getPortletPreferences();
	},
	onNubKeyPress: function (event) {
	  	if (event.keyCode == 13) {
		  	event.cancelBubble = true;
	  		event.returnValue = false;
	  		console.log ("in onNubKeyPress: pressed enter");
	  		this.toggleToolbar();
	  	}
	},
	toggleToolbar: function () {
		this.toggleTableToolbar();
		this.toggleChartToolbar();
	},
	toggleTableToolbar: function () {
		console.log ("enter toggle table toolbar");
		
		var tipTable = dijit.byId (this.global.namespace + "tbl_tipChartData");
		if (tipTable) {
			dojo.unsubscribe (this.toggleSubscription);
			this.toggleSubscription = null;
			tipTable.toggleToolbar();	
			// subscribe to table toolbar toggle, so we can also toggle the chart toolbar			
			this.toggleSubscription = dojo.subscribe (tipTable.id + "#ToggleToolbar", this, "tableToolbarToggled");
		}
	},
	tableToolbarToggled: function () {
		//this.resizeTable();
		this.toggleChartToolbar();
	},
	toggleChartToolbar: function () {
		console.log ("enter toggle toolbar. existing state: " + this.toolbarHidden_);
		if (this.toolbarHidden_) {
			this.openChartToolbar();
		} else {
			this.closeChartToolbar();
		}
		this.saveToolbarStatus(this.toolbarHidden_);
	},
	saveToolbarStatus: function(toolbarHidden_) {
	  	if(this.preferencesDialog === null || this.preferencesDialog == undefined){
			this.preferencesDialog = new ibm.tivoli.tip.charts.dijit.ChartPreferences ( 
					{ global: this.global,
					  chartBuilder: this.chartBuilder
					} 
				);
		}
		this.preferencesDialog.startup();
		this.preferencesDialog.builder.toolbarSelf.toolbarHidden_ = toolbarHidden_;
		this.preferencesDialog.saveUserPreferences(false);
	},
	openChartToolbar: function () {
		this.toolbarHidden_ = false;
		console.log ("showing toolbar");
		this.animateToolbarOpen();
		dojo.addClass (this.nub, "nubOpen");
		dojo.removeClass (this.nub, "nubClosed");
		dojo.style(this.nub,"position","absolute");
	},
	closeChartToolbar: function () {
		this.toolbarHidden_ = true;
		console.log ("hiding toolbar");
		this.animateToolbarClose();
		dojo.addClass (this.nub, "nubClosed");
		dojo.removeClass (this.nub, "nubOpen");
		dojo.style(this.nub,"position","relative");
	},
	animateToolbarOpen: function () {
		dojo.fx.wipeIn ({
				node: dojo.byId (this.id + "toolbar"),
		        duration: 500,
                onEnd: dojo.hitch(this, function() {
                })
			}).play();
	},
	animateToolbarClose: function () {
		dojo.fx.wipeOut ({
				node: dojo.byId (this.id + "toolbar"),
		        duration: 500,
                onEnd: dojo.hitch(this, function() {
                })
			}).play();
	},
	resizeTable: function () {
		var portletSize = this.getTableSize();
		var size = { w: portletSize.available.w, h: portletSize.available.h };
		console.log ("in resize table with size: ");
		console.log (size);
		dojo.byId (this.global.namespace + "tbl_tipChartData").resize (size);
	},
	resize: function (size) {
		try {
		console.log ("enter ChartToolbar.resize");
		var coords = dojo.coords(this.toolbarBody);
		var centerx =  coords.w/2;
		dojo.style(this.nub,"left",centerx+"px");
		console.log ("styled nub: " + centerx);
		console.log (this.nub);
		} catch (ex) {
			console.log (ex);
			alert (ex);
		}
	},
	getTableSize: function () {
		var size = iscPortlet.getPortletSize(this.global.iscPortletNamespace);
		console.log ("got portlet size, available.w: " + size.available.w + ", h: " + size.available.h);
		var coords = null;
		
		// temporary hack for IE
		if ((size.available.w === undefined) ||
			isNaN(size.available.w)) {
			// guess
			size.available.w = 450;
		}

		var tipTable = dojo.byId (this.global.namespace + "tbl_tipChartData");
		
		var node = this.metaDiv;
		//var showLastUpdated = dijit.byId (this.id + "showLastUpdated");
		if (node ) {
			// subtract size of date 
			coords = dojo.coords (node);
			if (coords.h < 16) {
				coords.h = 16;
			}
			size.available.h = size.available.h - coords.h;
			console.log ("removed meta data size: " + coords.h);
		} else 
		if (this.firstRender) {
			console.log ("firstRender and showLastUpdated is false.");
			// 1st render so those are not available yet, so use fixed size
			size.available.h = size.available.h - 16;
			this.firstRender = false;
		}
		
		// allow for extra space b/c we still get scroll bars on fixed size pages
		if (dojo.isIE) {
			size.available.h = size.available.h - 45;
			size.available.w = size.available.w - 11;
		} else {
			size.available.h = size.available.h - 0;
			size.available.w = size.available.w - 0;
		}

		// minimum height
		if (size.available.h <= 25) {
			size.available.h = 25;
		}
		
		console.log ("return chart size (after subtracting other UI elements): ");
		console.log (size);
		
		return (size);
	},


	onSave: function () {
		//this.savePrimitive ("saveShared", true);
		this.chartBuilder.chartUtil.savePrimitive ("saveShared", true);		
	},
	savePrimitive: function (action, showCompletion) {
		var callback = null;
		console.log ("enter savePrimitive with args: " + action + ", " + showCompletion);
		if (showCompletion) {
			callback = dojo.hitch (this, this.onSaveComplete);
		} 
		
		if (this.allowSubmit_) {
			var hiddenparams = this.getHiddenParameters();
			var url = "?" + hiddenparams;

			var kw = {
				 appendURL: url,
	             content:   {
                        command: "savePreferences",
                        arg_portletNamespace: this.global.namespace,
                        action: action
                        },
            	 onResponse:{
                         callback: callback
       	                }
    	     };
	        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		} else {
			this.allowSubmit_ = true;
		}
		
		// do not submit
		return (false);
	},
	onSaveComplete: function (data, ioArgs) {
		console.info ("enter onSaveComplete");

		var msgID = "TIPCH3005I";
		console.log("messages"+this.messages_);
		var msg = this.messages_.TIPCH3005I;
		this.global.showMessageDialog("info", msgID, msg, false);
	},
	getHiddenParameters: function () {
		var type = "TIPServer";
		var appID = this.chartBuilder.widgetDefinition.datasource.id;

		var urlfrag = this.global.namespace + "chartNameH=" + dojo.byId (this.global.namespace + "chartNameH").value;
		urlfrag += "&" + this.global.namespace + "descriptionH=" + dojo.byId (this.global.namespace + "descriptionH").value;
		urlfrag += "&" + this.global.namespace + "connectionNameH=" + dojo.byId (this.global.namespace + "connectionNameH").value;
		urlfrag += "&" + this.global.namespace + "chartIDH=" + this.chartBuilder.chartID;
		urlfrag += "&" + this.global.namespace+ "chartSourceTypeGroup=" + type;
		urlfrag += "&" + this.global.namespace + "appID=" + appID;
		urlfrag += "&arg_partialSave=false";

		return (urlfrag);	
	},

	
	
  	onRefresh: function () {
  		// TODO: Consider paging
//		var start = this.currentPage * this.chartFetcher.rowsPerPage + 1;
//		this.chartFetcher.fetchData (start, this.chartFetcher.rowsPerPage);
		var busyPane = dijit.byId(this.global.namespace+"busy_pane");
	    if ( busyPane ) {
	      console.log ("showing busypane");
	      busyPane.show(true); // size to portlet
		}
	    if(this.chartBuilder.chartPager){
	    	this.chartBuilder.chartPager.onRefresh();
	    }else{
	    	this.chartFetcher.fetchData();
	    }
  	},
  	onPreferences: function () {
  		console.log ("ChartToolbar.onPreferences: this.global: ");
  		console.log (this.global);
		this.preferencesDialog = new ibm.tivoli.tip.charts.dijit.ChartPreferences ( 
				{ global: this.global,
				  chartBuilder: this.chartBuilder
				} 
			);
		this.preferencesDialog.startup();
		this.preferencesDialog.show();
  	},
  	getToolbar: function () {
  		console.log ("ChartToolbar.getToolbar: returning this.toolbar: " + this.toolbar);
  		return (this.toolbar);
  	},
	saveSharedPreferences: function() {
		console.log("enter charttoolbar.saveSharedPreferences");
		if(this.chartBuilder.widgetDefinition.user == undefined) {
			return;
		}
		var prefs = dojo.toJson(this.chartBuilder.widgetDefinition.user);

		var encodedCID = encodeURIComponent(this.chartBuilder.chartID);		
		var encodedProviderID = encodeURIComponent(this.chartBuilder.providerID);		
		var encodedDS = encodeURIComponent(this.chartBuilder.datasourceID);		
		var encodedPID = encodeURIComponent(this.global.chartProfileID);
		var widgetDefVersion = this.chartBuilder.widgetDefinition.meta.version;	
		var widgetSchema = this.chartBuilder.widgetDefinition.meta.type;
		
		var controller = "/ibm/tivoli/rest";		
		console.log (controller + "/providers/tip.charts/datasources/" + encodedDS + "/widgets/" + widgetSchema + "/definitions/" + encodedCID + "-" + widgetDefVersion + "?type=shared&profileID=" + this.global.chartProfileID + "&registryLocation=local");
		
        var kw = {
        			  sync: true,
					  preventCache: true,
        			  headers: {'Content-Type': 'application/json',
        			  		   'Content-Length': prefs.length
        			  		  },
        			  url: controller + "/providers/" + encodedProviderID + "/datasources/" + encodedDS + "/widgets/" + widgetSchema + "/definitions/" + encodedCID + "-" + widgetDefVersion + "?type=shared&profileID=" + this.global.chartProfileID + "&registryLocation=local",
                      putData: prefs,
                      error: dojo.hitch (this, function (error) {											
							var errHandler = new ibm.tivoli.tip.charts.dojo.RESTErrorHandler(this.global);		
							errHandler.showError(dojo.fromJson(error.responseText));	
		 					})
                   };
          dojo.xhrPut (kw);         
          console.log ("exit charttoolbar.saveSharedPreferences");          
	},
  	destroy: function () {
		this.inherited ("destroy", arguments);
		if(this.toolbar){
			this.toolbar.destroyRecursive();
		}	
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartTitle"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartTitle"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.charts.dijit.ChartTitle");



// NLS


dojo.declare("ibm.tivoli.tip.charts.dijit.ChartTitle",
              [ dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true,
	templateString:"<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\" align=\"center\">\r\n\t<tbody>\r\n\t\t<tr>\r\n\t\t\t<td> \r\n\t\t\t\t<div class=\"chartIcon\" id=\"${id}icon\">\r\n\t\t\t\t</div>\r\n\t\t\t</td>\t\t\t\r\n\t\t\t<td> \r\n\t\t\t\t<div class=\"chartTitle\" id=\"${id}text\">\r\n\t\t\t\t</div>\r\n\t\t\t</td>\t\t\t\r\n\t\t</tr>  \r\n\t</tbody>\r\n</table>\r\n",
	title: null,
	namespace: "1234",
	chartid: null,
	
	constructor: function () {
	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");    
	},
	postCreate: function () {
		if (this.title !== null) {
			this.setTitle (this.title);
		}
		
		this.makeTitleContextMenu();
	},
	setTitle: function (title) {
		var textDiv = dojo.byId (this.id + "text");
		
		var titleHTML = "<span>" + title + "</span>";
		textDiv.innerHTML = titleHTML;
	},
	makeTitleContextMenu: function () {
		console.log ("enter makeTitleContextMenu");	
		var provider = new ibm.tivoli.tip.dijit.TIPContextMenuProvider(
					{ iscPortletNamespace: this.iscPortletNamespace,
					  getMenuItemsCallback: dojo.hitch (this, this.getMenuItems)
					});
		console.log ("after make provider");

		var menu = new ibm.tivoli.tip.dijit.TIPContextMenuProxy(
			{cmProvider: provider, 
			 targetNodeIds: [this.id + "icon"],
			 getContextCallback: dojo.hitch (this, this.getSelectedData)
			});
		console.log ("after create context menu");
	},
	getSelectedData: function () {
	},
	getMenuItems: function () {
		var menuItems = [
			     			{ id: this.namespace + "RefreshChartMenuItem", label: this.resources_.REFRESH, iconClass: "refreshIcon", launchType: "JavaScript", uri: "tipdijit.byId ('" + this.chartid + "').onRefresh();", enabled: true },
			     			{ id: this.namespace + "PreferencesMenuItem", label: this.resources_.PREFERENCES, iconClass: "propertiesIcon", launchType: "JavaScript", uri: "tipdijit.byId ('" + this.chartid + "').onPreferences();", enabled: true }
			     		];		
		return (menuItems);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.Builder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.Builder"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2010 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.charts.dijit.Builder");


dojo.declare("ibm.tivoli.tip.charts.dijit.Builder",
	          null,
{
	// Note: See https://w3.tap.ibm.com/w3ki03/display/TIP/Event%20Names
	//  and  https://w3.tap.ibm.com/w3ki03/display/TIP/Advanced%20Content%20Passing
	//  for information about this design
	global: null,
	// e.g. mssName: "ibm-cdm:///CDMMSS/Hostname=loco.raleigh.ibm.com+ManufacturerName=IBM+ProductName=IBM Tivoli Monitoring Services"
	mssName_: null,
	matchOn_: null,
	contextType_: null,
	event_: null,
	
	processEvent: function (event) {
		console.log ("enter Builder.processEvent");
		console.dir (event);
		try {
			if (event.name == "http://ibm.com/isclite#portletResize") {
				// handled outside of this code
			} else if (event.name == "http://ibm.com/tip#NodeClickedOn") {
				this.handleNodeClickedOnEvent (event);
			} else if (event.name == "http://ibm.com/tip#TimeSet") {
				this.handleTimeSetEvent (event);
			} else if (event.name == "http://ibm.com/isclite#launchPage") {
				this.handleNodeClickedOnEvent (event);
			} else if (event.name == "http://ibm.com/isclite#switchPage") {
				this.handleNodeClickedOnEvent (event);
			} else if (event.name == "http://ibm.com/isclite#launchPortlet") {
				console.log ("handling launchPortlet event");
			} else if (event.name == "http://ibm.com/tip#changeTheme") {
				console.log ("received changeTheme event");
				this.changeTheme(event);
			} else {
				console.log ("received unknown event: ");
				console.dir (event);
			}
		} catch (e) {
			console.error ("caught exception in processEvent");
			console.error (e);
		}
		console.log ("exit Builder.processEvent");
	},
	changeTheme: function (event) {
		// default is do nothing.  override if needed
	},
	handleNodeClickedOnEvent: function (event) {
		console.log ("enter Builder.handleNodeClickedOnEvent");

		if ((this.global.builder.widgetDefinition.datasets[0].supportsWire === undefined) ||
		    (this.global.builder.widgetDefinition.datasets[0].supportsWire)) {
		    console.log ("supportsWire is set to true. processing event: " + this.global.builder.widgetDefinition.datasets[0].supportsWire);
			this.event_ = null;
			if (this.mssName_ === null) {
				console.log ("mssName is null... fetching it: " + this.mssName_);
				this.event_ = event;
				this.fetchMSSName(false, dojo.hitch(this, this.afterFetchMSSName));
			} else {
				console.log ("already retreived mssName. it is: " + this.mssName_);
				this.matchMSSName(event);
			}
		} else {
			console.info ("via widget definition, wiring is off for this chart widget. ignoring event.");
		}
	},
	afterFetchMSSName: function (data) {
      	console.log ("return from fetching mss name: " + data.MSSName);
		this.mssName_ = data.MSSName;
		this.matchOn_ = data.matchOn;
		this.contextType_ = data.contextType;
		this.matchMSSName (this.event_);
		this.event_ = null;
	},
	matchMSSName: function (event) {							
		console.log ("enter matchMSSName"); 
//		var matchingMSSIndex = this.getMatchingMSSIndex (event, this.getSourceMSSName());		
			// alert ("matchingMSSIndex: " + matchingMSSIndex)
//			if (matchingMSSIndex < 0) {
				// no matching MSS found, so call to transform
//				event = this.transformEvent (event);
//			} else {
				// matching MSS found, so no need to transform, 
				// just invoke post transform processing
		this.onTransformEnd (event);
//			}
//		} else {
//			console.log ("receive context is off or not determined yet. cacheing event.");
//			this.event_ = event;
//		}
	
	},
	getMatchingMSSIndex : function (event, srcMssName) {
		console.log ("src MSS: " + srcMssName);
		var index = -1;
		
		if (this.equalsIgnoreCase (event.MSSName, srcMssName)) {
			index = 0;
		} else {
			var i = 1;
			console.log ("entering for loop");
			forloop:
			for (i=1; ; i++) {
				console.log ("in for loop with index: " + i);
				var value = event["MSSName" + i];
				console.log ("event.MSSName" + i + ": " + value);
				if (value) {
					if (this.equalsIgnoreCase (value, srcMssName)) {
						index = i;
						break forloop;
					}
				} else {
					break forloop;
				}
			}
		}
		
		console.log ("returning matching mss index: " + index);
		return (index);
	},
	equalsIgnoreCase: function (str1, str2) {
		if ((str1) && (str2)) {
			if ((str1 === undefined) || (str2 === undefined)) {
				return false;
			}
			if ((str1 === null) && (str2 === null)) {
				return (true);
			} else if ((str1 === null) || (str2 === null)) {
				return (false);
			} else {
				return (str1.toLowerCase() == str2.toLowerCase());
			}
		} else {
			return (false);
		}
	},	
	transformEvent: function (event) {
		console.log ("enter transformEvent with event:");
		console.dir (event);
		var transformInfo = {
                              "uniqueName": "com.ibm.tivoli.tip.TIPTransformation",
                              "locationUniqueName": "com.ibm.tivoli.tip.TIPTransformation",
                              "eventObj": event,
                              "srcPortletNamespace": this.global.iscPortletNamespace,
                              "handler": dojo.hitch (this, this.onTransformEnd)
                            };
        EventBroker.executeTransform(transformInfo);
	},
	onTransformEnd: function (event) {
		console.log ("enter onTransformEnd with event: ");
		console.dir (event);
		
		// check for multiple resources and copy first resource to root of event
		// We do not support multiple resources yet, just 1
		var util = new ibm.tivoli.tip.dojo.EventUtil();
		util.addOldContext (event);
		console.log ("after adding old context to event: ");
		console.dir (event);
		
		var matchingMSSIndex = this.getMatchingMSSIndex (event, this.getSourceMSSName());
		if (matchingMSSIndex > 0) {
			console.log("MSSName: " + event["MSSName" + matchingMSSIndex]);
			console.log ("SourceToken: " + event["SourceToken" + matchingMSSIndex]);

			if ((event["MSSName" + matchingMSSIndex] !== undefined) &&
			 	(event["MSSName" + matchingMSSIndex] !== null)) {
				event.MSSName = event["MSSName" + matchingMSSIndex];
			}
			if ((event["SourceToken" + matchingMSSIndex] !== undefined) &&
			 	(event["SourceToken" + matchingMSSIndex] !== null)) {
				event.SourceToken = event["SourceToken" + matchingMSSIndex];
			}
			
			console.log ("event: ");
			console.dir (event);
		}
		
		this.updateChart (event);
		/*
		// Update properties
		if (this.global.builder.isChart == true) {
			this.updateChart (event);
		} else if (this.global.builder.isTable == true) {	
			//rebuilds the table
			this.global.builder.updateTableData (event);
		} else {
			console.info ("Builder.onTransformEnd: no method for update");
		} 
		*/
	},
	fetchMSSName: function (sync, callback) {
		console.log ("enter fetchMSSName (sync, callback): " + sync + ", " + callback);
		if (this.mssName_ !== null) {
			console.log ("returning mssName_: " + this.mssName_); 
			return (this.mssName_);
		}
		
		var encodedProviderID = encodeURIComponent(this.global.builder.providerID);	
		var kw = {
        			  url: "/ibm/tivoli/rest/providers/" + encodedProviderID,
					  handleAs: "json",
    				  mimetype: "application/json",
                      error: dojo.hitch (this, function (error) {											
							var errHandler = new ibm.tivoli.tip.charts.dojo.RESTErrorHandler(this.global);		
							errHandler.showError(dojo.fromJson(error.responseText));	
		 			  })
                   };
          if (sync) {
          	kw.sync = sync;
          }

          if (callback) {
          	kw.load = callback;
          } else {
          	kw.load = dojo.hitch (this, function (data) {
          		this.mssName_ = data.MSSName;
          		this.matchOn_ = data.matchOn;
          		this.contextType_ = data.contextType;
          	});
          }

          dojo.xhrGet ( kw );
          
          // this is useful when calling synchronously
          return (this.mssName_);
	},
	getSourceMSSName: function () {
		if (this.mssName_ == null) {
			console.log ("getSourceMSSName: mssName is null... fetching it: " + this.mssName_);
			this.fetchMSSName(true);
		}
	
		//	e.g., "ibm-cdm:///CDMMSS/Hostname=pico.raleigh.ibm.com,ManufacturerName=IBM,ProductName=IBM Tivoli Monitoring"
		return this.mssName_;
	},
	eventParametersToRestParameters: function (event) {
		console.log ("enter Builder.eventParametersToRestParameters: ");
		if (event === undefined) {
			return (null);
		}
		
		var restParams = [];
		
		if(event.payload && event.payload.product){
			if(event.payload.cdm && event.payload.cdm.attributes){
				restParams.push ({
					id: "ClassType",
					value: event.payload.cdm.attributes.ClassType
				});
			}
			for (x in event.payload.product){				
				for(y in event.payload.product[x]){					
					if(y == "cdm"){
						for(z in event.payload.product[x][y]){
							restParams.push ({
								id: z,
								value: event.payload.product[x][y][z]
							});
						}
					}else{
						restParams.push ({
							id: y,
							value: event.payload.product[x][y]
						});
					}					
				}
			}			
		} 

		// we should always use these base event parameters as REST parameters				
		var param = null;
		
		for (var i in event) {
			if(i!="name" && i!="cdm" && i!="payload" && i!="product"){
				param = {};
				param.id = i;
				param.value = event[i];
				restParams.push (param);
			}
		}

		console.log ("exit Builder.eventParametersToRestParameters: ");
		console.log (restParams);
		
		return (restParams);
	},
	
	handleTimeSetEvent: function(event){		
		if(event.payload){
			this.updateChartByTimeSet(event.payload.Time);
		}else{
			this.updateChartByTimeSet(event.Time);
		}
	},	
	
	createCMSEventCallback: function(data){
		var event = {
			name: data.event,
			payload: {}
		};
		var mssName = null;
		var sourceToken = null;
		var cdmClass = null;		
		var contextData = {};
		for(var i=0;i<data.parameters.length;i++){			
			var param = data.parameters[i];			
			if(param.name=="SourceToken"){
				sourceToken = param.value.toString();
				if (sourceToken === "{SourceToken}") {
					sourceToken = null;
				}
			}else if(param.name=="MSSName"){
				mssName = param.value.toString();
				if (mssName === "{MSSName}") {
					mssName = null;
				}
			}else if(param.name=="ClassType"){
				cdmClass = param.value.toString();
			}else{
				contextData[param.name] = param.value.toString();
			}
		}
		if ((cdmClass) && (cdmClass !== null)) {
			event.payload.cdm = {
				attributes: {
					ClassType: cdmClass
				}
			};
		}
		if ((mssName) && (mssName !== null)) {	
			event.payload.product = {};
			var productName = mssName.substring(mssName.lastIndexOf('=')+1).replace(/ /g,"");
			event.payload.product[productName] = {
				cdm: {
					MSSName: mssName
				}
			};
			if((sourceToken) && (sourceToken !== null)) {
				event.payload.product[productName].cdm.SourceToken = sourceToken;
			}
			//  do not mixin generic properties under the productName... Only include SourceToken there (i.e., things specific to that MSSName)
			// dojo.mixin(event.payload.product[productName], contextData);
		}
		dojo.mixin(event.payload, contextData);
		
		return event;
	},		
	
	updateChart: function (event) {
		console.log ("enter Builder.updateChart");				
		var widgetDef = this.global.builder.widgetDefinition;		

		//To restore parameters in widgetDef
		//var oldParams = dojo.clone(widgetDef.datasets[0].parameters);
	
		// We are NOT implementing matchOn provider argument in this release... so commenting this out.
		// Beware that the matchOn code was NOT working anyway	
//		if ((this.matchOn_ !== null) && (this.matchOn_ !== undefined) && (this.matchOn_ === "all")) {
//			console.log ("provider matchOn is all... sending event context as JSON object");
//			widgetDef.datasets[0].parameters.eventContext = dojo.toJson(event);
//		} else {
			console.log ("using default value for provider matchOn. flattening parameters for REST call.");
			var newParams = this.eventParametersToRestParameters(event);
			if (newParams !== null) {
				//widgetDef.datasets[0].parameters = newParams;
				for(var i=0; i<widgetDef.datasets[0].parameters.length; i++){
					for(var j=0; j<newParams.length; j++){
						if(widgetDef.datasets[0].parameters[i].id == newParams[j].id){
							widgetDef.datasets[0].parameters[i] = newParams[j];
							newParams.splice(j,1);
							break;
						}
					}
				}
				widgetDef.datasets[0].parameters = widgetDef.datasets[0].parameters.concat(newParams);
			}
//		} 
		
		var builder = this.global.builder;	
		var encodedCID = null;
		if (this.global.builder.isChart == true) {			
			encodedCID = encodeURIComponent(builder.chartID);
		}
		else if (this.global.builder.isTable == true) {			
			encodedCID = encodeURIComponent(builder.tableID);
		}
		var encodedProviderID = encodeURIComponent(builder.providerID);		
		var encodedDS = encodeURIComponent(builder.datasourceID);
		var encodedPID = encodeURIComponent(this.global.chartProfileID);
		
		var drillUpItems = builder.drillUpItems;
		//show busyPane
		var busyPane = dijit.byId(this.global.namespace+"busy_pane");
	    if ( busyPane ) {	      
	      busyPane.show(true); // size to portlet
		}	
		if(!this.global.widgetFetcher){
			this.global.widgetFetcher = new ibm.tivoli.tip.charts.dojo.WidgetFetcher ({
				global: this.global
			});
		}	
		var fetcher = this.global.widgetFetcher;
		builder.destroyRecursive();		
		var domNode = dojo.byId (this.global.namespace+"widgetDiv");		
		dojo.destroy(domNode);
		domNode = dojo.doc.createElement("div");
		domNode.id = this.global.namespace+"widgetDiv";
		dojo.byId (this.global.namespace + "widgetContainer").appendChild(domNode);
		/* Separate Dojo Chart from Birt
		var domNode = dojo.byId (this.global.namespace+"widget");
		builder.destroy();		
		dojo.empty(domNode);
		dojo.place (domNode, dojo.byId (this.global.namespace + "birtChartWidget"), "after");			
		*/
		// There is no need to read user or shared preferences from the embedded JSON
		// Those values are merged into the base widget def by CURI before it is returned
		// from the server.
		var chartType = widgetDef.details.preferredView;

		if (chartType == "Table") {			
			this.global.builder = fetcher.buildTable (encodedProviderID, encodedDS, widgetDef, domNode, encodedCID, null, builder.isTableFirst, drillUpItems);
			this.global.builder.mssName_ = this.mssName_;						
		} else {
			this.global.builder = fetcher.buildChart (encodedProviderID, encodedDS, widgetDef, encodedCID, domNode, null, drillUpItems);
			this.global.builder.mssName_ = this.mssName_;			
		}		
		//this.global.builder.widgetDefinition.datasets[0].parameters = oldParams;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dojo.DojoChartUtil"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dojo.DojoChartUtil"] = true;
dojo.provide("ibm.tivoli.tip.charts.dojo.DojoChartUtil");







dojo.declare("ibm.tivoli.tip.charts.dojo.DojoChartUtil", null, {

	chartBuilder: null,
	namespace: null,
	providerID: null,
	datasourceID: null,
	savePrefsOption: true,

	constructor: function(mixinData) {
		console.log ("enter DojoChartUtil.ctor");
		dojo.mixin( this, mixinData );
	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.chartportlet.dijit", "chart_resources");
	    this.messages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.chartportlet.dijit", "chart_messages");
		this.global = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace);
		this.allowSubmit_ = true;
		this.interval = null;
		this.tipHelper_ = new ibm.tivoli.tip.dojo.TIPHelper();			
		if(!this.chartBuilder.widgetParams || !this.chartBuilder.widgetParams.standalone){
			this.addToolbarButtons();
		}
		if(this.savePrefsOption == true) {
			this.saveUserPrefs();
		}
		console.log ("exit DojoChartUtil.ctor");
	},
	addToolbarButtons: function () {
		console.log ("enter DojoChartUtil.addToolbarButtons");
		var toolbar = this.chartBuilder.getToolbar();		
		var oldPropertiesButton = dijit.byId (this.chartBuilder.id + "propertiesButton");
		toolbar.removeChild(oldDropDownButton);
		if(oldPropertiesButton != null || oldPropertiesButton != undefined) {
			oldPropertiesButton.destroy();
		}
		var toolbarWidget = dijit.byId (this.chartBuilder.id + "toolbar");
		var buttonPref = new ibm.tivoli.tip.dijit.TIPButton({	
			label: this.resources_.PREFERENCES,
			id: this.chartBuilder.id + "propertiesButton",
			iconClass: "propertiesIcon",
			showLabel: false,
			onClick: dojo.hitch (toolbarWidget, toolbarWidget.onPreferences) 
		});			
		toolbar.addChild(buttonPref);
		var ddMenu = new dijit.Menu ();
		var mi = null;			
		var context = this.global;
		if (context.roles.admin || context.roles.creator) {
			mi = new dijit.MenuItem ({
				label: this.resources_.SAVE_FOR_OTHERS
			});
			mi.onClick = dojo.hitch (this, "onSave");
			ddMenu.addChild (mi);
			
			mi = new dijit.MenuItem ({
				label: this.resources_.CLEAR_DEFAULT_PREFERENCES
			});
			mi.onClick = dojo.hitch (this, "onReset");
			ddMenu.addChild (mi);
		}
			
		mi = new dijit.MenuItem ({
				label: this.resources_.CLEAR_MY_PREFERENCES
			});
		mi.onClick = dojo.hitch (this, "onResetMyPreferences");
		ddMenu.addChild (mi);
			
		var oldDropDownButton = dijit.byId (this.chartBuilder.id + "dropDownButton");
		//toolbar.removeChild(oldDropDownButton);
		if(oldDropDownButton != null || oldDropDownButton != undefined) {
			oldDropDownButton.destroy();
		}

		button = new dijit.form.DropDownButton (
			{	label: this.resources_.PUBLISH,
				id: this.chartBuilder.id + "dropDownButton",
				showLabel: false,
				iconClass: "editIcon",
				dropDown: ddMenu
			});
		toolbar.addChild (button);
		console.log ("made user prefs button: ");
		console.log (button);	
		
		console.log ("exit DojoChartUtil.addToolbarButtons");
	},
	saveUserPrefs: function () {
		this.savePrimitive ("saveUser", false);
	},
	onSave: function () {
		this.savePrimitive ("saveShared", true);
	},
	savePrimitive: function (action, showCompletion) {
		var callback = null;
		console.log ("enter savePrimitive with args: " + action + ", " + showCompletion);
		if (showCompletion) {
			callback = dojo.hitch (this, this.onSaveComplete);
		} 
		
		if (this.allowSubmit_) {
			var hiddenparams = this.getHiddenParameters();
			var url = "?" + hiddenparams;

			var kw = {
				 appendURL: url,
	             content:   {
                        command: "savePreferences",
                        arg_portletNamespace: this.namespace,
                        action: action
                        },
            	 onResponse:{
                         callback: callback
       	                }
    	     };
	        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		} else {
			this.allowSubmit_ = true;
		}
		
		// do not submit
		return (false);
	},
	onResetPrimitive: function (action, reloadPrefs) {
		var callback = null;
		
		if (reloadPrefs === false) {
			callback = null;
		} else {
			callback = dojo.hitch(this, this.afterReset);
		}
		
		var kw = {
	             content:   {
                        command: "savePreferences",
                        arg_portletNamespace: this.namespace,
						arg_chartProfileID: this.global.chartProfileID,
                        action: action
                        },
            	 handle: callback
    	     };
	
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		
		// do not submit
		return (false);
	},
	onReset: function () {
		this.onResetPrimitive ("resetDefaults");
	},
	onResetMyPreferences: function (reloadDefaultPrefs) {
		this.onResetPrimitive ("resetMine", reloadDefaultPrefs);
	},
	afterReset: function () {
		if(!this.global.widgetFetcher){
			this.global.widgetFetcher = new ibm.tivoli.tip.charts.dojo.WidgetFetcher ({
				global: this.global
			});
		}		
		var providerID = this.providerID;
		var dsID = this.datasourceID;
		var widgetID = this.global.builder.chartID;
		var widgetVersion = this.global.builder.widgetDefinition.meta.version;		
		var domNode = dojo.byId (this.global.namespace+"widgetDiv");			
		this.global.builder.destroyRecursive();		
		dojo.destroy(domNode);
		domNode = dojo.doc.createElement("div");
		domNode.id = this.global.namespace+"widgetDiv";
		dojo.byId (this.global.namespace + "widgetContainer").appendChild(domNode);		
		this.global.builder = this.global.widgetFetcher.buildWidget (providerID, dsID, widgetID, domNode, widgetVersion);	
	},
	restorePortletTitle: function () {
		// restore portlet title to translated Charting
		this.tipHelper_.setPortletTitle (this.chartBuilder.domNode, this.global.getPortletTitle());
	},
	hide: function () {
		this.chartBuilder.domNode.style.display = "none";
	},
	show: function () {
		this.chartBuilder.domNode.style.display = "";
	},
	onSaveComplete: function (data, ioArgs) {
		console.info ("enter onSaveComplete");

		//PUT the shared preferences
		var toolbarWidget = dijit.byId (this.chartBuilder.id + "toolbar");			
		toolbarWidget.saveSharedPreferences();
		
		var msgID = "TIPCH3005I";
		var msg = this.messages_.TIPCH3005I;
		this.global.showMessageDialog("info", msgID, msg, false);
	},
	getHiddenParameters: function () {
		var type = "TIPServer";
		var appID = this.chartBuilder.widgetDefinition.datasource.id;
		var urlfrag = this.namespace + "connectionNameH=" + this.datasourceID;
		urlfrag += "&" + this.namespace + "chartIDH=" + this.chartBuilder.chartID;
		urlfrag += "&" + this.namespace + "chartSourceTypeGroup=" + type;
		urlfrag += "&" + this.namespace + "appID=" + this.datasourceID;
		urlfrag += "&" + this.namespace + "providerID=" + this.providerID;
		urlfrag += "&" + this.namespace + "versionH=" + this.chartBuilder.widgetDefinition.meta.version;
		urlfrag += "&arg_partialSave=false";

		return (urlfrag);	
	},
	destroy: function () {
		this.chartBuilder.destroyRecursive();
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartBuilder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartBuilder"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
// test version control - 2.2.0

dojo.provide("ibm.tivoli.tip.charts.dijit.ChartBuilder");












































dojo.require ("ibm.tivoli.tip.charts.dojo.DojoChartUtil");
dojo.require ("ibm.tivoli.tip.dojo.TIPHelper");

// NLS


dojo.declare("ibm.tivoli.tip.charts.dijit.ChartBuilder",
	          [ dijit._Widget, dijit._Templated, ibm.tivoli.tip.charts.dijit.Builder ],
{
	widgetsInTemplate: true,
	templateString:"<div class=\"chartBackground\">\r\n\t<div id=\"${id}title\"></div>\r\n\t<div id=\"${id}toolbar\"></div>\r\n\t<table id=\"${id}chartContainer\">\r\n\t\t<tr>\r\n\t\t\t<td style=\"vertical-align: middle;\"> <div  class=\"css-vertical-text\" id=\"${id}axistitle\"></div></td>\r\n\t\t\t<td> <div id=\"${id}chart0\" oncontextmenu=\"return false;\"></div>\r\n\t\t\t</td>\r\n\t\t\t<td style=\"vertical-align: middle;\">\r\n\t\t\t\t<div id=\"${id}rightLegend\" style=\"vertical-align:middle;\"></div>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t<tr align=\"center\">\r\n\t\t\t<td colspan=\"3\">\r\n\t\t\t\t<div id=\"${id}axistitle_X\" style=\"horizonal-align: center;\"></div>\t\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t\t<tr align=\"center\">\r\n\t\t\t<td colspan=\"3\">\r\n\t\t\t\t<div id=\"${id}bottomLegend\" style=\"horizonal-align: center;\"></div>\t\r\n\t\t\t</td>\r\n\t\t</tr>\r\n\t</table>\r\n\r\n\t<div id=\"${id}metaDataDiv\" dojoAttachPoint=\"metaDiv\">\r\n\t</div>\r\n\t<div id=\"${id}pager\"></div>\r\n</div>\r\n",

	namespace: "namespace1_",
	global: null,
	chartID: null,
	chartPreferences: null,
	widgetDefinition: null,
	updateMultiplier: 10,
    updateMinInterval: 30000,
    updateMaxInterval: 180000,
	interval: null,
	runReportStartTime: 0,
    runReportResponseTime: 5,
    datasourceID: null,
	dataSetID: null,
	providerID: "tip.charts",
	chartPager: null,
	axisLabelCount: 0,
	theme: ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandard.clone(),
	custom: ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandardCustom,
	standalone: false,
	ignoreContextMenu: false,
	ignoreTitlebar: false,
	ignoreToolbar: false,
	initSize: null,
    timeVar: null,
	trigger: null,
	drillUpItems: [],
	pieCMTargetNodeIds: [],
	isChart: true,
	themeIndex: 0,
	_currentSize: null,
	chartDivSize: null,

	constructor: function () {		
	    //dojo.mixin( this, mixinData );
		this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
		this.chartBuilder = true;		
		this.dc = dojox.charting;
		this.firstRun = true;
		this.chart = null;
		this.charts = [];
		this.columns = null;
		this.rows = null;
		this.store = null;
		this.chartFetcher = null;
		this.chartDef = null;
		this.legend = null;
		this.dataset = null;
		this.toolbarSelf  =  null;
        this.common = null;
        this.pieCMTargetNodeIds = [];
		this.seriesLength = 0;
		this._currentSize = null;
		this.chartDivSize = null;
	},
	startup: function () {
		this.chartDef = this.getChartDef (this.chartID);
		this.setupTheme();		 
		this.fetchChartData();
		this.dataset = this.getDataSet (this.dataSetID);
		this.common = this.getCommonData ();
		/*
		if(!this.standalone || (this.standalone && !this.ignoreContextMenu)){
			this.makeChartContextMenu();
		}
		*/
	},
	
	setupTheme: function(){
		if (this.chartDef.theme !== undefined) {
			console.log ("setting theme from widget definition: " + this.chartDef.theme.name);
			this.setTheme(this.chartDef.theme.name);
			this.themeIndex = this.chartDef.theme.startIndex;
			console.log ("themeIndex: " + this.themeIndex);
		} else if ((this.global.theme  !== undefined) && (this.global.theme !== null)) {
			console.log ("setting theme from global: " + this.global.theme);		
			this.setTheme(this.global.theme);
		} else {
			console.log ("no theme from widget def or global");
		}
		
		if ((this.theme.colors) && (this.theme.colors !== null)) {
			this.arrangeColorPalette();
		} else {
			this.arrangeGradientPalette();
		}
	},
	
	getChartDef: function (chartID) {
		var chartDef = null;		
		if(this.widgetDefinition.charts === undefined)
		{
			chartDef = this.widgetDefinition.tables[0];
			return(chartDef);
		}

		for (var i=0; i < this.widgetDefinition.charts.length; i++) {
			if (this.widgetDefinition.charts[i].id == chartID) {
				return (this.widgetDefinition.charts[i]);
			}			
		}
		
		console.error ("no chart definition found for: " + chartID + " in widgetDefinition:");
		
		return (null);
	},
	makeChartContextMenu: function () {
		
		var cmClientProvider = new ibm.tivoli.tip.dijit.TIPContextMenuProvider(
					{ iscPortletNamespace: this.iscPortletNamespace,
					  getMenuItemsCallback: dojo.hitch (this, this.getMenuItems)
					});
		
		var provider = new ibm.tivoli.tip.dijit.CMSMenuProviderV2(
				{ global: this.global,
				  menuName: "DojoTableV2",
				  getMenusCommandName: "getMenuItems",
				  getMenuActionCommandName: "getMenuCommand",	
				  createCMSEventCallback: this.createCMSEventCallback,			  
				  iscPortletNamespace: this.global.iscPortletNamespace
				});
		
		console.log ("after make provider");		

		this.chartMenu = new ibm.tivoli.tip.dijit.TIPContextMenuProxy(
			{cmProvider: provider, 
			 cmClientProvider: cmClientProvider,
			 //targetNodeIds: [this.id + "chart0"],
			 targetNodeIds: this.pieCMTargetNodeIds,
			 getContextCallback: dojo.hitch (this, this.getSelectedData),
			 id: this.global.namespace + "chartMenu"
			});
		console.log ("after create context menu");
	
	},
	getSelectedData: function (event) {
		var contextData = {};
		contextData.rcfs = [];
		var colNames = this.widgetDefinition.datasets[0].columns;
		var datasourceType = this.widgetDefinition.datasource.type;
		var datasourceId = this.datasourceID;		
		var datasetId = this.widgetDefinition.datasets[0].id;		
		var rowIndex = this.chartDef.type == "Pie" ? event.currentTarget.id.substr(event.currentTarget.id.length-1)-1 : event.index ;		
		var resourceAttrs = [];
		var attr = null;					
		for(var i=0; i<colNames.length; i++){
			attr = {};
			attr.name = colNames[i].label.key;			
			attr.value = this.store.getValue (this.rows[rowIndex], colNames[i].id);			
			attr.isNamingAttribute = false;
			resourceAttrs.push(attr);
		}			
		
		attr = {};
		attr.name = "Class";
		attr.value = "*";
		attr.isNamingAttribute = false;
		resourceAttrs.push(attr);
		contextData.rcfs.push (resourceAttrs);
		
		contextData.launchTypes = ["WEB_URL", "PORTAL_PAGE", "PORTLET", "CLIENT_EVENT"];
		contextData.excludedAppIDs = [];
		contextData.substitutionVariables = [];
		
		attr = {};
		attr.name = "RowIndex";
		attr.value = this.store.getIdentity(this.rows[rowIndex]);	
		contextData.substitutionVariables.push(attr);
		
		attr = {};
		attr.name = "MSSName";
		attr.value = this.fetchMSSName(true);
		if(attr.value!==undefined && attr.value!==null){
			contextData.substitutionVariables.push(attr);
		}
		
		attr = {};
		attr.name = "SourceToken";		
		attr.value = this.store.getValue (this.rows[rowIndex], ""+colNames.length);
		if(attr.value!==undefined && attr.value!==null){
			contextData.substitutionVariables.push(attr);
		}
		
		attr = {};
		attr.name = "DatasourceType";
		attr.value = datasourceType;	
		contextData.substitutionVariables.push(attr);
		
		attr = {};
		attr.name = "DatasourceId";
		attr.value = datasourceId;	
		contextData.substitutionVariables.push(attr);	
		
		attr = {};
		attr.name = "DatasetId";
		attr.value = datasetId;	
		contextData.substitutionVariables.push(attr);
		contextData.options = [];		
		return contextData;		
	},
	getCommonMenuItems: function () {
		var menuItems = [{ 
			id: this.id + "RefreshChartMenuItem", 
			label: this.resources_.REFRESH, 
			iconClass: "refreshIcon", 
			launchType: "JavaScript", 
			uri: "dijit.byId ('" + this.id + "toolbar" + "').onRefresh();", 
			enabled: true 
		}];
        var subMenuItems = null;
		if(this.widgetDefinition.drillDown !== undefined) {				
			subMenuItems = this.getDrillDownMenuItems();
			var oldDrillDownNode = dijit.byId (this.id + "DrillDown");		
			if(oldDrillDownNode) {
				oldDrillDownNode.destroy();
			}
			menuItems.push ({ 
				id: this.id + "DrillDown", 
				label: this.resources_.DRILL_DOWN, 
				icon: "", 
				launchType: "NOOP", 
				submenu_items: subMenuItems, 
				enabled: true 
			});
		}
		if(this.drillUpItems !== undefined && this.drillUpItems.length > 0) {	
			subMenuItems = this.getDrillUpMenuItems();
			var oldDrillUpNode = dijit.byId (this.id + "DrillUp");		
			if(oldDrillUpNode) {
				oldDrillUpNode.destroy();
			}				
			menuItems.push ({ 
				id: this.id + "DrillUp", 
				label: this.resources_.DRILL_UP, 
				icon: "", 
				launchType: "JavaScript", 
				submenu_items: subMenuItems, 
				enabled: true 
			});
		}	
		return (menuItems);
	},
	getCustomMenuItems: function () {
			var menuItems = [{ 
				id: this.id + "ShowHideToolbarMenuItem", 
				label: this.resources_.SHOW_HIDE_TOOLBAR, 
				iconClass: "toggleIcon", 
				launchType: "JavaScript", 
				uri: "dijit.byId ('" + this.id + "toolbar" +"').toggleToolbar();", 
				enabled: true 
			}];						
			if(!this.global.isFromPortletPref){
				menuItems.push({ 
					id: this.id + "ReturnToChartSelectorMenuItem", 
					label: this.resources_.RETURN_TO_CHART_SELECTOR, 
					iconClass: "backIcon", 
					launchType: "JavaScript", 
					uri: "dijit.byId ('" + this.id +"toolbar"+ "').showChartSelector();", 
					enabled: true 
				});
			}
			menuItems.push({ id: "separator" });
			menuItems.push({ 
				id: this.id + "PreferencesMenuItem", 
				label: this.resources_.PREFERENCES, 
				iconClass: "propertiesIcon", 
				launchType: "JavaScript", 
				uri: "dijit.byId ('" + this.id + "toolbar" + "').onPreferences();", 
				enabled: true 
			});
			var context = ibm.tivoli.tip.dojo.Global.getSingleton(this.global.namespace);
			if (context.roles.admin || context.roles.creator) {
			    menuItems.push ({ 
			    	id: this.id + "SaveForAll", 
			    	label: this.resources_.SAVE_FOR_OTHERS, 
			    	icon: "", 
			    	launchType: "JavaScript", 
			    	uri: "dijit.byId ('" + this.id +"toolbar"+ "').onSave();", 
			    	enabled: true 
			    });
				menuItems.push ({ 
					id: this.id + "ResetForAll", 
					label: this.resources_.CLEAR_DEFAULT_PREFERENCES, 
					icon: "", 
					launchType: "JavaScript", 
					uri: "dijit.byId ('" + this.id +"toolbar" + "').onReset();", 
					enabled: true 
				});
			}
			menuItems.push ({ 
				id: this.id + "ResetForMe", 
				label: this.resources_.CLEAR_MY_PREFERENCES, 
				icon: "", 
				launchType: "JavaScript", 
				uri: "dijit.byId ('" + this.id + "toolbar" +"').onResetMyPreferences();", 
				enabled: true 
			});				
			menuItems.push({ id: "separator" });
			return (menuItems);
	},
	
	getMenuItems: function () {
		var menuItems = null;
		var commonMenuItems = this.getCommonMenuItems();
		if(this.standalone){
			if(!this.ignoreContextMenu){
				menuItems = commonMenuItems;
			}
		}else{
			var customMenuItems = this.getCustomMenuItems();
			menuItems = commonMenuItems.concat(customMenuItems);
		}		
		return (menuItems);
	},
	
	getDrillDownMenuItems: function() {
		var subMenuItems = [];		
		for (var i=0; i<this.chartDef.contextMenu.length; i++) {
			var drillDownObj = this.getDrillDown(this.chartDef.contextMenu[i].reference);
			subMenuItems.push ({ id: this.id + this.chartDef.contextMenu[i].reference, label: drillDownObj.widgetId, icon: "", launchType: "JavaScript", uri: "dijit.byId ('" + this.id +"').drillDown('"+drillDownObj.id+"');", enabled: true });						
		}
		return (subMenuItems);
	
	},

	getDrillUpMenuItems: function() {
		var subMenuItems = [];
		for (var i=0; i<this.drillUpItems.length; i++) {
			var drillUpObj = this.drillUpItems[i];
			subMenuItems.push ({ id: this.id + drillUpObj.chartIdWithCount, label: drillUpObj.chartIdWithCount, icon: "", launchType: "JavaScript", uri: "dijit.byId ('" + this.id +"').drillUp('"+drillUpObj.chartIdWithCount+"');", enabled: true });						
		}		
		return (subMenuItems);

	},

	getDataSet: function (dataSetID) {
	/*	for (var i=0; i < this.widgetDefinition.dataSets.length; i++) {
			if (this.widgetDefinition.dataSets[i].id == dataSetID) {
				return (this.widgetDefinition.dataSets[i]);
			}
		}*/ 
		return (this.widgetDefinition.datasets[0]);
		
//		console.error ("no dataset found for: " + dataSetID + " in widgetDefinition:");		
//		return (null);
	
	},
	getCommonData: function () {

		return (this.chartDef.common);
	},
	makeSeriesData: function (dataColumn) {
		console.log ("enter ChartBuilder.makeSeriesData: " + dataColumn);
		var result = [];
		if(this.rows !== null){
			for (var i=0; i<this.rows.length; i++) {
				if (i !== undefined) {
					var values = this.store.getValues(this.rows[i], dataColumn, true); 
					var value = values[0]; 
                    result.push (dojo.number.parse(value));		
				}
			}
		}

		this.seriesLength = result.length;
		
		console.log ("exit ChartBuilder.makeSeriesData: ");
		console.log (result);
		return (result);
	},
	makeMultiPieSeriesData: function (i, dataColumn, textColumn, tooltipColumn) {
		//   {y: 4, text: "Red",   stroke: "black", tooltip: "Red is 50%"},
		var object = null;
		object = {};
		var values = this.store.getValues(this.rows[i], dataColumn, true); 
		object.y = dojo.number.parse(values[0]);
		//object.y = this.store.getValue (this.rows[i], dataColumn);		
		if (textColumn) {
			//object.text = this.store.getValue (this.rows[i], textColumn);			
			object.text = textColumn;
		}
				
		if (tooltipColumn) {
			values = this.store.getValues(this.rows[i], tooltipColumn, true); 
			object.tooltip = values[0];
			//object.tooltip = this.store.getValue (this.rows[i], tooltipColumn);
		}	
		//return({y: object.y, text: object.text, stroke: "black", tooltip: object.tooltip});
		var color = this.custom.piestroke.color;
		console.log ("makeMultiPieSeriesData color: " + color);
		return({y: object.y, text: object.text, stroke: color, tooltip: object.tooltip});
//		return({y: object.y, stroke: color, tooltip: object.tooltip});
	},
	makeXAxisLabels: function (columnID) {
		console.log ("enter makeXAxisLabels: " + columnID);
		console.log (this.rows);
		var result = [];
		var label = {};
		if(this.rows !== null){
		for (var i=0; i<this.rows.length; i++) {
			if (i !== undefined) {
				label = {};
				label.value = i+1;								
				var values = this.store.getValues (this.rows[i], columnID, false);
				//label.text = this.store.getValue (this.rows[i], columnID);
				label.text = values[0];			
				
				result.push (label);
				}
			}
		}
		
		console.log ("exit makeXAxisLabels: ");
		console.log (result);
		return (result);
	},
	afterFetch: function (columns, rows, store) {
		console.log ("enter ChartBuilder.afterFetch");
		
		var busyPane = dijit.byId(this.global.namespace+"busy_pane");
		if ( busyPane )
		{
			busyPane.hide();
		}
		
		this.columns = columns;
		this.rows = rows;
		this.store = store;
		
		if (this.chart === null) {
			this.buildChart ();
			this.resize(this.initSize);	
		} else {
			this.replaceData (rows);
		}

		this.addDrillButtons();
		if ((this.interval !== null) && (!this.chartDef.common.autoUpdate)) {
           clearTimeout (this.interval);
           this.interval = null;
        }
		
		this.updateMetaData();
		if (this.chartDef.common.showLastUpdateTime) {
			dojo.style (this.metaDiv, "display", "");
		} else {
			dojo.style (this.metaDiv, "display", "none");
		}
		
		this.configAutoUpdate();
		console.log ("exit ChartBuilder.afterFetch");		
	},
	
	addDrillButtons: function(){
		var toolbar = this.getToolbar();
		
		if(this.drillUpItems !== undefined && this.drillUpItems.length > 0) {			
			var drillUpOptions = [];
			drillUpOptions[0] = {
							"id": "Drill Up",
							"label": "Drill Up"
						};
			for (var j=0; j<this.drillUpItems.length; j++) {	
				drillUpOptions[j+1] = {
							"id": this.drillUpItems[j].chartIdWithCount,
							"label": this.drillUpItems[j].chartIdWithCount
						};
			}

			var oldDrillUpComboBox = dijit.byId(this.id + "drillUpCombo");
			if(oldDrillUpComboBox) {
				oldDrillUpComboBox.destroyRecursive();
				dijit.registry.remove (this.id + "drillUpCombo");
			}

			var selectOptions = new dojo.data.ItemFileWriteStore({data: {identifier: 'label', items:drillUpOptions}});			
			var drillUpComboBox = new dijit.form.ComboBox({
				id: this.id + "drillUpCombo",
				name: this.id + "drillUpCombo",
				value: "Drill Up",
				store: selectOptions,
				searchAttr: "label",
				onChange: dojo.hitch (this, this.drillUp)
         	},this.id + "drillUpCombo");
			
			if(toolbar!==undefined && toolbar!==null){
				toolbar.addChild(drillUpComboBox);	
			}
		 	//dojo.connect(drillUpComboBox,'onChange','drillUp'); 
		 	//this.toolbarSelf.addChild(drillUpComboBox);
		
		}
	},
	
	_connectEvents: function(o){
		this._connectSingleEvent(o, "oncontextmenu");		
	},
	
	constructChart: function(){
		this.buildTitle();
		//this.buildToolbar();
		
		console.log ("chart type: " + this.chartDef.type);

		if (this.chartDef.plot === undefined) {
			this.chartDef.plot = {};
		}
		if (this.chartDef.plot.gap === undefined) {
			this.chartDef.plot.gap = 5;
		}
		if (this.chartDef.plot.markers === undefined) {
			this.chartDef.plot.markers = true;
		}
		if (this.chartDef.plot.labels === undefined) {
			this.chartDef.plot.labels = true;
		}
		
		if ((this.chartDef.xAxis.majorTicks === false) &&
			(this.chartDef.yAxis.majorTicks === false)) {
			this.theme.axis.majorTick = "";
		}
	},
	
	buildPieChart: function(){
		console.log ("ChartBuilder.buildChart. type is Pie");
			var divs = [];
			var row2divs = [];
			var row3divs = [];
			var charts = [];
			var label = null;
			var div = dojo.byId (this.id + "chart0");
			
			var table = dojo.doc.createElement("table");
			dojo.place (table, div, 0);
			var tableBody = dojo.doc.createElement("tbody");
			table.appendChild(tableBody);

			//TODO rename var names
			var chartRow = dojo.doc.createElement("tr");
			tableBody.appendChild(chartRow);			
			var labelRow = dojo.doc.createElement("tr");
			tableBody.appendChild(labelRow);
			var row3 = dojo.doc.createElement("tr");
			tableBody.appendChild(row3);
			//table.style.border = "0";
			if(this.rows !== null){
			var labels = this.makeXAxisLabels (this.chartDef.xAxis.dataColumn);
			var pieSize = null;
			if(this.chartDivSize){
				pieSize = {w: this.chartDivSize.w/this.rows.length-10, h: this.chartDivSize.h-50};
			}	
			for (var i=0; i<this.rows.length; i++) {	
				
				var td = dojo.doc.createElement("td"); 
			//	td.style.border = "0";	
				chartRow.appendChild(td);
				divs[i] = dojo.doc.createElement("div");
				divs[i].name = this.id + "chart" + (i+1);
				divs[i].id = this.id + "chart" + (i+1);	
				if(pieSize){
					divs[i].style.width = pieSize.w + 'px';
					divs[i].style.height = pieSize.h + 'px';	
				}else{
					divs[i].style.width = '200px';
					divs[i].style.height = '200px';
				}
				this.pieCMTargetNodeIds.push (divs[i]);
				
				td.appendChild(divs[i]);

				charts[i] = new dojox.charting.Chart2D(divs[i]);
				charts[i].setTheme(this.theme);				
				
				var plotParms = { 
					type: this.chartDef.type,  
					hAxis: this.chartDef.xAxis.id, 
					vAxis: this.chartDef.yAxis.id
				};
				dojo.mixin (plotParms, this.chartDef.plot);
				
				charts[i].addPlot("myChart"+i, plotParms); 			
								
				this.addXAxis (charts[i]);				
				this.addPieSlices (charts[i], i);
				this.addTooltips (charts[i], "myChart"+i);
				
				var anim_a = new dojox.charting.action2d.MoveSlice(charts[i], "myChart"+i);

				this.addEvents (charts[i], "myChart"+i);		
				charts[i].render(); 
				
				var row2td = dojo.doc.createElement("td"); 
				labelRow.appendChild(row2td);
				
				label = labels[i];
				if (label.text !== undefined) {
					dojo.place ("<div class='chartLabels' style='color: " + this.custom.xlabels.color + "'>"+label.text+"</div>", row2td, i);
				} 					
			}			
			}
					
			var temp = 0;
			var serverNames = this.makeXAxisLabels ("0");     //Argument is the Server Name Column id from widget file
			var oldServerName = serverNames[0];
			if(this.rows !== null){
			for (var j=0; j<this.rows.length; j++) {	
				var str = j.toString();
				if(oldServerName.text == serverNames[j].text) {					
					temp = temp + 1;	
				}
				
				if(oldServerName.text != serverNames[j].text || j == (this.rows.length - 1)) {					
					var row3td = dojo.doc.createElement("td"); 			
					row3td.colSpan = temp;
					row3.appendChild(row3td);
					if (oldServerName.text !== undefined) {
						dojo.place ("<div class='chartLabels'>"+oldServerName.text+"</div>", row3td, 0);
					}
						 
					oldServerName = serverNames[j];
					temp = 1;
					}					
				}					
			}			
			this.chart = charts[charts.length-1];
			this.charts = charts;
	},
	
	buildAxisChart: function(){
		var type = this.chartDef.type;
			console.log ("chart type: " + type);
			
			var divChart = dojo.byId (this.id + "chart0");
			if(!this.chartDivSize){
				divChart.style.width = '5000px';
				divChart.style.height = '5000px';
			}
			
			if(this.chartDef.chartProperties && this.chartDef.chartProperties.chartMargins){				
				var chartMargins = this.chartDef.chartProperties.chartMargins;
				for(var i=0;i<chartMargins.length;i++){
					if( dojo.isIE && chartMargins[i].browser=="IE"){
						this.chartDef.chartProperties.margins = chartMargins[i].marginValue;
					} 
					if ( dojo.isFF && chartMargins[i].browser=="FF") {
						this.chartDef.chartProperties.margins = chartMargins[i].marginValue;
					}
				}				
			}
			
			var chart = new dojox.charting.Chart2D(this.id + "chart0", this.chartDef.chartProperties);
			this.chart = chart;
			chart.removePlot("myChart");
			if (type == "dojot.visual.StackedColumnsEmbossed") {
				type = dojot.visual.StackedColumnsEmbossed;
			} else if (type == "dojot.visual.ColumnsEmbossed") {
				type = dojot.visual.ColumnsEmbossed;			
			} else if (type == "dojot.visual.ClusteredColumnsEmbossed") {
				type = dojot.visual.ClusteredColumnsEmbossed;			
			} else if (type == "dojot.visual.BarsEmbossed") {
				type = dojot.visual.BarsEmbossed;
			} else if (type == "dojot.visual.StackedBarsEmbossed") {
				type = dojot.visual.StackedBarsEmbossed;
			} else if (type == "dojot.visual.ClusteredBarsEmbossed") {
				type = dojot.visual.ClusteredBarsEmbossed;
			}

			this.addThresholdsPlot (chart);
			chart.setTheme(this.theme);
			
			console.log ("adding plot of type: ");
			console.log (type);
			
			var plotParms = { 
				type: type,  
				hAxis: this.chartDef.xAxis.id, 
				vAxis: this.chartDef.yAxis.id
			};
			
			dojo.mixin (plotParms, this.chartDef.plot);
			chart.addPlot("myChart", plotParms); 
			
			var plotObj = chart.stack[chart.plots.myChart];
			dojo.connect (plotObj, "_connectEvents", plotObj, this._connectEvents);
			this.addAxisLabel();
			this.addGrid (chart);    			
			

	},
	
	feedAxisChartData: function(){
			var chart = this.chart;
			this.addYAxis (chart);		
			this.addXAxis (chart);			
			
			console.log ("adding animations");
			var anim_1 = new dojox.charting.action2d.Shake(chart, "myChart");
			var anim_2 = new dojox.charting.action2d.Magnify(chart, "myChart", {scale: 1.5});
			this.addTooltips (chart, "myChart");
			this.addEvents (chart, "myChart");				
			this.addThresholdsData (chart);			
			this.chart.render();
	 		// this call to render is so that after we add the legend, it will show the legend colors.
 			// If we add the legend before we render then the legend will not show the colors.
 			// It will show an 'X' instead.	
	},
	
	buildChart: function () {
		console.log ("enter ChartBuilder.buildChart");
		var start = new Date();
		
		try {
		this.constructChart();
				
		if(this.chartDef.type == "Pie") {
			this.buildPieChart();	
		} else {	
			this.buildAxisChart();	
			this.feedAxisChartData();
		}
 		
//		if (this.chartDef.xAxis.label.value) {
//			this.setXAxisTitle(this.chart,this.chartDef.xAxis.id,this.chartDef.xAxis.label.value);
//		}
 		
 		
		if ((this.chartDef.legend.hidden !== undefined) && (this.chartDef.legend.hidden === false)) {
	 		console.log ("calling addLegend");
			this.addLegend (this.chart);

			// If we have a legend, then we must render once more (seems to be a dojo bug!)
			// If we render only after adding legend, then the legend has no colors for its 
			// labels.
			console.log ("calling chart.resize to render the chart after adding legend");			
		}
		
		// make sure we have called resize at least once
		// if we have already called it with this size, then this method will
		// return immediately.
		this.resize (this.initSize);
		
	 	var renderTime  = new Date() - start;
 		console.log ("renderTime: " + renderTime);

		this.configAutoUpdate();
 		
 		dojo.publish ("ChartBuilder.chartBuilt" + this.id, []);
 		
 		if(!this.standalone || (this.standalone && !this.ignoreContextMenu)){
			this.makeChartContextMenu();			
		}
 		
		console.log ("exit ChartBuilder.buildChart");
		} catch (e) {
			console.warn (e);	
			//alert (e);
			//alert(dojo.toJson(e));		
		}				
		
 		
 		return (this.chart); 
	},
	addThresholdsPlot: function (chart) {
		if ((this.dataset.thresholds !== undefined) &&
			(this.dataset.thresholds.length > 0)) {
			var plotParms = { 
				type: "Lines", 
				hAxis: this.chartDef.xAxis.id, 
				vAxis: this.chartDef.yAxis.id
//				hAxis: "hThresholdAxis", 
//				vAxis: "vThresholdAxis"
			};
			
			chart.addPlot("thresholds", plotParms);

			// update theme color
			if ((this.theme.colors) && (this.theme.colors != null)) {
				this.addThresholdColorsToTheme (this.dataset.thresholds);
			} else {
				this.addThresholdGradientsToTheme (this.dataset.thresholds);
			}
		}
	},
	addThresholdsData: function (chart) {
		console.log ("enter addThresholds");
		if ((this.dataset.thresholds !== undefined) &&
			(this.dataset.thresholds.length > 0)) {
			for (var i=this.dataset.thresholds.length - 1; i>=0; i--) {
				// do not understand why but for the colors to map right, we have to add the thresholds backwards
				console.log ("adding threshold: " + this.dataset.thresholds[i].label.value + ", " + this.dataset.thresholds[i].start);
				var seriesData = this.makeThresholdSeriesData(this.dataset.thresholds[i]);
				chart.addSeries(this.dataset.thresholds[i].label.value, seriesData, { plot: "thresholds" });
			}
		} else {
			console.log ("no thresholds to add");
		}
				
		console.log ("exit addThresholds");
	},
	addThresholdGradientsToTheme: function (thresholds) {
		this.resizeGradientArray();
		
		for (var i=0; i<thresholds.length; i++) {
			if (thresholds[i].color !== undefined) {
				this.theme.seriesThemes[this.chartDef.yAxis.series.length + i] = {fill: dojox.charting.Theme.generateGradient(this.theme.defaultFill, thresholds[i].color, thresholds[i].color)};
			}
		}
	},
	addThresholdColorsToTheme: function (thresholds) {
		console.log ("enter addThresholdColorToTheme");
		
		this.resizeColorArray();
		
		for (var i=0; i<thresholds.length; i++) {
			if (thresholds[i].color !== undefined) {
				this.theme.colors[this.chartDef.yAxis.series.length + i] = thresholds[i].color;
			}
		}
		
		console.log ("colors");
		console.log (this.theme.colors);
	},
	makeThresholdSeriesData: function (threshold) {
		var seriesData = [];
		// this.seriesLength is set by the addSeriesData
		// We will make the same number of data points as 
		// the y-axis series has, but a minimum of 2 so it 
		// will draw a line (a line needs 2 points!)
		for (var i=0; i<this.seriesLength || i<2; i++) {
			if(this.chartDef.type.indexOf("Bar")!=-1){
				seriesData.push ({
					x: Number(threshold.start),
					y: i
				});		
			}else{
				seriesData.push (Number(threshold.start));
			}
		}
		
		console.log ("seriesData: ");
		console.log (seriesData);

		return (seriesData);
	},
	resizeGradientArray: function () {
		console.log ("enter resizeGradientArray");
		var maxSize = this.chartDef.yAxis.series.length + this.dataset.thresholds.length;
		
		if (maxSize > this.theme.seriesThemes.length) {
			var seriesThemes = [];
			for (var i=0; i<maxSize; i++) {
				seriesThemes.push (this.theme.seriesThemes[i % this.theme.seriesThemes.length]);
			}
			
			this.theme.seriesThemes = seriesThemes;
		}
		
		console.log ("exit resizeGradientArray");
		console.log (this.theme.seriesThemes);
	},
	resizeColorArray: function () {
		console.log ("enter resizeColorArray");
		var maxSize = this.chartDef.yAxis.series.length + this.dataset.thresholds.length;
		
		if (maxSize > this.theme.colors.length) {
			var colors = [];
			for (var i=0; i<maxSize; i++) {
				colors.push (this.theme.colors[i % this.theme.colors.length]);
			}
			
			this.theme.colors = colors;
		}
		
		console.log ("exit resizeColorArray");
		console.log (this.theme.colors);
	},
	replaceData: function (rows) {
		console.log ("enter ChartBuilder.replaceData");
		this.rows = rows;
		this.chart.removeAxis (this.chartDef.xAxis.id);
		this.chart.removeAxis (this.chartDef.yAxis.id);
		this.addYAxis (this.chart);
		this.addXAxis (this.chart);
		this.addAxisLabel();
		if(this.chartDef.type == "Pie") {
			for(var i=0; i<this.rows.length; i++) {	
				this.addPieSlices (this.chart, i);
			}
		}
		this.chart.render(); 

//		if (this.chartDef.xAxis.label.value) {
//			this.setXAxisTitle(this.chart,this.chartDef.xAxis.id,this.chartDef.xAxis.label.value);	
//		}

		console.log ("exit ChartBuilder.replaceData");
	},
	configAutoUpdate: function () {

		if ((this.interval === null) && (this.updateMultiplier > 0) &&
			this.chartDef.common.autoUpdate) {
			var fnc = dojo.hitch (this, this.updateChartData);
		    var nextUpdate = this.getUpdateInterval();
		    this.interval = setTimeout(fnc, nextUpdate);
		} else {
            if ((this.interval !== null) && (!this.chartDef.common.autoUpdate)) {
                clearTimeout (this.interval);
                this.interval = null;
            }
	}
	},
    getUpdateInterval: function() {

      var nextUpdateInterval = this.runReportResponseTime * this.updateMultiplier;
      
      if ( nextUpdateInterval > this.updateMaxInterval ) {
        nextUpdateInterval = this.updateMaxInterval;
      } else if ( nextUpdateInterval < this.updateMinInterval ) {
        nextUpdateInterval = this.updateMinInterval;
      }
      return nextUpdateInterval;
    },
    addLegend: function (chart) {
		var legend = null;	
		
		var orientation = (this.chartDef.legend.orientation === "horizontal") ? true : false;
		try
		{			
		if ((this.chartDef.legend.hidden !== undefined) && (this.chartDef.legend.hidden === false)) {	
				
			var oldLegendNode = dijit.byId (this.id + "legend");		
			if(oldLegendNode) {
				oldLegendNode.destroy();
			}

			var legendDomNode = dojo.doc.createElement("div");
			legendDomNode.id = this.id + "legend";
			if(this.chartDef.legend.location === "right") {						
				orientation = false;
				dojo.place (legendDomNode, dojo.byId (this.id + "rightLegend"), 0);
					
			} else {						
				orientation = true;
				dojo.place (legendDomNode, dojo.byId (this.id + "bottomLegend"), 0);
			}					
			REVERSED_SERIES = /\.(StackedColumns|StackedAreas|ClusteredBars|ClusteredBarsEmbossed|StackedColumnsEmbossed)$/;	
			legend = new dojox.charting.widget.Legend({chart: chart, horizontal: orientation}, this.id + "legend");
		}
		}
		catch (e)
		{		
			console.error (e.message);
			
		}		
		this.legend = legend;
		return (chart);
	},
	hideLegend: function () {

		var rightLegend = dojo.byId(this.id + "rightLegend");
		var bottomLegend = dojo.byId(this.id + "bottomLegend");		

		dojo.style(rightLegend, 'display', 'none');		
		dojo.style(bottomLegend, 'display', 'none');	
	},
	updateLegend: function (chart) {

		var legend = this.legend;
		if(legend === null)
		{
			this.addLegend (this.chart);
			return;
		}
		if(this.chartDef.legend.location == "right") {				
			
			var rightLegend = dojo.byId (this.id + "rightLegend");			
			dojo.removeClass(dojo.byId(this.id + "legend"), "dojoxLegendHorizontal");

			//TODO how to set the value of the "Horizontal" attribute ?
			//legend.attr("horizontal", false);	
			dojo.place (dojo.byId(this.id + "legend"), rightLegend, 0);
			legend.refresh();			
			dojo.style(rightLegend, 'display', 'block');
		}else {
			
			var bottomLegend = dojo.byId (this.id + "bottomLegend");			
			dojo.place (dojo.byId(this.id + "legend"), bottomLegend, 0);
			//legend.attr("horizontal", false);
			legend.refresh();
			dojo.style(bottomLegend, 'display', 'block');				
			}
		this.legend = legend;
	},
	addEvents: function (chart, plotName) {
		console.log ("enter addEvents");
		chart.connectToPlot(
    		plotName,    // the unique plot name you specified when creating a plot
    		this,      // both object and method are the same used by dojo.connect()
    		this.onChartEvent       // you can supply a function without an object
		);
		
		console.log ("exit addEvents");
		return (chart);
	},
	addTooltips: function (chart, plotName) {
		console.log ("enter addTooltips");
		if ((this.chartDef.showTooltips) || (this.chartDef.showTooltips == "true")) {
			console.log ("adding tooltips");
			var anim_tooltip = new this.dc.action2d.Tooltip(chart, plotName);			
		} else {
			console.log ("not adding tooltips");
		}
		
		console.log ("exit addTooltips");
		return (chart);
	},
	addGrid: function (chart) {		
		console.log ("enter addGrid");	
 		if (this.chartDef.showGrid === true) {
 			console.log ("showGrid is true.  Adding grid");					
 			chart.addPlot("Grid", {type: "Grid",
		      	hAxis: this.chartDef.xAxis.id,
      			vAxis: this.chartDef.yAxis.id,
      			hMajorLines: true,
				hMinorLines: true,
      			vMajorLines: true,
      			vMinorLines: true
			});
		} else {
			console.log ("not adding grid");
		}
		
		return (chart);
	},
	
	getTextBoxCoords: function(text, style){		
		var node =  dojo.doc.createElement("div");
		var nodeStyle = {
			position: "absolute",
			top: "-10000px",
			left: "0",
			borderWidth: "0",
			margin: "0",
			padding: "0",
			outline: "0"
		};
		dojo.mixin(nodeStyle, style);		
		dojo.style (node, nodeStyle);
		dojo.body().appendChild(node);		
		node.innerHTML = text;
		var coords = dojo.marginBox(node);
		dojo.destroy(node);		
		return coords;		
	},
	
	getLabelCoords: function(labels, font, isHorizontal){
		var text = null;
		if(isHorizontal){
			text = labels.join();
		}else{
			text = labels.join("<br>");
		}
		return this.getTextBoxCoords(text, {font: font,whiteSpace:"nowrap"});	
	},
	
	truncateLabels: function(labels, font, width){		
		var labelWidth = this.getLabelCoords(labels,font,true).w;
		var rate = width/labelWidth;		
		if(labelWidth<=width){
			return labels;
		}else{
			var truncateLabels = [];
			for(var i=0; i<labels.length; i++){
				var chartCount = labels[i].length;				
				truncateLabels[i] = labels[i].substring(0,Math.floor(rate*chartCount));			
			}	
			return truncateLabels;
		}		
	},
	
	getMajorTickStep: function(){		
		//var font = "normal normal normal 7pt Tahoma";
		var majorTickStep = null;
		var font = this.chart.theme.axis.tick.font;		
		var labels = dojox.lang.functional.map(this.labels, function(label){ return label.text; });			
		if(this.chartDef.type == "Lines" || this.chartDef.type.indexOf("Columns")!=-1){
			var labelTotalLength = this.getLabelCoords(labels,font,true).w;
			if(labelTotalLength/(this.chartDivSize.w-200) > this.labels.length){ //200 = this.chart.offsets.l + this.chart.offsets.r
				majorTickStep = this.labels.length-1;
			}else{
				majorTickStep = Math.ceil(labelTotalLength/(this.chartDivSize.w-200));
			}			
		}else{
			var labelTotalHeight = this.getLabelCoords(labels,font,false).h;
			if(labelTotalHeight/(this.chartDivSize.h-100) > this.labels.length){ //100 = this.chart.offsets.t + this.chart.offsets.b
				majorTickStep = this.labels.length-1;
			}else{
				majorTickStep = Math.ceil(labelTotalHeight/(this.chartDivSize.h-100));
			}			
		}
		return majorTickStep;
	},
	
	
	addXAxis: function (chart) {
		console.log ("enter ChartBuilder.addXAxis");
		var labels = this.makeXAxisLabels (this.chartDef.xAxis.dataColumn);
		this.labels = labels;
		console.log ("labels: " + dojo.toJson (labels));
		if (this.chartDef.xAxis.minorLabels === undefined) {
			this.chartDef.xAxis.minorLabels = false;
		}
		if (this.chartDef.xAxis.majorLabels === undefined) {
			this.chartDef.xAxis.majorLabels = true;
		}
		
		if ((this.chartDef.xAxis.minorTicks === undefined) || (this.chartDef.xAxis.minorTicks === false)) {
			this.chartDef.xAxis.minorTicks = false;
		} 
		
		if (this.chartDef.xAxis.majorTicks === undefined) {
			this.chartDef.xAxis.majorTicks = true;
		} 
		var majorTickStep = null;
		if(this.chartDivSize){
			if((!this.chartDef.xAxis.rotation || this.chartDef.xAxis.rotation==0) && this.chartDef.type != "Pie"){
				if(!this.chartDef.xAxis.majorTickStep){					
					majorTickStep = this.getMajorTickStep();				
				}else if(this.chartDef.type == "Lines" || this.chartDef.type.indexOf("Columns")!=-1){
					var labels2 = dojox.lang.functional.map(this.labels, function(label){ return label.text; });	
					var labels3 = this.truncateLabels(labels2, this.chart.theme.axis.tick.font, this.chartDivSize.w-200);				
					for(var i=0;i<labels.length;i++){
						labels[i].text = labels3[i];
					}
				}
			}
		}
		
		console.log ("majorTicks: " + this.chartDef.xAxis.majorTicks);
		console.log ("minorTicks: " + this.chartDef.xAxis.minorTicks);
		
		console.log ("xAxis.majorLabels: " + this.chartDef.xAxis.majorLabels);
		console.log ("xAxis.minorLabels: " + this.chartDef.xAxis.minorLabels);
		console.log ("before addYAxis color");
		var color = this.custom.xlabels.color;
		console.log ("addYAxis color: " + color);
		var axisParms = { 
			fontColor: color,
			majorTickStep: majorTickStep 
		};
		
		if ((this.chartDef.type == "StackedBars") || (this.chartDef.type == "Bars") || (this.chartDef.type == "ClusteredBars") ||
			(this.chartDef.type == "dojot.visual.StackedBarsEmbossed") || (this.chartDef.type == "dojot.visual.BarsEmbossed") || (this.chartDef.type == "dojot.visual.ClusteredBarsEmbossed")) {
			axisParms.vertical = true;
			axisParms.labels = labels;
			dojo.mixin (axisParms, this.chartDef.xAxis);
			chart.addAxis(this.chartDef.xAxis.id, axisParms);
		} else if ((this.chartDef.type == "Columns") || (this.chartDef.type == "StackedColumns") || (this.chartDef.type == "ClusteredColumns")) {
			axisParms.vertical = false;
			axisParms.labels = labels;
			dojo.mixin (axisParms, this.chartDef.xAxis);
			chart.addAxis(this.chartDef.xAxis.id, axisParms);
		} else if (this.chartDef.type == "Lines") {	
			axisParms.labels = labels;
			dojo.mixin (axisParms, this.chartDef.xAxis);
			chart.addAxis(this.chartDef.xAxis.id, axisParms);
		} else if ((this.chartDef.type == "dojot.visual.StackedColumnsEmbossed") || (this.chartDef.type == "dojot.visual.ColumnsEmbossed") ||
				   (this.chartDef.type == "dojot.visual.ClusteredColumnsEmbossed") || (this.chartDef.type == "Columns")) {
			console.log ("add x axis for stacked columns embossed");						
			axisParms.labels = labels;
			dojo.mixin (axisParms, this.chartDef.xAxis);
			chart.addAxis(this.chartDef.xAxis.id, axisParms);
		} else if (this.chartDef.type == "Pie") {
			axisParms.labels = labels;
			dojo.mixin (axisParms, this.chartDef.xAxis);
			chart.addAxis(this.chartDef.xAxis.id, axisParms);
		} else {
			console.error ("Unsupported chart type: " + this.chartDef.type);
		}
		
		console.log ("exit ChartBuilder.addXAxis");
		return (chart);
	},
	addPieSlices: function (chart, rowNumber, update) {
		var isUpdate = (update === undefined) ? false : update;
		if (!isUpdate) {
			chart.addAxis(this.chartDef.yAxis.id, {vertical: true});
		}
		var series = null;		
		var resultPie = [];

		for (var i in this.chartDef.yAxis.series) {
			if (i) {
				series = this.chartDef.yAxis.series[i];
				//resultPie[i] = this.makeMultiPieSeriesData (rowNumber, series.dataColumn, series.textColumn, series.tooltipColumn);
				resultPie[i] = this.makeMultiPieSeriesData (rowNumber, series.dataColumn, series.label.value, series.tooltipColumn);
			
			if (isUpdate) {
			    	chart.updateSeries (series.label.value, resultPie[i]);
			    } else {
					chart.addSeries(series.label.value, resultPie, { plot: "myChart"+rowNumber });
				}
			}
		}
		return (chart);	
	},
	addYAxis: function (chart, update, pieFlag) {
		console.log ("enter addYAxis (..., update, pieFlag): " + update + ", " + pieFlag);
		// e.g.,
		// chart1.addAxis("other y", {vertical: true,
      	//	leftBottom: false,
      	//	max: 7,
      	//	stroke: "green",
      	//	font: "normal normal bold 14pt Tahoma",
      	//	fontColor: "red",
      	//	majorTick: {color: "red", length: 6},
		//    minorTick: {stroke: "black", length: 3}
		// });
		console.log ("before addYAxis color");
		var color = this.custom.ylabels.color;
		console.log ("addYAxis color: " + color);
		
		var isUpdate = (update === undefined) ? false : update;
		
		if (!isUpdate) {
			var vertical = true;
			if ((this.chartDef.type == "StackedBars") || (this.chartDef.type == "Bars") || (this.chartDef.type == "ClusteredBars") ||
				(this.chartDef.type == "dojot.visual.StackedBarsEmbossed") || (this.chartDef.type == "dojot.visual.BarsEmbossed") || (this.chartDef.type == "dojot.visual.ClusteredBarsEmbossed")) {	
				vertical = false;			
			}
				    	
//			chart.addAxis(this.chartDef.yAxis.id, {min: 0, vertical: true, fontColor: color});
			var axisParms = { 
				vertical: vertical,
				fontColor: color
			};
			
			if (this.chartDef.yAxis.minorLabels === undefined) {
				this.chartDef.yAxis.minorLabels = false;
			}
			if (this.chartDef.yAxis.majorLabels === undefined) {
				this.chartDef.yAxis.majorLabels = true;
			}
			if (this.chartDef.yAxis.minorTicks === undefined) {
				this.chartDef.yAxis.minorTicks = false;
			} 
			if (this.chartDef.yAxis.majorTicks === undefined) {
				this.chartDef.yAxis.majorTicks = true;
			} 
		
			console.log ("majorTicks: " + this.chartDef.yAxis.majorTicks);
			console.log ("minorTicks: " + this.chartDef.yAxis.minorTicks);
		
			console.log ("xAxis.majorLabels: " + this.chartDef.yAxis.majorLabels);
			console.log ("xAxis.minorLabels: " + this.chartDef.yAxis.minorLabels);

			dojo.mixin (axisParms, this.chartDef.yAxis);
			
			chart.addAxis(this.chartDef.yAxis.id, axisParms);
	    }
	     
		var series = null;
		var seriesData = null;
		var resultBars = null;
		
		for (var i in this.chartDef.yAxis.series) {
			if (i) {
				series = this.chartDef.yAxis.series[i];				
				seriesData = this.makeSeriesData (series.dataColumn);	

				if (isUpdate) {
					console.log ("ChartBuiler.addYAxis.  calling updateSeries with seriesData: ");
					console.log (seriesData);
			    	chart.updateSeries (series.label.value, seriesData);
			    } else {					 											
					console.log ("ChartBuiler.addYAxis.  calling addSeries with seriesData: ");
					console.log (dojo.toJson(seriesData));
					chart.addSeries(series.label.value, seriesData, { plot: "myChart" });
				}
			}
		}
		
		console.log ("exit addYAxis");
		return (chart);
	},
	addAxisLabel: function(){
		var vertical = true;
		if ((this.chartDef.type == "StackedBars") || (this.chartDef.type == "Bars") || (this.chartDef.type == "ClusteredBars") ||
				(this.chartDef.type == "dojot.visual.StackedBarsEmbossed") || (this.chartDef.type == "dojot.visual.BarsEmbossed") || (this.chartDef.type == "dojot.visual.ClusteredBarsEmbossed")) {	
				vertical = false;			
		}
		if (this.chartDef.type !== "Pie"){
			var node_y = dojo.byId(this.id + "axistitle");
			var node_x = dojo.byId(this.id + "axistitle_X");
			if (node_y && this.chartDef.yAxis.label && this.chartDef.xAxis.label) {
				if(vertical == true){
					node_y.innerHTML = this.chartDef.yAxis.label.value;
					dojo.addClass(node_y, this.chartDef.yAxis.label.cssClass);
					node_x.innerHTML = this.chartDef.xAxis.label.value;
					dojo.addClass(node_x, this.chartDef.xAxis.label.cssClass);
				}else{
					node_y.innerHTML = this.chartDef.xAxis.label.value;
					dojo.addClass(node_y, this.chartDef.xAxis.label.cssClass);
					node_x.innerHTML = this.chartDef.yAxis.label.value;
					dojo.addClass(node_x, this.chartDef.yAxis.label.cssClass);
				}
			} else {
				console.log ("cannot find node_y for "+this.id + "axistitle");
			}
		}
	},
	onChartEvent: function (event) {	
		if(event.type == "oncontextmenu"){
			/*
			var menuEvent = {};
			menuEvent.coords = {};
			menuEvent.target = event.event.target ? event.event.target : event.event.srcElement;
			menuEvent.coords.x = event.event.clientX;
			menuEvent.coords.y = event.event.clientY;
			*/	
			this.chartMenu.contextMenuOpen(event.event, this.getSelectedData(event));
		}
		if (event.type == "onclick") {
		//	if (event.event.detail == 2) {
		//		clearTimeout(this.timeVar);
				this.getTrigger (event.hAxis, event.vAxis, event.type);				
					if (this.trigger.type == "DrillDown") {
						var builder = this.drillDown(this.trigger.reference);
						this.objReassign.call(builder);	

					} else if (this.trigger.type == "PORTAL_PAGE") {
						this.launchPage(this.trigger);

					} else if (this.trigger.type == "PORTLET") {
						this.launchPortlet(this.trigger);

					} else if (this.trigger.type == "WEB_URL") {		
						this.launchURL (this.trigger);

					} else if (this.trigger.type == "CLIENT_EVENT") {					
						this.sendEvent(this.trigger);

					} else if (this.trigger.type == "javascript") {
						this.handleJavaScriptTrigger (this.trigger, event);
					} else {
						console.error ("unknown Trigger type: " + this.trigger.type);
					}
				
			/*}
			else {
				this.timeVar = setTimeout(this.getTrigger (event.hAxis, event.vAxis, event.type), 250);				
				if (this.trigger !== null) {
					if (this.trigger.type == "javascript") {
						this.handleJavaScriptTrigger (this.trigger, event);
					} else {
						console.error ("unknown Trigger type: " + this.trigger.type);
				}
			}
			}*/
		}
	},
	launchURL: function(trigger) 
    {
      console.log ("enter launchURL");
      window.open (trigger.url);
    },	
	launchPortlet: function (trigger) 
   {		
      console.log ("enter launchPortlet");
      var portletEvent = {
                            name: "http://ibm.com/isclite#launchPortlet",
                            portletName: trigger.portletName,
                            portletApplicationID: trigger.portletApplicationId
                         };

      if (trigger.portletParameters) 
      {
         for (var j=0; j<trigger.portletParameters.length; j++) 
         {
			 try{
				 eval ("portletEvent." + trigger.portletParameters[j].id + "= '" + trigger.portletParameters[j].value + "'");
             }catch(e){
				 //tipChartsAll.js has variable names in a different format, so, eval will not work
             	portletEvent[trigger.portletParameters[j].id] = trigger.portletParameters[j].value;
             }
         }
      }

      try 
      {
         EventBroker.sendEvent (this.global.iscPortletNamespace, portletEvent);
      } catch (e) {
         console.error ("Caught javascript exception in ChartBuilder.launchPortlet: " + e);
         alert (e);
      }
   },
   
   launchPage: function (trigger) 
   {	  
      var pageEvent = {
                         name: "http://ibm.com/isclite#launchPage",
                         NavigationNode: trigger.pageId,
                         pageInstanceRef: null,
                         switchPage: true
                      };

      if (trigger.pageParameters) 
      {
         for (var k=0; k<trigger.pageParameters.length; k++) 
         {
			 try{
				 eval ("pageEvent." + trigger.pageParameters[k].id + "= '" + trigger.pageParameters[k].value + "'");
             }catch(e){
				  //tipChartsAll.js has variable names in a different format, so, eval will not work
             	pageEvent[trigger.pageParameters[k].id] = trigger.pageParameters[k].value;//shakeel
             }
         }
      }

      try 
      {
         EventBroker.sendEvent (this.global.iscPortletNamespace, pageEvent);
      } catch (ex) {
         console.error ("Caught javascript exception in ChartBuilder.launchPage: " + ex);
         alert (ex);
      }
   },
   
   sendEvent: function (trigger) 
   {
      var parms = trigger.payload;
      var event = {
                     name: trigger.eventName
                  };

      if (trigger.payload) 
      {
         for (var i=0; i<parms.length; i++) 
         {
            try{
                eval ("event." + parms[i].id + "= '" + parms[i].value + "'");
             }catch(e){
             	event[parms[i].id] = parms[i].value;
             }
         }
      }

	  console.log ("ChartBuilder: sending event: ");            
	  console.log (event);
      EventBroker.sendEvent (this.global.iscPortletNamespace, event);
    },
	
	getDrillDown: function (drillDownId) {		
		for (var i=0; i<this.widgetDefinition.drillDown.length; i++) {
				if (this.widgetDefinition.drillDown[i].id == drillDownId) {					
					return (this.widgetDefinition.drillDown[i]);
				}
		}		
		return (null);
	},

	getDrillUp: function (chartIdWithCount) {		
		for (var i=0; i<this.drillUpItems.length; i++) {				
				if(this.drillUpItems[i].chartIdWithCount == chartIdWithCount) {						
					return (this.drillUpItems[i]);
				}
		}		
		return (null);
	},

	objReassign: function() {
		console.log("builder");
		console.log(this);
	},	

	drillUp: function(chartIdWithCount) {		
		var drillUpObj = this.getDrillUp(chartIdWithCount);
		
		if(drillUpObj !== null) {
			
			var encodedChartId = encodeURIComponent(drillUpObj.chartId);
			var encodedProviderId = encodeURIComponent(drillUpObj.providerId);		
			var encodedDatasourceId = encodeURIComponent(drillUpObj.datasourceId);
			var encodedVersion = encodeURIComponent(drillUpObj.version);
			var fetcher = this.global.widgetFetcher;
			var domNode = dojo.byId (this.global.namespace+"widget");
			this.destroy();
			dojo.empty(domNode);
			
			var newDrillUpArray = [];
			
			for (var i=0; i<this.drillUpItems.length; i++) {	
				
				if(this.drillUpItems[i].chartIdWithCount == drillUpObj.chartIdWithCount) {	
					break;
				}
				newDrillUpArray[i] = this.drillUpItems[i];
			}				
						
			dojo.place (domNode, dojo.byId (this.global.namespace + "birtChartWidget"), "after");
			var builder = fetcher.buildWidget(encodedProviderId, encodedDatasourceId, encodedChartId, domNode, encodedVersion, null, newDrillUpArray);
			
			this.global.builder = builder;
			return builder;
		}
		return this;

	},
	
	drillDown: function(drillDownId) {

		var drillDownObj = this.getDrillDown(drillDownId);
		if(drillDownObj !== null) {

			var encodedCID = encodeURIComponent(drillDownObj.widgetId);
			var encodedProviderID = encodeURIComponent(drillDownObj.provider);	
					
			var encodedDS = null;
			if(drillDownObj.connectionID != undefined)
			{
				encodedDS = encodeURIComponent(drillDownObj.connectionID);			
			}
			else 
			{
				if(drillDownObj.datasource == this.widgetDefinition.datasource.type)
				{
					encodedDS = encodeURIComponent(this.datasourceID);
				}
			}
			
			var fetcher = this.global.widgetFetcher;
			var domNode = dojo.byId (this.global.namespace+"widget");
			this.destroy();
			dojo.empty(domNode);			
			var drillUpLength = this.drillUpItems.length;
			var idCount = 1;
			for (var i=0; i<this.drillUpItems.length; i++) {				
				if(this.drillUpItems[i].chartId == this.chartDef.id) {
					idCount = idCount + 1;
				}
			}
				
			this.drillUpItems[drillUpLength] = {
				"chartId": this.chartDef.id,
				"providerId": this.widgetDefinition.provider.label.key,
				"datasourceId": this.datasourceID,
				"version": this.widgetDefinition.meta.version,
				"idCount": idCount,
				"chartIdWithCount": this.chartDef.id + "(" + idCount + ")"
			};			
			dojo.place (domNode, dojo.byId (this.global.namespace + "birtChartWidget"), "after");
			var builder = fetcher.buildWidget(encodedProviderID, encodedDS, encodedCID, domNode, encodeURIComponent(drillDownObj.version), null, this.drillUpItems);
			
			this.global.builder = builder;
			return builder;
		}
		return this;
	},
	// TODO: Using a closure, add context to calling of trigger.  
	// (i.e., pass event to trigger function).
	handleJavaScriptTrigger: function (trigger, event) {
		trigger.script = this.substituteTriggerKeywords (trigger.script);
		eval (trigger.script);
	},
	substituteTriggerKeywords: function (map) {
		console.log ("trigger before substitution: " + map);
		map = map.replace (/<iscPortletAPI:namespace\/>/g, this.global.iscPortletNamespace);
		map = map.replace (/<portlet:namespace\/>/g, this.namespace);
		map = map.replace (/<script>/gi," <br> <script defer>");
		console.log ("trigger after substitution: " + map);
		
		return (map);
	},
	getTrigger: function (xAxis, yAxis, triggerName) {
		
		//if (this.chartDef.yAxis.id == yAxis) {		
			for (var i=0; i<this.chartDef.yAxis.triggers.length; i++) {
				if (this.chartDef.yAxis.triggers[i].id == triggerName) {
					this.trigger = this.chartDef.yAxis.triggers[i];					
					return (this.chartDef.yAxis.triggers[i]);
				}
			}
		//}
		
		return (null);
	},
	updateMetaData: function () {
		var lastUpdated = new Date();
		var msg = dojo.string.substitute(this.resources_.LAST_UPDATED, { 0: lastUpdated.toLocaleString() });
		this.metaDiv.innerHTML = "<p class=\"lastUpdated\">" + msg + "</p>";
	},
	updateChartDef: function () {
		this.chartDef.description.value = this.widgetDefinition.user.charts[0].description.value;
		this.chartDef.label.value = this.widgetDefinition.user.charts[0].label.value;
		this.chartDef.common = this.widgetDefinition.user.charts[0].common;
		this.chartDef.legend.hidden = this.widgetDefinition.user.charts[0].legend.hidden;
		this.chartDef.legend.location = this.widgetDefinition.user.charts[0].legend.location;	
	},
	update: function (prefs, newWidgetDef) {		
		this.widgetDefinition = newWidgetDef;
		this.updateChartDef();
		if(prefs.charts[0].type) {			
			if(this.chartDef.type != prefs.charts[0].type) {				
				this.chartDef.type = prefs.charts[0].type;				
				this.chart = null;
				var div = dojo.byId (this.id + "chart0");
				console.log(div);
				dojo.html._emptyNode(div);
				this.buildChart();
			}
		}		
		
					
		this.setTitle (this.chartDef.label.value);	
		
		if(this.chartDef.legend.hidden === false) {
			this.updateLegend (this.chart);		
		} else if(this.chartDef.legend.hidden === true) {
			this.hideLegend();
		}

		this.updateMetaData();
		if (prefs.charts[0].common.showLastUpdateTime) {
			dojo.style (this.metaDiv, "display", "");
		} else {
			dojo.style (this.metaDiv, "display", "none");
		}

		if(this.widgetDefinition.user.datasets[0].parameters.length > 0){
			this.parameters = this.widgetDefinition.user.datasets[0].parameters;
			this.updateChartData();
		}

		this.dataset = this.getDataSet (this.dataSetID);	
		this.configAutoUpdate();
			
	},
	buildTitle: function () {
		console.log ("enter ChartBuilder.buildTitle");
		if (this.standalone && !this.ignoreTitlebar) {
			console.log ("building titlebar");
			var titleBar = new ibm.tivoli.tip.charts.dijit.ChartTitle ({
					title: this.chartDef.label.value,
					chartid: this.id,
					namespace: this.namespace
				}, dojo.byId (this.id + "title"));
		} else {
			console.log ("not building titlebar");
			if ((this.widgetDefinition.user !== undefined) && (this.widgetDefinition.user.charts !== undefined)) {			
				if (this.widgetDefinition.charts[0].id == this.widgetDefinition.user.charts[0].id) {
				this.chartDef.label.value = this.widgetDefinition.user.charts[0].label.value;
				this.chartDef.description.value = this.widgetDefinition.user.charts[0].description.value;
			}		
			}else {
				if ((this.widgetDefinition.shared !== undefined) && (this.widgetDefinition.shared.charts !== undefined)) {			
				if (this.widgetDefinition.charts[0].id == this.widgetDefinition.shared.charts[0].id) {
				this.chartDef.label.value = this.widgetDefinition.shared.charts[0].label.value;
				this.chartDef.description.value = this.widgetDefinition.shared.charts[0].description.value;
				}
			}			
			}		
			this.setTitle (this.chartDef.label.value);
		}
		console.log ("exit ChartBuilder.buildTitle");
	},
	setTitle: function (title) {
		console.log ("enter ChartBuilder.setTitle");
		if ((this.global.iscPortletNamespace === undefined) || (this.global.iscPortletNamespace === null)) {
			//var titleBar = dijit.byId (this.id + "title");
		} else {			
			var tipHelper = new ibm.tivoli.tip.dojo.TIPHelper();
			tipHelper.setPortletTitle (this.domNode, title);
		}
		console.log ("exit ChartBuilder.setTitle");
	},
	buildToolbar: function () {
		console.log ("enter ChartBuilder.buildToolbar");
		if(this.toolbarSelf === null && (!this.standalone || (this.standalone && !this.ignoreToolbar))) {
			console.log ("creating chartToolbar. global: " + this.global);
			var toolbar = new ibm.tivoli.tip.charts.dijit.ChartToolbar ({
				chartFetcher: this.chartFetcher,
				toolbarHidden_: this.chartDef.toolbar.hidden,
				chartBuilder: this,
				global: this.global
			}, dojo.byId (this.id + "toolbar"));
			this.toolbarSelf = toolbar;
		}
		console.log ("exit ChartBuilder.buildToolbar");
	},
	getToolbar: function () {
		var toolbarWidget = dijit.byId (this.id + "toolbar");
		if(toolbarWidget!==undefined && toolbarWidget!==null){
			return (toolbarWidget.getToolbar());
		}
	},
	buildPager: function (rowCount) {
		if(this.chartDef.common.paging){
			if(rowCount>this.chartFetcher.rowsPerPage){
				var pager = new ibm.tivoli.tip.charts.dijit.ChartPager ({
					chartBuilder: this,
					chartDef: this.chartDef,
					chartFetcher: this.chartFetcher 
				}, dojo.byId (this.id + "pager"));
				this.chartPager = pager;
				pager.startup();
			}
		}		
	},
	fetchChartData: function () {
		console.log ("Enter fetchChartData");
	    var busyPane = dijit.byId(this.global.namespace+"busy_pane");
	    if ( busyPane ) {
	      console.log ("showing busypane");
	      busyPane.show(true); // size to portlet
		}
	    
		this.chartFetcher = new ibm.tivoli.tip.charts.dojo.ChartFetcher ({
			global: this.global, 
			chartPreferences: this.chartPreferences,
			chartDef: this.chartDef,
			rowsPerPage: this.chartDef.common.pagingSize,
			widgetDefinition: this.widgetDefinition,
			providerID: this.providerID,
			chartParams: this.widgetDefinition.datasets[0].parameters,
			datasourceID: this.datasourceID	
		});
		
		this.buildToolbar();
		
		if(!this.standalone) {
			//dojo.require ("ibm.tivoli.tip.chartportlet.dijit.DojoChartWidget");
			this.chartUtil = new ibm.tivoli.tip.charts.dojo.DojoChartUtil ({
				namespace: this.global.namespace,
				chartBuilder: this,
				providerID: this.providerID,
				datasourceID: this.datasourceID,
				savePrefsOption: false
			});
		}
		
		dojo.subscribe ("ChartFetcher.dataFetched" + this.chartFetcher.id, dojo.hitch (this, this.afterFetch)); 
		//this.chartFetcher.fetchData();
		dojo.subscribe ("ChartFetcher.totalRowsFetched" + this.chartFetcher.id, dojo.hitch (this, this.buildPager));	
		if(this.chartDef.common.paging){		
			this.chartFetcher.fetchData(0,this.chartFetcher.rowsPerPage);
		}else{
			this.chartFetcher.fetchData();
		}
		console.log ("Chart data fetched");
	},        
	
	updateChartData: function (event) {
		console.log ("enter ChartBuilder.updateChartData");
		if (this.interval !== null) {
        	clearTimeout (this.interval);
            this.interval = null;
        }		
		//dojo.subscribe ("ChartFetcher.dataFetched" + this.chartFetcher.id, dojo.hitch (this, this.afterFetch));
		
		if ((event !== undefined) && (event !== null)  && (event != 0)) {
			console.log ("ChartBuilder.updateChartData: using parameters from event to update chart data");
			this.chartFetcher.chartParams = this.eventParametersToRestParameters (event);
			//this.chartFetcher.fetchData();	
		} else if(this.parameters !== undefined && this.parameters !== null) {		
			console.log ("ChartBuilder.updateChartData: using parameters from prefs/widget def to update chart data");
			this.chartFetcher.chartParams = this.parameters;
			//this.chartFetcher.fetchData();	
		} else {
			console.info ("no parameters for updateChartData");
		}
		if(this.chartPager){
	    	this.chartPager.onRefresh();
	    }else{
	    	this.chartFetcher.fetchData();
	    }
		console.log ("exit ChartBuilder.updateChartData");
	},
	
	updateChartByTimeSet: function(time){		
		this.global.restTimeParam = {
			id: "Time",
			value: time
		};
		/*
		if(this.parameters){
			this.chartFetcher.chartParams = this.parameters;
		}
		*/		
		if(this.chartPager){
	    	this.chartPager.onRefresh();
	    }else{
	    	if(this.chartDef.common.paging){				
				this.chartFetcher.fetchData(0,this.chartFetcher.rowsPerPage);
			}else{
				this.chartFetcher.fetchData();
			}	    	
	    }
	},
	
	/*
	eventParametersToRestParameters: function (event) {
		console.log ("enter ChartBuilder.eventParametersToRestParameters: ");
		var restParams = [];
		var param = null;
		
		for (var i in event) {
			param = {};
			param.id = i;
			param.value = event[i];
			restParams.push (param);
		}
		
		console.log ("exit ChartBuilder.eventParametersToRestParameters: ");
		console.log (restParams);
		
		return (restParams);
	},
	*/
	
	getChart: function () {
		return (this.chart);
	},
	initialSize: function () {
		var vp = dijit.getViewport();
		var coords = dojo.coords (dojo.byId (this.id + "pager"));
		vp.h = vp.h - coords.h;
		coords = dojo.coords (dojo.byId (this.id + "legend"));
		vp.h = vp.h - coords.h;
		
		dojo.style (dojo.byId (this.id + "chart"), "width", vp.w);
		dojo.style (dojo.byId (this.id + "chart"), "height", vp.h);
		
	},
	resize: function (size, forceResize) {
		size = dojo.clone (size);		// clone the size because we will adjust it and do not want to affect the local variable
		console.log ("enter ChartBuilder.resize: ");
		console.log (size);
		
		if (this.chart !== null) {
			if ((size === undefined) || (size === null) || (size.w === undefined) || (size.h === undefined)) {
				if ((this.global.iscPortletNamespace === undefined) || (this.global.iscPortletNamespace === null)) {
					console.log ("ChartBuilder.resize: getting viewport size");
					var vp = dijit.getViewport();
					size = { w: vp.w, h: vp.h };	
				} else {
					console.log ("ChartBuilder.resize: getting iscPortlet size");
					var portletSize = iscPortlet.getPortletSize(this.global.iscPortletNamespace);
					size = { w: portletSize.available.w, h: portletSize.available.h };
				}
			} else {
				console.log ("ChartBuilder.resize: using size passed into resize");
			}

			if(forceResize || !this._currentSize || this._currentSize.w != size.w || this._currentSize.h != size.h){
				console.log ("sizing chart using base size: ");
				console.dir (size);
				
				this._currentSize = dojo.clone(size);
				dojo.style (this.domNode, {
					"width": size.w + "px",
					"height": size.h + "px"
				});

				var coords;			
				if(this.chartPager){
					coords = dojo.coords (dojo.byId (this.id + "pager"));
					size.h = size.h - coords.h;
				}
			
				if (this.chartDef.common.showLastUpdateTime) {
					coords = dojo.coords (this.metaDiv);
					size.h = size.h - 25;				
				} 
			
				var toolbarWidget = dijit.byId (this.id + "toolbar");
				if (toolbarWidget !== undefined && toolbarWidget !== null) {
					coords = dojo.coords (dojo.byId (this.id + "toolbar"));
					size.h = size.h - coords.h;
				}

				if(this.standalone && !this.ignoreTitlebar){
					coords = dojo.coords (dojo.byId (this.id + "title"));
					size.h = size.h - coords.h;
				}
			
				dojo.style (dojo.byId (this.id + "chartContainer"), {
					"width": size.w + "px",
					"height": size.h + "px"
				});
				var axisLabel = this.chartDef.yAxis.label;
				if(axisLabel && axisLabel.value && axisLabel.value != ""){
					var yAxisTitleNode = dojo.byId (this.id + "axistitle");
					if(this.chartDef.type != "Pie") {
						yAxisTitleNode.style.width = "20px";
					}
					coords = dojo.coords (yAxisTitleNode);
					size.w = size.w - coords.w;
				}
				axisLabel = this.chartDef.xAxis.label;
				if(axisLabel && axisLabel.value && axisLabel.value != ""){
					coords = dojo.coords (dojo.byId (this.id + "axistitle_X"));
					size.h = size.h - coords.h;
				}
			
				if ((this.chartDef.legend.hidden !== undefined) && (this.chartDef.legend.hidden === false)) {
					if(this.chartDef.legend.location == "right") {
						coords = dojo.coords (dojo.byId (this.id + "rightLegend"));
						if(dijit.byId (this.id + "legend")){
							size.w = size.w - coords.w;
						}
					} else {
						coords = dojo.coords (dojo.byId (this.id + "bottomLegend"));
						if(dijit.byId (this.id + "legend")){		
							size.h = size.h - coords.h;
						}else{
							size.h = size.h - 30;
						}
					}
				}

				// without this, we get scroll bars on FF freeform pages
				if (dojo.isFF) {
					size.w = size.w - 5;
					size.h = size.h - 5;
				}
				
				this.chartDivSize = size;
				
				if((!this.chartDef.xAxis.rotation || this.chartDef.xAxis.rotation==0) && this.chartDef.type != "Pie"){		
					this.chart.removeAxis(this.chartDef.xAxis.id);
					this.addXAxis(this.chart);
				}

				if((this.charts !== undefined) && (this.charts.length > 1))	{
					console.log ("resizing lots of little pies");
					for (var i=0; i<this.charts.length; i++) {
						this.chart = this.charts[i];					
						this.chart.resize ({w: (size.w / this.charts.length - 10), h: size.h - 50}); 
					}
				} else { 
					if(this.chartDef.type == "Pie") {
						console.log ("resizing a single pie chart... subtracting extra pixels");
						size.w = size.w - 30;
						size.h = size.h - 40;
					} else {			
						size.w = size.w - 10;
						size.h = size.h - 10;
					}
					
					console.log ("calling chart.resize with size: ");
					console.dir (size);
					this.chart.resize (size);
				}			
				if (toolbarWidget !== undefined && toolbarWidget !== null) {
					toolbarWidget.resize(size);
				}
			}
		} else {
			console.warn ("ChartBuilder.resize. this.chart is null");
		}
				
		console.log ("exit ChartBuilder.resize");
	},
	setYAxisTitle: function(chart, axisname, label) {
		console.log ("enter setYAxisTitle w/ label: " + label);
		var axis = chart.axes[axisname];
		var dim = chart.dim;
		var offsets=chart.offsets;
		var ta = chart.theme.axis;		
		var x;
		var y;
		var rotate=0;
		var title = null;

		if(axis.vertical) {
			rotate=90;
			y=dim.height/2  - offsets.b/2;
			x=0+12;
		} else {
			x=dim.width/2 + offsets.l/2;
			y=dim.height;
		}		
		var browserName = navigator.appName;		
		var m = dojox.gfx.matrix;
		var surface = dojox.gfx.createSurface(this.id + "axistitle", 45, 140);
		if (browserName == "Microsoft Internet Explorer") {
			title = surface.createText({x: x, y: y, text:label, align: "start"});
		} else {
			title = surface.createText({x: 32, y: -15, text:label, align: "start"});
		}
		
		title.moveToFront();
		var ylabels = this.custom.ylabels;
		title.setFont({family:ylabels.fontFamily, size: ylabels.size, weight: ylabels.weight});
		//title.setFont("Verdana");
		var color = this.custom.ylabels.color;
		console.log ("setYAxisTitle color: " + color);
		
		title.setFill(color);
		title.setTransform(m.rotategAt(rotate,x,y));

		return(chart);
	},
	setXAxisTitle: function(chart, axisname, label) {
		console.log ("enter ChartBuilder.setXAxisTitle: " + label);
		var axis = chart.axes[axisname];		
		var dim = chart.dim;
		var offsets=chart.offsets;
		var x;
		var y;
		var rotate=0;		
		
		x=dim.width/2 + offsets.l/2;
		y=dim.height;
			
		var m = dojox.gfx.matrix;	
		var title;

		title = axis.group.createText({x:x, y:y-5, text:label, align: "end"});
		var xlabels = this.custom.xlabels;
		title.setFont({family:xlabels.fontFamily, size: xlabels.size, weight: xlabels.weight});
		//title.setFill('black');
		console.log ("before setXAxisTitle color");
		var color = this.custom.xlabels.color;
		console.log ("setXAxisTitle color: " + color);
				
		title.setFill(color);
		title.setTransform(m.rotategAt(rotate,x,y));
		title.moveToFront();
								
		console.log ("exit ChartBuilder.setXAxisTitle");
	},
	onRefresh: function () {
		this.chartFetcher.fetchData();
	},
	onPreferences: function () {
		this.preferencesDialog = new ibm.tivoli.tip.charts.dijit.ChartPreferences ( 
				{ global: this.global,
				  chartBuilder: this
				} 
			);
		
		this.preferencesDialog.show();
	},
	setTheme: function (activeTheme) {
		console.log ("enter setTheme: " + activeTheme);
		//if (this.themeChangeSupported()) {
			if (activeTheme == "TIPDark") {
				this.theme = ibm.tivoli.tip.dijit.themes.ChartThemeTivoliDark.clone();
				this.custom = ibm.tivoli.tip.dijit.themes.ChartThemeTivoliDarkCustom;
				console.log ("setting theme to Dark");
			} else if ((activeTheme == "TIPLight") || (activeTheme == "ISCTheme")) {
				this.theme = ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandard.clone();
				this.custom = ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandardCustom;
				console.log ("setting theme to Light");
			} else if ((this.chartDef.theme) && (this.chartDef.theme.name)) {
				// no short theme defined, so just use the long theme name passed into the chart
				console.log ("using theme that is part of the chart: " + this.chartDef.theme.name);
				var str = 'dojo.' + 'require("' + this.chartDef.theme.name + '");';
				console.log ("require string: " + str);
				eval (str);
				console.log ("about to eval the theme");
				this.theme = eval (this.chartDef.theme.name);
				console.log ("cloning the theme");
				this.theme = this.theme.clone();
				console.log ("eval-ing the Custom");
				this.custom = eval (this.chartDef.theme.name + 'Custom');
			} else {
				console.log ("using theme passed into chart because activeTheme is not recognized or null: " + activeTheme);
			}
		//} else {
		//	console.log ("theme change is not supported");
		//}
	},
	arrangeGradientPalette: function () {
		if (this.theme.rearranged !== true) {
			// start the index
			var i = this.themeIndex % this.theme.seriesThemes.length;
			var newColors = [];
		
			for (var total =0; total < this.theme.seriesThemes.length; total++) {	
				newColors.push (this.theme.seriesThemes[i]);
				i = ++i % this.theme.seriesThemes.length;
			}
		 
			this.theme.seriesThemes = newColors;
			this.theme.rearranged = true;
		}
	},
	arrangeColorPalette: function () {
		if (this.theme.rearranged !== true) {
			// start the index
			var i = this.themeIndex % this.theme.colors.length;
			var newColors = [];
		
			for (var total =0; total < this.theme.colors.length; total++) {	
				newColors.push (this.theme.colors[i]);
				i = ++i % this.theme.colors.length;
			}
		 
			this.theme.colors = newColors;
			this.theme.rearranged = true;
		}
	},
	themeChangeSupported: function () {
		if ((this.theme == ibm.tivoli.tip.dijit.themes.ChartThemeTivoliStandard) ||
			(this.theme == ibm.tivoli.tip.dijit.themes.ChartThemeTivoliDark)) {
			return (true);
		} else {
			return (false);
		}
	},
	changeTheme: function (event) {
		console.log ("changing active theme to: " + event.activeTheme);
		this.setTheme (event.activeTheme);		
		this.updateChart();
	},
	destroy: function () {
		console.log ("enter ChartBuilder.destroy");
		this.inherited ("destroy", arguments);
		//this.getToolbar().destroyRecursive();		
		if(this.toolbarSelf){
			this.toolbarSelf.destroyRecursive();
		}
		if(this.chartPager){
			this.chartPager.destroyRecursive();
		}
		if(this.legend){
			this.legend.destroy();
		}
		// remove auto-refresh
		if (this.interval !== null) {
			clearTimeout (this.interval);
			delete this.interval;
		}		

		console.log ("exit ChartBuilder.destroy");
	}
});

}

if(!dojo._hasResource["dojox.data.ServiceStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.data.ServiceStore"] = true;
dojo.provide("dojox.data.ServiceStore");

// note that dojox.rpc.Service is not required, you can create your own services

// A ServiceStore is a readonly data store that provides a data.data interface to an RPC service.
// var myServices = new dojox.rpc.Service(dojo.moduleUrl("dojox.rpc.tests.resources", "test.smd"));
// var serviceStore = new dojox.data.ServiceStore({service:myServices.ServiceStore});
//
// The ServiceStore also supports lazy loading. References can be made to objects that have not been loaded.
//	For example if a service returned:
// {"name":"Example","lazyLoadedObject":{"$ref":"obj2"}}
//
// And this object has accessed using the dojo.data API:
// var obj = serviceStore.getValue(myObject,"lazyLoadedObject");
// The object would automatically be requested from the server (with an object id of "obj2").
//

dojo.declare("dojox.data.ServiceStore",
	// ClientFilter is intentionally not required, ServiceStore does not need it, and is more
	// lightweight without it, but if it is provided, the ServiceStore will use it.
	dojox.data.ClientFilter||null,{
		service: null,
		constructor: function(options){
			//summary:
			//		ServiceStore constructor, instantiate a new ServiceStore
			// 		A ServiceStore can be configured from a JSON Schema. Queries are just
			// 		passed through to the underlying services
			//
			// options:
			// 		Keyword arguments
			// The *schema* parameter
			//		This is a schema object for this store. This should be JSON Schema format.
			//
			// The *service* parameter
			// 		This is the service object that is used to retrieve lazy data and save results
			// 		The function should be directly callable with a single parameter of an object id to be loaded
			//
			// The *idAttribute* parameter
			//		Defaults to 'id'. The name of the attribute that holds an objects id.
			//		This can be a preexisting id provided by the server.
			//		If an ID isn't already provided when an object
			//		is fetched or added to the store, the autoIdentity system
			//		will generate an id for it and add it to the index.
			//
			// The *estimateCountFactor* parameter
			// 		This parameter is used by the ServiceStore to estimate the total count. When
			//		paging is indicated in a fetch and the response includes the full number of items
			//	 	requested by the fetch's count parameter, then the total count will be estimated
			//		to be estimateCountFactor multiplied by the provided count. If this is 1, then it is assumed that the server
			//		does not support paging, and the response is the full set of items, where the
			// 		total count is equal to the numer of items returned. If the server does support
			//		paging, an estimateCountFactor of 2 is a good value for estimating the total count
			//		It is also possible to override _processResults if the server can provide an exact
			// 		total count.
			//
			// The *syncMode* parameter
			//		Setting this to true will set the store to using synchronous calls by default.
			//		Sync calls return their data immediately from the calling function, so
			//		callbacks are unnecessary. This will only work with a synchronous capable service.
			//
			// description:
			//		ServiceStore can do client side caching and result set updating if
			// 		dojox.data.ClientFilter is loaded. Do this add:
			//	|	dojo.require("dojox.data.ClientFilter")
			//		prior to loading the ServiceStore (ClientFilter must be loaded before ServiceStore).
			//		To utilize client side filtering with a subclass, you can break queries into
			//		client side and server side components by putting client side actions in
			//		clientFilter property in fetch calls. For example you could override fetch:
			//	|	fetch: function(args){
				//	|		// do the sorting and paging on the client side
	 			//	|		args.clientFilter = {start:args.start, count: args.count, sort: args.sort};
	 			//	|		// args.query will be passed to the service object for the server side handling
	 			//	|		return this.inherited(arguments);
			//	|	}
			//		When extending this class, if you would like to create lazy objects, you can follow
			//		the example from dojox.data.tests.stores.ServiceStore:
			// |	var lazyItem = {
			// |		_loadObject: function(callback){
			// |			this.name="loaded";
			// |			delete this._loadObject;
			// |			callback(this);
			// |		}
			// |	};
			//setup a byId alias to the api call
			this.byId=this.fetchItemByIdentity;
			this._index = {};
			// if the advanced json parser is enabled, we can pass through object updates as onSet events
			if(options){
				dojo.mixin(this,options);
			}
			// We supply a default idAttribute for parser driven construction, but if no id attribute
			//	is supplied, it should be null so that auto identification takes place properly
			this.idAttribute = (options && options.idAttribute) || (this.schema && this.schema._idAttr);
		},
		schema: null,
		idAttribute: "id",
		labelAttribute: "label",
		syncMode: false,
		estimateCountFactor: 1,
		getSchema: function(){
			return this.schema;
		},

		loadLazyValues:true,

		getValue: function(/*Object*/ item, /*String*/property, /*value?*/defaultValue){
			// summary:
			//	Gets the value of an item's 'property'
			//
			//	item:
			//		The item to get the value from
			//	property:
			//		property to look up value for
			//	defaultValue:
			//		the default value

			var value = item[property];
			return value || // return the plain value since it was found;
						(property in item ? // a truthy value was not found, see if we actually have it
							value : // we do, so we can return it
							item._loadObject ? // property was not found, maybe because the item is not loaded, we will try to load it synchronously so we can get the property
								(dojox.rpc._sync = true) && arguments.callee.call(this,dojox.data.ServiceStore.prototype.loadItem({item:item}) || {}, property, defaultValue) : // load the item and run getValue again
								defaultValue);// not in item -> return default value
		},
		getValues: function(item, property){
			// summary:
			//		Gets the value of an item's 'property' and returns
			//		it.	If this value is an array it is just returned,
			//		if not, the value is added to an array and that is returned.
			//
			//	item: /* object */
			//	property: /* string */
			//		property to look up value for

			var val = this.getValue(item,property);
			if(val instanceof Array){
				return val;
			}
			if(!this.isItemLoaded(val)){
				dojox.rpc._sync = true;
				val = this.loadItem({item:val});
			}
			return val instanceof Array ? val : val === undefined ? [] : [val];
		},

		getAttributes: function(item){
			// summary:
			//	Gets the available attributes of an item's 'property' and returns
			//	it as an array.
			//
			//	item: /* object */

			var res = [];
			for(var i in item){
				if(item.hasOwnProperty(i) && !(i.charAt(0) == '_' && i.charAt(1) == '_')){
					res.push(i);
				}
			}
			return res;
		},

		hasAttribute: function(item,attribute){
			// summary:
			//		Checks to see if item has attribute
			//
			//	item: /* object */
			//	attribute: /* string */
			return attribute in item;
		},

		containsValue: function(item, attribute, value){
			// summary:
			//		Checks to see if 'item' has 'value' at 'attribute'
			//
			//	item: /* object */
			//	attribute: /* string */
			//	value: /* anything */
			return dojo.indexOf(this.getValues(item,attribute),value) > -1;
		},


		isItem: function(item){
			// summary:
			//		Checks to see if the argument is an item
			//
			//	item: /* object */
			//	attribute: /* string */

			// we have no way of determining if it belongs, we just have object returned from
			// 	service queries
			return (typeof item == 'object') && item && !(item instanceof Date);
		},

		isItemLoaded: function(item){
			// summary:
			//		Checks to see if the item is loaded.
			//
			//		item: /* object */

			return item && !item._loadObject;
		},

		loadItem: function(args){
			// summary:
			// 		Loads an item and calls the callback handler. Note, that this will call the callback
			// 		handler even if the item is loaded. Consequently, you can use loadItem to ensure
			// 		that an item is loaded is situations when the item may or may not be loaded yet.
			// 		If you access a value directly through property access, you can use this to load
			// 		a lazy value as well (doesn't need to be an item).
			//
			//	example:
			//		store.loadItem({
			//			item: item, // this item may or may not be loaded
			//			onItem: function(item){
			// 				// do something with the item
			//			}
			//		});

			var item;
			if(args.item._loadObject){
				args.item._loadObject(function(result){
					item = result; // in synchronous mode this can allow loadItem to return the value
					delete item._loadObject;
					var func = result instanceof Error ? args.onError : args.onItem;
					if(func){
						func.call(args.scope, result);
					}
				});
			}else if(args.onItem){
				// even if it is already loaded, we will use call the callback, this makes it easier to
				// use when it is not known if the item is loaded (you can always safely call loadItem).
				args.onItem.call(args.scope, args.item);
			}
			return item;
		},
		_currentId : 0,
		_processResults : function(results, deferred){
			// this should return an object with the items as an array and the total count of
			// items (maybe more than currently in the result set).
			// for example:
			//	| {totalCount:10, items: [{id:1},{id:2}]}

			// index the results, assigning ids as necessary

			if(results && typeof results == 'object'){
				var id = results.__id;
				if(!id){// if it hasn't been assigned yet
					if(this.idAttribute){
						// use the defined id if available
						id = results[this.idAttribute];
					}else{
						id = this._currentId++;
					}
					if(id !== undefined){
						var existingObj = this._index[id];
						if(existingObj){
							for(var j in existingObj){
								delete existingObj[j]; // clear it so we can mixin
							}
							results = dojo.mixin(existingObj,results);
						}
						results.__id = id;
						this._index[id] = results;
					}
				}
				for(var i in results){
					results[i] = this._processResults(results[i], deferred).items;
				}
				var count = results.length;
			}
			return {totalCount: deferred.request.count == count ? (deferred.request.start || 0) + count * this.estimateCountFactor : count, items: results};
		},
		close: function(request){
			return request && request.abort && request.abort();
		},
		fetch: function(args){
			// summary:
			//		See dojo.data.api.Read.fetch
			//
			// The *queryOptions.cache* parameter
			//		If true, indicates that the query result should be cached for future use. This is only available
			// 		if dojox.data.ClientFilter has been loaded before the ServiceStore
			//
			//	The *syncMode* parameter
			//		Indicates that the call should be fetch synchronously if possible (this is not always possible)
			//
			// The *clientFetch* parameter
			//		This is a fetch keyword argument for explicitly doing client side filtering, querying, and paging

			args = args || {};

			if("syncMode" in args ? args.syncMode : this.syncMode){
				dojox.rpc._sync = true;
			}
			var self = this;

			var scope = args.scope || self;
			var defResult = this.cachingFetch ? this.cachingFetch(args) : this._doQuery(args);
			defResult.request = args;
			defResult.addCallback(function(results){
				if(args.clientFetch){
					results = self.clientSideFetch({query:args.clientFetch,sort:args.sort,start:args.start,count:args.count},results);
				}
				var resultSet = self._processResults(results, defResult);
				results = args.results = resultSet.items;
				if(args.onBegin){
					args.onBegin.call(scope, resultSet.totalCount, args);
				}
				if(args.onItem){
					for(var i=0; i<results.length;i++){
						args.onItem.call(scope, results[i], args);
					}
				}
				if(args.onComplete){
					args.onComplete.call(scope, args.onItem ? null : results, args);
				}
				return results;
			});
			defResult.addErrback(args.onError && function(err){
				return args.onError.call(scope, err, args);
			});
			args.abort = function(){
				// abort the request
				defResult.cancel();
			};
			args.store = this;
			return args;
		},
		_doQuery: function(args){
			var query= typeof args.queryStr == 'string' ? args.queryStr : args.query;
			return this.service(query);
		},
		getFeatures: function(){
			// summary:
			// 		return the store feature set

			return {
				"dojo.data.api.Read": true,
				"dojo.data.api.Identity": true,
				"dojo.data.api.Schema": this.schema
			};
		},

		getLabel: function(item){
			// summary
			//		returns the label for an item. Just gets the "label" attribute.
			//
			return this.getValue(item,this.labelAttribute);
		},

		getLabelAttributes: function(item){
			// summary:
			//		returns an array of attributes that are used to create the label of an item
			return [this.labelAttribute];
		},

		//Identity API Support


		getIdentity: function(item){
			return item.__id;
		},

		getIdentityAttributes: function(item){
			// summary:
			//		returns the attributes which are used to make up the
			//		identity of an item.	Basically returns this.idAttribute

			return [this.idAttribute];
		},

		fetchItemByIdentity: function(args){
			// summary:
			//		fetch an item by its identity, by looking in our index of what we have loaded
			var item = this._index[(args._prefix || '') + args.identity];
			if(item){
				// the item exists in the index
				if(item._loadObject){
					// we have a handle on the item, but it isn't loaded yet, so we need to load it
					args.item = item;
					return this.loadItem(args);
				}else if(args.onItem){
					// it's already loaded, so we can immediately callback
					args.onItem.call(args.scope, item);
				}
			}else{
				// convert the different spellings
				return this.fetch({
						query: args.identity,
						onComplete: args.onItem,
						onError: args.onError,
						scope: args.scope
					}).results;
			}
			return item;
		}

	}
);

}

if(!dojo._hasResource["dojox.json.ref"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.json.ref"] = true;
dojo.provide("dojox.json.ref");



dojox.json.ref = {
	// summary:
	// 		Adds advanced JSON {de}serialization capabilities to the base json library.
	// 		This enhances the capabilities of dojo.toJson and dojo.fromJson,
	// 		adding referencing support, date handling, and other extra format handling.
	// 		On parsing, references are resolved. When references are made to
	// 		ids/objects that have been loaded yet, the loader function will be set to
	// 		_loadObject to denote a lazy loading (not loaded yet) object. 


	resolveJson: function(/*Object*/ root,/*Object?*/ args){
		// summary:
		// 		Indexes and resolves references in the JSON object.
		// description:
		// 		A JSON Schema object that can be used to advise the handling of the JSON (defining ids, date properties, urls, etc)
		//
		// root:
		//		The root object of the object graph to be processed
		// args:
		//		Object with additional arguments:
		//
		// The *index* parameter.
		//		This is the index object (map) to use to store an index of all the objects. 
		// 		If you are using inter-message referencing, you must provide the same object for each call.
		// The *defaultId* parameter.
		//		This is the default id to use for the root object (if it doesn't define it's own id)
		//	The *idPrefix* parameter.
		//		This the prefix to use for the ids as they enter the index. This allows multiple tables 
		// 		to use ids (that might otherwise collide) that enter the same global index. 
		// 		idPrefix should be in the form "/Service/".  For example,
		//		if the idPrefix is "/Table/", and object is encountered {id:"4",...}, this would go in the
		//		index as "/Table/4".
		//	The *idAttribute* parameter.
		//		This indicates what property is the identity property. This defaults to "id"
		//	The *assignAbsoluteIds* parameter.
		//		This indicates that the resolveJson should assign absolute ids (__id) as the objects are being parsed.
		//  
		// The *schemas* parameter
		//		This provides a map of schemas, from which prototypes can be retrieved
		// The *loader* parameter
		//		This is a function that is called added to the reference objects that can't be resolved (lazy objects)
		// return:
		//		An object, the result of the processing
		args = args || {};
		var idAttribute = args.idAttribute || 'id';
		var refAttribute = this.refAttribute;
		var idAsRef = args.idAsRef;
		var prefix = args.idPrefix || ''; 
		var assignAbsoluteIds = args.assignAbsoluteIds;
		var index = args.index || {}; // create an index if one doesn't exist
		var timeStamps = args.timeStamps;
		var ref,reWalk=[];
		var pathResolveRegex = /^(.*\/)?(\w+:\/\/)|[^\/\.]+\/\.\.\/|^.*\/(\/)/;
		var addProp = this._addProp;
		var F = function(){};
		function walk(it, stop, defaultId, needsPrefix, schema, defaultObject){
			// this walks the new graph, resolving references and making other changes
		 	var i, update, val, id = idAttribute in it ? it[idAttribute] : defaultId;
		 	if(idAttribute in it || ((id !== undefined) && needsPrefix)){
		 		id = (prefix + id).replace(pathResolveRegex,'$2$3');
		 	}
		 	var target = defaultObject || it;
			if(id !== undefined){ // if there is an id available...
				if(assignAbsoluteIds){
					it.__id = id;
				}
				if(args.schemas && (!(it instanceof Array)) && // won't try on arrays to do prototypes, plus it messes with queries 
		 					(val = id.match(/^(.+\/)[^\.\[]*$/))){ // if it has a direct table id (no paths)
		 			schema = args.schemas[val[1]];
				} 
				// if the id already exists in the system, we should use the existing object, and just 
				// update it... as long as the object is compatible
				if(index[id] && ((it instanceof Array) == (index[id] instanceof Array))){ 
					target = index[id];
					delete target.$ref; // remove this artifact
					delete target._loadObject;
					update = true;
				}else{
				 	var proto = schema && schema.prototype; // and if has a prototype
					if(proto){
						// if the schema defines a prototype, that needs to be the prototype of the object
						F.prototype = proto;
						target = new F();
					}
				}
				index[id] = target; // add the prefix, set _id, and index it
				if(timeStamps){
					timeStamps[id] = args.time;
				}
			}
			while(schema){
				var properties = schema.properties;
				if(properties){
					for(i in it){
						var propertyDefinition = properties[i];
						if(propertyDefinition && propertyDefinition.format == 'date-time' && typeof it[i] == 'string'){
							it[i] = dojo.date.stamp.fromISOString(it[i]);
						}
					}
				}
				schema = schema["extends"];
			}
			var length = it.length;
			for(i in it){
				if(i==length){
					break;		
				}
				if(it.hasOwnProperty(i)){
					val=it[i];
					if((typeof val =='object') && val && !(val instanceof Date) && i != '__parent'){
						ref=val[refAttribute] || (idAsRef && val[idAttribute]);
						if(!ref || !val.__parent){
							if(it != reWalk){
								val.__parent = target;
							}
						}
						if(ref){ // a reference was found
							// make sure it is a safe reference
							delete it[i];// remove the property so it doesn't resolve to itself in the case of id.propertyName lazy values
							var path = ref.toString().replace(/(#)([^\.\[])/,'$1.$2').match(/(^([^\[]*\/)?[^#\.\[]*)#?([\.\[].*)?/); // divide along the path
							if((ref = (path[1]=='$' || path[1]=='this' || path[1]=='') ? root : index[(prefix + path[1]).replace(pathResolveRegex,'$2$3')])){  // a $ indicates to start with the root, otherwise start with an id
								// if there is a path, we will iterate through the path references
								if(path[3]){
									path[3].replace(/(\[([^\]]+)\])|(\.?([^\.\[]+))/g,function(t,a,b,c,d){
										ref = ref && ref[b ? b.replace(/[\"\'\\]/,'') : d];
									});
								}
							}
							if(ref){
								val = ref;
							}else{
								// otherwise, no starting point was found (id not found), if stop is set, it does not exist, we have
								// unloaded reference, if stop is not set, it may be in a part of the graph not walked yet,
								// we will wait for the second loop
								if(!stop){
									var rewalking;
									if(!rewalking){
										reWalk.push(target); // we need to rewalk it to resolve references
									}
									rewalking = true; // we only want to add it once
									val = walk(val, false, val[refAttribute], true, propertyDefinition);
									// create a lazy loaded object
									val._loadObject = args.loader;
								}
							}
						}else{
							if(!stop){ // if we are in stop, that means we are in the second loop, and we only need to check this current one,
								// further walking may lead down circular loops
								val = walk(
									val,
									reWalk==it,
									id === undefined ? undefined : addProp(id, i), // the default id to use
									false,
									propertyDefinition, 
									// if we have an existing object child, we want to 
									// maintain it's identity, so we pass it as the default object
									target != it && typeof target[i] == 'object' && target[i] 
								);
							}
						}
					}
					it[i] = val;
					if(target!=it && !target.__isDirty){// do updates if we are updating an existing object and it's not dirty				
						var old = target[i];
						target[i] = val; // only update if it changed
						if(update && val !== old && // see if it is different 
								!target._loadObject && // no updates if we are just lazy loading
								!(i.charAt(0) == '_' && i.charAt(1) == '_') && i != "$ref" &&  
								!(val instanceof Date && old instanceof Date && val.getTime() == old.getTime()) && // make sure it isn't an identical date
								!(typeof val == 'function' && typeof old == 'function' && val.toString() == old.toString()) && // make sure it isn't an indentical function
								index.onUpdate){
							index.onUpdate(target,i,old,val); // call the listener for each update
						}
					}
				}
			}
	
			if(update && (idAttribute in it || target instanceof Array)){
				// this means we are updating with a full representation of the object, we need to remove deleted
				for(i in target){
					if(!target.__isDirty && target.hasOwnProperty(i) && !it.hasOwnProperty(i) && !(i.charAt(0) == '_' && i.charAt(1) == '_') && !(target instanceof Array && isNaN(i))){
						if(index.onUpdate && i != "_loadObject" && i != "_idAttr"){
							index.onUpdate(target,i,target[i],undefined); // call the listener for each update
						}
						delete target[i];
						while(target instanceof Array && target.length && target[target.length-1] === undefined){
							// shorten the target if necessary
							target.length--;
						}
					}
				}
			}else{
				if(index.onLoad){
					index.onLoad(target);
				}
			}
			return target;
		}
		if(root && typeof root == 'object'){
			root = walk(root,false,args.defaultId, true); // do the main walk through
			walk(reWalk,false); // re walk any parts that were not able to resolve references on the first round
		}
		return root;
	},


	fromJson: function(/*String*/ str,/*Object?*/ args){
	// summary:
	// 		evaluates the passed string-form of a JSON object.
	//
	// str:
	//		a string literal of a JSON item, for instance:
	//			'{ "foo": [ "bar", 1, { "baz": "thud" } ] }'
	// args: See resolveJson
	//
	// return:
	//		An object, the result of the evaluation
		function ref(target){ // support call styles references as well
			var refObject = {};
			refObject[this.refAttribute] = target;
			return refObject;
		}
		try{
			var root = eval('(' + str + ')'); // do the eval
		}catch(e){
			throw new SyntaxError("Invalid JSON string: " + e.message + " parsing: "+ str);
		}		
		if(root){
			return this.resolveJson(root, args);
		}
		return root;
	},
	
	toJson: function(/*Object*/ it, /*Boolean?*/ prettyPrint, /*Object?*/ idPrefix, /*Object?*/ indexSubObjects){
		// summary:
		//		Create a JSON serialization of an object.
		//		This has support for referencing, including circular references, duplicate references, and out-of-message references
		// 		id and path-based referencing is supported as well and is based on http://www.json.com/2007/10/19/json-referencing-proposal-and-library/.
		//
		// it:
		//		an object to be serialized.
		//
		// prettyPrint:
		//		if true, we indent objects and arrays to make the output prettier.
		//		The variable dojo.toJsonIndentStr is used as the indent string
		//		-- to use something other than the default (tab),
		//		change that variable before calling dojo.toJson().
		//
		// idPrefix: The prefix that has been used for the absolute ids
		//
		// return:
		//		a String representing the serialized version of the passed object.
		var useRefs = this._useRefs;
		var addProp = this._addProp;
		var refAttribute = this.refAttribute;
		idPrefix = idPrefix || ''; // the id prefix for this context
		var paths={};
		var generated = {};
		function serialize(it,path,_indentStr){
			if(typeof it == 'object' && it){
				var value;
				if(it instanceof Date){ // properly serialize dates
					return '"' + dojo.date.stamp.toISOString(it,{zulu:true}) + '"';
				}
				var id = it.__id;
				if(id){ // we found an identifiable object, we will just serialize a reference to it... unless it is the root
					if(path != '#' && ((useRefs && !id.match(/#/)) || paths[id])){
						var ref = id;	
						if(id.charAt(0)!='#'){
							if(it.__clientId == id){
								ref = "cid:" + id;
							}else if(id.substring(0, idPrefix.length) == idPrefix){ // see if the reference is in the current context
								// a reference with a prefix matching the current context, the prefix should be removed
								ref = id.substring(idPrefix.length);
							}else{
								// a reference to a different context, assume relative url based referencing
								ref = id;
							}
						}
						var refObject = {};
						refObject[refAttribute] = ref;
						return serialize(refObject,'#');
					}
					path = id;
				}else{
					it.__id = path; // we will create path ids for other objects in case they are circular
					generated[path] = it;
				}
				paths[path] = it;// save it here so they can be deleted at the end
				_indentStr = _indentStr || "";
				var nextIndent = prettyPrint ? _indentStr + dojo.toJsonIndentStr : "";
				var newLine = prettyPrint ? "\n" : "";
				var sep = prettyPrint ? " " : "";
	
				if(it instanceof Array){
					var res = dojo.map(it, function(obj,i){
						var val = serialize(obj, addProp(path, i), nextIndent);
						if(typeof val != "string"){
							val = "undefined";
						}
						return newLine + nextIndent + val;
					});
					return "[" + res.join("," + sep) + newLine + _indentStr + "]";
				}
	
				var output = [];
				for(var i in it){
					if(it.hasOwnProperty(i)){
						var keyStr;
						if(typeof i == "number"){
							keyStr = '"' + i + '"';
						}else if(typeof i == "string" && (i.charAt(0) != '_' || i.charAt(1) != '_')){
							// we don't serialize our internal properties __id and __clientId
							keyStr = dojo._escapeString(i);
						}else{
							// skip non-string or number keys
							continue;
						}
						var val = serialize(it[i],addProp(path, i),nextIndent);
						if(typeof val != "string"){
							// skip non-serializable values
							continue;
						}
						output.push(newLine + nextIndent + keyStr + ":" + sep + val);
					}
				}
				return "{" + output.join("," + sep) + newLine + _indentStr + "}";
			}else if(typeof it == "function" && dojox.json.ref.serializeFunctions){
				return it.toString();
			}
	
			return dojo.toJson(it); // use the default serializer for primitives
		}
		var json = serialize(it,'#','');
		if(!indexSubObjects){
			for(var i in generated)  {// cleanup the temporary path-generated ids
				delete generated[i].__id;
			}
		}
		return json;
	},
	_addProp: function(id, prop){
		return id + (id.match(/#/) ? id.length == 1 ? '' : '.' : '#') + prop;
	},
	//	refAttribute: String
	//		This indicates what property is the reference property. This acts like the idAttribute
	// 		except that this is used to indicate the current object is a reference or only partially 
	// 		loaded. This defaults to "$ref". 
	refAttribute: "$ref",
	_useRefs: false,
	serializeFunctions: false
}

}

if(!dojo._hasResource["dojox.rpc.Rest"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.rpc.Rest"] = true;
dojo.provide("dojox.rpc.Rest"); 
// Note: This doesn't require dojox.rpc.Service, and if you want it you must require it 
// yourself, and you must load it prior to dojox.rpc.Rest.

// summary:
// 		This provides a HTTP REST service with full range REST verbs include PUT,POST, and DELETE.
// description:
// 		A normal GET query is done by using the service directly:
// 		| var restService = dojox.rpc.Rest("Project");
// 		| restService("4");
//		This will do a GET for the URL "/Project/4".
//		| restService.put("4","new content");
//		This will do a PUT to the URL "/Project/4" with the content of "new content".
//		You can also use the SMD service to generate a REST service:
// 		| var services = dojox.rpc.Service({services: {myRestService: {transport: "REST",...
// 		| services.myRestService("parameters");
//
// 		The modifying methods can be called as sub-methods of the rest service method like:
//  	| services.myRestService.put("parameters","data to put in resource");
//  	| services.myRestService.post("parameters","data to post to the resource");
//  	| services.myRestService['delete']("parameters");
(function(){
	if(dojox.rpc && dojox.rpc.transportRegistry){
		// register it as an RPC service if the registry is available
		dojox.rpc.transportRegistry.register(
			"REST",
			function(str){return str == "REST";},
			{
				getExecutor : function(func,method,svc){
					return new dojox.rpc.Rest(
						method.name,
						(method.contentType||svc._smd.contentType||"").match(/json|javascript/), // isJson
						null,
						function(id, args){
							var request = svc._getRequest(method,[id]);
							request.url= request.target + (request.data ? '?'+  request.data : '');
							return request;
						}
					);
				}
			}
		);
	}
	var drr;

	function index(deferred, service, range, id){
		deferred.addCallback(function(result){
			if(deferred.ioArgs.xhr && range){
					// try to record the total number of items from the range header
					range = deferred.ioArgs.xhr.getResponseHeader("Content-Range");
					deferred.fullLength = range && (range=range.match(/\/(.*)/)) && parseInt(range[1]);
			}
			return result;
		});
		return deferred;
	}
	drr = dojox.rpc.Rest = function(/*String*/path, /*Boolean?*/isJson, /*Object?*/schema, /*Function?*/getRequest){
		// summary:
		//		Creates a REST service using the provided path.
		var service;
		// it should be in the form /Table/
		service = function(id, args){
			return drr._get(service, id, args);
		};
		service.isJson = isJson;
		service._schema = schema;
		// cache:
		//		This is an object that provides indexing service
		// 		This can be overriden to take advantage of more complex referencing/indexing
		// 		schemes
		service.cache = {
			serialize: isJson ? ((dojox.json && dojox.json.ref) || dojo).toJson : function(result){
				return result;
			}
		};
		// the default XHR args creator:
		service._getRequest = getRequest || function(id, args){
			if(dojo.isObject(id)){
				id = dojo.objectToQuery(id);
				id = id ? "?" + id: "";
			}
			if(args && args.sort && !args.queryStr){
				id += (id ? "&" : "?") + "sort("
				for(var i = 0; i<args.sort.length; i++){
					var sort = args.sort[i];
					id += (i > 0 ? "," : "") + (sort.descending ? '-' : '+') + encodeURIComponent(sort.attribute); 
				}
				id += ")";
			}
			var request = {
				url: path + (id == null ? "" : id),
				handleAs: isJson ? 'json' : 'text', 
				contentType: isJson ? 'application/json' : 'text/plain',
				sync: dojox.rpc._sync,
				headers: {
					Accept: isJson ? 'application/json,application/javascript' : '*/*'
				}
			};
			if(args && (args.start >= 0 || args.count >= 0)){
				request.headers.Range = "items=" + (args.start || '0') + '-' + ((args.count && args.count != Infinity && (args.count + (args.start || 0) - 1)) || '');
			}
			dojox.rpc._sync = false;
			return request;
		};
		// each calls the event handler
		function makeRest(name){
			service[name] = function(id,content){
				return drr._change(name,service,id,content); // the last parameter is to let the OfflineRest know where to store the item
			};
		}
		makeRest('put');
		makeRest('post');
		makeRest('delete');
		// record the REST services for later lookup
		service.servicePath = path;
		return service;
	};

	drr._index={};// the map of all indexed objects that have gone through REST processing
	drr._timeStamps={};
	// these do the actual requests
	drr._change = function(method,service,id,content){
		// this is called to actually do the put, post, and delete
		var request = service._getRequest(id);
		request[method+"Data"] = content;
		return index(dojo.xhr(method.toUpperCase(),request,true),service);
	};

	drr._get= function(service,id, args){
		args = args || {};
		// this is called to actually do the get
		return index(dojo.xhrGet(service._getRequest(id, args)), service, (args.start >= 0 || args.count >= 0), id);
	};
})();

}

if(!dojo._hasResource["dojox.rpc.JsonRest"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.rpc.JsonRest"] = true;
dojo.provide("dojox.rpc.JsonRest");

 // this provides json indexing

// summary:
// 		Provides JSON/REST utility functions
(function(){
	var dirtyObjects = [];
	var Rest = dojox.rpc.Rest;
	var jr;
	function resolveJson(service, deferred, value, defaultId){
		var timeStamp = deferred.ioArgs && deferred.ioArgs.xhr && deferred.ioArgs.xhr.getResponseHeader("Last-Modified");
		if(timeStamp && Rest._timeStamps){
			Rest._timeStamps[defaultId] = timeStamp;
		}
		var hrefProperty = service._schema && service._schema.hrefProperty;
		if(hrefProperty){
			dojox.json.ref.refAttribute = hrefProperty;
		}
		value = value && dojox.json.ref.resolveJson(value, {
			defaultId: defaultId, 
			index: Rest._index,
			timeStamps: timeStamp && Rest._timeStamps,
			time: timeStamp,
			idPrefix: service.servicePath.replace(/[^\/]*$/,''),
			idAttribute: jr.getIdAttribute(service),
			schemas: jr.schemas,
			loader:	jr._loader,
			idAsRef: service.idAsRef, 
			assignAbsoluteIds: true
		});
		dojox.json.ref.refAttribute  = "$ref";
		return value;
	}
	jr = dojox.rpc.JsonRest={
		serviceClass: dojox.rpc.Rest,
		conflictDateHeader: "If-Unmodified-Since",
		commit: function(kwArgs){
			// summary:
			//		Saves the dirty data using REST Ajax methods

			kwArgs = kwArgs || {};
			var actions = [];
			var alreadyRecorded = {};
			var savingObjects = [];
			for(var i = 0; i < dirtyObjects.length; i++){
				var dirty = dirtyObjects[i];
				var object = dirty.object;
				var old = dirty.old;
				var append = false;
				if(!(kwArgs.service && (object || old) && 
						(object || old).__id.indexOf(kwArgs.service.servicePath)) && dirty.save){
					delete object.__isDirty;
					if(object){
						if(old){
							// changed object
							var pathParts;
							if((pathParts = object.__id.match(/(.*)#.*/))){ // it is a path reference
								// this means it is a sub object, we must go to the parent object and save it
								object = Rest._index[pathParts[1]];
							}
							if(!(object.__id in alreadyRecorded)){// if it has already been saved, we don't want to repeat it
								// record that we are saving
								alreadyRecorded[object.__id] = object;
								if(kwArgs.incrementalUpdates 
									&& !pathParts){ // I haven't figured out how we would do incremental updates on sub-objects yet
									// make an incremental update using a POST
									var incremental = (typeof kwArgs.incrementalUpdates == 'function' ?
										kwArgs.incrementalUpdates : function(){
											incremental = {};
											for(var j in object){
												if(object.hasOwnProperty(j)){
													if(object[j] !== old[j]){
														incremental[j] = object[j];
													}
												}else if(old.hasOwnProperty(j)){
													// we can't use incremental updates to remove properties
													return null;
												}
											}
											return incremental;
										})(object, old);
								}
								
								if(incremental){
									actions.push({method:"post",target:object, content: incremental});
								}
								else{
									actions.push({method:"put",target:object,content:object});
								}
							}
						}else{
							// new object
							var service = jr.getServiceAndId(object.__id).service;
							var idAttribute = jr.getIdAttribute(service);
							if((idAttribute in object) && !kwArgs.alwaysPostNewItems){
								// if the id attribute is specified, then we should know the location
								actions.push({method:"put",target:object, content:object});
							}else{
								actions.push({method:"post",target:{__id:service.servicePath},
														content:object});
							}
						}
					}else if(old){
						// deleted object
						actions.push({method:"delete",target:old});
					}//else{ this would happen if an object is created and then deleted, don't do anything
					savingObjects.push(dirty);
					dirtyObjects.splice(i--,1);
				}
			}
			dojo.connect(kwArgs,"onError",function(){
				if(kwArgs.revertOnError !== false){
					var postCommitDirtyObjects = dirtyObjects;
					dirtyObjects = savingObjects;
					var numDirty = 0; // make sure this does't do anything if it is called again
					jr.revert(); // revert if there was an error
					dirtyObjects = postCommitDirtyObjects;
				}
				else{
					dirtyObjects = dirtyObject.concat(savingObjects); 
				}
			});
			jr.sendToServer(actions, kwArgs);
			return actions;
		},
		sendToServer: function(actions, kwArgs){
			var xhrSendId;
			var plainXhr = dojo.xhr;
			var left = actions.length;// this is how many changes are remaining to be received from the server
			var i, contentLocation;
			var timeStamp;
			var conflictDateHeader = this.conflictDateHeader;
			// add headers for extra information
			dojo.xhr = function(method,args){
				// keep the transaction open as we send requests
				args.headers = args.headers || {};
				// the last one should commit the transaction
				args.headers['Transaction'] = actions.length - 1 == i ? "commit" : "open";
				if(conflictDateHeader && timeStamp){
					args.headers[conflictDateHeader] = timeStamp; 
				}
				if(contentLocation){
					args.headers['Content-ID'] = '<' + contentLocation + '>';
				}
				return plainXhr.apply(dojo,arguments);
			};			
			for(i =0; i < actions.length;i++){ // iterate through the actions to execute
				var action = actions[i];
				dojox.rpc.JsonRest._contentId = action.content && action.content.__id; // this is used by OfflineRest
				var isPost = action.method == 'post';
				timeStamp = action.method == 'put' && Rest._timeStamps[action.content.__id];
				if(timeStamp){
					// update it now
					Rest._timeStamps[action.content.__id] = (new Date()) + '';
				}
				// send the content location to the server
				contentLocation = isPost && dojox.rpc.JsonRest._contentId;
				var serviceAndId = jr.getServiceAndId(action.target.__id);
				var service = serviceAndId.service; 
				var dfd = action.deferred = service[action.method](
									serviceAndId.id.replace(/#/,''), // if we are using references, we need eliminate #
									dojox.json.ref.toJson(action.content, false, service.servicePath, true)
								);
				(function(object, dfd, service){
					dfd.addCallback(function(value){
						try{
							// Implements id assignment per the HTTP specification
							var newId = dfd.ioArgs.xhr && dfd.ioArgs.xhr.getResponseHeader("Location");
							//TODO: match URLs if the servicePath is relative...
							if(newId){
								// if the path starts in the middle of an absolute URL for Location, we will use the just the path part 
								var startIndex = newId.match(/(^\w+:\/\/)/) && newId.indexOf(service.servicePath);
								newId = startIndex > 0 ? newId.substring(startIndex) : (service.servicePath + newId).
										// now do simple relative URL resolution in case of a relative URL. 
										replace(/^(.*\/)?(\w+:\/\/)|[^\/\.]+\/\.\.\/|^.*\/(\/)/,'$2$3');
								object.__id = newId;
								Rest._index[newId] = object;
							}
							value = resolveJson(service, dfd, value, object && object.__id);
						}catch(e){}
						if(!(--left)){
							if(kwArgs.onComplete){
								kwArgs.onComplete.call(kwArgs.scope, actions);
							}
						}
						return value;
					});
				})(action.content, dfd, service);
								
				dfd.addErrback(function(value){
					
					// on an error we want to revert, first we want to separate any changes that were made since the commit
					left = -1; // first make sure that success isn't called
					kwArgs.onError.call(kwArgs.scope, value);
				});
			}
			// revert back to the normal XHR handler
			dojo.xhr = plainXhr;
			
		},
		getDirtyObjects: function(){
			return dirtyObjects;
		},
		revert: function(service){
			// summary:
			//		Reverts all the changes made to JSON/REST data
			for(var i = dirtyObjects.length; i > 0;){
				i--;
				var dirty = dirtyObjects[i];
				var object = dirty.object;
				var old = dirty.old;
				var store = dojox.data._getStoreForItem(object || old);
				
				if(!(service && (object || old) && 
					(object || old).__id.indexOf(service.servicePath))){
					// if we are in the specified store or if this is a global revert
					if(object && old){
						// changed
						for(var j in old){
							if(old.hasOwnProperty(j) && object[j] !== old[j]){
								if(store){
									store.onSet(object, j, object[j], old[j]);
								}
								object[j] = old[j];
							}
						}
						for(j in object){
							if(!old.hasOwnProperty(j)){
								if(store){
									store.onSet(object, j, object[j]);
								}
								delete object[j];
							}
						}
					}else if(!old){
						// was an addition, remove it
						if(store){
							store.onDelete(object);
						}
					}else{
						// was a deletion, we will add it back
						if(store){
							store.onNew(old);
						}
					}
					delete (object || old).__isDirty;
					dirtyObjects.splice(i, 1);
				}
			}
		},
		changing: function(object,_deleting){
			// summary:
			//		adds an object to the list of dirty objects.  This object
			//		contains a reference to the object itself as well as a
			//		cloned and trimmed version of old object for use with
			//		revert.
			if(!object.__id){
				return;
			}
			object.__isDirty = true;
			//if an object is already in the list of dirty objects, don't add it again
			//or it will overwrite the premodification data set.
			for(var i=0; i<dirtyObjects.length; i++){
				var dirty = dirtyObjects[i];
				if(object==dirty.object){
					if(_deleting){
						// we are deleting, no object is an indicator of deletiong
						dirty.object = false;
						if(!this._saveNotNeeded){
							dirty.save = true;
						}
					}
					return;
				}
			}
			var old = object instanceof Array ? [] : {};
			for(i in object){
				if(object.hasOwnProperty(i)){
					old[i] = object[i];
				}
			}
			dirtyObjects.push({object: !_deleting && object, old: old, save: !this._saveNotNeeded});
		},
		deleteObject: function(object){
			// summary:
			//		deletes an object 
			//	object:
			//  	object to delete
			this.changing(object,true);
		},
		getConstructor: function(/*Function|String*/service, schema){
			// summary:
			// 		Creates or gets a constructor for objects from this service
			if(typeof service == 'string'){
				var servicePath = service;
				service = new dojox.rpc.Rest(service,true);
				this.registerService(service, servicePath, schema);
			}
			if(service._constructor){
				return service._constructor;
			}
			service._constructor = function(data){
				// summary:
				//		creates a new object for this table
				//
				//	data:
				//		object to mixed in
				var self = this;
				var args = arguments;
				var properties;
				var initializeCalled;
				function addDefaults(schema){
					if(schema){
						addDefaults(schema['extends']);
						properties = schema.properties;
						for(var i in properties){
							var propDef = properties[i]; 
							if(propDef && (typeof propDef == 'object') && ("default" in propDef)){
								self[i] = propDef["default"];
							}
						}
					}
					if(schema && schema.prototype && schema.prototype.initialize){
						initializeCalled = true;
						schema.prototype.initialize.apply(self, args);
					}
				}
				addDefaults(service._schema);
				if(!initializeCalled && data && typeof data == 'object'){
					dojo.mixin(self,data);
				}
				var idAttribute = jr.getIdAttribute(service);
				Rest._index[this.__id = this.__clientId = 
						service.servicePath + (this[idAttribute] || 
							Math.random().toString(16).substring(2,14) + '@' + ((dojox.rpc.Client && dojox.rpc.Client.clientId) || "client"))] = this;
				if(dojox.json.schema && properties){
					dojox.json.schema.mustBeValid(dojox.json.schema.validate(this, service._schema));
				} 
				dirtyObjects.push({object:this, save: true});
			};
			return dojo.mixin(service._constructor, service._schema, {load:service});
		},
		fetch: function(absoluteId){
			// summary:
			//		Fetches a resource by an absolute path/id and returns a dojo.Deferred.
			var serviceAndId = jr.getServiceAndId(absoluteId);
			return this.byId(serviceAndId.service,serviceAndId.id);
		},
		getIdAttribute: function(service){
			// summary:
			//		Return the ids attribute used by this service (based on it's schema).
			//		Defaults to "id", if not other id is defined
			var schema = service._schema;
			var idAttr;
			if(schema){
				if(!(idAttr = schema._idAttr)){
					for(var i in schema.properties){
						if(schema.properties[i].identity || (schema.properties[i].link == "self")){
							schema._idAttr = idAttr = i;
						}
					}
				}
			}
			return idAttr || 'id';
		},
		getServiceAndId: function(/*String*/absoluteId){
			// summary:
			//		Returns the REST service and the local id for the given absolute id. The result 
			// 		is returned as an object with a service property and an id property
			//	absoluteId:
			//		This is the absolute id of the object
			var serviceName = '';
			
			for(var service in jr.services){
				if((absoluteId.substring(0, service.length) == service) && (service.length >= serviceName.length)){
					serviceName = service;
				}
			}
			if (serviceName){
				return {service: jr.services[serviceName], id:absoluteId.substring(serviceName.length)};
			}			
			var parts = absoluteId.match(/^(.*\/)([^\/]*)$/);
			return {service: new jr.serviceClass(parts[1], true), id:parts[2]};
		},
		services:{},
		schemas:{},
		registerService: function(/*Function*/ service, /*String*/ servicePath, /*Object?*/ schema){
			//	summary:
			//		Registers a service for as a JsonRest service, mapping it to a path and schema
			//	service:
			//		This is the service to register
			//	servicePath:
			//		This is the path that is used for all the ids for the objects returned by service
			//	schema:
			//		This is a JSON Schema object to associate with objects returned by this service
			servicePath = service.servicePath = servicePath || service.servicePath;
			service._schema = jr.schemas[servicePath] = schema || service._schema || {};
			jr.services[servicePath] = service;
		},
		byId: function(service, id){
			// if caching is allowed, we look in the cache for the result
			var deferred, result = Rest._index[(service.servicePath || '') + id];
			if(result && !result._loadObject){// cache hit
				deferred = new dojo.Deferred();
				deferred.callback(result);
				return deferred;
			}
			return this.query(service, id);
		},
		query: function(service, id, args){
			var deferred = service(id, args);
			
			deferred.addCallback(function(result){
				if(result.nodeType && result.cloneNode){
					// return immediately if it is an XML document
					return result;
				}				
				return resolveJson(service, deferred, result, typeof id != 'string' || (args && (args.start || args.count)) ? undefined: id);
			});
			return deferred;			
		},
		_loader: function(callback){
			// load a lazy object
			var serviceAndId = jr.getServiceAndId(this.__id);
			var self = this;
			jr.query(serviceAndId.service, serviceAndId.id).addBoth(function(result){
				// if they are the same this means an object was loaded, otherwise it 
				// might be a primitive that was loaded or maybe an error
				if(result == self){
					// we can clear the flag, so it is a loaded object
					delete result.$ref;
					delete result._loadObject;
				}else{
					// it is probably a primitive value, we can't change the identity of an object to
					//	the loaded value, so we will keep it lazy, but define the lazy loader to always
					//	return the loaded value
					self._loadObject = function(callback){
						callback(result);
					};
				}
				callback(result);
			});
		},
		isDirty: function(item){
			// summary
			//		returns true if the item is marked as dirty or true if there are any dirty items
			if(!item){
				return !!dirtyObjects.length;
			}
			return item.__isDirty;
		}
		
	};
})();



}

if(!dojo._hasResource["dojox.data.JsonRestStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.data.JsonRestStore"] = true;
dojo.provide("dojox.data.JsonRestStore");




dojo.declare("dojox.data.JsonRestStore",
	dojox.data.ServiceStore,
	{
		constructor: function(options){
			//summary:
			//		JsonRestStore is a Dojo Data store interface to JSON HTTP/REST web
			//		storage services that support read and write through GET, PUT, POST, and DELETE.
			// options:
			// 		Keyword arguments
			//
			// The *schema* parameter
			//		This is a schema object for this store. This should be JSON Schema format.
			//
			// The *service* parameter
			// 		This is the service object that is used to retrieve lazy data and save results
			// 		The function should be directly callable with a single parameter of an object id to be loaded
			// 		The function should also have the following methods:
			// 			put(id,value) - puts the value at the given id
			// 			post(id,value) - posts (appends) the value at the given id
			// 			delete(id) - deletes the value corresponding to the given id
			//		Note that it is critical that the service parses responses as JSON.
			//		If you are using dojox.rpc.Service, the easiest way to make sure this
			// 		happens is to make the responses have a content type of
			// 		application/json. If you are creating your own service, make sure you
			//		use handleAs: "json" with your XHR requests.
			//
			// The *target* parameter
			// 		This is the target URL for this Service store. This may be used in place
			// 		of a service parameter to connect directly to RESTful URL without
			// 		using a dojox.rpc.Service object.
			//
			// The *idAttribute* parameter
			//		Defaults to 'id'. The name of the attribute that holds an objects id.
			//		This can be a preexisting id provided by the server.
			//		If an ID isn't already provided when an object
			//		is fetched or added to the store, the autoIdentity system
			//		will generate an id for it and add it to the index.
			//
			// The *syncMode* parameter
			//		Setting this to true will set the store to using synchronous calls by default.
			//		Sync calls return their data immediately from the calling function, so
			//		callbacks are unnecessary
			//
			//	description:
			//		The JsonRestStore will cause all saved modifications to be sent to the server using Rest commands (PUT, POST, or DELETE).
			// 		When using a Rest store on a public network, it is important to implement proper security measures to
			//		control access to resources.
			//		On the server side implementing a REST interface means providing GET, PUT, POST, and DELETE handlers.
			//		GET - Retrieve an object or array/result set, this can be by id (like /table/1) or with a
			// 			query (like /table/?name=foo).
			//		PUT - This should modify a object, the URL will correspond to the id (like /table/1), and the body will
			// 			provide the modified object
			//		POST - This should create a new object. The URL will correspond to the target store (like /table/)
			// 			and the body should be the properties of the new object. The server's response should include a
			// 			Location header that indicates the id of the newly created object. This id will be used for subsequent
			// 			PUT and DELETE requests. JsonRestStore also includes a Content-Location header that indicates
			//			the temporary randomly generated id used by client, and this location is used for subsequent
			// 			PUT/DELETEs if no Location header is provided by the server or if a modification is sent prior
			// 			to receiving a response from the server.
			// 		DELETE - This should delete an object by id.
			// 		These articles include more detailed information on using the JsonRestStore:
			//		http://www.sitepen.com/blog/2008/06/13/restful-json-dojo-data/
			//		http://blog.medryx.org/2008/07/24/jsonreststore-overview/
			//
			//	example:
			// 		A JsonRestStore takes a REST service or a URL and uses it the remote communication for a
			// 		read/write dojo.data implementation. A JsonRestStore can be created with a simple URL like:
			// 	|	new JsonRestStore({target:"/MyData/"});
			//	example:
			// 		To use a JsonRestStore with a service, you should create a
			// 		service with a REST transport. This can be configured with an SMD:
			//	|	{
			//	|		services: {
			//	|			jsonRestStore: {
			//	|				transport: "REST",
			//	|				envelope: "URL",
			//	|				target: "store.php",
			//	|				contentType:"application/json",
			//	|				parameters: [
			//	|					{name: "location", type: "string", optional: true}
			//	|				]
			//	|			}
			//	|		}
			//	|	}
			// 		The SMD can then be used to create service, and the service can be passed to a JsonRestStore. For example:
			//	|	var myServices = new dojox.rpc.Service(dojo.moduleUrl("dojox.rpc.tests.resources", "test.smd"));
			//	|	var jsonStore = new dojox.data.JsonRestStore({service:myServices.jsonRestStore});
			//	example:
			//		The JsonRestStore also supports lazy loading. References can be made to objects that have not been loaded.
			//		For example if a service returned:
			//	|	{"name":"Example","lazyLoadedObject":{"$ref":"obj2"}}
			// 		And this object has accessed using the dojo.data API:
			//	|	var obj = jsonStore.getValue(myObject,"lazyLoadedObject");
			//		The object would automatically be requested from the server (with an object id of "obj2").
			//

			dojo.connect(dojox.rpc.Rest._index,"onUpdate",this,function(obj,attrName,oldValue,newValue){
				var prefix = this.service.servicePath;
				if(!obj.__id){
					console.log("no id on updated object ", obj);
				}else if(obj.__id.substring(0,prefix.length) == prefix){
					this.onSet(obj,attrName,oldValue,newValue);
				}
			});
			this.idAttribute = this.idAttribute || 'id';// no options about it, we have to have identity

			if(typeof options.target == 'string'){
				options.target = options.target.match(/\/$/) || this.allowNoTrailingSlash ? options.target : (options.target + '/');
				if(!this.service){
					this.service = dojox.rpc.JsonRest.services[options.target] ||
							dojox.rpc.Rest(options.target, true);
					// create a default Rest service
				}
			}

			dojox.rpc.JsonRest.registerService(this.service, options.target, this.schema);
			this.schema = this.service._schema = this.schema || this.service._schema || {};
			// wrap the service with so it goes through JsonRest manager
			this.service._store = this;
			this.service.idAsRef = this.idAsRef;
			this.schema._idAttr = this.idAttribute;
			var constructor = dojox.rpc.JsonRest.getConstructor(this.service);
			var self = this;
			this._constructor = function(data){
				constructor.call(this, data);
				self.onNew(this);
			}
			this._constructor.prototype = constructor.prototype;
			this._index = dojox.rpc.Rest._index;
		},
		
		// summary:
		//		Will load any schemas referenced content-type header or in Link headers
		loadReferencedSchema: true,
		// summary:
		//		Treat objects in queries as partially loaded objects
		idAsRef: false,
		referenceIntegrity: true,
		target:"",
		// summary:
		// 		Allow no trailing slash on target paths. This is generally discouraged since
		// 		it creates prevents simple scalar values from being used a relative URLs.
		// 		Disabled by default.
		allowNoTrailingSlash: false,
		//Write API Support
		newItem: function(data, parentInfo){
			// summary:
			//		adds a new item to the store at the specified point.
			//		Takes two parameters, data, and options.
			//
			//	data: /* object */
			//		The data to be added in as an item.
			data = new this._constructor(data);
			if(parentInfo){
				// get the previous value or any empty array
				var values = this.getValue(parentInfo.parent,parentInfo.attribute,[]);
				// set the new value
				values = values.concat([data]);
				data.__parent = values;
				this.setValue(parentInfo.parent, parentInfo.attribute, values);
			}
			return data;
		},
		deleteItem: function(item){
			// summary:
			//		deletes item and any references to that item from the store.
			//
			//	item:
			//		item to delete
			//

			//	If the desire is to delete only one reference, unsetAttribute or
			//	setValue is the way to go.
			var checked = [];
			var store = dojox.data._getStoreForItem(item) || this;
			if(this.referenceIntegrity){
				// cleanup all references
				dojox.rpc.JsonRest._saveNotNeeded = true;
				var index = dojox.rpc.Rest._index;
				var fixReferences = function(parent){
					var toSplice;
					// keep track of the checked ones
					checked.push(parent);
					// mark it checked so we don't run into circular loops when encountering cycles
					parent.__checked = 1;
					for(var i in parent){
						if(i.substring(0,2) != "__"){
							var value = parent[i];
							if(value == item){
								if(parent != index){ // make sure we are just operating on real objects
									if(parent instanceof Array){
										// mark it as needing to be spliced, don't do it now or it will mess up the index into the array
										(toSplice = toSplice || []).push(i);
									}else{
										// property, just delete it.
										(dojox.data._getStoreForItem(parent) || store).unsetAttribute(parent, i);
									}
								}
							}else{
								if((typeof value == 'object') && value){
									if(!value.__checked){
										// recursively search
										fixReferences(value);
									}
									if(typeof value.__checked == 'object' && parent != index){
										// if it is a modified array, we will replace it
										(dojox.data._getStoreForItem(parent) || store).setValue(parent, i, value.__checked);
									}
								}
							}
						}
					}
					if(toSplice){
						// we need to splice the deleted item out of these arrays
						i = toSplice.length;
						parent = parent.__checked = parent.concat(); // indicates that the array is modified
						while(i--){
							parent.splice(toSplice[i], 1);
						}
						return parent;
					}
					return null;
				};
				// start with the index
				fixReferences(index);
				dojox.rpc.JsonRest._saveNotNeeded = false;
				var i = 0;
				while(checked[i]){
					// remove the checked marker
					delete checked[i++].__checked;
				}
			}
			dojox.rpc.JsonRest.deleteObject(item);

			store.onDelete(item);
		},
		changing: function(item,_deleting){
			// summary:
			//		adds an item to the list of dirty items.	This item
			//		contains a reference to the item itself as well as a
			//		cloned and trimmed version of old item for use with
			//		revert.
			dojox.rpc.JsonRest.changing(item,_deleting);
		},

		setValue: function(item, attribute, value){
			// summary:
			//		sets 'attribute' on 'item' to 'value'

			var old = item[attribute];
			var store = item.__id ? dojox.data._getStoreForItem(item) : this;
			if(dojox.json.schema && store.schema && store.schema.properties){
				// if we have a schema and schema validator available we will validate the property change
				dojox.json.schema.mustBeValid(dojox.json.schema.checkPropertyChange(value,store.schema.properties[attribute]));
			}
			if(attribute == store.idAttribute){
				throw new Error("Can not change the identity attribute for an item");
			}
			store.changing(item);
			item[attribute]=value;
			if(value && !value.__parent){
				value.__parent = item;
			}
			store.onSet(item,attribute,old,value);
		},
		setValues: function(item, attribute, values){
			// summary:
			//	sets 'attribute' on 'item' to 'value' value
			//	must be an array.


			if(!dojo.isArray(values)){
				throw new Error("setValues expects to be passed an Array object as its value");
			}
			this.setValue(item,attribute,values);
		},

		unsetAttribute: function(item, attribute){
			// summary:
			//		unsets 'attribute' on 'item'

			this.changing(item);
			var old = item[attribute];
			delete item[attribute];
			this.onSet(item,attribute,old,undefined);
		},
		save: function(kwArgs){
			// summary:
			//		Saves the dirty data using REST Ajax methods. See dojo.data.api.Write for API.
			//
			//	kwArgs.global:
			//		This will cause the save to commit the dirty data for all
			// 		JsonRestStores as a single transaction.
			//
			//	kwArgs.revertOnError
			//		This will cause the changes to be reverted if there is an
			//		error on the save. By default a revert is executed unless
			//		a value of false is provide for this parameter.
			//
			//	kwArgs.incrementalUpdates
			//		For items that have been updated, if this is enabled, the server will be sent a POST request
			// 		with a JSON object containing the changed properties. By default this is
			// 		not enabled, and a PUT is used to deliver an update, and will include a full
			// 		serialization of all the properties of the item/object.
			//		If this is true, the POST request body will consist of a JSON object with
			// 		only the changed properties. The incrementalUpdates parameter may also
			//		be a function, in which case it will be called with the updated and previous objects
			//		and an object update representation can be returned.
			//
			//	kwArgs.alwaysPostNewItems
			//		If this is true, new items will always be sent with a POST request. By default
			//		this is not enabled, and the JsonRestStore will send a POST request if
			//		the item does not include its identifier (expecting server assigned location/
			//		identifier), and will send a PUT request if the item does include its identifier
			//		(the PUT will be sent to the URI corresponding to the provided identifier).

			if(!(kwArgs && kwArgs.global)){
				(kwArgs = kwArgs || {}).service = this.service;
			}
			if("syncMode" in kwArgs ? kwArgs.syncMode : this.syncMode){
				dojox.rpc._sync = true;
			}

			var actions = dojox.rpc.JsonRest.commit(kwArgs);
			this.serverVersion = this._updates && this._updates.length;
			return actions;
		},

		revert: function(kwArgs){
			// summary
			//		returns any modified data to its original state prior to a save();
			//
			//	kwArgs.global:
			//		This will cause the revert to undo all the changes for all
			// 		JsonRestStores in a single operation.
			dojox.rpc.JsonRest.revert(kwArgs && kwArgs.global && this.service);
		},

		isDirty: function(item){
			// summary
			//		returns true if the item is marked as dirty.
			return dojox.rpc.JsonRest.isDirty(item);
		},
		isItem: function(item, anyStore){
			//	summary:
			//		Checks to see if a passed 'item'
			//		really belongs to this JsonRestStore.
			//
			//	item: /* object */
			//		The value to test for being an item
			//	anyStore: /* boolean*/
			//		If true, this will return true if the value is an item for any JsonRestStore,
			//		not just this instance
			return item && item.__id && (anyStore || this.service == dojox.rpc.JsonRest.getServiceAndId(item.__id).service);
		},
		_doQuery: function(args){
			var query= typeof args.queryStr == 'string' ? args.queryStr : args.query;
			var deferred = dojox.rpc.JsonRest.query(this.service,query, args);
			var self = this;
			if(this.loadReferencedSchema){
				deferred.addCallback(function(result){
					var contentType = deferred.ioArgs && deferred.ioArgs.xhr && deferred.ioArgs.xhr.getResponseHeader("Content-Type");
					var schemaRef = contentType && contentType.match(/definedby\s*=\s*([^;]*)/);
					if(contentType && !schemaRef){
						schemaRef = deferred.ioArgs.xhr.getResponseHeader("Link");
						schemaRef = schemaRef && schemaRef.match(/<([^>]*)>;\s*rel="?definedby"?/);
					}
					schemaRef = schemaRef && schemaRef[1];
					if(schemaRef){
						var serviceAndId = dojox.rpc.JsonRest.getServiceAndId((self.target + schemaRef).replace(/^(.*\/)?(\w+:\/\/)|[^\/\.]+\/\.\.\/|^.*\/(\/)/,"$2$3"));
						var schemaDeferred = dojox.rpc.JsonRest.byId(serviceAndId.service, serviceAndId.id);
						schemaDeferred.addCallbacks(function(newSchema){
							dojo.mixin(self.schema, newSchema);
							return result;
						}, function(error){
							console.error(error); // log it, but don't let it cause the main request to fail
							return result;
						});
						return schemaDeferred;
					}
					return undefined;//don't change anything, and deal with the stupid post-commit lint complaints
				});
			}
			return deferred;
		},
		_processResults: function(results, deferred){
			// index the results
			var count = results.length;
			// if we don't know the length, and it is partial result, we will guess that it is twice as big, that will work for most widgets
			return {totalCount:deferred.fullLength || (deferred.request.count == count ? (deferred.request.start || 0) + count * 2 : count), items: results};
		},

		getConstructor: function(){
			// summary:
			// 		Gets the constructor for objects from this store
			return this._constructor;
		},
		getIdentity: function(item){
			var id = item.__clientId || item.__id;
			if(!id){
				return id;
			}
			var prefix = this.service.servicePath.replace(/[^\/]*$/,'');
			// support for relative or absolute referencing with ids
			return id.substring(0,prefix.length) != prefix ?	id : id.substring(prefix.length); // String
		},
		fetchItemByIdentity: function(args){
			var id = args.identity;
			var store = this;
			// if it is an absolute id, we want to find the right store to query
			if(id.toString().match(/^(\w*:)?\//)){
				var serviceAndId = dojox.rpc.JsonRest.getServiceAndId(id);
				store = serviceAndId.service._store;
				args.identity = serviceAndId.id;
			}
			args._prefix = store.service.servicePath.replace(/[^\/]*$/,'');
			return store.inherited(arguments);
		},
		//Notifcation Support

		onSet: function(){},
		onNew: function(){},
		onDelete: 	function(){},

		getFeatures: function(){
			// summary:
			// 		return the store feature set
			var features = this.inherited(arguments);
			features["dojo.data.api.Write"] = true;
			features["dojo.data.api.Notification"] = true;
			return features;
		},

		getParent: function(item){
			//	summary:
			//		Returns the parent item (or query) for the given item
			//	item:
			//		The item to find the parent of

			return item && item.__parent;
		}


	}
);
dojox.data.JsonRestStore.getStore = function(options, Class){
	//	summary:
	//		Will retrieve or create a store using the given options (the same options
	//		that are passed to JsonRestStore constructor. Returns a JsonRestStore instance
	//	options:
	//		See the JsonRestStore constructor
	//	Class:
	//		Constructor to use (for creating stores from JsonRestStore subclasses).
	// 		This is optional and defaults to JsonRestStore.
	if(typeof options.target == 'string'){
		options.target = options.target.match(/\/$/) || options.allowNoTrailingSlash ?
				options.target : (options.target + '/');
		var store = (dojox.rpc.JsonRest.services[options.target] || {})._store;
		if(store){
			return store;
		}
	}
	return new (Class || dojox.data.JsonRestStore)(options);
};
dojox.data._getStoreForItem = function(item){
	if(item.__id){
		var serviceAndId = dojox.rpc.JsonRest.getServiceAndId(item.__id);
		if(serviceAndId && serviceAndId.service._store){
			return serviceAndId.service._store;
		}else{
			var servicePath = item.__id.toString().match(/.*\//)[0];
			return new dojox.data.JsonRestStore({target:servicePath});
		}
	}
	return null;
};
dojox.json.ref._useRefs = true; // Use referencing when identifiable objects are referenced

}

if(!dojo._hasResource["dojo.data.ItemFileReadStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.ItemFileReadStore"] = true;
dojo.provide("dojo.data.ItemFileReadStore");





dojo.declare("dojo.data.ItemFileReadStore", null,{
	//	summary:
	//		The ItemFileReadStore implements the dojo.data.api.Read API and reads
	//		data from JSON files that have contents in this format --
	//		{ items: [
	//			{ name:'Kermit', color:'green', age:12, friends:['Gonzo', {_reference:{name:'Fozzie Bear'}}]},
	//			{ name:'Fozzie Bear', wears:['hat', 'tie']},
	//			{ name:'Miss Piggy', pets:'Foo-Foo'}
	//		]}
	//		Note that it can also contain an 'identifer' property that specified which attribute on the items 
	//		in the array of items that acts as the unique identifier for that item.
	//
	constructor: function(/* Object */ keywordParameters){
		//	summary: constructor
		//	keywordParameters: {url: String}
		//	keywordParameters: {data: jsonObject}
		//	keywordParameters: {typeMap: object)
		//		The structure of the typeMap object is as follows:
		//		{
		//			type0: function || object,
		//			type1: function || object,
		//			...
		//			typeN: function || object
		//		}
		//		Where if it is a function, it is assumed to be an object constructor that takes the 
		//		value of _value as the initialization parameters.  If it is an object, then it is assumed
		//		to be an object of general form:
		//		{
		//			type: function, //constructor.
		//			deserialize:	function(value) //The function that parses the value and constructs the object defined by type appropriately.
		//		}
	
		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = [];
		this._loadFinished = false;
		this._jsonFileUrl = keywordParameters.url;
		this._ccUrl = keywordParameters.url;
		this.url = keywordParameters.url;
		this._jsonData = keywordParameters.data;
		this.data = null;
		this._datatypeMap = keywordParameters.typeMap || {};
		if(!this._datatypeMap['Date']){
			//If no default mapping for dates, then set this as default.
			//We use the dojo.date.stamp here because the ISO format is the 'dojo way'
			//of generically representing dates.
			this._datatypeMap['Date'] = {
											type: Date,
											deserialize: function(value){
												return dojo.date.stamp.fromISOString(value);
											}
										};
		}
		this._features = {'dojo.data.api.Read':true, 'dojo.data.api.Identity':true};
		this._itemsByIdentity = null;
		this._storeRefPropName = "_S"; // Default name for the store reference to attach to every item.
		this._itemNumPropName = "_0"; // Default Item Id for isItem to attach to every item.
		this._rootItemPropName = "_RI"; // Default Item Id for isItem to attach to every item.
		this._reverseRefMap = "_RRM"; // Default attribute for constructing a reverse reference map for use with reference integrity
		this._loadInProgress = false; //Got to track the initial load to prevent duelling loads of the dataset.
		this._queuedFetches = [];
		if(keywordParameters.urlPreventCache !== undefined){
			this.urlPreventCache = keywordParameters.urlPreventCache?true:false;
		}
		if(keywordParameters.hierarchical !== undefined){
			this.hierarchical = keywordParameters.hierarchical?true:false;
		}
		if(keywordParameters.clearOnClose){
			this.clearOnClose = true;
		}
		if("failOk" in keywordParameters){
			this.failOk = keywordParameters.failOk?true:false;
		}
	},
	
	url: "",	// use "" rather than undefined for the benefit of the parser (#3539)

	//Internal var, crossCheckUrl.  Used so that setting either url or _jsonFileUrl, can still trigger a reload
	//when clearOnClose and close is used.
	_ccUrl: "",

	data: null,	// define this so that the parser can populate it

	typeMap: null, //Define so parser can populate.
	
	//Parameter to allow users to specify if a close call should force a reload or not.
	//By default, it retains the old behavior of not clearing if close is called.  But
	//if set true, the store will be reset to default state.  Note that by doing this,
	//all item handles will become invalid and a new fetch must be issued.
	clearOnClose: false,

	//Parameter to allow specifying if preventCache should be passed to the xhrGet call or not when loading data from a url.  
	//Note this does not mean the store calls the server on each fetch, only that the data load has preventCache set as an option.
	//Added for tracker: #6072
	urlPreventCache: false,
	
	//Parameter for specifying that it is OK for the xhrGet call to fail silently.
	failOk: false,

	//Parameter to indicate to process data from the url as hierarchical 
	//(data items can contain other data items in js form).  Default is true 
	//for backwards compatibility.  False means only root items are processed 
	//as items, all child objects outside of type-mapped objects and those in 
	//specific reference format, are left straight JS data objects.
	hierarchical: true,

	_assertIsItem: function(/* item */ item){
		//	summary:
		//		This function tests whether the item passed in is indeed an item in the store.
		//	item: 
		//		The item to test for being contained by the store.
		if(!this.isItem(item)){ 
			throw new Error("dojo.data.ItemFileReadStore: Invalid item argument.");
		}
	},

	_assertIsAttribute: function(/* attribute-name-string */ attribute){
		//	summary:
		//		This function tests whether the item passed in is indeed a valid 'attribute' like type for the store.
		//	attribute: 
		//		The attribute to test for being contained by the store.
		if(typeof attribute !== "string"){ 
			throw new Error("dojo.data.ItemFileReadStore: Invalid attribute argument.");
		}
	},

	getValue: function(	/* item */ item, 
						/* attribute-name-string */ attribute, 
						/* value? */ defaultValue){
		//	summary: 
		//		See dojo.data.api.Read.getValue()
		var values = this.getValues(item, attribute);
		return (values.length > 0)?values[0]:defaultValue; // mixed
	},

	getValues: function(/* item */ item, 
						/* attribute-name-string */ attribute){
		//	summary: 
		//		See dojo.data.api.Read.getValues()

		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		// Clone it before returning.  refs: #10474
		return (item[attribute] || []).slice(0); // Array
	},

	getAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getAttributes()
		this._assertIsItem(item);
		var attributes = [];
		for(var key in item){
			// Save off only the real item attributes, not the special id marks for O(1) isItem.
			if((key !== this._storeRefPropName) && (key !== this._itemNumPropName) && (key !== this._rootItemPropName) && (key !== this._reverseRefMap)){
				attributes.push(key);
			}
		}
		return attributes; // Array
	},

	hasAttribute: function(	/* item */ item,
							/* attribute-name-string */ attribute){
		//	summary: 
		//		See dojo.data.api.Read.hasAttribute()
		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		return (attribute in item);
	},

	containsValue: function(/* item */ item, 
							/* attribute-name-string */ attribute, 
							/* anything */ value){
		//	summary: 
		//		See dojo.data.api.Read.containsValue()
		var regexp = undefined;
		if(typeof value === "string"){
			regexp = dojo.data.util.filter.patternToRegExp(value, false);
		}
		return this._containsValue(item, attribute, value, regexp); //boolean.
	},

	_containsValue: function(	/* item */ item, 
								/* attribute-name-string */ attribute, 
								/* anything */ value,
								/* RegExp?*/ regexp){
		//	summary: 
		//		Internal function for looking at the values contained by the item.
		//	description: 
		//		Internal function for looking at the values contained by the item.  This 
		//		function allows for denoting if the comparison should be case sensitive for
		//		strings or not (for handling filtering cases where string case should not matter)
		//	
		//	item:
		//		The data item to examine for attribute values.
		//	attribute:
		//		The attribute to inspect.
		//	value:	
		//		The value to match.
		//	regexp:
		//		Optional regular expression generated off value if value was of string type to handle wildcarding.
		//		If present and attribute values are string, then it can be used for comparison instead of 'value'
		return dojo.some(this.getValues(item, attribute), function(possibleValue){
			if(possibleValue !== null && !dojo.isObject(possibleValue) && regexp){
				if(possibleValue.toString().match(regexp)){
					return true; // Boolean
				}
			}else if(value === possibleValue){
				return true; // Boolean
			}
		});
	},

	isItem: function(/* anything */ something){
		//	summary: 
		//		See dojo.data.api.Read.isItem()
		if(something && something[this._storeRefPropName] === this){
			if(this._arrayOfAllItems[something[this._itemNumPropName]] === something){
				return true;
			}
		}
		return false; // Boolean
	},

	isItemLoaded: function(/* anything */ something){
		//	summary: 
		//		See dojo.data.api.Read.isItemLoaded()
		return this.isItem(something); //boolean
	},

	loadItem: function(/* object */ keywordArgs){
		//	summary: 
		//		See dojo.data.api.Read.loadItem()
		this._assertIsItem(keywordArgs.item);
	},

	getFeatures: function(){
		//	summary: 
		//		See dojo.data.api.Read.getFeatures()
		return this._features; //Object
	},

	getLabel: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getLabel()
		if(this._labelAttr && this.isItem(item)){
			return this.getValue(item,this._labelAttr); //String
		}
		return undefined; //undefined
	},

	getLabelAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getLabelAttributes()
		if(this._labelAttr){
			return [this._labelAttr]; //array
		}
		return null; //null
	},

	_fetchItems: function(	/* Object */ keywordArgs, 
							/* Function */ findCallback, 
							/* Function */ errorCallback){
		//	summary: 
		//		See dojo.data.util.simpleFetch.fetch()
		var self = this,
		    filter = function(requestArgs, arrayOfItems){
			var items = [],
			    i, key;
			if(requestArgs.query){
				var value,
				    ignoreCase = requestArgs.queryOptions ? requestArgs.queryOptions.ignoreCase : false;

				//See if there are any string values that can be regexp parsed first to avoid multiple regexp gens on the
				//same value for each item examined.  Much more efficient.
				var regexpList = {};
				for(key in requestArgs.query){
					value = requestArgs.query[key];
					if(typeof value === "string"){
						regexpList[key] = dojo.data.util.filter.patternToRegExp(value, ignoreCase);
					}else if(value instanceof RegExp){
						regexpList[key] = value;
					}
				}
				for(i = 0; i < arrayOfItems.length; ++i){
					var match = true;
					var candidateItem = arrayOfItems[i];
					if(candidateItem === null){
						match = false;
					}else{
						for(key in requestArgs.query){
							value = requestArgs.query[key];
							if(!self._containsValue(candidateItem, key, value, regexpList[key])){
								match = false;
							}
						}
					}
					if(match){
						items.push(candidateItem);
					}
				}
				findCallback(items, requestArgs);
			}else{
				// We want a copy to pass back in case the parent wishes to sort the array. 
				// We shouldn't allow resort of the internal list, so that multiple callers 
				// can get lists and sort without affecting each other.  We also need to
				// filter out any null values that have been left as a result of deleteItem()
				// calls in ItemFileWriteStore.
				for(i = 0; i < arrayOfItems.length; ++i){
					var item = arrayOfItems[i];
					if(item !== null){
						items.push(item);
					}
				}
				findCallback(items, requestArgs);
			}
		};

		if(this._loadFinished){
			filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
		}else{
			//Do a check on the JsonFileUrl and crosscheck it.
			//If it doesn't match the cross-check, it needs to be updated
			//This allows for either url or _jsonFileUrl to he changed to
			//reset the store load location.  Done this way for backwards 
			//compatibility.  People use _jsonFileUrl (even though officially
			//private.
			if(this._jsonFileUrl !== this._ccUrl){
				dojo.deprecated("dojo.data.ItemFileReadStore: ", 
					"To change the url, set the url property of the store," +
					" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
				this._ccUrl = this._jsonFileUrl;
				this.url = this._jsonFileUrl;
			}else if(this.url !== this._ccUrl){
				this._jsonFileUrl = this.url;
				this._ccUrl = this.url;
			}

			//See if there was any forced reset of data.
			if(this.data != null){
				this._jsonData = this.data;
				this.data = null;
			}

			if(this._jsonFileUrl){
				//If fetches come in before the loading has finished, but while
				//a load is in progress, we have to defer the fetching to be 
				//invoked in the callback.
				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs, filter: filter});
				}else{
					this._loadInProgress = true;
					var getArgs = {
							url: self._jsonFileUrl, 
							handleAs: "json-comment-optional",
							preventCache: this.urlPreventCache,
							failOk: this.failOk
						};
					var getHandler = dojo.xhrGet(getArgs);
					getHandler.addCallback(function(data){
						try{
							self._getItemsFromLoadedData(data);
							self._loadFinished = true;
							self._loadInProgress = false;
							
							filter(keywordArgs, self._getItemsArray(keywordArgs.queryOptions));
							self._handleQueuedFetches();
						}catch(e){
							self._loadFinished = true;
							self._loadInProgress = false;
							errorCallback(e, keywordArgs);
						}
					});
					getHandler.addErrback(function(error){
						self._loadInProgress = false;
						errorCallback(error, keywordArgs);
					});

					//Wire up the cancel to abort of the request
					//This call cancel on the deferred if it hasn't been called
					//yet and then will chain to the simple abort of the
					//simpleFetch keywordArgs
					var oldAbort = null;
					if(keywordArgs.abort){
						oldAbort = keywordArgs.abort;
					}
					keywordArgs.abort = function(){
						var df = getHandler;
						if(df && df.fired === -1){
							df.cancel();
							df = null;
						}
						if(oldAbort){
							oldAbort.call(keywordArgs);
						}
					};
				}
			}else if(this._jsonData){
				try{
					this._loadFinished = true;
					this._getItemsFromLoadedData(this._jsonData);
					this._jsonData = null;
					filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
				}catch(e){
					errorCallback(e, keywordArgs);
				}
			}else{
				errorCallback(new Error("dojo.data.ItemFileReadStore: No JSON source data was provided as either URL or a nested Javascript object."), keywordArgs);
			}
		}
	},

	_handleQueuedFetches: function(){
		//	summary: 
		//		Internal function to execute delayed request in the store.
		//Execute any deferred fetches now.
		if(this._queuedFetches.length > 0){
			for(var i = 0; i < this._queuedFetches.length; i++){
				var fData = this._queuedFetches[i],
				    delayedQuery = fData.args,
				    delayedFilter = fData.filter;
				if(delayedFilter){
					delayedFilter(delayedQuery, this._getItemsArray(delayedQuery.queryOptions)); 
				}else{
					this.fetchItemByIdentity(delayedQuery);
				}
			}
			this._queuedFetches = [];
		}
	},

	_getItemsArray: function(/*object?*/queryOptions){
		//	summary: 
		//		Internal function to determine which list of items to search over.
		//	queryOptions: The query options parameter, if any.
		if(queryOptions && queryOptions.deep){
			return this._arrayOfAllItems; 
		}
		return this._arrayOfTopLevelItems;
	},

	close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
		 //	summary: 
		 //		See dojo.data.api.Read.close()
		 if(this.clearOnClose && 
			this._loadFinished && 
			!this._loadInProgress){
			 //Reset all internalsback to default state.  This will force a reload
			 //on next fetch.  This also checks that the data or url param was set 
			 //so that the store knows it can get data.  Without one of those being set,
			 //the next fetch will trigger an error.

			 if(((this._jsonFileUrl == "" || this._jsonFileUrl == null) && 
				 (this.url == "" || this.url == null)
				) && this.data == null){
				 console.debug("dojo.data.ItemFileReadStore: WARNING!  Data reload " +
					" information has not been provided." + 
					"  Please set 'url' or 'data' to the appropriate value before" +
					" the next fetch");
			 }
			 this._arrayOfAllItems = [];
			 this._arrayOfTopLevelItems = [];
			 this._loadFinished = false;
			 this._itemsByIdentity = null;
			 this._loadInProgress = false;
			 this._queuedFetches = [];
		 }
	},

	_getItemsFromLoadedData: function(/* Object */ dataObject){
		//	summary:
		//		Function to parse the loaded data into item format and build the internal items array.
		//	description:
		//		Function to parse the loaded data into item format and build the internal items array.
		//
		//	dataObject:
		//		The JS data object containing the raw data to convery into item format.
		//
		// 	returns: array
		//		Array of items in store item format.
		
		// First, we define a couple little utility functions...
		var addingArrays = false,
		    self = this;
		
		function valueIsAnItem(/* anything */ aValue){
			// summary:
			//		Given any sort of value that could be in the raw json data,
			//		return true if we should interpret the value as being an
			//		item itself, rather than a literal value or a reference.
			// example:
			// 	|	false == valueIsAnItem("Kermit");
			// 	|	false == valueIsAnItem(42);
			// 	|	false == valueIsAnItem(new Date());
			// 	|	false == valueIsAnItem({_type:'Date', _value:'May 14, 1802'});
			// 	|	false == valueIsAnItem({_reference:'Kermit'});
			// 	|	true == valueIsAnItem({name:'Kermit', color:'green'});
			// 	|	true == valueIsAnItem({iggy:'pop'});
			// 	|	true == valueIsAnItem({foo:42});
			var isItem = (
				(aValue !== null) &&
				(typeof aValue === "object") &&
				(!dojo.isArray(aValue) || addingArrays) &&
				(!dojo.isFunction(aValue)) &&
				(aValue.constructor == Object || dojo.isArray(aValue)) &&
				(typeof aValue._reference === "undefined") && 
				(typeof aValue._type === "undefined") && 
				(typeof aValue._value === "undefined") &&
				self.hierarchical
			);
			return isItem;
		}
		
		function addItemAndSubItemsToArrayOfAllItems(/* Item */ anItem){
			self._arrayOfAllItems.push(anItem);
			for(var attribute in anItem){
				var valueForAttribute = anItem[attribute];
				if(valueForAttribute){
					if(dojo.isArray(valueForAttribute)){
						var valueArray = valueForAttribute;
						for(var k = 0; k < valueArray.length; ++k){
							var singleValue = valueArray[k];
							if(valueIsAnItem(singleValue)){
								addItemAndSubItemsToArrayOfAllItems(singleValue);
							}
						}
					}else{
						if(valueIsAnItem(valueForAttribute)){
							addItemAndSubItemsToArrayOfAllItems(valueForAttribute);
						}
					}
				}
			}
		}

		this._labelAttr = dataObject.label;

		// We need to do some transformations to convert the data structure
		// that we read from the file into a format that will be convenient
		// to work with in memory.

		// Step 1: Walk through the object hierarchy and build a list of all items
		var i,
		    item;
		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = dataObject.items;

		for(i = 0; i < this._arrayOfTopLevelItems.length; ++i){
			item = this._arrayOfTopLevelItems[i];
			if(dojo.isArray(item)){
				addingArrays = true;
			}
			addItemAndSubItemsToArrayOfAllItems(item);
			item[this._rootItemPropName]=true;
		}

		// Step 2: Walk through all the attribute values of all the items, 
		// and replace single values with arrays.  For example, we change this:
		//		{ name:'Miss Piggy', pets:'Foo-Foo'}
		// into this:
		//		{ name:['Miss Piggy'], pets:['Foo-Foo']}
		// 
		// We also store the attribute names so we can validate our store  
		// reference and item id special properties for the O(1) isItem
		var allAttributeNames = {},
		    key;

		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			for(key in item){
				if(key !== this._rootItemPropName){
					var value = item[key];
					if(value !== null){
						if(!dojo.isArray(value)){
							item[key] = [value];
						}
					}else{
						item[key] = [null];
					}
				}
				allAttributeNames[key]=key;
			}
		}

		// Step 3: Build unique property names to use for the _storeRefPropName and _itemNumPropName
		// This should go really fast, it will generally never even run the loop.
		while(allAttributeNames[this._storeRefPropName]){
			this._storeRefPropName += "_";
		}
		while(allAttributeNames[this._itemNumPropName]){
			this._itemNumPropName += "_";
		}
		while(allAttributeNames[this._reverseRefMap]){
			this._reverseRefMap += "_";
		}

		// Step 4: Some data files specify an optional 'identifier', which is 
		// the name of an attribute that holds the identity of each item. 
		// If this data file specified an identifier attribute, then build a 
		// hash table of items keyed by the identity of the items.
		var arrayOfValues;

		var identifier = dataObject.identifier;
		if(identifier){
			this._itemsByIdentity = {};
			this._features['dojo.data.api.Identity'] = identifier;
			for(i = 0; i < this._arrayOfAllItems.length; ++i){
				item = this._arrayOfAllItems[i];
				arrayOfValues = item[identifier];
				var identity = arrayOfValues[0];
				if(!this._itemsByIdentity[identity]){
					this._itemsByIdentity[identity] = item;
				}else{
					if(this._jsonFileUrl){
						throw new Error("dojo.data.ItemFileReadStore:  The json data as specified by: [" + this._jsonFileUrl + "] is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}else if(this._jsonData){
						throw new Error("dojo.data.ItemFileReadStore:  The json data provided by the creation arguments is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}
				}
			}
		}else{
			this._features['dojo.data.api.Identity'] = Number;
		}

		// Step 5: Walk through all the items, and set each item's properties 
		// for _storeRefPropName and _itemNumPropName, so that store.isItem() will return true.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			item[this._storeRefPropName] = this;
			item[this._itemNumPropName] = i;
		}

		// Step 6: We walk through all the attribute values of all the items,
		// looking for type/value literals and item-references.
		//
		// We replace item-references with pointers to items.  For example, we change:
		//		{ name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
		// into this:
		//		{ name:['Kermit'], friends:[miss_piggy] } 
		// (where miss_piggy is the object representing the 'Miss Piggy' item).
		//
		// We replace type/value pairs with typed-literals.  For example, we change:
		//		{ name:['Nelson Mandela'], born:[{_type:'Date', _value:'July 18, 1918'}] }
		// into this:
		//		{ name:['Kermit'], born:(new Date('July 18, 1918')) } 
		//
		// We also generate the associate map for all items for the O(1) isItem function.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i]; // example: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
			for(key in item){
				arrayOfValues = item[key]; // example: [{_reference:{name:'Miss Piggy'}}]
				for(var j = 0; j < arrayOfValues.length; ++j){
					value = arrayOfValues[j]; // example: {_reference:{name:'Miss Piggy'}}
					if(value !== null && typeof value == "object"){
						if(("_type" in value) && ("_value" in value)){
							var type = value._type; // examples: 'Date', 'Color', or 'ComplexNumber'
							var mappingObj = this._datatypeMap[type]; // examples: Date, dojo.Color, foo.math.ComplexNumber, {type: dojo.Color, deserialize(value){ return new dojo.Color(value)}}
							if(!mappingObj){ 
								throw new Error("dojo.data.ItemFileReadStore: in the typeMap constructor arg, no object class was specified for the datatype '" + type + "'");
							}else if(dojo.isFunction(mappingObj)){
								arrayOfValues[j] = new mappingObj(value._value);
							}else if(dojo.isFunction(mappingObj.deserialize)){
								arrayOfValues[j] = mappingObj.deserialize(value._value);
							}else{
								throw new Error("dojo.data.ItemFileReadStore: Value provided in typeMap was neither a constructor, nor a an object with a deserialize function");
							}
						}
						if(value._reference){
							var referenceDescription = value._reference; // example: {name:'Miss Piggy'}
							if(!dojo.isObject(referenceDescription)){
								// example: 'Miss Piggy'
								// from an item like: { name:['Kermit'], friends:[{_reference:'Miss Piggy'}]}
								arrayOfValues[j] = this._getItemByIdentity(referenceDescription);
							}else{
								// example: {name:'Miss Piggy'}
								// from an item like: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
								for(var k = 0; k < this._arrayOfAllItems.length; ++k){
									var candidateItem = this._arrayOfAllItems[k],
									    found = true;
									for(var refKey in referenceDescription){
										if(candidateItem[refKey] != referenceDescription[refKey]){ 
											found = false; 
										}
									}
									if(found){ 
										arrayOfValues[j] = candidateItem; 
									}
								}
							}
							if(this.referenceIntegrity){
								var refItem = arrayOfValues[j];
								if(this.isItem(refItem)){
									this._addReferenceToMap(refItem, item, key);
								}
							}
						}else if(this.isItem(value)){
							//It's a child item (not one referenced through _reference).  
							//We need to treat this as a referenced item, so it can be cleaned up
							//in a write store easily.
							if(this.referenceIntegrity){
								this._addReferenceToMap(value, item, key);
							}
						}
					}
				}
			}
		}
	},

	_addReferenceToMap: function(/*item*/ refItem, /*item*/ parentItem, /*string*/ attribute){
		 //	summary:
		 //		Method to add an reference map entry for an item and attribute.
		 //	description:
		 //		Method to add an reference map entry for an item and attribute. 		 //
		 //	refItem:
		 //		The item that is referenced.
		 //	parentItem:
		 //		The item that holds the new reference to refItem.
		 //	attribute:
		 //		The attribute on parentItem that contains the new reference.
		 
		 //Stub function, does nothing.  Real processing is in ItemFileWriteStore.
	},

	getIdentity: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Identity.getIdentity()
		var identifier = this._features['dojo.data.api.Identity'];
		if(identifier === Number){
			return item[this._itemNumPropName]; // Number
		}else{
			var arrayOfValues = item[identifier];
			if(arrayOfValues){
				return arrayOfValues[0]; // Object || String
			}
		}
		return null; // null
	},

	fetchItemByIdentity: function(/* Object */ keywordArgs){
		//	summary: 
		//		See dojo.data.api.Identity.fetchItemByIdentity()

		// Hasn't loaded yet, we have to trigger the load.
		var item,
		    scope;
		if(!this._loadFinished){
			var self = this;
			//Do a check on the JsonFileUrl and crosscheck it.
			//If it doesn't match the cross-check, it needs to be updated
			//This allows for either url or _jsonFileUrl to he changed to
			//reset the store load location.  Done this way for backwards 
			//compatibility.  People use _jsonFileUrl (even though officially
			//private.
			if(this._jsonFileUrl !== this._ccUrl){
				dojo.deprecated("dojo.data.ItemFileReadStore: ", 
					"To change the url, set the url property of the store," +
					" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
				this._ccUrl = this._jsonFileUrl;
				this.url = this._jsonFileUrl;
			}else if(this.url !== this._ccUrl){
				this._jsonFileUrl = this.url;
				this._ccUrl = this.url;
			}
			
			//See if there was any forced reset of data.
			if(this.data != null){
				this._jsonData = this.data;
				this.data = null;
			}

			if(this._jsonFileUrl){

				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs});
				}else{
					this._loadInProgress = true;
					var getArgs = {
							url: self._jsonFileUrl, 
							handleAs: "json-comment-optional",
							preventCache: this.urlPreventCache,
							failOk: this.failOk
					};
					var getHandler = dojo.xhrGet(getArgs);
					getHandler.addCallback(function(data){
						var scope = keywordArgs.scope?keywordArgs.scope:dojo.global;
						try{
							self._getItemsFromLoadedData(data);
							self._loadFinished = true;
							self._loadInProgress = false;
							item = self._getItemByIdentity(keywordArgs.identity);
							if(keywordArgs.onItem){
								keywordArgs.onItem.call(scope, item);
							}
							self._handleQueuedFetches();
						}catch(error){
							self._loadInProgress = false;
							if(keywordArgs.onError){
								keywordArgs.onError.call(scope, error);
							}
						}
					});
					getHandler.addErrback(function(error){
						self._loadInProgress = false;
						if(keywordArgs.onError){
							var scope = keywordArgs.scope?keywordArgs.scope:dojo.global;
							keywordArgs.onError.call(scope, error);
						}
					});
				}

			}else if(this._jsonData){
				// Passed in data, no need to xhr.
				self._getItemsFromLoadedData(self._jsonData);
				self._jsonData = null;
				self._loadFinished = true;
				item = self._getItemByIdentity(keywordArgs.identity);
				if(keywordArgs.onItem){
					scope = keywordArgs.scope?keywordArgs.scope:dojo.global;
					keywordArgs.onItem.call(scope, item);
				}
			} 
		}else{
			// Already loaded.  We can just look it up and call back.
			item = this._getItemByIdentity(keywordArgs.identity);
			if(keywordArgs.onItem){
				scope = keywordArgs.scope?keywordArgs.scope:dojo.global;
				keywordArgs.onItem.call(scope, item);
			}
		}
	},

	_getItemByIdentity: function(/* Object */ identity){
		//	summary:
		//		Internal function to look an item up by its identity map.
		var item = null;
		if(this._itemsByIdentity){
			item = this._itemsByIdentity[identity];
		}else{
			item = this._arrayOfAllItems[identity];
		}
		if(item === undefined){
			item = null;
		}
		return item; // Object
	},

	getIdentityAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Identity.getIdentifierAttributes()
		 
		var identifier = this._features['dojo.data.api.Identity'];
		if(identifier === Number){
			// If (identifier === Number) it means getIdentity() just returns
			// an integer item-number for each item.  The dojo.data.api.Identity
			// spec says we need to return null if the identity is not composed 
			// of attributes 
			return null; // null
		}else{
			return [identifier]; // Array
		}
	},
	
	_forceLoad: function(){
		//	summary: 
		//		Internal function to force a load of the store if it hasn't occurred yet.  This is required
		//		for specific functions to work properly.  
		var self = this;
		//Do a check on the JsonFileUrl and crosscheck it.
		//If it doesn't match the cross-check, it needs to be updated
		//This allows for either url or _jsonFileUrl to he changed to
		//reset the store load location.  Done this way for backwards 
		//compatibility.  People use _jsonFileUrl (even though officially
		//private.
		if(this._jsonFileUrl !== this._ccUrl){
			dojo.deprecated("dojo.data.ItemFileReadStore: ", 
				"To change the url, set the url property of the store," +
				" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
			this._ccUrl = this._jsonFileUrl;
			this.url = this._jsonFileUrl;
		}else if(this.url !== this._ccUrl){
			this._jsonFileUrl = this.url;
			this._ccUrl = this.url;
		}

		//See if there was any forced reset of data.
		if(this.data != null && this._jsonData == null){
			this._jsonData = this.data;
			this.data = null;
		}

		if(this._jsonFileUrl){
				var getArgs = {
					url: this._jsonFileUrl, 
					handleAs: "json-comment-optional",
					preventCache: this.urlPreventCache,
					failOk: this.failOk,
					sync: true
				};
			var getHandler = dojo.xhrGet(getArgs);
			getHandler.addCallback(function(data){
				try{
					//Check to be sure there wasn't another load going on concurrently 
					//So we don't clobber data that comes in on it.  If there is a load going on
					//then do not save this data.  It will potentially clobber current data.
					//We mainly wanted to sync/wait here.
					//TODO:  Revisit the loading scheme of this store to improve multi-initial
					//request handling.
					if(self._loadInProgress !== true && !self._loadFinished){
						self._getItemsFromLoadedData(data);
						self._loadFinished = true;
					}else if(self._loadInProgress){
						//Okay, we hit an error state we can't recover from.  A forced load occurred
						//while an async load was occurring.  Since we cannot block at this point, the best
						//that can be managed is to throw an error.
						throw new Error("dojo.data.ItemFileReadStore:  Unable to perform a synchronous load, an async load is in progress."); 
					}
				}catch(e){
					console.log(e);
					throw e;
				}
			});
			getHandler.addErrback(function(error){
				throw error;
			});
		}else if(this._jsonData){
			self._getItemsFromLoadedData(self._jsonData);
			self._jsonData = null;
			self._loadFinished = true;
		} 
	}
});
//Mix in the simple fetch implementation to this class.
dojo.extend(dojo.data.ItemFileReadStore,dojo.data.util.simpleFetch);

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.Formatters"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.Formatters"] = true;


dojo.provide("ibm.tivoli.tip.dijit.Formatters");

// TODO: these global variables need to be namespaced ... talk to Yu Xin about this.
// these variables must be put to a more global place, I have put them in TIPChartPortletView.jsp
//var storeCounter = 0;
//var contentStore = [];    

ibm.tivoli.tip.dijit.Formatters.checkBoxFormatter = function (val){  
	if(val == "true"){
        return ("<input type='checkbox' checked>");
    }else{
        return ("<input type='checkbox'>");
    }
};

ibm.tivoli.tip.dijit.Formatters.radioButtonFormatter = function (val){  
	if(val == "true"){
    	return ("<input type='radio' checked>");
    }else{
    	return ("<input type='radio'>");
    }
};

var rowSeveriry = null;

ibm.tivoli.tip.dijit.Formatters.iconBackgroundFormatter = function (val){
	var severity = null;
	var i = val.lastIndexOf("(");
	var j = val.lastIndexOf(")");	
	if(i!=-1&&j!=-1){
		severity = val.substring(i+1,j);
	}	
	rowSeveriry = severity;
	var bkColor = null;	
	var fontColor = null;
	if(severity=="Fatal"){
 		bkColor = "Black";
 		fontColor = "White";
    }else if(severity=="Critical"){	
    	bkColor = "Red";
    	fontColor = "White";
    }else if(severity=="Minor"){	
    	bkColor = "Darkorange";
    	fontColor = "Black";
    }else if(severity=="Warning"){	
    	bkColor = "Yellow";
    	fontColor = "Black";
    }else if(severity=="Harmless"){	
    	bkColor = "LimeGreen";
    	fontColor = "Black";
    }else if(severity=="Informational"){	
    	bkColor = "DodgerBlue";
    	fontColor = "Black";
    }else if(severity=="Unknown"){	
    	bkColor = "DarkGray";
    	fontColor = "Black";
    }
	if(bkColor==null){
		return (val);
	}else{
		return ("<table bgcolor='"+bkColor+"'><tr><td><font color='"+fontColor+"'>"+val.substring(0,i)+"</font></td></tr></table>");
	}	
};	


ibm.tivoli.tip.dijit.Formatters.comboBoxFormatter = function (val){
	var image = null;
	var sevStatus = rowSeveriry;
	if(sevStatus=="Fatal"){
    	image = "st18fatal_shad.png";
    }else if(sevStatus=="Critical"){	
    	image = "st18critical_shad.png";
    }else if(sevStatus=="Minor"){	
    	image = "st18majorwarning_shad.png";
    }else if(sevStatus=="Warning"){	
    	image = "st18minorwarning_shad.png";
    }else if(sevStatus=="Harmless"){	
    	image = "st18success_shad.png";
    }else if(sevStatus=="Informational"){	
    	image = "st18informational_shad.png";
    }else if(sevStatus=="Unknown"){	
    	image = "st18unknown_shad.png";
    }			
	var richContent = {            
            "items": []
    };	
    var optionsData = eval (val);	
    for(var i=0;i<optionsData.length;i++){
         richContent.items[i] = new Object();
     	 richContent.items[i].name = optionsData[i];
     	 if(image!=null){
     		var imageSrc = dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/status/"+image);
    	    richContent.items[i].label = "        <img src='"+imageSrc+"' />        "+optionsData[i];    
         }else{
    		richContent.items[i].label = optionsData[i];    
    	 }             
    }
    storeCounter ++;    
    contentStore[storeCounter] = new dojo.data.ItemFileReadStore ( { data: richContent } );    
    return ("<input dojoType='dijit.form.FilteringSelect' store='contentStore["+storeCounter+"]' labelAttr='label' labelType='html'/>");    
};

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.TableBuilder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.TableBuilder"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/


dojo.provide("ibm.tivoli.tip.charts.dijit.TableBuilder");










dojo.require ("ibm.tivoli.tip.charts.dojo.RESTErrorHandler");



dojo.require ("ibm.tivoli.tip.dojo.TIPHelper");


// NLS


dojo.declare("ibm.tivoli.tip.charts.dijit.TableBuilder",
	         [ dijit._Widget, dijit._Container, ibm.tivoli.tip.charts.dijit.Builder ],
{
	global: null,
	jsonStore: null,
	widgetDefinition: null,
    tableID: null,
	div: null,
	providerID: null,
	datasourceID: null,
    toolbarSelf: null,
	updateMultiplier: 10,
    updateMinInterval: 30000,
    updateMaxInterval: 180000,
	interval: null,
	runReportStartTime: 0,
    runReportResponseTime: 5,
	parameters: null,
	toolbarHidden_:null,
	CHART_SOURCE_TYPE_PAGE: "CHART_SOURCE_TYPE_PAGE",
	isTableFirst: false,
	trigger: null,
	drillUpItems: [],
	isTable: true,
	cmProvider: null,
	gridParms: null,
	tableClassName: "ibm.tivoli.tip.dijit.TIPTable",
	_isUserPrefSaved: false,

	constructor: function (mixinData) {
		this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
	    dojo.mixin( this, mixinData );
	    this.table = null;
	    this.tableBuilder = true;
		this.chartDef = null;
		this.dataset = null;
		this.preferencesDialog = null;
		this.connections = [];
		this.createStoreConnection = null;
		this.nub=null;
		if(!this.widgetParams || !this.widgetParams.standalone){
			this.messages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.chartportlet.dijit", "chart_messages");
		}
		this.currentPage = this.CHART_SOURCE_TYPE_PAGE;

		if ((this.global.iscPortletNamespace === undefined) || (this.global.iscPortletNamespace === null)) {
				//var titleBar = dijit.byId (this.id + "title");
		} else {			
			this.tipHelper_ = new ibm.tivoli.tip.dojo.TIPHelper();
		}
		this._isUserPrefSaved = false;
	},
	startup: function () {
		var busyPane = dijit.byId(this.global.namespace+"busy_pane");
	    if ( busyPane ) {
	      console.log ("showing busypane");
	      busyPane.show(true); // size to portlet
		}
	    
		this.titleDiv = dojo.doc.createElement( 'div' );
		this.titleDiv.id = this.id+"title";
		dojo.place (this.titleDiv, this.div, 0);
		
		this.tableDiv = dojo.doc.createElement( 'div' );
		this.tableDiv.id = this.id+"table";		
		dojo.place (this.tableDiv, this.div, 1);
		
		this.chartDef = this.getTableFromDef();		
		this.dataset = this.getDataSet (this.chartDef.dataset);
		this.toolbarHidden_ = (this.global.mode == "view") ? true : false;
		
		this.createDataset();
		this.createStoreConnection = dojo.connect(this, "createStore", dojo.hitch (this, function(){
			this._makeTIPTable(this.gridParms);
			this.buildTitle();
			//		if(this.table.filterBar){
			//			this.connections.push (dojo.connect (this.table.filterBar.nub, "onclick", this, "toggleToolbar"));
			//		}
        	this.connections.push (dojo.connect (this.table.grid, "onCellDblClick", this, "onDBClickEvent"));		
			this.connections.push (dojo.connect (this.table.grid, "onResizeColumn", this, "onResizeColumnEvent"));
			if(!this.widgetParams || !this.widgetParams.standalone){
				this.updateToolbar();
			}		
		}));
	},
	getTableFromDef: function () {
		var tableDef = null;		
		if(this.widgetDefinition.tables === undefined)
		{
			tableDef = this.widgetDefinition.charts[0];
			tableDef.dataset = this.widgetDefinition.datasets[0].id;			
			tableDef.view = [];			
			for(var i=0; i<this.widgetDefinition.datasets[0].columns.length; i++) {				
				tableDef.view[i] = {
					"id": this.widgetDefinition.datasets[0].columns[i].id,
					"width":"9em",
					"hidden": false,
					"searchable": true
				};
			}
		}
		else
		{
			tableDef = this.widgetDefinition.tables[0];
		}		
		return (tableDef);
	},
	getQuery: function () {
		var dataSetID = this.getTableFromDef().dataset;
		var dataSet = this.getDataSet (dataSetID);
		//var query = dataSet.query + "?widgetID=" + this.widgetDefinition.tables[0].id;
		
		return (dataSet.query);
	},
	getDataSet: function (dataSetID) {
		
		var dataSets = this.widgetDefinition.datasets;
		for (var i=0; i<dataSets.length; i++) {
			if (dataSets[i].id == dataSetID) {
				return (dataSets[i]);
			}
		}
		
		console.error ("could not find dataset with ID: " + dataSetID);
		return (null);
	},
	onNubKeyPress: function (event) {
	  	if (event.keyCode == 13) {
		  	
	  		event.cancelBubble = true;
	  		event.returnValue = false;
	  		console.log ("in onNubKeyPress: pressed enter");
	  		this.toggleToolbar();
	  	}
	},
	toggleToolbar: function () {		
		this.toggleTableToolbar();
		
	},
	toggleTableToolbar: function () {
		console.log ("enter toggle table toolbar");
		
	if (this.table) {
		if (this.toolbarHidden_) {
			this.table.toggleToolbar();
		} else {
			this.table.toggleToolbarOff();
		}
			
		}
	},
	updateToolbar: function () {
		
		var button = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.resources_.PREFERENCES, 
					showLabel: false, 
					iconClass: "propertiesIcon",
				 	onClick: dojo.hitch (this, this.onPreferences) 
				});
			this.table.toolbar.addChild(button);
			
			var backButton = new ibm.tivoli.tip.dijit.TIPButton(
					{	iconClass: "backIcon",
						label: this.resources_.RETURN_TO_CHART_SELECTOR,
						showLabel: false,
					 	onClick: dojo.hitch (this, this.showChartSelector) 
					});
			if(!this.global.isFromPortletPref){
				this.table.toolbar.addChild(backButton);
			}
						
			var ddMenu = new dijit.Menu();		
			var context = ibm.tivoli.tip.dojo.Global.getSingleton(this.global.namespace);			
			if (context.roles.admin || context.roles.creator) {
				var miShare = new dijit.MenuItem ({label: this.resources_.SAVE_FOR_OTHERS});
				miShare.onClick = dojo.hitch (this, "saveSharedPref");				
				ddMenu.addChild (miShare);				
				var miReset = new dijit.MenuItem ({label: this.resources_.CLEAR_DEFAULT_PREFERENCES});
				miReset.onClick = dojo.hitch (this, "onReset");				
				ddMenu.addChild (miReset);
			}			
			var miResetMine = new dijit.MenuItem ({label: this.resources_.CLEAR_MY_PREFERENCES});
			miResetMine.onClick = dojo.hitch (this, "onResetMyPreferences");			
			ddMenu.addChild (miResetMine);			
			button = new dijit.form.DropDownButton (
				{	label: this.resources_.PUBLISH,
					showLabel: false,
					iconClass: "editIcon",
					dropDown: ddMenu
				});			
			this.table.toolbar.addChild (button);

			//for drill up
			if(this.drillUpItems !== undefined && this.drillUpItems.length > 0) {			
				var drillUpOptions = [];
				drillUpOptions[0] = {
								"id": "Drill Up",
								"label": "Drill Up"
							};
				for (var j=0; j<this.drillUpItems.length; j++) {	
					drillUpOptions[j+1] = {
								"id": this.drillUpItems[j].chartIdWithCount,
								"label": this.drillUpItems[j].chartIdWithCount
							};
				}

				var oldDrillUpComboBox = dijit.byId(this.id + "drillUpCombo");
				if(oldDrillUpComboBox) {
					oldDrillUpComboBox.destroyRecursive();
					dijit.registry.remove (this.id + "drillUpCombo");
				}

				var selectOptions = new dojo.data.ItemFileWriteStore({data: {identifier: 'label', items:drillUpOptions}});			
				var drillUpComboBox = new dijit.form.ComboBox({
					id: this.id + "drillUpCombo",
					name: this.id + "drillUpCombo",
					value: "Drill Up",
					store: selectOptions,
					searchAttr: "label",
					onChange: dojo.hitch (this, this.drillUp)
			 },this.id + "drillUpCombo");
			
			 this.table.toolbar.addChild(drillUpComboBox);	
			}

	},
	
	saveUserPref: function () {
		if(!this._isUserPrefSaved){
			var callback = dojo.hitch (this, this.onSaveComplete);			
			var hiddenparams = this.getPageParameters();
			var url = "?" + hiddenparams;
			var kw = {
			 	appendURL: url,
	         	content:   {
                 	command: "savePreferences",
                 	arg_portletNamespace: this.global.namespace,
                 	action: "saveUser"
             	}             
    		};
	    	ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
	    	this._isUserPrefSaved = true;
	    }
		return (false);
	},
	
	saveSharedPref: function () {		
		var callback = dojo.hitch (this, this.onSaveComplete);			
		var hiddenparams = this.getPageParameters();
		var url = "?" + hiddenparams;
		var kw = {
			 appendURL: url,
	         content:   {
                 command: "savePreferences",
                 arg_portletNamespace: this.global.namespace,
                 action: "saveShared"
             },
             onResponse:{
                 callback: callback
       	     }
    	};
	    ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
		return (false);
	},
	
	getPageParameters: function () {
		var type = "TIPServer";	
		var urlfrag = this.namespace + "connectionNameH=" + this.datasourceID;
		urlfrag += "&" + this.global.namespace + "chartIDH=" + this.tableID;
		urlfrag += "&" + this.global.namespace + "versionH=" + this.widgetDefinition.meta.version;
		urlfrag += "&" + this.global.namespace + "chartSourceTypeGroup=" + type;
		urlfrag += "&" + this.global.namespace + "appID=" + this.datasourceID;
		urlfrag += "&" + this.global.namespace + "providerID=" + this.providerID;
		urlfrag += "&arg_partialSave=false";
		return (urlfrag);	
	},
	
	onPreferences: function () { 		
  		
		this.preferencesDialog = new ibm.tivoli.tip.charts.dijit.ChartPreferences ( 
				{ global: this.global,				  
				  tableBuilder: this,
				  isTableFirst: this.isTableFirst
				} 
			);	
		this.preferencesDialog.startup();
		this.preferencesDialog.show();
  	},
	createDataset: function () {
		//var url = this.global.controllerURL + "/dataset/";
		//url = url + this.getQuery();
		//this.jsonStore = new dojox.data.JsonRestStore({target:url, idAttribute:"field"});
		if ((this.interval !== null) && (!this.chartDef.common.autoUpdate)) {
                    clearTimeout (this.interval);
                    this.interval = null;
                }

		var url = "/ibm/tivoli/rest";		
		var dsParams = {};
		var tableParams = this.widgetDefinition.datasets[0].parameters;
		if(tableParams){
			for(var i=0;i<tableParams.length;i++){
				dsParams[tableParams[i].id] = tableParams[i].value;
			}
		}
		if(this.global.restTimeParam){
			dsParams[this.global.restTimeParam.id] = this.global.restTimeParam.value;
		}
		var dataSet = new ibm.tivoli.navmodel.Dataset({
			baseUrl: url,	
			provider: this.providerID,
			datasource: this.datasourceID,
			dataset: this.getTableFromDef().dataset,
			parameters: dsParams,
			onComplete: dojo.hitch(this, "createStore"),
			onError: function(error, exception, ioArgs) {
				var msgHandler = new ibm.tivoli.tip.navwidget.dijit.DefaultMessageHandler({});
				msgHandler.displayException(exception);
			}
		});

	},	
	
	createStore: function(dataSet){
		var navStore = new ibm.tivoli.navmodel.DatasetItemsStore({
			dataset: dataSet,						
			returnItem: false		
		});
		this.jsonStore = navStore;
	},
	
	getDataSetColumn: function (id) {
		var dataSetID = this.getTableFromDef().dataset;
		var dataSet = this.getDataSet (dataSetID);
		
		for (var i=0; i<dataSet.columns.length; i++) {
			if (dataSet.columns[i].id == id) {
				return dataSet.columns[i];
			}
		}
		
		console.error ("Cannot find dataSet column with ID: " + id);
		
		return (null);
	},
	getView: function () {
//		var url = this.global.controllerURL + "/queryMeta/";
//		var results = null;
//		var viewStore = new dojox.data.JsonRestStore({target:url});
//		viewStore.fetch ( {
//				query: this.getQuery(),
//				syncMode: true,
//				onComplete: function (result, request) {
//					results = result;
//				}
//			});
		var viewColumns = this.getTableFromDef().view;
		
		var cells = [];
		var cell = null;
		var viewColumn = null;
		
		for (var i=0; i<viewColumns.length; i++) {
			viewColumn = viewColumns[i];
			cell = {};
			cell.field = viewColumn.id;
			try{
			cell.name = this.getDataSetColumn(viewColumn.id).label.value;
			}catch(e)
			{
				alert(e.message);
			}
			cell.visible = !viewColumn.hidden;
			cell.hidden = viewColumn.hidden;
			cell.width = viewColumn.width;
			cell.isSearchable = viewColumn.searchable;
			/*
			if("ibm.tivoli.tip.dijit.Formatters.iconBackgroundFormatter"==viewColumn.formatter){
				cell.formatter = ibm.tivoli.tip.dijit.Formatters.iconBackgroundFormatter;
			}else if("ibm.tivoli.tip.dijit.Formatters.comboBoxFormatter"==viewColumn.formatter){
				cell.formatter = ibm.tivoli.tip.dijit.Formatters.comboBoxFormatter;
			}else if("ibm.tivoli.tip.dijit.Formatters.checkBoxFormatter"==viewColumn.formatter){
                cell.formatter = ibm.tivoli.tip.dijit.Formatters.checkBoxFormatter;
            }else if("ibm.tivoli.tip.dijit.Formatters.radioButtonFormatter"==viewColumn.formatter){
				cell.formatter = ibm.tivoli.tip.dijit.Formatters.radioButtonFormatter;
            }
            */
			cell.formatter = eval(viewColumn.formatter);
			cells.push(cell);	
		}
		var view = [ { cells: cells } ];

		return (view);
	},
	fetchRows: function () {
		this.jsonStore.fetch ( {
				query: this.getQuery(),
				onComplete: function (result, request) {
					
					
					this.printStore(result.items);
				}
			});
	},
	printStore: function (items) {
		for (var i=0; i<items.length; i++) {
			
		}
	},
	_makeTIPTable: function (gridParms) {
		console.log ("enter makeTIPTable");
		if (this.createStoreConnection !== null) {
			console.log ("disconnecting createStoreConnection: " + this.createStoreConnection);
			dojo.disconnect (this.createStoreConnection);
			this.createStoreConnection = null;
			console.log ("disconnected createStoreConnection");
		} else {
			console.log ("createStoreConnection is null. not disconnecting.");
		}

		var cmProvider = null;
		
		if (this.cmProvider === null) { 
			cmProvider = new ibm.tivoli.tip.dijit.CMSMenuProviderV2(
				{ global: this.global,
				  menuName: "DojoTableV2",
				  getMenusCommandName: "getMenuItems",
				  getMenuActionCommandName: "getMenuCommand",		
				  createCMSEventCallback: this.createCMSEventCallback,		  
				  iscPortletNamespace: this.global.iscPortletNamespace
				});
		} else {
			cmProvider = this.cmProvider;
		}
		
		var cmClientProvider = new ibm.tivoli.tip.dijit.TIPContextMenuProvider(
				{ iscPortletNamespace: this.global.iscPortletNamespace,
				  getMenuItemsCallback: dojo.hitch (this, this.getMenuItems)
				});
		
		if ((gridParms === null) || (gridParms === undefined)) {
			console.log ("creating gridParms");
			gridParms = {};
			gridParms.columnReordering = this.getTableFromDef().columnReordering;
			gridParms.rowsPerPage = this.getTableFromDef().common.pagingSize;
			gridParms.store = this.jsonStore;
			var query = {};			
			var filCond = this.widgetDefinition.datasets[0].conditions;
			var dsColumns = this.widgetDefinition.datasets[0].columns;
			if(filCond && filCond.filterString && filCond.filter.length>0){
				query.expression = filCond.filterString;	
				query.ids = {};			
				for(var i=0;i<filCond.filter.length;i++){
					if(filCond.filter[i].column == "anycolumn"){
						query.ids = {};
						for(var j=0;j<dsColumns.length;j++){
							query.ids[dsColumns[j].id] = true;
						}
						break;
					}else{
						query.ids[dsColumns[filCond.filter[i].column].id] = true;
					}
				}
			}	
			gridParms.query = query;
		} else {
			console.log ("using existing gridParms");
		}
		
		var view = this.getView();
		var multiSelect = true;
		if (this.getTableFromDef().selectionMode == "single" || this.widgetDefinition.tables === undefined) {
			multiSelect = false;
		}
		
		var tableClass = eval (this.tableClassName);
		var toolbarhidden = true;
		if(this.getTableFromDef().toolbar){
			toolbarhidden = this.getTableFromDef().toolbar.hidden;
		}
		var tipTable = new tableClass (
			{
				global: this.global,
				baseQuery: gridParms.query,
				toolbarHidden_: toolbarhidden,
				autoSavePreferences: false,
				tableOptionsEnabled: false,
				multiSelect: multiSelect,
				gridParms: gridParms,
				view: view,
				viewData: view,
				applicationID: this.widgetDefinition.datasource.id,
				widgetID: this.getTableFromDef().id,
				tableParams: this.widgetDefinition.datasets[0].parameters,
				tableBuilder: this,
				cmClientProvider: cmClientProvider,
				cmProvider: cmProvider	
			}, this.tableDiv);
		this.table = tipTable;

		var sub = dojo.subscribe (tipTable.id + "#LoadComplete", this, this.loadComplete);
		this.loadCmplteSub = sub;
		
		tipTable.startup();
		if (this.chartDef.common.showLastUpdateTime) {		
			var msg = this.updateMetaData();
			tipTable.getFooterToolbar().setTableUpdateAreaHTML(msg);
		} else {
			tipTable.getFooterToolbar().setTableUpdateAreaHTML("");
		}
		this.configAutoUpdate();
		
		return (tipTable);
	},	
	
	getCommonMenuItems: function () {
		var menuItems = [{ 
			id: this.namespace + "RefreshChartMenuItem", 
			label: this.resources_.REFRESH, 
			iconClass: "refreshIcon", 
			launchType: "JavaScript", 
			uri: "dijit.byId ('" + this.table.id + "').onRefresh();", enabled: true 
		}];
		var subMenuItems = null;		
		if(this.widgetDefinition.drillDown !== undefined) {			
			subMenuItems = this.getDrillDownMenuItems();
			var oldDrillDownNode = dijit.byId (this.id + "DrillDown");		
			if(oldDrillDownNode) {
				oldDrillDownNode.destroy();
			}
			menuItems.push ({ 
				id: this.id + "DrillDown", 
				label: this.resources_.DRILL_DOWN, 
				icon: "", launchType: "NOOP", 
				submenu_items: subMenuItems, 
				enabled: true 
			});
		}		
		if(this.drillUpItems !== undefined && this.drillUpItems.length > 0) {	
			subMenuItems = this.getDrillUpMenuItems();
			var oldDrillUpNode = dijit.byId (this.id + "DrillUp");		
			if(oldDrillUpNode) {
				oldDrillUpNode.destroy();
			}
			menuItems.push ({ 
				id: this.id + "DrillUp", 
				label: this.resources_.DRILL_UP, 
				icon: "", 
				launchType: "JavaScript", 
				submenu_items: subMenuItems, 
				enabled: true 
			});
		}		
		return (menuItems);
	},

	getCustomMenuItems: function () {
		var menuItems = [{ 
			id: this.namespace + "ShowHideToolbarMenuItem", 
			label: this.resources_.SHOW_HIDE_TOOLBAR, 
			iconClass: "toggleIcon", 
			launchType: "JavaScript", 
			uri: "dijit.byId ('" + this.id + "').toggleToolbar();", 
			enabled: true 
		}];				
		if(!this.global.isFromPortletPref){
			menuItems.push ({ 
				id: this.namespace + "ReturnToChartSelectorMenuItem", 
				label: this.resources_.RETURN_TO_CHART_SELECTOR, 
				iconClass: "backIcon", 
				launchType: "JavaScript", 
				uri: "dijit.byId ('" + this.id + "').showChartSelector();", 
				enabled: true 
			});		
		}
		menuItems.push ({ id: "separator" });  
		menuItems.push ({
			id: this.namespace + "PreferencesMenuItem", 
			label: this.resources_.PREFERENCES, 
			iconClass: "propertiesIcon", 
			launchType: "JavaScript", 
			uri: "dijit.byId ('" + this.id + "').onPreferences();", 
			enabled: true 
		});  		
		var context = ibm.tivoli.tip.dojo.Global.getSingleton(this.global.namespace);
		if (context.roles.admin || context.roles.creator) {
		    menuItems.push ({ 
		    	id: this.namespace + "SaveForAll", 
		    	label: this.resources_.SAVE_FOR_OTHERS, 
		    	icon: "", 
		    	launchType: "JavaScript", 
		    	uri: "dijit.byId ('" + this.id + "').onSave();", 
		    	enabled: true 
		    });
			menuItems.push ({ 
				id: this.namespace + "ResetForAll", 
				label: this.resources_.CLEAR_DEFAULT_PREFERENCES, 
				icon: "", 
				launchType: "JavaScript", 
				uri: "dijit.byId ('" + this.id + "').onReset();", 
				enabled: true 
			});
		}
		menuItems.push ({ 
			id: this.namespace + "ResetForMe", 
			label: this.resources_.CLEAR_MY_PREFERENCES, 
			icon: "", 
			launchType: "JavaScript", 
			uri: "dijit.byId ('" + this.id +"').onResetMyPreferences();", 
			enabled: true 
		});
		menuItems.push({ id: "separator" });
		return (menuItems);
	},

	
	getMenuItems: function () {
		var menuItems = null;
		var commonMenuItems = this.getCommonMenuItems();
		if(this.widgetParams && this.widgetParams.standalone){
			if(!this.widgetParams.ignoreContextMenu){
				menuItems = commonMenuItems;
			}
		}else{			
			var customMenuItems = this.getCustomMenuItems();
			menuItems = commonMenuItems.concat(customMenuItems);
		}		
		return (menuItems);
	},
	
	getDrillDownMenuItems: function() {
		var subMenuItems = [];		
		for (var i=0; i<this.chartDef.contextMenu.length; i++) {
			var drillDownObj = this.getDrillDown(this.chartDef.contextMenu[i].reference);
			subMenuItems.push ({ id: this.id + this.chartDef.contextMenu[i].reference, label: drillDownObj.widgetId, icon: "", launchType: "JavaScript", uri: "dijit.byId ('" + this.id +"').drillDown('"+drillDownObj.id+"');", enabled: true });						
		}
		return (subMenuItems);	
	},

	getDrillUpMenuItems: function() {
		var subMenuItems = [];
		for (var i=0; i<this.drillUpItems.length; i++) {
			var drillUpObj = this.drillUpItems[i];
			subMenuItems.push ({ id: this.id + drillUpObj.chartIdWithCount, label: drillUpObj.chartIdWithCount, icon: "", launchType: "JavaScript", uri: "dijit.byId ('" + this.id +"').drillUp('"+drillUpObj.chartIdWithCount+"');", enabled: true });						
		}		
		return (subMenuItems);
	},	
	
	buildTitle: function () {		
		var tableDef = this.getTableFromDef();
		if (this.widgetParams && this.widgetParams.standalone && !this.widgetParams.ignoreTitlebar) {
			var titleBar = new ibm.tivoli.tip.charts.dijit.ChartTitle ({
				title: tableDef.label.value
			}, dojo.byId (this.id + "title"));
		} else {
			// There is no need to read user or shared preferences from the embedded JSON
			// Those values are merged into the base widget def by CURI before it is returned
			// from the server.
			this.chartDef.label.value = this.getTableFromDef().label.value;
			this.chartDef.description.value = this.getTableFromDef().description.value;
			this.setTitle (this.chartDef.label.value);
		}
	},
	setTitle: function (title) {			
			var tipHelper = new ibm.tivoli.tip.dojo.TIPHelper();
			tipHelper.setPortletTitle (this.table.domNode, title);
	},
	loadComplete: function () {
		this.resizeTable();
		var busyPane = dijit.byId(this.global.namespace+"busy_pane");
		if ( busyPane ) {				
			busyPane.hide();
		}
	},
	resizeTable: function (size) {
		if ((size === undefined) || (size === null) || (size.w === undefined) || (size.h === undefined)) {
			if ((this.global.iscPortletNamespace === undefined) || (this.global.iscPortletNamespace === null)) {
				console.log ("no size sent to resize function, so using viewport size");
				var vp = dijit.getViewport();
				size = { w: vp.w, h: vp.h };
			} else {
				console.log ("no size sent to resize function, so using portlet size");
				var portletSize = iscPortlet.getPortletSize(this.global.iscPortletNamespace);
			
				size = { w: portletSize.available.w, h: portletSize.available.h };
			}
		} else {
			console.log ("using size received from resize function: " + size.w + ", " + size.h);
		}
	
		// without this, we get scroll bars on FF freeform pages
		if (dojo.isFF) {
			size.w = size.w - 2;
		}
		
		if ((this.table.filterBar) && (this.table.filterBar.stateMinimized === true)) {
			size.h = size.h - 7;
		} else if ((this.table.filterBar) && (this.table.filterBar.stateMinimized === false)) {
			size.h = size.h - 10;
		}
		
		this.table.resize ( { w: size.w, h: size.h } );
	},
	updateTableDef: function () {
		this.chartDef.description.value = this.widgetDefinition.tables[0].description.value;
		this.chartDef.label.value = this.widgetDefinition.tables[0].label.value;
		this.chartDef.common = this.widgetDefinition.tables[0].common;
	},
	update: function (prefs, newWidgetDef) {
		this.widgetDefinition = newWidgetDef;
		this.updateTableDef();		
		this.buildTitle();
		
		var msg = this.updateMetaData();
		if (prefs.tables[0].common.showLastUpdateTime) {			
			this.table.getFooterToolbar().setTableUpdateAreaHTML(msg);
		} else {
			
			this.table.getFooterToolbar().setTableUpdateAreaHTML("");
		}		
		this.dataset = this.widgetDefinition.datasets[0];

		if(this.widgetDefinition.user.datasets[0].parameters.length > 0){
			this.parameters = this.widgetDefinition.user.datasets[0].parameters;
			this.updateTableData();
		}else{
			var busyPane = dijit.byId(this.global.namespace+"busy_pane");
			if ( busyPane ) {				
				busyPane.hide();
			}
		}
		this.configAutoUpdate();
			
	},
	updateMetaData: function () {

		var lastUpdated = new Date();
		var msg = dojo.string.substitute(this.resources_.LAST_UPDATED, { 0: lastUpdated.toLocaleString() });
		return msg;
	},
	updateTableData: function () {

		if ((this.interval !== null)) {
           clearTimeout (this.interval);
           this.interval = null;
        }		
		if(this.parameters !== null) {
			this.table.tableParams = this.parameters;
		}

		var domNode = dojo.byId (this.id+"table");
		this.table.toolbar.destroyDescendants();
		this.table.destroyDescendants();
		//fix defect 248535
		dijit.registry.remove (this.id+"table");
		dojo.empty(domNode);
		this.tableDiv = dojo.doc.createElement( 'div' );
		this.tableDiv.id = this.id+"table";	
		
		dojo.place (this.tableDiv, this.div, 1);
		
		this.createDataset();
		this.createStoreConnection = dojo.connect(this, "createStore", dojo.hitch (this, function(){
			this._makeTIPTable(this.gridParms);
			this.connections.push (dojo.connect (this.table.grid, "onCellDblClick", this, "onDBClickEvent"));		
			if(!this.widgetParams || !this.widgetParams.standalone){
				this.updateToolbar();
			}
			var msg = this.updateMetaData();
			if (this.chartDef.common.showLastUpdateTime) {			
				this.table.getFooterToolbar().setTableUpdateAreaHTML(msg);
			} else {
				this.table.getFooterToolbar().setTableUpdateAreaHTML("");
			}
			this.configAutoUpdate();		
		}));
	},
	
	updateChartByTimeSet: function(time){	
		this.global.restTimeParam = {
			id: "Time",
			value: time
		};
		this.createDataset();
		this.connections.push (dojo.connect(this, "createStore", function(){
			this.table.grid.setStore(this.jsonStore);
		}));
	},
	
	configAutoUpdate: function () {		
		
		if ((this.interval === null) && (this.updateMultiplier > 0) &&
			this.chartDef.common.autoUpdate) {			
			var fnc = dojo.hitch (this, this.updateTableData);
		    var nextUpdate = this.getUpdateInterval();
		    this.interval = setTimeout(fnc, nextUpdate);
		} else {
            if ((this.interval !== null) && (!this.chartDef.common.autoUpdate)) {
                clearTimeout (this.interval);
                this.interval = null;
            }
		}		
	},
	onSave: function () {
		this.savePrimitive ("saveShared", true);
	},
	savePrimitive: function (action, showCompletion) {
		var callback = null;
		console.log ("enter savePrimitive with args: " + action + ", " + showCompletion);
		if (showCompletion) {
			callback = dojo.hitch (this, this.onSaveComplete);
		} 
		
		if (this.allowSubmit_) {
			var hiddenparams = this.getHiddenParameters();
			var url = "?" + hiddenparams;

			var kw = {
				 appendURL: url,
	             content:   {
                        command: "savePreferences",
                        arg_portletNamespace: this.global.namespace,
                        action: action
                        },
            	 onResponse:{
                         callback: callback
       	                }
    	     };
	        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		} else {
			this.allowSubmit_ = true;
		}
	},
	onSaveComplete: function (data, ioArgs) {
		console.info ("enter onSaveComplete");
                this.saveSharedPreferences();

		var msgID = "TIPCH3005I";
		console.log("messages"+this.messages_);
		var msg = this.messages_.TIPCH3005I;
		this.global.showMessageDialog("info", msgID, msg, false);
	},
	saveSharedPreferences: function() {
		console.log("enter tableBuilder.saveSharedPreferences");

		if(this.widgetDefinition.user == undefined) {
			return;
		}
		var prefs = dojo.toJson(this.widgetDefinition.user);
		
		var encodedCID = encodeURIComponent(this.tableID);		
		var encodedProviderID = encodeURIComponent(this.providerID);		
		var encodedDS = encodeURIComponent(this.datasourceID);		
		var encodedPID = encodeURIComponent(this.global.chartProfileID);
		var widgetDefVersion = this.widgetDefinition.meta.version;	
		var widgetSchema = this.widgetDefinition.meta.type;
		
		var controller = "/ibm/tivoli/rest";		
		console.log (controller + "/providers/tip.charts/datasources/" + encodedDS + "/widgets/" + widgetSchema + "/definitions/" + encodedCID + "-" + widgetDefVersion + "?type=shared&profileID=" + this.global.chartProfileID + "&registryLocation=local");
		
        var kw = {
        			  sync: true,
					  preventCache: true,
        			  headers: {'Content-Type': 'application/json',
        			  		   'Content-Length': prefs.length
        			  		  },
        			  url: controller + "/providers/" + encodedProviderID + "/datasources/" + encodedDS + "/widgets/"+ widgetSchema + "/definitions/" + encodedCID + "-" + widgetDefVersion + "?type=shared&profileID=" + this.global.chartProfileID + "&registryLocation=local",
                      putData: prefs,
                      error: dojo.hitch (this, function (error) {											
							var errHandler = new ibm.tivoli.tip.charts.dojo.RESTErrorHandler(this.global);		
							errHandler.showError(dojo.fromJson(error.responseText));	
						})
                   };
          dojo.xhrPut (kw);
          console.log ("exit tableBuilder.saveSharedPreferences");         
	},
	getUpdateInterval: function() {
      var nextUpdateInterval = this.runReportResponseTime * this.updateMultiplier;
      
      if ( nextUpdateInterval > this.updateMaxInterval ) {
        nextUpdateInterval = this.updateMaxInterval;
      } else if ( nextUpdateInterval < this.updateMinInterval ) {
        nextUpdateInterval = this.updateMinInterval;
      }
      return nextUpdateInterval;
    },
    getHiddenParameters: function () {
		var type = "TIPServer";
		var appID = this.widgetDefinition.datasource.id;
		var urlfrag = this.namespace + "connectionNameH=" + this.datasourceID;
		urlfrag += "&" + this.namespace + "chartIDH=" + this.tableID;
		urlfrag += "&" + this.namespace + "chartSourceTypeGroup=" + type;
		urlfrag += "&" + this.namespace + "appID=" + this.datasourceID;
		urlfrag += "&" + this.namespace + "providerID=" + this.providerID;
		urlfrag += "&" + this.namespace + "versionH=" + this.widgetDefinition.meta.version;
		urlfrag += "&arg_partialSave=false";

		return (urlfrag);	
	},
	showChartSelector: function () {
    	if (this.interval !== null) {
      		// if auto update is on, cancel it
			clearTimeout (this.interval);
			this.interval = null;
		}
		var chartNavigator = dijit.byId (this.global.namespace + "mainChartNavigatorDiv");
		chartNavigator.setCurrentPage (chartNavigator.CHART_SOURCE_TYPE_PAGE);
		chartNavigator.showChartNavigator();
		this.restorePortletTitle();
		this.destroyRecreate();
	},
	destroyRecreate: function () {
		this.destroy();
		var node = dojo.doc.createElement ("div");
		node.id = this.global.namespace + "widget";
		dojo.place (node, dojo.byId (this.global.namespace + "birtChartWidget"), "after");
	},
	restorePortletTitle: function () {
		// restore portlet title to translated Charting
		this.tipHelper_.setPortletTitle (this.table.domNode, this.global.getPortletTitle());
	},
	onResetPrimitive: function (action, reloadPrefs) {
		var callback = null;
		
		if (reloadPrefs === false) {
			callback = null;
		} else {
			callback = dojo.hitch(this, this.afterReset);
		}
		
		var kw = {
	             content:   {
                        command: "savePreferences",
                        arg_portletNamespace: this.global.namespace,
                        arg_chartProfileID: this.global.chartProfileID,
                        action: action
                        },
                 handle: callback
    	     };
	
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		
		// do not submit
		return (false);
	},
	onReset: function () {
		this.onResetPrimitive ("resetDefaults");
	},
	afterReset: function () {
		if(!this.global.widgetFetcher){
			this.global.widgetFetcher = new ibm.tivoli.tip.charts.dojo.WidgetFetcher ({
				global: this.global
			});
		}		
		var providerID = this.providerID;
		var dsID = this.datasourceID;
		var widgetID = this.tableID;
		var widgetVersion = this.global.builder.widgetDefinition.meta.version;		
		var domNode = dojo.byId (this.global.namespace+"widgetDiv");			
		this.global.builder.destroyRecursive();		
		dojo.destroy(domNode);
		domNode = dojo.doc.createElement("div");
		domNode.id = this.global.namespace+"widgetDiv";
		dojo.byId (this.global.namespace + "widgetContainer").appendChild(domNode);		
		this.global.builder = this.global.widgetFetcher.buildWidget (providerID, dsID, widgetID, domNode, widgetVersion);	
	},

	onResetMyPreferences: function (reloadDefaultPrefs) {
		this.onResetPrimitive ("resetMine", reloadDefaultPrefs);
	},
	resize: function (size) {
		this.resizeTable (size);
	},
	onDBClickEvent: function () {
		this.getTrigger ("ondblclick");
		if (this.trigger.type == "DrillDown") {
			var builder = this.drillDown(this.trigger.reference);
			this.objReassign.call(builder);	
		}  else if (this.trigger.type == "PORTAL_PAGE") {
			this.launchPage(this.trigger);

		} else if (this.trigger.type == "PORTLET") {
			this.launchPortlet(this.trigger);

		} else if (this.trigger.type == "WEB_URL") {		
			this.launchURL (this.trigger);

		} else if (this.trigger.type == "CLIENT_EVENT") {					
			this.sendEvent(this.trigger);

		} else {
			console.error ("unknown Trigger type: " + this.trigger.type);
		}
	},
	onResizeColumnEvent: function() {
		if(this.widgetDefinition) {
			var columns = this.table.originalStructure.cells;
			if(columns){
				var ifColumnWidthChanged = false;
				var userPrefs;
				if(this.widgetDefinition.user){
					userPrefs = this.widgetDefinition.user;
				}else{
					userPrefs = this.widgetDefinition;
					userPrefs.details = {"preferredView": "Table"};
				}
				for (var i in columns) {
					if (i) {
						var column = columns[i];
						if(userPrefs.tables[0].view[i].width != columns[i].unitWidth){
							ifColumnWidthChanged = true;
						}
						userPrefs.tables[0].view[i].width = columns[i].unitWidth;
					}
				}

				if(ifColumnWidthChanged){
					var prefs = dojo.toJson(userPrefs);
					if(true){
						var encodedCID = encodeURIComponent(this.tableID);                
						var encodedProviderID = encodeURIComponent(this.providerID);                
						var encodedDS = encodeURIComponent(this.datasourceID);                
						var encodedPID = encodeURIComponent(this.global.chartProfileID);
						var widgetDefVersion = this.widgetDefinition.meta.version;        
						var widgetSchema = this.widgetDefinition.meta.type;
						
						var controller = "/ibm/tivoli/rest";                
						console.log (controller + "/providers/tip.charts/datasources/" + encodedDS + "/widgets/" + widgetSchema + "/definitions/" + encodedCID + "-" + widgetDefVersion + "?type=user&profileID=" + this.global.chartProfileID + "&registryLocation=local");
						
						var kw = {
								  sync: true,
								  preventCache: true,
								  headers: {'Content-Type': 'application/json',
												   'Content-Length': prefs.length
												  },
								  url: controller + "/providers/" + encodedProviderID + "/datasources/" + encodedDS + "/widgets/"+ widgetSchema + "/definitions/" + encodedCID + "-" + widgetDefVersion + "?type=user&profileID=" + this.global.chartProfileID + "&registryLocation=local",
								  putData: prefs,
								  error: dojo.hitch (this, function (error) {                                                                                        
												var errHandler = new ibm.tivoli.tip.charts.dojo.RESTErrorHandler(this.global);                
												errHandler.showError(dojo.fromJson(error.responseText));        
									})
							};
						dojo.xhrPut (kw);
					}
				}
			}
		}
	},
	launchURL: function(trigger) 
    {
      console.log ("enter launchURL");
      window.open (trigger.url);
    },	
	launchPortlet: function (trigger) 
   {		
      console.log ("enter launchPortlet");
      var portletEvent = {
                            name: "http://ibm.com/isclite#launchPortlet",
                            portletName: trigger.portletName,
                            portletApplicationID: trigger.portletApplicationId
                         };

      if (trigger.portletParameters) 
      {
         for (var j=0; j<trigger.portletParameters.length; j++) 
         {
            try{
            	eval ("portletEvent." + trigger.portletParameters[j].id + "= '" + trigger.portletParameters[j].value + "'");
             }catch(e){
				 //tipChartsAll.js has variable names in a different format, so, eval will not work
             	portletEvent[trigger.portletParameters[j].id] = trigger.portletParameters[j].value;
             }
         }
      }

      try 
      {
         EventBroker.sendEvent (this.global.iscPortletNamespace, portletEvent);
      } catch (e) {
         console.error ("Caught javascript exception in TableBuilder.launchPortlet: " + e);
         alert (e);
      }
   },
   
   launchPage: function (trigger) 
   {	  
      var pageEvent = {
                         name: "http://ibm.com/isclite#launchPage",
                         NavigationNode: trigger.pageId,
                         pageInstanceRef: null,
                         switchPage: true
                      };

      if (trigger.pageParameters) 
      {
         for (var k=0; k<trigger.pageParameters.length; k++) 
         {
             try{
            	 eval ("pageEvent." + trigger.pageParameters[k].id + "= '" + trigger.pageParameters[k].value + "'");
             }catch(e){
				  //tipChartsAll.js has variable names in a different format, so, eval will not work
             	pageEvent[trigger.pageParameters[k].id] = trigger.pageParameters[k].value;//shakeel
             }
         }
      }

      try 
      {
         EventBroker.sendEvent (this.global.iscPortletNamespace, pageEvent);
      } catch (ex) {
         console.error ("Caught javascript exception in TableBuilder.launchPage: " + ex);
         alert (ex);
      }
   },
   
   sendEvent: function (trigger) 
   {
      var parms = trigger.payload;
      var event = {
                     name: trigger.eventName
                  };

      if (trigger.payload) 
      {
         for (var i=0; i<parms.length; i++) 
         {
            try{
             	eval ("event." + parms[i].id + "= '" + parms[i].value + "'");
             }catch(e){
             	event[parms[i].id] = parms[i].value;
             }
         }
      }

	  console.log ("TableBuilder: sending event: ");            
	  console.log (event);
      EventBroker.sendEvent (this.global.iscPortletNamespace, event);
    },
	getTrigger: function (triggerName) {
		if(this.chartDef.triggers === undefined) {
			if(this.chartDef.yAxis !== undefined) {		//it is a change from a chart to a table
				this.chartDef.triggers = this.chartDef.yAxis.triggers;
			}else {
				return null;
			}
		}
			for (var i=0; i<this.chartDef.triggers.length; i++) {
				if (this.chartDef.triggers[i].id == triggerName) {
					this.trigger = this.chartDef.triggers[i];					
					return (this.chartDef.triggers[i]);
				}
			}		
		return (null);
	},
	getDrillDown: function (drillDownId) {
		for (var i=0; i<this.widgetDefinition.drillDown.length; i++) {
				if (this.widgetDefinition.drillDown[i].id == drillDownId) {					
					return (this.widgetDefinition.drillDown[i]);
				}
		}		
		return (null);
	},

	getDrillUp: function (chartIdWithCount) {		
		for (var i=0; i<this.drillUpItems.length; i++) {				
				if(this.drillUpItems[i].chartIdWithCount == chartIdWithCount) {						
					return (this.drillUpItems[i]);
				}
		}		
		return (null);
	},
	objReassign: function() {
		console.log("builder");
		console.log(this);
	},	

	drillUp: function(chartIdWithCount) {		
		var drillUpObj = this.getDrillUp(chartIdWithCount);
		
		if(drillUpObj !== null) {
			
			var encodedChartId = encodeURIComponent(drillUpObj.chartId);
			var encodedProviderId = encodeURIComponent(drillUpObj.providerId);		
			var encodedDatasourceId = encodeURIComponent(drillUpObj.datasourceId);
			var encodedVersion = encodeURIComponent(drillUpObj.version);
			var fetcher = this.global.widgetFetcher;
			var domNode = dojo.byId (this.global.namespace+"widget");
			this.destroy();
			dojo.empty(domNode);
			
			var newDrillUpArray = [];
			
			for (var i=0; i<this.drillUpItems.length; i++) {	
				
				if(this.drillUpItems[i].chartIdWithCount == drillUpObj.chartIdWithCount) {	
					break;
				}
				newDrillUpArray[i] = this.drillUpItems[i];
			}				
						
			dojo.place (domNode, dojo.byId (this.global.namespace + "birtChartWidget"), "after");
			var builder = fetcher.buildWidget(encodedProviderId, encodedDatasourceId, encodedChartId, domNode, encodedVersion, null, newDrillUpArray);
			this.global.builder = builder;
			return builder;
		}
		return this;

	},
	
	drillDown: function(drillDownId) {

		var drillDownObj = this.getDrillDown(drillDownId);
		if(drillDownObj !== null) {

			var encodedCID = encodeURIComponent(drillDownObj.widgetId);
			var encodedProviderID = encodeURIComponent(drillDownObj.provider);	
					
			var encodedDS = null;
			if(drillDownObj.connectionID != undefined)
			{
				encodedDS = encodeURIComponent(drillDownObj.connectionID);			
			}
			else 
			{
				if(drillDownObj.datasource == this.widgetDefinition.datasource.type)
				{
					encodedDS = encodeURIComponent(this.datasourceID);
				}
			}
			
			var fetcher = this.global.widgetFetcher;
			var domNode = dojo.byId (this.global.namespace+"widget");
			this.destroy();
			dojo.empty(domNode);			
			var drillUpLength = this.drillUpItems.length;
			var idCount = 1;
			for (var i=0; i<this.drillUpItems.length; i++) {				
				if(this.drillUpItems[i].chartId == this.chartDef.id) {
					idCount = idCount + 1;
				}
			}
				
			this.drillUpItems[drillUpLength] = {
				"chartId": this.chartDef.id,
				"providerId": this.widgetDefinition.provider.label.key,
				"datasourceId": this.datasourceID,
				"version": this.widgetDefinition.meta.version,
				"idCount": idCount,
				"chartIdWithCount": this.chartDef.id + "(" + idCount + ")"
			};			
			dojo.place (domNode, dojo.byId (this.global.namespace + "birtChartWidget"), "after");
			var builder = fetcher.buildWidget(encodedProviderID, encodedDS, encodedCID, domNode, encodeURIComponent(drillDownObj.version), null, this.drillUpItems);
			this.global.builder = builder;
			return builder;
		}
		return this;
	},
	destroy: function () {		
		this.inherited ("destroy", arguments);
		dojo.unsubscribe(this.loadCmplteSub);
		dojo.forEach (this.connections, function (item) {
			dojo.disconnect (item);
		});
			
		//this.getToolbar().destroyRecursive();	
		//this.table.toolbar.destroyRecursive();	
		//this.table.destroyRecursive();	

		// remove auto-refresh
		if (this.interval !== null) {
			clearTimeout (this.interval);
			delete this.interval;
		}	
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dojo.ChartPreferences"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dojo.ChartPreferences"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/


dojo.provide("ibm.tivoli.tip.charts.dojo.ChartPreferences");

dojo.declare("ibm.tivoli.tip.charts.dojo.ChartPreferences",
	          null,
{
	parameters: null,

	constructor: function (mixinData) {
	    dojo.mixin( this, mixinData );
	
	},
	getParameters: function () {
		return (this.parameters);
	},
	setParameters: function (params) {
		this.parameters = params;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dojo.WidgetFetcher"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dojo.WidgetFetcher"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/


dojo.provide("ibm.tivoli.tip.charts.dojo.WidgetFetcher");


dojo.require ("ibm.tivoli.tip.charts.dijit.ChartBuilder");
dojo.require ("ibm.tivoli.tip.charts.dijit.TableBuilder");
dojo.require ("ibm.tivoli.tip.charts.dojo.ChartPreferences");
dojo.require ("ibm.tivoli.tip.charts.dojo.RESTErrorHandler");
dojo.declare("ibm.tivoli.tip.charts.dojo.WidgetFetcher",
	          null,
{
	/* Default timeout for requests made to the server */
	requestTimeout: 30000,
	global: null,
	autoUpdateConfig: null,

	constructor: function (mixinData) {
	    dojo.mixin( this, mixinData );
	    this.widgetDef = null;
	    this.preferences = null;
	    this.autoUpdateConfig = {};
	    if(this.global.autoUpdateConfig){
	    if(this.global.autoUpdateConfig.maxUpdateInterval){
			this.autoUpdateConfig.updateMaxInterval = this.global.autoUpdateConfig.maxUpdateInterval * 1000;
		}
		if(this.global.autoUpdateConfig.minUpdateInterval){
			this.autoUpdateConfig.updateMinInterval = this.global.autoUpdateConfig.minUpdateInterval * 1000;
		}
		if(this.global.autoUpdateConfig.multiplier){
			this.autoUpdateConfig.updateMultiplier = this.global.autoUpdateConfig.multiplier;
		}
		}
	},
	buildWidgetAPI: function (parameters) {
		return (this.buildWidget (parameters.providerID, parameters.datasourceID, parameters.widgetID,
						  parameters.domNode, parameters.chartVersion, parameters.widgetParameters,
						  parameters.drillUpItems, parameters.initSize));
	},
	fetchWidgetDefinitionAPI: function (parameters) {
		return (this.fetchWidgetDefinition (parameters.providerID, 
							parameters.datasourceID, parameters.widgetID, 
							parameters.widgetVersion, parameters.profileID));
	},
	/*
	 * widgetParams: {
	 * 	 standalone: boolean   run as a standalone widget if true
	 *   ignoreTitlebar: boolean	without titlebar if true
	 *   ignoreToolbar: boolean	 	without toolbar if true
	 *   ignoreContextMenu: boolean     without contextmenu if true
	 * }
	 */
	buildWidget: function (providerID, datasourceID, widgetID, domNode, chartVersion, widgetParams, drillUpItems, initSize) {
		console.log ("enter WidgetFetcher.buildWidget");
		this.widgetDef = this.fetchWidgetDefinition (providerID, datasourceID, widgetID, chartVersion);
		// If "all" is passed in as the datasource, then we must assume the datasource type in the widget definition
		// provides the ID of the datasource.  This is because tip.charts does not list datasources for providers
		// other than tip.charts provider.  The datasource is assumed to be "any" or "all".  However, to allow providers
		// to use datasource as organization mechanism, we provide this mechanism for assuming the datasource is just
		// organziation and not connection instance.
		if (datasourceID == "all") {
			datasourceID = this.widgetDef.datasource.type;
		}
		
		if ((this.widgetDef.charts) && (this.widgetDef.charts[0].type == "Table") || ((this.widgetDef.tables) && (this.widgetDef.tables.length > 0))) {
			var isTableFirst = false;
			if(this.widgetDef.tables) {
	            isTableFirst = true;
			}
			return (this.buildTable (providerID, datasourceID, this.widgetDef, domNode, widgetID, widgetParams, isTableFirst, drillUpItems));
		} else {
			return (this.buildChart (providerID, datasourceID, this.widgetDef, widgetID, domNode, widgetParams, drillUpItems, initSize));
		}
	},
	fetchWidgetDefinition: function (providerID, datasourceID, widgetID, chartVersion, profileID) {	
		var decodedPID = null;
		if(profileID === null || profileID === undefined) {
			//encodedPID = encodeURIComponent(this.global.chartProfileID);
			decodedPID = this.global.chartProfileID;
		}
		else {
			//encodedPID = encodeURIComponent(profileID);
			decodedPID = decodeURIComponent(profileID);
		}		
		// e.g, /ibm/tivoli/rest/providers/tip.charts/datasources/ITM/widgets/charts-2.1.0/definitions/ProcessorUsage
		// TODO: Do not hard code widgetVersion
		//var widgetVersion = "2.1.0";
		var widgetVersion = chartVersion;
		var controller = "/ibm/tivoli/rest";
		var result = null;
		//TODO passing profile Id within content modofies the profileId 
		var profileIdUriParam = decodedPID==null||decodedPID==undefined||decodedPID=="null"||decodedPID=="undefined" ? "" : "&profileID=" + decodedPID;
        var kw = {
        			  sync: true,
					  handleAs: "json",
					  preventCache: true,
    				  mimetype: "application/json",
        			  //url: controller + "/providers/" + providerID + "/datasources/" + datasourceID + "/widgets/charts"+ "-" + widgetVersion + "/definitions/" + widgetID + "-" + widgetVersion + "?profileID=" + decodedPID,
        			  url: controller + "/providers/" + providerID + "/datasources/" + datasourceID + "/widgets/charts-all/definitions/" + widgetID + "-" + widgetVersion + "?registryLocation=local" + profileIdUriParam,
                      load: function (data) {
                           	result = data;
                      },
                      error: dojo.hitch (this, function (error) {												
							var errHandler = new ibm.tivoli.tip.charts.dojo.RESTErrorHandler(this.global);		
							errHandler.showError(dojo.fromJson(error.responseText));	
	 				   })
                   };

          //ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
          dojo.xhrGet (kw);
        
          console.log ("exit WidgetFetcher.fetchWidgetDefinition");
          return (result);
	},
	buildChart: function (providerID, datasourceID, widgetDef, widgetID, domNode, widgetParams, drillUpItems, initSize) {
		console.log ("enter buildChart with providerID & datasourceID: " + providerID + ", " + datasourceID);
		//this.fetchPreferences();
        if(drillUpItems === null || drillUpItems === undefined) {
			drillUpItems = [];
		}
		var params = {
				global: this.global,
				providerID: providerID,
				datasourceID: datasourceID,
				chartID: widgetID,
				chartPreferences: this.preferences,
				widgetDefinition: widgetDef,
				drillUpItems: drillUpItems,
				initSize: initSize
		};
		
		dojo.mixin (params, widgetParams);
		dojo.mixin (params, this.autoUpdateConfig);		
		
		var chartBuilder = new ibm.tivoli.tip.charts.dijit.ChartBuilder(params, domNode); 
		console.log ("created chartBuilder");
		chartBuilder.startup();
		
		return (chartBuilder);
	},
	buildTable: function (providerID, datasourceID, widgetDef, domNode, widgetID, widgetParams, isTableFirst, drillUpItems) {
		console.log ("enter buildTable");
		if(isTableFirst === null || isTableFirst === undefined)
		{
			isTableFirst = false;
		}
		//this.fetchPreferences();
		if(drillUpItems === null || drillUpItems === undefined) {
				drillUpItems = [];
		}
		
		var params = {
			global: this.global,
			providerID: providerID,
			datasourceID: datasourceID,
            tableID: widgetID,
			widgetDefinition: widgetDef,
			widgetParams: widgetParams,
			isTableFirst: isTableFirst,
			drillUpItems: drillUpItems,
			div: domNode
		};
		
		dojo.mixin (params, this.autoUpdateConfig);	
		
		var tableBuilder = new ibm.tivoli.tip.charts.dijit.TableBuilder(params);
		
		console.log ("created tableBuilder");
		tableBuilder.startup();
		
		return (tableBuilder);
	},
	fetchPreferences: function () {
		this.preferences = new ibm.tivoli.tip.charts.dojo.ChartPreferences();
		//this.preferences.setParameters ("?param_ManagedSystemName=*NT_SYSTEM");
					
		return (this.preferences);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.DojoChartBuilder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.DojoChartBuilder"] = true;
dojo.provide("ibm.tivoli.tip.charts.dijit.DojoChartBuilder");



dojo.declare("ibm.tivoli.tip.charts.dijit.DojoChartBuilder",[ibm.tivoli.tip.charts.dijit.ChartBuilder],{		
	
	constructor: function(){
		this.isChartBuilt = false;
		this._widgetSize = null;
		this.isResizeLate = false;
	},	
	
	startup: function(){
		this.chartDef = this.getChartDef (this.chartID);
		this.preBuild = setTimeout(dojo.hitch(this,function(){
			this.setSize(this._widgetSize);
			this.setupTheme();
			this.dataset = this.getDataSet (this.dataSetID);
			this.common = this.getCommonData ();			
			this.constructChart();	
			if(this.chartDef.type != "Pie") {
				this.buildAxisChart();	
			}
			this.buildToolbar();				
			clearTimeout (this.preBuild);
		}),0);					
		this.fetchData();				
	},
	
	fetchData: function(){
		this.chartFetcher = new ibm.tivoli.tip.charts.dojo.ChartFetcher ({
			global: this.global, 
			chartPreferences: this.chartPreferences,
			chartDef: this.chartDef,
			rowsPerPage: this.chartDef.common.pagingSize,
			widgetDefinition: this.widgetDefinition,
			providerID: this.providerID,
			chartParams: this.widgetDefinition.datasets[0].parameters,
			datasourceID: this.datasourceID	
		});
		this.dataFetchSub = dojo.subscribe ("ChartFetcher.dataFetched" + this.chartFetcher.id, dojo.hitch (this, this.dataFetched)); 		
		this.subscribe ("ChartFetcher.totalRowsFetched" + this.chartFetcher.id, dojo.hitch (this, this.buildPager));	
		if(this.chartDef.common.paging){		
			this.chartFetcher.fetchData(0,this.chartFetcher.rowsPerPage);
		}else{
			this.chartFetcher.fetchData();
		}		
	},
	
	renderChart: function(){
		try{
		if(this.chartDef.type == "Pie") {			
			this.buildPieChart();	
		} else {	
			this.feedAxisChartData();		
		} 			
		if ((this.chartDef.legend.hidden !== undefined) && (this.chartDef.legend.hidden === false)) {
			this.addLegend (this.chart);			
		}	
		}catch(e){
			console.error("Render Chart: "+e.message);
		}finally{
			this.isChartBuilt = true;
		}		
		if(this.isResizeLate){
			this.resize();
		}
		dojo.publish ("ChartBuilder.chartBuilt" + this.id, []);		
		this.updateMetaData();
		if (this.chartDef.common.showLastUpdateTime) {
			dojo.style (this.metaDiv, "display", "");
		} else {
			dojo.style (this.metaDiv, "display", "none");
		}
		if(this.postRenderChart){
			clearTimeout(this.postRenderChart);
		}
	},
	
	dataFetched: function(columns, rows, store){		
		dojo.unsubscribe(this.dataFetchSub);
		this.subscribe ("ChartFetcher.dataFetched" + this.chartFetcher.id, dojo.hitch (this, this.afterFetch)); 
		this.columns = columns;
		this.rows = rows;
		this.store = store;		
		if(this.chart){
			this.renderChart();
		}else{
			this.postRenderChart = setTimeout(dojo.hitch(this,this.renderChart),0);
		}
		this.afterBuild = setTimeout(dojo.hitch(this,function(){
			if(!this.standalone) {			
				this.chartUtil = new ibm.tivoli.tip.charts.dojo.DojoChartUtil ({
					namespace: this.global.namespace,
					chartBuilder: this,
					providerID: this.providerID,
					datasourceID: this.datasourceID,
					savePrefsOption: false
				});
			}
			this.addDrillButtons();	
			if(!this.standalone || (this.standalone && !this.ignoreContextMenu)){
				this.makeChartContextMenu();			
			}	
			if ((this.interval !== null) && (!this.chartDef.common.autoUpdate)) {
				clearTimeout (this.interval);
				this.interval = null;
			}	
			this.configAutoUpdate();				
			clearTimeout (this.afterBuild);
		}),0);					
	},
	
	setWidgetSize: function(size){
		this._widgetSize = size;		
	},
	
	setSize: function (size) {	
		dojo.style (this.domNode, {"width": size.w + "px","height": size.h + "px"});					
		if (this.chartDef.common.showLastUpdateTime) {			
			size.h = size.h - 15;				
		} 				
		if (!this.standalone || (this.standalone && !this.ignoreToolbar)) {			
			if(this.chartDef.toolbar.hidden){				
				size.h = size.h - 16;
			}else{					
				size.h = size.h - 35;
			}			
		}
		if(this.standalone && !this.ignoreTitlebar){			
			size.h = size.h - 25;
		}			
		if(this.chartDef.common.paging){			
			size.h = size.h - 25;
		}
		dojo.style (dojo.byId (this.id + "chartContainer"), {
			"width": size.w + "px",
			"height": size.h + "px"
		});	
		var label = this.chartDef.yAxis.label;
		if(label && label.value && label.value != ""){
			var yAxisTitleNode = dojo.byId (this.id + "axistitle");
			if(this.chartDef.type != "Pie") {
				yAxisTitleNode.style.width = "20px";
			}			
			size.w = size.w - 20;
		}			
		label = this.chartDef.xAxis.label;
		if(label && label.value && label.value != ""){			
			size.h = size.h - 15;
		}		
		if ((this.chartDef.legend.hidden !== undefined) && (this.chartDef.legend.hidden === false)) {
			if(this.chartDef.legend.location == "right") {				
				size.w = size.w - 150;	
			} else {				
				size.h = size.h - 30;				
			}		
		}		
		if (dojo.isIE) {			
			size.h = size.h - 10;
			size.w = size.w - 10;
		}		
		// without this, we get scroll bars on FF freeform pages
		if (dojo.isFF) {
			size.w = size.w - 4;
			size.h = size.h - 4;
		}		
		this.chartDivSize = size;
		dojo.style (dojo.byId(this.id + "chart0"), {
			"width": this.chartDivSize.w + "px",
			"height": this.chartDivSize.h + "px"
		});		
	}
		
})

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dojo.WidgetController"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dojo.WidgetController"] = true;

dojo.provide("ibm.tivoli.tip.charts.dojo.WidgetController");

dojo.require ("ibm.tivoli.tip.charts.dijit.DojoChartBuilder");
dojo.require ("ibm.tivoli.tip.charts.dijit.TableBuilder");

dojo.declare("ibm.tivoli.tip.charts.dojo.WidgetController", null,{

	global: null,
	namespace: null,
	iscNamespace: null,
	widgetDef: null,
	widgetParams: null,
	builder: null,
	widgetDomNode: null,
	widgetSize: null,
	_isChart: null,
	
	constructor: function (params) {
		console.log("enter WidgetController.constructor");
		this.widgetDef = null;
		this.widgetParams = null;
		this.global = null;
		this.namespace = null;
		this.iscNamespace = null;
		this.builder = null;
		this.widgetDomNode = null;
		this.widgetSize = null;
		this._isChart = true;
	    dojo.mixin( this, params );	
		console.log("exit WidgetController.constructor");
	},
	
	_buildWidgetParams: function(){
		console.log("enter WidgetController._buildWidgetParams");
		var isResizeLate = false;
		if(!this.widgetSize){
			this.widgetSize = {w:400,h:400};
			isResizeLate = true;
		}
		this.widgetParams = {
			global: this.global,
			_widgetSize: this.widgetSize,
			isResizeLate: isResizeLate,
			providerID: this.widgetDef.provider.id,
			datasourceID: this.widgetDef.datasource.id,
			chartID: this.widgetDef.meta.id,		
			tableID: this.widgetDef.meta.id,
			widgetDefinition: this.widgetDef
		};		
		console.log("exit WidgetController._buildWidgetParams");
	},

	buildWidget: function () {
		console.log("enter WidgetController.buildWidget");
		this.busyPane = new ibm.tivoli.tip.dijit.BusyPane({id: this.global.namespaceID('busy_pane')}, dojo.byId(this.namespace+"busy_pane"));
		this.busyPane.show(true);
		this._buildWidgetParams();
		this.widgetDomNode = dojo.byId(this.namespace+"widgetDiv");
		if ((this.widgetDef.charts) && (this.widgetDef.charts[0].type == "Table") || ((this.widgetDef.tables) && (this.widgetDef.tables.length > 0))) {
			this._isChart = false;			
			this._buildTable();
		} else {
			this._isChart = true;
			this._buildChart();
		}
		this.global.builder = this.builder;
		console.log("exit WidgetController.buildWidget");
	},
	
	resizeWidget: function(size){
		console.log("enter WidgetController.resizeWidget");		
		if(!(size && size.w && size.h)){			
			var portletSize = iscPortlet.getPortletSize(this.iscNamespace);			
			size = { w: portletSize.available.w-10, h: portletSize.available.h-5 };
		}				
		if(!this.widgetSize||(this.widgetSize.w!=size.w || this.widgetSize.h!=size.h)){			
			this.widgetSize = size;				
			if(this.builder.isChartBuilt == true){				
				this.builder.resize(dojo.clone(this.widgetSize));	
			}else{		
				this.builder.setWidgetSize(dojo.clone(this.widgetSize));				
			}					
		}
		console.log("exit WidgetController.resizeWidget");
	},

	_buildChart: function () {	
		console.log("enter WidgetController._buildChart");
		this.builder = new ibm.tivoli.tip.charts.dijit.DojoChartBuilder(this.widgetParams,this.widgetDomNode); 
		this.builder.startup();		
		this.chartBuildSub = dojo.subscribe ("ChartBuilder.chartBuilt" + this.builder.id, dojo.hitch(this,function(){			
			this.busyPane.hide();			
		})); 
		console.log("exit WidgetController._buildChart");
	},
	
	_buildTable: function () {	
		console.log("enter WidgetController._buildTable");
		this.widgetParams.div = this.widgetDomNode;		
		this.builder = new ibm.tivoli.tip.charts.dijit.TableBuilder(this.widgetParams);			
		this.builder.startup();			
		console.log("exit WidgetController._buildTable");
	},
	
	cleanup: function(){
		this.builder.destroyRecursive();
		this.busyPane.destroyRecursive();
		dojo.unsubscribe(this.chartBuildSub);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.TIPDojoAll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.TIPDojoAll"] = true;
dojo.provide("ibm.tivoli.tip.TIPDojoAll");















































	




}


dojo.i18n._preloadLocalizations("ibm.tivoli.tip.nls.TIPDojoAll", ["ROOT","ar","ca","cs","da","de","de-de","el","en","en-gb","en-us","es","es-es","fi","fi-fi","fr","fr-fr","he","he-il","hu","it","it-it","ja","ja-jp","ko","ko-kr","nb","nl","nl-nl","pl","pt","pt-br","pt-pt","ru","sk","sl","sv","th","tr","xx","zh","zh-cn","zh-tw"]);
